<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-13 14:26:50 +0000 (Wed, 13 Feb 2019) $
 * @version        $Rev: 10548 $
 * @package		Clientix.Classes.Field
 * @abstract		General Field Class
 * CHANGE LOG
 *  Included label
 */
class clx_field { 

    var $data;
    var $label;
    var $prefix;
    var $sufix;
    var $required = false;
    var $readonly = false;
    var $js = array();

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        $this->data = $data;
        $this->readonly = (!empty($data['readonly']));
        $this->js = (!empty($data['js'])) ? $data['js'] : array();
        if (!empty($data['validations'])) {
            foreach ($data['validations'] as $key => $val) {
                $this->required = (!empty($val['required']));
                if (!empty($this->required))
                    break;
            }
        }else {
            $this->required = false;
        }
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }


    }

    function editMsg($field_name, $msg, &$table) {
        if (isset($msg[$field_name])) {
            $table->addCell($msg[$field_name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT));
        }
    }

    function viewFormField($toolcode, $record) {
        $contents = '';
        $field_name = $toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
        if (isset($this->data['invisible'])) {
            // do nothing
        } else {
            $source = $field_name_val;
            $contents .= $this->label . $this->prefix . $source . $this->sufix;
        }
        return $contents;
    }

    function viewList(&$fieldrecord, &$record) {
        return $this->buildPrefixSufix($record[$fieldrecord['name']]);
    }

    function checkForm(&$form, &$record, &$msg) {
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
        if (isset($this->data['validations']) && is_array($this->data['validations'])) {
            foreach ($this->data['validations'] as $validation) {
                $type = 'clx_validate_' . $validation['type'];
                $val = new $type($validation);
                $val->check($field_name, $record, $msg, $form);
            }
        } else {

        }
    }

    /**
     * Returns Prefix, Contents and Sufix
     * @param string $contents
     * @return string
     */
    function buildPrefixSufix($contents) {
        $sep1 = '';
        $sep2 = '';
        if (strlen($this->prefix) > CLX_NUM_ZERO)
            $sep1 = ' ';
        if (strlen($this->sufix) > CLX_NUM_ZERO)
            $sep2 = ' ';
        return $this->prefix . $sep1 . $contents . $sep2 . $this->sufix;
    }

}

class clx_field_string extends clx_field {

    var $name;
    var $size;
    var $displaysize;
    var $label;
    var $prefix;
    var $sufix;

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_TEN;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_TEN;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? ((substr($data['prefix'], CLX_NUM_ZERO, CLX_NUM_ONE) === '=') ? substr($data['prefix'], CLX_NUM_ONE) : dict($data['prefix'])): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        if (isset($this->data['invisible'])) {
            $form->htmlform->addArgument($field_name, $field_name_val);
        } else {
            if (isset($msg[$field_name_tex])) {
                $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
            }
            if ((!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && !empty($form->modify)) || (!empty($this->data['readonly']) && !empty($this->data['readonly']))) {
                $source = $field_name_val;
                $form->htmlform->addArgument($field_name, $field_name_val);
            } else {
                $field = new HTMLField($field_name, (array_key_exists($field_name_tex, $record)) ? $field_name_val : '', $this->displaysize, $this->size);
                $source = $field->source();
            }
            $contents .= $this->label . $this->prefix . $source . $this->sufix;
        }
        return $contents;
    }

    function checkForm(&$form, &$record, &$msg) {
        if (!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) {
            // do nothing
        } elseif (!empty($this->data['invisible']) && $this->data['invisible']) {
            // do nothing
        } elseif (!empty($this->data['readonly']) && !empty($this->data['readonly'])) {
            // do nothing
        } else {
            parent::checkForm($form, $record, $msg);
        }
    }

}

class clx_field_date extends clx_field {

    var $name;
    var $size;
    var $displaysize;
    var $label;
    var $prefix;
    var $sufix;
    var $format = 'M d, y';

    function __construct(&$form, $data) {
	$this->format= $_SESSION['prefs']['DATE_FORMAT'];

		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_TEN;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_TEN;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? ((substr($data['prefix'], CLX_NUM_ZERO, CLX_NUM_ONE) === '=') ? substr($data['prefix'], CLX_NUM_ONE) : dict($data['prefix'])): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        if (isset($this->data['invisible'])) {
            $form->htmlform->addArgument($field_name, $field_name_val);
        } else {
            if (!empty($msg[$field_name_tex])) {
                $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
            }
            if ((!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) || (!empty($this->data['readonly']) && !empty($this->data['readonly']))) {
                $source = $field_name_val;
                $form->htmlform->addArgument($field_name, $field_name_val);
            } else {
                $field = new HTMLField($field_name, (array_key_exists($field_name, $record)) ? $field_name_val : '', $this->displaysize, $this->size);
		$format_javascript = $_SESSION['prefs']['DATE_FORMAT'];
		$format_javascript = str_replace('d', 'dd', $format_javascript);
		$format_javascript = str_replace('m', 'mm', $format_javascript);
		$format_javascript = str_replace('Y', 'yy', $format_javascript);
                $picker = '<script type="text/javascript">
            			    $(function() {
            			    	var pickOpts = {
            			    	 dateFormat: "'.$format_javascript.'",
            			    	 showTime: true,
  							     stepMinutes: 1,
         						 stepHours: 1,
         						 altTimeField: "",
            			    	 duration: "",
                                 showTime: false,
                                 constrainInput: false,
                                 time24h: true,
                                 changeMonth: true,
								 changeYear: true,
                                 showOn: "button",
                                 buttonImage: "./' . kPathToSkins . $_SESSION['skin'] . '/images/calendar/cal.gif",
                                 buttonImageOnly: true
            			    	};
                        		$("#' . $field_name . '").datepicker(pickOpts);
                        	});
                        	</script>';
                $source = $field->source() . CLX_CHAR_NL . $picker;
            }
            $contents .= $this->label . $this->prefix . $source . $this->sufix;
        }
        return $contents;
    }

    function checkForm(&$form, &$record, &$msg) {
        if (!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) {
            // do nothing
        } elseif (!empty($this->data['invisible']) && $this->data['invisible']) {
            // do nothing
        } elseif (!empty($this->data['readonly']) && !empty($this->data['readonly'])) {
            // do nothing
        } else {
            parent::checkForm($form, $record, $msg);
        }
    }

    function viewList(&$fieldrecord, &$record) {
        if (!empty($record[$fieldrecord['name']])) {
            return $this->buildPrefixSufix(xFormat::date($record[$fieldrecord['name']], $this->format));
        } else {
            return $this->buildPrefixSufix('');
        }
    }

}

class clx_field_dati extends clx_field {

    var $name;
    var $size;
    var $displaysize;
    var $label;
    var $prefix;
    var $sufix;
    var $format = 'M d, y H:i:s';

    function __construct(&$form, $data) {
	$this->format= $_SESSION['prefs']['DATE_FORMAT'].' H:i:s';
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

		
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_20;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_20;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? ((substr($data['prefix'], CLX_NUM_ZERO, CLX_NUM_ONE) === '=') ? substr($data['prefix'], CLX_NUM_ONE) : dict($data['prefix'])): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
        if (isset($this->data['invisible'])) {
            $form->htmlform->addArgument($field_name, $field_name_val);
        } else {
            if (!empty($msg[$field_name_tex])) {
                $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
            }
            if ((!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && !empty($form->modify)) || (!empty($this->data['readonly']) && !empty($this->data['readonly']))) {
                $source = $field_name_val;
                $form->htmlform->addArgument($field_name, $field_name_val);
            } else {
                $field = new HTMLField($field_name, (array_key_exists($field_name, $record)) ? $field_name_val : '', $this->displaysize, $this->size);
		$format_javascript = $_SESSION['prefs']['DATE_FORMAT'];
		$format_javascript = str_replace('d', 'dd', $format_javascript);
		$format_javascript = str_replace('m', 'mm', $format_javascript);
		$format_javascript = str_replace('Y', 'yy', $format_javascript);

                $picker = '<script type="text/javascript">
            			    $(function() {
            			    	var pickOpts = {
            			    	 dateFormat: "'.$format_javascript.'",
            			    	 showTime: true,
  							     stepMinutes: 1,
         						 stepHours: 1,
         						 altTimeField: "",
            			    	 duration: "",
                                 showTime: true,
                                 constrainInput: false,
                                 time24h: true,
                                 changeMonth: true,
								 changeYear: true,
                                 showOn: "button",
                                 buttonImage: "./' . kPathToSkins . $_SESSION['skin'] . '/images/calendar/cal.gif",
                                 buttonImageOnly: true
            			    	};
                        		$("#' . $field_name . '").datepicker(pickOpts);
                        	});
                        	</script>';
                $source = $field->source() . CLX_CHAR_NL . $picker;
            }
            $contents .= $this->label . $this->prefix . $source . $this->sufix;
        }
        return $contents;
    }

    function checkForm(&$form, &$record, &$msg) {
        if (!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) {
            // do nothing
        } elseif (!empty($this->data['invisible']) && $this->data['invisible']) {
            // do nothing
        } elseif (!empty($this->data['readonly']) && !empty($this->data['readonly'])) {
            // do nothing
        } else {
            parent::checkForm($form, $record, $msg);
        }
    }

    function viewList(&$fieldrecord, &$record) {
        if (!empty($record[$fieldrecord['name']])) {
            return $this->buildPrefixSufix(xFormat::date($record[$fieldrecord['name']], $this->format));
        } else {
            return $this->buildPrefixSufix('');
        }
    }

}

class clx_field_password extends clx_field_string {

    var $name;
    var $size;
    var $displaysize;
    var $label;
    var $prefix;
    var $sufix;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

		
        if (isset($msg[$field_name_tex])) {
            $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
        }
        if ((!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) || (!empty($this->data['readonly']) && !empty($this->data['readonly']))) {
            $source = $field_name_val;
        } else {
            $field = new HTMLPassword($field_name, $field_name_val, $this->displaysize, $this->size);
            $source = $field->source();
        }
        $contents .= $this->label . $this->prefix . $source . $this->sufix;
        return $contents;
    }

    function checkForm(&$form, &$record, &$msg) {
        if ((!empty($this->data['dontmodify']) && !empty($this->data['dontmodify']) && $form->modify) || (!empty($this->data['readonly']) && !empty($this->data['readonly']))) {
            // do nothing
        } else {
            parent::checkForm($form, $record, $msg);
        }
    }

}

class clx_field_dict extends clx_field_string {

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_255;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_48;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
        $form->htmlform->addArgument($field_name, $field_name_val);
		
		if (isset($msg[$field_name_tex . '_DICT'])) {
            $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex . '_DICT'] . '</p>';
        }
        $field = new HTMLField($field_name_tex . '_DICT', $record[$field_name_tex . '_DICT'], $this->displaysize, $this->size);
        $source = $field->source();
        $contents .= $this->label . $this->prefix . $source . $this->sufix;
        return $contents;
    }

    function checkForm(&$form, &$record, &$msg) {
        $field_name = $form->toolcode . '_' . $this->name;
        $DICT = &getTool('DICT');
        $DICT->validateEntry($field_name, $record, $this->data['required'], $msg);
    }

}

class clx_field_number extends clx_field {

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_TEN;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_TEN;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? ((substr($data['prefix'], CLX_NUM_ZERO, CLX_NUM_ONE) === '=') ? substr($data['prefix'], CLX_NUM_ONE) : dict($data['prefix'])): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

}

class clx_field_text extends clx_field {

    var $name;
    var $cols;
    var $rows;
    var $label;
    var $prefix;
    var $sufix;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->cols = isset($data['cols']) ? $data['cols'] : CLX_NUM_48;
        $this->rows = isset($data['rows']) ? $data['rows'] : CLX_NUM_FOUR;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1654;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,'15555 EDITFORM INPUTTEXT'.chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
		if (isset($msg[$field_name_tex])) {
            $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
        }
		$field = new HTMLTextArea($field_name, $field_name_val, $this->rows, $this->cols);
        $contents .= $this->label . $this->prefix . $field->source() . $this->sufix;
        return $contents;
    }

    function viewFormFields($toolcode, $record) {
        return '<pre>' . parent::viewFormFields($toolcode, $record) . '</pre>';
    }

}

class clx_field_lookup extends clx_field {

    var $name;
    var $tool;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->tool = $data['tool'];
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        if (!empty($msg[$field_name_tex])) {
            $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
        }
        $tool = &getTool($this->tool);

        if (!empty($this->readonly)) {
            if ($tool->tablename <> '' && $tool->primary_field <> '') {
                $fields = $tool->primary_field;
                if (!empty($tool->secondary_field)) {
                    $fields = 'CONCAT(' . $tool->primary_field . ',CONCAT(" (", CONCAT(' . $tool->secondary_field . ',")"))) X ';
                }
                $source = ClxCommon::$db->getOne('SELECT ' . $fields . ' FROM ' . $tool->tablename . ' WHERE ' . $tool->primary_field . ' =' . singleQuote($field_name_val));
                if (empty($source)) {
                    $source = $field_name_val;
                }
            } else {
                $source = $field_name_val;
            }
            $form->htmlform->addArgument($field_name, $field_name_val);
        } else {
			list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

            $source = $tool->chooseRecord($field_name, (array_key_exists($field_name, $record)) ? $record[$field_name] : null, $this->data);

            if (!isset($this->label)) {
                $this->label = '';
            }
            if (!isset($this->prefix)) {
                $this->prefix = '';
            }
            if (!isset($this->sufix)) {
                $this->sufix = '';
            }
        }
        $contents .= $this->label . $this->prefix . $source . $this->sufix;
        return $contents;
    }

function viewFormField($toolcode, $record) {
	    $contents = "";

	    if (isset($this->data["invisible"])) {
		    // do nothing
	    } else {

		$field_name = $toolcode  ."_". $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

    $tool = &getTool($this->tool);
		if($tool->tablename<>'' && $tool->primary_field<>''){
		    $fields = $tool->primary_field;
		    if(!empty($tool->secondary_field)){
			$fields ="CONCAT(CONCAT(CONCAT($tool->secondary_field,' ('), $tool->primary_field),')') X ";
		    }
		    $source = ClxCommon::$db->getOne("SELECT
						$fields
					    FROM $tool->tablename
					   WHERE $tool->primary_field ='".$field_name_val."'");
		if(empty($source)){
		    $source = $field_name_val;
		}
		}else{
		    $source = $field_name_val;
		}
		    $contents .= $this->label . $this->prefix . $source . $this->sufix;
	    }

	    return $contents;
    }

    function viewFormFields($toolcode, $record) {
        $contents = '';
        if (isset($this->data['invisible'])) {
            // do nothing
        } else {
            $field_name = $toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);



            $tool = &getTool($this->tool);
            if ($tool->tablename <> '' && $tool->primary_field <> '') {
                $fields = $tool->primary_field;
                if (!empty($tool->secondary_field)) {
                    $fields = 'CONCAT(' . $tool->primary_field . ',CONCAT(" (", CONCAT(' . $tool->secondary_field . ',")"))) X ';
                }
                $source = ClxCommon::$db->getOne('SELECT ' . $fields . ' FROM ' . $tool->tablename . ' WHERE ' . $tool->primary_field . ' =' . singleQuote($field_name_val));
                if (empty($source)) {
                    $source = $field_name_val;
                }
            } else {
                $source = $field_name_val;
            }
            $contents .= $this->label . $this->prefix . $source . $this->sufix;
        }
        return $contents;
    }

    function viewList(&$fieldrecord, &$record) {

        $tool = &getTool($this->tool);
        if ($tool->tablename <> '' && $tool->primary_field <> '') {
            $fields = $tool->primary_field;
            if (!empty($tool->secondary_field)) {
                $fields = 'CONCAT(' . $tool->secondary_field . ',CONCAT(" (",CONCAT(' . $tool->primary_field . ',")"))) X ';
            }
            $source = ClxCommon::$db->getOne('SELECT ' . $fields . ' FROM ' . $tool->tablename . ' WHERE ' . $tool->primary_field . ' =' . singleQuote($record[$fieldrecord['name']]));
            if (empty($source)) {
                $source = $record[$fieldrecord['name']];
            }
        } else {
            $source = $record[$fieldrecord['name']];
        }
        return $this->buildPrefixSufix($source);
    }

}

class clx_field_file extends clx_field {

    var $name;
    var $label;
    var $prefix = '';
    var $sufix = '';

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $form->htmlform->setProp('enctype', 'multipart/form-data');
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
        if (isset($msg[$field_name_tex])) {
			$contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
        }

        $field = new htmlObject('input', '', array('type' => 'file', 'name' => $field_name, 'value' => ((array_key_exists($field_name, $record)) ? $field_name_val : '')));
        return $contents . $this->label . $this->prefix . $field->source() . $this->sufix;
    }

    function checkForm(&$form, &$record, &$msg) {
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		$field_name_val = UserRequest::$files[$field_name]['name'];
        $something = (strlen($field_name_val) > CLX_NUM_ZERO);
        if (isset($this->data['validations'])) {
            foreach ($this->data['validations'] as $validation) {
                $type = 'clx_validate_' . $validation['type'];
                $val = new $type($validation);
                $val->check($field_name, $record, $msg, $form);
            }
        }
        if (!empty($something) and UserRequest::$files[$field_name]['error']) {
            $msg[$field_name_tex] = dict('GEN_ERR_UPLOAD_FILE', UserRequest::$files[$field_name]['error']);
        }
    }

}

class clx_field_list extends clx_field {

    var $name;
    var $label;
    var $prefix;
    var $sufix;
    var $size;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;
	
	
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->size = isset($data['size']) ? dict($data['size']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
		$contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        if (isset($msg[$field_name_tex])) {
            $contents .= '<p class=' . singleQuote('alert') . '>' . $msg[$field_name_tex] . '</p>';
        }
        if (!array_key_exists($field_name_tex, $record)) {
            $field_name_val = '';
        }
        if (is_array($this->data['options'])) {
            $pop = new HTMLSelect($field_name);
            if (!empty($this->size)) {
                $pop->setProp('size', $this->size);
            }
			foreach ($this->data['options'] as $key => $value) {
				$valuex=( (isset($this->data['dict_nochange']) && $this->data['dict_nochange']) ) ? $value:dict($value, false) ;
				if (UserRequest::$request['ac'] == 'IMPORT' or UserRequest::$request['ac'] == 'IMPORT_BATCH') {
                    $pop->addItem($valuex, $key, ($this->data['default']==$key) ? true : false);
                }
				else if (UserRequest::$request['ac'] !== 'UNFILTER') {
                    $pop->addItem($valuex, $key, ((array_key_exists($field_name_tex, $record) && (!is_array($field_name_val) && strval($field_name_val) == strval($key))) ? true : false));
                } else {
                    $pop->addItem($valuex, $key, false);
                }
            }
        } else {
            $DBLT = &getTool('DBLT');
            $pop = $DBLT->getList($this->data['options'], $field_name, $field_name_val);
        }
        foreach ($this->js as $key => $val) {
            $pop->setProp($key, $val);
        }
        ClxFreeReturn( new htmlObject('input', '', array('type' => 'file', 'name' => $field_name, 'value' => $field_name_val)));
        $contents .= $this->label . $this->prefix . $pop->source() . $this->sufix;
        return $contents;
    }

    function viewFormFields($toolcode, $record) {
        $contents = '';
        $field_name = $toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
        if (isset($this->data['invisible'])) {
            // do nothing
        } else {
            $source = $field_name_val;
            if (!empty($this->data['options'])) {
                $contents .= $this->label . $this->prefix . dict($this->data['options'][$source]). $this->sufix;
            } else {
                $contents .= $this->label . $this->prefix . $source . $this->sufix;
            }
        }
        return $contents;
    }
    function viewFormField($toolcode, $record) {
	    $contents = "";
	    $field_name = $toolcode  ."_". $this->name;

	    if (isset($this->data["invisible"])) {
		    // do nothing
	    } else {

		    $source = $record[$field_name];
		    if(!empty($this->data['options'])){
				$contents .= $this->label . $this->prefix . ( (isset($this->data['options'][$source])) ? dict($this->data['options'][$source]):''). $this->sufix;
		    }else{
			    $contents .= $this->label . $this->prefix . $source . $this->sufix;
		    }
	    }

	    return $contents;
    }


    function viewList(&$fieldrecord, &$record) {
        if (!empty($this->data['options'])) {
            if (!isset($this->data['options'][$record[$fieldrecord['name']]]))
                $this->data['options'][$record[$fieldrecord['name']]] = '';
            return $this->buildPrefixSufix(dict($this->data['options'][$record[$fieldrecord['name']]]));
        }else {
            return $this->buildPrefixSufix($record[$fieldrecord['name']]);
        }
    }

}

class clx_field_objects extends clx_field {

    var $name;
    var $objects;
    var $label;
    var $prefix;
    var $sufix;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->objects = $data['objects'];
        $this->prefix = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $this->editMsg($field_name, $msg, $table);

        $pop = new HTMLSelect($field_name);
        $pop->setProp('onchange', 'updateForm();');
        $panellist = CLX_CHAR_NL . 'panellist = new Array();' . CLX_CHAR_NL . CLX_CHAR_NL;
        $panelsources = '';
        $i = CLX_INIT_ZERO;

        $OBJS = &getTool('OBJS');
        foreach ($OBJS->objectList($this->objects) as $objectcode) {

            $object = $OBJS->getObject_($this->objects, $objectcode);
            if (!empty($object->hidden))
                continue;

            $pop->addItem($object->name, $object->code, ($field_name_val == $object->code));
            $panellist .= sprintf('panellist[%s] = ' . singleQuote('%s') . ';' . CLX_CHAR_NL, $i, $objectcode);
            $paneldata = $object->editPanel;
            $paneldata['code'] = $objectcode;
            $panelsobj = new clx_form_panel_dynamic($form, $paneldata);
            $panelsources .= $panelsobj->editForm_($form, $record, $msg);
            $i++;
        }

        $form->topscripts .= setJavaScriptTag($panellist, false);
        $variables = array('VISIBLEPANEL' => $field_name);
        $form->topscripts .= setJavaScriptTag(getJavaScript('dynform_top.js', $variables));
        $form->btmscripts .= setJavaScriptTag(getJavaScriptFilename('dynform_btm.js'), true);

        $table->addCell($this->label . $this->prefix . $pop->source() . $this->sufix, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addCell($panelsources, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $table->source();
    }

    function checkForm(&$form, &$record, &$msg) {
        parent::checkForm($form, $record, $msg);
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        $field_value = $field_name_val;
        $object = getToolObject($this->objects, $field_value);
        $paneldata = $object->editPanel;
        $paneldata['code'] = $field_value;
        $panelsobj = new clx_form_panel_dynamic($form, $paneldata);
        $panelsobj->checkForm($form, $record, $msg);
    }

}

class clx_field_dynamic extends clx_field {

    var $name;
    var $panels;
    var $label;
    var $prefix;
    var $sufix;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->panels = $data['panels'];
        $this->prefix = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? dict($data['prefix']): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $this->editMsg($field_name, $msg, $table);
        $pop = new HTMLSelect($field_name);
        $pop->setProp('onchange', 'updateForm();');
        $panellist = CLX_CHAR_NL . 'panellist = new Array();' . CLX_CHAR_NL . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
		
        foreach ($this->panels as $panel_code => $panel_name) {
			$pop->addItem(dict($panel_name), $panel_code, (array_key_exists($field_name, $record) && $field_name_val == $panel_code));
            $panellist .= sprintf('panellist[%s] = ' . singleQuote('%s') . ';' . CLX_CHAR_NL, $i, $panel_code);
	        $i++;
        }
	    $form->topscripts .= setJavaScriptTag($panellist, false);
        $variables = array();
        $variables['VISIBLEPANEL'] = $field_name;
        $form->topscripts .= setJavaScriptTag(getJavaScript('dynform_top.js', $variables));
        $form->btmscripts .= setJavaScriptTag(getJavaScriptFilename('dynform_btm.js'), true);
        $table->addCell($this->label . $this->prefix . $pop->source() . $this->sufix, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $table->source();
    }

    function checkForm(&$form, &$record, &$msg) {
        parent::checkForm($form, $record, $msg);
    }

}

class clx_field_function extends clx_field {

    var $name;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
		
        if (isset($msg[$field_name_tex])) {
            $contents .= '<p class="alert">' . $msg[$field_name] . '</p>';
        }
        $tool = &getTool($form->toolcode);
        $function = $this->data['function'];
        $source = $tool->$function($field_name, $record);
        if (empty($this->label)) {
            $this->label = '';
        }
        if (empty($this->prefix)) {
            $this->prefix = '';
        }
        if (empty($this->sufix)) {
            $this->sufix = '';
        }
        $contents .= $this->label . $this->prefix . $source . $this->sufix;
        return $contents;
    }

}

class clx_field_radiobutton extends clx_field {

    var $name;
    var $value;
    var $prompt;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->value = $data['value'];
        $this->prompt = $data[CLX_CLASS_PROMPT];
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) { IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $contents = '';
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);
        if (array_key_exists('options', $this->data)) {
            foreach ($this->data['options'] as $key => $value) {
                $radiobutton = new htmlRadioButton($field_name, (array_key_exists($field_name, $record) && $field_name_val == $key), $key);
                $contents .= $radiobutton->source() . dict($value);
            }
        } else {
            $radiobutton = new htmlRadioButton($field_name, (array_key_exists($field_name, $record) && $field_name_val == $this->value), $this->value);
            $contents = $radiobutton->source() . dict($this->prompt);
        }
        return $contents;
    }

}

class clx_field_checkbox extends clx_field {

    var $name;
    var $value;
    var $prompt;
    var $checked;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->value = $data['value'];
        $this->prompt = $data[CLX_CLASS_PROMPT];
        $this->checked = empty($data['checked'])?false:$data['checked'];
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) {	IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; } if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1654;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,'15555 EDITFORM PREINPUTPRETEXT'.chr(13).chr(10));$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
        $field_name = $form->toolcode . '_' . $this->name;
		list($field_name_tex,$field_name_val)=parsing_array_input($field_name,$record);

        $radiobutton = new htmlCheckbox($field_name, $this->checked);
        if (strlen($this->value) > CLX_NUM_ZERO)
            $radiobutton->setProp('value', $this->value);
        return $radiobutton->source() . dict($this->prompt);
    }

}

class clx_field_fieldgroup extends clx_field {

    var $name;
    var $value;
    var $prompt;
    var $checked;

    function __construct(&$form, $data) {
   		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($_SESSION['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;

        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        parent::loadArray($form, $data);
        $this->name = $data['name'];
        $this->size = isset($data['size']) ? $data['size'] : CLX_NUM_TEN;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_TEN;
        $this->label = isset($data['label']) ? dict($data['label']): '';
        $this->prefix = isset($data['prefix']) ? ((substr($data['prefix'], CLX_NUM_ZERO, CLX_NUM_ONE) === '=') ? substr($data['prefix'], CLX_NUM_ONE) : dict($data['prefix'])): '';
        $this->sufix = isset($data['sufix']) ? dict($data['sufix']): '';
        $this->data = $data;
    }

    function editForm_(&$form, $record, $msg) { IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        return 'GEN_LBL_NO_DEFINED';
    }

    function editMsg($field_name, $msg, &$table) {
        return 'GEN_LBL_NO_DEFINED';
    }

    function viewForm_2($toolcode, $record) {
        return 'GEN_LBL_NO_DEFINED';
    }

    function viewList(&$fieldrecord, &$record) {
        $content = '';
        foreach ($fieldrecord['fields'] as $fl) {
            $type = 'clx_field_' . $fl['type'];
            $tmp = new $type($fl, $fl);
            $content .= $tmp->viewList($fl, $record);
        }
        return $this->buildPrefixSufix($content);
    }

    function checkForm(&$form, &$record, &$msg) {
        return 'GEN_LBL_NO_DEFINED';
    }

}
