<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version        $Rev: 10222 $
 * @package		Clientix.Classes.Form
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 */
class clx_form_panel {

    var $title;
    var $caption;  // Panel caption, optional
    var $entries;  // array of clx_form_panel_entries
    var $tableWidth;

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        $this->title = (!empty($data[CLX_CLASS_TITLE])) ? $data[CLX_CLASS_TITLE] : '';
        $this->caption = (!empty($data['caption'])) ? $data['caption'] : '';
        $this->tableWidth = (isset($data['cols'])) ? $data['cols'] : '2';
        $this->entries = array();
        if (empty($data['entries'])) {
            $data['entries'] = array();
        }
        foreach ($data['entries'] as $entrydata) {
            $this->entries[] = new clx_form_panel_entry($form, $entrydata);
        }
    }

    function editForm_(&$form, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $table = new htmlTable($this->tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        if (strlen($this->title) > CLX_NUM_ZERO) {
            $title = dict($this->title);
        } else {
            $title = '';
        }
        if (strlen($this->caption) > CLX_NUM_ZERO) {
            $table->addCell(dict($this->caption), array(CLX_PROP_CLASS => 'caption', CLX_PROP_COLSPAN => $this->tableWidth));
        }
        foreach ($this->entries as $entry) {
            $entry->editForm($form, $record, $msg, $table);
        }
        $source = buildPanelInterface($title, '', $table->source(), CLX_NUM_ONE);
        return $source;
    }

    function viewForm_2($toolcode, $record) {
        $table = new htmlTable($this->tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        if (strlen($this->title) > CLX_NUM_ZERO) {
            $title = dict($this->title);
        } else {
            $title = '';
        }
        if (strlen($this->caption) > CLX_NUM_ZERO) {
            $table->addCell(dict($this->caption), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $this->tableWidth));
        }
        foreach ($this->entries as $entry) {
            $entry->viewForm($toolcode, $record, $table);
        }
        $source = buildPanelInterface($title, '', $table->source(), CLX_NUM_MINUS_1);
        return $source;
    }

    function checkForm(&$form, &$record, &$msg) {
        if (!empty($this->entries)) {
            foreach ($this->entries as $entry) {
                $entry->checkForm($form, $record, $msg);
            }
        }
    }

}

class clx_form_panel_normal extends clx_form_panel {

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
    }

}

class clx_form_panel_colapsable extends clx_form_panel {

    var $initial_state;
    var $code;

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        $this->code = $data['code'];
        $this->initial_state = (isset($data['initial_state'])) ? $data['initial_state'] : $_SESSION['prefs']['PANEL_STATE'];
        parent::loadArray($form, $data);
    }

    function editForm_(&$form, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $table = new htmlTable($this->tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        if (strlen($this->title) > CLX_NUM_ZERO) {
            $title = $this->title;
        } else {
            $title = $this->caption;
        }
        if (strlen($this->caption) > CLX_NUM_ZERO) {
            $table->addCell(dict($this->caption), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $this->tableWidth));
        }
        foreach ($this->entries as $entry) {
            $entry->editForm($form, $record, $msg, $table);
        }
        $source = buildPanelInterface(dict($title), $this->code, $table->source(), $this->initial_state);
        return $source;
    }

}

class clx_form_panel_dynamic extends clx_form_panel {

    var $code;

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        $this->code = $data['code'];
        parent::loadArray($form, $data);
    }

    function editForm_(&$form, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $table = new htmlTable($this->tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        if (strlen($this->caption) > CLX_NUM_ZERO) {
            $table->addCell(dict($this->caption), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => $this->tableWidth));
        }
        foreach ($this->entries as $entry) {
            $entry->editForm($form, $record, $msg, $table);
        }
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return $div->source();
    }

}

class clx_form_panel_callback extends clx_form_panel {

    var $function;

    function __construct(&$form, $data) {
        $this->loadArray($form, $data);
    }

    function loadArray(&$form, $data) {
        $this->function = $data['function'];
    }

    function editForm_(&$form, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $tool = &getTool($form->toolcode);
        $f = $this->function;
        return $tool->$f($record);
    }

}
