<?php

/**
 * Report Header
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version        $Rev: 9902 $
 * @package		Clientix.Classes.Report
 * @abstract		Abstract Class
 */
class clx_report_data {

    /**
     * Primary Title
     * @var string
     */
    public $data;
    private $style;
    private $excelCell = array();
    private $excelBlank = false;
    private $nl = false;

    /**
     * Report Header Constructor
     * @param mixed $data
     */
    function __construct($data, clx_style $style = null, $col = -1, $row = -1, $writeBlank = false, $nl = false) {
        $this->setData($data);
        $this->setExcellCell($col, $row);
        $this->excelBlank = $writeBlank;
        $this->nl = $nl;
        if ($style !== null) {
            $this->setStyle($style);
        }
    }

    /**
     * Returns if new line is requiered
     * @return type
     */
    function getNl(){
        return $this->nl;
    }
    /**
     * Sets data coordinates, in excel = cell
     * @param type $col
     * @param type $row
     */
    function setExcellCell($col, $row) {
        $this->excelCell = array($col, $row);
    }

    /**
     * Returns coordinate
     * @return array
     */
    function getExcellCell() {
        return $this->excelCell;
    }

    /**
     * Returns excell col
     * @return type
     */
    function getExcellCellCol() {
        return $this->excelCell[CLX_INIT_ZERO];
    }

    /**
     * Returns excell row
     * @return type
     */
    function getExcellCellRow() {
        return $this->excelCell[1];
    }

    function setData($text) {
        $this->data = (string) $text;
    }

    function setStyle(clx_style $style) {
        $this->style = $style;
    }

    function getStyle() {
        return $this->style;
    }

    //FORMATING
    function sourceText() {
        return strip_tags(str_replace(CLX_CHAR_HTML_NBSP, ' ', $this->style->formatData($this->data)));
    }

    function sourceHtml() {
        //temporal para probar formato de html
        $html = new htmlObject('div');
        if (!empty($this->style)) {
            $html->setProp('style', $this->style->sourceHtml());
        }
        //Check Formatting
        $html->setContents($this->style->formatData($this->data));
        return $html->source();
    }

    function sourceXml() {
        return htmlentities(htmlentities($this->style->formatData($this->data)));
    }

    function sourcePdf(&$pdf) {
        //Setting windows encoding
        $this->data = utf8_decode($this->data);
        $pdf->SetFont($this->style->getFontFace('pdf'), $this->style->getFont('pdf'), $this->style->getFontSize('pdf'));
        $pdf->Cell($this->style->getBoxWidth('pdf'), $this->style->getBoxHeight('pdf'), $this->style->formatData($this->data), $this->style->getBoxBorder('pdf'), CLX_NUM_ZERO, $this->style->getFontAligment('pdf'));
        $pdf->Ln();
    }

    function sourceMsExcel(&$ws, &$wb) {
        $a = CLX_INIT_ZERO;
        //Setting windows encoding
        $this->data = utf8_decode($this->data);
        $r = ($this->getExcellCellRow() >= 0) ? $this->getExcellCellRow() : $ws->getRow();
        $c = ($this->getExcellCellCol() >= 0) ? $this->getExcellCellCol() : $ws->getCol();
        if ($this->excelBlank === true) {
            $ws->writeBlank($r, $c, $this->getStyle()->sourceMsExcelStyle($wb));
        } else {
            $ws->write($r, $c, strip_tags(str_replace(CLX_CHAR_HTML_NBSP, ' ', $this->style->formatData($this->data, 'msexcel'))), $this->getStyle()->sourceMsExcelStyle($wb));
        }

        //setting merge cels
        $colspan = $this->getStyle()->getBoxColSpan('msexcel');
        $rowspan = $this->getStyle()->getBoxRowSpan('msexcel');
        /*if($colspan>0){
            $ws->writeBlank($r, 1, $this->getStyle()->sourceMsExcelStyle($wb));
        }*/

        if (($rowspan <> CLX_NUM_ZERO || $colspan <> CLX_NUM_ZERO)) {

            $ws->mergeCells($r, $c, ($r + $rowspan), ($c + $colspan));

            /*if ($colspan > CLX_NUM_ZERO) {
                for ($i = CLX_INIT_ZERO; $i < $colspan; $i++) {
                    $ws->nextCol();
                }
            }*/
        } else {
            //setting sizes
            if ($this->getStyle()->getBoxWidth('')>0) {
                $ws->setColumn($c, $c, $this->getStyle()->getBoxWidth(''));
            }
            if ($this->getStyle()->getBoxHeight('')>0) {
                $ws->setRow($ws->getRow(), $this->getStyle()->getBoxHeight(''));
            }
        }

        $ws->nextCol();
        if($this->getNl()){
            $ws->Ln();
        }
    }

}
