<?php

/**
 * Clientix Validation Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-09 11:22:02 -0400 (Tue, 09 Oct 2018) $
 * @version    	$Rev: 10222 $
 * @package		Clientix.Classes.Validate
 * @abstract	Abstract Class
 */
class clx_validate {

    var $data;
    var $required;
    var $something;

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        $this->data = $data;
        $this->required = (!empty($this->data['required'])) ? $this->data['required'] : CLX_NUM_ZERO;
    }

    function check($field_name, &$record, &$msg, &$form) {
        $field_value = $record[$field_name];
        //$this->something = (strlen($field_value) > CLX_NUM_ZERO);
        $this->something = (!is_null($field_value) && strlen($field_value) > CLX_NUM_ZERO);
        if (!empty($this->required) and empty($this->something)) {
            $msg[$field_name] = dict('GEN_ERR_REQUIRED_FIELD') . '<br>';
        }
    }

}

class clx_validate_number extends clx_validate {

    var $check_minimum;
    var $check_maximum;
    var $minimum;
    var $maximum;

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->check_minimum = isset($this->data['minimum']);
        if (!empty($this->check_minimum))
            $this->minimum = $this->data['minimum'];
        $this->check_maximum = isset($this->data['maximum']);
        if (!empty($this->check_maximum))
            $this->maximum = $this->data['maximum'];
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $number = floatval($field_value);
        if (!empty($this->something) or ! empty($this->required)) {
            if (!is_numeric($field_value) || (substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '-' && substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '.' && substr($field_value, CLX_NUM_ONE, CLX_NUM_ONE) !== '.' && substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) == CLX_NUM_ZERO && $number <> CLX_NUM_ZERO)) {
                if (!array_key_exists($field_name, $msg))
                    $msg[$field_name] = '';
                $msg[$field_name] .= dict('GEN_ERR_NOT_NUMERIC', $field_value) . '<br>';
            }
            if (!empty($this->check_minimum) and ( $field_value < $this->minimum)) {
                if (!array_key_exists($field_name, $msg))
                    $msg[$field_name] = '';
                $msg[$field_name] .= dict('GEN_ERR_LESS_THAN', $this->minimum) . '<br>';
            }
            if (!empty($this->check_maximum) and ( $field_value > $this->maximum)) {
                if (!array_key_exists($field_name, $msg))
                    $msg[$field_name] = '';
                $msg[$field_name] .= dict('GEN_ERR_GREATER_THAN', $this->maximum) . '<br>';
            }
        }
    }

}

class clx_validate_integer extends clx_validate_number {

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $number = intval($field_value);
        if (!empty($this->something) or ! empty($this->required)) {
            if ($number != $field_value || (substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '-' && substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '.' && substr($field_value, CLX_NUM_ONE, CLX_NUM_ONE) !== '.' && substr($field_value, CLX_NUM_ZERO, CLX_NUM_ONE) == CLX_NUM_ZERO && $number <> CLX_NUM_ZERO)) {
                $msg[$field_name] .= dict('GEN_ERR_NOT_INTEGER', $field_value);
            }
        }
    }

}

class clx_validate_string extends clx_validate {

    var $allowedchars;
    var $check_min_length;
    var $check_max_length;
    var $min_length;
    var $max_length;

    function __construct($data) {
        $this->loadArray($data);
        $this->allowedchars = (!empty($this->data['allowedchars'])) ? $this->data['allowedchars'] : '';
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->check_min_length = isset($this->data['min_length']);
        if (!empty($this->check_min_length))
            $this->min_length = $this->data['min_length'];
        $this->check_max_length = isset($this->data['max_length']);
        if (!empty($this->check_max_length))
            $this->maximum = $this->data['max_length'];
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent::check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        if (!empty($this->something) or ! empty($this->required)) {
            if ($this->check_min_length and ( strlen($field_value) < $this->min_length)) {
                $msg[$field_name] .= dict('GEN_ERR_STRING_TOO_SHORT', $this->min_length);
            }
            if (!empty($this->check_max_length) and ( strlen($field_value) > $this->max_length)) {
                $msg[$field_name] .= dict('GEN_ERR_STRING_TOO_LONG', $this->max_length);
            }
            if (strlen($this->allowedchars) > CLX_NUM_ZERO) {
                $ok = true;
                $la = strlen($field_value);
                $i = CLX_INIT_ZERO;
                while ($ok and ( $i < $la)) {
                    $ok = (strpos($this->allowedchars, $field_value[$i]) !== false);
                    $i++;
                }
                if (empty($ok))
                    $msg[$field_name] = dict('GEN_ERR_WRONG_CHARS');
            }
        }
    }

}

class clx_validate_unique extends clx_validate {

    var $tablename;
    var $fieldname;

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->tablename = $this->data['tablename'];
        $this->fieldname = $this->data['fieldname'];
    }

    function check($field_name, &$record, &$msg, &$form) {
		if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { 	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,'111-validate record'.var_Export($record,true).chr(13).chr(10));	fwrite($tta,chr(13).chr(10));		fclose($tta);}
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $crit = $this->fieldname . '=' . singleQuote($record[$field_name]);
        if (isset($record[$form->toolcode . '_id']) && !empty($form->modify)) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $crit . ' AND ID<>' . $record[$form->toolcode . '_id']);
        } else {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $crit);
        }
        if ($qty > CLX_NUM_ZERO) {
            $msg[$field_name] = dict('GEN_ERR_UNIQUE_KEY', $field_value);
        }
    }

}

class clx_validate_lookup extends clx_validate {

    var $fieldname;
    var $tablename;

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->fieldname = $this->data['fieldname'];
        $this->tablename = $this->data['tablename'];
    }

    function check($field_name, &$record, &$msg, &$form) {
		if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,'112-validate record  '.var_Export($record,true)."\n and form \n".var_Export($form,true).chr(13).chr(10));	fwrite($tta,chr(13).chr(10));		fclose($tta);}
        parent::check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        //if (strlen($field_value) > CLX_NUM_ZERO) {
        if (!is_null($field_value) && strlen($field_value) > CLX_NUM_ZERO) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->fieldname . '=' . singleQuote($field_value));
            if ($qty == CLX_NUM_ZERO) {
                $msg[$field_name] = dict('GEN_ERR_KEY_DONT_EXIST', $field_value);
            } else {
                $record[$field_name] = ClxCommon::$db->getField($this->tablename, $this->fieldname, $this->fieldname . '=' . singleQuote($field_value));
            }
        }
    }

}

class clx_validate_query extends clx_validate {

    var $query;

    function __construct($data) {
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->query = $this->data['query'];
    }

    function check($field_name, &$record, &$msg, &$form) {
		if ( isset($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { 	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,'112b-validate record and form'.var_Export($record,true).var_Export($form,true).chr(13).chr(10));	fwrite($tta,chr(13).chr(10));		fclose($tta);}
        parent:: check($field_name, $record, $msg, $form);
        $query = $this->query;
        foreach ($record as $field => $value) {
            $query = str_replace(singlequote($field), singlequote($value), $query);
        }
        $field_value = $record[$field_name];
        $rs = ClxCommon::$db->executeQuery($query);
        $r = ClxCommon::$db->getNextRecord($rs);
        //if (ClxCommon::$db->num_rows($rs) == 0) {
        if (empty($r)) {
            $msg[$field_name] = (!empty($this->data['msg'])) ? dict($this->data['msg'], $field_value) :  dict('GEN_ERR_KEY_DONT_EXIST', $field_value);
        } else {
            $record[$field_name] = $r[uppercase($field_name)];
        }
    }

}

class clx_validate_sqlobject extends clx_validate_string {

    function __construct($data) {
        $this->loadArray($data);
        $this->allowedchars = 'qwertyuioplkjhgfdsazxcvbnmQWERTYUIOPLKJHGFDSAZXCVBNM1234567890_';
    }

}

class clx_validate_codeformat extends clx_validate_string {

    function __construct($data) {
        $data['allowedchars'] = 'A#.';
        $this->loadArray($data);
    }

}

class clx_validate_codesubformat extends clx_validate_string {

    function __construct($data) {
        $data['allowedchars'] = 'A#';
        $this->loadArray($data);
    }

}

class clx_validate_code extends clx_validate_string {

    var $format;

    function __construct($data) {
        $data['allowedchars'] = 'QWERTYUIOPLKJHGFDSAZXCVBNM1234567890.';
        $this->loadArray($data);
    }

    function loadArray($data) {
        parent::loadArray($data);
        $this->format = $this->data['format'];
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $format_len = strlen($this->format);
        $value_len = strlen($field_value);
        if ($format_len != $value_len) {
            $msg[$field_name] = dict('GEN_ERR_INVALID_ENTRY');
        } else {
            $error = false;
            $done = false;
            $i = CLX_INIT_ZERO;
            while (!$done and ! $error) {
                $digit = substr($this->format, $i, CLX_NUM_ONE);
                $letter = substr($field_value, $i, CLX_NUM_ONE);
                if ($digit === 'A') {
                    $pattern = 'A-Z';
                } elseif ($digit === 'a') {
                    $pattern = 'a-z';
                } elseif ($digit === '#') {
                    $pattern = '0-9';
                } else {
                    $pattern = 'NOPATERNWILLBELIKETHIS';
                }
                $error = (preg_match('/[^' . $pattern . ']/', $letter) != CLX_NUM_ZERO);
                $i++;
                $done = ($i > $format_len);
            }
            if (!empty($error)) {
                if ($digit === 'A') {
                    $msg[$field_name] .= dict('GEN_ERR_EXPECTING_UPPERALPHA');
                } elseif ($digit === 'a') {
                    $msg[$field_name] .= dict('GEN_ERR_EXPECTING_LOWERALPHA');
                } elseif ($digit === '#') {
                    $msg[$field_name] .= dict('GEN_ERR_EXPECTING_NUMBER');
                }
            }
        }
    }

}

class clx_validate_url extends clx_validate_string {

    function __construct($data) {
        $this->loadArray($data);
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        if (substr($field_value, CLX_NUM_ZERO, CLX_NUM_7) !== 'http://') {
            $msg[$field_name] .= dict('GEN_ERR_BAD_URL');
        }
    }

}

class clx_validate_mail extends clx_validate_string {

    function __construct($data) {
        $this->loadArray($data);
    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        if (stristr($field_value, '@') === '') {
            $msg[$field_name] .= dict('GEN_ERR_EMAIL_MISSING_CHAR', '@');
        } elseif (stristr($field_value, '.') === '') {
            $msg[$field_name] .= dict('GEN_ERR_EMAIL_MISSING_CHAR', '.');
        }
    }

}

class clx_validate_password extends clx_validate_string {

    function __construct($data) {
        $data['min_length'] = CLX_NUM_SIX;
        $this->loadArray($data);
    }

}

class clx_validate_date extends clx_validate {

    function __construct($data) {

    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $timestamp = adodb_strtotime($field_value);
        if ($timestamp == -CLX_NUM_ONE) {
            $msg[$field_name] .= dict('GEN_ERR_BAD_DATE', $field_value);
        }
    }

}

class clx_validate_datetime extends clx_validate {

    function __construct($data) {

    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $timestamp = adodb_strtotime($field_value);
        if ($timestamp == -CLX_NUM_ONE) {
            $msg[$field_name] .= dict('GEN_ERR_BAD_DATETIME', $field_value);
        }
    }

}

class clx_validate_time extends clx_validate {

    function __construct($data) {

    }

    function check($field_name, &$record, &$msg, &$form) {
        parent:: check($field_name, $record, $msg, $form);
        $field_value = $record[$field_name];
        $timestamp = adodb_strtotime($field_value);
        if ($timestamp == -CLX_NUM_ONE) {
            $msg[$field_name] .= dict('GEN_ERR_BAD_TIME', $field_value);
        }
    }

}
