<?php

/**
 * General HTML class
 * Objects to create HTML tables, forms and form elements.
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-12-04 12:39:06 -0400 (ma., 04 dic. 2018) $
 * @version    	$Rev: 10405 $
 * @package		Clientix.Classes.Html
 * @abstract		Abstract Html Class
 */
class htmlForm extends htmlObject {

    var $arguments;

    function __construct() {
        // The arguments are property key/value pairs
        $this->arguments = array();
        parent::__construct(CLX_CLASS_FORM, '', array());
        $this->openingDelim = CLX_CHAR_NL;
        $this->closingDelim = CLX_CHAR_NL;
        $this->setProp(CLX_PROP_ID, 'theForm');
        $this->setProp('name', 'theForm');
        $this->setProp(CLX_PROP_ACTION, 'self');
        $this->setProp('method', 'post');
        $this->setProp('enctype', 'application/x-www-form-urlencoded');
        $this->setProp('accept-charset', 'utf-8');
        $this->setProp('onsubmit', 'return formPreventResubmit(this)');
        $qty = func_num_args();
        if ($qty > CLX_NUM_ZERO) {
            for ($i = CLX_INIT_ZERO; $i < $qty; $i = $i + CLX_NUM_TWO) {
                $key = func_get_arg($i);
                $value = func_get_arg($i + CLX_NUM_ONE);
                $this->setProp($key, $value);
            }
        }
    }

    function addArguments($array_arguments) {
        foreach ($array_arguments as $key => $value) {
            $this->addArgument($key, $value);
        }
    }

    function addArgument($key, $value) {
        $this->arguments[$key] = $value;
    }

    function source() {
        $buffer = $this->openTag();
        foreach ($this->arguments as $key => $value) {
            $c = new htmlObject('input', '', array('type' => 'hidden', 'name' => $key, 'value' => $value));
            $c->closingDelim = CLX_CHAR_NL;
            $buffer .= $c->source();
        }
        if ($this->contents <> '') {
            $buffer .= $this->contents;
            $buffer .= $this->closeTag();
        }
        $buffer .= $this->closingDelim;
        return $this->prefix . $buffer . $this->sufix;
    }

}

class htmlTable extends htmlObject {

    var $cells;
    var $cellqty;
    var $width; // cells accross
    var $height;
    var $colwidths;

    function __construct($w) {
        // The first argument is always the width (in columns) of the table
        // The rest are property key/value pairs
        parent::__construct('table', '', array());
        $this->cellqty = CLX_INIT_ZERO;
        $this->width = $w;
        $this->openingDelim = CLX_CHAR_NL;
        $this->closingDelim = CLX_CHAR_NL;
        $this->setProp('border', '0');
        $this->setProp('cellspacing', '0');
        $this->setProp('cellpadding', '0');
        $qty = func_num_args();
        if ($qty > CLX_NUM_ONE) {
            for ($i = CLX_NUM_ONE; $i < $qty; $i = $i + CLX_NUM_TWO) {
                $key = func_get_arg($i);
                $value = func_get_arg($i + CLX_NUM_ONE);
                $this->setProp($key, $value);
            }
        }
    }

    function addCell($value, $propiedades) {

        // First argument = contents of the cell
        // The rest are property key/value pairs
        $c = new htmlCell('', array());
		
        $c->setContents($value);
		if (gettype($propiedades)=='array'){
			foreach ($propiedades as $key => $value) {
				$c->setProp($key, $value);
			}
		}
        $this->cellqty++;
        $this->cells[$this->cellqty] = $c;
    }

    function addCells($class, $propiedades) {
	$max=count($propiedades);
        for ($t=0;$t<$max;$t++) {
	    $this->addCell($propiedades[$t], array(CLX_PROP_CLASS => $class));
        }
    }

    function addData($prompt, $data, $promptclass = CLX_CLASS_PROMPT, $dataclass = CLX_CLASS_DATA) {
        $this->addCell($prompt, array(CLX_PROP_CLASS => $promptclass));
        $this->addCell($data, array(CLX_PROP_CLASS => $dataclass));
    }

    function addField($msg, $prompt, $name, $default, $length, $maxlength, $required = false) {
        if (!empty($msg[$name])) {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $c = new htmlField($name, $default, $length, $maxlength);
        $this->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addDate($msg, $prompt, $name, $default, $length, $maxlength, $required = false, $include_time = false) {
        if (!empty($msg[$name])) {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $c = new htmlField($name, $default, $length, $maxlength);
		$format_javascript = $_SESSION['prefs']['DATE_FORMAT'];
		$format_javascript = str_replace('d', 'dd', $format_javascript);
		$format_javascript = str_replace('m', 'mm', $format_javascript);
		$format_javascript = str_replace('Y', 'yy', $format_javascript);

        $picker = '<script type="text/javascript">
            			    $(function() {
            			    	var pickOpts = {
            			    	 dateFormat: "'.$format_javascript.'",
            			    	 showTime: true,
  							     stepMinutes: 1,
         						 stepHours: 1,
         						 altTimeField: "",
            			    	 duration: "",
                                 showTime: ' . (($include_time) ? 'true' : 'false') . ',
                                 constrainInput: false,
                                 time24h: true,
                                 changeMonth: true,
								 changeYear: true,
                                 showOn: "button",
                                 buttonImage: "./' . kPathToSkins . $_SESSION['skin'] . '/images/calendar/cal.gif",
                                 buttonImageOnly: true
            			    	};
                        		$("#' . $name . '").datepicker(pickOpts);
                        	});
                        	</script>';
        $picker = CLX_CHAR_NL . $picker;
        $this->addCell($c->source() . $picker, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addText($msg, $prompt, $name, $default, $rows, $cols, $required = false) {
		//if ( isset($_SESSION["_dEBUG"]) ) { $te=debug_backtrace();$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."html- hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));for ($t=0;$t<18;$t++) {if (isset($te[$t]["file"])) {fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}fwrite($tta,chr(13).chr(10));	fclose($tta);}}
	
        if (isset($msg[$name])) {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $c = new htmlTextArea($name, $default, $rows, $cols);
        $this->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addYesNo($prompt, $name, $default, $width, $required = false) {
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED, CLX_PROP_WIDTH => $width));
        $pop = new htmlSelect($name);
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($default == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($default == CLX_NUM_ZERO));
        $this->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addYesNoMaybe($prompt, $name, $default, $required = false) {
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $pop = new htmlSelect($name);
        $pop->addItem(dict('GEN_LBL_YES'), '1', ($default == CLX_NUM_ONE));
        $pop->addItem(dict('GEN_LBL_NO'), '0', ($default == CLX_NUM_ZERO));
        $pop->addItem('', '-1', ($default == -CLX_NUM_ONE));
        $this->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addPassword($msg, $prompt, $name, $default, $lenght, $maxlenght, $required = false) {
        if (array_key_exists($name, $msg) && $msg[$name] <> '') {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $c = new htmlObject('input', '', array('type' => 'password', 'name' => $name, 'value' => $default, 'size' => $lenght, 'maxlenght' => $maxlenght));
        $this->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addLookup($msg, $prompt, $name, $default, $toolcode, $lenght, $maxlenght, $required = false) {
        if (array_key_exists($name, $msg) && $msg[$name] <> '') {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $tool = getTool($toolcode);
        $look = $tool->chooseRecord($name, $default, array(), $lenght, $maxlenght);
        $this->addCell($look, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    /**
     * Builds html list
     * @param string $prompt
     * @param string $name
     * @param string $default
     * @param array $list_items
     * @param bool $required
     */
    function addList($prompt, $name, $default, $list_items, $required = false) {
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $pop = new htmlSelect($name);
        foreach ($list_items as $key => $val) {
            $pop->addItem(dict($val), $key, ($default == $key));
        }
        $this->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addFile($msg, $prompt, $name, $default, $required = false) {
        if (array_key_exists($name, $msg) && $msg[$name] <> '') {
            $this->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $this->width));
        }
        $this->addCell($prompt, array(CLX_PROP_CLASS => (!$required) ? CLX_CLASS_PROMPT : CLX_CLASS_PROMPTREQUIRED));
        $c = new htmlObject('input', '', array('type' => 'file', 'name' => $name, 'value' => $default));
        $this->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
    }

    function addSeparator($class, $caption = false, $colspan = false) {
        if (empty($caption)) {
            $caption = CLX_CHAR_HTML_NBSP;
        }
        if (empty($colspan)) {
            $colspan = $this->width;
        }
        $this->addCell($caption, array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => $colspan));
    }

    function source() {
        $contenidoHTML = '';
        $buffer = parent::openTag();
        $y = CLX_NUM_ONE;
        $this->height = CLX_NUM_ONE;
        $this->colwidths[CLX_NUM_ONE] = $this->width;
        $x = $this->colwidths[$y];
        $opencol = true;
        for ($i = CLX_NUM_ONE; $i <= $this->cellqty; $i++) {
            if (!empty($opencol)) {
                $buffer .= '<tr>';
                $opencol = false;
            }
            $buffer .= $this->cells[$i]->source();
            $x -= $this->cells[$i]->getProp(CLX_PROP_COLSPAN);
            if ($this->cells[$i]->getProp('rowspan') > CLX_NUM_ONE) {
                for ($j = $y; $j <= ($y + $this->cells[$i]->getProp('rowspan') - CLX_NUM_ONE); $j++) {
                    if ($j > $this->height) {
                        $this->colwidths[$j] = $this->width;
                        $this->height++;
                    }
                    $this->colwidths[$j] -= $this->cells[$i]->getProp(CLX_PROP_COLSPAN);
                }
            }
            if ($x <= CLX_NUM_ZERO) {
                $buffer .= '</tr>' . CLX_CHAR_NL;
                $y++;
                if ($y > $this->height) {
                    $this->colwidths[$y] = $this->width;
                    $this->height++;
                }
                $x = $this->colwidths[$y];
                $opencol = true;
            }
        }
        if (empty($opencol)) { // This means that you have orphan cells in the last row
            $contenidoHTML.= '<!--This table has orphan rows!<br>-->';
            $buffer .= '</tr>' . CLX_CHAR_NL;
        }
        $buffer .= parent::closeTag() . $this->closingDelim;
        echo $contenidoHTML;
        return $buffer;
    }

}

class htmlCell extends htmlObject {

    function __construct($contenido, $argumentos = null) {
        // The first argument if present is the contents of the cell
        // The rest are key/value pairs
        parent::__construct('td', '', array());
        $this->setProp(CLX_PROP_COLSPAN, '1');
        $this->setProp('rowspan', '1');
        $this->setProp('valign', 'top');
        $this->contents = $contenido;
        foreach ($argumentos as $key => $value) {
            $this->setProp($key, $value);
        }
    }

}

class htmlSelect extends htmlObject {

    var $items;
    var $itemqty;

    function __construct($name) {
        parent::__construct('select', '', array('name' => $name, CLX_PROP_ID => $name));
        $this->items = array(); // array of htmlSelectItems
        $this->itemqty = CLX_INIT_ZERO;
    }
	function setProp($key, $value) {
        $this->properties[$key] = $value;
    }
    function addItem($prompt, $value, $selected) {if ( isset($_SESSION["_dEBUG"]) && $_SESSION["_dEBUG"]==99 ) {$posx=0;$pos=1629;$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");$te=debug_backtrace(); for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."_html:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}}fclose($tta);}
		$this->items[$this->itemqty++] = new htmlSelectItem($prompt, $value, $selected);
    }

    function addItems($items) {
        foreach ($items as $key => $value) {
            $this->items[$this->itemqty++] = new htmlSelectItem($value, $key, false);
        }
    }

    function source() {
        $buffer = $this->openTag();
        for ($i = CLX_INIT_ZERO; $i < $this->itemqty; $i++) {
            $this->contents .= $this->items[$i]->source();
        }
        $buffer .= $this->contents;
        $buffer .= $this->closeTag();
        $buffer .= $this->closingDelim;
		return $this->prefix . $buffer . $this->sufix;
	}

}

class htmlSelectItem extends htmlObject {

    function __construct($prompt, $value, $selected) {
        if (!strpos($prompt, CLX_CHAR_SINGLE_QUOTE) === false) {
            $prompt = htmlspecialchars($prompt);
        }
        parent::__construct('option', $prompt, array('label' => $prompt, 'value' => $value));
        if ($selected === true) {
            $this->setProp('selected', '');
        }
    }

}

class htmlImage extends htmlObject {

    function __construct($url, $title) {
        parent::__construct('img', '', array('src' => $url, CLX_CLASS_TITLE => $title, 'border' => '0'));
    }

}

class htmlFieldHidden extends htmlObject {

    function __construct($name, $default) {
        parent::__construct('input', '', array('type' => 'hidden', 'name' => $name, 'value' => htmlspecialchars($default)));
    }

}

class htmlField extends htmlObject {

    function __construct($name, $default, $length, $maxlength) {
        parent::__construct('input', '', array('type' => 'text', 'name' => $name, CLX_PROP_ID => str_replace("[","",str_replace("]","", $name)), 'value' => htmlspecialchars($default), 'size' => $length, 'maxlength' => $maxlength));
    }

}

class htmlPassword extends htmlObject {

    function __construct($name, $default, $length, $maxlength) {
        parent::__construct('input', '', array('type' => 'password', 'name' => $name, CLX_PROP_ID => str_replace("[","",str_replace("]","", $name)), 'value' => $default, 'size' => $length, 'maxlength' => $maxlength));
    }

}

class htmlTextArea extends htmlObject {

    function __construct($name, $default, $rows, $cols) {
        parent::__construct('textarea', $default, array('name' => $name, CLX_PROP_ID => str_replace("[","",str_replace("]","", $name)), 'rows' => $rows, 'cols' => $cols));
        $this->forceclose = true;
    }

}

class htmlAnchor extends htmlObject {

    function __construct($contents, $url) {
        parent::__construct('a', $contents, array('href' => $url));
    }

}

class htmlButton extends htmlObject {

    function __construct($name) {
		
		parent::__construct('input', '', array('type' => 'button', 'name' => $name, CLX_PROP_ID => $name, 'value' => $name, CLX_PROP_CLASS => 'button'));
		
        $qty = func_num_args();
        if ($qty > CLX_NUM_ONE) {
            for ($i = CLX_NUM_ONE; $i < $qty; $i = $i + CLX_NUM_TWO) {
                $key = func_get_arg($i);
                $value = func_get_arg($i + CLX_NUM_ONE);

                $this->setProp($key, $value);
            }
        }

    }

    /**
     * Creates button and return object of type selected
     * @param type $type
     * @param type $name
     * @param type $properties
     * @return \htmlButton
     */
    static function getButton($type, $name, $properties) {
		$button = null;
		switch ($type) {
            case 'submit':
                $button = new htmlFormSubmit($name, $properties['action'], $properties['form']);
                if (!empty($properties['id'])) {
                    $button->setProp('id', $properties['id']);
					}
                break;
            case 'cancel':
                $button = new htmlButtonCancel($name, (!empty($properties['anchor'])) ? $properties['anchor'] : CLX_CHAR_EMPTY_STRING, (!empty($properties['action'])) ? $properties['action'] : CLX_CHAR_EMPTY_STRING);
                break;
            case 'goto':
                $button = new htmlButtonGoto($name, $properties['url'], (!empty($properties['anchor'])) ? $properties['anchor'] : CLX_CHAR_EMPTY_STRING);
                break;
            default:
                $button = new htmlButton($name);
				$button->setProp('deb', $_SESSION["ADICT_$name"].'('.$fi.'_'.$li.')');
                break;
        }
        return $button;
    }

}

class htmlButtonGotoPopNew extends htmlButton {
    function __construct($name, $url, $anchor = '') {
        parent::__construct($name, 'onclick', 'javascript:openWindow(\'\', '. singleQuote($url) . ',700,500);');
    }

}


class htmlButtonGoto extends htmlButton {
    function __construct($name, $url, $anchor = '') {
        parent::__construct($name, 'onclick', 'javascript:window.location.href=clearUriAnchor(' . singleQuote($url) . ');');
    }
}




class htmlButtonCancel extends htmlButtonGoto {

    function __construct($title = '', $anchor = '', $action = 'tool.php') {
        if (!isset($title) || strlen($title) == CLX_NUM_ZERO) {
            $title = dict('GEN_LBL_CANCEL');
        }
        $url = $action . '?pop=1&relfm=' . ((array_key_exists('relfm', UserRequest::$request)) ? UserRequest::$request['relfm'] : '');
        parent::__construct($title, $url, $anchor);
    }

}

class htmlButtonCloseWindow extends htmlButton {

    function __construct($title) {
        $url = "parent.newDivw[0].dialog('close');";
        parent::__construct($title, 'onclick', $url);
    }

}

class htmlButtonReturn extends htmlButtonGoto {

    function __construct($title = '', $anchor = '', $target = 'tool.php') {
        if (strlen($title) == CLX_NUM_ZERO)
            $title = 'GEN_LBL_RETURN';
        $url = $target . '?pop=2&relfm=' . UserRequest::$request['relfm'];
        parent::__construct(dict($title), $url, $anchor);
    }

}

class htmlImageButton extends htmlObject {

    function __construct($image, $url) {
        $qty = func_num_args();
        $title = ($qty == CLX_NUM_THREE) ? func_get_arg(CLX_NUM_TWO) : $title = $url;
        $path = (strpos($image, '/') === false) ? 'images/buttons/' . $image . '.gif' : $image;
        parent::__construct('a', '<img border="0" src="' . $path . '">', array('href' => $url, 'border' => '0', CLX_CLASS_TITLE => $title));
    }

}

class htmlSubmit extends htmlObject {

    function __construct($value) {
		parent::__construct('input', '', array('type' => 'submit', 'name' => 'submit1', 'value' => $value, CLX_PROP_CLASS => 'button', CLX_PROP_ID => 'submit1'));
		
	}

}

class htmlFormSubmit extends htmlSubmit {

    function __construct($title, $action, $form) {
        if (strlen($title) === CLX_NUM_ZERO) {
            $title = $action;
        }
        parent::__construct($title);
        if(!empty($action)){
            $this->setProp('onclick', 'setFormAction(' . singleQuote($form) . ',' . singleQuote($action) . ')');
        }
    }

}

class htmlCheckbox extends htmlObject {
    function __construct($name, $checked, $value = CLX_NUM_ONE) {	if ( isset($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+"); 	fwrite($tta,chr(13).chr(10)."1198-hora:".date("h:i:s d/m/Y").chr(13).chr(10));	$te=debug_backtrace(); for ($t=0;$t<12;$t++) {	if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}fclose($tta);}	
        parent::__construct('input', '', array('type' => 'checkbox', 'name' => $name, CLX_PROP_ID => str_replace("[","",str_replace("]","", $name)), 'value' => $value));
        if (!empty($checked)) {
            $this->setProp('checked', '');
        }
    }

}

class htmlRadioButton extends htmlObject {

    function __construct($name, $checked, $value) {
        parent::__construct('input', '', array('type' => 'radio', 'name' => $name, CLX_PROP_ID => str_replace("[","",str_replace("]","", $name)), 'value' => $value));
        if ($checked === true) {
            $this->setProp('checked', '');
        }
    }

}

// <div id='DOCS' style='visibility:hidden;display:none;'>;
class htmlDiv extends htmlObject {

    function __construct($id, $style, $contents) {
        parent::__construct('div', $contents, array(CLX_PROP_ID => $id));
        if (!empty($style)) {
            $this->setProp('style', $style);
        }
    }

}

class htmlDivTable extends htmlObject {

    var $cells;
    var $cellqty;
    var $width; // cells accross
    var $height;
    var $colwidths;

    function __construct($w, $propiedades) {
        parent::__construct('div', '', array());
        $this->cellqty = CLX_INIT_ZERO;
        $this->width = $w;
        $this->openingDelim = CLX_CHAR_NL;
        $this->closingDelim = CLX_CHAR_NL;
        foreach ($propiedades as $key => $value) {
            $this->setProp($key, $value);
        }
    }

    function addCell($value, $propiedades
    ) {
        // First argument = contents of the cell
        // The rest are property key/value pairs
        $c = new htmlObject('div', '', array());
        $c->setContents($value);
        foreach ($propiedades as $key => $value) {
            $c->setProp($key, $value);
        }
        $this->cellqty++;
        $this->cells[$this->cellqty] = $c;
    }

    function addCells($class, $propiedades) {
        foreach ($propiedades as $value) {
            $this->addCell($value, array(CLX_PROP_CLASS => $class));
        }
    }

    function addData($prompt, $data, $promptclass = CLX_CLASS_PROMPT, $dataclass = CLX_CLASS_DATA) {
        $this->addCell($prompt, array(CLX_PROP_CLASS => $promptclass));
        $this->addCell($data, array(CLX_PROP_CLASS => $dataclass));
    }

    function source() {
        $buffer = parent::openTag();
        for ($i = CLX_NUM_ONE; $i <= $this->cellqty; $i++) {
            $buffer .= $this->cells[$i]->source();
        }
        $buffer .= parent::closeTag() . $this->closingDelim;
		return $this->prefix . $buffer . $this->sufix;
    }

}

class htmlSpan extends htmlObject {

    function __construct($id, $style, $contents) {
        parent::__construct('span', $contents, array(CLX_PROP_ID => $id));
        if (!empty($style)) {
            $this->setProp('style', $style);
        }
    }

}

class htmlObject {

    var $properties;
    var $contents;
    var $tag;
    var $openingDelim;
    var $closingDelim;
    var $forceclose;
    var $prefix = '';
    var $sufix = '';

    function __construct($tagname, $default = '', $propiedades = array()) {
        $this->tag = $tagname;
        $this->contents = '';
        $this->openingDelim = '';
        $this->closingDelim = '';
        $this->forceclose = false;
        $this->properties = array();
        $this->contents = $default;
		foreach ($propiedades as $key => $value) {
            $this->setProp($key, $value);
        }
    }

    function setProp($key, $value) {
        $this->properties[$key] = $value;
    }

    /**
     * Set multiple properties from array
     * @param type $array_prop
     */
    public function setProps($array_prop) {
        foreach ($array_prop as $key => $value) {
            $this->setProp($key, $value);
        }
    }

    function getProp($key) {
        return $this->properties[$key];
    }

    function setContents($value) {
        $this->contents = $value;
    }

    function addContents($value) {
        $this->contents .= $value;
    }

    function getContents() {
        return $this->contents;
    }

    function openTag() {
        $buffer = '<' . $this->tag;
        foreach ($this->properties as $key => $value) {
            if (strlen($value) != CLX_NUM_ZERO) {
                $buffer .= ' ' . $key . '=';
                $buffer .= (is_numeric($value)) ? $value : quote($value);
            } else {
                $buffer .= ' ' . $key . '=' . quote('');
            }
        }
        $buffer .= '>' . $this->openingDelim;
        return $buffer;
    }

    function closeTag() {
        return '</' . $this->tag . '>' . CLX_CHAR_NL;
    }

    function addPrefix($value) {
        $this->prefix .= $value;
    }

    function addSufix($value) {
        $this->sufix .= $value;
    }

    function source() {
        $buffer = $this->openTag();
        if (($this->contents != '') or ($this->forceclose)) {
			if (is_array($this->contents)) {
				$this->contents=implode('',$this->contents);
			}
            $buffer .= $this->contents;
            $buffer .= $this->closeTag();
        }
        $buffer .= $this->closingDelim;
			return $this->prefix . $buffer . $this->sufix;
		
		
    }

    function dump() {
        print $this->source();
    }

}
