<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-11-19 15:19:17 -0500 (Thu, 19 Nov 2015) $
 * @version        $Rev: 7521 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class fidu_accounting {

    var $code = 'FIAC';
    var $modulecode; // Module Code
    var $ficoname;  // Contracts
    var $fiainame;  // Instructions
    var $fipaname;  // Participants
    var $fiatname;  // Contract Transactions
    var $fitbname;  // Transactions Table
    var $fipdname;  // Participant Distribution
    var $fidaname;  // Asset Participant Distribution
    var $procname;  // Processes
    var $fiasname;  // Assets
    var $glacname;  // Gl Accounts
    var $glagname;  // Gl Account Groups
    var $jehdname;  // Journal Headers
    var $jelnname;  // Journal Lines
    var $acclname;  // Accounting Closings
    var $sdate;
    var $edate;
    var $summary;   // Summary
    var $balances = array(); // gj code balances
    var $keepLog = true;
    var $debug = false;
    var $datestamp = '';
    var $contents = '';
    var $nolink = false;

    /**
     * Constructor
     */
    function _construct() {
        if (!empty($this->debug))
            echoLog('Financial Balance Starting <br>');
        $this->datestamp = date('YmdHis');
        if (!empty(UserRequest::$request[$this->code . '_SDATE'])) {
            $this->sdate = UserRequest::$request[$this->code . '_SDATE'];
            $this->edate = UserRequest::$request[$this->code . '_EDATE'];
        } else {
            $this->sdate = xFormat::getNow('Y-m-01');
            $this->edate = xFormat::getNow('Y-m-d');
        }
        $this->modulecode = 'FIDU';
        $this->ficoname = tableName('FICO');
        $this->fiainame = tableName('FIAI');
        $this->fipaname = tableName('FIPA');
        $this->fiatname = tableName('FIAT');
        $this->fitbname = tableName('FITB');
        $this->fipdname = tableName('FIPD');
        $this->fidaname = tableName('FIDA');
        $this->procname = tableName('PROC');
        $this->fiasname = tableName('FIAS');
        $this->acclname = tableName('ACCL');
        $this->glacname = tableName('GLAC');
        $this->glagname = tableName('GLAG');
        $this->jehdname = tableName('JEHD');
        $this->jelnname = tableName('JELN');
    }

    /**
     * Change start date
     */
    function sDate($date) {
        $this->sdate = $date;
    }

// End method
    /**
     * Change end date
     */
    function eDate($date) {
        $this->edate = $date;
    }

// End method
    /**
     * Validate GL Account
     */
    function glValidate($record, $msg) {
        $form = '';
        if (!empty($record['GLAC_GLGROUP_CODE'])) {
            $format = ClxCommon::$db->getField($this->glagname, 'IDCODE_FORMAT', 'IDCODE = ' . singleQuote($record['GLAC_GLGROUP_CODE']));
            if (!empty($format)) {
                $r = array('format' => $format);
                $a = new clx_validate_code($r);
                $a->check('GLAC_IDCODE', $record, $msg, $form);
            } else {
                $msg['GLAC_GLGROUP_CODE'] = dict('GEN_LBL_REQUIRED');
            }
        }
        return $msg;
    }

// End method
    /**
     * Builds GL Chart of Account Trees
     */
    function rollup($contract_code = '', $gl_types_in = '') {
        if ($gl_types_in === '')
            $gl_types_in = '' . singleQuote('AS') . ',' . singleQuote('LI') . ',' . singleQuote('EQ') . '';
        $cc = (strlen($contract_code) > CLX_NUM_ZERO) ? ' AND JEHD.CONTRACT_CODE IN (' . singleQuote($contract_code) . ')' : CLX_CHAR_EMPTY_STRING;
        $this->balances = array();
        $results = ClxCommon::$db->getRecords($this->glacname, 'GL_TYPE IN (' . $gl_types_in . ') ORDER BY IDCODE DESC');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            if (!array_key_exists($result['IDCODE'], $this->balances)) {
                $this->balances[$result['IDCODE']]['NAME'] = $result['IDNAME'];
                $this->balances[$result['IDCODE']]['TYPE'] = $result['GL_TYPE'];
                $this->balances[$result['IDCODE']]['BALANCE'] = CLX_INIT_ZERO;
            }
            if (empty($this->balances[$result['IDCODE']]['NAME'])) {
                $this->balances[$result['IDCODE']]['NAME'] = $result['IDNAME'];
                $this->balances[$result['IDCODE']]['TYPE'] = $result['GL_TYPE'];
            }
            if (!empty($this->nolink))
                $dates = ' AND (JEHD.EFFTDATE > ' . singleQuote($this->sdate) . ' AND JEHD.EFFTDATE <= ' . singleQuote($this->edate) . ') ';
            else
                $dates = '';
            $sql = 'SELECT JEHD.CONTRACT_CODE, JELN.CREDIT_AMT, JELN.DEBIT_AMT, JELN.GLACCOUNTCODE FROM ' . $this->jehdname . ' JEHD,' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JELN.GLACCOUNTCODE = ' . singleQuote($result['IDCODE']) . ' ' . $cc . ' ' . $dates;
            $results2 = ClxCommon::$db->executeQuery($sql);
            while (($result2 = ClxCommon::$db->getNextRecord($results2))) {
                $sum = $result2['DEBIT_AMT'] - $result2['CREDIT_AMT'];
                $this->balances[$result['IDCODE']]['BALANCE'] += $sum;
            }
            if (!empty($result['ACPARENT'])) {
                if (!array_key_exists($result['ACPARENT'], $this->balances)) {
                    $this->balances[$result['ACPARENT']]['BALANCE'] = CLX_INIT_ZERO;
                    $this->balances[$result['ACPARENT']]['NAME'] = '';
                    $this->balances[$result['ACPARENT']]['TYPE'] = '';
                }
                $this->balances[$result['ACPARENT']]['BALANCE'] += $this->balances[$result['IDCODE']]['BALANCE'];
            }
        }
        unset($results, $results2, $result, $result2, $gl_types_in, $contract);
    }

// End method
    function dump() {
        print_r($this->balances);
    }

    function dailyClosing() {

    }

    /**
     * Accounting Closing
     */
    function accountingClosing() {
        if (!empty($this->debug))
            echoLog('Closing Process Started<br>');
        $contracts = ClxCommon::$db->executeQuery('SELECT IDCODE FROM ' . $this->ficoname . ' WHERE CONTRACT_STATUS = ' . singleQuote('ACTI') . '');
        while (($contract = ClxCommon::$db->getNextRecord($contracts))) {
            $this->contents = '';
            $count = ClxCommon::$db->oneExistsRecords($this->acclname, 'CONTRACT_CODE = ' . singleQuote($contract['IDCODE']) . ' AND SDATE = ' . singleQuote($this->sdate) . ' AND EDATE = ' . singleQuote($this->edate));
            if ($count > CLX_NUM_ZERO)
                continue;
            if (!empty($this->debug))
                echoLog('Processing Contract:' . $contract['IDCODE'] . '<br>');
            $this->nolink = true;
            $this->closingBuild($contract['IDCODE']);
            $this->closingRegiter($contract['IDCODE']);
            break;
        }
        if (!empty($this->debug))
            echoLog('Closing Process Ended<br>');
    }

// End method
    /**
     * Balance Contents
     */
    function closingBuild($contract_code) {
        //Preferences
        $this->summary = CLX_NUM_ONE;

        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_ALIGN => 'center'));

        $tableFrame->addcell('<hr size="1" noshade>', array());
        $a = '<h3><IMG SRC="./data/files/banklogo_sml.jpg" align = "left" vspace=3 hspace =10></h3>';
        $a .= '<b>' . xFormat::date($this->sdate, $_SESSION['prefs']['DATE_FORMAT']) . ' ' . xFormat::date($this->edate, 'd - M - Y') . '</b>';
        $tableFrame->addcell($a, array());
        $tableFrame->addcell('<hr size="1" noshade>', array());
        // Load balances
        $this->rollup($contract_code);
        $tableFrame->addcell($this->buildbalances());
        $tableFrame->addcell('<hr size="1" noshade>', array());
        $this->contents = $tableFrame->source();
    }

    /**
     * Accounting clousing Register
     */
    function closingRegiter($contract_code) {
        $rec = array();
        $rec['IDCODE'] = $contract_code;
        $rec['IDNAME'] = $contract_code . ' ' . $this->sdate . ' - ' . $this->edate;
        $rec['CONTRACT_CODE'] = $contract_code;
        $rec['SDATE'] = $this->sdate;
        $rec['EDATE'] = $this->edate;
        $rec['DESCRIPTION'] = $this->contents;
        ClxCommon::$db->createRecord($this->acclname, $rec, false);
        $this->contents = '';
    }

// Resultados Financieros
    /**
     * Builds GL Chart of Account Trees for results
     */
    function rollup_results($contract_code = '', $gl_types_in = '') {
        if ($gl_types_in === '')
            $gl_types_in = '' . singleQuote('IN') . ',' . singleQuote('EX') . '';
        $cc = (strlen($contract_code) > CLX_NUM_ZERO) ? ' AND JEHD.CONTRACT_CODE = ' . singleQuote($contract_code) : CLX_CHAR_EMPTY_STRING;
        $this->balances = array();
        $results = ClxCommon::$db->getRecords($this->glacname, 'GL_TYPE IN (' . $gl_types_in . ') ORDER BY IDCODE DESC');
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            if (!array_key_exists($result['IDCODE'], $this->balances)) {
                $this->balances[$result['IDCODE']]['NAME'] = $result['IDNAME'];
                $this->balances[$result['IDCODE']]['TYPE'] = $result['GL_TYPE'];
                $this->balances[$result['IDCODE']]['BALANCE'] = CLX_INIT_ZERO;
            }
            if (empty($this->balances[$result['IDCODE']]['NAME'])) {
                $this->balances[$result['IDCODE']]['NAME'] = $result['IDNAME'];
                $this->balances[$result['IDCODE']]['TYPE'] = $result['GL_TYPE'];
            }
            if (!empty($this->nolink))
                $dates = ' AND (JEHD.EFFTDATE > ' . singleQuote($this->sdate) . ' AND JEHD.EFFTDATE <= ' . singleQuote($this->edate) . ') ';
            else
                $dates = '';
            $sql = 'SELECT JEHD.CONTRACT_CODE, JELN.CREDIT_AMT, JELN.DEBIT_AMT, JELN.GLACCOUNTCODE FROM ' . $this->jehdname . ' JEHD ,' . $this->jelnname . ' JELN  WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JELN.GLACCOUNTCODE = ' . singleQuote($result['IDCODE']) . ' ' . $cc . ' ' . $dates;
            $results2 = ClxCommon::$db->executeQuery($sql);
            while (($result2 = ClxCommon::$db->getNextRecord($results2))) {
                $sum = $result2['DEBIT_AMT'] - $result2['CREDIT_AMT'];
                $this->balances[$result['IDCODE']]['BALANCE'] += $sum;
            }
            if (!empty($result['ACPARENT'])) {
                if (!array_key_exists($result['ACPARENT'], $this->balances)) {
                    $this->balances[$result['ACPARENT']]['BALANCE'] = CLX_INIT_ZERO;
                    $this->balances[$result['ACPARENT']]['NAME'] = '';
                    $this->balances[$result['ACPARENT']]['TYPE'] = '';
                }
                $this->balances[$result['ACPARENT']]['BALANCE'] += $this->balances[$result['IDCODE']]['BALANCE'];
            }
        }
        unset($results, $results2, $result, $result2, $gl_types_in, $contract);
    }

// End method
    /**
     * Resultados Financieros
     */
    function build_results() {
        $msg = array();
        $this->contents = '';
        if (!empty(UserRequest::$request['idlist'])) {
            $contract_code = ClxCommon::$db->getField($this->ficoname, 'IDCODE', 'ID=' . UserRequest::$request['idlist']);
        } else {
            $contract_code = '';
        }
        if (empty(UserRequest::$request['confirm']) || !$this->checkDialog($msg)) {
            $this->print_dialog_results($msg);
        } elseif (!empty(UserRequest::$request['gl'])) { // Display Transactions
            $this->contents = $this->getTransactions(UserRequest::$request['gl'], $contract_code);
        } else {
            //Preferences
            $this->edate = UserRequest::$request[$this->code . '_EDATE'];
            $this->summary = !empty(UserRequest::$request[$this->code . '_DETAIL']);
            $tableFrame = new htmlTable(CLX_WIDTH_ONE);
            $tableFrame->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_ALIGN => 'center'));

            $tableFrame->addcell('<hr size="1" noshade>', array());
            $a = '';
            if ($contract_code === '') {
                $a .= '<b>' . dict('FIDU_CODE_CONSRESULTS') . '<br><font size=3>' . xFormat::date($this->edate, $_SESSION['prefs']['DATE_FORMAT']) . '</font></b>';
            } else {
                $a .= dict('FIDU_CODE_CONTRESULTS') . '<br><font size=3>' . xFormat::date($this->edate, $_SESSION['prefs']['DATE_FORMAT']) . '</font><br>';
                $a .= '<br>' . dict('FIDU_CODE_ACCOUNTNO') . ':<i>' . $contract_code . '</b>';
            }
            $a = '<h3><IMG SRC="./data/files/banklogo_sml.jpg" align = "left" vspace=3 hspace =10><center>' . $a . '<center></h3>';
            $tableFrame->addcell($a, array());
            $tableFrame->addcell(CLX_CHAR_HTML_NBSP, array());
            // Load balances
            $this->rollup_results($contract_code);
            $tableFrame->addcell($this->buildbalances_results(), array());
            $tableFrame->addcell('<hr size="1" noshade>', array());
            $this->contents = $tableFrame->source();
        }
    }

// End method
    /**
     * Build user interfase balances resutls
     */
    function buildbalances_results() {
        $ini = CLX_NUM_100000000000; //Level of characters.
        $summary_lbl = ($this->summary) ? CLX_NUM_FIVE : CLX_NUM_HUNDRED;
        ksort($this->balances);

        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $tincome = CLX_INIT_ZERO; // Total Income
        $texpense = CLX_INIT_ZERO; // Total expenses
        $i = CLX_INIT_ZERO;
        foreach ($this->balances as $key => $val) {
            $char = strlen(str_replace('.', '', ($key / $ini)));
            $i++;
            //Reporting Label
            if (!$char > $summary_lbl === true)
                continue;
            $tkey = substr($key, CLX_NUM_ZERO, $summary_lbl);
            if ($char === '1' && stristr($tkey, '50000') !== false) {
                $tincome = $val['BALANCE'];
                //e cho '<hr><hr>Income' . $tkey.'<hr><hr>';
            }
            if ($char === '1' && stristr($tkey, '60000') !== false) {
                $table->addcell('<hr size="1" noshade>&nbsp;', array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
                $table->addcell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
                $texpense = $val['BALANCE'];
                //e cho '<hr><hr>Expense' . $tkey.'<hr><hr>';
            }
            $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
            if ($char <= CLX_NUM_TWO)
                $table->addcell('<hr size="1" noshade>', array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
            if ($char <= CLX_NUM_TWO) {
                $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' <b>' . $val['NAME'] . '</b>', array(CLX_PROP_CLASS => $class));
            } else {
                $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' ' . $val['NAME'], array(CLX_PROP_CLASS => $class));
            }
            $table->addcell(xFormat::number(abs($val['BALANCE'])), array(CLX_PROP_CLASS => $class));
            $table->addcell(CLX_CHAR_HTML_NBSP, CLX_PROP_CLASS, $class);
            if ($char == CLX_NUM_TWO)
                $table->addcell('<hr size="1" noshade>', array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
        }
        $table->addcell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
        $table->addcell('' . dict('FICO_CODE_TOTAL') . ': <b>' . xFormat::number(($tincome + $texpense)) . '</b>', array(CLX_PROP_COLSPAN => CLX_NUM_THREE, CLX_PROP_ALIGN => CLX_PROP_RIGHT));
        return $table->source();
    }

// End method
    /**
     * Prompt period && Client to display on financial results
     */
    function print_dialog_results($msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'pop.php', CLX_PROP_ID, 'theForm', 'name', 'theForm');
        $form->addArgument(CLX_PROP_TO, UserRequest::$request[CLX_PROP_TO]);
        $form->addArgument('ac', UserRequest::$request['ac']);
        $form->addArgument('idlist', UserRequest::$request['idlist']);
        $form->addArgument('wind', '1');
        $form->addArgument('confirm', '1');

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));


        $table->addCell(dict('FIDU_CODE_CONSRESULTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $field = getToolObject('DBFT', 'DATE');
        //To date
        $cell = $field->placeEditFieldObj('theForm', $msg, (empty(UserRequest::$post[$this->code . '_EDATE'])) ? $this->edate : UserRequest::$post[$this->code . '_EDATE'], 'DATE', $this->code . '_EDATE', 'EDATE', 'GEN_LBL_TO', CLX_NUM_20, CLX_NUM_20, '', '', '', true, false, false);
        $table->addCell($cell->source(), array(CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $checkbox = new htmlCheckBox($this->code . '_DETAIL', true);
        $table->addCell(dict('FICO_CODE_SUMARY'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($checkbox->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $c = new htmlSubmit(dict('GEN_LBL_CONTINUE'), 'PRINT_DIALOG');
        $buttons = $c->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $form->setContents($table->source());
        $this->contents = $form->source();
    }

// End method
    /**
     * Balance Sheet Start
     */
    function build() {
        $msg = array();
        $this->contents = '';
        if (!empty(UserRequest::$request['idlist'])) {
            $contract_code = ClxCommon::$db->getField($this->ficoname, 'IDCODE', 'ID=' . UserRequest::$request['idlist']);
        } else {
            $contract_code = '';
        }
        if (empty(UserRequest::$request['confirm']) || !$this->checkDialog($msg)) {
            $this->print_dialog($msg);
        } elseif (!empty(UserRequest::$request['gl'])) { // Display Transactions
            $this->contents = $this->getTransactions(UserRequest::$request['gl'], $contract_code);
        } else {
            //Preferences
            $this->edate = UserRequest::$request[$this->code . '_EDATE'];
            $this->summary = !empty(UserRequest::$request[$this->code . '_DETAIL']);

            $table = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_ALIGN => 'center'));

            $tableFrame->addcell('<hr size="1" noshade>');
            $a = '<h3><IMG SRC="./data/files/banklogo_sml.jpg" align = "left" vspace=3 hspace =10><center>' . dict('FIDU_CODE_BALANCESHEET') . '<center></h3>';
            if ($contract_code === '') {
                $a .= '<b>' . dict('FIDU_CODE_CONSBALACE', xFormat::date($this->edate, 'd - M - Y')) . '</b>';
            } else {
                $a .= dict('FIDU_CODE_CONTBALACE', xFormat::date($this->edate, $_SESSION['prefs']['DATE_FORMAT']));
                $a .= '<br>' . dict('FIDU_CODE_ACCOUNTNO') . ':<i>' . $contract_code . '</b>';
            }
            $tableFrame->addcell($a);
            $tableFrame->addcell('<hr size="1" noshade>');
            // Load balances
            $this->rollup($contract_code);
            $tableFrame->addcell($this->buildbalances());
            $tableFrame->addcell('<hr size="1" noshade>');
            $this->contents = $tableFrame->source();
        }
    }

// End method
    /**
     * Build user interfase balances
     */
    function buildbalances() {
        $ini = CLX_NUM_100000000000; //Level of characters.
        $summary_lbl = ($this->summary) ? CLX_NUM_FIVE : CLX_NUM_HUNDRED;
        ksort($this->balances);

        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $i = CLX_INIT_ZERO;
        foreach ($this->balances as $key => $val) {
            $char = strlen(str_replace('.', '', ($key / $ini)));
            $i++;
            //Reporting Label
            if ($char > $summary_lbl === true)
                continue;
            $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
            if ($char <= CLX_NUM_TWO)
                $table->addcell('<hr size="1" noshade>', array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
            $tkey = substr($key, CLX_NUM_ZERO, $summary_lbl);
            if (empty($this->nolink)) {
                $url = 'pop.php?idlist=' . UserRequest::$request['idlist'] . '&to=' . UserRequest::$request['to'] . '&ac=' . UserRequest::$request['ac'] . '&wind=1&confirm=1&gl=' . substr($tkey, CLX_NUM_ZERO, $char);
                if ($char <= CLX_NUM_TWO) {
                    $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' <b>' . $val['NAME'] . '</b> [<a href=' . singleQuote($url) . ' target=blank>+</a>] ', CLX_PROP_CLASS, $class);
                } else {
                    $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' ' . $val['NAME'] . '  [<a href=' . singleQuote($url) . ' target=blank>+</a>]', CLX_PROP_CLASS, $class);
                }
            } else {
                if ($char <= CLX_NUM_TWO) {
                    $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' <b>' . $val['NAME'] . '</b>', CLX_PROP_CLASS, $class);
                } else {
                    $table->addcell((str_repeat(CLX_CHAR_HTML_NBSP, $char)) . $tkey . ' ' . $val['NAME'], CLX_PROP_CLASS, $class);
                }
            }
            $table->addcell(xFormat::number(abs($val['BALANCE'])), array(CLX_PROP_CLASS => $class));
            $table->addcell(CLX_CHAR_HTML_NBSP, CLX_PROP_CLASS, $class);
            if ($char == CLX_NUM_TWO)
                $table->addcell('<hr size="1" noshade>', array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
        }
        return $table->source();
    }

// End method
    /**
     * Prompt period && Client to display on statement
     */
    function print_dialog($msg) {
        $form = new htmlForm(CLX_PROP_ACTION, 'pop.php', CLX_PROP_ID, 'theForm', 'name', 'theForm');
        $form->addArgument(CLX_PROP_TO, UserRequest::$request[CLX_PROP_TO]);
        $form->addArgument('ac', UserRequest::$request['ac']);
        $form->addArgument('idlist', UserRequest::$request['idlist']);
        $form->addArgument('wind', '1');
        $form->addArgument('confirm', '1');

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));

        $table->addCell(dict('FIDU_CODE_BALANCESHEET'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        //If not contract all trust but we need company
        if (empty(UserRequest::$request['idlist'])) {
            $select = new htmlSelect();
            $companies = ClxCommon::$db->getRecords(tableName('COMP'));
            while (($company = ClxCommon::$db->getNextRecord($companies))) {
                $select->addItem($company['IDCODE'] . ',' . $company['IDNAME'], $company['IDCODE'], false);
            }
            $table->addCell(dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME = ' . singleQuote('COMP_CODE') . '')), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        $field = getToolObject('DBFT', 'DATE');
        //To date
        $cell = $field->placeEditFieldObj('theForm', $msg, (empty(UserRequest::$post[$this->code . '_EDATE'])) ? $this->edate : UserRequest::$post[$this->code . '_EDATE'], 'DATE', $this->code . '_EDATE', 'EDATE', 'GEN_LBL_TO', CLX_NUM_20, CLX_NUM_20, '', '', '', true, false, false);
        $table->addCell($cell->source(), array(CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $checkbox = new htmlCheckBox($this->code . 'DBFT', true);
        $table->addCell(dict('FICO_CODE_SUMARY'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($checkbox->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $c = new htmlSubmit(dict('GEN_LBL_CONTINUE'), 'PRINT_DIALOG');
        $buttons = $c->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        $form->setContents($table->source());
        $this->contents = $form->source();
    }

// End method
    /**
     * Return Trasaction for specific gl code
     */
    function getTransactions($gl_code, $contract_code = '') {
        $start = '';
        if (!empty($contract_code)) {
            $contract_code = ' FIAT.CONTRACT_CODE = ' . singleQuote($contract_code) . ' AND ';
        }
        $sql = 'SELECT FIAT.* FROM ' . $this->fiatname . ' FIAT,' . $this->jelnname . ' JELN,' . $this->jehdname . ' JEHD  WHERE JELN.JEHDR_CODE = JEHD.IDCODE AND JEHD.TRANS_CODE = FIAT.IDCODE AND FIAT.APPROVAL_DATE >= ' . singleQuote($start) . ' AND ' .
                $contract_code . ' JELN.GLACCOUNTCODE LIKE ' . singleQuote($gl_code . '%') . ' ORDER BY FIAT.APPROVAL_DATE ASC';
        $results = ClxCommon::$db->executeQuery($sql);

        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));

        $table->addCell(dict('FIDU_CODE_TRANSACTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_SIX));
        $i = CLX_INIT_ZERO;
        $c = 'CLX_DB_FIELDS';
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('APPROVAL_DATE') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('CONTRACT_CODE') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('ASSET_CODE') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('IDCODE') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('IDNAME') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict(ClxCommon::$db->getField($c, 'FIELDLISTLABEL', 'FIELDNAME=' . singleQuote('BASE_TRANS_AMT') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        unset($c);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
            $i++;
            $table->addCell($result['APPROVAL_DATE'], array(CLX_PROP_CLASS => $class));
            $table->addCell($result['CONTRACT_CODE'], array(CLX_PROP_CLASS => $class));
            $table->addCell($result['ASSET_CODE'], array(CLX_PROP_CLASS => $class));
            $table->addCell($result['IDCODE'], array(CLX_PROP_CLASS => $class));
            $table->addCell($result['IDNAME'], array(CLX_PROP_CLASS => $class));
            $table->addCell(xFormat::number($result['BASE_TRANS_AMT']), array(CLX_PROP_CLASS => $class));
        }
        $b = new htmlButtonCloseWindow(dict('GEN_LBL_CLOSEWINDOW'));
        $table->addCell($b->source(), array(CLX_PROP_CLASS => 'footer', CLX_PROP_COLSPAN => CLX_NUM_SIX));
        return $table->source();
    }

    /**
     * Validate if correct values
     */
    function checkDialog(&$msg) {
        $msg = array();
        xValidate::date($msg, $this->code . '_EDATE', $this->edate, dict('GEN_LBL_DATE'));
        return count($msg) == CLX_NUM_ZERO;
    }

// End method
    /**
     * Returns source code
     */
    function source() {
        return $this->contents;
    }

// End method
// Reporte de Estado de Resultados - DRAFT
    /**
     * Buiild reports
     */
    function buildResults() {

    }

}
