<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2016-10-14 16:13:51 -0400 (Fri, 14 Oct 2016) $
 * @version        $Rev: 8826 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class fiai {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('FIAT', 'FIAI', 'FICO', 'GLCP', 'FITS', 'FITB', 'FIBK', 'PROD', 'PRAT', 'JCBV', 'FIAS'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice fiai form
     * Executed on init event
     */
    function fiai_init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            if (!class_exists('Fico'))
                includeCodeLib('FICO');
            $fico = new Fico();
            $comprec = $fico->fico_comp_operation_parameters($record['FIAI_CHART_FIELD01']);
            $record['FIAI_IDCODE'] = 'AUTO';
            $record['FIAI_CLX_EFFTDATE'] = $comprec['OPERATION_DATE'];
            $record['FIAI_NEXT_RUN_DATE'] = $comprec['OPERATION_DATE'];
            $record['FIAI_INSTSTAT'] = 'PRE';
        }
        if (UserRequest::$request['ac'] === 'CREATE') {
            $record['FIAI_CURRENCY_CODE'] = $comprec['CURRENCY_CODE'];
            $record['FIAI_BRANCH_PRN_prim'] = companyParameter('BRANCH_PRN', $comprec['IDCODE']);
            $record['FIAI_FIDU_BANKACT_CODE_prim'] = companyParameter('FIDU_BANK_ACCOUNT', $comprec['IDCODE']);
        }
    }

    /**
     * Post Authorization Method
     *
     * @param unknown_type $record
     */
    function post_autorize(&$record) {
        if (UserRequest::$request['ac'] === 'DELETE')
            return;
        if (array_key_exists('FIAI_IDCODE', $record)) {
            $prefix = 'FIAI_';
        } else {
            $prefix = '';
        }
        $sql = 'UPDATE ' . $this->fiainame . ' SET EFFTSTAT = ' . singleQuote('A') . ', INSTSTAT = "PEND"  WHERE IDCODE =' . singleQuote($record[$prefix . 'IDCODE']) . ' AND CLX_EFFTDATE = ' . singleQuote($record[$prefix . 'CLX_EFFTDATE']);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function fiai_validate_form(&$record, &$msg) {
        if (isset($record['FIAI_GLACCOUNTCODE_prim']) and $record['FIAI_GLACCOUNTCODE_prim'] !== '' and $record['FIAI_GLACCOUNTCODE'] === '')
            $record['FIAI_GLACCOUNTCODE'] = $record['FIAI_GLACCOUNTCODE_prim'];
        switch ($record['FIAI_FORMULA']) {
            case 'F001':
                if ($record['FIAI_FIXED_AMOUNT'] === '' || $record['FIAI_FIXED_AMOUNT'] == CLX_NUM_ZERO) {
                    $msg['FIAI_FIXED_AMOUNT'] = smsg('MSG-407');
                }
                break;
            case 'F002':
                if (isset($record['FIAI_GLACCOUNTCODE']) and $record['FIAI_GLACCOUNTCODE'] === '') {
                    $msg['FIAI_GLACCOUNTCODE'] = smsg('MSG-408');
                } elseif ($record['FIAI_GLACCOUNTCODE_prim'] === '') {
                    $msg['FIAI_GLACCOUNTCODE_prim'] = smsg('MSG-408');
                }
                break;
            case 'F003':
                if ($record['FIAI_GLACCOUNTCODE'] === '') {
                    $msg['FIAI_GLACCOUNTCODE'] = smsg('MSG-408');
                }
                if ($record['FIAI_PCT_CTA_CONT'] === '' || $record['FIAI_PCT_CTA_CONT'] == CLX_NUM_ZERO) {
                    $msg['FIAI_PCT_CTA_CONT'] = smsg('MSG-409');
                }
                break;
            default:
                break;
        }
        //Validar parametros de desembolso asi como la primera ejecucion se encuentre correcta.
        //Si existe retiro de efectivo verificar que exista la cuenta de banco del fideicomiso
        if (ClxCommon::$db->oneExistsRecords($this->fitsname, 'TRANSTBL_CODE =' . singleQuote($record['FIAI_TRANSTBL_CODE']) . ' AND AFFECT_TYPE = ' . singleQuote('CA') . '') > CLX_NUM_ZERO && empty($record['FIAI_FIDU_BANKACT_CODE'])) {
            $msg['FIAI_FIDU_BANKACT_CODE'] = smsg('MSG-26', $record['FIAI_IDCODE']);
        }
        //Test fi bank account has gl
        if (!empty($record['FIAI_FIDU_BANKACT_CODE'])) {
            $sql = 'SELECT  PRAT.PRO_GL_CODE FROM  ' . $this->fibkname . ' FIBK, ' . $this->pratname . ' PRAT WHERE FIBK.BANK_PROD_CODE = PRAT.PRODUCT_CODE AND FIBK.BANK_ACCOUNT_NO =' . singleQuote($record['FIAI_FIDU_BANKACT_CODE']);
            if (ClxCommon::$db->getOne($sql) === '') {
                $msg['FIAI_FIDU_BANKACT_CODE'] = smsg('MSG-27', $record['FIAI_IDCODE']);
            }
        }
        getLookupValue('FIAI_BENEFICIARY_CODE', 'BENEFICIARY_CODE', $record); //beneficiario
        getLookupValue('FIAI_CHART_FIELD01', 'CHART_FIELD01', $record); //contracto
        $is_disbursement = (ClxCommon::$db->oneExistsRecords($this->fitsname, 'TRANSTBL_CODE =' . singleQuote($record['FIAI_TRANSTBL_CODE']) . ' AND AFFECT_TYPE = ' . singleQuote('CA') . ' AND DEBIT_CREDIT=' . "'CR'" . '') > CLX_NUM_ZERO);
        if ($is_disbursement && (empty($record['FIAI_BENEFICIARY_CODE']) && empty($record['FIAI_BENEFICIARY_NAME']))) {
            $msg['FIAI_BENEFICIARY_CODE'] = smsg('MSG-35', $record['FIAI_IDCODE']);
        }
        //Si hay beneficiario que exista la cuenta de banco
        if ($is_disbursement === true && empty($record['FIAI_DISBURSEMENT_TYPE'])) {
            $msg['FIAI_DISBURSEMENT_TYPE'] = smsg('MSG-36', $record['FIAI_IDCODE']);
        }
        //Si el desembolso es de tipo cheque solicitar BRANCH_PRN
        getLookupValue('FIAI_BRANCH_PRN', 'BRANCH_PRN', $record); //beneficiario
        if ($record['FIAI_DISBURSEMENT_TYPE'] === 'CH' && empty($record['FIAI_BRANCH_PRN'])) {
            $msg['FIAI_BRANCH_PRN'] = smsg('MSG-37', $record['FIAI_IDCODE']);
        }
        //Si es cheque solicitar el numero de cheque,
        //Nota esta validacion se mando a detener ya que es capturada por el ibs
        //Validar que el tipo de transaccion
        $sql = 'SELECT COUNT(FITB.ID)  FROM ' . $this->fitbname . ' FITB, ' . $this->ficoname . ' FICO  WHERE (FITB.FI_TYPE = FICO.FI_TYPE  OR FITB.FI_TYPE IS NULL OR FITB.FI_TYPE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND FITB.IDCODE =' . singleQuote($record['FIAI_TRANSTBL_CODE']) . ' AND FICO.IDCODE =' . singleQuote($record['FIAI_CHART_FIELD01']);
        if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            $msg['FIAI_TRANSTBL_CODE'] = smsg('MSG-39', $record['FIAI_IDCODE']);
        }
        //Validar que linea de efectivo coincida con moneda de cuenta de banco
        $sql = 'SELECT  COUNT(FIBK.ID)  FROM  ' . $this->fibkname . ' FIBK, ' . $this->prodname . ' PROD WHERE FIBK.BANK_PROD_CODE = PROD.IDCODE AND FIBK.BANK_ACCOUNT_NO = ' . singleQuote($record['FIAI_FIDU_BANKACT_CODE']) . ' AND FIBK.CURRENCY_CODE =' . singleQuote($record['FIAI_CURRENCY_CODE']);
        if (!empty($record['FIAI_FIDU_BANKACT_CODE']) && ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
            //Existe una linea de otro grupo contable
            $msg['FIAI_FIDU_BANKACT_CODE'] = smsg('MSG-88', $record['FIAI_IDCODE']);
        }
    }

    /**
     * Runs Instruction by Btn from fico_reset listing
     * @return string
     */
    function fiai_run_instruction_btn() {
        $idlist = getIdlist();
        $contents = '';
        $sql = 'SELECT FICO.COMP_CODE, FICO.IDCODE, GLCP.OPERATION_DATE  FROM ' . $this->ficoname . ' FICO, ' . $this->glcpname . ' GLCP  WHERE GLCP.COMP_CODE = FICO.COMP_CODE AND FICO.ID IN (' . toString($idlist) . ')';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $this->fiai_build_transactions($r['COMP_CODE'], $r['OPERATION_DATE'], $r['IDCODE'], null, $r, null);
        }
        return $contents;
    }

    //Preparar ejecucion para el cierre
    function fiai_run_cron($comp_code, $efftdate, $parameters) {
        $this->fiai_build_transactions($comp_code, $efftdate, null, null, $parameters, null);
    }

    function fiai_run_instruction_listing() {
        $idlist = toString(getIdlist());
        if (empty($idlist))
            return;
        $sql = 'SELECT FICO.COMP_CODE, GLCP.OPERATION_DATE, FIAI.IDCODE, FIAI.CHART_FIELD01 CONTRACT_CODE, FIAI.CHART_FIELD02 PARTICIPANT_ID FROM FIDU_INSTRUCTIONS FIAI, FIDU_CONTRACTS FICO, GL_PARAMETERS_TBL GLCP WHERE FIAI.CHART_FIELD01 = FICO.IDCODE AND FICO.COMP_CODE = GLCP.COMP_CODE AND FIAI.ID IN (' . $idlist . ')';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $parameters = array();
            $this->fiai_build_transactions($result['COMP_CODE'], $result['OPERATION_DATE'], $result['CONTRACT_CODE'], $result['PARTICIPANT_ID'], $parameters, $result['IDCODE'], false);
        }
    }

    /**
     * Build transaction instructions
     * @param: string comp_code
     * @param: string [efftdate]
     * @param: string [contract_code]
     * @param: string [participant_id]
     */
    function fiai_build_transactions($comp_code, $operation_date = '', $contract_code = '', $participant_id = '', $parameters = array(), $fiai_code = '', $display = true) {
        //$now =  xFormat::getNow('Y-m-d H:i:s');
        //Determine batch id
        $batch_id = batch_id();
        //If efftdate is not provided then search operation date day as operation date
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        if ($operation_date === ''){
            $operation_date = $comp->comp_operation_date($comp_code);
        }
        $fiscal_period = $comp->comp_fiscal_period($comp_code, $operation_date);
        if ($display === true) {
            echoLog(' PROCESSING CONTRACT INTRUCTIONS. BATCH ID: ' . $batch_id);
            echoLog('Efective charge date : ' . $operation_date);
            echoLog('Company: ' . (($comp_code === '') ? 'ALL' : $comp_code));
            echoLog('Contract: ' . (($contract_code === '') ? 'ALL' : $contract_code));
            echoLog('Participant: ' . (($participant_id === '') ? 'ALL' : $participant_id));
        }
        $sql = 'SELECT COUNT(FIAI.ID) COUNTER ' . 'FROM ' . $this->fiainame . ' FIAI, ' . $this->ficoname . ' FICO ' . ' WHERE FICO.IDCODE = FIAI.CHART_FIELD01 AND FIAI.CLX_AUTH_STATUS = '.singleQuote('AP').' AND FIAI.NEXT_RUN_DATE = ' . singleQuote($operation_date) . '  AND FICO.COMP_CODE = ' . singleQuote($comp_code) . '  AND FIAI.EFFTSTAT = ' . singleQuote('A') . '  AND FICO.CONTRACT_STATUS = ' . singleQuote('ACTI') . '  AND FIAI.INSTSTAT = ' . singleQuote('PEND') . '  AND FIAI.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE) ' . 'FROM ' . $this->fiainame . ' X  WHERE X.IDCODE = FIAI.IDCODE  AND X.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') ';
        if ($fiai_code <> ''){
            $sql .= ' AND FIAI.IDCODE = ' . singleQuote($fiai_code);
        }
        if ($display === true){
            echoLog('Total instruction to process: ' . ClxCommon::$db->getOne($sql));
        }
        if (!class_exists('Gloq')){
            includeCodeLib('GLOQ');
        }
        $gloq = new Gloq();
        if (!class_exists('Jeca')){
            includeCodeLib('JECA');
        }
        $jeca = new Jeca();
        if (!function_exists('fiat_build')){
            includeCodeLib('FIAT');
        }
        $fiat = new Fiat();
        $field = getToolObject('DBFT', 'RECU');
        //Search all instruction with next run date = operation date
        $sql = 'SELECT FICO.COMP_CODE, FIAI.*  FROM  ' . $this->fiainame . ' FIAI,  ' . $this->ficoname . ' FICO  WHERE  FICO.IDCODE = FIAI.CHART_FIELD01  AND FIAI.NEXT_RUN_DATE = ' . singleQuote($operation_date) . '  AND FICO.COMP_CODE = ' . singleQuote($comp_code) . '  AND FIAI.EFFTSTAT = ' . singleQuote('A') . '  AND FICO.CONTRACT_STATUS = ' . singleQuote('ACTI') . '  AND FIAI.INSTSTAT = ' . singleQuote('PEND') . '  AND FIAI.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE)  FROM ' . $this->fiainame . ' X  WHERE X.IDCODE = FIAI.IDCODE  AND X.CLX_EFFTDATE<=' . singleQuote($operation_date) . ') ';
        if($fiai_code <> '') $sql .= " AND FIAI.IDCODE = '$fiai_code'";
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {

            $trans_rec = array('COMP_CODE' => $result['COMP_CODE'],
                'IDNAME' => $result['IDNAME'],
                'INSTRUCTION_CODE' => $result['IDCODE'],
                'TRANSTBL_CODE' => $result['TRANSTBL_CODE'],
                'EFFTDATE' => $operation_date,
                'REAL_DATE' => $operation_date,
                'AVAILABILITY_DATE' => $result['NEXT_RUN_DATE'],
                'TRANS_AMT' => $result['FIXED_AMOUNT'],
                'CURRENCY_CODE' => $result['CURRENCY_CODE'],
                'FIDU_BANKACT_CODE' => $result['FIDU_BANKACT_CODE'],
                'CHART_FIELD01' => $result['CHART_FIELD01'],
                'CHART_FIELD02' => $result['CHART_FIELD02'],
                'CHART_FIELD03' => $result['CHART_FIELD03'],
                'CHART_FIELD04' => $result['CHART_FIELD04'],
                'GUARANTEE_CODE' => $result['GUARANTEE_CODE'],
                'GARNISHMENT_CODE' => $result['GARNISHMENT_CODE'],
                'DISBURSEMENT_TYPE' => $result['DISBURSEMENT_TYPE'],
                'BENEFICIARY_CODE' => $result['BENEFICIARY_CODE'],
                'BENEFICIARY_NAME' => $result['BENEFICIARY_NAME'],
                'LGGROUP_CODE' => $result['LGGROUP_CODE'],
                'FISCAL_PERIOD' => $fiscal_period,
                'COMMISSION_CODE' => 'null',
                'BRANCH_PRN' => $result['BRANCH_PRN'],
                'ITEM_REF_NO' => $result['IDCODE'],
                'DISBURSEMENT_TYPE' => $result['DISBURSEMENT_TYPE'],
                'BANKAC_CODE' => $result['BANKAC_CODE'],
                'GLACCOUNTCODE2' => $result['GLACCOUNTCODE2'],
                'BANK_ACCOUNT_NO' => $result['BANK_ACCOUNT_NO'],
                'BANK_NAME' => $result['BANK_NAME'],
                'BANK_ROUTING_NO' => $result['BANK_ROUTING_NO'],
                'BANK_ADDRESS' => $result['BANK_ADDRESS'],
                'BANK_COUNTRY' => $result['BANK_COUNTRY'],
                'BANK_STATE' => $result['BANK_STATE'],
                'BANK_CITY' => $result['BANK_CITY'],
                'BANK_SWIFT' => $result['BANK_SWIFT'],
                'ACH_ACC_TYPE' => $result['ACH_ACC_TYPE'],
                'CHECK_NO' => $result['CHECK_NO'],
                'CHECK_PRINT_FMT' => $result['CHECK_PRINT_FMT'],
                'BRANCH_PRN' => $result['BRANCH_PRN'],
                'CHECK_ALLOW_PR_BL' => $result['CHECK_ALLOW_PR_BL'],
                'INVOICE_ITEM' => $result['INVOICE_ITEM'],
                'INVOICE_NO' => $result['INVOICE_NO'],
                'INVOICE_NO_02' => $result['INVOICE_NO_02'],
                'PROJECT_CODE' => $result['PROJECT_CODE'],
                'BATCH_ID' => $batch_id);

            //test if date is holiday, if true increase a day
            $i = CLX_INIT_ZERO;
            while ($jeca->is_holiday($trans_rec ['COMP_CODE'], $trans_rec ['EFFTDATE'])) {
                //Update date + 1
                $trans_rec ['EFFTDATE'] = adodb_next_day($trans_rec ['EFFTDATE']);
                $trans_rec ['REAL_DATE'] = $trans_rec ['EFFTDATE'];
                $i++;
                if ($i == CLX_NUM_HUNDRED)
                    break;
            }
            $trans_rec ['AVAILABILITY_DATE'] = $trans_rec ['EFFTDATE'];
            $xrate = $comp->comp_current_exchange($trans_rec ['COMP_CODE'], $trans_rec ['CURRENCY_CODE'], $trans_rec ['EFFTDATE']);
            $trans_rec ['EXCHANGE_RT'] = $xrate ['EXCHANGE_RT'];
            $trans_rec ['CUR_OPERATOR'] = $xrate ['CUR_OPERATOR'];
            $formula = $result['FORMULA'];
            $pctCtaCont = $result['PCT_CTA_CONT'];
            switch ($formula) {
                case 'F002':
                    $sql = 'SELECT SUM(JCBV.BALANCE_AMT) MONTO FROM ' . $this->jcbvname . ' JCBV, ' . $this->ficoname . ' FICO WHERE JCBV.GLACCOUNTCODE =' . singleQuote($result['GLACCOUNTCODE']) . ' AND JCBV.CHART_FIELD01 = ' . singleQuote($result['CHART_FIELD01']).' AND JCBV.CHART_FIELD02 = ' . singleQuote($result['CHART_FIELD02']) . ' AND JCBV.CHART_FIELD01 = FICO.IDCODE AND JCBV.CURRENCY_CODE =' . singleQuote($result['CURRENCY_CODE']);
                    $monto_def = ClxCommon::$db->getOne($sql);
                    break;
                case 'F003':
                    $sql = 'SELECT SUM(JCBV.BALANCE_AMT) MONTO FROM ' . $this->jcbvname . ' JCBV, ' . $this->ficoname . ' FICO WHERE JCBV.GLACCOUNTCODE =' . singleQuote($result['GLACCOUNTCODE']) . ' AND JCBV.CHART_FIELD01 = ' . singleQuote($result['CHART_FIELD01']).' AND JCBV.CHART_FIELD02 = ' . singleQuote($result['CHART_FIELD02']) . ' AND JCBV.CHART_FIELD01 = FICO.IDCODE AND JCBV.CURRENCY_CODE =' . singleQuote($result['CURRENCY_CODE']);
                    $monto_def = (ClxCommon::$db->getOne($sql) * $pctCtaCont) / CLX_NUM_HUNDRED;
                    break;
                case 'F001':
                default:
                    $monto_def = $result['FIXED_AMOUNT'];
                    break;
            }
            $trans_code = $fiat->fiat_build($comp_code, $result['TRANSTBL_CODE'], $result['CHART_FIELD01'], $result['CHART_FIELD02'], $monto_def, $trans_rec);
            unset($trans_rec);
            if (!empty($result['APPROVE_PAYMENT'])){
                if(defined('kDefaultIsCron') && !empty(kDefaultIsCron)){
                    $fiat->fiat_approve_by_transcode($trans_code); //Aprovando directo
                }else{
                    $gloq->pushFiatListing($trans_code);//Mandando a cola de operaciones
                }
            }

            //Set next run day
            $last_date = $result['NEXT_RUN_DATE'];
            if ($last_date === ''){
                $last_date = $operation_date;
            }
            $next_run_date = xFormat :: date($field->next_run_date('RECURRENT', $last_date, $result), 'Y-m-d');
            if ($next_run_date === '') {
                $status = 'COMP';
                $sql = 'UPDATE ' . $this->fiainame . ' SET LAST_RUN_DATE = ' . singleQuote($last_date) . ', ' . 'NEXT_RUN_DATE = NULL, ' . 'INSTSTAT = ' . singleQuote($status) . '  WHERE ID = ' . singleQuote($result['ID']);
            } else {
                $status = 'PEND';
                $sql = 'UPDATE ' . $this->fiainame . ' SET LAST_RUN_DATE = ' . singleQuote($last_date) . ', ' . 'NEXT_RUN_DATE = ' . singleQuote($next_run_date) . ', ' . 'INSTSTAT = ' . singleQuote($status) . ' ' . ' WHERE ID = ' . singleQuote($result['ID']);
            }
            ClxCommon::$db->executeQuery($sql);
        }//end while
        $sql = 'SELECT COUNT(ID) COUNTER FROM ' . $this->fiatname . ' WHERE BATCH_ID = ' . singleQuote($batch_id);
        if ($display === true)
            echoLog('Total instruction completed: ' . ClxCommon::$db->getOne($sql));
    }

}
