<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-12 07:55:36 -0400 (Mon, 12 Oct 2015) $
 * @version        $Rev: 7215 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Fign {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('FIPA', 'FICO', 'FIGN'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Initialice fipa form
     * Executed on init event
     */
    function fign_init_form(&$record) {
        if (!class_exists('Fico'))
            includeCodeLib('FICO');
        $fico = new Fico();
        $comprec = $fico->fico_comp_operation_parameters($record['FIGN_CONTRACT_CODE']);
        unset($fico);
        $ficostart = ClxCommon::$db->getField(tableName('FICO'), 'START_DATE', 'IDCODE = ' . singleQuote($record['FIGN_CONTRACT_CODE']));
        $record['FIGN_CLX_EFFTDATE'] = ($ficostart > $comprec['OPERATION_DATE']) ? $ficostart : $comprec['OPERATION_DATE'];
    }

    function fign_validate_form(&$record, &$msg) {
        getLookupValue('FIGN_CONTRACT_CODE', 'CONTRACT_CODE', $record);
        if (($record['FIGN_GARNISH_AMT'] <> CLX_NUM_ZERO && $record['FIGN_GARNISH_PCT'] <> CLX_NUM_ZERO) || ($record['FIGN_GARNISH_AMT'] == CLX_NUM_ZERO && $record['FIGN_GARNISH_PCT'] == CLX_NUM_ZERO)) {
            /*
             * CODE_ERR_MUSTBEAMOUNTORPCT
             * Please choose one amount or porcentage.
             * Emgargo solo puede ser por monto o porcentage, no ambas.
             */
            $msg['FIGN_GARNISH_AMT'] = smsg('MSG-59', $record['FIGN_IDCODE']);
        }
        //Test if garnishment already exists for other contract
        if (ClxCommon::$db->oneExistsRecords($this->fignname, 'IDCODE = ' . singleQuote($record['FIGN_IDCODE']) . ' AND CONTRACT_CODE <> ' . singleQuote($record['FIGN_CONTRACT_CODE'])) <> CLX_NUM_ZERO) {
            $msg['FIGN_CONTRACT_CODE'] = smsg('MSG-60', $record['FIGN_IDCODE']);
        }
        $fignold = ClxCommon::$db->getRecord($this->fignname, 'CONTRACT_CODE =' . singleQuote($record['FIGN_CONTRACT_CODE']) . ' AND IDCODE =' . singleQuote($record['FIGN_IDCODE']) . ' AND CLX_EFFTDATE<>' . singleQuote($record['FIGN_CLX_EFFTDATE']));
        if (!empty($fignold)) {
            if ($record['FIGN_GARNISH_AMT'] <> CLX_NUM_ZERO && $fignold['GARNISH_PCT'] <> CLX_NUM_ZERO) {
                /*
                 * CODE_ERR_MUSTBEPCT
                 * Percentage allowed only.
                 * Emgargo solo puede ser por porcentage.
                 */
                $msg['FIGN_GARNISH_AMT'] = smsg('MSG-61', $record['FIGN_IDCODE']);
            }
            if ($fignold['GARNISH_AMT'] <> CLX_NUM_ZERO && $record['FIGN_GARNISH_PCT'] <> CLX_NUM_ZERO) {
                /*
                 * CODE_ERR_MUSTBEAMT
                 * Amount allowed only.
                 * Emgargo solo puede ser por monto.
                 */
                $msg['FIGN_GARNISH_PCT'] = smsg('MSG-62', $record['FIGN_IDCODE']);
            }
        }
        if ($record['FIGN_GARNISH_AMT'] <> CLX_NUM_ZERO && $record['FIGN_MAXBLOCKED_AMT'] <> CLX_NUM_ZERO) {
            /*
             * CODE_ERR_MAXNOTALLOWED
             * Max Amount allowed only on percentages.
             * Monto maximo solo permitido para porcentajes
             */
            $msg['FIGN_MAXBLOCKED_AMT'] = smsg('MSG-63', $record['FIGN_IDCODE']);
        }
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            //Validate that modifications must be under efftdate
            if (ClxCommon::$db->oneExistsRecords($this->fignname, 'CONTRACT_CODE =' . singleQuote($record['FIGN_CONTRACT_CODE']) . ' AND IDCODE =' . singleQuote($record['FIGN_IDCODE'])) <> CLX_NUM_ZERO) {
                /**
                 * CODE_ERR_FIGNEXISTSMUSTEFFTDATE
                 * Already created please used modify effective dated instead.
                 * Emgargo ya se encuentra registrado, favor modifique con fecha efectiva.
                 */
                $msg['FIGN_IDCODE'] = smsg('MSG-64', $record['FIGN_IDCODE']);
            }
        }
        //Validate contract status
        if (empty($record['FIGN_CONTRACT_CODE']) || ClxCommon::$db->oneExistsRecords($this->ficoname, 'CONTRACT_STATUS IN (' . singleQuoteArray(array('ACTI', 'NEG')) . ') AND IDCODE =' . singleQuote($record['FIGN_CONTRACT_CODE'])) == CLX_NUM_ZERO) {
            $msg['FIGN_CONTRACT_CODE'] = smsg('MSG-65', $record['FIGN_IDCODE']);
        }
        //Validate participant status
        if (empty($record['FIGN_PARTICIPANT_ID']) || ClxCommon::$db->oneExistsRecords($this->fipaname, 'STATUS = ' . singleQuote('A') . ' AND CONTRACT_CODE =' . singleQuote($record['FIGN_CONTRACT_CODE']) . ' AND ACCOUNTCODE =' . singleQuote($record['FIGN_PARTICIPANT_ID'])) == CLX_NUM_ZERO) {
            $msg['FIGN_PARTICIPANT_ID'] = smsg('MSG-66', $record['FIGN_IDCODE']);
        }
        //Validando que fecha de creacion de embargo sea mayor a la fecha de la contabilidad
        if (!class_exists('Fico'))
            includeCodeLib('FICO');
        $fico = new Fico();
        $comprec = $fico->fico_comp_operation_parameters($record['FIGN_CONTRACT_CODE']);
        unset($fico);
        if (xFormat::date($record['FIGN_CLX_EFFTDATE'], 'Y-m-d') < $comprec['OPERATION_DATE']) {
            /*
             * CODE_ERR_EFFTDATENOTACCDATE
             * Effective date must be greater than accounting date.
             * Fecha efectiva debe ser mayor a la fecha de la contabilidad.
             */
            $msg['FIGN_CLX_EFFTDATE'] = smsg('MSG-67', $record['FIGN_IDCODE']);
        }
    }

}
