<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-08 15:32:52 -0400 (Thu, 08 Oct 2015) $
 * @version        $Rev: 7191 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Glpf {

    var $parameters; //Process parameters if provided.
    var $approve = true;
    var $load_credi_balances = false;

    function __construct($procParameters = array()) {
        multiTableName($this, array('GLPF', 'PROD', 'INVT', 'APIL', 'GLAC', 'INVL', 'INVO', 'FIAS', 'FILO', 'GLPF', 'JEHD', 'JELN', 'GDVW', 'PRAT', 'FIAT', 'FTMP', 'FICO', 'FIPA', 'INPY'));
        $this->batch_id = (!empty($procParameters['RUNID'])) ? $procParameters['RUNID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->parameters = $procParameters;
    }

//end constructor
    /**
     * Initialices form
     *
     * @param array $record
     */
    function init_form(&$record) {
        //Verificar ultimo seqnum
        $sql = 'SELECT IFNULL(MAX(SEQNUM),0) FROM ' . $this->glpfname . ' WHERE PRODUCT_CODE = ' . singleQuote($record['GLPF_PRODUCT_CODE']);
        $record['GLPF_SEQNUM'] = intVal(ClxCommon::$db->getOne($sql)) + CLX_NUM_ONE;
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function validate_form(&$record, &$msg) {
        //Seqnum must be integer from 1-5
        if (intVal($record['GLPF_SEQNUM']) < CLX_NUM_ONE || intVal($record['GLPF_SEQNUM']) > CLX_NUM_FIVE) {
            $msg['GLPF_SEQNUM'] = smsg('MSG-170');
        }
        $sql = 'SELECT COUNT(GLPF.SEQNUM) FROM ' . $this->glpfname . ' GLPF WHERE GLPF.FEE2_GL_CODE = ' . singleQuote($record['GLPF_FEE2_GL_CODE']) . ' AND GLPF.PRODUCT_CODE = ' . singleQuote($record['GLPF_PRODUCT_CODE']) . ' AND GLPF.PROD_STATUS = ' . singleQuote($record['GLPF_PRODUCT_CODE']) . ' AND GLPF.ID <> ' . ((empty($record['GLPF_ID'])) ? CLX_NUM_ZERO : $record['GLPF_ID']);
        if (!empty($record['GLPF_FEE2_GL_CODE']) && $record['GLPF_FEE2_GL_CODE'] <> 'null' && ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $msg['GLPF_SEQNUM'] = smsg('MSG-208');
        }
        //Validar que en productos de credito no se repitan las comisiones
        //en caso de productos de garantias deben ser los mismos.
        if ($record['GLPF_CPAY_FREQUENCY'] <> 'CHGR' && !empty($record['GLPF_PAY_TX_CODE'])) {
            $msg['GLPF_PAY_TX_CODE'] = smsg('MSG-375');
        }
        //en comision de bancor se implementa directamente en el cargo y no en transaccion como en global
        //Validando grupo contable de transacion seleccionada
        if (!empty($record['GLPF_PAY_TX_CODE'])) {
            $sql = 'SELECT COUNT(FITS.ID) QTY FROM FIDU_TRANSACTION_SPLIT FITS, GL_CODES_TBL GLAC WHERE FITS.TRANSTBL_CODE = ' . singleQuote($record['GLPF_PAY_TX_CODE']) . ' AND FITS.GLACCOUNTCODE = GLAC.IDCODE AND GLAC.GLGROUP_CODE <> ' . singleQuote($record['GLPF_GLGROUP_CODE']);
            if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                $msg['GLPF_PAY_TX_CODE'] = smsg('MSG-377');
            }
        }
    }

    /* Contabilizacion de garantias segun */

    /**
     * Ftmp payment from glpf adm invoice
     * @param string $comp_code
     * @param string $je_source
     */
    function build_invoice_pay_fiat_from_ftmp($batch_id, $je_source) {
        if (ClxCommon::$db->oneExistsRecords($this->ftmpname, 'BATCH_ID = ' . singleQuote($batch_id)) == CLX_NUM_ZERO) {
            return;
        }
        if (!class_exists('Fiat'))
            includeCodeLib('FIAT');
        $fiat = new Fiat ();
        // build transactions /*CALC_AMT_02 RESERVED FOR APPROVE_PAYMENT*/
        $sql = 'SELECT ' . singleQuote('AUTO') . ' IDCODE, FTMP.IDNAME, FTMP.EFFTDATE, FTMP.TRANSTBL_CODE, ' . singleQuote('PEND') . ' CLX_AUTH_STATUS, FTMP.CURRENCY_CODE, FTMP.EXCHANGE_RT, FTMP.CUR_OPERATOR, FTMP.LGGROUP_CODE, FTMP.ITEM_REF_NO, FTMP.ITEM_REF_NO REFERENCE_NO, FTMP.COMP_CODE, FTMP.CHART_FIELD01, FTMP.CHART_FIELD02, FTMP.CHART_FIELD03, FTMP.CHART_FIELD04, FTMP.BANKACCOUNT_CODE FIDU_BANKACT_CODE, FTMP.CALC_AMT_01 TRANS_AMT, FTMP.CALC_AMT_02, FTMP.CALC_AMT_03 TAX_AMT, 0 RECONCILIED, FTMP.DISBURSEMENT_TYPE, FTMP.ITEM_REF_NO, FTMP.BANKACCOUNT_CODE BANKAC_CODE, FTMP.EFFTDATE AVAILABILITY_DATE, FTMP.BENEFICIARY_NAME, FTMP.BRANCH_PRN, 0 RECORD_BLOCKING, FTMP.BATCH_ID, ' . singleQuote($je_source) . ' JE_SOURCE, ' . singleQuote($this->username) . ' CREATEDBY, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->username) . ' MODIFIEDBY, ' . singleQuote($this->now) . ' MDATE, FTMP.REC_ID FROM ' . $this->ftmpname . ' FTMP, ' . $this->ficoname . ' FICO, '.$this->glpfname.' GLPF WHERE FTMP.CHART_FIELD01 = FICO.IDCODE AND FTMP.GLPF_ID = GLPF.ID AND FTMP.CALC_AMT_01<>0 AND FTMP.BATCH_ID = ' . singleQuote($batch_id) . ' AND FTMP.REC_STATUS = ' . singleQuote('RUN') . ' ';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $approve = $r ['CALC_AMT_02'];
            if ($approve == CLX_NUM_ZERO) {

                $r ['EFFTDATE'] = $r ['EFFTDATE'];
                $r ['AVAILABILITY_DATE'] = $r ['EFFTDATE'];
            }
            unset($r ['CALC_AMT_02']);
            $inpy_id = $r['REC_ID'];
            unset($r ['REC_ID']);
            $trans_code = $fiat->fiat_build($r['COMP_CODE'], $r ['TRANSTBL_CODE'], $r ['CHART_FIELD01'], $r ['CHART_FIELD02'], $r ['TRANS_AMT'], $r);
            if ($je_source === 'GLPFADMPAY' && $inpy_id !== '') {
                ClxCommon::$db->updateRecord($this->inpyname, array('REL_TRANS_CODE' => $trans_code), 'ID = ' . singleQuote($inpy_id));
            }// Approving all transactions that may require approval
            if ($approve > CLX_NUM_ZERO) {
                $fiat->fiat_approve_by_transcode($trans_code);
            }
        }
    }

    /**
     * Build Bills or Invoices.
     * Run by process engine GLPF_001
     * @param string $comp_code
     * @param bool $rebuild = false
     */
    function build_invoices($comp_code, $rebuild = false) {
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        $comprec = $comp->comp_operation_parameters($comp_code);
        $operation_date = $comprec['OPERATION_DATE'];
		if (empty($operation_date)) {
			echoLog('<br> ERROR not operation date for company: '.$comp_code.' <br>');
			return;
		}
        if (!class_exists('Apin'))
            includeCodeLib('APIN');
        if (!class_exists('Invo'))
            includeCodeLib('INVO');
        $apin = false;
        $invo = false;
        $DBTB = &getTool('DBTB');
        $DBTB->buildTemporaryTable('INVT'); //Clear tmp
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->invtname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
        $oper_mn = xFormat::date($operation_date, 'm');
        $oper_mn_day = xFormat::date($operation_date, 'm-j');
        $oper_mn_day_last_day = adodb_month_lastday($operation_date, 'm-d');
        $oper_bisiesto = ($oper_mn_day_last_day === '02-29') ? '1' : '0';
        //Account Payables and recievables
        $sql = 'INSERT INTO ' . $this->invtname . ' (IDNAME, VENDOR_CODE, INVOICEDATE, DUE_DATE, CURRENCY_CODE, INVOICE_AMT, EXT_REF_NO, INVO_STAT, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, GL_TYPE, GL01_CODE, GL02_CODE, CDATE, CREATEDBY, JEHDR_CODE, RESTRICTED, BATCH_ID) SELECT GLPF.IDNAME, GLPF.VENDOR_CODE, ' . singleQuote($operation_date) . ' INVOICEDATE, ' . singleQuote($operation_date) . ' DUE_DATE, GDVW.CURRENCY_CODE, (case GDVW.GL_TYPE WHEN ' . singleQuote('LI') . ' THEN -1 ELSE 1 END)*(GDVW.DEBIT_AMT-GDVW.CREDIT_AMT) INVOICE_AMT, FILO.IDCODE EXT_REF_NO, ' . "'01'" . ' INVO_STAT, GDVW.COMP_CODE, GDVW.CHART_FIELD01, GDVW.CHART_FIELD02, GDVW.CHART_FIELD03, GDVW.CHART_FIELD04, GLPF.FEE_ITEM_CODE ITEM_CODE, GDVW.GL_TYPE, GLPF.FEE2_GL_CODE GL01_CODE, GLPF.FEE_GL_CODE GL02_CODE, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->username) . ' CREATEDBY, GDVW.IDCODE JEHDR_CODE, 1 RESTRICTED, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->fiasname . ' FIAS, ' . $this->filoname . ' FILO, ' . $this->glpfname . ' GLPF, ' . $this->gdvwname . ' GDVW WHERE GDVW.COMP_CODE = ' . singleQuote($comp_code) . ' AND GDVW.HAS_INVOICE = 0 AND GLPF.STATUS = ' . singleQuote('A') . ' AND GLPF.BILLING = 1 AND GDVW.REVERSED = 0 AND GDVW.GLACCOUNTCODE = GLPF.FEE2_GL_CODE AND FILO.IDCODE = FIAS.LOAN_CODE AND GDVW.ITEM_REF_NO = FILO.IDCODE AND FILO.PRODUCT_CODE = GLPF.PRODUCT_CODE AND FILO.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE) FROM CRED_CREDITS X WHERE X.IDCODE = FILO.IDCODE) AND ( /*diario D*/ GLPF.CPAY_FREQUENCY IN (' . singleQuoteArray(array('D', '360', '365', '366')) . ') /*mensual 2 */ OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND CONCAT(' . singleQuote($oper_mn . '-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('01-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('02-28') . ' AND ' . $oper_bisiesto . ' = 0) OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('02-29') . ' AND ' . $oper_bisiesto . ' = 1) OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('03-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('04-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('05-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('06-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('07-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('08-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('09-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('10-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('11-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('2') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('12-31') . ') /*trimestral 4*/ OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND CONCAT(' . singleQuote('03-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND CONCAT(' . singleQuote('06-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND CONCAT(' . singleQuote('09-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND CONCAT(' . singleQuote('12-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('03-31') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('06-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('09-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('4') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('12-31') . ') /*semestral 5 */ OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('5') . ' AND CONCAT(' . singleQuote('06-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('5') . ' AND CONCAT(' . singleQuote('12-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('5') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('06-30') . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('5') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('12-31') . ') /*anual 6 */ OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('6') . ' AND CONCAT(' . singleQuote('12-') . ', GLPF.PAY_DAY) = ' . singleQuote($oper_mn_day) . ') OR (GLPF.CPAY_FREQUENCY = ' . singleQuote('6') . ' AND GLPF.PAY_DAY = 31 AND ' . singleQuote($oper_mn_day) . '=' . singleQuote('12-31') . '))';
        if (empty($rebuild)) {
            ClxCommon::$db->executeQuery($sql);
        }
        /* PCT of interest will calculate invoice from payment
          nota: la taza tomando el interes desde la transaccion debe
          conciderar el porcentaje que representa el interes vs la comision en este caso debe quedar (((pct com*100)/filo inte)/100)*monto */
        $sql = 'INSERT INTO ' . $this->invtname . ' (IDNAME, VENDOR_CODE, INVOICEDATE, DUE_DATE, CURRENCY_CODE, FEE_01_RATE, INVOICE_AMT, EXT_REF_NO, INVO_STAT, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, GL_TYPE, GL01_CODE, GL02_CODE, GL03_CODE, CDATE, CREATEDBY, JEHDR_CODE, BATCH_ID) SELECT DISTINCT GLPF.IDNAME, GLPF.VENDOR_CODE, FIAT.EFFTDATE INVOICEDATE, FIAT.EFFTDATE DUE_DATE, FIAT.CURRENCY_CODE, (case GLAC.GL_TYPE WHEN ' . singleQuote('LI') . ' THEN 1 ELSE -1 END)* ROUND((( (((case GLPF.SEQNUM WHEN 1 THEN FILO.FEE_01_RATE WHEN 2 THEN FILO.FEE_02_RATE WHEN 3 THEN FILO.FEE_03_RATE WHEN 4 THEN FILO.FEE_04_RATE WHEN 5 THEN FILO.FEE_05_RATE ELSE 0 END)*100)/FILO.LOAN_INTEREST))), 5) FEE_01_RATE, FIAT.TRANS_AMT INVOICE_AMT, FILO.IDCODE EXT_REF_NO, ' . "'01'" . ' INVO_STAT, FIAT.COMP_CODE, FIAT.CHART_FIELD01, FIAT.CHART_FIELD02, FIAT.CHART_FIELD03, FIAT.CHART_FIELD04, GLPF.FEE_ITEM_CODE ITEM_CODE, ' . singleQuote('TOUPDATE') . ' GL_TYPE, GLPF.FEE2_GL_CODE GL01_CODE, GLPF.FEE_GL_CODE GL02_CODE, PRAT.AR_GL_CODE GL03_CODE, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->username) . ' CREATEDBY, FIAT.IDCODE JEHDR_CODE, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->glacname . ' GLAC, ' . $this->pratname . ' PRAT, ' . $this->glpfname . ' GLPF, ' . $this->fiasname . ' FIAS, ' . $this->filoname . ' FILO, ' . $this->fiatname . ' FIAT WHERE FIAS.LOAN_CODE = FILO.IDCODE AND GLAC.IDCODE = PRAT.AR_GL_CODE AND GLAC.GLGROUP_CODE = PRAT.GLGROUP_CODE AND FIAS.IDCODE = FIAT.CHART_FIELD03 AND FILO.PRODUCT_CODE = PRAT.PRODUCT_CODE AND FIAT.COMP_CODE = ' . singleQuote($comp_code) . ' ' . (($rebuild) ? '' : ' AND FIAT.EFFTDATE = ' . singleQuote($operation_date . ' ') . '') . ' AND GLPF.FEE_FORMULA = ' . singleQuote('P0005') . ' AND PRAT.PRODUCT_CODE = GLPF.PRODUCT_CODE AND PRAT.PROD_STATUS = GLPF.PROD_STATUS AND GLPF.PROD_STATUS = FILO.LOAN_STATUS AND GLPF.STATUS = ' . singleQuote('A') . ' AND FIAT.REVERSED = 0 AND FIAT.TRANSTBL_CODE = ' . singleQuote('SYS_CRED-PYMT') . ' /*SOLO PAGOS NO AJUSTES*/ AND FILO.CLX_EFFTDATE = (SELECT MAX(FILOX.CLX_EFFTDATE) FROM CRED_CREDITS FILOX WHERE FILOX.IDCODE = FILO.IDCODE AND FILOX.CLX_EFFTDATE<=' . singleQuote($this->now) . ') AND GLPF.CPAY_FREQUENCY = ' . singleQuote('PAY') . ' AND NOT EXISTS(SELECT JEHD.ID FROM ' . $this->jehdname . ' JEHD WHERE JEHD.TRANS_CODE = FIAT.IDCODE AND JEHD.HAS_INVOICE = 1)';
        ClxCommon::$db->executeQuery($sql);
        $sql = 'DELETE FROM ' . $this->invtname . ' WHERE INVOICE_AMT <= 0 AND BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE ' . $this->invtname . ' INVT SET GL_TYPE = (SELECT MAX(GLAC.GL_TYPE) FROM ' . $this->glacname . ' GLAC WHERE GLAC.IDCODE = INVT.GL01_CODE) WHERE INVT.BATCH_ID = ' . singleQuote($this->batch_id) . ' ';
        ClxCommon::$db->executeQuery($sql);
        $apin = new Apin();
        $apin->buildInvoceFromInvt($this->batch_id, 'LI');
        unset($apin);
        $invo = new Invo();
        $invo->buildInvoceFromInvt($this->batch_id, 'AS');
        unset($invo);
        //Mark Journal Entries as invoices
        $sql = 'UPDATE ' . $this->jehdname . ' SET HAS_INVOICE = 1 WHERE IDCODE IN (SELECT DISTINCT JEHDR_CODE FROM ' . $this->invtname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ')';
        ClxCommon::$db->executeQuery($sql);
        //Accounts Recivables
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->invtname . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
    }

}
