<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-25 14:47:34 -0400 (Fri, 25 Sep 2015) $
 * @version        $Rev: 7014 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Guab {

    var $display = false;

    function __construct($procParameters = array()) {
        multiTableName($this, array('GUAB', 'GUAR', 'FICO', 'FIAS', 'GLT1', 'JCBV', 'GUAB'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->je_source = '';
    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     */
    function validate_form(&$record, &$msg) {
        $sql = 'SELECT GUAR.GUARANTEE_PCT, GUAR.AVAILABLE_PCT, GUAR.BALANCE_AMT FROM GUAR_GUARANTEES GUAR, GL_PARAMETERS_TBL GLCP WHERE GLCP.COMP_CODE = GUAR.COMP_CODE AND GUAR.IDCODE =' . singleQuote($record['GUAB_GUAR_CODE']) . ' AND GUAR.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE) FROM GUAR_GUARANTEES X WHERE X.IDCODE = GUAR.IDCODE AND X.CLX_EFFTDATE<=GLCP.OPERATION_DATE)';
        $guar = ClxCommon::$db->getOneRecord($sql);
        $guar['TRANS_AMT'] = ($guar['BALANCE_AMT'] - round((($record['GUAB_PRINCIPAL_BAL'] * ($guar['GUARANTEE_PCT'] / CLX_NUM_HUNDRED)) * ($guar['AVAILABLE_PCT'] / CLX_NUM_HUNDRED)), CLX_NUM_TWO));
        //Validar que monto de prestamo no sea mayor a monto de garantia.
        if ($guar['TRANS_AMT'] < CLX_NUM_ZERO) {
            $msg['GUAB_PRINCIPAL_BAL'] = smsg('MSG-219');
        }
        if ($record['GUAB_PRINCIPAL_BAL'] < CLX_NUM_ZERO) {
            $msg['GUAB_PRINCIPAL_BAL'] = smsg('MSG-220');
        }
    }

    function post_save($record) {
        $sql = 'SELECT DISTINCT COMP_CODE, IDCODE FROM ' . $this->guarname . ' WHERE IDCODE =' . singleQuote($record['GUAR_CODE']);
        $r = ClxCommon::$db->getOneRecord($sql);
        if (!empty($r)) {
            $this->notification($r['COMP_CODE'], $r['IDCODE']);
        }
    }

    // Amortizacion
    /**
     * Method to execute cron for interest
     *
     * @param unknown_type $comp_code
     */
    function run_notification_cron($comp_code) {
        $this->display = true;
        $this->notification($comp_code, '');
    }

    /**
     * Runs interest calculation
     *
     */
    function run_notification() {
        $idlist = getIdlist();
        if (empty($idlist))
            return;
        $sql = 'SELECT DISTINCT COMP_CODE, IDCODE FROM ' . $this->guarname . ' WHERE ID IN (' . toString($idlist) . ')';
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
            $this->notification($r[CLX_NUM_ZERO], $r[CLX_NUM_ONE]);
        }
    }

    function notification($comp_code, $guar_code = '') {
        $this->je_source = 'GUAB'; //Notificacion de saldos
        $DBLT = getTool('DBLT');
        $idname = $DBLT->getListItem('SOTY', $this->je_source);
        $idname = substr($idname, CLX_NUM_ZERO, CLX_NUM_60);
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        $comprec = $comp->comp_operation_parameters($comp_code);
        $operation_date = $comprec['OPERATION_DATE'];
        if ($this->display === true) {
            echoLog($idname . '. BATCH ID: ' . $this->batch_id);
            echoLog('Efective date: ' . $operation_date);
            echoLog('Guarantee: ' . (($guar_code === '') ? 'ALL' : $guar_code));
        }
        if (empty($comprec) || empty($comp_code)) {
            echoLog('Error, no company operation parameters found');
        }
        $DBTB = getTool('DBTB');
        $DBTB->buildTemporaryTable('GLT1'); //Clear tmp
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->glt1name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
        //loading temporary table
        $sql = 'INSERT INTO ' . $this->glt1name . ' (BATCH_ID, EFFTDATE, GLACCOUNTCODE, GL01_CODE, GL02_CODE, GL_TYPE, BALANCE_AMT, CALC_AMT_01, CALC_AMT_02, CALC_AMT_03, CALC_AMT_04, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, CURRENCY_CODE, CUR_OPERATOR, EXCHANGE_RT, DEBIT_AMT, CREDIT_AMT, LOAN_CODE, PRODUCT_CODE) SELECT ' . singleQuote($this->batch_id) . ' BATCH_ID, ' . singleQuote($operation_date) . ' EFFTDATE, PRAT.PRO_GL_CODE GLACCOUNTCODE, GUAR.GUARANTEE_PCT GL01_CODE, GUAR.AVAILABLE_PCT GL02_CODE, ' . singleQuote('AS') . ' GL_TYPE, -1*(JCBV.BALANCE_AMT+(ROUND(((GUAB.PRINCIPAL_BAL* (GUAR.GUARANTEE_PCT/100))* (GUAR.AVAILABLE_PCT/100)), 2)))	BALANCE_AMT, GUAR.AMORTIZATION_PCT CALC_AMT_01, 0 CALC_AMT_02, -1*JCBV.BALANCE_AMT CALC_AMT_03, GUAB.ID CALC_AMT_04, JCBV.CHART_FIELD01, JCBV.CHART_FIELD02, JCBV.CHART_FIELD03, JCBV.CHART_FIELD04, JCBV.CURRENCY_CODE, 1 CUR_OPERATOR, 1 EXCHANGE_RT, 0 DEBIT_AMT, 0 CREDIT_AMT, GUAR.IDCODE LOAN_CODE, GUAR.PRODUCT_CODE FROM ' . $this->guarname . ' GUAR, ' . $this->jcbvname . ' JCBV, GL_PROD_PRMTR_TBL PRAT, ' . $this->fiasname . ' FIAS, ' . $this->guabname . ' GUAB, GL_PARAMETERS_TBL GLCP WHERE GUAB.GUAR_CODE = GUAR.IDCODE AND GLCP.COMP_CODE = GUAR.COMP_CODE AND FIAS.IDCODE = JCBV.CHART_FIELD03 AND PRAT.PRODUCT_CODE = GUAR.PRODUCT_CODE AND PRAT.PRO_GL_CODE = JCBV.GLACCOUNTCODE AND GUAB.GUAR_CODE = FIAS.GUAR_CODE AND PRAT.PROD_STATUS = ' . singleQuote('VIG') . ' AND GUAR.START_DATE <= ' . singleQuote($operation_date) . ' AND GUAR.END_DATE >= ' . singleQuote($operation_date) . ' AND GUAR.COMP_CODE = ' . singleQuote($comp_code) . ' ' . (($guar_code <> '') ? ' AND GUAR.IDCODE = ' . singleQuote($guar_code) . ' ' : '') . ' AND GUAR.CLX_EFFTDATE = (SELECT MAX(X.CLX_EFFTDATE) FROM GUAR_GUARANTEES X WHERE X.IDCODE = GUAR.IDCODE AND X.CLX_EFFTDATE<=GLCP.OPERATION_DATE) AND GUAB.EFFTDATE = (SELECT MAX(X.EFFTDATE) FROM GUAR_BENEFICIARIES X WHERE X.GUAR_CODE = GUAR.IDCODE)';
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->glt1name . ' WHERE BALANCE_AMT = 0 AND BATCH_ID = ' . singleQuote($this->batch_id));
        $sql = 'SELECT DISTINCT CURRENCY_CODE, EFFTDATE OPERATION_DATE FROM ' . $this->glt1name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' ';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $rate = $comp->comp_current_exchange($comp_code, $r['CURRENCY_CODE'], $r['OPERATION_DATE']);
            $sql = 'UPDATE ' . $this->glt1name . ' SET CUR_OPERATOR = ' . singleQuote($rate['CUR_OPERATOR']) . ', EXCHANGE_RT = ' . singleQuote($rate['EXCHANGE_RT']) . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CURRENCY_CODE =' . singleQuote($r['CURRENCY_CODE']);
            ClxCommon::$db->executeQuery($sql);
        }//end while
        //Guardar monto en distribucion
        $sql = 'UPDATE ' . $this->glt1name . ' SET CALC_AMT_04 = BALANCE_AMT WHERE  BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Generando Distribucion
        $sql = 'SELECT LOAN_CODE, SUM(CALC_AMT_03) TOTAL FROM ' . $this->glt1name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' GROUP BY LOAN_CODE';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $sql = 'UPDATE ' . $this->glt1name . ' SET BALANCE_AMT = ROUND(BALANCE_AMT*(CALC_AMT_03/' . $r['TOTAL'] . '), 2) WHERE LOAN_CODE =' . singleQuote($r['LOAN_CODE']) . ' AND BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
        }
        //Ajustando rounding de distribucion
        $sql = 'SELECT LOAN_CODE, (SUM(BALANCE_AMT)-CALC_AMT_04) DIF FROM ' . $this->glt1name . '  WHERE BATCH_ID = ' . singleQuote($this->batch_id) . ' GROUP BY LOAN_CODE, CALC_AMT_04 HAVING (SUM(BALANCE_AMT)-CALC_AMT_04)<>0';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $sql = 'SELECT CHART_FIELD01, CHART_FIELD02 FROM ' . $this->glt1name . '  WHERE LOAN_CODE =' . singleQuote($r['LOAN_CODE']) . ' LIMIT 1';
            $part = ClxCommon::$db->getOneRecord($sql);
            $sql = 'UPDATE ' . $this->glt1name . ' SET BALANCE_AMT = BALANCE_AMT + ' . $r['DIF'] . ' WHERE LOAN_CODE =' . singleQuote($r['LOAN_CODE']) . ' AND BATCH_ID = ' . singleQuote($this->batch_id) . ' AND CHART_FIELD01 = ' . singleQuote($part['CHART_FIELD01']) . ' AND CHART_FIELD02 = ' . singleQuote($part['CHART_FIELD02']);
            ClxCommon::$db->executeQuery($sql);
        }
        if (!class_exists('Fiat'))
            includeCodeLib('FIAT');
        $fiat = new Fiat();
        $sql = 'SELECT EFFTDATE, EFFTDATE AVAILABILITY_DATE, BALANCE_AMT TRANS_AMT, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, CURRENCY_CODE, CUR_OPERATOR, EXCHANGE_RT, GL01_CODE GUARANTEE_PCT, GL02_CODE AVAILABLE_PCT, null FIDU_BANKACT_CODE FROM ' . $this->glt1name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $r['IDNAME'] = $idname;
            $r['LGGROUP_CODE'] = $comprec['LGGROUP_CODE'];
            $r['FISCAL_PERIOD'] = $comprec['FISCAL_PERIOD'];
            $fiat->fiat_buildVoid($comp_code, 'SYS-GUAR_DISC', $r['CHART_FIELD01'], $r['CHART_FIELD02'], $r['TRANS_AMT'], $r);
        }
        ClxCommon::$db->executeQuery('DELETE FROM ' . $this->glt1name . ' WHERE BATCH_ID = ' . singleQuote($this->batch_id));
    }

}
