<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2016-04-20 09:04:31 -0400 (Wed, 20 Apr 2016) $
 * @version        $Rev: 8138 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Loit {

    var $parameters; //Process parameters if provided.
    var $approve = true;
    var $load_credi_balances = false;

    function __construct($procParameters = array()) {
        multiTableName($this, array('COMP', 'GLCP', 'JELN', 'JEHD', 'LOIT', 'LOTT', 'JABV', 'FILO', 'JCBV', 'LEGI', 'FIAS', 'FICO', 'GLCP', 'TMP1', 'FIPA', 'FIBK', 'COMP', 'PROD', 'PRAT', 'INSI', 'FILC'));
        $this->batch_id = (!empty($procParameters['RUNID'])) ? $procParameters['RUNID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->parameters = $procParameters;
    }

//end constructor
    /**
     * Validate record information
     * Executed on table validate
     */
    function loit_validate_form(&$record, &$msg) {
        $qty = ClxCommon::$db->oneExistsRecords($this->ficoname, 'IDCODE = ' . singleQuote($record['LOIT_CONTRACT_CODE']) . ' AND CONTRACT_STATUS<>' . singleQuote('ACT') . '');
        //Validar que debitos y creditos no puede ser igual a cero.
        if ($qty > CLX_NUM_ZERO) {
            $msg['LOIT_CONTRACT_CODE'] = smsg('MSG-93');
        }
        $qty = ClxCommon::$db->coneExistsRecords($this->fipaname, 'CONTRACT_CODE = ' . singleQuote($record['LOIT_CONTRACT_CODE']) . ' AND ACCOUNTCODE = ' . singleQuote($record['LOIT_PARTICIPANT_ID']) . ' AND STATUS <> ' . singleQuote(CLX_CHAR_EMPTY_STRING) . '');
        //Validar que debitos y creditos no puede ser igual a cero.
        if ($qty > CLX_NUM_ZERO) {
            $msg['LOIT_PARTICIPANT_ID'] = smsg('MSG-94');
        }
        //Validar que exista la operacion en configuracion
        $qty = ClxCommon::$db->oneExistsRecords($this->lottname, 'EXT_TRANS_CODE = ' . singleQuote($record['LOIT_EXT_TRANS_CODE']) . ' AND EXT_ITEM_CODE = ' . singleQuote($record['LOIT_EXT_ITEM_CODE']));
        if (($record['LOIT_EXT_ITEM_CODE'] <> '' || $record['LOIT_EXT_TRANS_CODE'] <> '') && $qty == CLX_NUM_ZERO) {
            $msg['LOIT_EXT_TRANS_CODE'] = smsg('MSG-95');
            $msg['LOIT_EXT_ITEM_CODE'] = smsg('MSG-95');
        }
    }

    /**
     * Drop process exceptions
     *
     * @param string $msg
     * @param string $status
     * @param string $tablecode
     */
    function loit_report_exception($msg, $status = 'ERRO', $tablecode = '') {
        $msg .= ' ' . dict('GEN_LBL_BATCH_ID') . ' ' . $this->batch_id;
        $msg .= ' ' . dict('GEN_LBL_SCHEDULE') . ' ' . $this->parameters['SCHEDULECODE'];
        $msg .= ' ' . dict('GEN_LBL_START_DATE') . ': ' . $this->parameters['START_TIME'] . ' ';
        $PRPR = getTool('PRPR');
        $PRPR->report_exception($this->parameters['SCHEDULECODE'], $this->parameters['START_TIME'], $this->parameters['RUNID'], $msg, $tablecode, $status);
        echoLog($msg);
        //Sending alert to user alerts
        alrt_send($msg, 'now', $this->parameters['USERNAME']);
    }

    /**
     * Loan interfase process main method
     *
     * @return boolean
     */
    function loit_build() {
        echoLog('');
        echoLog('Starting LOIT (IBS FILO, FIBK Interfase) Batch-id: ' . $this->batch_id);
        //Cambiar status
        $this->loit_set_status('IDLE', 'RUN');
        $this->loit_set_status('ENDR', 'RUN');
        //Validations
        ClxFreeReturn($this->loit_build_validate());
        //Build bank accounts
        $this->loit_build_fibk();
        //Build loan record
        $this->loit_build_filo();
        //Build sub account
        $this->loit_build_fias();
        //Builds collaterals relation
        $this->loit_build_filc();
        //Build accounting for operations
        $this->loit_build_oper_accounting();
        //Build accounting for balances
        $this->loit_build_bal_accounting();
        $this->loit_set_status('RUN', 'END');
        echoLog('End LOIT');
        return true;
    }

    /**
     * Changes loan status
     *
     * @param String $from_status
     * @param String $to_status
     */
    private function loit_set_status($from_status, $to_status) {
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote($to_status) . ', LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' WHERE LOIT.REC_STATUS = ' . singleQuote($from_status) . ' ';
        /* AND EXISTS(SELECT GLCP.OPERATION_DATE FROM ' . $this->glcpname.' GLCP, ' . $this->ficoname.' FICO WHERE FICO.COMP_CODE = GLCP.COMP_CODE AND FICO.IDCODE = LOIT.CONTRACT_CODE AND LOIT.EFFTDATE<=GLCP.OPERATION_DATE) */
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Validate Interfase
     *
     */
    private function loit_build_validate() {
        $msg = array();
        $return = true;
        //Validate contract code on loans and colaterals
        $sql = 'SELECT COUNT(LOIT.ID) QTY FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXGARAN') . ') AND NOT EXISTS( SELECT FICO.ID FROM ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.CONTRACT_STATUS  IN (' . singleQuote('ACT') . ',' . singleQuote('CAN') . '))';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXGARAN') . ') AND NOT EXISTS(SELECT FICO.ID FROM ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.CONTRACT_STATUS  IN (' . singleQuote('ACT') . ',' . singleQuote('CAN') . ')) ';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-93');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar existencia de participantes en prestamos y garantias
        $sql = 'SELECT COUNT(LOIT.ID) QTY FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXGARAN') . ') AND NOT EXISTS(SELECT FIPA.ID FROM ' . $this->fipaname . ' FIPA WHERE LOIT.CONTRACT_CODE = FIPA.CONTRACT_CODE AND LOIT.PARTICIPANT_ID = FIPA.ACCOUNTCODE AND FIPA.STATUS =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXGARAN') . ') AND NOT EXISTS( SELECT FIPA.ID FROM ' . $this->fipaname . ' FIPA WHERE LOIT.CONTRACT_CODE = FIPA.CONTRACT_CODE AND LOIT.PARTICIPANT_ID = FIPA.ACCOUNTCODE AND FIPA.STATUS =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-94');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validate contract code en cuentas de banco
        $sql = 'SELECT COUNT(LOIT.ID) QTY FROM ' . $this->fibkname . ' FIBK, ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND NOT EXISTS( SELECT FICO.ID FROM ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.CONTRACT_STATUS  IN (' . singleQuote('ACT') . ',' . singleQuote('CAN') . '))';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT, ' . $this->fibkname . ' FIBK SET LOIT.REC_STATUS = ' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND NOT EXISTS(SELECT FICO.ID FROM ' . $this->ficoname . ' FICO WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.CONTRACT_STATUS  IN (' . singleQuote('ACT') . ',' . singleQuote('CAN') . ')); ';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-93');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar existencia de participantes para cuentas de banco
        $sql = 'SELECT COUNT(LOIT.ID) QTY FROM ' . $this->fibkname . ' FIBK, ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND NOT EXISTS(SELECT FIPA.ID FROM ' . $this->fipaname . ' FIPA WHERE LOIT.CONTRACT_CODE = FIPA.CONTRACT_CODE AND LOIT.PARTICIPANT_ID = FIPA.ACCOUNTCODE AND FIPA.STATUS =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->fibkname . ' FIBK, ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND NOT EXISTS( SELECT FIPA.ID FROM ' . $this->fipaname . ' FIPA WHERE LOIT.CONTRACT_CODE = FIPA.CONTRACT_CODE AND LOIT.PARTICIPANT_ID = FIPA.ACCOUNTCODE AND FIPA.STATUS =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-94');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar que exista la operacion en
        //configuracion en operaciones de credito
        $sql = 'SELECT COUNT(LOIT.ID) FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' AND NOT EXISTS( SELECT LOTT.ID FROM ' . $this->lottname . ' LOTT WHERE LOTT.EXT_TRANS_CODE = LOIT.EXT_TRANS_CODE AND LOTT.EXT_ITEM_CODE = LOIT.EXT_ITEM_CODE)';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS =' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT LOTT.ID FROM ' . $this->lottname . ' LOTT WHERE LOTT.EXT_TRANS_CODE = LOIT.EXT_TRANS_CODE AND LOTT.EXT_ITEM_CODE = LOIT.EXT_ITEM_CODE)';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-95');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar que exista codigo de moneda
        $sql = 'SELECT COUNT(LOIT.ID) FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND (LOIT.CURRENCY_CODE IS NULL OR LOIT.CURRENCY_CODE =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS =' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND (LOIT.CURRENCY_CODE IS NULL OR LOIT.CURRENCY_CODE =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-98');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar que exista codigo de producto externo
        $sql = 'SELECT COUNT(LOIT.ID) FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND (LOIT.EXT_PRODUCT_CODE IS NULL OR LOIT.EXT_PRODUCT_CODE =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS =' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND (LOIT.EXT_PRODUCT_CODE IS NULL OR LOIT.EXT_PRODUCT_CODE =' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-99');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        //Validar que existan productos definidos
        $sql = 'SELECT COUNT(LOIT.ID) FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ',' . singleQuote('CLXCREDI') . ', ' . singleQuote('CLXGARAN') . ') AND NOT EXISTS( SELECT PRAT.ID FROM ' . $this->pratname . ' PRAT WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE)';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS =' . singleQuote('END') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ',' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXGARAN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT PRAT.ID FROM ' . $this->pratname . ' PRAT WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE)';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-100');
            $this->loit_report_exception($m);
            $msg[] = $m;
            $return = false;
        }
        foreach ($msg as $key => $val)
            echoLog($val);
        return $return;
    }

    /**
     * Builds bank accounts from record
     *
     */
    private function loit_build_fibk() {
        $rec = array();
        //Load fields
        $sql = 'SELECT TF.FIELDNAME, CASE F.FIELDTYPE WHEN ' . singleQuote('DATE') . ' THEN NOW() WHEN ' . singleQuote('DATE') . ' THEN NOW() ELSE TF.FIELD_DEFAULT END FIELD_DEFAULT FROM CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE F.FIELDNAME = TF.FIELDNAME AND TF.TABLECODE = ' . singleQuote('FIBK') . ' AND TF.FIELDNAME<>' . singleQuote('ID');
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $rec[$r['FIELDNAME']] = ($r['FIELD_DEFAULT'] === '') ? 'null' : $r['FIELD_DEFAULT'];
        }
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
        setCreationData($rec);
        $sql = 'SELECT DISTINCT LOIT.PARTICIPANT_ID ACCOUNTCODE, 0 BALANCE_AMT, COMP.VE_INST_NO BANK_CODE, COMP.IDNAME BANK_NAME, LOIT.BANK_PROD_CODE, LOIT.BANK_PROD_TYPE, LOIT.EXT_BRANCH_CODE BRANCH, 0 CALC_INTEREST, FICO.COMP_CODE, LOIT.CONTRACT_CODE, COMP.COUNTRY, LOIT.CURRENCY_CODE, LOIT.IDNAME, LOIT.PRODUCT_CODE BANK_PROD_CODE, LOIT.EXT_PRODUCT_CODE BANK_ACCOUNT_NO, LOIT.EXT_PRODUCT_CODE, PRAT.PRO_GL_CODE GLACCOUNTCODE, PRAT.INC_GL_CODE INC_GL_CODE, 1 BK_INTRFSE_MODE, (SELECT MIN(INSI.IDCODE) FROM ' . $this->insiname . ' INSI) BANK_CODE FROM ' . $this->loitname . ' LOIT, ' . $this->ficoname . ' FICO, ' . $this->compname . ' COMP, ' . $this->pratname . ' PRAT, ' . $this->prodname . ' PROD WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.COMP_CODE = COMP.IDCODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND LOIT.PRODUCT_CODE = PRAT.PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND PROD.PRODUCT_CLASS IN (' . singleQuote('S') . ',' . singleQuote('C') . ') AND NOT EXISTS(SELECT FIBK.BANK_ACCOUNT_NO FROM ' . $this->fibkname . ' FIBK WHERE FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE)';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $nrec = array_merge($rec, $r);
            ClxCommon::$db->createRecord($this->fibkname, $nrec, false);
        }
        //Updating Information for does that exists
        $sql = 'SELECT DISTINCT LOIT.PARTICIPANT_ID ACCOUNTCODE, -1*LOIT.BALANCE_AMT BALANCE_AMT, LOIT.INTEARNED_AMT, LOIT.INTPAID_AMT, /*COMP.VE_INST_NO BANK_CODE,*/ COMP.IDNAME BANK_NAME, LOIT.BANK_PROD_CODE, LOIT.BANK_PROD_TYPE, LOIT.EXT_BRANCH_CODE BRANCH, 0 CALC_INTEREST, FICO.COMP_CODE, LOIT.CONTRACT_CODE, COMP.COUNTRY, LOIT.CURRENCY_CODE, LOIT.PRODUCT_CODE BANK_PROD_CODE, LOIT.IDNAME, LOIT.EXT_PRODUCT_CODE BANK_ACCOUNT_NO, LOIT.EXT_PRODUCT_CODE FROM ' . $this->fibkname . ' FIBK, ' . $this->loitname . ' LOIT, ' . $this->ficoname . ' FICO, ' . $this->compname . ' COMP WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE IN (' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND FICO.COMP_CODE = COMP.IDCODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . '';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            unset($r['IDNAME'], $r['BANK_PROD_CODE'], $r['BANK_PROD_TYPE']);
            ClxCommon::$db->updateRecord($this->fibkname, $r, 'BANK_ACCOUNT_NO = ' . singleQuote($r['BANK_ACCOUNT_NO']));
        }
    }

    /**
     * Builds Loan record from interface
     *
     */
    private function loit_build_filo() {
        //Updating Information for does that exists
        $sql = 'SELECT LOIT.EXT_PRODUCT_CODE IDCODE, LOIT.ADJ_POINTS_OVERINDEX, LOIT.CURRENCY_CODE, FILO.CLX_EFFTDATE, LOIT.EXT_PRODUCT_CODE, LOIT.GRACE_PERIOD_DAYS, LOIT.INTEREST_CALC, LOIT.INTEREST_TYPE, LOIT.LOAN_AMT, LOIT.LOAN_INTEREST, LOIT.LOAN_PERIOD, LOIT.LOAN_STATUS, LOIT.PAYMENT_FREQ, LOIT.PERIOD_TYPE, LOIT.PRODUCT_CODE, LOIT.START_AMT LOAN_AMT, LOIT.LOAN_AMT PRINCIPAL_BAL, LOIT.BALANCE_AMT BALANCE_AMT, FILO.INTEARNED_AMT PREV_INTEARNED_AMT, LOIT.INTEARNED_AMT INTEARNED_AMT, LOIT.INTPAID_AMT INTPAID_AMT, LOIT.LOAN_INTEREST LOAN_INTEREST, LOIT.LOAN_PERIOD LOAN_PERIOD, LOIT.LOAN_PERIOD NO_PAYMENTS, LOIT.LATE_FEE_BALANCE, FILO.LATE_FEE_BALANCE PREV_LATE_FEE_AMT, LOIT.COMMISSIONS_AMT, FILO.COMMISSIONS_AMT PREV_COMM_AMT FROM ' . $this->glcpname . ' GLCP, ' . $this->loitname . ' LOIT, ' . $this->filoname . ' FILO WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND GLCP.COMP_CODE = FILO.COMP_CODE AND FILO.IDCODE = LOIT.EXT_PRODUCT_CODE AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND FILO.CLX_EFFTDATE = (SELECT MAX(FILO1.CLX_EFFTDATE) FROM ' . $this->filoname . ' FILO1 WHERE FILO1.IDCODE = FILO.IDCODE AND FILO1.CLX_EFFTDATE<=GLCP.OPERATION_DATE) ';
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($result = ClxCommon::$db->getNextRecord($rs))) {
            ClxCommon::$db->updateRecord($this->filoname, $result, 'IDCODE = ' . singleQuote($result['IDCODE']) . ' AND CLX_EFFTDATE = ' . singleQuote($result['CLX_EFFTDATE']));
        }
        //Creating new entries creation must be later to asign paid as initial earning
        //and prevent double accounting
        $r = array('IDNAME' => dict('FICO_CODE_LOANS'),
            'LOAN_TYPE' => 'OTH',
            'INTEREST_TYPE' => 'FIX',
            'LOAN_INTEREST' => '0',
            'LOAN_PERIOD' => 'M',
            'PERIOD_TYPE' => 'M',
            'ADJ_PATTERN' => '0',
            'ADJ_CAPPOINTS_YEAR' => '0',
            'ADJ_CAPPOINTS_LIFE' => '0',
            'ADJ_INDEX' => 'N/A',
            'ADJ_POINTS_OVERINDEX' => CLX_NUM_ZERO,
            'PENALTY_RATE' => CLX_NUM_ZERO,
            'PENALTY_CALC_TYPE' => 'N/A',
            'GRACE_PERIOD_DAYS' => CLX_NUM_ZERO,
            'LOAN_PYMT_AMT' => CLX_NUM_ZERO,
            'PAYMENT_FREQ' => CLX_NUM_TWO,
            'PRINCIPAL_BAL' => CLX_NUM_ZERO,
            'INTEREST_CALC' => '360',
            'CREATEDBY' => $this->username, 'CDATE' => $this->now, 'MODIFIEDBY' => $this->username, 'MDATE' => $this->now, 'LOAN_STATUS' => 'VIG',
            'GL01_CODE' => 'null',
            'GL02_CODE' => 'null',
            'CALC_INTEREST' => '0',
            'LOAN_INTEREST' => '0',
            'EARNED_INT_AMT' => '0',
            'NO_PAYMENTS' => '0');
        /* colocando pagado en caso de no ser cero */
        $sql = 'SELECT DISTINCT GLCP.OPERATION_DATE CLX_EFFTDATE, LOIT.EXT_PRODUCT_CODE IDCODE, LOIT.EXT_PRODUCT_CODE, MAX(LOIT.PRODUCT_CODE) PRODUCT_CODE, MAX(FICO.COMP_CODE) COMP_CODE, MAX(LOIT.PARTICIPANT_ID) DEBTOR_CODE, MAX(LOIT.CURRENCY_CODE) CURRENCY_CODE, SUM(LOIT.START_AMT) LOAN_AMT, SUM(LOIT.LOAN_AMT) PRINCIPAL_BAL, SUM(LOIT.BALANCE_AMT) BALANCE_AMT, SUM(LOIT.INTEARNED_AMT) INTEARNED_AMT, SUM(LOIT.INTPAID_AMT) INTPAID_AMT, SUM(LOIT.INTPAID_AMT) PREV_INTEARNED_AMT, SUM(LOIT.LATE_FEE_BALANCE) LATE_FEE_BALANCE, SUM(LOIT.LATE_FEE_BALANCE) PREV_LATE_FEE_AMT, SUM(LOIT.COMMISSIONS_AMT) COMMISSIONS_AMT, SUM(LOIT.COMMISSIONS_AMT) PREV_COMM_AMT, MAX(LOIT.LOAN_PERIOD) LOAN_PERIOD, MAX(LOIT.LOAN_PERIOD) NO_PAYMENTS FROM ' . $this->loitname . ' LOIT, ' . $this->ficoname . ' FICO, ' . $this->glcpname . ' GLCP WHERE FICO.IDCODE = LOIT.CONTRACT_CODE AND FICO.COMP_CODE = GLCP.COMP_CODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ') AND NOT EXISTS(SELECT FILO.IDCODE FROM ' . $this->filoname . ' FILO WHERE FILO.IDCODE = LOIT.EXT_PRODUCT_CODE AND FILO.CLX_EFFTDATE<=GLCP.OPERATION_DATE) GROUP BY LOIT.EXT_PRODUCT_CODE';
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($result = ClxCommon::$db->getNextRecord($rs))) {
            $result = array_merge($result, $r);
            ClxCommon::$db->createRecord($this->filoname, $result, false);
        }
    }

    /**
     * Builds subaccount record from interface if not exists
     *
     */
    private function loit_build_fias() {
        if (!class_exists('Fias'))
            includeCodeLib('FIAS');
        $fias = new Fias();
        $sql = 'SELECT DISTINCT LOIT.EXT_PRODUCT_CODE LOAN_CODE, LOIT.EXT_PRODUCT_CODE, ' . singleQuote('AS') . ' GL_TYPE, ' . singleQuote('CR') . ' TRUST_ITEM_TYPE, LOIT.CONTRACT_CODE, LOIT.PARTICIPANT_ID ACCOUNTCODE, LOIT.CURRENCY_CODE, LOIT.IDNAME FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ') AND LOIT.BATCH_ID=' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT FIAS.EXT_PRODUCT_CODE FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) UNION SELECT DISTINCT LOIT.EXT_PRODUCT_CODE LOAN_CODE, LOIT.EXT_PRODUCT_CODE, ' . singleQuote('A') . ' GL_TYPE, ' . singleQuote('C') . ' TRUST_ITEM_TYPE, LOIT.CONTRACT_CODE, LOIT.PARTICIPANT_ID ACCOUNTCODE, LOIT.CURRENCY_CODE, LOIT.IDNAME FROM ' . $this->fibkname . ' FIBK, ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ') AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.CONTRACT_CODE <>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.BATCH_ID=' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT FIAS.EXT_PRODUCT_CODE FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) UNION /*Garantias*/ SELECT DISTINCT LOIT.EXT_PRODUCT_CODE LOAN_CODE, LOIT.EXT_PRODUCT_CODE, ' . singleQuote('A') . ' GL_TYPE, ' . singleQuote('G') . ' TRUST_ITEM_TYPE, LOIT.CONTRACT_CODE, LOIT.PARTICIPANT_ID ACCOUNTCODE, LOIT.CURRENCY_CODE, PROD.IDNAME FROM ' . $this->loitname . ' LOIT, ' . $this->prodname . ' PROD WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND PROD.IDCODE = LOIT.PRODUCT_CODE AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXGARAN') . ') AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.CONTRACT_CODE <>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.BATCH_ID=' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT FIAS.EXT_PRODUCT_CODE FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE)';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if ($r['TRUST_ITEM_TYPE'] === 'CA') {
                $r['FIDU_BANKACT_CODE'] = $r['LOAN_CODE'];
                $r['LOAN_CODE'] = 'null';
            } elseif ($r['TRUST_ITEM_TYPE'] === 'GA') {
                $r['ASSET_CODE'] = $r['LOAN_CODE'];
                $r['LOAN_CODE'] = 'null';
            } else {
                $r['FIDU_BANKACT_CODE'] = 'null';
            }
            $r['PORTFOLIO_CODE'] = 'null';
            $fias->fias_build($r);
        }
    }

    /**
     * Adds collater and loan relation
     *
     */
    private function loit_build_filc() {
        $sql = 'INSERT INTO ' . $this->filcname . ' (FIAS_COLL_CODE, FIAS_LOAN_CODE) SELECT FIAS.IDCODE FIAS_COLL_CODE, FIAS2.IDCODE FIAS_LOAN_CODE FROM ' . $this->loitname . ' LOIT, ' . $this->fiasname . ' FIAS, ' . $this->fiasname . ' FIAS2 WHERE LOIT.CONTRACT_CODE = FIAS.CONTRACT_CODE AND FIAS.CONTRACT_CODE = FIAS2.CONTRACT_CODE AND LOIT.EXT_PRODUCT_CODE = FIAS.EXT_PRODUCT_CODE AND LOIT.EXT_PROD01_CODE = FIAS2.EXT_PRODUCT_CODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXGARAN') . ' AND LOIT.BATCH_ID=' . singleQuote($this->batch_id) . ' AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND NOT EXISTS (SELECT FILC.ID FROM ' . $this->filcname . ' FILC WHERE FILC.FIAS_COLL_CODE = FIAS.IDCODE AND FILC.FIAS_LOAN_CODE = FIAS2.IDCODE)';
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Sets default values after import
     *
     */
    private function loit_build_oper_accounting() {
        $je_source = 'LOIT';
        $idname = dict('FICO_CODE_LOANS');
        //Marcando en error aquellos registros que no cuentan
        //con prestamo definido
        //Validar que exista codigo de producto externo o excepcion a registros
        $sql = 'SELECT COUNT(LOIT.ID) FROM ' . $this->loitname . ' LOIT WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' AND NOT EXISTS(SELECT X.ID FROM FIDU_ITEMS X WHERE X.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE)';
        if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
            $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS =' . singleQuote('END') . ' WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' AND NOT EXISTS(SELECT X.ID FROM ' . $this->filoname . ' X WHERE X.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE)';
            ClxCommon::$db->executeQuery($sql);
            $m = smsg('MSG-101');
            $this->loit_report_exception($m);
            echoLog($m);
        }
        //Actualizando informacion de creditos en cuanto a contrato y participante
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.CONTRACT_CODE = (SELECT MAX(FIAS.CONTRACT_CODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' ';
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.PARTICIPANT_ID = (SELECT MAX(FIPA.ACCOUNTCODE) FROM ' . $this->fipaname . ' FIPA WHERE FIPA.CONTRACT_CODE = LOIT.CONTRACT_CODE AND FIPA.ISPRIMARY = 1) WHERE LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPAGOS') . ' ';
        ClxCommon::$db->executeQuery($sql);
        if (!class_exists('Jehd'))
            includeCodeLib('JEHD');
        $jehd = new Jehd();
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        //Build Header for this process, one pre currency
        $sql = 'SELECT DISTINCT GLCP.OPERATION_DATE, GLCP.COMP_CODE FROM ' . $this->loitname . ' LOIT, ' . $this->lottname . ' LOTT, ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS, ' . $this->filoname . ' FILO, ' . $this->glcpname . ' GLCP WHERE GLCP.COMP_CODE = FICO.COMP_CODE AND FILO.IDCODE = FIAS.LOAN_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE AND LOIT.EXT_PRODUCT_CODE = FILO.EXT_PRODUCT_CODE AND LOTT.EXT_TRANS_CODE = LOIT.EXT_TRANS_CODE AND LOTT.EXT_ITEM_CODE = LOIT.EXT_ITEM_CODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND ( LOIT.DEBIT_AMT<> 0 OR  LOIT.CREDIT_AMT<>0)';
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $gl_rec = $jehd->jehd_init_record();
            $gl_rec['IDNAME'] = $idname;
            $gl_rec['JE_SOURCE'] = $je_source;
            $gl_rec['EFFTDATE'] = $result['OPERATION_DATE'];
            $gl_rec['COMP_CODE'] = $result['COMP_CODE'];
            $gl_rec['TRANS_CODE'] = 'null';
            $gl_rec['REQ_CHANNEL'] = 'SYS';
            $gl_rec['BATCH_ID'] = $this->batch_id;
            $jehd->jehd_build($gl_rec);
            //Build debit lines
            $sql = 'INSERT INTO ' . $this->jelnname . '  (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, LOTT.CREDIT_GL_CODE GLACCOUNTCODE, LOTT.ITEM_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, FIAS.IDCODE CHART_FIELD03, null CHART_FIELD04, LOIT.EXT_PRODUCT_CODE ITEM_REF_NO, 1 SEQNUM, LOIT.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, LOIT.DEBIT_AMT DEBIT_AMT, LOIT.CREDIT_AMT CREDIT_AMT FROM ' . $this->loitname . ' LOIT, ' . $this->lottname . ' LOTT, ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS, ' . $this->filoname . ' FILO, ' . $this->glcpname . ' GLCP WHERE GLCP.COMP_CODE = FICO.COMP_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE AND LOTT.EXT_TRANS_CODE = LOIT.EXT_TRANS_CODE AND LOTT.EXT_ITEM_CODE = LOIT.EXT_ITEM_CODE AND FILO.IDCODE = FIAS.LOAN_CODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND FICO.COMP_CODE = ' . singleQuote($result['COMP_CODE']) . ' AND ( LOIT.DEBIT_AMT<> 0 OR  LOIT.CREDIT_AMT<>0) UNION ALL SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, LOTT.DEBIT_GL_CODE GLACCOUNTCODE, LOTT.ITEM_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, FIAS.IDCODE CHART_FIELD03, null CHART_FIELD04, LOIT.EXT_PRODUCT_CODE ITEM_REF_NO, 2 SEQNUM, LOIT.CURRENCY_CODE, 1 EXCHANGE_RT, 1 CUR_OPERATOR, LOIT.CREDIT_AMT DEBIT_AMT, LOIT.DEBIT_AMT CREDIT_AMT FROM ' . $this->loitname . ' LOIT, ' . $this->lottname . ' LOTT, ' . $this->fiasname . ' FIAS, ' . $this->ficoname . ' FICO, ' . $this->filoname . ' FILO, ' . $this->glcpname . ' GLCP WHERE GLCP.COMP_CODE = FICO.COMP_CODE AND LOTT.EXT_TRANS_CODE = LOIT.EXT_TRANS_CODE AND LOTT.EXT_ITEM_CODE = LOIT.EXT_ITEM_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FILO.IDCODE = FIAS.LOAN_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND FICO.COMP_CODE = ' . singleQuote($result['COMP_CODE']) . ' AND ( LOIT.DEBIT_AMT<> 0 OR  LOIT.CREDIT_AMT<>0)';
            ClxCommon::$db->executeQuery($sql);
            //Updating Exchange rates and
            $sql = 'SELECT DISTINCT JEHD.COMP_CODE, JELN.CURRENCY_CODE, JEHD.EFFTDATE FROM ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN WHERE JEHD.IDCODE = JELN.JEHDR_CODE AND JEHD.IDCODE = ' . singleQuote($gl_rec['IDCODE']);
            $rs = ClxCommon::$db->executeQuery($sql);
            while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
                $xrate = $comp->comp_current_exchange($r[CLX_NUM_ZERO], $r[CLX_NUM_ONE], $r[CLX_NUM_TWO]);
                if (empty($xrate['EXCHANGE_RT']))
                    $xrate['EXCHANGE_RT'] = CLX_NUM_ONE;
                if (empty($xrate['CUR_OPERATOR']))
                    $xrate['CUR_OPERATOR'] = CLX_NUM_ONE;
                ClxCommon::$db->updateRecord($this->jelnname, array('EXCHANGE_RT' => $xrate['EXCHANGE_RT'], 'CUR_OPERATOR' => $xrate['CUR_OPERATOR']), 'JEHDR_CODE = ' . singleQuote($gl_rec['IDCODE']) . ' AND CURRENCY_CODE = ' . singleQuote($r[CLX_NUM_ONE]));
            }
            //Aprovar asiento de ser necesario
            if (!empty($this->approve))
                $jehd->jehd_approve_by_code($gl_rec['IDCODE']);
        }
    }

    /**
     * Ajuste products according balances
     *
     */
    private function loit_build_bal_accounting() {

        $je_source = 'SYS';
        if (!class_exists('Jehd'))
            includeCodeLib('JEHD');
        $jehd = new Jehd();
        if (!class_exists('Comp'))
            includeCodeLib('COMP');
        $comp = new Comp();
        /* CLEAR ANY POSIBLE RECORD */
        $DBTB = getTool('DBTB');
        $DBTB->buildTemporaryTable('TMP1');
        $sql = 'DELETE FROM ' . $this->tmp1name;
        ClxCommon::$db->executeQuery($sql);
        //Lineas para productos de cuenta de banco CLXPRODU
        $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) CHART_FIELD03, LOIT.CURRENCY_CODE, IFNULL(PRAT.GL01_CODE, (SELECT MAX(TTPA.EQ_GL_CODE) FROM FIDU_PARAMETERS_TBL TTPA, ' . $this->ficoname . ' FICO WHERE FICO.FI_TYPE = TTPA.FI_TYPE AND FICO.CONTRACT_TYPE = TTPA.CONTRACT_TYPE AND TTPA.TRUST_FUND_SRC = FICO.TRUST_FUND_SRC AND FICO.IDCODE = LOIT.CONTRACT_CODE AND TTPA.CONTRACT_TYPE = FICO.CONTRACT_TYPE)) EQ_GL_CODE, PRAT.PRO_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, -1*ROUND((LOIT.BALANCE_AMT+LOIT.LOAN_AMT), 2) NEW_BALANCE_AMT, IFNULL((SELECT SUM(JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV, ' . $this->fiasname . ' FIAS WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.PRO_GL_CODE),0) CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ') AND NOT EXISTS(SELECT FIBK.ID FROM FIDU_BANK_ACCOUNTS FIBK WHERE FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE IN (' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')) AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Solo para carga inicial de prestamos luego se apaga ya que
        //llegan por interfase CLXPAGOS
        if ($this->load_credi_balances) {
            $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) CHART_FIELD03, LOIT.CURRENCY_CODE, IFNULL(PRAT.GL01_CODE, (SELECT MAX(TTPA.EQ_GL_CODE) FROM FIDU_PARAMETERS_TBL TTPA, ' . $this->ficoname . ' FICO WHERE FICO.FI_TYPE = TTPA.FI_TYPE AND FICO.CONTRACT_TYPE = TTPA.CONTRACT_TYPE AND TTPA.TRUST_FUND_SRC = FICO.TRUST_FUND_SRC AND FICO.IDCODE = LOIT.CONTRACT_CODE)) EQ_GL_CODE, PRAT.PRO_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, ROUND((LOIT.BALANCE_AMT+LOIT.LOAN_AMT), 2) NEW_BALANCE_AMT, IFNULL((SELECT SUM(JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV, ' . $this->fiasname . ' FIAS WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.PRO_GL_CODE),0) CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT WHERE 1=1 AND PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND NOT EXISTS(SELECT FIBK.ID FROM FIDU_BANK_ACCOUNTS FIBK WHERE FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE IN (' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')) AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
            ClxCommon::$db->executeQuery($sql);
        }
        //Lineas de devengo de cuenta de banco
        $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 1 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) CHART_FIELD03, LOIT.CURRENCY_CODE, (SELECT MAX(TTPA.EQ_GL_CODE) FROM FIDU_PARAMETERS_TBL TTPA, ' . $this->ficoname . ' FICO WHERE FICO.FI_TYPE = TTPA.FI_TYPE AND FICO.CONTRACT_TYPE = TTPA.CONTRACT_TYPE AND TTPA.TRUST_FUND_SRC = FICO.TRUST_FUND_SRC AND FICO.IDCODE = LOIT.CONTRACT_CODE) EQ_GL_CODE, PRAT.INC_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, -1*(IFNULL(LOIT.INTPAID_AMT-(SELECT SUM(INI_INTPAID_AMT) FROM ' . $this->fibkname . ' FIBK WHERE FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE),0)) NEW_BALANCE_AMT, IFNULL((SELECT (case JABV.GL_TYPE WHEN ' . singleQuote('L') . ' THEN -1 ELSE 1 END)*SUM(JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV, ' . $this->fiasname . ' FIAS WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.INC_GL_CODE),0) CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT WHERE 1=1 AND PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND NOT EXISTS(SELECT FIBK.ID FROM FIDU_BANK_ACCOUNTS FIBK WHERE FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE IN (' . singleQuote(CLX_CHAR_EMPTY_STRING) . ',' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')) AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Lineas para devengo de prestamos CLXCREDI
        $sql = 'INSERT INTO ' . $this->tmp1name . '  ( REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, FIAS.IDCODE CHART_FIELD03, LOIT.CURRENCY_CODE, PRAT.INC_GL_CODE EQ_GL_CODE, PRAT.AR_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, ROUND(LOIT.INTEARNED_AMT, 2) NEW_BALANCE_AMT, /*ROUND(LOIT.INTEARNED_AMT-LOIT.INTPAID_AMT, 2) NEW_BALANCE_AMT,*/ ROUND(FILO.PREV_INTEARNED_AMT, 2) /*IFNULL((SELECT SUM((case JABV.GL_TYPE WHEN ' . singleQuote('L') . ' THEN -1 ELSE 1 END)*JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.AR_GL_CODE),0)*/ CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT, ' . $this->filoname . ' FILO, ' . $this->fiasname . ' FIAS WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND FILO.IDCODE = LOIT.EXT_PRODUCT_CODE AND LOIT.CONTRACT_CODE = FIAS.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Lineas de mora para de prestamos CLXCREDI
        $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, (SELECT MAX(FIAS.IDCODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE) CHART_FIELD03, LOIT.CURRENCY_CODE, PRAT.LATEF2_GL_CODE EQ_GL_CODE, PRAT.LATEF_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, ROUND(LOIT.LATE_FEE_BALANCE, 2) NEW_BALANCE_AMT, ROUND(FILO.PREV_LATE_FEE_AMT, 2) /*IFNULL((SELECT SUM((case JABV.GL_TYPE WHEN ' . singleQuote('L') . ' THEN -1 ELSE 1 END)*JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.LATEF_GL_CODE),0)*/ CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT, ' . $this->filoname . ' FILO, ' . $this->fiasname . ' FIAS WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND FILO.IDCODE = LOIT.EXT_PRODUCT_CODE AND LOIT.CONTRACT_CODE = FIAS.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Lineas de comisiones para de prestamos CLXCREDI
        $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, PROD.IDCODE, PROD.IDNAME, NOW() EFFTDATE, (SELECT MAX(FICO.COMP_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIAS.CONTRACT_CODE = FICO.IDCODE) COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, FIAS.IDCODE CHART_FIELD03, LOIT.CURRENCY_CODE, PRAT.FEE2_GL_CODE EQ_GL_CODE, PRAT.FEE_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, ROUND(LOIT.COMMISSIONS_AMT, 2) NEW_BALANCE_AMT, ROUND(FILO.PREV_COMM_AMT, 2) /*IFNULL((SELECT SUM((case JABV.GL_TYPE WHEN ' . singleQuote('L') . ' THEN -1 ELSE 1 END)*JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.FEE_GL_CODE),0)*/ CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT, ' . $this->filoname . ' FILO, ' . $this->fiasname . ' FIAS WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND LOIT.CONTRACT_CODE = FIAS.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FILO.IDCODE = LOIT.EXT_PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXCREDI') . ' AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        /*
         * NOTA: REC_TYPE CLASIFICA COMO CONTABLILIZARA, EN CASO DE
         * QUERE CONTABILIZAR DEBITO, CREDITO USAR TIPO 2 Y COLOCAR
         * EQ_GL_CODE COMO CUENTA DE DEBITO Y CAMPO GLACCOUNTCODE
         */
        //lineas de interfaz de garantias CLXGARAN
        $sql = 'INSERT INTO ' . $this->tmp1name . '  (  REC_TYPE, IDCODE, IDNAME, EFFTDATE, COMP_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CURRENCY_CODE, EQ_GL_CODE, GLACCOUNTCODE, EXCHANGE_RT, CUR_OPERATOR, NEW_BALANCE_AMT, CUR_BALANCE_AMT, DIF_BALANCE_AMT, DEBIT_AMT, CREDIT_AMT, BATCH_ID) SELECT DISTINCT 0 REC_TYPE, MAX(PROD.IDCODE) IDCODE, MAX(PROD.IDNAME) IDNAME, NOW() EFFTDATE, FICO.COMP_CODE, LOIT.CONTRACT_CODE CHART_FIELD01, LOIT.PARTICIPANT_ID CHART_FIELD02, FIAS.IDCODE CHART_FIELD03, LOIT.CURRENCY_CODE, MAX(IFNULL(PRAT.GL01_CODE, (SELECT MAX(TTPA.EQ_GL_CODE) FROM FIDU_PARAMETERS_TBL TTPA, ' . $this->ficoname . ' FICO WHERE FICO.FI_TYPE = TTPA.FI_TYPE AND FICO.CONTRACT_TYPE = TTPA.CONTRACT_TYPE AND TTPA.TRUST_FUND_SRC = FICO.TRUST_FUND_SRC AND FICO.IDCODE = LOIT.CONTRACT_CODE))) EQ_GL_CODE, PRAT.PRO_GL_CODE GLACCOUNTCODE, 1.00000 EXCHANGE_RT, 1 CUR_OPERATOR, ROUND(SUM(LOIT.BALANCE_AMT), 2) NEW_BALANCE_AMT, IFNULL((SELECT SUM((case JABV.GL_TYPE WHEN ' . singleQuote('L') . ' THEN -1 ELSE 1 END)*JABV.BALANCE_AMT) FROM ' . $this->jabvname . ' JABV WHERE JABV.CHART_FIELD01 = LOIT.CONTRACT_CODE AND JABV.CHART_FIELD02 = LOIT.PARTICIPANT_ID AND JABV.CHART_FIELD03 = FIAS.IDCODE AND JABV.CURRENCY_CODE = LOIT.CURRENCY_CODE AND JABV.GLACCOUNTCODE = PRAT.PRO_GL_CODE),0) CUR_BALANCE_AMT, 0.00 DIF_BALANCE_AMT, 0.00 DEBIT_AMT, 0.00 CREDIT_AMT, ' . singleQuote($this->batch_id) . ' BATCH_ID FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS, ' . $this->prodname . ' PROD, ' . $this->loitname . ' LOIT, ' . $this->pratname . ' PRAT WHERE PRAT.PRODUCT_CODE = LOIT.PRODUCT_CODE AND FICO.IDCODE = FIAS.CONTRACT_CODE AND FIAS.CONTRACT_CODE = LOIT.CONTRACT_CODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND LOIT.PRODUCT_CODE = PROD.IDCODE AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXGARAN') . ' AND LOIT.CONTRACT_CODE IS NOT null AND LOIT.REC_STATUS = ' . singleQuote('RUN') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' GROUP BY FICO.COMP_CODE, LOIT.CONTRACT_CODE, LOIT.PARTICIPANT_ID, FIAS.IDCODE, LOIT.CURRENCY_CODE, PRAT.PRO_GL_CODE';
        ClxCommon::$db->executeQuery($sql);
        /* UPDATE DIFERENCES AND REMOVE NO CHANGE AND SET DEBITS OR CREDITS */
        $sql = 'UPDATE ' . $this->tmp1name . '  SET DIF_BALANCE_AMT = NEW_BALANCE_AMT - CUR_BALANCE_AMT WHERE BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        $sql = 'DELETE FROM ' . $this->tmp1name . '  WHERE DIF_BALANCE_AMT = 0 AND BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        /* SET PRODUCT DEBITS AND CREDITS */
        $sql = 'UPDATE ' . $this->tmp1name . '  SET DEBIT_AMT = DIF_BALANCE_AMT WHERE DIF_BALANCE_AMT>0 AND REC_TYPE = 0 AND BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE ' . $this->tmp1name . '  SET CREDIT_AMT = ABS(DIF_BALANCE_AMT) WHERE DIF_BALANCE_AMT<0  AND REC_TYPE = 0 AND BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        /* SET INCOME DEBITS AND CREDITS */
        $sql = 'UPDATE ' . $this->tmp1name . '  SET CREDIT_AMT = DIF_BALANCE_AMT WHERE DIF_BALANCE_AMT>0 AND REC_TYPE = 1 AND BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE ' . $this->tmp1name . '  SET DEBIT_AMT = ABS(DIF_BALANCE_AMT) WHERE DIF_BALANCE_AMT<0  AND REC_TYPE = 1 AND BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        /* SET INCOME DEBITS AND CREDITS */
        $sql = 'UPDATE FIDU_INTERFASE_TMP SET DEBIT_AMT = DIF_BALANCE_AMT WHERE DIF_BALANCE_AMT>0 AND REC_TYPE = 2';
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE FIDU_INTERFASE_TMP SET CREDIT_AMT = ABS(DIF_BALANCE_AMT) WHERE DIF_BALANCE_AMT<0  AND REC_TYPE = 2';
        ClxCommon::$db->executeQuery($sql);
        $sql = 'UPDATE FIDU_INTERFASE_TMP X SET X.EFFTDATE = (SELECT GLCP.OPERATION_DATE FROM GL_PARAMETERS_TBL GLCP WHERE GLCP.COMP_CODE = X.COMP_CODE) WHERE BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        switch (kDatabaseType) {
            case 'oci':
                ClxCommon::$db->executeQuery('CREATE OR REPLACE TABLE DEBUG_TMP SELECT * FROM FIDU_INTERFASE_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id));
                break;
            default:
                ClxCommon::$db->executeQuery('DROP TABLE IF EXISTS DEBUG_TMP');
                ClxCommon::$db->executeQuery('CREATE TABLE DEBUG_TMP SELECT * FROM FIDU_INTERFASE_TMP WHERE BATCH_ID = ' . singleQuote($this->batch_id));
        }
        /* ACCORDING GL_TYPE MOVE DEBITS AND CREDITS ACCORDINGLY */
        /* UPDATING EXCHANGE RATES */
        $sql = 'SELECT DISTINCT COMP_CODE, CURRENCY_CODE, EFFTDATE FROM ' . $this->tmp1name . '  WHERE BATCH_ID=' . singleQuote($this->batch_id);
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($r = ClxCommon::$db->getNextRecord($rs, NUMERIC))) {
            $xrate = $comp->comp_current_exchange($r[CLX_NUM_ZERO], $r[CLX_NUM_ONE], $r[CLX_NUM_TWO]);
            if (empty($xrate['EXCHANGE_RT']))
                $xrate['EXCHANGE_RT'] = CLX_NUM_ONE;
            if (empty($xrate['CUR_OPERATOR']))
                $xrate['CUR_OPERATOR'] = CLX_NUM_ONE;
            ClxCommon::$db->updateRecord($this->tmp1name, array('EXCHANGE_RT' => $xrate['EXCHANGE_RT'], 'CUR_OPERATOR' => $xrate['CUR_OPERATOR']), 'CURRENCY_CODE = ' . singleQuote($r[CLX_NUM_ONE]) . ' AND BATCH_ID = ' . singleQuote($this->batch_id));
        }
        /* SELECCIONANDO CANDIDATOS PARA ENCABEZADO */
        $sql = 'SELECT DISTINCT COMP_CODE, EFFTDATE, IDNAME FROM ' . $this->tmp1name . '  WHERE BATCH_ID=' . singleQuote($this->batch_id) . ' AND (DEBIT_AMT<>0 OR CREDIT_AMT<>0)';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            //pre(print_r($r));
            /* GENERARDO ASIENTO CONTABLE */
            $gl_rec = $jehd->jehd_init_record();
            $gl_rec['IDNAME'] = $r['IDNAME'];
            $gl_rec['JE_SOURCE'] = $je_source;
            $gl_rec['EFFTDATE'] = $r['EFFTDATE'];
            $gl_rec['COMP_CODE'] = $r['COMP_CODE'];
            $gl_rec['TRANS_CODE'] = 'null';
            $gl_rec['REQ_CHANNEL'] = 'SYS';
            $gl_rec['BATCH_ID'] = $this->batch_id;
            $jehd->jehd_build($gl_rec);
            /* LINEAS DE ASIENTO ACTIVOS */
            $sql = 'INSERT INTO ' . $this->jelnname . '  (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 1 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.DEBIT_AMT, X.CREDIT_AMT FROM ' . $this->tmp1name . ' X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND GLAC.GL_TYPE NOT IN (' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ') AND X.BATCH_ID=' . singleQuote($this->batch_id) . ' AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE<>2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            /* LINEAS DE ASIENTO PASIVOS */
            $sql = 'INSERT INTO ' . $this->jelnname . '  (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 2 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.CREDIT_AMT, X.DEBIT_AMT FROM ' . $this->tmp1name . ' X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND GLAC.GL_TYPE IN (' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ') AND X.BATCH_ID=' . singleQuote($this->batch_id) . ' AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE<>2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            /* CONTRAPARTIDA CON SUMATORIA DE REGISTROS ACTIVOS */
            $sql = 'INSERT INTO ' . $this->jelnname . '  (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.EQ_GL_CODE GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 3 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.CREDIT_AMT, X.DEBIT_AMT FROM ' . $this->tmp1name . ' X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND GLAC.GL_TYPE NOT IN (' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ') AND X.BATCH_ID=' . singleQuote($this->batch_id) . ' AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE<>2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            /* LINEAS DE ASIENTO PASIVOS CONTRAPARTIDA */
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.EQ_GL_CODE GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 4 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.DEBIT_AMT, X.CREDIT_AMT FROM ' . $this->tmp1name . ' X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND GLAC.GL_TYPE IN (' . singleQuote('LI') . ',' . singleQuote('IN') . ',' . singleQuote('EX') . ') AND X.BATCH_ID=' . singleQuote($this->batch_id) . ' AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE<>2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            //Procesando casos de REC_TYPE 2 donde registrs se operan db y cr
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 5 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.DEBIT_AMT, X.CREDIT_AMT FROM FIDU_INTERFASE_TMP X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE=2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            //Contrapartida
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, GLACCOUNTCODE, ITEM_CODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, SEQNUM, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_AMT, CREDIT_AMT) SELECT ' . singleQuote($gl_rec['IDCODE']) . ' JEHDR_CODE, X.EQ_GL_CODE GLACCOUNTCODE, null ITEM_CODE, X.CHART_FIELD01, X.CHART_FIELD02, X.CHART_FIELD03, null CHART_FIELD04, X.CHART_FIELD02 ITEM_REF_NO, 6 SEQNUM, X.CURRENCY_CODE, X.EXCHANGE_RT, X.CUR_OPERATOR, X.CREDIT_AMT, X.DEBIT_AMT FROM FIDU_INTERFASE_TMP X, GL_PARAMETERS_TBL GLCP, GL_CODES_TBL GLAC WHERE GLAC.IDCODE = X.GLACCOUNTCODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = X.COMP_CODE AND X.IDNAME = ' . singleQuote($r['IDNAME']) . ' AND X.REC_TYPE=2 AND (X.DEBIT_AMT<>0 OR X.CREDIT_AMT<>0) ';
            ClxCommon::$db->executeQuery($sql);
            //Aprovar asiento de ser necesario
            if (!empty($this->approve))
                $jehd->jehd_approve_by_code($gl_rec['IDCODE']);
        }//end while
        /* APROBAR ASIENTO Y BORRAR DE SER NECESARIO */
        $sql = 'DELETE FROM ' . $this->tmp1name . ' WHERE BATCH_ID=' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Builds default
     *
     */
    private function loit_build_import_defaults() {
        $sql = 'UPDATE ' . $this->loitname . ' SET BATCH_ID = ' . singleQuote($this->batch_id) . ' WHERE REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ')';
        ClxCommon::$db->executeQuery($sql);
        //Import loan amount as AVAILABLE_AMT-BALANCE_AMT from CLXGARAN since
        //USED AMOUNT REPRESENTS THE AMOUNT AVAILABLE
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.LOAN_AMT = AVAILABLE_AMT-BALANCE_AMT WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXGARAN') . '';
        ClxCommon::$db->executeQuery($sql);
        //SETTING CONTRACT AND PARTICIPANT CODE CLXGARAN
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.CONTRACT_CODE = (SELECT MAX(FIAS.CONTRACT_CODE) FROM ' . $this->fiasname . ' FIAS WHERE FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PROD01_CODE), LOIT.PARTICIPANT_ID = (SELECT FIPA.ACCOUNTCODE FROM ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIPA.CONTRACT_CODE = FICO.IDCODE AND FIAS.CONTRACT_CODE = FICO.IDCODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PROD01_CODE AND FIPA.ISPRIMARY = 1) WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXGARAN') . ')';
        ClxCommon::$db->executeQuery($sql);
        //SETTING CASES ON LOANS
        $sql = 'UPDATE FIDU_INTERFASE LOIT SET LOIT.CONTRACT_CODE = (SELECT MAX(FICO.IDCODE) FROM ' . $this->ficoname . ' FICO WHERE FICO.CONTRACT_NUMBER = LOIT.EXT_COST_CENTER), LOIT.PARTICIPANT_ID = (SELECT FIPA.ACCOUNTCODE FROM ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO WHERE FIPA.CONTRACT_CODE = FICO.IDCODE AND FICO.CONTRACT_NUMBER = LOIT.EXT_COST_CENTER AND FIPA.ISPRIMARY = 1) WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EXT_COST_CENTER IS NOT null AND LOIT.EXT_CUST_NO IS NOT null AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //SETTING CONTRACTS FROM FIAS FOR NON CLXCREDI RECORD (NO COST CENTER)
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.CONTRACT_CODE = (SELECT MAX(FIAS.CONTRACT_CODE) FROM ' . $this->ficoname . ' FICO, ' . $this->fiasname . ' FIAS WHERE FIAS.CONTRACT_CODE = FICO.IDCODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE), LOIT.PARTICIPANT_ID = (SELECT MAX(FIPA.ACCOUNTCODE) FROM ' . $this->fiasname . ' FIAS, ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO WHERE FIAS.CONTRACT_CODE = FICO.IDCODE AND FIAS.EXT_PRODUCT_CODE = LOIT.EXT_PRODUCT_CODE AND FIPA.CONTRACT_CODE = FICO.IDCODE AND FIPA.ISPRIMARY = 1) WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND (LOIT.CONTRACT_CODE IS NULL OR LOIT.CONTRACT_CODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //SETTING IDENTIFICATION     	    /*ACCO.IDCODE*/
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.IDENTIFICATION = (SELECT MAX(ACCO.IDENTIFICATION) FROM CRM_ACCOUNTS ACCO WHERE ACCO.IDCODE = LOIT.PARTICIPANT_ID), LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ')';
        ClxCommon::$db->executeQuery($sql);
        //SETING PRODUCT AND OTHERS
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.LOAN_TYPE = LOIT.BANK_PROD_TYPE, LOIT.PRODUCT_CODE = CONCAT(LOIT.BANK_PROD_TYPE,IFNULL(LOIT.EXT_SUBPROD_CODE,' . singleQuote(CLX_CHAR_EMPTY_STRING) . ')), LOIT.OPERATION_DATE = LOIT.EFFTDATE, LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ', LOIT.EXT_TRANS_CODE = null, LOIT.EXT_ITEM_CODE = null WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXPRODU') . ',' . singleQuote('CLXGARAN') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Cambiando a cancelado (CANC) todos los registros de banco
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote('CAN') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXPRODU') . ',' . singleQuote('CLXCREDI') . ') AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND NOT EXISTS(SELECT PROD.ID FROM ' . $this->prodname . ' PROD WHERE PROD.IDCODE = LOIT.BANK_PROD_TYPE)';
        ClxCommon::$db->executeQuery($sql);
        //No se tienen creditos en banco 1
        $sql = 'UPDATE ' . $this->loitname . ' LOIT SET LOIT.REC_STATUS = ' . singleQuote('CAN') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ') AND LOIT.EBSK_MSG_CODE IN (' . singleQuote('CLXCREDI') . ',' . singleQuote('CLXPAGOS') . ') AND LOIT.EXT_BANK_CODE = ' . singleQuote('0') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
        //Cancelar todas aquellas que no requiran actualizacion
        $sql = 'UPDATE ' . $this->loitname . ' LOIT, ' . $this->fibkname . ' FIBK SET LOIT.REC_STATUS = ' . singleQuote('CAN') . ' WHERE LOIT.REC_STATUS IN (' . singleQuote('IDLE') . ',' . singleQuote('END') . ',' . singleQuote('RUN') . ') AND FIBK.BANK_ACCOUNT_NO = LOIT.EXT_PRODUCT_CODE AND FIBK.BK_INTRFSE_MODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND LOIT.EBSK_MSG_CODE = ' . singleQuote('CLXPRODU') . ' AND LOIT.BATCH_ID = ' . singleQuote($this->batch_id);
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Execute LOIT Interfase on process
     */
    function loit_read_interfase() {
        //e cho '<hr>En prueba loit::loit_read_interfase()<hr>';
        //Load socket tools
        $EBSK = getTool('EBSK');
        $EBSK->rename_file = true; //Rename file after read
        //$EBSK->rename_file = false;//Rename file after read
        $this->approve = true; //set false to stop gl approval
        //$this->approve = false; //set false to stop gl approval
        $this->load_credi_balances = false; //for initial load only not for production
        //Read all products
        $EBSK->read('CLXPRODU');
        //Import credit details
        $EBSK->read('CLXCREDI');
        //Import credit operations
        $EBSK->read('CLXPAGOS');
        //Import credit operations
        $EBSK->read('CLXGARAN');
        //Actualizaciones para los registros importados
        $this->loit_build_import_defaults();
        //Place to run interfase
        ClxFreeReturn($this->loit_build());
    }

    /**
     * Method will read file and it pending until processing
     */
    function storeFile() {
        echoLog('Starting process: ' . $this->parameters['PROCESS_CODE']);
        $EBSK = &getTool('EBSK');
        //$EBSK->rename_file = true;//Rename file after read
        //$EBSK->rename_file = false;//Rename file after read
        //Read all products
        $EBSK->read('CLXPRODU');
        //Import credit details
        $EBSK->read('CLXCREDI');
        //Import credit operations
        $EBSK->read('CLXPAGOS');
        //Import credit operations
        $EBSK->read('CLXGARAN');
        echoLog('Ending process: ' . $this->parameters['PROCESS_CODE']);
    }

    function loit_reschedule() {
        $x = getTool('PRSC');
        $x->runScheduleOneTimeNow('LOIT01', array());
        return note(dict('UTIL_DBBK_PROC_SCHEDULED'));
    }

}
