<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Code Event.
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-02-14 07:33:40 -0500 (Wed, 14 Feb 2018) $
 * @version        $Rev: 9715 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_code extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'CODE';
        $this->name = 'Code';
        $this->type = 'ACTN';
        $this->version = '1.1.1.0';
        $this->description = 'Executes code.';
        $this->debug = false;
        $this->loadedlibraries = array();
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
            $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell('Code Properties', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        //$table->addField($msg, dict('GEN_LBL_CALL') . ':', 'CODE_call', $actionrecord['CODE_call'], 64, 255);
        $table->addText($msg, dict('GEN_LBL_CALL') . ':', 'CODE_call', $actionrecord['CODE_call'], CLX_NUM_30, CLX_NUM_120);
        $CODE = &getTool('CODE');
        $scripts = $CODE->placeChooseCodeLibrary($msg, 'EVNT_language', dict('GEN_LBL_LIBRARY') . ':', 'CODE_library', $actionrecord['CODE_library'], $table);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return $scripts . CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (empty(UserRequest::$post['CODE_call'])) {
            $msg['CODE_call'] = dict('EVNT_LBL_TYPE_CALL');
        }
        if (!empty(UserRequest::$post['CODE_library']) && !$this->libExists(UserRequest::$post['CODE_library'])) {
            $msg['CODE_library'] = dict('EVNT_LBL_LIB_EXISTS');
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['EXTRA1'] = CLX_INIT_ZERO;
        $actionrecord['EXTRA2'] = clx_html_spchrs_decode(UserRequest::$post['CODE_call']);
        $actionrecord['EXTRA3'] = UserRequest::$post['CODE_library'];
        $actionrecord['EXTRA4'] = '';
        $actionrecord['EXTRA5'] = '';
        $actionrecord['EXTRA6'] = '';
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
            $actionrecord['CODE_call'] = $values['EXTRA2'];
            $actionrecord['CODE_library'] = $values['EXTRA3'];
        } else {
            $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['CODE_call'] = '';
        $actionrecord['CODE_library'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm_2(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_CALL') . ':', '<pre>' . $record['CODE_call'] . '</pre>');
        if (!empty($record['CODE_contents'])) {
            $table->addData(dict('GEN_LBL_CONTENTS') . ':', '<pre>' . $record['CODE_contents'] . '<pre>');
        }
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_EXECUTE_CODE') . ':' . $record['EXTRA2'] . ' ' . dict('GEN_LBL_LIBRARY') . ': ' . $record['EXTRA3'];
    }

    /**
     * Includes event library
     * @param type $event
     * @param type $record
     */
    private function includeLibrary($event, &$record) {
        if (strlen($event['EXTRA3']) > CLX_NUM_ZERO) {
			$fetched = false;
			if (kApcTtl_4>0) $librec = apc_fetchFct($findVar=kSpaceName . '_codelib_' . $event['EXTRA3'], $fetched);
			if (empty($fetched)) {
				$librec = ClxCommon::$db->getRecord('CLX_CODELIB', 'UPPER(IDCODE)=' . singleQuote(uppercase($event['EXTRA3'])));

				if (empty($librec)) {
					$BUGS = getTool('BUGS');
					$BUGS->report(E_USER_ERROR, dict('TOOL_CODE_ERR_NO_RECORD') . ': ' . dict('GEN_LBL_CODELIB') . ' ' . $event['EXTRA2'], 'objects/actn_event.php', 'unknown line', $record, CLX_NUM_ONE, 'Lib code: ' . $event['EXTRA3']);
				}
				else if (kApcTtl_4>0 && !empty($librec)) { 
					apc_addFct($findVar, $librec, kApcTtl_4);
				}
			}
		    include_once kPathToWebSite .  (($librec['TYPE'] === 'ENBD') ? kPathToCode.lowercase($librec['IDCODE']).'.php' : $librec['FILENAME']);
        }
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
			
	/*	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivoxx.txt","a+");
		$te=debug_backtrace(); 
		for ($t=0;$t<12;$t++) {
			if (isset($te[$t]["file"])){
				fwrite($tta,"evento16:".substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).
					$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
			}
		}fclose($tta);
		*/						
        $this->includeLibrary($event, $record);
        if (!empty($this->debug)) {
            pre($msg);
        }
        //Crear archivo fisico
		
        if (!empty($event['EXTRA2']) and file_exists(kPathToWebSite . kPathToCodeEvents . 'evtn_' . $event['ID'] . '.php')) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivoxx.txt","a+");fwrite($tta,"<br>".kPathToWebSite . kPathToCodeEvents . 'evtn_' . $event['ID'] . '.php');fclose($tta);
			 
			include kPathToWebSite . kPathToCodeEvents . 'evtn_' . $event['ID'] . '.php';
		}
		else   {
//			include kPathToWebSite . kPathToCodeEvents . 'evtn_' . $event['ID'] . '.php';
		}

    }

    /**
     * Process step execution according object type.
     * @param string $event
     * @param array $record
     * @param array $parameters
     * @param array $msg
     * @param bool $debug
     * @return mixed
     */
    function execute_process_event($event, &$record, &$parameters, &$msg, $debug = false) {
        $inittime=microtime(true);
        $this->includeLibrary($event, $record);
        
        //Crear archivo fisico
        if (!empty($event['EXTRA2'])) {
            $archivo=kPathToWebSite . kPathToCodeEvents . 'evtn_' . $event['ID'] . '.php';
            if (!file_exists($archivo))  {
                $a=fopen($archivo,"w+");
                fwrite($a,"<?PHP".chr(13).chr(10).$event['EXTRA2'].chr(13).chr(10)."?>");
                fclose($a);
                
                
            }
			$_SESSION["code_parameters"]=$parameters;
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions_events_code.txt","a+");
			fwrite($tta,chr(13).chr(10).'Init sql , record: '.print_r($record,true).chr(13).chr(10));
			//fwrite($tta,chr(13).chr(10).'Code section:'. print_r($sql,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).'Event section:'. print_r($event,true).chr(13).chr(10));
			fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
			fclose($tta);
            include $archivo;
			$tta=fopen(kPathToWebSite."/".kPathToTemp."/process_executions_events_code.txt","a+");
			fwrite($tta,chr(13).chr(10).'Tiempo section:'. (microtime(true)-$inittime).chr(13).chr(10));
			fclose($tta);
        }
    }
}
