<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - SQL Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_sqlb extends actn {

    var $do_section_type = array('NOT' => 'GEN_LBL_NONE',
        'DS' => 'GEN_LBL_DO_SELECT',
        'DW' => 'GEN_LBL_DO_WHILE',
        'DWH' => 'GEN_LBL_DO_WHEN',
        'DU' => 'GEN_LBL_DO_UNTIL');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'SQLB';
        $this->name = 'Sql Script For Business Process';
        $this->type = 'ACTN';
        $this->version = '1.2.1.0';
        $this->description = 'Executes sql code.';
        $this->debug = false;
        $this->loadedlibraries = array();
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
            $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell('Code Properties', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        //$table->addField($msg, dict('GEN_LBL_CALL') . ':', 'SQL_call', $actionrecord['SQL_call'], 64, 255);
        $table->addText($msg, dict('GEN_LBL_CALL') . ':', 'SQLB_call', $actionrecord['SQLB_call'], CLX_NUM_30, CLX_NUM_120);
        $SQL = &getTool('CODE');
        $scripts = $SQL->placeChooseCodeLibrary($msg, 'EVNT_language', dict('GEN_LBL_LIBRARY') . ':', 'SQL_library', $actionrecord['SQLB_library'], $table);
        $sel = new htmlSelect('SQLB_database');
        $sel->addItem(dict('GEN_LBL_ALL'), 'all', ($actionrecord['SQLB_database'] === 'all'));
        foreach (ClxCommon::$db->dbTypes as $key => $val) {
            $sel->addItem(dict($val), $key, ($actionrecord['SQLB_database'] == $key));
        }
        $table->addCell(dict('GEN_LBL_DATABASE'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($sel->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return $scripts . CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (empty(UserRequest::$post['SQLB_call'])) {
            $msg['SQL_call'] = dict('EVNT_LBL_TYPE_CALL');
        }
        if (!empty(UserRequest::$post['SQLB_library']) && !$this->libExists(UserRequest::$post['SQLB_library'])) {
            $msg['SQLB_library'] = dict('EVNT_LBL_LIB_EXISTS');
        }
        if (!empty(UserRequest::$post['SQLB_call']) && !empty(UserRequest::$post['SQLB_library'])) {
            $msg['SQLB_call'] = dict('EVNT_ONLY_LIB_OR_CALL');
            $msg['SQLB_library'] = $msg['SQLB_call'];
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = UserRequest::$post['SQLB_call'];
        $actionrecord['extra3'] = (isset(UserRequest::$post['SQLB_library']) ?UserRequest::$post['SQLB_library'] :'');
        $actionrecord['extra4'] = UserRequest::$post['SQLB_database'];
        $actionrecord['extra5'] = '';
        $actionrecord['extra6'] = '';
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
            $actionrecord['SQLB_call'] = $values['EXTRA2'];
            $actionrecord['SQLB_library'] = $values['EXTRA3'];
            $actionrecord['SQLB_database'] = $values['EXTRA4'];
        } else {
            $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['SQLB_call'] = '';
        $actionrecord['SQLB_library'] = '';
        $actionrecord['SQLB_database'] = 'all';
        $actionrecord['SQLB_do_section_type'] = 'NOT';
        $actionrecord['SQLB_do_section'] = '';
        $actionrecord['SQLB_seqnum'] = '0';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_CALL') . ':', '<pre>' . $record['SQLB_call'] . '</pre>');
        if ($record['SQLB_database'] === 'all') {
            $lbl = 'GEN_LBL_ALL';
        } else {
            $lbl = ClxCommon::$db->database_type[$record['SQLB_database']];
        }
        $table->addData(dict('GEN_LBL_database') . ':', dict($lbl));
        if (!empty($record['SQLB_library'])) {
            $table->addData(dict('GEN_LBL_CODE_LIBRARY') . ':', '<pre>' . $record['SQLB_library'] . '<pre>');
        }
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        $return = dict('GEN_LBL_STATEMENT') . ': <pre>' . $record['EXTRA2'] . '</pre>';
        if ($record['EXTRA3'] !== '') {
            $return .= '<br>' . dict('GEN_LBL_CODELIB') . ': ' . $record['EXTRA3'];
        }
        if (!empty($record['EXTRA4']) && $record['EXTRA4'] !== '') {
            $return .= '<br>' . dict('GEN_LBL_DATABASE') . ': ' . (($record['EXTRA4'] === 'all') ? dict('GEN_LBL_ALL') : dict(ClxCommon::$db->dbTypes[$record['EXTRA4']]));
        }
        return $return;
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        $code = '';
        $libcode = $event['EXTRA3'];
        if ($libcode !== '') {
            $librec = ClxCommon::$db->getRecord('CLX_CODELIB', 'IDCODE=' . singleQuote($libcode));
            if (empty($librec)) {
                $BUGS = getTool('BUGS');
                $BUGS->report(E_USER_ERROR, dict('TOOL_CODE_ERR_NO_RECORD') . ': ' . dict('GEN_LBL_CODELIB') . ' ' . $event['EXTRA3'], 'objects/actn_event.php', 'unknown line', $record, CLX_NUM_ONE, 'Lib code: ' . $libcode);
            } elseif ($librec['TYPE'] === 'ENBD') {
                $code = stripslashes($librec['CONTENTS']);
            } elseif ($librec['TYPE'] === 'LINK') {
                include_once kPathToWebSite . $librec['FILENAME'];
                $code = file_get_contents(kPathToWebSite . $librec['FILENAME']);
            }
        } else {
            $code = $event['EXTRA2'];
        }
        if (!empty($this->debug)) {
            echo '<!-- ', $code, ' ', $librec['FILENAME'], ' ', $event['EXTRA2'], ' -->';
        }
        $rs = false;
        if (strlen($code) > CLX_NUM_ZERO) {
            $rs = ClxCommon::$db->executeQuery($code);
        }
        echo 'Ejecutando sql ', CLX_CHAR_NL, $code, CLX_CHAR_NL;
        return ($rs) ? true : false;
    }

}
