<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('util') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'util.php';

class util_db_backup extends util {

    var $verbose = false; //Set true to display output

    // CONSTRUCTOR //

    function __construct() {
        $this->code = 'DBBK';
        $this->name = 'Database Backup Utilities';
        $this->type = 'UTIL';
        $this->version = '0.0.0.1';
        $this->description = 'Database Backup Utilities.';
        $this->haspanel = true;
        $this->debug = false;
        multiTableName($this, array('PROC', 'PRSC'));
        if (empty(UserRequest::$post[$this->code . '_output_folder'])) {
            UserRequest::$post[$this->code . '_output_folder'] = $this->dump_file_name();
        }
        $this->panelform = array(CLX_CLASS_TITLE => 'UTIL_DBBK_TITLE_EXECUTE',
            CLX_CLASS_SUBTITLE => 'GEN_LBL_LONGPROCESSWAIT',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_OUTPUT_FOLDER',
                            'fields' => array(
                                array('name' => 'output_folder', 'type' => 'string', 'displaysize' => '45', 'size' => '255', 'value' => $this->bkp_folder_name(), 'validations' => array(
                                        array('type' => 'string', 'required' => '1'),)
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function bkp_folder_name() {
        // Instance UTIL tool to load system parameters
        // UTIL_DBBK_BKP_DIR
        // Backup folder on general parameter
        $Gp_BackupFolder = (defined('UTIL_DBBK_BKP_DIR')) ? UTIL_DBBK_BKP_DIR : '';
        if ($Gp_BackupFolder === '' || $Gp_BackupFolder === 'null' || is_null($Gp_BackupFolder)) {
            $Gp_BackupFolder = '';
        }
        $BackupFolder = '';
        // Backup folder defined in configurator
        if (!defined('kDbBackupFolder')) {
            define('kDbBackupFolder', '/temp');
            $BackupFolder = kDbBackupFolder;
        }
        // Backup folder on general parameter takes precedence over configurator one
        if ($Gp_BackupFolder !== '') {
            $BackupFolder = $Gp_BackupFolder;
        }
        if (substr($BackupFolder, -1 * CLX_NUM_ONE) !== '/') {
            $BackupFolder .= '/';
        }
        return str_replace('/', DIRECTORY_SEPARATOR, str_replace('\\', DIRECTORY_SEPARATOR, $BackupFolder));
    }

    function dump_file_name() {

        $BackupFolder = $this->bkp_folder_name();
        return $BackupFolder . kDatabaseName . '_' . date('Ymd_Hi') . '.sql';
    }

    function dump_cron() {
        UserRequest::$post[$this->code . '_output_folder'] = $this->dump_file_name();
    }

    /**
     * Normal execution for system->utilites.
     * Process will be sent to process engine
     */
    function execute_() {
        $contents = '';
        $status = true;
        //Check that process exists.
        if (ClxCommon::$db->oneExistsRecords($this->procname, 'IDCODE = ' . singleQuote($this->code)) == CLX_NUM_ZERO) {
            /*
              UTIL_DBBK_PROCESSMISSING Database backup process (^1) missing. Please install process before continuing. Proceso de respaldo de base de datos (^1) no pudo ser encontrado. Favor verificar
             */
            $contents = dict('UTIL_DBBK_PROCESSMISSING', $this->code);
            $status = false;
        }
        if ($status === true) {
            //set schedule
            xProcess::runScheduleOneTimeNow($this->code, array('IDNAME' => UserRequest::$post[$this->code . '_output_folder']));
            $contents = dict('UTIL_DBBK_PROC_SCHEDULED');
            $contents = dialog(dict('UTIL_DBBK_TITLE_EXECUTE'), '', $contents, '');
        } else {
            $contents = dialog(dict('UTIL_DBBK_TITLE_EXECUTE'), CLX_CLASS_ALERT, $contents, '');
        }
        return $contents;
    }

    /**
     * The actual execution
     */
    function run($parameters = array()) {

        $contents = '';
        UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
        $bkp_time_limit = $this->get_bkp_time_limit();
        UserResponse::setRunTimeLimit($bkp_time_limit * CLX_NUM_60 * CLX_NUM_60);

        if ($parameters['IS_DEBUG'] === true) {
            $is_debug = ($parameters['IS_DEBUG'] === '1');
        } else {
            // No debug by default
            $is_debug = false;
        }

        if ($parameters['IDNAME'] === true) {
            $output_folder = trim($parameters['IDNAME']);
        } else {
            // Default backup filespec
            $output_folder = $this->dump_file_name();
        }

        // (separated by '/')
        if ($parameters['TABLES'] === true) {
            $tables = $parameters['TABLES'];
        } else {
            // All tables by default
            $tables = '';
        }

        if ($parameters['NAME_SUFFIX'] === true) {
            $suffix = trim($parameters['NAME_SUFFIX']);
        } else {
            // No suffix by default
            $suffix = '';
        }

        // (separated by '/')
        if ($parameters['DUMP_OPTIONS'] === true) {
            $dump_options = trim($parameters['DUMP_OPTIONS']);
        } else {
            // No dump optional parameters by default
            $dump_options = '';
        }
        // Local Backup Script optional parameters: Don't stop minute-by-minute process
        // (only for specific local_bkp_script_name)
        if ($parameters['LBS_OPT_PAR'] === true) {
            $lbs_opt_par = trim($parameters['LBS_OPT_PAR']);
        } else {
            // No optional backup parameters by default
            $lbs_opt_par = '';
        }
        $cmd = '';
        $script_backup_call = false;
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Verifying constant existance to avoid notice error message
        $zip_cmd_name = '';
        if (defined('UTIL_DBBK_ZIP_CMD')) {
            $zip_cmd_name = UTIL_DBBK_ZIP_CMD;
        }
        if ($zip_cmd_name === '' || $zip_cmd_name === 'null' || is_null($zip_cmd_name)) {
            $zip_cmd_name = '';
        }
        $local_bkp_script_name = '';
        if (defined('UTIL_DBBK_LOC_BKP_SCRIPT')) {
            $local_bkp_script_name = UTIL_DBBK_LOC_BKP_SCRIPT;
        }
        if ($local_bkp_script_name === '' || $local_bkp_script_name === 'null' || is_null($local_bkp_script_name)) {
            $local_bkp_script_name = '';
        }
        if ($local_bkp_script_name !== '') {
            //   - Caso 1002598 - BNP-Limpieza Audit en Batch
            // Se agrega la opcion de poder llamar a un Script o programa externo
            // que haga el trabajo
            if ($tables !== '') {
                $tables = ' -tables ' . $tables;
            }
            if ($suffix !== '') {
                $suffix = ' -nsuf ' . $suffix;
            }
            if ($dump_options !== '') {
                $dump_options = ' -dmpopt ' . $dump_options;
            }
            $remote_bkp_script_name = '';
            if (defined('UTIL_DBBK_REM_BKP_SCRIPT')) {
                $remote_bkp_script_name = UTIL_DBBK_REM_BKP_SCRIPT;
            }
            if ($remote_bkp_script_name !== '' && $remote_bkp_script_name !== 'null' && !is_null($remote_bkp_script_name)) {
                $remote_bkp_script_name = ' -rbsn ' . $remote_bkp_script_name;
            }
            $cmd = $local_bkp_script_name . ' -h ' . kDatabaseURL . ' -u ' . kDatabaseUsername . ' -p ' . kDatabasePassword . ' -dbn ' . kDatabaseName . $suffix . $tables . $remote_bkp_script_name . $dump_options . ' ' . $lbs_opt_par . ' -of ' . $output_folder;
            $script_backup_call = true;
        } elseif (kDatabaseType === 'mysql' or kDatabaseType === 'mysqli' ) {
            if ($tables !== '') {
                $tables = '--tables ' . str_replace('/', ' ', $tables);
            }
            $dump_options = ' ' . str_replace('/', ' ', $dump_options) . ' ';
            $output_folder = str_replace('.sql', $suffix . '.sql', $output_folder);
            $host = kDatabaseURL;
            $port = '';
            if (!(strpos($host, ':') === false)) {
                $cmd = explode(':', $host);
                $host = $cmd[CLX_NUM_ZERO];
                $port = ' --port ' . $cmd[CLX_NUM_ONE];
            }
            $cmd = 'mysqldump -h' . $host . $port . ' -u' . kDatabaseUsername . ' -p' . kDatabasePassword . $dump_options . '--lock-tables=false ' . kDatabaseName . ' ' . $tables . ' > ' . $output_folder;
        } elseif (kDatabaseType === 'oci') {
            if ($tables !== '') {
                $tables = 'tables=' . str_replace('/', ',', $tables);
            }
            $a_dump_options = explode('/', $dump_options);
            $dump_options = '';
            foreach ($a_dump_options as $key => $val) {
                if ($val === '--skip-add-drop-table') {
                    // how add drop table when i use exp command
                    // https://community.oracle.com/thread/843861?start=0&tstart=0
                }
            }
            $output_folder = str_replace('.sql', $suffix . '.dmp', $output_folder);
            $cmd = 'exp ' . kDatabaseUsername . '/' . kDatabasePassword . '@' . kDatabaseName . ' file=' . $output_folder . ' ' . $tables . ' statistics = ' . singleQuote('non') . ' ' . $dump_options;
        }
        if ($cmd <> '') {
            if (!empty($is_debug)) {
                $msgout = $cmd;
                $msgout = str_replace(kDatabasePassword, '***', $msgout);
                $msgout = str_replace(kDatabaseUsername, '***', $msgout);
                $msgout = str_replace(kDatabaseURL, '***', $msgout);
                echoLog('<br>' . CLX_CHAR_NL . 'Calling external program:<br>' . CLX_CHAR_NL . $msgout . '<br>' . CLX_CHAR_NL);
                echoLog('Time Limit: ' . $bkp_time_limit . '<br>' . CLX_CHAR_NL);
            }
            if ($script_backup_call === true) {
                //   - Caso 1002598 - BNP-Limpieza Audit en Batch
                // Llama a Script Backup
                // Este script debe escribir en la salida 'OK' el respaldo finalizo con saliro
                $cmd_result = shell_exec($cmd);
                $cmd_result = $this->fix_cmd_result($cmd_result);
                $newline_string = $this->get_newline_string($cmd_result);
                $cmd_result = $this->delete_ending_crlf($cmd_result);
                $contents = $cmd_result;
                if (!is_null($newline_string)) {
                    $a_tmp_resp_recs = explode($newline_string, $cmd_result);
                    // echoLog('<br>'.CLX_CHAR_NL.''.CLX_CHAR_NL.' COUNT(A_TMP_RESP_RECS): '.count($a_tmp_resp_recs).'<br>'.CLX_CHAR_NL);
                    if (count($a_tmp_resp_recs) > CLX_NUM_ONE) {
                        // OK Result must be in the last line
                        $cmd_result = $a_tmp_resp_recs[count($a_tmp_resp_recs) - CLX_NUM_ONE];
                    }
                }
                //e choLog('<br>'.CLX_CHAR_NL.''.CLX_CHAR_NL.' CMD_RESULT 2: ' . $cmd_result.'<br>'.CLX_CHAR_NL);
                if ($cmd_result === 'OK') {
                    echoLog('<br>' . CLX_CHAR_NL . 'Script backup completed');
                    $contents = ''; // No error
                } else {
                    $PRPR = getTool('PRPR');
                    $PRPR->report_exception($parameters['SCHEDULECODE'], $parameters['START_TIME'], $parameters['RUNID'], $contents, '', 'ERRO');
                }
            } else {
                //exec($cmd);
                $cmd_result = shell_exec($cmd);
                $cmd_result = $this->fix_cmd_result($cmd_result);
                if (!file_exists($output_folder) || filesize($output_folder) < CLX_NUM_THOUSAND) {
                    $contents .= 'File creation fail during (Mysqldump or Oracle EXP) database dump execution. Maybe, user has no privileges to complete this action. Please verify. <br><br>No se pudo completar el backup por favor verifique si el usuario tiene privilegios para realizar esta accion.';
                    if (!empty($is_debug))
                        $contents .= '<br><br>' . CLX_CHAR_NL . 'Command Result:<br>' . CLX_CHAR_NL . $cmd_result;
                    $PRPR = getTool('PRPR');
                    $PRPR->report_exception($parameters['SCHEDULECODE'], $parameters['START_TIME'], $parameters['RUNID'], $contents, '', 'ERRO');
                    if (file_exists($output_folder))
                        unlink($output_folder);
                }else {
                    if ($zip_cmd_name !== '') {
                        $output_zip = $output_folder . '.zip';
                        $cmd = $zip_cmd_name . ' ' . $output_zip . ' ' . $output_folder;
                        if (!empty($is_debug))
                            echoLog('<br>' . CLX_CHAR_NL . 'Compressing backup file:<br>' . $cmd . '<br>');
                        //exec($cmd);
                        $cmd_result = shell_exec($cmd);
                        $cmd_result = $this->fix_cmd_result($cmd_result);
                        if (file_exists($output_zip) && filesize($output_zip) > CLX_NUM_ZERO) {
                            unlink($output_folder);
                            echoLog('<br>' . CLX_CHAR_NL . 'Backup completed: ' . $output_folder . '.zip');
                        } else {
                            $contents .= 'Compressed file creation Fails during [' . $zip_cmd_name . '] execution. Please verify. <br><br>No se pudo completar la compresi&oacute;n del backup.';
                            if (!empty($is_debug))
                                $contents .= '<br><br>' . CLX_CHAR_NL . 'Command Result:<br>' . CLX_CHAR_NL . $cmd_result;
                            //   - Caso 1002598 - BNP-Limpieza Audit en Batch
                            // Report Compressed file creation error
                            $PRPR = getTool('PRPR');
                            $PRPR->report_exception($parameters['SCHEDULECODE'], $parameters['START_TIME'], $parameters['RUNID'], $contents, '', 'ERRO');
                        }
                    } else {
                        echoLog('<br>' . CLX_CHAR_NL . 'Backup completed (without compression): ' . $output_folder);
                    }
                }
            }
        } else {
            $contents = 'Invalid command only available for mysql or oracle.';
            $PRPR = getTool('PRPR');
            $PRPR->report_exception($parameters['SCHEDULECODE'], $parameters['START_TIME'], $parameters['RUNID'], $contents, '', 'ERRO');
        }
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // OJO [ $contents === '' ] indica que no hubo error
        if ($contents !== '') {
            if (!empty($is_debug))
                echoLog($contents);
        }
        return $contents;
    }

    /*
     * Returns time limit based on UTIL_DBBK_TIME_LIMIT system parameter
     * or default value of 24 hours
     */

    function get_bkp_time_limit() {
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Set backup time limit. Initially was 6 hours but
        // CLX_AC_AUDIT archival backup could be very huge the first time it runs,
        // so default value was raised to 24 hours and the option to specify longer
        // time limit was added by the optional UTIL_DBBK_TIME_LIMIT system parameter
        // If process is killed after ClxBackup finished the 7za.exe run,
        // the process engine will repeat execution of CLX_ACAD_CLEAN forever... =(
        $bkp_time_limit = null;
        if (defined('UTIL_DBBK_TIME_LIMIT')) {
            $bkp_time_limit = UTIL_DBBK_TIME_LIMIT;
        }
        if ($bkp_time_limit === 'null' || is_null($bkp_time_limit)) {
            $bkp_time_limit = '';
        }
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Tired of schedule to re-run process... allow to specify 0=unlimited
        // with the UTIL_DBBK_TIME_LIMIT parameter
        if ($bkp_time_limit === '')
            $bkp_time_limit = CLX_NUM_24; // 24 hours by default
        $bkp_time_limit = CLX_NUM_ZERO + $bkp_time_limit;
        return $bkp_time_limit;
    }

    /**
     * Fix external operating system command response to enable effective
     * recording of entire message in the process exceptions log
     */
    function fix_cmd_result($str) {
        $str = encode_string($str);
        return $str;
    }

// Method End
    /**
     * Removes the trailing CR/LFs (line endings)
     */
    function delete_ending_crlf($s_ch) {
        return trim($s_ch, CLX_CHAR_CR . CLX_CHAR_NL);
    }

// Method End
    /**
     * Detect new line combination in a external operating system
     * command response
     */
    function get_newline_string($str) {
        // null if it doesn´t find any new line string
        $response = null;
        // CR+LF
        if (!(mb_strpos($str, CLX_CHAR_CR . CLX_CHAR_NL) === false)) {
            //e choLog(CLX_CHAR_NL.'GET_NEWLINE_STRING = CR+LF'.CLX_CHAR_NL);
            $response = CLX_CHAR_CR . CLX_CHAR_NL;
        }
        // LF+CR
        if (!(mb_strpos($str, '' . CLX_CHAR_NL . CLX_CHAR_CR) === false)) {
            //e choLog(CLX_CHAR_NL.'GET_NEWLINE_STRING = LF+CR'.CLX_CHAR_NL);
            $response = '' . CLX_CHAR_NL . CLX_CHAR_CR;
        }
        // CR
        if (!(mb_strpos($str, CLX_CHAR_CR) === false)) {
            //e choLog(CLX_CHAR_NL.'GET_NEWLINE_STRING = CR'.CLX_CHAR_NL);
            $response = CLX_CHAR_CR;
        }
        // ** LF **
        if (!(mb_strpos($str, CLX_CHAR_NL) === false)) {
            //e choLog(CLX_CHAR_NL.'GET_NEWLINE_STRING = LF'.CLX_CHAR_NL);
            $response = CLX_CHAR_NL;
        }
        return $response;
    }

// Method End
}
