<?php

/**
 * User Group Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:25:56 -0400 (ma., 22 ene. 2019) $
 * @version    	$Rev: 10494 $
 * CHANGE LOG
 *  Included label
 */
class ac_groups extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ACGR';
        $this->name = 'User Group Manager';
        $this->system = 'Security';
        $this->version = '0.3.3.5';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_AC_GROUPS';
        $this->record_name_singular = 'user group';
        $this->record_name_plural = 'user groups';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->status_types = array('1' => 'GEN_LBL_ACTIVE', '0' => 'GEN_LBL_INACTIVE');
        $this->yesno = array('1' => 'GEN_LBL_YES', '0' => 'GEN_LBL_NO');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_ACGR_TITLE_MODIFY',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48',
                                    'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48')
                            )
                        ), array('label' => 'GEN_LBL_GROUPAUDIT',
                            'fields' => array(
                                array('name' => 'audit', 'type' => CLX_CLASS_LIST, 'default' => '0',
                                    'options' => $this->yesno, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_STATUS',
                            'fields' => array(
                                array('name' => 'status', 'type' => CLX_CLASS_LIST, 'default' => '0',
                                    'options' => $this->status_types, 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_GROUPS', 'TOOL', $this->code, 'ac=LIST');
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote('admi') . '') == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->tablename, array('IDCODE' => 'admin', 'IDNAME' => 'administrators', 'DESCRIPTION' => 'Clientix System Administrators',
                'STATUS' => CLX_NUM_ONE, 'CDATE' => date('YmdHis'), 'MDATE' => date('YmdHis'), 'username' => 'admin', 'AUDITU' => CLX_NUM_ZERO), false);
        }
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['AUDITU'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ACGM', 'SELECT * FROM CLX_AC_GROUP_MEMBERS WHERE UPPER(GROUPCODE)=' . singleQuote(uppercase($code)));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $code = uppercase($record['IDCODE']);
        ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($code));
        $newid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $newid));
            }
        }
    }

    //Acc Group package do not has version.
    function packdescribe($o, &$i, &$table) {
        $rv = '';
        $ov = '';
        $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
        if ($o->hasSubObject('MDATE')) {
            $ov = $o->getSubObjectContents('MDATE');
            $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($o->getSubObjectContents($this->primary_field))) > CLX_NUM_ZERO);
            $ov1 = str_replace('.', '', $ov);
            $ov1 = str_replace('-', '', $ov1);
            if ($exists === true) {
                $record = ClxCommon::$db->getRecord($this->tablename, $this->primary_field . '=' . singleQuote($o->getSubObjectContents($this->primary_field)));
                $rv = $record['MDATE'];
            }
            $rv1 = str_replace('.', '', $rv);
            $rv1 = str_replace('-', '', $rv1);
            if (empty($exists)) {
                $table->addCell(dict('GEN_LBL_NEW'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: yellow', CLX_PROP_WIDTH => '25%'));
            } elseif ($ov1 > $rv1) {
                $table->addCell(dict('GEN_LBL_UPGRADE'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: green', CLX_PROP_WIDTH => '25%'));
            } elseif ($ov1 == $rv1) {
                $table->addCell(dict('GEN_LBL_SAME'), array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
            } else {
                $table->addCell(dict('GEN_LBL_DOWNGRADE'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: red', CLX_PROP_WIDTH => '25%'));
            }
            $table->addCell($rv, array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
            $table->addCell($ov, array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
            $table->addCell($this->packname($o), array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
            $i++;
        } else {
            $table->addCell(dict('GEN_LBL_OBJECTHASNOVERSION'), array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '4'));
        }
    }

    function action($ac, $idlist) {
        $msg = array();
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'INCLUDE':
                    if ($this->getAccess()->canModify()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($this->checkIncludeForm($msg)) {
                                $record = array();
                                $record['GROUPCODE'] = uppercase(UserRequest::$post['ACGR_idcode']);
                                $record['USERNAME'] = UserRequest::$post['ACGR_username'];
                                ClxCommon::$db->createRecord('CLX_AC_GROUP_MEMBERS', $record, false);
                                $info = popHistory();
                                $to = &getTool($info[CLX_PROP_TO]);
                                $contents = $to->action($info['ac'], toArray($info['idlist']));
                            } else {
                                $contents = $this->includeForm($ac, UserRequest::$post, $msg);
                            }
                        } else {
                            $record = array();
                            $record['ACGR_username'] = UserRequest::$request['username'];
                            $contents = $this->includeForm($ac, $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_GROUPS-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'EXCLUDE':
                    if ($this->getAccess()->canDelete()) {
                        foreach ($idlist as $id) {
                            $groupcode = ClxCommon::$db->getField('CLX_AC_GROUPS', 'IDCODE', 'ID=' . $id);
                            ClxCommon::$db->deleteRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(GROUPCODE)=' . singleQuote(uppercase($groupcode)) . ' AND USERNAME=' . singleQuote(UserRequest::$post['username']));
                        }
                        $info = popHistory();
                        $to = &getTool($info[CLX_PROP_TO]);
                        $contents = $to->action($info['ac'], toArray($info['idlist']));
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_GROUPS-CAN_DELETE][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_GROUPS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACGRList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AC_GROUPS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACGR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_STATUS'), dict('GEN_LBL_USERS')));
        $i = CLX_INIT_ZERO;
        $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $active = ClxCommon::$db->countRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(GROUPCODE)=' . singleQuote(uppercase($record['IDCODE'])));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(uppercase($record['IDCODE']), $record['IDNAME'], xFormat::text($record['DESCRIPTION']), $status[$record['STATUS']], $active));
            $i++;
        }
        $id = $form->getProp(CLX_PROP_ID);
        $buttons = parent::listingButtons($id);
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function makepanel($username = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'ACGRPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('username', $username);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = 'g.*';
            $fromclause = 'CLX_AC_GROUPS g, CLX_AC_GROUP_MEMBERS u';
            $whereclause = 'UPPER(g.IDCODE) = UPPER(u.GROUPCODE) AND UPPER(u.USERNAME)=' . singleQuote(uppercase($username));
            $sortclause = 'g.IDCODE, g.IDNAME';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_NUM_FIVE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_STATUS')));
            $i = CLX_INIT_ZERO;
            $status = array('Inactive', 'Active');
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], xFormat::text($record['DESCRIPTION']), $status[$record['STATUS']]));
                $i++;
            }
            $buttons = '';
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $b = new htmlFormSubmit(dict('GEN_LBL_EXCLUDE'), 'EXCLUDE', $form->getProp(CLX_PROP_ID));
                $b->setProp(CLX_PROP_ID, 'MULT');
                $buttons .= $b->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_GROUP')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
                $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('TOOL_ACGR_PROMPT_INAGROUP')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
                $b->setProp(CLX_PROP_ID, 'ON');
                $buttons .= $b->source();
            }
            $form->addContents($page->footers($table, $buttons));
            $modulecode = ''; // Verify $modulecode?
            return buildPanelInterface(dict('TOOL_ACGR_TITLE_GROUPLIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Export listing
     *
     * @param string $listingcode
     */
    function exportList($listingcode = null) {
        $this->export_notes .= dict('GEN_LBL_STATUS') . ': ';
        $comma = '';
        foreach ($this->status_types as $key => $val) {
            $this->export_notes .= $comma . $key . '=' . dict($val, true);
            $comma = ', ';
        }
        $this->export_notes .= dict('GEN_LBL_GROUPAUDIT') . ': ';
        $comma = '';
        foreach ($this->yesno as $key => $val) {
            $this->export_notes .= $comma . $key . '=' . dict($val, true);
            $comma = ', ';
        }
        return parent::exportList($listingcode);
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACGR_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('ACGR_idcode', UserRequest::$post))
            UserRequest::$post['ACGR_idcode'] = '';
        if (!array_key_exists('ACGR_idname', UserRequest::$post))
            UserRequest::$post['ACGR_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACGR_idcode', UserRequest::$post['ACGR_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACGR_idname', UserRequest::$post['ACGR_idname'], CLX_NUM_32, CLX_NUM_32);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            $boolean = '';
            if (UserRequest::$post['ACGR_idcode'] <> '') {
                $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACGR_idcode']) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['ACGR_idname'] <> '') {
                $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACGR_idname'])) . '%');
                $boolean = ' AND ';
            }
            $qty = ClxCommon::$db->oneExistsRecords('CLX_AC_GROUPS', $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell('No record match your description.', array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords('CLX_AC_GROUPS', $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=acgroup></page><a href=' . singleQuote($js) . '>' . $record['IDNAME'] . '</a>', $record['IDCODE']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        parent::checkForm($msg);
        if ($action === 'CREATE') {
            xValidate::unique($msg, 'ACGR_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['ACGR_idcode']));
        }
        if (UserRequest::$post['ACGR_idname'] === '') {
            $msg['ACGR_idname'] = 'You have to write a name.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function includeForm($ac, $record, $msg) {
        $contents = '';
        $title = dict('TOOL_ACGR_TITLE_INCLUDEUSER');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ACGR_username', $record['ACGR_username']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_USERNAME') . ':', (array_key_exists('ACGR_username', $record)) ? $record['ACGR_username'] : '');
        $contents .= $this->placeChooseGroup($msg, dict('GEN_LBL_GROUPCODE') . ':', 'ACGR_idcode', (array_key_exists('ACGR_idcode', $record)) ? $record['ACGR_idcode'] : '', $table);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE', dict('GEN_LBL_USER')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents;
    }

    function checkIncludeForm(&$msg) {
        $msg = array();
        //Validate Group
        if (!$this->groupExists(UserRequest::$post['ACGR_idcode'])) {
            $msg['ACGR_idcode'] = dict('TOOL_ACGR_NOGROUPFOUND');
        }
        //Validate if user exists
        if (ClxCommon::$db->oneExistsRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(GROUPCODE) =' . singleQuote(uppercase(UserRequest::$post['ACGR_idcode'])) . ' AND USERNAME = ' . singleQuote(UserRequest::$post['ACGR_username'])) > CLX_NUM_ZERO) {
            $msg['ACGR_idcode'] = dict('GEN_ACGR_USERALREADYEXISTS');
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACGR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACGR_idname']);
    }

    function viewFormFill($record) {
        addHistory('groupcode', $record['ACGR_idcode']);
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $status = array(dict('GEN_LBL_INACTIVE'), dict('GEN_LBL_ACTIVE'));
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACGR_TITLE_VIEW') . ': ' . $record['ACGR_idcode'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACGR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACGR_idname']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ACGR_description']));
        $table->addData(dict('GEN_LBL_GROUPAUDIT') . ':', $yesno[$record['ACGR_audit']]);
        $table->addData(dict('GEN_LBL_STATUS') . ':', $status[$record['ACGR_status']]);
        $contents .= $table->source();
        $ACUS = &getTool('ACUS');
        $contents .= $ACUS->makepanel($record['ACGR_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['ACGR_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['ACGR_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ACGR_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ACGR_idcode', UserRequest::$post))
            UserRequest::$post['ACGR_idcode'] = '';
        if (!array_key_exists('ACGR_idname', UserRequest::$post))
            UserRequest::$post['ACGR_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ACGR_idcode', UserRequest::$post['ACGR_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ACGR_idname', UserRequest::$post['ACGR_idname'], CLX_NUM_32, CLX_NUM_32);
        $yesno = new htmlSelect('ACGR_audit');
        $yesno->addItem('', '', true);
        $yesno->addItem(dict('GEN_LBL_YES'), '1', false);
        $yesno->addItem(dict('GEN_LBL_NO'), '0', false);
        $table->addData(dict('GEN_LBL_GROUPAUDIT') . ':', $yesno->source());
        $status = new htmlSelect('ACGR_status');
        $status->addItem('', '', true);
        $status->addItem(dict('GEN_LBL_INACTIVE'), '0', false);
        $status->addItem(dict('GEN_LBL_ACTIVE'), '1', false);
        $table->addData(dict('GEN_LBL_STATUS') . ':', $status->source());
        $sort = new HtmlSelect('ACGR_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Code', 'idcode', false);
        $sort->addItem('Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACGR_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACGR_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACGR_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACGR_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACGR_audit'])) {
            $filter .= $boolean . ' AUDITU=' . UserRequest::$post['ACGR_audit'];
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACGR_status'])) {
            $filter .= $boolean . ' STATUS=' . UserRequest::$post['ACGR_status'];
            $boolean = 'AND';
        }
        $_SESSION['paging']['ACGRList']['sortby'] = UserRequest::$post['ACGR_sort'];
        $_SESSION['paging']['ACGRList']['criteria'] = $filter;
        $_SESSION['paging']['ACGRList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACGRList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_AC_GROUPS', 'ID=' . $recordid);
        $record = array();
        $record['ACGR_id'] = $recordid;
        $record['ACGR_idcode'] = $values['IDCODE'];
        $record['ACGR_idname'] = $values['IDNAME'];
        $record['ACGR_description'] = $values['DESCRIPTION'];
        $record['ACGR_status'] = $values['STATUS'];
        $record['ACGR_audit'] = $values['AUDITU'];
        $record['ACGR_cdate'] = xFormat::date($values['CDATE']);
        $record['ACGR_mdate'] = xFormat::date($values['MDATE']);
        $record['ACGR_username'] = $values['USERNAME'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['ACGR_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['ACGR_description'];
        $record['STATUS'] = UserRequest::$post['ACGR_status'];
        $record['AUDITU'] = UserRequest::$post['ACGR_audit'];
        $record['MDATE'] = $d;
        ClxCommon::$db->updateRecord('CLX_AC_GROUPS', $record, 'ID=' . UserRequest::$post['ACGR_id']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ACGR_idcode'];
        $record['IDNAME'] = UserRequest::$post['ACGR_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['ACGR_description'];
        $record['STATUS'] = UserRequest::$post['ACGR_status'];
        $record['AUDITU'] = UserRequest::$post['ACGR_audit'];
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $recordid = ClxCommon::$db->createRecord('CLX_AC_GROUPS', $record, false);
        $callingpage = peekHistory();
        if ($callingpage[CLX_PROP_TO] === 'ACUS') {
            $record = array();
            $record['USERNAME'] = $callingpage['username'];
            $record['GROUPCODE'] = uppercase(UserRequest::$post['ACGR_idcode']);
            ClxCommon::$db->createRecord('CLX_AC_GROUP_MEMBERS', $record, false);
        }
        return $recordid;
    }

    function deleteRecordSk($id) {
        $grouprec = ClxCommon::$db->getRecord('CLX_AC_GROUPS', 'ID=' . $id);
        $groupcode = uppercase($grouprec['IDCODE']);
        ClxCommon::$db->deleteRecords('CLX_AC_GROUP_MEMBERS', 'UPPER(GROUPCODE)=' . singleQuote($groupcode));
        ClxCommon::$db->deleteRecords('CLX_AC_TOOLS', 'OWNERCODE=' . singleQuote('GRUP') . ' AND UPPER(OWNERREF)=' . singleQuote($groupcode));
        ClxCommon::$db->deleteRecords('CLX_AC_MENUITEMS', 'OWNERCODE=' . singleQuote('GRUP') . ' AND UPPER(OWNERREF)=' . singleQuote($groupcode));
        ClxCommon::$db->deleteRecords('CLX_AC_TABLES', 'OWNERCODE=' . singleQuote('GRUP') . ' AND UPPER(OWNERREF)=' . singleQuote($groupcode));
        ClxCommon::$db->deleteRecords('CLX_AC_GROUPS', 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['ACGR_id'] = CLX_INIT_ZERO;
        $record['ACGR_idcode'] = '';
        $record['ACGR_idname'] = '';
        $record['ACGR_description'] = '';
        $record['ACGR_status'] = CLX_NUM_ONE;
        $record['ACGR_audit'] = CLX_INIT_ZERO;
        $record['ACGR_cdate'] = '';
        $record['ACGR_mdate'] = '';
        $record['ACGR_username'] = $_SESSION['user_record']['username'];
        return $record;
    }

    function placeChooseGroup($msg, $prompt, $name, $default, &$table) {
        $searchIcon = getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=ACGR&ac=SEARCH&target=' . $name) . ', 500, 300);', 'Search for a user group');
        if (!empty($msg[$name])){
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_32);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function groupExists(&$groupcode) {
        if (empty($groupcode)) {
            $qty = CLX_INIT_ZERO;
        } else {
            $qty = ClxCommon::$db->oneExistsRecords('CLX_AC_GROUPS', 'UPPER(IDCODE)=' . singleQuote(uppercase($groupcode)));
            if ($qty > CLX_NUM_ZERO) {
                $groupcode = ClxCommon::$db->getField('CLX_AC_GROUPS', 'IDCODE', 'UPPER(IDCODE)=' . singleQuote(uppercase($groupcode)));
            }
        }
        return $qty > CLX_NUM_ZERO;
    }

}
