<?php

/**
 * Reports Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Groups
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class an_report_groups extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ANRG';
        $this->name = 'Report Groups';
        $this->system = 'Analytics';
        $this->version = '0.1.2.1';


        $this->accesskey = 'ANRP';

        $this->tablename = 'CLX_AN_REPORT_GROUPS';
        $this->record_name_singular = 'report group';
        $this->record_name_plural = 'report groups';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
        $this->seqnum_field = 'SEQNUM';
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['REPORTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FORMAT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['BOOKMARKNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['REPQRYCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');

        $fields['FIELDLISTLABEL'] = array('type' => 'varchar(64)', 'properties' => '', 'dict' => true);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'REPORTCODE,SEQNUM');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'REPQRYCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- R E P O R T    G R O U P S
    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $id = $record['ID'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANRX', 'SELECT * FROM CLX_AN_REPORT_GROUP_FUNC WHERE GROUPID=' . $id);

        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_AN_REPORT_GROUPS X WHERE DICT.ENTRY=X.FIELDLISTLABEL AND X.ID=' . singleQuote($id));
        $o->addObject($objs);
        return $o;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $groupid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $groupid));
            }
        }
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($reportcode = null) {
        if ($this->getAccess()->canRead()) {

            $DICT = &getTool('DICT');
            $report_type = ClxCommon::$db->getField('CLX_AN_REPORTS', 'REPORT_TYPE', 'IDCODE = ' . singleQuote($reportcode));
            $title = ($report_type == CLX_NUM_ZERO) ? dict('TOOL_ANRG_TITLE_LIST') : dict('TOOL_ANRG_BOOKMARKS');
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $yesno = array();
            foreach ($this->yesno as $key => $val)
                $yesno[$key] = dict($val);
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'ANRGPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('reportcode', $reportcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            if ($report_type == CLX_NUM_ONE) {//Template type
                $selectclause = 'G.*, Q.IDNAME QRYNAME';
                $fromclause = 'CLX_AN_REPORT_GROUPS G, CLX_AN_REPORT_QUERIES Q';
                $whereclause = 'G.REPQRYCODE = Q.IDCODE AND G.REPORTCODE=' . singleQuote($reportcode);
                $sortclause = 'G.SEQNUM';
            } else {
                $selectclause = 'T.IDNAME TABLENAME, T.MODULECODE MODULECODE, G.*';
                $fromclause = 'CLX_AN_REPORT_GROUPS G, CLX_DB_TABLES T';
                $whereclause = 'G.REPORTCODE=' . singleQuote($reportcode) . ' AND G.TABLECODE=T.IDCODE';
                $sortclause = 'G.SEQNUM';
            }
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            if ($report_type == CLX_NUM_ONE) {//Template type
                $table = new htmlTable(CLX_WIDTH_FOUR);
                $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

                $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_BOOKMARKNAME'), dict('GEN_LBL_REPQRYCODE'), dict('GEN_LBL_ACTIVE')));
            } else {
                $table = new htmlTable(CLX_WIDTH_SIX);
                $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

                //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_SEL'), dict('GEN_LBL_TABLE'),dict('GEN_LBL_FIELDNAME'),dict('GEN_LBL_FORMAT'), dict('GEN_LBL_ACTIVE')));
                $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_COLLABEL_DEF'), dict('GEN_LBL_COLLABEL_EV'), dict('GEN_LBL_FORMAT'), dict('GEN_LBL_ACTIVE')));
            }
            $page->header($table);
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                if ($report_type == CLX_NUM_ONE) {//Template type
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                        $record['BOOKMARKNAME'], $record['REPQRYCODE'] . ', ' . $record['QRYNAME'], $yesno[$record['ACTIVE']]));
                } else {

                    $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($record['FIELDNAME']));
                    if (is_null($record['FIELDLISTLABEL']) || $record['FIELDLISTLABEL'] === '') {
                        $collabel = '';
                    } else {
                        $collabel = $DICT->dictFastEdit($record['FIELDLISTLABEL']);
                    }
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
                        $record['TABLECODE'] . '.' . $record['FIELDNAME'], dict($entry), $collabel, dict($this->format_types[$record['FORMAT']]), $yesno[$record['ACTIVE']]));
                }
                $i++;
            }
            $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                if ($report_type === CLX_NUM_ONE) {
                    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
                } else {
                    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
                }
            }
            if ($this->getAccess()->canDisplay()) {
                if ($report_type === CLX_NUM_ONE) {
                    $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
                } else {
                    $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_FIELD')), 'CREATE', $form->getProp(CLX_PROP_ID));
                    $b->setProp(CLX_PROP_ID, 'ON');
                    $buttons .= $b->source();
                }
            }
            if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface($title, $reportcode . 'GROUPS', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {

        $DICT = &getTool('DICT');
        $reportcode = $record['ANRG_reportcode'];
        $reportrec = ClxCommon::$db->getRecord('CLX_AN_REPORTS', 'IDCODE=' . singleQuote($reportcode));
        $report_type = $reportrec['REPORT_TYPE'];
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        if ($reportrec['REPORT_TYPE'] === CLX_NUM_ONE) {
            $title = ($modify) ? dict('TOOL_ANRG_BOOKMARK_MODIFY') : dict('TOOL_ANRG_BOOKMARK_CREATE');
        } else {
            $title = ($modify) ? dict('TOOL_ANRG_TITLE_MODIFY') : dict('TOOL_ANRG_TITLE_CREATE');
        }
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('report_type', $reportrec['REPORT_TYPE']); //just to simplify validation on checkForm()
        $form->addArgument('ANRG_id', $record['ANRG_id']);
        $form->addArgument('ANRG_reportcode', $record['ANRG_reportcode']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if ($reportrec['REPORT_TYPE'] === CLX_NUM_ONE) {//type template
            $table->addField($msg, dict('GEN_LBL_BOOKMARKNAME'), 'ANRG_bookmarkname', $record['ANRG_bookmarkname'], CLX_NUM_30, CLX_NUM_64, true);
            $ANRQ = getTool('ANRQ');
            if (!empty($msg['ANRG_repqrycode'])) {
                $table->addCell($msg['ANRG_repqrycode'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
            }
            $table->addCell(dict('GEN_LBL_REPQRYCODE'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPTREQUIRED));
            $table->addCell($ANRQ->chooseRecord('ANRG_repqrycode', $record['ANRG_repqrycode'], array('ANRG_reportcode' => $record['ANRG_reportcode'])), CLX_PROP_CLASS, CLX_CLASS_DATA);
            $form->addArgument('ANRG_tablecode_fieldname', $record['ANRG_tablecode'] . '.' . $record['ANRG_fieldname']);
            $form->addArgument('ANRG_tablecode', $record['ANRG_tablecode']);
            $form->addArgument('ANRG_fieldname', $record['ANRG_fieldname']);
            $form->addArgument('ANRG_format', $record['ANRG_format']);
        } else {
            //set not used argument according type
            $form->addArgument('ANRG_repqrycode', $record['ANRG_repqrycode']);
            $form->addArgument('ANRG_bookmarkname', $record['ANRG_bookmarkname']);
            $DBQT = &getTool('DBQT');
            $fields = $DBQT->getTableFieldsSql($reportrec['QUERYCODE']);
            $pop = new htmlSelect('ANRG_tablecode_fieldname');
            while ($field = ClxCommon::$db->getNextRecord($fields)) {
                $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($field['FIELDNAME']));
                if (array_key_exists('MODULECODE', $field)) {
                    $fieldname = $field['MODULECODE'] . '_' . $field['IDNAME'] . '.' . $field['FIELDNAME'];
                } else {
                    $fieldname = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
                }
                $fieldcode = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
                $pop->addItem($fieldname . ' - ' . dict($entry), $fieldcode, ($fieldcode == ($record['ANRG_tablecode'] . '.' . $record['ANRG_fieldname'])));
            }
            $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $pop->source());
            //$table->addYesNo(dict('GEN_LBL_FORMAT'),'ANRG_format',$record['ANRG_format'],0);
            //$table->addList(dict('GEN_LBL_FORMAT'), 'ANRG_format', $record['ANRG_format'], $this->yesno, false);
            $table->addList(dict('GEN_LBL_FORMAT'), 'ANRG_format', $record['ANRG_format'], $this->format_types, false);
        }//end if report format
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'ANRG_active', $record['ANRG_active'], CLX_NUM_ZERO);

        //-- Label
        if ($report_type === CLX_NUM_ONE) {
            $form->addArgument('ANRG_fieldlistlabel', '');
        } else {
            $table2 = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
            /*
             * GEN_LBL_COLLABEL_EV
             * Column Title (Specific)
             * Título Columna (Específico)
             */
            $DICT->editEntry($msg, dict('GEN_LBL_COLLABEL_EV') . ':', 'ANRG_fieldlistlabel', $record, $table2, $form);
            $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'CREATE', $form->getProp(CLX_PROP_ID));
            $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        if (UserRequest::$post['report_type'] == CLX_NUM_ONE && empty(UserRequest::$post['ANRG_bookmarkname'])) {
            $msg['ANRG_bookmarkname'] = dict('GEN_LBL_FIELD_REQUIRED');
        }
        if (UserRequest::$post['report_type'] == CLX_NUM_ONE && empty(UserRequest::$post['ANRG_repqrycode'])) {
            $msg['ANRG_repqrycode'] = dict('GEN_LBL_FIELD_REQUIRED');
        }

        $DICT = &getTool('DICT');
        $DICT->validateEntry('ANRG_fieldlistlabel', UserRequest::$post, false, $msg);
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        if (!empty($record['ANRG_bookmarkname'])) {
            $table->addData(dict('GEN_LBL_BOOKMARKNAME') . ':', $record['ANRG_bookmarkname']);
            $table->addData(dict('GEN_LBL_REPQRYCODE') . ':', $record['ANRG_repqrycode']);
        } else {
            $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ANRG_tablecode']);
            $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANRG_fieldname']);
        }
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        addHistory('groupid', $record['ANRG_id']);
        addHistory('reportcode', $record['ANRG_reportcode']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANRG_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($record['ANRG_bookmarkname'])) {
            $table->addData(dict('GEN_LBL_BOOKMARKNAME') . ':', $record['ANRG_bookmarkname']);
            $table->addData(dict('GEN_LBL_REPQRYCODE') . ':', $record['ANRG_repqrycode']);
        } else {
            $table->addData(dict('GEN_LBL_TABLECODE') . ':', tableName($record['ANRG_tablecode']));
            $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANRG_fieldname']);
        }
        $table->addData(dict('GEN_LBL_ACTIVE') . ':', dict($this->yesno[$record['ANRG_active']]));
        if (!empty($record['ANRG_fieldlistlabel'])) {
            $table->addData(dict('GEN_LBL_COLLABEL_EV') . ':', dict($record['ANRG_fieldlistlabel']));
        }
        $contents .= $table->source();
        $ANRX = &getTool('ANRX');
        $contents .= $ANRX->makepanel($record['ANRG_id']);
        return $contents;
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $sql = 'SELECT G.FIELDNAME, G.ID, G.BOOKMARKNAME, T.MODULECODE, T.IDNAME FROM CLX_AN_REPORT_GROUPS G LEFT JOIN CLX_DB_TABLES T ON G.TABLECODE = T.IDCODE WHERE G.REPORTCODE=' . singleQuote(UserRequest::$post['reportcode']) . ' 				ORDER BY G.SEQNUM';
        $rs = ClxCommon::$db->executeQuery($sql);
        $list = array();
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (!empty($r['BOOKMARKNAME'])) {
                $list[$r['ID']] = $r['BOOKMARKNAME'];
            } else {
                $list[$r['ID']] = $r['MODULECODE'] . '_' . $r['IDNAME'] . '.' . $r['FIELDNAME'];
            }
        }
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANRG_id'] = $recordid;
        $record['ANRG_reportcode'] = $values['REPORTCODE'];
        $record['ANRG_tablecode'] = $values['TABLECODE'];
        $record['ANRG_fieldname'] = $values['FIELDNAME'];
        $record['ANRG_bookmarkname'] = $values['BOOKMARKNAME'];
        $record['ANRG_repqrycode'] = $values['REPQRYCODE'];
        $record['ANRG_seqnum'] = $values['SEQNUM'];
        $record['ANRG_format'] = $values['FORMAT'];
        $record['ANRG_active'] = $values['ACTIVE'];

        $DICT = &getTool('DICT');
        $DICT->loadEntry('ANRG_fieldlistlabel', $values['FIELDLISTLABEL'], $record);
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $data = UserRequest::$post['ANRG_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['REPORTCODE'] = UserRequest::$post['ANRG_reportcode'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['FORMAT'] = UserRequest::$post['ANRG_format'];
        $record['BOOKMARKNAME'] = UserRequest::$post['ANRG_bookmarkname'];
        $record['REPQRYCODE'] = UserRequest::$post['ANRG_repqrycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRG_active'];

        if (!isset(UserRequest::$post['ANRG_fieldlistlabel_DICT']) || UserRequest::$post['ANRG_fieldlistlabel_DICT'] === '') {
            $record['FIELDLISTLABEL'] = 'null';
        } else {
            $DICT = &getTool('DICT');
            $record['FIELDLISTLABEL'] = $DICT->createEntry('ANRG_fieldlistlabel', UserRequest::$post);
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANRG_id']);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $data = UserRequest::$post['ANRG_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['REPORTCODE'] = UserRequest::$post['ANRG_reportcode'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['FORMAT'] = UserRequest::$post['ANRG_format'];
        $record['BOOKMARKNAME'] = UserRequest::$post['ANRG_bookmarkname'];
        $record['REPQRYCODE'] = UserRequest::$post['ANRG_repqrycode'];
        $record['ACTIVE'] = UserRequest::$post['ANRG_active'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'REPORTCODE=' . singleQuote($record['REPORTCODE']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;

        if (!isset(UserRequest::$post['ANRG_fieldlistlabel_DICT']) || UserRequest::$post['ANRG_fieldlistlabel_DICT'] === '') {
            $record['FIELDLISTLABEL'] = 'null';
        } else {
            $DICT = &getTool('DICT');
            $record['FIELDLISTLABEL'] = $DICT->createEntry('ANRG_fieldlistlabel', UserRequest::$post);
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $record['REPORTCODE']);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $grouprec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_AN_REPORT_GROUP_FUNC', 'GROUPID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $grouprec['REPORTCODE']);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        $record = array();
        $record['ANRG_id'] = CLX_INIT_ZERO;
        $record['ANRG_reportcode'] = $reportcode;
        $record['ANRG_tablecode'] = '';
        $record['ANRG_fieldname'] = '';
        $record['ANRG_bookmarkname'] = '';
        $record['ANRG_repqrycode'] = '';
        $record['ANRG_seqnum'] = '0';
        $record['ANRG_format'] = '0';
        $record['ANRG_active'] = '1';

        $record['ANRG_fieldlistlabel'] = false;
        $record['ANRG_fieldlistlabel_DICT'] = '';
        return $record;
    }

}
