<?php

/**
 * Reports Query Criterias Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-15 20:57:54 -0400 (Mon, 15 Oct 2018) $
 * @version    	$Rev: 10266 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Columns
 */
class an_report_query_crits extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ANQC';
        $this->name = 'Report Queries Criteries';
        $this->system = 'Analytics';
        $this->version = '0.1.2.0';


        $this->accesskey = 'ANRP';

        $this->tablename = 'CLX_AN_REP_QRY_CRITS';
        $this->record_name_singular = 'report query criteria';
        $this->record_name_plural = 'report query criterias';
        $this->primary_field = 'ID';
        $this->secondary_field = 'REPQRYCODE';

        $ops = array();
        foreach ($this->operators as $key => $val) {
            $ops[$key] = $val[CLX_NUM_ZERO];
        }
        $this->viewform = array(CLX_CLASS_TITLE => 'TOOL_ANQC_TITLE_DISPLAY',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_REPQRYCODE',
                            'fields' => array(
                                array('name' => 'repqrycode',
                                    'type' => 'lookup',
                                    'default' => '',
                                    'readonly' => true,
                                    'tool' => 'ANRQ',
                                    'validations' => array(
                                        array('type' => 'lookup',
                                            'fieldname' => 'idcode',
                                            'tablename' => 'CLX_AN_REPORT_QUERIES'
                                        )
                                    )
                                )
                            )
                        ), array('label' => 'GEN_LBL_FIELDNAME1',
                            'fields' => array(
                                array('name' => 'fieldname1', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16'
                                )
                            )
                        ), array('label' => 'GEN_LBL_OPERATOR',
                            'fields' => array(
                                array('name' => 'operator',
                                    'type' => CLX_CLASS_LIST,
                                    'options' => $ops
                                )
                            )
                        ), array('label' => 'GEN_LBL_FIELDNAME2',
                            'fields' => array(
                                array('name' => 'fieldname2', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16'
                                )
                            )
                        ), array('label' => 'GEN_LBL_VALUE',
                            'fields' => array(
                                array('name' => 'value',
                                    'type' => 'string',
                                    'default' => '',
                                    'size' => '64',
                                    'displaysize' => '48'
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->toolpanelkeys = array(array('name' => 'REPQRYCODE', 'type' => 'string'));
        $this->toolpanel = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'has_reorder' => true,
            'fields' =>
            array(array('name' => 'ID', 'type' => 'number'), array('name' => 'FIELDNAME1', 'type' => 'string'), array('name' => 'OPERATOR', 'type' => CLX_CLASS_LIST, 'options' => $ops), array('name' => 'REPQRYCODE2', 'type' => 'string'), array('name' => 'FIELDNAME2', 'type' => 'string'), array('name' => 'VALUE', 'type' => 'string'))
        );
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['REPQRYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'default' => '');
        $fields['FIELDNAME1'] = array('type' => 'varchar(64)', 'null' => 'NO');
        //$fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'default' => '"EQ"', 'default_raw' => 'EQ');
        $fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'default' => singleQuote('EQ'), 'default_raw' => 'EQ');
        $fields['REPQRYCODE2'] = array('type' => 'varchar(16)', 'null' => 'YES', 'default' => '');
        $fields['FIELDNAME2'] = array('type' => 'varchar(64)', 'null' => 'YES');
        $fields['FIELDALIAS2'] = array('type' => 'varchar(64)', 'null' => 'YES');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'REPQRYCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        return parent::pack($record);
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $reportid) = $parameters;
        parent::unpack(array('package' => $package, 'id' => $reportid));
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        $rep_qry_rec = ClxCommon::$db->getRecord('CLX_AN_REPORT_QUERIES', 'IDCODE=' . singleQuote($record['ANQC_repqrycode']));
        // Query Fields -------------------------
        $DBQT = &getTool('DBQT');
        $fields = $DBQT->getTableFieldsSql($rep_qry_rec['QUERYCODE'], null, true);
        $queryfields = new htmlSelect('ANQC_fieldname1');
        $queryfields->addItem('', '', ($record['ANQC_fieldname1'] === ''));
        while (($field = ClxCommon::$db->getNextRecord($fields))) {
            $fieldname = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
            $fieldcode = $field['TABLECODE'] . '.' . $field['FIELDNAME'];
            $queryfields->addItem($fieldname, $fieldcode, ($fieldcode == $record['ANQC_fieldname1']));
        }
        // Form Fields -------------------------
        $formfields = new htmlSelect('ANQC_fieldname2');
        $formfields->addItem('', '', ($record['ANQC_fieldname1'] === ''));
        $sql = 'SELECT IDCODE, IDNAME, QUERYCODE FROM CLX_AN_REPORT_QUERIES  WHERE REPORTCODE = ' . singleQuote($rep_qry_rec['REPORTCODE']) . ' AND IDCODE <>' . singleQuote($rep_qry_rec['IDCODE']);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $formfields->addItem(uppercase('---- ' . $r['IDCODE'] . ': ' . substr($r['IDNAME'], CLX_NUM_ZERO, CLX_NUM_30) . ' (' . $r['QUERYCODE'] . ') ----'), '', false);
            $fields = $DBQT->getTableFieldsSql($r['QUERYCODE'], null, false);
            while (($field = ClxCommon::$db->getNextRecord($fields))) {
                $fieldname = '[' . $r['IDCODE'] . '] ' . $field['TABLECODE'] . '_' . $field['FIELDNAME'];
                $fieldcode = $r['IDCODE'] . '.' . $field['TABLECODE'] . '.' . $field['FIELDNAME'];
                $formfields->addItem($fieldname, $fieldcode, ($fieldcode == $record['ANQC_repqrycode2'] . '.' . $record['ANQC_fieldname2']));
            }
        }//end while
        $title = dict('TOOL_ANQC_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANQC_id', $record['ANQC_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('ANQC_repqrycode', $record['ANQC_repqrycode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('GEN_LBL_REPQRYCODE'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($rep_qry_rec['IDCODE'] . ', ' . $rep_qry_rec['IDNAME'] . '(' . $rep_qry_rec['QUERYCODE'] . ')', array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addData(dict('TOOL_ANQC_PROMPT_QUERYFIELD') . ':', $queryfields->source());
        // The operators
        $pop = new HTMLSelect($this->code . '_operator');
        foreach ($this->operators as $key => $value) {
            $pop->addItem(dict($value[CLX_NUM_ZERO]), $key, ($key == $record[$this->code . '_operator']));
        }
        $table->addData(dict('GEN_LBL_OPERATORS') . ':', $pop->source());
        $field = new htmlTextArea('ANQC_value', $record['ANQC_value'], CLX_NUM_7, CLX_NUM_50);
        if (!empty($msg[$this->code . '_frm_fieldname']))
            $table->addCell($msg[$this->code . '_frm_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addData(dict('TOOL_ANQC_PROMPT_FORMFIELDORVALUE') . ':', $formfields->source() . '<br>' . dict('GEN_LBL_OR') . '<br>' . $field->source());
        $form->addContents($table->source());
        $buttons = '';

        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANQC_id'] = $recordid;
        $record['ANQC_repqrycode2'] = $values['REPQRYCODE2'];
        $record['ANQC_repqrycode'] = $values['REPQRYCODE'];
        $record['ANQC_fieldname1'] = $values['FIELDNAME1'];
        $record['ANQC_fieldname2'] = $values['FIELDNAME2'];
        $record['ANQC_operator'] = $values['OPERATOR'];
        $record['ANQC_value'] = $values['VALUE'];
        return $record;
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['REPQRYCODE'] = UserRequest::$post['ANQC_repqrycode'];
        $record['FIELDNAME1'] = UserRequest::$post['ANQC_fieldname1'];
        if (strlen(UserRequest::$post['ANQC_value']) > CLX_NUM_ZERO) {
            $record['VALUE'] = UserRequest::$post['ANQC_value'];
            $record['REPQRYCODE2'] = 'null';
            $record['FIELDNAME2'] = 'null';
            $record['FIELDALIAS2'] = 'null';
        } else {
            $a = explode('.', UserRequest::$post['ANQC_fieldname2']);
            $record['REPQRYCODE2'] = $a[CLX_NUM_ZERO];
            $record['FIELDNAME2'] = $a[CLX_NUM_ONE] . '.' . $a[CLX_NUM_TWO];
            $record['FIELDALIAS2'] = $a[CLX_NUM_ONE] . '_' . $a[CLX_NUM_TWO];
            $record['VALUE'] = 'null';
        }
        $record['OPERATOR'] = UserRequest::$post['ANQC_operator'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANRQ = &getTool('ANRQ');
        $ANRQ->versionIncrease('IDCODE', UserRequest::$post['ANQC_repqrycode']);
        return $recordid;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $record = array();
        $record['REPQRYCODE'] = UserRequest::$post['ANQC_repqrycode'];
        $record['FIELDNAME1'] = UserRequest::$post['ANQC_fieldname1'];
        if (strlen(UserRequest::$post['ANQC_value']) > CLX_NUM_ZERO) {
            $record['VALUE'] = clx_html_spchrs_decode(UserRequest::$post['ANQC_value']);
            $record['REPQRYCODE2'] = 'null';
            $record['FIELDNAME2'] = 'null';
            $record['FIELDALIAS2'] = 'null';
        } else {
            $a = explode('.', UserRequest::$post['ANQC_fieldname2']);
            $record['REPQRYCODE2'] = $a[CLX_NUM_ZERO];
            $record['FIELDNAME2'] = $a[CLX_NUM_ONE] . '.' . $a[CLX_NUM_TWO];
            $record['FIELDALIAS2'] = $a[CLX_NUM_ONE] . '_' . $a[CLX_NUM_TWO];
            $record['VALUE'] = 'null';
        }
        $record['OPERATOR'] = UserRequest::$post['ANQC_operator'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANQC_id']);
        $ANRQ = &getTool('ANRQ');
        $ANRQ->versionIncrease('IDCODE', UserRequest::$post['ANQC_repqrycode']);
        return UserRequest::$post['ANQC_id'];
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $repqry = ClxCommon::$db->getField($this->tablename, 'REPQRYCODE', 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANRQ = &getTool('ANRQ');
        $ANRQ->versionIncrease('IDCODE', $repqry);
        return $id;
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['ANQC_id'] = '';
        $record['ANQC_repqrycode'] = $callingpage['repqrycode'];
        $record['ANQC_repqrycode2'] = '';
        $record['ANQC_fieldname1'] = '';
        $record['ANQC_fieldname2'] = '';
        $record['ANQC_operator'] = '';
        $record['ANQC_value'] = '';
        return $record;
    }

}
