<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class codelib extends clx_tool {

    var $types_code;

    function __construct() {
        $this->code = 'CODE';
        $this->name = 'Code Library Manager';
        $this->system = 'System';
        $this->version = '0.5.6.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_CODELIB';
        $this->record_name_singular = 'code library';
        $this->record_name_plural = 'code libraries';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->primary_key = array('IDCODE');
        $this->types_code = array('ENBD' => 'GEN_LBL_EMBEDDED', 'LINK' => 'GEN_LBL_LINKED');
        $this->languages = array('PHP' => 'PHP', 'JS' => 'JavaScript', 'PCRE' => 'Regular expression pattern (preg_match)', 'SQL' => 'SQL Script');
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_CODELIB', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['CONTENTS'] = array('type' => 'text', 'properties' => '');
        $fields['FILENAME'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['LANGUAGE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'TYPE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'LANGUAGE');
        $indexes[$ipf . '07'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function packRecord($record) {
        if ($record['TYPE'] === 'LINK') {
            if ($record['FILENAME'] === '') {
                $BUGS = getTool('BUGS');
                if (!empty($BUGS)) {
                    $BUGS->report(E_USER_ERROR, 'Missing Codelib File Path ', 'tools/codelib.php', '108', $record, CLX_NUM_ONE, '');
                }
            }//pre( $record);
            $record['CONTENTS'] = file_get_contents($record['FILENAME']);
        }
        $record['CONTENTS'] = base64_encode($record['CONTENTS']);
        return parent::packRecord($record);
    }

    function unpackRecord($o) {
        $record = parent::unpackRecord($o);
        $record['CONTENTS'] = base64_decode($record['CONTENTS']);
        //$record['CONTENTS'] = stripslashes($record['CONTENTS']);
        if ($record['TYPE'] === 'LINK') {
            $file = new clx_file($record['FILENAME'], 'w');
            $file->open();
            if(!$file->hasError() ){

                $file->write($record['CONTENTS']);
                $file->close();
            }else{
                echo $file->getError().'<br>';
            }

            /*if (($fd = fopen($record['FILENAME'], 'w')) === true) {
                fwrite($fd, $record['CONTENTS']);
                fclose($fd);
            }*/
        }
        return $record;
    }

    function action($ac, $idlist) {
        $msg = array();
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'CREATE':
                    if ($this->getAccess()->canDisplay()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($this->checkForm($msg)) {
                                $id = $this->createRecord_();
                                $record = $this->loadToolRecord($id);
                                $msg = array();
                                if ($record['CODE_type'] === 'LINK') {
                                    $msg['CODE_filename'] = 'Please type a filename';
                                } else {
                                    $msg['CODE_contents'] = 'Please enter the code';
                                }
                                $contents = $this->editForm('MODIFY', array(), $record, $msg);
                            } else {
                                $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
                            }
                        } else {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm($ac, array(), $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CODELIB-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'WRITE_EMBD_CODE':
                    $codelib = '';
                    $this->writeEmbedPhpSources($codelib);
                    $contents = note(dict('GEN_LBL_COMPLETED'));
                    break;
                case 'EVNT_COMPILE_CODE':
                    $this->clearEmbededSources();
                    $contents = $this->compileEmbededPhpSourcesRun();
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CODELIB-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Buils listing for tool
     * @param type $listingcode
     * @return type
     */
    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'CODEList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_CODELIB';
        $whereclause = '';
        $sortclause = 'IDCODE, IDNAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_NUM_8);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_CODE_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_8));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_LANGUAGE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['MODULECODE'], $record['IDCODE'], $record['IDNAME'], $this->languages[$record['LANGUAGE']], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], dict($this->types_code[$record['TYPE']]), xFormat::text($record['DESCRIPTION'])));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_LIBRARY')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }

        if($this->getAccess()->isAdmin()){
            $buttons .=htmlButton::getButton('goto', dict('GEN_LBL_COMPILE'), array('url' => 'tool.php?to=EVNT&ac=EVNT_COMPILE_CODE'))->source();
        }


        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    /**
     * Search form for tool
     * @return type
     */
    function searchForm() {
        $language = (array_key_exists('language', UserRequest::$request)) ? UserRequest::$request['language'] : CLX_CHAR_EMPTY_STRING;
        $action = (array_key_exists(CLX_PROP_ACTION, UserRequest::$request)) ? UserRequest::$request[CLX_PROP_ACTION] : CLX_CHAR_EMPTY_STRING;
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument('language', $language);
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $form->addArgument(CLX_PROP_ACTION, $action);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_CODE_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'CODE_idcode', (array_key_exists('CODE_idcode', UserRequest::$post)) ? UserRequest::$post['CODE_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'CODE_idname', (array_key_exists('CODE_idname', UserRequest::$post)) ? UserRequest::$post['CODE_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'CODE_modulecode', (array_key_exists('CODE_modulecode', UserRequest::$post)) ? UserRequest::$post['CODE_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        $boolean = '';
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            if (empty($language)) {
                $criteria = '';
            } else {
                if (!empty($action)) {
                    switch ($action) {
                        case 'CODE':
                            $lang = 'PHP';
                            break;
                        case 'SQL':
                            $lang = 'SQL';
                            break;
                        default:
                            $lang = $language;
                            break;
                    }
                    $criteria = 'LANGUAGE=' . singleQuote($lang);
                    $boolean = ' AND ';
                } else {
                    $criteria = 'LANGUAGE=' . singleQuote($language);
                    $boolean = ' AND ';
                }
            }
            if (UserRequest::$post['CODE_idcode'] <> '') {
                $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['CODE_idcode']) . '%');
            }
            if (UserRequest::$post['CODE_idname'] <> '') {
                $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['CODE_idname'])) . '%');
            }
            if (UserRequest::$post['CODE_modulecode'] <> '') {
                $criteria .= $boolean . 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['CODE_modulecode']) . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=codelib></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = ($ac === 'MODIFY');
        $title = ($modify) ? dict('TOOL_CODE_TITLE_MODIFY') : dict('TOOL_CODE_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('CODE_id', $record['CODE_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $form->addArgument('CODE_idcode', $record['CODE_idcode']);
            $table->addData(dict('GEN_LBL_CODE') . ':', $record['CODE_idcode']);
        } else {
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'CODE_idcode', $record['CODE_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'CODE_idname', $record['CODE_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'CODE_version', $record['CODE_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'CODE_description', $record['CODE_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $language = new htmlSelect('CODE_language');
        foreach ($this->languages as $key => $val) {
            $language->addItem($val, $key, ($record['CODE_language'] == $key));
        }
        $table->addData(dict('GEN_LBL_LANGUAGE'), $language->source());
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('GEN_LBL_MODULECODE') . ':', 'CODE_modulecode', $record['CODE_modulecode'], $table);

        if (empty($modify)) {
            $pop = new HTMLSelect('CODE_type');
            foreach ($this->types_code as $key => $value) {
                $pop->addItem(dict($value), $key, ($record['CODE_type'] == $key));
            }
            $table->addData(dict('GEN_LBL_TYPE') . ':', $pop->source());
        } else {
            $form->addArgument('CODE_type', $record['CODE_type']);
            if ($record['CODE_type'] === 'LINK') {
                $table->addField($msg, dict('GEN_LBL_FILENAME') . ':', 'CODE_filename', $record['CODE_filename'], CLX_NUM_48, CLX_NUM_255);
            } elseif ($record['CODE_type'] === 'ENBD') {
                $table->addCell(dict('GEN_LBL_CONTENTS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                if (isset($msg['CODE_contents'])) {
                    $table->addCell($msg['CODE_contents'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                }
                $c = new htmlObject('textarea', htmlspecialchars(stripslashes($record['CODE_contents'])), array('name'=> 'CODE_contents', 'rows'=>CLX_NUM_30, 'cols'=>CLX_NUM_120));
                $c->forceclose = true;
                $table->addCell($c->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            }
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        if (empty($msg))
            $msg = array();
        $action = UserRequest::$post['ac'];
        if ($action === 'CREATE') {
            xValidate::required($msg, 'CODE_idcode', UserRequest::$post['CODE_idcode']);
            if (empty($msg)) {
                xValidate::unique($msg, 'CODE_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['CODE_idcode']));
            }
        }
        if (empty(UserRequest::$post['CODE_idname'])) {
            $msg['CODE_idname'] = dict('GEN_LBL_FIELD_REQUIRED');
        }
        if ($action === 'MODIFY') {
            if (UserRequest::$post['CODE_type'] === 'LINK') {
                xValidate::required($msg, 'CODE_filename', UserRequest::$post['CODE_filename']);
                if (!file_exists(UserRequest::$post['CODE_filename'])) {
                    $msg['CODE_filename'] = dict('GEN_ERR_FILE_NOT_FOUND');
                }
            } elseif (!empty(UserRequest::$post['CODE_type']) && UserRequest::$post['CODE_type'] === 'ENBD') {
                xValidate::required($msg, 'CODE_contents', UserRequest::$post['CODE_contents']);
            }
        }
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post['CODE_modulecode'])) {
            $msg['CODE_modulecode'] = 'There is no module with this code.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['CODE_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['CODE_idname']);
    }

    function viewFormFill($record) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_CODE_TITLE_VIEW') . ': ' . $record['CODE_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['CODE_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['CODE_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['CODE_version'] . '-' . $record['CODE_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['CODE_description']));
        $table->addData(dict('GEN_LBL_LANGUAGE') . ':', $this->languages[$record['CODE_language']]);
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['CODE_modulecode']);
        if ($record['CODE_type'] === 'LINK') {
            $table->addData(dict('GEN_LBL_FILENAME') . ':', $record['CODE_filename']);
        }
        $contents = $table->source();
        if ($record['CODE_type'] === 'ENBD') {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $record['CODE_contents'] = stripslashes($record['CODE_contents']);
            $text = new htmlTextArea('', $record['CODE_contents'], CLX_NUM_30, CLX_NUM_120);
            $text->setProp('readonly', 'true');
            $table->addCell($text->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $contents .= buildPanelInterface(dict('GEN_LBL_CONTENTS'), 'CONTENTS', $table->source(), CLX_NUM_MINUS_1);
        }
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table2->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table2->addData(dict('GEN_LBL_CDATE') . ':', $record['CODE_cdate']);
        $table2->addData(dict('GEN_LBL_MDATE') . ':', $record['CODE_mdate']);
        $table2->addData(dict('GEN_LBL_USERNAME') . ':', $record['CODE_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table2->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('CODE_idcode', UserRequest::$post)) {
            UserRequest::$post['CODE_idcode'] = '';
        }
        if (!array_key_exists('CODE_idname', UserRequest::$post)) {
            UserRequest::$post['CODE_idname'] = '';
        }
        if (!array_key_exists('CODE_language', UserRequest::$post)) {
            UserRequest::$post['CODE_language'] = '';
        }
        if (!array_key_exists('CODE_modulecode', UserRequest::$post)) {
            UserRequest::$post['CODE_modulecode'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_CODE'), 'CODE_idcode', UserRequest::$post['CODE_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'CODE_idname', UserRequest::$post['CODE_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_LANGUAGES'), 'CODE_language', UserRequest::$post['CODE_language'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_MODULECODE'), 'CODE_modulecode', UserRequest::$post['CODE_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['CODE_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['CODE_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['CODE_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['CODE_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['CODE_language'])) {
            $filter .= $boolean . ' UPPER(LANGUAGE) LIKE ' . singleQuote(uppercase(UserRequest::$post['CODE_language']). '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['CODE_modulecode'])) {
            $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['CODE_modulecode']) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['CODEList']['criteria'] = $filter;
        $_SESSION['paging']['CODEList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['CODEList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['CODE_id'] = $recordid;
        $record['CODE_idcode'] = $values['IDCODE'];
        $record['CODE_idname'] = $values['IDNAME'];
        $record['CODE_version'] = $values['VERSION'];
        $record['CODE_version_counter'] = $values['VERSION_COUNTER'];
        $record['CODE_cdate'] = xFormat::date($values['CDATE']);
        $record['CODE_mdate'] = xFormat::date($values['MDATE']);
        $record['CODE_username'] = $values['USERNAME'];
        $record['CODE_description'] = $values['DESCRIPTION'];
        $record['CODE_contents'] = clx_html_spchrs_decode($values['CONTENTS']);
        $record['CODE_language'] = $values['LANGUAGE'];
        $record['CODE_modulecode'] = $values['MODULECODE'];
        $record['CODE_type'] = $values['TYPE'];
        $record['CODE_filename'] = $values['FILENAME'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['CODE_idcode'];
        $record['IDNAME'] = UserRequest::$post['CODE_idname'];
        $record['VERSION'] = UserRequest::$post['CODE_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['CODE_description'];
        $record['LANGUAGE'] = UserRequest::$post['CODE_language'];
        $record['MODULECODE'] = UserRequest::$post['CODE_modulecode'];
        if (UserRequest::$post['CODE_type'] === 'LINK') {
            $record['FILENAME'] = UserRequest::$post['CODE_filename'];
            $record['CONTENTS'] = 'null';
        } elseif (UserRequest::$post['CODE_type'] === 'ENBD') {
            $record['CONTENTS'] = addslashes(clx_html_spchrs_decode(UserRequest::$post['CODE_contents']));
            $record['FILENAME'] = 'null';
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['CODE_id']);
        $this->versionIncrease('ID', UserRequest::$post['CODE_id']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['CODE_idcode'];
        $record['IDNAME'] = UserRequest::$post['CODE_idname'];
        $record['VERSION'] = UserRequest::$post['CODE_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['CODE_description'];
        $record['LANGUAGE'] = UserRequest::$post['CODE_language'];
        $record['MODULECODE'] = UserRequest::$post['CODE_modulecode'];
        $record['TYPE'] = UserRequest::$post['CODE_type'];
        $record['FILENAME'] = 'null';
        $record['CONTENTS'] = 'null';
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $DBMO = &getTool('DBMO');
        $coderec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO->versionIncrease('IDCODE', $coderec['MODULECODE']);
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function defaultRecord() {
        $record = array();
        $record['CODE_id'] = CLX_INIT_ZERO;
        $record['CODE_idcode'] = '';
        $record['CODE_idname'] = '';
        $record['CODE_version'] = '1.0.0';
        $record['CODE_cdate'] = '';
        $record['CODE_mdate'] = '';
        $record['CODE_username'] = $_SESSION['user_record']['username'];
        $record['CODE_description'] = '';
        $record['CODE_contents'] = '';
        $record['CODE_language'] = 'JS';
        $record['CODE_modulecode'] = '';
        $record['CODE_filename'] = '';
        $record['CODE_type'] = 'ENBD';
        return $record;
    }

    function placeChooseCodeLibrary($msg, $languagefieldname, $prompt, $name, $default, &$table) {
        $contents = setJavaScriptTag(getJavaScriptFilename('CODE_searchwindow.js'), true);
        $searchIcon = getSkinIcon('view', 'javascript:CODE_searchwindow(' . singleQuote($languagefieldname) . ', ' . singleQuote($name) . ')', 'Search for a Code Library');
        if (!empty($msg[$name])) {
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $contents;
    }

    /**
     * Test if code lib exists
     * @param type $idcode
     * @param string $codetype
     * @return type
     */
    function codeLibraryExists(&$idcode, $codetype = '') {
        if ($codetype <> '') {
            $codetype = ' AND TYPE = ' . singleQuote($codetype);
        }
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode) . ' ' . $codetype);
        if ($qty > CLX_NUM_ZERO) {
            $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    /**
     * Returns source code from file
     * @param type $codelib
     * @return string
     */
    function codeGetSource($codelib) {
        $code = '';
        $codelibrec = ClxCommon::$db->getRecord('CLX_CODELIB', 'IDCODE=' . singleQuote($codelib));
        if (empty($codelibrec)) {
            $code = '';
        } elseif ($codelibrec['TYPE'] === 'ENBD') {
            $code = str_replace(array('<?PHP', '<?php', '?>'), '', stripslashes($codelibrec['CONTENTS']));
        } elseif ($codelibrec['TYPE'] === 'LINK') {
            $code = str_replace(array('<?PHP', '<?php', '?>'), '', file_get_contents(kPathToWebSite . $codelibrec['FILENAME']));
        }
        return $code;
    }

    /**
     *
     * @param type $filename
     * @param type $relative
     * @return type
     */
    private function sourceFullPathFileName($filename, $relative = false) {
        return (($relative) ? '' : kPathToWebSite) . kPathToCode . lowercase($filename);
    }

    /**
     * Used on audit
     * @return type
     */
    function compileEmbededPhpSourcesRun(){
        $note = dict('GEN_LBL_COMPILE').': ';
        $directory = kPathToWebSite . kPathToCodeEvents;
        if(is_writable($directory)){
            $this->compileEmbedPhpSources();
            $note .= dict('GEN_LBL_COMPLETED');
        }else{
            $note .= dict('GEN_ERR_FILE_UNABLETOWRITE',kPathToCodeEvents);
        }
        return note($note);
    }

    /**
     *
     */
    private function clearEmbededSources(){
         $directory = kPathToWebSite . kPathToCodeEvents;
        if ($handle = opendir($directory)) {
            while (false !== ($entry = readdir($handle))) {
                if ($entry != '.' && $entry !== '..' && $entry!=='index.php' && strpos($entry, '.php')!==false) {
                    if (is_readable($directory.$entry)) {
                        unlink($directory.$entry);
                    }
                }
            }
            closedir($handle);
        }
    }

    /**
     * Writes all sources to file
     */
    private function compileEmbedPhpSources() {
         $rs = ClxCommon::$db->getRecords('CLX_CODELIB', 'TYPE = '.  singleQuote('ENBD') . ' AND LANGUAGE = ' . singleQuote('PHP'));
         while($codelibrec = ClxCommon::$db->getNextRecord($rs)){
            $this->writeEmbedPhpfile($codelibrec);

         }
    }
    /**
     * Writes php codes to phisical file
     * @param type $codelib
     */
    private function writeEmbedPhpSources($codelibid) {
        $codelibrec = ClxCommon::$db->getRecord('CLX_CODELIB', 'ID=' . singleQuote($codelibid).' AND TYPE = '.  singleQuote('ENBD') . ' AND LANGUAGE = ' . singleQuote('PHP'));
        $this->writeEmbedPhpfile($codelibrec);

    }

    /**
     * Writes php codes to phisical file
     * @param type $codelib
     */
    private function writeEmbedPhpfile($codelibrec) {

        if (!empty($codelibrec) && $codelibrec['TYPE'] === 'ENBD' && !empty($codelibrec['CONTENTS'])) {
            $code = stripslashes($codelibrec['CONTENTS']);
            $filename = $codelibrec['IDCODE'] . '.php';
            $this->writePhpCodeSource($this->sourceFullPathFileName($filename, false), $code);
        }
    }
    /**
     * Writes code to
     * @param string $filename
     * @param string $code
     */
    function writePhpCodeSource($filename, $filecontent) {
        /*$code = '<?php ' . CLX_CHAR_NL . str_replace(array('<?PHP', '<?php', ''), '?>', $code);*/
        $code = '<?php ' . CLX_CHAR_NL . '/**' . CLX_CHAR_NL . '* Auto generated code file from embeded sources. Changes will be lost.' . CLX_CHAR_NL . '* For more information contact  support@alenet.com' . CLX_CHAR_NL . '* @since        $Date: ' . getNow('Y-m-d H:i:s') . ' ' . CLX_CHAR_NL . '*/' . CLX_CHAR_NL . CLX_CHAR_NL . str_replace(array('<?PHP', '<?php', ''), '?>', stripslashes($filecontent));
        $f = new clx_file($filename, 'w');
        $f->open();
        $f->write($code);
        $f->close();

    }



}
