<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @abstract		Database Dynamic Relations
 * @package		Clientix.Tool.Database
 * @author	   $Author: alenetdev $
 * @copyright	Alenet, Inc.
 * @since		$Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version		$Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_dynamic_relations extends clx_tool {

    /**
     * Search Scope Options
     * @var array
     */
    var $search_scope;

    /**
     * Relation Type
     * @var array
     */
    var $relation_type = array('GEN_LBL_DYNAMIC_REL', 'GEN_LBL_SIMPLE_REL', 'GEN_LBL_MN_REL');

    /**
     * Relation Type
     * @var array
     */
    var $on_action = array('GEN_LBL_NO_ACTION', 'GEN_LBL_CASCADE', 'GEN_LBL_SET_null', 'GEN_LBL_RESTRICT');

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'DBDR';
        $this->name = 'Dynamic Relations Manager';
        $this->system = 'Database';
        $this->version = '5.5.1.0';

        $this->has_access = true;


        $this->tablename = 'CLX_DB_DYNRELATIONS';
        $this->record_name_singular = 'dynamic relation';
        $this->record_name_plural = 'dynamic relations';
        $this->primary_field = 'PARENT_TABLECODE';
        $this->secondary_field = 'CHILD_TABLECODE';

        $this->primary_key = array('PARENT_TABLECODE', 'PRIMARY_SEARCHFIELD', 'CHILD_TABLECODE', 'SECONDARY_SEARCHFIELD');
        $this->search_scope = array('255' => 'DBDR_SEARCH_ALL', '1' => 'DBDR_SEARCH_1LEVEL', '2' => 'DBDR_SEARCH_2LEVEL', '3' => 'DBDR_SEARCH_3LEVEL', '4' => 'DBDR_SEARCH_4LEVEL');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_DBDR_TITLE_MODIFY',
            'panels' => array(
                array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_TABLEPRMTS',
                    'entries' => array(
                        array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_PARENT_TABLECODE',
                            'fields' => array(
                                array('name' => 'parent_tablecode', 'type' => 'lookup', 'tool' => 'DBTB', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_TABLES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_PARENT_FIELDNAME',
                            'fields' => array(
                                array('name' => 'parent_fieldname', 'type' => 'lookup', 'tool' => 'DBTF', 'tablecode' => 'DBDR_parent_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'),
                                        array('type' => 'query', 'query' => 'SELECT FIELDNAME DBDR_PARENT_FIELDNAME FROM CLX_DB_TABLEFIELDS WHERE TABLECODE=' . singleQuote('DBDR_parent_tablecode') . ' AND FIELDNAME=' . singleQuote('DBDR_parent_fieldname')))
                                )
                            )
                        ), array('label' => 'GEN_LBL_CHILD_TABLECODE',
                            'fields' => array(
                                array('name' => 'child_tablecode', 'type' => 'lookup', 'tool' => 'DBTB', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_TABLES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_CHILD_FIELDNAME',
                            'fields' => array(
                                array('name' => 'child_fieldname', 'type' => 'lookup', 'tool' => 'DBTF', 'tablecode' => 'DBDR_child_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'query', 'query' => 'SELECT FIELDNAME DBDR_CHILD_FIELDNAME FROM CLX_DB_TABLEFIELDS WHERE TABLECODE=' . singleQuote('DBDR_child_tablecode') . ' AND FIELDNAME=' . singleQuote('DBDR_child_fieldname')))
                                )
                            )
                        ), array('label' => 'GEN_LBL_ACTIVE',
                            'fields' => array(
                                array('name' => 'active', 'type' => CLX_CLASS_LIST, 'options' => $this->yesno)
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_SEARCH_PRMT',
                    'entries' => array(
                        array('label' => 'GEN_LBL_SRCH_LISTING',
                            'fields' => array(
                                array('name' => 'srch_listing', 'type' => 'lookup', 'tool' => 'UILG', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_UI_LISTINGS'))
                                )
                            )
                        ),
                        array('label' => 'GEN_LBL_EXCLUSIVE',
                            'fields' => array(
                                array('name' => 'exclusive', 'type' => CLX_CLASS_LIST, 'options' => $this->yesno)
                            )
                        )
                    )
                ), array('type' => 'normal',
                    CLX_CLASS_TITLE => 'GEN_LBL_RELATION_TBL_PRMT',
                    /* //'caption'=>'GEN_LBL_NEED_NEW_TABLE',
                      Idea para colocar link a crear tabla GEN_LBL_NEED_NEW_TABLE Need a new table? <a href='pop.php?to=DBTB&ac=CREATE&nomenus=1&noalerts=1' target=blank>Click Here</a>
                      ¿Necesita una nueva tabla? <a href='pop.php?to=DBTB&ac=CREATE&nomenus=1&noalerts=1' target=blank>Presione Aquí</a>
                     */
                    'entries' => array(
                        array('label' => 'GEN_LBL_REL_TABLECODE',
                            'fields' => array(
                                array('name' => 'tablecode', 'type' => 'lookup', 'tool' => 'DBTB', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_DB_TABLES'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_RELTBL_PFIELD',
                            'fields' => array(
                                array('name' => 'relt_pfield', 'type' => 'lookup', 'tool' => 'DBTF', 'tablecode' => 'DBDR_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'query', 'query' => 'SELECT FIELDNAME DBDR_RELT_PFIELD FROM CLX_DB_TABLEFIELDS WHERE TABLECODE=' . singleQuote('DBDR_tablecode') . ' AND FIELDNAME=' . singleQuote('DBDR_relt_pfield')))
                                )
                            )
                        ), array('label' => 'GEN_LBL_RELTBL_CFIELD',
                            'fields' => array(
                                array('name' => 'relt_cfield', 'type' => 'lookup', 'tool' => 'DBTF', 'tablecode' => 'DBDR_tablecode', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'query', 'query' => 'SELECT FIELDNAME DBDR_RELT_CFIELD FROM CLX_DB_TABLEFIELDS WHERE TABLECODE=' . singleQuote('DBDR_tablecode') . ' AND FIELDNAME=' . singleQuote('DBDR_relt_cfield')))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    /**
     * Install procedure
     * @see clx_tool::install()
     */
    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_DATABASE:MENU_DYNAMIC_RELATIONS', 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * Describes the tool table structure.
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PARENT_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['CHILD_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['EXCLUSIVES'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PARENT_FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '', 'default' =>  singleQuote('ID'), 'default_raw' => 'ID');
        $fields['CHILD_FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '', 'default' => singleQuote('ID'), 'default_raw' => 'ID');
        //additional information regarding dinamyc relations
        $fields['RELT_PFIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '', 'default' => singleQuote('PARENT_RECORDID'), 'default_raw' => 'PARENT_RECORDID');
        $fields['RELT_CFIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '', 'default' => singleQuote('CHILD_RECORDID'), 'default_raw' => 'CHILD_RECORDID');
        $fields['ACTIVE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => '1', 'properties' => '');
        $fields['RELATION_TYPE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DUPLICATE_ACTION'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['UPDATE_ACTION'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DELETE_ACTION'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SRCH_LISTING'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'PARENT_TABLECODE,CHILD_TABLECODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'PARENT_FIELDNAME,CHILD_FIELDNAME');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::pack()
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'DBDC', 'SELECT * FROM CLX_DB_DYNRELATION_COLS WHERE PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE'])) > CLX_NUM_ZERO);
        if ($exists === true) {
            ClxCommon::$db->deleteRecords('CLX_DB_DYNRELATION_COLS', 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']));
            ClxCommon::$db->deleteRecords($this->tablename, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']));
        }
        ClxCommon::$db->createRecordVoid ($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o));
            }
        }
    }

    /**
     * Action is called for every user interaction with the module...
     * @see clx_tool::action()
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_DYNAMIC_RELATIONS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds listing on tool
     * @see clx_tool::makelist()
     */
    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        $DBDC = &getTool('DBDC');
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');

        $form->setProp(CLX_PROP_ID, 'DBDRList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_DB_DYNRELATIONS';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBDR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_IDNAME'), dict('GEN_LBL_PARENT_TABLECODE'), dict('GEN_LBL_CHILD_TABLECODE'), dict('GEN_LBL_RELATION_TYPE'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_PROPERTIES')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $properties = '';
            if ($record['RELATION_TYPE'] == CLX_NUM_ONE) {
                $properties .= dict('GEN_LBL_ON_DELETE') . ':' . dict($this->on_action[$record['DELETE_ACTION']]) . ', ';
                $properties .= dict('GEN_LBL_ON_UPDATE') . ':' . dict($this->on_action[$record['UPDATE_ACTION']]) . ', ';
                $properties .= dict('GEN_LBL_ON_DUPLICATE') . ':' . dict($this->on_action[$record['DUPLICATE_ACTION']]) . ', ';
                $fls = $DBDC->getRelationFieldsArray($record['PARENT_TABLECODE'], $record['CHILD_TABLECODE']);
                $properties .= dict('GEN_LBL_NO_CRITERIAS') . ':' . count($fls);
            } else {
                $properties = dict('GEN_LBL_SRCH_LISTING') . ':' . $record['SRCH_LISTING'] . ', ';
                if ($record['RELATION_TYPE'] == CLX_NUM_ONE) {
                    $properties .= dict('GEN_LBL_EXCLUSIVE') . ':' . $record['EXCLUSIVES'] . ', ';
                }
                $properties .= dict('GEN_LBL_REL_TABLECODE') . ':' . $record['TABLECODE'];
            }
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['PARENT_TABLECODE'], $record['CHILD_TABLECODE'],
                dict($this->relation_type[$record['RELATION_TYPE']]), dict($this->yesno[$record['ACTIVE']]), $properties));
            $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }

        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_RELATION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    /**
     * Builds Panel listing on tool
     * @see clx_tool::makepanel()
     */
    function makepanel($groupcode = null) {
        $parent_tablecode = $groupcode;
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $DBDC = &getTool('DBDC');
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'DBDRList');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('parent_tablecode', $parent_tablecode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = 'CLX_DB_DYNRELATIONS';
            $whereclause = '(PARENT_TABLECODE=' . singleQuote($parent_tablecode) . ' OR CHILD_TABLECODE=' . singleQuote($parent_tablecode) . ')';
            $sortclause = '';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_7);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

            $table->addCell(dict('TOOL_DBDR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_IDNAME'), dict('GEN_LBL_PARENT_TABLECODE'), dict('GEN_LBL_CHILD_TABLECODE'), dict('GEN_LBL_RELATION_TYPE'), dict('GEN_LBL_ACTIVE'), dict('GEN_LBL_PROPERTIES') //dict('GEN_LBL_SRCH_LISTING'),
                    //dict('GEN_LBL_PRIMARY_SEARCH_FIELD'),
                    // dict('GEN_LBL_SECONDARY_SEARCH_FIELD'),*/
                    //dict('GEN_LBL_TABLECODE')
            ));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $properties = '';
                if (!empty($record['RELATION_TYPE'])) {
                    $properties .= dict('GEN_LBL_ON_DELETE') . ':' . dict($this->on_action[$record['DELETE_ACTION']]) . ', ';
                    $properties .= dict('GEN_LBL_ON_UPDATE') . ':' . dict($this->on_action[$record['UPDATE_ACTION']]) . ', ';
                    $properties .= dict('GEN_LBL_ON_DUPLICATE') . ':' . dict($this->on_action[$record['DUPLICATE_ACTION']]) . ', ';
                    $fls = $DBDC->getRelationFieldsArray($record['PARENT_TABLECODE'], $record['CHILD_TABLECODE']);
                    $properties .= dict('GEN_LBL_NO_CRITERIAS') . ':' . count($fls);
                } else {
                    $properties = dict('GEN_LBL_SRCH_LISTING') . ':' . $record['SRCH_LISTING'] . ', ';
                    if (empty($record['RELATION_TYPE'])) {
                        $properties .= dict('GEN_LBL_EXCLUSIVE') . ':' . $record['EXCLUSIVES'] . ', ';
                    }
                    $properties .= dict('GEN_LBL_REL_TABLECODE') . ':' . $record['TABLECODE'];
                }
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['PARENT_TABLECODE'], $record['CHILD_TABLECODE'], dict($this->relation_type[$record['RELATION_TYPE']]), dict($this->yesno[$record['ACTIVE']]), $properties));
                $i++;
            }
            $buttons = '';
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_RELATION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_DBDR_TITLE_LIST'), $parent_tablecode . '_RELATIONS', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Builds delete form
     * @see clx_tool::confirmDeleteFill()
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_IDNAME') . ':', $record['DBDR_idname']);
        $parentrec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($record['DBDR_parent_tablecode']));
        $parentname = $parentrec['IDNAME'];
        $parentcode = $parentrec['IDCODE'];
        $table->addData(dict('GEN_LBL_PARENT_TABLECODE') . ':', $parentname . ' (' . $parentcode . ')');
        $childrec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($record['DBDR_child_tablecode']));
        $childname = $childrec['IDNAME'];
        $childcode = $childrec['IDCODE'];
        $table->addData(dict('GEN_LBL_CHILD_TABLECODE') . ':', $childname . ' (' . $childcode . ')');
    }

    /**
     * Display Form Contents
     * @see clx_tool::viewFormFill()
     */
    function viewFormFill($record) {
        $contents = '';
        //General Parameters
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBDR_TITLE_DISPLAY'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_IDNAME') . ': ', $record['DBDR_idname']);
        $table->addData(dict('GEN_LBL_ACTIVE') . ': ', dict($this->yesno[$record['DBDR_active']]));
        $table->addData(dict('GEN_LBL_RELATION_TYPE') . ': ', dict($this->relation_type[$record['DBDR_relation_type']]));
        $table->addData(dict('GEN_LBL_PARENT_TABLECODE') . ': ', $record['DBDR_parent_tablecode']);
        $table->addData(dict('GEN_LBL_CHILD_TABLECODE') . ': ', $record['DBDR_child_tablecode']);
        $contents .= $table->source() . '<br>';
        //Relation Parameters
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if ($record['DBDR_relation_type'] == CLX_NUM_ONE) {//Single relation
            $table->addCell(dict('GEN_LBL_SIMPLE_REL'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $fields = '';
            foreach ($record['DBDR_relation_fields'] as $key => $val) {
                $fields .= '<li>' . $val['PARENT_TABLECODE'] . '.' . $val['PARENT_FIELD'] . ' = ' . $val['CHILD_TABLECODE'] . '.' . $val['CHILD_FIELD'] . '</li>';
            }
            $table->addData(dict('GEN_LBL_FIELDS') . ': ', $fields);
            $table->addData(dict('GEN_LBL_ON_DUPLICATE') . ': ', dict($this->on_action[$record['DBDR_duplicate_action']]));
            $table->addData(dict('GEN_LBL_ON_DELETE') . ': ', dict($this->on_action[$record['DBDR_delete_action']]));
            $table->addData(dict('GEN_LBL_ON_UPDATE') . ': ', dict($this->on_action[$record['DBDR_update_action']]));
        } elseif ($record['DBDR_relation_type'] == CLX_NUM_TWO) {//MN relation
            $table->addCell(dict('GEN_LBL_MN_REL'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addData(dict('GEN_LBL_REL_TABLECODE') . ': ', $record['DBDR_tablecode']);
            $fields = '';
            foreach ($record['DBDR_relation_fields_mn'] as $key => $val) {
                $fields .= '<li>' . $val['MN_TABLECODE'] . '.' . $val['PARENT_FIELD'] . ' = ' . $record['DBDR_tablecode'] . '.' . $val['CHILD_FIELD'] . '</li>';
            }
            $table->addData(dict('GEN_LBL_FIELDS') . ': ', $fields);
        } else {//dynamic relation
            //Search Parameters
            $table->addCell(dict('GEN_LBL_SEARCH_PRMT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addData(dict('GEN_LBL_SRCH_LISTING') . ': ', $record['DBDR_srch_listing']);
            $table->addData(dict('GEN_LBL_EXCLUSIVE') . ': ', dict($this->yesno[$record['DBDR_exclusive']]));
            $table->addSeparator(CLX_CLASS_DATA);
            $table->addCell(dict('GEN_LBL_RELATION_TBL_PRMT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addData(dict('GEN_LBL_REL_TABLECODE') . ': ', $record['DBDR_tablecode']);
            $table->addData(dict('GEN_LBL_RELTBL_PFIELD') . ': ', $record['DBDR_relt_pfield']);
            $table->addData(dict('GEN_LBL_RELTBL_CFIELD') . ': ', $record['DBDR_relt_cfield']);
        }
        $contents .= $table->source() . '<br>';
        return $contents;
    }

    /**
     * Tool Edit Form
     * @see clx_tool::editForm()
     */
    function editForm($ac, $idlist, $record, $msg) {
        $DBTB = &getTool('DBTB');
        $DBTF = &getTool('DBTF');
        $UILG = &getTool('UILG');
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_' . $this->code . '_TITLE_MODIFY') : dict('TOOL_' . $this->code . '_TITLE_CREATE');
        $scripts = setJavaScriptTag(getJavaScriptFilename('DBDR_editform.js'), true);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('DBDR_id', $record['DBDR_id']);
        $form->addArgument('DBDR_relation_fields', '');
        $form->addArgument('DBDR_relation_fields_mn', '');
        $form->setProp('onsubmit', 'return processForm()');
        $form->addArgument('selected', '');
        //General Parameters
        $tbprops = array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT);

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps($tbprops);
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_IDNAME'), 'DBDR_idname', $record['DBDR_idname'], CLX_NUM_30, CLX_NUM_20, true);
        $table->addYesNo(dict('GEN_LBL_ACTIVE'), 'DBDR_active', $record['DBDR_active'], CLX_NUM_ZERO, false);
        $table->addList(dict('GEN_LBL_RELATION_TYPE'), 'DBDR_relation_type', $record['DBDR_relation_type'], $this->relation_type, false);
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_PARENT_TABLECODE') . ':', 'DBDR_parent_tablecode', $record['DBDR_parent_tablecode'], $table, true);
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_CHILD_TABLECODE') . ':', 'DBDR_child_tablecode', $record['DBDR_child_tablecode'], $table, true);
        $form->addContents($table->source() . '<br>');
        //Relation Parameters
        $content = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps($tbprops);
        $table->addCell(dict('GEN_LBL_TABLEPRMTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBTF->placeChooseTableField($msg, 'DBDR_parent_tablecode', dict('GEN_LBL_PARENT_FIELDNAME') . ':', 'DBDR_parent_fieldname', $record['DBDR_parent_fieldname'], $table);
        $DBTF->placeChooseTableField($msg, 'DBDR_child_tablecode', dict('GEN_LBL_CHILD_FIELDNAME') . ':', 'DBDR_child_fieldname', $record['DBDR_child_fieldname'], $table);
        $content .= $table->source() . '<br>';
        //Search Parameters
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps($tbprops);
        $table->addCell(dict('GEN_LBL_SEARCH_PRMT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $UILG->placeChooseListing($msg, dict('GEN_LBL_SRCH_LISTING'), 'DBDR_srch_listing', $record['DBDR_srch_listing'], $table);
        $table->addYesNo(dict('GEN_LBL_EXCLUSIVE'), 'DBDR_exclusive', $record['DBDR_exclusive'], CLX_NUM_ZERO, false);
        $content .= $table->source() . '<br>';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps($tbprops);
        $table->addCell(dict('GEN_LBL_RELATION_TBL_PRMT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_REL_TABLECODE') . ':', 'DBDR_tablecode', $record['DBDR_tablecode'], $table);
        $DBTF->placeChooseTableField($msg, 'DBDR_tablecode', dict('GEN_LBL_RELTBL_PFIELD') . ':', 'DBDR_relt_pfield', $record['DBDR_relt_pfield'], $table);
        $DBTF->placeChooseTableField($msg, 'DBDR_tablecode', dict('GEN_LBL_RELTBL_CFIELD') . ':', 'DBDR_relt_cfield', $record['DBDR_relt_cfield'], $table);
        $content .= $table->source() . '<br>';
        //GEN_LBL_MN_REL
        $div = new htmlDiv('DBDR_DYNAMIC', 'panel', $content);
        $form->addContents($div->source());
        //Single Ralation
        //Get Table Fields
        $allfields = array();
        $sql = 'SELECT TABLECODE, FIELDNAME, SEQNUM FROM CLX_DB_TABLEFIELDS WHERE TABLECODE IN (' . singleQuote($record['DBDR_parent_tablecode']) . ',' . singleQuote($record['DBDR_child_tablecode']) . ') ORDER BY TABLECODE ASC, SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $allfields[$r['TABLECODE']][] = $r;
        }
        $tbl = '';
        //load default options
        //Adding note
        if (!empty($msg['DBDR_relation_fields'])) {
            $tbl .='<tr><td colspan=3  class=CLX_CLASS_ALERT>' . $msg['DBDR_relation_fields'] . '</td></tr>';
        }
        $tbl .= '<tr><td class=CLX_CLASS_HEADERS width=100>' . getSkinIcon('modify', 'javascript:dbdr_expand_list()', dict('GEN_LBL_ADD')) . '</td><td class="CLX_CLASS_HEADERS">' . dict('GEN_DBDR_REL_PARENT_CHILD') . ' </td><td class=CLX_CLASS_HEADERS>' . dict('GEN_DBDR_REL_COPY_FIELD') . ' </td></tr>';
        $i = CLX_INIT_ZERO;

        foreach ($record['DBDR_relation_fields'] as $r) {
            $tbl .= '<tr id="relfield' . $i . '" class=CLX_CLASS_DATA><td>' . getSkinIcon('delete', 'javascript:dbdr_contract_list(' . $i . ');', '') . '</td><td><select id="parentfls' . $i . '" name="parentfl" ' . $i . '"  it=' . $i . '><option value=""></option>';
            foreach ($allfields[$record['DBDR_parent_tablecode']] as $key => $val) {
                $tbl .='<option value="' . $val['FIELDNAME'] . '" ' . (($val['FIELDNAME'] == $r['PARENT_FIELD']) ? 'SELECTED' : '') . '>' . $val['FIELDNAME'] . '</option>';
            }
            $tbl .='</select> = <select id="childfls' . $i . '" name="childfl' . $i . '"  it=' . $i . '><option value=""></option>';
            foreach ($allfields[$record['DBDR_child_tablecode']] as $key => $val) {
                $tbl .= '<option value="' . $val['FIELDNAME'] . '" ' . (($val['FIELDNAME'] == $r['CHILD_FIELD']) ? 'SELECTED' : '') . '>' . $val['FIELDNAME'] . '</option>';
            }
            $tbl .= '</select></td><td><input type="checkbox" id="copyfls' . $i . '" name="copyfls' . $i . '" value=1 ' . (($r['COPY_FLAG'] == CLX_NUM_ONE) ? 'checked=true' : '') . '/></td></tr>';
            $i++;
        }
        $tbl = '<table class="' . CLX_CLASS_FORM . '"  width = "' . CLX_PROP_100PCT . '" id = "relfields" trows=' . $i . ' ic = "' . kPathToSkins . $_SESSION['skin'] . '/icons/">' . $tbl . '</table>';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $table->addCell(dict('GEN_LBL_SIMPLE_REL'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('GEN_LBL_FIELDS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_PROMPTREQUIRED));
        $table->addCell($tbl, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $table->addList(dict('GEN_LBL_ON_DELETE'), 'DBDR_delete_action', $record['DBDR_delete_action'], $this->on_action, false);
        $table->addList(dict('GEN_LBL_ON_UPDATE'), 'DBDR_update_action', $record['DBDR_update_action'], $this->on_action, false);
        unset($this->on_action[CLX_NUM_TWO], $this->on_action[CLX_NUM_THREE]);
        $table->addList(dict('GEN_LBL_ON_DUPLICATE'), 'DBDR_duplicate_action', $record['DBDR_duplicate_action'], $this->on_action, false);
        $div = new htmlDiv('DBDR_ONETOONE', 'panel', $table->source());
        $form->addContents($div->source());
        //-Adding M-N Relations
        $allfields = array();
        $sql = 'SELECT TABLECODE, FIELDNAME, SEQNUM FROM CLX_DB_TABLEFIELDS WHERE TABLECODE IN (' . singleQuote($record['DBDR_parent_tablecode']) . ',' . singleQuote($record['DBDR_child_tablecode']) . ',' . singleQuote($record['DBDR_tablecode']) . ') ORDER BY TABLECODE ASC, SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $allfields[$r['TABLECODE']][] = $r;
        }
        $tbl = '';
        //load default options
        //Adding note
        if (!empty($msg['DBDR_relation_fields_mn'])) {
            $tbl .='<tr><td colspan=3  class=CLX_CLASS_ALERT>' . $msg['DBDR_relation_fields_mn'] . '</td></tr>';
        }
        $tbl .= '<tr><td class=CLX_CLASS_HEADERS width=100>' . getSkinIcon('modify', 'javascript:dbdr_expand_list_mn()', dict('GEN_LBL_ADD')) . '</td><td class=CLX_CLASS_HEADERS>' . dict('GEN_DBDR_REL_PARENT_CHILD') . ' </td><td class=CLX_CLASS_HEADERS>' . dict('GEN_DBDR_REL_COPY_FIELD') . ' </td></tr>';
        $i = CLX_INIT_ZERO;
        foreach ($record['DBDR_relation_fields_mn'] as $r) {
            $tbl .= '<tr id="relfieldmn' . $i . '" class="' . CLX_CLASS_DATA . '"><td>' . getSkinIcon('delete', 'javascript:dbdr_contract_list_mn(' . $i . ');', '') . '</td><td> 			<select id="parentflsmn' . $i . '" name="parentflmn' . $i . '"  it=' . $i . '>';
            $tbl .='<option value=""></option>';
            foreach ($allfields[$record['DBDR_parent_tablecode']] as $key => $val) {
                $tbl .='<option value="' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '" ' . (($val['TABLECODE'] . '.' . $val['FIELDNAME'] == $r['MN_TABLECODE'] . '.' . $r['PARENT_FIELD']) ? 'SELECTed' : '') . '>' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '</option>';
            }
            //second table
            foreach ($allfields[$record['DBDR_child_tablecode']] as $key => $val) {
                $tbl .='<option value="' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '" ' . (($val['TABLECODE'] . '.' . $val['FIELDNAME'] == $r['MN_TABLECODE'] . '.' . $r['PARENT_FIELD']) ? 'SELECTed' : '') . '>' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '</option>';
            }
            $tbl .='</select> = <select id="childflsmn' . $i . '" name="childfl' . $i . '"  it=' . $i . '><option value=""></option>';
            foreach ($allfields[$record['DBDR_tablecode']] as $key => $val) {
                $tbl .= '<option value="' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '" ' . (($val['FIELDNAME'] == $r['CHILD_FIELD']) ? 'SELECTed' : '') . '>' . $val['TABLECODE'] . '.' . $val['FIELDNAME'] . '</option>';
            }
            $tbl .= '</select></td><td><input type="checkbox" id="copyflsmn' . $i . '" name="copyflsmn' . $i . '" value=1 ' . (($r['COPY_FLAG'] == CLX_NUM_ONE) ? 'checked=true' : '') . '/></td></tr>';
            $i++;
        }
        $tbl = '<table class=CLX_CLASS_FORM  width = CLX_PROP_100PCT id = "relfieldsmn" trows=' . $i . ' ic = "' . kPathToSkins . $_SESSION['skin'] . '/icons/">' . $tbl . '</table>';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $tbprops = array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT);
        $table->setProps($tbprops);
        $table->addCell(dict('GEN_LBL_MN_REL'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_REL_TABLECODE') . ':', 'DBDR_tablecode_mn', $record['DBDR_tablecode_mn'], $table);
        $table->addCell(dict('GEN_LBL_FIELDS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_PROMPTREQUIRED));
        $table->addCell($tbl, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $UILG->placeChooseListing($msg, dict('GEN_LBL_SRCH_LISTING'), 'DBDR_srch_listing_mn', $record['DBDR_srch_listing_mn'], $table);
        $div = new htmlDiv('DBDR_MTON', 'panel', $table->source());
        $form->addContents($div->source());
        $form->addContents('<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps($tbprops);
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        if (UserRequest::$request['ac'] === 'MODIFY') {
            $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_MODIFY';
        } else {
            $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_CREATE';
        }
        return $form->source() . $scripts;
    }

    /**
     * Adds Filter Form contents
     * @see clx_tool::filterFormFill()
     */
    function filterFormFill($msg, &$table) {
        if (!array_key_exists('DBDR_idname', UserRequest::$post))
            UserRequest::$post['DBDR_idname'] = '';
        if (!array_key_exists('DBDR_parent_tablecode', UserRequest::$post))
            UserRequest::$post['DBDR_parent_tablecode'] = '';
        if (!array_key_exists('DBDR_child_tablecode', UserRequest::$post))
            UserRequest::$post['DBDR_child_tablecode'] = '';
        $table->addField($msg, dict('GEN_LBL_IDNAME'), 'DBDR_idname', UserRequest::$post['DBDR_idname'], CLX_NUM_30, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_PARENT_TABLECODE'), 'DBDR_parent_tablecode', UserRequest::$post['DBDR_parent_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_CHILD_TABLECODE'), 'DBDR_child_tablecode', UserRequest::$post['DBDR_child_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('DBDR_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_PARENT_TABLECODE'), 'parent_tablecode', false);
        $sort->addItem(dict('GEN_LBL_CHILD_TABLECODE'), 'child_tablecode', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    /**
     * Register filter information on user session
     * @see clx_tool::setfilter()
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['DBDR_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['DBDR_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBDR_parent_tablecode'])) {
            $filter .= $boolean . ' PARENT_TABLECODE LIKE ' . singleQuote(UserRequest::$post['DBDR_parent_tablecode'] . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBDR_child_tablecode'])) {
            $filter .= $boolean . ' CHILD_TABLECODE LIKE ' . singleQuote(UserRequest::$post['DBDR_child_tablecode'] . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['DBDRList']['sortby'] = UserRequest::$post['DBDR_sort'];
        $_SESSION['paging']['DBDRList']['criteria'] = $filter;
        $_SESSION['paging']['DBDRList']['startat'] = '0';
    }

    /**
     * Clear any applied filter on session for user
     * @see clx_tool::clearfilter()
     */
	 ///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
   /*function clearfilter() {
        $_SESSION['paging']['DBDRList']['criteria'] = '';
    }*/

    /**
     * Verify Tool Edit Forms
     * @see clx_tool::checkForm()
     */
    function checkForm(&$msg) {
        $msg = array();
        //DBDR_srch_listing$this->editform
        //turn off simple relation search listing validation
        if (UserRequest::$post['DBDR_relation_type'] == CLX_NUM_ONE) {
            $this->editform['panels'][CLX_NUM_ONE]['entries'][CLX_NUM_ZERO]['fields'][CLX_NUM_ZERO]['validations'] = array();
        }
        if (UserRequest::$post['DBDR_relation_type'] == CLX_NUM_TWO) {
            UserRequest::$post['DBDR_srch_listing'] = UserRequest::$post['DBDR_srch_listing_mn'];
            $this->editform['panels'][CLX_NUM_TWO]['entries'][CLX_NUM_ONE]['fields'][CLX_NUM_ZERO]['validations'] = array();
            $this->editform['panels'][CLX_NUM_TWO]['entries'][CLX_NUM_TWO]['fields'][CLX_NUM_ZERO]['validations'] = array();
        }
        parent::checkForm($msg);

        if (UserRequest::$request['ac'] === 'CREATE' && ClxCommon::$db->oneExistsRecords($this->tablename, 'PARENT_TABLECODE = ' . singleQuote(UserRequest::$post['DBDR_parent_tablecode']) . ' AND CHILD_TABLECODE = ' . singleQuote(UserRequest::$post['DBDR_child_tablecode'])) > 0) {
            $msg['DBDR_parent_tablecode'] = dict('GEN_LBL_ALREADYEXISTS');
            $msg['DBDR_child_tablecode'] = $msg['DBDR_parent_tablecode'];
        }

        if (UserRequest::$post['DBDR_relation_type'] == CLX_NUM_ZERO) {//dynamic
            if (!$this->formFieldTypeEqual(UserRequest::$post['DBDR_parent_fieldname'], UserRequest::$post['DBDR_relt_pfield'])) {
                //wrong not same type
                $msg['DBDR_parent_fieldname'] = dict('GEN_LBL_PFIELD_MUSTBESAMETYPE');
                $msg['DBDR_relt_pfield'] = $msg['DBDR_parent_fieldname'];
            }
            if (!$this->formFieldTypeEqual(UserRequest::$post['DBDR_child_fieldname'], UserRequest::$post['DBDR_relt_cfield'])) {
                //wrong not same type
                $msg['DBDR_child_fieldname'] = dict('GEN_LBL_CFIELD_MUSTBESAMETYPE');
                $msg['DBDR_relt_cfield'] = $msg['DBDR_child_fieldname'];
            }
            $store = array();
            $store[UserRequest::$post['DBDR_relt_pfield']] = CLX_NUM_ONE;
            if (!empty($store[UserRequest::$post['DBDR_relt_cfield']])) {
                //Field could no be the same
                $msg['DBDR_relt_cfield'] = dict('GEN_LBL_FIELDSHOUDBEDIFFERENT');
            }
            $store[UserRequest::$post['DBDR_relt_cfield']] = CLX_NUM_ONE;
            $DBTF = &getTool('DBTF');
            $fieldname = 'PARENT_TABLECODE';
            if (!$DBTF->fieldExists(UserRequest::$post['DBDR_tablecode'], $fieldname)) {
                $msg['DBDR_tablecode'] = dict('GEN_LBL_PLEASEINCLUDEFIELD', $fieldname, UserRequest::$post['DBDR_tablecode']);
            }
            $fieldname = 'CHILD_TABLECODE';
            if (!$DBTF->fieldExists(UserRequest::$post['DBDR_tablecode'], $fieldname)) {
                $msg['DBDR_tablecode'] = dict('GEN_LBL_PLEASEINCLUDEFIELD', $fieldname, UserRequest::$post['DBDR_tablecode']);
            }
        }
        $fields = explode(',', UserRequest::$post['DBDR_relation_fields']);
        UserRequest::$post['DBDR_relation_fields'] = array();
        $done = array();
        $fieldsin = array();
        foreach ($fields as $k => $v) {
            if (empty($v[CLX_NUM_ONE]))
                continue;
            $v = explode(':', $v);
            UserRequest::$post['DBDR_relation_fields'][] = array('PARENT_FIELD' => $v[CLX_NUM_ZERO], 'CHILD_FIELD' => $v[CLX_NUM_ONE], 'COPY_FLAG' => $v[CLX_NUM_TWO]);
            $fieldsin[] = $v[CLX_NUM_ONE];
            if (!empty($done[$v[CLX_NUM_ZERO] . $v[CLX_NUM_ONE]])) {
                $msg['DBDR_relation_fields'] = dict('DBDR_ERR_DUPLICATE_REL', $v[CLX_NUM_ZERO] . '=' . $v[CLX_NUM_ONE]);
            }
            $done[$v[CLX_NUM_ZERO] . $v[CLX_NUM_ONE]] = CLX_NUM_ONE;
        }
        $fields = explode(',', UserRequest::$post['DBDR_relation_fields_mn']);
        UserRequest::$post['DBDR_relation_fields_mn'] = array();
        $done = array();
        $fieldsin = array();
        foreach ($fields as $k => $v) {
            if (empty($v[CLX_NUM_ONE]))
                continue;
            $v = explode(':', $v);
            $t1 = explode('.', $v[CLX_NUM_ZERO]);
            $t2 = explode('.', $v[CLX_NUM_ONE]);
            UserRequest::$post['DBDR_relation_fields_mn'][] = array('MN_TABLECODE' => $t1[CLX_NUM_ZERO], 'PARENT_FIELD' => $t1[CLX_NUM_ONE], 'CHILD_FIELD' => $t2[CLX_NUM_ONE], 'COPY_FLAG' => $v[CLX_NUM_TWO]);
            $fieldsin[] = $v[CLX_NUM_ONE];
            if (!empty($done[$v[CLX_NUM_ZERO] . $v[CLX_NUM_ONE]])) {
                $msg['DBDR_relation_fields_mn'] = dict('DBDR_ERR_DUPLICATE_REL', $v[CLX_NUM_ZERO] . '=' . $v[CLX_NUM_ONE]);
            }
            $done[$v[CLX_NUM_ZERO] . $v[CLX_NUM_ONE]] = CLX_NUM_ONE;
        }
        //Simple Relation Validations
        if (UserRequest::$post['DBDR_relation_type'] == CLX_NUM_ONE) {
            //Check if relation type null then child must be nulable
            if (count($fieldsin) > CLX_NUM_ZERO && (UserRequest::$post['DBDR_delete_action'] == CLX_NUM_TWO || UserRequest::$post['DBDR_update_action'] == CLX_NUM_TWO)) {
                $sql = 'SELECT COUNT(ID) FROM CLX_DB_TABLEFIELDS WHERE TABLECODE = ' . singleQuote(UserRequest::$post['DBDR_child_tablecode']) . ' AND FIELDNAME IN (' . singleQuote(implode('' . singleQuote(CLX_CHAR_EMPTY_STRING) . '', $fieldsin)) . ') AND FIELD_REQUIRED = 1';
                if (ClxCommon::$db->getOne($sql) > CLX_NUM_ZERO) {
                    $msg['DBDR_relation_fields'] = dict('DBDR_ERR_null_REQUIRED');
                }
            }
        }//end relation type
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Compares field to see if same type
     * @param string $fielda
     * @param string $fieldb
     * @return boolean
     */
    function formFieldTypeEqual($fielda, $fieldb) {
        $fieldatype = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDTYPE', 'FIELDNAME = ' . singleQuote($fielda));
        if ($fieldatype === 'PRIM')
            $fieldatype = 'INTN';
        $fieldbtype = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDTYPE', 'FIELDNAME = ' . singleQuote($fieldb));
        if ($fieldbtype === 'PRIM')
            $fieldatype = 'INTN';
        if ($fieldatype === 'LOOK') {
            $sql = 'SELECT X.FIELDTYPE FROM CLX_DB_FIELDS X, CLX_DB_FIELDS Y WHERE X.FIELDNAME = Y.FIELD_EXTRA2 AND Y.FIELDNAME = ' . singleQuote($fielda);
            $fieldatype = ClxCommon::$db->getOne($sql);
        }
        if ($fieldbtype === 'LOOK') {
            $sql = 'SELECT X.FIELDTYPE FROM CLX_DB_FIELDS X, CLX_DB_FIELDS Y WHERE X.FIELDNAME = Y.FIELD_EXTRA2 AND Y.FIELDNAME = ' . singleQuote($fieldb);
            $fieldbtype = ClxCommon::$db->getOne($sql);
        }
        return ($fieldatype <> $fieldbtype) ? false : true;
    }

    /**
     * Check if field is type list.
     * @param string $fieldname
     * @return boolean
     */
    function formCheckIsFieldList($fieldname) {
        $qty = ClxCommon::$db->oneExistsRecordsS('CLX_DB_FIELDS', 'FIELDNAME = ' . singleQuote($fieldname) . ' AND FIELDTYPE = ' . singleQuote('LIS') . '');
        return ($qty == CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Loads current record on selected id
     * @see clx_tool::loadRecord()
     */
    function loadToolRecord($recordid) {

        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBDR_id'] = $recordid;
        $record['DBDR_idname'] = $values['IDNAME'];
        $record['DBDR_parent_tablecode'] = $values['PARENT_TABLECODE'];
        $record['DBDR_child_tablecode'] = $values['CHILD_TABLECODE'];
        $record['DBDR_srch_listing'] = $values['SRCH_LISTING'];
        $record['DBDR_srch_listing_mn'] = $values['SRCH_LISTING'];
        $record['DBDR_tablecode'] = $values['TABLECODE'];
        $record['DBDR_exclusive'] = $values['EXCLUSIVES'];
        $record['DBDR_parent_fieldname'] = $values['PARENT_FIELDNAME'];
        $record['DBDR_child_fieldname'] = $values['CHILD_FIELDNAME'];
        $record['DBDR_relt_pfield'] = $values['RELT_PFIELD'];
        $record['DBDR_relt_cfield'] = $values['RELT_CFIELD'];
        $record['DBDR_active'] = $values['ACTIVE'];
        $record['DBDR_relation_type'] = $values['RELATION_TYPE'];
        $record['DBDR_update_action'] = $values['UPDATE_ACTION'];
        $record['DBDR_delete_action'] = $values['DELETE_ACTION'];
        $record['DBDR_duplicate_action'] = $values['DUPLICATE_ACTION'];
        $record['DBDR_relation_fields'] = array();
        $record['DBDR_relation_fields_mn'] = array();
        $record['DBDR_tablecode_mn'] = '';
        $DBDC = &getTool('DBDC');
        switch ($values['RELATION_TYPE']) {
            case CLX_NUM_ONE:
                $record['DBDR_relation_fields'] = $DBDC->getRelationFieldsArray($record['DBDR_parent_tablecode'], $record['DBDR_child_tablecode']);
                break;
            case CLX_NUM_TWO:
                $record['DBDR_relation_fields_mn'] = $DBDC->getRelationFieldsArray($record['DBDR_parent_tablecode'], $record['DBDR_child_tablecode']);
                $record['DBDR_tablecode_mn'] = $values['TABLECODE'];
                break;
            default:
                $record['DBDR_relation_fields'] = array();
                $record['DBDR_relation_fields_mn'] = array();
                break;
        }
        return $record;
    }

    /**
     * Update records
     * @see clx_tool::saveRecord()
     */
    function saveRecord() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['DBDR_idname'];
        $record['PARENT_TABLECODE'] = UserRequest::$post['DBDR_parent_tablecode'];
        $record['CHILD_TABLECODE'] = UserRequest::$post['DBDR_child_tablecode'];
        $record['SRCH_LISTING'] = UserRequest::$post['DBDR_srch_listing'];
        $record['EXCLUSIVES'] = UserRequest::$post['DBDR_exclusive'];
        $record['PARENT_FIELDNAME'] = (empty(UserRequest::$post['DBDR_parent_fieldname'])) ? 'ID' : UserRequest::$post['DBDR_parent_fieldname'];
        $record['CHILD_FIELDNAME'] = (empty(UserRequest::$post['DBDR_child_fieldname'])) ? 'ID' : UserRequest::$post['DBDR_child_fieldname'];
        $record['RELT_PFIELD'] = UserRequest::$post['DBDR_relt_pfield'];
        $record['RELT_CFIELD'] = UserRequest::$post['DBDR_relt_cfield'];
        $record['ACTIVE'] = UserRequest::$post['DBDR_active'];
        $record['RELATION_TYPE'] = UserRequest::$post['DBDR_relation_type'];
        $record['UPDATE_ACTION'] = UserRequest::$post['DBDR_update_action'];
        $record['DELETE_ACTION'] = UserRequest::$post['DBDR_delete_action'];
        $record['DUPLICATE_ACTION'] = UserRequest::$post['DBDR_duplicate_action'];
        if (strlen(UserRequest::$post['DBDR_tablecode']) == CLX_NUM_ZERO) {
            $record['TABLECODE'] = 'DYNR';
        } else {
            $record['TABLECODE'] = UserRequest::$post['DBDR_tablecode'];
        }
        //mn relation type
        if ($record['RELATION_TYPE'] == CLX_NUM_TWO) {
            $record['TABLECODE'] = UserRequest::$post['DBDR_tablecode_mn'];
            $record['SRCH_LISTING'] = UserRequest::$post['DBDR_srch_listing_mn'];
        }
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['DBDR_id']);
        //Setting Columns
        $sql = 'UPDATE CLX_DB_DYNRELATION_COLS SET DELETE_FLAG = 1 WHERE PARENT_TABLECODE =' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE =' . singleQuote($record['CHILD_TABLECODE']);
        ClxCommon::$db->executeQuery($sql);
        if ($record['RELATION_TYPE'] == CLX_NUM_ONE) {
            foreach (UserRequest::$post['DBDR_relation_fields'] as $k => $v) {
                ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATION_COLS', array('PARENT_TABLECODE' => $record['PARENT_TABLECODE'], 'CHILD_TABLECODE' => $record['CHILD_TABLECODE'], 'SEQNUM' => $k, 'PARENT_FIELD' => $v['PARENT_FIELD'], 'CHILD_FIELD' => $v['CHILD_FIELD'], 'COPY_FLAG' => $v['COPY_FLAG'], 'DELETE_FLAG' => CLX_NUM_ZERO), false, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND PARENT_FIELD =' . singleQuote($v['PARENT_FIELD']) . ' AND CHILD_FIELD =' . singleQuote($v['CHILD_FIELD']));
            }
        } elseif ($record['RELATION_TYPE'] == CLX_NUM_TWO) {
            foreach (UserRequest::$post['DBDR_relation_fields_mn'] as $k => $v) {
                ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATION_COLS', array('PARENT_TABLECODE' => $record['PARENT_TABLECODE'], 'CHILD_TABLECODE' => $record['CHILD_TABLECODE'], 'SEQNUM' => $k, 'MN_TABLECODE' => $v['MN_TABLECODE'], 'PARENT_FIELD' => $v['PARENT_FIELD'], 'CHILD_FIELD' => $v['CHILD_FIELD'], 'COPY_FLAG' => $v['COPY_FLAG'], 'DELETE_FLAG' => CLX_NUM_ZERO), false, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND MN_TABLECODE =' . singleQuote($v['MN_TABLECODE']) . ' AND PARENT_FIELD =' . singleQuote($v['PARENT_FIELD']) . ' AND CHILD_FIELD =' . singleQuote($v['CHILD_FIELD']));
            }
        }
        $sql = 'DELETE FROM CLX_DB_DYNRELATION_COLS WHERE PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND DELETE_FLAG = 1';
        ClxCommon::$db->executeQuery($sql);
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['PARENT_TABLECODE']);
        $DBTB->versionIncrease('IDCODE', $record['CHILD_TABLECODE']);
    }

    /**
     * Creates record
     * @see clx_tool::createRecord()
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['DBDR_idname'];
        $record['PARENT_TABLECODE'] = UserRequest::$post['DBDR_parent_tablecode'];
        $record['CHILD_TABLECODE'] = UserRequest::$post['DBDR_child_tablecode'];
        $record['SRCH_LISTING'] = UserRequest::$post['DBDR_srch_listing'];
        $record['PARENT_FIELDNAME'] = (empty(UserRequest::$post['DBDR_parent_fieldname'])) ? 'ID' : UserRequest::$post['DBDR_parent_fieldname'];
        $record['CHILD_FIELDNAME'] = (empty(UserRequest::$post['DBDR_child_fieldname'])) ? 'ID' : UserRequest::$post['DBDR_child_fieldname'];
        $record['RELT_PFIELD'] = UserRequest::$post['DBDR_relt_pfield'];
        $record['RELT_CFIELD'] = UserRequest::$post['DBDR_relt_cfield'];
        $record['ACTIVE'] = UserRequest::$post['DBDR_active'];
        $record['RELATION_TYPE'] = UserRequest::$post['DBDR_relation_type'];
        $record['UPDATE_ACTION'] = UserRequest::$post['DBDR_update_action'];
        $record['DELETE_ACTION'] = UserRequest::$post['DBDR_delete_action'];
        $record['DUPLICATE_ACTION'] = UserRequest::$post['DBDR_duplicate_action'];
        if (strlen(UserRequest::$post['DBDR_tablecode']) == CLX_NUM_ZERO) {
            $record['TABLECODE'] = 'DYNR';
        } else {
            $record['TABLECODE'] = UserRequest::$post['DBDR_tablecode'];
        }
        //mn relation type
        if ($record['RELATION_TYPE'] == CLX_NUM_TWO) {
            $record['TABLECODE'] = UserRequest::$post['DBDR_tablecode_mn'];
            $record['SRCH_LISTING'] = UserRequest::$post['DBDR_srch_listing_mn'];
        }
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        //Setting Columns
        $sql = 'UPDATE CLX_DB_DYNRELATION_COLS SET DELETE_FLAG = 1 WHERE PARENT_TABLECODE =' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE =' . singleQuote($record['CHILD_TABLECODE']);
        ClxCommon::$db->executeQuery($sql);
        if ($record['RELATION_TYPE'] == CLX_NUM_ONE) {
            foreach (UserRequest::$post['DBDR_relation_fields'] as $k => $v) {
                ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATION_COLS', array('PARENT_TABLECODE' => $record['PARENT_TABLECODE'], 'CHILD_TABLECODE' => $record['CHILD_TABLECODE'], 'SEQNUM' => $k, 'PARENT_FIELD' => $v['PARENT_FIELD'], 'CHILD_FIELD' => $v['CHILD_FIELD'], 'COPY_FLAG' => $v['COPY_FLAG'], 'DELETE_FLAG' => CLX_NUM_ZERO), false, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND PARENT_FIELD =' . singleQuote($v['PARENT_FIELD']) . ' AND CHILD_FIELD =' . singleQuote($v['CHILD_FIELD']));
            }
        } elseif ($record['RELATION_TYPE'] == CLX_NUM_TWO) {
            foreach (UserRequest::$post['DBDR_relation_fields_mn'] as $k => $v) {
                ClxCommon::$db->loadDbRecord('CLX_DB_DYNRELATION_COLS', array('PARENT_TABLECODE' => $record['PARENT_TABLECODE'], 'CHILD_TABLECODE' => $record['CHILD_TABLECODE'], 'SEQNUM' => $k, 'MN_TABLECODE' => $v['MN_TABLECODE'], 'PARENT_FIELD' => $v['PARENT_FIELD'], 'CHILD_FIELD' => $v['CHILD_FIELD'], 'COPY_FLAG' => $v['COPY_FLAG'], 'DELETE_FLAG' => CLX_NUM_ZERO), false, 'PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND MN_TABLECODE =' . singleQuote($v['MN_TABLECODE']) . ' AND PARENT_FIELD =' . singleQuote($v['PARENT_FIELD']) . ' AND CHILD_FIELD =' . singleQuote($v['CHILD_FIELD']));
            }
        }
        $sql = 'DELETE FROM CLX_DB_DYNRELATION_COLS WHERE PARENT_TABLECODE = ' . singleQuote($record['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($record['CHILD_TABLECODE']) . ' AND DELETE_FLAG = 1';
        ClxCommon::$db->executeQuery($sql);
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['PARENT_TABLECODE']);
        $DBTB->versionIncrease('IDCODE', $record['CHILD_TABLECODE']);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     * @see clx_tool::deleteRecord()
     */
    function deleteRecordSk($id) {
        $relationrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        //Deleting Childs
        ClxCommon::$db->deleteRecords('CLX_DB_DYNRELATION_COLS', 'PARENT_TABLECODE =' . singleQuote($relationrec['PARENT_TABLECODE']) . ' AND CHILD_TABLECODE = ' . singleQuote($relationrec['CHILD_TABLECODE']));
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $relationrec['PARENT_TABLECODE']);
        $DBTB->versionIncrease('IDCODE', $relationrec['CHILD_TABLECODE']);
    }

    /**
     * Defines default record on initializacion
     * @see clx_tool::defaultRecord()
     */
    function defaultRecord() {
        $record = array();
        $record['DBDR_id'] = '0';
        $record['DBDR_idname'] = '';
        $record['DBDR_parent_tablecode'] = '';
        $record['DBDR_child_tablecode'] = '';
        $record['DBDR_srch_listing'] = '';
        $record['DBDR_srch_listing_mn'] = '';
        $record['DBDR_tablecode'] = 'DYRL';
        $record['DBDR_exclusive'] = '1';
        $record['DBDR_parent_fieldname'] = 'ID';
        $record['DBDR_child_fieldname'] = 'ID';
        $record['DBDR_rel_list_code'] = '';
        $record['DBDR_relt_pfield'] = 'PARENT_RECORDID';
        $record['DBDR_relt_cfield'] = 'CHILD_RECORDID';
        $record['DBDR_active'] = CLX_NUM_ONE;
        $record['DBDR_relation_type'] = CLX_INIT_ZERO;
        $record['DBDR_update_action'] = CLX_INIT_ZERO;
        $record['DBDR_delete_action'] = CLX_INIT_ZERO;
        $record['DBDR_duplicate_action'] = CLX_INIT_ZERO;
        $record['DBDR_relation_fields'] = array();
        $record['DBDR_relation_fields_mn'] = array();
        $record['DBDR_tablecode_mn'] = '';
        return $record;
    }

    /**
     * Returns Parent Table Relations
     * @param string $parent_table
     * @param string $child_table
     * @return multitype:unknown
     */
    function getChildRelation($parent_table, $child_tablecode = '') {
        $relation = array();
        $sql = 'SELECT DBDR.RELATION_TYPE, DBDR.UPDATE_ACTION, DBDR.DELETE_ACTION, DBDR.DUPLICATE_ACTION, DBDR.PARENT_TABLECODE, DBDR.CHILD_TABLECODE, DBDC.PARENT_FIELD, DBDC.CHILD_FIELD FROM CLX_DB_DYNRELATIONS DBDR, CLX_DB_DYNRELATION_COLS DBDC WHERE DBDR.PARENT_TABLECODE = DBDC.PARENT_TABLECODE AND DBDR.CHILD_TABLECODE = DBDC.CHILD_TABLECODE AND DBDR.ACTIVE = 1 AND DBDR.PARENT_TABLECODE = ' . singleQuote($parent_table) . ' ' . (($child_tablecode <> '') ? ' AND DBDR.CHILD_TABLECODE = ' . singleQuote($child_tablecode) : '') . ' ORDER BY DBDC.SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if (!array_key_exists($r['CHILD_TABLECODE'], $relation)) {
                $relation[$r['CHILD_TABLECODE']] = array();
            }
            $relation[$r['CHILD_TABLECODE']]['RELATION_TYPE'] = $r['RELATION_TYPE'];
            $relation[$r['CHILD_TABLECODE']]['UPDATE_ACTION'] = $r['UPDATE_ACTION'];
            $relation[$r['CHILD_TABLECODE']]['DELETE_ACTION'] = $r['DELETE_ACTION'];
            $relation[$r['CHILD_TABLECODE']]['DUPLICATE_ACTION'] = $r['DUPLICATE_ACTION'];
            $relation[$r['CHILD_TABLECODE']]['PARENT_TABLECODE'] = $r['PARENT_TABLECODE'];
            $relation[$r['CHILD_TABLECODE']]['CHILD_TABLECODE'] = $r['CHILD_TABLECODE'];
            $relation[$r['CHILD_TABLECODE']]['CRITERIAS'][] = array('PARENT_FIELD' => $r['PARENT_FIELD'], 'CHILD_FIELD' => $r['CHILD_FIELD']);
        }
        return $relation;
    }

    /**
     * Test if relation exists between two tables
     * @param string $parent_tablecode
     * @param string $child_tablecode
     * @return boolean
     */
    function relationExists(&$parent_tablecode, &$child_tablecode) {
        $DBTB = &getTool('DBTB');
        if ($DBTB->tableExists($parent_tablecode) and $DBTB->tableExists($child_tablecode)) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'PARENT_TABLECODE=' . singleQuote($parent_tablecode) . ' AND CHILD_TABLECODE=' . singleQuote($child_tablecode) . ' AND ACTIVE = 1 AND RELATION_TYPE = 0');
		} else {
            $qty = CLX_INIT_ZERO;
        }
        return $qty > CLX_NUM_ZERO;
    }

    /**
     * Prepares Criteria for Dynamic Relation to be set on listing.
     * @param string $parent_tablecode
     * @param string $child_tablecode
     * @param int $parent_id
     * @param array $queryrec
     * @param bool $isrelated
     * @return string
     */
    function buildListingCriteria($parent_tablecode, $child_tablecode, $parent_id, &$queryrec, &$isrelated) {
        $relrec = $this->getRelationInfo($parent_tablecode, $child_tablecode);
        if (empty($relrec))
            return CLX_NUM_ZERO; //no relation
        if (stristr($queryrec['FROMCLAUSE'], $relrec['REL_TABLENAME'])) {
            $isrelated = true;
            //Asume que se hara la relacion en la consulta
            return CLX_CHAR_EMPTY_STRING;
        }
        //set if exclusive so listing will not display link button.
        $isrelated = (!$relrec['EXCLUSIVES']) ? true : false;
        //despues de cambio  si llega aca es porque no tiene caso
        $queryrec['FROMCLAUSE'] = $relrec['REL_TABLENAME'] . ' ' . $relrec['REL_TABLECODE'] . ', ' . $queryrec['FROMCLAUSE'];
        if (!stristr($queryrec['FROMCLAUSE'], $relrec['PARENT_TABLENAME'])) {
            $queryrec['FROMCLAUSE'] = $relrec['PARENT_TABLENAME'] . ' ' . $relrec['PARENT_TABLECODE'] . ', ' . $queryrec['FROMCLAUSE'];
        }
        if (!stristr($queryrec['FROMCLAUSE'], $relrec['CHILD_TABLENAME'])) {
            $queryrec['FROMCLAUSE'] = $relrec['CHILD_TABLENAME'] . ' ' . $relrec['CHILD_TABLECODE'] . ', ' . $queryrec['FROMCLAUSE'];
        }
        if (empty($parent_id))
            $parent_id = CLX_INIT_ZERO;
		
		if ($relrec['CHILD_TABLENAME']==$relrec['PARENT_TABLENAME']) {
			$criteria = '('.$relrec['REL_TABLECODE'] . '.'. $relrec['RELT_PFIELD'] .'=' . $parent_id ./*' AND ' . $relrec['REL_TABLECODE'] . '.' . $relrec['RELT_PFIELD'] . '=' . $relrec['PARENT_TABLECODE'] . '.' . $relrec['PARENT_FIELDNAME'] . */' AND ' . $relrec['REL_TABLECODE'] . '.' . $relrec['RELT_CFIELD'] . '=' . $relrec['CHILD_TABLECODE'] . '.' . $relrec['CHILD_FIELDNAME'] . ' AND ' . $relrec['REL_TABLECODE'] . '.PARENT_TABLECODE=' . singleQuote($relrec['PARENT_TABLECODE']) . ' AND ' . $relrec['REL_TABLECODE'] . '.CHILD_TABLECODE=' . singleQuote($relrec['CHILD_TABLECODE']) . ')';
		}
		else {
			$criteria = '(' . $relrec['PARENT_TABLECODE'] . '.ID = ' . $parent_id . ' AND ' . $relrec['REL_TABLECODE'] . '.' . $relrec['RELT_PFIELD'] . '=' . $relrec['PARENT_TABLECODE'] . '.' . $relrec['PARENT_FIELDNAME'] . ' AND ' . $relrec['REL_TABLECODE'] . '.' . $relrec['RELT_CFIELD'] . '=' . $relrec['CHILD_TABLECODE'] . '.' . $relrec['CHILD_FIELDNAME'] . ' AND ' . $relrec['REL_TABLECODE'] . '.PARENT_TABLECODE=' . singleQuote($relrec['PARENT_TABLECODE']) . ' AND ' . $relrec['REL_TABLECODE'] . '.CHILD_TABLECODE=' . singleQuote($relrec['CHILD_TABLECODE']) . ')';
		}
		return $criteria;
    }

    /**
     * Retrive Result Set with relation information.
     * @param $parent_tablecode
     * @param $child_tablecode
     * @return resource
     */
    function getRelationInfo($parent_tablecode, $child_tablecode) {
        $record = array();
        $sql = 'SELECT DBDR.RELATION_TYPE, DBDR.PARENT_TABLECODE, DBDR.CHILD_TABLECODE, DBDR.PARENT_FIELDNAME, DBDR.CHILD_FIELDNAME, DBDR.TABLECODE REL_TABLECODE, DBDR.RELT_PFIELD, DBDR.RELT_CFIELD, CONCAT(CONCAT(DBTB.MODULECODE,' . singleQuote('_') . '),DBTB.IDNAME) REL_TABLENAME, CONCAT(CONCAT(DBTB2.MODULECODE,' . singleQuote('_') . '),DBTB2.IDNAME) PARENT_TABLENAME, CONCAT(CONCAT(DBTB3.MODULECODE,' . singleQuote('_') . '),DBTB3.IDNAME) CHILD_TABLENAME, DBDR.EXCLUSIVES, DBDR.SRCH_LISTING FROM CLX_DB_DYNRELATIONS DBDR, CLX_DB_TABLES DBTB, CLX_DB_TABLES DBTB2, CLX_DB_TABLES DBTB3 WHERE DBDR.TABLECODE = DBTB.IDCODE AND DBDR.PARENT_TABLECODE = DBTB2.IDCODE AND DBDR.CHILD_TABLECODE = DBTB3.IDCODE AND DBDR.PARENT_TABLECODE=' . singleQuote($parent_tablecode) . ' AND DBDR.CHILD_TABLECODE=' . singleQuote($child_tablecode) . ' AND DBDR.ACTIVE = 1';
        $record = ClxCommon::$db->getOneRecord($sql);
        if (!empty($record)) {
            $record['RELATIONS'] = array();
            $sql = 'SELECT * FROM CLX_DB_DYNRELATION_COLS WHERE PARENT_TABLECODE=' . singleQuote($parent_tablecode) . ' AND CHILD_TABLECODE=' . singleQuote($child_tablecode);
            $rs = ClxCommon::$db->executeQuery($sql);
            while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                $record['RELATIONS'][] = $r;
            }
        }
        return $record;
    }

}
