<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_query_join_criteria extends clx_tool {

    var $logicals;
    var $operators;
    var $variables;
    var $debug = false;

    function __construct() {
        $this->code = 'DBQJ';
        $this->name = 'Query Join Criteria';
        $this->system = 'Database';
        $this->version = '0.2.2.1';


        $this->accesskey = 'DBQY';

        $this->tablename = 'CLX_DB_QUERY_JOIN_CRITERIA';
        $this->record_name_singular = 'query join criteria';
        $this->record_name_plural = 'query join criteria';
        $this->primary_field = 'QUERYCODE';
        $this->secondary_field = 'TABLECODE';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('QUERYCODE', 'TABLECODE', 'FIELDNAME', 'SEQNUM');
        $this->logicals = array('AND' => 'GEN_LOG_AND');
        $this->operators = array('EQ' => 'GEN_OPE_EQUAL');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['LOGICAL'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '', 'default' => '');
        $fields['OPEN_BRACKETS'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '', 'default' => '');
        $fields['CLOSE_BRACKETS'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '', 'default' => '');
        $fields['SPECIAL'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'SEQNUM');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'FIELDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
		foreach($record as $key=>&$value) {
			if (empty($value) && $value!==0 ) {
				if ($key=='SEQNUM' ||$key=='SPECIAL')
					$value=0;
				else	
					$value=singlequote('');
			}
			else {
//				echo $record[$key];
			}
		}
		ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function makepanel($querycode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'DBQJPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('querycode', $querycode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = 'T.IDNAME TABLENAME, QC.*';
            $fromclause =  $this->tablename.' QC, CLX_DB_TABLES T';
            $whereclause = 'UPPER(QC.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QC.TABLECODE=T.IDCODE';
            $sortclause = 'SEQNUM';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('selector', false);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_LOGICAL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_OPERATOR'), dict('GEN_LBL_VALUE')));
            $i = CLX_INIT_ZERO;
			//$existsx=array();
            while ($record = $page->nextRecord()) {//print_r($record);echo "<br><br>";
			//	if (!isset($existsx[$record['TABLECODE'].'.'.$record['FIELDNAME']]) AND !isset($existsx[$record['VALUE']]]) ) {
			//		$existsx[$record['TABLECODE'].'.'.$record['FIELDNAME']]=1;
			//		$existsx[$record['VALUE']]]=1;
					
					if ($i > CLX_NUM_ZERO) {
						$logical = dict($this->logicals[$record['LOGICAL']]);
					} else {
						$logical = CLX_CHAR_HTML_NBSP;
					}
					if ($record['TYPE'] === 'f') {
						$value = $record['VALUE'];
					}	
					$table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($logical, $record['OPEN_BRACKETS'] . $record['TABLENAME'] . ' (' . $record['TABLECODE'] . ') . ' . $record['FIELDNAME'], dict($this->operators[$record['OPERATOR']]), $value . $record['CLOSE_BRACKETS']));
					$i++;
				}
           // }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EDIT'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_DBQJ_TITLE_LIST'), $querycode . '_CRITERIA', $form->source(), -1 * CLX_NUM_ONE);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $callingpage = peekHistory();
	
        $querycode = $callingpage['querycode'];
        $criteria = $this->getCriterias($querycode);
        $scripts = setJavaScriptTag(getJavaScriptFilename('DBQJ_editform.js'), true);
        $select_size = CLX_NUM_12;
        $tables = array();
        //$fields = array();
        $lasttable = '';
        $tab_count = CLX_NUM_MINUS_1;
        $fld_count = CLX_INIT_ZERO;

        $js = 'tables = new Array();' . CLX_CHAR_NL . 'fields = new Array();' . CLX_CHAR_NL;
        $records = ClxCommon::$db->executeQuery('SELECT T.IDCODE TABLECODE, T.IDNAME TABLENAME, TB.FIELDNAME FIELDNAME, DF.FIELDTYPE FIELDTYPE FROM CLX_DB_TABLES T, CLX_DB_TABLEFIELDS TB, CLX_DB_QUERY_TABLES QT, CLX_DB_FIELDS DF WHERE UPPER(QT.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QT.TABLECODE=T.IDCODE AND QT.TABLECODE = TB.TABLECODE AND QT.INVISIBLE = 0 AND DF.FIELDNAME = TB.FIELDNAME ORDER BY T.IDCODE, TB.FIELDNAME');
		
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            //Test if field object is a valid field based on object definition
            $fieldobj = getToolObject('DBFT', $record['FIELDTYPE']);
            if (empty($fieldobj->has_question)) {
                continue;
            }

            if ($lasttable != $record['TABLECODE']) {
                $lasttable = $record['TABLECODE'];
                $tables[$record['TABLECODE']] = array('code' => $record['TABLECODE'], 'name' => $record['TABLENAME']);
                $tab_count++;
                $js .= 'tables[' . $tab_count . '] = ' . singleQuote($record['TABLECODE'] . '|DOT|' . $record['TABLENAME'] . ' (' . $record['TABLECODE'] . ')') . ';' . CLX_CHAR_NL;
                $js .= 'fields[' . $tab_count . '] = new Array();' . CLX_CHAR_NL;
                $fld_count = CLX_INIT_ZERO;
            }
            $tables[$lasttable][$record['FIELDNAME']] = array('code' => $record['FIELDNAME'], 'name' => $record['FIELDNAME']);
            $js .= 'fields[' . $tab_count . '][' . $fld_count . '] = ' . singleQuote($record['FIELDNAME'] . '|DOT|' . $record['FIELDNAME']) . ';' . CLX_CHAR_NL;
            $fld_count++;
        }
        $js .= CLX_CHAR_NL . CLX_CHAR_NL;
        $js .= 'logicals = new Array();' . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
        foreach ($this->logicals as $key => $value) {
            $js .= 'logicals[' . $i . '] = ' . singleQuote($key . '|DOT|' . dict($value)) . ';' . CLX_CHAR_NL;
            $i++;
        }
        $js .= CLX_CHAR_NL . CLX_CHAR_NL;
        $js .= 'operators = new Array();' . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
        foreach ($this->operators as $key => $value) {
            $js .= 'operators[' . $i . '] = ' . singleQuote($key . '|DOT|' . dict($value)) . ';' . CLX_CHAR_NL;
            $i++;
        }
        $js .= CLX_CHAR_NL . CLX_CHAR_NL;
        $js .= 'variables = new Array();' . CLX_CHAR_NL;
        $i = CLX_INIT_ZERO;
		if (is_array($this->variables) && count($this->variables)>0) {
			foreach ($this->variables as $key => $value) {
				$js .= 'variables[' . $i . '] = ' . singleQuote($key . '|DOT|' . dict($value)) . ';' . CLX_CHAR_NL;
				$i++;
			}
		}
		$js .= 'form_init();' . CLX_CHAR_NL;
        $scripts .= setJavaScriptTag($js, false);

        // Create the from that will encompass everything -------------------------------------------------------------------------------
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBQJ_TITLE_MODIFY') : dict('TOOL_DBQJ_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('querycode', $querycode);
        $form->setProp('onsubmit', 'return form_process()');
        if (empty($this->debug)) {
            $form->addArgument('criteria', $criteria);
            $form->addArgument('selected_row', '');
            $form->addArgument('row_qty', '0');
        }
        // Titles ---------------------------------------------------------------------------

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CHAR_EMPTY_STRING, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($this->debug)) {
            $table->addField($msg, 'selected_row:', 'selected_row', '', CLX_NUM_TEN, CLX_NUM_TEN);
            $table->addField($msg, 'row_qty:', 'row_qty', '0', CLX_NUM_TEN, CLX_NUM_TEN);
        }
        $form->addContents($table->source());

        // The criteria editor -------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TABLES'), dict('GEN_LBL_FIELDS'), dict('GEN_LBL_OPERATORS'), dict('GEN_LBL_VALUES')));
        // The list of tables inside the criteria editor
        $pop = new htmlSelect('DBQJ_tables');
        $pop->setProp('onChange', 'form_set_fields();');
        $pop->setProp('size', $select_size);
        $table->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        // The list of fields
        $pop = new htmlSelect('DBQJ_fields');
        $pop->setProp('size', $select_size);
        $table->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        // The operators
        $pop = new htmlSelect('DBQJ_operators');
        $pop->setProp('size', CLX_NUM_ONE	);
        $table->addCell($pop->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));

        $buttons = '';
        // Constant Value
        if (!array_key_exists('DBQJ_constant', UserRequest::$post)) {
            UserRequest::$post['DBQJ_constant'] = '';
        }
        $check = new htmlRadioButton('DBQJ_value', true, 'constant');
		
        // Fields
        $check = new htmlRadioButton('DBQJ_value', false, 'field');
        $pop = new htmlSelect('DBQJ_joinfields');
		$pop->setProp('size', $select_size);
        $buttons .= $check->source() . CLX_CHAR_HTML_NBSP . $pop->source() . '<br>';
        // Buttons
        $buttons .= '<br><hr><br><p style="text-align: center;"><input type="button" name="add_btn" value="' . dict('GEN_LBL_ADD') . '" onclick="criteria_add();">';
        $buttons .= '<input type="button" name="save_btn" value="' . dict('GEN_LBL_MODIFY') . '" onclick="criteria_modify();">';
        $buttons .= '<input type="button" name="delete_btn" value="' . dict('GEN_LBL_DELETE') . '" onclick="criteria_delete();">';
        if (!empty($this->debug)) {
            $buttons .= '<input type="button" name="test_btn" value="' . dict('GEN_LBL_TEST') . '" onclick="form_process()">';
        }
        $buttons .= '</p>';
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $form->addContents($table->source());
        // The criteria listing ------------------------------------------------------------------

        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT, CLX_PROP_ID => 'criteria_table'));
        $table->setProp('onclick', 'row_select()');
        if (!empty($this->debug)) {
            $table->setProp('border', '2');
        }
        $table->addCell(dict('GEN_LBL_CRITERIAS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $form->addContents($table->source());
        // Form Controls and stuff ---------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));


        $buttons = (!empty($modify)) ? htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source() : htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $buttons .= htmlButton::getButton('cancel', CLX_CHAR_EMPTY_STRING, array())->source();

        htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'JOINS', 'form' => $form->getProp(CLX_PROP_ID)))->source();

        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents . $scripts;
    }

    function checkForm(&$msg) {
        return (empty($msg)) ? true : false;
    }

    function loadToolRecord($recordid) {
        $record = array();
        return $record;
    }

    function saveRecord() {
        $querycode = UserRequest::$request['querycode'];
        $criteria = UserRequest::$request['criteria'];
		apc_clear_querycode(UserRequest::$request['querycode']);
        $lines = explode('|END|', $criteria);
        ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND SPECIAL=0');
		ClxCommon::$db->deleteRecords('CLX_DB_QUERY_JOIN_TYPE_RELATION', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) );
		$record = array();
        $record['QUERYCODE'] = $querycode;
        $i = CLX_INIT_ZERO;
		$creados_join=array();
		$creados_join_table=array();
        if (strlen($criteria) > CLX_NUM_ZERO) {

            foreach ($lines as $line) {

                $info = explode('|PRE|', $line);
					$record['LOGICAL'] = $info[CLX_NUM_ZERO];
					$record['OPEN_BRACKETS'] = $info[CLX_NUM_ONE];
					$record['TABLECODE'] = $info[CLX_NUM_TWO];
					$record['FIELDNAME'] = $info[CLX_NUM_THREE];
					$record['OPERATOR'] = $info[CLX_NUM_FOUR];
					$record['TYPE'] = $info[CLX_NUM_FIVE];
					$record['VALUE'] = clx_html_spchrs_decode($info[CLX_NUM_SIX]);
					$record['CLOSE_BRACKETS'] = $info[CLX_NUM_7];
					$record['SEQNUM'] = $i + CLX_NUM_HUNDRED;
					$record['SPECIAL'] = '0';
					ClxCommon::$db->createRecord($this->tablename, $record, false);
					$i++;
            }
        }

        $DBQY = &getTool('DBQY');
        $DBQY->versionIncrease('IDCODE', $querycode);
		apc_clear_querycode($querycode);
    }
    /**
     * Action is called for every user interaction with the module...
     *
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
			if ( isset($_SESSION["_dEBUG"])) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
					                                      fwrite($tta,chr(13).chr(10)."14001- hora: ac:$ac".date("h:i:s d/m/Y").chr(13).chr(10)); 
														  $te=debug_backtrace();for ($t=0;$t<18;$t++) { 
															if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	
														  }	
														  fwrite($tta,chr(13).chr(10).chr(13).chr(10));fclose($tta);}
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'MODIFY':
                    if ($this->getAccess()->canModify()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($this->checkForm($msg)) {
                                ClxFreeReturn($this->saveRecord());
								
                                if (count($idlist) > CLX_NUM_ZERO) {
                                    $id = array_pop($idlist);
                                    $record = $this->loadToolRecord($id);
								    $contents = $this->editForm($ac, $idlist, $record, $msg);
							    } else if ( isset($GLOBALS["process"])  ) { 
                                    $contents="";      
								}
								else{
                                    $info = popHistory();
                                    $contents = executeHistory($info);
                                }
                            } else {
                                $contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
                            }
                        } else {
                            $id = array_pop($idlist);
                            $record = $this->loadToolRecord($id);
                            $contents = $this->editForm($ac, $idlist, $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
               
                default:
				
                    
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
	}	
    function getCriterias($querycode) {
        $criteria = '';
        $criterias = ClxCommon::$db->executeQuery('SELECT T.IDNAME TABLENAME, QC.* FROM CLX_DB_QUERY_JOIN_CRITERIA QC, CLX_DB_TABLES T WHERE UPPER(QC.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QC.TABLECODE = T.IDCODE AND QC.SPECIAL = 0 ORDER BY QC.SEQNUM');
        $nl = '';
        while ($criteriarec = ClxCommon::$db->getNextRecord($criterias)) {
            $criteria .= $nl . $criteriarec['LOGICAL'] . '|SEP|' .
                    $criteriarec['OPEN_BRACKETS'] . '|SEP|' .
                    $criteriarec['TABLECODE'] . '|SEP|' .
                    $criteriarec['FIELDNAME'] . '|SEP|' .
                    $criteriarec['OPERATOR'] . '|SEP|' .
                    $criteriarec['TYPE'] . '|SEP|';
            if ($criteriarec['TYPE'] === 'v') {
                $criteria .= $criteriarec['VALUE'] . '|DOT|' . dict($this->variables[$criteriarec['VALUE']]);
            } else {
                $criteria .= htmlentities($criteriarec['VALUE']);
            }
            $criteria .= '|SEP|' . $criteriarec['CLOSE_BRACKETS'];
            $nl = '|END|';
        }
        return $criteria;
    }

}
