<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_query_tables extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBQT';
        $this->name = 'Query Tables';
        $this->system = 'Database';
        $this->version = '0.1.2.1';


        $this->accesskey = 'DBQY';

        $this->tablename = 'CLX_DB_QUERY_TABLES';
        $this->record_name_singular = 'query table';
        $this->record_name_plural = 'query tables';
        $this->primary_field = 'QUERYCODE';
        $this->secondary_field = 'TABLECODE';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('QUERYCODE', 'TABLECODE');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['INVISIBLE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'QUERYCODE,TABLECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    function unpack($parameters) {  // not working?
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $msg = array();
            switch ($ac) {
                case 'FIELDS_XML':
                    $querycode = (!empty(UserRequest::$request['qc'])) ? UserRequest::$request['qc'] : CLX_CHAR_EMPTY_STRING;
                    $tablecode = (!empty(UserRequest::$request['tc'])) ? UserRequest::$request['tc'] : CLX_CHAR_EMPTY_STRING;
                    sendXmlHeader();
                    echo $this->getTableFields($querycode, $tablecode,false)->saveXml();
                    salir(CLX_NUM_ZERO);
                    break;
                case 'MODIFY':
                    if ($this->getAccess()->canModify()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            if ($this->checkForm($msg)) {
                                $this->saveRecord();
                                $querycode = UserRequest::$request['querycode'];
                                $contents = $this->joinsForm('JOINS', $querycode);
                            } else {
                                $contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
                            }
                        } else {
                            $id = array_pop($idlist);
                            $record = $this->loadToolRecord($id);
                            $contents = $this->editForm($ac, $idlist, $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_QUERY_TABLES-CAN_MODIFY_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'JOINS':
                    if ($this->getAccess()->canModify()) {
                        $this->saveJoins();
                        $info = popHistory();
                        $contents = executeHistory($info);
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_QUERY_TABLES-CAN_MODIFY_2][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_QUERY_TABLES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($querycode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'DBQTPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('querycode', $querycode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = 'T.MODULECODE, T.IDCODE, T.IDNAME';
            $fromclause = 'CLX_DB_QUERY_TABLES QT, CLX_DB_TABLES T';
            $whereclause = 'QT.TABLECODE = T.IDCODE AND UPPER(QT.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QT.INVISIBLE=0';
            $sortclause = 'QT.ID';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('selector', false);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_THREE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_TABLENAME')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['MODULECODE'], $record['IDCODE'], $record['IDNAME']));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EDIT'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_DBQT_TITLE_LIST'), $querycode . '_TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $scripts = '';
        /* $variables = array();
          $scripts .= getJavaScript('DBQT_editform.js', $variables); */

        $scripts = setJavaScriptTag(getJavaScriptFilename('DBQT_editform.js'), true);
        $modules = array();
        $tables = array();
        $lastmodule = '';
        $mod_count = -1 * CLX_NUM_ONE;
        $tab_count = CLX_INIT_ZERO;
        $js = 'modules = new Array();' . CLX_CHAR_NL . 'tables = new Array();' . CLX_CHAR_NL;
        $records = ClxCommon::$db->executeQuery('SELECT m.IDCODE MODULECODE, m.IDNAME MODULENAME, t.IDCODE TABLECODE, t.IDNAME TABLENAME FROM CLX_DB_MODULES m, CLX_DB_TABLES t WHERE t.MODULECODE = m.IDCODE ORDER BY m.IDCODE, t.IDCODE');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            if ($lastmodule != $record['MODULECODE']) {
                $lastmodule = $record['MODULECODE'];
                $modules[$record['MODULECODE']] = array('code' => $record['MODULECODE'], 'name' => $record['MODULENAME']);
                $mod_count++;
                $js .= 'modules[' . $mod_count . '] = ' . singleQuote($record['MODULECODE'] . ' (' . $record['MODULENAME'] . ')') . ';' . CLX_CHAR_NL;
                $js .= 'tables[' . $mod_count . '] = new Array();' . CLX_CHAR_NL;
                $tab_count = CLX_INIT_ZERO;
            }
            $tables[$lastmodule][$record['TABLECODE']] = array('code' => $record['TABLECODE'], 'name' => $record['TABLENAME']);
            $js .= 'tables[' . $mod_count . '][' . $tab_count . '] = ' . singleQuote($record['TABLECODE'] . '?' . $record['TABLECODE'] . ' (' . $record['TABLENAME'] . ')') . ';' . CLX_CHAR_NL;
            $tab_count++;
        }
        $js .= 'updateTables();' . CLX_CHAR_NL;
        $scripts .= setJavaScriptTag($js, false);
        $callingpage = peekHistory();
        $querycode = $callingpage['querycode'];
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBQT_TITLE_MODIFY') : dict('TOOL_DBQT_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('querycode', $querycode);
        $form->setProp('onsubmit', 'return processForm()');
        $form->addArgument('tables', '');

        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_MODULES'), dict('GEN_LBL_TABLES'), CLX_CHAR_HTML_NBSP, dict('GEN_LBL_SELECTED')));

        $pop = new HTMLSelect('DBQT_modules');
        $pop->setProp('onChange', 'updateTables()');
        $pop->setProp('size', '10');
        $selected = true;
        foreach ($modules as $code => $info) {
            $pop->addItem($code . ' (' . $info['name'] . ')', $code, $selected);
            $selected = false;
        }
        $table->addCell($pop->source(), array());
        $pop = new HTMLSelect('DBQT_tables');
        $pop->setProp('size', '10');
        $table->addCell($pop->source(), array());

        $buttons = '<input type="button" name="select_btn" value="' . dict('GEN_LBL_SELECT') . '" onclick=' . singleQuote('copyItem("DBQT_tables","DBQT_selected")') . '><br>';
        $buttons .= '<input type="button" name="delete_btn" value="' . dict('GEN_LBL_DELETE') . '" onclick=' . singleQuote('deleteItem("DBQT_selected")') . '><br>';
        $table->addCell($buttons, array());
        $pop = new HTMLSelect('DBQT_selected');
        $pop->setProp('size', '10');
        $pop->setProp('onChange', 'updateTables();');
        $records = ClxCommon::$db->executeQuery('SELECT T.IDCODE, T.IDNAME FROM CLX_DB_TABLES T, CLX_DB_QUERY_TABLES QT WHERE T.IDCODE=QT.TABLECODE AND UPPER(QT.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QT.INVISIBLE=0 ORDER BY T.IDCODE');
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {

            $pop->addItem($record['IDCODE'] . ' (' . $record['IDNAME'] . ')', $record['IDCODE'], false);
        }
        $table->addCell($pop->source(), array());
        $buttons = (!empty($modify)) ? htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source() : htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $buttons .= htmlButton::getButton('cancel', CLX_CHAR_EMPTY_STRING, array())->source();

        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents . $scripts;
    }

    function checkForm(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    function joinsForm($ac, $querycode) {

        $idlist = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('querycode', $querycode);

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell(dict('TOOL_DBQT_TITLE_JOINS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_JOINS')));
        $records = ClxCommon::$db->getRecords('CLX_DB_QUERY_TABLES', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' ORDER BY ID');
        $tables = array();
        $joins = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $tables[] = $record['TABLECODE'];
        }
        $tableqty = count($tables);
        foreach ($tables as $fathercode) {
            for ($i = CLX_NUM_ZERO; $i < $tableqty; $i++) {
                if ($fathercode != $tables[$i]) {
                    $this->discoverJoins($fathercode, $tables[$i], $joins);
                }
            }
        }
        $form->addArgument('joinqty', count($joins));
        $i = CLX_INIT_ZERO;
        foreach ($joins as $join) {

            $father_rec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($join['FATHER_TABLECODE']));
            $child_rec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($join['CHILD_TABLECODE']));
            if ($join['type'] === 'LOOK') {
                $relation = $father_rec['IDNAME'] . ' (' . $father_rec['IDCODE'] . ') . ' . $join['FATHER_FIELDNAME'] . ' = ' . $child_rec['IDNAME'] . ' (' . $child_rec['IDCODE'] . ') . ' . $join['CHILD_FIELDNAME'];
                $value = 'LOOK.' . $father_rec['IDCODE'] . '.' . $join['FATHER_FIELDNAME'] . '.' . $child_rec['IDCODE'] . '.' . $join['CHILD_FIELDNAME'];
            } else {
                $relation = $father_rec['IDNAME'] . ' (' . $father_rec['IDCODE'] . ') = ' . $child_rec['IDNAME'] . ' (' . $child_rec['IDCODE'] . ')';
                $value = 'DYN.' . $father_rec['IDCODE'] . '.' . $child_rec['IDCODE'];
            }
            $select = new htmlCheckbox('selector_' . $i, false);
            $select->setProp('value', $value);
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => CLX_PROP_CENTER));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($relation));
            $i++;
        }
        $table->addCell(dict('GEN_LBL_OPTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $select = new htmlCheckbox('delete_relations', false);
        $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
        $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(dict('TOOL_DBQT_DELETE_RELATIONS')));

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'JOINS', 'form' => $form->getProp(CLX_PROP_ID)))->source() . htmlButton::getButton('cancel', CLX_CHAR_EMPTY_STRING, array())->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function discoverJoins($fathercode, $childcode, &$joins) {
        $records = ClxCommon::$db->executeQuery('SELECT F.FIELD_EXTRA1 FATHER_TABLECODE, TF.TABLECODE CHILD_TABLECODE, F.FIELDNAME CHILD_FIELDNAME, F.FIELD_EXTRA2 FATHER_FIELDNAME FROM CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TF.TABLECODE = ' . singleQuote($childcode) . ' AND TF.FIELDNAME = F.FIELDNAME AND F.FIELDTYPE = ' . singleQuote('LOOK') . ' AND F.FIELD_EXTRA1 = ' . singleQuote($fathercode));
        if (ClxCommon::$db->numRows($records) > CLX_NUM_ZERO) {
            $join = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE);
            $join['type'] = 'LOOK';
            $joins[] = $join;
        }
        $records = ClxCommon::$db->getRecords('CLX_DB_DYNRELATIONS', 'PARENT_TABLECODE=' . singleQuote($fathercode), 'CHILD_TABLECODE=' . singleQuote($childcode) . '  AND RELATION_TYPE = 0');
        if (ClxCommon::$db->numRows($records) > CLX_NUM_ZERO) {
            $join = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE);
            $join['FATHER_TABLECODE'] = $fathercode;
            $join['CHILD_TABLECODE'] = $childcode;
            $join['FATHER_FIELDNAME'] = '';
            $join['CHILD_FIELDNAME'] = '';
            $join['TYPE'] = 'DYN';
            $joins[] = $join;
        }
    }

    function saveJoins() {

        $querycode = (array_key_exists('querycode', UserRequest::$request)) ? UserRequest::$request['querycode'] : CLX_CHAR_EMPTY_STRING;
        $qty = (array_key_exists('joinqty', UserRequest::$request)) ? UserRequest::$request['joinqty'] : CLX_CHAR_EMPTY_STRING;
        if (!empty(UserRequest::$request['delete_relations'])) {
            ClxCommon::$db->deleteRecords('CLX_DB_QUERY_CRITERIA', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)));
            $seqnum = CLX_INIT_ZERO;
        }

        $seqnum = (ClxCommon::$db->countRecords('CLX_DB_QUERY_CRITERIA', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode))) > CLX_NUM_ZERO) ? CLX_INIT_ZERO : ClxCommon::$db->getOne('SELECT SEQNUM+1 SEQNUM FROM CLX_DB_QUERY_CRITERIA WHERE QUERYCODE = ' . singleQuote($querycode) . ' ORDER BY SEQNUM DESC');
        $special = CLX_NUM_ONE;
        for ($i = CLX_NUM_ZERO; $i < $qty; $i++) {
            if (empty(UserRequest::$request['selector_' . $i])) {
                continue;
            }
            $info = explode('.', UserRequest::$request['selector_' . $i]);

            if ($info[CLX_NUM_ZERO] === 'DYN') {
                $rel_rec = ClxCommon::$db->getRecord('CLX_DB_DYNRELATIONS', 'PARENT_TABLECODE=' . singleQuote($info[CLX_NUM_ONE]) . ' AND CHILD_TABLECODE=' . singleQuote($info[CLX_NUM_TWO]) . '  AND RELATION_TYPE = 0');
                $record = array();
                $record['QUERYCODE'] = $querycode;
                $record['LOGICAL'] = 'AND';
                $record['TABLECODE'] = $rel_rec['TABLECODE'];
                $record['FIELDNAME'] = 'PARENT_RECORDID';
                $record['OPERATOR'] = 'EQ';
                $record['TYPE'] = 'f';
                $record['VALUE'] = $info[CLX_NUM_ONE] . '.ID';
                $record['OPEN_BRACKETS'] = '(';
                $record['CLOSE_BRACKETS'] = '';
                $record['SEQNUM'] = $seqnum;
                $record['SPECIAL'] = $special;
                $seqnum++;
                ClxCommon::$db->createRecord('CLX_DB_QUERY_CRITERIA', $record, false);
                $record = array();
                $record['QUERYCODE'] = $querycode;
                $record['LOGICAL'] = 'AND';
                $record['TABLECODE'] = $rel_rec['TABLECODE'];
                $record['FIELDNAME'] = 'PARENT_TABLECODE';
                $record['OPERATOR'] = 'EQ';
                $record['TYPE'] = 'k';
                $record['VALUE'] = $info[CLX_NUM_ONE];
                $record['OPEN_BRACKETS'] = '';
                $record['CLOSE_BRACKETS'] = '';
                $record['SEQNUM'] = $seqnum;
                $record['SPECIAL'] = $special;
                $seqnum++;
                ClxCommon::$db->createRecord('CLX_DB_QUERY_CRITERIA', $record, false);
                $record = array();
                $record['QUERYCODE'] = $querycode;
                $record['LOGICAL'] = 'AND';
                $record['TABLECODE'] = $rel_rec['TABLECODE'];
                $record['FIELDNAME'] = 'CHILD_TABLECODE';
                $record['OPERATOR'] = 'EQ';
                $record['TYPE'] = 'k';
                $record['VALUE'] = $info[CLX_NUM_TWO];
                $record['OPEN_BRACKETS'] = '';
                $record['CLOSE_BRACKETS'] = '';
                $record['SEQNUM'] = $seqnum;
                $record['SPECIAL'] = $special;
                $seqnum++;
                ClxCommon::$db->createRecord('CLX_DB_QUERY_CRITERIA', $record, false);
                $record = array();
                $record['QUERYCODE'] = $querycode;
                $record['LOGICAL'] = 'AND';
                $record['TABLECODE'] = $rel_rec['TABLECODE'];
                $record['FIELDNAME'] = 'CHILD_RECORDID';
                $record['OPERATOR'] = 'EQ';
                $record['TYPE'] = 'f';
                $record['VALUE'] = $info[CLX_NUM_TWO] . '.ID';
                $record['OPEN_BRACKETS'] = '';
                $record['CLOSE_BRACKETS'] = ')';
                $record['SEQNUM'] = $seqnum;
                $record['SPECIAL'] = $special;
                $seqnum++;
                $special++;
                ClxCommon::$db->createRecord('CLX_DB_QUERY_CRITERIA', $record, false);
                if (ClxCommon::$db->oneExistsRecords($this->tablename, 'TABLECODE=' . singleQuote($rel_rec['TABLECODE'])) == CLX_NUM_ZERO) {
                    $record = array();
                    $record['QUERYCODE'] = $querycode;
                    $record['TABLECODE'] = $rel_rec['TABLECODE'];
                    $record['INVISIBLE'] = '1';
                    ClxCommon::$db->createRecord($this->tablename, $record, false);
                }
            } elseif ($info[CLX_NUM_ZERO] === 'LOOK') {
                $record = array();
                $record['QUERYCODE'] = $querycode;
                $record['LOGICAL'] = 'AND';
                $record['TABLECODE'] = $info[CLX_NUM_ONE];
                $record['FIELDNAME'] = $info[CLX_NUM_TWO];
                $record['OPERATOR'] = 'EQ';
                $record['TYPE'] = 'f';
                $record['VALUE'] = $info[CLX_NUM_THREE] . '.' . $info[CLX_NUM_FOUR];
                $record['OPEN_BRACKETS'] = '';
                $record['CLOSE_BRACKETS'] = '';
                $record['SEQNUM'] = $seqnum;
                $record['SPECIAL'] = $special;
                $seqnum++;
                $special++;
                ClxCommon::$db->createRecord('CLX_DB_QUERY_CRITERIA', $record, false);
            }
        }
    }

    function loadToolRecord($recordid) {
        $record = array();
        return $record;
    }

    function saveRecord() {

        $querycode = UserRequest::$request['querycode'];
        $tables = toArray(UserRequest::$request['tables']);
		apc_clear_querycode(UserRequest::$request['querycode']);
        $savedTables = array();
        ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)));
        $record = array();
        $record['QUERYCODE'] = $querycode;
        $record['INVISIBLE'] = '0';
        foreach ($tables as $tablecode) {
            if (!in_array($tablecode, $savedTables)) {
                $savedTables[] = $tablecode;
                $record['TABLECODE'] = $tablecode;
                ClxCommon::$db->createRecord($this->tablename, $record, false);
            }
        }
        $DBQY = &getTool('DBQY');
        $DBQY->versionIncrease('IDCODE', $querycode);
		apc_clear_querycode($querycode);
    }

    /**
     * Return sql resource with fields on table
     * @param string $querycode
     * @param string $tablecode
     * @param bool $noderived
     * @return resource
     */
    function getTableFieldsSql($querycode, $tablecode = '', $noderived = false) {
        $crits = (empty($tablecode)) ? '' : ' AND X.TABLECODE = ' . singleQuote($tablecode) . ' ';
        $noderived = ($noderived) ? ' AND X.DERIVED = 0 ' : CLX_CHAR_EMPTY_STRING;
        $sql = 'SELECT DISTINCT X.*, CONCAT(CONCAT(X.TABLECODE,' . singleQuote('_') . '),X.FIELDNAME) ALIAS FROM (SELECT DBQT.QUERYCODE, DBTF.TABLECODE, DBTF.FIELDNAME, DBTF.SEQNUM, 0 DERIVED FROM CLX_DB_QUERY_TABLES DBQT, CLX_DB_TABLEFIELDS DBTF WHERE DBQT.TABLECODE = DBTF.TABLECODE AND DBQT.QUERYCODE = ' . singleQuote($querycode) . ' UNION SELECT DBQT.QUERYCODE, DBTF.TABLECODE, DBTF.FIELDNAME, DBTF.SEQNUM, 1 DERIVED FROM CLX_DB_QUERY_TABLES DBQT, CLX_DB_TABLEDFIELDS DBTF WHERE DBQT.TABLECODE = DBTF.TABLECODE AND DBQT.QUERYCODE = ' . singleQuote($querycode) . ') X WHERE 1=1 ' . $crits . ' ' . $noderived . ' ORDER BY X.TABLECODE ASC, X.DERIVED ASC, X.SEQNUM ASC, X.FIELDNAME ASC';
        return ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Return zml object with posible fields on tables of query
     * @param string $querycode
     * @param string $tablecode
     * @param bool $noderived
     * @return object
     */
    function getTableFields($querycode, $TABLECODE = '', $noderived = false) {
        // create a new XML document
        $doc = new DomDocument('1.0');
        if ($querycode === '')
            return $doc->saveXML();
        // create root node
        $node = $doc->createElement('TOOL');
        $node->setAttribute('CODE', $this->code);
        //adding id if not exists
        $rs = $this->getTableFieldsSql($querycode, $tablecode, $noderived);
        $numrow = CLX_NUM_ONE;
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            // add node for each row
            $occ = $doc->createElement($this->code);
            $occ->setAttribute('NUMROW', $numrow);
            $numrow++;
            foreach ($r as $key => $val) {
                $field = $doc->createElement($key);
                $value = $doc->createTextNode(utf8_encode($val));
                $field->appendChild($value);
                $occ->appendChild($field);
            }
            $node->appendChild($occ);
        }
        $doc->appendChild($node);
        return $doc;
    }

}
