<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.IbsSockets
 * @abstract		IBS Socket Messages Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class eb_socket_exceptions extends clx_tool {

    var $operators;

    function __construct() {
        $this->code = 'EBIP';
        $this->name = 'IBS Socket Manager Import Exceptions';
        $this->system = 'Sockets';
        $this->version = '1.0.0.0';

        $this->accesskey = 'EBSK';


        $this->tablename = 'CLX_EB_SOCKET_EXCEPTIONS';
        $this->record_name_singular = 'eb socket exception';
        $this->record_name_plural = 'ib socket exceptions';
        $this->primary_field = 'ID';
        $this->secondary_field = 'EBSOCKETCODE';

        $this->operators = array('EQ' => 'GEN_OPE_EQUAL',
            'NQ' => 'GEN_OPE_NOTEQUAL',
            'LS' => 'GEN_OPE_LESS',
            'LQ' => 'GEN_OPE_LESS_EQUAL',
            'GQ' => 'GEN_OPE_GREATER_EQUAL',
            'GT' => 'GEN_OPE_GREATER'
        );
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_EBIP_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_POSITION',
                            'fields' => array(
                                array('name' => 'position', 'type' => 'string', 'default' => '', 'size' => '10', 'displaysize' => '10', 'validations' => array(
                                        array('type' => 'integer', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_OPERATOR',
                            'fields' => array(
                                array('name' => 'operator', 'type' => CLX_CLASS_LIST, 'default' => '', 'options' => $this->operators))
                        ), array('label' => 'GEN_LBL_VALUE',
                            'fields' => array(
                                array('name' => 'value', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['EBSOCKETCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['POSITION'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['VALUE'] = array('type' => 'mediumtext', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'EBSOCKETCODE,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- I M P O R T    E X C E P T I O N S
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [EB_SOCKET_EXCEPTIONS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'EBIPPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('ebsocketcode', $groupcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = $this->tablename;
            $whereclause = 'EBSOCKETCODE=' . singleQuote($groupcode);
            $sortclause = 'SEQNUM';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_START_POSITION'), dict('GEN_LBL_OPERATOR'), dict('GEN_LBL_VALUE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['POSITION'], dict($this->operators[$record['OPERATOR']]), $record['value']));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_EBIP_TITLE_LIST'), 'EB_SOCKET_EXCEPTIONS_' . $groupcode, $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_POSITION') . ':', $record['EBIP_position']);
        $table->addData(dict('GEN_LBL_VALUE') . ':', $record['EBIP_value']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['EBIP_id'] = $recordid;
        $record['EBIP_position'] = $values['POSITION'];
        $record['EBIP_value'] = $values['value'];
        $record['EBIP_operator'] = $values['OPERATOR'];
        $record['EBIP_ebsocketcode'] = $values['EBSOCKETCODE'];
        $record['EBIP_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['POSITION'] = UserRequest::$post['EBIP_position'];
        $record['value'] = UserRequest::$post['EBIP_value'];
        $record['OPERATOR'] = UserRequest::$post['EBIP_operator'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['EBIP_id']);
        $this->versionIncrease('ID', UserRequest::$post['EBIP_id']);
    }

    function createRecord_() {
        $callingpage = peekHistory();
        $record = array();
        $record['POSITION'] = UserRequest::$post['EBIP_position'];
        $record['value'] = UserRequest::$post['EBIP_value'];
        $record['OPERATOR'] = UserRequest::$post['EBIP_operator'];
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'EBSOCKETCODE=' . singleQuote($callingpage['ebsocketcode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $record['EBSOCKETCODE'] = $callingpage['ebsocketcode'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, 'ID=' . UserRequest::$post['EBIP_id'], false);
        $this->versionIncrease('ID', $recordid);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $rec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $id = $rec['ID'];
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $this->versionIncrease('ID', $id);
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $ebsocketcode = ClxCommon::$db->getField($this->tablename, 'EBSOCKETCODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBIM = &getTool('EBIM');
        $DBIM->versionIncrease('IDCODE', $ebsocketcode);
    }

}
