<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.IbsSockets
 * @abstract		IBS Socket Messages Manager
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-10-15 20:57:54 -0400 (Mon, 15 Oct 2018) $
 * @version        $Rev: 10266 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class eb_socket_translations extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'EBIT';
        $this->name = 'IBS Socket Manager Import Translations';
        $this->system = 'Sockets';
        $this->version = '5.5.0.0';


        $this->accesskey = 'EBSK';

        $this->tablename = 'CLX_EB_SOCKET_TRANSLATIONS';
        $this->record_name_singular = 'eb import translation';
        $this->record_name_plural = 'eb import translations';
        $this->primary_field = 'ID';
        $this->secondary_field = 'MAPID';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_EBIT_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_MAPID',
                            'fields' => array(
                                array('name' => 'mapid', 'type' => 'string', 'readonly' => '1'))
                        ), array('label' => 'GEN_LBL_TRANSL_CODE',
                            'caption' => 'GEN_LBL_TRANSL_SET_DESC',
                            'fields' => array(
                                array('name' => 'transl_code', 'type' => 'lookup', 'default' => '', 'tool' => 'TRTS', 'validations' => array(
                                        array('type' => 'lookup', 'fieldname' => 'IDCODE', 'tablename' => 'CLX_SYS_TRANSL_SETS'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_ORIGINAL_VALUE',
                            'fields' => array(
                                array('name' => 'original_value', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '64'))
                        ), array('label' => 'GEN_LBL_REPLACE_VALUE',
                            'fields' => array(
                                array('name' => 'replace_value', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '64'))
                        ), array('label' => 'GEN_LBL_CASE_SENSITIVE',
                            'fields' => array(
                                array('name' => 'case_sensitive', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => array('0' => 'GEN_LBL_NO', '1' => 'GEN_LBL_YES')))
                        )
                    )
                )
            )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['MAPID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TRANSL_CODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['ORIGINAL_VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['REPLACE_VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['CASE_SENSITIVE'] = array('type' => 'int(1) unsigned', 'null' => 'YES', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'MAPID,TRANSL_CODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'ORIGINAL_VALUE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes,
            'case_sensitive' => true);
    }

//-- I M P O R T   T R A N S L A T I O N S
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $mapid) = $parameters;
        $record = $this->unpackRecord($package);
        $record['MAPID'] = $mapid;
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $contents = parent::action($ac, $idlist);
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [EB_SOCKET_TRANSLATIONS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'EBITPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('mapid', $groupcode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = $this->tablename;
            $whereclause = 'MAPID=' . $groupcode;
            $sortclause = 'SEQNUM';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FIVE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_TRANSL_CODE'), dict('GEN_LBL_ORIGINAL_VALUE'), dict('GEN_LBL_REPLACE_VALUE'), dict('GEN_LBL_CASE_SENSITIVE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['TRANSL_CODE'], $record['ORIGINAL_VALUE'], $record['REPLACE_VALUE'], $yesno[$record['CASE_SENSITIVE']]));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_TRANSLATION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_EBIT_TITLE_LIST'), 'EB_SOCKET_TRANSLATIONS_' . $groupcode, $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_ORIGINAL_VALUE') . ':', $record['EBIT_original_value']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['EBIT_id'] = $recordid;
        $record['EBIT_mapid'] = $values['MAPID'];
        $record['EBIT_transl_code'] = $values['TRANSL_CODE'];
        $record['EBIT_original_value'] = $values['ORIGINAL_VALUE'];
        $record['EBIT_replace_value'] = $values['REPLACE_VALUE'];
        $record['EBIT_case_sensitive'] = $values['CASE_SENSITIVE'];
        $record['EBIT_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['TRANSL_CODE'] = UserRequest::$post['EBIT_transl_code'];
        $record['ORIGINAL_VALUE'] = UserRequest::$post['EBIT_original_value'];
        $record['REPLACE_VALUE'] = UserRequest::$post['EBIT_replace_value'];
        $record['CASE_SENSITIVE'] = UserRequest::$post['EBIT_case_sensitive'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['EBIT_id']);
        $this->versionIncrease('ID', UserRequest::$post['EBIT_id']);
    }

    function createRecord_() {
        $record = array();
        $record['MAPID'] = UserRequest::$post['EBIT_mapid'];
        $record['SEQNUM'] = '0';
        $record['TRANSL_CODE'] = UserRequest::$post['EBIT_transl_code'];
        $record['ORIGINAL_VALUE'] = UserRequest::$post['EBIT_original_value'];
        $record['REPLACE_VALUE'] = UserRequest::$post['EBIT_replace_value'];
        $record['CASE_SENSITIVE'] = UserRequest::$post['EBIT_case_sensitive'];
        ClxCommon::$db->createRecordVoid($this->tablename, $record, false);
        $this->versionIncrease('ID', UserRequest::$post['EBIT_id']);
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $mapid = $record['MAPID'];
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $EBIC = &getTool('EBIC');
        $EBIC->versionIncrease('ID', $mapid);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['EBIM_id'] = CLX_INIT_ZERO;
        $record['EBIT_mapid'] = $callingpage['mapid'];
        $record['EBIT_transl_code'] = '';
        $record['EBIT_original_value'] = '';
        $record['EBIT_replace_value'] = '';
        $record['EBIT_case_sensitive'] = '0';
        $record['EBIT_seqnum'] = '0';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $mapid = ClxCommon::$db->getField($this->tablename, 'MAPID', $fieldname . '=' . singleQuote($fieldvalue));
        $EBIC = &getTool('EBIC');
        $EBIC->versionIncrease('ID', $mapid);
    }

    /**
     * Performs translation of a value
     */
    function translate($mapid, &$value, $direction = CLX_NUM_ZERO) {
        $sql = 'SELECT  Z.REPLACE_VALUE FROM  CLX_EB_SOCKET_TRANSLATIONS Z WHERE  Z.MAPID = ' . $mapid . '  AND Z.ORIGINAL_VALUE = ' . singleQuote($db->addslashes($value)) . '  UNION ALL SELECT  Z.REPLACE_VALUE FROM CLX_EB_SOCKET_TRANSLATIONS X, CLX_SYS_TRANSL_SETS Y, CLX_SYS_TRANSL_ITEMS Z WHERE  Y.IDCODE = Z.TRANSL_CODE AND X.TRANSL_CODE = Y.IDCODE AND Z.ORIGINAL_VALUE = ' . singleQuote($db->addslashes($value)) . '  AND X.MAPID = ' . $mapid . ' AND Z.DIRECTION = ' . $direction;
        $translation = ClxCommon::$db->getOne($sql);
        if (strlen($translation) > CLX_NUM_ZERO) {
            $value = $translation;
        }
    }

}
