<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-06-21 08:03:11 -0400 (Thu, 21 Jun 2018) $
 * @version        $Rev: 9841 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class package_contents extends clx_tool {

    // CONSTRUCTOR //
    var $other_actions = array();

    function __construct() {
        $this->code = 'PACC';
        $this->name = 'Package Contents';
        $this->system = 'System';
        $this->version = '5.6.0.0';
        $this->accesskey = 'PACK';



        $this->tablename = 'CLX_PACKAGE_CONTENTS';
        $this->record_name_singular = 'package content';
        $this->record_name_plural = 'package contents';
        $this->primary_field = 'PACKAGECODE';
        $this->secondary_field = 'TOOLCODE';
        $this->seqnum_field = 'SEQNUM';
        $this->other_actions = array('OTH_SQL' => 'GEN_LBL_STATEMENT', 'OTH_AUTO' => 'GEN_LBL_AUDIT_TOOLS',
            'OTH_AUTB' => 'GEN_LBL_AUDIT_TABLES',
            'OTH_AUPR' => 'GEN_LBL_AUDIT_PROCEDURE',
            'OTH_AUAL' => 'GEN_LBL_AUDIT_ALL',
            'OTH_NOTE' => 'GEN_LBL_NOTE_OR_DESC');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['PACKAGECODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TOOLCODE'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $fields['RECORDID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TABLENAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['PRIMARY_FIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SECONDARY_FIELD'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        //$fields['PRI_FIELD_VALUE'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        //$fields['SEC_FIELD_VALUE'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['PRI_FIELD_VALUE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['SEC_FIELD_VALUE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['RECORD_NAME_SINGULAR'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['RECORD_NAME_PLURAL'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '', 'null' => 'YES');
        $fields['NOT_TOOL_ACTION'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO, 'default_raw' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO, 'default_raw' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'PACKAGECODE,TOOLCODE,RECORDID');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

//-- P A C K A G E   C O N T E N T S
    function action($ac, $idlist) {
        $msg = array();
        $contents = '';
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'MODIFY':
                    if ($this->getAccess()->canDisplay()) {
                        //same as CTOOL but adding tool code from selection
                        UserRequest::$post['PACC_toolcode'] = ClxCommon::$db->getField($this->tablename, 'TOOLCODE', 'ID IN (' . toString($idlist) . ')');
                        $record = $this->defaultRecord();
                        $contents = $this->editForm('CREATE', array(), $record, $msg);
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGE_CONTENTS-CAN_APPEND_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'CTOOL':
                    if ($this->getAccess()->canDisplay()) {
                        if (!empty(UserRequest::$post['PACC_toolcode']) && isset(UserRequest::$post['confirm'])) {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm('CREATE', array(), $record, $msg);
                        } else {
                            $contents = $this->chooseToolForm();
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGE_CONTENTS-CAN_APPEND_1][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'CREATE':
                    if ($this->getAccess()->canDisplay()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            $this->createRecord($idlist, false);
                            $info = popHistory();
                            $contents = executeHistory($info);
                        } else {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm($ac, array(), $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGE_CONTENTS-CAN_APPEND_2][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGE_CONTENTS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($packagecode = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'PACCPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('packagecode', $packagecode);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = 'CLX_PACKAGE_CONTENTS';
            $whereclause = 'PACKAGECODE=' . singleQuote($packagecode);
            $sortclause = 'SEQNUM, TOOLCODE';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_TOOL'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_MODULECODE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                if ($this->if_other_type($record['TOOLCODE'])) {
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['TOOLCODE'], '<b>' . dict($record['RECORD_NAME_SINGULAR']) . ':</b><pre>' . wordwrap($record['DESCRIPTION'], CLX_NUM_120) . '</pre>', dict('GEN_LBL_NONE')));
                } else {
                    $info = ClxCommon::$db->getRecord($record['TABLENAME'], 'ID=' . $record['RECORDID']);
                    $label = $info[uppercase($record['SECONDARY_FIELD'])];
                    if ($record['TOOLCODE'] === 'UIMI') {
                        $label = dict($label);
                    }
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['TOOLCODE'], '<b>' . ucfirst($record['RECORD_NAME_SINGULAR']) . ':</b><br> ' . $info[uppercase($record['PRIMARY_FIELD'])] . ' ' . $label, (isset($info['MODULECODE'])) ? $info['MODULECODE'] : dict('GEN_LBL_NONE')));
                }
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_OBJECT')), 'CTOOL', $form->getProp(CLX_PROP_ID));
                $b->setProp(CLX_PROP_ID, 'ON');
                $buttons .= $b->source();
            }
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
            }
            if (($page->getProp('qty') > CLX_NUM_ONE) and ( $this->getAccess()->canModify() )) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_PACC_TITLE_LIST'), 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function chooseToolForm() {
        $title = dict('TOOL_PACC_TITLE_CHOOSETOOL');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'CTOOL');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $separator = str_repeat('-', CLX_NUM_60);
        $pop = new HTMLSelect('PACC_toolcode');
        $pop->addItem('', '', false);
        $pop->addItem($separator, '', false);
        $pop->addItem(dict('GEN_LBL_TOOL_COMPONENTS'), '', false);
        $pop->addItem($separator, '', false);
        $tools = ClxCommon::$db->getRecords('CLX_TOOLS', 'HAS_PACKAGE=1 ORDER BY IDNAME');
        while (($tool = ClxCommon::$db->getNextRecord($tools))) {
            $pop->addItem($tool['IDNAME'] . ' (' . $tool['IDCODE'] . ')', $tool['IDCODE'], false);
        }
        //Add extra elements
        $pop->addItem($separator, '', false);
        $pop->addItem(dict('GEN_LBL_OTHER_ACTIONS'), '', false);
        $pop->addItem($separator, '', false);
        foreach ($this->other_actions as $key => $val) {
            $pop->addItem(dict($val), $key, false);
        }
        $table->addData(dict('GEN_LBL_TOOL') . ':', $pop->source());
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_SELECT'), 'CTOOL', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $form->addContents($table->source());
        return $form->source();
    }

    function editForm($ac, $idlist, $record, $msg) {
        $toolcode = $record['PACC_toolcode'];
        $idlist = toString(getIdlist());
        if (!empty($idlist)) {
            $rec = $this->loadToolRecord($idlist);
            if ($this->if_other_type($toolcode)) {
                $record['PACC_description'] = $rec['DESCRIPTION'];
                $record['PACC_id'] = $rec['ID'];
            }
            $record['PACC_seqnum'] = $rec['SEQNUM'];
        }
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('PACC_id', $record['PACC_id']);
        $form->addArgument('PACC_seqnum', $record['PACC_seqnum']);
        $form->addArgument('PACC_packagecode', $record['PACC_packagecode']);
        $form->addArgument('PACC_toolcode', $record['PACC_toolcode']);
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACC_TITLE_CREATE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $table->addData(dict('GEN_LBL_PACKAGECODE') . ':', $record['PACC_packagecode']);
        if ($this->if_other_type(UserRequest::$post['PACC_toolcode'])) {
            $this->editFormOther($form, $table, $record);
        } else {
            $this->editFormTools($form, $table, $record);
        }
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'CREATE', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Build form contents for packages of tools
     *
     * @param type $form
     * @param type $table
     * @param type $record
     */
    private function editFormTools(&$form, &$table, &$record) {
        $table->addData(dict('GEN_LBL_TOOLCODE') . ':', $record['PACC_toolcode']);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACC_TITLE_OBJECTS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $packcode = $record['PACC_packagecode'];
        $toolcode = $record['PACC_toolcode'];
        $is_menu_tool = ($toolcode === 'UIMI') ? true : false;
        $tool = &getTool($record['PACC_toolcode']);
        //En caso de ser tool
        $tstructure = $tool->toolTableStructure();
        $has_module_code = (isset($tstructure['fields']['MODULECODE'])) ? true : false;
        if (!empty($is_menu_tool)) {
            $table->addCell(ucfirst($tool->record_name_plural), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => CLX_NUM_THREE));
            $sql = 'SELECT * FROM ' . $tool->tablename . ' WHERE PARENT = ' . singleQuote(CLX_NUM_ZERO) . ' ORDER BY ' . $tool->seqnum_field . ' ASC';
        } else {
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict($tool->record_name_plural),dict('GEN_LBL_MDATE')));
            if (!empty($tool->seqnum_field)) {
                $sql = 'SELECT * FROM ' . $tool->tablename . ' ORDER BY ' . $tool->seqnum_field . ',' . $tool->primary_field . ',' . $tool->secondary_field;
            } else {
                $sql = 'SELECT * FROM ' . $tool->tablename . ' ORDER BY ' . $tool->primary_field . ',' . $tool->secondary_field;
            }
            if ($has_module_code === true)
                $sql = str_replace('ORDER BY', 'ORDER BY MODULECODE, ', $sql);
        }
        $objects = ClxCommon::$db->executeQuery($sql);
        $i = CLX_INIT_ZERO;
        $last_module = '';
        while (($object = ClxCommon::$db->getNextRecord($objects))) {
            if (!empty($is_menu_tool)) {
                $this->listMenuItems($table, $object, $packcode, $toolcode, CLX_NUM_ZERO, CLX_NUM_ZERO);
            } else {
                if (isset($object['MODULECODE']) && $last_module <> $object['MODULECODE']) {
                    $last_module = $object['MODULECODE'];
                    $table->addCell('<b>' . uppercase($last_module . ' - ' . ClxCommon::$db->getField('CLX_DB_MODULES', 'IDNAME', 'IDCODE = ' . singleQuote($last_module))) . '</b>', array(CLX_PROP_CLASS => 'caption', CLX_PROP_COLSPAN => CLX_NUM_THREE));
                }
                $recordid = $object['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                if (ClxCommon::$db->oneExistsRecords($this->tablename, 'PACKAGECODE=' . singleQuote($packcode) . ' AND TOOLCODE=' . singleQuote($toolcode) . ' AND RECORDID=' . $recordid) > CLX_NUM_ZERO) {
                    $select->setProp('checked', '');
                }
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $lbl = '[';
                if (isset($tstructure['fields'][$tool->primary_field]['dict']) && $tstructure['fields'][$tool->primary_field]['dict'])
                    $lbl .= dict($object[$tool->primary_field]);
                else
                    $lbl .= $object[$tool->primary_field];
                $lbl .= '] -';
                if (isset($tstructure['fields'][$tool->secondary_field]['dict']) && $tstructure['fields'][$tool->secondary_field]['dict'])
                    $lbl .= dict($object[$tool->secondary_field]);
                else
                    $lbl .= $object[$tool->secondary_field];
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($lbl,SUBSTR($object['MDATE'],0,10)));
                $i++;
            }
        }
        $form->addContents($table->source());
    }

    /**
     *
     * @param type $form
     * @param type $table
     * @param type $record
     */
    private function editFormOther(&$form, &$table, &$record) {
        $toolcode = $record['PACC_toolcode'];
        $table->addData(dict('GEN_LBL_OTHER_ACTIONS') . ':', dict($this->other_actions[$toolcode]));
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACC_TITLE_OBJECTS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $msg = array();
        $table->addText($msg, dict($this->other_actions[$toolcode])
                , 'PACC_description', $record['PACC_description'], CLX_NUM_30, CLX_NUM_HUNDRED, true);
        $form->addContents($table->source());
    }

    function confirmDeleteFill(&$table, $record) {
        if ($this->if_other_type($record['TOOLCODE'])) {
            $table->addData(dict('GEN_LBL_OBJECT') . ':', $record['TOOLCODE'] . ': ' . dict($this->other_actions[$record['TOOLCODE']]));
        } else {
            $info = ClxCommon::$db->getRecord($record['TABLENAME'], 'ID=' . $record['RECORDID']);
            $table->addData(dict('GEN_LBL_OBJECT') . ':', ucfirst($record['RECORD_NAME_SINGULAR']) . ': ' . $info[$record['PRIMARY_FIELD']] . ' ' . $info[$record['SECONDARY_FIELD']]);
        }
    }

    /**
     * Returns true if object is of type other.
     * @param string $toolcode
     */
    private function if_other_type($toolcode) {
        return (substr($toolcode, CLX_NUM_ZERO, CLX_NUM_FOUR) === 'OTH_') ? true : false;
    }

    /**
     * Loads record
     * @param int $recordid
     * @return array
     */
    function loadToolRecord($recordid) {
        return ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
    }

    function createRecord($idlist = null) {
        ClxCommon::$db->deleteRecords($this->tablename, 'PACKAGECODE=' . singleQuote(UserRequest::$post['PACC_packagecode']) . ' AND TOOLCODE=' . singleQuote(UserRequest::$post['PACC_toolcode']));
        $record = array();
        $record['PACKAGECODE'] = UserRequest::$post['PACC_packagecode'];
        $record['TOOLCODE'] = UserRequest::$post['PACC_toolcode'];
        if ($this->if_other_type(UserRequest::$post['PACC_toolcode'])) {
            //other elements
            $record['NOT_TOOL_ACTION'] = CLX_NUM_ONE;
            $record['DESCRIPTION'] = UserRequest::$post['PACC_description'];
            $record['TABLENAME'] = 'NA';
            $record['PRIMARY_FIELD'] = 'NA';
            $record['SECONDARY_FIELD'] = 'NA';
            $record['RECORD_NAME_SINGULAR'] = $this->other_actions[UserRequest::$post['PACC_toolcode']];
            $record['RECORD_NAME_PLURAL'] = $record['RECORD_NAME_SINGULAR'];
            $record['SEQNUM'] = (!empty(UserRequest::$post['PACC_seqnum'])) ? UserRequest::$post['PACC_seqnum'] : ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'PACKAGECODE=' . singleQuote(UserRequest::$post['PACC_packagecode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
            ClxCommon::$db->createRecord($this->tablename, $record, false);
        } else {
            //Packages
            $tool = &getTool(UserRequest::$post['PACC_toolcode']);
            $record['TABLENAME'] = $tool->tablename;
            $record['PRIMARY_FIELD'] = $tool->primary_field;
            $record['SECONDARY_FIELD'] = $tool->secondary_field;
            $record['RECORD_NAME_SINGULAR'] = $tool->record_name_singular;
            $record['RECORD_NAME_PLURAL'] = $tool->record_name_plural;
            $record['NOT_TOOL_ACTION'] = CLX_INIT_ZERO;
            foreach ($idlist as $id) {
                $record['RECORDID'] = $id;
                $record['SEQNUM'] = (!empty(UserRequest::$post['PACC_seqnum'])) ? UserRequest::$post['PACC_seqnum'] : ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'PACKAGECODE=' . singleQuote(UserRequest::$post['PACC_packagecode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;

                if (!empty($tool->primary_field)) {
                    $record['PRI_FIELD_VALUE'] = ClxCommon::$db->getField($tool->tablename, $tool->primary_field, 'ID = ' . $id);
                }
                if (!empty($tool->secondary_field)) {
                    $record['SEC_FIELD_VALUE'] = ClxCommon::$db->getField($tool->tablename, $tool->secondary_field, 'ID = ' . $id);
                }
                ClxCommon::$db->createRecord($this->tablename, $record, false);
            }
        }
        $PACK = &getTool('PACK');
        $PACK->versionIncrease('IDCODE', $record['PACKAGECODE']);
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $PACK = &getTool('PACK');
        $PACK->versionIncrease('IDCODE', $record['PACKAGECODE']);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['PACC_id'] = CLX_INIT_ZERO;
        $record['PACC_packagecode'] = $callingpage['pc'];
        $record['PACC_toolcode'] = UserRequest::$post['PACC_toolcode'];
        $record['PACC_not_tool_action'] = CLX_INIT_ZERO;
        $record['PACC_description'] = '';
        $record['PACC_seqnum'] = CLX_INIT_ZERO;
        return $record;
    }

    //To be reviewed
    function listMenuItems(&$table, $menurec, $packcode, $toolcode, $level, $parent) {
        $menuid = $menurec['IDCODE'];
        $qty = ClxCommon::$db->oneExistsRecords('CLX_UI_MENUITEMS', 'PARENT=' . singleQuote($menurec['IDCODE']));
        $tabs = '';
        $i = CLX_INIT_ZERO;
        $name = $level . ' ' . $menurec['IDCODE'] . ' - ' . dict($menurec['NAME']);
        $tabs .= str_repeat(CLX_CHAR_HTML_NBSP, $level);
        $select = new htmlCheckbox('selector_' . $menurec['ID'], false);
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'PACKAGECODE=' . singleQuote($packcode) . ' AND TOOLCODE=' . singleQuote($toolcode) . ' AND RECORDID=' . $menurec['ID']) > CLX_NUM_ZERO) {
            $select->setProp('checked', '');
        }
        $table->addCells(CLX_CLASS_DATA, array($tabs . $select->source() . CLX_CHAR_HTML_NBSP . $name, '',''));
        $i++;
        if ($qty > CLX_NUM_ZERO) {
            $sonstable = new htmlTable(CLX_WIDTH_ONE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $sons = ClxCommon::$db->getRecords('CLX_UI_MENUITEMS', 'PARENT=' . singleQuote($menuid) . ' ORDER BY SEQNUM');
            while (($son = ClxCommon::$db->getNextRecord($sons))) {
                if ($son['PARENT'] <> $menurec['PARENT'])
                    $level++;
                $this->listMenuItems($sonstable, $son, $packcode, $toolcode, $level, $menuid);
            }
            $table->addCell($sonstable->source(), array(CLX_PROP_COLSPAN => CLX_NUM_THREE, CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'PACKAGECODE=' . singleQuote(UserRequest::$post['packagecode']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
            $list[$record['ID']] = $record['TOOLCODE'] . ': ' . dict($record['RECORD_NAME_SINGULAR']);
        }
        return $list;
    }

    // package other action method
    /**
     * Method called by import
     * @param type $action_code
     * @param type $contents
     * @return type
     */
    public function importRunOtherAction($action_code, $contents) {
        $PACC = &getTool('PACC');
        if (!array_key_exists($action_code, $PACC->other_actions))
            return;
        $callback = 'otherAction_' . $action_code;
        if (method_exists($this, $callback)) {
            $this->$callback($contents);
        }
    }

    /**
     * Called on importRun according to type of action
     * @param type $contents
     */
    private function otherAction_OTH_SQL($contents){
        global $db;
        $sqls = explode(";",$contents);
        foreach($sqls as $sql){
	    if (!empty($sql)) {
		$db->executeQuery($sql);
		if($db->errorNum ()!=0){
		    $contents = dict("GEN_LBL_ERROR").$this->errorMsg ().CLX_CHAR_NL.$sql;
		    $LOGR = &getTool('LOGR');
		    $LOGR->appendLog(CLX_PACK_RUN_ID, $contents);
		    $LOGR->setLogStatys(CLX_PACK_RUN_ID, 'ERR');
		    exit();
		}
	    }	
        }
    }


    /**
     * Called on importRun according to type of action
     * @param type $contents
     */
    function otherAction_OTH_AUTO($contents) {
        $DBAU = getToolObject('UTIL', 'DBAU');
        $fix = true;
        $fd = null;
        $fields_only = false;
        $indexes_only = false;
        $contents = $DBAU->checkTools($fix, $fd, $fields_only, $indexes_only);
        $LOGR = &getTool('LOGR');
        $LOGR->appendLog(CLX_PACK_RUN_ID, $contents);
    }

    /**
     * Called on importRun according to type of action
     * @param type $contents
     */
    function otherAction_OTH_AUTB($contents) {
        $DBAU = getToolObject('UTIL', 'DBAU');
        $fix = true;
        $fd = null;
        $fields_only = false;
        $indexes_only = false;
        $drop_fields = false;
        $contents = $DBAU->checkTables($fix, $fd, $drop_fields, $fields_only, $indexes_only);
        $LOGR = &getTool('LOGR');
        $LOGR->appendLog(CLX_PACK_RUN_ID, $contents);
    }

    /**
     * Called on importRun according to type of action
     * @param type $contents
     */
    function otherAction_OTH_AUPR($contents) {
        $DBAU = getToolObject('UTIL', 'DBAU');
        $fix = true;
        $fd = null;
        $return = $DBAU->checkProceduresFunctions($fix, $fd);
        $LOGR = &getTool('LOGR');
        $LOGR->appendLog(CLX_PACK_RUN_ID, $return);
    }

    /**
     * Called on importRun according to type of action
     * @param type $contents
     */
    function otherAction_OTH_AUAL($contents) {
        $DBAU = getToolObject('UTIL', 'DBAU');
        $fix = true;
        $fd = null;
        $fields_only = false;
        $indexes_only = false;
        $drop_fields = false;
        $contents = $DBAU->checkTools($fix, $fd, $fields_only, $indexes_only);
        $contents .= $DBAU->checkTables($fix, $fd, $drop_fields, $fields_only, $indexes_only);
        $contents .= $DBAU->checkProceduresFunctions($fix, $fd);
        $LOGR = &getTool('LOGR');
        $LOGR->appendLog(CLX_PACK_RUN_ID, $contents);
    }

}
