<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class packages extends clx_tool {

    // CONSTRUCTOR //
    var $dump_import_sql_script = false;

    function __construct() {
        $this->code = 'PACK';
        $this->name = 'Package Manager';
        $this->system = 'System';
        $this->version = '5.6.1.1';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_PACKAGES';
        $this->record_name_singular = 'package';
        $this->record_name_plural = 'packages';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_PACKAGES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields ['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields ['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields ['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields ['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields ['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['USERNAME'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields ['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes [$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes [$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes [$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    // P A C K A G E S
    function action($ac, $idlist) {
        $msg = array();
        $record = array();
        if ($this->getAccess()->canRead() === true) {
            switch ($ac) {
                case 'EXPORT' :
                    if (isset(UserRequest::$post ['confirm'])) {
                        $bool = $this->checkExport($idlist);
                        if ($bool === true) {
                            $contents = $this->export($idlist);
                        } else {
                            $contents = $bool;
                        }
                    } else {
                        $contents = $this->exportForm($idlist);
                    }
                    break;
                case 'CONTENTS' :
                    if (isset(UserRequest::$post ['confirm'])) {
                        if ($this->checkImportForm($msg)) {
                            $contents = $this->contents();
                        } else {
                            $contents = $this->importForm(UserRequest::$post, $msg);
                        }
                    } else {
                        $contents = $this->importForm($record, $msg);
                    }
                    break;
                case 'IMPORT' :
                    if ($this->getAccess()->canDisplay()) {
                        $contents = $this->import();
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGES-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                default :
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [PACKAGES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'PACC', 'SELECT * FROM CLX_PACKAGE_CONTENTS WHERE PACKAGECODE=' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {

        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $recid = $recid = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
        ClxCommon::$db->deleteRecords('CLX_PACKAGE_CONTENTS', 'PACKAGECODE=' . singleQuote($record['IDCODE']));
		if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = getTool($toolcode);
                $packitem = $tool->unpackRecord($o);
                unset($packitem['ID']);
                ClxCommon::$db->createRecordVoid('CLX_PACKAGE_CONTENTS', $packitem, false);
            }
        }

        //Update local version ids if pri field is found.
        $sql = 'SELECT DISTINCT TOOLCODE, TABLENAME, PRIMARY_FIELD FROM CLX_PACKAGE_CONTENTS WHERE PACKAGECODE = ' . singleQuote($record['IDCODE']) . ' AND TABLENAME IS NOT NULL AND PRI_FIELD_VALUE<>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND PRI_FIELD_VALUE IS NOT NULL';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs)) {
			$sql='UPDATE CLX_PACKAGE_CONTENTS PC SET PC.RECORDID = 100000000-PC.RECORDID WHERE PC.PACKAGECODE = ' . singleQuote($record['IDCODE']) . ' AND PC.TABLENAME = ' . singleQuote($r['TABLENAME']) . ' AND PC.PRI_FIELD_VALUE<>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND PC.PRI_FIELD_VALUE IS NOT NULL';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE CLX_PACKAGE_CONTENTS PC SET PC.RECORDID = (SELECT MAX(TB.ID) FROM ' . $r['TABLENAME'] . ' TB WHERE TB.' . $r['PRIMARY_FIELD'] . ' = PC.PRI_FIELD_VALUE) WHERE PC.PACKAGECODE = ' . singleQuote($record['IDCODE']) . ' AND PC.TABLENAME = ' . singleQuote($r['TABLENAME']) . ' AND PC.PRI_FIELD_VALUE<>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND PC.PRI_FIELD_VALUE IS NOT NULL';
            ClxCommon::$db->executeQuery($sql);
        }
    }

    /**
     * Build Listing
     * @param type $listingcode
     * @return type
     */
    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());

        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'PACKList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_PACKAGES';
        $whereclause = CLX_CHAR_EMPTY_STRING;
        $sortclause = CLX_CHAR_EMPTY_STRING;
        $page = new xPaging ();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, CLX_CHAR_EMPTY_STRING);
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record ['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ')');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record ['IDCODE'], $record ['IDNAME'], $record ['VERSION'] . '-' . $record ['VERSION_COUNTER'], xFormat::text($record ['DESCRIPTION'])));
            $i++;
        }

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->isAdmin()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'CONTENTS', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_PACKAGE')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_PACK_TITLE_MODIFY') : dict('TOOL_PACK_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('PACK_id', $record ['PACK_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $form->addArgument('PACK_idcode', $record ['PACK_idcode']);
            $table->addData(dict('GEN_LBL_CODE') . ':', $record ['PACK_idcode']);
        } else {
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'PACK_idcode', $record ['PACK_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'PACK_idname', $record ['PACK_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'PACK_version', $record ['PACK_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'PACK_description', $record ['PACK_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel ();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        if (empty($msg))
            $msg = array();
        $action = UserRequest::$post ['ac'];
        if ($action === 'CREATE') {
            xValidate::required($msg, 'PACK_idcode', UserRequest::$post ['PACK_idcode']);
            xValidate::unique($msg, 'PACK_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post ['PACK_idcode']));
        }
        if (strlen(UserRequest::$post ['PACK_idname']) == CLX_NUM_ZERO) {
            $msg ['PACK_idname'] = 'You have to fill this field';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function exportForm($idlist) {
        $msg = array();
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'EXPORT');
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_EXPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $packid = $idlist [CLX_NUM_ZERO];
        $packrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $packid);
        $table->addData(dict('GEN_LBL_CODE') . ':', $packrec ['IDCODE']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $packrec ['IDNAME']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $packrec ['VERSION']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($packrec ['DESCRIPTION']));
        $method = new HTMLSelect('PACK_method');
        $method->addItem(dict('GEN_LBL_PACKAGE'), 'ZIP', true);
        $method->addItem(dict('GEN_LBL_FILE'), 'FILE', false);
        $method->addItem(dict('GEN_LBL_WINDOW'), 'wind', false);
        $method->addItem(dict('GEN_LBL_MAIL'), 'MAIL', false);
        $table->addData(dict('GEN_LBL_METHOD'), $method->source());
        $record ['PACK_target'] = uppercase(str_replace(' ', '', $packrec ['IDCODE'] . date('_Ymd')));
        $table->addField($msg, dict('GEN_LBL_TARGET') . ':', 'PACK_target', $record ['PACK_target'], CLX_NUM_48, CLX_NUM_255);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel ();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Builds Import Form
     *
     * @param array $record
     * @param array $msg
     * @return string
     */
    function importForm($record, $msg) {
        $idlist = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'CONTENTS');
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->setProp('enctype', 'multipart/form-data');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_IMPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(dict('TOOL_PACK_PROMPT_IMPORTEXPLAIN'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('PACK_localfile', $record))
            $record ['PACK_localfile'] = '';
        if (!array_key_exists('PACK_uploadfile', $record))
            $record ['PACK_uploadfile'] = '';
        $table->addField($msg, dict('TOOL_PACK_PROMPT_FILEONSERVER') . ':', 'PACK_localfile', $record ['PACK_localfile'], CLX_NUM_48, CLX_NUM_255);
        $table->addFile($msg, dict('TOOL_PACK_PROMPT_UPLOADFILE') . ':', 'PACK_uploadfile', $record ['PACK_uploadfile']);
        // $table->addYesNo(dict('TOOL_PACK_VERSION_OVERRIDE'),
        $table->addYesNo(dict('GEN_LBL_REPLACE_LOCAL_FILE'), 'replace_local_file', CLX_NUM_ZERO, CLX_NUM_ONE);
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'CONTENTS', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel ();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Validate Import form
     *
     * @param array $msg
     * @return boolean
     */
    function checkImportForm(&$msg) {
        $msg = array();
        if (empty(UserRequest::$post ['PACK_localfile'])) {
            if (UserRequest::$files ['PACK_uploadfile'] ['error']) {
                $msg ['PACK_uploadfile'] = dict('ERR_PACK_FILETOOBIG');
            }
        } elseif (!file_exists(UserRequest::$post ['PACK_localfile'])) {
            if (strpos(UserRequest::$post ['PACK_localfile'], 'http') === false) {
                $msg ['PACK_localfile'] = dict('GEN_ERR_FILE_NOT_FOUND');
            } else {

                $fd = fopen(UserRequest::$post ['PACK_localfile'], 'r');
                if (empty($fd)) {
                    $msg ['PACK_localfile'] = dict('GEN_ERR_FILE_NOT_FOUND');
                }
            }
        }

        return (count($msg) == CLX_NUM_ZERO) ? true : false;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::confirmDeleteFill()
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record ['PACK_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record ['PACK_idname']);
    }

    function viewFormFill($record) {
        addHistory('pc', $record ['PACK_idcode']);
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_VIEW') . ': ' . $record ['PACK_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record ['PACK_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record ['PACK_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record ['PACK_version'] . '-' . $record ['PACK_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record ['PACK_description']));
        $contents .= $table->source();
        $PACC = &getTool('PACC');
        $contents .= $PACC->makepanel($record ['PACK_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record ['PACK_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record ['PACK_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record ['PACK_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('PACK_idcode', UserRequest::$post))
            UserRequest::$post ['PACK_idcode'] = '';
        if (!array_key_exists('PACK_idname', UserRequest::$post))
            UserRequest::$post ['PACK_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'PACK_idcode', UserRequest::$post ['PACK_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'PACK_idname', UserRequest::$post ['PACK_idname'], CLX_NUM_32, CLX_NUM_64);
        $sort = new HtmlSelect('PACK_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Code', 'idcode', false);
        $sort->addItem('Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post ['PACK_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['PACK_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['PACK_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post ['PACK_idname'])) . '%');
            $boolean = 'AND';
        }
        $_SESSION ['paging'] ['PACKList'] ['sortby'] = UserRequest::$post ['PACK_sort'];
        $_SESSION ['paging'] ['PACKList'] ['criteria'] = $filter;
        $_SESSION ['paging'] ['PACKList'] ['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
   /* function clearfilter() {
        $_SESSION ['paging'] ['PACKList'] ['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record ['PACK_id'] = $recordid;
        $record ['PACK_idcode'] = $values ['IDCODE'];
        $record ['PACK_idname'] = $values ['IDNAME'];
        $record ['PACK_version'] = $values ['VERSION'];
        $record ['PACK_version_counter'] = $values ['VERSION_COUNTER'];
        $record ['PACK_cdate'] = xFormat::date($values ['CDATE']);
        $record ['PACK_mdate'] = xFormat::date($values ['MDATE']);
        $record ['PACK_username'] = $values ['USERNAME'];
        $record ['PACK_description'] = $values ['DESCRIPTION'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record ['IDNAME'] = UserRequest::$post ['PACK_idname'];
        $record ['VERSION'] = UserRequest::$post ['PACK_version'];
        $record ['MDATE'] = $d;
        $record ['USERNAME'] = $_SESSION ['user_record'] ['username'];
        $record ['DESCRIPTION'] = UserRequest::$post ['PACK_description'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post ['PACK_id']);
        $this->versionIncrease(CLX_PROP_ID, UserRequest::$post ['PACK_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record ['IDCODE'] = UserRequest::$post ['PACK_idcode'];
        $record ['IDNAME'] = UserRequest::$post ['PACK_idname'];
        $record ['VERSION'] = UserRequest::$post ['PACK_version'];
        $record ['VERSION_COUNTER'] = '0';
        $record ['CDATE'] = $d;
        $record ['MDATE'] = $d;
        $record ['USERNAME'] = $_SESSION ['user_record'] ['username'];
        $record ['DESCRIPTION'] = UserRequest::$post ['PACK_description'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $packrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $packcode = $packrec ['IDCODE'];
        ClxCommon::$db->deleteRecords('CLX_PACKAGE_CONTENTS', 'PACKAGECODE=' . singleQuote($packcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record ['PACK_id'] = '';
        $record ['PACK_idcode'] = '';
        $record ['PACK_idname'] = '';
        $record ['PACK_version'] = '1.0.0';
        $record ['PACK_cdate'] = '';
        $record ['PACK_mdate'] = '';
        $record ['PACK_username'] = $_SESSION ['user_record'] ['username'];
        $record ['PACK_description'] = '';
        return $record;
    }

    /**
     *
     * @param string $idlist
     * @return string
     */
    function export($idlist) {

        UserResponse::setRunTimeLimit(CLX_NUM_1800);
        $method = UserRequest::$request ['PACK_method'];
        $target = kPathToPackages . UserRequest::$request ['PACK_target'];
        $packid = $idlist [CLX_NUM_ZERO];
        $packrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $packid);
        $packcode = $packrec ['IDCODE'];
        if ($method === 'ZIP') {
            $target = $target . '.zip';
            if (!class_exists('clx_zip')) {
                include_once kPathToWebSite . 'classes/clx_zip.php';
            }

            // create zip file
            $file = new clx_file($target, 'w');
            $file->open();
            $file->close();
            $zip = new clx_zip ();
            $zip->open($target);
            $objects = ClxCommon::$db->getRecords('CLX_PACKAGE_CONTENTS', 'PACKAGECODE = ' . singleQuote($packcode) . '  ORDER BY SEQNUM ASC');
            $i = CLX_INIT_ZERO;
            while (($objectrec = ClxCommon::$db->getNextRecord($objects))) {
                //Get Tool
                if (!empty($objectrec['NOT_TOOL_ACTION'])) {
                    //process other elements in package
                    $package = new xmlObject('PACK');
                    $package->setProp('ID', CLX_NUM_ZERO);
                    $package_objs = new xmlObject('PACK_OBJECTS');
                    $o = new xmlObject($objectrec ['TOOLCODE']);
                    $o->setProp('NOT_TOOL_CODE', CLX_NUM_ONE);
                    $f = new xmlObject('DESCRIPTION');
                    $f->addContents($objectrec['DESCRIPTION']);
                    $o->addObject($f);
                    $package_objs->addObject($o);
                    $package->addObject($package_objs);
                    // Package Name
                    $pn = $objectrec ['SEQNUM'] . '_' . $objectrec ['PACKAGECODE'] . '_' . $objectrec ['TOOLCODE'] . '_' . $objectrec ['ID'] . '.txt';
                    $zip->addFromString($pn, $package->source_(array('level' => CLX_NUM_ZERO)));
                } else {
                    //Procesing elements that are tools
                    $tool = &getTool($objectrec ['TOOLCODE']);
                    $record = ClxCommon::$db->getRecord($objectrec ['TABLENAME'], 'ID=' . $objectrec ['RECORDID']);
				//	pre($record);
                    if ($objectrec ['TOOLCODE'] === 'DBMO') {
                        //Added to split module into parts
                        $tool->pack_split($record, $this, $zip, $objectrec, $packrec, $i);
                    } else {
                        $package = $this->packRecord($packrec);
                        $package_objs = new xmlObject($this->code . '_OBJECTS');
                        $paquete = $tool->pack($record);
                        $package_objs->addObject($paquete);
                        $package->addObject($package_objs);
                        // Package Name
                        $pn = $objectrec ['SEQNUM'] . '_' . $objectrec ['PACKAGECODE'] . '_' . $objectrec ['TOOLCODE'] . '_' . $objectrec ['RECORDID'] . '.xml';
                        $zip->addFromString($pn, $package->source_(array('level' => CLX_NUM_ZERO)));
                        $i++;
                    }
                }
            }
            $zip->setArchiveComment('clx_package.' . $i);
            $zip->close();
            /*
             * GEL_LBL_OPER_COMP_SUCCESS
             * Operation completed succesfully
             * Operaci�n realizada con �xito
             */
            $contents = note(dict('GEL_LBL_OPER_COMP_SUCCESS'), 'OK');
        } else {
            $package = $this->packRecord($packrec);
            $package_objs = new xmlObject($this->code . '_OBJECTS');
            $objects = ClxCommon::$db->getRecords('CLX_PACKAGE_CONTENTS', 'PACKAGECODE = ' . singleQuote($packcode) . ' AND NOT_TOOL_ACTION = 0');
            while (($objectrec = ClxCommon::$db->getNextRecord($objects))) {
                $tool = &getTool($objectrec ['TOOLCODE']);
                $record = ClxCommon::$db->getRecord($objectrec ['TABLENAME'], 'ID=' . $objectrec ['RECORDID']);
                $package_objs->addObject($tool->pack($record));
            }
            $package->addObject($package_objs);
            switch ($method) {
                case 'FILE' :
                    $contents = $this->exportFile($package, $target);
                    break;
                case 'MAIL' :
                    $contents = note(dict('ERR_NOTIMPLEMENTED'));
                    break;
                case 'WIND' :
                    $contents = '<pre>' . htmlspecialchars($package->source()) . '</pre>';
                    break;
                default :
                    break;
            }
        }
        unset($package_objs, $package, $target);
        return $contents;
    }

    /**
     * Export Items
     * @param string $idlist
     * @return string
     */
    function exportByItem() {

        UserResponse::setRunTimeLimit(CLX_NUM_1800);
        $target = kPathToPackages . 'PAQUETE';
        $packid = CLX_NUM_123;
        $packrec = array('ID' => $packid);
        $packcode = 'FS';
        $target = $target . '.zip';
        if (!class_exists('clx_zip')) {
            include_once kPathToWebSite . 'classes/clx_zip.php';
        }

        // create zip file
        $file = new clx_file($target, 'w');
        $file->open();
        $file->close();
        $i = CLX_INIT_ZERO;
        $zip = new clx_zip ();
        $zip->open($target);
        $objects = ClxCommon::$db->getRecords('CLX_PACKAGE_LIST', 'PACKLISTCODE = ' . singleQuote($packcode) . ' ORDER BY SEQNUM ASC');
        while (($objectrec = ClxCommon::$db->getNextRecord($objects))) {
            //Procesing elements that are tools
            $tool = &getTool($objectrec ['TOOLCODE']);
            $record = ClxCommon::$db->getRecord($tool->tablename, $tool->primary_field . '=' . singleQuote($objectrec ['IDCODE']));
            $package = $this->packRecord($packrec);
            $package_objs = new xmlObject($this->code . '_OBJECTS');
            $package_objs->addObject($tool->pack($record));
            $package->addObject($package_objs);
            // Package Name
            $pn = uppercase($objectrec ['PACKLISTCODE'] . '_' . $objectrec ['TOOLCODE'] . '_' . $objectrec ['IDCODE'] . '_' . $i . '.XML');
            $zip->addFromString($pn, $package->source_(array('level' => CLX_NUM_ZERO)));
            $i++;
        }
        $zip->setArchiveComment('clx_package.' . $i);
        $zip->close();
        /*
         * GEL_LBL_OPER_COMP_SUCCESS
         * Operation completed succesfully
         * Operaci�n realizada con �xito
         */
        $contents = note(dict('GEL_LBL_OPER_COMP_SUCCESS'), 'OK');
        unset($package_objs, $package, $target);
        return $contents;
    }

    /**
     * Builds File
     *
     * @param object $o
     * @param string $target
     * @return string
     */
    function exportFile($o, $target) {
        $fd = fopen($target, 'w');
        if (empty($fd)) {
            $contents = alert('[PACK-010] Can' . CLX_CHAR_SINGLE_QUOTE . 't open file: '.$target, dict('GEN_LBL_CONTINUE'));
        } elseif (!fwrite($fd, $o->source_(array('level' => CLX_NUM_ZERO)))) {
            $contents = alert('[PACK-020] Can' . CLX_CHAR_SINGLE_QUOTE . 't write to file: '.$target, dict('GEN_LBL_CONTINUE'));
        } else {
            fclose($fd);
            $contents = note(dict('GEL_LBL_OPER_COMP_SUCCESS'), 'OK');
        }
        return $contents;
    }

    /**
     * Describes package content
     *
     * @return string
     */
    function contents() {
        $contents = '';

        UserResponse::setRunTimeLimit(CLX_NUM_1800);
        $error = false;
        if (empty(UserRequest::$post ['PACK_localfile'])) {
            // $filename = kPathToWebSite . kPathToTemp . 'tmp' . time ( 'U' ) .
            $filename = UserRequest::$files ['PACK_uploadfile'] ['name'];
            $full_filename = kPathToWebSite . kPathToPackages . $filename;
			if (!file_exists(kPathToWebSite . kPathToPackages) || !file_exists(UserRequest::$files ['PACK_uploadfile'] ['tmp_name']) || !move_uploaded_file(UserRequest::$files ['PACK_uploadfile'] ['tmp_name'], $full_filename)) {
                $contents = alert('[PACK-040] Can' . CLX_CHAR_SINGLE_QUOTE . 't move: ' . $filename . ' to ' . $filename, dict('GEN_LBL_CONTINUE'));
                $error = true;
            }
        } else {
            $filename = UserRequest::$post ['PACK_localfile'];
            $file = new clx_file($filename, 'r');
            $fileinfo = $file->filePathInfo();
            $filename = $fileinfo['filename'] . '.' . $fileinfo['extension'];
            $full_filename = kPathToWebSite . kPathToPackages . $filename;
            if (!empty(UserRequest::$post['replace_local_file']) || !file_exists($full_filename)) {
                //copy file to local
                $file->copy(UserRequest::$post ['PACK_localfile'], $full_filename);
            }
            $file->close();
        }
        if (empty($error)) {
            // verify if zip file
            include_once kPathToWebSite . 'classes/clx_zip.php';
            $zip = new clx_zip ();
            if ($zip->open($full_filename) === true) {
                if (strpos($zip->getArchiveComment(), 'clx_package.') !== false) {
                    // Search all xml files and format contents
                    $form = false;
                    $this->contentsFormHeader($filename, $form);
                    for ($i = CLX_INIT_ZERO; $i < $zip->numFiles; $i++) {
                        $xml = $zip->getFromIndex($i);
                        $o = new XMLObject('ROOT');
                        $o->parse($xml);
                        $this->contentsFormItem($o, $form, $filename, ($i == CLX_NUM_ZERO));
                    }
                    $this->contentsFormFooter($form);
                    $contents = $form->source();
                } else {
                    $contents = alert('[PACK-050] Invalid package file', dict('GEN_LBL_CONTINUE'));
                }
                // Procesing zip file
                $zip->close();
            } else {
                $fd = fopen($full_filename, 'r');
                if (empty($fd)) {
                    $contents = alert('[PACK-030] Can' . CLX_CHAR_SINGLE_QUOTE . 't open file: ' . $full_filename, dict('GEN_LBL_CONTINUE'));
                } else {
                    // $xml = fread($fd, filesize($filename));
                    $xml = '';
                    while (!feof($fd)) {
                        $xml .= fread($fd, CLX_NUM_8192);
                    }
                    fclose($fd);
                    $o = new XMLObject('ROOT');
                    $o->parse($xml);
                    $contents = $this->contentsForm($filename, $o);
                }
            }
        }
        unset($filename, $o);

        $_SESSION['CLX_PACK_CONTENTS'] = strip_tags($contents, '<br><br><table><td><tr>');
        return $contents;
    }

    /**
     * Presents contents form
     *
     * @param string $filename
     * @param obj $package
     * @return string
     */
    function contentsFormHeader($filename, &$form) {

        UserResponse::setRunTimeLimit(CLX_NUM_1800);
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'IMPORT');
        $form->addArgument('fn', $filename);
    }

    /**
     * Adds content form footer
     *
     * @param object $form
     */
    function contentsFormFooter(&$form) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'IMPORT', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel ();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
    }

    /**
     * Adds items to package content form
     *
     * @param unknown_type $package
     * @param unknown_type $form
     */
    function contentsFormItem($package, &$form, $filename, $should_packinfo = false) {
        //Pinta el encabezado
        if (!empty($should_packinfo)) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $table->addCell(dict('TOOL_PACK_TITLE_PACKCONTENTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $table->addData(dict('GEN_LBL_FILENAME') . ':', $filename);
            $table->addData(dict('GEN_LBL_CODE') . ':', $package->getSubObjectContents('IDCODE'));
            $table->addData(dict('GEN_LBL_NAME') . ':', $package->getSubObjectContents('IDNAME'));
            $table->addData(dict('GEN_LBL_VERSION') . ':', $package->getSubObjectContents('VERSION') . '-' . $package->getSubObjectContents('VERSION_COUNTER'));
            $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($package->getSubObjectContents('DESCRIPTION')));
            $table->addData(dict('GEN_LBL_CDATE') . ':', xFormat::date($package->getSubObjectContents('CDATE')));
            $table->addData(dict('GEN_LBL_MDATE') . ':', xFormat::date($package->getSubObjectContents('MDATE')));
            $table->addData(dict('GEN_LBL_USERNAME') . ':', $package->getSubObjectContents('USERNAME'));
            $table->addCell(dict('TOOL_PACK_TITLE_PACKDETAIL'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            $form->addContents($table->source());
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $table->addCell(dict('GEN_LBL_STATUS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '25%'));
            $table->addCell(dict('TOOL_PACK_PROMPT_LOCALVERSION'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '25%'));
            $table->addCell(dict('TOOL_PACK_PROMPT_PACKVERSION'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '25%'));
            $table->addCell(dict('GEN_LBL_DESCRIPTION'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '25%'));
            $form->addContents($table->source());
        }
        $PACC = &getTool('PACC');
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        // $table->addCells ( CLX_CLASS_HEADERS, dict ( 'GEN_LBL_STATUS' ), dict (
        // 'TOOL_PACK_PROMPT_LOCALVERSION' ), dict (
        $objs = $package->getObjectXml('PACK_OBJECTS');
        $i = CLX_INIT_ZERO;
        if (is_object($objs) && property_exists($objs, 'objects')) {
            foreach ($objs->objects as $o) {
                if (isset($o->properties['NOT_TOOL_CODE'])) {
                    $table->addCell(dict('GEN_LBL_ACTION') . '<br><b>' . dict($PACC->other_actions[$o->tag]) . '</b>', array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_WIDTH => '25%'));
                    $table->addCell('<pre>' . wordwrap($o->objects[CLX_NUM_ZERO]->contents, CLX_NUM_120) . '</pre>', array(CLX_PROP_CLASS => CLX_CLASS_DATA));
                } else {
                    $toolcode = $o->tag;
                    $tool = &getTool($toolcode);
                    $tool->packcheck($o, $i, $table);
                }
            }
        }
        $form->addContents($table->source());
    }

    /**
     * Presents contents form
     *
     * @param string $filename
     * @param obj $package
     * @return string
     */
    function contentsForm($filename, $package) {

        UserResponse::setRunTimeLimit(CLX_NUM_1800);
        $buttons = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'IMPORT');
        $form->addArgument('fn', $filename);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_PACKCONTENTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FILENAME') . ':', $filename);
        $table->addData(dict('GEN_LBL_CODE') . ':', $package->getSubObjectContents('IDCODE'));
        $table->addData(dict('GEN_LBL_NAME') . ':', $package->getSubObjectContents('IDNAME'));
        $table->addData(dict('GEN_LBL_VERSION') . ':', $package->getSubObjectContents('VERSION') . '-' . $package->getSubObjectContents('VERSION_COUNTER'));
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($package->getSubObjectContents('DESCRIPTION')));
        $table->addData(dict('GEN_LBL_CDATE') . ':', xFormat::date($package->getSubObjectContents('CDATE')));
        $table->addData(dict('GEN_LBL_MDATE') . ':', xFormat::date($package->getSubObjectContents('MDATE')));
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $package->getSubObjectContents('USERNAME'));
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_PACK_TITLE_PACKDETAIL'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_STATUS'), dict('TOOL_PACK_PROMPT_LOCALVERSION'), dict('TOOL_PACK_PROMPT_PACKVERSION'), dict('GEN_LBL_DESCRIPTION')));
        $objs = $package->getObjectXml('PACK_OBJECTS');
        $i = CLX_INIT_ZERO;
        foreach ($objs->objects as $o) {
            $toolcode = $o->tag;
            $tool = &getTool($toolcode);
            $tool->packcheck($o, $i, $table);
        }
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'IMPORT', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Import Form
     *
     * @return string
     */
    function import() {
        $timetolast = CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_60;
        $LOGR = &getTool('LOGR');
        unset($_SESSION['CLX_PACK_RUN_ID']);
        //Insertar entrada en bitacora para monitorear
        $url = '<a href="tool.php?to=LOGR&ac=LIST">' . dict('GEN_LBL_SEE_LOG_HERE') . '</a>';
        $contents = note(dict('GEN_LBL_PACKAGE_IMPORT_WAIT', $url));

        UserResponse::setIgnoreUserAbort(true);
        UserResponse::setRunTimeLimit((CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_60));
        
        if (empty($_SESSION['CLX_PACK_RUN_ID'])) {
            //creando elemento en session y continuando
            $log = array('CDATE' => xFormat::getNow('Y-m-d H:i:s'), 'ACTION' => 'IMPORT', 'TOOLCODE' => $this->code, 'USERNAME' => $_SESSION['user_record']['username'], 'TRANSACTION' => null, 'OWNERCODE' => $this->code, 'STATUS' => 'IDLE', 'PARAMETERS' => '', 'RESULT' => '');
            $_SESSION['CLX_PACK_RUN_ID'] = ClxCommon::$db->createRecord('CLX_LOGGER', $log, false);
            ClxCommon::$db->commit();
            //sendLongProcessWait($contents, $timetolast, 'master.html', true);
        } else {
            //si existe flag en session ya se envio no reprocesar
            //return $contents;
        }
        if (!defined('CLX_PACK_RUN_ID'))
            define('CLX_PACK_RUN_ID', $_SESSION['CLX_PACK_RUN_ID']);
        //done in sendLongProcessWait
        //UserResponse::setRunTimeLimit(1800);
        ClxCommon::$db->updateRecord('CLX_LOGGER', array('STATUS' => 'RUN'), 'STATUS = ' . singleQuote('IDLE') . ' AND ID = ' . CLX_PACK_RUN_ID);
        ClxCommon::$db->commit();
        $filename = UserRequest::$request ['fn'];

        if (!defined('UTIL_DUMP_AUPK_SQL_SCRIPT'))
            define('UTIL_DUMP_AUPK_SQL_SCRIPT', CLX_NUM_ZERO);
        $this->dump_import_sql_script = (UTIL_DUMP_AUPK_SQL_SCRIPT === strval(CLX_NUM_ONE));

        if (!empty($this->dump_import_sql_script)) {
            ClxCommon::$db->scriptFileOut = kPathToTemp . clx_mcript('PACK_IMP_' . $filename) . '.sql';
            ClxCommon::$db->dumpScript = true;
        }
        // verify if zip file
        $full_filename = kPathToWebSite . kPathToPackages . $filename;
        include_once kPathToWebSite . 'classes/clx_zip.php';
        $zip = new clx_zip ();
        if ($zip->open($full_filename) === true) {
            if (strpos($zip->getArchiveComment(), 'clx_package.') !== false) {
                // Search all xml files and format contents
                for ($i = CLX_INIT_ZERO; $i < $zip->numFiles; $i++) {
                    $xml = $zip->getFromIndex($i);
                    //Import package
                    $this->importUnpackXml($xml);
                }
            }
        } else {
            $fd = fopen($full_filename, 'r');
            $xml = '';
            while (!feof($fd)) {
                $xml .= fread($fd, CLX_NUM_8192);
            }
            fclose($fd);
            $this->importUnpackXml($xml);
        }
        if (!empty($this->dump_import_sql_script))
            ClxCommon::$db->dumpScript = false;
        unset($_SESSION['CLX_PACK_RUN_ID']);
        $contents = dict('GEN_LBL_PACKAGE') . ': ' . $filename . '<br>' . dict('GEL_LBL_OPER_COMP_SUCCESS');
        if(!empty($_SESSION['CLX_PACK_CONTENTS'])){
            $contents .= '<br><br>'.$_SESSION['CLX_PACK_CONTENTS'];
            unset($_SESSION['CLX_PACK_CONTENTS']);
        }

        $LOGR->appendLog(CLX_PACK_RUN_ID, $contents);
        ClxCommon::$db->updateRecord('CLX_LOGGER', array('STATUS' => 'END', 'RESULT' => $contents), 'STATUS = ' . singleQuote('RUN') . ' AND ID = ' . CLX_PACK_RUN_ID);
        ClxCommon::$db->commit();
        return note($contents);
    }

    /**
     * Process XML elements
     * @param string $xml
     */
    function importUnpackXml(&$xml) {
        if ($xml === '')
            return;
        $package = new XMLObject('ROOT');
        $package->parse($xml);
        $objs = $package->getObjectXml('PACK_OBJECTS');
        foreach ($objs->objects as $o) {
            if (isset($o->properties['NOT_TOOL_CODE'])) {
                //Importando otras acciones
                $PACC = &getTool('PACC');
                $PACC->importRunOtherAction($o->tag, $o->objects[CLX_NUM_ZERO]->contents);
            } else {
                //Importando paquetes de herramientas
                $toolcode = $o->tag;
                $tool = &getTool($toolcode);
                $tool->unpack(array('package' => $o));
            }
        }
    }

    /**
     * Validates Import Form
     *
     * @param unknown_type $idlist
     * @return string boolean
     */
    function checkExport($idlist) {
        // Get the package's idcode
        $id = $idlist [CLX_NUM_ZERO];
        $sql = 'SELECT IDCODE FROM ' . $this->tablename . ' WHERE ID =' . singleQuote($id);
        $key = ClxCommon::$db->getOneRecord($sql);
        // Get all the objects in the package
        $PACC = &getTool('PACC');
        $sql = 'SELECT TOOLCODE,RECORDID,TABLENAME,RECORD_NAME_SINGULAR FROM ' . $PACC->tablename . ' WHERE PACKAGECODE = ' . singleQuote($key ['IDCODE']) . ' AND NOT_TOOL_ACTION = 0';
        $results = ClxCommon::$db->executeQuery($sql);
        // Create the data structures for the report
        $report = array();
        $counter = CLX_INIT_ZERO;
        // Now we check is those objects exists.
        while (($record = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE))) {
            if (ClxCommon::$db->oneExistsRecords($record ['TABLENAME'], 'ID=' . singleQuote($record ['RECORDID'])) == CLX_NUM_ZERO) {
                // We report the object as missing.
                $report [$counter] = $record;
                $counter++;
            }
        }
        if ($counter != CLX_NUM_ZERO) {
            $table = new htmlTable(CLX_WIDTH_FOUR);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $table->addCell(dict('TOOL_PACK_ERROR'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
            $table->addCell(dict('TOOL_PACK_NO_OBJECTS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TOOLCODE'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_ID'), dict('GEN_LBL_TABLENAME')));
            for ($i = CLX_INIT_ZERO; $i < $counter; $i++) {
                $table->addCells(CLX_CLASS_DATA, array($report [$i] ['TOOLCODE'], $report [$i] ['RECORD_NAME_SINGULAR'], $report [$i] ['RECORDID'], $report [$i] ['TABLENAME']));
            }
            return $table->source();
        } else {
            return true;
        }
    }

}

// End class
