<?php

/**
 * Translation Items Manager
 *
 * @package		Clientix.Tool.System.Translations
 * @abstract		Translation Items from system translations
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-24 13:34:28 -0400 (ju., 24 ene. 2019) $
 * @version    	$Rev: 10499 $
 * CHANGE LOG
 *  Included label
 */
class sys_translation_items extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'TRTI';
        $this->name = 'Translation Items Manager';
        $this->system = 'System';
        $this->version = '5.6.0.0';


        $this->accesskey = 'TRTS';

        $this->tablename = 'CLX_SYS_TRANSL_ITEMS';
        $this->record_name_singular = 'translation';
        $this->record_name_plural = 'translations';
        $this->primary_field = 'TRANSL_CODE';
        $this->secondary_field = 'OLD_VALUE';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_TRTI_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_TRANSL_CODE',
                            'fields' => array(
                                array('name' => 'transl_code', 'type' => 'string', 'readonly' => '1'))
                        ), array('label' => 'GEN_LBL_ORIGINAL_VALUE',
                            'fields' => array(
                                array('name' => 'original_value', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '64'))
                        ), array('label' => 'GEN_LBL_REPLACE_VALUE',
                            'fields' => array(
                                array('name' => 'replace_value', 'type' => 'string', 'default' => '', 'size' => '255', 'displaysize' => '64'))
                        ), array('label' => 'GEN_LBL_CASE_SENSITIVE',
                            'fields' => array(
                                array('name' => 'case_sensitive', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => array('0' => 'GEN_LBL_NO', '1' => 'GEN_LBL_YES')))
                        ), array('label' => 'GEN_LBL_DIRECTION',
                            'fields' => array(
                                array('name' => 'direction', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => array('0' => 'GEN_LBL_IN', '1' => 'GEN_LBL_OUT')))
                        )
                    )
                )
            )
        );
        $this->importform = array(CLX_CLASS_TITLE => 'TOOL_TRTI_TITLE_IMPORT',
            CLX_CLASS_SUBTITLE => 'TOOL_TRTI_PROMPT_IMPORT',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_FILE',
                            'fields' => array(
                                array('name' => 'filename', 'type' => 'file', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        )
                    )
                )
            )
        );
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['TRANSL_CODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['ORIGINAL_VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        //case insensitives search without penalty
        $fields['ORIGINAL_VALUE_CI'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '', 'collate'=>'utf8_general_ci');
        $fields['REPLACE_VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['CASE_SENSITIVE'] = array('type' => 'int(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DIRECTION'] = array('type' => 'int(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'TRANSL_CODE,ORIGINAL_VALUE,DIRECTION');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'TRANSL_CODE,ORIGINAL_VALUE_CI,DIRECTION');
        //CHARSET=utf8 COLLATE=utf8_bin
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB','fields' => $fields,'indexes' => $indexes,'case_sensitive' => true);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $transl_code) = $parameters;
        $record = $this->unpackRecord($package);
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'TRANSL_CODE = ' . singleQuote($record['TRANSL_CODE']) . ' AND ORIGINAL_VALUE = ' . singleQuote($record['ORIGINAL_VALUE']) . ' AND DIRECTION = ' . singleQuote($record['DIRECTION'])) == CLX_NUM_ZERO) {
            ClxCommon::$db->createRecord($this->tablename, $record, false);
        } else {
            unset($record['ID']);
            ClxCommon::$db->updateRecord($this->tablename, $record, 'TRANSL_CODE = ' . singleQuote($record['TRANSL_CODE']) . ' AND ORIGINAL_VALUE = ' . singleQuote($record['ORIGINAL_VALUE']) . ' AND DIRECTION = ' . singleQuote($record['DIRECTION']));
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            $msg = array();
            switch ($ac) {
                case 'IMPORT':
                    if (isset(UserRequest::$post['confirm'])) {
                        if ($this->checkImportForm($msg)) {
                            $contents = $this->import();
                            if (empty($contents)) {
                                $info = popHistory();
                                $contents = executeHistory($info);
                            }
                        } else {
                            $contents = $this->importForm($ac, array(), UserRequest::$post, $msg);
                        }
                    } else {
                        $record = array();
                        $contents = $this->importForm($ac, $idlist, $record, $msg);
                    }
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SYS_TRANSLATION_ITEMS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($transl_code = null) {
        if ($this->getAccess()->canRead()) {
            $t = time();
            $to = $this->code;
            $here = 'tool.php?to=' . $to . '&t=' . $t;
            $direction = array(dict('GEN_LBL_IN'), dict('GEN_LBL_OUT'));
            $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
            $form->setProp(CLX_PROP_ID, 'TRTIPanel');
            $form->addArgument(CLX_PROP_TO, $this->code);
            $form->addArgument('transl_code', $transl_code);
            $form->addArgument('ac', 'NOT_DEFINED');
            $selectclause = '*';
            $fromclause = $this->tablename;
            $whereclause = 'TRANSL_CODE=' . singleQuote($transl_code);
            $sortclause = 'DIRECTION ASC, ORIGINAL_VALUE ASC';
            $page = new xPaging();
            $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
            $page->setProp('printheaders', false);
            $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
            $table = new htmlTable(CLX_WIDTH_FIVE);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $page->header($table);
            $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_DIRECTION'), dict('GEN_LBL_ORIGINAL_VALUE'), dict('GEN_LBL_REPLACE_VALUE'), dict('GEN_LBL_CASE_SENSITIVE')));
            $i = CLX_INIT_ZERO;
            while ($record = $page->nextRecord()) {
                $recordid = $record['ID'];
                $select = new htmlCheckbox('selector_' . $recordid, false);
                $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
                $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
                $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($direction[$record['DIRECTION']], $record['ORIGINAL_VALUE'], $record['REPLACE_VALUE'], dict($this->yesno[$record['CASE_SENSITIVE']])));
                $i++;
            }
            $buttons = '';
            if ($this->getAccess()->canModify()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDelete()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_TRANSLATION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            if ($this->getAccess()->canDisplay()) {
                $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_IMPORT'), array('action' => 'IMPORT', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
            }
            $form->addContents($page->footers($table, $buttons));
            return buildPanelInterface(dict('TOOL_TRTI_TITLE_LIST'), 'EB_SOCKET_TRANSLATIONS_' . $transl_code, $form->source(), CLX_NUM_MINUS_1);
        } else {
            return CLX_CHAR_EMPTY_STRING;
        }
    }

    function importForm($ac, $idlist, $record, $msg) {
        $form = new clx_form($this->code, $this->importform);
        return $form->editForm($ac, $idlist, $record, $msg);
    }

    function checkImportForm(&$msg) {
        $form = new clx_form($this->code, $this->importform);
        $result = $form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg);
        return $result;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_TRANSL_CODE') . ':', $record['TRTI_transl_code']);
        $table->addData(dict('GEN_LBL_ORIGINAL_VALUE') . ':', $record['TRTI_original_value']);
        $table->addData(dict('GEN_LBL_REPLACE_VALUE') . ':', $record['TRTI_replace_value']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['TRTI_id'] = $recordid;
        $record['TRTI_transl_code'] = $values['TRANSL_CODE'];
        $record['TRTI_original_value'] = $values['ORIGINAL_VALUE'];
        $record['TRTI_replace_value'] = $values['REPLACE_VALUE'];
        $record['TRTI_case_sensitive'] = $values['CASE_SENSITIVE'];
        $record['TRTI_direction'] = $values['DIRECTION'];
        $record['TRTI_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['ORIGINAL_VALUE'] = UserRequest::$post['TRTI_original_value'];
        $record['ORIGINAL_VALUE_CI'] = UserRequest::$post['TRTI_original_value'];
        $record['REPLACE_VALUE'] = UserRequest::$post['TRTI_replace_value'];
        $record['CASE_SENSITIVE'] = UserRequest::$post['TRTI_case_sensitive'];
        $record['DIRECTION'] = UserRequest::$post['TRTI_direction'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['TRTI_id']);
        $this->versionIncrease('ID', UserRequest::$post['TRTI_id']);
    }

    function createRecord_() {
        $record = array();
        $record['TRANSL_CODE'] = UserRequest::$post['TRTI_transl_code'];
        $record['SEQNUM'] = '0';
        $record['ORIGINAL_VALUE'] = UserRequest::$post['TRTI_original_value'];
        $record['ORIGINAL_VALUE_CI'] = UserRequest::$post['TRTI_original_value'];
        $record['REPLACE_VALUE'] = UserRequest::$post['TRTI_replace_value'];
        $record['CASE_SENSITIVE'] = UserRequest::$post['TRTI_case_sensitive'];
        $record['DIRECTION'] = UserRequest::$post['TRTI_direction'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->versionIncrease('ID', $recordid);
    }

    function deleteRecordSk($id) {
        $this->versionIncrease('ID', $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['TRTI_id'] = CLX_INIT_ZERO;
        $record['TRTI_transl_code'] = $callingpage['transl_code'];
        $record['TRTI_original_value'] = '';
        $record['TRTI_original_value_ci'] = '';
        $record['TRTI_replace_value'] = '';
        $record['TRTI_case_sensitive'] = '0';
        $record['TRTI_direction'] = '0';
        $record['TRTI_seqnum'] = '0';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $transl_code = ClxCommon::$db->getField($this->tablename, 'TRANSL_CODE', $fieldname . '=' . singleQuote($fieldvalue));
        $TRTS = &getTool('TRTS');
        $TRTS->versionIncrease('IDCODE', $transl_code);
    }

    /**
     * Performs translation of a value
     */
    function translate($transl_code, &$value, $direction = CLX_NUM_ZERO) {
        $translation = ClxCommon::$db->getField($this->tablename, 'REPLACE_VALUE', 'TRANSL_CODE = ' . singleQuote($transl_code) . ' AND ORIGINAL_VALUE = ' . singleQuote(clxCommon::$db->addslashes($value)) . ' AND DIRECTION = ' . $direction . ' ORDER BY SEQNUM DESC');
        if (strlen($translation) > CLX_NUM_ZERO) {
            $value = $translation;
        }
    }

    function import() {
        require_once'classes/clx_file_delimited.php';
        $filename = UserRequest::$files['TRTI_filename']['tmp_name'];
        $callingpage = peekHistory();
        $contents = '';
        $fd = new clx_file_delimited($filename);
        if ($fd->error) {
            $contents = alert($fd->errorMsg, dict('GEN_LBL_RETURN'));
        } else {
            $record = array();
            $record['TRANSL_CODE'] = $callingpage['transl_code'];
            $i = CLX_INIT_ZERO;
            while ($buffer = $fd->getNextRecord2()) {
                if (strlen(trim($buffer)) == CLX_NUM_ZERO) {
                    continue;
                }
                //$line = explode($fd->field_delim, $buffer);
                $line = $fd->toArray($buffer);
                $record['ORIGINAL_VALUE'] = encode_string($line[CLX_NUM_ZERO], kDefaultEncoding);
                $record['ORIGINAL_VALUE_CI'] = encode_string($line[CLX_NUM_ZERO], kDefaultEncoding);
                $record['REPLACE_VALUE'] = encode_string($line[CLX_NUM_ONE], kDefaultEncoding);
                $record['CASE_SENSITIVE'] = (empty($line[CLX_NUM_TWO])) ? 0 : 1;
                $record['DIRECTION'] = (empty($line[CLX_NUM_THREE])) ? 0 : 1;
                $record['SEQNUM'] = $i;
                $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'TRANSL_CODE = ' . singleQuote($record['TRANSL_CODE']) . ' AND ORIGINAL_VALUE = ' . singleQuote($line[CLX_NUM_ZERO]));
                unset($id);
                $i++;
            }
            $fd->close();
        }
        return $contents;
    }

}
