<?php

/**
 * Translation Sets Manager
 *
 * @package		Clientix.Tool.System.Translations
 * @abstract		System Translation Tool
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version    	$Rev: 10496 $
 *
 * CHANGE LOG
 */
class sys_translation_sets extends clx_tool {

    var $debug = false;

    function __construct() {
        $this->code = 'TRTS';
        $this->name = 'Translation Sets';
        $this->system = 'System';
        $this->version = '5.5.1.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->primary_key = array('IDCODE');
        $this->tablename = 'CLX_SYS_TRANSL_SETS';
        $this->record_name_singular = 'TRANSLATION';
        $this->record_name_plural = 'TRANSLATIONS';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_TRTS_TITLE_CREATE',
            'panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE',
                            'fields' => array(
                                array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16', 'dontmodify' => '1',
                                    'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
                                )
                            )
                        ), array('label' => 'GEN_LBL_NAME',
                            'fields' => array(
                                array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_DESCRIPTION',
                            'fields' => array(
                                array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))
                        ), array('label' => 'GEN_LBL_VERSION',
                            'fields' => array(
                                array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(
                                        array('type' => 'string', 'required' => '1'))
                                )
                            )
                        ), array('label' => 'GEN_LBL_MODULECODE',
                            'fields' => array(
                                array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array(
                                        array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_MODULES'))
                                )
                            )
                        )
                    )
                )
            )
        );
        $this->viewform = $this->editform;
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_TRANSLN_SET', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
            'fields' => $fields,
            'indexes' => $indexes);
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::pack()
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'TRTI', 'SELECT * FROM CLX_SYS_TRANSL_ITEMS WHERE TRANSL_CODE =' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE']))) > CLX_NUM_ZERO);
        if ($exists === true) {
            ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
        }
        $TRTI = &getTool('TRTI');
        ClxCommon::$db->deleteRecords($TRTI->tablename, 'TRANSL_CODE =' . singleQuote($record['IDCODE']));
        $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $toolcode = $o->tag;
                $tool = getTool($toolcode);
                $tool->unpack(array('package' => $o, 'id' => $id));
            }
        }
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::action()
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'LIST':
                    if (!empty(UserRequest::$get['menuid'])) {
                        if (!empty(UserRequest::$request['mc'])) {//Adding parameter to the filter if mc is set on menu to display only mc.
                            $_SESSION['paging'][$this->code . 'List']['mc'] = UserRequest::$request['mc'];
                        } else {
                            unset($_SESSION['paging'][$this->code . 'List']['mc']);
                        }
                    }
                    if (!empty($_SESSION['paging'][$this->code . 'List']['mc'])) {
                        $_SESSION['paging'][$this->code . 'List']['criteria'] = ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase($_SESSION['paging'][$this->code . 'List']['mc']) . '%');
                        if (empty($_SESSION['paging'][$this->code . 'List']['sortby']))
                            $_SESSION['paging'][$this->code . 'List']['sortby'] = CLX_NUM_ZERO;
                        if (empty($_SESSION['paging'][$this->code . 'List']['startat']))
                            $_SESSION['paging'][$this->code . 'List']['startat'] = CLX_NUM_ZERO;
                    }
                    $contents = parent::action($ac, $idlist);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [SYS_TRANSLATION_SETS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::makelist()
     */
    function makelist($listingcode = null) {
        $this->applyUserAccessFromMenu();
        ClxFreeReturn($listingcode);
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'TRTSList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_TRTS_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_SIX));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_MODULECODE')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION']), $record['MODULECODE']));
            $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canDisplay()) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        //if ($this->getAccess()->canDisplay()) {
        if ($this->getAccess()->canAppend()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_TRTS_CREATEMSG')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_TRTS_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'TRTS_idcode', (array_key_exists('TRTS_idcode', UserRequest::$post)) ? UserRequest::$post['TRTS_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'TRTS_idname', (array_key_exists('TRTS_idname', UserRequest::$post)) ? UserRequest::$post['TRTS_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'TRTS_modulecode', (array_key_exists('TRTS_idname', UserRequest::$post)) ? UserRequest::$post['TRTS_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            if (UserRequest::$post['TRTS_idcode'] <> '') {
                $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['TRTS_idcode']) . '%');
            }
            if (UserRequest::$post['TRTS_idname'] <> '') {
                $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['TRTS_idname'])) . '%');
            }
            if (UserRequest::$post['TRTS_modulecode'] <> '') {
                $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['TRTS_modulecode']) . '%');
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=set_traslations_set></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['TRTS_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['TRTS_idname']);
    }

    function viewFormFill($record) {
        addHistory('transl_code', $record['TRTS_idcode']);
        $contents = '';
        // Elements
        $TRTI = &getTool('TRTI');
        $contents .= $TRTI->makepanel($record['TRTS_idcode']);
        return $contents;
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::filterFormFill()
     */
    function filterFormFill($msg, &$table) {

        $table->addField($msg, dict('GEN_LBL_CODE'), 'TRTS_idcode', (array_key_exists('TRTS_idcode', UserRequest::$post)) ? UserRequest::$post['TRTS_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'TRTS_idname', (array_key_exists('TRTS_idname', UserRequest::$post)) ? UserRequest::$post['TRTS_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULE'), 'TRTS_modulecode', (array_key_exists('TRTS_modulecode', UserRequest::$post)) ? UserRequest::$post['TRTS_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('TRTS_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::setfilter()
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['TRTS_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['TRTS_idcode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['TRTS_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['TRTS_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['TRTS_modulecode'])) {
            $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['TRTS_modulecode']) . '%');
            $boolean = 'AND';
        }
        $_SESSION['paging']['TRTSList']['sortby'] = UserRequest::$post['TRTS_sort'];
        $_SESSION['paging']['TRTSList']['criteria'] = $filter;
        $_SESSION['paging']['TRTSList']['startat'] = '0';
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::clearfilter()
     */
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	////////////////////////////////////////////////// 
    /*function clearfilter() {
        $_SESSION['paging']['TRTSList']['criteria'] = '';
    }*/

    /**
     * (non-PHPdoc)
     * @see clx_tool::loadRecord()
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['TRTS_id'] = $recordid;
        $record['TRTS_idcode'] = $values['IDCODE'];
        $record['TRTS_idname'] = $values['IDNAME'];
        $record['TRTS_version'] = $values['VERSION'];
        $record['TRTS_version_counter'] = $values['VERSION_COUNTER'];
        $record['TRTS_cdate'] = xFormat::date($values['CDATE']);
        $record['TRTS_mdate'] = xFormat::date($values['MDATE']);
        $record['TRTS_description'] = $values['DESCRIPTION'];
        $record['TRTS_modulecode'] = $values['MODULECODE'];
        return $record;
    }

    /**
     * (non-PHPdoc)
     * @see clx_tool::saveRecord()
     */
    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['TRTS_idname'];
        $record['VERSION'] = UserRequest::$post['TRTS_version'];
        $record['MDATE'] = $d;
        $record['DESCRIPTION'] = UserRequest::$post['TRTS_description'];
        $record['MODULECODE'] = UserRequest::$post['TRTS_modulecode'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['TRTS_id']);
        $this->versionIncrease('ID', UserRequest::$post['TRTS_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['TRTS_idcode'];
        $record['IDNAME'] = UserRequest::$post['TRTS_idname'];
        $record['VERSION'] = UserRequest::$post['TRTS_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['DESCRIPTION'] = UserRequest::$post['TRTS_description'];
        $record['MODULECODE'] = UserRequest::$post['TRTS_modulecode'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->versionIncrease('IDCODE', $record['IDCODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $idcode = $record['IDCODE'];
        $TRTI = getTool('TRTI');
        $elements = ClxCommon::$db->getRecords($TRTI->tablename, 'TRANSL_CODE=' . singleQuote($idcode));
        while (($element = ClxCommon::$db->getNextRecord($elements))) {
            $TRTI->deleteRecordSk($element['ID']);
        }
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['TRTS_id'] = CLX_INIT_ZERO;
        $record['TRTS_idcode'] = '';
        $record['TRTS_idname'] = '';
        $record['TRTS_version'] = '1.0.0';
        $record['TRTS_cdate'] = '';
        $record['TRTS_mdate'] = '';
        $record['TRTS_description'] = '';
        $record['TRTS_modulecode'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
    }

}

// End class
