<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-31 13:55:38 +0000 (Thu, 31 Jan 2019) $
 * @version        $Rev: 10516 $
 *
 * CHANGE LOG
 *  Included label
 */
class ui_filters extends clx_tool {

    var $access_types;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'UIFT';
        $this->name = 'Filter Manager';
        $this->system = 'Interface';
        $this->version = '0.3.3.3';

        $this->has_access = true;


        $this->tablename = 'CLX_UI_FILTERS';
        $this->record_name_singular = 'FILTER';
        $this->record_name_plural = 'filters';
        $this->primary_field = 'QUERYCODE';
        $this->secondary_field = 'IDNAME';

        $this->access_types = array('Private', 'Public');
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_INTERFACE:MENU_FILTERS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['ACCESSES'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SQL_COMMANDS'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $msg = array();
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'CQUERY':
                    if (isset(UserRequest::$post['confirm'])) {
                        if ($this->checkchooseQueryForm($msg)) {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm('CREATE', array(), $record, $msg);
                        } else {
                            $contents = $this->chooseQueryForm(UserRequest::$post, $msg);
                        }
                    } else {
                        $info = peekHistory();
                        if (isset($info['qy'])) {
                            UserRequest::$post['UIFT_querycode'] = $info['qy'];
                            $record = $this->defaultRecord();
                            $contents = $this->editForm('CREATE', array(), $record, $msg);
                        } else {
                            $record = array();
                            $contents = $this->chooseQueryForm($record, $msg);
                        }
                    }
                    break;
                case 'CREATE':
                    if ($this->getAccess()->canDisplay()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            $this->createRecord_();
                            $info = popHistory();
                            $contents = executeHistory($info);
                        } else {
                            $record = $this->defaultRecord();
                            $contents = $this->editForm($ac, array(), $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FILTERS-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'MODIFY':
                    if ($this->getAccess()->canModify()) {
                        if (isset(UserRequest::$post['confirm'])) {
                            $this->saveRecord();
                            if (count($idlist) > CLX_NUM_ZERO) {
                                $id = array_pop($idlist);
                                $record = $this->loadToolRecord($id);
                                $contents = $this->editForm($ac, $idlist, $record, $msg);
                            } else {
                                $info = popHistory();
                                $contents = executeHistory($info);
                            }
                        } else {
                            $id = array_pop($idlist);
                            $record = $this->loadToolRecord($id);
                            $contents = $this->editForm($ac, $idlist, $record, $msg);
                        }
                    } else {
                        $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FILTERS-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
                    }
                    break;
                case 'PREF':
                    $id = array_pop($idlist);
                    $this->setPref($id);
                    $info = popHistory();
                    $contents = executeHistory($info);
                    break;
                case 'UNPREF':
                    $id = array_pop($idlist);
                    $this->unsetPref($id);
                    $info = popHistory();
                    $contents = executeHistory($info);
                    break;
                case 'SHORTLIST':
                    $querycode = UserRequest::$request['qy'];
                    $contents = $this->makeshortlist($querycode);
                    break;
                case 'QUICK_FILTER':
                    if (UserRequest::$request['filterid'] == CLX_NUM_ZERO) {
                        $ac = 'CREATE';
                        UserRequest::$post['UIFT_querycode'] = UserRequest::$request['qy'];
                        UserRequest::$post['UIFT_id'] = CLX_INIT_ZERO;
                        $record = $this->defaultRecord();
                        $contents = $this->editForm($ac, array(), $record, $msg);
                    } else {
                        $ac = 'MODIFY';
                        $record = $this->loadToolRecord(UserRequest::$request['filterid']);
                        $contents = $this->editForm($ac, array(), $record, $msg);
                    }
                    break;
                case 'USE':
                    $querycode = UserRequest::$request['qy'];
                    $filterid = UserRequest::$request[CLX_PROP_ID];
                    $this->setPref($filterid);
                    $info = popHistory();
                    $contents = executeHistory($info);
                    break;
                case 'CLEAR':
                    $querycode = UserRequest::$request['qy'];
                    $this->clearPref($querycode);
                    $info = popHistory();
                    $contents = executeHistory($info);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FILTERS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        ClxFreeReturn($listingcode);
        //$t = time('U'); genera un warning
		$t = time();
        $to = $this->code;
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $here = $action . '?to=' . $to . '&t=' . $t;
        $username = $_SESSION['user_record']['username'];
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->setProp(CLX_PROP_ID, 'UIFTList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_FILTERS';
        $whereclause = ($this->getAccess()->isAdmin()) ? CLX_CHAR_EMPTY_STRING : '(UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' OR ACCESSES=1)';
        $sortclause = '';

        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFT_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_ACCESS')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $querycode = $record['QUERYCODE'];
            if (ClxCommon::$db->oneExistsRecords('CLX_UI_FILTER_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND UPPER( USERNAME)=' . singleQuote(uppercase($username)) . ' AND FILTERID=' . $recordid) > CLX_NUM_ZERO) {
                $name = '<b>' . $record['IDNAME'] . '</b>';
            } else {
                $name = $record['IDNAME'];
            }
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($name, $querycode, xFormat::text($record['DESCRIPTION']), $this->access_types[$record['ACCESSES']]));
            $i++;
        }
        $buttons = '';
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $b = new htmlFormSubmit(dict('GEN_LBL_SETPREFERED'), 'PREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $buttons .= $b->source();
        $b = new htmlFormSubmit(dict('GEN_LBL_UNSETPREFERED'), 'UNPREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $buttons .= $b->source();
        if ($this->getAccess()->canDisplay()) {
			$buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_FILTER')), array('action' => 'CQUERY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function makeshortlist($querycode) {

        $t = time();
        $to = $this->code;
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $here = $action . '?to=' . $to . '&t=' . $t;
        $username = $_SESSION['user_record']['username'];
        pushHistory($this->code, 'SHORTLIST', array(), 'qy', $querycode);
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->setProp(CLX_PROP_ID, 'UIFTShortList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_FILTERS';
        $whereclause = '(UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' OR ACCESSES=' . singleQuote(CLX_CHAR_EMPTY_STRING) . ') AND UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode));
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFT_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_QUERYCODE'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_ACCESS')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
            $querycode = $record['QUERYCODE'];
            if (ClxCommon::$db->oneExistsRecords('CLX_UI_FILTER_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND UPPER( USERNAME)=' . singleQuote(uppercase($username)) . ' AND FILTERID=' . singleQuote($recordid)) > CLX_NUM_ZERO) {
                $name = '<b>' . $record['IDNAME'] . '</b>';
            } else {
                $name = $record['IDNAME'];
            }
            $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($name, $querycode, xFormat::text($record['DESCRIPTION']), $this->access_types[$record['ACCESSES']]));
            $i++;
        }
        $buttons = '';
        $selectButtons = '';
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_FILTER')), array('action' => 'CQUERY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        if ($this->getAccess()->canModify()) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        $selectGroup = new htmlSpan('SELECTorbuttons', '', $selectButtons);
        $singleSelectorButtons = '';

        $b = new htmlFormSubmit(dict('GEN_LBL_SETPREFERED'), 'PREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $singleSelectorButtons .= $b->source();

        $b = new htmlFormSubmit(dict('GEN_LBL_UNSETPREFERED'), 'UNPREF', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ONE');
        $singleSelectorButtons .= $b->source();

        $singleSelectGroup = new htmlSpan('singleselectorbuttons', '', $singleSelectorButtons);

        $b = new htmlButtonReturn('', '', $action);
        $form->addContents($page->footers($table, $buttons.$selectGroup->source() . $singleSelectGroup->source().$b->source()));
        return $form->source();
    }

    function chooseQueryForm($record, $msg) {
        $title = dict('TOOL_UIFT_TITLE_CHOOSEQUERY');
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'CQUERY');
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBQY = &getTool('DBQY');
        $DBQY->placeChooseQuery($msg, dict('GEN_LBL_QUERYCODE') . ':', 'UIFT_querycode', (array_key_exists('UIFT_querycode', $record)) ? $record['UIFT_querycode'] : '', $table);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlFormSubmit(dict('GEN_LBL_SELECT'), 'CQUERY', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $c = new htmlButtonCancel('', '', $action);
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkchooseQueryForm(&$msg) {
        $msg = array();
        $DBQY = &getTool('DBQY');
        if (!$DBQY->queryExists(UserRequest::$post['UIFT_querycode'])) {
            $msg['UIFT_querycode'] = 'There is no query with this code.';
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UIFT_TITLE_MODIFY') : dict('TOOL_UIFT_TITLE_CREATE');
        $filterid = $record['UIFT_id'];
        $querycode = $record['UIFT_querycode'];
        $queryfields = $this->queryFields($querycode);
        $contents = '';

        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        $form = new htmlForm(CLX_PROP_ACTION, $action);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('UIFT_id', $filterid);
        $form->addArgument('UIFT_querycode', $querycode);
        // D A T A  R E C O R D  -------------------------------------------------
        $datarecord = array();
        foreach ($queryfields as $fieldrec) {
            $form_fieldname = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
            if (!array_key_exists($form_fieldname, $datarecord)) {
                $datarecord[$form_fieldname] = '';
                $datarecord[$form_fieldname . '_operator'] = '';
                $datarecord[$form_fieldname . '_boolean'] = '';
            }
            if ($filterid > CLX_NUM_ZERO) {
                if (ClxCommon::$db->oneExistsRecords('CLX_UI_FILTER_CRITERIA', 'FILTERID=' . $filterid . ' AND TABLECODE=' . singleQuote($fieldrec['TABLECODE']) . ' AND FIELDNAME=' . singleQuote($fieldrec['FIELDNAME'])) > CLX_NUM_ZERO) {
                    $rec = ClxCommon::$db->getRecord('CLX_UI_FILTER_CRITERIA', 'FILTERID=' . $filterid . ' AND TABLECODE=' . singleQuote($fieldrec['TABLECODE']) . ' AND FIELDNAME=' . singleQuote($fieldrec['FIELDNAME']));
                    $datarecord[$form_fieldname] = $rec['FIELDVALUE'];
                    $datarecord[$form_fieldname . '_operator'] = $rec['OPERATOR'];
                    $datarecord[$form_fieldname . '_boolean'] = $rec['BOOLEAN'];
                }
            }
        }
        // H E A D E R S
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['UIFT_querycode']);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFT_idname', $record['UIFT_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'UIFT_description', $record['UIFT_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $pop = new HTMLSelect('UIFT_accesses');
        foreach ($this->access_types as $key => $value) {
            $pop->addItem($value, $key, ($record['UIFT_accesses'] == $key));
        }
        $table->addData(dict('GEN_LBL_ACCESS') . ':', $pop->source());
        $form->addContents($table->source());

        // C O L U M N S
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_FIELD'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '200'));
        $table->addCell(dict('GEN_LBL_OPERATOR'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '150'));
        $table->addCell(dict('GEN_LBL_VALUE'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS));
        $table->addCell(dict('GEN_LBL_BOOLEAN'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => '20'));
        $lasttable = '';
        foreach ($queryfields as $fieldrec) {
            if ($lasttable != $fieldrec['TABLECODE']) {
                $lasttable = $fieldrec['TABLECODE'];
                $table->addCell($fieldrec['TABLENAME'] . ' (' . $fieldrec['MODULECODE'] . ')', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
            }
            $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
            $fieldrec['FIELD_READONLY'] = CLX_INIT_ZERO;
            $fieldobj = $fieldinstance->placeSearchField($fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'], $fieldrec, $datarecord, $form);
            $table->addCell($fieldobj->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        }
        $form->addContents($table->source());


        // F O O T E R S
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
            $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel('', '', $action);
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents;
    }

    function queryFields($querycode) {
        // Restricted to fields on the query that are visible
		switch (kDatabaseType) {
            case 'oci': // Oracle
				$extraquerycriteria= 'AND (F.FIELDTYPE NOT IN ('.singleQuoteArray(array('TEXT','CLOB')).') ) ';
                        break;
            default: // MS SQL Server
				$extraquerycriteria= '';
                break;
        }
        $sql = 'SELECT QF.TABLECODE TABLECODE, T.IDNAME TABLENAME, T.MODULECODE MODULECODE, F.*, 0 ADD_LIST_ENTRY, 0 FIELD_DB_SIZE, 0 FIELD_DB_DISPLAYSIZE  FROM CLX_DB_QUERIES Q, CLX_DB_QUERY_FIELDS QF, CLX_DB_FIELDS F, CLX_DB_TABLES T WHERE Q.IDCODE=' . singleQuote($querycode) . ' AND QF.FILTER = 1 AND QF.QUERYCODE = Q.IDCODE AND QF.FIELDNAME = F.FIELDNAME AND QF.TABLECODE = T.IDCODE '.$extraquerycriteria.' ORDER BY QF.TABLECODE ASC, QF.SEQNUM ASC';
        $fields = ClxCommon::$db->executeQuery($sql);
        if (ClxCommon::$db->numRows($fields) == CLX_NUM_ZERO) {
            // If no field is set on query get all fields on the table using this query
            $sql = 'SELECT T.IDCODE TABLECODE, T.IDNAME TABLENAME, T.MODULECODE MODULECODE, F.*, 0 ADD_LIST_ENTRY, 0 FIELD_DB_SIZE, 0 FIELD_DB_DISPLAYSIZE  FROM CLX_DB_QUERIES Q, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF, CLX_DB_TABLES T, CLX_DB_FIELDS F WHERE Q.IDCODE=' . singleQuote($querycode) . ' AND Q.IDCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND TF.TABLECODE = T.IDCODE AND F.FIELDNAME = TF.FIELDNAME '.$extraquerycriteria.' ORDER BY QT.TABLECODE ASC, TF.SEQNUM ASC';
            $fields = ClxCommon::$db->executeQuery($sql);
        }
        $list = array();
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
            $list[] = $field;
        }
        return $list;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_QUERYCODE') . ':', $record['UIFT_querycode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIFT_idname']);
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('UIFT_querycode', UserRequest::$post)) {
            UserRequest::$post['UIFT_querycode'] = '';
        }
        if (!array_key_exists('UIFT_idname', UserRequest::$post)) {
            UserRequest::$post['UIFT_idname'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'UIFT_querycode', UserRequest::$post['UIFT_querycode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFT_idname', UserRequest::$post['UIFT_idname'], CLX_NUM_32, CLX_NUM_64);
        $pop = new HTMLSelect('UIFT_accesses');
        $pop->addItem('', '', true);
        foreach ($this->access_types as $key => $value) {
            $pop->addItem($value, $key, false);
        }
        $table->addData(dict('GEN_LBL_ACCESS') . ':', $pop->source());
        $sort = new HtmlSelect('UIFT_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Query Code', 'querycode', false);
        $sort->addItem('Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['UIFT_querycode'])) {
            $filter .= $boolean . ' UPPER(QUERYCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFT_querycode']) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UIFT_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['UIFT_idname'])) . '%');
            $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UIFT_accesses'])) {
            $filter .= $boolean . ' ACCESSES = ' . UserRequest::$post['UIFT_accesses'];
            $boolean = 'AND';
        }
        $_SESSION['paging']['UIFTList']['sortby'] = UserRequest::$post['UIFT_sort'];
        $_SESSION['paging']['UIFTList']['criteria'] = $filter;
        $_SESSION['paging']['UIFTList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['UIFTList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIFT_id'] = $recordid;
        $record['UIFT_idname'] = $values['IDNAME'];
        $record['UIFT_username'] = $values['USERNAME'];
        $record['UIFT_description'] = $values['DESCRIPTION'];
        $record['UIFT_querycode'] = $values['QUERYCODE'];
        $record['UIFT_accesses'] = $values['ACCESSES'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIFT_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFT_description'];
        $record['ACCESSES'] = UserRequest::$post['UIFT_accesses'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIFT_id']);
        $this->saveDetails(UserRequest::$post['UIFT_id'], UserRequest::$post['UIFT_querycode']);
    }

    function createRecord_() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIFT_idname'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFT_description'];
        $record['QUERYCODE'] = UserRequest::$post['UIFT_querycode'];
        $record['ACCESSES'] = UserRequest::$post['UIFT_accesses'];
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->saveDetails($recordid, UserRequest::$post['UIFT_querycode']);
        $this->setPref($recordid);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $filterrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $_SESSION['paging'][$filterrec['QUERYCODE'] . '_LIST']['criteria'] = '';
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_CRITERIA', 'FILTERID=' . $id);
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_PREFS', 'FILTERID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['UIFT_id'] = CLX_INIT_ZERO;
        $record['UIFT_idname'] = '';
        $record['UIFT_description'] = '';
        $record['UIFT_querycode'] = UserRequest::$post['UIFT_querycode'];
        $record['UIFT_username'] = $_SESSION['user_record']['username'];
        $record['UIFT_accesses'] = '1';
        return $record;
    }

    function saveDetails($filterid, $querycode) {
        $queryfields = $this->queryFields($querycode);
        $sql = '';
        $boolean = '';
        $record = array();
        $record['FILTERID'] = $filterid;
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_CRITERIA', 'FILTERID=' . $filterid);
        foreach ($queryfields as $fieldrec) {
            $sufix = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
            $db_fldname = $fieldrec['TABLECODE'] . '.' . $fieldrec['FIELDNAME'];
            if (!array_key_exists($sufix, UserRequest::$post)) {
                continue;
            } elseif (is_array(UserRequest::$post[$sufix]) && empty(UserRequest::$post[$sufix])) {
                continue;
            } elseif (!is_array(UserRequest::$post[$sufix]) && strlen(UserRequest::$post[$sufix]) == CLX_NUM_ZERO) {
                continue;
            }
            if (strlen(UserRequest::$post[$sufix . '_operator']) > CLX_NUM_ZERO) {
                //storing criterias
                $record['TABLECODE'] = $fieldrec['TABLECODE'];
                $record['FIELDNAME'] = $fieldrec['FIELDNAME'];
				
                $record['OPERATOR'] = (!empty(UserRequest::$post[$sufix . '_operator'])) ? UserRequest::$post[$sufix . '_operator'] : '===';
                switch ($record['OPERATOR']){
					case '&lt;':
						$record['OPERATOR']="<";
						break;
					case '&gt;':
						$record['OPERATOR']=">";
						break;
					case '&lt;=':
						$record['OPERATOR']="<=";
						break;
					case '&gt;=':
						$record['OPERATOR']=">=";
						break;	
				}
				
				$record['BOOLEAN'] = (!empty(UserRequest::$post[$sufix . '_boolean'])) ? UserRequest::$post[$sufix . '_boolean'] : 'AND';
                if (is_array(UserRequest::$post[$sufix])) {
                    $record['FIELDVALUE'] = '';
                    $comma = '';
                    foreach (UserRequest::$post[$sufix] as $val) {
                        if ($record['OPERATOR'] === '==') {
                            $record['FIELDVALUE'] .= $comma . $db_fldname . ' LIKE ' . singleQuote('%' . $val . '%');
                            $comma = ' OR ';
                        } else {
                            $record['FIELDVALUE'] .= $comma . $db_fldname . ' NOT LIKE ' . singleQuote('%' . $val . '%');
                            $comma = ' AND ';
                        }
                    }
                    $record['FIELDVALUE'] = '(' . $record['FIELDVALUE'] . ')';
                    if (strlen($sql) > CLX_NUM_ZERO)
                        $record['FIELDVALUE'] = ' ' . $record['BOOLEAN'] . ' ' . $record['FIELDVALUE'];
                    $sql .= $record['FIELDVALUE'];
                }else {
					$record['FIELDVALUE'] = UserRequest::$post[$sufix];
                    $sql .= $this->makeCriteria($db_fldname, $record['OPERATOR'], $record['FIELDVALUE'], $boolean);
                }
                ClxCommon::$db->createRecord('CLX_UI_FILTER_CRITERIA', $record, false);
                $boolean = $record['BOOLEAN'] . ' ';
            }
        }
        $record = array();
        $record['SQL_COMMANDS'] = $sql; /// comentado produce textos con doble comillas simples error filtro //ClxCommon::$db->addslashes($sql);
        ClxCommon::$db->updateRecord('CLX_UI_FILTERS', $record, 'ID=' . $filterid);
        $prefid = $this->getPref($querycode);
        if ($filterid == $prefid) {
            $_SESSION['paging'][$querycode . '_LIST']['criteria'] = $sql;
            unset($_SESSION['paging'][$querycode . '_LIST']['qfilter']);
        }
    }

    function getPref($querycode) {
        $username = $_SESSION['user_record']['username'];
        if (ClxCommon::$db->oneExistsRecords('CLX_UI_FILTER_PREFS', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' AND UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode))) > CLX_NUM_ZERO) {
            $filterid = ClxCommon::$db->getField('CLX_UI_FILTER_PREFS', 'FILTERID', 'UPPER(USERNAME)=' . singleQuote(uppercase($username)) . ' AND UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)));
        } else {
            $filterid = CLX_INIT_ZERO;
        }
        return $filterid;
    }

    function setPref($filterid) {
        $filterrec = ClxCommon::$db->getRecord('CLX_UI_FILTERS', 'ID=' . $filterid);
        if ($filterrec==null){
            $filterrec['QUERYCODE']=null;
            $filterrec['SQL_COMMANDS']=null;
        }
        $querycode = $filterrec['QUERYCODE'];
        $username = $_SESSION['user_record']['username'];
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        $record = array();
        $record['FILTERID'] = $filterid;
        $record['QUERYCODE'] = $querycode;
        $record['USERNAME'] = $username;
        ClxCommon::$db->createRecord('CLX_UI_FILTER_PREFS', $record, false);
        $_SESSION['paging'][$querycode . '_LIST']['criteria'] = $filterrec['SQL_COMMANDS'];
        unset($_SESSION['paging'][$querycode . '_LIST']['qfilter']);
    }

    function unsetPref($filterid) {
        $filterrec = ClxCommon::$db->getRecord('CLX_UI_FILTERS', 'ID=' . $filterid);
        $querycode = $filterrec['QUERYCODE'];
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_PREFS', 'FILTERID=' . singleQuote($filterid) . ' AND USERNAME=' . singleQuote($_SESSION['user_record']['username']));
        $_SESSION['paging'][$querycode . '_LIST']['criteria'] = '';
        unset($_SESSION['paging'][$querycode . '_LIST']['qfilter']);
    }

    function clearPref($querycode) {
        $username = $_SESSION['user_record']['username'];
        ClxCommon::$db->deleteRecords('CLX_UI_FILTER_PREFS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND  UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        $_SESSION['paging'][$querycode . '_LIST']['criteria'] = '';
        unset($_SESSION['paging'][$querycode . '_LIST']['qfilter']);
    }

    function initPref($querycode) {
        $this->setPref($this->getPref($querycode));
    }

    /**
     * Load login prefference
     *
     */
    function loadFilterPreferences() {
        $sql = 'SELECT DISTINCT UIFU.FILTERID FROM CLX_UI_FILTERS UIFT, CLX_UI_FILTER_PREFS UIFU, CLX_UI_LISTINGS UILG, CLX_DB_QUERIES DBQY WHERE UIFT.ID = UIFU.FILTERID AND UIFU.QUERYCODE = DBQY.IDCODE AND UILG.IDCODE = DBQY.IDCODE AND UIFU.USERNAME=' . singleQuote($_SESSION['user_record']['username']);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $this->setPref($r['FILTERID']);
        }
    }

    function pop($querycode, $form) {
        $username = $_SESSION['user_record']['username'];
        $formname = $form->getProp(CLX_PROP_ID);
        $to = $this->code;
        $filterid = $this->getPref($querycode);
        $action = (!empty(UserRequest::$server['PHP_SELF'])) ? UserRequest::$server['PHP_SELF'] : 'tool.php';
        // Create the popup with filters...
        $optionname = $querycode . '_options';
        $options = new htmlSelect($optionname);
        //***LSA-20040809, Changed window.open to window.location. Jscript not working ***
        //$options->setProp('onchange', 'window.location=document.getElementsByName(' . singleQuote($optionname) . ').value');
        $options->setProp('onchange', 'window.location=this.value');
        $options->addItem(dict('TOOL_UIFT_MENU_OPTIONS'), $action . '?to=' . $to . '&ac=QUICK_FILTER&qy=' . $querycode . '&filterid=' . $filterid, false);
        $options->addItem(dict('TOOL_UIFT_MENU_ALLRECORDS'), $action . '?to=' . $to . '&ac=CLEAR&qy=' . $querycode, (($filterid == CLX_NUM_ZERO) and empty($_SESSION['paging'][$formname]['criteria'])));
        $options->addItem(dict('TOOL_UIFT_MENU_MANAGE'), $action . '?to=' . $to . '&ac=SHORTLIST&qy=' . $querycode, false);
        
		if (kApcTtl_10>0) $qitemsrecord = apc_fetchFct($findVar='itemrecord_'.UPPERCASE($querycode).'_'.uppercase($username).'_' , $fetched);
		
		if (empty($fetched)) {
			$qitemsrecord=array();
			$records = ClxCommon::$db->getRecords('CLX_UI_FILTERS', 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)), '(USERNAME=' . singleQuote(uppercase($username)) . ' OR ACCESSES=1) ORDER BY IDNAME');
			while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
				$qitemsrecord[]=array('ID'=>$record['ID'],'IDNAME'=>$record['IDNAME']);
			}
			if (kApcTtl_10>0 && count($qitemsrecord)>0){
				apc_addFct($findVar,$qitemsrecord,kApcTtl_10);
			}
		}
		foreach($qitemsrecord as $record) {
		
            $options->addItem($record['IDNAME'], $action . '?to=' . $to . '&ac=USE&qy=' . $querycode . '&id=' . $record['ID'], (intval($filterid) ===intval($record['ID'])));
        }
        if (!empty($_SESSION['paging'][$formname]['qfilter'])) {
            $options->addItem('...' . dict('GEN_LBL_QFILTER') . '...', '', true);
        }
        return $options->source();
    }

    // the following is also used by EDIT
    function makeCriteria($fieldname, $operator, $fieldvalue, $boolean) {
        $sql = ' ' . $boolean . ' (' . $fieldname;
        switch ($operator) {
            case '===':
			case '==':
                $sql .= '=' . singleQuote($fieldvalue);
                break;
            case '!== ':
			case '!= ':
			 $sql .= '<>' . singleQuote($fieldvalue). ' OR '.$fieldname.' IS NULL ';
                break;
            case '%=':
                $sql .= ' LIKE ' . singleQuote($fieldvalue . '%');
                break;
            case '=%':
                $sql .= ' LIKE ' . singleQuote('%' . $fieldvalue);
                break;
            case '%%':
                $sql .= ' LIKE ' . singleQuote('%' . $fieldvalue . '%');
                break;
            case '>':
            case '>=':
            case '<=':
            case '<':
            case 'IN':
                $sql .= $operator . singleQuote($fieldvalue);
                break;
            default :
                break;
        }
        return $sql.')';
    }

}
