<?php

/**
 * Application Form Manager
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Application Forms
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Forms
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version    	$Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 */
class ui_forms extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {

        $this->code = 'UIFM';
        $this->name = 'Form Manager';
        $this->system = 'Interface';
        $this->version = '0.6.16.3';

        $this->has_access = true;
        $this->has_package = true;
        $this->has_pagelet = true;
        $this->tablename = 'CLX_UI_FORMS';
        $this->record_name_singular = CLX_CLASS_FORM;
        $this->record_name_plural = 'forms';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->primary_key = array('IDCODE');
        $this->quickfilterform = array('panels' => array(
                array('type' => 'normal',
                    'entries' => array(
                        array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '16', 'displaysize' => '16'))),
                        array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48'))),
                        array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'readonly' => false, 'tool' => 'DBMO'))),
                        array('label' => 'GEN_LBL_TABLECODE', 'fields' => array(array('name' => 'tablecode', 'type' => 'lookup', 'default' => '', 'readonly' => false, 'tool' => 'DBTB')))
        ))));
		IF (ISSET($_SESSION['ADDRDEBUGUIFORMS']) && $_SESSION['ADDRDEBUGUIFORMS']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}	
    }

    function install() {
        $UIMI = &getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_INTERFACE:MENU_FORMS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['DEFAULT_FORM'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['NUMCOLS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ONE);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'DEFAULT_FORM');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        $indexes[$ipf . '07'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

//-- F O R M S
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'UIFP', 'SELECT * FROM CLX_UI_FORM_PANELS WHERE FORMCODE=' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'UIFL', 'SELECT * FROM CLX_UI_FORM_LISTINGS WHERE FORMCODE=' . singleQuote($record['IDCODE']));
        $this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote($record['ID']));
        //Packing Dictionary
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_UI_FORM_PANELS X WHERE (DICT.ENTRY=X.TITLE OR DICT.ENTRY=X.CAPTION) AND X.FORMCODE=' . singleQuote($record['IDCODE']));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);

        $formcode = $record['IDCODE'];
        if (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($formcode)) > CLX_NUM_ZERO) {
            ClxCommon::$db->updateRecord($this->tablename, $record, 'IDCODE=' . singleQuote($formcode));
            $formid = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($formcode));
            $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode));
            while (($panel = ClxCommon::$db->getNextRecord($panels))) {
                $panelid = $panel['IDCODE'];
				$id_panelid=$panel['ID'];
                $fields = ClxCommon::$db->getRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($panelid));
                while ($field = ClxCommon::$db->getNextRecord($fields)) {
                    ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFF') . ' AND OWNERID=' . singleQuote($field['ID']));
                }
				
				$fields = ClxCommon::$db->getRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID=' . singleQuote($id_panelid));
				while (($field = ClxCommon::$db->getNextRecord($fields))) {
					$pfid=$field['PFID'];
					ClxCommon::$db->deleteRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID=' . singleQuote($id));
				}
                ClxCommon::$db->deleteRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($panelid));
                ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFP') . ' AND OWNERID=' . singleQuote($panelid));
            }
            ClxCommon::$db->deleteRecords('CLX_UI_FORM_FLD_PROP', 'FORMCODE=' . singleQuote($formcode));
            ClxCommon::$db->deleteRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode));
			$listings = ClxCommon::$db->getRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($formcode));
            while (($listingx = ClxCommon::$db->getNextRecord($listings))) {
				$listingid = $listingx['ID'];
				ClxCommon::$db->deleteRecords('CLX_UI_FORM_LISTING_CRITS', 'FORMLISTINGID=' . singleQuote($listingid));
            }
            ClxCommon::$db->deleteRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($formcode));
            ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFM') . ' AND OWNERID=' . singleQuote($formid));
            //ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($formcode));
        } else {
            $formid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        }
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
            $objs = $package->getObjectXml($this->code . '_OBJECTS');
            foreach ($objs->objects as $o) {
                $tool = &getTool($o->tag);
                $tool->unpack(array('package' => $o, 'id' => $formid));
            }
        }
    }

    function action($ac, $idlist) {
		IF (ISSET($_SESSION['ADDRDEBUGUIFORMS']) && $_SESSION['ADDRDEBUGUIFORMS']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}	
        if ($this->getAccess()->canRead()) {
            switch ($ac) {
                case 'LIST_AJAX'://sdded to enable list from form security
                    $DBLT = &getTool('DBLT');
                    $contents = $DBLT->action($ac, $idlist);
                    break;
                case 'PREVIEW'://sdded to enable list from form security
                    $contents = $this->userPreviewForm(UserRequest::$request['fm']);
                    break;
                default:
                    $contents = parent::action($ac, $idlist);
                    break;
            }
        } else {
            $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_FORMS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        ClxFreeReturn($listingcode);
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'UIFMList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		$table = new htmlTable((ISSET($_SESSION['ADDRDEBUGFORM']))?CLX_WIDTH_NINE:CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFM_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => (ISSET($_SESSION['ADDRDEBUGFORM']))?CLX_WIDTH_NINE:CLX_WIDTH_EIGHT));
        $page->header($table);
		$arreglocells=array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_DEFAULT'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION'));
      
		if (ISSET($_SESSION['ADDRDEBUGFORM'])) $arreglocells[]='list';
		$table->addCells(CLX_CLASS_HEADERS,$arreglocells );
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
            $recordid = $record['ID'];
            $select = new htmlCheckbox('selector_' . $recordid, false);
            $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
            $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			
			$arreglocells=array($record['IDCODE'], $record['IDNAME'], $record['MODULECODE'], $record['TABLECODE'], dict($this->yesno[$record['DEFAULT_FORM']]), $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION']));
			if (ISSET($_SESSION['ADDRDEBUGFORM'])) {
				$listings = ClxCommon::$db->getRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($record['IDCODE']));
				$info='';
				while ($listingrec = ClxCommon::$db->getNextRecord($listings)) {
					$info.=' '.$listingrec['LISTINGCODE'];
				}
				if (empty($info))
					$info='&nbsp;';
				$arreglocells[]=$info;           
			}
		    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT,$arreglocells );
            $i++;
        }

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
            $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_FORM')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFM_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UIFM_idcode', (!empty(UserRequest::$post['UIFM_idcode'])) ? UserRequest::$post['UIFM_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFM_idname', (!empty(UserRequest::$post['UIFM_idname'])) ? UserRequest::$post['UIFM_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'UIFM_tablecode', (!empty(UserRequest::$post['UIFM_tablecode'])) ? UserRequest::$post['UIFM_tablecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UIFM_modulecode', (!empty(UserRequest::$post['UIFM_modulecode'])) ? UserRequest::$post['UIFM_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (!empty(UserRequest::$post['confirm'])) {
            $table = new htmlTable(CLX_WIDTH_TWO);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $criteria = '';
            $boolean = '';
            if (UserRequest::$post['UIFM_idcode'] <> '') {
                $criteria .= $boolean . 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_idcode']) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['UIFM_idname'] <> '') {
                $criteria .= $boolean . 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_idname']) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['UIFM_modulecode'] <> '') {
                $criteria .= $boolean . 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_modulecode']) . '%');
                $boolean = ' AND ';
            }
            if (UserRequest::$post['UIFM_tablecode'] <> '') {
                $criteria .= $boolean . 'UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_tablecode']) . '%');
                $boolean = ' AND ';
            }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
            if ($qty == CLX_NUM_ZERO) {
                $table->addCell(dict('TOOL_UIFM_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
            } else {
                $table->addCell(dict('TOOL_UIFM_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
                $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
                $i = CLX_INIT_ZERO;
                while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
                    $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
                    $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ui_forms></page><a href=' . singlequote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
                    $i++;
                }
            }
            $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY') ? true : false;
        $title = ($modify) ? dict('TOOL_UIFM_TITLE_MODIFY') : dict('TOOL_UIFM_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UIFM_id', $record['UIFM_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('UIFM_numcols', $record['UIFM_numcols']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
            $table->addData(dict('GEN_LBL_CODE') . ':', $record['UIFM_idcode']);
            $form->addArgument('UIFM_idcode', $record['UIFM_idcode']);
        } else {
            $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UIFM_idcode', $record['UIFM_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFM_idname', $record['UIFM_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'UIFM_version', $record['UIFM_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'UIFM_description', $record['UIFM_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $DBTB = &getTool('DBTB');
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_TABLECODE') . ':', 'UIFM_tablecode', $record['UIFM_tablecode'], $table);
        $form->addArgument('UIFM_querycode', '');
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('GEN_LBL_MODULE') . ':', 'UIFM_modulecode', $record['UIFM_modulecode'], $table);
        $table->addYesNo(dict('GEN_LBL_DEFAULTFORM'), 'UIFM_default', $record['UIFM_default'], CLX_NUM_ZERO, false);
        $table->addField($msg, dict('GEN_LBL_NUMCOL') . ':', 'UIFM_numcols', $record['UIFM_numcols'], CLX_NUM_TWO, CLX_NUM_TWO);
        $form->addContents($table->source());
        $form->addContents('<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => '', 'form' => $form->getProp(CLX_PROP_ID)))->source();

        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        if (empty($msg))
            $msg = array();
        $action = UserRequest::$post['ac'];
        xValidate::required($msg, 'UIFM_idcode', UserRequest::$post['UIFM_idcode']);
        xValidate::required($msg, 'UIFM_idname', UserRequest::$post['UIFM_idname']);
        if ($action !== 'MODIFY') {
            xValidate::unique($msg, 'UIFM_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['UIFM_idcode']));
        }
        $DBTB = &getTool('DBTB');
        if (!$DBTB->tableExists(UserRequest::$post['UIFM_tablecode'])) {
            $msg['UIFM_tablecode'] = dict('GEN_ERR_TABLE_DOES_NOT_EXISTS');
        }

        $DBMO = &getTool('DBMO');
        $DBMO->recordExists('UIFM_modulecode', UserRequest::$post['UIFM_modulecode'], $msg);

        if (strlen(UserRequest::$post['UIFM_querycode']) > CLX_NUM_ZERO) {
            $DBQY = &getTool('DBQY');
            if (!$DBQY->queryExists(UserRequest::$post['UIFM_querycode'])) {
                $msg['UIFM_querycode'] = dict('TOOL_UILG_ERR_NOQUERYCODE');
            }
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['UIFM_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIFM_idname']);
    }

    function viewFormFill($record) {
        addHistory('formcode', $record['UIFM_idcode']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record['UIFM_id']);
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UIFM_TITLE_VIEW') . ': ' . $record['UIFM_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['UIFM_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UIFM_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['UIFM_version'] . '-' . $record['UIFM_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['UIFM_description']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['UIFM_modulecode']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['UIFM_tablecode']);
        $table->addData(dict('GEN_LBL_DEFAULTFORM') . ':', $yesno[$record['UIFM_default']]);
        $table->addData(dict('GEN_LBL_NUMCOL') . ':', $record['UIFM_numcols']);
        $a = new htmlAnchor(dict('GEN_LBL_CLICK_HERE_PREVIEW'), 'pop.php?to=' . $this->code . '&ac=PREVIEW&fm=' . $record['UIFM_idcode']);
        $a->setProp(CLX_PROP_TARGET, 'blank');
        $table->addCell($a->source(), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => '2'));
        $contents .= $table->source();
        $UIFP = &getTool('UIFP');
        $contents .= $UIFP->makepanel($record['UIFM_idcode']);
        $UIFL = &getTool('UIFL');
        $contents .= $UIFL->makepanel($record['UIFM_idcode']);
        $EVNT = &getTool('EVNT');
        $contents .= $EVNT->makepanel($this->code, $record['UIFM_id']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['UIFM_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['UIFM_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['UIFM_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('UIFM_idcode', UserRequest::$post))
            UserRequest::$post['UIFM_idcode'] = '';
        if (!array_key_exists('UIFM_idname', UserRequest::$post))
            UserRequest::$post['UIFM_idname'] = '';
        if (!array_key_exists('UIFM_modulecode', UserRequest::$post))
            UserRequest::$post['UIFM_modulecode'] = '';
        if (!array_key_exists('UIFM_tablecode', UserRequest::$post))
            UserRequest::$post['UIFM_tablecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UIFM_idcode', UserRequest::$post['UIFM_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UIFM_idname', UserRequest::$post['UIFM_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UIFM_modulecode', UserRequest::$post['UIFM_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'UIFM_tablecode', UserRequest::$post['UIFM_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('UIFM_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $sort->addItem(dict('GEN_LBL_TABLECODE'), 'tablecode', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['UIFM_idcode'])) {
            $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_idcode']) . '%');
            $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['UIFM_idname'])) {
            $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['UIFM_idname'])) . '%');
            $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['UIFM_modulecode'])) {
            $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_modulecode']) . '%');
            $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['UIFM_tablecode'])) {
            $filter .= $boolean . ' UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UIFM_tablecode']) . '%');
            $boolean = ' AND ';
        }
        if (array_key_exists('UIFM_sort', UserRequest::$post)) {
            $_SESSION['paging']['UIFMList']['sortby'] = UserRequest::$post['UIFM_sort'];
        }

        $_SESSION['paging']['UIFMList']['criteria'] = $filter;
        $_SESSION['paging']['UIFMList']['startat'] = '0';
        $_SESSION['paging'][$this->code . 'List']['qfilter'] = UserRequest::$post;
        unset($_SESSION['paging'][$this->code . 'List']['qfilter'][CLX_PROP_TO], $_SESSION['paging'][$this->code . 'List']['qfilter']['ac'], $_SESSION['paging'][$this->code . 'List']['qfilter']['qy'], $_SESSION['paging'][$this->code . 'List']['qfilter']['fm'], $_SESSION['paging'][$this->code . 'List']['qfilter']['li'], $_SESSION['paging'][$this->code . 'List']['qfilter']['obj_uid'], $_SESSION['paging'][$this->code . 'List']['qfilter']['startat'], $_SESSION['paging'][$this->code . 'List']['qfilter']['pc'], $_SESSION['paging'][$this->code . 'List']['qfilter']['nofr'], $_SESSION['paging'][$this->code . 'List']['qfilter']['pfid'], $_SESSION['paging'][$this->code . 'List']['qfilter']['rf'], $_SESSION['paging'][$this->code . 'List']['qfilter']['rfi'], $_SESSION['paging'][$this->code . 'List']['qfilter']['submit1'], $_SESSION['paging'][$this->code . 'List']['qfilter']['confirm'], $_SESSION['paging'][$this->code . 'List']['qfilter']['idlist']);
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['UIFMList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIFM_id'] = $recordid;
        $record['UIFM_idcode'] = $values['IDCODE'];
        $record['UIFM_idname'] = $values['IDNAME'];
        $record['UIFM_version'] = $values['VERSION'];
        $record['UIFM_version_counter'] = $values['VERSION_COUNTER'];
        $record['UIFM_cdate'] = xFormat::date($values['CDATE']);
        $record['UIFM_mdate'] = xFormat::date($values['MDATE']);
        $record['UIFM_username'] = $values['USERNAME'];
        $record['UIFM_description'] = $values['DESCRIPTION'];
        $record['UIFM_modulecode'] = $values['MODULECODE'];
        $record['UIFM_tablecode'] = $values['TABLECODE'];
        $record['UIFM_querycode'] = $values['QUERYCODE'];
        $record['UIFM_default'] = $values['DEFAULT_FORM'];
        $record['UIFM_numcols'] = $values['NUMCOLS'];
        return $record;
    }

    /**
     * Returns raw form record
     * @param array $formcode
     */
    function getFormRecord($formcode) {
        return ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($formcode));
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UIFM_idname'];
        $record['VERSION'] = UserRequest::$post['UIFM_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFM_description'];
        $record['QUERYCODE'] = UserRequest::$post['UIFM_querycode'];
        $record['TABLECODE'] = UserRequest::$post['UIFM_tablecode'];
        $record['MODULECODE'] = UserRequest::$post['UIFM_modulecode'];
        $record['DEFAULT_FORM'] = UserRequest::$post['UIFM_default'];
        $record['NUMCOLS'] = UserRequest::$post['UIFM_numcols'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIFM_id']);
        $this->versionIncrease(CLX_PROP_ID, UserRequest::$post['UIFM_id']);
        if (UserRequest::$post['UIFM_default'])
            $this->setDefaultForm(UserRequest::$post['UIFM_idcode']);
    }

    function createRecord_() {

        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['UIFM_idcode'];
        $record['IDNAME'] = UserRequest::$post['UIFM_idname'];
        $record['VERSION'] = UserRequest::$post['UIFM_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['UIFM_description'];
        $record['QUERYCODE'] = UserRequest::$post['UIFM_querycode'];
        $record['TABLECODE'] = UserRequest::$post['UIFM_tablecode'];
        $record['NUMCOLS'] = UserRequest::$post['UIFM_numcols'];
        if (!empty(UserRequest::$post['UIFM_modulecode'])) {
            $record['MODULECODE'] = UserRequest::$post['UIFM_modulecode'];
        } else {
            $record['MODULECODE'] = ClxCommon::$db->getField('CLX_DB_TABLES', 'MODULECODE', 'IDCODE=' . singleQuote(UserRequest::$post['UIFM_tablecode']));
        }
        $record['DEFAULT_FORM'] = UserRequest::$post['UIFM_default'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if (UserRequest::$post['UIFM_default'])
            $this->setDefaultForm(UserRequest::$post['UIFM_idcode']);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        return $recordid;
    }

    /**
     * Duplicates record
     * @return type
     */
    function duplicateRecord() {

        $formid = $this->createRecord_();
        $formcode = UserRequest::$post['UIFM_idcode'];
        $originalformid = UserRequest::$post['UIFM_id'];
        $originalformrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalformid);
        $originalformcode = $originalformrec['IDCODE'];
        //Duplicando criterios de formas
        $rs = ClxCommon::$db->getRecords('CLX_UI_FORM_FLD_PROP', 'FORMCODE=' . singleQuote($originalformcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            unset($r['ID']);
            $r['FORMCODE'] = $formcode;
            ClxCommon::$db->createRecordVoid('CLX_UI_FORM_FLD_PROP', $r, false);
        }

        $panelcounter = 0;
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($originalformcode) . ' ORDER BY SEQNUM ASC');
        while ($panelrec = ClxCommon::$db->getNextRecord($panels)) {
            $originalpanelid = $panelrec['IDCODE'];
            unset($panelrec['ID']);
            /* if (strpos($panelrec['IDCODE'], $originalformcode) === null) {
              $panelrec['IDCODE'] = $formcode . '_' . $panelrec['IDCODE'];
              } else {
              $panelrec['IDCODE'] = str_replace($originalformcode, $formcode, $panelrec['IDCODE']);
              } */
            $panelcounter++;
            $panelrec["IDCODE"] = 'P'.$formid.$panelcounter;
            $panelrec['FORMCODE'] = $formcode;
            $panelrec['TITLE'] = dict_parent_entry($panelrec['TITLE']);
            $panelrec['CAPTION'] = dict_parent_entry($panelrec['CAPTION']);
            ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANELS', $panelrec, false);
            $panelid = $panelrec['IDCODE'];
            $panelfields = ClxCommon::$db->getRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($originalpanelid));
            while ($fieldrec = ClxCommon::$db->getNextRecord($panelfields)) {
                $originalfieldid = $fieldrec['ID'];
                unset($fieldrec['ID']);
                $fieldrec['PANELID'] = $panelid;
                $fieldrec['FIELDFORMLABEL'] = dict_parent_entry($fieldrec['FIELDFORMLABEL']);
                $fieldrec['FIELDPREFIX'] = dict_parent_entry($fieldrec['FIELDPREFIX']);
                $fieldrec['FIELDSUFIX'] = dict_parent_entry($fieldrec['FIELDSUFIX']);
                $fieldrec['CAPTION'] = dict_parent_entry($fieldrec['CAPTION']);
                $fieldid = ClxCommon::$db->createRecord('CLX_UI_FORM_PANEL_FIELDS', $fieldrec, false);
                //events
                $events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFF'), 'OWNERID=' . singleQuote($originalfieldid));
                while ($eventrec = ClxCommon::$db->getNextRecord($events)) {
                    unset($eventrec['ID']);
                    $eventrec['OWNERID'] = $fieldid;
                    ClxCommon::$db->createRecordVoid('CLX_EVENTS', $eventrec, false);
                }
                //Search listing Criterias
                $rs = ClxCommon::$db->getRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID=' . $originalfieldid);
                while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                    unset($r['ID']);
                    $r['PFID'] = $fieldid;
                    ClxCommon::$db->createRecordVoid('CLX_UI_SRCH_LIST_CRITS', $r, false);
                }
            }
            $events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFP') . '', 'OWNERID=' . singleQuote($originalpanelid));
            while ($eventrec = ClxCommon::$db->getNextRecord($events)) {
                unset($eventrec['ID']);
                $eventrec['OWNERID'] = $panelid;
                ClxCommon::$db->createRecordVoid('CLX_EVENTS', $eventrec, false);
            }
        }
        //Duplicate Form Listings
        $listings = ClxCommon::$db->getRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($originalformcode));
        while ($listingrec = ClxCommon::$db->getNextRecord($listings)) {
            $oldformlistingid = $listingrec['ID'];
            unset($listingrec['ID']);
            $listingrec['FORMCODE'] = $formcode;
            $newformlistingid = ClxCommon::$db->createRecord('CLX_UI_FORM_LISTINGS', $listingrec, false);
            //Duplicate Form Listing Criterias
            $rs_crit= ClxCommon::$db->executequery($sql='SELECT * FROM CLX_UI_FORM_LISTING_CRITS WHERE FORMLISTINGID=' . singlequote( $oldformlistingid));
 			while ($list_crit = ClxCommon::$db->getNextRecord($rs_crit, ASSOCIATIVE)) {
                //FORMLISTINGID
                $list_crit['ID']=0;
                $list_crit['FORMLISTINGID'] = $newformlistingid;
                ClxCommon::$db->createRecordVoid('CLX_UI_FORM_LISTING_CRITS', $list_crit, false);
            }
        }
        $events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFM'), 'OWNERID=' . singleQuote($originalformid));
        while ($eventrec = ClxCommon::$db->getNextRecord($events)) {
            unset($eventrec['ID']);
            $eventrec['OWNERID'] = $formid;
            ClxCommon::$db->createRecordVoid('CLX_EVENTS', $eventrec, false);
        }
        return $formid;
    }

    function deleteRecordSk($id) {
        $UIFP = &getTool('UIFP');
        $formrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $formcode = $formrec['IDCODE'];
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode));
        while (($panel = ClxCommon::$db->getNextRecord($panels))) {
			$panelid = $panel['ID'];
			$fields = ClxCommon::$db->getRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($panelid));
			while ($field = ClxCommon::$db->getNextRecord($fields)) {
				ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFF') . ' AND OWNERID=' . singleQuote($field['ID']));
			}
			$fields = ClxCommon::$db->getRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID=' . singleQuote($panelid));
			while (($field = ClxCommon::$db->getNextRecord($fields))) {
				$pfid=$field['PFID'];
				ClxCommon::$db->deleteRecords('CLX_UI_SRCH_LIST_CRITS', 'PFID=' . singleQuote($id));
			}
            $UIFP->deleteRecordSk($panel['ID']);
			ClxCommon::$db->deleteRecords('CLX_UI_FORM_PANEL_FIELDS', 'PANELID=' . singleQuote($panelid));
			ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('UIFP') . ' AND OWNERID=' . singleQuote($panelid));
		}
		ClxCommon::$db->deleteRecords('CLX_UI_FORM_FLD_PROP', 'FORMCODE=' . singleQuote($formcode));
		ClxCommon::$db->deleteRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode));
		$listings = ClxCommon::$db->getRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($formcode));
		while (($listing = ClxCommon::$db->getNextRecord($listings))) {
			$listingID = $listing['ID'];
			ClxCommon::$db->deleteRecords('CLX_UI_FORM_LISTING_CRITS', 'FORMLISTINGID=' . $listingID);
        }
        ClxCommon::$db->deleteRecords('CLX_UI_FORM_LISTINGS', 'FORMCODE=' . singleQuote($formcode));
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote($id));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $formrec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['UIFM_id'] = CLX_INIT_ZERO;
        $record['UIFM_idcode'] = '';
        $record['UIFM_idname'] = '';
        $record['UIFM_version'] = '1.0.0';
        $record['UIFM_cdate'] = '';
        $record['UIFM_mdate'] = '';
        $record['UIFM_username'] = $_SESSION['user_record']['username'];
        $record['UIFM_description'] = '';
        $record['UIFM_modulecode'] = '';
        $record['UIFM_querycode'] = '';
        $record['UIFM_tablecode'] = '';
        $record['UIFM_default'] = '0';
        $record['UIFM_numcols'] = CLX_NUM_ONE;
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function setDefaultForm($formcode) {
        $tablecode = ClxCommon::$db->getField($this->tablename, 'TABLECODE', 'IDCODE=' . singleQuote($formcode));
        $record = array();
        $record['DEFAULT_FORM'] = '0';
        ClxCommon::$db->updateRecord($this->tablename, $record, 'TABLECODE=' . singleQuote($tablecode));
        $record = array();
        $record['DEFAULT_FORM'] = '1';
        ClxCommon::$db->updateRecord($this->tablename, $record, 'IDCODE=' . singleQuote($formcode));
    }

    function placeChooseForm($msg, $prompt, $name, $default, &$table) {

        $searchIcon = getSkinIconPop('view', dict('GEN_LBL_SEARCH'), $this->code, 'SEARCH', 'SEARCH');
        if (!empty($msg[$name])) {
            $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_16, CLX_NUM_16);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        if (!empty($msg) && $table->width > CLX_NUM_TWO) {
            for ($i = CLX_NUM_THREE; $i <= $table->width; $i++) {
                $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
            }
        }
        return '';
    }

    function formExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
            $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    function formTable($formcode, $tablecode) {
        $tc = ClxCommon::$db->getField($this->tablename, 'TABLECODE', 'IDCODE=' . singleQuote($formcode));
        return $tc == $tablecode;
    }

    function userPreviewForm($formcode) {
        $record = array();
        $formrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($formcode));
        $formcode = $formrec ['IDCODE'];
        $tablecode = $formrec ['TABLECODE'];
        $title = dict('GEN_LBL_FORM_PREVIEW');
        $buttonloc = $_SESSION ['prefs'] ['EDIT_BUTTON_LOCATION'];
        $action = '';
        $target = '';
        $msg = array();
        $EDIT = &getTool('EDIT');
        ClxFreeReturn($EDIT->getAppAccess($formrec ['TABLECODE'], true));
        // Create the Form
        $form = new htmlForm(CLX_PROP_ACTION, $action, CLX_PROP_TARGET, $target);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'PREVIEW');
        $form->addArgument('fm', $formcode);
        $form->addArgument('it', '');
        $form->addArgument('pc', '');
        $form->addArgument('confirm', '0');
        $form->addArgument('idlist', CLX_NUM_ZERO);
        $form->addArgument($tablecode . CLX_PROP_ID, (array_key_exists($tablecode . CLX_PROP_ID, $record)) ? $record [$tablecode . CLX_PROP_ID] : '' );
        // Buttons Table
        $btable = '';
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CLOSE'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons'));
        $btable = $table->source();
        if ($buttonloc <= CLX_NUM_TWO)
            $form->addContents($btable);
        // Call the form manager to draw the form...
        $UIFM = &getTool('UIFM');
        $form->addContents($UIFM->userEditForm($formrec, $title, $record, $msg, $form));
        if ($buttonloc >= CLX_NUM_TWO)
            $form->addContents($btable);
        return $form->source();
    }

    function userEditForm($formrec, $title, $record, $msg, &$form) {
		IF ($this->debug===true) { 
			pre( 'Metodo linea formrec:'. __METHOD__ .' '. __FILE__ .':'.__LINE__); 
			pre($formrec);
			pre( 'Metodo linea record:'. __METHOD__ .' '. __FILE__ .':'.__LINE__);
			pre($record);
			$pos=$posx=0;
			$te=debug_backtrace(); 
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"])){
					if ($t==$posx)
						echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";
					echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
				}
			}
		}       
		$EVNT = &getTool('EVNT');
        if (empty($formrec)) {
		  report_error(E_USER_ERROR, 'Missing form', 'edit.php', __LINE__, UserRequest::$request);
        }
        $UIFF = &getTool('UIFF');
		//PRE($formrec);
		//pre(UserRequest::$post);
		$EVNT->execute5('UIFM', $formrec ['ID'], 'INFO', $record, $msg);
		
        $formcode = $formrec['IDCODE'];
        $tablecode = $formrec['TABLECODE'];
        $contents = $this->getJavaScriptLibraries($formcode);
        //Adding FRLO events
        $contents .= $this->getJavaScriptEventsForm($formrec['ID']);
        if (!empty($title)) {
            $table_frame = new htmlTable(CLX_WIDTH_ONE);
            $table_frame->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
            $contents .= $table_frame->source();
        }
        //Incorporando javascripts de formas para criterios adicionales
        $UIFR = &getTool('UIFR');
        $contents .= $UIFR->getJavaScriptEventsForm($formrec['IDCODE']);
        $table_frame = new htmlTable($formrec['NUMCOLS']);
        $table_frame->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, 'cellspacing' => '5'));
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode) . ' AND ACTIVE = 1 ORDER BY SEQNUM');
        $panelpos = CLX_NUM_ONE;
        $panels_contents = array();
        while ($panelrec = ClxCommon::$db->getNextRecord($panels)) {

            if (!array_key_exists($panelpos, $panels_contents)) {
                $panels_contents[$panelpos] = '';
            }
            $panelid = $panelrec['IDCODE'];
            $tableWidth = $panelrec['NUMCOLS'];
            $table = new htmlTable(CLX_WIDTH_ONE);
            if (($panelrec['TYPE'] == CLX_NUM_ZERO) and ( !empty($panelrec['TITLE']))) {
                $table->addCell(dict($panelrec['TITLE']), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => $tableWidth));
            }
            if (!empty($panelrec['CAPTION'])) {
                if (empty($panelrec['CAPTION_CLASS'])) {
                    $panelrec['CAPTION_CLASS'] = '';
                }
                $table->addCell(dict($panelrec['CAPTION']), array(CLX_PROP_CLASS => $panelrec['CAPTION_CLASS'], CLX_PROP_COLSPAN => $tableWidth));
            }
            $panels_contents[$panelpos] .= $table->source();
            $table = new htmlTable($tableWidth);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, 'cellspacing' => '1'));
            if ($panelrec['PANEL_CLASS'] === '') {
                $panelrec['PANEL_CLASS'] = CLX_CLASS_FORM;
            }

            //Adding subform Panel
            if ($panelrec['FORM_OBJ_CODE'] <> '') {
                //Adding form panel object
                $subformtitle = dict($panelrec['TITLE']);
                $form_obj_rec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($panelrec['FORM_OBJ_CODE']));
                $table->addCell($this->userEditForm($form_obj_rec, $subformtitle, $record, $msg, $form), array(CLX_PROP_COLSPAN => $tableWidth));
            } else {
                $fields = $this->getFormPanelFields($panelid, false);
                $i = CLX_INIT_ZERO;
                $y = CLX_INIT_ZERO;
				while ($fieldrec = ClxCommon::$db->getNextRecord($fields, ASSOCIATIVE)) {
                   	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

					if ($fieldrec['PF_TYPE'] === 'SPACE') {  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                        //Just add plase Holder and continue
                        $lbl = dict($fieldrec['FIELDFORMLABEL']);
                        if ($lbl === '')
                            $lbl = CLX_CHAR_HTML_NBSP;
                        $table->addCell($lbl, array(CLX_PROP_CLASS => $fieldrec['CAPTION_CLASS']));
                        $y++;
                        continue;
                    }
					
                    if ($fieldrec['FIELDNAME'] === '' || $fieldrec['FIELDNAME'] === null) {  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                        $missingfl = ClxCommon::$db->getField('CLX_UI_FORM_PANEL_FIELDS', 'FIELDNAME', 'ID = ' . $fieldrec['PFID']);
                        //missing field drop error
                        $BUGS = &getTool('BUGS');
                        if (!empty($BUGS)) {
                            $BUGS->report(E_USER_ERROR, 'Missing field:  ' . $missingfl, 'tools/ui_forms.php', '976', UserRequest::$request, CLX_NUM_ONE, '');
                        }
                    }
                    $form_fieldname = '';
                    //Setting Tablecode from field
                    if ($fieldrec['FIELD_TABLECODE'] <> '') {  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                        $fieldrec['TABLECODE'] = $fieldrec['FIELD_TABLECODE'];
                        $form_fieldname = $fieldrec['FIELD_TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
                    } else {  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                        $fieldrec['TABLECODE'] = $tablecode;
                        $form_fieldname = $tablecode . '_' . $fieldrec['FIELDNAME'];
                    }
					//Adding user prefrences for default values
                    $UIFF->setUserPrefFieldDefault($form_fieldname, $fieldrec);  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                    //Setting Readonly
                    if ($fieldrec['FIELD_READONLY'] > CLX_NUM_ONE) {  	IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre('-------'. __LINE__ .'-->entrada') ;}

                        if (UserRequest::$request['ac'] === 'CREATE' && $fieldrec['FIELD_READONLY'] == CLX_NUM_TWO) {
                            $fieldrec['FIELD_READONLY'] = CLX_NUM_ONE;
                        } elseif ($fieldrec['FIELD_READONLY'] == CLX_NUM_TWO) {
                            $fieldrec['FIELD_READONLY'] = CLX_INIT_ZERO;
                        }
                        if ((UserRequest::$request['ac'] === 'MODIFY' || UserRequest::$request['ac'] === 'DUPLICATE_EFFTDATE') && $fieldrec['FIELD_READONLY'] == CLX_NUM_THREE) {
                            $fieldrec['FIELD_READONLY'] = CLX_NUM_ONE;
                        } elseif ($fieldrec['FIELD_READONLY'] == CLX_NUM_THREE) {
                            $fieldrec['FIELD_READONLY'] = CLX_INIT_ZERO;
                        }
                    }

                    $jsevents = $this->getJavaScriptEvents($fieldrec['PFID']);
                    $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
					IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  { pre( '<font color="green">'.__LINE__ ." ".$form_fieldname." ". $fieldrec['FIELDTYPE'].'</font>') ;}
                    $cell = $fieldinstance->placeEditField($msg, $form_fieldname, $fieldrec, $record, $form, $jsevents);
                    if ($fieldrec['EDIT_MODE'] === 'WYSIWYG') {
                        $pre = wysiwyg($form_fieldname, $fieldrec['FIELD_READONLY']);
                    } else {
                        $pre = '';
                    }
                    $table->addCell($pre . $cell->source(), array(CLX_PROP_CLASS => 'formdata'));
                    $i++;
                }//end while fields
                $dif = (($panelrec['NUMCOLS'] * (round($i / $panelrec['NUMCOLS']))) + $y) - $i;
                $dif = round($dif);
                for ($x = CLX_INIT_ZERO; $x < $dif; $x++) {
                    $table->addCell('', array(CLX_PROP_CLASS => 'formdata'));
                }
            }
            if ($panelrec['TYPE'] == CLX_NUM_ZERO) {
                $panels_contents[$panelpos] .= $table->source() . '<br>';
            } else {

                $panels_contents[$panelpos] .= buildPanelInterface(dict($panelrec['TITLE']), $panelrec['IDCODE'], $table->source(), ($panelrec['INITIAL_STATE'] == CLX_NUM_ONE)) . '<br>';
            }
            //swich amount panels
            if ($panelpos >= $formrec['NUMCOLS']) {
                $panelpos = CLX_NUM_ONE;
            } else {
                $panelpos++;
            }
        }//end while panels
		$wt=5;
		if (((int) $formrec['NUMCOLS'])>0)
			$wt = rounder((CLX_NUM_HUNDRED / (int) $formrec['NUMCOLS']), CLX_NUM_TWO) . '%';
        foreach ($panels_contents as $val) {
            $table_frame->addCell($val, array(CLX_PROP_WIDTH => $wt, 'valign' => 'top'));
        }
        $contents .= $table_frame->source();
        //Add mask js to every field that apply
        $UIMA = &getTool('UIMA');
        $contents .= $UIMA->getFormMasks($formcode); //return js scripts for masks
        return $contents;
    }

    function userCheckForm($formrec, &$record, &$msg) {
        $EVNT = &getTool('EVNT');
        $DBTI = &getTool('DBTI');
        if (empty($formrec) && !empty($record['fm'])) {
            $formrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($record['fm']));
        }
        if (empty($formrec)) {
            $formrec = array('IDCODE' => '', 'TABLECODE' => '');
        }
        $formcode = $formrec['IDCODE'];
        $tablecode = $formrec['TABLECODE'];
        $tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($tablecode));
        //Validating security if no access then return
        if (!empty($tablecode)) {
            $EDIT = &getTool('EDIT');
            $hascode = ((UserRequest::$request['ac'] === 'CODE' || UserRequest::$request['ac'] === 'REPORT_BUILD') && $EDIT->getAccess()->getCanRead() == CLX_NUM_ONE) ? CLX_NUM_ONE : CLX_NUM_ZERO;
            if (($hascode + $EDIT->getAccess()->getCanmodify() + $EDIT->getAccess()->getCandelete() + $EDIT->getAccess()->getDisplay() + $EDIT->getAccess()->getIsadmin()) == CLX_NUM_ZERO) {
                $msg['CLX_MSG'] = dict('GEN_ERR_NO_ACCESS') . ' [UI_FORMS-CAN_MODIFY|CAN_DELETE|CAN_APPEND|ISADMIN][USERCHECKFORM] ' . $this->getCode(false) . ' - Table: ' . $tablecode;
                addMessage($msg['CLX_MSG']);
            }
        }
        $fields = $this->getFormPanelFields2($formcode, $record, true);
        $EVNT->execute5('UIFM', $formrec['ID'], 'VAFO', $record, $msg); //  Validate Form
		if ( isset($_SESSION["_dEBUG"]) ) { 
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
				fwrite($tta,chr(13).chr(10)."uf0- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
				fwrite($tta,chr(13).chr(10));
				fclose($tta);
		}

        foreach ($fields as $fieldrec) {

            $panelid = $fieldrec['PANELID'];
            $form_fieldname = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
            $fieldexists = array_key_exists($form_fieldname, $record);
            $array_data = ($fieldexists === true && is_array($record[$form_fieldname])) ? true : false;
            $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
            $table = false;
            $form = false;
            if ( isset($_SESSION["_dEBUG"]) ) { 
					$te=debug_backtrace();
					$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
					fwrite($tta,chr(13).chr(10)."ufx- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
					fwrite($tta,chr(13).chr(10)."fieldrec:".var_export($fieldrec,true).chr(13).chr(10));
					fwrite($tta,chr(13).chr(10)."fieldname:".var_export($form_fieldname,true).chr(13).chr(10));
					fwrite($tta,chr(13).chr(10)."record:".var_export($record,true).chr(13).chr(10));
					for ($t=0;$t<18;$t++) {
						if (isset($te[$t]["file"]))
							fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


					}	
					fwrite($tta,chr(13).chr(10));
					fclose($tta);
			}
			IF ($this->debug==true) {
				echo "=====form_fieldname== in validateField=======";
				pre($form_fieldname);
				echo "=====fieldinstance=========";
				
				pre($fieldinstance);
				echo "=====fieldrec=========";
				pre($fieldrec);
				echo "=====pre msg==========";
				PRE($msg);
			}
            if(UserRequest::$request['ac']!='DELETE')
			    $fieldinstance->validateField($msg, $form_fieldname, $fieldrec, $record, $table, $form);
			IF ($this->debug==true) {
			
				echo "=====post msg==========";
				PRE($msg);
			}
			$EVNT->execute5('UIFF', $fieldrec['ID'], 'VAFI', $record, $msg); //  Validate Field
			
			$ideventVafi= ClxCommon::$db->getone($SQL='SELECT X.ID FROM CLX_UI_FORM_PANEL_FIELDS X,CLX_UI_FORM_PANELS Z ,CLX_EVENTS Y WHERE X.FIELDNAME='.singlequote($fieldrec['FIELDNAME']).' AND Z.FORMCODE='.singlequote($formcode).' AND X.PANELID=Z.IDCODE AND '.((kDatabaseType === 'oci') ? 'TO_CHAR(X.ID)' : 'X.ID').'=Y.OWNERID AND Y.EVENT='.singlequote('VAFI')); 
			if (!empty($ideventVafi))
				$EVNT->execute5('UIFF', $ideventVafi , 'VAFI', $record, $msg); //  Validate Field
            
            //Validate Masking
			
            $required = $fieldrec['FIELD_REQUIRED'];
            if ($fieldrec['FIELD_FORM_REQUIRED'] <> CLX_NUM_TWO)
                $required = $fieldrec['FIELD_FORM_REQUIRED'];
            if ($fieldexists === true) {
                if ($array_data === true) {
                    $datalength = strlen(toString($record[$form_fieldname], CLX_CHAR_SINGLE_QUOTE));
                } else {
                    $datalength = strlen($record[$form_fieldname]);
                }
            } else {
                $datalength = CLX_INIT_ZERO;
            }

            if (UserRequest::$request['ac']!='DELETE' && ($required == CLX_NUM_ONE || $datalength <> CLX_NUM_ZERO) && !empty($fieldrec['MASK_CODE'])) {

                $CODE = &getTool('CODE');
                $maskinfo = $CODE->codeGetSource($fieldrec['MASK_CODE']);

                //Masking Invalid
                //e cho 'A match was not found.';
                if ($array_data === true) {
                    foreach (!empty($maskinfo) && $record[$form_fieldname] as $k => $v) {
                        if (!preg_match($maskinfo, $v)) {

                            $msg[$form_fieldname] = dict('GEN_LBL_INVALIDMASK');
                            break;
                        }
                    }
                } else {

                    if (!empty($maskinfo) && !preg_match($maskinfo, $record[$form_fieldname])) {
                        $msg[$form_fieldname] = dict('GEN_LBL_INVALIDMASK');
                    }
                }

            }
        }

        $EVNT->execute5('UIFP', $panelid, 'VAPA', $record, $msg); //  Validate Panel
		if ( isset($_SESSION["_dEBUG"]) ) { 
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
				fwrite($tta,chr(13).chr(10)."uf1- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
				fwrite($tta,chr(13).chr(10));
				fclose($tta);
		}


        if (UserRequest::$request['ac'] <> 'CODE' && UserRequest::$request['ac'] <> 'REPORT_BUILD') {//Donot check actions that arent create
            $indexes = $DBTI->getTableIndexes($tablecode, 'UNI');
            foreach ($indexes as $key => $indexrec) {
                $DBTI->checkField($indexrec, $tablerec, $record, $msg);
            }
        }

        //Validacion de campos
        $UIFR = &getTool('UIFR');
		if ( isset($_SESSION["_dEBUG"]) ) { 
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
				fwrite($tta,chr(13).chr(10)."uf2- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
				fwrite($tta,chr(13).chr(10));
				fclose($tta);
		}
        $UIFR->userCheckForm($formrec, $record, $msg);
        if (UserRequest::$request['ac'] <> 'REPORT_BUILD') {
            $EVNT->execute5('DBTB', $tablerec['ID'], 'VREC', $record, $msg); //  Validate Record
        }
		if ( isset($_SESSION["_dEBUG"]) ) { 
				$te=debug_backtrace();
				$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");
				fwrite($tta,chr(13).chr(10)."uf3- hora:".date("h:i:s d/m/Y").chr(13).chr(10));
				fwrite($tta,chr(13).chr(10).var_export($msg,true).chr(13).chr(10));
				for ($t=0;$t<18;$t++) {
					if (isset($te[$t]["file"]))
						fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));


				}	
				fwrite($tta,chr(13).chr(10));
				fclose($tta);
		}
        UserRequest::$post = $record; //This is added to include record lookvalues into _POST for latter save
    }

    function userCheckQuickSearchForm($formrec, &$record, &$msg) {
        $EVNT = &getTool('EVNT');
        $formcode = $formrec['IDCODE'];
        $tablecode = $formrec['TABLECODE'];
        //Validating security if no access then return
        if (!empty($tablecode)) {
            $EDIT = &getTool('EDIT');
            $taccess = $EDIT->getAppAccess($tablecode, true);
            //FS  added QFILTER
            $hascode = ((UserRequest::$request['ac'] === 'CODE' || UserRequest::$request['ac'] === 'QFILTER') && $taccess->canRead()) ? CLX_NUM_ONE : CLX_NUM_ZERO;
            if (($hascode + $taccess->canModify() + $taccess->canDelete() + $taccess->canDisplay() + $taccess->isAdmin()) == CLX_NUM_ZERO) {
                $msg['CLX_MSG'] = dict('GEN_ERR_NO_ACCESS') . ' [UI_FORMS-CAN_MODIFY|CAN_DELETE|CAN_APPEND|ISADMIN][USERCHECKFORM] ' . $this->getCode(false) . ' - Table: ' . $tablecode;
                addMessage($msg['CLX_MSG']);
            }
        }
        $EVNT->execute5('UIFM', $formrec['ID'], 'VAFO', $record, $msg); //  Validate Form
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode) . ' AND ACTIVE = 1 ORDER BY SEQNUM');
        while (($panelrec = ClxCommon::$db->getNextRecord($panels))) {
            $panelid = $panelrec['IDCODE'];
            $fields = $this->getFormPanelFields($panelid, true);
            while (($fieldrec = ClxCommon::$db->getNextRecord($fields))) {
                if ($fieldrec['FIELD_TABLECODE'] <> '')
                    $fieldrec['TABLECODE'] = $fieldrec['FIELD_TABLECODE'];
                if ($fieldrec['TABLECODE'] === '')
                    $fieldrec['TABLECODE'] = $tablecode;
                $form_fieldname = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
                $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
                $table = false;
                $form = false;
                $fieldinstance->validateField($msg, $form_fieldname, $fieldrec, $record, $table, $form);
                $EVNT->execute5('UIFF', $fieldrec['ID'], 'VAFI', $record, $msg); //  Validate Field
                //Validate Masking
                ClxFreeReturn($fieldrec['FIELD_REQUIRED']);
            }
        }
    }

    function userViewForm($formrec, $title, $record) {
        //	$callingpage = peekHistory();
        $contents = '';
        $formcode = $formrec['IDCODE'];
        $tablecode = $formrec['TABLECODE'];
        $tableWidth = $formrec['NUMCOLS'];
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => 'gen_title', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($title)) {
            $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => $tableWidth));
        }
        $contents .= $table->source();
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode) . ' AND ACTIVE = 1 ORDER BY SEQNUM');
        while (($panelrec = ClxCommon::$db->getNextRecord($panels))) {
            $panelid = $panelrec['IDCODE'];
            $tableWidth = $panelrec['NUMCOLS'];
            $table = new htmlTable($tableWidth);
            $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));
            $table = new htmlTable($tableWidth);
            $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));

            $panel_title = dict($panelrec['TITLE']);
            if (($panelrec['TYPE'] == CLX_NUM_ZERO) && (strlen($panel_title) > CLX_NUM_ZERO)) {
                $table->addCell($panel_title, array(CLX_PROP_CLASS => 'panel', CLX_PROP_COLSPAN => $tableWidth));
            }
            $cpt = dict($panelrec['CAPTION']);
            if (empty($panelrec['CAPTION_CLASS'])) {
                $panelrec['CAPTION_CLASS'] = '';
            }
            if (strlen($cpt) > CLX_NUM_ZERO) {
                $table->addCell($cpt, array(CLX_PROP_CLASS => $panelrec['CAPTION_CLASS'], CLX_PROP_COLSPAN => $tableWidth));
            }
            $contents .=$table->source();
            $table = new htmlTable($tableWidth);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $fields = $this->getFormPanelFields($panelid, false);
            $i = CLX_INIT_ZERO;
            while (($fieldrec = ClxCommon::$db->getNextRecord($fields))) {
                if ($fieldrec['PF_TYPE'] === 'SPACE') {
                    //Just add plase Holder and continue
                    $lbl = dict($fieldrec['FIELDFORMLABEL']);
                    if ($lbl === '')
                        $lbl = CLX_CHAR_HTML_NBSP;
                    $table->addCell($lbl, array(CLX_PROP_CLASS => $fieldrec['CAPTION_CLASS']));
                    continue;
                }
                $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
                if ($fieldrec['FIELD_TABLECODE'] <> '') {
                    $cell = $fieldinstance->placeViewField($fieldrec['FIELD_TABLECODE'] . '_' . $fieldrec['FIELDNAME'], $fieldrec, $record);
                } else {
                    $cell = $fieldinstance->placeViewField($tablecode . '_' . $fieldrec['FIELDNAME'], $fieldrec, $record);
                }
                $table->addCell($cell->source(), array(CLX_PROP_CLASS => 'formdata'));
                $i++;
            }

            $dif = $i % $panelrec['NUMCOLS'];
            for ($x = CLX_INIT_ZERO; $x < $dif; $x++) {
                $table->addCell('', array(CLX_PROP_CLASS => 'formdata'));
            }
            if ($panelrec['TYPE'] == CLX_NUM_ZERO) {
                $contents .= $table->source();
            } else {
                $contents .= buildPanelInterface($panel_title, $panelrec['IDCODE'], $table->source(), ($panelrec['INITIAL_STATE'] == CLX_NUM_ONE));
            }
        }
        return $contents;
    }

    function userSearchForm($formrec, $title, $record) {
        //	$callingpage = peekHistory();
        $formcode = $formrec['IDCODE'];
        $tablecode = $formrec['TABLECODE'];
        $form = false;

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'gen_title'));
        if (!empty($title))
            $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $contents = $table->source();
        $panels = ClxCommon::$db->getRecords('CLX_UI_FORM_PANELS', 'FORMCODE=' . singleQuote($formcode) . ' ORDER BY seqnum');
        while ($panelrec = ClxCommon::$db->getNextRecord($panels)) {
            //Field not found on panel table. setting a default
            $panelrec['CAPTION_CLASS'] = 'caption';
            $panelid = $panelrec['IDCODE'];
            $tableWidth = $panelrec['NUMCOLS'];
            $table = new htmlTable($tableWidth);
            $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
            $panel_title = dict($panelrec['TITLE']);
            if (($panelrec['TYPE'] == CLX_NUM_ZERO) && ( strlen($panel_title) > CLX_NUM_ZERO)) {
                $table->addCell($panel_title, array(CLX_PROP_CLASS => 'panel', CLX_PROP_COLSPAN => $tableWidth));
            }
            $cpt = dict($panelrec['CAPTION']);
            if (strlen($cpt) > CLX_NUM_ZERO) {
                $table->addCell($cpt, array(CLX_PROP_CLASS => $panelrec['CAPTION_CLASS'], CLX_PROP_COLSPAN => $tableWidth));
            }
            $fields = $this->getFormPanelFields($panelid, false);
            $i = CLX_INIT_ZERO;
            while (($fieldrec = ClxCommon::$db->getNextRecord($fields))) {
                if ($fieldrec['PF_TYPE'] === 'SPACE') {
                    //Just add plase Holder and continue
                    $lbl = dict($fieldrec['FIELDFORMLABEL']);
                    if ($lbl === '')
                        $lbl = CLX_CHAR_HTML_NBSP;
                    $table->addCell($lbl, array(CLX_PROP_CLASS => $fieldrec['CAPTION_CLASS']));
                    continue;
                }
                $fieldrec['FIELD_DEFAULT'] = ''; //eliminada la opcion de valor por defecto en busquedas
                $fieldinstance = getToolObject('DBFT', $fieldrec['FIELDTYPE']);
                $cell = $fieldinstance->placeSearchField($tablecode . '_' . $fieldrec['FIELDNAME'], $fieldrec, $record, $form);
                $table->addCell($cell->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
                $i++;
            }
            $dif = $i % $panelrec['NUMCOLS'];
            for ($x = CLX_INIT_ZERO; $x < $dif; $x++) {
                $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
            }
            if ($panelrec['TYPE'] == CLX_NUM_ZERO) {
                $contents .= $table->source();
            } else {
                $contents .= buildPanelInterface(dict($panelrec['IDNAME']), $panelrec['IDCODE'], $table->source(), ($panelrec['INITIAL_STATE'] == CLX_NUM_ONE));
            }
        }
        return $contents;
    }

    function getFormPanelFields2($formcode, &$record, $fieldsonly = true) {
        $sql = 'SELECT FP.IDCODE PANELID, FP.FORMCODE,PF.ID PFID, F.ID, F.VERSION, F.VERSION_COUNTER, F.CDATE, F.MDATE, F.USERNAME, F.DESCRIPTION,
		    F.FIELDNAME, PF.FIELDFORMLABEL, F.FIELDLISTLABEL, PF.FIELDPREFIX, PF.FIELDSUFIX, F.FIELDTYPE,
		    F.FIELD_ANCHOR, F.FIELD_SIZE FIELD_DB_SIZE, F.FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN,
		    PF.FIELD_DEFAULT, F.FIELD_MINVALUE, F.FIELD_MAXVALUE, F.FIELD_DISPLAYSIZE FIELD_DB_DISPLAYSIZE,
		    PF.FIELD_DISPLAYSIZE, PF.FIELD_SIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4,
		    F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9, PF.FIELD_READONLY,
		    PF.FIELD_PASSWORD, PF.FIELD_HIDDEN, PF.FIELD_REQUIRED FIELD_FORM_REQUIRED, PF.FIELD_RELATED FIELD_RELATED,
		    PF.PRELOAD_LIST, PF.CAPTION, PF.CAPTION_CLASS, FM.TABLECODE, PF.TABLECODE FIELD_TABLECODE, PF.MASK_CODE,
		    PF.EDIT_MODE, PF.SRCH_LISTING, PF.PF_TYPE, PF.USE_USR_DEFAULT, PF.FIELD_PARENT, PF.ADD_LIST_ENTRY
		FROM CLX_UI_FORMS FM
		JOIN CLX_UI_FORM_PANELS FP ON FM.IDCODE = FP.FORMCODE
		JOIN CLX_UI_FORM_PANEL_FIELDS PF ON FP.IDCODE = PF.PANELID
		JOIN CLX_DB_FIELDS F ON PF.FIELDNAME=F.FIELDNAME
		WHERE FP.FORMCODE=' . singleQuote($formcode) . ' AND FP.ACTIVE = 1 AND PF.ACTIVE = 1  ' . (($fieldsonly) ? ' AND PF.PF_TYPE = ' . singleQuote('FIELD') : '') . '
		ORDER BY FP.SEQNUM, PF.FIELDSEQNUM ';
        IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR']) {
			pre( "==============================");$pos=$posx=0;
			$te=debug_backtrace(); 
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"])){
					if ($t==$posx)
						echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";
					echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
				}
			}
			echo '<font color="blue">getFormPanelFields2:'. print_r ($sql,true).':</font><br>';}			

        //$sql = ' SELECT PF.ID PFID, F.ID, F.VERSION, F.VERSION_COUNTER, F.CDATE, F.MDATE, F.USERNAME, F.DESCRIPTION, F.FIELDNAME, PF.FIELDFORMLABEL, F.FIELDLISTLABEL, PF.FIELDPREFIX, PF.FIELDSUFIX, F.FIELDTYPE, F.FIELD_ANCHOR, F.FIELD_SIZE FIELD_DB_SIZE, F.FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN, PF.FIELD_DEFAULT, F.FIELD_MINVALUE, F.FIELD_MAXVALUE, F.FIELD_DISPLAYSIZE FIELD_DB_DISPLAYSIZE, PF.FIELD_DISPLAYSIZE, PF.FIELD_SIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9, PF.FIELD_READONLY, PF.FIELD_PASSWORD, PF.FIELD_HIDDEN, PF.FIELD_REQUIRED FIELD_FORM_REQUIRED, PF.FIELD_RELATED FIELD_RELATED, PF.PRELOAD_LIST, PF.CAPTION, PF.CAPTION_CLASS, FM.TABLECODE, PF.TABLECODE FIELD_TABLECODE, PF.MASK_CODE, PF.EDIT_MODE, PF.SRCH_LISTING, PF.PF_TYPE, PF.USE_USR_DEFAULT, PF.FIELD_PARENT, PF.ADD_LIST_ENTRY FROM CLX_UI_FORMS FM, CLX_UI_FORM_PANELS FP, CLX_UI_FORM_PANEL_FIELDS PF LEFT JOIN CLX_DB_FIELDS F ON PF.FIELDNAME=F.FIELDNAME WHERE FM.IDCODE = FP.FORMCODE AND FP.IDCODE = PF.PANELID AND PF.PANELID=' . singleQuote($panelid) . ' AND PF.ACTIVE = 1 ' . (($fieldsonly) ? ' AND PF.PF_TYPE = ' . singleQuote('FIELD') : '') . ' 			ORDER BY PF.FIELDSEQNUM';
        $fields = ClxCommon::$db->executeQuery($sql);
        $return = array();
        while (($fieldrec = ClxCommon::$db->getNextRecord($fields))) {
            if ($fieldrec['FIELD_TABLECODE'] <> '')
                $fieldrec['TABLECODE'] = $fieldrec['FIELD_TABLECODE'];
            if ($fieldrec['TABLECODE'] === '')
                $fieldrec['TABLECODE'] = $tablecode;
            $fieldinstance = getToolObject('DBFT', $fieldrec["FIELDTYPE"]);
            $form_fieldname = $fieldrec['TABLECODE'] . '_' . $fieldrec['FIELDNAME'];
            $fieldinstance->parseField($form_fieldname, $fieldrec, $record);
            $return[] = $fieldrec;
        }
        return $return;
    }

    function getFormPanelFields($panelid, $fieldsonly = true) {
        $sql = ' SELECT PF.ID PFID, F.ID, F.VERSION, F.VERSION_COUNTER, F.CDATE, F.MDATE, F.USERNAME, F.DESCRIPTION, F.FIELDNAME, PF.FIELDFORMLABEL, F.FIELDLISTLABEL, PF.FIELDPREFIX, PF.FIELDSUFIX, F.FIELDTYPE, F.FIELD_ANCHOR, F.FIELD_SIZE FIELD_DB_SIZE, F.FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN, PF.FIELD_DEFAULT, F.FIELD_MINVALUE, F.FIELD_MAXVALUE, F.FIELD_DISPLAYSIZE FIELD_DB_DISPLAYSIZE, PF.FIELD_DISPLAYSIZE, PF.FIELD_SIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9, PF.FIELD_READONLY, PF.FIELD_PASSWORD, PF.FIELD_HIDDEN, PF.FIELD_REQUIRED FIELD_FORM_REQUIRED, PF.FIELD_RELATED FIELD_RELATED, PF.PRELOAD_LIST, PF.CAPTION, PF.CAPTION_CLASS, FM.TABLECODE, PF.TABLECODE FIELD_TABLECODE, PF.MASK_CODE, PF.EDIT_MODE, PF.SRCH_LISTING, PF.PF_TYPE, PF.USE_USR_DEFAULT, PF.FIELD_PARENT, PF.ADD_LIST_ENTRY FROM CLX_UI_FORMS FM, CLX_UI_FORM_PANELS FP, CLX_UI_FORM_PANEL_FIELDS PF LEFT JOIN CLX_DB_FIELDS F ON PF.FIELDNAME=F.FIELDNAME WHERE FM.IDCODE = FP.FORMCODE AND FP.IDCODE = PF.PANELID AND PF.PANELID=' . singleQuote($panelid) . ' AND PF.ACTIVE = 1 ' . (($fieldsonly) ? ' AND PF.PF_TYPE = ' . singleQuote('FIELD') : '') . ' 			ORDER BY PF.FIELDSEQNUM';
        //pre($sql);
		IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR']) {
			pre( "==============================");$posx=$pos=0;
			$te=debug_backtrace(); 
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"])){
					if ($t==$posx)
						echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";
					echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
				}
			}
			
			echo '<font color="blue">getFormPanelFields:'. print_r ($sql,true).':</font><br>';}			
        return ClxCommon::$db->executeQuery($sql);
    }

// End method
    function getFormFields($formcode, $fieldsonly = true) {
        $sql = ' SELECT PF.ID PFID, F.ID, F.FIELDNAME, PF.FIELDFORMLABEL, F.FIELDLISTLABEL, PF.FIELDPREFIX, PF.FIELDSUFIX, F.FIELDTYPE, F.FIELD_ANCHOR, F.FIELD_SIZE FIELD_DB_SIZE, F.FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN, PF.FIELD_DEFAULT, F.FIELD_MINVALUE, F.FIELD_MAXVALUE, F.FIELD_DISPLAYSIZE FIELD_DB_DISPLAYSIZE, PF.FIELD_DISPLAYSIZE, PF.FIELD_SIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9, PF.FIELD_READONLY, PF.FIELD_PASSWORD, PF.FIELD_HIDDEN, PF.FIELD_REQUIRED FIELD_FORM_REQUIRED, PF.FIELD_RELATED FIELD_RELATED, PF.PRELOAD_LIST, PF.CAPTION, PF.CAPTION_CLASS, FM.TABLECODE, PF.TABLECODE FIELD_TABLECODE, PF.MASK_CODE, PF.EDIT_MODE, PF.SRCH_LISTING, PF.PF_TYPE, PF.USE_USR_DEFAULT, PF.FIELD_PARENT FROM CLX_UI_FORMS FM, CLX_UI_FORM_PANELS FP, CLX_UI_FORM_PANEL_FIELDS PF LEFT JOIN CLX_DB_FIELDS F ON PF.FIELDNAME=F.FIELDNAME WHERE FM.IDCODE = FP.FORMCODE AND FP.IDCODE = PF.PANELID AND FP.FORMCODE = ' . singleQuote($formcode) . ' AND PF.ACTIVE = 1 AND PF.ACTIVE = 1 ' . (($fieldsonly) ? ' AND PF.PF_TYPE = ' . singleQuote('FIELD') : "''") . ' 			ORDER BY PF.FIELDSEQNUM';
		IF (ISSET($_SESSION['ADDRDEBUGDICT']) && $_SESSION['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR']) {echo '<font color="blue">getFormFields:'. print_r ($sql,true).':</font><br>';}			
        return ClxCommon::$db->executeQuery($sql);
    }

// End method
    function getJavaScriptEvents($panelfieldid) {
        $EVNT = &getTool('EVNT');
        $codelist = $EVNT->getOwnerEventsByAction('UIFF', $panelfieldid, 'CODE');
        $fieldevents = array();
        foreach ($codelist as $key => $codeinfo) {
            if ($EVNT->events[$codeinfo['EVENT']]['language'] === 'JS') {
                $eventname = $EVNT->events[$codeinfo['EVENT']]['name'];
                $codeinfo['EXTRA2'] = stripslashes($codeinfo['EXTRA2']);
                if (array_key_exists($eventname, $fieldevents)) {
                    $fieldevents[$eventname].=' ' . $codeinfo['EXTRA2'];
                } else {
                    $fieldevents[$eventname] = $codeinfo['EXTRA2'];
                }
            }
        }
        return $fieldevents;
    }

    function getJavaScriptEventsForm($formid) {
        $sql = 'SELECT DISTINCT CLIB.TYPE, ' . ((kDatabaseType === 'oci') ? 'TO_CHAR(CLIB.CONTENTS)' : 'CLIB.CONTENTS') . ' CONTENTS, CLIB.FILENAME, ' . ((kDatabaseType === 'oci') ? 'TO_CHAR(EVT.EXTRA2)' : 'EVT.EXTRA2') . ' USER_CALL FROM CLX_EVENTS EVT, CLX_CODELIB CLIB WHERE EVT.OWNERCODE = ' . singleQuote('UIFM') . '  AND EVT.ACTION = ' . singleQuote('CODE') . '  AND EVT.OWNERCODE = ' . singleQuote('UIFM') . '  AND EVT.OWNERID = ' . singlequote($formid) . ' AND CLIB.IDCODE = EVT.EXTRA3 AND CLIB.LANGUAGE = ' . singleQuote('JS') . ' AND EVT.STATUS = 1';
        $rs = ClxCommon::$db->executeQuery($sql);
        $scripts_src = '';
        while (($script = ClxCommon::$db->getNextRecord($rs))) {
            if ($script['TYPE'] === 'LINK') {
                $script['CONTENTS'] = file_get_contents($script['FILENAME']);
            }
            $scripts_src .=stripslashes($script['CONTENTS']) . CLX_CHAR_NL . CLX_CHAR_NL;
            $scripts_src .=$script['USER_CALL'];
        }
        if ($scripts_src <> '') {
            $scripts_src = setJavaScriptTag($scripts_src, false);
        }
        return $scripts_src;
    }

    function getJavaScriptLibraries($formcode) {
        //'.((kDatabaseType==='oci')?'TO_CHAR(CLIB.CONTENTS)':'CLIB.CONTENTS') . ' CONTENTS,
//        $sql = ' SELECT DISTINCT CLIB.TYPE, ' . ((kDatabaseType === 'oci') ? 'TO_CHAR(CLIB.CONTENTS)' : 'CLIB.CONTENTS') . ' CONTENTS, CLIB.FILENAME FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS P, CLX_UI_FORM_PANEL_FIELDS FLD, CLX_EVENTS EVT, CLX_CODELIB CLIB WHERE F.IDCODE=' . singleQuote($formcode) . ' AND P.FORMCODE=F.IDCODE AND P.ACTIVE = 1 AND FLD.ACTIVE = 1 AND FLD.PANELID=P.IDCODE AND EVT.OWNERCODE=' . singleQuote('UIFF') . '  AND EVT.OWNERID=FLD.ID AND EVT.ACTION=' . singleQuote('CODE') . '  AND CLIB.IDCODE=EVT.EXTRA3 AND CLIB.LANGUAGE=' . singleQuote('JS') . '  AND EVT.STATUS = 1 ';
		IF (kDatabaseType === 'oci')
			$sql = ' SELECT CLIB.TYPE, CLIB.CONTENTS CONTENTS, CLIB.FILENAME FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS P, CLX_UI_FORM_PANEL_FIELDS FLD, CLX_EVENTS EVT, CLX_CODELIB CLIB WHERE F.IDCODE=' . singleQuote($formcode) . ' AND P.FORMCODE=F.IDCODE AND P.ACTIVE = 1 AND FLD.ACTIVE = 1 AND FLD.PANELID=P.IDCODE AND EVT.OWNERCODE=' . singleQuote('UIFF') . '  AND EVT.OWNERID=TO_CHAR(FLD.ID) AND EVT.ACTION=' . singleQuote('CODE') . '  AND CLIB.IDCODE=EVT.EXTRA3 AND CLIB.LANGUAGE=' . singleQuote('JS') . '  AND EVT.STATUS = 1 ';
		ELSE
			$sql = ' SELECT DISTINCT CLIB.TYPE, ' . ((kDatabaseType === 'oci') ? 'CLIB.CONTENTS' : 'CLIB.CONTENTS') . ' CONTENTS, CLIB.FILENAME FROM CLX_UI_FORMS F, CLX_UI_FORM_PANELS P, CLX_UI_FORM_PANEL_FIELDS FLD, CLX_EVENTS EVT, CLX_CODELIB CLIB WHERE F.IDCODE=' . singleQuote($formcode) . ' AND P.FORMCODE=F.IDCODE AND P.ACTIVE = 1 AND FLD.ACTIVE = 1 AND FLD.PANELID=P.IDCODE AND EVT.OWNERCODE=' . singleQuote('UIFF') . '  AND EVT.OWNERID=FLD.ID AND EVT.ACTION=' . singleQuote('CODE') . '  AND CLIB.IDCODE=EVT.EXTRA3 AND CLIB.LANGUAGE=' . singleQuote('JS') . '  AND EVT.STATUS = 1 ';
     
        $scripts = ClxCommon::$db->executeQuery($sql);
        $contents = '';
        while (($script = ClxCommon::$db->getNextRecord($scripts))) {
            if ($script['TYPE'] === 'LINK') {
                $script['CONTENTS'] = file_get_contents($script['FILENAME']);
            }
            $contents .= CLX_CHAR_NL . CLX_CHAR_NL . stripslashes($script['CONTENTS']);
        }
        if (empty($contents)) {
            return '';
        } else {
            return setJavaScriptTag($contents, false);
        }
    }

    function pagelet($code, $ac = null, $objectid = null) {
		$action = UserRequest::$server['PHP_SELF'];
        $ac = (!empty($ac)) ? $ac : UserRequest::$request['ac'];
        $to = (array_key_exists(CLX_PROP_TO, UserRequest::$request)) ? UserRequest::$request[CLX_PROP_TO] : '';
        $it = (array_key_exists('it', UserRequest::$request)) ? UserRequest::$request['it'] : '';
        $pc = (array_key_exists('pc', UserRequest::$request)) ? UserRequest::$request['pc'] : '';
        $title = '';
        $record = UserRequest::$request;
        $EDIT = &getTool('EDIT');
        $msg = array();
        $formrec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($code));
        if (UserRequest::$request['ac'] === 'DISPLAY') {
            $UIPC = &getTool('UIPC');
            $crits = ClxCommon::$db->getRecords($UIPC->tablename, 'OBJECTID = ' . $objectid);
            $whereclause = '';
            $boolean = '';
            while (($crit = ClxCommon::$db->getNextRecord($crits))) {
                if (array_key_exists($pc, $_SESSION['page_filters']) &&
                        array_key_exists($crit['FRM_FIELDNAME'], $_SESSION['page_filters'][$pc]))
                    $value = $_SESSION['page_filters'][$pc] [$crit['FRM_FIELDNAME']];
                else
                    continue;
                $whereclause.= $boolean . $UIPC->buildOperator($crit['OPERATOR'], $crit['QRY_FIELDNAME'], $value, false);
                $boolean = ' AND ';
            }
            $DBTB = &getTool('DBTB');
            $tablerec = $DBTB->getTable($formrec['TABLECODE']);
            $sql = 'SELECT ' . $tablerec['IDCODE'] . '.ID FROM ' . $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'] . ' ' . $tablerec['IDCODE'] . ' ';
            if ($whereclause <> '')
                $sql .= ' WHERE ' . $whereclause;
            $recordid = ClxCommon::$db->getOne($sql);
            if (empty($recordid)) {
                return note(dict('GEN_LBL_NORECORDFOUND'));
            }
            $record = $EDIT->loadToolRecord($tablerec, $recordid, true);
            $contents = $this->userViewForm($formrec, $title, $record);
        } elseif (UserRequest::$request['ac'] === 'QFILTER') {
            $buttons = array('QFILTER' => dict('GEN_LBL_FILTER'),
                'CLEAR' => dict('GEN_LBL_CLEAR_FILTER'));
            $form = $EDIT->defaultForm($formrec, $record, $msg, $ac, $to, $it, $action, CLX_CLASS_FORM, '', $buttons);
            $contents = $form->source();
        } else {
            $buttons = array('QFILTER' => dict('GEN_LBL_FILTER'),
                'CLEAR' => dict('GEN_LBL_CLEAR_FILTER'));
            $form = $EDIT->defaultForm($formrec, $record, $msg, $ac, $to, $it, $action, CLX_CLASS_FORM, '', $buttons);
            $contents = $form->source();
        }
        return $contents;
    }

// End method
}
