<?php

/**
 * UserRequest handler
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date:  08:53:00 -0400 (Fri, 04 Jun 2010) $
 * @version    	$Rev: 1854 $
 */
class UserRequest {

    public static $post = array();
    public static $get = array();
    public static $request = array();
    public static $cookies = array();
    public static $server = array();
    public static $files = array();
    public static $idlist = null;
    public static $file_upload_errcodes = array(
		0 => 'UPLOAD_ERR_OK',
		1 => 'UPLOAD_ERR_INI_SIZE',
		2 => 'UPLOAD_ERR_FORM_SIZE',
		3 => 'UPLOAD_ERR_PARTIAL',
		4 => 'UPLOAD_ERR_NO_FILE',
		6 => 'UPLOAD_ERR_NO_TMP_DIR',
		7 => 'UPLOAD_ERR_CANT_WRITE',
		8 => 'UPLOAD_ERR_EXTENSION'
	);

    /**
     * Default arguments
     * @var type array
     */
    public static $arg = FILTER_SANITIZE_FULL_SPECIAL_CHARS;

    /**
     * Returns array of request type
     * @param type $type
     * @param type $args
     * @return type array
     */
    public static function getArray($type = INPUT_POST, $args = null) {
        $ar = (isset($args)) ? $args : self::$arg;
        return filter_input_array($type, $ar, true);
    }

    /**
     * Get sanitized input
     * @param type $variable_name
     * @param type $type
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getInput($variable_name, $type, $filter = FILTER_DEFAULT, $options = null) {
        return filter_input($type, $variable_name, $filter, $options);
    }
    public static function setInput(&$variable_name,$inputname,$newvalue) {
		if (isset($variable_name[$inputname])) {
			$variable_name[$inputname]=$newvalue;
		}    
		
    }

    /**
     * Get Post from field
     * @param type $variable_name
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getPostField($variable_name, $filter = FILTER_DEFAULT, $options = null) {
        return self::getInput($variable_name, INPUT_POST, $filter, $options);
    }
    /**
     * Get field from get
     * @param type $variable_name
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getGetField($variable_name, $filter = FILTER_DEFAULT, $options = null) {
        return self::getInput($variable_name, INPUT_GET, $filter, $options);
    }

    /**
     * Returs input request
     * @param type $variable_name
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getRequestField($variable_name, $filter = FILTER_DEFAULT, $options = null) {
        return self::getInput($variable_name, INPUT_REQUEST, $filter, $options);
    }

    

    /**
     * Returs input request
     * @param type $variable_name
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getCookieField($variable_name, $filter = FILTER_DEFAULT, $options = null) {
        return self::getInput($variable_name, INPUT_COOKIE, $filter, $options);
    }

    /**
     * Get Server Variable
     * @param type $variable_name
     * @param type $filter
     * @param type $options
     * @return type
     */
    public static function getServerField($variable_name, $filter = FILTER_DEFAULT, $options = null) {
        return self::getInput($variable_name, INPUT_SERVER, $filter, $options);
    }

    /**
     * General Capture of request
     * @param type $filters
     */
    public static function sanitizeUserInput($filters = null) {

        if ($filters === -1) {
		  $filters = self::$arg;
        }

        self::$get = self::getArray(INPUT_GET, $filters);
        self::$post = self::getArray(INPUT_POST, $filters);
        if (empty(self::$get)) {
		  self::$get = array();
        }
        if (empty(self::$post)) {
		  self::$post = array();
        }
		foreach($_FILES as $key=>$value) {
			if (!isset(self::$post[$key])) {
				self::$post[$key]=$value["name"];
			}
			if (!isset(self::$get[$key])) {
				self::$get[$key]=$value["name"];
			}
		}	
		
        //self::$request = self::getArray(INPUT_REQUEST, $filters);
		self::setupperIdcodes();
        self::$request = array_merge(self::$get, self::$post);
        self::$cookies = self::getArray(INPUT_COOKIE, $filters);
        self::$server = self::getArray(INPUT_SERVER, $filters);
        self::setDefaultRequestItems();
		
        if (empty(self::$cookies)) {
		  self::$cookies = array();
        }
        if (empty(self::$server)) {
		  self::$server = array();
        }
    }
	private static function setupperIdcodes() {
		foreach (self::$get as $key=>&$value) {
			if (stripos(" ".$key,"IDCODE")>0) {
				$value=uppercase($value);
			}
		}
		foreach (self::$post as $key=>&$value) {
			if (stripos(" ".$key,"IDCODE")>0) {
				$value=uppercase($value);
			}
		}
		
	}
    private static function setDefaultRequestItems() {
        if (empty(self::$request)) {
		  self::$request = array();
        }
        if (empty(self::$request['ac'])) {
		  self::$request['ac'] = '';
        }
        if (empty(self::$request['idlist'])) {
		  self::$request['idlist'] = '';
        }
        if (empty(self::$request['to'])) {
		  self::$request['to'] = '';
        }
        if (empty(self::$request['mc'])) {
		  self::$request['mc'] = '';
        }
        if (empty(self::$request['relfm'])) {
		  self::$request['relfm'] = '';
        }
        self::$files = $_FILES;
    }

    /**
     * Resolves lookup values
     */
    public static function getLookUpValues() {
	
        foreach (UserRequest::$request as $key => $val) {
		  if (lowercase(substr($key, -CLX_NUM_FIVE, CLX_NUM_FIVE)) === '_prim') {//parse lookup
			 $form_fieldname = substr($key, CLX_NUM_ZERO, -CLX_NUM_FIVE);
			 $fieldname = substr($form_fieldname, CLX_NUM_FIVE);
			 getLookupValue($form_fieldname, $fieldname, UserRequest::$request);
			 UserRequest::$request[$form_fieldname] = clx_html_spchrs_decode(UserRequest::$request[$form_fieldname]);
			 if (array_key_exists($form_fieldname, UserRequest::$post)) {
					UserRequest::$post[$form_fieldname] = UserRequest::$request[$form_fieldname];
			 }
			 if (array_key_exists($form_fieldname, UserRequest::$get)) {
			     UserRequest::$get[$form_fieldname] = UserRequest::$request[$form_fieldname];
			 }
			 if (array_key_exists($form_fieldname . '_prim', UserRequest::$request)) {
			     UserRequest::$request[$form_fieldname . '_prim'] = clx_html_spchrs_decode(UserRequest::$request[$form_fieldname . '_prim']);
			     UserRequest::$post[$form_fieldname . '_prim'] = UserRequest::$request[$form_fieldname . '_prim'];
			     UserRequest::$get[$form_fieldname . '_prim'] = UserRequest::$request[$form_fieldname . '_prim'];
			 }
			 if (array_key_exists($form_fieldname . '_secd', UserRequest::$request)) {
			     UserRequest::$request[$form_fieldname . '_secd'] = clx_html_spchrs_decode(UserRequest::$request[$form_fieldname . '_secd']);
			     UserRequest::$post[$form_fieldname . '_secd'] = UserRequest::$request[$form_fieldname . '_secd'];
			     UserRequest::$get[$form_fieldname . '_secd'] = UserRequest::$request[$form_fieldname . '_secd'];
			 }

			 if (array_key_exists($form_fieldname . '_terc', UserRequest::$request)) {
			     UserRequest::$request[$form_fieldname . '_terc'] = clx_html_spchrs_decode(UserRequest::$request[$form_fieldname . '_terc']);
			     UserRequest::$post[$form_fieldname . '_terc'] = UserRequest::$request[$form_fieldname . '_terc'];
			     UserRequest::$get[$form_fieldname . '_terc'] = UserRequest::$request[$form_fieldname . '_terc'];
			 }

				foreach($_FILES as $key=>$value) {
					if (!isset(UserRequest::$request[$key])) {
						UserRequest::$request[$key]=$value[$name];
					}
					if (!isset(UserRequest::$post[$key])) {
						UserRequest::$post[$key]=$value[$name];
					}
					if (!isset(UserRequest::$get[$key])) {
						UserRequest::$get[$key]=$value[$name];
					}

				}	
				
		  }
        }
    }

}
