<?php

/**
 * Report Table Object
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2016-10-10 20:40:45 +0000 (Mon, 10 Oct 2016) $
 * @version        $Rev: 8796 $
 * @package		Clientix.Classes.Report
 * @abstract		Abstract Class
 */
class clx_report_object_table extends clx_report_object {

    /**
     * Table Columns
     * @var integer
     */
    private $width;

    /**
     * Columns prefixes
     * @var array
     */
    public $prefix;

    /**
     * Column Sufixes
     * @var array
     */
    public $sufix;

    /**
     * Column Labels
     * @var array
     */
    private $columns;

    /**
     * Row of data
     * @var array
     */
    private $data;
    private $column_sizes;
    private $count_width; //Count w>0
    private $count_height; //Count h>0
    private $total_width; //Add width>0
    public $total_height; //Add h>0

    private $header_visible;
    public $auto_width = true;
    var $disable_borders = false;
    private $ln = CLX_INIT_ZERO;
    private $ln_post = CLX_INIT_ZERO;
    private $sw = CLX_INIT_ZERO; //para no pintar la línea de abajo del título con la de data

    public $html_align = CLX_PROP_CENTER;
    public $html_border = CLX_NUM_ZERO;
    public $html_cellspacing = CLX_NUM_ZERO;
    public $html_cellpadding = CLX_NUM_ZERO;
    public $html_tbl_pre_br = false;
    public $html_tbl_post_br = false;

    private $pdfAnrpMode = false;
    /**
     * Constructor
     */

    function __construct() {
        $this->setObjectType('TBL');
        $this->showTableHeader();
        $this->width = CLX_INIT_ZERO;
    }

    /**
     * Returns columns
     */
    function getColumns() {
        return $this->columns;
    }

    /**
     * Returns the column at column number
     * @param unknown_type $colnumber
     * @return multitype:
     */
    function getColumn($colnumber) {
        return $this->columns[$colnumber];
    }

    function calculaAncho($w) {
        $ancho = CLX_INIT_ZERO;
        $total = count($w);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $ancho += $w[$i];
        }
        return $ancho;
    }

    /**
     * Returns array of column labels
     */
    function getColumnLabels() {
        //Add Columns
        $colsd = array();
        foreach ($this->columns as $cols) {
		  $colsd[] = utf8_decode($cols->sourceText());
        }
        return $colsd;
    }

    /**
     * Add Colums to table
     * @param $name
     * @param $type
     * @param $format
     */
    function addColumn(clx_report_data $column, $format = 'pdf') {
        $this->columns[] = $column;
        //Set colunm Dimmensions
        $this->count_width = CLX_INIT_ZERO;
        $this->count_height = CLX_INIT_ZERO;
        $this->width = count($this->columns);
        $last = ($this->width > CLX_NUM_ZERO) ? $this->width - CLX_NUM_ONE : CLX_NUM_ZERO;
        $this->setColumnDimensions($last, $column->getStyle()->getBoxWidth($format), $column->getStyle()->getBoxHeight($format));
        //Test if autoWithd could be enable
        $this->setAutoWidth();
    }

    /**
     * Enables auto with if all elements are 0
     */
    private function setAutoWidth() {
        $this->auto_width = true;
        $this->count_width = CLX_INIT_ZERO;
        $this->count_height = CLX_INIT_ZERO;
        foreach ($this->column_sizes as $val) {
		  if ($val['w'] > CLX_NUM_ZERO) {
			 $this->auto_width = false;
			 $this->count_width++;
		  }
		  if ($val['h'] > CLX_NUM_ZERO) {
			 $this->count_height++;
		  }
        }
    }

    /**
     * Sets total width
     */
    private function setTotalWidth() {
        $this->total_width = CLX_INIT_ZERO;
        foreach ($this->column_sizes as $val) {
		  if (!empty($val['w'])) {
			 $this->total_width += $val['w'];
		  }
        }
        $this->setAutoWidth();
    }

    /**
     * Sets total height
     */
    private function setTotalHeight() {
        $this->total_height = CLX_INIT_ZERO;
        foreach ($this->column_sizes as $val) {
		  if (!empty($val['h'])) {
			 $this->total_height += $val['h'];
		  }
        }
    }

    /**
     * Return Dimenssion at location
     * @param unknown_type $column_no
     * @param unknown_type $width
     * @param unknown_type $height
     */
    function setColumnDimensions($column_no, $width = CLX_NUM_ZERO, $height = CLX_NUM_ZERO) {
        $this->column_sizes[$column_no] = array('w' => $width, 'h' => $height);
        $this->setTotalWidth();
        $this->setTotalHeight();
    }

    /**
     * Changes the with of a colunm
     * @param unknown_type $column_no
     * @param unknown_type $width
     */
    function setColumnWidth($column_no, $width = CLX_NUM_ZERO) {
        $this->column_sizes[$column_no]['w'] = $width;
        $this->setTotalWidth();
    }

    /**
     * Returns column withd at location
     * @param integer $column_no
     * @return number
     */
    function getColumnWidth($column_no) {
        if (empty($this->column_sizes[$column_no]['w'])) {
		  return CLX_NUM_ZERO;
        } else {
		  return $this->column_sizes[$column_no]['w'];
        }
    }

    /**
     * Returns array of calumn withds
     * @return multitype:unknown
     */
    function getColumnWidths($max_page_with = CLX_NUM_ZERO) {
        $w = array();
		if (is_array($this->column_sizes)) { 
	        foreach ($this->column_sizes as $key => $val) {
			  //Tray
			  $v = $this->getColumnWidth($key);
			  if ($v == CLX_NUM_ZERO)
				 $v = ($max_page_with - $this->total_width) / ($this->getWidth() - ($this->count_width - CLX_NUM_ONE)); //Remove a column to allow space
			  $w[] = $v;
        	}
    	}
        return $w;
	}
    /**
     * Return Colum Alignment
     * @return array
     */
    function getColumnAligns() {
        $w = array();
		if (is_array($this->getColumns())) {
	        foreach ($this->getColumns() as $val) {
			  $w[] = $val->getStyle()->getFontAligment('pdf');
	        }
		}
        return $w;
    }

    /**
     * Return column Hights at location number
     * @param integer $column_no
     * @return number
     */
    function getColumnHeight($column_no) {
        if (empty($this->column_sizes[$column_no]['h'])) {
		  return CLX_NUM_ZERO;
        } else {
		  return $this->column_sizes[$column_no]['h'];
        }
    }

    /**
     * Returns width
     */
    function getWidth() {
        return $this->width;
    }

    /**
     * Sets table witdh
     * @param type $w
     */
    function setWidth($w) {
        $this->width = $w;
    }

    /**
     * Adds single row to table
     * @param $row
     */
    function addRow(clx_report_data $row) {
        $this->data[] = $row;
    }

    function addRows(clx_style $style, array $row) {
        foreach ($row as $val) {
		  $this->addRow(new clx_report_data($val, $style));
        }
    }

    /**
     * Add Colums Prefixes
     * @param $prefix
     */
    function addPrefix(array $prefix) {
        $this->prefix = $prefix;
    }

    /**
     * Add Colums Sufixes
     * @param $sufix
     */
    function addSufix(array $sufix) {
        $this->sufix = $sufix;
    }

    /**
     * Return table rows
     */
    function getData() {
        return $this->data;
    }

    function hideTableHeader() {
        $this->header_visible = false;
    }

    function showTableHeader() {
        $this->header_visible = true;
    }

    function setLn($num) {
        $this->ln = $num;
    }

    function setLnPost($num) {
        $this->ln_post = $num;
    }

    function getLn() {
        return $this->ln;
    }

    function getLnPost() {
        return $this->ln_post;
    }

    private function reest_index($w) {
        $return = array();
        $indices = array_keys($w);
        $total = count($indices);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
		  $return[] = $w[$indices[$i]];
        }
        return $return;
    }

    //FORMATING
    function sourceText() {
        $contents = '';
        $cell_delim = ' | ';
        $row_delim = CLX_CHAR_NL;
        //Adding Headers
        if (!empty($this->header_visible)) {
		  $comma = '';
		  foreach ($this->columns as $cols) {
			 $contents .= $comma . $cols->sourceText();
			 $comma = $cell_delim;
		  }
		  $contents .= $row_delim;
        }
        //Adding Rows
        $i = CLX_NUM_ONE;
        $comma = '';
        foreach ($this->getData() as $cell) {
		  $contents .= $comma . $cell->sourceText();
		  $comma = $cell_delim;
		  if ($i >= $this->width) {
			 $contents .= $row_delim;
			 $comma = '';
			 $i = CLX_INIT_ZERO;
		  }
		  $i++;
        }
        return $contents;
    }

    function sourceHtml() {
        $table = new htmlTable(count($this->columns));

        $table->setProp('style', 'table-layout:auto');
        $table->setProp('border', $this->html_border);
        $table->setProp('cellspacing', $this->html_cellspacing);
        $table->setProp('cellpadding',  $this->html_cellpadding);
        $table->setProp('align', $this->html_align);

        //Adding Headers
        if (!empty($this->header_visible)) {
		  foreach ($this->columns as $cols) {
			 $table->addCell($cols->sourceHtml(), array(CLX_PROP_COLSPAN => $cols->getStyle()->getBoxColSpan('html')));
		  }
        }
        //Adding Rows
        if(!empty($this->getData())){
	        foreach ($this->getData() as $cell) {
			  $table->addCell($cell->sourceHtml(), array(CLX_PROP_COLSPAN => $cell->getStyle()->getBoxColSpan('html')));
			  //$table->addCell($cell->sourceHtml(), array(CLX_PROP_COLSPAN => $cell->getStyle()->getBoxColSpan('html'), 'style'=>$cell->getStyle()->sourceHtml()));
	
	        }
        }
        return ((true === $this->html_tbl_pre_br)?'<br>':'').$table->source().((true === $this->html_tbl_post_br)?'<br>':'');
    }

    function sourceXml() {
        $table = new xmlObject('object', true);
        $table->setProp('type', $this->getType());
        $headers = new xmlObject(CLX_CLASS_HEADERS, true);
        //Adding Headers
        if (!empty($this->header_visible)) {
		  foreach ($this->columns as $cols) {
			 $xml_temp = new xmlObject('header', true);
			 $xml_temp->addContents($cols->sourceXml());
			 $headers->addObject($xml_temp);
		  }
        }
        $table->addObject($headers);
        //Adding Rows
        $data = new xmlObject('row', true);
        $i = CLX_NUM_ONE;
        foreach ($this->getData() as $cell) {
		  $xml_temp = new xmlObject('col', true);
		  $xml_temp->addContents($cell->sourceXml());
		  $data->addObject($xml_temp);
		  if ($i >= $this->width) {
			 $table->addObject($data);
			 $data = new xmlObject('row', true);
			 $i = CLX_INIT_ZERO;
		  }
		  $i++;
        }
        return $table;
    }

    function setPdfAnrpMode($mode){
        $this->pdfAnrpMode = (!empty($mode))?true:false;
    }

    function getPdfAnrpMode(){
        return $this->pdfAnrpMode;
    }

    private function sourcePdfAnrpRow(&$pdf) {

        //Set color
        /*$colstyle = $this->getColumn(CLX_NUM_ZERO)->getStyle();
        $colorFont = $colstyle->getFontColor('pdf');
        $bgcolor = $colstyle->getBoxBgColor('pdf');
        $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
        $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);*/
        $pdf->SetLineWidth(CLX_NUM_0_PT_1);
        $cols = count($this->getColumnWidths($pdf->w));
        $pdf->SetWidths($this->getColumnWidths($pdf->w));
        $pdf->SetAligns($this->getColumnAligns());

        //Adding headers
        $a = array(); $row = array();
        $i = 0;
		if (is_array($this->columns)) { 
	        foreach ($this->columns as $c) {
			  $row[] = $c->sourceText();
			  $a[] = $c->getStyle()->getFontAligment('pdf');
			  $i++;
			  if($i==$cols){
				 $colorFont = $c->getStyle()->getFontColor('pdf');
				 $bgcolor = $c->getStyle()->getBoxBgColor('pdf');
				 $color = $c->getStyle()->getBoxBorderColor('pdf');
				 $pdf->SetDrawColor($color['r'], $color['g'], $color['b']);
				 $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
				 $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
				 $pdf->SetFont($c->getStyle()->getFontFace(), $c->getStyle()->getFont('pdf'), $c->getStyle()->getFontSize('pdf'));
				 $pdf->SetAligns($a);
				 $pdf->Row($row, true);
				 $i=0;
				 $row = array();
				 $a = array();
			  }
	        }
		}
        $a = array(); $row = array();
        //Adding data
        $i = 0;
        $getData=$this->getData();
        if (!is_array($getData))
        	$getData=(array)$getData;
	
        foreach ($getData as $c) {

		  $row[] = $c->sourceText();
		  $a[] = $c->getStyle()->getFontAligment('pdf');
		  $i++;
		  $colspan = $c->getStyle()->getBoxColSpan('pdf');
		  for($t=1; $t<$colspan; $t++){
			 //$row[] = '';
			 $i++;
		  }

		  if($i==$cols){
			 $colorFont = $c->getStyle()->getFontColor('pdf');
			 $bgcolor = $c->getStyle()->getBoxBgColor('pdf');
			 $color = $c->getStyle()->getBoxBorderColor('pdf');
			 $pdf->SetDrawColor($color['r'], $color['g'], $color['b']);
			 $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			 $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			 $pdf->SetFont($c->getStyle()->getFontFace(), $c->getStyle()->getFont('pdf'), $c->getStyle()->getFontSize('pdf'));
			 $pdf->SetAligns($a);
			 $pdf->Row($row, true);
			 $i=0;
			 $row = array();
			 $a = array();
		  }
        }

    }

    function sourcePdf(&$pdf) {

        //User on reports
        if($this->getPdfAnrpMode()){
		  $this->sourcePdfAnrpRow($pdf);
		  return;
        }

        if (!empty($this->header_visible)) {
		  //contador de columnas
		  $contaColum = CLX_NUM_ONE;
		  //$pdf->SetLineWidth(2);
		  $pdf->SetLineWidth(CLX_NUM_0_PT_2);
		  //Set column withds
		  $pdf->SetWidths($this->getColumnWidths($pdf->w));
		  $anchoTotal = $pdf->GetTotalWidth();
		  $pdf->SetAligns($this->getColumnAligns());
		  //Setting a default
		  $colstyle = $this->getColumn(CLX_NUM_ZERO)->getStyle();

		  //Set color
		  $colorFont = $colstyle->getFontColor('pdf');
		  $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
		  $bgcolor = $colstyle->getBoxBgColor('pdf');
		  $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
		  $oldw = $pdf->GetWidths();
		  $w = array();
		  $colw = array();
		  $al = array();
		  $i = CLX_NUM_ZERO;
		  $cant_colum = CLX_INIT_ZERO;
		  $arrayRowSpan = array();
		  $colNum = CLX_INIT_ZERO;
		  //Add the Columns

		  foreach ($this->columns as $column) {
			 $altura = $column->getStyle()->getBoxHeight('pdf');
			 $colorFont = $column->getStyle()->getFontColor('pdf');

			 if ($column->getStyle()->getBoxBgColor('pdf'))
			 //$bgcolor =Array ( 'r' => 17, 'g' => 217 ,'b' => 187 ); // $column->getStyle()->getBoxBgColor('pdf');
			     $bgcolor = $column->getStyle()->getBoxBgColor('pdf');

			 if (!empty($this->disable_borders)) {
			     $form_border = CLX_INIT_ZERO;
			     $bord_color['r'] = $bgcolor['r'];
			     $bord_color['g'] = $bgcolor['g'];
			     $bord_color['b'] = $bgcolor['b'];
			 } else {
			     $form_border = $column->getStyle()->getBoxBorderForm('pdf');
			     $color = $column->getStyle()->getBoxBorderColor('pdf');
			     $bord_color['r'] = $color['r'];
			     $bord_color['g'] = $color['g'];
			     $bord_color['b'] = $color['b'];
			 }

			 $arrayAligns[] = $column->getStyle()->getFontAligment('pdf');
			 $arrayAltura[] = $altura;
			 $arrayFillR[] = CLX_NUM_ONE;
			 $arrayFuente[] = array('letra' => $column->getStyle()->getFontFace(),
			     'estilo' => $column->getStyle()->getFont('pdf'),
			     'tamano' => $column->getStyle()->getFontSize('pdf'),
			     'cR' => $colorFont['r'],
			     'cG' => $colorFont['g'],
			     'cB' => $colorFont['b'],
			     'fR' => $bgcolor['r'],
			     'fG' => $bgcolor['g'],
			     'fB' => $bgcolor['b'],
			     'bR' => $bord_color['r'],
			     'bG' => $bord_color['g'],
			     'bB' => $bord_color['b'],
			     'borde' => $form_border);
			 //pre($column->getStyle());

			 $colw[] = $column->sourceText();
			 $al[] = $column->getStyle()->getFontAligment('pdf');

			 
			 //Test for col span
			 $colspan = $column->getStyle()->getBoxColSpan('pdf');
			 if ($colspan == CLX_NUM_ZERO)
			     $colspan = CLX_NUM_ONE;
			 //Set new colspan
			 if ($colspan > CLX_NUM_ONE) {
			     $w[$i] = CLX_INIT_ZERO;
			     for ($a = CLX_INIT_ZERO; $a < $colspan; $a++) {
			         $w[$i] += $oldw[$i + $a];
			     }
			     $cant_colum = $cant_colum + $colspan;
			 } else {
			     $w[] = $oldw[$i];
			     $cant_colum++;
			 }
			 $i += $colspan;
			 if ($i >= $this->width) {
			     $i = CLX_INIT_ZERO;
			     //Set new cols to include span
			     $pdf->SetWidths($w);
			     //Get Style Info
			     $colstyle = $column->getStyle();
			     //Set back color
			     $colorFont = $column->getStyle()->getFontColor('pdf');
			     $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			     //Set Font
			     $pdf->SetFont($column->getStyle()->getFontFace(), $column->getStyle()->getFont('pdf'), $column->getStyle()->getFontSize('pdf'));
			     //Set alingments for row
			     $pdf->SetAligns($al);
			 }
			 $pdf->SetWidths($oldw);
			 $arrayRowSpan[$colNum]['cantidad'] = CLX_NUM_ONE;
			 $arrayRowSpan[$colNum][CLX_PROP_COLSPAN] = CLX_NUM_ONE;
			 $arrayRowSpan[$colNum]['marca'] = CLX_INIT_ZERO;
			 $colNum++;
		  }
		  $pdf->SetWidths($w);
		  $pdf->SetAligns($arrayAligns);
		  $pdf->SetAltura($arrayAltura);
		  $pdf->SetFillR($arrayFillR);
		  //Paint Row
		  //   pre($arrayFuente);
		  $pdf->Row1($colw, true, true, $this->getLn(), true, $arrayFuente, '', $arrayRowSpan);
		  $arrayAligns = array();
		  $arrayAltura = array();
		  $arrayFillR = array();
		  $arrayFuente = array();
		  $pdf->resetAltura();
		  $colw = array();
		  //Store withd to handle span, Try to complete elements in a different maner
		  $oldw = $pdf->GetWidths();
		  $w = array();
		  //Add data
		  $colw = array();
		  $al = array();
		  $i = CLX_NUM_ZERO;
		  $columnasRow = CLX_INIT_ZERO;
		  $colNum = CLX_INIT_ZERO;
		  foreach ($this->getData() as $row) {
			 $altura = $row->getStyle()->getBoxHeight('pdf') * $row->getStyle()->getBoxRowSpan('pdf');
			 $colorFont = $row->getStyle()->getFontColor('pdf');
			 if ($row->getStyle()->getBoxBgColor('pdf')) {
			     $bgcolor = $row->getStyle()->getBoxBgColor('pdf');
			 }
			 $color = $row->getStyle()->getBoxBorderColor('pdf');
			 $form_border_aux = $row->getStyle()->getBoxBorderForm('pdf');

			 if (empty($form_border_aux))
			     $form_border_aux = 0;
			 if (!empty($this->disable_borders)) {
			     $form_border = CLX_INIT_ZERO;
			     $bord_color['r'] = $bgcolor['r'];
			     $bord_color['g'] = $bgcolor['g'];
			     $bord_color['b'] = $bgcolor['b'];
			 } else {
			     $bord_color['r'] = $color['r'];
			     $bord_color['g'] = $color['g'];
			     $bord_color['b'] = $color['b'];
			     $form_border = $form_border_aux;

			     $bord_color['r'] = $color['r'];
			     $bord_color['g'] = $color['g'];
			     $bord_color['b'] = $color['b'];
			     if ($this->sw < count($this->columns))
			         $this->sw++;
			 }
			 $arrayAligns[] = $row->getStyle()->getFontAligment('pdf');
			 $arrayAltura[] = $altura;
			 $arrayFillR[] = CLX_NUM_ONE;
			 $arrayFuente[] = array('nombre' => $row->getStyle()->getName(), 'letra' => $row->getStyle()->getFontFace(), 'estilo' => $row->getStyle()->getFont('pdf'), 'tamano' => $row->getStyle()->getFontSize('pdf'), 'cR' => $colorFont['r'],
			     'cG' => $colorFont['g'], 'cB' => $colorFont['b'], 'fR' => $bgcolor['r'], 'fG' => $bgcolor['g'],
			     'fB' => $bgcolor['b'], 'bR' => $bord_color['r'], 'bG' => $bord_color['g'], 'bB' => $bord_color['b'],
			     'borde' => $form_border_aux/* $form_border */, 'xls_border_type' => $row->getStyle()->xls_border_type);

			 $colw[] = $row->sourceText();
			 $al[] = $row->getStyle()->getFontAligment('pdf');
			 //Test for col span
			 $colspan = $row->getStyle()->getBoxColSpan('pdf');
			 if ($colspan == CLX_NUM_ZERO)
			     $colspan = CLX_NUM_ONE;
			 //Set new colspan
			 if ($colspan > CLX_NUM_ONE) {
			     $w[$i] = CLX_INIT_ZERO;
			     for ($a = CLX_INIT_ZERO; $a < $colspan; $a++) {
			         $w[$i] += $oldw[$i + $a];
			     }
			     $columnasRow = $columnasRow + $colspan;
			 } else {
			     $w[] = $oldw[$i];
			     $columnasRow++;
			 }
			 //Test for row span
			 for ($xy = CLX_INIT_ZERO; $xy < $colspan; $xy++) {
			     if ($arrayRowSpan[$colNum]['cantidad'] == CLX_NUM_ZERO && $arrayRowSpan[$colNum]['marca'] == CLX_NUM_ONE) {
			         $arrayRowSpan[$colNum]['cantidad'] = ($row->getStyle()->getBoxRowSpan('pdf') == CLX_NUM_ZERO ? CLX_NUM_ONE : $row->getStyle()->getBoxRowSpan('pdf'));
			         if ($xy == CLX_NUM_ZERO)
					   $arrayRowSpan[$colNum][CLX_PROP_COLSPAN] = $colspan;
			         else
					   $arrayRowSpan[$colNum][CLX_PROP_COLSPAN] = CLX_INIT_ZERO;
			         $arrayRowSpan[$colNum]['marca'] = CLX_INIT_ZERO;
			     }
			     $colNum++;
			 }
			 $i += $colspan;
			 if ($i >= $this->width) {
			     $i = CLX_INIT_ZERO;
			     //Set new cols to include span
			     $pdf->SetWidths($w);
			     //Get Style Info
			     $colstyle = $row->getStyle();
			     //Set back color
			     $colorFont = $row->getStyle()->getFontColor('pdf');
			     $pdf->SetTextColor($colorFont['r'], $colorFont['g'], $colorFont['b']);
			     //Set Font
			     $pdf->SetFont($row->getStyle()->getFontFace(), $row->getStyle()->getFont('pdf'), $row->getStyle()->getFontSize('pdf'));
			     //Set alingments for row
			     $pdf->SetAligns($al);
			 }
			 $pdf->SetWidths($oldw);
			 if ($anchoTotal <= $this->calculaAncho($w) || $columnasRow >= $cant_colum) {
			     $columnasRow = $columnasRow - $cant_colum;
			     $contaColum = CLX_NUM_ONE;
			     $w_new = $this->reest_index($w);

			     $pdf->SetWidths($w_new);
			     $pdf->SetAligns($arrayAligns);
			     $pdf->SetAltura($arrayAltura);
			     $pdf->SetFillR($arrayFillR);
			     //Paint Row
			     $pdf->Row1($colw, true, true, $this->getLn(), true, $arrayFuente, '', $arrayRowSpan);
			     $arrayAligns = array();
			     $arrayAltura = array();
			     $arrayFillR = array();
			     $arrayFuente = array();
			     $colw = array();
			     $w = array();
			     $pdf->resetAltura();
			     $colNum = CLX_INIT_ZERO;
			 } else
			     $contaColum++;
		  }
		  //Init
		  $colw = array();
		  $w = array();
		  $al = array();
        }
    }

    /**
     * Build pdf table from report
     * @param unknown_type $pdf
     */
    function sourcePdfOld(&$pdf) {
        $pdf->SetLineWidth(CLX_NUM_0_PT_2);
        //Get Header size to allow formating with wrap
        $colw = array();
        foreach ($this->columns as $key => $cols) {
		  if ($cols->getStyle()->getBoxWidth('pdf') == CLX_NUM_ZERO) {//try to auto adjust column
			 $colw[$key] = array((float) CLX_NUM_40, (float) $cols->getStyle()->getBoxHeight('pdf'));
		  } else {
			 $colw[$key] = array((float) $cols->getStyle()->getBoxWidth('pdf'), (float) $cols->getStyle()->getBoxHeight('pdf'));
		  }
        }
        //Adding Headers
        if (!empty($this->header_visible)) {
		  $set_top_line = false;
		  $set_bot_line = false;
		  foreach ($this->columns as $key => $cols) {
			 //catch x and y for lines
			 $x = $pdf->GetX();
			 $y = $pdf->GetY();
			 $borders = $cols->getStyle()->getBoxBorderStyle('');
			 if (!empty($borders['top'])) {
			     //Dolor de cabeza con las lineas nulas de las celdas sobre la linea
			     //Cuando se pinta la linea si se coloca la celda arriba la oculta
			     $pdf->Line($x, $y, ($x + $cols->getStyle()->getBoxWidth('pdf')), $y);
			     $set_top_line = true;
			 }
			 if (!empty($borders['bottom'])) {
			     $pdf->Line($x, ($y + $cols->getStyle()->getBoxHeight('pdf')), ($x + $cols->getStyle()->getBoxWidth('pdf')), ($y + $cols->getStyle()->getBoxHeight('pdf')));
			     $set_bot_line = true;
			 }
			 $bgcolor = $cols->getStyle()->getBoxBgColor('pdf');
			 $pdf->SetFillColor($bgcolor['r'], $bgcolor['g'], $bgcolor['b']);
			 $pdf->SetFont($cols->getStyle()->getFontFace(), $cols->getStyle()->getFont('pdf'), $cols->getStyle()->getFontSize('pdf'));
			 $y1 = $pdf->GetY();
			 $pdf->MultiCell($colw[$key][CLX_NUM_ZERO], $colw[$key][CLX_NUM_ONE], $cols->sourceText(), $cols->getStyle()->getBoxBorder('pdf'), $cols->getStyle()->getFontAligment('pdf'), null);
			 $y2 = $pdf->GetY();
			 $yH = $y2 - $y1;
			 $pdf->SetXY($x + $colw[$key][CLX_NUM_ZERO], $pdf->GetY() - $yH);
		  }//end for each
		  if (!empty($set_top_line))
			 $pdf->Ln(CLX_NUM_0_PT_4);
		  if (!empty($set_bot_line))
			 $pdf->Ln(CLX_NUM_0_PT_4);
		  $pdf->SetFillColor(CLX_NUM_255, CLX_NUM_255, CLX_NUM_255);
        }
        //Adding Rows
        $set_top_line = false;
        $set_bot_line = false;
        $i = CLX_INIT_ZERO;
        foreach ($this->getData() as $row) {
		  //catch x and y for lines.
		  $x = $pdf->GetX();
		  $y = $pdf->GetY();
		  $pdf->SetFont($row->getStyle()->getFontFace(), $row->getStyle()->getFont('pdf'), $row->getStyle()->getFontSize('pdf'));
		  //MultiCell(float w, float h, string txt [, mixed border [, string align [, boolean fill]]])
		  $y1 = $pdf->GetY();
		  $pdf->MultiCell($colw[$i][CLX_NUM_ZERO], $colw[$i][CLX_NUM_ONE], $row->sourceText(), $row->getStyle()->getBoxBorder('pdf'), $row->getStyle()->getFontAligment('pdf'), null);
		  $yH = $y2 - $y1;
		  $pdf->SetXY($x + $colw[$key][CLX_NUM_ZERO], $pdf->GetY() - $yH);
		  $borders = $row->getStyle()->getBoxBorderStyle('');
		  if (!empty($borders['top'])) {
			 $pdf->Line($x, $y, ($x + $row->getStyle()->getBoxWidth('pdf')), $y);
			 $set_top_line = true;
		  }
		  if (!empty($borders['bottom'])) {
			 $pdf->Line($x, ($y + $row->getStyle()->getBoxHeight('pdf')), ($x + $row->getStyle()->getBoxWidth('pdf')), ($y + $row->getStyle()->getBoxHeight('pdf')));
			 $set_bot_line = true;
		  }
		  $pdf->SetXY($x + $colw[$key][CLX_NUM_ZERO], $y);
		  if ($i >= $this->width - CLX_NUM_ONE) {
			 if (!empty($set_top_line))
			     $pdf->Ln(CLX_NUM_0_PT_4);
			 if (!empty($set_bot_line))
			     $pdf->Ln(CLX_NUM_0_PT_4);
			 $set_top_line = false;
			 $set_bot_line = false;
			 $pdf->Ln();
			 $i = CLX_INIT_ZERO;
		  }else {
			 $i++;
		  }
        }
    }

    function sourceMsExcel(&$ws, &$wb) {
        if (!empty($this->header_visible) and ! empty($this->columns)) {
		  foreach ($this->columns as $cols) {
			 $cols->sourceMsExcel($ws, $wb);
		  }
		  $ws->Ln();
        }

        //Adding Rows
        $i = CLX_NUM_ONE;
        if (!empty($this->getData())) {
		  foreach ($this->getData() as $cell) {
			 $cell->sourceMsExcel($ws, $wb);
			 if ($i >= $this->width) {
			     $ws->Ln();
			     $i = 0;
			 }
			 $i++;
		  }
        }
        $ws->Ln();
    }

}
