<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 * 
 * @package		Clientix.Tool
 * @abstract		Abstract Class for Clientix Tools
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-11 19:24:26 +0000 (Mon, 11 Feb 2019) $
 * @version        $Rev: 10544 $
 * CHANGE LOG
 *  Included label
 */
class clx_tool {

    /**
     * Tool Unique identifier
     * @var string
     */
    var $code;

    /**
     * The name of the tool system
     * @var string
     */
    var $system;

    /**
     * The name of this tool
     * @var string
     */
    var $name;

    /**
     * Tool version
     * @var string
     */
    var $version;

    /**
     * Stores tool code of security to apply to this tool.
     * If child tool this may conteins parent tool code
     * so child tool will work with parent security instead
     * of its own. usualy = $this->code unless $has_access = false.
     * @var string
     */
    protected $accesskey = '';

    /**
     * Stores the user access privileges for tool
     * @var array
     */
    private $access;

    /**
     * This tool has a preference panel?
     * @var Bool
     */
    protected $has_preferences = false;

    /**
     * This tool has a preference panel?
     * @var bool
     */
    protected $has_parameters = false;

    /**
     * This tool has an access panel?
     * @var bool
     */
    protected $has_access = false;

    /**
     * This tool can be saved in a package?
     * @var bool
     */
    protected $has_package = false;

    /**
     * This tool can be used as a pagelet?
     * @var bool
     */
    protected $has_pagelet = false;

    /**
     * Associated table to tool
     * @var string
     */
    var $tablename = '';

    /**
     * Singular record label - dictionary entry
     * @var string
     */
    var $record_name_singular;

    /**
     * Plural record label - dictionary entry
     * @var string
     */
    var $record_name_plural;

    /**
     * Tool Primary key used on synch
     * @var array
     */
    var $primary_key;

    /**
     * Message to display at top or export option
     * @var string
     */
    var $export_notes;

    /**
     * Yes / No List
     * @var array
     */
    var $yesno = array('GEN_LBL_NO', 'GEN_LBL_YES');

    /**
     * Yes / No List
     * @var array
     */
    var $truefalse = array('GEN_LBL_false', 'GEN_LBL_true');

    /**
     * Status type list
     * @var array
     */
    var $status = array('GEN_LBL_INACTIVE', 'GEN_LBL_ACTIVE');

    /**
     * Sort Types
     * @var array
     */
    var $sorting_types = array('NONE' => 'GEN_LBL_NONE', 'ASC' => 'GEN_LBL_ASCENDING', 'DESC' => 'GEN_LBL_DESCENDING');

    /**
     * Date formats used by imports and services. Type is interpreted on
     * fromIsoDate($date, $format) on adodb_date
     * @var array
     */
    var $date_in_formats = array('' => '',
        'd/m/Y' => 'GEN_LBL_DATEFORMAT01',
        'Y-m-d' => 'GEN_LBL_DATEFORMAT02',
        'Y-m-d H:g' => 'GEN_LBL_DATEFORMAT03',
        'Y-m-d H:i:s' => 'GEN_LBL_DATEFORMAT05',
        'c' => 'GEN_LBL_DATEFORMAT04',
        'mdy' => 'GEN_LBL_DATEFORMAT06',
        'mdY' => 'GEN_LBL_DATEFORMAT07',
        '0m0d0y' => 'GEN_LBL_DATEFORMAT08',
        '0m0d0Y' => 'GEN_LBL_DATEFORMAT09',
        'n j y' => 'GEN_LBL_DATEFORMAT10',
        'n j Y' => 'GEN_LBL_DATEFORMAT11',
        ' n j y' => 'GEN_LBL_DATEFORMAT12',
        ' n j Y' => 'GEN_LBL_DATEFORMAT13',
        'ymdHis' => 'GEN_LBL_DATEFORMAT14',
        'YmdHis' => 'GEN_LBL_DATEFORMAT15',
        'ymd' => 'GEN_LBL_DATEFORMAT16',
        'Ymd' => 'GEN_LBL_DATEFORMAT17',
        'dmy' => 'GEN_LBL_DATEFORMAT18',
        'dmY' => 'GEN_LBL_DATEFORMAT19',
        ' j n y' => 'GEN_LBL_DATEFORMAT20',
        ' j n Y' => 'GEN_LBL_DATEFORMAT21',
        'md Y' => 'GEN_LBL_DATEFORMAT22',
        'M d, y' => 'GEN_LBL_DATEFORMAT23',
        'Y/m/d' => 'GEN_LBL_DATEFORMAT24',
        'd-m-Y' => 'GEN_LBL_DATEFORMAT25',);

    /**
     * Notify status list
     * @var array
     */
    var $notify_status = array('' => '', 'NOT' => 'GEN_LBL_NOT', 'IDLE' => 'GEN_LBL_IDLE', 'STOP' => 'GEN_LBL_STOP', 'RUN' => 'GEN_LBL_RUN', 'CANC' => 'GEN_LBL_CANC', 'EXPI' => 'GEN_LBL_EXPIRED', 'ERRO' => 'GEN_LBL_ERRO', 'END' => 'GEN_LBL_END', 'ENDR' => 'GEN_LBL_ENDR', 'WAIT' => 'GEN_LBL_WAITING');
    /* see if move to process schedule,
      'RWOC'=>'GEN_LBL_RWOC',
      'RWAC'=>'GEN_LBL_RWAC' */

    /**
     * Read only types
     * @var array
     */
    var $read_only = array('0' => 'GEN_LBL_NEVER', '1' => 'GEN_LBL_ALWAYS', '2' => 'GEN_LBL_ONCREATE', '3' => 'GEN_LBL_ONMODIFY');

    /**
     * Query Operators
     * @var array
     */
    var $operators = array(
        'EQ' => array('GEN_OPE_EQUAL', '=VALUE'),
        'NQ' => array('GEN_OPE_NOTEQUAL', '<>VALUE'),
        'LS' => array('GEN_OPE_LESS', '<VALUE'), 'LQ' => array('GEN_OPE_LESS_EQUAL', '<=VALUE'),
        'GQ' => array('GEN_OPE_GREATER_EQUAL', '>=VALUE'),
        'GT' => array('GEN_OPE_GREATER', '>VALUE'),
        'IN' => array('GEN_OPE_IN', 'IN (VALUE) '),
        'NI' => array('GEN_OPE_NOTIN', 'NOT IN (VALUE)'),
        'SW' => array('GEN_OPE_STARTS_WITH', 'LIKE \'VALUE%\''),
        'EW' => array('GEN_OPE_ENDS_WITH', 'LIKE \'%VALUE%\''),
        'CT' => array('GEN_OPE_CONTAINS', 'LIKE %\'%VALUE%\''),
        'EX' => array('GEN_OPE_EXPRESSIONS', '(VALUE)'),
        'IS' => array('GEN_OPE_IS', 'IS VALUE'),
        'IT' => array('GEN_OPE_ISNOT', 'IS NOT VALUE'));

    /**
     * Access types
     * @var array
     */
    var $access_types;

    /**
     * Primary Search Field
     * @var string
     */
    var $primary_field;

    /**
     * Secondary Search Field
     * @var string
     */
    var $secondary_field;

    /**
     * Terciary Search Field
     * @var string
     */
    var $terciary_field;

    /**
     * Sequence number field
     * @var string
     */
    var $seqnum_field;

    /**
     * The form used to create/modify this module
     * @var array
     */
    var $editform;

    /**
     * The form used to view/display this module
     * @var array
     */
    var $viewform;

    /**
     * The Listing Definition
     * @var array
     */
    var $toollist;

    /**
     * Panel definition
     * @var array
     */
    var $toolpanel;

    /**
     * Parent field keys
     * @var array
     */
    var $toolpanelkeys;

    /**
     * Field Format Types
     * @var string
     */
    var $format_types = array(CLX_NUM_ZERO => 'GEN_LBL_DBQF_DEFAULT', CLX_NUM_ONE => 'GEN_LBL_DBQF_NO_FORMAT', CLX_NUM_TWO => 'GEN_LBL_DBQF_FORMAT', CLX_NUM_THREE => 'GEN_LBL_DBQF_SECNAME', CLX_NUM_FOUR => 'GEN_LBL_DBQF_SECNAMECOD', CLX_NUM_FIVE => 'GEN_LBL_DBQF_NAMESECNAME', CLX_NUM_SIX => 'GEN_LBL_DBQF_NAMESECCODE');
    //@TODO implement 7=>'GEN_LBL_TEXTUAL_NUMBER' for DBQC and ANRC, Etc*
    /**
     * Debug boolean, when true tools outputs debug info
     * @var bool
     */
    var $debug = false;

    /**
     * Use Menu Security instead Tool Security. Property stores corresponding Menu ID (which identifies "use Menu Security") or blank (which identifies "use Tool Security") [2016-05-04-CR]
     * @var string
     */
    var $useMenuSecurity = '';

    /**
     *Quick Filter form
     * @var type
     */
    var $quickfilterform;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = '';
        $this->name = 'Father of all Tools';
        $this->system = 'System';
        $this->version = '0.2.4.0';
        $this->tablename = '';
        $this->record_name_singular = '';
        $this->record_name_plural = '';
        $this->primary_key = array();
        $this->primary_field = '';
        $this->secondary_field = '';
        $this->terciary_field = '';
        $this->seqnum_field = '';
        $this->toollist = array();
        $this->toolpanel = array();
        $this->export_notes = '';
        $this->access = new ClxAccess();
		//PRE($_SESSION);
		IF (ISSET($GLOBALS['ADDRDEBUGCLXTOOL']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGCLXTOOL']==$_SERVER['REMOTE_ADDR'])) {
			$this->debug=true;
		}	
		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
		IF (ISSET($GLOBALS['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;
	    $this->access = new ClxAccess();
	
    }

    /**
     * Returs string describing the tool code
     * @param bool $codeonly
     * @return string
     */
    function getCode($codeonly = true) {
        return ((!$codeonly) ? 'Tool: ' : '') . $this->code;
    }

    /**
     * Install procedure
     *
     */
    function install() {

    }

    /**
     * Creates physical table for tool based on structure.
     * used maintly on install tools
     *
     */
    function createToolTable() {
        $tableInfo = $this->toolTableStructure();
        if (isset($tableInfo['name']) && !ClxCommon::$db->tableExists($tableInfo['name'])) {
		  $tableInfo = ClxCommon::$db->translateTableStructure($tableInfo);
		  ClxCommon::$db->createTable($tableInfo['name'], $tableInfo['type'], $tableInfo['fields'], false);
        }
    }

    /**
     * Archive Tool Records
     */
    function toolArchive() {
    }

    function deleteLogsUtil() {

    }

    function deleteUtil() {
        ClxCommon::$db->deleteRecords($this->tablename, '1=1');
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 'index'=>'', default = '', 'dict'=>true)
     * dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $tableinfo = array();
        return $tableinfo;
    }

    /**
     * Returns Listing record
     * @return array
     */
    function listTableStructure() {
        $table = $this->toolTableStructure();
        unset($table['idcode'], $table['idname'], $table['type'], $table['indexes']);
        foreach ($table['fields'] as $key => $val) {
		  $table['fields'][$key] = array('name' => $key,
			 'type' => 'string');
		  if (array_key_exists('default', $val)) {
			 $table['fields'][$key]['default'] = $val['default'];
		  }
        }
        $table[CLX_CLASS_TITLE] = $this->record_name_plural;
        return $table;
    }

    /**
     * Loaded during tool startup. Method loads security parameters
     */
    function init() {
        $ACTO = getTool('ACTO');
        if (clx_is_empty($this->accesskey)) {
		  $this->accesskey = $this->code;
        }
        $this->setAccess($ACTO->getUserAccess($this->accesskey));
    }

    /**
     * Describes tool based on record
     * @param string $primary_field_value
     */
    function describe($primary_field_value) {
        return $primary_field_value;
    }

    /**
     * Add this tool preferences to the preferences form,
     * $record are the saved preferences.
     *
     * @param array $record
     * @param object $form
     * @param object $table
     */
    function preferencesForm($record, &$form, &$table) {

    }

    /**
     * Check the user input in the above form.
     * Use UserRequest::$post to check user input.
     *
     * @param array $msg
     */
    function preferencesCheckForm(&$msg) {

    }

    /**
     * Load preferences from the database.
     * $record will be used as first argument int preferencesForm
     *
     * @param array $record
     */
    function preferencesLoad(&$record) {
        ClxFreeReturn($record);
    }

    /**
     * Save user input.  Use UserRequest::$post to check user input.
     *
     * @param array $record
     * @return boolean
     */
    function preferencesSave() {
        return; // return true if you want to save $record
    }

    /**
     * Create preferences for new users.
     *
     * @param array $record
     * @return boolean
     */
    function preferencesDefault(&$record) {
        ClxFreeReturn($record);
        return false; // return true if you want to save $record
    }

    /**
     * Add this tool parameters to the preferences form,
     * $record are the saved parameters.
     *
     * @param array $record
     * @param object $form
     * @param object $table
     */
    function parametersForm($record, &$form, &$table) {
        ClxFreeReturn($record);
        ClxFreeReturn($form);
        ClxFreeReturn($table);
    }

    /**
     * Check the user input in the above form.  Use UserRequest::$post to check user input.
     *
     * @param array $msg
     */
    function parametersCheckForm(&$msg) {
        ClxFreeReturn($msg);
    }

    /**
     * Load preferences from the database.
     * $record will be used as first argument int preferencesForm
     *
     * @param array $record
     */
    function parametersLoad(&$record) {
        ClxFreeReturn($record);
    }

    /**
     * Save user input.  Use UserRequest::$post to check user input.
     * @param array
     * @return bool
     */
    function parametersSave(&$record) {
        ClxFreeReturn($record);
        return false; // return true if you want to save $record
    }

    /**
     * Create parameters for new users.
     * @param array
     * @return bool
     */
    function parametersDefault(&$record) {
        ClxFreeReturn($record);
        return false; // return true if you want to save $record
    }

    /**
     * Pack tool record on xml Format
     * @param array record
     * @return obj
     */
    function pack($record) {
        return $this->packRecord($record);
    }

    /**
     * Creates an xml object from record
     * @param array $record
     * @return obj
     */
    function packRecord($record) {
        $o = new xmlObject($this->code);
        $o->setProp('ID', $record['ID']);
		//pre($record);
        foreach ((array) $record as $key => $value) {
		  //$f = new xmlObject($key);
		  $f = new xmlObject(uppercase($key));
		  $f->setContents(htmlspecialchars($value));
		  $o->addObject($f);
        }
        return $o;
    }

    /**
     * Builds Xml Objects from dependant tool
     * @param obj $objs
     * @param string $toolcode
     * @param string $query
     * @return obj
     */
    function packsub(&$objs, $toolcode, $query) {
        $records = ClxCommon::$db->executeQuery($query);
        $tool = &getTool($toolcode);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $o = $tool->pack($record);
		  $objs->addObject($o);
        }
		ClxCommon::$db->freeResult($records);
        unset($o);
    }

    /**
     * Verify Package Object
     * @param obj $package
     * @param int $i
     * @param obj $table
     */
    function packcheck($package, &$i, &$table) {
        $this->packdescribe($package, $i, $table);
    }

    /**
     * Compare bpackage vs Object
     * @param obj $o
     * @param int $i
     * @param obj $table
     */
    function packdescribe($o, &$i, &$table) {
        $rv = '';
        $ov = '';
        $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
        if ($o->hasSubObject('VERSION')) {
		  $ov = $o->getSubObjectContents('VERSION') . '-' . $o->getSubObjectContents('VERSION_COUNTER');
            $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($o->getSubObjectContents($this->primary_field))) > CLX_NUM_ZERO);
		  $ov1 = str_replace('.', '', $ov);
		  $ov1 = str_replace('-', '', $ov1);
		  if (!clx_is_empty($exists)) {
			 $record = ClxCommon::$db->getRecord($this->tablename, $this->primary_field . '=' . singleQuote($o->getSubObjectContents($this->primary_field)));
			 $rv = $record['VERSION'] . '-' . $record['VERSION_COUNTER'];
		  }
		  $rv1 = str_replace('.', '', $rv);
		  $rv1 = str_replace('-', '', $rv1);
		  if (clx_is_empty($exists)) {
			 $table->addCell(dict('GEN_LBL_NEW'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: yellow', CLX_PROP_WIDTH => '25%'));
		  } elseif ($ov1 > $rv1) {
			 $table->addCell(dict('GEN_LBL_UPGRADE'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: green', CLX_PROP_WIDTH => '25%'));
		  } elseif ($ov1 == $rv1) {
			 $table->addCell(dict('GEN_LBL_SAME'), array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
		  } else {
			 $table->addCell(dict('GEN_LBL_DOWNGRADE'), array(CLX_PROP_CLASS => $class, 'style' => 'background-color: red', CLX_PROP_WIDTH => '25%'));
		  }
		  $table->addCell($rv, array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
		  $table->addCell($ov, array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
		  $table->addCell($this->packname($o), array(CLX_PROP_CLASS => $class, CLX_PROP_WIDTH => '25%'));
		  $i++;
        } else {
		  $table->addCell(dict('GEN_LBL_OBJECTHASNOVERSION'), array(CLX_PROP_CLASS => $class, CLX_PROP_COLSPAN => '4'));
        }
    }

    /**
     * Defines package name
     * @param obj $o
     * @return string
     */
    function packname($o) {
        $t = '';
        $t .= ucfirst($this->record_name_singular) . ': ';
        $t .= $o->getSubObjectContents(uppercase($this->primary_field)) . ' ';
        $t .= $o->getSubObjectContents(uppercase($this->secondary_field)) . ' ';
        return $t;
    }

    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        if (!array_key_exists($this->primary_field, $record)) {
		  $record[$this->primary_field] = '';
        }
        $record['ID'] = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $this->primary_field . '=' . singleQuote($record[$this->primary_field]));
    }

    /**
     * Unpack Record into Metadata
     * @param obj $o
     * @return array
     */
    function unpackRecord($o) {
        $record = array();
        $structure = $this->toolTableStructure();
        $fields = $structure['fields'];
        foreach ($fields as $fieldname => $fieldrec) {
		  if ($fieldname !== 'ID') {
			 //$record[$fieldname] = html_entity_decode($o->getSubObjectContents(uppercase($fieldname)));
			 $record[uppercase($fieldname)] = html_entity_decode($o->getSubObjectContents(uppercase($fieldname)));
		  }
        }
        return $record;
    }

    /**
     * Action is called for every user interaction with the module...
     *
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function action($ac, $idlist) {
		if (!isset($GLOBALS['ADDRDEBUGEDITFORM_']))
			$GLOBALS['ADDRDEBUGEDITFORM_']=0;
	
		IF (!empty($GLOBALS['ADDRDEBUGIMPORT']) )
			$GLOBALS['ADDRDEBUGEDITFORM_']=1;
        $contents = '';
        $msg = array();
			if ( !empty($_SESSION["_dEBUG"])) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
											        fwrite($tta,chr(13).chr(10)."14001- hora: ac:$ac".date("h:i:s d/m/Y").chr(13).chr(10)); 
														  $te=debug_backtrace();for ($t=0;$t<18;$t++) { 
															if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	
														  }	
														  fwrite($tta,chr(13).chr(10).chr(13).chr(10));fclose($tta);}
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'LIST':
					if (empty(UserRequest::$request['mc'])) {
						UserRequest::$request['mc']='';	
					}	
			     $this->checkUserAccessFromMenu(UserRequest::$request['mc']);
			     if ($this->getAccess()->canRead()) {
		             $contents .= $this->filterQuickForm($msg);
			         $contents .= $this->makelist();
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_READ][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'EXPORT':
			     $contents = $this->exportList();
			     break;
			 case 'EXPORT_LIST_XML':
			     $criteria = array();
			     $tb = $this->toolTableStructure();
			     foreach ($tb['fields'] as $key => $val) {
			         if (array_key_exists($key, UserRequest::$request)) {
					   $criteria[$key] = UserRequest::$request[$key];
			         }
			     }
			     unset($tb);
			     $fields = (!clx_is_empty(UserRequest::$request['fl'])) ? UserRequest::$request['fl'] : '*';
			     sendXmlHeader();
			     $contents.= $this->getListXml($fields, $criteria)->saveXml();
			     break;
			 case 'DISPLAY':
			     if (isset(UserRequest::$request['next'])) {
			         popHistory(); // forget where we here
			         array_pop($idlist);
			         $contents = $this->viewForm_($idlist);
			     } else {
			         $contents = $this->viewForm_($idlist);
			     }
			     break;
			 case 'SEARCH':
			     $contents = $this->searchForm();
			     break;
			 case 'QFILTER':
			     if (isset(UserRequest::$post['confirm'])) {
			         $this->setfilter();
			         $info = popHistory();
			         $contents = executeHistory($info);
			     }
			     break;
			 case 'FILTER':
			     if (isset(UserRequest::$post['confirm'])) {
			         $this->setfilter();
			         $info = popHistory();
			         $contents = executeHistory($info);
			     } else {
			         $record = array();
			         $contents = $this->filterForm($msg);
			     }
			     break;
			 case 'UNFILTER':
			     $this->clearfilter();
			     $info = popHistory();
			     $contents = executeHistory($info);
			     break;
			 case 'REORDER':
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   $this->reorderAction();
					   $info = popHistory();
					   $contents = executeHistory($info);
			         } else {
					   $contents = $this->reorderForm();
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'CREATE':
			     if ($this->getAccess()->canDisplay()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  $this->createRecordVoid_();
						  $info = popHistory();
						  $contents = executeHistory($info);
					   } else {
						  $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
					   }
			         } else {
					   $record = $this->defaultRecord();
					   $contents = $this->editForm($ac, array(), $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'MODIFY':
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  ClxFreeReturn($this->saveRecord());
						  if (count($idlist) > CLX_NUM_ZERO) {
						      $id = array_pop($idlist);
						      $record = $this->loadToolRecord($id);
								    $contents = $this->editForm($ac, $idlist, $record, $msg);
							    } else if ( isset($GLOBALS["process"])  ) { 
						      $contents="";      
								}
								else{
						      $info = popHistory();
						      $contents = executeHistory($info);
						  }
					   } else {
						  $contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
					   }
			         } else {
					   $id = array_pop($idlist);
					   $record = $this->loadToolRecord($id);
					   $contents = $this->editForm($ac, $idlist, $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_MODIFY][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'DUPLICATE':
			     if ($this->getAccess()->canModify() && ($this->getAccess()->canDisplay())) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  ClxFreeReturn($this->duplicateRecord());
								if ( isset($GLOBALS["process"])  ) { 
						      $contents="";      
								} else {
									$info = popHistory();
									$contents = executeHistory($info);
								}
					   } else {
						  $contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
					   }
			         } else {
					   $id = array_pop($idlist);
					   $record = $this->loadToolRecord($id);
					   $contents = $this->editForm($ac, $idlist, $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_MODIFY & CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'DELETE':
					if ($this->debug==true) {
						pre('action delete clx_tool');
						pre('idlist');
						pre($idlist);
						pre('UserRequest::post');
						pre(UserRequest::$post);
						
					}
			     if ($this->getAccess()->canDelete()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   foreach ($idlist as $id) {
						  $this->deleteRecordSk($id);
					   }
							if ( isset($GLOBALS["process"])  ) { 
						      $contents="";      
							} else {
								$info = popHistory();
								$contents = executeHistory($info);
							}
			         } else {
					   $contents = $this->confirmDelete($ac, $idlist);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_DELETE][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'TOOL_ARCHIVE':
			     $this->toolArchive();
			     //Return to caller
			     $info = popHistory();
			     $contents = executeHistory($info);
			     break;
			 default:
					if ( !empty($_SESSION["_dEBUG"])) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	
											        fwrite($tta,chr(13).chr(10)."14001- hora: Not Action".date("h:i:s d/m/Y").chr(13).chr(10)); 
														  $te=debug_backtrace();
														  for ($t=0;$t<18;$t++) { 
															if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	
														  }	
														  fwrite($tta,chr(13).chr(10).chr(13).chr(10));fclose($tta);
														  }
			     $contents = xDebug::dump(UserRequest::$request);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Builds listing on tool
     *
     * @param string $listingcode
     * @return string
     */
    function makelist($listingcode = null) {
        $this->applyUserAccessFromMenu();
        if (isset($this->toollist)) {
		  $table = $this->toollist;
        } else {
		  $table = $this->listTableStructure();
        }
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $list = new clx_listing($this->code, $table);
        return $list->makeList(true);
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($groupcode = null) {
        if (isset($this->toolpanel)) {
		  $table = $this->toolpanel;
        } else {
		  return buildPanelInterface($this->name, $this->name . 'panel', dict('GEN_LBL_NOPANELINTOOL') . ': ' . $this->code);
        }
        $table['rpp'] = CLX_NUM_TEN_THOUSAND;
        $list = new clx_listing($this->code, $table);
        $list->ispanel = true;
        $list->panel_arguments = func_get_args();
        return $list->makeList(false);
    }

    /**
     * Construct listing buttons
     *
     * @param string $form_id
     * @return string
     */
    function listingButtons(&$form_id) {
        $buttons = '';
        if ($this->getAccess()->canRead()) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form_id, CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form_id, CLX_PROP_ID => 'ON'))->source();
        }
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form_id, CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form_id, CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form_id, CLX_PROP_ID => 'ONE'))->source();
        }
        if ($this->getAccess()->canRead()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form_id))->source();
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form_id, CLX_PROP_ID => 'ON'))->source();
        }
        return $buttons;
    }

    /**
     * Export listing
     *
     * @param string $listingcode
     */
    function exportList($listingcode = null) {
        ClxFreeReturn($listingcode);
        if ($this->tablename === '') {
		  return CLX_CHAR_EMPTY_STRING;
        }
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        $table_structure = $this->toolTableStructure();

        $ACAD = &getTool('ACAD');
        $auditrec = (!clx_is_empty($_SESSION['paging'][$this->code . 'List'])) ? print_r($_SESSION['paging'][$this->code . 'List'], true) : '';
        $ACAD->log(array(), '', 'EXPORT_REC', $this->code, $auditrec);
        unset($auditrec);

        $contents = '';
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        //$page->debug = true;
        $page->specs['rpp'] = '65530';
        $page->setProp('singular', dict($this->record_name_singular, true));
        $page->setProp('plural', dict($this->record_name_plural, true));
        $page->pageInit($this->code . 'List', $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        foreach ($table_structure['fields'] as $key => $val) {
		  //remove protected filds to prevent been displayed
		  if ($key === 'ID' || !clx_is_empty($val['protected'])) {
			 unset($table_structure['fields'][$key]);
		  }
        }
        //add some header to the file
        $contents .= uppercase(dict('GEN_LBL_TOOL')) . ': ';
        $contents .= CLX_CHAR_TAB . $this->code . CLX_CHAR_NL;
        $contents .= uppercase(dict('GEN_LBL_SYSTEM')) . ': ' . CLX_CHAR_TAB . kDatabaseDisplay . '@' . kWebSite . ' -> ' . $this->system . CLX_CHAR_NL;
        $contents .= uppercase(dict('GEN_LBL_DATE')) . ': ';
        $contents .= CLX_CHAR_TAB . xFormat::getNow('Y-m-d G:i:s') . CLX_CHAR_NL;
        $contents .= uppercase(dict('GEN_LBL_CREATEDBY')) . ': ' . CLX_CHAR_TAB;
        $contents .= $_SESSION['user_record']['username'] . CLX_CHAR_NL;
        if (!clx_is_empty($this->export_notes)) {
		  $contents .= uppercase(dict('GEN_LBL_COMMENTS')) . ': ' . CLX_CHAR_TAB;
		  $contents .= $this->export_notes . CLX_CHAR_NL;
        }
        $contents .= CLX_CHAR_NL;
        $comma = '';
        foreach ($table_structure['fields'] as $key => $val) {
		  $key = str_replace(CLX_CHAR_TAB, ' ', $key);
		  $key = str_replace(CLX_CHAR_NL, ' ', $key);
		  $contents .= $comma . uppercase(dict('GEN_LBL_' . $key, false));
		  $comma = CLX_CHAR_TAB;
        }
        $contents .= CLX_CHAR_NL;
        while ($record = $page->nextRecord()) {
		  $comma = '';
		  foreach ($table_structure['fields'] as $key => $val) {
			 $val = html_entity_decode($record[$key]);
			 if ($this->code === 'ACAD') { //cleaning audit html
			     $val = '.' . str_replace('</tr><tr>', ', ', str_replace('</td><td>', ': ', $val));
			 }
			 $parts = array(CLX_CHAR_TAB, CLX_CHAR_NL, CLX_CHAR_CR, CLX_CHAR_BACKSLASH);
			 $val = str_replace($parts, '', strip_tags($val));
			 $contents .= $comma . $val;
			 $comma = CLX_CHAR_TAB;
		  }
		  $contents .= CLX_CHAR_NL;
        }
		ob_clean();
        sendDownloadFile('download', 'xls');
        return $contents;
    }

    /**
     * Dump tool record list on xml format
     * @param $fields
     * @param $criteria
     * @return obj
     */
    function getListXml($fields = '*', $criteria = array()) {
        // create a new XML document
        $doc = new DomDocument('1.0');
        $doc->encoding = 'UTF-8';
        if ($this->tablename === '') {
		  return $doc->saveXML();
        }
        // create root node
        $node = $doc->createElement('TOOL');
        $node->setAttribute('CODE', $this->code);
        //adding id if not exists
        $sql = 'SELECT ' . $fields . ' FROM ' . $this->tablename . ' WHERE 1=1 ';
        foreach ($criteria as $key => $val) {
		  $sql .= 'AND ' . $key . ' = ' . singleQuote($val) . ' ';
        }
        if (!clx_is_empty($this->seqnum_field)) {
		  $sql .= 'ORDER BY ' . $this->seqnum_field . ' ASC';
        }
        $rs = ClxCommon::$db->executeQuery($sql);
        $numrow = CLX_NUM_ONE;
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  // add node for each row
		  $occ = $doc->createElement($this->code);
		  $occ->setAttribute('NUMROW', $numrow);
		  $numrow++;
		  foreach ($r as $key => $val) {
			 $field = $doc->createElement($key);
			 $value = $doc->createTextNode(utf8_encode($val));
			 $field->appendChild($value);
			 $occ->appendChild($field);
		  }
		  $node->appendChild($occ);
        }
		ClxCommon::$db->freeResult($rs);
		
        $doc->appendChild($node);
        return $doc;
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {IF (isset($GLOBALS['ADDRDEBUGEDITFORM_']) && $GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }

        if (UserRequest::$request['ac'] === 'MODIFY') {
		  $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_MODIFY';
        } else {
		  $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_CREATE';
        }
        $form = new clx_form($this->code, $this->editform);
        return $form->editForm($ac, $idlist, $record, $msg);
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $form = new clx_form($this->code, $this->editform);
        return $form->checkForm1(UserRequest::$post['ac'], UserRequest::$post, $msg);
    }

    /**
     * Updates global variable ac. Used on action
     * @global type $ac
     * @param type $acIn
     */
    protected function setAc($acIn) {
        UserRequest::$request['ac'] = $acIn;
    }

    /**
     * Builds Tool Search form
     * @return string
     */
    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $fields = array();
        $pf = $this->code . '_' . $this->primary_field;
        if (!array_key_exists($pf, UserRequest::$post)) {
		  if (array_key_exists(lowercase($this->primary_field), UserRequest::$get)) {
			 UserRequest::$post[$pf] = UserRequest::$get[lowercase($this->primary_field)];
		  } else {
			 UserRequest::$post[$pf] = '';
		  }
        }
        if ($this->primary_field !== '') {
		  $fields[] = $this->primary_field;
		  $table->addField($msg, dict('GEN_LBL_' . $this->primary_field) . ':', $pf, UserRequest::$post[$pf], CLX_NUM_20, '');
        }
        $sf = $this->code . '_' . $this->secondary_field;
        if (!array_key_exists($sf, UserRequest::$post)) {
		  if (array_key_exists(lowercase($this->secondary_field), UserRequest::$get)) {
			 UserRequest::$post[$sf] = UserRequest::$get[lowercase($this->secondary_field)];
		  } else {
			 UserRequest::$post[$sf] = '';
		  }
        }
        if ($this->secondary_field !== '') {
		  $fields[] = $this->secondary_field;
		  $table->addField($msg, dict('GEN_LBL_' . $this->secondary_field) . ':', $sf, UserRequest::$post[$sf], CLX_NUM_20, '');
        }
        $tf = $this->code . '_' . $this->terciary_field;
        if (!array_key_exists($tf, UserRequest::$post)) {
		  if (array_key_exists(lowercase($this->terciary_field), UserRequest::$get)) {
			 UserRequest::$post[$tf] = UserRequest::$get[lowercase($this->terciary_field)];
		  } else {
			 UserRequest::$post[$tf] = '';
		  }
        }
        if ($this->terciary_field !== '') {
		  $fields[] = $this->terciary_field;
		  $table->addField($msg, dict('GEN_LBL_' . $this->terciary_field) . ':', $tf, UserRequest::$post[$tf], CLX_NUM_20, '');
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();

        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {

		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));

		  $criteria = '';
		  $comma = '';
		  if (UserRequest::$post[$pf] !== '') {
			 $criteria .= $comma . ' ' . $this->primary_field . ' LIKE ' . singleQuote(UserRequest::$post[$pf] . '%');
			 $comma = ' AND ';
		  }
		  if (UserRequest::$post[$sf] !== '') {
			 $criteria .= $comma . ' ' . $this->secondary_field . ' LIKE ' . singleQuote(UserRequest::$post[$sf] . '%');
			 $comma = ' AND ';
		  }
		  if (UserRequest::$post[$tf] !== '') {
			 $criteria .= $comma . ' ' . $this->terciary_field . ' LIKE ' . singleQuote(UserRequest::$post[$tf] . '%');
			 $comma = ' AND ';
		  }
			switch (kDatabaseType) {
				case 'oci':
					$sql = 'SELECT COUNT(1) FROM ( SELECT DISTINCT ' . implode(', ', $fields) . '  A FROM ' . $this->tablename . ' WHERE 1=1 ' . $comma . ' ' . $criteria. ')X';
					break;
				default:
		  $sql = 'SELECT COUNT(DISTINCT ' . implode(', ', $fields) . ' ) A FROM ' . $this->tablename . ' WHERE 1=1 ' . $comma . ' ' . $criteria;
		  		break;
			}
		  ClxCommon::$db->ci_search = true;
		  $qty = ClxCommon::$db->getOne($sql);
		  ClxCommon::$db->ci_search = false;
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));

			 $sql = 'SELECT DISTINCT ' . implode(',', $fields) . ' FROM ' . $this->tablename . ' WHERE 1=1 ' . $comma . ' ' . $criteria;
			 $records = ClxCommon::$db->executeQueryCi($sql);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     if (!array_key_exists($this->primary_field, $record)) {
			         $record[$this->primary_field] = '-';
			     }
			     if (!array_key_exists($this->secondary_field, $record)) {
			         $record[$this->secondary_field] = '';
			     }
			     if (!array_key_exists($this->terciary_field, $record)) {
			         $record[$this->terciary_field] = '-';
			     }
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record[$this->primary_field]) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=clx_tool></page><a href=' . singleQuote($js) . '>' . $record[$this->primary_field] . '</a>', $record[$this->secondary_field], $record[$this->terciary_field]));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
			 	
			 ClxCommon::$db->executeQueryCiResetSesVars();
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    /**
     * Build tool display form
     * @param $idlist
     * @return string
     */
    function viewForm_($idlist) {
        $contents = '';
        $buttons = '';
        $id = $idlist[count($idlist) - CLX_NUM_ONE];
        $record = $this->loadToolRecord($id);
        pushHistory($this->code, 'DISPLAY', $idlist);
        // Buttons Form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, CLX_CLASS_FOOTERS);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('next', CLX_NUM_ONE);
        $form->addArgument('relfm', UserRequest::$request['relfm']);

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));


        $c = new htmlButtonReturn();
        $buttons .= $c->source();
        if (count($idlist) > CLX_NUM_ONE) {
		  $c = new htmlFormSubmit(dict('GEN_LBL_VIEWNEXT_SOMETHING', dict('GEN_LBL_RECORD')), 'DISPLAY', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        $buttons .= $this->viewFormButtons($form);
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $buttons_form = $form->source();
        // View Form -----------------------------------------------------------
        if (!clx_is_empty($this->viewform)) {
		  $form = new clx_form($this->code, $this->viewform);
		  $record = $this->loadToolRecord($id);
		  $contents .= $form->viewForm_($record);
		  $contents .= $buttons_form;
		  $contents .= $this->viewFormFill($record);
        } else {
		  $contents .= $this->viewFormFill($record);
		  $contents .= $buttons_form;
        }
        return  $contents;
    }

    /**
     * Adds buttons to to edit form
     * @param obj $form
     */
    function viewFormButtons($form) {
        if (!clx_is_empty($this->debug)) {
		  pre($form);
        }
        return '';
    }

    /**
     * Build display form contents
     * @param array $record
     * @return string
     */
    function viewFormFill($record) {
        ClxFreeReturn($record);
        if (!clx_is_empty($this->debug)) {
		  pre($form);
        }
        return '';
    }

//end method
    /**
     * Builds tool reorder form
     * @return string
     */
    function reorderForm() {
        $contents = setJavaScriptTag(getJavaScriptFilename('reorder.js'), true);
        $jsscript = 'var arrList = new Array();' . CLX_CHAR_NL;
        $records = $this->reorderRecords();
        $i = CLX_INIT_ZERO;
        foreach ($records as $key => $value) {
		  $value = str_replace(array(CLX_CHAR_CR, CLX_CHAR_NL), '', $value);
		  $jsscript .= 'arrList[' . $i . '] = new Array(' . $key . ', ' . singleQuote($value) . ');' . CLX_CHAR_NL;
		  $i++;
        }
        $contents .= setJavaScriptTag($jsscript, false);
        // Build the form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp('onsubmit', 'processReorderForm()');
        $form->addArgument('ac', 'REORDER');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('neworder', 'this is the new order');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => 400));

        $table->addCell(dict('GEN_LBL_REORDER'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $listing = setJavaScriptTag('document.write(outputReorderList(arrList) + ' . singleQuote('<br>') . ');', false);
        $table->addCell($listing, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $jsscript2 = 'document.write(outputReorderButton(true) + ' . singleQuote('<br>') . ');' . CLX_CHAR_NL
			 . 'document.write(outputReorderButton(false) + ' . singleQuote('<br>') . ');' . CLX_CHAR_NL;
        $options = setJavaScriptTag($jsscript2, false);
        $table->addCell($options, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        $c = new htmlSubmit(dict('GEN_LBL_REORDER'));
        $buttons = $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= str_repeat(CLX_CHAR_NL, CLX_NUM_THREE) . $form->source();
        return $contents;
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        return 'ARRAY value pair';
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        $order = toArray(UserRequest::$post['neworder']);
        $qty = count($order);
        $fields = array();
        for ($i = CLX_NUM_ONE; $i <= $qty; $i++) {
		  $fields[$this->seqnum_field] = $i;
		  ClxCommon::$db->updateRecord($this->tablename, $fields, 'ID=' . $order[$i - CLX_NUM_ONE]);
        }
    }

    /**
     * Builds delete form
     * @param string $ac
     * @param string $idlist
     * @return string
     */
    function confirmDelete($ac, $idlist) {
        $form = new htmlForm();
        $form->setProp(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument('ac', 'NOT_SET');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', implode(',', $idlist));

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));

        $table->addCell(dict('GEN_TTL_DELETE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $qty = count($idlist);
        if ($qty == CLX_NUM_ONE) {
		  $record = $this->loadToolRecord($idlist[CLX_NUM_ZERO]);
		  $table->addCell(dict('GEN_TTL_DELETE_CONFIRM', dict($this->record_name_singular)), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $this->confirmDeleteFill($table, $record);
        } else {
		  $table->addCell(dict('GEN_TTL_DELETE_CONFIRM', dict($this->record_name_plural)), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  foreach ($idlist as $id) {
			 $record = $this->loadToolRecord($id);
			 $this->confirmDeleteFill($table, $record);
		  }
        }
        $buttons = '';
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        return $form->source();
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        if (!clx_is_empty($this->primary_field)) {
		  $table->addData(dict($this->code. '_LBL_' . $this->primary_field) . ':', $record[$this->code . '_' . lowercase($this->primary_field)]);
        }
        if (!clx_is_empty($this->secondary_field)) {
		  $table->addData(dict($this->code. '_LBL_' . $this->secondary_field) . ':', $record[$this->code . '_' . lowercase($this->secondary_field)]);
        }
        if (!clx_is_empty($this->seqnum_field)) {
		  $table->addData(dict($this->code. '_LBL_' . $this->seqnum_field) . ':', $record[$this->code . '_' . lowercase($this->seqnum_field)]);
        }
    }

    /**
     * Builds Filter Form
     * @param array $msg
     * @return string
     */
    function filterQuickForm($msg) {
        $return = '';
        $toollist = $this->toollist;
        if (clx_is_empty($toollist)) {
		  $toollist = array();
        }
        if (!empty($this->quickfilterform) || !empty($toollist['has_filter'])) {
		  $idlist = '';
		  $form = new clx_form($this->code, $this->quickfilterform);
		  $data = (!empty($_SESSION['paging'][$this->code . 'List']['qfilter'])) ? $_SESSION['paging'][$this->code . 'List']['qfilter'] : array();
		  $return = $form->quickFilterForm('NOT_DEFINED', $idlist, $data, $msg);
        }
        return $return;
    }

    /**
     * Builds Tool Filter Form
     * @param array $msg
     * @return  string
     */
    function filterForm($msg) {
        $contents = '';
        $title = dict('GEN_TTL_FILTER');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'FILTER');
        $form->addArgument('confirm', '1');

        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));

        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $this->filterFormFill($msg, $table);
        $c = new htmlSubmit(dict('GEN_LBL_SET_FILTER'), 'FILTER');
        $buttons = $c->source();
		// 2018-12-05 CR - Caso 1003486 - GB-Rep BATCH no arranca
        //$c = new htmlSubmit(dict('GEN_LBL_CLEAR_FILTER'), 'UNFILTER');
        $c = new htmlFormSubmit(dict('GEN_LBL_CLEAR_FILTER'), 'UNFILTER', 'theForm');
        $buttons .= $c->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    /**
     * Adds Filter Form contents
     * @param array $msg
     * @param obj $table
     */
    function filterFormFill($msg, &$table) {
        $tbl_def = $this->toolTableStructure();
        foreach ($tbl_def['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  if (!array_key_exists($formfield, UserRequest::$post))
			 UserRequest::$post[$formfield] = '';
		  $table->addField($msg, dict('GEN_LBL_' . $key) . ':', $formfield, UserRequest::$post[$formfield], CLX_NUM_40, CLX_NUM_HUNDRED);
        }
    }

    /**
     * Register filter information on user session
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
		$tbl_def = $this->toolTableStructure();
        foreach ($tbl_def['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  if (array_key_exists($formfield, UserRequest::$post) && strlen(UserRequest::$post[$formfield]) > CLX_NUM_ZERO) {
			 $filter .= $boolean . ' ' . $key . '  LIKE ' . singleQuote(UserRequest::$post[$formfield]);
			 $boolean = 'AND';
		  }
        }

        $this->setFilterKeys($filter, '', 0);
    }

    /**
     * Set filter keys
     * @param string $filter
     * @param string $sortby
     * @param int $startat
     */
    protected function setFilterKeys($filter, $sortby, $startat) {
		$_SESSION['paging'][$this->code . 'List']['sortby'] = $sortby;
		$_SESSION['paging'][$this->code . 'List']['criteria'] = $filter;
		if (empty(UserRequest::$get['to'] )|| UserRequest::$get['to']=='EDIT' ||UserRequest::$get['to']==$this->code)
			$_SESSION['paging'][$this->code . 'List']['startat'] = $startat;
        $this->setFilterHistory();
    }

    /**
     * Loads quick filter history to display query on serch form.
     * @param type $filter
     */
    protected function setFilterHistory() {
        $qfilter = array();
        foreach (UserRequest::$post as $key => $val) {
		  if ($key === 'to' || $key === 'ac' || $key === 'qy' || $key === 'fm' || $key === 'li' || $key === 'obj_uid' || $key === 'startat' || $key === 'pc' || $key === 'nofr' || $key === 'pfid' || $key === 'rf' || $key === 'rfi' || $key === 'submit1' || $key === 'confirm' || $key === 'idlist') {
			 continue;
		  }
		  $qfilter[$key] = $val;
        }
        $_SESSION['paging'][$this->code . 'List']['qfilter'] = $qfilter;
    }

    /**
     * Clear any applied filter on session for user
     */
    function clearfilter() {
        $_SESSION['paging'][$this->code . 'List']['criteria'] = '';
        $_SESSION['paging'][$this->code . 'List']['startat'] = '0';
        $_SESSION['paging'][$this->code . 'List']['qfilter'] = array();
		// 2018-12-05 CR - Caso 1003486 - GB-Rep BATCH no arranca
        $_SESSION['paging'][$this->code . 'List']['sortby'] = '';
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $table = $this->toolTableStructure();
        $record = array();
        foreach ($table['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  if (array_key_exists($formfield, UserRequest::$post)) {
			 $record[$key] = UserRequest::$post[$formfield];
		  }
		  if ($key === 'CDATE') {
			 $record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
		  }
		  if ($key === 'MDATE') {
			 $record['MDATE'] = xFormat::getNow('Y-m-d H:i:s');
		  }
		  if ($key === 'USERNAME') {
			 $record['USERNAME'] = $_SESSION['user_record']['username'];
		  }
		  if ($key === 'CREATEDBY') {
			 $record['CREATEDBY'] = $_SESSION['user_record']['username'];
		  }
		  if ($key === 'MODIFIEDBY') {
			 $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
		  }
		  if ($key === 'VERSION_COUNTER') {
			 $record['VERSION_COUNTER'] = CLX_INIT_ZERO;
		  }
		  //$date_fmt = $_SESSION['prefs']['DATE_FORMAT'];
		  $date_fmt = 'Y-m-d';
		  if ($val['type'] === 'date') {
			 $record[$key] = xFormat::date($record[$key], $date_fmt);
		  } elseif ($val['type'] === 'datetime') {
			 $record[$key] = xFormat::date($record[$key], $date_fmt . ' h:i:s');
		  } elseif ($val['type'] === 'time') {
			 $record[$key] = xFormat::date($record[$key], $date_fmt . ' h:i:s');
		  }
        }
        return ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * Duplicates record
     *
     * @return integer
     */
    function duplicateRecord() {
        UserRequest::$post[$this->code . '_id'] = CLX_INIT_ZERO;
        $recordid = $this->createRecord_();
        return $recordid;
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $table = $this->toolTableStructure();
        $record = array();
        foreach ($table['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  $record[$formfield] = (array_key_exists('default', $val)) ? $val['default'] : '';
        }
        return $record;
    }

    /**
     * Initialices a record array based on table structure
     *
     * @return array
     */
    function defaultTableRecord() {
        $record = array();
        $tbstruc = $this->toolTableStructure();
        foreach ($tbstruc['fields'] as $key => $val) {
		  if ($key === 'ID') {
			 $record['ID'] = CLX_INIT_ZERO;
			 continue;
		  }
		  if (array_key_exists('default_raw', $val)) {
			 $val['default'] = $val['default_raw'];
		  }
		  if (array_key_exists('default', $val)) {
			 $record[$key] = $val['default'];
		  } else {
			 $record[$key] = 'null';
		  }
        }
        return $record;
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($id) {
        $row = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . $id);
        $table = $this->toolTableStructure();
        $record = array();
        foreach ($table['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  $record[$formfield] = $row[$key];
        }
        return $record;
    }

    /**
     * Returns tool record from metadata
     * @param string $fieldname
     * @return array
     */
    function getToolRecord($primarykey, $secundarykey = '') {
        $crit = $this->primary_field . ' = ' . singleQuote($primarykey);
        if (!clx_is_empty($this->secondary_field) && $secundarykey <> '') {
		  $crit .=' AND ' . $this->secondary_field . ' = ' . singleQuote($secundarykey);
        }
		  $fetched = false;
        if (kApcTtl_21>0) $toolrec = apc_fetchFct($findVar=$key = kSpaceName . '.' . $this->code . '.' . $primarykey . ' ' . $secundarykey,$fetched);
        
		if (empty($fetched)) {
			 $toolrec = ClxCommon::$db->getRecord($this->tablename, $crit);
			if (kApcTtl_21>0&&!empty($toolrec)) {
				apc_addFct($key, $toolrec, kApcTtl_21);
			 }
		  }
        return $toolrec;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {	IF (ISSET($GLOBALS['ADDRDEBUGCLXTOOL']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGCLXTOOL']==$_SERVER['REMOTE_ADDR'])) {$this->debug=true;}	
        $table = $this->toolTableStructure();
        $record = array();
        $recordid = UserRequest::$post[$this->code . '_id'];
    	IF($this->debug==true){
			pre('saveRecord clx_tool');
			pre($table);
		}
        foreach ($table['fields'] as $key => $val) {
		  $formfield = $this->code . '_' . lowercase($key);
		  if (array_key_exists($formfield, UserRequest::$post)) {
			 $record[$key] = UserRequest::$post[$formfield];
		  }
		  if ($key === 'MDATE') {
			 $record['MDATE'] = xFormat::getNow('Y-m-d H:i:s');
		  }
		  if ($key === 'USERNAME') {
			 $record['USERNAME'] = $_SESSION['user_record']['username'];
		  }
		  if ($key === 'MODIFIEDBY') {
			 $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
		  }
			
		  //$date_fmt = $_SESSION['prefs']['DATE_FORMAT'];
		  $date_fmt = 'Y-m-d';
		  if (array_key_exists($key, $record) && $val['type'] === 'date') {
			 //$record[$key] = xFormat::date($record[$key], $_SESSION['prefs']['DATE_FORMAT']);
			 $record[$key] = xFormat::date($record[$key], $date_fmt);
		  } elseif (array_key_exists($key, $record) && $val['type'] === 'datetime') {
			 //$record[$key] = xFormat::date($record[$key], $_SESSION['prefs']['DATE_FORMAT'] . ' h:i:s');
			 $record[$key] = xFormat::date($record[$key], $date_fmt . ' h:i:s');
		  } elseif (array_key_exists($key, $record) && $val['type'] === 'time') {
			 //$record[$key] = xFormat::date($record[$key], $_SESSION['prefs']['DATE_FORMAT'] . ' h:i:s');
			 $record[$key] = xFormat::date($record[$key], $date_fmt . ' h:i:s');
		  }
			
        }

        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID = ' . $recordid);
        $this->versionIncrease('ID', $recordid);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {

        if ($id !== '') {
		  $this->versionIncrease('ID', $id);
		  ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        }
    }

    /**
     * Increase tool version
     * @param string $fieldname
     * @param string $fieldvalue
     */
    function versionIncrease($fieldname, $fieldvalue) {
        $table = $this->toolTableStructure();
        if (array_key_exists('VERSION_COUNTER', $table['fields'])){
		  $sql = 'UPDATE ' . $this->tablename . ' SET VERSION_COUNTER = VERSION_COUNTER + 1 WHERE ' . $fieldname . '=' . singleQuote($fieldvalue);
		  ClxCommon::$db->executeQuery($sql);
        }
    }

    /**
     * Build Tool Lookup field in forms.
     * @param string $name
     * @param string $default
     * @param array $data
     * @param int $display_size
     * @param int $maxlength
     * @return string
     */
    function chooseRecord($name, $default, $data = array(), $display_size = CLX_NUM_32, $maxlength = CLX_NUM_64) {
        ClxFreeReturn($data);
        $code = $this->code;
        $alt = 'TOOL_' . $code . '_ALT_SEARCH';
        $searchIcon = '';
        $url = 'pop.php?to=' . $code . '&ac=SEARCH&target=' . $name;
        $parent = peekHistory();
        foreach ($parent as $key => $val) {
		  if ($key == CLX_PROP_TO || $key === 'ac' || $key === 'idlist' || $key == CLX_PROP_TARGET) {
			 continue;
		  }
		  if (is_string($val)) {
			 $url .= '&' . $key . '=' . urlencode($val);
		  }
        }
        $url .= '&no_menu=1&no_alerts=1';
        $searchIcon .= getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote($url) . ',' . CLX_SRCH_WIN_DEF_WIDTH . ', ' . CLX_SRCH_WIN_DEF_HEIGHT . ',null,'.singlequote($name).');', dict($alt), null, null, null);
        $c = new htmlField($name, $default, $display_size, $maxlength);
        return '<!--'. __FILE__ .'_'. __LINE__ .'-->'.$c->source() . $searchIcon;
    }

    /**
     * Validates if record exists
     * @param string $field_label
     * @param string $field_value
     * @param array $msg
     * @return bool
     */
    function recordExists($field_label, &$field_value, &$msg) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($field_value));
        if ($qty == CLX_NUM_ZERO) {
		  $msg[$field_label] = dict('TOOL_' . $this->code . '_ERR_NO_RECORD');
        } else {
		  $field_value = ClxCommon::$db->getField($this->tablename, $this->primary_field, $this->primary_field . '=' . singleQuote($field_value));
        }
        return $qty > CLX_NUM_ZERO;
    }

    /**
     * Builds Pagelet for Tool
     * @param string $code
     * @param string $ac
     * @param integer $objectid
     * @return string
     */
    function pagelet($code, $ac = null, $objectid = null) {
        ClxFreeReturn($ac);
        ClxFreeReturn($objectid);
        return $code;
    }

    /**
     * Check if object exists on tool
     * @param type $code
     * @return boolean
     */
    function pageCheckObject($code) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($code));
        return ($qty === CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Build Pagelet Form for tool
     * @return string
     */
    function pageletform() {
        return '';
    }

    /**
     * Construct operator based on $this->operators types and value
     * Method is used by tools that required additional criteras as
     * listing criterias or report query criteria.
     * @param string $operator
     * @param string $fieldname
     * @param string $value
     * @param bool $is_numeric
     * @return string
     */
    function buildOperator($operator, $fieldname, $value, $is_numeric = false) {
        if ($operator === '') {
		  $operator = 'EQ';
        }
        $opval = $this->operators[$operator];
        switch ($operator) {
		  case 'EX':
			 return '(' . $value . ')';
			 break;
		  case 'IN': //LET IT BE NOT QUOTES
			 $value = str_replace('VALUE', $value, $opval[CLX_NUM_ONE]);
			 break;
		  case 'NI': //LET IT BE NOT QUOTES
			 $value = str_replace('VALUE', $value, $opval[CLX_NUM_ONE]);
			 break;
		  case 'SW':
			 $value = str_replace('VALUE', $value, $opval[CLX_NUM_ONE]);
			 break;
		  case 'EW':
			 $value = str_replace('VALUE', $value, $opval[CLX_NUM_ONE]);
			 break;
		  case 'CT':
			 $value = str_replace('VALUE', $value, $opval[CLX_NUM_ONE]);
			 break;
		  default: //NI,IN
			 //LET IT BE NOT QUOTES
			 if (clx_is_empty($is_numeric)) {
			     $value = str_replace('VALUE', singleQuote(str_replace(CLX_CHAR_SINGLE_QUOTE, '\\\'', $value)), $opval[CLX_NUM_ONE]);
			 }
			 break;
        }
        return $fieldname . ' ' . $value;
    }

    /**
     * Setter for tool access
     * @param ClxAccess $access
     */
    function setAccess(ClxAccess $access) {
        $this->access = $access;
    }

    /**
     * Getter for tool access
     * @return ClxAccess
     */
    function getAccess() {
        return $this->access;
    }

    function getUserAccess($toolcode) {
        $access = new ClxAccess($toolcode);
        if (!ClxCommon::sessionHasUser()) {
		  $access->setNoAccess();
        } elseif ($this->adminAccess()) {
		  $access->setAdminUser();
        } else {
		  $sq = $this->securityQuery();
		  $sql = 'SELECT MAX(CANREAD) CANREAD, MAX(CANMODIFY) CANMODIFY, MAX(CANDELETE) CANDELETE, MAX(CANAPPEND) CANAPPEND, MAX(ISADMIN) ISADMIN FROM CLX_AC_TOOLS WHERE ' . $sq . ' AND TOOLCODE = ' . singleQuote($toolcode);
		  $r = ClxCommon::$db->getOneRecord($sql);
		  if (!clx_is_empty($r)) {
			 $access->setUserAccess(intval($r['CANREAD']), intval($r['CANMODIFY']), intval($r['CANDELETE']), intval($r['CANAPPEND']), intval($r['CANREAD']), intval($r['ISADMIN']));
		  }
        }
        return $access;
    }

    function getUserAccessFromMenu($toolcode, $useMenuSecurity, $currentAccess) {
        if (!clx_is_empty($this->debug)) {
		  pre('Valor de menuid: ' . $useMenuSecurity);
        }
        if (!ClxCommon::sessionHasUser() || $this->adminAccess() || clx_is_empty($useMenuSecurity)) {
		  if (!clx_is_empty($this->debug)) {
			 pre('No tiene menu');
		  }
		  // Get the already calculated tool security settings when no user session or is superuser or no menu ID assigned
		  return $currentAccess;
        }
        $access = new ClxAccess($toolcode);
        $sq = $this->securityQuery();
        // Aplica seguridad del menu
        $sql = 'SELECT MAX(AT.CANREAD) CANREAD, MAX(AT.CANMODIFY) CANMODIFY, MAX(AT.CANDELETE) CANDELETE, MAX(AT.CANAPPEND) CANAPPEND, MAX(AT.ISADMIN) ISADMIN FROM CLX_AC_MENUITEMS AT WHERE TOOLCODE = ' . singleQuote($toolcode) . ' AND AT.MENUID = ' . singleQuote($useMenuSecurity) . ' AND ' . $sq . ' GROUP BY AT.TOOLCODE';
        if (!clx_is_empty($this->debug)) {
		  pre($sql);
        }
        $r = ClxCommon::$db->getOneRecord($sql);
        if (!clx_is_empty($r)) {
		  $access->setUserAccess(intval($r['CANREAD']), intval($r['CANMODIFY']), intval($r['CANDELETE']), intval($r['CANAPPEND']), intval($r['CANREAD']), intval($r['ISADMIN']));
		  if (!clx_is_empty($this->debug)) {
			 pre($access);
		  }
        }
        return $access;
    }

    function applyUserAccessFromMenu() {
        if (clx_is_empty($this->useMenuSecurity)) {
		  return;
        }
        if (!clx_is_empty($this->debug)) {
		  pre('Valor -makelist- de useMenuSecurity = ' . $this->useMenuSecurity);
        }
        $ACTO = getTool('ACTO'); // If there's any filter, get menu security insted tool's / Si se especifica filtro, toma la seguridad del menu [2016-04-05]
        $this->setAccess($ACTO->getUserAccessFromMenu($this->accesskey, $this->useMenuSecurity, $this->getAccess()));
        if (!clx_is_empty($this->debug)) {
		  pre($this->getAccess());
        }
    }

    function checkUserAccessFromMenu($filterTokenValue) {
        if (!isset($_SESSION['paging'][$this->code . 'List']['useMenuSecurity'])) {
		  $_SESSION['paging'][$this->code . 'List']['useMenuSecurity'] = '';
        }
        // If there's a filter from menu, stores it into session to apply menu based security, instead tool based security. This only occurs the first time option is called from menu, further times it will get values from session var [2016-04-05]
        if (!clx_is_empty(UserRequest::$request['menuid']) && UserRequest::$request['menuid'] != $_SESSION['paging'][$this->code . 'List']['useMenuSecurity']) {
		  if (!clx_is_empty($filterTokenValue)) {
			 // Set useMenuSecurity option only the first time menu calls and when a filter is applied. Filter means NO ClientoTools menu option...
			 $_SESSION['paging'][$this->code . 'List']['useMenuSecurity'] = UserRequest::$request['menuid'];
		  } else {
			 // No filter, no useMenuSecurity option (for example when it's the ClientTools option). It behaves like before, that is: using tool security
			 $_SESSION['paging'][$this->code . 'List']['useMenuSecurity'] = '';
		  }
		  if (!clx_is_empty($this->debug)) {
			 pre('Hace nuevo filtro de la Lista ' . singleQuote($filterTokenValue) . ' y asigna nuevo useMenuSecurity = ' . singleQuote($_SESSION['paging'][$this->code . 'List']['useMenuSecurity']));
		  }
        }
        if (isset($_SESSION['paging'][$this->code . 'List']['useMenuSecurity'])) {
		  $this->useMenuSecurity = $_SESSION['paging'][$this->code . 'List']['useMenuSecurity'];
		  if (!clx_is_empty($this->debug)) {
			 pre('Asigno useMenuSecurity = ' . $this->useMenuSecurity);
		  }
        }
    }

    private function adminAccess() {
        return $_SESSION['user_record']['isadmin'];
    }

    /**
     * Export diccionary records
     */
    function exportDictRecords() {

        if (clx_is_empty($_SESSION['paging'][$this->code . 'List']['criteria'])) {
		  addMessage(dict('GEN_LBL_FILTER_REQ_FOR_EXP'));
		  return;
        }
        $tree = new xmlObject($this->code);
        $rec = new xmlObject('FILTER');
        xmlObject::addNewObjectToNode('SORT', $_SESSION['paging'][$this->code . 'List']['sortby'], $rec);
        xmlObject::addNewObjectToNode('CRITERIA', $_SESSION['paging'][$this->code . 'List']['criteria'], $rec);
        xmlObject::addNewObjectToNode('STARTAT', $_SESSION['paging'][$this->code . 'List']['startat'], $rec);
        $tree->addObject($rec);

        $records = ClxCommon::$db->getRecords($this->tablename, $_SESSION['paging'][$this->code . 'List']['criteria']);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  unset($record['ID']);
		  $rec = new xmlObject('ENTRIES');
		  foreach ($record as $key => $val) {
			 $vale = ($key == 'ENTRY' || $key == 'VALUE' || $key == 'DESCRIPTION' || $key == 'RESOLUTION') ? htmlspecialchars($val) : $val;
			 xmlObject::addNewObjectToNode($key, $vale, $rec);
		  }
		  $tree->addObject($rec);
        }
			
		ClxCommon::$db->freeResult($records);
		

        $tree->save(UserRequest::$request[$this->code . '_filename']);
    }

    /**
     * Imports diccionary and sys messages from record.
     */
    protected function importDictRecords() {

        $filename = (!clx_is_empty(UserRequest::$request[$this->code . '_localfile'])) ? UserRequest::$request[$this->code . '_localfile'] : UserRequest::$files[$this->code . '_filename']['tmp_name'];
        $tree = new xmlObject($this->code);
        $tree->load($filename);
        $dump = (defined('UTIL_DUMP_AUPK_SQL_SCRIPT') && UTIL_DUMP_AUPK_SQL_SCRIPT === strval(CLX_NUM_ONE)) ? true : false;
        foreach ($tree->objects as $entry) {

		  ClxCommon::$db->dumpScript = ($dump) ? true : false;
		  $this->importEntryDictFromPack($entry);
		  ClxCommon::$db->dumpScript = false;
		  $this->importRecordsFilterFromPack($entry);
        }
    }

    /**
     * Runs format entry
     * @param type $entry
     */
    protected function importEntryDictFromPack(&$entry) {

        if ($entry->tag === 'ENTRIES' || $entry->tag === 'ENTRY') {
		  $record = array('ENTRY' => html_entity_decode($entry->getSubObjectContents('ENTRY')),
			 'VALUE' => html_entity_decode($entry->getSubObjectContents('VALUE')),
			 'LANGUAGE' => clx_language_lower_upper($entry->getSubObjectContents('LANGUAGE')),
			 'MDATE' => $entry->getSubObjectContents('MDATE'),
			 'TOOL_ENTRY' => $entry->getSubObjectContents('TOOL_ENTRY'),
			 'SOURCE_LANGUAGE' => (clx_is_empty($entry->getSubObjectContents('SOURCE_LANGUAGE'))) ? clx_language_lower_upper(kSystemLanguage) : clx_language_lower_upper($entry->getSubObjectContents('SOURCE_LANGUAGE')),
			 'ENTRY_VERSION' => (clx_is_empty($entry->getSubObjectContents('ENTRY_VERSION'))) ? CLX_INIT_ZERO : $entry->getSubObjectContents('ENTRY_VERSION'));
		  if ($this->code == 'SMSG') {
			 $record['DESCRIPTION'] = html_entity_decode($entry->getSubObjectContents('DESCRIPTION'));
			 $record['RESOLUTION'] = html_entity_decode($entry->getSubObjectContents('RESOLUTION'));
		  }
		  $id = 0;
		  $crits = 'ENTRY = ' . singleQuote($record['ENTRY']) . ' AND LANGUAGE = ' . singleQuote(clx_language_lower_upper($record['LANGUAGE']));
		  switch (UserRequest::$request[$this->code . '_options']) {
			 case 'NEW':
			     $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crits);
			     break;
			 case 'UPD':
			     $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crits);
			     break;
			 case 'REP':
			     $id = ClxCommon::$db->loadDbRecord($this->tablename, $record, false, $crits);
			     break;
			 default:
			     break;
		  }
		  unset($id);

		  $this->clearDictHeap($record['ENTRY']);
        }
    }

    /**
     * Sets session filter
     * @param obj $entry
     */
    protected function importRecordsFilterFromPack(&$entry) {
        if ($entry->tag === 'FILTER') {
		  $_SESSION['paging'][$this->code . 'List']['sortby'] = $entry->getSubObjectContents('SORT');
		  $_SESSION['paging'][$this->code . 'List']['criteria'] = $entry->getSubObjectContents('CRITERIA');
		  $_SESSION['paging'][$this->code . 'List']['startat'] = $entry->getSubObjectContents('STARTAT');
        }
    }

}
