<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @since        $Date: 2015-09-03 16:24:19 +0000 (Thu, 03 Sep 2015) $
 * @version        $Rev: 6814 $
 * @package		Clientix.Classes
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 *
 * Queue class - under PHP 4
 *
 * @description This is an implementation of FIFO (First In First Out) queue.
 *
 * @copyright (c) 2003 Michal 'Seth' Golebiowski <sethmail at poczta dot fm>
 *   Released under the GNU General Public License
 *   license is attached to package in license.txt file
 *
 * @updated 10.08.2003
 *
 * @example example.php4 Simple example of puting ang geting datas from queue
 *
 * @requirement PHP 4
 * @greetings goes to all developers from Poland especially from php.pl :-)
 */

/**
 * Implementation of FIFO queue
 * @version 1.9
 */
class Queue {

    var
		  $arrQueue, // Array of queue items
		  $intBegin, // Begin of queue - head
		  $intEnd, // End of queue - tail
		  $intArraySize, // Size of array
		  $intCurrentSize; // Current size of array

    /**
     * Queue constructor
     * @param int $intQueue - size of queue
     */

    function Queue($intSize = CLX_NUM_20) {
        $this->arrQueue = array();
        $this->intArraySize = $intSize;
        $this->Clear();
    }

    /**
     * Add item to queue
     * @param obj &$objQueueItem - queue item object
     * @return true if added to queue or false if queue is full and item could not be added
     */
    function Put(&$objQueueItem) {
        if ($this->intCurrentSize >= $this->intArraySize) {
		  return false;
        }
        if ($this->intEnd == $this->intArraySize - CLX_NUM_ONE) {
		  $this->intEnd = CLX_INIT_ZERO;
        } else {
		  $this->intEnd++;
        }
        $this->arrQueue[$this->intEnd] = $objQueueItem;
        $this->intCurrentSize++;
        return true;
    }

    /**
     * Get item from queue
     * @return object (queue iteme) or false if there is now items in queue
     */
    function Get() {
        if ($this->IsEmpty()) {
		  return false;
        }
        $objItem = $this->arrQueue[$this->intBegin];
        if ($this->intBegin == $this->intArraySize - CLX_NUM_ONE) {
		  $this->intBegin = CLX_INIT_ZERO;
        } else {
		  $this->intBegin++;
        }
        $this->intCurrentSize--;
        return $objItem;
    }

    /**
     * Return number of entries on stock
     * @return Integer
     */
    function sizeQueue() {
        return count($this->arrQueue);
    }

    /**
     * Check if queue is empty
     * @return true if it is empty or false if not
     */
    function IsEmpty() {
        return $this->intCurrentSize == CLX_NUM_ZERO ? true : false;
    }

    /**
     * Drop Next Item
     * @return boolean
     */
    function DropNext() {
        if ($this->IsEmpty()) {
		  return false;
        }
        unset($this->arrQueue[$this->intBegin]);
        if ($this->intBegin == $this->intArraySize - CLX_NUM_ONE) {
		  $this->intBegin = CLX_INIT_ZERO;
        } else {
		  $this->intBegin++;
        }
        $this->intCurrentSize--;
        return true;
    }

    /**
     * Clear queue
     */
    function Clear() {
        $this->arrCurrentSize = CLX_INIT_ZERO;
        $this->intBegin = CLX_INIT_ZERO;
        $this->intEnd = $this->intArraySize - CLX_NUM_ONE;
    }

}
