<?php

/**
 * General Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-10-02 20:00:43 +0000 (Mon, 02 Oct 2017) $
 * @version        $Rev: 9552 $
 * @package		Clientix.Classes.Validate
 * @abstract		General Class
 * CHANGE LOG
 *  Included label
 */
class xValidate {

    static function required(&$msg, $prompt, $value) {
        if (strlen(trim($value)) == CLX_NUM_ZERO) {
		  $msg[$prompt] = dict('GEN_ERR_REQUIRED_FIELD');
        }
    }

    static function format(&$msg, $prompt, $format) {
        if (preg_match('/[^Aa#\.]/', $format) != CLX_NUM_ZERO) {
		  $msg[$prompt] = dict('GEN_ERR_FORMAT_NOTVALID');
        }
    }

    static function subformat(&$msg, $prompt, $format) {
        if (preg_match('/[^Aa#]/', $format) != CLX_NUM_ZERO) {
		  $msg[$prompt] = dict('GEN_ERR_FORMAT_NOTVALID');
        }
    }

    static function formated(&$msg, $prompt, $value, $format) {
        $format_len = strlen($format);
        $value_len = strlen($value);
        if ($format_len != $value_len) {
		  $msg[$prompt] = dict('GEN_ERR_INVALID_ENTRY');
        } else {
		  $error = false;
		  $done = false;
		  $i = CLX_INIT_ZERO;
		  while (!$done and ! $error) {
			 $digit = substr($format, $i, CLX_NUM_ONE);
			 $letter = substr($value, $i, CLX_NUM_ONE);
			 if ($digit === 'A') {
			     $pattern = 'A-Z';
			 } elseif ($digit === 'a') {
			     $pattern = 'a-z';
			 } elseif ($digit === '#') {
			     $pattern = '0-9';
			 }
			 $error = (preg_match('/[^' . $pattern . ']/', $letter) != CLX_NUM_ZERO);
			 $i++;
			 $done = ($i > $format_len);
		  }
		  if (!empty($error)) {
			 if ($digit === 'A') {
			     $msg[$prompt] = dict('GEN_ERR_EXPECTING_UPPERALPHA');
			 } elseif ($digit === 'a') {
			     $msg[$prompt] = dict('GEN_ERR_EXPECTING_LOWERALPHA');
			 } elseif ($digit === '#') {
			     $msg[$prompt] = dict('GEN_ERR_EXPECTING_NUMBER');
			 }
		  }
        }
    }

    static function url(&$msg, $prompt, $url) {
        if (empty($url)) {
		  $msg[$prompt] = dict('GEN_ERR_EXPECTING_URL') . '<br>';
        } elseif (substr($url, CLX_NUM_ZERO, CLX_NUM_7) !== 'http://') {
		  $msg[$prompt] = dict('GEN_ERR_EXPECTING_URL_HTTP') . '<br>';
        }
    }

    static function mail(&$msg, $prompt, $mail) {
        if (empty($mail)) {
		  $msg[$prompt] = dict('GEN_ERR_EXPECTING_EMAIL') . '<br>';
        } elseif (!self::is_valid_email($mail)) {
		  $msg[$prompt] = dict('GEN_ERR_EXPECTING_EMAIL_BAD') . '<br>';
        }
    }

    static function is_valid_email($mail) {
        $status = true;
        if (version_compare(PHP_VERSION, '5.2.0') >= 0) {
		  $status = filter_var($mail, FILTER_VALIDATE_EMAIL);
        } else {
		  if (empty($mail) || stristr($mail, '@') === '' || stristr($mail, '.') === '') {
			 $status = false;
		  }
        }
        return (boolean) $status;
    }

    static function password(&$msg, $passname, $passvalue, $confirmname, $confirmvalue) {
        if (empty($passvalue)) {
		  $msg[$passname] = dict('GEN_ERR_PASSWORD') . '<br>';
        }
        if (!defined('ACUS_PWD_MIN_LENGTH'))
		  $legth = CLX_NUM_SIX;
        else
		  $legth = ACUS_PWD_MIN_LENGTH;
        if (strlen($passvalue) < $legth) {
		  $msg[$passname] = dict('GEN_ERR_PASSWORD_LONG', $legth) . '<br>';
        }
        //Validating password max longitude
        if (!defined('ACUS_PWD_MAX_LENGTH'))
		  $legth = CLX_INIT_ZERO;
        else
		  $legth = ACUS_PWD_MAX_LENGTH;
        if ($legth > CLX_NUM_ZERO && strlen($passvalue) > $legth) {
		  $msg[$passname] = dict('GEN_ERR_PASSWORD_TO_LONG', $legth) . '<br>';
        }
        if (empty($confirmvalue)) {
		  $msg[$confirmname] = dict('GEN_ERR_REPASSWORD') . '<br>';
        }
        if ($passvalue <> $confirmvalue) {
		  $msg[$confirmname] = dict('GEN_ERR_REPASSWORD_MATCH') . '<br>';
        }
    }

    /**
     * Additional password validations base on user parameters
     *
     * @param array $msg
     * @param string $passname
     * @param string $passvalue
     * @param string $confirmname
     * @param string $confirmvalue
     */
    static function password_extras(&$msg, $passname, $passvalue) {
        /*
		ACUS_PWD_CHAR_REP Set to 1, to prevent user to type passwords that repeat characters. */
        if (defined('ACUS_PWD_CHAR_REP') && ACUS_PWD_CHAR_REP > CLX_NUM_ZERO) {
		  if (xValidate::checkCharDup($passvalue, false, ACUS_PWD_CHAR_REP)) {
			 $msg[$passname] = dict('GEN_ACUS_PWD_CHAR_REP');
		  }
        }
        /*
		ACUS_PWD_CHAR_CONT Set to 1, to prevent user to type continuous characters or numbers. Example: 123ABC
         */
        if (defined('ACUS_PWD_CHAR_CONT') && ACUS_PWD_CHAR_CONT > CLX_NUM_ZERO) {
		  if (xValidate::checkCharCont($passvalue, false, ACUS_PWD_CHAR_CONT)) {
			 $msg[$passname] = dict('GEN_ACUS_PWD_CHAR_CONT');
		  }
        }
        $validchars = 'abcdefghijklmnñopqrstuvwxyz';
        $validcharsandnums = 'abcdefghijklmnñopqrstuvwxyz1234567890';
        $validcharsspecial = 'abcdefghijklmnñopqrstuvwxyz1234567890!@#$&%_=+-';
        /*
		ACUS_PWD_CHAR_MIX Defines the character type allowed on passwords: Set to 0 for any type.
		Set to 1 for characters only, Set to 2 for characters and numbers Set to 3 for characters+number+special characters. */
        if (defined('ACUS_PWD_CHAR_MIX') && ACUS_PWD_CHAR_MIX > CLX_NUM_ZERO) {
		  switch (ACUS_PWD_CHAR_MIX) {
			 case CLX_NUM_ONE:
			     if (!xValidate::checkChars($validchars, lowercase($passvalue))) {
			         $msg[$passname] = dict('ACUS_PWD_CHAR_MIX1');
			     }
			     break;
			 case CLX_NUM_TWO:
			     if (!xValidate::checkChars($validcharsandnums, lowercase($passvalue))) {
			         $msg[$passname] = dict('ACUS_PWD_CHAR_MIX2');
			     }
			     break;
			 default:
			     if (!xValidate::checkChars($validcharsspecial, lowercase($passvalue))) {
			         $msg[$passname] = dict('ACUS_PWD_CHAR_MIX3');
			     }
			     break;
		  }
        }
        $validchars = 'abcdefghijklmnñopqrstuvwxyz';
        $validnums = '1234567890';
        $validspecial = '!@#$&%_=+-';
        //Validate number of characters
        if (defined('ACUS_PWD_COUNT_CHAR') && ACUS_PWD_COUNT_CHAR > CLX_NUM_ZERO) {
		  if (!xValidate::countChars($validchars, $passvalue, ACUS_PWD_COUNT_CHAR)) {
			 $msg[$passname] = dict('ACUS_PWD_COUNT_CHAR', ACUS_PWD_COUNT_CHAR);
		  }
        }
        //validate number of numeric characters
        if (defined('ACUS_PWD_COUNT_NUM') && ACUS_PWD_COUNT_NUM > CLX_NUM_ZERO) {
		  if (!xValidate::countChars($validnums, $passvalue, ACUS_PWD_COUNT_NUM)) {
			 $msg[$passname] = dict('ACUS_PWD_COUNT_NUM', ACUS_PWD_COUNT_NUM);
		  }
        }
        //Validate number of special characters
        if (defined('ACUS_PWD_COUNT_SCHAR') && ACUS_PWD_COUNT_SCHAR > CLX_NUM_ZERO) {
		  if (!xValidate::countChars($validspecial, $passvalue, ACUS_PWD_COUNT_SCHAR)) {
			 $msg[$passname] = dict('ACUS_PWD_COUNT_SCHAR', ACUS_PWD_COUNT_SCHAR);
		  }
        }
        //Validate character case ACUS_PWD_CHAR_CASE
        if (defined('ACUS_PWD_CHAR_CASE') && ACUS_PWD_CHAR_CASE > CLX_NUM_ZERO) {
		  switch (ACUS_PWD_CHAR_CASE) {
			 case '1'://Uppercase
			     if ($passvalue <> uppercase($passvalue)) {
			         $msg[$passname] = dict('ACUS_PWD_CHAR_MUST_UPPER');
			     }
			     break;
			 case '2'://lowercase
			     if ($passvalue <> lowercase($passvalue)) {
			         $msg[$passname] = dict('ACUS_PWD_CHAR_MUST_LOWER');
			     }
			     break;
			 default:
			     break;
		  }
        }
    }

    /**
     * Validate integer value
     *
     * @param array $msg
     * @param string $prompt
     * @param integer $value
     * @param integer $min
     * @param integer $max
     * @param string $name
     * @param integer  0 = Unsigned, 1 = Signed
     */
    static function integer(&$msg, $prompt, $value, $min, $max, $name, $sign = CLX_NUM_ONE) {
        //$number = intval($value);//intval overflow in number like 6593128416
        if (extension_loaded('bcmath')) {//prefered
		  $number = bcadd($value, CLX_NUM_ZERO, CLX_NUM_ZERO);
        } else {
		  $number = intval($value); //intval overflow in number like 6593128416
        }
        if ($value === '' || (substr($value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '-' && substr($value, CLX_NUM_ZERO, CLX_NUM_ONE) <> '.' && substr($value, CLX_NUM_ONE, CLX_NUM_ONE) !== '.' && substr($value, CLX_NUM_ZERO, CLX_NUM_ONE) == CLX_NUM_ZERO && $number <> CLX_NUM_ZERO)) {
		  $msg[$prompt] = dict('GEN_ERR_ISNUM');
        } elseif (!is_numeric($value)) {
		  $msg[$prompt] = dict('GEN_ERR_ISNUM');
        } elseif ($value != $number) {
		  $msg[$prompt] = dict('GEN_ERR_ISINT');
        } elseif (($sign == CLX_NUM_ZERO) and ( $number < CLX_NUM_ZERO)) {
		  $msg[$prompt] = dict('GEN_ERR_ISINTPOS');
        } else {
		  xValidate::number($msg, $prompt, $number, $min, $max, $name);
        }
    }

    static function float(&$msg, $prompt, $value, $min, $max, $name) {
        $number = $value;
        if (strlen($number) == CLX_NUM_ZERO) {
		  $msg[$prompt] = dict('GEN_ERR_ISNUM');
        } else if (!is_numeric($value)) {
		  $msg[$prompt] = dict('GEN_ERR_ISNUM');
        } elseif ($value != $value) {
		  $msg[$prompt] = dict('GEN_ERR_ISFLOAT');
        } else {
		  xValidate::number($msg, $prompt, $value, $min, $max, $name);
        }
    }

    static function number(&$msg, $prompt, $number, $min, $max, $name) {
        if (strlen($min) > CLX_NUM_ZERO && ($number < $min)) {
		  $msg[$prompt] = dict('GEN_NUM_GREATER') . ' ' . $min;
        } elseif (strlen($max) > CLX_NUM_ZERO && ($number > $max)) {
		  $msg[$prompt] = dict('GEN_NUM_LESS') . ' ' . $max;
        }
    }

    static function datetime(&$msg, $prompt, $adate, $name) {
        xValidate::anytimedate($msg, $prompt, $adate, $name);
    }

    static function date(&$msg, $prompt, $adate, $name) {
        xValidate::anytimedate($msg, $prompt, $adate, $name);
    }

    static function time(&$msg, $prompt, $atime, $name) {
        if ($atime === '') {
		  $msg[$prompt] = dict('GEN_ERR_NONAME') . ' ' . $name;
        } else {
		  $timestamp = adodb_strtotime($atime);
		  if (UserRequest::$request['ac']!='DELETE' &&$timestamp == -CLX_NUM_ONE) {
			 $msg[$prompt] = dict('GEN_ERR_CANTPARSE');
		  }
        }
    }

    // This only works on unix or windows 95!
    static function anytimedate(&$msg, $prompt, &$adate, $name) {
        if ($adate === '') {
		  $msg[$prompt] = dict('GEN_ERR_NONAME') . ' ' . $name;
		}
		else if ($adate=='00-00-00') {
			$adate=null;
        } else {
		  $timestamp = adodb_strtotime($adate);
		  if (UserRequest::$request['ac']!='DELETE' &&$timestamp == -CLX_NUM_ONE) {
			 $msg[$prompt] = dict('GEN_ERR_CANTPARSE');
		  }
        }
    }

    static function sqlobject(&$msg, $prompt, $acode) {
        if ($acode === '') {
		  $msg[$prompt] = dict('GEN_ERR_REQUIRED');
        } else {
		  $validchars = 'qwertyuioplkjhgfdsazxcvbnmQWERTYUIOPLKJHGFDSAZXCVBNM1234567890_';
		  if (!xValidate::checkChars($validchars, $acode)) {
			 $msg[$prompt] = dict('GEN_ERR_REQUIRED_CHAR');
		  }
        }
    }

    /**
     *
     * @param string $validchars
     * @param type $acode
     */
    static function checkFilenameChars($acode) {

        $invalidchars = '"*+,;<=>?[]|';
        $ok = true;
        for ($i = 0; $i < strlen($acode); $i++) {
		  $ok = (strpos($invalidchars, $acode[$i]) !== false) ? false : true;
		  if (empty($ok)) {
			 break;
		  }
        }
        return $ok;
    }

    /**
     * Unique key
     * @param type $msg
     * @param type $prompt
     * @param type $name
     * @param type $tablename
     */
    static function unique(&$msg, $prompt, $name, $tablename) {
        $qty = func_num_args();
        $criteria = '';
        if ($qty > CLX_NUM_FOUR) {
		  $boolean = '';
		  for ($i = CLX_NUM_FOUR; $i < $qty; $i++) {
			 $c = func_get_arg($i);
			 if (!empty($c)) {
			     $criteria .= $boolean . $c;
			     $boolean = ' AND ';
			 }
		  }
        }
        $qty = ClxCommon::$db->oneExistsRecords($tablename, $criteria);
        if ($qty > CLX_NUM_ZERO) {
		  $msg[$prompt] = dict('GEN_ERR_UNIQUE');
        }
    }

    /**
     * General string utilities
     */
    static function checkChars($validchars, $source) {
        $ok = true;
        for ($i = 0; $i < strlen($source); $i++) {

		  $ok = (strpos($validchars, $source[$i]) !== false) ? true : false;
		  if (empty($ok)) {
			 break;
		  }
        }

        return $ok;
    }

    /**
     * Return true if repeated character is found on string. Optional parameters
     * defines case sensitivity and number of repetitions to check.
     *
     * @param string $input
     * @param boolean $case_sensitive
     * @param integer $no_occurrences
     * @return boolean
     */
    static function checkCharDup($value, $case_sensitive = false, $no_occurrences = CLX_NUM_ONE) {
        $return = false;
        if (empty($case_sensitive)) {
		  $value = lowercase($value);
        }
        $counter = array();
        for ($i = CLX_INIT_ZERO; $i < strlen($value); $i++) {
		  $char = substr($value, $i, CLX_NUM_ONE);
		  if (!array_key_exists($char, $counter))
			 $counter[$char] = (int) CLX_NUM_ONE;
		  else
			 $counter[$char] ++;
		  if ($counter[$char] > $no_occurrences) {
			 $return = true;
			 break;
		  }
        }
        return $return;
    }

    /**
     * Return true is string conteins continuos characters or according to tolerance
     * Example: if tolerance = 1 then 'ABH' will error since AB are continuous
     *
     * @param string $input
     * @param boolean $case_sensitive
     * @param integer $tolerance
     * @return boolean
     */
    static function checkCharCont($value, $case_sensitive = false, $tolerance = CLX_NUM_ONE) {
        $return = false;
        if (empty($case_sensitive)) {
		  $value = lowercase($value);
        }
        $last_char = CLX_INIT_ZERO;
        for ($i = CLX_INIT_ZERO; $i < strlen($value); $i++) {
		  $char = ord(substr($value, $i, CLX_NUM_ONE));
		  if (abs($char - $last_char) <= $tolerance) {
			 $return = true;
			 break;
		  } else {
			 $last_char = $char;
		  }
        }
        return $return;
    }

    /**
     * Counts the number of characters on string and return true if
     * number of occurrences is >= than total count
     *
     * @param string $validchars
     * @param string $source
     * @param string $no_ocurrences
     * @return boolean
     */
    static function countChars($validchars, $source, $no_ocurrences = CLX_NUM_ZERO) {
        $source = lowercase((string) $source);
        $validchars = lowercase((string) $validchars);
        $i = CLX_INIT_ZERO;
        $chrs = str_split($source);
        foreach ($chrs as $val) {

		  if (strpos($validchars, (string) $val) !== false) {
			 $i++;
		  }
        }
        return ($no_ocurrences <= $i) ? true : false;
    }

}
