<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-03 15:14:44 -0400 (Thu, 03 Sep 2015) $
 * @version        $Rev: 6820 $
 *
 * CHANGE LOG
 *  Included svn prop
 */

/**
 * QUERY INSPECTOR - helps to get the complete SQL statement from a Clientix Query,
 * in order to copy and paste it in a SQL agent, such as the ClientTools > System > Utilities > SQL Tool
 * For more information contact  support@alenet.com
 *
 * @author       $Author: fsantana $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-09-03 15:14:44 -0400 (Thu, 03 Sep 2015) $
 * @version    	$Rev: 6820 $
 */
/*
  Record of changes:
  Desarrollo inicial */
class query_inspector {

    var $code;
    var $name;
    var $tablename;
    var $contents = '';
    var $general_ope_mode;
    var $general_querycode;
    var $listing_code = 'DBQYINS';
    var $debug = false;   // print debbuging info

    /**
     * Class constructor
     *
     * @param void
     * @return void
     */

    function __construct() {
        $this->code = 'DBQYINS';
        $this->name = 'Query Inspector';
        $this->tablename = 'CLX_DB_QUERIES';
    }

    /**
     * Execute queries for campaign for a given Query IDCODE. Validates the parameters
     *
     * @param int $queryid
     * @return void
     */
    function run($querycode, $mode) {
        $error = false;
        if ($querycode === '') {
            $this->contents .= 'Query Code ism' . CLX_CHAR_SINGLE_QUOTE . 'tspecified';
            $error = true;
        }
        if ($error === true) {
            return;
        }
        $this->general_ope_mode = $mode;
        $this->executeQuery($querycode);
    }

    /**
     * Execute queries for campaign for a given Query ID
     *
     * @param int $queryid
     * @return void
     */
    function executeQueryById($queryid) {
        $querycode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'ID=' . singleQuote($queryid));
        $this->executeQuery($querycode);
    }

    /**
     * Execute queries for campaign for a given Query IDCODE
     *
     * @param char $querycode
     * @return void
     */
    function executeQuery($querycode) {
        $max_time = CLX_NUM_60 * CLX_NUM_60 * CLX_NUM_ONE;
        
        UserResponse::setRunTimeLimit($max_time);
        $this->general_querycode = $querycode;
        $DBQY = &getTool('DBQY'); // The query manager
        $queryrec = $DBQY->getQuery($this->code, $querycode);
        $where_clause = $queryrec['WHERECLAUSE'];
        if (!empty($where_clause)) {
            $where_clause = str_replace('(1=0) AND', '', $where_clause);
        }
        $where_clause = 'WHERE ' . CLX_CHAR_NL . ' ' . $where_clause;
        // Columnas del Select
        $columns = str_replace(',', ',' . CLX_CHAR_NL, $queryrec['SELECTCLAUSE']);
        // From
        $from = str_replace(',', ',' . CLX_CHAR_NL, $queryrec['FROMCLAUSE']);
        $from = str_replace('FROM', 'FROM' . CLX_CHAR_NL, $queryrec['FROMCLAUSE']);
        // Order By
        $orderby = '';
        if (!empty($queryrec['SORTCLAUSE'])) {
            $orderby = str_replace(',', ',' . CLX_CHAR_NL, $queryrec['SORTCLAUSE']);
            $orderby = 'ORDER BY' . CLX_CHAR_NL . $orderby;
        }
        // Group By
        $groupby = '';
        if (!empty($queryrec['GROUPCLAUSE'])) {
            $groupby = str_replace(',', ',' . CLX_CHAR_NL, $queryrec['GROUPCLAUSE']);
            $groupby = 'GROUP BY' . CLX_CHAR_NL . $groupby;
        }
        // Finalmente construye el SQL
        $sql = 'SELECT ' . CLX_CHAR_NL . ' ' . $columns . ' ' . CLX_CHAR_NL . ' FROM ' . CLX_CHAR_NL . ' ' . $from . ' ' . CLX_CHAR_NL . ' ' . $where_clause . ' ' . CLX_CHAR_NL . ' ' . $orderby . ' ' . CLX_CHAR_NL . ' ' . $groupby . ' ' . CLX_CHAR_NL;
        // Link to Listing in a separate window
        $this->update_listing($queryrec);
        $link_to_listing = '<BR>' . '<A HREF=' . singleQuote('pop.php?to=EDIT&ac=LIST&li=' . $this->listing_code) . ' TARGET=' . singleQuote('_blank') . '>' . 'View Query Results' . '</A>';
        // Finalize output formating
        $ToPrint = 'DATA BASE QUERY TOOL INSPECTOR' . CLX_CHAR_NL . CLX_CHAR_NL . 'QUERY: ' . $this->general_querycode . CLX_CHAR_NL . $link_to_listing . ' ' . CLX_CHAR_NL . CLX_CHAR_NL . 'SQL:' . CLX_CHAR_NL . CLX_CHAR_NL . $sql . ' ' . CLX_CHAR_NL;
        // write sql statement to the output stream
        $this->contents .= str_replace(CLX_CHAR_NL, '<br>', $ToPrint);
        if (!empty($this->debug)) {
            $cmd = $queryrec['FROMWHERESORT'];
            $result = ClxCommon::$db->executeQuery('SELECT COUNT(*) ' . $cmd);
            $value = ClxCommon::$db->getNextRecord($result, NUMERIC);
        }
        if (!empty($this->debug))
            $this->contents .= '<br>COUNTER=' . $value[CLX_NUM_ZERO];
        if (!empty($this->debug))
            $this->contents .= debug_dump('queryrec', $queryrec);
    }

    /**
     * Updates the standard listing in order to show the query results
     *
     * @param array
     * @return void
     */
    function update_listing($queryrec) {
        $a_get_main_table = $this->get_main_table($queryrec['FROMCLAUSE']);
        $alias_main_table = $a_get_main_table['alias_tabla'];
        $record = array();
        $record['QUERYCODE'] = $this->general_querycode;
        $record['TABLECODE'] = $alias_main_table;
        /*
          ID, IDCODE, IDNAME, VERSION, VERSION_COUNTER, CDATE, MDATE, USERNAME, DESCRIPTION, TITLE, CAPTION, MODULECODE, TABLECODE, QUERYCODE, CREATE_FORM, MODIFY_FORM, DISPLAY_FORM, DELETE_FORM, SEARCH_FORM, CONTROL_VIEWS, CONTROL_FILTERS, VIEW_SUMMARY, VIEW_SUMMARY_EXP, EXEONSEARCH, CONTROL_EXPORT, DUPLICATE_FORM, EFFTDATE_FORM */
        ClxCommon::$db->updateRecord('CLX_UI_LISTINGS', $record, 'IDCODE = ' . singleQuote($this->listing_code));
    }

    /**
     * Returns array with the first table of the query, assuming that's the main table
     *
     * @param char
     * @return array
     */
    function get_main_table($from_clause) {
        $s_aliases_sin_from = str_replace('FROM ', '', trim($from_clause));
        $a_aliases = explode(',', $s_aliases_sin_from);
        $sw_encontro_alias_ppal = false;
        $nombre_tabla = '';
        $alias_tabla = '';
        $total = count($a_aliases);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            list($nombre_tabla, $alias_tabla) = explode(' ', trim($a_aliases[$i]));
            $sw_encontro_alias_ppal = true;
            break;
        }
        return array('nombre_tabla' => $nombre_tabla,
            'alias_tabla' => $alias_tabla,
            'sw_encontro_alias_ppal' => $sw_encontro_alias_ppal
        );
    }

    /**
     * Returns the output stream
     *
     * @param void
     * @return char
     */
    function source() {
        return $this->contents;
    }

}

// Class end
/*
Para las llamadas:
http://localhost/clx55dev/qry_insp.php
*/