<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: imrsnik $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2015-10-17 18:43:20 -0400 (Sat, 17 Oct 2015) $
 * @version        $Rev: 7344 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
/*
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
  RECORD OF CHANGES 24/01/2008		Initial development
  22/02/2008 		se agreg� la interfaz sudeban
  17/03/2008		se corrigio problema de Estatus = 'C' cuando hay problemas copiando el archivo al destino al crear registro en IBIQ
  07/07/2008		se puso <?php en vez de <? al principio para evitar conflictos al hacer el include en Windows
  05/09/2008		se agreg� la cola para la creacion de clientes de CRM
  08/09/2008		se corrigio problema con una variabe no definida: display_file() se le puso $s_filespec en vez de $filename
  //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
 */
class ibs_interface_queue {

    var $ibiqname;
    var $s_par_filespec_in;
    var $s_par_filespec_out;
    var $s_dir_repository;
    var $s_idcode = null;
    var $a_record = null;
    var $s_ibs_interface_type;
    var $s_comp_code;
    var $s_file_extension = '.txt';
    var $s_sufix_ansfile = '.ansfil';
    var $s_sufix_ans_rptfile = '.ansrpt';
    var $s_sufix_src_rptfile = '.srcrpt';
    var $s_sufix_in = '.INP.';
    var $s_sufix_out = '.OUT.';
    var $s_prefix_export_dir_file = 'ibiq_tmp.';
    var $ar_pars;
    var $s_errormsg = '';
    var $debug = false;
    var $s_msgcode_gen_file;
    var $s_msgcode_rpt_file;
    var $s_msgcode_res_file;
    var $s_msgcode_res_rpt_file;
    var $s_dir_tmp_download;
    var $s_rel_path_tmp_download;

    /**
     * Constructor
     * Parameters: [$ar_pars] = array with this values:
      'comp_code' => company code
      'interface' => interface type: 'CLI' or 'CTA' (See 'equivalence' method).
      'submodule' => 'PRES' for Severances or 'FON' for Fondafa (Venezuela)
      'subtype' => Depending on the 'interface' parameter. For 'CLI', 'I' = Individual, 'C' = Corporate client.
     * */
    function __construct($ar_pars = null) {
        if (!empty($this->debug))
            echoLog(debug_dump('ibs_interface_queue-ar_pars', $ar_pars));
        // Additional tools
        if (!function_exists('setCreationData'))
            includeCodeLib('GEN_REC_UTIL');
       
        /**
         * GEN_LBL_IBS_IBIQ_GENRPT
         * Generation Report (^1)
         * Reporte de Generaci�n (^1)
         * */
        $this->s_msgcode_rpt_file = 'GEN_LBL_IBS_IBIQ_GENRPT';
        /**
         * GEN_LBL_IBS_IBIQ_GENFILE
         * Interface File (^1)
         * Archivo de Interfaz (^1)
         * */
        $this->s_msgcode_gen_file = 'GEN_LBL_IBS_IBIQ_GENFILE';
        /**
         * GEN_LBL_IBS_IBIQ_RESFILE
         * Response File (^1)
         * Archivo de Respuesta (^1)
         * */
        $this->s_msgcode_res_file = 'GEN_LBL_IBS_IBIQ_RESFILE';
        $this->s_msgcode_res_rpt_file = 'GEN_LBL_IBS_IBIQ_RESRPT';
        $this->ar_pars = $ar_pars;
        if ($this->ar_pars != null) {
            if (array_key_exists('comp_code', $this->ar_pars)) {
                $this->s_comp_code = $this->ar_pars['comp_code'];
            }
            if (array_key_exists('interface', $this->ar_pars) && array_key_exists('submodule', $this->ar_pars) && array_key_exists('subtype', $this->ar_pars)
            ) {
                $this->s_ibs_interface_type = $this->equivalence($this->ar_pars['interface'], $this->ar_pars['submodule'], $this->ar_pars['subtype']);
            }
        }
        if ($this->s_comp_code == null)
            $this->s_comp_code = $this->ar_pars['comp_code'];
        if ($this->s_ibs_interface_type == null)
            $this->s_ibs_interface_type = $this->ar_pars['ibs_interface_type'];
        // Nombres de las tablas involucradas
        multiTableName($this, array('IBIQ'));
        // Directorio para mostrar archivos cuya extension es distinta de .TXT. Por defecto asume 'export'
        $this->s_rel_path_tmp_download = 'export/';
        $this->s_dir_tmp_download = kPathToWebSite . $this->s_rel_path_tmp_download;
    }

// Method end
    /**
     * Inicializa los parametros. Devuelve false si algo salio mal
     * */
    function initpar($s_comp_code = null, $s_ibs_interface_type = null) {
        // Asigna parametros externos en las propiedades de esta clase
        if ($s_comp_code != null)
            $this->s_comp_code = $s_comp_code;
        if ($s_ibs_interface_type != null)
            $this->s_ibs_interface_type = $s_ibs_interface_type;
        // asigna propiedades seg�n el tipo de interface
        $b_response = $this->interface_data_selector($this->s_ibs_interface_type);
        return $b_response;
    }

// Method end
    /**
     * asigna propiedades seg�n el tipo de interface. Devuelve false si el tipo de interfaz no es reconocida
     * */
    function interface_data_selector() {
        $b_response = true;
        switch ($this->s_ibs_interface_type) {
            // PRE_CL1		Prestaciones: Clientes Pers. Naturales
            // FON_CL1 	Fondafa: Clientes Pers. Naturales
            // CRM_CL1	CRM: Clientes Pers. Naturales
            case 'PRE_CL1':
            case 'FON_CL1':
            case 'CRM_CL1':
                $this->s_par_filespec_in = companyParameter('EIBS_CLIEX_FILESPEC_I', $this->s_comp_code);
                $this->s_par_filespec_out = companyParameter('EIBS_CLIEX_RESP_FILESPEC_I', $this->s_comp_code);
                break;
            // PRE_CL2 	Prestaciones: Clientes Pers. Jur�dicas
            // FON_CL2 	Fondafa: Clientes Pers. Jur�dicas
            // CRM_CL2 	CRM: Clientes Pers. Jur�dicas
            case 'PRE_CL2':
            case 'FON_CL2':
            case 'CRM_CL2':
                $this->s_par_filespec_in = companyParameter('EIBS_CLIEX_FILESPEC_C', $this->s_comp_code);
                $this->s_par_filespec_out = companyParameter('EIBS_CLIEX_RESP_FILESPEC_C', $this->s_comp_code);
                break;
            // PRE_AC 		Prestaciones: Creaci�n Ctas. de Bco.
            case 'PRE_AC':
                $this->s_par_filespec_in = companyParameter('PRES_ACC_REQ_EXPRT_FILE', $this->s_comp_code);
                $this->s_par_filespec_out = companyParameter('PRES_ACC_REQ_IMPRT_FILE', $this->s_comp_code);
                break;
            // FON_AC 	Fondafa: Creaci�n de Ctas. de Bco.
            case 'FON_AC':
                $this->s_par_filespec_in = companyParameter('FONDAFA_ACC_REQ_EXPRT_FILE', $this->s_comp_code);
                $this->s_par_filespec_out = companyParameter('FONDAFA_ACC_REQ_IMPRT_FILE', $this->s_comp_code);
                break;
            // SUDEBAN export
            case 'SUDEBAN':
                $this->s_par_filespec_in = '';
                $this->s_par_filespec_out = null;
                $this->s_msgcode_rpt_file = 'SUDEBAN_DEXP_EXPRT_TITLE_ERRORS';
                break;
            // Es la verificacion de la cola....
            case null:
                break;
            // Error en el parametro de tipo de interfaz
            default:
                /**
                 * GEN_ERR_IBS_IBIQ_WRONG_DATA
                 * Inconsistency error: wrong IBS interface type: '^1'. Interface: '^2', submodule: '^3', person type: '^4'
                 * Error de inconsistencia: tipo errado de interfaz IBS: '^1'. Interface: '^2', subm�dulo: '^3', tipo de persona: '^4'
                 */
                $this->s_errormsg = 'IDS.01=' . dict('GEN_ERR_IBS_IBIQ_WRONG_DATA', $this->s_ibs_interface_type, $this->ar_pars['interface'], $this->ar_pars['submodule'], $this->ar_pars['subtype']);
                $b_response = false;
        }
        // Parametros generales de trabajo
        if ($this->s_comp_code != null) {
            ClxFreeReturn($this->ass_dir_repository());
        }
        return $b_response;
    }

// Method end
    /**
     * asigna el directorio de IBIQ para la compa�ia corriente
     * */
    function ass_dir_repository() {
        // debe tener el / final
        $this->s_dir_repository = companyParameter('EIBS_IBIQ_REPOSITORY', $this->s_comp_code);
        return $this->s_dir_repository;
    }

    /**
     * Procesa un archivo que deber� ser encolado. Devuelve true si todo fue hecho correctamente
     * */
    function enter_queue($s_src_filename, $s_rep_filename = null) {
        if (!empty($this->debug))
            echoLog('<br>enter_queue(' . singleQuote($s_src_filename) . ', ' . singleQuote($s_rep_filename) . ')');
        // Respuesta de este metodo
        $b_response = true;
        // Inserta el registro en la cola
        ClxFreeReturn($this->ibiq_row_insert());
        $a_update_record = array();
        $a_update_record['COMMENTS'] = '';
        // Copia al repositorio el archivo fuente
        $s_title = $this->s_msgcode_gen_file;
        $s_filename = $s_src_filename;
        if (is_array($s_filename)) {
            $s_title = $s_filename[CLX_NUM_ONE];
            $s_filename = $s_filename[CLX_NUM_ZERO];
        }
        $path_parts = pathinfo($s_filename);
        $a_data = array();
        $a_data['FTYPE'] = 'IFO';
        $a_data['ORIG_DIR'] = $path_parts['dirname'];
        $a_data['ORIG_FILENAME'] = $path_parts['basename'];
        $a_data['TITLE'] = $s_title;
        $a_data['FILENAME'] = $this->s_idcode . $this->s_sufix_in . $path_parts['basename'];
        $s_output_filename = $this->s_dir_repository . $a_data['FILENAME'];
        if (!empty($this->debug))
            echoLog('<br>enter_queue-1-copy($s_src_filename, $s_output_filename)');
        if (!copy($s_src_filename, $s_output_filename)) {
            $this->s_errormsg = 'EQ.01=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_src_filename, $s_output_filename);
            $b_response = false;
        }
        $a_data['ERRORMSG'] = $this->s_errormsg;
        ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));

        if (!empty($this->debug))
            echoLog('<br>s_rep_filename = $s_rep_filename | b_response: ' . ($b_response ? 'true' : 'false'));
        // Copia al repositorio el reporte asociado al archivo fuente
        if ($b_response && $s_rep_filename != null) {
            if (!empty($this->debug))
                echoLog('<br>enter_queue-qqq | s_rep_filename = $s_rep_filename');
            $s_title = $this->s_msgcode_rpt_file;
            $s_filename = $s_rep_filename;
            if (is_array($s_filename)) {
                $s_title = $s_filename[CLX_NUM_ONE];
                $s_filename = $s_filename[CLX_NUM_ZERO];
            }
            $path_parts = pathinfo($s_filename);
            $a_data = array();
            $a_data['FTYPE'] = 'RFO';
            $a_data['ORIG_DIR'] = $path_parts['dirname'];
            $a_data['ORIG_FILENAME'] = $path_parts['basename'];
            $a_data['TITLE'] = $s_title;
            $a_data['FILENAME'] = $this->s_idcode . $this->s_sufix_in . $path_parts['basename'];
            $s_output_filename = $this->s_dir_repository . $a_data['FILENAME'];
            if (!empty($this->debug))
                echoLog('<br>enter_queue-2-copy(' . $s_filename . ', ' . $s_output_filename . ')');
            if (!copy($s_filename, $s_output_filename)) {
                $this->s_errormsg = 'EQ.02=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_filename, $s_output_filename);
                $b_response = false;
            }
            $a_data['ERRORMSG'] = $this->s_errormsg;
            ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));
        }
        // Updates comment columns of created IBIQ row
        if (empty($b_response))
            $a_update_record['REC_STATUS'] = 'ENDR';
        $this->ibiq_update_record($a_update_record);
        return $b_response;
    }

// Method end
    /**
     * Verifica la cola, si hay algo pendiente, verifica si el servicio batch est� desocupado, si no espera...
     * */
    function verif_queue() {
        $a_update_record = array();
        $this->s_errormsg = '';
        $sql = 'SELECT * FROM ' . $this->ibiqname . ' IBIQ WHERE IBIQ.REC_STATUS = ' . singleQuote('IDLE') . ' ORDER BY CDATE ASC ';
        if (!empty($this->debug))
            echoLog('verif_queue<BR>' . $sql . '<BR>');
        $results = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $this->s_ibs_interface_type = $record['IBS_INTERFACE_TYPE'];
            ClxFreeReturn($this->get_ibiq_rec_and_set_locals($record['IDCODE'], $record));

            if (!empty($this->debug))
                echoLog('IDOCDE=' . $record['IDCODE'] . ', s_ibs_interface_type=' . $this->s_ibs_interface_type . ', s_comp_code=' . $this->s_comp_code . '<BR>');
            if (!$this->interface_data_selector()) {
                // error en el registro de la cola
                $a_update_record['REC_STATUS'] = 'ENDR';
                $a_data = array();
                $a_data['FTYPE'] = 'ERR';
                $a_data['ERRORMSG'] = $this->s_errormsg;
                ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));
                $this->ibiq_update_record($a_update_record);
            } else {
                if (!file_exists($this->s_par_filespec_in) && !file_exists($this->s_par_filespec_out)) {
                    // Ejecuta la copia del archivo del repositorio al directorio destino y el cambio de status en la cola
                    $a_resp = $this->get_comments_element(CLX_NUM_ZERO);
                    if (!empty($a_resp['error'])) {
                        $this->s_errormsg = $a_resp['errormsg'];
                        $a_update_record['REC_STATUS'] = 'ENDR';
                        $a_data = array();
                        $a_data['FTYPE'] = 'ERR';
                        $a_data['ERRORMSG'] = $this->s_errormsg;
                        ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));

                        $this->ibiq_update_record($a_update_record);
                    } else {
                        $s_input_filename = $this->s_dir_repository . $a_resp[CLX_CLASS_DATA]['FILENAME'];
                        if (!empty($this->debug))
                            echoLog('<br>verif_queue-1-copy(' . $s_input_filename . ', ' . $this->s_par_filespec_in . ')');
                        if (!copy($s_input_filename, $this->s_par_filespec_in)) {
                            // No se pudo hacer la copia
                            /*
                             * GEN_ERR_FILESYS_COPYFAIL
                             * Error copying from file  ^1 to file ^2
                             * Error realizando la copia del archivo ^1 al archivo ^2
                             */
                            $this->s_errormsg = 'VQ.01=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_input_filename, $this->s_par_filespec_in);
                            $a_update_record['REC_STATUS'] = 'ENDR';
                            $a_data = array();
                            $a_data['FTYPE'] = 'ERR';
                            $a_data['ERRORMSG'] = $this->s_errormsg;
                            ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));
                            $this->ibiq_update_record($a_update_record);
                        } else {
                            // Cambia estatus al record
                            $a_update_record['REC_STATUS'] = 'RUN';
                            $this->ibiq_update_record($a_update_record);
                        }
                    }
                    // Aun no se puede...
                }
            }
            // sigue procesando los varios registros de la cola...
        }
        unset($results);
    }

// Method end
    /**
     * Procesa una solicitud de salida de la cola. Devuelve true si todo fue hecho correctamente
     * En $a_result_reports esta la lista de reportes de resultados para la interfaz en particular, los cuales ser�n copiados al directorio repositorio
     * luego de haber terminado el proceso
     * */
    function salir_queue($a_result_reports, $s_qstatus = 'RUN') {
        if (!empty($this->debug))
            echoLog(debug_dump('ibs_interface_queue-salir_queue-a_result_reports', $a_result_reports));
        $a_update_record = array();
        $a_update_record['COMMENTS'] = '';
        // Trae el registro en proceso para el tipo de interfaz especificada. Se supone que solo debe haber uno por cada tipo de interfaz con status RUN
        $record = ClxCommon::$db->getRecord($this->ibiqname, 'IBS_INTERFACE_TYPE = ' . singleQuote($this->s_ibs_interface_type) . ' AND REC_STATUS = ' . singleQuote($s_qstatus));
        $b_response = !empty($record);
        if ($b_response == false) {
            /**
             * GEN_ERR_IBS_IBIQ_NOPENDREC
             * Inconsistency error: there's no record type ^1 pending for termination in the interace queue
             * Error de inconsistencia: no hay registro de tipo ^1 pendiente por terminar en la cola de interfaces
             * */
            $this->s_errormsg = 'XQ.03=' . dict('GEN_ERR_IBS_IBIQ_NOPENDREC', $this->s_ibs_interface_type);
            $b_response = false;
        } else {
            // Para futuras referencias (ver cop_file_to_ibiq_dir())
            $a_update_record['COMMENTS'] = $record['COMMENTS'];
            ClxFreeReturn($this->get_ibiq_rec($record['IDCODE'], $record));

            // Copia los resultados al repositorio
            $s_title = $this->s_msgcode_res_file;
            $s_filename = $this->s_par_filespec_out;
            if (is_array($s_filename)) {
                $s_title = $s_filename[CLX_NUM_ONE];
                $s_filename = $s_filename[CLX_NUM_ZERO];
            }
            if ($s_filename != null && $s_filename !== '') {
                $path_parts = pathinfo($s_filename);
                $a_data = array();
                $a_data['FTYPE'] = 'IFR';
                $a_data['ORIG_DIR'] = $path_parts['dirname'];
                $a_data['ORIG_FILENAME'] = $path_parts['basename'];
                $a_data['TITLE'] = $s_title;
                $a_data['FILENAME'] = $this->s_idcode . $this->s_sufix_out . $path_parts['basename'];
                $s_output_filename = $this->s_dir_repository . $a_data['FILENAME'];
                if (!empty($this->debug))
                    echoLog('<br>salir_queue-1-copy($s_filename, $s_output_filename)');
                if (!copy($s_filename, $s_output_filename)) {
                    /*
                     * GEN_ERR_FILESYS_COPYFAIL
                     * Error copying from file  ^1 to file ^2
                     * Error realizando la copia del archivo ^1 al archivo ^2
                     */
                    $this->s_errormsg = 'XQ.01=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_filename, $s_output_filename);
                    $b_response = false;
                }
                $a_data['ERRORMSG'] = $this->s_errormsg;
                ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));
            }
            if (!empty($b_response)) {
                // Copia los reportes al repositorio
                $total = count($a_result_reports);
                for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
                    $s_title = $this->s_msgcode_res_rpt_file;
                    $s_filename = $a_result_reports[$i];
                    if (is_array($s_filename)) {
                        $s_title = $s_filename[CLX_NUM_ONE];
                        $s_filename = $s_filename[CLX_NUM_ZERO];
                    }
                    $path_parts = pathinfo($s_filename);
                    $a_data = array();
                    $a_data['FTYPE'] = 'RFR';
                    $a_data['ORIG_DIR'] = $path_parts['dirname'];
                    $a_data['ORIG_FILENAME'] = $path_parts['basename'];
                    $a_data['TITLE'] = $s_title;
                    $a_data['FILENAME'] = $this->s_idcode . $this->s_sufix_out . $path_parts['basename'];
                    $s_output_filename = $this->s_dir_repository . $a_data['FILENAME'];
                    if (!empty($this->debug))
                        echoLog('<br>salir_queue-2-copy($s_filename, $s_output_filename)');
                    if (!copy($s_filename, $s_output_filename)) {
                        /*
                         * GEN_ERR_FILESYS_COPYFAIL
                         * Error copying from file  ^1 to file ^2
                         * Error realizando la copia del archivo ^1 al archivo ^2
                         */
                        $this->s_errormsg = 'XQ.02=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_filename, $s_output_filename);
                        $b_response = false;
                    }
                    $a_data['ERRORMSG'] = $this->s_errormsg;
                    ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));
                }
            }
            if (!empty($b_response)) {
                // Asigna estatus END y la fecha y hora de terminacion
                $a_update_record['REC_STATUS'] = 'END';
                $a_update_record['COMPLETED_DATE'] = xFormat::getNow('Y-m-d H:i:s');
            } else {
                // Asigna estatus ENDR ya que hubo errores
                $a_update_record['REC_STATUS'] = 'ENDR';
            }
            $this->ibiq_update_record($a_update_record);
        }
        return $b_response;
    }

// Method end
    /**
     * Devuelve el token que se pone entre el IDCODE del record de IBIQ y el nombre del archivo original
     * al momento de asignar el nombre del archivo en el directorio del respositorio de ibiq
     * */
    function sufix_type($s_comments_record_type) {
        switch ($s_comments_record_type) {
            case 'IFO': // interface file sent (original),
            case 'RFO': // Report file cooresponding to IFO
                $s_sufix = $this->s_sufix_in;
                break;
            case 'IFR': // interface file received,
            case 'RFR': // Report file cooresponding to IFR
                $s_sufix = $this->s_sufix_out;
                break;
            case 'ERR': // General Error
                $s_sufix = '';
                break;
            default:
                $s_sufix = '?';
        }
        return $s_sufix;
    }

    /**
     * copia un archivo al directorio de la cola
     * */
    function cop_file_to_ibiq_dir($s_input_filespec, $s_title, $s_sufix_out = null, $b_make_comments_entry = true, $s_idcode = null) {
        // Valorespordefecto para los parametros
        if ($s_idcode == null) {
            $s_idcode = $this->s_idcode;
        } else {
            ClxFreeReturn($this->get_ibiq_rec($s_idcode));
        }
        if ($s_comments_record_type == null)
            $s_comments_record_type = 'RFR';
        $s_sufix_out = $this->sufix_type($s_comments_record_type);
        // Copia el archivo
        if (!empty($this->debug))
            echoLog('<br>cop_file_to_ibiq_dir("' . $s_input_filespec . '", "' . $s_title . '", "' . $s_sufix_out . '", "' . $b_make_comments_entry . '", "' . $s_idcode . '")');
        $a_response = array();
        $a_response['error'] = false;
        $s_realpath = realpath($s_input_filespec);
        $path_parts = pathinfo($s_realpath);
        $s_filename = $s_idcode . $s_sufix_out . $path_parts['basename'];
        $a_response['output_filename'] = $this->s_dir_repository . $s_filename;
        if (!empty($this->debug))
            echoLog('<br>copy($s_realpath, ' . $a_response['output_filename'] . ')');
        if (!copy($s_realpath, $a_response['output_filename'])) {
            $a_response['error'] = true;
            $a_response['errormsg'] = 'CFTI.01=' . dict('GEN_ERR_FILESYS_COPYFAIL', $s_realpath, $a_response['output_filename']);
            $this->s_errormsg = $a_response['errormsg'];
        }
        // Agrega la entrada en el campo de comentarios
        if (!empty($b_make_comments_entry)) {
            // Carga el contenido actual
            $a_update_record = array();
            $a_update_record['COMMENTS'] = $this->a_record['COMMENTS'];
            // Agrega el nuevo archivo
            $a_data = array();
            $a_data['FTYPE'] = $s_comments_record_type;
            $a_data['ORIG_DIR'] = $path_parts['dirname'];
            $a_data['ORIG_FILENAME'] = $path_parts['basename'];
            $a_data['TITLE'] = $s_title;
            $a_data['FILENAME'] = $s_filename;
            $a_data['ERRORMSG'] = $this->s_errormsg;
            ClxFreeReturn($this->add_to_comment_array($a_update_record, $a_data));

            // Guarda el dato actualizado
            $this->ibiq_update_record($a_update_record);
        }
        return $a_response;
    }

// Method end
    /**
     * Calculates the s_ibs_interface_type variable according to interface type, submodule and person type combination
     * Parameters:
     * 	$s_opt => interface type: 'CLI' or 'CTA' (See 'equivalence' method).
     * 	$s_submodule => 'PRES' for Severances or 'FON' for Fondafa (Venezuela)
     * 	$s_subtype => Depending on the 'interface' parameter. For 'CLI', 'I' = Individual, 'C' = Corporate client.
     * */
    function equivalence($s_opt, $s_submodule, $s_subtype) {
        $s_ibs_interface_type = '??';
        if ($s_submodule !== '') {
            if ($s_opt === 'CLI') {
                // Massive Client Creation
                if ($s_subtype === 'I') { // Individuals | Personas naturales
                    $s_ibs_interface_type = $this->equivalence_submod_cli($s_submodule, '1');
                } elseif ($s_subtype === 'C') { // corporate | Personas juridicas
                    $s_ibs_interface_type = $this->equivalence_submod_cli($s_submodule, '2');
                }
            } elseif ($s_opt === 'CTA') {
                // Massive Banck Account Creation
                $s_ibs_interface_type = ($s_submodule === 'PRES' ? 'PRE_AC' : 'FON_AC');
            }
        } elseif ($s_opt === 'SUDEBAN') {
            // Venezuela: SUDEBAN export
            $s_ibs_interface_type = 'SUDEBAN';
        }
        if (!empty($this->debug))
            echoLog('equivalence: s_opt=' . $s_opt . ', s_submodule=' . $s_submodule . ', s_subtype=' . $s_subtype . ',  s_ibs_interface_type=' . $s_ibs_interface_type . '<br>');
        return $s_ibs_interface_type;
    }

// Method end
    function equivalence_submod_cli($s_submodule, $numero) {
        $s_response = null;
        if ($s_submodule === 'PRES') {
            $s_response = 'PRE_CL' . $numero;
        } elseif ($s_submodule === 'CRM') {
            $s_response = 'CRM_CL' . $numero;
        } else {
            $s_response = 'FON_CL' . $numero;
        }
        return $s_response;
    }

    /**
     * Returns the last error message generated
     * */
    function get_errormsg() {
        return $this->s_errormsg;
    }

// Method end
    /**
     * Returns file contents to the response stream
     * */
    function display_file($s_idcode, $s_cual) {
        if (!empty($this->debug))
            echolog('display_file-s_idcode=' . singleQuote($s_idcode) . ', s_cual=' . singleQuote($s_cual));
        $s_response = '';
        $n_cual = CLX_NUM_ZERO + $s_cual;
        if (empty($this->get_ibiq_rec_and_set_locals($s_idcode))) {
            /*
             * GEN_MSG_RECORD_NOTFOUND
             * Record not found: ^1
             * No se encontr� el registro: ^1
             */
            $s_response .= dict('GEN_MSG_RECORD_NOTFOUND', $s_idcode);
        } else {
            $a_response = $this->get_comments_element($n_cual);
            if (!empty($this->debug))
                echoLog(debug_dump('get_comments_element(' . $n_cual . ')', $a_response));
            if ($a_response['error'] === true) {
                // entrada especificada es invalida
                $s_response .= $a_response['errormsg'];
            } else {
                $b_con_pre = false;
                $q = strpos('.txt.', '.' . lowercase($a_response[CLX_CLASS_DATA]['ORIG_FILENAME_EXT']));
                if (!empty($this->debug))
                    echoLog('<br>q = $q | Extension = ' . singleQuote(lowercase($a_response[CLX_CLASS_DATA]['ORIG_FILENAME_EXT'])));
                if ($q !== false) {
                    // Aplica el <PRE> para que se vean bien los espacios en blanco en los archivos de texto plano
                    $b_con_pre = true;
                }
                $s_filespec = $this->s_dir_repository . $a_response[CLX_CLASS_DATA]['FILENAME'];
                if (!empty($this->debug))
                    echoLog('<br>s_filespec=$s_filespec');
                if (!file_exists($s_filespec)) {
                    /*
                     * GEN_ERR_CANT_OPEN_FILE
                     * Invalid file
                     * Archivo invalido
                     */
                    $s_response .= dict('GEN_ERR_CANT_OPEN_FILE') . ' (' . $s_filespec . ')';
                } else {
                    $s_response .= $this->response_file(dict($a_response[CLX_CLASS_DATA]['TITLE'], $a_response[CLX_CLASS_DATA]['FILENAME'], $a_response[CLX_CLASS_DATA]['ORIG_FILENAME']), $s_filespec, $b_con_pre) . '<BR>'; // show the file
                }
            }
        }
        return $s_response;
    }

// Method end
    /**
     * Returns 'comments' column to display in editor with links instead of real coded content.
     * */
    function display_comments_field($s_idcode) {
        $s_response = '';
        if (empty($this->get_ibiq_rec_and_set_locals($s_idcode))) {
            /*
             * GEN_MSG_RECORD_NOTFOUND
             * Record not found: ^1
             * No se encontr� el registro: ^1
             */
            $s_response .= dict('GEN_MSG_RECORD_NOTFOUND', $s_idcode);
        } else {
            $a_response = $this->get_comments_array();
            $total = count($a_response[CLX_CLASS_DATA]);
            for ($i = $total - CLX_NUM_ONE; $i >= CLX_INIT_ZERO; $i--) {
                $s_response .= $this->make_link($a_response[CLX_CLASS_DATA][$i], $this->get_ibiq_rec_and_set_locals($s_idcode), $i);
            }
        }
        return $s_response;
    }

// Method end
    /**
     * Returns a link to display file contents. The call for this method must be located on Load Exception of IBIQ Table
     * */
    function make_link($a_comment_element, $record, $s_cual) {
        if (!empty($this->debug)) {
            echoLog('make_link (s_cual=' . $s_cual . ')');
            echoLog(debug_dump('a_comment_element', $a_comment_element));
            echoLog(debug_dump('record', $record));
        }
        $s_idcode = $record['IDCODE'];
        $s_title = dict($a_comment_element['TITLE'], $a_comment_element['ORIG_FILENAME']);
        // Por defecto llama a ibiq_show.php
        $s_response = '<A href="ibiq_show.php?idcode=' . $s_idcode . '&mod=' . $s_cual . '") TARGET="new")>' . $s_title . '</A>';
        // Verifica si la extension del archivo es distinta de html, htm, txt
        if (strpos('.txt.htm.html.log.', '.' . lowercase($a_comment_element['ORIG_FILENAME_EXT'])) === false) {
            // Es necesario copiar el file a un directorio bajo el �mbito del webserver para poder hacer el download
            $s_src_filename = $this->s_dir_repository . $a_comment_element['FILENAME'];
            $s_output_filename = $this->s_dir_tmp_download . $a_comment_element['ORIG_FILENAME'];
            if (!empty($this->debug))
                echoLog('<br>copy(' . $s_src_filename . ', ' . $s_output_filename . ')');
            if (!copy($s_src_filename, $s_output_filename)) {
                $s_response = dict('GEN_ERR_FILESYS_COPYFAIL', $s_src_filename, $s_output_filename);
            } else {
                $s_response = '<A href="' . $this->s_rel_path_tmp_download . $a_comment_element['ORIG_FILENAME'] . '">' . $s_title . '</A>';
            }
        }
        if ($a_comment_element['ERRORCODE'] !== '') {
            $s_response .= '<BR>' .
                    $a_comment_element['ERRORMSG'] . ' (' . $a_comment_element['ERRORCODE'] . ')';
        }
        $s_response .= '<BR><BR>';
        return $s_response;
    }

// Method end
    /**
     * File contents output to response stream
     * */
    function response_file($s_title, $s_filespec, $b_con_pre) {
        if (!empty($this->debug))
            echolog('response_file-s_title=' . singleQuote($s_title) . ', s_filespec=' . singleQuote($s_filespec));
        $s_response = '';
        $o_clx_file = new clx_file(trim($s_filespec), 'r');
        $o_clx_file->open();
        if ($o_clx_file->error === true) {
            $s_response .= $o_clx_file->errorMsg . '<BR>';
        } else {
            $s_response .= ($b_con_pre ? '<pre>' : '');
            $buffer = $o_clx_file->read_line();
            while (!$o_clx_file->eof) {
                $s_response .= singleQuote($buffer);
                $buffer = $o_clx_file->read_line();
            }
            $s_response .= ($b_con_pre ? '</pre>' : '');
            $o_clx_file->close();
            if ($o_clx_file->error === true) {
                $s_response .= $o_clx_file->errorMsg . '<BR>';
            }
        }
        return singleQuote($s_response);
    }

// Method end
    /**
     * Calls get_ibiq_rec() method, set company code and IBIQ directory/folder
     * */
    function get_ibiq_rec_and_set_locals($s_idcode, $record = null) {
        $record = $this->get_ibiq_rec($s_idcode, $record);
        $this->s_comp_code = $record['COMP_CODE'];
        ClxFreeReturn($this->ass_dir_repository());
        return $record;
    }

    /**
     * Get one IBIQ record and putdata in class properties
     * */
    function get_ibiq_rec($s_idcode, $record = null) {
        $this->s_idcode = $s_idcode;
        if ($record == null) {
            // It isn') . 't loaded
            $this->a_record = ClxCommon::$db->getRecord($this->ibiqname, 'IDCODE = ' . singleQuote($s_idcode));
        } else {
            // already loaded, so copy data in the property
            $this->a_record = $record;
        }
        return $this->a_record;
    }

// Method end
    /**
     * Returns one element in the comments column array
     * */
    function get_comments_element($n_cual, $record = null) {
        $a_response = array();
        $a_response['error'] = false;
        $a_response['errormsg'] = '';
        $a_response[CLX_CLASS_DATA] = null;
        if ($record == null)
            $record = $this->a_record;
        $a_resp1 = $this->get_comments_array($record);
        if ($n_cual < CLX_NUM_ZERO || $n_cual > count($a_resp1[CLX_CLASS_DATA]) - CLX_NUM_ONE) {
            $a_response['error'] = true;
            /*
             * GEN_ERR_INVALID_ENTRY
             * Invalid Entry
             * Entrada Inv�lida
             */
            $a_response['errormsg'] = dict('GEN_ERR_INVALID_ENTRY');
        } else {
            $a_response[CLX_CLASS_DATA] = $a_resp1[CLX_CLASS_DATA][$n_cual];
        }
        return $a_response;
    }

// Method end
    /**
     * Returns comments column in array structured and ready to be processed
     * */
    function get_comments_array($record = null) {
        if ($record == null)
            $record = $this->a_record;
        $a_response = array();
        $a_response['error'] = ($record['REC_STATUS'] === 'ENDR');
        $a_response['errormsg'] = '';
        $a_response[CLX_CLASS_DATA] = array();
        $a_tmp = explode(CLX_CHAR_NL, $record['COMMENTS']); // split CR+LFs
        if (!empty($this->debug))
            echoLog(debug_dump('get_comments_array-a_tmp', $a_tmp));
        $total = count($a_tmp);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            if ($a_tmp[$i] !== '') {
                $a_tmp_2 = explode(' | ', $a_tmp[$i]);  // Split the specific file data
                $a_response[CLX_CLASS_DATA][] = array();
                $j = count($a_response[CLX_CLASS_DATA]) - CLX_NUM_ONE;
                $a_response[CLX_CLASS_DATA][$j]['FTYPE'] = $a_tmp_2[CLX_NUM_ZERO];
                $a_response[CLX_CLASS_DATA][$j]['ORIG_DIR'] = $a_tmp_2[CLX_NUM_ONE];
                $a_response[CLX_CLASS_DATA][$j]['ORIG_FILENAME'] = $a_tmp_2[CLX_NUM_TWO];
                $a_response[CLX_CLASS_DATA][$j]['TITLE'] = $a_tmp_2[CLX_NUM_THREE];
                $a_response[CLX_CLASS_DATA][$j]['FILENAME'] = $a_tmp_2[CLX_NUM_FOUR];
                $a_response[CLX_CLASS_DATA][$j]['ERRORMSG'] = '';
                $a_response[CLX_CLASS_DATA][$j]['ERRORCODE'] = '';
                if ($a_tmp_2[CLX_NUM_FIVE] !== '') {
                    $a_tmp_3 = explode('=', $a_tmp_2[CLX_NUM_FIVE]);  // Split the specific error data
                    $a_response[CLX_CLASS_DATA][$j]['ERRORMSG'] = $a_tmp_3[CLX_NUM_ONE];
                    $a_response[CLX_CLASS_DATA][$j]['ERRORCODE'] = $a_tmp_3[CLX_NUM_ZERO];
                    $a_response['errormsg'] = $a_tmp_3[CLX_NUM_ZERO] . '=' . $a_tmp_3[CLX_NUM_ONE];
                }
                // Original file extension
                $path_parts = pathinfo($a_response[CLX_CLASS_DATA][$j]['ORIG_FILENAME']);
                $a_response[CLX_CLASS_DATA][$j]['ORIG_FILENAME_EXT'] = $path_parts['extension'];
            }
        }
        if (!empty($this->debug))
            echoLog(debug_dump('get_comments_array-a_response', $a_response));
        return $a_response;
    }

// Method end
    /**
     * Add an entry to comment column array
     * */
    function add_to_comment_array(&$record, $a_data) {
        if ($a_data['FTYPE'] === 'ERR') {
            // By default, those elements isn') . 't necessary for ERR entry type
            $a_data['ORIG_DIR'] = '';
            $a_data['ORIG_FILENAME'] = '';
            $a_data['TITLE'] = '';
            $a_data['FILENAME'] = '';
        }
        $s_data = $a_data['FTYPE'] . ' | ' . $a_data['ORIG_DIR'] . ' | ' . $a_data['ORIG_FILENAME'] . ' | ' . $a_data['TITLE'] . ' | ' . $a_data['FILENAME'] . ' | ' . $a_data['ERRORMSG'];
        if (!isset($record['COMMENTS']))
            $record['COMMENTS'] = '';
        $record['COMMENTS'] .= $s_data . CLX_CHAR_NL;
        return $record;
    }

// Method end
    /**
     * Inserta un registro en la tabla de la cola
     * */
    function ibiq_row_insert() {
        $record = array();
        $record['IBS_INTERFACE_TYPE'] = $this->s_ibs_interface_type;
        $record['REC_STATUS'] = 'IDLE';
        $record['COMP_CODE'] = $this->s_comp_code;
        $record['CREATEDBY'] = '';
        $record['CDATE'] = '';
        $record = setCreationData($record);
        $record['IDCODE'] = 'AUTO';
        $record = setCreationIdCode($record, 'IBIQ');
        if (!empty($this->debug))
            ClxCommon::$db->debug = true;
        $i_id = ClxCommon::$db->createRecord($this->ibiqname, $record, true);
        if (!empty($this->debug))
            ClxCommon::$db->debug = false;
        $this->s_idcode = $record['IDCODE'];
        return $i_id;
    }

// Method end
    /**
     * Updates the current record on IBIQ Table
     * */
    function ibiq_update_record($a_update_record, $s_idcode = null) {
        if ($s_idcode == null)
            $s_idcode = $this->s_idcode;
        ClxCommon::$db->updateRecord($this->ibiqname, $a_update_record, 'IDCODE = ' . singleQuote($s_idcode));
        ClxFreeReturn($this->get_ibiq_rec($s_idcode));
    }

// Method end
}

// Class End
/*
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- DOCUMENTACION -----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------  Caso 1000370 - FONDAFA/PRESTACIONES interfaz multiusuario
  Analisis  - Desarrollar una clase que maneje una cola de procesos batch de tal forma que varios usuarios puedan
  disparar estos procesos de manera concurrente. - Cada caso debe tener una tabla con una cola de los procesos pendientes o una cola com�n clasificada por
  el tipo de interfaz.  Desarrollo  * Tabla IBIQ
  Nombre:				IBS_INTERFACE_QUEUE)
  Descripci�n:		IBS Interface Queue: To handle multiple IBS batch interface user requests
  C�digo de M�dulo	FIDU Debe tener la siguiente estructura:
  IDCODE				=	numero correlativo COMP_CODE			=	compa�ia IBS_INTERFACE_TYPE	=	tipo de interfaz (Lista IITY) REC_STATUS			=	Indica si esta pendiente o ya se proceso CREATEDBY			=	usuario que dispar� el proceso CDATE				=	fecha en que se dispar� el proceso COMPLETED_DATE		=	fecha en que fue procesado COMMENTS			=	Con los nombres de los archivos a ser inclu�dos en el repositorio  * Campo IBS_INTERFACE_TYPE
  Descripci�n:		IBS Interface type: to know interface caracteristics of pending and processed requests in queue
  Tipo:				List
  Required:			No
  List Code:			IITY
  Default Item:
  Etiqueta de Lista:	Tipo de Interfaz
  Etiqueta de Forma:	Tipo de Interfaz
  La idea es que el proceso haga la entrada en la cola, guarde el archivo de entrada en un repositorio y ponga el estatus IDLE al registro. El nombre del archivo ser� el IDCODE con la extensi�n .txt Seg�n el tipo de proceso, se verifica el directorio/archivo de entrada y el directorio/archivo de salida.
  Si existe alguno de los dos, el proceso no copia el archivo correspondiente. Se parte de la premisa que el archivo de entrada y el de salida son eliminados por los correspondientes procesos (el de IBS y el de CLX respectivamente) una vez procesados. Cuando no existe ninguno de los dos, se copia el archivo al directorio de entrada y se asigna al registro el status RUN. Una vez procesado el registro de entrada por el IBS, se borra el archivo de entrada y se genera el archivo de salida. CLX tiene un proceso que verifica si ya se gener� ese archivo de salida, busca en la cola
  un registro de su tipo espec�fico que est� en RUN y toma el IDCODE; procesa la respuesta, copia ese archivo
  con la respuesta en el repositorio con el IDCODE agreg�ndole '.ansfil.txt' al final del nombre y generando un archivo en el repositorio con el reporte de procesamiento de esa respuesta, cuyo nombre ser� el IDCODE agreg�ndole '.rpt.txt' al final. Finalmente al terminar el procesamiento, asignar� END al registro de la cola.  * Se necesita un directorio especial para guardar estos archivos, por ejemplo
  /home/fideicomiso/fide/ibs_int_queue/
  Configurado mediante el par�metro: EIBS_IBIQ_REPOSITORY
 * Es necesario tener una entrada en la tabla de C�digos correspondiente a la tabla IBIQ
 * Listado, Query, Forma IBIQ
  Habr� una pantalla en donde se pueda consultar la cola, en donde aparecer�n todos las entradas pendientes y procesadas ordenadas por la fecha de creacion del registro.  * Lista IITY
  Las interfaces a ser tratadas mediante esta clase son:
  PRE_CL1 		Prestaciones: Clientes Pers. Naturales
  PRE_CL2 		Prestaciones: Clientes Pers. Jur�dicas
  PRE_AC 		Prestaciones: Creaci�n Ctas. de Bco.
  FON_CL1 		Fondafa: Clientes Pers. Naturales
  FON_CL2 		Fondafa: Clientes Pers. Jur�dicas
  FON_AC 		Fondafa: Creaci�n de Ctas. de Bco.
  //////////////////
  Mensaje no utilizado default:
  /*
 * GEN_ERR_IBS_IBIQ_NOTHG2DISP
 * Nothing to display (record status '^1')
 * No hay nada para mostrar (estatus '^1')
 * /
  $s_response .= dict('GEN_ERR_IBS_IBIQ_NOTHG2DISP', $record['REC_STATUS']);
  ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------- */