<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author	   $Author: fsantana $
 * @copyright	Alenet, Inc.
 * @since		$Date: 2015-09-25 14:47:34 -0400 (Fri, 25 Sep 2015) $
 * @version		$Rev: 7014 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Aras {

    /**
     * Constructor
     * @param array [$procParameters]
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('ARAS', 'INVO', 'INVL', 'PROD', 'PRAT', 'GLAC'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->today = xFormat::getNow('Y-m-d');
        $this->je_source = '';
    }

    /**
     * Initialice form - Executed on init event
     *
     * @param array $record
     */
    function init_form(&$record) {
        if (UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE') {
            $apinrec = ClxCommon::$db->getFields($this->invoname, 'AR_AMORT_METHOD, INVOICEDATE, AR_AMORT_FREQ, INVOICE_AMT, AMORTIZATION_INIT', 'IDCODE = ' . singleQuote($record['ARAS_INVOICECODE']));
            $sql = 'SELECT ARAS.SEQNUM, ARAS.PERIOD_END_DATE, ARAS.FINAL_BAL_AMT FROM ' . $this->arasname . ' ARAS WHERE ARAS.INVOICECODE =' . singleQuote($record['ARAS_INVOICECODE']) . ' AND ARAS.SEQNUM = (SELECT MAX(ARASX.SEQNUM) FROM ' . $this->arasname . ' ARASX WHERE ARASX.INVOICECODE = ARAS.INVOICECODE)';
            $r = ClxCommon::$db->getOneRecord($sql);
            if (empty($r)) {
                $record['ARAS_SEQNUM'] = CLX_NUM_ONE;
                $record['ARAS_PERIOD_START_DATE'] = $apinrec['INVOICEDATE'];
                $record['ARAS_INIT_BAL_AMT'] = round($apinrec['INVOICE_AMT'] - $apinrec['AMORTIZATION_INIT'], CLX_NUM_TWO);
                $record['ARAS_FINAL_BAL_AMT'] = round($apinrec['INVOICE_AMT'] - $apinrec['AMORTIZATION_INIT'], CLX_NUM_TWO);
            } else {
                $record['ARAS_SEQNUM'] = $r['SEQNUM'] + CLX_NUM_ONE;
                $record['ARAS_PERIOD_START_DATE'] = adodb_next_day($r['PERIOD_END_DATE']);
                $record['ARAS_INIT_BAL_AMT'] = $r['FINAL_BAL_AMT'];
                $record['ARAS_FINAL_BAL_AMT'] = $r['FINAL_BAL_AMT'];
            }
            $record['ARAS_PERIOD_END_DATE'] = $this->find_period_end_date($apinrec['AR_AMORT_FREQ'], $record['ARAS_PERIOD_START_DATE'], $record['ARAS_SEQNUM']);
        }
    }

    /**
     * Events post aras save commit
     * @param array $record
     */
    function post_save_form($record) {
        //Mark for deletition next records
        $sql = 'UPDATE ' . $this->arasname . '  SET IS_REMOVE = 1 WHERE INVOICECODE =' . singleQuote($record['INVOICECODE']) . ' AND SEQNUM>' . $record['SEQNUM'];
        ClxCommon::$db->executeQuery($sql);
        $final_bal_amt = $record['FINAL_BAL_AMT'];
        $sql = 'SELECT * FROM ' . $this->arasname . '  WHERE INVOICECODE =' . singleQuote($record['INVOICECODE']) . ' AND SEQNUM>' . $record['SEQNUM'] . ' ORDER BY SEQNUM ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            if ($final_bal_amt == CLX_NUM_ZERO)
                break;
            $r['INIT_BAL_AMT'] = $final_bal_amt;
            if ($r['INIT_BAL_AMT'] < $r['AMORTIZATION_AMT']) {
                $r['AMORTIZATION_AMT'] = $r['INIT_BAL_AMT'];
            }
            $r['FINAL_BAL_AMT'] = $r['INIT_BAL_AMT'] - $r['AMORTIZATION_AMT'];
            $final_bal_amt = $r['FINAL_BAL_AMT'];
            $r['IS_REMOVE'] = CLX_INIT_ZERO;
            ClxCommon::$db->updateRecord($this->arasname, $r, 'ID = ' . $r['ID']);
        }
        $sql = 'DELETE FROM ' . $this->arasname . '  WHERE INVOICECODE =' . singleQuote($record['INVOICECODE']) . ' AND IS_REMOVE = 1';
        ClxCommon::$db->executeQuery($sql);
    }

    /**
     * Validate record information
     * Executed on table validate
     *
     * @param array $record
     * @param array $msg
     * @return boolean
     */
    function validate_form(&$record, &$msg) {
        if (UserRequest::$request['ac'] == 'DELETE') {
                return true;
        }    

        //Calculate formula and update balances.
        $apinrec = ClxCommon::$db->getFields($this->invoname, 'AR_AMORT_METHOD, INVOICEDATE', 'IDCODE = ' . singleQuote($record['ARAS_INVOICECODE']));
        if ($apinrec['AR_AMORT_METHOD'] <> CLX_NUM_THREE) {
            $msg['ARAS_INVOICECODE'] = smsg('MSG-282');
        }
        if ($record['ARAS_PERIOD_START_DATE'] > $record['ARAS_PERIOD_END_DATE']) {
            $msg['ARAS_PERIOD_START_DATE'] = smsg('MSG-283');
        }
        if ($record['ARAS_PERIOD_START_DATE'] < $apinrec['INVOICEDATE']) {
            $msg['ARAS_PERIOD_START_DATE'] = smsg('MSG-284');
        }
        if ($record['ARAS_SEQNUM'] === '')
            $record['ARAS_SEQNUM'] = CLX_NUM_ZERO;
        $sql = 'SELECT MAX(PERIOD_END_DATE) PERIOD_END_DATE FROM ' . $this->arasname . '  WHERE INVOICECODE =' . singleQuote($record['ARAS_INVOICECODE']) . ' AND SEQNUM < ' . $record['ARAS_SEQNUM'];
        $last_end_date = ClxCommon::$db->getOne($sql);
        if (!empty($last_end_date) && $record['ARAS_PERIOD_START_DATE'] < $last_end_date) {
            $msg['ARAS_PERIOD_START_DATE'] = smsg('MSG-285', $last_end_date);
        }
        return (count($msg) > CLX_NUM_ZERO) ? false : true;
    }

    /**
     * Build amortization schedule
     * @param $invoice_code
     * @param $efftdate
     */
    function build_amortization($invoice_code) {
        //Calculate formula and update balances.
        $apinrec = ClxCommon::$db->getRecord($this->invoname, 'IDCODE = ' . singleQuote($invoice_code));
        if ($apinrec['AR_AMORT_METHOD'] == CLX_NUM_ONE || $apinrec['AR_AMORT_METHOD'] == CLX_NUM_TWO) {//fixed or Pct formula
            /* mark all current */
            $sql = 'UPDATE ' . $this->arasname . '  SET IS_REMOVE = 1 WHERE INVOICECODE = ' . singleQuote($invoice_code);	
            ClxCommon::$db->executeQuery($sql);
            $DBTB = &getTool('DBTB');			
            $arasfls = $DBTB->tableDefaultRecord('ARAS');
            $arasfls['CREATEDBY'] = $this->username;
            $arasfls['CDATE'] = $this->now;
            $arasfls['MODIFIEDBY'] = $this->username;
            $arasfls['MDATE'] = $this->now;
            $arasfls['INVOICECODE'] = $invoice_code;
			//pre ('aja4'); die;
            unset($arasfls['ID']);
            $invoice_amt = $apinrec['INVOICE_AMT'] - $apinrec['AMORTIZATION_INIT'];
            $sdate = $apinrec['INVOICEDATE'];
            if ($apinrec['AR_AMORT_METHOD'] == CLX_NUM_ONE) {
                $amortization_amt = $apinrec['AMORTIZATION_AMT'];
            } elseif ($apinrec['AR_AMORT_METHOD'] == CLX_NUM_TWO) {
                $amortization_amt = round($invoice_amt * ($apinrec['AMORTIZATION_RT'] / CLX_NUM_HUNDRED), CLX_NUM_TWO);
            } else {
                return;
            }
			
            $i = CLX_NUM_ONE;
            $amortized_amt = CLX_INIT_ZERO;
            while ($invoice_amt > CLX_NUM_ZERO) {
                $r = $arasfls;
                $r['PERIOD_START_DATE'] = $sdate;
                $sdate = $this->find_period_end_date($apinrec['AR_AMORT_FREQ'], $sdate, $i);
                //Evaluate if amortization amount is greater than amortized amount
                if ($amortized_amt > $invoice_amt)
                    $amortized_amt = $invoice_amt;
                $r['IS_REMOVE'] = CLX_INIT_ZERO;
                $r['SEQNUM'] = $i;
                $r['PERIOD_END_DATE'] = adodb_prev_day($sdate);
                $r['INIT_BAL_AMT'] = $invoice_amt;
                if ($invoice_amt < $amortization_amt) {
                    $amortization_amt = $invoice_amt;
                }
                if (round(($invoice_amt - $amortization_amt), CLX_NUM_TWO) <= CLX_NUM_0_PT_10) {
                    $amortization_amt = $invoice_amt;
                }
                $r['AMORTIZATION_AMT'] = $amortization_amt;
                $invoice_amt -= $amortization_amt;
                $r['FINAL_BAL_AMT'] = $invoice_amt;
                ClxCommon::$db->loadDbRecord($this->arasname, $r, false, 'INVOICECODE =' . singleQuote($r['INVOICECODE']) . ' AND SEQNUM = ' . singleQuote($r['SEQNUM']));
                $i++;
                //just break in case of some extrenge long loop
                if ($i >= CLX_NUM_HUNDRED_THOUSAND)
                    break;
            }//end while
            $sql = 'DELETE FROM ' . $this->arasname . '  WHERE IS_REMOVE = 1	AND INVOICECODE = ' . singleQuote($invoice_code);
            ClxCommon::$db->executeQuery($sql);
        }
    }

    /**
     * Sets period of dates
     * @param string $interval
     * @param string $sdate
     * @param string $seqnum
     */
    private function find_period_end_date($interval, $sdate, $seqnum) {
        switch ($interval) {
            case '360':
                $sdate = adodb_next_day($sdate, 'Y-m-d');
                break;
            case '12.0':
                if ($seqnum === CLX_NUM_ONE) {//first month partial
                    $sdate = adodb_next_day(adodb_month_lastday($sdate, 'Y-m-d'));
                } else {
                    $sdate = adodb_period_date($sdate, 'M', CLX_NUM_ONE, 'Y-m-d');
                }
                break;
            case '12'://Monthly
                $sdate = adodb_period_date($sdate, 'M', CLX_NUM_ONE, 'Y-m-d');
                break;
            case '6'://Bimonthly(6)
                $sdate = adodb_period_date($sdate, 'M', CLX_NUM_TWO, 'Y-m-d');
                break;
            case '4'://Quarterly(4)
                $sdate = adodb_period_date($sdate, 'M', CLX_NUM_THREE, 'Y-m-d');
                break;
            case '3'://Four-monthly 1/3(3)
                $sdate = adodb_period_date($sdate, 'M', CLX_NUM_FOUR, 'Y-m-d');
                break;
            case '2'://Semiannually(2)
                $sdate = adodb_period_date($sdate, 'M', CLX_NUM_SIX, 'Y-m-d');
                break;
            case '1'://Annually(1)
                $sdate = adodb_next_date($sdate, 'Y');
                break;
            default:
                $sdate = 'null';
                break;
        }
        return $sdate;
    }

}
