<?php
/**
 * frpe.php
 * PBL-Interfaz Revisión de Clientes
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-16 09:23:00 -0500 (Thu, 16 Jan 2019) $
 * @version    	$Rev: 10045 $
 */

class frpe {

    private $frpe_param = array();
    private $frpe_objs = array();
    private $frpe_err = array();
	
	const FRPE_SCHED_CODE = 'FRPE';
	
    
    /**
     * Constructor
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = null) {
		if(is_null($procParameters)) {
			$this->frpe_param = $this->get_sched_par_values(self::FRPE_SCHED_CODE);
		} else {
			$this->frpe_param = $procParameters;
		}
		$this->verify_def_values();
		//$this->include_encription_class();
	}

	function include_encription_class() {
		include_once 'classes/Encription.php';
		if(!defined('kPathToRoot')) {
			// The root directory to look for ambients.
			define('kPathToRoot', filter_input(INPUT_SERVER, 'DOCUMENT_ROOT', FILTER_SANITIZE_STRING) . '/');
		}
		// Ambient file
		if(!defined('ambFile')) {
			define('ambFile','amb.php');
		}
		// The data directory.
		if(!defined('kData')) {
			define('kData','data/');
		}
		$this->frpe_objs['Encription'] = new Encription(array('show_msg'=>'1'));
	}

	function get_sched_par_values($sched_code) {
		$response = array();
		$PRSP = getTool('PRSP');
        $sql = 'SELECT FIELDNAME, PARAMETER_VALUE FROM ' . $PRSP->tablename . ' WHERE SCHEDULECODE = ' . singleQuote($sched_code);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $response[$r['FIELDNAME']] = $r['PARAMETER_VALUE'];
        }
		return $response;
	}
	
	function action($ac, $idlist) {
        $contents = '';
        $msg = array();
		switch ($ac) {
			case 'DOLO':
				// 'toolapp.php?to=FRPE&ac=DOLO&i='.$s_idcode
				if(is_null($idlist) || count($idlist) === CLX_NUM_ZERO) {
					if(isset(UserRequest::$get['i'])) {
						$idlist = UserRequest::$get['i'];
					}
				}
				$contents = $this->GetInterfaceFile($idlist);
				break;
			default:
				$contents = 'Invalid action: "'.$ac.'" [FRPE-100]';
		}
		return $contents;
	}
	
	function GetInterfaceFile($file_name_par) {
		$file_name = urldecode($file_name_par);
		//$filepath_orig = $this->frpe_objs['Encription']->start('DoDecrypt', $file_name);
		$filepath_orig = $this->frpe_param['FILE_DIR'] . DIRECTORY_SEPARATOR . $file_name;
		$filepath_orig = str_replace(DIRECTORY_SEPARATOR.DIRECTORY_SEPARATOR, DIRECTORY_SEPARATOR, $filepath_orig);
		$filepath_orig = realpath($filepath_orig); // realpath() elimina la combinacion de / y \ en el path
		//pre('file_name_par = '.$file_name_par);pre('file_name = '.$file_name);pre('filepath_orig =' . $filepath_orig); exit();
		if(!file_exists($filepath_orig)) {
			if($this->frpe_param['IS_DEBUG'] === '1') {
				echo 'FRPE interface file "'.$this->frpe_param['FILE_DIR'] . DIRECTORY_SEPARATOR . $file_name.'" not found [FRPE-015]';
			} else {
				echo 'FRPE interface file "'.$file_name.'" not found [FRPE-010]';
			}
			exit();
		}
		AppDownloadFile($filepath_orig, $file_name, 'txt');
	}
	
    /**
     * Proceso principal de FRPE
     * @param void
     * @return boolean
     */
    function main_process() {
		$this->verify_list_file_existance();
		$this->verify_export_dir_existance();
		if($this->flush_errors() === false) {
			return false;
		}
		UserRequest::$request['EFFTDATE'] = $this->frpe_param['EFFTDATE'];
		$DBEX = &getTool('DBEX');
        if (!defined('DBEX_DEF_LIST_IMP_USER')) {
			define('DBEX_DEF_LIST_IMP_USER', $this->frpe_param['USERNAME']);
		}
		$DBEX->utf8_output = true;	// 2019-02-18 CR | SA | Caso 1003786 - PBL-Interf Rev Client
		$DBEX->export_from_list($this->frpe_param['FILE_NAME']);
		$this->frpe_param['result'] = $DBEX->get_dbex_pars('export_from_list');
		$this->generate_file_links();
		return true;
	}
	
    /**
     * Verifica valores por defecto
     * @param void
     * @return boolean
     */
	function verify_def_values() {
		// Fecha para el nombre del archivo
		$this->parse_par_def_var('EFFTDATE', xFormat::getNow('Y-m-d H:i:s'));
		// Directorio de exportacion
		$this->parse_par_def_var('FILE_DIR', kPathToWebSite . kPathToExport);
		// Archivo con la lista de export a realizar
		$this->parse_par_def_var('FILE_NAME', 'export/FRPE_export_list.txt');
		// Debug
		$this->parse_par_def_var('IS_DEBUG', '0');
		//$this->parse_par_def_var('IS_DEBUG', '1');
	}

	function parse_par_def_var($par_name, $def_val) {
		if(!isset($this->frpe_param[$par_name])) {
			$this->frpe_param[$par_name] = CLX_CHAR_EMPTY_STRING;
		}
		if(trim($this->frpe_param[$par_name]) === CLX_CHAR_EMPTY_STRING) {
			$this->frpe_param[$par_name] = $def_val;
		} else {
			$this->frpe_param[$par_name] = trim($this->frpe_param[$par_name]);
		}
	}
	
	function verify_list_file_existance() {
		if(file_exists($this->frpe_param['FILE_NAME'])) {
			return;
		}
        $f = new clx_file($this->frpe_param['FILE_NAME'], 'w');
        $f->open();
        if (!empty($f->error)) {
            $this->frpe_err[] = dict('GEN_ERR_CANT_CREATE_FILE', $this->frpe_param['FILE_NAME']) . ' [FRPE-030]';
            return;
        }
		$f->write('EXPORT_CODE, FILE_NAME, FILE_DIR, OTHER_PARAMETERS' . CLX_CHAR_NL);
		$f->write('FRPE_01, , ' . $this->frpe_param['FILE_DIR'] . CLX_CHAR_NL);
		$f->write('FRPE_02, , ' . $this->frpe_param['FILE_DIR'] . CLX_CHAR_NL);
		$f->write('FRPE_03, , ' . $this->frpe_param['FILE_DIR'] . CLX_CHAR_NL);
        $f->close();
	}

	function verify_export_dir_existance() {
		if(is_dir($this->frpe_param['FILE_DIR'])) {
			return;
		}
		if(!mkdir($this->frpe_param['FILE_DIR'])) {
            $this->frpe_err[] = dict('CODE_ERR_FILE_INVALIDDIR', $this->frpe_param['FILE_DIR']) . ' [FRPE-020]';
            return;
		}
	}

	function flush_errors() {
		if(count($this->frpe_err) == CLX_NUM_ZERO) {
			return true;
		}
		$bullet_char = '- ';
		echo $bullet_char . implode(CLX_CHAR_NL . $bullet_char, $this->frpe_err);
		return false;
	}

	function generate_file_links() {
		$cli_envir = (isset($argv)); // true = invoked from command line
		if($cli_envir === true) {
			return;
		}
		$lbreak = (isset($argv) ? CLX_CHAR_NL : '<BR/>'); // Line Break on CLI or Web environments
		foreach($this->frpe_param['result']['response'] as $filespec) {
			//$file_name_raw = $this->frpe_objs['Encription']->start('DoEncrypt', $filespec);
			$file_name_raw = basename($filespec);
			$file_name = urlencode($file_name_raw);
			echo '<A HREF="toolapp.php?to=FRPE&ac=DOLO&i=', $file_name, '">Download file ', basename($filespec), '</A>', $lbreak;
		}
	}
}
