<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 10:58:27 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9614 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Gltt {

    /**
     * Contructor
     *
     * @param array $procParameters
     * @return void
     */
    function __construct($procParameters = array()) {
        multiTableName($this, array('GLTT', 'JTMP', 'JEHD', 'JELN', 'GLAC', 'FICO', 'ACCO', 'GLCP', 'FTMP', 'RUCO'));
        $this->batch_id = (!empty($procParameters ['BATCH_ID'])) ? $procParameters ['BATCH_ID'] : batch_id();
        $this->username = $_SESSION ['user_record'] ['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function validate_form(&$record, &$msg) {
		if (UserRequest::$request['ac'] == 'DELETE'){
		   return true;
		}
		switch ($record ['GLTT_INTERFACE_CODE']) {
            case 'FITYP_ADM' :
                if (empty($record ['GLTT_FI_TYPE'])) {
                    $msg ['GLTT_FI_TYPE'] = dict('GEN_LBL_FIELD_REQUIRED');
                }
                if (empty($record['GLTT_TGT_COMP_CODE'])) {
                    $msg ['GLTT_TGT_COMP_CODE'] = dict('GEN_LBL_FIELD_REQUIRED');
                }
                if (empty($record['GLTT_TGT_CHART_FIELD01'])) {
                    $msg ['GLTT_TGT_CHART_FIELD01'] = dict('GEN_LBL_FIELD_REQUIRED');
                }
                if (empty($record['GLTT_TGT_CHART_FIELD02'])) {
                    $msg ['GLTT_TGT_CHART_FIELD02'] = dict('GEN_LBL_FIELD_REQUIRED');
                }
                break;
            default :
                if (empty($record ['GLTT_GLACCOUNTCODE'])) {
                    $msg ['GLTT_GLACCOUNTCODE'] = dict('GEN_LBL_FIELD_REQUIRED');
                }
                break;
        } // end switch
    }

// end function

    function test() {
        $comp_code = 'FFG';
        $efftdate = '';
        $this->run_interface_fityp_adm($comp_code, $efftdate);
    }

    /**
     * Build journal for interface
     */
    function run_interface_fityp_adm($comp_code, $operation_date = '') {
        if ($comp_code === '')
            return;
        $je_source = 'GLTT_FITY';
        $DBLT = &getTool('DBLT');
        $idname = $DBLT->getListItem('SOTY', $je_source, CLX_NUM_TWO);
        if (!class_exists('Comp')) {
            includeCodeLib('COMP');
        }
        $comp = new Comp ();
        if ($operation_date === '') {
            $op = $comp->comp_operation_parameters($comp_code);
            $operation_date = $op ['OPERATION_DATE'];
        }
        echoLog('<br><br> Accounting Interface ' . $je_source . '. BATCH ID: ' . $this->batch_id);
        echoLog('<br> Comp Code: ' . $comp_code);
        echoLog('<br> Efective date: ' . $operation_date);
        $DBTB = &getTool('DBTB');
        $DBTB->buildTemporaryTable('FTMP');
        ClxCommon::$db->deleteRecords($this->ftmpname, 'BATCH_ID = ' . singleQuote($this->batch_id));
        //FTMP 	BALANCES_TMP
        $sql = 'INSERT INTO ' . $this->ftmpname . ' (COMP_CODE, GL01_CODE, GL02_CODE, EFFTDATE, JE_SOURCE, BATCH_ID, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, LGGROUP_CODE, FISCAL_PERIOD, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_REF_NO, CALC_AMT_01, CALC_AMT_02, COMMISSION_CODE) SELECT GLTT.TGT_COMP_CODE COMP_CODE, GLTT.DEBIT_GL_CODE GL01_CODE, GLTT.CREDIT_GL_CODE GL02_CODE, JEHD.EFFTDATE EFFTDATE, ' . singleQuote($je_source) . ' JE_SOURCE, ' . singleQuote($this->batch_id) . ' BATCH_ID, JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, JELN.CUR_OPERATOR, JEHD.LGGROUP_CODE, JEHD.FISCAL_PERIOD, GLTT.TGT_CHART_FIELD01 CHART_FIELD01, GLTT.TGT_CHART_FIELD02 CHART_FIELD02, null CHART_FIELD03, null CHART_FIELD04, JEHD.COMP_CODE ITEM_REF_NO, SUM(JELN.DEBIT_TRANS_AMT-JELN.CREDIT_TRANS_AMT) CALC_AMT_01, SUM(JELN.DEBIT_AMT-JELN.CREDIT_AMT) CALC_AMT_02, GLTT.ITEM_CODE COMMISSION_CODE FROM ' . $this->ficoname . ' FICO, ' . $this->glacname . ' GLAC, ' . $this->glcpname . ' GLCP, ' . $this->jehdname . ' JEHD, ' . $this->jelnname . ' JELN, ' . $this->glttname . ' GLTT WHERE GLTT.INTERFACE_CODE = ' . singleQuote('FITYP_ADM') . ' AND GLTT.DIRECTION = ' . CLX_NUM_ZERO . ' AND GLTT.FI_TYPE = FICO.FI_TYPE AND GLTT.CONTRACT_TYPE = FICO.CONTRACT_TYPE AND GLTT.GL_TYPE = GLAC.GL_TYPE AND JEHD.IDCODE = JELN.JEHDR_CODE AND FICO.IDCODE = JELN.CHART_FIELD01 AND GLCP.COMP_CODE = FICO.COMP_CODE AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLAC.IDCODE = JELN.GLACCOUNTCODE AND JEHD.JE_STATUS = ' . singleQuote('AP') . ' AND JEHD.COMP_CODE = ' . singleQuote($comp_code) . ' AND JEHD.REAL_DATE = ' . singleQuote($operation_date) . ' AND NOT EXISTS (SELECT RUCO.ID FROM ' . $this->ruconame . ' RUCO WHERE RUCO.JE_SOURCE = ' . singleQuote('GLTT_FITY') . ' AND RUCO.COMP_CODE = ' . singleQuote($comp_code) . ' AND RUCO.STATUS = ' . singleQuote('I') . ') GROUP BY JEHD.COMP_CODE, GLTT.TGT_COMP_CODE, GLTT.TGT_CHART_FIELD01, GLTT.TGT_CHART_FIELD02, GLTT.DEBIT_GL_CODE, GLTT.CREDIT_GL_CODE, FICO.COMP_CODE, JEHD.EFFTDATE, FICO.CONTRACT_TYPE, FICO.FI_TYPE, JELN.CURRENCY_CODE, JELN.EXCHANGE_RT, GLTT.ITEM_CODE ';
        ClxCommon::$db->executeQuery($sql);
        if (ClxCommon::$db->oneExistsRecords($this->ftmpname, 'BATCH_ID = ' . singleQuote($this->batch_id)) == CLX_NUM_ZERO) {
            //nothing to do
            echoLog('<br> No record found ');
            return;
        }
        if (!class_exists('Jehd'))
            includeCodeLib('JEHD');
        $jehd = new Jehd ();
        $sql = 'SELECT DISTINCT FTMP.COMP_CODE, FTMP.EFFTDATE, FTMP.LGGROUP_CODE, FTMP.FISCAL_PERIOD, ' . singleQuote($je_source) . ' JE_SOURCE, 0 CLOSING_ENTRY, 0 REVERSED, FTMP.BATCH_ID, ' . singleQuote($this->username) . ' CREATEDBY, ' . singleQuote($this->username) . ' MODIFIEDBY, ' . singleQuote($this->now) . ' CDATE, ' . singleQuote($this->now) . ' MDATE FROM ' . $this->ftmpname . ' FTMP WHERE FTMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND FTMP.CALC_AMT_01 <>0';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $op = $comp->comp_operation_parameters($comp_code); //Cambiando por compania de origan para que caiga en misma fecha
            //$r['FISCAL_PERIOD'] = $op['FISCAL_PERIOD'];
            $r['FISCAL_PERIOD'] = $comp->comp_fiscal_period ( $r['COMP_CODE'] , $r ['EFFTDATE'] );
            $r['REAL_DATE'] = $op['OPERATION_DATE'];
            //$r['EFFTDATE'] = $r['EFFTDATE'];
            $r['LGGROUP_CODE'] = $op['LGGROUP_CODE'];
            $r['IDNAME'] = $idname;
            // Generar encabezado para portafolio
            $jehd->jehd_build($r);
            $jehdr_code = $r ['IDCODE'];
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT, RECONCILIED) SELECT ' . singleQuote($jehdr_code) . ' JEHDR_CODE, 1 SEQNUM, FTMP.GL01_CODE GLACCOUNTCODE, FTMP.CHART_FIELD01, FTMP.CHART_FIELD02, FTMP.CHART_FIELD03, FTMP.CHART_FIELD04, FTMP.COMMISSION_CODE ITEM_CODE, FTMP.ITEM_REF_NO, FTMP.CURRENCY_CODE, FTMP.EXCHANGE_RT, FTMP.CUR_OPERATOR, FTMP.CALC_AMT_01 DEBIT_TRANS_AMT, 0 CREDIT_TRANS_AMT, FTMP.CALC_AMT_02 DEBIT_AMT, 0 CREDIT_AMT, 0 RECONCILIED FROM ' . $this->ftmpname . ' FTMP WHERE FTMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND FTMP.EFFTDATE = ' . singleQuote($r['EFFTDATE']) . ' AND FTMP.CALC_AMT_01 <>0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'INSERT INTO ' . $this->jelnname . ' (JEHDR_CODE, SEQNUM, GLACCOUNTCODE, CHART_FIELD01, CHART_FIELD02, CHART_FIELD03, CHART_FIELD04, ITEM_CODE, ITEM_REF_NO, CURRENCY_CODE, EXCHANGE_RT, CUR_OPERATOR, DEBIT_TRANS_AMT, CREDIT_TRANS_AMT, DEBIT_AMT, CREDIT_AMT, RECONCILIED) SELECT ' . singleQuote($jehdr_code) . ' JEHDR_CODE, 2 SEQNUM, FTMP.GL02_CODE GLACCOUNTCODE, FTMP.CHART_FIELD01, FTMP.CHART_FIELD02, FTMP.CHART_FIELD03, FTMP.CHART_FIELD04, FTMP.COMMISSION_CODE ITEM_CODE, FTMP.ITEM_REF_NO, FTMP.CURRENCY_CODE, FTMP.EXCHANGE_RT, FTMP.CUR_OPERATOR, 0 DEBIT_TRANS_AMT, FTMP.CALC_AMT_01 CREDIT_TRANS_AMT, 0 DEBIT_AMT, FTMP.CALC_AMT_02 CREDIT_AMT, 0 RECONCILIED FROM ' . $this->ftmpname . ' FTMP WHERE FTMP.BATCH_ID = ' . singleQuote($this->batch_id) . ' AND FTMP.EFFTDATE = ' . singleQuote($r['EFFTDATE']) . ' AND FTMP.CALC_AMT_01 <>0';
            ClxCommon::$db->executeQuery($sql);
            //UPDATING LINES REVERSE IF EXISTS -1
            $sql = 'UPDATE ' . $this->jelnname . ' SET CREDIT_AMT = (- 1 *DEBIT_AMT), DEBIT_AMT = 0, CREDIT_TRANS_AMT = (- 1 *DEBIT_TRANS_AMT), DEBIT_TRANS_AMT = 0 WHERE JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND DEBIT_AMT<0';
            ClxCommon::$db->executeQuery($sql);
            $sql = 'UPDATE ' . $this->jelnname . '  SET DEBIT_AMT = (- 1 *CREDIT_AMT), CREDIT_AMT = 0, DEBIT_TRANS_AMT = (- 1 *CREDIT_TRANS_AMT), CREDIT_TRANS_AMT = 0 WHERE JEHDR_CODE = ' . singleQuote($jehdr_code) . ' AND CREDIT_AMT<0';
            ClxCommon::$db->executeQuery($sql);
            //Verificando conversion;
            $jehd->balance_exchange_rounding($jehdr_code);
            // Cambiar estatus de asientos generados por el proceso
            $jehd->jehd_approve_by_code($jehdr_code);
            echoLog('<Br>Approving jehd: ' . $jehdr_code . '-' . $je_source);
        }
    }

}
