<?php

/**
 * General Code File
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-03-22 13:55:41 -0400 (Thu, 22 Mar 2018) $
 * @version        $Rev: 9771 $
 *
 * CHANGE LOG
 *  Included svn prop
 */
class Jeln {

    private $flg_migration_process = false;

    function __construct($procParameters = array()) {
        multiTableName($this, array('FIPA', 'FICO', 'JEHD', 'GLAC', 'GLAG', 'COMP', 'INFU', 'FIAS', 'FIAC', 'JELN', 'GLCP', 'PROJ', 'BLAK', 'ACCO'));
        $this->batch_id = (!empty($procParameters['BATCH_ID'])) ? $procParameters['BATCH_ID'] : batch_id();
        $this->username = $_SESSION['user_record']['username'];
        $this->now = xFormat::getNow('Y-m-d H:i:s');
        $this->flg_migration_process = ( companyParameter('FLG_MIGRATION_PROCESS') === (''.CLX_NUM_ONE) );
    }

    /**
     * Initialice jeln form
     * Executed on init event
     */
    function jeln_init_form(&$record) {
        if (!empty($record['JELN_JEHDR_CODE'])) {
            $record['JELN_COMP_CODE'] = ClxCommon::$db->getField($this->jehdname, 'COMP_CODE', 'IDCODE = ' . singleQuote($record['JELN_JEHDR_CODE']));
        }
    }

    /**
     * Validate record information
     * Executed on table validate
     */
    function jeln_validate_form(&$record, &$msg) {

        if (!empty(UserRequest::$request['fm']) && (UserRequest::$request['fm'] === 'JELN_REC' || UserRequest::$request['fm'] === 'GLRL_JELN')) {//Means reconciliation
            return true;
        }

        $record_list = explode(",", $record['idlist']);
        foreach($record_list as $record_item){
            $record['idlist'] =  $record_item;

            if(UserRequest::$request['ac'] === 'DELETE' && empty($record['JELN_JEHDR_CODE'])){
                 $record_table = ClxCommon::$db->getRecord($this->jelnname, 'ID=' . $record['idlist']);
                foreach($record_table as $rtk => $rtv){
                    $record['JELN_'.$rtk] = $rtv;
                }
            }
            $jehdrec = ClxCommon::$db->getFields($this->jehdname, 'EFFTDATE, COMP_CODE, JE_STATUS,IDCODE, JE_SOURCE, REV_JEHDR_CODE', 'IDCODE = ' . singleQuote($record['JELN_JEHDR_CODE']));
            //Validar que operaciones de reverso no puedan ser modificadas
            if ((UserRequest::$request['ac'] === 'CREATE' || UserRequest::$request['ac'] === 'DUPLICATE' || UserRequest::$request['ac'] === 'MODIFY' || UserRequest::$request['ac'] === 'DELETE')) {
                if (!empty($jehdrec ['REV_JEHDR_CODE'])) {
                    $msg ['JELN_JEHDR_CODE'] = smsg('MSG-89', $jehdrec ['IDCODE']);
                    addMessage($msg['JELN_JEHDR_CODE']);
                    return false;
                }
            }

            $ficorec = ClxCommon::$db->getFields($this->ficoname, 'CONTRACT_STATUS, EXPIRATION_DATE, CANCEL_DATE,  START_DATE, COMP_CODE, IDCODE', 'IDCODE = ' . singleQuote($record['JELN_CHART_FIELD01']));
            //Validar status del encabezado. Debe estar en pendiente para modificar.
            if ($jehdrec['JE_STATUS'] <> 'PEND' && UserRequest::$request['fm'] <> 'GLRL_JELN') {
                $msg['JELN_JEHDR_CODE'] = smsg('MSG-3', $jehdrec['IDCODE']);
                if (UserRequest::$request['ac'] === 'DELETE') {
                    addMessage($msg['JELN_JEHDR_CODE']);
                    return false;
                }
            }
            //Validar que debitos y creditos no puede ser igual a cero.
            if (UserRequest::$request['ac'] <> 'DELETE' && ($record['JELN_DEBIT_AMT'] + $record['JELN_CREDIT_AMT']) == CLX_NUM_ZERO) {
                $msg['JELN_DEBIT_AMT'] = smsg('MSG-4', $jehdrec['IDCODE']);
            }
            //Validar que exista participante en contrato o subcontrato (Siempre que no sea durante la migracion inicial de datos)
            if($this->flg_migration_process === false &&UserRequest::$request['ac'] <> 'DELETE') {
                $sql = 'SELECT COUNT(FIPA.ID) ' . 'FROM ' . $this->fipaname . ' FIPA, ' . $this->ficoname . ' FICO ' . ' WHERE (FIPA.CONTRACT_CODE = FICO.IDCODE OR FIPA.CONTRACT_CODE = FICO.PARENTCONTRACT_CODE) AND FIPA.STATUS = ' . singleQuote('A') . ' ' . 'AND FICO.IDCODE =' . singleQuote($record['JELN_CHART_FIELD01']) . 'AND FIPA.ACCOUNTCODE =' . singleQuote($record['JELN_CHART_FIELD02']);
                if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                    $msg['JELN_CHART_FIELD02'] = smsg('MSG-16', $jehdrec['IDCODE'], $record['JELN_CHART_FIELD01'], $jehdrec['IDCODE']);
                }
            }
            //Validar que compania salire en el contrato.
            if ($ficorec['COMP_CODE'] <> $jehdrec['COMP_CODE']) {
                $msg['JELN_CHART_FIELD01'] = smsg('MSG-6', $jehdrec['IDCODE']);
            }
            //Validar que contrato esta activo y fechas.
            if ($ficorec['CONTRACT_STATUS'] === 'CANC' ||
                $ficorec['CONTRACT_STATUS'] === 'EXPI' || $ficorec['CONTRACT_STATUS'] === 'INAC') {
                $msg['JELN_CHART_FIELD01'] = smsg('MSG-7', $jehdrec['IDCODE']);
            }
            //Contract Start Date
            if (strtotime($ficorec['START_DATE']) > strtotime($jehdrec['EFFTDATE'])) {
                $msg['JELN_CHART_FIELD01'] = smsg('MSG-8', $jehdrec['IDCODE']);
            }
            //Contract Expiration date
            /* comentada ya en aunque la fecha de expiracion llegue puede
         * que haga falta continuar operando.
          if (!empty ($ficorec['EXPIRATION_DATE']) &&
          strtotime($ficorec['EXPIRATION_DATE'])  < strtotime($jehdrec['EFFTDATE']))
          {
          $msg['JELN_CHART_FIELD01'] = smsg('MSG-9',$jehdrec['IDCODE']);
          } */
            //Contract Cancel date
            if (!empty($ficorec['CANCEL_DATE']) &&
                strtotime($ficorec['CANCEL_DATE']) < strtotime($jehdrec['EFFTDATE'])) {
                $msg['JELN_CHART_FIELD01'] = smsg('MSG-9', $jehdrec['IDCODE']);
            }
            //Validar que sea una cuenta contable con movimientos.
            $sql = 'SELECT COUNT(GLAC.ID)  FROM ' . $this->glacname . ' GLAC, ' . $this->glcpname . ' GLCP  WHERE GLAC.ALLOW_ACTIVITY = 0 AND GLAC.EFFTSTAT = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' AND GLCP.GLGROUP_CODE = GLAC.GLGROUP_CODE AND GLCP.COMP_CODE = ' . singleQuote($jehdrec['COMP_CODE']) . '   AND GLAC.IDCODE = ' . singleQuote($record['JELN_GLACCOUNTCODE']);
            if (ClxCommon::$db->getOne($sql)) {
                $msg['JELN_GLACCOUNTCODE'] = smsg('MSG-10', $jehdrec['IDCODE']);
            }
            //Validar que sea una cuenta contable de grupo contable correcto.
            $sql = 'SELECT COUNT(GLAC.ID)  FROM  ' . $this->glacname . ' GLAC,  ' . $this->glcpname . ' GLCP  WHERE GLAC.GLGROUP_CODE = GLCP.GLGROUP_CODE  AND GLCP.COMP_CODE = ' . singleQuote($jehdrec['COMP_CODE']) . '  AND GLAC.IDCODE = ' . singleQuote($record['JELN_GLACCOUNTCODE']);
            if (UserRequest::$request['ac'] <> 'DELETE' && ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                $msg['JELN_GLACCOUNTCODE'] = smsg('MSG-11', $jehdrec['IDCODE']);
            }
            //Validar que el saldo no sobregira la cuenta. Recordar que puede ser
            //de provisiones(Ver si en aprobacion)
            //Validar que comision existe en contrato.
            if (array_key_exists('JELN_CHART_FIELD04_prim', $record) && $record['JELN_CHART_FIELD04_prim'] <> '') {
                $sql = 'SELECT COUNT(PROJ.ID)  FROM ' . $this->projname . ' PROJ,  ' . $this->ficoname . ' FICO  WHERE PROJ.CONTRACT_CODE = FICO.IDCODE  AND FICO.IDCODE =' . singleQuote($record['JELN_CHART_FIELD01']) . '  AND PROJ.IDCODE =' . singleQuote($record['JELN_CHART_FIELD04']) . ' ';
                if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                    $msg['JELN_CHART_FIELD04'] = smsg('MSG-14', $jehdrec['IDCODE']);
                }
            }
            if (array_key_exists('JELN_CHART_FIELD03_prim', $record) && $record['JELN_CHART_FIELD03_prim'] <> '') {
                $sql = 'SELECT COUNT(FIAS.ID)  FROM ' . $this->fiasname . ' FIAS,  ' . $this->ficoname . ' FICO ' . ' WHERE (FIAS.CONTRACT_CODE = FICO.IDCODE  OR FIAS.CONTRACT_CODE = FICO.PARENTCONTRACT_CODE) ' . 'AND FICO.IDCODE =' . singleQuote($record['JELN_CHART_FIELD01']) . ' ' . 'AND FIAS.IDCODE =' . singleQuote($record['JELN_CHART_FIELD03']) . ' ';
                if (ClxCommon::$db->getOne($sql) == CLX_NUM_ZERO) {
                    $msg['JELN_CHART_FIELD03'] = smsg('MSG-15', $jehdrec['IDCODE']);
                }
            }
            /*if (!empty($this->blakname) && !empty(UserRequest::$post[CLX_PROP_TO]) && UserRequest::$post[CLX_PROP_TO] === 'EDIT') {
                $sql = 'SELECT DISTINCT ACCO.IDCODE, ACCO.IDNAME FROM ' . $this->acconame . ' ACCO WHERE ACCO.IDCODE =' . singleQuote($record['JELN_CHART_FIELD02']) . ' AND ACCO.BLACKLIST = 1';
                $rs = ClxCommon::$db->executeQuery($sql);
                while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
                    $msg['JELN_CHART_FIELD02'] = smsg('MSG-268', $jehdrec['IDCODE'], $r['IDCODE'] . ' ' . $r['IDNAME']);
                }
            }*/
            if (UserRequest::$request['ac'] === 'DELETE' && !empty($msg)) {
                addMessage($msg);
                return false;
            }
        }



    }

    /**
     * Update Parent (JEHD) Information one record is saved
     */
    function jeln_save_form(&$record) {

        if (!empty(UserRequest::$post['fm']) && UserRequest::$post['fm'] === 'GLRL_JELN') {
            //Asignar la fecha de la conciliacion.
            $callingpage = peekHistory();
            if (intVal(UserRequest::$post['JELN_RECONCILIED']) === CLX_NUM_ONE) {
                if (!empty($callingpage['parentrecord']['GLRC_TO_DATE'])) {
                    $record['RECONCILIED_DT'] = $callingpage['parentrecord']['GLRC_TO_DATE'];
                }
            } else {
                $record['RECONCILIED_DT'] = 'null';
            }
            if (!empty($callingpage['parentrecord']['GLRC_IDCODE'])) {
                if (!class_exists('Glrc'))
                    includeCodeLib('GLRC');
                $glrc = new Glrc();

                $glrc->setGlrcAmt($callingpage['parentrecord']['GLRC_IDCODE']);
                unset($glrc);
            }
        }
    }

    /**
     * Clear all lines for header
     */
    function jeln_delete_lines($jehdr_code) {
        if (!class_exists('Jehd'))
            includeCodeLib('JEHD');
        $jehd = new Jehd();
        if ($jehd->jehd_is_editable($jehdr_code)) {
            ClxCommon::$db->deleteRecords($this->jelnname, 'JEHDR_CODE = ' . singleQuote($jehdr_code));
        }
    }

    /**
     * Initialices jehb record
     */
    function jeln_init_record() {
        $fields = array();
        $sql = 'SELECT FIELDNAME  FROM CLX_DB_TABLEFIELDS  WHERE TABLECODE=' . singleQuote('JELN') . ' AND FIELDNAME <> ' . singleQuote('ID');
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
            $value = 'null';
            switch ($result['FIELDNAME']) {
                case 'DEBIT_AMT':
                    $value = CLX_INIT_ZERO;
                    break;
                case 'CREDIT_AMT':
                    $value = CLX_INIT_ZERO;
                    break;
                case 'DEBIT_TRANS_AMT':
                    $value = CLX_INIT_ZERO;
                    break;
                case 'CREDIT_TRANS_AMT':
                    $value = CLX_INIT_ZERO;
                    break;
                case 'RECONCILIED':
                    $value = CLX_INIT_ZERO;
                    break;
                default:
                    $value = 'null';
                    break;
            }
            $fields[$result['FIELDNAME']] = $value;
        }
        return $fields;
    }

    /**
     * Builds jeln
     */
    function jeln_build(&$record, $revert_exchange = false) {

        if (!function_exists('setCreationData')) {
            includeCodeLib('GEN_REC_UTIL');
        }
        $jehd_build = false;
        if (!empty(UserRequest::$post['fm']) && UserRequest::$post['fm'] === 'JEHD' && UserRequest::$request['ac'] === 'CREATE') {
            $jehd_build = true;
        }
        //Permitir que se cree lineas en 0 si el asiento.
        if (empty($jehd_build)) {
            if ($record['DEBIT_AMT'] == CLX_NUM_ZERO && $record['CREDIT_AMT'] == CLX_NUM_ZERO)
                return;
        }
        //Invertir lineas en caso de ser negativo
        if ($record['DEBIT_AMT'] < CLX_NUM_ZERO) {
            $record['CREDIT_AMT'] = abs($record['DEBIT_AMT']);
            $record['DEBIT_AMT'] = CLX_INIT_ZERO;
        }
        if ($record['CREDIT_AMT'] < CLX_NUM_ZERO) {
            $record['DEBIT_AMT'] = abs($record['CREDIT_AMT']);
            $record['CREDIT_AMT'] = CLX_INIT_ZERO;
        }
        if ($record['CHART_FIELD01'] === '' || $record['CHART_FIELD01'] === 'null') {
            $record['CHART_FIELD01'] = 'pend';
        }
        if ($record['CHART_FIELD02'] === '' || $record['CHART_FIELD02'] === 'null') {
            $record['CHART_FIELD02'] = 'pend';
        }
        if ($record['CHART_FIELD03'] === '') {
            $record['CHART_FIELD03'] = 'null';
        }
        if ($record['CHART_FIELD04'] === '' || $record['CHART_FIELD04'] === null) {
            $record['CHART_FIELD04'] = 'null';
        }
        if ($record['SEQNUM'] === '') {
            $record['SEQNUM'] = CLX_INIT_ZERO;
        }
        if ($record['DEBIT_TRANS_AMT'] === 'null') {
            $record['DEBIT_TRANS_AMT'] = CLX_INIT_ZERO;
        }
        if ($record['CREDIT_TRANS_AMT'] === 'null') {
            $record['CREDIT_TRANS_AMT'] = CLX_INIT_ZERO;
        }
        $record['IS_REMOVE'] = CLX_INIT_ZERO;
        //Esto se implementa luego de encontrar la necesidad de trans amount
        //para no cambiar los demas programas se coloca el monto de debit
        //en campo trans y se realiza la conversion asumiendo que este es el monto
        //al que se aplica el cambio.
        if (empty($revert_exchange)) {
            if ($record['DEBIT_AMT'] > CLX_NUM_ZERO) {
                $record['DEBIT_TRANS_AMT'] = $record['DEBIT_AMT'];
            }
            if ($record['CREDIT_AMT'] > CLX_NUM_ZERO) {
                $record['CREDIT_TRANS_AMT'] = $record['CREDIT_AMT'];
            }
            $record['DEBIT_AMT'] = rounder($record['DEBIT_AMT'] * pow($record['EXCHANGE_RT'], $record['CUR_OPERATOR']), CLX_NUM_TWO);
            $record['CREDIT_AMT'] = rounder($record['CREDIT_AMT'] * pow($record['EXCHANGE_RT'], $record['CUR_OPERATOR']), CLX_NUM_TWO);
        } else {
            //if ($record['DEBIT_AMT'] > CLX_NUM_ZERO && $record['DEBIT_TRANS_AMT'] == CLX_NUM_ZERO) {
            if ($record['DEBIT_AMT'] > CLX_NUM_ZERO) {
                $record['DEBIT_TRANS_AMT'] = rounder($record['DEBIT_AMT'] / $record ['EXCHANGE_RT'], CLX_NUM_TWO);
            }
            //if ($record['CREDIT_AMT'] > CLX_NUM_ZERO && $record['CREDIT_TRANS_AMT'] == CLX_NUM_ZERO) {
            if ($record['CREDIT_AMT'] > CLX_NUM_ZERO) {
                $record['CREDIT_TRANS_AMT'] = rounder($record['CREDIT_AMT'] / $record ['EXCHANGE_RT'], CLX_NUM_TWO);
            }
        }
        //Ajustando activos si son diferentes a la moneda de la transaccion. La moneda se cambia en fiat.
        $sql = 'SELECT COMP.CURRENCY_CODE FROM ' . $this->compname . ' COMP, ' . $this->jehdname . ' JEHD WHERE COMP.IDCODE = JEHD.COMP_CODE AND JEHD.IDCODE =' . singleQuote($record['JEHDR_CODE']);
        $base_currency = ClxCommon::$db->getOne($sql);
        if ($base_currency == $record['CURRENCY_CODE'] && $record['EXCHANGE_RT'] <> CLX_NUM_ONE) {
            //Verificar que cambio de moneda cambia a conversion 1
            $record ['EXCHANGE_RT'] = CLX_NUM_ONE;
            $record ['CUR_OPERATOR'] = CLX_NUM_ONE;
            $record['DEBIT_TRANS_AMT'] = $record['DEBIT_AMT'];
            $record['CREDIT_TRANS_AMT'] = $record['CREDIT_AMT'];
        }
        //Check if update needed
        if ($record['SEQNUM'] === '')
            $record['SEQNUM'] = CLX_INIT_ZERO;
        $crit = 'JEHDR_CODE =' . singleQuote($record['JEHDR_CODE']) . ' AND SEQNUM = ' . $record['SEQNUM'] . ' AND GLACCOUNTCODE =' . singleQuote($record['GLACCOUNTCODE']) . ' AND CHART_FIELD01 = ' . singleQuote($record['CHART_FIELD01']) . '  AND CHART_FIELD02 = ' . singleQuote($record['CHART_FIELD02']);
        $crit .= ($record['CHART_FIELD03'] === 'null') ? ' AND CHART_FIELD03 IS NULL ' : ' AND CHART_FIELD03 = ' . singleQuote($record['CHART_FIELD03']);
        $crit .= ($record['CHART_FIELD04'] === 'null') ? ' AND CHART_FIELD04 IS NULL ' : ' AND CHART_FIELD04 = ' . singleQuote($record['CHART_FIELD04']);
        if ($record['GLACCOUNTCODE'] === 'null' && $record['GLACCOUNTCODE'] == '') {
            $record['GLACCOUNTCODE'] = '';
        }

        if (ClxCommon::$db->countRecords($this->jelnname, $crit) == CLX_NUM_ZERO) {
            setCreationData($record);
            ClxCommon::$db->createRecord($this->jelnname, $record, false);
        } else {
            setModificationData($record);
            ClxCommon::$db->updateRecord($this->jelnname, $record, $crit);
        }
    }

}
