<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - System Alert Message.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_alert extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ALER';
        $this->name = 'Alert Submition';
        $this->type = 'ACTN';
        $this->version = '1.1.0.0';
        $this->description = 'Creates a system Alert.';
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('CODE_LBL_ALER_PROPS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $ACUS = &getTool('ACUS');
        $ACUS->placeChooseUser($msg, dict('GEN_LBL_ASSIGNED_TO') . ':', 'ALER_assignedto', $actionrecord['ALER_assignedto'], $table);
        $ACGR = &getTool('ACGR');
        if (!empty($msg['ALER_groupto'])) {
		  $table->addCell($msg['ALER_groupto'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addData(dict('GEN_LBL_GROUP_TO') . ':', $ACGR->chooseRecord('ALER_groupto', $actionrecord['ALER_groupto'], $actionrecord, CLX_NUM_20, CLX_NUM_16));
        $table->addField($msg, dict('GEN_LBL_DELIVERY_DATI') . ':', 'ALER_start_dati', $actionrecord['ALER_start_dati'], CLX_NUM_32, CLX_NUM_32);
        $table->addCell(dict('GEN_LBL_MESSAGE'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (isset($msg['ALER_description'])) {
		  $table->addCell($msg['ALER_description'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $c = new htmlObject('textarea', htmlspecialchars($actionrecord['ALER_description']), array('name' => 'ALER_description', 'rows' => CLX_NUM_15, 'cols' => CLX_NUM_80));
        $c->forceclose = true;
        $table->addCell($c->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
		  $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
		  $ACUS = &getTool('ACUS');
		  if (!$ACUS->userExists(UserRequest::$post['ALER_assignedto'])) {
			 $msg['ALER_assignedto'] = dict('GEN_LBL_USER_DONT_EXIST');
		  }
		  if (!$ACUS->userExists(UserRequest::$post['ALER_groupto'])) {
			 $msg['ALER_groupto'] = dict('GEN_LBL_GROUP_DONT_EXIST');
		  }
		  if (empty(UserRequest::$post['ALER_description'])) {
			 $msg['ALER_description'] = dict('GEN_LBL_DESCRIPTION_MISS');
		  }
		  xValidate::datetime($msg, dict('GEN_LBL_DELIVERY_DATI'), UserRequest::$post['ALER_start_dati'], 'ALER_start_dati');
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['EXTRA1'] = CLX_INIT_ZERO;
        $actionrecord['EXTRA2'] = UserRequest::$post['ALER_assignedto'];
        $actionrecord['EXTRA3'] = UserRequest::$post['ALER_groupto'];
        $actionrecord['EXTRA4'] = UserRequest::$post['ALER_start_dati'];
        $actionrecord['EXTRA5'] = 'null';
        $actionrecord['EXTRA6'] = UserRequest::$post['ALER_description'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  $actionrecord['ALER_assignedto'] = $values['EXTRA2'];
		  $actionrecord['ALER_groupto'] = $values['EXTRA3'];
		  $actionrecord['ALER_start_dati'] = $values['EXTRA4'];
		  $actionrecord['ALER_description'] = $values['EXTRA6'];
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['ALER_assignedto'] = $_SESSION['user_record']['username'];
        $actionrecord['ALER_groupto'] = '';
        $actionrecord['ALER_start_dati'] = 'now';
        $actionrecord['ALER_description'] = '';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm_2(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_ASSIGNED_TO') . ':', $record['ALER_assignedto']);
        $table->addData(dict('GEN_LBL_GROUP_TO') . ':', $record['ALER_groupto']);
        $table->addData(dict('GEN_LBL_DELIVERY_DATI') . ':', $record['ALER_start_dati']);
        $table->addData(dict('GEN_LBL_MESSAGE') . ':', $record['ALER_description']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_CREATE_ALER_TO') . ': ' . $record['EXTRA2'] . ' (' . $record['EXTRA3'] . ')';
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        $message = $event['EXTRA6'];
        $delivery_date = $event['EXTRA4'];
        $username = $event['EXTRA2'];
        $groupcode = $event['EXTRA3'];
        if (empty($username) && empty($groupcode))
		  return false;
        return alrt_send($message, $delivery_date, $username, $groupcode);
    }

}
