<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 * @package		Clientix.Tool.Event
 * @abstract		System Events Manager - Webservice Event.
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
if (!class_exists('actn') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'actn.php';

class actn_listing extends actn {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ACLS';
        $this->name = 'Listing Event';
        $this->type = 'ACTN';
        $this->version = '5.5.0.0';
        $this->description = 'Executes Listing Event.';
        $this->set_bp_req_user_wait(true);
        $this->events = array();
        $this->events['BCNO'] = array('name' => 'GEN_LBL_NONE', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => false);
        $this->events['BCCR'] = array('name' => 'GEN_LBL_CREATE_RECORD', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => true);
        $this->events['BPMR'] = array('name' => 'GEN_LBL_MODIFY_RECORD', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => false);
        $this->events['BPDR'] = array('name' => 'GEN_LBL_DELETE_RECORD', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => false);
        $this->events['BCRC'] = array('name' => 'GEN_LBL_ON_CREATION', 'object' => 'BPPR', 'language' => 'PHP', 'wait_user_action' => false);
    }

    /**
     * Build code object panel on edit event.
     * @param array $msg
     * @param array $actionrecord
     * @return string
     */
    function panel($msg, $actionrecord) {
        if ($actionrecord['EVNT_action'] != $this->code) {
		  $this->defaultPanel($actionrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => '90%', CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_ACLS_PROPS') . ':', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $UILG = &getTool('UILG');
        if (!empty($msg['ACLS_listing_code'])) {
		  $table->addCell($msg['ACLS_listing_code'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addData(dict('GEN_LBL_LISTING') . ':', $UILG->chooseRecord('ACLS_listing_code', $actionrecord['ACLS_listing_code'], $actionrecord, CLX_NUM_20, CLX_NUM_16));
        $event = new htmlSelect('ACLS_event_type');
        foreach ($this->events as $code => $info) {
		  $event->addItem(dict($info['name']), $code, ($code == $actionrecord ['ACLS_event_type']));
        }
        $table->addData(dict('GEN_LBL_LISTING_ACTION') . ':', $event->source());
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    /**
     * Validate Panel User Input before saving record.
     * @param array $msg
     */
    function checkPanel(&$msg) {
        if (UserRequest::$post['EVNT_language'] <> 'PHP' && UserRequest::$post['EVNT_language'] <> 'COMP') {
		  $msg['EVNT_action'] = dict('EVNT_LBL_PHP_ONLY');
        } else {
		  if (empty(UserRequest::$post['ACLS_listing_code'])) {
			 $msg['ACLS_listing_code'] = dict('GEN_LBL_INVALID_LISTING');
		  } else {
                if (ClxCommon::$db->oneExistsRecords('CLX_UI_LISTINGS', 'IDCODE = ' . singleQuote(UserRequest::$post['ACLS_listing_code'])) == CLX_NUM_ZERO) {
			     $msg['ACLS_listing_code'] = dict('GEN_LBL_INVALID_LISTINGS');
			 }
		  }
        }
    }

    /**
     * Build object record from user post
     * @param array $actionrecord
     */
    function getPanel(&$actionrecord) {
        $actionrecord['extra1'] = CLX_INIT_ZERO;
        $actionrecord['extra2'] = UserRequest::$post['ACLS_listing_code'];
        $actionrecord['extra3'] = UserRequest::$post['ACLS_event_type'];
    }

    /**
     * Sets edit record required on forms of user interface.
     * @param array $actionrecord
     * @param array $values
     */
    function setPanel(&$actionrecord, $values) {
        if ($values['ACTION'] == $this->code) {
		  $actionrecord['ACLS_listing_code'] = $values['EXTRA2'];
		  $actionrecord['ACLS_event_type'] = $values['EXTRA3'];
        } else {
		  $this->defaultPanel($actionrecord);
        }
    }

    /**
     * Sets the default values for form on create event.
     * @param array $actionrecord
     */
    function defaultPanel(&$actionrecord) {
        $actionrecord['ACLS_listing_code'] = '';
        $actionrecord['ACLS_event_type'] = 'BCNO';
    }

    /**
     * Build panel for display form
     * @param obj $table
     * @param array $record
     */
    function viewForm_2(&$table, $record) {
        $table->addData(dict('GEN_LBL_ACTION') . ':', $this->name);
        $table->addData(dict('GEN_LBL_LISTING') . ':', $record['ACLS_listing_code']);
        $table->addData(dict('GEN_LBL_TRIGGER_EVENT') . ':', $record['ACLS_event_type']);
    }

    /**
     * Builds description about the object on event panel listing.
     * @param array $record
     * @return string
     */
    function describe($record) {
        return dict('GEN_LBL_LISTING', '') . ': ' . $record['EXTRA2'];
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute3($event, &$record, &$msg) {IF (ISSET($GLOBALS['ADDRDEBUGEVENTS'])) { echo __METHOD__.' '. __FILE__ .':'.__LINE__; pre($event);pre($record); }
        return true;
    }

    /**
     * Execute code according object type.
     * @param string $event
     * @param array $record
     * @param array $msg
     */
    function execute_bp_manual_event($event, &$record, &$msg, $debug = false) {

        switch ($event['EXTRA3']) {
		  case 'BCCR': //Create record from listing
			 $ac = 'CREATE';
			 $li = $event['EXTRA2'];
			 header('location: tool.php?to=EDIT&ac=' . $ac . '&li=' . $li);
			 salir();
			 break;
		  default:
			 break;
        }
        return true;
    }

}
