<?php

/**
 * Object Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		System Chart Manager
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 */
class cpty extends clx_object {

    var $functions = array();
    var $classname;
    var $classfile;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'CPTY';
        $this->name = 'Generic Chart Property Adaptor';
        $this->type = '----';
        $this->version = '0.3.0.0';
        $this->description = 'Generic Chart Property Adaptor';
    }

    function panel($msg, $seriesrecord, $visible = false) {
        if ($seriesrecord['ANCS_type'] <> $this->code) {
		  $this->defaultPanel($seriesrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO, CLX_PROP_WIDTH, '90%');
        $table->addCell(dict('OBJ_' . $this->code . '_TITLE_PROPERTIES'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        foreach ($this->functions as $function) {
		  $prompt = dict($function[CLX_CLASS_PROMPT]) . ':';
		  foreach ($function['arguments'] as $argument) {
			 $name = $this->code . '_' . $function['name'] . '_' . $argument['name'];
			 if (!array_key_exists($name, $seriesrecord)) {
			     $seriesrecord[$name] = '';
			 }
			 if (!empty($msg[$name])) {
			     $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
			 }
			 switch ($argument['type']) {
			     case 'NUMBER':
			     case 'STRING':
			         $dsize = isset($argument['displaysize']) ? $argument['displaysize'] : CLX_NUM_TEN;
			         $size = isset($argument['size']) ? $argument['size'] : CLX_NUM_TEN;
			         $f = new HTMLField($name, $seriesrecord[$name], $dsize, $size);
			         break;
			     case 'TEXT':
			     case 'ARRAY':
			         $cols = isset($argument['cols']) ? $argument['cols'] : CLX_NUM_48;
			         $rows = isset($argument['rows']) ? $argument['rows'] : CLX_NUM_FOUR;
			         $f = new HTMLTextArea($name, $seriesrecord[$name], $rows, $cols);
			         break;
			     case 'LIST':
			         $f = new HTMLSelect($name);
			         $options = explode(',', $argument['options']);
			         $f->addItem('', '', ($seriesrecord[$name] === ''));
			         foreach ($options as $item) {
					   $pos = strpos($item, ':');
					   if ($pos !== false) {
						  $text = dict(substr($item, $pos + CLX_NUM_ONE));
						  $item = substr($item, CLX_NUM_ZERO, $pos);
					   } else {
						  $text = $item;
					   }
					   $f->addItem($text, $item, ($seriesrecord[$name] == $item));
			         }
			         break;
			     case 'BOOLEAN':
			         if (!array_key_exists($name, $seriesrecord))
					   $seriesrecord[$name] = '';
			         $f = new HTMLSelect($name);
			         $f->addItem('', '', ($seriesrecord[$name] === ''));
			         if (isset($argument['options'])) {
					   $options = explode(',', $argument['options']);
					   $f->addItem(dict($options[CLX_NUM_ZERO]), '1', ($seriesrecord[$name] === '1'));
					   $f->addItem(dict($options[CLX_NUM_ONE]), '0', ($seriesrecord[$name] === '0'));
			         } else {
					   $f->addItem(dict('GEN_LBL_YES'), '1', ($seriesrecord[$name] === '1'));
					   $f->addItem(dict('GEN_LBL_NO'), '0', ($seriesrecord[$name] === '0'));
			         }
			         break;
			     default :
			         break;
			 }
			 $p = (!empty($argument[CLX_CLASS_PROMPT])) ? dict($argument[CLX_CLASS_PROMPT]) . ': ' : CLX_CHAR_HTML_NBSP;
			 $prefix = (!empty($argument['prefix'])) ? dict($argument['prefix']) . ' ' : '';
			 $sufix = (!empty($argument['postfix'])) ? ' ' . dict($argument['postfix']) : '';
			 $table->addData($prompt, $p . $prefix . $f->source() . $sufix);
			 $prompt = CLX_CHAR_HTML_NBSP;
		  }
        }
        if ($visible === true) {
		  $div = new htmlDiv($this->code, '', $table->source());
        } else {
		  $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        }
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        foreach ($this->functions as $function) {
		  foreach ($function['arguments'] as $argument) {
			 $name = $this->code . '_' . $function['name'] . '_' . $argument['name'];
			 $required = (array_key_exists('required', $argument)) ? $argument['required'] : CLX_NUM_ZERO;
			 $something = (array_key_exists($name, UserRequest::$post) && strlen(UserRequest::$post[$name]) > CLX_NUM_ZERO) ? true : false;
			 switch ($argument['type']) {
			     case 'NUMBER':
			         if (!array_key_exists('min', $argument))
					   $argument['min'] = '';
			         if (!array_key_exists('max', $argument))
					   $argument['max'] = '';
			         if (!empty($argument['decimal'])) {
					   if (!empty($something) or ! empty($required))
						  xValidate::float($msg, $name, UserRequest::$post[$name], $argument['min'], $argument['max'], 'this number');
			         } else {
					   if (!empty($something) or ! empty($required))
						  xValidate::integer($msg, $name, UserRequest::$post[$name], $argument['min'], $argument['max'], 'this number');
			         }
			         break;
			     case 'STRING':
			         if (!empty($required) and empty($something)) {
					   $msg[$name] = dict('GEN_ERR_REQUIRED_FIELD');
			         }
			         break;
			     case 'BOOLEAN':
			         break;
			     case 'LIST':
			         break;
			     case 'TEXT':
			         break;
			     case 'ARRAY':
			         break;
			     default :
			         break;
			 }
		  }
        }
    }

    function savePanel(&$seriesrecord) {
        $properties = new XMLObject('properties', false);
        foreach ($this->functions as $function) {
		  $group = new XMLObject($function['name'], false);
		  foreach ($function['arguments'] as $argument) {
			 $name = $this->code . '_' . $function['name'] . '_' . $argument['name'];
			 $o = new XMLObject($argument['name'], false);
			 $o->setContents(UserRequest::$post[$name]);
			 $group->addObject($o);
		  }
		  $properties->addObject($group);
        }
        $seriesrecord['SERIES_PROPERTIES'] = $properties->source_(array('level' => CLX_NUM_ZERO));
    }

    function loadPanel(&$seriesrecord, $values) {
        $properties = new XMLObject('properties', false);
        $properties->parse($values['SERIES_PROPERTIES']);
        foreach ($this->functions as $function) {
		  if ($properties->hasObject($function['name'])) {
			 $group = $properties->getObject($function['name']);
			 foreach ($function['arguments'] as $argument) {
			     $name = $this->code . '_' . $function['name'] . '_' . $argument['name'];
			     if ($group->hasObject($argument['name'])) {
			         $o = $group->getObject($argument['name']);
			         $seriesrecord[$name] = $o->getContents();
			     }
			 }
		  }
        }
    }

    function defaultPanel(&$seriesrecord) {
        foreach ($this->functions as $function) {
		  foreach ($function['arguments'] as $argument) {
			 $name = $this->code . '_' . $function['name'] . '_' . $argument['name'];
			 $seriesrecord[$name] = (array_key_exists('default', $argument)) ? $argument['default'] : '';
		  }
        }
    }

    function viewForm(&$table, $record) {

    }

    function getPlot(&$graph, $args, $series) {
        return null;
    }

    function setFunctions(&$graph, $args, $series) {
        $contenidoHTML = $this->code . ' called setFunctions<br>';
        echo $contenidoHTML;
    }

    function setFunction(&$object, $function, $func_arguments) {
        $command = $object->$function['function'] . '(';
        $comma = '';
        $doit = false;
        foreach ($function['arguments'] as $argument) {
		  switch ($argument['type']) {
			 case 'STRING':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         if (strlen($value) == CLX_NUM_ZERO && array_key_exists('default', $argument))
					   $value = $argument['default'];
			         if (strlen($value) == CLX_NUM_ZERO) {
					   $command .= $comma . 'null';
			         } else {
					   if (!empty($argument['constant'])) {
						  $command .= $comma . $value;
					   } else {
						  $command .= $comma . singleQuote($value);
					   }
			         }
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			     }
			     break;
			 case 'LIST':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         if (!empty($argument['constant'])) {
					   $command .= $comma . $value;
			         } else {
					   $command .= $comma . quote($value);
			         }
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			     }
			     break;
			 case 'NUMBER':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         if (strlen($value) == CLX_NUM_ZERO && array_key_exists('default', $argument))
					   $value = $argument['default'];
			         $command .= $comma . $value;
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			     }
			     break;
			 case 'BOOLEAN':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         if ($value === true) {
					   $command .= $comma . 'true';
			         } else {
					   $command .= $comma . 'false';
			         }
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			     }
			     break;
			 case 'TEXT':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         $command .= $comma . quote($value);
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			     }
			     break;
			 case 'ARRAY':
			     if ($func_arguments->hasObject($argument['name'])) {
			         $o = $func_arguments->getObject($argument['name']);
			         $value = $o->getContents();
			         $doit = ($doit or ( strlen($value) > CLX_NUM_ZERO));
			         if ($doit === true) {
					   $delim = (isset($argument['delim'])) ? $argument['delim'] : ',';
					   $list = explode($delim, $value);
					   $values = array();
					   foreach ($list as $item)
						  $values[] = quote($item);
					   $command .= $comma . 'array(' . implode(',', $values) . ')';
			         }
			     }
			     break;
			 default :
			     break;
		  }
		  $comma = ', ';
        }
        $command .= ');';
        if ($doit === true) {
		  $cal = new ClxExpression();
		  $command = $cal->parseRecordFormula($command);
        }
    }

}
