<?php

/**
 * Date field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 *
 * Change log
 *  Added Oracle on field properties
 *
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_date extends dbft {

    private $date_edit_format = 'Y-m-d';

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DATE';
        $this->name = 'Date';
        $this->type = 'DBFT';
        $this->version = '0.1.4.4';
        $this->description = 'Saves a date in the database';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO', '>' => 'GEN_LBL_GREATERTHAN', '>=' => 'GEN_LBL_GREATERTHANOREQUEALTO', '<=' => 'GEN_LBL_LESSTHANOREQUALTO', '<' => 'GEN_LBL_LESSTHAN');
    	IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_DATEFIELDPROP'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $yesno = new htmlSelect($c . '_required');
        $yesno->addItem(dict('GEN_LBL_NO'), '0', ($fieldrecord[$c . '_required'] == CLX_NUM_ZERO));
        $yesno->addItem(dict('GEN_LBL_YES'), '1', ($fieldrecord[$c . '_required'] == CLX_NUM_ONE));
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno->source());
        $table->addField($msg, dict('GEN_LBL_FORMAT') . ':', $c . '_format', $fieldrecord[$c . '_format'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_DEFAULT') . ':', $c . '_default', $fieldrecord[$c . '_default'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_SIZE') . ':', $c . '_displaysize', $fieldrecord[$c . '_displaysize'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_SIZE') . ':', $this->code . '_size', $fieldrecord[$c . '_size'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        xValidate::integer($msg, $c . '_displaysize', UserRequest::$post[$c . '_displaysize'], CLX_NUM_ONE, CLX_NUM_255, 'display size');
        if (!empty(UserRequest::$post[$c . '_required'])) {
		  xValidate::required($msg, $c . '_default', UserRequest::$post[$c . '_default']);
        }
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = UserRequest::$post[$c . '_size'];
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = UserRequest::$post[$c . '_format'];
        $fieldrecord['FIELD_ALIGN'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_default'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_DISPLAYSIZE'] = UserRequest::$post[$c . '_displaysize'];
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
		  $fieldrecord[$c . '_format'] = $values['FIELD_FORMAT'];
		  $fieldrecord[$c . '_default'] = $values['FIELD_DEFAULT'];
		  $fieldrecord[$c . '_displaysize'] = $values['FIELD_DISPLAYSIZE'];
		  $fieldrecord[$c . '_size'] = $values['FIELD_SIZE'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_NUM_30;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = $this->date_edit_format;//$this->format = $_SESSION['prefs']['DATE_FORMAT'];
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = 'today';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_displaysize'] = '30';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $yesno = array(CLX_NUM_ZERO => dict('GEN_LBL_NO'), CLX_NUM_ONE => dict('GEN_LBL_YES'));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno[$record['DATE_required']]);
        $table->addData(dict('GEN_LBL_FORMAT') . ':', $record['DATE_format']);
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $record['DATE_default']);
        $table->addData(dict('GEN_LBL_DISPLAY_SIZE') . ':', $record['DATE_displaysize']);
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        //Setting form field requirement if any diferent than table default
        if ($fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        if (!empty($fieldrecord['FIELD_HIDDEN'])) {
		  $form->addArgument($form_fieldname, (array_key_exists($form_fieldname, $record)) ? $record[$form_fieldname] : '');
		  $table = $this->placeViewField($form_fieldname, $fieldrecord, $record, $msg);
        } else {
		  $cpt = dict($fieldrecord['CAPTION']);
		  if (strlen($cpt) > CLX_NUM_ZERO) {
			 $table->addCell($cpt, array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  if (isset($msg[$form_fieldname])) {
			 $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  }
		  $fieldobj = $this->fieldObject($form_fieldname, $fieldrecord, $record);
		  foreach ($jsevents as $key => $value) {
			 $fieldobj->setProp($key, $value);
		  }
		  $pfx = dict($fieldrecord['FIELDPREFIX'],true,false); 
		  $fieldsource = (strlen($pfx) > CLX_NUM_ZERO) ? $pfx : '';
		  $fieldsource .= $fieldobj->source();
		  $sfx = dict($fieldrecord['FIELDSUFIX'],true,false); 
		  $fieldsource .= (strlen($sfx) > CLX_NUM_ZERO) ? $sfx : '';
		  $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
		  //Added to Display Date Format
		  $EDIT = &getTool('EDIT');
		  //$format_label_dict = $EDIT->date_in_formats[$_SESSION['prefs']['DATE_FORMAT']];
		  $format_label_dict = $EDIT->date_in_formats[$this->date_edit_format];
		  $fieldlabel = dict($fieldrecord['FIELDFORMLABEL']) . '<br><span class=' . singleQuote('promptdateformat') . '>' . dict($format_label_dict) . '</spam>';
		  $table->addCell($fieldlabel, array(CLX_PROP_CLASS => $class, CLX_PROP_ID => 'LBL_' . $form_fieldname));
		  //Added to display calendar link
		  if (empty($fieldrecord['FIELD_READONLY'])) {
			 $format_javascript = $this->date_edit_format;//$_SESSION['prefs']['DATE_FORMAT'];
			 $format_javascript = str_replace('d', 'dd', $format_javascript);
			 $format_javascript = str_replace('m', 'mm', $format_javascript);
			 $format_javascript = str_replace('Y', 'yy', $format_javascript);
			 $format_javascript = str_replace(' H:i:s','',$format_javascript);
			 $picker = '$(function() { '
			         //.'  if(typeof(axVar) == "undefined") { '
			         //.'	var    axVar=["' . $form_fieldname . '"]; '
			         //.'  }else{'
			         //.'     axVar.push("'.$form_fieldname . '"); '
			         //.' } '
			         //. 'var	forma=$("#' . $form_fieldname . '").prop("form").name; '
			         //  'var submittext=$("#"+forma).data( "events" ).submit.toSource(); '
			         //	. ' alert(submittext)';
			         //. '$("#"+forma).submit(function() { alert(axVar[0]);'
			         //. '$( "#"+axVar[0] ).datepicker("option","dateFormat", "yy-mm-dd").val();'
			         //. 'alert(1);'
			         //	. '});'
			         //. 'var pos=indexOf("function () {"+12)
			         //. 'submit=submit+";"+function'
			         . 'var pickOpts = {dateFormat: "' . $format_javascript . '", showTime: false, stepMinutes: 1, stepHours: 1, altTimeField: "", duration: "", showTime: false, constrainInput: false, time24h: true, changeMonth: true, changeYear: true, showOn: "button", buttonImage: "./' . kPathToSkins . $_SESSION['skin'] . '/images/calendar/cal.gif", buttonImageOnly: true};'
			         . '$("#' . $form_fieldname . '").datepicker(pickOpts);'
			         . '});';
			 $fieldsource .= setJavaScriptTag($picker, false);
		  }

		  if (array_key_exists('SRCH_LISTING', $fieldrecord) && $fieldrecord['SRCH_LISTING'] <> '') {
                $fieldsource .= getSkinIcon('view', 'javascript:formToCookie(' . singleQuote('theForm') . ');openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=EDIT&nofr=1&pfid=' . $fieldrecord['PFID'] . '&ac=POPLIST&li=' . $fieldrecord['SRCH_LISTING']) . ',700, 400,null,'.singlequote($form_fieldname).','.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');', dict('GEN_LBL_SEARCH'));
		  }
		  $table->addCell($fieldsource, array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_ID => 'FLD_' . $form_fieldname));
        }
		if ($this->debug==true ) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_date-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
			}

			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldsource,true));
			fclose($tta);
			if (!isset($GLOBALS['placelistcount']))
				$GLOBALS['placelistcount']=1;
			else
				$GLOBALS['placelistcount']++;			
		}
        return $table;
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        $props['type'] = 'DATE';
        $def = str_replace(' ', '', str_replace(':', '', str_replace('-', '', $fieldrecord['FIELD_DEFAULT'])));
        if (is_numeric($def)) {
		  if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
			 $props['default'] = 'null';
		  } else {
			 $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
			 $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
		  }
        } else if (uppercase($fieldrecord['FIELD_DEFAULT']) === 'TODAY' OR uppercase($fieldrecord['FIELD_DEFAULT']) === 'CURRENT_DATE' ) {
			$props['default'] = 'TODAY';
		}
		else
		{
		  $props['default'] = 'null';
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    function defaultField($form_fieldname, $fieldrecord, &$record) {
        $record[$form_fieldname] = xFormat::date($fieldrecord['FIELD_DEFAULT'], $this->date_edit_format);
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        $field_displaysize = $fieldrecord['FIELD_DISPLAYSIZE'];
        if (empty($field_displaysize))
		  $field_displaysize = $fieldrecord['FIELD_DB_DISPLAYSIZE'];
        $field_size = (array_key_exists('FIELD_DB_SIZE', $fieldrecord)) ? $fieldrecord['FIELD_DB_SIZE'] : '';
        if (!empty($fieldrecord['FIELD_SIZE'])) {
		  $field_size = $fieldrecord['FIELD_SIZE'];
        }
        if (empty($field_size)) {
		  $field_size = $field_displaysize;
        }
        if (!array_key_exists($form_fieldname, $record)){
		  $record[$form_fieldname] = '';
        }
        //$fieldobj = new htmlField($form_fieldname, xformat::date($record[$form_fieldname], $_SESSION['prefs']['DATE_FORMAT']), $field_displaysize, $field_size);
        $fieldobj = new htmlField($form_fieldname, xformat::date($record[$form_fieldname], $this->date_edit_format), $field_displaysize, $field_size);
        //Added instead of displaying read only files
        if (array_key_exists('FIELD_READONLY', $fieldrecord) && $fieldrecord['FIELD_READONLY']) {
		  $fieldobj->setProp('readonly', 'true');
		  $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
        }
        return $fieldobj;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        //Setting form require pref if any
        if (array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        if (empty($fieldrecord['FIELD_HIDDEN'])) {
		  if (!$fieldrecord['FIELD_REQUIRED'] === '1' && $record[$form_fieldname] === '0000-00-00') {
			 $record[$form_fieldname] = '';
		  }
		  if (UserRequest::$request['ac']!='DELETE' &&(!empty($record[$form_fieldname]) || $fieldrecord['FIELD_REQUIRED'] === '1')) {
			 xValidate::date($msg, $form_fieldname, $record[$form_fieldname], dict($fieldrecord['FIELDLISTLABEL']));
		  }
        }//end hidden
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
		  return;
        } else {
		  if (($fieldrecord['FIELD_REQUIRED'] && $fieldvalue === '') || !empty($fieldvalue)) {
			 xValidate::date($msg, $fieldrecord['FIELDNAME'], $fieldvalue, dict($fieldrecord['FIELDLISTLABEL']));
		  }
        }
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid = null) {
        if (strlen(UserRequest::$post[$form_fieldname]) == CLX_NUM_ZERO) {
		  $record[$fieldrecord['FIELDNAME']] = 'null';
        } else {
		  $record[$fieldrecord['FIELDNAME']] = xFormat::date(UserRequest::$post[$form_fieldname], 'Y-m-d');
        }
    }

    /**
     *
     * @param type $form_fieldname
     * @param type $fieldrecord
     * @param type $record
     * @return format de campo
     */
    function formatField($form_fieldname, $fieldrecord, $record) {
        $form_fieldname = uppercase($form_fieldname);
        $record = array_change_key_case($record, CASE_UPPER);
        if (!array_key_exists($form_fieldname, $record)) {
		  $record[$form_fieldname] = '';
        }
        if(!empty($fieldrecord['DBQF_FIELD_FORMAT']) &&   $fieldrecord['DBQF_FIELD_FORMAT'] === '1'){
		  if (array_key_exists($form_fieldname, $record)) {
			 $original_value = $record[$form_fieldname];
		  } else {
			 $original_value = '';
		  }
		  return $original_value;
        }


        if ($record[$form_fieldname] === '0000-00-00') {
		  return CLX_CHAR_EMPTY_STRING;
        } elseif (strlen($record[$form_fieldname]) == CLX_NUM_ZERO) {
		  return CLX_CHAR_EMPTY_STRING;
        } else {
		  return xFormat::date($record[$form_fieldname], $_SESSION['prefs']['DATE_FORMAT']);
		  //return xFormat::date($record[$form_fieldname], $fieldrecord['FIELD_FORMAT']);
        }
    }

    /**
     * Build session key for filters
     */
    function setQuickFilters($tablecode, $fieldname, $fieldrecord, $record) {
        $criteria = '';
        if (array_key_exists($tablecode . '_' . $fieldname, $record) && strlen($record[$tablecode . '_' . $fieldname]) > CLX_NUM_ZERO) {
		  if (strpos($record[$tablecode . '_' . $fieldname], ',') > CLX_NUM_ZERO) {
			 $values = explode(',', $record[$tablecode . '_' . $fieldname]);
			 $criteria = $tablecode . '.' . $fieldname . ' >= ' . singleQuote(trim($values[CLX_NUM_ZERO])) . ' ';
			 $criteria .= ' AND ' . $tablecode . '.' . $fieldname . ' <= ' . singleQuote(trim($values[CLX_NUM_ONE])) . ' ';
		  } else {
			 $criteria = $tablecode . '.' . $fieldname . ' LIKE ' . singleQuote(str_replace(CLX_CHAR_SINGLE_QUOTE, '', $record[$tablecode . '_' . $fieldname]) . '%');
		  }
        }
        return $criteria;
    }

    function parseField($form_fieldname, $fieldrec, &$record) {
       if (array_key_exists($form_fieldname, $record)) {
	        $tolower = lowercase($record[$form_fieldname]);
	        if ($tolower <> 'now' and $tolower <> 'yesterday' and $tolower <> '') {
			  $fechas = array();
			  $fechas = explode(',', $record[$form_fieldname]);
			  $count = count($fechas);
			  for ($k = 0; $k < $count; $k++) {
				 //$fechas[$k] = fromIsoDate($fechas[$k], $_SESSION['prefs']['DATE_FORMAT']);
				 $fechas[$k] = fromIsoDate($fechas[$k], $this->date_edit_format);
			  }
			  $record[$form_fieldname] = implode(',', $fechas);
			  UserRequest::setInput(UserRequest::$post, $form_fieldname, $record[$form_fieldname]);
			  UserRequest::setInput(UserRequest::$request, $form_fieldname, $record[$form_fieldname]);
			  UserRequest::setInput(UserRequest::$get, $form_fieldname, $record[$form_fieldname]);
	        }
	    }
   }     

}
