<?php

/**
 * List field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-01 05:44:03 -0400 (Wed, 01 Aug 2018) $
 * @version    	$Rev: 9874 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_list extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'LIST';
        $this->name = 'List';
        $this->type = 'DBFT';
        $this->version = '0.2.3.2';
        $this->description = 'Saves a reference to a list item';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO', '!== ' => 'GEN_LBL_NOTEQUALTO');
        $this->yesno = array('GEN_LBL_NO', 'GEN_LBL_YES');
    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_LISTFIELDPROPS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addYesNo(dict('GEN_LBL_REQUIRED'), $c . '_required', $fieldrecord[$c . '_required'], CLX_NUM_ZERO, false);
        $DBLT = &getTool('DBLT');
        $DBLI = &getTool('DBLI');
        $DBLT->placeChooseList($msg, dict('GEN_LBL_LISTCODE') . ':', $c . '_extra1', $fieldrecord[$c . '_extra1'], $table);
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $DBLI->placeChooseListItem($msg, $c . '_extra1', 'Default Item:', $c . '_extra2', $fieldrecord[$c . '_extra2'], $table);
        $table->addYesNo(dict('GEN_LBL_MULTI_SEL'), $c . '_extra3', $fieldrecord[$c . '_extra3'], CLX_NUM_ZERO, false);
        $table->addField($msg, dict('GEN_LBL_DISPLAY_OPTIONS') . ':', $c . '_extra4', $fieldrecord[$c . '_extra4'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        $contents .= CLX_CHAR_NL . CLX_CHAR_NL . $div->source();
        return $contents . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        $DBLT = &getTool('DBLT');
        if (empty(UserRequest::$post[$c . '_extra1'])) {
		  $msg[$c . '_extra1'] = dict('GEN_LBL_FIELDREQUIRED');
        } elseif (!$DBLT->listExists(UserRequest::$post[$c . '_extra1'])) {
		  $msg[$c . '_extra1'] = dict('GEN_LBL_INVALID_LIST');
        } elseif (!$DBLT->listItemExists(UserRequest::$post[$c . '_extra1'], UserRequest::$post[$c . '_extra2'])) {
		  $msg[$c . '_extra2'] = dict('GEN_LBL_INVALID_LIST_ITEM');
        }
        $c = $this->code;
    }

    function savePanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord['FIELD_SIZE'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_REQUIRED'] = UserRequest::$post[$c . '_required'];
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_DEFAULT'] = UserRequest::$post[$c . '_extra2'];
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = UserRequest::$post[$c . '_extra1'];
        $fieldrecord['FIELD_EXTRA2'] = UserRequest::$post[$c . '_extra2'];
        $fieldrecord['FIELD_EXTRA3'] = (empty(UserRequest::$post[$c . '_extra3'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $fieldrecord['FIELD_EXTRA4'] = (empty(UserRequest::$post[$c . '_extra4'])) ? CLX_NUM_ZERO : UserRequest::$post[$c . '_extra4'];
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $c = $this->code;
        if ($values['FIELDTYPE'] == $c) {
		  $fieldrecord[$c . '_required'] = $values['FIELD_REQUIRED'];
		  $fieldrecord[$c . '_extra1'] = $values['FIELD_EXTRA1'];
		  $fieldrecord[$c . '_extra2'] = $values['FIELD_EXTRA2'];
		  $fieldrecord[$c . '_extra3'] = (empty($values['FIELD_EXTRA3'])) ? CLX_NUM_ZERO : CLX_NUM_ONE;
		  $fieldrecord[$c . '_extra4'] = $values['FIELD_EXTRA4'];
        } else {
		  $this->defaultPanel($fieldrecord);
        }
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_required'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_extra4'] = CLX_NUM_ONE;
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $yesno = array(CLX_NUM_ZERO => dict('GEN_LBL_NO'), CLX_NUM_ONE => dict('GEN_LBL_YES'));
        $table->addData(dict('GEN_LBL_TYPE') . ':', $this->name);
        $table->addData(dict('GEN_LBL_REQUIRED') . ':', $yesno[$record['LIST_required']]);
        $table->addData(dict('GEN_LBL_LIST_CODE') . ':', $record['LIST_extra1']);
        $table->addData(dict('GEN_LBL_DEFAULT') . ':', $record['LIST_extra2']);
        $table->addData(dict('GEN_LBL_MULTI_SEL') . ':', dict($this->yesno[$record['LIST_extra3']]));
        $table->addData(dict('GEN_LBL_DISPLAY_OPTIONS') . ':', $record['LIST_extra4']);
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        // DB implementation
        if (empty($fieldrecord['FIELD_EXTRA3'])) {
		  $lenght = CLX_NUM_TEN;
        } else {
		  $lenght = CLX_NUM_255;
        }
        switch (kDatabaseType) {
		  case 'oci':
			 $props['type'] = 'NVARCHAR2(' . $lenght . ')';
//			 $props['type'] = 'VARCHAR2(' . $lenght . ')';
			 break;
		  default:
			 $props['type'] = 'varchar(' . $lenght . ')';
			 break;
        }
        if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
		  $props['default'] = 'null';
        } else {
		  $props['default_raw'] = $fieldrecord['FIELD_DEFAULT'];
		  $props['default'] = singleQuote($fieldrecord['FIELD_DEFAULT']);
        }
        $props['properties'] = '';
        $props['null'] = ($fieldrecord['FIELD_REQUIRED']) ? 'NO' : 'YES';
        return $props;
    }

    function fieldObject(&$form_fieldname, &$fieldrecord, $record) {
        //If preloaded all item from list will be added during form load
        $preloaded = (!empty($fieldrecord['PRELOAD_LIST'])) ? true : false;
        $DBLT = &getTool('DBLT');
        if (!array_key_exists($form_fieldname, $record)) {
		  $record[$form_fieldname] = $fieldrecord['FIELD_DEFAULT'];
        }IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {pre( "=". __LINE__ ."====>dbft ". $form_fieldname);}
        if (!empty($fieldrecord['FIELD_EXTRA3'])) {//multiple
		  $pop = new htmlSelect($form_fieldname . '[]');
		  $pop->setProp(CLX_PROP_ID, $form_fieldname);
        } else {
		  $pop = new htmlSelect($form_fieldname);
        }
        //En ibs se hace un wrap extrano. por eso se elimina la funcionalidad de ajustar la dimension en este caso.
        if (!empty($preloaded)) {
		  $pop->setProp('clxlock', '1');
        } else {
		  $pop->setProp('clxlock', '0');
        }IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {pre( "=". __LINE__ ."====>dbft ". $form_fieldname);}
        if (!array_key_exists('PFID', $fieldrecord))
		  $fieldrecord['PFID'] = '';
        if (!array_key_exists('FIELD_PARENT', $fieldrecord))
		  $fieldrecord['FIELD_PARENT'] = '';
        $pop->setProp('clxuiff', $fieldrecord['PFID']);
        if (strlen($fieldrecord['FIELD_PARENT']) > CLX_NUM_ZERO) {
		  $pop->setProp('clxprnt', $fieldrecord['TABLECODE'] . '_' . $fieldrecord['FIELD_PARENT']);
        } else {
		  $pop->setProp('clxprnt', '');
        }
        if (!empty($fieldrecord['FIELD_EXTRA3'])) {//multiple
		  $pop->setProp('multiple', true);
        }
        if (!empty($fieldrecord['FIELD_EXTRA4'])) {
		  $pop->setProp('size', $fieldrecord['FIELD_EXTRA4']);
        } else {
		  $pop->setProp('size', CLX_NUM_ONE);
        }
        //If option selected then add single item to display to user
        if (array_key_exists($form_fieldname, $record) > CLX_NUM_ZERO) {
		  if (!empty($fieldrecord['FIELD_EXTRA3'])) {//multiple
                $crit = '';IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {pre( "=". __LINE__ ."====>dbftLIST ". $form_fieldname);}
			 ##$selopts = toArray(str_replace(CLX_CHAR_SINGLE_QUOTE,'',$record[$form_fieldname]));
			 $selopts = (is_array($record[$form_fieldname]))?$record[$form_fieldname]:toArray(str_replace(CLX_CHAR_SINGLE_QUOTE,'',$record[$form_fieldname]));
			 /*if (!is_array($record[$form_fieldname])) {
			     if (strlen($record[$form_fieldname]) == CLX_NUM_ZERO) {
			         //$crit = ' AND DBLI.IDCODE = ' . singleQuote(CLX_CHAR_EMPTY_STRING) ;
			     } else {
			         $crit = ' AND DBLI.IDCODE IN (' . $record[$form_fieldname] . ')';
			     }
			 } else {
			     $crit = 'AND DBLI.IDCODE IN (' . singleQuote(implode( CLX_CHAR_SINGLE_QUOTE, $record[$form_fieldname])) . ')';
			 }*/
			 $rs = $DBLT->getListSql($fieldrecord['FIELD_EXTRA1'], $crit);
			 while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			     $pop->addItem($r['IDNAME'], $r['IDCODE'], (is_array($selopts) && in_array(htmlspecialchars_decode($r['IDCODE']), $selopts)) ? true : false);
			 }
			 ClxCommon::$db->freeResult( $rs );
		  } else {
                if (!empty($preloaded)) {IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {pre( "=". __LINE__ ."====>dbftLIST ". $form_fieldname);}
			     $options = $DBLT->getListItems($fieldrecord['FIELD_EXTRA1'], CLX_NUM_ZERO);
			     foreach ($options as $k => $v) {
			         $pop->addItem($v, $k, (strval(htmlspecialchars_decode($record[$form_fieldname])) === strval($k)) ? true : false);
			     }
                } else {IF (ISSET($GLOBALS['ADDRDEBUGDICT']) && $GLOBALS['ADDRDEBUGDICT']==$_SERVER['REMOTE_ADDR'])  {pre( "=". __LINE__ ."====>dbfT ". $form_fieldname);}
			     $option = $DBLT->getListItem($fieldrecord['FIELD_EXTRA1'], $record[$form_fieldname], CLX_NUM_ZERO);
			     $pop->addItem($option, htmlspecialchars_decode($record[$form_fieldname]), true);
			 }
		  }
        }
        if (!empty($fieldrecord['FIELD_READONLY']) and (!$fieldrecord["FIELD_REQUIRED"] or ($fieldrecord["FIELD_REQUIRED"] and $record[$form_fieldname]<>'' and $record[$form_fieldname]<>'I' )) ) {
		  $pop->setProp('style', 'display:block;');
		  $pop->setProp(CLX_PROP_CLASS, 'readonly');
		  return $pop;
        }
        //Add list item if enabled

        if ($fieldrecord['ADD_LIST_ENTRY'] == CLX_NUM_ONE && $fieldrecord['FIELD_READONLY'] == CLX_NUM_ZERO) {
		  $pop->addSufix(getSkinIcon('modify', 'javascript:promptValueList(' . singleQuote($form_fieldname) . ')', 'Add Item'));
        }
        //Attach javascript for loading if not preloaded items
        if (empty($preloaded)) {
		  $script =  setJavaScriptTag('jQuery(document).ready(function() {DBTL_set_evnt(' . singleQuote($fieldrecord['PFID']) . ',' . singleQuote($form_fieldname) . ')});', false);
		  $pop->addSufix($script);
        }
        return $pop;
    }

    /**
     * Build session key for filters
     */
    function setQuickFilters($tablecode, $fieldname, $fieldrecord, $record) {
        $criteria = '';
        if (array_key_exists($tablecode . '_' . $fieldname, $record)) {
		  if (is_array($record[$tablecode . '_' . $fieldname])) {      //comprueba si la variable es array
			 $criteria = '';
			 $comma = '';
			 foreach ($record[$tablecode . '_' . $fieldname] as $val) {
			     $criteria .= $comma . $tablecode . '.' . $fieldname . ' LIKE "%' . singleQuote($val) . '%"';
			     $comma = ' OR ';
			 }
			 if (strlen($criteria) > CLX_NUM_ZERO) {
			     $criteria = '(' . $criteria . ')';
			 }
		  } else {
			 if (strlen($record[$tablecode . '_' . $fieldname]) > CLX_NUM_ZERO) {
			     $criteria = $tablecode . '.' . $fieldname . ' = ' . singleQuote(str_replace(CLX_CHAR_SINGLE_QUOTE, '', $record[$tablecode . '_' . $fieldname]));
			 }
		  }
        }
        return $criteria;
    }

    function searchFieldObject($form_fieldname, $fieldrecord, $record) {
        $fieldobj = $this->fieldObject($form_fieldname, $fieldrecord, $record);
        $selected = (!isset($record[$form_fieldname]) or ( strlen($record[$form_fieldname]) == CLX_NUM_ZERO));
        $fieldobj->addItem('', '', $selected);
        return $fieldobj;
    }

    function validateField(&$msg, $form_fieldname, $fieldrecord, &$record, &$table, &$form) {
        if (!empty($fieldrecord['FIELD_EXTRA3'] === true) &&
			 is_array($record[$form_fieldname]) && strlen(toString($record[$form_fieldname], CLX_CHAR_SINGLE_QUOTE)) > CLX_NUM_255) {
		  $msg[$form_fieldname] = dict('DBFL_LIST_TOO_MANY_OPTIONS', '255', strlen(toString($record[$form_fieldname], CLX_CHAR_SINGLE_QUOTE)));
        }
        //Setting form require pref if any
        if (UserRequest::$request['ac']!='DELETE' &&array_key_exists('FIELD_FORM_REQUIRED', $fieldrecord) && $fieldrecord['FIELD_FORM_REQUIRED'] != CLX_NUM_TWO) {
		  $fieldrecord['FIELD_REQUIRED'] = $fieldrecord['FIELD_FORM_REQUIRED'];
        }
		
        // Note: !array_key_exists was added to enable lookups to validate
        if (!array_key_exists('FIELD_HIDDEN', $fieldrecord))
		  $fieldrecord['FIELD_HIDDEN'] = false;
        if (empty($fieldrecord['FIELD_HIDDEN']) === true) {
		  if (UserRequest::$request['ac']!='DELETE' && !empty($fieldrecord['FIELD_REQUIRED'])) {
			 xValidate::required($msg, $form_fieldname, $record[$form_fieldname]);
		  }
        }
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {
        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
		  return;
        } else {
		  $something = (strlen($fieldvalue) != CLX_NUM_ZERO);
		  $required = $fieldrecord['FIELD_REQUIRED'];
		  if (empty($something) && !empty($required)) {
			 $msg[$fieldrecord['FIELDNAME']] = dict('GEN_LBL_FIELDREQUIRED');
		  } elseif (!empty($something)) {
			 $DBLT = &getTool('DBLT');
			 if (!$DBLT->listItemExists($fieldrecord['FIELD_EXTRA1'], $fieldvalue)) {
			     //$msg[$fieldrecord['FIELDNAME']] = dict('GEN_LBL_INVALID_LIST_ITEM') . ': ' . $fieldvalue;
			     $msg[$fieldrecord['FIELDNAME']] = dict('GEN_LBL_INVALID_LIST_ITEM') . ': ' . $fieldvalue . ' [' . $fieldrecord['FIELD_EXTRA1'] . ']';
			 }
		  }
        }
    }

    function set_obj_cache(&$fieldrecord) {
        if (empty($this->obj_cache[$fieldrecord["FIELD_EXTRA1"]])) {
		  $DBLT = &getTool('DBLT');
		  if(empty($fieldrecord['DBQF_FIELD_FORMAT'])) $fieldrecord['DBQF_FIELD_FORMAT'] = 0;
		  $this->obj_cache[$fieldrecord["FIELD_EXTRA1"]] = $DBLT->getListItems($fieldrecord['FIELD_EXTRA1'], $fieldrecord['DBQF_FIELD_FORMAT']);
        }
    }

    function formatField($form_fieldname, $fieldrecord, $record) {
        if (!array_key_exists($form_fieldname, $record)) {
		  $record[$form_fieldname] = '';
        }

        if(!empty($fieldrecord['DBQF_FIELD_FORMAT']) &&   $fieldrecord['DBQF_FIELD_FORMAT'] === '1'){
		  if (array_key_exists($form_fieldname, $record)) {
			 $original_value = $record[$form_fieldname];
		  } else {
			 $original_value = '';
		  }
		  return $original_value;
        }

        if (!empty($this->obj_cache) && array_key_exists($fieldrecord['FIELD_EXTRA1'], $this->obj_cache)) {
			IF (ISSET($GLOBALS['ADDRDEBUGUILISTING']) && $GLOBALS['ADDRDEBUGUILISTING']==$_SERVER['REMOTE_ADDR']) {
				PRE('REEMPLAZO DE LISTA POR CACHE INTERNO DE LISTAS DBFT_LIST '.$fieldrecord['FIELD_EXTRA1'].'(('. $record[$form_fieldname].'))' );
			}

            $xvalores=explode(',',$record[$form_fieldname]);
            $xreturn='';
            foreach ($xvalores as $xkey=>$xval){
                $xval=str_replace("'",'',$xval);
				if (isset($this->obj_cache[$fieldrecord['FIELD_EXTRA1']][$xval]))
					$xreturn.=$this->obj_cache[$fieldrecord['FIELD_EXTRA1']][$xval].'<br>';
            }
            IF (!EMPTY($xreturn))
                return $xreturn;
			else if ((!empty($record[$form_fieldname]) OR $record[$form_fieldname]==='0') &&(!empty($fieldrecord['FIELD_EXTRA1']) OR $fieldrecord['FIELD_EXTRA1']==='0') && array_key_exists($record[$form_fieldname], $this->obj_cache[$fieldrecord['FIELD_EXTRA1']])) {
			 return $this->obj_cache[$fieldrecord['FIELD_EXTRA1']][$record[$form_fieldname]];
		  }else{
			 return '';
		  }
        }

        //$form_fieldname = uppercase($form_fieldname);
        //$record = array_change_key_case($record, CASE_UPPER);
        if (strlen($record[$form_fieldname]) > CLX_NUM_ZERO) {
		  $DBLT = &getTool('DBLT');
		  if (!empty($fieldrecord['FIELD_EXTRA3'])) {//multiple
			 $record[$form_fieldname] = toArray($record[$form_fieldname], CLX_CHAR_SINGLE_QUOTE);
		  }
		  if (!array_key_exists('DBQF_FIELD_FORMAT', $fieldrecord))
			 $fieldrecord['DBQF_FIELD_FORMAT'] = CLX_INIT_ZERO;
		  if (!empty($fieldrecord['NOCODES'])) {
			 $fieldrecord['DBQF_FIELD_FORMAT'] = CLX_NUM_TWO; // pintear solo nombre   nombre(codigo)
		  }
		  return $DBLT->getListItem($fieldrecord['FIELD_EXTRA1'], $record[$form_fieldname], $fieldrecord['DBQF_FIELD_FORMAT']);
        } else {
		  return CLX_CHAR_HTML_NBSP;
        }
    }

    function saveField($form_fieldname, $fieldrecord, &$record, $recordid) {
        parent::saveField($form_fieldname, $fieldrecord, $record, $recordid);
        $record[$fieldrecord['FIELDNAME']] = clx_html_spchrs_decode($record[$fieldrecord['FIELDNAME']]);
    }
}
