<?php

/**
 * Primary field object class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 */
if (!class_exists('dbft') && !defined('clxInstallProcess'))
    include_once kPathToObjects . 'dbft.php';

class dbft_primary extends dbft {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'PRIM';
        $this->name = 'Unique Record Identifier';
        $this->type = 'DBFT';
        $this->version = '0.2.3.2';
        $this->description = 'This field creates a  primary, unique index in the table';
        $this->multifield = false; // Describes it field is a multi field see properties;
        $this->has_lookup = true; // Decribes if field can be set as copy field on lookup
        $this->has_question = true; // Describes if field can be used on queries criteria
        $this->operators = array('===' => 'GEN_LBL_EQUALTO',
		  '!== ' => 'GEN_LBL_NOTEQUALTO',
		  'IN' => 'GEN_LBL_ISIN'
        );
		IF (ISSET($GLOBALS['ADDRDEBUGEDIT']) && $GLOBALS['ADDRDEBUGEDIT']==$_SERVER['REMOTE_ADDR']) {
			$this->debug=true;
		}

    }

    function panel($msg, $fieldrecord, &$form) {
        $c = $this->code;
        if ($fieldrecord['DBFL_fieldtype'] != $c) {
		  $this->defaultPanel($fieldrecord);
        }
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_NOPROPERTIES'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $div = new htmlDiv($this->code, 'visibility:hidden; display:none', $table->source());
        return CLX_CHAR_NL . CLX_CHAR_NL . $div->source() . CLX_CHAR_NL . CLX_CHAR_NL;
    }

    function checkPanel(&$msg) {
        $c = $this->code;
        $fieldid = (!empty(UserRequest::$post['FLD_id'])) ? UserRequest::$post['FLD_id'] : '';
        if (!empty(UserRequest::$post['FLD_tablecode'])) {
		  $tableid = ClxCommon::$db->getField('CLX_DB_TABLES', 'ID', 'IDCODE=' . singleQuote(UserRequest::$post['FLD_tablecode']));
            $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_FIELDS', 'TABLEID=' . $tableid . ' AND FIELDTYPE=' . singleQuote('PRI') . ' AND ID<>' . $fieldid);
		  if ($qty != CLX_NUM_ZERO) {
			 $msg['DBFL_fieldtype'] = dict('GEN_LBL_PRIMEXISTS');
		  }
        }
        if (!empty(UserRequest::$post[$c . '_required'])) {
		  xValidate::required($msg, $c . '_default', trim(UserRequest::$post[$c . '_default']));
        }
    }

    function savePanel(&$fieldrecord) {
        $fieldrecord['FIELD_SIZE'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_REQUIRED'] = CLX_NUM_ONE;
        $fieldrecord['FIELD_FORMAT'] = '';
        $fieldrecord['FIELD_ALIGN'] = CLX_INIT_ZERO;
        $fieldrecord['FIELD_DEFAULT'] = '';
        $fieldrecord['FIELD_MINVALUE'] = '';
        $fieldrecord['FIELD_MAXVALUE'] = '';
        $fieldrecord['FIELD_EXTRA1'] = '';
        $fieldrecord['FIELD_EXTRA2'] = '';
        $fieldrecord['FIELD_EXTRA3'] = '';
        $fieldrecord['FIELD_EXTRA4'] = '';
        $fieldrecord['FIELD_EXTRA5'] = '';
    }

    function loadPanel(&$fieldrecord, $values) {
        $this->defaultPanel($fieldrecord);
    }

    function defaultPanel(&$fieldrecord) {
        $c = $this->code;
        $fieldrecord[$c . '_size'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_required'] = CLX_NUM_ONE;
        $fieldrecord[$c . '_format'] = '';
        $fieldrecord[$c . '_align'] = CLX_INIT_ZERO;
        $fieldrecord[$c . '_default'] = '';
        $fieldrecord[$c . '_minvalue'] = '';
        $fieldrecord[$c . '_maxvalue'] = '';
        $fieldrecord[$c . '_extra1'] = '';
        $fieldrecord[$c . '_extra2'] = '';
        $fieldrecord[$c . '_extra3'] = '';
        $fieldrecord[$c . '_extra4'] = '';
        $fieldrecord[$c . '_extra5'] = '';
    }

    function viewForm(&$table, $record) {
        $table->addData('Index:', 'Primary (Unique Record Identifier)');
    }

    function fieldProperties($fieldrecord) {
        $props = array();
        // DB implementation
        switch (kDatabaseType) {
		  case 'oci':
			 $props['type'] = 'NUMBER(11)';
			 break;
		  default:
			 $props['type'] = 'INT(11) UNSIGNED';
			 break;
        }
        $props['index'] = 'PRI';
        if (!array_key_exists('FIELD_DEFAULT', $fieldrecord)) {
		  $props['default'] = 'null';
        } else {
		  if ($fieldrecord['FIELD_DEFAULT'] === 'null' || strlen($fieldrecord['FIELD_DEFAULT']) == CLX_NUM_ZERO) {
			 $props['default'] = 'null';
		  } else {
			 $props['default'] = $fieldrecord['FIELD_DEFAULT'];
		  }
        }
        $props['properties'] = 'auto_increment';
        $props['null'] = 'NO';
        return $props;
    }

    function defaultField($form_fieldname, $fieldrecord, &$record) {
        $record[$form_fieldname] = '';
    }

    function placeEditField($msg, $form_fieldname, $fieldrecord, $record, &$form, $jsevents) {
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($fieldrecord['CAPTION'])) {
		  $table->addCell(dict($fieldrecord['CAPTION']), array(CLX_PROP_CLASS => $fieldrecord['CAPTION_CLASS'], CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        if (!empty($msg[$form_fieldname])) {
		  $table->addCell($msg[$form_fieldname], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $class = ($fieldrecord['FIELD_REQUIRED']) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT;
        $table->addCell(dict($fieldrecord['FIELDFORMLABEL']), array(CLX_PROP_CLASS => $class));
        $table->addCell(dict($fieldrecord['FIELDPREFIX'],true,false)  . ((array_key_exists(uppercase($form_fieldname), $record)) ? $record[uppercase($form_fieldname)] : '') . dict($fieldrecord['FIELDSUFIX'],true,false) , array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        
		if ( $this->debug==true &&!isset($GLOBALS['placelistcount']) && $GLOBALS['placelistcount']<200) {
			$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo4.txt","a+");
			fwrite($tta,chr(13).chr(10).$pos."_".$te[$posx]["function"]."dbft_primary-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();
			for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"]))
					fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));
				
			}

			fwrite($tta,chr(13).chr(10)."placeListFieldText: ".print_r($fieldrecord,true));
			fclose($tta);
			if (!isset($GLOBALS['placelistcount']))
				$GLOBALS['placelistcount']=1;
			else
				$GLOBALS['placelistcount']++;			
		}
		
        return $table;
    }

    function searchFieldObject($form_fieldname, $fieldrecord, $record) {
        if (!array_key_exists($form_fieldname, $record))
		  $record[$form_fieldname] = '';
        $fieldobj = new htmlField($form_fieldname, $record[$form_fieldname], CLX_NUM_48, CLX_NUM_256);
        //Added instead of displaying read only files
        if (array_key_exists('FIELD_READONLY', $fieldrecord) && $fieldrecord['FIELD_READONLY']) {
		  $fieldobj->setProp('readonly', 'true');
		  $fieldobj->setProp(CLX_PROP_CLASS, 'readonly');
        }
        return $fieldobj;
    }

    function validateImportField(&$msg, $fieldrecord, $fieldvalue) {

        if ($fieldrecord['FIELD_REQUIRED'] == CLX_NUM_ZERO && $fieldvalue === 'null') {
		  return;
        } else {
		  xValidate::integer($msg, $fieldrecord['FIELDNAME'], $fieldvalue, CLX_NUM_ZERO, '', $fieldrecord['FIELDLISTLABEL']);
        }
    }

}
