<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:26:29 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10495 $
 *
 * CHANGE LOG
 * 2009/04/06 Included label
 */
class ac_audit extends clx_tool {

    // CONSTRUCTOR //
    var $audit_type;

    function __construct() {
        $this->code = 'ACAD';
        $this->name = 'Audit and Logging';
        $this->system = 'Security';
        $this->version = '0.4.4.0';

        $this->has_access = true;


        $this->tablename = 'CLX_AC_AUDIT';
        $this->record_name_singular = 'audit record';
        $this->record_name_plural = 'audit records';
        $this->primary_field = 'TABLECODE';
        $this->secondary_field = 'IDCODE';

        $this->audit_type = array('LOGIN' => 'GEN_LBL_LOGIN', 'LOGOUT' => 'GEN_LBL_LOGOUT', 'LOAD_REC' => 'GEN_LBL_LOAD_RECORD', 'PRINT_REC' => 'GEN_LBL_PRINT_REC', 'SAVE_REC' => 'GEN_LBL_SAVE_REC', 'CREATE_REC' => 'GEN_LBL_CREATE_REC', 'DEL_REC' => 'GEN_LBL_DEL_REC', 'EXPORT_REC' => 'GEN_LBL_EXPORT_REC', 'ARCH_REC' => 'GEN_LBL_ARCHIVE_RECS', 'TIMEOUT' => 'GEN_LBL_TIMEOUT', 'OTH' => 'GEN_LBL_OTHER');
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SECURITY:MENU_AUDIT', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['RECORDID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['SESSION_IP'] = array('type' => 'varchar(32)', 'null' => 'YES', 'properties' => '');
        $fields['INFORMATION'] = array('type' => 'mediumblob', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'CDATE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'RECORDID');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'MyISAM',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    /**
     * Archive Tool Records
     */
    function toolArchive() {
        if (!defined('UTIL_ARCHIVE_MODE'))
		  define('UTIL_ARCHIVE_MODE', CLX_NUM_12);
        //Archive
        $sql = 'SELECT * FROM ' . $this->tablename;
        if (UTIL_ARCHIVE_MODE == CLX_NUM_ONE) {
		  $archive_tbl = $this->tablename . '_' . getNow('m') . '_' . getNow('Y');
        } else {
		  $archive_tbl = $this->tablename . '_' . getNow('Y');
        }
        $tableinfo = $this->toolTableStructure();
        unset($tableinfo['fields']['ID']);
        $tableinfo['name'] = $archive_tbl;
        $tableinfo['type'] = 'ARCHIVE';
        $tableinfo['indexes'] = array();
        $structure = ClxCommon::$db->translateTableStructure($tableinfo);
        //Create table if not exists
        if (!ClxCommon::$db->tableExists($structure['name'])) {
		  //Create table if not exists
		  ClxCommon::$db->createTable($structure['name'], $structure['type'], $structure['fields'], $structure['case_sensitive']);
        }
        $comma = '';
        $select = '';
        foreach ($structure['fields'] as $key => $val) {
		  $select .= $comma . $key;
		  $comma = ',';
        }
        //Lock table to archive
        $sql = 'SELECT MAX(ID) FROM ' . $this->tablename;
        $maxid = ClxCommon::$db->getOne($sql);
        if (empty($maxid))
		  return;
        //Backup
        $sql = 'INSERT INTO ' . $structure['name'] . '(' . $select . ') SELECT ' . $select . ' FROM ' . $this->tablename . ' WHERE ID <=' . $maxid;
        ClxCommon::$db->executeQuery($sql);
        //Unlock table
        $sql = 'DELETE FROM ' . $this->tablename . ' WHERE ID<=' . $maxid;
        ClxCommon::$db->executeQuery($sql);
        //Restore stats if apply
        ClxCommon::$db->buldStats($this->tablename);
    }

    function toolArchive_batch($parameters) {
        $contenidoHTML = '';
//e cho 'TOOLARCHIVE_BATCH parameters:<br>'.CLX_CHAR_NL; print_r($parameters);
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Method 'get_bkp_time_limit' implemented to avoid schedule CLX_ACAD_CLEAN
        // to remain un 'In progress(RUN) status due to PHP timeout.
        $DBBK = getToolObject('UTIL', 'DBBK');
        // Set it to twice the time limit of backup, just in case
        $bkp_time_limit = $DBBK->get_bkp_time_limit();
        UserResponse::setRunTimeLimit($bkp_time_limit * CLX_NUM_TWO * CLX_NUM_60 * CLX_NUM_60);
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Se agrega este método para realizar el archival en batch,
        // ya que en los casos que CLX_AC_AUDIT tenga muchos registros,
        // se cae cuando se hace en el interactivo.
        // Realiza la llamada al método $this->toolArchive()
        $this->toolArchive();
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Hace commit para que no de error al hacer respaldo de los binarios
        ClxCommon::$db->commit();
        // Extrae la lista de las tablas con los archives existentes
        //   - Caso 1002670 - BNP-Limpieza Proc Log en batch
        // Fix size of SUBSTR with calculated strlen($this->tablename)+1
        // Substitutes table name prefix with $this->tablename
        /*
         * Para saber los nombres de los Archives de Audit en la base de datos corriente
         */
        $sql = 'SELECT TABLE_NAME FROM INFORMATION_SCHEMA.TABLES WHERE SUBSTR(TABLE_NAME, 1, ' . (strlen($this->tablename) + CLX_NUM_ONE) . ') = ' . singleQuote($this->tablename . '_') . ' AND TABLE_SCHEMA = ' . singleQuote(kDatabaseName);
        $results = ClxCommon::$db->executeQuery($sql);
        if (defined('ACAD_BIG_TABLE_REC_LIMIT')) {
		  $big_table_record_limit = ACAD_BIG_TABLE_REC_LIMIT;
        } else {
		  // Default value: 5000 records to consider to backup ISAM binary file instead
		  $big_table_record_limit = '5000';
        }
        $big_table_record_limit = CLX_NUM_ZERO + $big_table_record_limit;
        $big_tables = false;
        $tables = array();
        while (($rec = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE))) {
		  $tables[] = $rec['TABLE_NAME'];
            if (ClxCommon::$db->oneExistsRecords($rec['TABLE_NAME']) > $big_table_record_limit) {
			 $big_tables = true;
		  }
        }
		ClxCommon::$db->freeResult($results);
        //   - Caso 1002598 - BNP-Limpieza Audit en Batch
        // Hace 2do commit para asegurar que no de error luego de estos ultimos SELECT al hacer respaldo de los binarios
        ClxCommon::$db->commit();
        // Verifica su hay archives
        if (count($tables) > CLX_NUM_ZERO) {
		  // List of archive tables to backup (separated by '/')
		  $parameters['TABLES'] = implode('/', $tables);
		  // Eventually adds a suffix to backup file name
		  // to avoid Recycler to delete those bkp files
		  // Le agrega lo necesario para que el respaldo se conserve del reciclaje
		  if (defined('ACAD_ARC_NAME_SUFFIX')) {
			 $archive_name_suffix = ACAD_ARC_NAME_SUFFIX;
		  } else {
			 $archive_name_suffix = '';
		  }
		  if ($archive_name_suffix === '' || $archive_name_suffix === 'null' || is_null($archive_name_suffix)
		  ) {
			 $archive_name_suffix = '';
		  }
		  // Backup file name
		  $parameters['NAME_SUFFIX'] = '_' . $this->tablename . $archive_name_suffix;
		  // No Drop Table option
		  $parameters['DUMP_OPTIONS'] = '--skip-add-drop-table';
		  // Local Backup Script optional parameters:
		  $parameters['LBS_OPT_PAR'] = '-ncss 0';
		  if ($big_tables === true) {
			 $parameters['LBS_OPT_PAR'] .= ' -rmode LOC_DB_BKP_BINDAT';
		  }
		  $contenidoHTML.= '<br>' .
			     'Calling backup procedure<br>' .
			     'Tables: ' . str_replace('/', ', ', $parameters['TABLES']) . '<br>' .
			     'File name suffix: ' . $parameters['NAME_SUFFIX'] . '<br>';
		  // Llama al respaldo pasandole como parametro los nombres de los archives
		  $results = $DBBK->run($parameters);
		  if ($results === '') {
			 // Si el respaldo fue existoso, elimina los archives de la BDD
			 $contenidoHTML.= '<br>';
			 foreach ($tables as $key => $val) {
			     $contenidoHTML.= 'Dropping Table: ' . $val . '<br>';
			     $sql = 'DROP TABLE IF EXISTS ' . $val;
			     ClxCommon::$db->executeQuery($sql);
			 }
		  } else {
			 $contenidoHTML.= '<br>' .
			         'ERROR: excecuting backup<br>' .
			         $results . '<br>';
		  }
        }
        echo $contenidoHTML;
    }

//--  A U D I T
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AC_AUDIT-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ACADList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AC_AUDIT';
        $whereclause = '';
        $sortclause = 'CDATE DESC';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_NINE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACAD_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_NINE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_DATE'), dict('GEN_LBL_USER'), dict('GEN_LBL_ACTION'), dict('GEN_LBL_CODE'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_RECORD'), dict('GEN_LBL_SESSION_IP')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(xFormat::datetime($record['CDATE']), $record['USERNAME'], $record['IDNAME'], $record['IDCODE'], $record['MODULECODE'], $record['TABLECODE'], $record['RECORDID'], $record['SESSION_IP']));
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXPORT'), array('action' => 'EXPORT', 'form' => $form->getProp(CLX_PROP_ID)))->source();

        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $b = new htmlFormSubmit(dict('GEN_LBL_TOOL_ARCHIVE'), 'TOOL_ARCHIVE', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ON');
        $buttons .= $b->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ACAD_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_USERNAME') . ':', xFormat::datetime($record['ACAD_username']));
        $table->addData(dict('GEN_LBL_DATETIME') . ':', xFormat::datetime($record['ACAD_cdate']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ACAD_modulecode']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ACAD_tablecode']);
        $table->addData(dict('GEN_LBL_RECORDID') . ':', $record['ACAD_recordid']);
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ACAD_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ACAD_idname']);
        $table->addData(dict('GEN_LBL_SESSION_IP') . ':', $record['ACAD_session_ip']);
        $contents .= $table->source();
        if (!empty($record['ACAD_information'])) {
		  $contents .= buildPanelInterface(dict('GEN_LBL_DATA'), 'AUDITDATA', html_entity_decode($record['ACAD_information']), true);
        }
        $contents .= '<br>';
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        //$from = adodb_month_lastday(xFormat::getNow('Y-m-d'),'Y-m-d');
        if (!array_key_exists('ACAD_modulecode', UserRequest::$post))
		  UserRequest::$post['ACAD_modulecode'] = '';
        if (!array_key_exists('ACAD_tablecode', UserRequest::$post))
		  UserRequest::$post['ACAD_tablecode'] = '';
        if (!array_key_exists('ACAD_username', UserRequest::$post))
		  UserRequest::$post['ACAD_username'] = '';
        if (!array_key_exists('ACAD_from_date', UserRequest::$post))
		  UserRequest::$post['ACAD_from_date'] = '';
        if (!array_key_exists('ACAD_to_date', UserRequest::$post))
		  UserRequest::$post['ACAD_to_date'] = '';
        if (!array_key_exists('ACAD_session_ip', UserRequest::$post))
		  UserRequest::$post['ACAD_session_ip'] = '';
        if (!array_key_exists('ACAD_action', UserRequest::$post))
		  UserRequest::$post['ACAD_action'] = '';
        $table->addLookup($msg, dict('GEN_LBL_USERNAME') . ':', 'ACAD_username', UserRequest::$post['ACAD_username'], 'ACUS', CLX_NUM_30, CLX_NUM_30);
        $table->addDate($msg, dict('GEN_LBL_FROM') . ':', 'ACAD_from_date', UserRequest::$post['ACAD_from_date'], CLX_NUM_16, CLX_NUM_16, false, true);
        $table->addDate($msg, dict('GEN_LBL_TO') . ':', 'ACAD_to_date', UserRequest::$post['ACAD_to_date'], CLX_NUM_16, CLX_NUM_16, false, true);
        $action = new HtmlSelect('ACAD_action');
        $action->addItem('', '', true);
        foreach ($this->audit_type as $key => $val) {
		  $action->addItem(dict($val, true), $key, (UserRequest::$post['ACAD_action'] == $key));
        }
        $table->addData(dict('GEN_LBL_ACTION') . ':', $action->source());
        unset($action);
        $table->addSeparator('');
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ACAD_modulecode', UserRequest::$post['ACAD_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'ACAD_tablecode', UserRequest::$post['ACAD_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_SESSION_IP') . ':', 'ACAD_session_ip', UserRequest::$post['ACAD_session_ip'], CLX_NUM_32, CLX_NUM_32);
        $table->addSeparator('');
        $sort = new HtmlSelect('ACAD_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $sort->addItem(dict('GEN_LBL_TABLECODE'), 'tablecode', false);
        $sort->addItem(dict('GEN_LBL_USERNAME'), 'username', false);
        $sort->addItem('Date (Asc.)', 'cdate ASC', false);
        $sort->addItem('Date (Desc.)', 'cdate DESC', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ACAD_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACAD_modulecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_tablecode'])) {
		  $filter .= $boolean . ' UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ACAD_tablecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_username'])) {
		  $filter .= $boolean . ' UPPER(USERNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ACAD_username'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_from_date'])) {
		  $filter .= $boolean . ' CDATE >= ' . singleQuote(UserRequest::$post['ACAD_from_date']);
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_to_date'])) {
		  $filter .= $boolean . ' CDATE <= ' . singleQuote(UserRequest::$post['ACAD_to_date']);
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_session_ip'])) {
		  $filter .= $boolean . ' SESSION_IP LIKE ' . singleQuote(UserRequest::$post['ACAD_session_ip']);
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ACAD_action'])) {
		  $filter .= $boolean . ' IDCODE = ' . singleQuote(UserRequest::$post['ACAD_action']);
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ACADList']['sortby'] = UserRequest::$post['ACAD_sort'];
        $_SESSION['paging']['ACADList']['criteria'] = $filter;
        $_SESSION['paging']['ACADList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ACADList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_AC_AUDIT', 'ID=' . $recordid);
        $record = array();
        $record['ACAD_id'] = $recordid;
        $record['ACAD_cdate'] = xFormat::date($values['CDATE']);
        $record['ACAD_modulecode'] = $values['MODULECODE'];
        $record['ACAD_tablecode'] = $values['TABLECODE'];
        $record['ACAD_recordid'] = $values['RECORDID'];
        $record['ACAD_username'] = $values['USERNAME'];
        $record['ACAD_idcode'] = $values['IDCODE'];
        $record['ACAD_idname'] = $values['IDNAME'];
        $record['ACAD_information'] = $values['INFORMATION'];
        $record['ACAD_session_ip'] = $values['SESSION_IP'];
        return $record;
    }

    /**
     * Stores main session audit trails
     */
    function log_main_request() {
        //Edit handled individualy
        if (!empty(UserRequest::$request[CLX_PROP_TO]) && UserRequest::$request[CLX_PROP_TO] === 'EDIT')
		  return;
        if (!empty(UserRequest::$request['ac']) && UserRequest::$request['ac'] === 'LOGIN')
		  return;
        $tablerecord = array('ID' => '', 'MODULECODE' => 'CLX', 'IDCODE' => UserRequest::$request[CLX_PROP_TO]);
        $recordid = CLX_INIT_ZERO;
        $idcode = 'OTH';
        if (!empty(UserRequest::$post['ac'])) {
		  switch (UserRequest::$post['ac']) {
			 case 'MODIFY':
			     $idcode = 'SAVE_REC';
			     break;
			 case 'DELETE':
			     $idcode = 'DEL_REC';
			     break;
			 case 'CREATE':
			     $idcode = 'CREATE_REC';
			     break;
			 case 'DUPLICATE':
			     $idcode = 'CREATE_REC';
			     break;
			 default:
			     $idcode = 'LOAD_REC';
			     break;
		  }
        }
        $idname = 'Main Request';
        $data = array_merge(UserRequest::$post, UserRequest::$request, UserRequest::$get);
        unset($data['PHPSESSID']);
        $this->log($tablerecord, $recordid, $idcode, $idname, $data);
    }

    function log($tablerecord, $recordid, $idcode, $idname, $data) {
        $audit = false;
        if (!empty($tablerecord)) {
		  if (is_array($tablerecord)) {
			 $tablecode = $tablerecord['IDCODE'];
		  } else {
			 // In this case $tablerecord is the idcode of the table
			 $tablerecord = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($tablerecord));
			 $tablecode = $tablerecord['IDCODE'];
		  }
		  $modulecode = $tablerecord['MODULECODE'];
		  $modulerecord = ClxCommon::$db->getRecord('CLX_DB_MODULES', 'IDCODE=' . singleQuote($modulecode));

		  if (isset($modulerecord['AUDITU']) && $modulerecord['AUDITU']) {
			 $audit = true;
		  } elseif (isset($tablerecord['AUDITU']) && $tablerecord['AUDITU']) {
			 $audit = true;
		  }
        } else {
		  $tablecode = '';
		  $modulecode = '';
		  $recordid = CLX_INIT_ZERO;
        }
        if (empty($audit)) {
		  if (array_key_exists('user_record', $_SESSION) && !empty($_SESSION['user_record']['auditu']) && $_SESSION['user_record']['auditu'] === true) {
			 $audit = true;
		  } elseif (array_key_exists('user_record', $_SESSION)) {
			 $ACTO = &getTool('ACTO');
			 $list = $ACTO->userGroups($_SESSION['user_record']['username']);
			 if (count($list) > CLX_NUM_ZERO) {
			     $list = toString($list);
			     //Verify was ID IN instead of IDCODE IN
                    $audit = (ClxCommon::$db->oneExistsRecords('CLX_AC_GROUPS', 'AUDITU=1 AND IDCODE IN (' . $list . ')') > CLX_NUM_ZERO);
			 }
		  }
        }
        if (!empty($audit)) {
		  $record = array();
		  $record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
		  $record['MODULECODE'] = $modulecode;
		  $record['TABLECODE'] = $tablecode;
		  $record['RECORDID'] = $recordid;
		  $record['USERNAME'] = (array_key_exists('username', $_SESSION['user_record'])) ? $_SESSION['user_record']['username'] : '';
		  $record['IDCODE'] = $idcode;
		  if (!array_key_exists($idcode, $this->audit_type)) {
			 $record['IDNAME'] = $idcode;
		  } else {
			 $record['IDNAME'] = dict($this->audit_type[$idcode], true);
			 if (strlen($idname) > CLX_NUM_ZERO)
			     $record['IDNAME'] .=': ' . $idname;
		  }
		  $record['IDNAME'] = substr($record['IDNAME'], CLX_NUM_ZERO, CLX_NUM_255);
		  //$data = htmlentities(print_r($data,true));
		  //$data = xDebug::dump($data);
		  $data = str_replace(CLX_CHAR_SINGLE_QUOTE, '\\\'', xDebug::dump($data));
		  ClxCommon::$db->addslashes($data);
		  $record['INFORMATION'] = $data;
		  $record['SESSION_IP'] = ClxSession::getSessionIp();
		  $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
		  unset($id);
		  ClxCommon::$db->commit();
        }
    }

}
