<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:19:44 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10483 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class an_questionary extends clx_tool {

    var $next_question;
    var $test; //Testing Questionary Flag, if true no response will be saved on result table
    var $parent_table; //Parent table, associated record to questionary answer
    var $parent_id; //Parent table record ID
    var $is_telemarketing = false; //set to true will be running as a telemarketing quest.
    var $end_questionary = false; //set to true if last question was the end of questionary
    var $update_parent_at_end = false;
    var $runid = CLX_INIT_ZERO; //result table id or runid
    var $mark_camp_tele_code = '';
    var $camp_target_id = CLX_INIT_ZERO;

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANQN';
        $this->name = 'Questionary Manager';
        $this->system = 'Interface';
        $this->version = '0.1.4.0';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_AN_QUESTIONARIES';
        $this->record_name_singular = 'questionary';
        $this->record_name_plural = 'questionaries';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->next_question = CLX_NUM_ONE;
        $this->test = false;
        $this->parent_table = '';
        $this->parent_id = '';
        $this->is_telemarketing = (!empty(UserRequest::$request[CLX_PROP_TO]) && UserRequest::$request[CLX_PROP_TO] === 'MATE');
        $this->end_questionary = false;
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_ANALYTICS:MENU_QUESTIONARIES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['RESULT_TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['RESULT_FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['RESULT_FLDVALUE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'RESULT_TABLECODE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'ANQQ', 'SELECT * FROM CLX_AN_QUESTIONARY_QUESTS WHERE QUESTIONARYCODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $idcode = $record['IDCODE'];
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode)) > CLX_NUM_ZERO);
        if ($exists === true) {
		  $id = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($idcode));
		  $this->deleteRecordSk($id);
        }
        $id = ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o));
		  }
        }
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'EXECUTE_TEST':
			     $contents = $this->executeTest();
			     break;
			 case 'EXECUTE_QUESTIONARY':
			     $contents = $this->execute_questionary();
			     break;
			 case 'QUESTIONARY_DIALOG':
			     $tablecode = UserRequest::$request['tc'];
			     $recs = UserRequest::$request['recs'];
			     $contents = $this->questionary_dialog($tablecode, $recs);
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [AN_QUESTIONARY-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'ANQNList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_AN_QUESTIONARIES';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_EIGHT);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANQN_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_EIGHT));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_TABLECODE'), dict('GEN_LBL_RESULT_TABLECODE'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_VERSION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ')');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], xFormat::text($record['DESCRIPTION']), $record['TABLECODE'], $record['RESULT_TABLECODE'], $record['MODULECODE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER']));
		  $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_QUESTIONARY')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXECUTE_TEST'), array('action' => 'EXECUTE_TEST', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $contents = '';
        $msg = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANQN_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANQN_idcode', (!array_key_exists('ANQN_idcode', UserRequest::$post)) ? '' : UserRequest::$post['ANQN_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANQN_idname', (!array_key_exists('ANQN_idname', UserRequest::$post)) ? '' : UserRequest::$post['ANQN_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_WORKSONTABLECODE') . ':', 'ANQN_tablecode', (!array_key_exists('ANQN_tablecode', UserRequest::$post)) ? '' : UserRequest::$post['ANQN_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_RESULT_TABLECODE') . ':', 'ANQN_result_tablecode', (!array_key_exists('ANQN_result_tablecode', UserRequest::$post)) ? '' : UserRequest::$post['ANQN_result_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANQN_modulecode', (!array_key_exists('ANQN_modulecode', UserRequest::$post)) ? '' : UserRequest::$post['ANQN_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['ANQN_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_idcode']) . '%');
		  }
		  if (UserRequest::$post['ANQN_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ANQN_idname']) ). '%');
		  }
		  if (UserRequest::$post['ANQN_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_modulecode']) . '%');
		  }
		  if (UserRequest::$post['ANQN_tablecode'] <> '') {
			 $criteria .= 'UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_tablecode']) . '%');
		  }
		  if (UserRequest::$post['ANQN_result_tablecode'] <> '') {
			 $criteria .= 'RESULT_TABLECODE LIKE ' . singleQuote(UserRequest::$post['ANQN_result_tablecode'] . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=questionary></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANQN_TITLE_MODIFY') : dict('TOOL_ANQN_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANQN_id', $record['ANQN_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANQN_idcode']);
		  $form->addArgument('ANQN_idcode', $record['ANQN_idcode']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANQN_idcode', $record['ANQN_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANQN_idname', $record['ANQN_idname'], CLX_NUM_48, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_VERSION') . ':', 'ANQN_version', $record['ANQN_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION') . ':', 'ANQN_description', $record['ANQN_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $DBTB = &getTool('DBTB');
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('TOOL_ANQN_PROMPT_MODULECODE') . ':', 'ANQN_modulecode', $record['ANQN_modulecode'], $table);
        if (!empty($modify)) {
		  $table->addData(dict('TOOL_ANQN_EXPLAIN_TABLECODE') . ':', $record['ANQN_result_tablecode']);
		  $form->addArgument('ANQN_result_tablecode', $record['ANQN_result_tablecode']);
        } else {
		  $table->addCell(dict('TOOL_ANQN_EXPLAIN_TABLECODE'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $DBTB->placeChooseTable($msg, dict('GEN_LBL_RESULT_TABLECODE') . ':', 'ANQN_result_tablecode', $record['ANQN_result_tablecode'], $table);
        }
        $DBTF = &getTool('DBTF');
        $table->addCell(dict('TOOL_ANQN_RESULTFIELDNAME'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_WORKSONTABLECODE') . ':', 'ANQN_tablecode', $record['ANQN_tablecode'], $table);
        $DBTF->placeChooseTableField($msg, 'ANQN_tablecode', dict('GEN_LBL_FIELDNAME'), 'ANQN_result_fieldname', $record['ANQN_result_fieldname'], $table);
        $table->addField($msg, dict('GEN_LBL_RESULTVALUE') . ':', 'ANQN_result_fldvalue', $record['ANQN_result_fldvalue'], CLX_NUM_48, CLX_NUM_64);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        xValidate::sqlobject($msg, 'ANQN_idcode', UserRequest::$post['ANQN_idcode']);
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post['ANQN_modulecode'])) {
		  $msg['ANQN_modulecode'] = dict('TOOL_DBMO_ERR_NO_RECORD');
        }
        if ($action !== 'MODIFY') {
		  xValidate::required($msg, 'ANQN_idcode', UserRequest::$post['ANQN_idcode']);
		  xValidate::unique($msg, 'ANQN_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['ANQN_idcode']));
        }
        //Testing
        $DBTB = &getTool('DBTB');
        if (!empty(UserRequest::$post['ANQN_tablecode']) && !$DBTB->tableExists(UserRequest::$post['ANQN_tablecode'])) {
		  $msg['ANQN_tablecode'] = dict('TOOL_UILG_ERR_NOTABLECODE');
        }
        if ($action !== 'MODIFY') {
            $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_TABLES', 'IDCODE=' . singleQuote(UserRequest::$post['ANQN_result_tablecode']));
		  if ($qty > CLX_NUM_ZERO) {
			 $msg['ANQN_result_tablecode'] = dict('GEN_ERR_UNIQUE_KEY', UserRequest::$post['ANQN_result_tablecode']);
		  }
		  xValidate::sqlobject($msg, 'ANQN_result_tablecode', UserRequest::$post['ANQN_result_tablecode']);
        }
        //Check if table Exists
        if ($action !== 'MODIFY') {
            $qty = ClxCommon::$db->oneExistsRecords('CLX_DB_TABLES', 'IDNAME=' . singleQuote(UserRequest::$post['ANQN_idname']) . ' AND MODULECODE=' . singleQuote(UserRequest::$post['ANQN_modulecode']));
        } else {
		  $qty = CLX_INIT_ZERO;
        }
        if ($qty > CLX_NUM_ZERO) {
		  $msg['ANQN_idname'] = dict('GEN_ERR_TABLE_NOT_UNIQUE', UserRequest::$post['ANQN_idname']);
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANQN_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANQN_idname']);
    }

    function viewFormFill($record) {
        addHistory('questionarycode', $record['ANQN_idcode']);
        $contents = '';

        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_ANQN_TITLE_VIEW') . ': ' . $record['ANQN_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['ANQN_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['ANQN_idname']);
        $table->addData(dict('GEN_LBL_VERSION') . ':', $record['ANQN_version'] . '-' . $record['ANQN_version_counter']);
        $table->addData(dict('GEN_LBL_DESCRIPTION') . ':', xFormat::text($record['ANQN_description']));
        $table->addData(dict('GEN_LBL_MODULECODE') . ':', $record['ANQN_modulecode']);
        $table->addData(dict('GEN_LBL_WORKSONTABLECODE') . ':', $record['ANQN_tablecode']);
        $table->addData(dict('GEN_LBL_RESULT_TABLECODE') . ':', $record['ANQN_result_tablecode']);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANQN_result_fieldname']);
        $table->addData(dict('GEN_LBL_VALUE') . ':', $record['ANQN_result_fldvalue']);
        $contents .= $table->source();
        $ANQQ = &getTool('ANQQ');
        $contents .= $ANQQ->makepanel($record['ANQN_idcode']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['ANQN_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['ANQN_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['ANQN_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('ANQN_idcode', UserRequest::$post))
		  UserRequest::$post['ANQN_idcode'] = '';
        if (!array_key_exists('ANQN_idname', UserRequest::$post))
		  UserRequest::$post['ANQN_idname'] = '';
        if (!array_key_exists('ANQN_modulecode', UserRequest::$post))
		  UserRequest::$post['ANQN_modulecode'] = '';
        if (!array_key_exists('ANQN_result_tablecode', UserRequest::$post))
		  UserRequest::$post['ANQN_result_tablecode'] = '';
        if (!array_key_exists('ANQN_tablecode', UserRequest::$post))
		  UserRequest::$post['ANQN_tablecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'ANQN_idcode', UserRequest::$post['ANQN_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'ANQN_idname', UserRequest::$post['ANQN_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'ANQN_modulecode', UserRequest::$post['ANQN_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_WORKSONTABLECODE') . ':', 'ANQN_tablecode', UserRequest::$post['ANQN_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_RESULT_TABLECODE') . ':', 'ANQN_result_tablecode', UserRequest::$post['ANQN_result_tablecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('ANQN_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'modulecode', false);
        $sort->addItem(dict('GEN_LBL_RESULT_TABLECODE'), 'result_tablecode', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'idcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['ANQN_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANQN_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['ANQN_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANQN_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_modulecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANQN_tablecode'])) {
		  $filter .= $boolean . ' UPPER(TABLECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['ANQN_tablecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['ANQN_result_tablecode'])) {
		  $filter .= $boolean . ' RESULT_TABLECODE LIKE ' . singleQuote(UserRequest::$post['ANQN_result_tablecode'] . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['ANQNList']['sortby'] = UserRequest::$post['ANQN_sort'];
        $_SESSION['paging']['ANQNList']['criteria'] = $filter;
        $_SESSION['paging']['ANQNList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['ANQNList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANQN_id'] = $recordid;
        $record['ANQN_idcode'] = $values['IDCODE'];
        $record['ANQN_idname'] = $values['IDNAME'];
        $record['ANQN_version'] = $values['VERSION'];
        $record['ANQN_version_counter'] = $values['VERSION_COUNTER'];
        $record['ANQN_cdate'] = xFormat::date($values['CDATE']);
        $record['ANQN_mdate'] = xFormat::date($values['MDATE']);
        $record['ANQN_username'] = $values['USERNAME'];
        $record['ANQN_description'] = $values['DESCRIPTION'];
        $record['ANQN_tablecode'] = $values['TABLECODE'];
        $record['ANQN_result_tablecode'] = $values['RESULT_TABLECODE'];
        $record['ANQN_result_fieldname'] = $values['RESULT_FIELDNAME'];
        $record['ANQN_result_fldvalue'] = $values['RESULT_FLDVALUE'];
        $record['ANQN_modulecode'] = $values['MODULECODE'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['IDCODE'] = UserRequest::$post['ANQN_idcode'];
        $record['IDNAME'] = UserRequest::$post['ANQN_idname'];
        $record['VERSION'] = UserRequest::$post['ANQN_version'];
        $record['MDATE'] = xFormat::getNow('Y-m-d');
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANQN_description'];
        $record['TABLECODE'] = UserRequest::$post['ANQN_tablecode'];
        $record['RESULT_TABLECODE'] = UserRequest::$post['ANQN_result_tablecode'];
        $record['RESULT_FIELDNAME'] = UserRequest::$post['ANQN_result_fieldname'];
        $record['RESULT_FLDVALUE'] = UserRequest::$post['ANQN_result_fldvalue'];
        $record['MODULECODE'] = UserRequest::$post['ANQN_modulecode'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANQN_id']);
        $this->createTable($record);
        $this->versionIncrease('ID', UserRequest::$post['ANQN_id']);
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = uppercase(UserRequest::$post['ANQN_idcode']);
        $record['IDNAME'] = UserRequest::$post['ANQN_idname'];
        $record['VERSION'] = UserRequest::$post['ANQN_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['ANQN_description'];
        $record['TABLECODE'] = UserRequest::$post['ANQN_tablecode'];
        $record['RESULT_TABLECODE'] = UserRequest::$post['ANQN_result_tablecode'];
        $record['RESULT_FIELDNAME'] = UserRequest::$post['ANQN_result_fieldname'];
        $record['RESULT_FLDVALUE'] = UserRequest::$post['ANQN_result_fldvalue'];
        $record['MODULECODE'] = UserRequest::$post['ANQN_modulecode'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->createTable($record);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        return $recordid;
    }

    function duplicateRecord() {
        $questid = $this->createRecord();
        $ANQQ = getTool('ANQQ');
        $questcode = UserRequest::$post['ANQN_idcode'];
        $originalquestid = UserRequest::$post['ANQN_id'];
        $originalquestrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originalquestid);
        $originalquestcode = $originalquestrec['IDCODE'];
        $questions = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($originalquestcode));
        while (($question = ClxCommon::$db->getNextRecord($questions))) {
		  $oldquestionid = $question['ID'];
		  unset($question['ID']);
		  $question['QUESTIONARYCODE'] = $questcode;
		  $questionid = ClxCommon::$db->createRecord('CLX_AN_QUESTIONARY_QUESTS', $question, false);
		  $questionopts = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . singleQuote($oldquestionid));
		  while (($questionopt = ClxCommon::$db->getNextRecord($questionopts))) {
			 unset($questionopt['ID']);
			 $questionopt['QUESTIONID'] = $questionid;
			 ClxCommon::$db->createRecord('CLX_AN_QUESTIONARY_OPTIONS', $questionopt, false);
		  }
		  //Executing Field creation
		  $question['ID'] = $oldquestionid;
		  $ANQQ->createQuestTableField($question);
        }
        return $questid;
    }

    function deleteRecordSk($id) {
        $questrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $questcode = $questrec['IDCODE'];
        $questions = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($questcode));
        while (($question = ClxCommon::$db->getNextRecord($questions))) {
		  ClxCommon::$db->deleteRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . singleQuote($question['ID']));
        }
        ClxCommon::$db->deleteRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($questcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . singleQuote($id));
        $DBTB = &getTool('DBTB');
        $tableid = ClxCommon::$db->getField($DBTB->tablename, 'ID', 'IDCODE = ' . singleQuote($questrec['RESULT_TABLECODE']));
        if (!empty($tableid)) {
		  $DBTB->deleteRecordSk($tableid);
        }
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $questrec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['ANQN_id'] = CLX_INIT_ZERO;
        $record['ANQN_idcode'] = '';
        $record['ANQN_idname'] = '';
        $record['ANQN_version'] = '1.0.0';
        $record['ANQN_cdate'] = '';
        $record['ANQN_mdate'] = '';
        $record['ANQN_username'] = $_SESSION['user_record']['username'];
        $record['ANQN_description'] = '';
        $record['ANQN_tablecode'] = '';
        $record['ANQN_result_tablecode'] = '';
        $record['ANQN_result_fieldname'] = '';
        $record['ANQN_result_fldvalue'] = '';
        $record['ANQN_modulecode'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    // DEPRECATED in favor of chooseRecord($name, $default)
    function placeChooseQuestionary($msg, $prompt, $name, $default, &$table) {
        if (!empty($msg[$name]))
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($this->chooseRecord($name, $default), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function questionaryExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, $this->primary_field, $this->primary_field . '=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    /**
     * Executing Test.
     */
    function executeTest() {
        $idlist = getIdlist();
        //$this->test = true;//Setting Test Flag
        if (!empty($idlist[CLX_NUM_ZERO])) {
		  $contents = $this->executeQuestinary($idlist[CLX_NUM_ZERO]);
        } else {
		  $contents = dict('GEN_ERR_INVALIDQUEST');
        }
        return $contents;
    }

    function questionary_dialog($tablecode, $recs) {
        //Search Record to use as filter
        $datarec = ClxCommon::$db->getRecord(tableName($tablecode), 'ID = ' . singleQuote($recs));
        $contents = '';
        $form = new htmlForm(CLX_PROP_ACTION, 'pop.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'EXECUTE_QUESTIONARY');
        $form->addArgument('confirm', '1');
        $form->addArgument('tc', $tablecode);
        $form->addArgument('recs', $recs);
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_ALIGN => 'center'));
        $table->addCell(dict('TOOL_ANQN_CHOOSE_QUESTIONARY'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $table->addCell(dict('TOOL_ANQN_CHOOSE_QUESTIONARY_EXPLAIN'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SELECT'), dict('GEN_LBL_NAME'), dict('GEN_LBL_CODE')));
        $i = CLX_INIT_ZERO;
        $counter = CLX_INIT_ZERO;
        //Check the ones that already have
        $records = ClxCommon::$db->getRecords($this->tablename, 'TABLECODE=' . singleQuote($tablecode), 'RESULT_FIELDNAME<>' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' ORDER BY IDNAME'); //   - Caso 1001929 - MYSQL>ORA: PARA LOS STRING SE DEBE USAR COMILLA SIMPLE
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  if (!array_key_exists($record['RESULT_FIELDNAME'], $datarec) || stristr($datarec[$record['RESULT_FIELDNAME']], $record['RESULT_FLDVALUE']) === false) {
			 continue;
		  }
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['IDCODE']));
		  $i++;
		  $counter++;
        }
        if ($counter == CLX_NUM_ZERO) {
		  //Other Entries with no criteria
		  $records = ClxCommon::$db->getRecords($this->tablename, 'TABLECODE=' . singleQuote($tablecode), 'RESULT_FIELDNAME=' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR RESULT_FIELDNAME IS NULL ORDER BY IDNAME'); //   - Caso 1001929 - MYSQL>ORA: PARA LOS STRING SE DEBE USAR COMILLA SIMPLE
		  while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDNAME'], $record['IDCODE']));
			 $i++;
		  }
        }
        if ($i > CLX_NUM_ZERO) {
		  $c = new htmlSubmit(dict('GEN_LBL_EXECUTE'), 'EXECUTE');
		  $buttons = $c->source();
        }
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $form->setContents($table->source());
        $contents = $form->source();
        return $contents;
    }

    /**
     * Executing Test.
     */
    function execute_questionary() {
        $idlist = getIdlist();
        if (!empty($idlist[CLX_NUM_ZERO])) {
		  $contents = $this->executeQuestinary($idlist[CLX_NUM_ZERO]);
        } else {
		  $contents = dict('GEN_ERR_INVALIDQUEST');
        }
        return $contents;
    }

    function resultRecordExists($result_table, $result_id) {
        return (!empty($result_id)) ? ClxCommon::$db->oneExistsRecords(tableName($result_table), 'ID=' . $result_id) : false;
    }

    /**
     * Build Question
     * Build Question Form According Sequnce number
     * @param: (string) $questionary_code
     * @param: (integer) $sequence number
     */
    function executeQuestinary($recordid) {
        $anqn_rec = ClxCommon::$db->getRecord($this->tablename, 'ID = ' . singleQuote($recordid));


        //test if any table problem
        ClxFreeReturn($this->checkResultTable($anqn_rec));

        if (ClxCommon::$db->oneExistsRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE'])) == CLX_NUM_ZERO) {
		  return alert(dict('GEN_LBL_NOQUESTIONS'));
        }

        $this->setParentTable($anqn_rec['IDCODE'], $anqn_rec['TABLECODE']);

        //Initialize runid if needed
        if (!$this->existsRunid($anqn_rec)) {
		  $runid = $this->initAnswers($anqn_rec);
		  $this->setRunid($anqn_rec['IDCODE'], $runid);
        }

        $msg = array();
        $record = array();
        $this->loadAnswer($record, $anqn_rec);

        //Store Result, Init and Continue
        if ((array_key_exists('last_question', $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]) && $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['last_question'] == $record['nq']) && $this->validateAnswer($msg, $record, $anqn_rec) && array_key_exists('confirm', UserRequest::$post) && !array_key_exists('tc', UserRequest::$post)) {
		  $this->saveAnswer($record, $anqn_rec);
		  $this->nextQuestion($record, $anqn_rec);
        }

        $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['last_question'] = $record['nq'];

        //Init Session and
        $this->next_question = $record['nq'];
        $question_rec = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']) . ' AND SEQNUM = ' . singleQuote($this->next_question));

        //Set Answers in case of any
        $this->getAnswers($record, $anqn_rec, $question_rec);

        //Test if not end quest
        if (!$this->existsRunid($anqn_rec)) {
		  //Questionary End
		  $this->end_questionary = true;
		  $this->setResultStatus('END', $anqn_rec['RESULT_TABLECODE']);
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, 'valign' => 'top', CLX_PROP_WIDTH => CLX_PROP_100PCT, 'cellspacing' => CLX_NUM_THREE));
		  $table->addCell(dict('GEN_LBL_ENDQUESTIONARY'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  if (!empty($this->is_telemarketing)) {
			 $url = UserRequest::$server['PHP_SELF'] . '?to=MATE&ac=EXECUTE&ri=' . ((!empty(UserRequest::$request['ri'])) ? UserRequest::$request['ri'] : '');
			 $c = new htmlButtonGoto(dict('GEN_LBL_CONTINUE'), $url);
		  } else {
			 if (strpos(UserRequest::$server['PHP_SELF'], 'tool.php') !== false) {
			     $c = new htmlButtonCancel(' ' . dict('GEN_LBL_salir') . ' ');
			 } else {
			     $c = new htmlButtonCloseWindow(dict('GEN_LBL_salir'));
			 }
		  }
		  $table->addCell($c->source(), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        } else {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
		  $table->addCell($this->showQuestionary($anqn_rec), array(CLX_PROP_WIDTH => CLX_NUM_200));
		  $table->addCell($this->buildQuestion($anqn_rec, $question_rec, $record, $msg), array());
        }
        $contents = $table->source();
        return $contents;
    }

    function isQuestionaryEnd() {
        return $this->end_questionary;
    }

    /**
     * Validate Question Answer
     */
    function validateAnswer(&$msg, $record, $anqn_rec) {
        $question_rec = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']) . ' AND SEQNUM = ' . singleQuote($record['nq']));
        $mult = ($question_rec['TYPE'] === 'MULT');
        $required = ($question_rec['FIELD_REQUIRED'] == CLX_NUM_ONE);
        switch ($question_rec['DATA_TYPE']) {
		  case 'LOOK':
			 //Solving Lookups
			 if (array_key_exists('_' . $question_rec['FIELDNAME'] . '_prim', $record)) {
			     $something = (strlen($record['_' . $question_rec['FIELDNAME'] . '_prim'] . $record['_' . $question_rec['FIELDNAME'] . '_secd']) > CLX_NUM_ZERO);
			     if ($something && (count($record['_' . $question_rec['FIELDNAME']]) > CLX_NUM_ONE || strlen($record['_' . $question_rec['FIELDNAME']]) == CLX_NUM_ZERO))
			         $msg['ANSWER_TEXT'] = dict('GEN_ERR_KEY_DONT_EXIST');
			     if (!empty($required))
			         xvalidate::required($msg, 'ANSWER_TEXT', $record['_' . $question_rec['FIELDNAME']]);
			 }
			 $fieldrecord = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($question_rec['FIELDNAME']));
			 $fieldrecord['TABLECODE'] = $anqn_rec['RESULT_TABLECODE'];
			 $field = getToolObject('DBFT', $fieldrecord['FIELDTYPE']);
			 $runid = $this->getRunid($anqn_rec);
			 $field->validateFieldQuestionary($msg, $fieldrecord['TABLECODE'] . '_' . $question_rec['FIELDNAME'], $fieldrecord, $anqn_rec['RESULT_TABLECODE'], $record, $runid);
			 break;
		  case ($question_rec['DATA_TYPE'] === 'BOOL' || $question_rec['DATA_TYPE'] === 'CHAR'):
			 if (!empty($mult)) {
			     if ($question_rec['DISPLAY_TYPE'] === 'CHCK') {//set for checkboxes
			         $options = $this->getOptions($question_rec['ID']);
			         foreach ($options as $optkey => $optval) {
					   if (empty($record['ANSWER_TEXT']) || !is_array($record['ANSWER_TEXT']))
						  $val = '0';
					   elseif (is_array($record['ANSWER_TEXT']) && !array_key_exists($optval['SEQNUM'], $record['ANSWER_TEXT']))
						  $val = '0';
					   else
						  $val = $record['ANSWER_TEXT'] = $record['ANSWER_TEXT'][$optval['SEQNUM']];
					   if (!empty($required))
						  xvalidate::required($msg, 'ANSWER_TEXT[' . $optval['SEQNUM'] . ']', $val);
			         }
			     }else {
			         foreach ($record['ANSWER_TEXT'] as $key => $val) {
					   if (!empty($required))
						  xvalidate::required($msg, 'ANSWER_TEXT[]', $val);
			         }
			     }
			 }else {
			     if (!empty($required)) {
			         xvalidate::required($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT']);
			     }
			 }
			 break;
		  case 'INTN':
			 if (!empty($mult)) {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         if (!empty($required) || !empty($val))
					   xvalidate::integer($msg, 'ANSWER_TEXT', $val, '', '', '');
			     }
			 }else {
			     if (!empty($required) || !empty($record['ANSWER_TEXT'])) {
			         xvalidate::integer($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT'], '', '', '');
			     }
			 }
			 break;
		  case 'DECN':
			 if (!empty($mult)) {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         if (!empty($required) || !empty($val))
					   xvalidate::float($msg, 'ANSWER_TEXT', $val, '', '', '');
			     }
			 }else {
			     if (!empty($required) || !empty($record['ANSWER_TEXT'])) {
			         xvalidate::float($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT'], '', '', '');
			     }
			 }
			 break;
		  case 'DATE':
			 if (!empty($mult)) {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         if (!empty($required) || !empty($val))
					   xvalidate::date($msg, 'ANSWER_TEXT', $val, '');
			     }
			 }else {
			     if (!empty($required) || !empty($record['ANSWER_TEXT'])) {
			         xvalidate::date($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT'], '');
			     }
			 }
			 break;
		  case 'DATI':
			 if (!empty($mult)) {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         if (!empty($required) || !empty($val))
					   xvalidate::datetime($msg, 'ANSWER_TEXT', $val, '');
			     }
			 }else {
			     if (!empty($required) || !empty($record['ANSWER_TEXT'])) {
			         xvalidate::datetime($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT'], '');
			     }
			 }
			 break;
		  case 'TIME':
			 if (!empty($mult)) {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         if (!empty($required) || !empty($val))
					   xvalidate::time($msg, 'ANSWER_TEXT', $val, '');
			     }
			 }else {
			     if (!empty($required) || !empty($record['ANSWER_TEXT'])) {
			         xvalidate::time($msg, 'ANSWER_TEXT', $record['ANSWER_TEXT'], '');
			     }
			 }
			 break;
		  default:
			 break;
        }
        return empty($msg);
    }

    /**
     * Retrive Run id from Session
     */
    function getRunid($anqn_rec) {
        $this->runid = ClxCommon::$db->getField(tableName($anqn_rec['RESULT_TABLECODE']), 'ID', 'CREATEDBY=' . singleQuote($_SESSION['user_record']['username']) . ' AND RESULT_CODE = ' . singleQuote('WORK') . '');
        if (empty($this->runid))
		  $this->runid = false;
        return $this->runid;
    }

    function setResultStatus($status, $result_tablecode) {
        ClxCommon::$db->updateRecord(tableName($result_tablecode), array('RESULT_CODE' => $status), 'CREATEDBY=' . singleQuote($_SESSION['user_record']['username']) . ' AND RESULT_CODE = ' . singleQuote('WORK') . '');
    }

    /**
     * Test if runid exists
     */
    function existsRunid($anqn_rec) {
        return $this->getRunid($anqn_rec);
    }

    function setRunid($questionary_code, $result_id) {
        $_SESSION['QUESTIONARY'][$questionary_code]['runid'] = $result_id;
    }

    /**
     * Init parent table
     */
    function parentTable($anqn_rec) {
        $this->parent_table = (!empty($_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_table'])) ? $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_table'] : '';
        $this->parent_id = (!empty($_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_id'])) ? $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_id'] : '';
    }

    function setParentTable($questionary_code, $parent_table_code, $parent_record_id = '') {
        $this->parent_table_code = $parent_table_code;
        $this->parent_table = tableName($parent_table_code);
        $_SESSION['QUESTIONARY'][$questionary_code]['parent_table'] = $this->parent_table;
        if ($parent_record_id <> '')
		  $this->setParentTableId($questionary_code, $parent_record_id);
    }

    function initParentTable($questionary_code) {
        $parent_table_code = ClxCommon::$db->getField('CLX_AN_QUESTIONARIES', 'TABLECODE', 'IDCODE = ' . singleQuote($questionary_code));
        $this->parent_table_code = $parent_table_code;
        $this->parent_table = tableName($parent_table_code);
    }

    function setParentTableId($questionary_code, $parent_record_id) {
        $this->parent_id = $parent_record_id;
        $_SESSION['QUESTIONARY'][$questionary_code]['parent_id'] = $this->parent_id;
    }

    /**
     * Initializes response record and set run id.
     */
    function endAnswers($anqn_rec) {
        ClxCommon::$db->updateRecord(tableName($anqn_rec['RESULT_TABLECODE']), array('RESULT_CODE' => 'END'), 'CREATEDBY=' . singleQuote($_SESSION['user_record']['username']) . ' AND RESULT_CODE = ' . singleQuote('WORK') . '');
        unset($_SESSION['QUESTIONARY']);
    }

    /**
     * Initializes response record and set run id.
     */
    function initAnswers($anqn_rec) {
        $d = xFormat::getNow();
        $record = array('QUESTIONARY_CODE' => $anqn_rec['IDCODE']);
        //Store all options on the database
        if ((strlen(trim($this->parent_table)) > CLX_NUM_ZERO && $this->parent_id <> '') || array_key_exists('tc', UserRequest::$request)) {
		  if (array_key_exists('tc', UserRequest::$request))
			 $this->parent_table = tableName(UserRequest::$request['tc']);
		  if (array_key_exists('recs', UserRequest::$request))
			 $this->parent_id = UserRequest::$request['recs'];
		  if ($this->parent_table <> '_')
			 $result = ClxCommon::$db->getRecord($this->parent_table, 'ID=' . singleQuote($this->parent_id));
		  $question_recs = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']), 'FIELDNAME<>' . "''");
		  while (($question_rec = ClxCommon::$db->getNextRecord($question_recs))) {
			 if (is_array($result) && array_key_exists($question_rec['FIELDNAME'], $result)) {
			     $record[$question_rec['FIELDNAME']] = $result[$question_rec['FIELDNAME']];
			 }
		  }
        }
        $record['CAMPSECTION_TELE_CODE'] = $this->mark_camp_tele_code;
        $record['CAMPTARGET_ID'] = $this->camp_target_id;
        $record['CREATEDBY'] = $_SESSION['user_record']['username'];
        $record['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['RESULT_CODE'] = 'WORK';
        $result_table = tableName($anqn_rec['RESULT_TABLECODE']);
        //CAMPTARGET_ID, CAMPSECTION_TELE_CODE
        // Test some basic fields Required
        $recordid = ClxCommon::$db->createRecord($result_table, $record, false);
        if (empty($this->test)) {
		  $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_table'] = $this->parent_table;
		  $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['parent_id'] = $this->parent_id;
		  $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['runid'] = $recordid;
		  $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['src'] = array(); //Source Record
        }
        $_SESSION['QUESTIONARY'][$anqn_rec['IDCODE']]['last_question'] = ''; //Source Record
        return $recordid;
    }

    /**
     * Load or Init Answer Record Question Answer
     */
    function loadAnswer(&$record, $anqn_rec) {
        $record['nq'] = (array_key_exists('nq', UserRequest::$request)) ? UserRequest::$request['nq'] : CLX_NUM_ONE;
        $record['COMMENTS'] = (array_key_exists('COMMENTS', UserRequest::$post)) ? UserRequest::$post['COMMENTS'] : '';
        $question_rec = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']) . ' AND SEQNUM = ' . singleQuote($record['nq']));
        if ($question_rec['DATA_TYPE'] === 'LOOK') {
		  //Solving Lookups
		  $formfieldname = $anqn_rec['RESULT_TABLECODE'] . '_' . $question_rec['FIELDNAME'];
		  if (array_key_exists($formfieldname . '_prim', UserRequest::$post)) {
			 $field = getToolObject('DBFT', 'LOOK');
			 $fieldrecord = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($question_rec['FIELDNAME']));
			 $look_results = $field->getLookupValue($formfieldname, $fieldrecord, UserRequest::$post);
			 if (is_array($look_results[$formfieldname]))
			     UserRequest::$post[$formfieldname] = $look_results[$formfieldname][CLX_NUM_ZERO];
			 else
			     UserRequest::$post[$formfieldname] = $look_results[$formfieldname];
			 $record[$formfieldname] = UserRequest::$post[$formfieldname];
			 $record[$formfieldname . '_prim'] = UserRequest::$post[$formfieldname . '_prim'];
			 $record[$formfieldname . '_secd'] = UserRequest::$post[$formfieldname . '_secd'];
			 $record['ANSWER_TEXT'] = $record[$formfieldname];
		  }else {
			 $record['ANSWER_TEXT'] = (array_key_exists($formfieldname, UserRequest::$post)) ? UserRequest::$post[$formfieldname] : '';
			 $record[$formfieldname] = $record['ANSWER_TEXT'];
		  }
		  //e cho count($look_results);
        } elseif ($question_rec['TYPE'] === 'END') {
		  $record['ANSWER_TEXT'] = (array_key_exists('COMMENTS', UserRequest::$post)) ? UserRequest::$post['COMMENTS'] : '';
        } else {
		  $record['ANSWER_TEXT'] = (array_key_exists('ANSWER_TEXT', UserRequest::$post)) ? UserRequest::$post['ANSWER_TEXT'] : '';
        }
    }

    /**
     * Return Answers
     */
    function getAnswers(&$record, $anqn_rec, $question_rec) {
        $tablecode = tableName($anqn_rec['RESULT_TABLECODE']);
        $rec = ClxCommon::$db->getRecord($tablecode, 'ID=' . singleQuote($this->getRunid($anqn_rec)));
        if ($question_rec['DATA_TYPE'] === 'LOOK') {
		  $formfieldname = $anqn_rec['RESULT_TABLECODE'] . '_' . $question_rec['FIELDNAME'];
		  $fieldrecord = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($question_rec['FIELDNAME']));
		  $field = getToolObject('DBFT', $fieldrecord['FIELDTYPE']);
		  $field->loadField($formfieldname, $fieldrecord, $record, array($question_rec['FIELDNAME'] => $rec[$question_rec['FIELDNAME']]));
		  $record['ANSWER_TEXT'] = $rec[$question_rec['FIELDNAME']];
        } else {
		  if ($question_rec['TYPE'] === 'MULT') {
			 $options = $this->getOptions($question_rec['ID']);
			 foreach ($options as $key => $val) {
			     $s_entry_name = 'Q' . $question_rec['QUESTIONARYCODE'] . '_' . $question_rec['SEQNUM'] . '_' . ($key + CLX_NUM_ONE);
			     $record['ANSWER_TEXT'][] = (isset($rec[$s_entry_name]) ? $rec[$s_entry_name] : '');
			 }
		  } else {
			 $s_entry_name = 'Q' . $question_rec['QUESTIONARYCODE'] . '_' . $question_rec['SEQNUM'];
			 $record['ANSWER_TEXT'] = (isset($rec[$s_entry_name]) ? $rec[$s_entry_name] : '');
		  }
        }
    }

    /**
     * Validate Question Answer
     */
    function saveAnswer($record, $anqn_rec) {
        $runid = $this->getRunid($anqn_rec);
        $ANQQ = &getTool('ANQQ');
        $fldprefix = 'Q' . $anqn_rec['IDCODE'];
        $question_rec = ClxCommon::$db->getRecord($ANQQ->tablename, 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']) . ' AND SEQNUM = ' . singleQuote($record['nq']));
        if ($question_rec['DATA_TYPE'] === 'LOOK') {
		  $rec = array($question_rec['FIELDNAME'] => $record['ANSWER_TEXT']);
        } else {
		  if (is_array($record['ANSWER_TEXT'])) {
			 $rec[$fldprefix . '_' . $record['nq']] = ''; //MULTI Means blank
			 if ($question_rec['DISPLAY_TYPE'] === 'CHCK') {//Check Boxes
			     $options = $this->getOptions($question_rec['ID']);
			     foreach ($options as $key => $val) {
			         if (!empty($record['ANSWER_TEXT']) && array_key_exists($key, $record['ANSWER_TEXT'])) {
					   $rec[$fldprefix . '_' . $record['nq'] . '_' . $val['SEQNUM']] = $record['ANSWER_TEXT'][$key];
			         } else {
					   $rec[$fldprefix . '_' . $record['nq'] . '_' . ($key + CLX_NUM_ONE)] = 'null';
			         }
			     }
			 } else {
			     foreach ($record['ANSWER_TEXT'] as $key => $val) {
			         $rec[$fldprefix . '_' . $record['nq'] . '_' . ($key + CLX_NUM_ONE)] = $val;
			     }
			 }
		  } else {
			 $rec = array($fldprefix . '_' . $record['nq'] => $record['ANSWER_TEXT']);
		  }
        }
        //Modification Data
        $rec['MODIFIEDBY'] = $_SESSION['user_record']['username'];
        $rec['MDATE'] = xFormat::getNow();
        if ($question_rec['FIELD_READONLY'] == CLX_NUM_ZERO) {
		  if ($question_rec['DATA_TYPE'] === 'LOOK') {
			 $fieldrecord = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($question_rec['FIELDNAME']));
			 $fieldrecord['TABLECODE'] = $anqn_rec['RESULT_TABLECODE'];
			 $field = getToolObject('DBFT', $fieldrecord['FIELDTYPE']);
			 $field->saveField($fieldrecord['TABLECODE'] . '_' . $question_rec['FIELDNAME'], $fieldrecord, $rec, $runid);
			 if (empty($this->update_parent_at_end)) {
			     $this->parentTable($anqn_rec); //Load parent table
			     //Test if field exists
			     if (!empty($question_rec['UPDATE_PARENT']) && !empty($this->parent_table) && !empty($this->parent_id)) {
			         ClxCommon::$db->updateRecord($this->parent_table, array($question_rec['FIELDNAME'] => $rec[$question_rec['FIELDNAME']]), 'ID = ' . singleQuote($this->parent_id));
			     }
			 }
		  }
		  if ($this->resultRecordExists($anqn_rec['RESULT_TABLECODE'], $runid)) {
			 ClxCommon::$db->updateRecord(tableName($anqn_rec['RESULT_TABLECODE']), $rec, 'ID =' . singleQuote($runid));
		  } else {
			 $rec['CREATEDBY'] = $_SESSION['user_record']['username'];
			 $rec['CDATE'] = xFormat::getNow();
			 $runid = ClxCommon::$db->createRecord(tableName($anqn_rec['RESULT_TABLECODE']), $rec, false);
			 $this->setRunid($anqn_rec['IDCODE'], $runid);
		  }
        }
        if ($question_rec['TYPE'] === 'END') {
		  $this->endAnswers($anqn_rec);
        }
    }

    /**
     * Will update all parents on one call
     */
    function updateAllParents($questionary_code) {
        if (empty($this->runid))
		  return;
        $sql = 'SELECT Q.IDCODE, Q.RESULT_TABLECODE, QQ.FIELDNAME ' . 'FROM CLX_AN_QUESTIONARIES Q, CLX_AN_QUESTIONARY_QUESTS QQ ' . ' WHERE Q.IDCODE = QQ.QUESTIONARYCODE ' . 'AND Q.IDCODE = ' . singleQuote($questionary_code) . ' ' . 'AND QQ.DATA_TYPE = ' . singleQuote('LOOK') . ' AND QQ.UPDATE_PARENT=1 ' . 'AND EXISTS(SELECT X.ID FROM CLX_DB_TABLEFIELDS X ' . ' WHERE X.TABLECODE = Q.TABLECODE AND X.FIELDNAME = QQ.FIELDNAME) ';
        $results = ClxCommon::$db->executeQuery($sql);
        $sql = 'SELECT ';
        $comma = '';
        $ok = false;
        $result_tablecode = '';
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  if (empty($ok))
			 $result_tablecode = $result['RESULT_TABLECODE'];
		  $ok = true;
		  $sql .= $comma . ' ' . $result['FIELDNAME'];
		  $comma = ', ';
        }
        if (empty($ok))
		  return;
        $sql .= ' FROM ' . tableName($result_tablecode) . ' WHERE ID = ' . $this->runid;
        $results = ClxCommon::$db->executeQuery($sql);
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  ClxCommon::$db->updateRecord($this->parent_table, $result, 'ID =  ' . $this->parent_id);
        }
        return;
    }

    /**
     * Validate Question Answer
     */
    function nextQuestion(&$record, $anqn_rec) {
        //Test if this is a jump menu or just continue
        if (!is_array($record['ANSWER_TEXT'])) {
		  $next_question = ClxCommon::$db->getOne('SELECT ANQO.NEXT_QUESTION FROM CLX_AN_QUESTIONARY_QUESTS ANQQ, CLX_AN_QUESTIONARY_OPTIONS ANQO WHERE ANQQ.SEQNUM = ' . singleQuote($record['nq']) . ' AND ANQQ.TYPE = ' . singleQuote('SING') . ' AND ANQO.OPTION_VALUE = ' . singleQuote($record['ANSWER_TEXT']) . ' AND ANQQ.ID = ANQO.QUESTIONID AND ANQQ.QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']));
        } else {
		  $next_question = false;
        }
        //Test Next Question.
        if (empty($next_question)) {
		  //Total Questions
		  $anqqQty = ClxCommon::$db->getOne('SELECT NEXT_QUESTION FROM CLX_AN_QUESTIONARY_QUESTS WHERE SEQNUM = ' . singleQuote($record['nq']) . ' AND QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']));
		  if (!empty($anqqQty)) {
			 $record['nq'] = $anqqQty;
		  } else {
			 $anqqQty = ClxCommon::$db->getOne('SELECT MAX(SEQNUM) FROM CLX_AN_QUESTIONARY_QUESTS WHERE QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']));
			 if ($anqqQty > $record['nq'])
			     $record['nq'] ++;
			 else
			     $record['nq'] = CLX_NUM_ONE;
		  }
        }else {
		  $record['nq'] = $next_question;
        }
        $record['ANSWER_TEXT'] = false;
        $record['COMMENTS'] = false;
    }

    /**
     * Build Question
     * Build Question Form According Sequnce number
     * @param: (string) $questionary_code
     * @param: (integer) $sequence number
     */
    function buildQuestion($anqn_rec, $question_rec, $record, $msg) {
        //Process Questionary
        $ac = UserRequest::$request['ac'];
        $to = UserRequest::$request[CLX_PROP_TO];
        $options = $this->getOptions($question_rec['ID']);
        $questionLabel = $question_rec['SEQNUM'] . '. ' . dict($question_rec['QUESTION']) . '<br>';
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF'], 'ID', 'theForm', 'NAME', 'theForm');
        $form->addArgument(CLX_PROP_TO, $to);
        $form->addArgument('ac', $ac);
        $form->addArgument('nq', $this->next_question);
        $form->addArgument('ri', (!empty(UserRequest::$request['ri'])) ? UserRequest::$request['ri'] : ''); //marketing campaign id if exists
        $form->addArgument('idlist', $anqn_rec['ID']);
        $form->addArgument('confirm', CLX_NUM_ONE);
        $required = ($question_rec['FIELD_REQUIRED'] == CLX_NUM_ONE);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array('valign' => 'top', CLX_PROP_WIDTH => CLX_PROP_100PCT, 'cellspacing' => '3'));
        $table->addCell($questionLabel . '<br>', array(CLX_PROP_CLASS => 'panel', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        switch ($question_rec['TYPE']) {
		  case 'OPEN':
			 if ($question_rec['DATA_TYPE'] === 'DATE' || $question_rec['DATA_TYPE'] === 'DATI') {
			     $form_fieldname = 'ANSWER_TEXT';
			     $class = 'hasDatepicker';
			     $picker = '<script type="text/javascript">
						  $(function() {
						      var pickOpts = {
								    dateFormat: "yy-mm-dd",
								    showTime: true,
								    stepMinutes: 1,
								    stepHours: 1,
								    altTimeField: "",
								    duration: "",
								    showTime: ' . (($question_rec['DATA_TYPE'] === 'DATI') ? 'true' : 'false') . ',' .
								    'constrainInput: false,
								    time24h: true,
								    changeMonth: true,
								    changeYear: true,
								    showOn: "button",
								    buttonImage: "./' . kPathToSkins . $_SESSION["skin"] . '/images/calendar/cal.gif",
								    buttonImageOnly: true
								};
						      $("#' . $form_fieldname . '").datepicker(pickOpts);
						  });
					   </script>';
			 }else{
			     $class = '';
			     $picker = '';
			 }
			 $b = new htmlTable(CLX_WIDTH_ONE);
			 if (!empty($question_rec['FIELD_READONLY'])) {
			     $b->addCells($class, array($record['ANSWER_TEXT']));
			 } elseif (!empty($question_rec['FIELD_HIDDEN'])) {
			     $form->addArgument('ANSWER_TEXT', $record['ANSWER_TEXT']);
			 } else {
			     if($question_rec['DATA_TYPE']==='DATE' || $question_rec['DATA_TYPE']==='DATI'){
			         $r = new htmlField('ANSWER_TEXT', $record['ANSWER_TEXT'], CLX_NUM_30, '');
			     } else {
			         $r = new htmlTextArea('ANSWER_TEXT', $record['ANSWER_TEXT'], CLX_NUM_FOUR, CLX_NUM_60);
			     }
			     $b->addCells($class, array($r->source() . $picker));
			 }
			 break;
		  case 'END':
			 $b = false;
			 break;
		  case 'SING':
			 switch ($question_rec['DISPLAY_TYPE']) {
			     case 'RADI':
			         if (!empty($question_rec['FIELD_READONLY'])) {
					   $b = new htmlTable(CLX_WIDTH_ONE);
					   foreach ($options as $key => $value) {
						  if ($record['ANSWER_TEXT'] == $value['OPTION_VALUE'])
						      $b->addCells('', array(dict($value['OPTION_TEXT'])));
					   }
			         }elseif (!empty($question_rec['FIELD_HIDDEN'])) {
					   foreach ($options as $key => $value) {
						  $form->addArgument('ANSWER_TEXT', $value['OPTION_VALUE']);
					   }
			         } else {
					   $b = new htmlTable(CLX_WIDTH_TWO);
					   foreach ($options as $key => $value) {
						  $r = new htmlRadioButton('ANSWER_TEXT', ($record['ANSWER_TEXT'] == $value['OPTION_VALUE']), $value['OPTION_VALUE']);
						  $b->addCells('', array($r->source(), dict($value['OPTION_TEXT'])));
					   }
			         }
			         break;
			     case 'CHCK':
			         if (!empty($question_rec['FIELD_READONLY'])) {
					   $b = new htmlTable(CLX_WIDTH_ONE);
					   foreach ($options as $key => $value) {
						  if ($record['ANSWER_TEXT'] == $value['OPTION_VALUE'])
						      $b->addCells('', array(dict($value['OPTION_TEXT'])));
					   }
			         }elseif (!empty($question_rec['FIELD_HIDDEN'])) {
					   foreach ($options as $key => $value) {
						  $form->addArgument('ANSWER_TEXT', $value['OPTION_VALUE']);
					   }
			         } else {
					   $b = new htmlTable(CLX_WIDTH_TWO);
					   foreach ($options as $key => $value) {
						  $r = new htmlCheckbox('ANSWER_TEXT', ($record['ANSWER_TEXT'] == $value['OPTION_VALUE']), $value['OPTION_VALUE']);
						  $b->addCells('', array($r->source(), dict($value['OPTION_TEXT'])));
					   }
			         }
			         break;
			     default:
			         if (!empty($question_rec['FIELD_HIDDEN'])) {
					   $form->addArgument('ANSWER_TEXT', $value['OPTION_VALUE']);
			         } else {
					   switch ($question_rec['DATA_TYPE']) {
						  case 'LOOK':
						      $fieldrecord = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME = ' . singleQuote($question_rec['FIELDNAME']));
						      if (!empty($fieldrecord)) {
								$field = getToolObject('DBFT', $fieldrecord['FIELDTYPE']);
								$fieldrecord['FIELD_FORM_REQUIRED'] = $question_rec['FIELD_REQUIRED'];
								$fieldrecord['FIELD_FORM_REQUIRED'] = $question_rec['FIELD_REQUIRED'];
								$fieldrecord['FIELD_HIDDEN'] = $question_rec['FIELD_HIDDEN'];
								$fieldrecord['CAPTION'] = '';
								$fieldrecord['FIELD_READONLY'] = $question_rec['FIELD_READONLY'];
								$fieldrecord['TABLECODE'] = $anqn_rec['RESULT_TABLECODE'];
								if (!empty($question_rec['FIELD_READONLY']) || $question_rec['FIELD_HIDDEN'] === true) {
								    $b = $field->placeViewField($fieldrecord['TABLECODE'] . '_' . $fieldrecord['FIELDNAME'], $fieldrecord, $record);
								} else {
								    $jsevents = array();
								    $b = $field->placeEditField($msg, $fieldrecord['TABLECODE'] . '_' . $fieldrecord['FIELDNAME'], $fieldrecord, $record, $form, $jsevents);
								}
						      } else {
								$b = new htmlTable(CLX_WIDTH_ONE);
								$b->addCells('', array(dict('GEN_LBL_INVALIDFIELDNAME')));
						      }
						      break;
						  default:
						      //Select Type
						      if (!empty($question_rec['FIELD_READONLY'])) {
								$b = new htmlTable(CLX_WIDTH_ONE);
								foreach ($options as $key => $value) {
								    if ($record['ANSWER_TEXT'] == $value['OPTION_VALUE'])
									   $b->addCells('', array(dict($value['OPTION_TEXT'])));
								}
						      }elseif (!empty($question_rec['FIELD_HIDDEN'])) {
								$form->addArgument('ANSWER_TEXT', $value['OPTION_VALUE']);
						      } else {
								$b = new htmlSelect('ANSWER_TEXT');
								foreach ($options as $key => $value) {
								    $b->addItem(uppercase(dict($value['OPTION_TEXT'])), $value['OPTION_VALUE'], ($record['ANSWER_TEXT'] == $value['OPTION_VALUE']));
								}
						      }
						      break;
					   }//end switch
			         }
			         break;
			 }
			 break;
		  case 'MULT':
			 if (!empty($question_rec['FIELD_HIDDEN'])) {
			     $form->addArgument('ANSWER_TEXT', $value['OPTION_VALUE']);
			 } else {
			     switch ($question_rec['DISPLAY_TYPE']) {// No radio bottons on single choice
			         case ($question_rec['DISPLAY_TYPE'] === 'TIME' || $question_rec['DISPLAY_TYPE'] === 'CHAR'):
					   if (!empty($question_rec['FIELD_READONLY']) || !empty($question_rec['FIELD_HIDDEN'])) {
						  $b = new htmlTable(CLX_WIDTH_ONE);
						  foreach ($options as $key => $value) {
						      $b->addCells('', array($record['ANSWER_TEXT'][$key]));
						  }
					   } else {
						  //Select Type
						  $b = new htmlTable(CLX_WIDTH_TWO);
						  foreach ($options as $key => $value) {
						      if ($question_rec['DATA_TYPE'] === 'DATE' || $question_rec['DATA_TYPE'] === 'DATI') {
								//PARA REVISAR CALENDARIOS CON ARREGLOS
								$form_fieldname = 'ANSWER_TEXT['.$key.']';
								$class = 'hasDatepicker';
								$picker = '<script type="text/javascript">
									       $(function() {
											 var pickOpts = {
												    dateFormat: "yy-mm-dd",
												    showTime: true,
												    stepMinutes: 1,
												    stepHours: 1,
												    altTimeField: "",
												    duration: "",
												    showTime: ' . (($question_rec['DATA_TYPE'] === 'DATI') ? 'true' : 'false') . ',' .
												    'constrainInput: false,
												    time24h: true,
												    changeMonth: true,
												    changeYear: true,
												    showOn: "button",
												    buttonImage: "./' . kPathToSkins . $_SESSION["skin"] . '/images/calendar/cal.gif",
												    buttonImageOnly: true
												};
											 $("#' . $form_fieldname . '").datepicker(pickOpts);
									       });
									   </script>';
						      }else{
								$class = '';
								$picker = '';
						      }
						      $r = new htmlField('ANSWER_TEXT[' . $key . ']', $record['ANSWER_TEXT'][$key], CLX_NUM_30, '');
						      $b->addCells($class, array(dict($value['OPTION_TEXT']), $r->source() . $picker));
						  }
					   }
					   break;
			         case 'CHCK':
					   //Select Type
					   if (!empty($question_rec['FIELD_READONLY']) || !empty($question_rec['FIELD_HIDDEN'])) {
						  $b = new htmlTable(CLX_WIDTH_ONE);
						  foreach ($options as $key => $value) {
						      if ($record['ANSWER_TEXT'][$key] == $value['OPTION_VALUE'])
								$b->addCells(CLX_CHAR_EMPTY_STRING, array(dict($value['OPTION_TEXT'])));
						  }
					   }else {
						  $b = new htmlTable(CLX_WIDTH_TWO);
						  foreach ($options as $key => $value) {
						      $r = new htmlCheckbox('ANSWER_TEXT[' . $key . ']', (is_array($record['ANSWER_TEXT']) && array_key_exists($key, $record['ANSWER_TEXT']) && $record['ANSWER_TEXT'][$key] == $value['OPTION_VALUE']), $value['OPTION_VALUE']);
						      $b->addCells(CLX_CHAR_EMPTY_STRING, array($r->source(), dict($value['OPTION_TEXT'])));
						  }
					   }
					   break;
			         default:
					   //Select Type
					   if ($question_rec['FIELD_READONLY'] === true || $question_rec['FIELD_HIDDEN'] === true) {
						  $b = new htmlTable(CLX_WIDTH_ONE);
						  foreach ($options as $key => $value) {
						      if ($record['ANSWER_TEXT'][$key] == $value['OPTION_VALUE'])
								$b->addCells(CLX_CHAR_EMPTY_STRING, array(dict($value['OPTION_TEXT'])));
						  }
					   }else {
						  $b = new htmlSelect('ANSWER_TEXT[]');
						  $b->setProp('size', '5');
						  $b->setProp('multiple', CLX_CHAR_EMPTY_STRING);
						  foreach ($options as $key => $value) {
						      $b->addItem(dict($value['OPTION_TEXT']), $value['OPTION_VALUE'], ($record['ANSWER_TEXT'][$key] == $value['OPTION_VALUE']));
						  }
					   }
					   break;
			     }//End switch
			 }//end if
			 break;
		  default:
			 $b = false;
			 break;
        }//end switch
        if (!empty($b)) {
		  if (!empty($question_rec['FIELD_READONLY']) || !empty($question_rec['FIELD_HIDDEN'])) {
			 $table->addCell($b->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  } else {
			 if (array_key_exists('ANSWER_TEXT', $msg))
			     $table->addCell($msg['ANSWER_TEXT'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 if ($question_rec['DATA_TYPE'] === 'DATE') {
			     $table->addCell(dict('GEN_LBL_ANSWER') . '<br><span class="promptdateformat">' . dict('GEN_DATE_INPUT_FORMAT') . '</spam>', array(CLX_PROP_CLASS => ($required) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT));
			 } elseif ($question_rec['DATA_TYPE'] === 'DATI') {
			     $table->addCell(dict('GEN_LBL_ANSWER') . '<br><span class="promptdateformat">' . dict('GEN_DATETIME_INPUT_FORMAT') . '</spam>', array(CLX_PROP_CLASS => ($required) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT));
			 } else {
			     $table->addCell(dict('GEN_LBL_ANSWER'), array(CLX_PROP_CLASS => ($required) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT));
			 }
			 $table->addCell($b->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  }
        }
        //Comments for the last option
        if ($question_rec['TYPE'] === 'END') {
		  $b = new htmlTextArea('COMMENTS', $record['ANSWER_TEXT'], CLX_NUM_FOUR, CLX_NUM_60);
		  $table->addCell(dict('GEN_LBL_COMMENTS'), array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
		  $table->addCell($b->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        }
        $buttons = '';
        $c = new htmlSubmit(dict('GEN_LBL_CONTINUE'));
        $buttons .= $c->source();
        if (strpos(UserRequest::$server['PHP_SELF'], 'tool.php') !== false) {
		  $c = new htmlButtonCancel(' ' . dict('GEN_LBL_salir') . ' ');
        } else {
		  $c = new htmlButtonCloseWindow(dict('GEN_LBL_salir'));
        }
        $buttons .= $c->source();
        if (!empty($question_rec['FIELD_HIDDEN'])) {
		  $submit_hidden = '<script language="javascript">theFormSubmit(' . singleQuote('theForm') . ');</script>';
        } else {
		  $submit_hidden = '';
        }
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addCell(CLX_CHAR_HTML_NBSP . $submit_hidden, array());
        $table->addCell($buttons, array());
        $form->addContents($table->source());
        return $form->source();
    }

// End method
    /**
     * Rvise
     */
    function showQuestionary($anqn_rec) {
        $contents = '';
        $format = new xFormat();
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('GEN_LBL_QUESTIONARY'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $ri = (!empty(UserRequest::$request['ri'])) ? '&ri=' . UserRequest::$request['ri'] : ''; //marketing campaign id if exists
        $results = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($anqn_rec['IDCODE']) . ' AND FIELD_HIDDEN = 0 ORDER BY SEQNUM ASC');
        $i = CLX_NUM_ONE;
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  $contents = $i . '. ';
		  $i++;
		  if ($this->next_question == $result['SEQNUM']) {
			 $url = new htmlAnchor('<font color="red">' . dict($result['QUESTION']). '</font>', UserRequest::$server['PHP_SELF'] . '?to=' . UserRequest::$request['to'] . '&nq=' . $result['SEQNUM'] . '&ac=' . UserRequest::$request['ac'] . '&idlist=' . $anqn_rec['ID'] . $ri);
		  } else {
			 $url = new htmlAnchor(dict($result['QUESTION']), UserRequest::$server['PHP_SELF'] . '?to=' . UserRequest::$request['to'] . '&nq=' . $result['SEQNUM'] . '&ac=' . UserRequest::$request['ac'] . '&idlist=' . $anqn_rec['ID'] . $ri);
		  }
		  $contents .= $url->source();
		  $table->addCell($contents, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $contents = '';
		  //Test if options are found
		  $options = $this->getOptions($result['ID']);
		  $contents = '<blockquote>';
		  foreach ($options as $key2 => $value) {
			 if ($result['TYPE'] === 'MULT') {
			     $contents .= $format->text(uppercase(dict($value['OPTION_TEXT']))) . '<br>';
			 } else {
			     $contents .= $format->text(uppercase(dict($value['OPTION_TEXT']))) . ' -> ' . $value['NEXT_QUESTION'] . '<br>';
			 }
		  }
		  $contents .= '</blockquote>';
		  $table->addCell($contents, array(CLX_PROP_CLASS => CLX_CLASS_DATAALT));
        }
        return $table->source();
    }

    /**
     * Rvise
     */
    function getQuestion($questionId) {
        $ANQQ = &getTool('ANQQ');
        $question = ClxCommon::$db->getRecord($ANQQ->tablename, 'ID=' . singleQuote($questionId));
        return $question;
    }

    /**
     * Rvise
     */
    function getOptions($questionid) {
        $ANQO = &getTool('ANQO');
        $options = array();
        $sqlResult = ClxCommon::$db->getRecords($ANQO->tablename, 'QUESTIONID=' . singleQuote($questionid) . ' ORDER BY SEQNUM ASC');
        while ($row = ClxCommon::$db->getNextRecord($sqlResult)) {
		  $options[] = $row;
        }
        return $options;
    }

    /**
     * Returns questionary id
     * @param type $optionId
     * @return type
     */
    function getOption($optionId) {
        $ANQO = &getTool('ANQO');
        $option = ClxCommon::$db->getRecord($ANQO->tablename, 'ID=' . $optionId);
        return $option;
    }

    private function getResultFields() {
        // Test some basic fields Required
	$d=date('Y/m/d');
        $fields = array(
		  array('FIELDNAME' => 'QUESTIONARY_CODE', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Questionary Code', 'FIELDTYPE' => 'SOBJ', 'FIELDLISTLABEL' => 'GEN_LBL_QUESTIONARY_CODE', 'FIELDFORMLABEL' => 'GEN_LBL_QUESTIONARY_CODE', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '0', 'FIELD_REQUIRED' => '1', 'FIELD_ALIGN' => '0', 'FIELD_DISPLAYSIZE' => '1', 'FIELD_EXTRA1' => 'ANQN'),
		  array('FIELDNAME' => 'CREATEDBY', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Created By', 'FIELDTYPE' => 'TEXT', 'FIELDLISTLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDFORMLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '1', 'FIELD_REQUIRED' => '0', 'FIELD_EXTRA1' => '4', 'FIELD_EXTRA2' => '48'),
		  array('FIELDNAME' => 'MODIFIEDBY', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Modified By', 'FIELDTYPE' => 'CHAR', 'FIELDLISTLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDFORMLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '48', 'FIELD_REQUIRED' => '0', 'FIELD_ALIGN' => '0', 'FIELD_DISPLAYSIZE' => '8', 'FIELD_DEFAULT' => CLX_NUM_ONE, 'FIELD_EXTRA1' => CLX_NUM_ZERO, 'FIELD_EXTRA2' => CLX_NUM_ZERO),
		  array('FIELDNAME' => 'CDATE', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Creation Date', 'FIELDTYPE' => 'CHAR', 'FIELDLISTLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDFORMLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '255', 'FIELD_REQUIRED' => '0', 'FIELD_ALIGN' => '0', 'FIELD_DISPLAYSIZE' => '48'),
		  array('FIELDNAME' => 'MDATE', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Modification Date', 'FIELDTYPE' => 'CHAR', 'FIELDLISTLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDFORMLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '255', 'FIELD_REQUIRED' => '0', 'FIELD_ALIGN' => '0', 'FIELD_DISPLAYSIZE' => '48'),
		  array('FIELDNAME' => 'RESULT_CODE', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Result code', 'FIELDTYPE' => 'CHAR', 'FIELDLISTLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDFORMLABEL' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => '10', 'FIELD_REQUIRED' => '0', 'FIELD_ALIGN' => '0', 'FIELD_DISPLAYSIZE' => '10'),
		  array('FIELDNAME' => 'CAMPSECTION_TELE_CODE', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Campaign section code', 'FIELDTYPE' => 'LOOK', 'FIELDLISTLABEL' => '61167397C8F189929E097A492C363A64', 'FIELDFORMLABEL' => '61167397C8F189929E097A492C363A64', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => CLX_NUM_ZERO, 'FIELD_REQUIRED' => CLX_NUM_ZERO, 'FIELD_ALIGN' => CLX_NUM_ZERO, 'FIELD_DISPLAYSIZE' => '10', 'FIELD_EXTRA1' => 'CTMK', 'FIELD_EXTRA2' => 'IDCODE', 'FIELD_EXTRA3' => 'IDCODE', 'FIELD_EXTRA4' => 'IDNAME'),
            array('FIELDNAME' => 'CAMPTARGET_ID', 'VERSION' => '1.0.0', 'VERSION_COUNTER' => '0', 'CDATE' => $d, 'MDATE' => $d, 'USERNAME' => $_SESSION['user_record']['username'], 'DESCRIPTION' => 'Campaign section code', 'FIELDTYPE' => 'LOOK', 'FIELDLISTLABEL' => '0438bc5aa19805ab394b16a9674a9bb3', 'FIELDFORMLABEL' => '0438bc5aa19805ab394b16a9674a9bb3', 'FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e', 'FIELD_SIZE' => CLX_NUM_ZERO, 'FIELD_REQUIRED' => CLX_NUM_ZERO, 'FIELD_ALIGN' => CLX_NUM_ZERO, 'FIELD_DISPLAYSIZE' => '10', 'FIELD_EXTRA1' => 'CTGT', 'FIELD_EXTRA2' => 'ID', 'FIELD_EXTRA3' => 'CQRY_CODE', 'FIELD_EXTRA4' => 'FIRSTNAME'),
			array('FIELDNAME' =>'CAMPTELERES_ID','VERSION' => '1.0.0','VERSION_COUNTER' => '0','CDATE' => $d,'MDATE' => $d,'USERNAME' => $_SESSION["user_record"]["username"],'DESCRIPTION' => 'Result telemark code','FIELDTYPE' => 'LOOK','FIELDLISTLABEL' => '0438bc5aa19805ab394b16a9674a9bb3','FIELDFORMLABEL' => '0438bc5aa19805ab394b16a9674a9bb3','FIELDPREFIX' => 'd41d8cd98f00b204e9800998ecf8427e','FIELDSUFIX' => 'd41d8cd98f00b204e9800998ecf8427e',	'FIELD_SIZE' => CLX_NUM_ZERO,'FIELD_REQUIRED' => CLX_NUM_ZERO,'FIELD_ALIGN' => CLX_NUM_ZERO,'FIELD_DISPLAYSIZE' => '10','FIELD_EXTRA1'=>'CTGT','FIELD_EXTRA2'=>'ID','FIELD_EXTRA3'=>'CQRY_CODE','FIELD_EXTRA4'=>'FIRSTNAME')
        );
        return $fields;
    }

    /**
     * Create questionary result table
     * @param type $anqn_rec
     * @return type
     */
    function createTable($anqn_rec) {

        $d = xformat::getNow();
        $record = array();
        $record['IDCODE'] = uppercase($anqn_rec['RESULT_TABLECODE']);
        if (ClxCommon::$db->oneExistsRecords('CLX_DB_TABLES', 'IDCODE = ' . singleQuote($anqn_rec['RESULT_TABLECODE'])) > CLX_NUM_ZERO) {
		  return;
        }
        $record['IDNAME'] = uppercase($anqn_rec['RESULT_TABLECODE']);
        $record['VERSION'] = '1.0.0';
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = $anqn_rec['IDCODE'] . ' ' . $anqn_rec['IDNAME'];
        $record['MODULECODE'] = $anqn_rec['MODULECODE'];
        $record['TYPE'] = 'MyISAM';
        $record['AUDITU'] = '0';
        $record['RECORDNAME_SINGULAR'] = '';
        $record['RECORDNAME_PLURAL'] = '';
	$record['STATUS'] = '1';
        ClxCommon::$db->createRecordVoid('CLX_DB_TABLES', $record, false);

        $DBTB = &getTool('DBTB'); //  Table
        $DBTB->createTable2($record);

        $DBTF = &getTool('DBTF'); //  Table fields
	$DBTF->createTable($record);
	$DBTI = &getTool('DBTI'); // Table indexes
        $DBTI->createTable($record);
	$DBFL = &getTool('DBFL');
        foreach ($this->getResultFields() as $val) {
		  if (!$DBFL->fieldExists($val['FIELDNAME'])) {
			 ClxCommon::$db->createRecordVoid($DBFL->tablename, $val, false);
		  }
		  // Adding Additional Fields required
		  $DBTF->includeField($record['IDCODE'], $val['FIELDNAME']);
        }
    }

    function checkResultTable($anqn_rec) {

        $record = array();
        $record['IDCODE'] = uppercase($anqn_rec['RESULT_TABLECODE']);
        if (ClxCommon::$db->oneExistsRecords('CLX_DB_TABLES', 'IDCODE = ' . singleQuote($anqn_rec['RESULT_TABLECODE'])) === CLX_NUM_ZERO) {
		  return $this->createTable($anqn_rec);
        }

        $DBFL = &getTool('DBFL');
        $DBTF = &getTool('DBTF'); //  Table fields
        foreach ($this->getResultFields() as $val) {

		  if (!$DBFL->fieldExists($val['FIELDNAME'])) {
			 ClxCommon::$db->createRecordVoid($DBFL->tablename, $val, false);

			 // Adding Additional Fields required
			 $DBTF->includeField($record['IDCODE'], $val['FIELDNAME']);
		  }
        }
		$sql="DELETE FROM CLX_AC_TABLES WHERE TABLECODE = '".$record["IDCODE"]."'";
		ClxCommon::$db->executeQuery($sql);
    }

}
