<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class an_questionary_options extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'ANQO';
        $this->name = 'Questionary Options';
        $this->system = 'Interface';
        $this->version = '0.1.3.1';


        $this->accesskey = 'ANQN';

        $this->tablename = 'CLX_AN_QUESTIONARY_OPTIONS';
        $this->record_name_singular = 'questionary option';
        $this->record_name_plural = 'questionary options';
        $this->primary_field = 'ID';
        $this->secondary_field = 'QUESTION';
        $this->seqnum_field = 'SEQNUM';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['QUESTIONID'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'tinyint(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['OPTION_TEXT'] = array('type' => 'mediumtext', 'properties' => '', 'dict' => true);
        $fields['OPTION_VALUE'] = array('type' => 'varchar(255)', 'properties' => '');
        $fields['NEXT_QUESTION'] = array('type' => 'int(11) unsigned', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'QUESTIONID,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $questionid) = $parameters;
        $record = $this->unpackRecord($package);
        $record['QUESTIONID'] = $questionid;
        ClxFreeReturn(ClxCommon::$db->createRecord($this->tablename, $record, false));
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'ANQOPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('questionid', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_AN_QUESTIONARY_OPTIONS';
		  $whereclause = 'QUESTIONID=' . singleQuote($groupcode);
		  $sortclause = 'SEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FOUR);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_OPTION'), dict('GEN_LBL_VALUE'), dict('GEN_LBL_NEXTQUESTION')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $questionarycode = ClxCommon::$db->getField('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE', 'ID=' . singleQuote($record['QUESTIONID']));
			 $next = xFormat::text(ClxCommon::$db->getField('CLX_AN_QUESTIONARY_QUESTS', 'question', 'QUESTIONARYCODE = ' . singleQuote($questionarycode), 'SEQNUM=' . $record['NEXT_QUESTION']));
			 //$table->addCells(($i%2==0)? CLX_CLASS_DATA : CLX_CLASS_DATAALT,array( xFormat::text($record['OPTION_TEXT']), $record['OPTION_VALUE'], $next));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(dict($record['OPTION_TEXT']), $record['OPTION_VALUE'], dict($next)));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_OPTION')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_ANQO_TITLE_LIST'), 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_ANQO_TITLE_MODIFY') : dict('TOOL_ANQO_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANQO_id', $record['ANQO_id']);
        $form->addArgument('idlist', toString($idlist));
        $questionid = $record['ANQO_questionid'];
        $form->addArgument('ANQO_questionid', $questionid);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DICT = &getTool('DICT');
        $DICT->editEntry($msg, dict('GEN_LBL_OPTION') . ':', 'ANQO_option_text', $record, $table, $form);
        //$table->addText($msg, dict('GEN_LBL_OPTION') . ':', 'ANQO_option_text', $record['ANQO_option_text'], 4, 64);
        $table->addField($msg, dict('GEN_LBL_VALUE') . ':', 'ANQO_option_value', $record['ANQO_option_value'], CLX_NUM_48, CLX_NUM_255);
        $questionrec = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_QUESTS', 'ID=' . $questionid);
        $questionarycode = $questionrec['QUESTIONARYCODE'];
        $questions = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_QUESTS', 'QUESTIONARYCODE=' . singleQuote($questionarycode), 'ID<>' . $questionid . ' ORDER BY SEQNUM');
        $pop = new HTMLSelect('ANQO_next_question');
        $pop->addItem('', CLX_NUM_ZERO, false);
        while (($questionrec = ClxCommon::$db->getNextRecord($questions))) {
		  $pop->addItem($questionrec['SEQNUM'] . '. ' . substr(dict($questionrec['QUESTION']), CLX_NUM_ZERO, CLX_NUM_50) . '...', $questionrec['SEQNUM'], ($record['ANQO_next_question'] == $questionrec['SEQNUM']));
        }
        $table->addData(dict('GEN_LBL_NEXTQUESTION'), $pop->source());
        $form->addContents($table->source() . '<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {

		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        xValidate::required($msg, 'ANQO_option_text_DICT', UserRequest::$post['ANQO_option_text_DICT']);
        xValidate::required($msg, 'ANQO_option_value', UserRequest::$post['ANQO_option_value']);
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        //$table->addData(dict('GEN_LBL_OPTION') . ':', xFormat::text($record['ANQO_option_text']));
        $table->addData(dict('GEN_LBL_OPTION') . ':', dict($record['ANQO_option_text']));
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords('CLX_AN_QUESTIONARY_OPTIONS', 'QUESTIONID=' . singleQuote(UserRequest::$post['questionid']) . ' ORDER BY SEQNUM');
        $list = array();
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $t = str_replace(CLX_CHAR_NL, '', $record['OPTION_TEXT']);
		  $t = str_replace(chr(CLX_NUM_13), '', $t);
		  $list[$record['ID']] = dict($t);
        }
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $questionid = $callingpage['questionid'];
        $ANQQ = &getTool('ANQQ');
        $ANQQ->versionIncrease('ID', $questionid);
        //Creating Tablefields
        $ANQQ->createQuestTableField(ClxCommon::$db->getRecord($ANQQ->tablename, 'ID=' . singleQuote($questionid)));
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_OPTIONS', 'ID=' . $recordid);
        $record = array();
        $record['ANQO_id'] = $recordid;
        $record['ANQO_questionid'] = $values['QUESTIONID'];
        $DICT = &getTool('DICT');
        $DICT->loadEntry('ANQO_option_text', $values['OPTION_TEXT'], $record);
        $record['ANQO_option_value'] = $values['OPTION_VALUE'];
        $record['ANQO_seqnum'] = $values['SEQNUM'];
        $record['ANQO_next_question'] = $values['NEXT_QUESTION'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $DICT = &getTool('DICT');
        $record['OPTION_TEXT'] = $DICT->saveEntry('ANQO_option_text', UserRequest::$post);
        $record['OPTION_VALUE'] = UserRequest::$post['ANQO_option_value'];
        $record['NEXT_QUESTION'] = UserRequest::$post['ANQO_next_question'];
        ClxCommon::$db->updateRecord('CLX_AN_QUESTIONARY_OPTIONS', $record, 'ID=' . UserRequest::$post['ANQO_id']);
        $ANQQ = &getTool('ANQQ');
        $ANQQ->versionIncrease('ID', UserRequest::$post['ANQO_questionid']);
        //Creating Tablefields
        $ANQQ->createQuestTableField(ClxCommon::$db->getRecord($ANQQ->tablename, 'ID=' . singleQuote(UserRequest::$post['ANQO_questionid'])));
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['QUESTIONID'] = UserRequest::$post['ANQO_questionid'];
        $DICT = &getTool('DICT');
        $record['OPTION_TEXT'] = $DICT->createEntry('ANQO_option_text', UserRequest::$post);
        $record['OPTION_VALUE'] = UserRequest::$post['ANQO_option_value'];
        $record['NEXT_QUESTION'] = UserRequest::$post['ANQO_next_question'];
        $record['SEQNUM'] = ClxCommon::$db->getField('CLX_AN_QUESTIONARY_OPTIONS', 'SEQNUM', 'QUESTIONID=' . singleQuote(UserRequest::$post['ANQO_questionid']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord('CLX_AN_QUESTIONARY_OPTIONS', $record, false);
        $ANQQ = &getTool('ANQQ');
        $ANQQ->versionIncrease('ID', UserRequest::$post['ANQO_questionid']);
        //Creating Tablefields
        $ANQQ->createQuestTableField(ClxCommon::$db->getRecord($ANQQ->tablename, 'ID=' . singleQuote(UserRequest::$post['ANQO_questionid'])));
        return $recordid;
    }

    function deleteRecordSk($id) {
        $optionrec = ClxCommon::$db->getRecord('CLX_AN_QUESTIONARY_OPTIONS', 'ID=' . singleQuote($id));
        ClxCommon::$db->deleteRecords('CLX_AN_QUESTIONARY_OPTIONS', 'ID=' . $id);
        $ANQQ = &getTool('ANQQ');
        $ANQQ->versionIncrease('ID', $optionrec['QUESTIONID']);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['ANQO_id'] = '0';
        $record['ANQO_questionid'] = $callingpage['questionid'];
        $record['ANQO_option_text'] = '';
        $record['ANQO_option_value'] = '';
        $record['ANQO_seqnum'] = '0';
        $record['ANQO_next_question'] = '0';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        $questionid = ClxCommon::$db->getField($this->tablename, 'QUESTIONID', $fieldname . '=' . singleQuote($fieldvalue));
        $ANQQ = &getTool('ANQQ');
        $ANQQ->versionIncrease('ID', $questionid);
    }

}
