<?php

/**
 * Reports Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version    	$Rev: 9615 $
 * @package		Clientix.Tool.Analytics
 * @abstract		Report Group Functions
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class an_report_group_func extends clx_tool {

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'ANRX';
        $this->name = 'Report Group Functions';
        $this->system = 'Analytics';
        $this->version = '0.1.3.1';


        $this->accesskey = 'ANRP';

        $this->tablename = 'CLX_AN_REPORT_GROUP_FUNC';
        $this->record_name_singular = 'report group function';
        $this->record_name_plural = 'report group functions';
        $this->primary_field = 'GROUPID';
        $this->secondary_field = 'TABLECODE';
        $this->seqnum_field = 'SEQNUM';
    }

    /**
     * Describes the tool table structure.
     * @return array ('type' => 'varchar(255)', 'properties'=>'auto increment',
     * 				'index'=>'', default = '', 'dict'=>true)
     * 				 dict true or false means that field holds dictionary entry
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['GROUPID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['SUM'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['AVG'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['MIN'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['MAX'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['COUNT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FORMAT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'GROUPID,SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

//-- R E P O R T    G R O U  P    F U N C T I O N S
    /**
     * Loads package into system
     * @param string $package
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => null), $parameters));
        list($package, $groupid) = $parameters;
        $record = $this->unpackRecord($package);
        $record['GROUPID'] = $groupid;
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($groupid = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $sql = 'SELECT R.REPORT_TYPE FROM CLX_AN_REPORT_GROUPS G, CLX_AN_REPORTS R WHERE R.IDCODE = G.REPORTCODE AND G.ID = ' . $groupid;
		  $report_type = ClxCommon::$db->getOne($sql);

		  $title = ($report_type === CLX_NUM_ONE) ? 'TOOL_ANRX_BOOKMARKFIELD' : 'TOOL_ANRX_TITLE_LIST';

		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'ANRXPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('groupid', $groupid);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = 'T.IDCODE TABLE_CODE, T.IDNAME TABLENAME, T.MODULECODE MODULECODE, Q.REPQRYCODE, F.*';
		  $fromclause = 'CLX_DB_TABLES T, CLX_AN_REPORT_GROUP_FUNC F LEFT JOIN CLX_AN_REPORT_GROUPS Q ON Q.ID = F.GROUPID';
		  $whereclause = 'F.GROUPID=' . singleQuote($groupid) . ' AND F.TABLECODE=T.IDCODE';
		  $sortclause = 'F.SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');

		  $i = CLX_INIT_ZERO;
		  if ($report_type === CLX_NUM_ONE) {//type template
			 $table = new htmlTable(CLX_WIDTH_FIVE);
			 $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
			 $page->header($table);
			 //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_SEL'), dict('GEN_LBL_TABLE'), dict('GEN_LBL_FIELDNAME'),dict('GEN_LBL_FORMAT')));
			 $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_FIELDNAME_WT'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_FORMAT')));
			 while ($record = $page->nextRecord()) {
			     $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($record['FIELDNAME']));
			     $recordid = $record['ID'];
			     $select = new htmlCheckbox('selector_' . $recordid, false);
			     $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			     $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
			         $record['TABLECODE'] . '.' . $record['FIELDNAME'], $record['REPQRYCODE'] . '.' . $record['TABLECODE'] . '_' . $record['FIELDNAME'], dict($entry), dict($this->format_types[$record['FORMAT']])));
			     $i++;
			 }
		  } else {

			 $table = new htmlTable(CLX_WIDTH_FOUR);
			 $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
			 $page->header($table);
			 $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_FUNCTIONS')));
			 while ($record = $page->nextRecord()) {
			     $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($record['FIELDNAME']));
			     $recordid = $record['ID'];
			     $select = new htmlCheckbox('selector_' . $recordid, false);
			     $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			     $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			     $function = array();
			     if (!empty($record['SUM'])) {
			         $function[] = dict('TOOL_ANRX_PROMPT_SUM');
			     }
			     if (!empty($record['AVG'])) {
			         $function[] = dict('TOOL_ANRX_PROMPT_AVG');
			     }
			     if (!empty($record['MIN'])) {
			         $function[] = dict('TOOL_ANRX_PROMPT_MIN');
			     }
			     if (!empty($record['MAX'])) {
			         $function[] = dict('TOOL_ANRX_PROMPT_MAX');
			     }
			     if (!empty($record['COUNT'])) {
			         $function[] = dict('TOOL_ANRX_PROMPT_COUNT');
			     }
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['TABLECODE'] . '.' . $record['FIELDNAME'], dict($entry), implode(', ', $function)));
			     $i++;
			 }
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE_SOMETHING', dict('GEN_LBL_FIELD')), 'CREATE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) and $this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  $reportcode = '';
		  return buildPanelInterface(dict($title), $reportcode . 'COLUMNS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $sql = 'SELECT R.REPORT_TYPE,  (case R.REPORT_TYPE  WHEN 1 THEN	Q.QUERYCODE ELSE R.QUERYCODE END) QUERYCODE FROM CLX_AN_REPORTS R, CLX_AN_REPORT_GROUPS G LEFT JOIN CLX_AN_REPORT_QUERIES Q ON Q.IDCODE = G.REPQRYCODE WHERE R.IDCODE = G.REPORTCODE  AND G.ID = ' . $record['ANRX_groupid'];
        $r = ClxCommon::$db->getOneRecord($sql);
        $report_type = $r['REPORT_TYPE'];
        $querycode = $r['QUERYCODE'];
        unset($r);
        if ($report_type === CLX_NUM_ONE) {
		  $title = ($modify) ? dict('TOOL_ANRX_TITLE2_MODIFY') : dict('TOOL_ANRX_TITLE2_CREATE');
        } else {
		  $title = ($modify) ? dict('TOOL_ANRX_TITLE_MODIFY') : dict('TOOL_ANRX_TITLE_CREATE');
        }
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('ANRX_id', $record['ANRX_id']);
        $form->addArgument('ANRX_groupid', $record['ANRX_groupid']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBQT = &getTool('DBQT');
        $rs = $DBQT->getTableFieldsSql($querycode);
        $pop = new htmlSelect('ANRX_tablecode_fieldname');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $entry = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDFORMLABEL', 'FIELDNAME  = ' . singleQuote($r['FIELDNAME']));
		  $fieldname = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  $fieldcode = $r['TABLECODE'] . '.' . $r['FIELDNAME'];
		  $pop->addItem($fieldname . ' - ' . dict($entry), $fieldcode, ($fieldcode == ($record['ANRX_tablecode'] . '.' . $record['ANRX_fieldname'])));
        }
		ClxCommon::$db->freeResult($rs);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $pop->source());
        // functions
        if ($report_type == CLX_NUM_ZERO) {
		  $celprops = array(CLX_PROP_COLSPAN => CLX_WIDTH_FOUR, CLX_PROP_CLASS => CLX_CLASS_DATA);
		  $table->addCell(dict('TOOL_ANRX_TITLE_FUNCTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
		  $c = new htmlCheckbox('ANRX_sum', (!empty($record['ANRX_sum'])) ? 1 : 0);
		  $table->addCell($c->source() . CLX_CHAR_HTML_NBSP . dict('TOOL_ANRX_PROMPT_SUM'), $celprops);
		  $c = new htmlCheckbox('ANRX_avg', (!empty($record['ANRX_avg'])) ? 1 : 0);
		  $table->addCell($c->source() . CLX_CHAR_HTML_NBSP . dict('TOOL_ANRX_PROMPT_AVG'), $celprops);
		  $c = new htmlCheckbox('ANRX_min', (!empty($record['ANRX_min'])) ? 1 : 0);
		  $table->addCell($c->source() . CLX_CHAR_HTML_NBSP . dict('TOOL_ANRX_PROMPT_MIN'), $celprops);
		  $c = new htmlCheckbox('ANRX_max', (!empty($record['ANRX_max'])) ? 1 : 0);
		  $table->addCell($c->source() . CLX_CHAR_HTML_NBSP . dict('TOOL_ANRX_PROMPT_MAX'), $celprops);
		  $c = new htmlCheckbox('ANRX_count', (!empty($record['ANRX_count'])) ? 1 : 0);
		  $table->addCell($c->source() . CLX_CHAR_HTML_NBSP . dict('TOOL_ANRX_PROMPT_COUNT'), $celprops);
		  $form->addArgument('ANRX_format', CLX_NUM_ZERO);
        } else {
		  //$table->addYesNo(dict('GEN_LBL_FORMAT'),'ANRX_format',$record['ANRX_format'],0);
		  $table->addList(dict('GEN_LBL_FORMAT'), 'ANRX_format', $record['ANRX_format'], $this->format_types, false);
        }
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE'), 'CREATE', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Verify Tool Edit Forms
     * @param array $msg
     * @return bool
     */
    function checkForm(&$msg) {
        $msg = array();
        return count($msg) == CLX_NUM_ZERO;
    }

    /**
     * Builds confirm delete form for tool
     * @param obj $table
     * @param array $record
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['ANRX_tablecode']);
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['ANRX_fieldname']);
    }

    /**
     * Creates reorder list
     * @return string
     */
    function reorderRecords() {
        $sql = 'SELECT GF.TABLECODE, GF.FIELDNAME,  GF.ID FROM CLX_AN_REPORT_GROUP_FUNC GF WHERE GF.GROUPID = ' . UserRequest::$post['groupid'] . '			  ORDER BY GF.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[$record['ID']] = $record['TABLECODE'] . '.' . $record['FIELDNAME'];
        }
		ClxCommon::$db->freeResult($records);
        return $list;
    }

    /**
     * Performs reorder record action
     */
    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
    }

    /**
     * Loads current record on selected id
     *
     * @param integer $id
     * @return array
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['ANRX_id'] = $recordid;
        $record['ANRX_groupid'] = $values['GROUPID'];
        $record['ANRX_tablecode'] = $values['TABLECODE'];
        $record['ANRX_fieldname'] = $values['FIELDNAME'];
        $record['ANRX_seqnum'] = $values['SEQNUM'];
        $record['ANRX_sum'] = $values['SUM'];
        $record['ANRX_avg'] = $values['AVG'];
        $record['ANRX_min'] = $values['MIN'];
        $record['ANRX_max'] = $values['MAX'];
        $record['ANRX_count'] = $values['COUNT'];
        $record['ANRX_count'] = $values['COUNT'];
        $record['ANRX_format'] = $values['FORMAT'];
        return $record;
    }

    /**
     * Update records
     *
     * @return integer
     */
    function saveRecord() {
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        $data = UserRequest::$post['ANRX_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['SUM'] = (!empty(UserRequest::$post['ANRX_sum'])) ? UserRequest::$post['ANRX_sum'] : CLX_NUM_ZERO;
        $record['AVG'] = (!empty(UserRequest::$post['ANRX_avg'])) ? UserRequest::$post['ANRX_avg'] : CLX_NUM_ZERO;
        $record['MIN'] = (!empty(UserRequest::$post['ANRX_min'])) ? UserRequest::$post['ANRX_min'] : CLX_NUM_ZERO;
        $record['MAX'] = (!empty(UserRequest::$post['ANRX_max'])) ? UserRequest::$post['ANRX_max'] : CLX_NUM_ZERO;
        $record['COUNT'] = (!empty(UserRequest::$post['ANRX_count'])) ? UserRequest::$post['ANRX_count'] : CLX_NUM_ZERO;
        $record['FORMAT'] = UserRequest::$post['ANRX_format'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['ANRX_id']);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
    }

    /**
     * Creates record
     *
     * @return integer
     */
    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        $data = UserRequest::$post['ANRX_tablecode_fieldname'];
        $pos = strpos($data, '.');
        $record = array();
        $record['GROUPID'] = UserRequest::$post['ANRX_groupid'];
        $record['TABLECODE'] = substr($data, CLX_NUM_ZERO, $pos);
        $record['FIELDNAME'] = substr($data, $pos + CLX_NUM_ONE);
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'GROUPID=' . singleQuote($record['GROUPID']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $record['SUM'] = (!empty(UserRequest::$post['ANRX_sum'])) ? UserRequest::$post['ANRX_sum'] : CLX_NUM_ZERO;
        $record['AVG'] = (!empty(UserRequest::$post['ANRX_avg'])) ? UserRequest::$post['ANRX_avg'] : CLX_NUM_ZERO;
        $record['MIN'] = (!empty(UserRequest::$post['ANRX_min'])) ? UserRequest::$post['ANRX_min'] : CLX_NUM_ZERO;
        $record['MAX'] = (!empty(UserRequest::$post['ANRX_max'])) ? UserRequest::$post['ANRX_max'] : CLX_NUM_ZERO;
        $record['COUNT'] = (!empty(UserRequest::$post['ANRX_count'])) ? UserRequest::$post['ANRX_count'] : CLX_NUM_ZERO;
        $record['FORMAT'] = UserRequest::$post['ANRX_format'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
        return $recordid;
    }

    /**
     * Deletes record on selected id
     *
     * @param integer $id
     */
    function deleteRecordSk($id) {
        $callingpage = peekHistory();
        $reportcode = $callingpage['reportcode'];
        ClxFreeReturn(ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $ANRP = &getTool('ANRP');
        $ANRP->versionIncrease('IDCODE', $reportcode);
    }

    /**
     * Defines default record on initializacion
     *
     * @return array
     */
    function defaultRecord() {
        $callingpage = peekHistory();
        $groupid = $callingpage['groupid'];
        $record = array();
        $record['ANRX_id'] = CLX_INIT_ZERO;
        $record['ANRX_groupid'] = $groupid;
        $record['ANRX_tablecode'] = '';
        $record['ANRX_fieldname'] = '1.0.0';
        $record['ANRX_seqnum'] = '0';
        $record['SUM'] = '0';
        $record['AVG'] = '0';
        $record['MIN'] = '0';
        $record['MAX'] = '0';
        $record['COUNT'] = '0';
        $record['ANRX_format'] = CLX_INIT_ZERO;
        return $record;
    }

}
