<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Business Process
 * @abstract		Business Process Manager
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Included label
 */
class bp_processes extends clx_tool {

    var $expires_on_uom;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'BPPR';
        $this->name = 'Business Process';
        $this->system = 'Business Process';
        $this->version = '5.5.0.1';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_BP_PROCESSES';
        $this->record_name_singular = 'GEN_LBL_BUSINESS_PROCESS';
        $this->record_name_plural = 'GEN_LBL_BUSINESS_PROCESSES';
        $this->primary_key = array('PROCESSCODE');
        $this->primary_field = 'PROCESSCODE';
        $this->secondary_field = 'IDNAME';
        $this->expires_on_uom = array(CLX_NUM_ZERO => 'GEN_LBL_NEVER', CLX_NUM_ONE => 'GEN_LBL_MINUTE-S', CLX_NUM_60 => 'GEN_LBL_HOUR-S', CLX_NUM_1440 => 'GEN_LBL_DAY-S', CLX_NUM_43200 => 'GEN_LBL_MONTH-S');
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_BPPR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     CLX_CLASS_TITLE => 'GEN_LBL_PROCESS',
			     'entries' => array(
			         array('label' => 'GEN_LBL_PROCESSCODE',
					   'fields' => array(
						  array('name' => 'processcode',
						      'type' => 'string',
						      'default' => '',
						      'size' => '16',
						      'displaysize' => '16',
						      'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject',
								    'required' => '1'
								), array('type' => 'unique',
								    'fieldname' => 'processcode',
								    'tablename' => $this->tablename)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_IDNAME',
					   'fields' => array(
						  array('name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64',
						      'displaysize' => '48',
						      'validations' => array(
								array('type' => 'string',
								    'required' => '1'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_DESCRIPTION',
					   'fields' => array(
						  array('name' => 'description',
						      'type' => 'text',
						      'default' => '',
						      'cols' => '48',
						      'rows' => '4'
						  )
					   )
			         ), array('label' => 'GEN_LBL_STATUS',
					   'fields' => array(
						  array('name' => 'status',
						      'type' => CLX_CLASS_LIST,
						      'default' => '1',
						      'options' => $this->status)
					   )
			         ), array('label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode',
						      'type' => 'lookup',
						      'default' => '',
						      'tool' => 'DBMO',
						      'validations' => array(
								array('type' => 'sqlobject',
								    'required' => '1'
								), array('type' => 'lookup',
								    'fieldname' => 'idcode',
								    'tablename' => 'CLX_DB_MODULES'
								)
						      )
						  )
					   )
			         )
			     )
			 ), array('type' => 'normal',
			     CLX_CLASS_TITLE => 'GEN_LBL_EXPIRE_OPTIONS',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CASE_EXPIRES_ON',
					   'fields' => array(
						  array('name' => 'expires_on',
						      'type' => 'string',
						      'default' => '',
						      'size' => '4',
						      'displaysize' => '4',
						      'default' => '0',
						      'validations' => array(
								array('type' => 'number',
								    'required' => '1'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_CASE_EXPIRES_ON_UOM',
					   'fields' => array(
						  array('name' => 'expires_on_uom',
						      'type' => CLX_CLASS_LIST,
						      'default' => '',
						      'options' => $this->expires_on_uom, 'validations' => array(
								array('type' => 'number',
								    'required' => '1'
								)
						      )
						  )
					   )
			         )
			     )
			 ), array('type' => 'normal',
			     CLX_CLASS_TITLE => 'GEN_LBL_MSG_OPTIONS',
			     'entries' => array(
			         array('label' => 'GEN_LBL_NOTIFY_STATUS',
					   'fields' => array(
						  array('name' => 'notify_status',
						      'type' => CLX_CLASS_LIST,
						      'default' => '',
						      'options' => $this->notify_status
						  )
					   )
			         ), array('label' => 'GEN_LBL_TOUSER',
					   'fields' => array(
						  array('name' => 'touser',
						      'type' => 'lookup',
						      'default' => '',
						      'readonly' => false,
						      'tool' => 'ACUS',
						      'validations' => array(
								array('type' => 'lookup',
								    'fieldname' => 'username',
								    'tablename' => 'CLX_AC_USERS'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_CREATE_TASK',
					   'fields' => array(
						  array('name' => 'create_task',
						      'type' => 'radiobutton',
						      'checked' => false,
						      CLX_CLASS_PROMPT => '',
						      'value' => '0',
						      'options' => $this->yesno, 'validations' => array(
								array('type' => 'number',
								    'required' => '0'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_CREATE_ALERT',
					   'fields' => array(
						  array('name' => 'create_alert',
						      'type' => 'radiobutton',
						      'checked' => false,
						      CLX_CLASS_PROMPT => '',
						      'value' => '0',
						      'options' => $this->yesno, 'validations' => array(
								array('type' => 'number',
								    'required' => '0'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_SEND_EMAIL',
					   'fields' => array(
						  array('name' => 'send_email',
						      'type' => 'radiobutton',
						      'checked' => false,
						      CLX_CLASS_PROMPT => '',
						      'value' => '0',
						      'options' => $this->yesno, 'validations' => array(
								array('type' => 'number',
								    'required' => '0'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_SUBJECT',
					   'fields' => array(
						  array('name' => 'subject',
						      'type' => 'string',
						      'default' => '',
						      'size' => '255',
						      'displaysize' => '35',
						      'validations' => array(
								array('type' => 'string',
								    'required' => '0'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_MESSAGE',
					   'fields' => array(
						  array('name' => 'message',
						      'type' => 'text',
						      'default' => '',
						      'cols' => '48',
						      'rows' => '4'
						  )
					   )
			         )
			     )
			 ), array('type' => 'normal',
			     CLX_CLASS_TITLE => 'GEN_LBL_RECORD_INFO',
			     'entries' => array(
			         array('label' => 'GEN_LBL_VERSION',
					   'fields' => array(
						  array('name' => 'version',
						      'type' => 'string',
						      'default' => '1.0.0',
						      'size' => '32',
						      'displaysize' => '16',
						      'validations' => array(
								array('type' => 'string',
								    'required' => '1'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_VERSION_COUNTER',
					   'fields' => array(
						  array('name' => 'version_counter',
						      'type' => 'string',
						      'default' => '1.0.0',
						      'readonly' => true,
						      'size' => '32',
						      'displaysize' => '16',
						      'validations' => array()
						  )
					   )
			         ), array('label' => 'GEN_LBL_CDATE',
					   'fields' => array(
						  array('name' => 'cdate',
						      'type' => 'dati',
						      'dontmodify' => '1',
						      'displaysize' => '35',
						      'readonly' => false,
						      'options' => getNow('Y-m-d h:i:s'),
						      'validations' => array(
								array('type' => 'string'
								)
						      )
						  )
					   )
			         ), array('label' => 'GEN_LBL_MDATE',
					   'fields' => array(
						  array('name' => 'mdate',
						      'type' => 'dati',
						      'dontmodify' => '1',
						      'displaysize' => '35',
						      'readonly' => false,
						      'options' => getNow('Y-m-d h:i:s'),
						      'validations' => array(
								array('type' => 'string'
								)
						      )
						  )
					   )
			         )
			     )
			 )
		  )
        );
        $this->viewform = &$this->editform;
        $this->toollist = array('name' => $this->tablename, CLX_CLASS_TITLE => $this->record_name_plural, 'fields' => array(
			 array('name' => 'ID',
			     'type' => 'number'
			 ), array('name' => 'PROCESSCODE',
			     'type' => 'string'
			 ), array('name' => 'IDNAME',
			     'type' => 'string'
			 ), array('name' => 'DESCRIPTION',
			     'type' => 'string'
			 ), array('name' => 'MODULECODE',
			     'type' => 'string'
			 ), array('name' => 'STATUS',
			     'type' => CLX_CLASS_LIST,
			     'label' => 'GEN_LBL_STATUS',
			     'options' => $this->status, 'default' => '1'
			 ), array('name' => 'CDATE',
			     'type' => 'dati',
			     'dontmodify' => '1',
			     'displaysize' => '35',
			     'readonly' => false,
			     'validations' => array(
			         array('type' => 'string'
			         )
			     )
			 ), array('name' => 'MDATE',
			     'type' => 'dati',
			     'dontmodify' => '1',
			     'displaysize' => '35',
			     'readonly' => false,
			     'validations' => array(
			         array('type' => 'string'
			         )
			     )
			 ), array('name' => 'VERSION',
			     'type' => 'fieldgroup',
			     'fields' => array(
			         array('name' => 'VERSION',
					   'type' => 'string',
					   'sufix' => '.'
			         ), array('name' => 'VERSION_COUNTER',
					   'type' => 'number'
			         )
			     )
			 )
		  )
        );
        $this->quickfilterform = array(CLX_CLASS_TITLE => 'TOOL_BPPR_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_PROCESSCODE',
					   'fields' => array(
						  array('name' => 'processcode',
						      'type' => 'string',
						      'default' => '',
						      'size' => '16',
						      'displaysize' => '16'
						  )
					   )
			         ), array('label' => 'GEN_LBL_IDNAME',
					   'fields' => array(
						  array('name' => 'idname',
						      'type' => 'string',
						      'default' => '',
						      'size' => '64'
						  )
					   )
			         ), array('label' => 'GEN_LBL_MODULECODE',
					   'fields' => array(
						  array('name' => 'modulecode',
						      'type' => 'lookup',
						      'default' => '',
						      'tool' => 'DBMO'
						  )
					   )
			         )
			     )
			 )
		  )
        );
        if (empty($_SESSION))
		  $_SESSION ['user_record'] ['username'] = 'system';
        $this->executedby = $_SESSION ['user_record'] ['username'];
        $this->batch_pid = getmypid() . '.' . getmyinode();
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::install()
     */
    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_BUSINESS_PROCESS:MENU_' . $this->code, 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::toolTableStructure()
     */
    function toolTableStructure() {
        $fields = array();
        $fields ['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields ['PROCESSCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields ['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        // Expiration
        $fields ['EXPIRES_ON'] = array('type' => 'int(8) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['EXPIRES_ON_UOM'] = array('type' => 'int(4) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        // Notification options
        $fields ['TOUSER'] = array('type' => 'varchar(30)', 'null' => 'YES');
        $fields ['NOTIFY_STATUS'] = array('type' => 'varchar(10)', 'null' => 'YES');
        $fields ['SUBJECT'] = array('type' => 'varchar(255)', 'null' => 'YES');
        $fields ['MESSAGE'] = array('type' => 'tinytext');
        $fields ['CREATE_TASK'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields ['CREATE_ALERT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields ['SEND_EMAIL'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO);
        $fields ['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields ['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields ['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields ['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields ['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields ['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields ['STATUS'] = array('type' => 'varchar(1)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes [$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes [$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'PROCESSCODE');
        $indexes [$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::pack()
     */
    function pack($record) {
        $o = $this->packRecord($record);
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'BPLA', 'SELECT * FROM CLX_BP_LANES WHERE PROCESSCODE =' . singleQuote($record ['PROCESSCODE']));
        $this->packsub($objs, 'BPTR', 'SELECT * FROM CLX_BP_TRANSITIONS WHERE PROCESSCODE =' . singleQuote($record ['PROCESSCODE']));
        $this->packsub($objs, 'BPCT', 'SELECT * FROM CLX_BP_CATEGORIES WHERE PROCESSCODE =' . singleQuote($record ['PROCESSCODE']));
        $o->addObject($objs);
        return $o;
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::unpack()
     */
    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        ClxCommon::$db->deleteRecords($this->tablename, $this->primary_field . '=' . singleQuote($record [$this->primary_field]));
        ClxCommon::$db->createRecord($this->tablename, $record, false);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = &getTool($toolcode);
			 $tool->unpack(array('package' => $o));
		  }
        }
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::action()
     */
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 default :
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . '[BP_PROCESSES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::checkForm()
     */
    function checkForm(&$msg) {
        $result = parent::checkForm($msg);
        return $result;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::searchForm()
     */
    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server ['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request [CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        //$table->addCell(dict('TOOL_BPPR_TITLE_SEARCH'), CLX_PROP_CLASS, CLX_CLASS_TITLE, CLX_PROP_COLSPAN, CLX_WIDTH_TWO);
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('BPPR_processcode', UserRequest::$post))
		  UserRequest::$post ['BPPR_processcode'] = '';
        if (!array_key_exists('BPPR_idname', UserRequest::$post))
		  UserRequest::$post ['BPPR_idname'] = '';
        if (!array_key_exists('BPPR_modulecode', UserRequest::$post))
		  UserRequest::$post ['BPPR_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'BPPR_processcode', UserRequest::$post ['BPPR_processcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'BPPR_idname', UserRequest::$post ['BPPR_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'BPPR_modulecode', UserRequest::$post ['BPPR_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, CLX_PROP_CLASS, CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN, CLX_WIDTH_TWO);
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post ['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post ['BPPR_processcode'] !== '') {
			 $criteria .= 'PROCESSCODE LIKE ' . singleQuote(UserRequest::$post ['BPPR_processcode'] . '%');
		  }
		  if (UserRequest::$post ['BPPR_idname'] !== '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post ['BPPR_idname'])) . '%');
		  }
		  if (UserRequest::$post ['BPPR_modulecode'] !== '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['BPPR_modulecode']) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), CLX_PROP_CLASS, CLX_CLASS_ALERT, CLX_PROP_COLSPAN, CLX_WIDTH_TWO);
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), CLX_PROP_CLASS, CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN, CLX_WIDTH_TWO);
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request [CLX_PROP_TARGET]) . ', ' . quote($record ['PROCESSCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, '<page t=bp_process></page><a href=' . singleQuote($js) . '>' . $record ['PROCESSCODE'] . '</a>', $record ['IDNAME']);
			     $i ++;
			 }
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::confirmDeleteFill()
     */
    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record ['BPPR_processcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record ['BPPR_processcode']);
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::viewFormFill()
     */
    function viewFormFill($record) {
        addHistory('processcode', $record ['BPPR_processcode']);
        $BPLA = &getTool('BPLA'); // Secciones lanes
        $contents = $BPLA->makepanel($record ['BPPR_processcode']);
        $BPCT = &getTool('BPCT');
        $contents .= $BPCT->makepanel($record ['BPPR_processcode']);
        $BPTR = &getTool('BPTR');
        $contents .= $BPTR->makepanel($record ['BPPR_processcode']);
        return $contents;
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::filterFormFill()
     */
    function filterFormFill($msg, &$table) {
        if (!array_key_exists('BPPR_processcode', UserRequest::$post))
		  UserRequest::$post ['BPPR_processcode'] = '';
        if (!array_key_exists('BPPR_idname', UserRequest::$post))
		  UserRequest::$post ['BPPR_idname'] = '';
        if (!array_key_exists('BPPR_modulecode', UserRequest::$post))
		  UserRequest::$post ['BPPR_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'BPPR_processcode', UserRequest::$post ['BPPR_processcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'BPPR_idname', UserRequest::$post ['BPPR_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'BPPR_modulecode', UserRequest::$post ['BPPR_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('BPPR_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('GEN_LBL_MODULECODE'), 'processcode', false);
        $sort->addItem(dict('GEN_LBL_CODE'), 'processcode', false);
        $sort->addItem(dict('GEN_LBL_NAME'), 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::setfilter()
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post ['BPPR_processcode'])) {
		  $filter .= $boolean . ' PROCESSCODE LIKE ' . singleQuote(UserRequest::$post ['BPPR_processcode'] . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['BPPR_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post ['BPPR_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post ['BPPR_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post ['BPPR_modulecode']) . '%');
		  $boolean = 'AND';
        }
        $_SESSION ['paging'] ['BPPRList'] ['sortby'] = UserRequest::$post ['BPPR_sort'];
        $_SESSION ['paging'] ['BPPRList'] ['criteria'] = $filter;
        $_SESSION ['paging'] ['BPPRList'] ['startat'] = '0';
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::clearfilter()
     */
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	////////////////////////////////////////////////// 
    /*function clearfilter() {
        $_SESSION ['paging'] ['BPPRList'] ['criteria'] = '';
    }*/

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::duplicateRecord()
     */
    function duplicateRecord() {
        return $this->createRecord();
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::deleteRecord()
     */
    function deleteRecordSk($id) {
        $processcode = ClxCommon::$db->getField($this->tablename, 'PROCESSCODE', 'ID=' . $id);
        $TOOL = &getTool('BPLA');
        $rs = ClxCommon::$db->getRecords($TOOL->tablename, 'PROCESSCODE=' . singleQuote($processcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $TOOL->deleteRecordSk($r ['ID']);
        }
        $TOOL = &getTool('BPCT');
        $rs = ClxCommon::$db->getRecords($TOOL->tablename, 'PROCESSCODE=' . singleQuote($processcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $TOOL->deleteRecordSk($r ['ID']);
        }
        $TOOL = &getTool('BPTR');
        $rs = ClxCommon::$db->getRecords($TOOL->tablename, 'PROCESSCODE=' . singleQuote($processcode));
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $TOOL->deleteRecordSk($r ['ID']);
        }
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

// end method
    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::defaultRecord()
     */
    function defaultRecord() {
        $record = parent::defaultRecord();
        $record ['BPPR_username'] = $_SESSION ['user_record'] ['username'];
        $record ['BPPR_version'] = '1.0.0';
        $record ['BPPR_status'] = '1';
        $record ['BPPR_cdate'] = getNow('Y-m-d h:i:s');
        $record ['BPPR_mdate'] = getNow('Y-m-d h:i:s');
        return $record;
    }

    /**
     * (non-PHPdoc)
     *
     * @see clx_tool::versionIncrease()
     */
    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

// end method
    function start_process($processcode) {
        // Lee el proceso.
        // Genera Caso
    }

}

// end class
