<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_export_executions extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBEC';
        $this->name = 'Export Executions';
        $this->system = 'Database';
        $this->version = '0.1.2.0';

        $this->accesskey = 'DBIM';


        $this->tablename = 'CLX_DB_EXPORT_EXECUTIONS';
        $this->record_name_singular = 'export execution';
        $this->record_name_plural = 'export executions';
        $this->primary_field = 'ID';
        $this->secondary_field = 'EXPORTID';
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['EXPORTCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['RECORD_QTY'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['OWNERCODE'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $fields['FILENAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'EXPORTCODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_EXPORT_EXECUTIONS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBECPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('exportcode', $groupcode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_DB_EXPORT_EXECUTIONS';
		  $whereclause = 'EXPORTCODE=' . singleQuote($groupcode);
		  $sortclause = 'CDATE DESC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->setProp('selector', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FIVE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_DATE'), dict('GEN_LBL_FILENAME'), dict('GEN_LBL_RECORD_QTY'), dict('GEN_LBL_BY'), dict('GEN_LBL_DESCRIPTION')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(xformat::datetime($record['CDATE']), $record['FILENAME'], $record['RECORD_QTY'], $record['OWNERCODE'], xFormat::text($record['DESCRIPTION'])));
			 $i++;
		  }
		  $form->addContents($page->footers($table, ''));
		  return buildPanelInterface(dict('TOOL_DBEC_TITLE_LIST'), 'EXPORT_EXECS_' . $groupcode, $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

}
