<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (vi., 03 nov. 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_query_fields extends clx_tool {

    //see clx_tools var $format_types = array(0=>'GEN_LBL_DBQF_DEFAULT', 1=>'GEN_LBL_DBQF_NO_FORMAT', 2=>'GEN_LBL_DBQF_FORMAT', 3=>'GEN_LBL_DBQF_SECNAME', 4=>'GEN_LBL_DBQF_SECNAMECOD', 5=>'GEN_LBL_DBQF_NAMESECNAME', 6=>'GEN_LBL_DBQF_NAMESECCODE');
    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'DBQF';
        $this->name = 'Query Fields';
        $this->system = 'Database';
        $this->version = '0.3.3.2';


        $this->accesskey = 'DBQY';

        $this->tablename = 'CLX_DB_QUERY_FIELDS';
        $this->record_name_singular = 'query field';
        $this->record_name_plural = 'query fields';
        $this->secondary_field = 'TABLECODE';
        $this->terciary_field = 'FIELDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('QUERYCODE', 'TABLECODE', 'FIELDNAME', 'SEQNUM');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['SORT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['VISIBLE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FILTER'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['GROUPED'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['EXPRESSION'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
		$fields['POSSELECT'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CURRENCY_FIELD_TBL'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['CURRENCY_FIELD'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['FIELD_FORMAT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        //CURRENCY_FIELD_TBL
        //CURRENCY_FIELD
        //FIELD_FORMAT: 0= DEfault Nombre(codigo), 1 = Solo Codigo, 2 = Solo Secundario name, 4 = Nombre (Secundario)
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE,SEQNUM');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'FIELDNAME');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE,VISIBLE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function unpack($parameters) {  // not working?
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['FIELDNAME'] = uppercase($record['FIELDNAME']);
        $record['EXPRESSION'] = stripslashes($record['EXPRESSION']);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function makepanel($querycode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBQFPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('querycode', $querycode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = 't.IDCODE TABLECODE, t.IDNAME TABLENAME, qf.FIELDNAME, qf.SORT, qf.VISIBLE, qf.FILTER, qf.GROUPED, qf.EXPRESSION, qf.CURRENCY_FIELD, qf.FIELD_FORMAT,qf.POSSELECT';
		  $fromclause = 'CLX_DB_QUERY_FIELDS qf, CLX_DB_TABLES t';
		  $whereclause = 'qf.TABLECODE = t.IDCODE and UPPER(qf.QUERYCODE)=' . singleQuote(UPPERCASE($querycode));
		  $sortclause = 'qf.SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('selector', false);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_EIGHT);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TABLENAME'), dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_SORT'), dict('GEN_LBL_VISIBLE'), dict('GEN_LBL_FILTER'), dict('GEN_LBL_GROUP'), dict('GEN_LBL_EXPRESSION'), dict('GEN_LBL_FIELD_FORMAT')));
		  $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
		  $sort = array(dict('GEN_LBL_NO'), dict('GEN_LBL_ASCENDING'), dict('GEN_LBL_DESCENDING'));
		  $field_format = array(dict('GEN_LBL_DBQF_DEFAULT'), dict('GEN_LBL_DBQF_NO_FORMAT'), dict('GEN_LBL_DBQF_FORMAT'), dict('GEN_LBL_DBQF_SECNAME'), dict('GEN_LBL_DBQF_SECNAMECOD'), dict('GEN_LBL_DBQF_NAMESECNAME'), dict('GEN_LBL_DBQF_NAMESECCODE'));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 if (empty($record['FILTER']))
			     $record['FILTER'] = CLX_NUM_ZERO;
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
			     $record['TABLENAME'] . ' (' . $record['TABLECODE'] . ')', $record['FIELDNAME'], $sort[$record['SORT']], $yesno[$record['VISIBLE']], $yesno[$record['FILTER']], $yesno[$record['GROUPED']], $record['EXPRESSION'],
			     $field_format[$record['FIELD_FORMAT']]));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EDIT'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_DBQF_TITLE_LIST'), $querycode . '_FIELDS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {

        $contents = '';
        $select_size = CLX_NUM_12;
        $callingpage = peekHistory();
        $querycode = $callingpage['querycode'];
        $available_fields = $this->getFields($querycode);
        $available_dfields = $this->getdFields($querycode);

        // array que obtiene los campos de la tabla TESTS
        $scripts = setJavaScriptTag(getJavaScriptFilename('DBQF_editform.js'), true);
        $tables = array();
        $fields = array();
        $lasttable = '';
        $tab_count = -1 * CLX_NUM_ONE;

        $tab_count_tableref = array();
        $fld_count = CLX_INIT_ZERO;
        $js = 'var tables = new Array();' . CLX_CHAR_NL . 'var fields = new Array();' . CLX_CHAR_NL . 'var dfields = new Array();' . CLX_CHAR_NL;
        foreach ($available_fields as $record) {
		  if ($lasttable != $record['TABLECODE']) {
			 $lasttable = $record['TABLECODE'];
			 $tables[$record['TABLECODE']] = array('code' => $record['TABLECODE'], 'name' => $record['TABLENAME']);
			 $tab_count++;

			 $tab_count_tableref[$record['TABLECODE']] = $tab_count;
			 $js .= 'tables[' . $tab_count . '] = ' . singleQuote($record['TABLENAME'] . ' (' . $record['TABLECODE'] . ')') . ';' . CLX_CHAR_NL;
			 $js .= 'fields[' . $tab_count . '] = new Array();' . CLX_CHAR_NL;
			 $fld_count = CLX_INIT_ZERO;
		  }
		  $fields[$lasttable][] = $lasttable . '||' . $record['FIELDNAME'];
		  $js .= 'fields[' . $tab_count . '][' . $fld_count . '] = ' . singleQuote($lasttable . '||' . $record['FIELDNAME']) . ';' . CLX_CHAR_NL;
		  $fld_count++;
        }
        $js .= 'updateFields();' . CLX_CHAR_NL;
        $lasttable = '';
        $tab_count = -1 * CLX_NUM_ONE;
        $fld_count = CLX_INIT_ZERO;
        //Adding Derived Fields
        foreach ($available_dfields as $record) {
		  if ($lasttable != $record['TABLECODE']) {
			 $lasttable = $record['TABLECODE'];
			 $tables[$record['TABLECODE']] = array('code' => $record['TABLECODE'], 'name' => $record['TABLENAME']);
			 $tab_count = $tab_count_tableref[$record['TABLECODE']];
			 $js .= 'tables[' . $tab_count . '] = ' . singleQuote($record['TABLENAME'] . ' (' . $record['TABLECODE'] . ')') . ';' . CLX_CHAR_NL;
			 $js .= 'dfields[' . $tab_count . '] = new Array();' . CLX_CHAR_NL;
			 $fld_count = CLX_INIT_ZERO;
		  }
		  $fields[$lasttable][] = $lasttable . '||' . $record['FIELDNAME'];
		  $js .= 'dfields[' . $tab_count . '][' . $fld_count . '] = ' . singleQuote($lasttable . '||' . $record['FIELDNAME']) . ';' . CLX_CHAR_NL;
		  $fld_count++;
        }
        $js .= 'updatedFields();' . CLX_CHAR_NL;

        $scripts .= setJavaScriptTag($js, false);

        // The Form ---------------------------------------------------------------------------------------------

        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArguments(array(CLX_PROP_TO => $this->code, 'ac' => $ac, 'confirm' => 1, 'idlist' => toString($idlist), 'querycode' => $querycode, 'selected' => ''));
        $form->setProp('onsubmit', 'return processForm()');

        // Table Title  ---------------------------------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell(dict('TOOL_DBQF_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $form->addContents($table->source());

        // Fields ---------------------------------------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_FIVE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        $table->addCell(dict('TOOL_DBQF_FIELDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FIVE));
        //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_TABLES'), dict('GEN_LBL_FIELDS'),CLX_CHAR_HTML_NBSP,dict('GEN_LBL_DFIELDS'), CLX_CHAR_HTML_NBSP, dict('GEN_LBL_SELECTED'), dict('GEN_LBL_OPTIONS'));
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TABLES'), dict('GEN_LBL_FIELDS'), CLX_CHAR_HTML_NBSP, dict('GEN_LBL_DFIELDS'), CLX_CHAR_HTML_NBSP));
        $pop = new HTMLSelect('DBQF_tables');
        $pop->setProps(array('onChange' => 'updateFields();updatedFields();', 'size' => $select_size));
        $selected = true;
        foreach ($tables as $code => $info) {
		  $pop->addItem($info['name'] . ' (' . $code . ')', $code, $selected);
		  $selected = false;
        }
        $table->addCell($pop->source(), array());
        $pop = new HTMLSelect('DBQF_fields');
        $pop->setProps(array('size' => $select_size, 'multiple' => true));
        $table->addCell($pop->source(), array());
        $buttons = '<input type="button" name="select_btn" value="' . dict('GEN_LBL_SELECT') . '" onclick="selectItem(' . singleQuote('DBQF_fields') . ', ' . singleQuote('DBQF_selected') . ')"><br>';
        $table->addCell($buttons, array());

        // Derived Fields -------------------
        $pop = new HTMLSelect('DBQF_dfields');
        $pop->setProps(array('size' => $select_size, 'multiple' => true));
        $table->addCell($pop->source(), array());
        $buttons = '<input type="button" name="select_btn" value="' . dict('GEN_LBL_SELECT') . '" onclick="selectItem(' . singleQuote('DBQF_dfields') . ', ' . singleQuote('DBQF_selected') . ')"><br>';
        $table->addCell($buttons, array());
        $table->addCells(CLX_CHAR_EMPTY_STRING, array(CLX_CHAR_HTML_NBSP));
        $table->addCell(dict('GEN_LBL_SELECTED'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => '1'));
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => '1'));
        $table->addCell(dict('GEN_LBL_OPTIONS'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_COLSPAN => '2'));
        $table->addCells(CLX_CHAR_EMPTY_STRING, array(CLX_CHAR_HTML_NBSP));

        // Selected Items -------------------
        $pop = new HTMLSelect('DBQF_selected');
        $pop->setProps(array('onChange' => 'updateForm();', 'size' => $select_size, 'multiple' => true));
        $sql = 'SELECT  T.IDCODE TABLECODE, T.IDNAME TABLENAME, QF.FIELDNAME FIELDNAME, QF.SORT SORT, QF.VISIBLE VISIBLE, QF.FILTER FILTER, QF.GROUPED, QF.EXPRESSION, QF.CURRENCY_FIELD, QF.CURRENCY_FIELD_TBL, QF.FIELD_FORMAT  FROM  CLX_DB_TABLES T, CLX_DB_QUERY_FIELDS QF WHERE  T.IDCODE=QF.TABLECODE  AND UPPER(qf.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . '  ORDER BY  QF.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $codes = $record['TABLECODE'] . '||' . $record['FIELDNAME'] .
			     '||' . $record['SORT'] .
			     '||' . $record['VISIBLE'] .
			     '||' . $record['FILTER'] .
			     '||' . $record['GROUPED'] .
			     '||' . $record['EXPRESSION'] .
			     '||' . $record['FIELD_FORMAT'] .
                    '||' . $record['CURRENCY_FIELD_TBL'] . '.' . $record['CURRENCY_FIELD'];
		  $pop->addItem($record['TABLECODE'] . '.' . $record['FIELDNAME'], $codes, false);
        }
		ClxCommon::$db->freeResult($records);
        $table->addCell($pop->source(), array(CLX_PROP_COLSPAN => '1'));
        //Ups and Down --------------------
        $options = '<p style="text-align: center;">'
			 . '<input type="button" name="moveup_btn" value="' . dict('GEN_LBL_MOVEUP') . '" onclick="moveItem(1, ' . singleQuote('DBQF_selected') . ')"><br>'
			 . '<input type="button" name="movedown_btn" value="' . dict('GEN_LBL_MOVEDOWN') . '" onclick="moveItem(-1, ' . singleQuote('DBQF_selected') . ')"><br>'
			 . '<input type="button" name="delete_btn" value="' . dict('GEN_LBL_DELETE') . '" onclick="deleteItem(' . singleQuote('DBQF_selected') . ')"><br></p>';
        $table->addCell($options, array(CLX_PROP_COLSPAN => '1'));
        // Options ----------------
        $options = '';
        $pop = new HTMLSelect('visible');
        $pop->addItem(dict('GEN_LBL_NO'), '0', false);
        $pop->addItem(dict('GEN_LBL_YES'), '1', false);
        $pop->setProp('onChange', 'setOptions();');
        $options .= dict('GEN_LBL_VISIBLE') . ': ' . $pop->source() . '<br>';
        $pop = new HTMLSelect('FILTER');
        $pop->addItem(dict('GEN_LBL_NO'), '0', false);
        $pop->addItem(dict('GEN_LBL_YES'), '1', false);
        $pop->setProp('onChange', 'setOptions();');
        $options .= dict('GEN_LBL_FILTER') . ': ' . $pop->source() . '<br>';

        $pop = new HTMLSelect('sort');
        $pop->addItem(dict('GEN_LBL_NO'), '0', false);
        $pop->addItem(dict('GEN_LBL_ASCENDING'), '1', false);
        $pop->addItem(dict('GEN_LBL_DESCENDING'), '2', false);
        $pop->setProp('onChange', 'setOptions();');
        $options .= dict('GEN_LBL_SORT') . ': ' . $pop->source() . '<br>';
        $pop = new HTMLSelect('grouped');
        $pop->addItem(dict('GEN_LBL_NO'), '0', false);
        $pop->addItem(dict('GEN_LBL_YES'), '1', false);
        $pop->setProp('onChange', 'setOptions();');
        $options .= dict('GEN_LBL_GROUP') . ': ' . $pop->source() . '<br>';
        $pop = new HTMLSelect('posselect');
        $pop->addItem(dict('GEN_LBL_INOUT1'), '0', false);	
		$pop->addItem(dict('GEN_LBL_INOUT2'), '1', false);
        $pop->setProp('onChange', 'setOptions();');
		$options .= dict('POS_SELECT') . ': ' . $pop->source() . '<br>';
        
        //$field = new HTMLField('expression', '', 40, 255);
        $field = new HTMLTextArea('expression', '', CLX_NUM_FIVE, CLX_NUM_40);
        $field->setProp('onChange', 'setOptions();');
        $options .= '<br>' . dict('GEN_LBL_EXPRESSION') . ':<br>' . $field->source() . '<br>';
        $pop = new HTMLSelect('field_format');
        foreach ($this->format_types as $k => $v) {
		  $pop->addItem(dict($v), $k, false);
        }
        $pop->setProp('onChange', 'setOptions();');
        $options .= '<br>' . dict('GEN_LBL_FIELD_FORMAT') . ':<br>' . $pop->source() . '<br>';
        $pop = new HTMLSelect('currency_field');
        $pop->addItem('', '.', false);
        foreach ($available_fields as $k => $v) {
		  $pop->addItem($v['TABLECODE'] . '.' . $v['FIELDNAME'], $v['TABLECODE'] . '.' . $v['FIELDNAME'], false);
        }
        $pop->setProp('onChange', 'setOptions();');
        $options .= '<br>' . dict('GEN_LBL_CURRENCY_FIELD') . ':<br>' . $pop->source() . '<br>';
        $table->addCell($options, array(CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $form->addContents($table->source());

        // Controls ans Such ---------------------------------------------------------------------------------------------
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source()
			 . htmlButton::getButton('cancel', CLX_CHAR_EMPTY_STRING, array())->source();

        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_NUM_ONE));
        $form->addContents($table->source());
        $contents .= $form->source() . $scripts;
        return $contents;
    }

    function checkForm(&$msg) {
        return (empty($msg)) ? true : false;
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $form->addArgument('confirm', '1');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBQF_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('DBQF_querycode', UserRequest::$post)) {
		  UserRequest::$post['DBQF_querycode'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'DBQF_querycode', UserRequest::$post['DBQF_querycode'], CLX_NUM_16, CLX_NUM_16);
        if (!array_key_exists('DBQF_tablecode', UserRequest::$post)) {
		  UserRequest::$post['DBQF_tablecode'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_TABLECODE') . ':', 'DBQF_tablecode', UserRequest::$post['DBQF_tablecode'], CLX_NUM_16, CLX_NUM_16);
        if (!array_key_exists('DBQF_fieldname', UserRequest::$post)) {
		  UserRequest::$post['DBQF_fieldname'] = '';
        }
        $table->addField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBQF_fieldname', UserRequest::$post['DBQF_fieldname'], CLX_NUM_16, CLX_NUM_16);

        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source()
			 . htmlButton::getButton('goto', dict('GEN_LBL_CANCEL'), array('url' => 'javascript:closeWindow();'))->source();


        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();

        // Search Procedure
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $criteria = '';
        $boolean = '';
        if (!empty(UserRequest::$request['DBQF_querycode'])) {
		  $criteria .= $boolean . 'QUERYCODE = ' . singleQuote(UserRequest::$request['DBQF_querycode']);
		  $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['DBQF_querycode'])) {
		  $criteria .= $boolean . 'QUERYCODE LIKE ' . singleQuote(UserRequest::$post['DBQF_querycode'] . '%');
		  $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['DBQF_tablecode'])) {
		  $criteria .= $boolean . 'TABLECODE LIKE ' . singleQuote(UserRequest::$post['DBQF_tablecode'] . '%');
		  $boolean = ' AND ';
        }
        if (!empty(UserRequest::$post['DBQF_fieldname'])) {
		  $criteria .= $boolean . 'FIELDNAME LIKE ' . singleQuote(UserRequest::$post['DBQF_fieldname'] . '%');
		  $boolean = ' AND ';
        }
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
        if ($qty == CLX_NUM_ZERO) {
		  $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        } else {
		  $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
		  $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
		  $i = CLX_INIT_ZERO;
		  while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			 $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['FIELDNAME']) . ');';
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_query_fields></page><a href=' . singleQuote($js) . '>' . $record['FIELDNAME'] . '</a>'));
			 $i++;
		  }
		  ClxCommon::$db->freeResult($records);
        }
        $contents .= $table->source();
        return $contents;
    }

    function loadToolRecord($recordid) {
        $record = array();
        return $record;
    }

    function saveRecord() {
        $querycode = UserRequest::$request['querycode'];
        $fields = explode('-END-', UserRequest::$request['selected']);
        ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)));
        apc_clear_querycode(UserRequest::$request['querycode']);
        $record = array();
        $record['QUERYCODE'] = $querycode;
        $i = CLX_INIT_ZERO;
		$fields_unique=array();
        foreach ($fields as $info) {
		  if (empty($info)) {
			 continue;
		  }

		  list($tablecode, $fieldname, $sort, $visible, $filter, $group, $expression, $field_format, $currency_field,$posselect) = explode('||', $info);
			if (in_array($tablecode.'_'.$fieldname,$fields_unique))
				continue;
			$fields_unique[]=$tablecode.'_'.$fieldname;	
		  $record['TABLECODE'] = $tablecode;
		  $record['FIELDNAME'] = $fieldname;
		  $record['SORT'] = $sort;
		  $record['VISIBLE'] = $visible;
		  $record['FILTER'] = $filter;
		  $record['GROUPED'] = $group;
		  $record['EXPRESSION'] = clx_html_spchrs_decode($expression);
		  $record['SEQNUM'] = $i;
			$record['POSSELECT'] = $posselect;
			
		  $crufl = explode('.', $currency_field);
		  if (!empty($crufl[CLX_NUM_ZERO])) {
			 $record['CURRENCY_FIELD_TBL'] = $crufl[CLX_NUM_ZERO];
			 $record['CURRENCY_FIELD'] = $crufl[CLX_NUM_ONE];
		  } else {
			 $record['CURRENCY_FIELD_TBL'] = 'null';
			 $record['CURRENCY_FIELD'] = 'null';
		  }
		  if ($field_format === '')
			 $field_format = CLX_INIT_ZERO;
		  $record['FIELD_FORMAT'] = $field_format;
		  ClxCommon::$db->createRecord($this->tablename, $record, false);
		  $i++;
        }
        $DBQY = &getTool('DBQY');
        $DBQY->versionIncrease('IDCODE', $querycode);
		apc_clear_querycode($querycode);
    }

    function getFields($querycode) {
        $sql = '  SELECT t.IDCODE TABLECODE, t.IDNAME TABLENAME, tf.FIELDNAME FIELDNAME FROM CLX_DB_TABLES t, CLX_DB_TABLEFIELDS tf, CLX_DB_QUERY_TABLES qt WHERE qt.QUERYCODE = ' . singleQuote($querycode) . ' AND qt.TABLECODE = tf.TABLECODE AND qt.TABLECODE = t.IDCODE ORDER BY qt.TABLECODE, tf.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $fields[] = $record;
        }
		ClxCommon::$db->freeResult($records);
        return $fields;
    }

    /**
     * Get Derived Fields
     */
    function getdFields($querycode) {
        $sql = ' SELECT t.IDCODE TABLECODE, t.IDNAME TABLENAME, tf.FIELDNAME FIELDNAME FROM CLX_DB_TABLES t, CLX_DB_TABLEDFIELDS tf, CLX_DB_QUERY_TABLES qt WHERE qt.QUERYCODE = ' . singleQuote($querycode) . ' AND qt.TABLECODE = tf.TABLECODE AND qt.TABLECODE = t.IDCODE ORDER BY qt.TABLECODE, tf.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $fields[] = $record;
        }
		ClxCommon::$db->freeResult($records);
        return $fields;
    }

    function chooseRecord($name, $default, $data = array(), $display_size = CLX_NUM_32, $maxlength = CLX_NUM_64) {
	$contents = setJavaScriptTag(getJavaScriptFilename('DBQF_searchwindow.js'), true);
//        $contents = getJavaScript('DBQF_searchwindow.js', $variables);
        $code = $this->code;
        $alt = 'TOOL_' . $code . '_ALT_SEARCH';
        $querycode = $data['DBQF_querycode'];
        $searchIcon = getSkinIcon('view', 'javascript:dbqf_searchwindow(' . singleQuote($querycode) . ', ' . singleQuote($name) . ');', dict($alt));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        return $contents . $c->source() . $searchIcon;
    }

}
