<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: jclerma $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2017-11-03 15:09:40 -0400 (Fri, 03 Nov 2017) $
 * @version        $Rev: 9615 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_query_join_relation extends clx_tool {

    var $logicals;
    var $operators;
    var $variables;
    var $debug = false;

    function __construct() {
        $this->code = 'DBQR';
        $this->name = 'Query Join Type Relation';
        $this->system = 'Database';
        $this->version = '0.2.2.1';


        $this->accesskey = 'DBQY';

        $this->tablename = 'CLX_DB_QUERY_JOIN_TYPE_RELATION';
        $this->record_name_singular = 'query join type relation';
        $this->record_name_plural = 'query join type relation';
        $this->primary_field = 'QUERYCODE';
        $this->secondary_field = 'TABLECODE';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('QUERYCODE', 'TABLECODE',  'SEQNUM');
		$this->typeRelation=array(0=>'GEN_LBL_LEFT_JOIN',1=>'GEN_LBL_RIGHT_JOIN',2=>'GEN_LBL_INNER_JOIN',3=>'GEN_LBL_FULL_JOIN');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        
		$fields['QUERYCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['JOIN_TABLE1'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '', 'default' => '');
		$fields['JOIN_TABLE2'] = array('type' => 'varchar(100)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['TYPE_JOIN'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'QUERYCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'SEQNUM');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }
	function unpackRecord($o) {
        $record = array();
        $structure = $this->toolTableStructure();
        $fields = $structure['fields'];
		foreach ($fields as $fieldname => $fieldrec) {
		  if ($fieldname !== 'ID') {
        
			 //$record[$fieldname] = html_entity_decode($o->getSubObjectContents(uppercase($fieldname)));
			 $record[uppercase($fieldname)] = html_entity_decode($o->getSubObjectContents(uppercase($fieldname)));
		  }
        }
        return $record;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
		
        
		
        list($package) = $parameters;
		
        
		
        $record = $this->unpackRecord($package);

        foreach($record as $key=>&$value) {
			if (empty($value) && $value!==0 ) {
				if ($key=='SEQNUM' || $key=='TYPE_JOIN' )
					$value=0;
				else	
					$value=singlequote();
			}
			else {
	
			}
		}
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function makepanel($querycode = null) {	
        if ($this->getAccess()->canRead()) {
            $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBRPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('querycode', $querycode);
		
		  $form->addArgument('ac', 'NOT_DEFINED');
        
			$sql = 'SELECT DISTINCT T.IDCODE TABLECODE, T.IDNAME TABLENAME,QR.VALUE,QR.SEQNUM FROM CLX_DB_TABLES T, CLX_DB_QUERY_JOIN_CRITERIA QR WHERE  T.IDCODE=QR.TABLECODE  AND UPPER(QR.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' ORDER BY QR.SEQNUM';
			$rs = ClxCommon::$db->executeQuery($sql);
			$nameTable=array();
			$CODIGOS='';
			while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
				 $nameTable[$record['TABLECODE']]=$record['TABLENAME'];
				 $CODIGOS.=singleQuote(explode(".",$record['VALUE'])[0]);
			}
			ClxCommon::$db->freeResult($rs);
			if (!empty($CODIGOS)) {
				$CODIGOS=str_replace("''","','",$CODIGOS);
				$sql = 'SELECT DISTINCT T.IDCODE TABLECODE, T.IDNAME TABLENAME,"" VALUE,1000 SEQNUM FROM CLX_DB_TABLES T WHERE  T.IDCODE IN ('.$CODIGOS.') ';
				$rs = ClxCommon::$db->executeQuery($sql);
				while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
					$nameTable[$record['TABLECODE']]=$record['TABLENAME'];
					$CODIGOS.=singleQuote(explode(".",$record['VALUE'])[0]);
				}
				ClxCommon::$db->freeResult($rs);				
			}
        
			$selectclause = 'JOIN_TABLE1, TYPE_JOIN,JOIN_TABLE2';
		  $fromclause =  $this->tablename.' QR ';
		  $whereclause = 'UPPER(QR.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) ;
		  $sortclause = 'SEQNUM';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('selector', false);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_JOIN_TABLE') ,dict('GEN_LBL_TYPE_JOIN'), dict('GEN_LBL_JOIN_TABLE') ));
		  $i = CLX_INIT_ZERO;
		    while ($record = $page->nextRecord()) {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT,  array($nameTable[$record['JOIN_TABLE2']].'('.$record['JOIN_TABLE2'].')' , dict($this->typeRelation[ $record['TYPE_JOIN']]), $nameTable[$record['JOIN_TABLE1']].'('.$record['JOIN_TABLE1'].')' ));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EDIT'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_DBQR_TITLE_LIST'), $querycode . '_TYPE_RELATION', $form->source(), -1 * CLX_NUM_ONE);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $callingpage = peekHistory();
		$here='';
        $querycode = $callingpage['querycode'];
		$relationsJoin=$this->getjoin('DBQY',$querycode);

		$sql = 'SELECT DISTINCT T.IDCODE TABLECODE, T.IDNAME TABLENAME,QR.VALUE,QR.SEQNUM FROM CLX_DB_TABLES T, CLX_DB_QUERY_JOIN_CRITERIA QR WHERE  T.IDCODE=QR.TABLECODE  AND UPPER(QR.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' ORDER BY QR.SEQNUM';
		$rs = ClxCommon::$db->executeQuery($sql);
		$nameTable=array();
		$CODIGOS='';
		while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 $nameTable[$record['TABLECODE']]=$record['TABLENAME'];
			 $CODIGOS.=singleQuote(explode(".",$record['VALUE'])[0]);
		}
		ClxCommon::$db->freeResult($rs);
		$CODIGOS=str_replace("''","','",$CODIGOS);
		if (empty($CODIGOS)) {
			$CODIGOS=singlequote($CODIGOS);	
		}	
		$sql = 'SELECT DISTINCT T.IDCODE TABLECODE, T.IDNAME TABLENAME,"" VALUE,1000 SEQNUM FROM CLX_DB_TABLES T WHERE  T.IDCODE IN ('.$CODIGOS.') ';
		$rs = ClxCommon::$db->executeQuery($sql);
		while ($record = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			 $nameTable[$record['TABLECODE']]=$record['TABLENAME'];
			 $CODIGOS.=singleQuote(explode(".",$record['VALUE'])[0]);
		}	
		ClxCommon::$db->freeResult($rs);		
		$form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		$form->setProp(CLX_PROP_ID, 'DBQRPanel');
		$form->addArgument(CLX_PROP_TO, $this->code);
		$form->addArgument('querycode', $querycode);
		$form->addArgument('ac', 'NOT_DEFINED');
		$form->addArgument('confirm', '1');
		$selectclause = 'JOIN_TABLE1, TYPE_JOIN,JOIN_TABLE2';
		$fromclause =  $this->tablename.' QR ';
		$whereclause = 'UPPER(QR.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) ;
		$sortclause = 'SEQNUM';
		$page = new xPaging();
		$page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		$page->setProp('selector', false);
		$page->setProp('printheaders', false);
		$page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		$table = new htmlTable(CLX_WIDTH_THREE);
		$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBQR_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));

		$page->header($table);

		$table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_JOIN_TABLE') ,dict('GEN_LBL_TYPE_JOIN'), dict('GEN_LBL_JOIN_TABLE') ));
		$i = CLX_INIT_ZERO;
	//	pre($relationsJoin);
		while ($record = $page->nextRecord()) {
			for($t=0;$t<count($relationsJoin);$t++) {
				if	($record['JOIN_TABLE2'] == $relationsJoin[$record['JOIN_TABLE1'].'____'.$record['JOIN_TABLE2']]['JOIN_TABLE2'] and $record['JOIN_TABLE1'] == $relationsJoin[$record['JOIN_TABLE1'].'____'.$record['JOIN_TABLE2']]['JOIN_TABLE1']){
				    $relationsJoin[$record['JOIN_TABLE1'].'____'.$record['JOIN_TABLE2']]['TYPE_JOIN']=$record['TYPE_JOIN']  ;
				}
			}	
		}
		
		foreach($relationsJoin as $key =>$record) {//pre($record);
			$c = new htmlField('DBQR_JOIN_TABLE2[]', $record['JOIN_TABLE2'], 2, 4);
			$table->addCell($nameTable[$record['JOIN_TABLE2']].' ('.$c->source().')', array(CLX_PROP_CLASS => CLX_CLASS_DATA));
			$sel = new htmlSelect('DBQR_TYPE_JOIN[]');
			foreach ($this->typeRelation as $key => $val) {
				$sel->addItem(dict($val), $key, $record['TYPE_JOIN']==$key);
			}
			$table->addCell($sel->source(), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
			$c = new htmlField('DBQR_JOIN_TABLE1[]', $record['JOIN_TABLE1'], 2, 4);
			$table->addCell($nameTable[$record['JOIN_TABLE1']].' ('.$c->source().')', array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		}
		$buttons = htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source()
			 . htmlButton::getButton('cancel', CLX_CHAR_EMPTY_STRING, array())->source();

        $form->addContents($page->footers($table, $buttons));
		$contents .= $form->source();
		return $contents.setJavaScriptTag(getJavaScriptFilename('DBQR_editform.js'), true) ;
    }
	function action($ac, $idlist) {
		$msg=null;
		if ($ac=='MODIFY' and $this->getAccess()->canModify() ) {
			if (isset(UserRequest::$post['confirm'])) {
				if ($this->checkForm($msg)) {
					ClxFreeReturn($this->saveRecord());
					if (count($idlist) > CLX_NUM_ZERO ) {
						$id = array_pop($idlist);
						UserRequest::$post['DBIC_id']=array();
						$record = $this->loadToolRecord($id);
					
						$contents = $this->editForm($ac, $idlist, $record, $msg);
					} else if ( isset($GLOBALS["process"])  ) { 
						$contents="";      
					}
					else if ($ac=='MODIFY') {
						$info = popHistory();
						$contents = executeHistory($info);
					}
					
				} else {
					$contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
				}
			} else {
				$id = array_pop($idlist);
				$record = $this->loadToolRecord($id);
				   
				$contents = $this->editForm($ac, $idlist, $record, $msg);
			}
		}
		return $contents;
    }
    function checkForm(&$msg) {
        return (empty($msg)) ? true : false;
    }

    function loadToolRecord($recordid) {
        $record = array();
        return $record;
    }

    function saveRecord() {
        $querycode = UserRequest::$request['querycode'];
		apc_clear_querycode(UserRequest::$request['querycode']);
		ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(QUERYCODE)=' . singleQuote(UPPERCASE($querycode)));
		$record=array();
	
		$record['QUERYCODE']=UserRequest::$request['querycode'];
	//	pre(UserRequest::$request);
		for ($t=0;$t<count(UserRequest::$request['DBQR_JOIN_TABLE1']);$t++) {
			$record['JOIN_TABLE1']	= UserRequest::$request['DBQR_JOIN_TABLE1'][$t];
			$record['TYPE_JOIN']	= UserRequest::$request['DBQR_TYPE_JOIN'][$t];
			$record['JOIN_TABLE2']	= UserRequest::$request['DBQR_JOIN_TABLE2'][$t];
			ClxCommon::$db->createRecord($this->tablename, $record, false);
		}
		apc_clear_querycode($querycode);	
    }

    function getCriterias($querycode) {
        $criteria = '';
        $criterias = ClxCommon::$db->executeQuery('SELECT T.IDNAME TABLENAME, QC.* FROM CLX_DB_QUERY_JOIN_CRITERIA QC, CLX_DB_TABLES T WHERE UPPER(QC.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND QC.TABLECODE = T.IDCODE AND QC.SPECIAL = 0 ORDER BY QC.SEQNUM');
        $nl = '';
        while ($criteriarec = ClxCommon::$db->getNextRecord($criterias)) {
		  $criteria .= $nl . $criteriarec['LOGICAL'] . '|SEP|' .
			     $criteriarec['OPEN_BRACKETS'] . '|SEP|' .
			     $criteriarec['TABLECODE'] . '|SEP|' .
			     $criteriarec['FIELDNAME'] . '|SEP|' .
			     $criteriarec['OPERATOR'] . '|SEP|' .
			     $criteriarec['TYPE'] . '|SEP|';
		  if ($criteriarec['TYPE'] === 'v') {
			 $criteria .= $criteriarec['VALUE'] . '|DOT|' . dict($this->variables[$criteriarec['VALUE']]);
		  } else {
			 $criteria .= htmlentities($criteriarec['VALUE']);
		  }
		  $criteria .= '|SEP|' . $criteriarec['CLOSE_BRACKETS'];
		  $nl = '|END|';
        }
		ClxCommon::$db->freeResult($criterias);
        return $criteria;
    }
    /**
     * Query method used in ui_listing that builds sql
     * @param string $tool
     * @param string $querycode
     * @param bool $on_efftdate
     */
    function getjoin($tool, $querycode) {
        //init some values
        $queryrec = array('IDCODE', $querycode, 'WHERECLAUSE' => '');
        //   Create a list of tables used in this query (later it will be used to create the from clause, among other things...) */
        switch (kDatabaseType) {
		  case 'oci':
			 $records = ClxCommon::$db->executeQuery('SELECT t.MODULECODE|| ' . "'_'" . '||t.IDNAME TABLENAME, t.IDCODE TABLECODE FROM CLX_DB_QUERY_TABLES qt, CLX_DB_TABLES t WHERE UPPER(qt.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND t.IDCODE = qt.TABLECODE AND qt.INVISIBLE = 0');
			 break;
		  default:
			 $records = ClxCommon::$db->executeQuery('SELECT CONCAT(t.MODULECODE, CONCAT(' . "'_'" . ' , t.IDNAME)) TABLENAME, t.IDCODE TABLECODE FROM CLX_DB_QUERY_TABLES qt, CLX_DB_TABLES t WHERE UPPER(qt.QUERYCODE)=' . singleQuote(UPPERCASE($querycode)) . ' AND t.IDCODE = qt.TABLECODE AND qt.INVISIBLE = 0');
			 break;
        }
        $tables = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $tables[$record['TABLECODE']] = $record['TABLENAME'];
        }
        ClxCommon::$db->freeResult($records);
        $sql = 'SELECT * FROM CLX_DB_QUERY_JOIN_CRITERIA WHERE QUERYCODE = ' . singleQuote($querycode);
        if (empty($this->efftdate_enabled)) {
		  $sql .= ' AND OPERATOR NOT IN (' . singleQuote('EF') . ' , ' . singleQuote('FQ') . ')';
		  $efftdone = true;
        }
        $sql .= 'ORDER BY SEQNUM ASC';

		$records = ClxCommon::$db->executeQuery($sql);
        $nextTablecode = CLX_NUM_65;
		$joinTable=array();
		$orderJoin=array();
		$tablasofjoin=array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
            $is_dyna = (ClxCommon::$db->oneExistsRecords('CLX_DB_DYNRELATIONS', 'TABLECODE=' . singleQuote($record['TABLECODE']) . '  AND RELATION_TYPE = 0') > CLX_NUM_ZERO);
		  if ($is_dyna === true) {
			 
			 $dyn_table_rec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote($record['TABLECODE']));
			 if (!array_key_exists($record['TABLECODE'], $tables)) {
			     $tablecode = chr($nextTablecode);
			     $tables[chr($nextTablecode)] = $dyn_table_rec['MODULECODE'] . '_' . $dyn_table_rec['IDNAME'];
			 } else {
			     $tablecode = $record['TABLECODE'];
			 }
		  } else {
			 $tablecode = $record['TABLECODE'];
		  }
		  //skeep if not enabled effective date
		  if ($record['OPERATOR'] === 'EF' && !$this->efftdate_enabled) {
			 $record['LOGICAL'] = '';
		  }
		  $val_to_compare = $this->getQueryCriteriaValue('', $record);

			$cut=explode('.',$val_to_compare);
			$orderJ='!!!'.implode('!!!',$orderJoin);
			$pos1=strpos('!'.trim($record['TABLECODE']).'!',$orderJ);
			$pos2=strpos('!'.trim($cut[0]).'!',$orderJ);
			
			if ($pos1<=$pos2 ) {
				if (!in_array(trim($record['TABLECODE']),$orderJoin)) {
					$orderJoin[]=trim($record['TABLECODE']);
				}	
				if ($pos2==0) {
					if (!isset($tablasofjoin[$record['TABLECODE'].'____'.$cut[0]])) {
						$tablasofjoin[$cut[0].'____'.$record['TABLECODE']]=array('JOIN_TABLE1'=>$cut[0],'TYPE_JOIN'=>1,'JOIN_TABLE2'=>$record['TABLECODE']);
					}
					if (!in_array(trim($cut[0]),$orderJoin)) {
						$orderJoin[]=trim($cut[0]);
					}	
				}
				else {
					$tablasofjoin[$record['TABLECODE'].'____'.$cut[0]]=array('JOIN_TABLE1'=>$cut[0],'TYPE_JOIN'=>1,'JOIN_TABLE2'=>$record['TABLECODE']);
					
				}
			}	
			else {
				$tablasofjoin[$record['TABLECODE'].'____'.$cut[0]]=array('JOIN_TABLE1'=>$cut[0],'TYPE_JOIN'=>1,'JOIN_TABLE2'=>$record['TABLECODE']);
				if (!in_array(trim($cut[0]),$orderJoin)) {
					$orderJoin[]=trim($cut[0]);
				}
			}
			
        }//End while
		ClxCommon::$db->freeResult($records);
		return $tablasofjoin; 
        
    }
	function getQueryCriteriaValue($tool, $criteria_rec) {
        $value = '';
        $criteria_rec['VALUE'] = ClxCommon::$db->extrac_dbms_sql($criteria_rec['VALUE']);
        switch ($criteria_rec['TYPE']) {
		  case 'f':
			 $value = $criteria_rec['VALUE'];
			 break;
		  case 'k':
			 $value = $criteria_rec['VALUE'];
			 $value = $this->formatCriteriaValue($criteria_rec['OPERATOR'], $value);
			 break;
		  case 'u':
			 $c = $criteria_rec['TABLECODE'] . '_' . $criteria_rec['FIELDNAME'];
			 $value = (array_key_exists($c, UserRequest::$request)) ? UserRequest::$request[$c] : '';
			 if ($this->include_empty_answers === true || ($this->include_empty_answers == false && strlen(trim($value)) > CLX_NUM_ZERO)) {
			     $value = $this->formatCriteriaValue($criteria_rec['OPERATOR'], $value);
			 }
			 break;
		  case 'v':
			 $value = clx_keywords_simple($criteria_rec['VALUE']);
			 $value = $this->formatCriteriaValue($criteria_rec['OPERATOR'], $value);
			 break;
		  default:
			 break;
        }
        return $value;
    }
	 

}
