<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version        $Rev: 9902 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_tabledfields extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'DBTD';
        $this->name = 'Table Derived Fields';
        $this->system = 'Database';
        $this->version = '1.0.0.0';



        $this->accesskey = 'DBTB';
        $this->tablename = 'CLX_DB_TABLEDFIELDS';
        $this->record_name_singular = 'table derived field';
        $this->record_name_plural = 'table derived fields';
        $this->primary_field = 'TABLECODE';
        $this->secondary_field = 'FIELDNAME';
        $this->seqnum_field = 'SEQNUM';
        $this->primary_key = array('TABLECODE', 'FIELDNAME');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['SEQNUM'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'TABLECODE,FIELDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $record['FIELDNAME'] = uppercase($record['FIELDNAME']);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'INCLUDE':
			     if ($this->getAccess()->canModify()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  $this->createRecordVoid_();
						  $info = popHistory();
						  $to = &getTool($info[CLX_PROP_TO]);
						  $contents = $to->action($info['ac'], toArray($info['idlist']));
					   } else {
						  if (array_key_exists('ERROR', $msg) && $msg['ERROR'] === 'NO_FIELD') {
						      pushHistory($this->code, 'INCLUDE_NEW', $idlist, 'tablecode', UserRequest::$post['DBTD_tablecode']);
						      //pushHistory($this->code, 'INCLUDE_NEW', $idlist, 'tablecode', UserRequest::$post['DBTD_tablecode']); // this is not an error! (I think)
						      $DBFL = &getTool('DBFL');
						      $record = $DBFL->defaultRecord();
						      $record['DBFL_fieldname'] = UserRequest::$post['DBTD_fieldname'];
						      $contents = $DBFL->editForm('CREATE', array(), $record, array());
						  } else {
						      $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
						  }
					   }
			         } else {
					   $record = $this->defaultRecord();
					   $contents = $this->editForm($ac, array(), $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEDFIELDS-CAN_MODIFY_1][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'INCLUDE_NEW':
			     if ($this->getAccess()->canModify()) {
			         if (!empty(UserRequest::$request['DBFL_fieldname'])) {
					   UserRequest::$post['DBTD_fieldname'] = UserRequest::$request['DBFL_fieldname'];
					   //$info = popHistory();
					   $info = peekHistory();
					   UserRequest::$post['DBTD_tablecode'] = $info['tablecode'];
					   $this->createRecordVoid_();
			         } else {
					   //popHistory();
			         }
			         //$info = popHistory();
			         $info = peekHistory();
			         $to = &getTool($info[CLX_PROP_TO]);
			         $contents = $to->action($info['ac'], toArray($info['idlist']));
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEDFIELDS-CAN_MODIFY_2][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLEDFIELDS-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makepanel($tablecode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'tabledfieldspanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $form->addArgument('DBTD_tablecode', $tablecode);
		  $selectclause = '*';
		  $fromclause = 'CLX_DB_TABLEDFIELDS';
		  $whereclause = 'TABLECODE=' . singleQuote($tablecode);
		  $sortclause = 'SEQNUM ASC';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_7);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_FORMLABEL'), dict('GEN_LBL_REQUIRED'), dict('GEN_LBL_DESCRIPTION')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $fieldrec = ClxCommon::$db->getRecord('CLX_DB_FIELDS', 'FIELDNAME=' . singleQuote($record['FIELDNAME']));
			 $fieldname = $record['FIELDNAME'];
			 if (empty($fieldrec['FIELD_REQUIRED'])) {
			     $fieldrec['FIELD_REQUIRED'] = CLX_INIT_ZERO;
			 }
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($fieldname, $fieldrec['FIELDTYPE'], $fieldrec['VERSION'] . '-' . $fieldrec['VERSION_COUNTER'], dict($fieldrec['FIELDFORMLABEL']), dict($this->yesno[$fieldrec['FIELD_REQUIRED']]), xFormat::text($fieldrec['DESCRIPTION'])));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_EXCLUDE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_INCLUDE', dict('GEN_LBL_FIELD')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  if (($page->getProp('qty') > CLX_NUM_ONE) && ( $this->getAccess()->canModify())) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_REORDER'), array('action' => 'REORDER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  $recordcode = 'DBTD'; // Verify why recordcode?
		  return buildPanelInterface(dict('TOOL_DBTD_TITLE_FIELDS'), $recordcode . '_FIELDS', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $contents = '';
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBTD_TITLE_MODIFY') : dict('TOOL_DBTD_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('DBTD_id', (!empty($record['DBTD_id'])) ? $record['DBTD_id'] : '');
        $form->addArgument('DBTD_tablecode', $record['DBTD_tablecode']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $DBFL = &getTool('DBFL');
        $contents .= $DBFL->placeChooseField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBTD_fieldname', $record['DBTD_fieldname'], $table);
        $form->addContents($table->source() . '<br>');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $c = new htmlFormSubmit(dict('GEN_LBL_INCLUDE', dict('GEN_LBL_FIELD')), 'INCLUDE', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        $contents .= $form->source();
        return $contents;
    }

    function checkForm(&$msg) {
        $msg = array();
        $DBFL = &getTool('DBFL');
        UserRequest::$post['DBTD_fieldname'] = uppercase(UserRequest::$post['DBTD_fieldname']);
        if (!$DBFL->fieldexists(UserRequest::$post['DBTD_fieldname'])) {
		  $msg['DBTD_fieldname'] = dict('TOOL_DBTD_ERR_NOFIELD');
		  $msg['ERROR'] = 'NO_FIELD';
        } else {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'TABLECODE=' . singleQuote(UserRequest::$post['DBTD_tablecode']) . ' AND FIELDNAME=' . singleQuote(UserRequest::$post['DBTD_fieldname']));
		  if ($qty > CLX_NUM_ZERO) {
			 $msg['DBTD_fieldname'] = dict('TOOL_DBTD_ERR_ALREADYINCLUDED');
		  }
        }
        return (count($msg) == CLX_NUM_ZERO) ? true : false;
    }

    function searchForm() {
        $DBTB = &getTool('DBTB');
        $contents = '';
        $msg = array();
        if (empty(UserRequest::$request['tablecode'])) {
		  $contents = note(dict('TOOL_DBTD_ERR_MUSTSELECTFIELD'));
        } elseif (!$DBTB->tableExists(UserRequest::$request['tablecode'])) {
		  $contents = note(dict('TOOL_DBTD_ERR_TABLECODESELNOTFOUND', UserRequest::$request['tablecode']));
        } else {
		  $tablecode = UserRequest::$request['tablecode'];
		  // Search Form
		  $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('ac', 'SEARCH');
		  $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
		  $form->addArgument('tablecode', $tablecode);
		  $form->addArgument('confirm', '1');
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCell(dict('TOOL_DBTD_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  if (!array_key_exists('DBTD_fieldname', UserRequest::$post))
			 UserRequest::$post['DBTD_fieldname'] = '';
		  $table->addField($msg, dict('GEN_LBL_FIELDNAME') . ':', 'DBTD_fieldname', UserRequest::$post['DBTD_fieldname'], CLX_NUM_32, CLX_NUM_48);
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
		  $c = new htmlButtonGoto(dict('GEN_LBL_CANCEL'), 'javascript:closeWindow();');
		  $buttons .= $c->source();
		  $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  $form->setContents($table->source());
		  $contents .= $form->source();
		  // Search Procedure
		  $table = new htmlTable(CLX_WIDTH_ONE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = 'TABLECODE=' . singleQuote($tablecode);
		  $boolean = ' AND ';
		  if (!empty(UserRequest::$post['DBTD_fieldname'])) {
			 $criteria .= $boolean . 'FIELDNAME LIKE ' . singleQuote(UserRequest::$post['DBTD_fieldname'] . '%');
			 $boolean = ' and ';
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['FIELDNAME']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_tabledfields></page><a href=' . singleQuote($js) . '>' . $record['FIELDNAME'] . '</a>'));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_FIELDNAME') . ':', $record['DBTD_fieldname']);
    }

    function reorderRecords() {
        $records = ClxCommon::$db->getRecords($this->tablename, 'TABLECODE=' . singleQuote(UserRequest::$post['DBTD_tablecode']) . ' ORDER BY SEQNUM');
        $list = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $list[$record['ID']] = $record['FIELDNAME'];
        }
		ClxCommon::$db->freeResult($records);
        return $list;
    }

    function reorderAction() {
        parent::reorderAction();
        $callingpage = peekHistory();
        $tablecode = $callingpage['tablecode'];
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $tablecode);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBTD_id'] = $recordid;
        $record['DBTD_tablecode'] = $values['TABLECODE'];
        $record['DBTD_fieldname'] = $values['FIELDNAME'];
        $record['DBTD_seqnum'] = $values['SEQNUM'];
        return $record;
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
		if (isset(ClxCommon::$db->debugCF) && ClxCommon::$db->debugCF==true){
		
			echo "post en tablefields";
				pre($_POST);
				echo "DUMP";
				
				
		}
        $record = array();
        $record['TABLECODE'] = UserRequest::$post['DBTD_tablecode'];
        $record['FIELDNAME'] = uppercase(UserRequest::$post['DBTD_fieldname']);
        $record['SEQNUM'] = ClxCommon::$db->getField($this->tablename, 'SEQNUM', 'TABLECODE=' . singleQuote(UserRequest::$post['DBTD_tablecode']) . ' ORDER BY SEQNUM DESC') + CLX_NUM_ONE;
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $record['ID'] = $recordid;
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
        $this->clearHeap($record['TABLECODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
        $this->clearHeap($record['TABLECODE']);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['DBTD_tablecode'] = UserRequest::$post['DBTD_tablecode'];
        $record['DBTD_fieldname'] = '';
        $record['DBTD_seqnum'] = '0';
        return $record;
    }

    function placeChooseTableField($msg, $tablefieldname, $prompt, $name, $default, &$table) {

        $contents = setJavaScriptTag(getJavaScriptFilename('DBTD_searchwindow.js'), true);
        $searchIcon = getSkinIcon('view', 'javascript:dbtd_searchwindow(' . singleQuote($tablefieldname), singleQuote($name) . ')', 'Search for a field');
        if (!empty($msg[$name]))
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $contents;
    }

    function fieldExists(&$tablecode, &$fieldname) {
        $DBTB = &getTool('DBTB');
        if ($DBTB->tableExists($tablecode)) {
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'TABLECODE=' . singleQuote($tablecode) . ' AND FIELDNAME=' . singleQuote(uppercase($fieldname)));
		  if ($qty > CLX_NUM_ZERO) {
			 $fieldname = ClxCommon::$db->getField($this->tablename, 'FIELDNAME', 'TABLECODE=' . singleQuote($tablecode) . ' AND FIELDNAME=' . singleQuote(uppercase($fieldname)));
		  }
        } else {
		  $qty = CLX_INIT_ZERO;
        }
        return $qty > CLX_NUM_ZERO;
    }

    function chooseRecord($name, $default, $data = array(), $display_size = CLX_NUM_32, $maxlength = CLX_NUM_64) {
        $variables = '';
        $contents = getJavaScript('DBTD_searchwindow.js', $variables);
        $code = $this->code;
        $alt = 'TOOL_' . $code . '_ALT_SEARCH';
        $tablecode = $data['tablecode'];
        $searchIcon = getSkinIcon('view', 'javascript:dbtd_searchwindow(' . singleQuote($tablecode), singleQuote($name) . ');', dict($alt));
        $c = new htmlField($name, $default, CLX_NUM_32, CLX_NUM_64);
        return $contents . $c->source() . $searchIcon;
    }

    /**
     * Clear field form heap
     * @param string $fieldname
     */
    private function clearHeap($tablecode) {
		apc_deleteFct(kSpaceName . '.dbtf.' . $tablecode);
		apc_deleteFct(kSpaceName . '.dbtb.dbfl.' . $tablecode);
    }

}
