<?php

/**
 * db_tableindexes
 * Abstract Database Object for communicating with a Database
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-30 05:15:57 -0400 (Thu, 30 Aug 2018) $
 * @version    	$Rev: 9946 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_tableindexes extends clx_tool {

    var $indexes;

    function __construct() {
        $this->code = 'DBTI';
        $this->name = 'Table Indexes';
        $this->system = 'Database';
        $this->version = '0.4.1.1';

        $this->accesskey = 'DBTB';


        $this->tablename = 'CLX_DB_TABLEINDEXES';
        $this->record_name_singular = 'table index';
        $this->record_name_plural = 'table indexes';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'TABLECODE';

        $this->primary_key = array('IDCODE');
        $this->indexes = array('PRI' => 'GEN_LBL_PRIMARY', 'MUL' => 'GEN_LBL_KEY', 'UNI' => 'GEN_LBL_UNIQUE', 'COM' => 'GEN_LBL_MULTIPLE');
		$this->debugAU=false;
		IF (ISSET($GLOBALS['ADDRDEBUGAUDIT']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGAUDIT']==$_SERVER['REMOTE_ADDR'])) {
			$this->debugAU=true;
			
		}	
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['FIELDNAME_SEC'] = array('type' => 'varchar(64)', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'TABLECODE');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'FIELDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    function pack($record) {
        $o = $this->packRecord($record);
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'DBTC', 'SELECT * FROM CLX_DB_INDEX_COLUMNS WHERE INDEX_CODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $indexcode = uppercase($record['IDCODE']);
        $exists = (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($indexcode)) > CLX_NUM_ZERO);
        if ($exists === true) {
		  ClxCommon::$db->deleteRecords('CLX_DB_INDEX_COLUMNS', 'INDEX_CODE=' . singleQuote($indexcode));
		  ClxCommon::$db->deleteRecords($this->tablename, 'IDCODE=' . singleQuote($indexcode));
        }
        $indexid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        unset($indexid);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = getTool($toolcode);
			 $tool->unpack(array('package' => $o));
		  }
        }
    }

    /**
     * Builds Panel listing on tool
     *
     * @param string $groupcode
     * @return string
     */
    function makepanel($groupcode = null) {
        $tablecode = $groupcode;
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $modulecode = '';
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBTIPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('modulecode', $modulecode);
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = 'ID, IDCODE, TYPE';
		  $fromclause = 'CLX_DB_TABLEINDEXES';
		  $whereclause = 'TABLECODE=' . singleQuote($tablecode);
		  $sortclause = '';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_THREE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_FIELDNAME')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $record['FIELDNAME'] = '';
			 $comma = '';
			 $fields = ClxCommon::$db->getRecords('CLX_DB_INDEX_COLUMNS', 'INDEX_CODE =' . singleQuote($record['IDCODE']) . ' ORDER BY SEQNUM ASC');
			 while ($field = ClxCommon::$db->getNextRecord($fields)) {
			     $record['FIELDNAME'] .= $comma . $field['FIELDNAME'];
			     $comma = ', ';
			 }
			 ClxCommon::$db->freeResult($fields);
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<b>' . $record['IDCODE'] . '</b> <i>' . dict('GEN_LBL_TYPE') . ': ' . dict($this->indexes[$record['TYPE']]) . '</i>', $record['FIELDNAME']));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_INDEX')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_DBTI_TITLE_LIST'), $modulecode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return '';
        }
    }

    function editForm($ac, $idlist, $record, $msg) {
        $scripts = '';
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_DBTI_TITLE_MODIFY') : dict('TOOL_DBTI_TITLE_CREATE');
        $scripts .= setJavaScriptTag(getJavaScriptFilename('DBTI_editform.js'), true);
        $js = 'fields = new Array();' . CLX_CHAR_NL;
        $available_fields = $this->getFields($record['DBTI_tablecode']);
        //Adding Fields
        $scripts .= setJavaScriptTag($js, false);
        $form = new htmlForm(CLX_PROP_ACTION, quote('tool.php'));
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('DBTI_id', $record['DBTI_id']);
        $form->addArgument('DBTI_tablecode', $record['DBTI_tablecode']);
        $form->addArgument('DBTI_fieldname', $record['DBTI_fieldname']); //Backward Compatibility
        $form->addArgument('DBTI_fieldname_sec', $record['DBTI_fieldname_sec']); //Backward Compatibility
        $form->addArgument('selected', '');
        $form->setProp('onsubmit', 'return processForm()');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (empty($modify)) {
		  $table->addField($msg, dict('GEN_LBL_IDCODE'), 'DBTI_idcode', $record['DBTI_idcode'], CLX_NUM_30, CLX_NUM_20, true);
        } else {
		  $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBTI_idcode']);
		  $form->addArgument('DBTI_idcode', $record['DBTI_idcode']);
        }
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['DBTI_tablecode']);
        //Index Type
        $type = new htmlSelect('DBTI_type');
        foreach ($this->indexes as $key => $value) {
		  if ($key === 'COM') {
			 continue;
		  }
		  $type->addItem(dict($value), $key, ($record['DBTI_type'] == $key));
        }
        $table->addData(dict('GEN_LBL_INDEX_TYPE') . ':', $type->source());
        $table2 = new htmlTable(CLX_WIDTH_TWO + CLX_NUM_ONE+ CLX_NUM_TWO);
        $table2->setProps(array(CLX_PROP_CLASS => CLX_CLASS_PROMPT, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table2->addCell(dict('GEN_LBL_AVA_COL'), array(CLX_PROP_COLSPAN => CLX_NUM_TWO, CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_PROP_30PCT));
		$table2->addCell(dict('GEN_LBL_FUNCTION'), array(CLX_PROP_COLSPAN => CLX_NUM_ONE, CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_PROP_30PCT));
        $table2->addCell(dict('GEN_LBL_SEL_COL'), array(CLX_PROP_COLSPAN => CLX_NUM_TWO, CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_PROP_30PCT));
        if (!empty($msg['DBTI_selected'])) {
		  $table2->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_COLSPAN => CLX_NUM_THREE));
		  $table2->addCell($msg['DBTI_selected'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        }
        //New fieldname select
        $select_size = CLX_NUM_TEN;
        $pop = new HTMLSelect('DBTI_fields');
        $pop->setProp('size', $select_size);
        $pop->setProp('multiple', true);
        $selected = true;
        foreach ($available_fields as $code => $info) {
		  $pop->addItem($code, $code, $selected);
		  $selected = false;
        }
        $table2->addCell($pop->source(), array());
        $buttons = '<input type="button" name="select_btn" value="' . dict('GEN_LBL_SELECT') . '" onclick="selectItem(' . singleQuote('DBTI_fields') . ', ' . singleQuote('DBTI_selected') . ')"><br>';
        $table2->addCell($buttons, array());
        
		
		$pop = new HTMLSelect('DBTI_fields_function');
        $pop->setProp('size', $select_size);
        $pop->setProp('multiple', false);
        $selected = true;
		$available_functions=Array("0"=>"n/a","1"=>"SUM","2"=>"MAX","3"=>"UPPER","4"=>"LOWER","5"=>"NVL","6"=>"COALESCE","7"=>"ROUND","8"=>"SUBSTR");
        foreach ($available_functions as $code => $info) {
		  $pop->addItem($info, $code, $selected);
		  $selected = false;
        }
        $table2->addCell($pop->source(), array());
        // Selected Items -------------------
        $pop = new HTMLSelect('DBTI_selected');
        $pop->setProp('onChange', 'updateForm();');
        $pop->setProp('size', $select_size);
        $pop->setProp('multiple', true);
        $current_fields = $this->getIndexFields($record['DBTI_idcode']);
        foreach ($current_fields as $key => $val) {
		  $pop->addItem($key, $key, false);
        }
        $table2->addCell($pop->source(), array(CLX_PROP_COLSPAN => '1'));
        //Ups and Down --------------------
        $options = '<p style="text-align: center;">';
        $options .= '<input type="button" name="moveup_btn" value="' . dict('GEN_LBL_MOVEUP') . '" onclick="moveItem(1, ' . singleQuote('DBTI_selected') . ')"><br>';
        $options .= '<input type="button" name="movedown_btn" value="' . dict('GEN_LBL_MOVEDOWN') . '" onclick="moveItem(-1, ' . singleQuote('DBTI_selected') . ')"><br>';
        $options .= '<input type="button" name="delete_btn" value="' . dict('GEN_LBL_DELETE') . '" onclick="deleteItem(' . singleQuote('DBTI_selected') . ')"><br></p>';
        $table2->addCell($options, array(CLX_PROP_COLSPAN => '1'));
        $table->addCell($table2->source(), array(CLX_PROP_COLSPAN => CLX_NUM_TWO, CLX_PROP_CLASS => ''));
        unset($table2);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $scripts . $form->source();
    }

    /**
     * Re
     * @param type $msg
     * @return type
     */
    function checkForm(&$msg) {
        $msg = array();
        $DBTF = &getTool('DBTF');
        $fields = explode(',', UserRequest::$post['selected']);
        if (count($fields) == CLX_NUM_ZERO) {
		  $msg['DBTI_selected'] = dict('GEN_LBL_FIELD_REQUIRED');
        }
        foreach ($fields as $key => $val) {
			if ($this->debugAU==true) {
				pre('revisando checkform revisando field: '.$val);
			}
		  if (!$DBTF->fieldExists(UserRequest::$post['DBTI_tablecode'], $val)) {
			 $msg['DBTI_selected'] = dict('GEN_LBL_INVALID_TABLEFIELD', UserRequest::$post['DBTI_tablecode'], $val);
		  }
        }
        if (UserRequest::$post['DBTI_type'] === 'PRI') {
			if ($this->debugAU==true) {
				pre('revisando indice primary paso 1');
			}	
            //$prim_qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE <> ' . singleQuote(UserRequest::$post['DBTI_idcode'] . ' AND TABLECODE= ' . singleQuote(UserRequest::$post['DBTI_tablecode']) . ' AND TYPE = ' . singleQuote('PRI')));
            $prim_qty1 = ClxCommon::$db->oneExistsRecords($this->tablename, /*'IDCODE <> ' . singleQuote(UserRequest::$post['DBTI_idcode']) . ' AND */ ' TABLECODE= ' . singleQuote(UserRequest::$post['DBTI_tablecode']) . ' AND TYPE = ' . singleQuote('PRI'));
            $prim_qty2 = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE = ' . singleQuote(UserRequest::$post['DBTI_idcode']) . ' AND TABLECODE= ' . singleQuote(UserRequest::$post['DBTI_tablecode']) . ' AND TYPE = ' . singleQuote('PRI'));
		  if ($this->debugAU==true) {
				PRE('INDICE PRIMARIO A:');
				PRE( $prim_qty1);
				PRE('INDICE PRIMARIO B:');
				PRE( $prim_qty2);
			}
			if ($prim_qty1 > CLX_NUM_ZERO) {
				
				IF ($prim_qty2 == CLX_NUM_ZERO) {
					$msg['DBTI_selected'] = dict('GEN_LBL_ONLY_ONE_PRIM_INDEX');
				}
				ELSE {
			 $tablerec = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'IDCODE=' . singleQuote(UserRequest::$post['DBTI_tablecode']));
			 $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
			 if (ClxCommon::$db->hasIndex($tablename, UserRequest::$post['DBTI_fieldname'], 'PRIMARY')) {
			     $msg['DBTI_selected'] = dict('GEN_LBL_ONLY_ONE_PRIM_INDEX');
			 }
		  }
        }
        }
		if (empty($msg['DBTI_selected'])) {
			$sql='SELECT SEQNUM,B.TABLECODE,B.IDCODE,INDEX_CODE,A.FIELDNAME FROM CLX_DB_INDEX_COLUMNS A,CLX_DB_TABLEINDEXES B WHERE B.IDCODE=A.INDEX_CODE AND TABLECODE='.singlequote(UserRequest::$post['DBTI_tablecode']).' ORDER BY INDEX_CODE,SEQNUM';
			$records = ClxCommon::$db->executeQuery($sql);
			$index_array=array();
			while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {  
				IF (!ISSET($index_array[$record['TABLECODE']."_".$record['INDEX_CODE']]))
					$index_array[$record['TABLECODE']."_".$record['INDEX_CODE']]=$record['FIELDNAME'];
				else
					$index_array[$record['TABLECODE']."_".$record['INDEX_CODE']].=','.$record['FIELDNAME'];
			}
			ClxCommon::$db->freeResult($records);
			if (UserRequest::$post['ac']!='MODIFY') {
				foreach($index_array as $key=>$value) {
					if (UserRequest::$post['selected']==$value)
						$msg['DBTI_selected']=dict('GEN_LBL_EXIST_INDEX');
				}
			}
		}	
        return (count($msg) == CLX_NUM_ZERO) ? true : false;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_IDCODE') . ':', $record['DBTI_idcode']);
        $table->addData(dict('GEN_LBL_TABLECODE') . ':', $record['DBTI_tablecode']);
        $table->addData(dict('GEN_LBL_INDEX_TYPE') . ':', dict($this->indexes[$record['DBTI_type']]));
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBTI_id'] = $recordid;
        $record['DBTI_idcode'] = $values['IDCODE'];
        $record['DBTI_tablecode'] = $values['TABLECODE'];
        $record['DBTI_fieldname'] = $values['FIELDNAME'];
        $record['DBTI_type'] = $values['TYPE'];
        $record['DBTI_fieldname_sec'] = $values['FIELDNAME_SEC'];
        return $record;
    }

    function saveRecord() {
        $this->deleteRecordSk(UserRequest::$post['DBTI_id']);
        $this->createRecordVoid_();
    }

    function createRecordVoid_() {
        ClxFreeReturn($this->createRecord_());
    }

    function createRecord_() {
        $record = array();
        $record['IDCODE'] = UserRequest::$post['DBTI_idcode'];
        $record['TABLECODE'] = UserRequest::$post['DBTI_tablecode'];
        $record['FIELDNAME'] = UserRequest::$post['DBTI_fieldname'];
        $record['TYPE'] = UserRequest::$post['DBTI_type'];
        $record['FIELDNAME_SEC'] = UserRequest::$post['DBTI_fieldname_sec'];
        //Creating index record
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBTC = getTool('DBTC');
        $fields = explode(',', UserRequest::$post['selected']);
        foreach ($fields as $key => $val) {
		  $f = array('INDEX_CODE' => $record['IDCODE'], 'FIELDNAME' => $val, 'SEQNUM' => $key);
		  ClxCommon::$db->createRecord($DBTC->tablename, $f, false);
        }
        //Creating index fields
        $tablerec = ClxCommon::$db->getFields('CLX_DB_TABLES', 'IDCODE,MODULECODE,IDNAME,TYPE', 'IDCODE=' . singleQuote($record['TABLECODE']));

        if (empty($tablerec['STATUS']) || ($tablerec['TYPE'] === 'TEMP' && kDatabaseType <> 'oci')) {
		  //mysql temporary table always remove any aditional field
        } else {
		  $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
		  $indexname = $record['IDCODE'];
		  switch ($record['TYPE']) {
			 case 'UNI':
			 case 'COM':
			     ClxCommon::$db->createUniqueIndex($tablename, UserRequest::$post['selected'], $indexname);
			     break;
			 case 'MUL':
			     ClxCommon::$db->createIndex($tablename, UserRequest::$post['selected'], $indexname);
			     break;
			 case 'PRI':
			     ClxCommon::$db->createPrimaryIndex($tablename, UserRequest::$post['selected'], $indexname);
			     break;
			 default:
			     break;
		  }
        }
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $record['TABLECODE']);
        return $recordid;
    }

    function deleteRecordSk($id) {
        $DBTC = &getTool('DBTC');
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $tablerec = ClxCommon::$db->getFields('CLX_DB_TABLES', 'IDCODE,MODULECODE,IDNAME,TYPE,STATUS', 'IDCODE=' . singleQuote($record['TABLECODE']));
        
		if ($tablerec['STATUS']==='' || ($tablerec['TYPE'] === 'TEMP' && kDatabaseType <> 'oci')) {
		  //mysql temporary table always remove any aditional field
        } else {
		  if (!(UserRequest::$post['ac'] === 'DELETE' && UserRequest::$post[CLX_PROP_TO] === 'DBTB')) {
			 $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
			 $fieldname = $record['FIELDNAME'];
			 $indexname = $record['IDCODE'];
			 if ($record['TYPE'] === 'PRI') {
			     if (UserRequest::$post['ac'] <> 'MODIFY') {
			         ClxCommon::$db->dropPrimaryIndex($tablename, $fieldname, $indexname);
			     }
			 } else {
			     ClxCommon::$db->dropIndex($tablename, $fieldname, $indexname);
			 }
		  }
        }
        ClxCommon::$db->deleteRecords($DBTC->tablename, 'INDEX_CODE=' . singleQuote($record['IDCODE']));
        ClxCommon::$db->deleteRecords($this->tablename, 'UPPER(IDCODE)=' . singleQuote(uppercase($record['IDCODE'])));
        $DBTB = &getTool('DBTB');
        $DBTB->versionIncrease('IDCODE', $tablerec['IDCODE']);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['DBTI_id'] = CLX_INIT_ZERO;
        $record['DBTI_idcode'] = $this->indexDefCode($callingpage['tablecode']);
        $record['DBTI_tablecode'] = $callingpage['tablecode'];
        $record['DBTI_fieldname'] = '';
        $record['DBTI_type'] = 'MUL';
        $record['DBTI_fieldname_sec'] = '';
        return $record;
    }

    function indexDefCode($tablecode) {
        return $tablecode . '_' . date('Ymdhis');
    }

    function createTable($tablerec) { // called by the table manager every time a table is created.
        $record = array();
        $record['IDCODE'] = $this->indexDefCode($tablerec['IDCODE']) . '1';
        $record['TABLECODE'] = $tablerec['IDCODE'];
        $record['FIELDNAME'] = 'ID';
        $record['TYPE'] = 'PRI';
        $record['FIELDNAME_SEC'] = '';
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        unset($recordid);
        $DBTC = getTool('DBTC');
        ClxCommon::$db->createRecord($DBTC->tablename, array('INDEX_CODE' => $record['IDCODE'], 'FIELDNAME' => 'ID', 'SEQNUM' => CLX_NUM_ONE), false);
        if (kDatabaseType === 'oci' && $tablerec['STATUS']==='1') {
		  $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
		  ClxCommon::$db->createPrimaryIndex($tablename, 'ID', $record['IDCODE']);
        }
    }

    /**
     * Check tool record
     * @param type $indexrec
     * @param type $tablerec
     * @param type $record
     * @param string $msg
     * @return type
     */
    function checkField($indexrec, $tablerec, &$record, &$msg) {
        $index_fields = explode(',', $indexrec['FIELDNAME']);
        if (count($index_fields) == CLX_NUM_ZERO) {
		  return;
        }
        if (!empty($record[$tablerec['IDCODE'] . '_ID'])) {
		  $id = $record[$tablerec['IDCODE'] . '_ID'];
        } else {
		  $id = CLX_INIT_ZERO;
        }
        $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
        $sql = 'SELECT COUNT(*) FROM ' . $tablename . ' WHERE 1=1 ';
        //verificar que no se habla del registro modificado
        if ($id <> CLX_NUM_ZERO) {
		  $sql .= ' AND ID <> ' . $id;
        }
        $dupvalue = array();
        //Prepare sql to check
        foreach ($index_fields as $key => $fieldname) {
		  $form_fieldname = $tablerec['IDCODE'] . '_' . $fieldname;
		  //Return fieldtype for futher operations
		  if (array_key_exists($form_fieldname, $record)) { // : ver caso 1001031
			 $fieldname_type = ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDTYPE', 'FIELDNAME = ' . singleQuote($fieldname));
			 switch ($fieldname_type) {
			     case 'DATE':
			         $record[$form_fieldname] = xFormat::date($record[$form_fieldname], 'Y-m-d');
			         $sql .=' AND ' . $fieldname . '=' . singleQuote($record[$form_fieldname]) . ' ' . CLX_CHAR_NL;
			         break;
			     case 'DATI':
			         $record[$form_fieldname] = xFormat::date($record[$form_fieldname], 'Y-m-d H:i:s');
			         $sql .=' AND ' . $fieldname . '=' . singleQuote($record[$form_fieldname]) . ' ' . CLX_CHAR_NL;
			         break;
			     case 'TIME':
			         $record[$form_fieldname] = xFormat::date($record[$form_fieldname], 'H:i:s');
			         $sql .=' AND ' . $fieldname . '=' . singleQuote($record[$form_fieldname]) . ' ' . CLX_CHAR_NL;
			         break;
			     case 'INTN':
			         if (trim($record[$form_fieldname]) === '') {
					   $record[$form_fieldname] = 'null';
			         }
			         $sql .=' AND ' . $fieldname . '=' . $record[$form_fieldname] . ' ' . CLX_CHAR_NL;
			         break;
			     case 'DECN':
			         if (trim($record[$form_fieldname]) === '') {
					   $record[$form_fieldname] = 'null';
			         }
			         $sql .=' AND ' . $fieldname . '=' . $record[$form_fieldname] . ' ' . CLX_CHAR_NL;
			         break;
			     default:
			         $sql .=' AND ' . $fieldname . '=' . singleQuote($record[$form_fieldname]) . ' ' . CLX_CHAR_NL;
			         break;
			 }
			 $dupvalue[$form_fieldname] = $record[$form_fieldname];
		  }
        }
        $r = ClxCommon::$db->getOne($sql);
        if ($r > CLX_NUM_ZERO) {
		  foreach ($dupvalue as $key => $val) {
			 $msg[$key] = dict('GEN_ERR_DBDUPKEY') . ' ' . implode('-', $dupvalue);
		  }
        }
    }

    /**
     * Test index Information to see if correct. Used by check table and audit
     * see also DBTB->checkTable()
     *
     * @param string $tablecode
     * @param boolean $fix
     */
    function checkIndex($tablecode, $fix, &$msg) {
        if ($this->debug) {
		  print_r($tablecode);
		  print_r($fix);
		  print_r($msg);
        }
    }

    function getFields($tablecode) {
        $sql = 'SELECT T.IDCODE TABLECODE, T.IDNAME TABLENAME, TF.FIELDNAME FIELDNAME FROM CLX_DB_TABLES T, CLX_DB_TABLEFIELDS TF WHERE T.IDCODE = TF.TABLECODE AND T.IDCODE = ' . singleQuote($tablecode) . ' ORDER BY TF.SEQNUM ASC';
        $records = ClxCommon::$db->executeQuery($sql);
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $fields[$record['FIELDNAME']] = $record;
        }
		ClxCommon::$db->freeResult($records);
        return $fields;
    }

    function getTableIndexes($tablecode, $type = '') {
        $cc = ($type <> '') ? 'AND DBTI.TYPE = ' . singleQuote($type) : '';
        $sql = 'SELECT DBTI.IDCODE INDEX_CODE, DBTI.TYPE TYPE, DBTC.FIELDNAME FROM ' . $this->tablename . ' DBTI, CLX_DB_INDEX_COLUMNS DBTC WHERE DBTI.IDCODE = DBTC.INDEX_CODE AND DBTI.TABLECODE = ' . singleQuote($tablecode) . ' ' . $cc . '  ORDER BY DBTC.SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  if (!array_key_exists($record['INDEX_CODE'], $fields)) {
			 $fields[$record['INDEX_CODE']] = $record;
		  } else {
			 $fields[$record['INDEX_CODE']]['FIELDNAME'] .= ',' . $record['FIELDNAME'];
		  }
        }
		ClxCommon::$db->freeResult($records);
        reset($fields);
        return $fields;
    }

    function getIndexFields($indexcode) {
        $sql = 'SELECT FIELDNAME FROM CLX_DB_INDEX_COLUMNS WHERE INDEX_CODE = ' . singleQuote($indexcode) . ' ORDER BY SEQNUM';
        $records = ClxCommon::$db->executeQuery($sql);
        $fields = array();
        while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $fields[$record['FIELDNAME']] = $record;
        }
		ClxCommon::$db->freeResult($records);
        reset($fields);
        return $fields;
    }

}
