<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-31 18:12:58 +0000 (Thu, 31 Jan 2019) $
 * @version        $Rev: 10517 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class db_tables extends clx_tool {

    function __construct() {
        $this->code = 'DBTB';
        $this->name = 'Table Manager';
        $this->system = 'Database';
        $this->version = '0.5.19.1';

        $this->has_access = true;
        $this->has_package = true;

        $this->tablename = 'CLX_DB_TABLES';
        $this->record_name_singular = 'table';
        $this->record_name_plural = 'tables';
        $this->primary_key = array('IDCODE');
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->table_types = (kDatabaseType === 'oci') ? array('TEMP' => 'TEMPORARY', 'VIEW' => 'VIEW', 'InnoDB' => 'DEFAULT', 'QUERY' => 'GEN_LBL_QUERY2') : array('InnoDB' => 'DB_INNODB', 'TEMP' => 'TEMPORARY', 'VIEW' => 'VIEW', 'ISAM' => 'DB_ISAM', 'MyISAM' => 'DB_MYISAM', 'HEAP' => 'DB_HEAP', 'BDB' => 'DB_BERKELEY', 'QUERY' => 'GEN_LBL_QUERY2');
		$fetched = false;
	    if(kApcTtl_16>0) $this->query_associate= apc_fetchFct($findVar=kSpaceName . '_IDCODE_IDNAME_CLX_DBQUERIES_' , $fetched);
		if (empty($fetched)) {
			$this->query_associate=array();
		$rs = ClxCommon::$db->executeQuery('SELECT IDCODE, IDNAME FROM CLX_DB_QUERIES');
		while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			$this->query_associate[$r['IDCODE']]=$r['IDNAME'].' ('.$r['IDCODE'].')';
		}
		ClxCommon::$db->freeResult($rs);
			if (kApcTtl_16>0 && count($this->query_associate)>0  ) {
				apc_addFct($findVar, $this->query_associate, kApcTtl_16);
			}
		}

        $this->table_types_aux = array_merge(array('' => ''), $this->table_types);
        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_DBTB_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '4', 'displaysize' => '4', 'dontmodify' => '1', 'validations' => array(array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))))),
			         array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(array('type' => 'sqlobject', 'required' => '1'))))),
			         array('label' => 'GEN_LBL_TYPE', 'fields' => array(array('name' => 'type', 'type' => CLX_CLASS_LIST, 'default' => 'InnoDB', 'options' => $this->table_types))),
			         array('label' => 'GEN_LBL_STATUS', 'fields' => array(array('name' => 'status', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => $this->status))),
						array('label' => 'GEN_LBL_CREATE_HISTORIC', 'fields' => array(array('name' => 'historic', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => $this->yesno))),
						array('label' => 'GEN_LBL_VERSION', 'fields' => array(array('name' => 'version', 'type' => 'string', 'default' => '1.0.0', 'size' => '32', 'displaysize' => '16', 'validations' => array(array('type' => 'string', 'required' => '1'))))),
			         array('label' => 'GEN_LBL_DESCRIPTION', 'fields' => array(array('name' => 'description', 'type' => 'text', 'default' => '', 'cols' => '48', 'rows' => '4'))),
			         array('label' => 'TOOL_DBMO_LABEL_AUDIT', 'fields' => array(array('name' => 'audit', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => $this->yesno))),
			         array('label' => 'TOOL_DBMO_LABEL_TRUNCABLE', 'fields' => array(array('name' => 'truncable', 'type' => CLX_CLASS_LIST, 'default' => '1', 'options' => $this->yesno))),
			         array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO', 'validations' => array(array('type' => 'sqlobject', 'required' => '1'), array('type' => 'lookup', 'fieldname' => 'idcode', 'tablename' => 'CLX_DB_MODULES'))))),
					##	array('label' => 'GEN_LBL_QUERY_ASSOCIATE', 'fields' => array(array('name' => 'query_associate', 'type' => CLX_CLASS_LIST, 'default' => '', 'options' => $this->query_associate,'dict_nochange'=>'1'))),
					
			     )
			 ),
			 array(CLX_CLASS_TITLE => 'TOOL_DBTB_VIEW',
			     'type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_OR_SQL_VIEW', 'fields' => array(array('name' => 'view_query', 'type' => 'text', 'default' => '', 'cols' => '120', 'rows' => '20'))))),
			 array(CLX_CLASS_TITLE => 'TOOL_DBTB_PARTITIONS',
			     'type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_PTTN_ENABLED', 'fields' => array(array('name' => 'pttn_enabled', 'type' => CLX_CLASS_LIST, 'default' => '0', 'options' => $this->yesno))),
			         array('label' => 'GEN_LBL_PTTN_OPTIONS', 'fields' => array(array('name' => 'pttn_options', 'type' => 'text', 'default' => '', 'cols' => '120', 'rows' => '10'))))),
			 array(CLX_CLASS_TITLE => 'TOOL_DBTB_CAPTION_RECNAMES',
			     'type' => 'normal',
			     'entries' => array(
			         array('label' => 'TOOL_DBTB_PROMPT_RECNAMES', 'fields' => array(array('name' => 'recordname_singular', 'type' => 'dict', 'required' => '1'))),
			         array('label' => 'TOOL_DBTB_PROMPT_RECNAMEP', 'fields' => array(array('name' => 'recordname_plural', 'type' => 'dict', 'required' => '1')))
			     )
			 )
		  )
        );
        $this->quickfilterform = array('panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '4', 'displaysize' => '4'))),
			         array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48'))),
			         array('label' => 'GEN_LBL_TYPE', 'fields' => array(array('name' => 'type', 'type' => 'list', 'options' => $this->table_types_aux))),
			         array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'lookup', 'default' => '', 'tool' => 'DBMO')))
			     )
			 )
		  )
        );
        $this->viewform = array(CLX_CLASS_TITLE => 'TOOL_DBTB_TITLE_VIEW',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_TYPE', 'fields' => array(array('name' => 'type', 'type' => CLX_CLASS_LIST, 'default' => 'MYISAM', 'options' => $this->table_types))),
			         array('label' => 'GEN_LBL_VERSION', 'fields' => array(array('name' => 'fullversion', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_DESCRIPTION', 'fields' => array(array('name' => 'description', 'type' => 'text'))),
			         array('label' => 'GEN_LBL_MODULECODE', 'fields' => array(array('name' => 'modulecode', 'type' => 'string')))
			     )
			 )
		  )
        );
	  	$this->orphans_fields = array('0' => 'GEN_LBL_NO', '1' => 'GEN_LBL_YES');
				
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_DATABASE:MENU_TABLES', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['AUDITU'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
		$fields['DROPORPHANSFIELDS'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        //
        $fields['TRUNCABLE'] = array('type' => 'tinyint(1) unsigned', 'properties' => '');
        //
        $fields['STATUS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => 0, 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(16)', 'properties' => '');
        $fields['VIEW_QUERY'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['RECORDNAME_SINGULAR'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        $fields['RECORDNAME_PLURAL'] = array('type' => 'varchar(255)', 'properties' => '', 'dict' => true);
        //Opciones de particiones
        $fields['PTTN_ENABLED'] = array('type' => 'int(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['PTTN_OPTIONS'] = array('type' => 'mediumtext', 'null' => 'YES', 'properties' => '');
        $fields['HISTORIC'] = array('type' => 'tinyint(1) unsigned', 'null' => 'YES', 'default' => '0', 'properties' => '');
		$indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    //-- T A B L E S
    function pack($record) {
        $o = $this->packRecord($record);
        $id = $record['ID'];
        $code = $record['IDCODE'];
        $objs = new XMLObject($this->code . '_OBJECTS');
        $this->packsub($objs, 'DBTF', 'SELECT * FROM CLX_DB_TABLEFIELDS WHERE TABLECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBTD', 'SELECT * FROM CLX_DB_TABLEDFIELDS WHERE TABLECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBTI', 'SELECT * FROM CLX_DB_TABLEINDEXES WHERE TABLECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBDR', 'SELECT * FROM CLX_DB_DYNRELATIONS WHERE PARENT_TABLECODE=' . singleQuote($code));
        $this->packsub($objs, 'DBFL', 'SELECT f.* FROM CLX_DB_TABLEFIELDS tb, CLX_DB_FIELDS f  WHERE tb.TABLECODE=' . singleQuote($code) . ' AND tb.FIELDNAME=f.FIELDNAME ORDER BY tb.SEQNUM');
        $this->packsub($objs, 'DBFL', 'SELECT f.* FROM CLX_DB_TABLEDFIELDS tb, CLX_DB_FIELDS f  WHERE tb.TABLECODE=' . singleQuote($code) . ' AND tb.FIELDNAME=f.FIELDNAME ORDER BY tb.SEQNUM');
        $this->packsub($objs, 'EVNT', 'SELECT * FROM CLX_EVENTS WHERE OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singlequote($id));
        $this->packsub($objs, 'DICT', 'SELECT DICT.* FROM CLX_DICTIONARY DICT, CLX_DB_TABLES X WHERE (DICT.ENTRY=X.RECORDNAME_SINGULAR OR DICT.ENTRY=X.RECORDNAME_PLURAL) AND X.IDCODE=' . singleQuote($code));
        $o->addObject($objs);
        return $o;
    }
	  /**
     * Tool Edit Form
     * @param string $ac
     * @param string $idlist
     * @param array $record
     * @param array $msg
     * @return string
     */
    function editForm($ac, $idlist, $record, $msg) {

        if (UserRequest::$request['ac'] === 'MODIFY') {
		  $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_MODIFY';
        } else {
		  $this->editform[CLX_CLASS_TITLE] = 'TOOL_' . $this->code . '_TITLE_CREATE';
        }
        $form = new clx_form($this->code, $this->editform);
		$extra='';
		if ($ac=='DUPLICATE') {
			$extra='<script>setTimeout(\'$("#DBTB_historic option[value=\\\'1\\\']").remove();$("#DBTB_historic option[value=\\\'0\\\']").attr("selected", "selected");\',5000)</script>';
		}
		return $form->editForm($ac, $idlist, $record, $msg).$extra;
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null), $parameters));
        list($package) = $parameters;
        $record = $this->unpackRecord($package);
        $tablecode = uppercase($record['IDCODE']);

        ClxCommon::$db->loadDbRecord($this->tablename, $record, false, 'IDCODE=' . singleQuote($tablecode));

        $tableid = ClxCommon::$db->getField($this->tablename, 'ID', 'IDCODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEDFIELDS', 'TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEFIELDS', 'TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_DB_DYNRELATIONS', 'PARENT_TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_DB_DYNRELATION_COLS', 'PARENT_TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->executeQuery('DELETE FROM CLX_DB_INDEX_COLUMNS WHERE INDEX_CODE IN (SELECT IDCODE FROM CLX_DB_TABLEINDEXES WHERE TABLECODE=' . singleQuote($tablecode) . ')');
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEINDEXES', 'TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code) . ' AND OWNERID=' . singleQuote($tableid));

        $record['IDNAME'] = uppercase($record['IDNAME']);
        if ($package->hasSubObject($this->code . '_OBJECTS')) {
		  $objs = $package->getObjectXml($this->code . '_OBJECTS');
		  foreach ($objs->objects as $o) {
			 $toolcode = $o->tag;
			 $tool = getTool($toolcode);
			 $tool->unpack(array('package' => $o, 'id' => $tableid));
		  }
        }
		apc_clear_xppaging_count_all_user($tablecode);
    }

    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'CREATE_UI':
			     $contents = $this->create_ui($idlist);
			     break;
			 case 'CHECK':
			     $contents = $this->checkTableForm($idlist, false);
			     break;
			 case 'FIX':
			     // Audit dump to .SQL Script file instead of excecuting SQL senences directly
			     $dump = (defined('UTIL_DUMP_AUPK_SQL_SCRIPT') && UTIL_DUMP_AUPK_SQL_SCRIPT === strval(CLX_NUM_ONE)) ? true : false;
			     $flmsg = '';
			     if (!empty($dump)) {
			         ClxCommon::$db->scriptFileOut = kPathToTemp . clx_mcript('FORM_DB_TABLE_FIX') . '.sql';
			         $flmsg = '<br>' . dict('GEN_LBL_TO') . ' <a href="' . ClxCommon::$db->scriptFileOut . '" target=_blank>' . ClxCommon::$db->scriptFileOut . '</a>';
			     }
			     $contents = $this->checkTableForm($idlist, true, $dump);
			     //  Audit dump to .SQL Script file instead of excecuting SQL senences directly
			     if (!empty($dump)) {
			         $contents .= '<br><br>' . note(dict('GEL_LBL_OPER_COMP_SUCCESS') . ' ' . $flmsg, 'OK');
			     }
			     break;
			 case 'IMPORT_STRUCTURE':
			     $contents = $this->importStructure();
			     break;
				case 'DUPLICATE':
			     if ($this->getAccess()->canModify() && ($this->getAccess()->canDisplay())) {
			         if (isset(UserRequest::$post['confirm'])) {
					   if ($this->checkForm($msg)) {
						  ClxFreeReturn($this->duplicateRecord());
								if ( UserRequest::$post['DBTB_historic']==1 OR UserRequest::$post['MODULECODE']=='USER' ) {
									$this->gen_Historic(UserRequest::$post);
								}	
								if ( isset($GLOBALS["process"])  ) { 
						      $contents="";      
								} else {
									$info = popHistory();
									$contents = executeHistory($info);
								}
					   } else {
						  $contents = $this->editForm($ac, $idlist, UserRequest::$post, $msg);
					   }
			         } else {
					   $id = array_pop($idlist);
					   $record = $this->loadToolRecord($id);
							if (!isset($msg))
								$msg=array();
							$contents = $this->editForm($ac, $idlist, $record, $msg);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [CLX_TOOL-CAN_MODIFY & CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
				case 'CHANGEH':
					IF (isset(UserRequest::$request['IDCODE'])) {
						$contents= $this->changeh();
					}
					else {
						$contents = parent::action('LIST', array())	;
					}	
					break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [DB_TABLES-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'DBTBList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_DB_TABLES';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_NUM_TEN);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBTB_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_TEN));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_STATUS'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION'), dict('GEN_LBL_PTTN_ENABLED'),dict('GEN_LBL_HIST')));
        $i = CLX_INIT_ZERO;
		
		$sqlhist='SELECT TABLE_CODE,TABLE_CODE_HIST FROM CLX_DB_TABLE_HIST WHERE
				EXISTS (SELECT 1 FROM CLX_DB_TABLES WHERE IDCODE=TABLE_CODE) AND
				EXISTS (SELECT 1 FROM CLX_DB_TABLES WHERE IDCODE=TABLE_CODE_HIST)' ;
		$recordhists = ClxCommon::$db->executequery($sqlhist);
		//echo "-----".$tablecode."------1";
		$recordh=array();
		while ($recx = ClxCommon::$db->getNextRecord($recordhists)) {
			$recordh[$recx['TABLE_CODE']]=dict('GEN_LBL_HIST1'). $recx['TABLE_CODE_HIST'];
			$recordh[$recx['TABLE_CODE_HIST']]=dict('GEN_LBL_HIST2'). $recx['TABLE_CODE'];
		}
		ClxCommon::$db->freeResult($recordhists);
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ')');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));

		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['MODULECODE'], $record['TYPE'], dict($this->status[$record['STATUS']]), $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION']), dict($this->yesno[$record['PTTN_ENABLED']]), '<a href="tool.php?to=DBTB&ac=CHANGEH&IDCODE='.$record['IDCODE'].'" style="text-decoration:none"> '. (isset($recordh[$record['IDCODE']])? $recordh[$record['IDCODE']]:'&nbsp; &nbsp; &nbsp; &nbsp; &nbsp;&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;').'</a>')); //'<a href="xxx"> '.(isset($recordh[$record['IDCODE']])? $recordh[$record['IDCODE']]:'&nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;').'</a>'
																																																																																																														
		  $i++;
        }
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canAppend()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function makepanel($groupcode = null) {
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'DBTBPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('modulecode', uppercase($groupcode));
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_DB_TABLES';
		  $whereclause = 'MODULECODE=' . singleQuote(uppercase($groupcode));
		  $sortclause = 'IDCODE';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_SIX);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 if (kDatabaseType === 'oci' && $record['TYPE'] <> 'TEMP') {
			     $record['TYPE'] = 'InnoDB';
			 }
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['TYPE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
			 $i++;
		  }
		  $buttons = '';
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDisplay()) {
			 $b = new htmlFormSubmit(dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_TABLE')), 'CREATE', $form->getProp(CLX_PROP_ID));
			 $b->setProp(CLX_PROP_ID, 'ON');
			 $buttons .= $b->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_DBTB_TITLE_LIST'), $groupcode . 'TABLES', $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function checkForm(&$msg) {
		UserRequest::$post['DBTB_modulecode']=uppercase(UserRequest::$post['DBTB_modulecode'] );
		UserRequest::$post['DBTB_idname']=uppercase(UserRequest::$post['DBTB_idname']);
        $result = parent::checkForm($msg);
        if (kDatabaseType === 'oci') {
			$qty = (UserRequest::$request['ac'] !== 'MODIFY') ? ClxCommon::$db->oneExistsRecords($this->tablename, 'UPPER(IDNAME)=' . singleQuote(UserRequest::$post['DBTB_idname']) . ' AND MODULECODE=' . singleQuote(uppercase(UserRequest::$post['DBTB_modulecode']))) : CLX_NUM_ZERO;
        
        } else {
            $qty = (UserRequest::$request['ac'] !== 'MODIFY') ? ClxCommon::$db->oneExistsRecords($this->tablename, 'IDNAME=' . singleQuote(UserRequest::$post['DBTB_idname']) . ' AND MODULECODE=' . singleQuote(uppercase(UserRequest::$post['DBTB_modulecode']))) : CLX_NUM_ZERO;
        }
		
        if ($qty > CLX_NUM_ZERO) {
		  $msg['DBTB_idname'] = dict('GEN_ERR_TABLE_NOT_UNIQUE', UserRequest::$post['DBTB_idname']);
		  $result = false;
        }
        if (UserRequest::$post['DBTB_type'] === 'VIEW') {
		  xValidate::required($msg, 'DBTB_view_query', UserRequest::$post['DBTB_view_query']);
        }
        if (!empty(UserRequest::$post['DBTB_pttn_enabled'])) {
		  xValidate::required($msg, 'DBTB_pttn_enabled', UserRequest::$post['DBTB_pttn_options']);
        }
        if (!empty($msg)) {
		  $result = false;
        }
        return $result;
    }

    function searchForm() {
        $contents = '';
        $msg = array();
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_DBTB_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('DBTB_idcode', UserRequest::$post))
		  UserRequest::$post['DBTB_idcode'] = '';
        if (!array_key_exists('DBTB_idname', UserRequest::$post))
		  UserRequest::$post['DBTB_idname'] = '';
        if (!array_key_exists('DBTB_modulecode', UserRequest::$post))
		  UserRequest::$post['DBTB_modulecode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBTB_idcode', UserRequest::$post['DBTB_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'DBTB_idname', UserRequest::$post['DBTB_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'DBTB_modulecode', uppercase(UserRequest::$post['DBTB_modulecode']), CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['DBTB_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBTB_idcode']) . '%');
		  }
		  if (UserRequest::$post['DBTB_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['DBTB_idname'])) . '%');
		  }
		  if (UserRequest::$post['DBTB_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBTB_modulecode']) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=db_tables></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['DBTB_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['DBTB_idname']);
    }

    /**
     * Build tool display form
     * @param $idlist
     * @return string
     */
    function viewForm_($idlist) {
        $contents = '';
        $buttons = '';
        $id = $idlist[count($idlist) - CLX_NUM_ONE];
        $record = $this->loadToolRecord($id);
        pushHistory($this->code, 'DISPLAY', $idlist);
        // Buttons Form
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, CLX_CLASS_FOOTERS);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('next', CLX_NUM_ONE);
        $form->addArgument('relfm', UserRequest::$request['relfm']);
        $table2 = new htmlTable(CLX_WIDTH_TWO);
        $table2->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		$table2->addList(dict('GEN_LBL_DROP_ORPHANS_FIELDS'), 'DBTB_drop_orphans_fields', '0', $this->orphans_fields, false);
			
		$table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
	
        $c = new htmlButtonReturn();
        $buttons .= $c->source();
        if (count($idlist) > CLX_NUM_ONE) {
		  $c = new htmlFormSubmit(dict('GEN_LBL_VIEWNEXT_SOMETHING', dict('GEN_LBL_RECORD')), 'DISPLAY', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        $buttons .= $this->viewFormButtons($form);
		$table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table2->source().$table->source());
        $buttons_form = $form->source();
        // View Form -----------------------------------------------------------
		if (!empty($GLOBALS['DETAILS_ACUMULATION']))
			$contents =$GLOBALS['DETAILS_ACUMULATION'].'<br>';
		$GLOBALS['DETAILS_ACUMULATION']='';
        if (!clx_is_empty($this->viewform)) {
		  $form = new clx_form($this->code, $this->viewform);
		  $record = $this->loadToolRecord($id);
		  $contents .= $form->viewForm_($record);
		  $contents .= $buttons_form;
		  $contents .= $this->viewFormFill($record);
        } else {
		  $contents .= $this->viewFormFill($record);
		  $contents .= $buttons_form;
        }
        return $contents;
    }
    function viewFormFill($record) {
        $contents = '';
        addHistory('tablecode', $record['DBTB_idcode']);
        addHistory('ownercode', $this->code);
        addHistory('ownerid', $record['DBTB_id']);
        $DBTF = &getTool('DBTF');
        $contents .= $DBTF->makepanel($record['DBTB_idcode']);
        if ($record['DBTB_type'] <> 'VIEW') {
		  $DBTI = &getTool('DBTI');
		  $contents .= $DBTI->makepanel($record['DBTB_idcode']);
		  $DBDR = &getTool('DBDR');
		  $contents .= $DBDR->makepanel($record['DBTB_idcode']);
        }
        $DBTD = &getTool('DBTD');
        $contents .= $DBTD->makepanel($record['DBTB_idcode']);
        $DBTE = &getTool('EVNT');
        $contents .= $DBTE->makepanel($this->code, $record['DBTB_id']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addData(dict('GEN_LBL_STATUS') . ':', $this->yesno[$record['DBTB_status']]);
        $table->addData(dict('GEN_LBL_TYPE') . ':', $record['DBTB_type']);
        $table->addData(dict('GEN_LBL_AUDIT_SOMETHING', dict('GEN_LBL_TABLE')) . ':', $this->yesno[$record['DBTB_audit']]);
        $table->addData(dict('TOOL_DBTB_PROMPT_RECNAMES') . ':', $record['DBTB_recordname_singular_DICT']);
        $table->addData(dict('TOOL_DBTB_PROMPT_RECNAMEP') . ':', $record['DBTB_recordname_plural_DICT']);
        $table->addData(dict('GEN_LBL_CDATE') . ':', $record['DBTB_cdate']);
        $table->addData(dict('GEN_LBL_MDATE') . ':', $record['DBTB_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME') . ':', $record['DBTB_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function viewFormButtons($form) {
        $buttons = '';
        if ($this->getAccess()->canModify()) {
		  $c = new htmlFormSubmit(dict('TOOL_DBTB_GENERATE_UI'), 'CREATE_UI', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        if ($this->getAccess()->getIsadmin()) {
			$c = new htmlFormSubmit(dict('GEN_LBL_CHECK_TABLE'), 'CHECK', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
		  $c = new htmlFormSubmit(dict('GEN_LBL_FIX_TABLE'), 'FIX', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
		  $idlist = toString(getIdlist());
            $url = 'window.php?to=' . $this->code . '&ac=IMPORT_STRUCTURE&DBTB_id=' . $idlist;
            $url = 'javascript:openWindow(' . singleQuote('pop') . ', ' . singleQuote($url) . ', 700, 500,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');';
		  $c = new htmlButton(dict('GEN_LBL_DESCRIBE'));
		  $c->setProp('onClick', $url);
		  $buttons .= $c->source();
        }
        return $buttons;
    }

    function checkTableForm($idlist, $fix, $dump = false,$analize= true) {
        if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");fwrite($tta,chr(13).chr(10)."101- hora:".date("h:i:s d/m/Y").chr(13).chr(10));	fwrite($tta,chr(13).chr(10).'idlist: '.var_export($idlist,true).chr(13).chr(10));fclose($tta);}
        //$tableid = $idlist[CLX_NUM_ZERO];
        $idlst_ndx = count($idlist)-1;
        $tableid = $idlist[$idlst_ndx];
        $tablerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $tableid);
        $contents = '';
        $htmltable = new htmlTable(CLX_WIDTH_TWO);
        $htmltable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $htmltable->addCell(dict('GEN_LBL_RESULTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $htmltable->addData(dict('GEN_LBL_TABLECODE'), $tablerec['IDCODE']);
        $htmltable->addData(dict('GEN_LBL_TABLENAME'), $tablerec['IDNAME']);
        $contents .= $htmltable->source() . '<br>';
        $htmltable = new htmlTable(CLX_WIDTH_THREE);
        $htmltable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $htmltable->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_OBJECT'), dict('GEN_LBL_ERROR'), dict('GEN_LBL_COMMENTS')));
        $msg = $this->checkTable($tablerec, $fix, false, false, false, $dump,$analize);
        $i = CLX_INIT_ZERO;
		$fixorphan=false;
        $arregloOrphans=array();
		if ($tablerec['TYPE']!='VIEW') {
			if (!empty(UserRequest::$post['DBTB_drop_orphans_fields']) && $fix) {
				$fixorphan=true;
			}
			
			$arregloOrphans=ClxCommon::$db->drop_real_orphans_fisic_fields($tablerec['IDCODE'],$tablerec['MODULECODE'].'_'.$tablerec['IDNAME'],$fixorphan);
		}
		//$contents .= $htmltable->source();
		
		if (count($msg) == CLX_NUM_ZERO && count($arregloOrphans) ==CLX_NUM_ZERO ) {
		  $htmltable->addCell(dict('GEN_LBL_NOERROR_FOUND'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        } else {
			if (count($msg) != CLX_NUM_ZERO) {
		  foreach ($msg as $line) {
			 $htmltable->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($line['fieldname'], $line['error'], $line['comments']));
			 $i++;
		  }
        }
			if ( count($arregloOrphans) !=CLX_NUM_ZERO ) {
				foreach ($arregloOrphans as $line) {
					$htmltable->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($line[0], '',$line[2]));
					$i++;
				}
				
			}	
        }
        $contents .= $htmltable->source() . '<br>';
        $htmltable = new htmlTable(CLX_WIDTH_ONE);
        $htmltable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        $c = new HTMLButtonCancel(dict('GEN_LBL_RETURN'));
        $buttons .= $c->source();
        $htmltable->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
		$contents .= $htmltable->source() . '<br>';
        
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('DBTB_idcode', UserRequest::$post))
		  UserRequest::$post['DBTB_idcode'] = '';
        if (!array_key_exists('DBTB_idname', UserRequest::$post))
		  UserRequest::$post['DBTB_idname'] = '';
        if (!array_key_exists('DBTB_modulecode', UserRequest::$post))
		  UserRequest::$post['DBTB_modulecode'] = '';
        if (!array_key_exists('DBTB_type', UserRequest::$post))
		  UserRequest::$post['DBTB_type'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'DBTB_idcode', UserRequest::$post['DBTB_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'DBTB_idname', UserRequest::$post['DBTB_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addLookup($msg, dict('GEN_LBL_MODULECODE') . ':', 'DBTB_modulecode', uppercase(UserRequest::$post['DBTB_modulecode']), 'DBMO', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $sort = new HtmlSelect('DBTB_type');
        $sort->addItem('', '', true);
        foreach ($this->table_types as $key => $val) {
		  $sort->addItem(dict($val), $key, ($key == UserRequest::$post['DBTB_type']));
        }
        $table->addData(dict('GEN_LBL_TYPE') . ':', $sort->source());
        $sort = new HtmlSelect('DBTB_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Module Code', 'modulecode', false);
        $sort->addItem('Table Code', 'idcode', false);
        $sort->addItem('Table Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['DBTB_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBTB_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBTB_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(clx_change_especial_char_html (uppercase(UserRequest::$post['DBTB_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBTB_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['DBTB_modulecode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['DBTB_type'])) {
		  $filter .= $boolean . ' TYPE = ' . singleQuote(UserRequest::$post['DBTB_type']);
		  $boolean = 'AND';
        }

        $sortby = (array_key_exists('DBTB_sort', UserRequest::$post)) ? UserRequest::$post['DBTB_sort'] : CLX_CHAR_EMPTY_STRING;
        $this->setFilterKeys($filter, $sortby, CLX_NUM_ZERO);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['DBTB_id'] = $recordid;
        $record['DBTB_idcode'] = $values['IDCODE'];
        $record['DBTB_idname'] = $values['IDNAME'];
        $record['DBTB_version'] = $values['VERSION'];
        $record['DBTB_fullversion'] = $values['VERSION'] . '-' . $values['VERSION_COUNTER'];
        $record['DBTB_version_counter'] = $values['VERSION_COUNTER'];
        $record['DBTB_cdate'] = xFormat::date($values['CDATE']);
        $record['DBTB_mdate'] = xFormat::date($values['MDATE']);
        $record['DBTB_username'] = $values['USERNAME'];
        $record['DBTB_pttn_enabled'] = $values['PTTN_ENABLED'];
        $record['DBTB_pttn_options'] = $values['PTTN_OPTIONS'];
        $record['DBTB_description'] = $values['DESCRIPTION'];
        $record['DBTB_modulecode'] = uppercase($values['MODULECODE']);
        if (kDatabaseType === 'oci') {
		  if ($values['TYPE'] <> 'TEMP') {
			 $record['DBTB_type'] = 'InnoDB';
		  }
		  $record['DBTB_type'] = $values['TYPE'];
        } else {
		  $record['DBTB_type'] = $values['TYPE'];
        }
        $record['DBTB_view_query'] = $values['VIEW_QUERY'];
        $record['DBTB_audit'] = $values['AUDITU'];
        //
        $record['DBTB_truncable'] = (!isset($values['TRUNCABLE']) || is_null($values['TRUNCABLE']) ? 1 : $values['TRUNCABLE']);
        //
        $record['DBTB_status'] = (!empty($values['STATUS'])) ? 1 : 0;
        $DICT = &getTool('DICT');
        $DICT->loadEntry('DBTB_recordname_singular', $values['RECORDNAME_SINGULAR'], $record);
        $DICT->loadEntry('DBTB_recordname_plural', $values['RECORDNAME_PLURAL'], $record);
        return $record;
    }

    function saveRecord() {
        $DICT = &getTool('DICT');
        $oldrecord = ClxCommon::$db->getRecord($this->tablename, 'ID=' . UserRequest::$post['DBTB_id']);
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['DBTB_idcode'];
        $record['IDNAME'] = UserRequest::$post['DBTB_idname'];
        $record['VERSION'] = UserRequest::$post['DBTB_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBTB_description'];
        $record['MODULECODE'] = uppercase(UserRequest::$post['DBTB_modulecode']);
        $record['TYPE'] = UserRequest::$post['DBTB_type'];
        $record['VIEW_QUERY'] = clx_html_spchrs_decode(UserRequest::$post['DBTB_view_query']);
        $record['PTTN_ENABLED'] = UserRequest::$post['DBTB_pttn_enabled'];
        $record['PTTN_OPTIONS'] = UserRequest::$post['DBTB_pttn_options'];
        $record['AUDITU'] = UserRequest::$post['DBTB_audit'];
        //
        $record['TRUNCABLE'] = UserRequest::$post['DBTB_truncable'];
        //
		$sql='select VIEW_QUERY FROM ' .$this->tablename. ' WHERE ID=' . UserRequest::$post['DBTB_id'];
        //$this->debug=true;
		if ($this->debug==true) {
			pre('sql view'.__LINE__);
			pre($sql);
		}
		if (($dat=trim(ClxCommon::$db->getOne($sql)))==trim($record['VIEW_QUERY'])) {	
        $record['STATUS'] = (!empty(UserRequest::$post['DBTB_status'])) ? 1 : 0;
        }
		else {
			if ($this->debug==true) {
				pre('echo diferentes db_tables_'.__LINE__);
				
				pre($dat);
				echo '------------';
				pre($record['VIEW_QUERY']);
			}
			$record['STATUS'] = 1;
		}
        $record['RECORDNAME_SINGULAR'] = $DICT->saveEntry('DBTB_recordname_singular', UserRequest::$post);
        $record['RECORDNAME_PLURAL'] = $DICT->saveEntry('DBTB_recordname_plural', UserRequest::$post);
        ClxFreeReturn(ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['DBTB_id']));
        if ($record['TYPE'] == 'VIEW') {
		  $sql = 'DELETE FROM CLX_DB_INDEX_COLUMNS WHERE INDEX_CODE IN (SELECT IDCODE FROM CLX_DB_TABLEINDEXES WHERE TABLECODE =' . singleQuote($record['IDCODE']) . ')';
		  ClxCommon::$db->executeQuery($sql);
		  $sql = 'DELETE FROM CLX_DB_TABLEINDEXES WHERE TABLECODE =' . singleQuote($record['IDCODE']);
		  ClxCommon::$db->executeQuery($sql);
        } else {
		  if ((($oldrecord['IDNAME'] != $record['IDNAME']) || ($oldrecord['MODULECODE'] != uppercase($record['MODULECODE']))) && ClxCommon::$db->tableExists(uppercase($oldrecord['MODULECODE']) . '_' . $oldrecord['IDNAME'])) {
			 ClxCommon::$db->renameTable(uppercase($oldrecord['MODULECODE'] . '_' . $oldrecord['IDNAME']), uppercase($record['MODULECODE'] . '_' . $record['IDNAME']));
		  }
        }
        $this->versionIncrease('ID', UserRequest::$post['DBTB_id']);
		IF (isset($this->debugCF) && $this->debugCF==true) {
			echo 'record_'.__LINE__;
			PRE($record);
        }
        if (!empty($record['STATUS'])) {
		  if ($record['TYPE'] <> 'VIEW') {

			 if ($oldrecord['TYPE'] != $record['TYPE']) {
			     if ($record['TYPE'] === 'TEMP') {
			         ClxCommon::$db->dropTable(uppercase($record['MODULECODE'] . '_' . $record['IDNAME']));
			     } else {
			         ClxCommon::$db->changeTableType(uppercase($record['MODULECODE'] . '_' . $record['IDNAME']), $record['TYPE']);
			     }
			 }
		  } else {
			 ClxCommon::$db->alterView(uppercase($record['MODULECODE'] . '_' . $record['IDNAME']), $record['VIEW_QUERY']);
		  }
		}	
		if ( UserRequest::$post['DBTB_historic']==1 OR $record['MODULECODE']=='USER' ) {
			$this->gen_Historic(UserRequest::$post);
		}
        
    }

    function createRecord_() {
        $DICT = &getTool('DICT');
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = uppercase(UserRequest::$post['DBTB_idcode']);
        $record['IDNAME'] = uppercase(UserRequest::$post['DBTB_idname']);
        $record['VERSION'] = UserRequest::$post['DBTB_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['DBTB_description'];
        $record['MODULECODE'] = uppercase(UserRequest::$post['DBTB_modulecode']);
        $record['TYPE'] = UserRequest::$post['DBTB_type'];
        $record['AUDITU'] = UserRequest::$post['DBTB_audit'];
        //
        $record['TRUNCABLE'] = UserRequest::$post['DBTB_truncable'];
        //
		IF ($record['MODULECODE']=='USER')
			UserRequest::$post['DBTB_historic']=1;
		$record['HISTORIC']= (!empty(UserRequest::$post['DBTB_historic'])) ? 1 : 0; 
        
        $record['STATUS'] = (!empty(UserRequest::$post['DBTB_status'])) ? 1 : 0;
        $record['VIEW_QUERY'] = clx_html_spchrs_decode(UserRequest::$post['DBTB_view_query']);
        $record['PTTN_ENABLED'] = UserRequest::$post['DBTB_pttn_enabled'];
        $record['PTTN_OPTIONS'] = UserRequest::$post['DBTB_pttn_options'];
        $record['RECORDNAME_SINGULAR'] = $DICT->createEntry('DBTB_recordname_singular', UserRequest::$post);
        $record['RECORDNAME_PLURAL'] = $DICT->createEntry('DBTB_recordname_plural', UserRequest::$post);
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', uppercase($record['MODULECODE']));
        if ($record['TYPE'] === 'VIEW') {
		  if ($record['STATUS'] === '1') {
			 ClxCommon::$db->createView(uppercase($record['MODULECODE'] . '_' . $record['IDNAME']), $record['VIEW_QUERY']);
		  }
        } else {
		  $record['ID'] = $recordid;
		  $this->createTable($record);
		  $DBTF = &getTool('DBTF'); //  Table fields
		  $DBTF->createTable($record);
		  $DBTI = &getTool('DBTI'); // Table indexes
		  $DBTI->createTable($record);
		
			$id=ClxCommon::$db->getOne('SELECT ID FROM CLX_DB_TABLES WHERE IDCODE='.singleQuote($record['IDCODE']));

			$antpost=UserRequest::$post;
			$antget=UserRequest::$get;
			$antrequest=UserRequest::$request;

			UserRequest::$request=UserRequest::$post=UserRequest::$post= array ( 'ac' => 'FIX',  'to' => 'DBTB', 'idlist' => $id,  'next' => '1', 'relfm' => '', 'submit1' => 'Arreglar Tabla');
			$content=$this->action('FIX',array($id));

			UserRequest::$post=$antpost;
			UserRequest::$request=$antrequest;
			UserRequest::$get=$antget;

        }
        // Create basic security in order to allow current user
        $this->create_def_sec($record);
		
		if (UserRequest::$post['DBTB_historic']==1 && UserRequest::$post['ac']!=='DUPLICATE') {
			$this->gen_Historic(UserRequest::$post);
		}

        return $recordid;
    }
    
    function create_def_sec($dbtb_record) {
        $create_def_sec_menu = (defined('DBTB_CREATE_DEF_SEC_MENU') ? trim(DBTB_CREATE_DEF_SEC_MENU) : '');
        $create_def_sec_menu_code_prefix = (defined('DBTB_CREATE_DEF_SEC_MENU_CODE') ? trim(DBTB_CREATE_DEF_SEC_MENU_CODE) : '');
        $create_def_sec_grp = (defined('DBTB_CREATE_DEF_SEC_GRP') ? trim(DBTB_CREATE_DEF_SEC_GRP) : '');
        if(is_null($create_def_sec_menu_code_prefix) || $create_def_sec_menu_code_prefix === CLX_CHAR_EMPTY_STRING) {
		  $create_def_sec_menu_code_prefix = 'USRM_';
        }
        $UIMI_idcode = '';
        if($create_def_sec_menu !== '') {
		  // Default parent menu defined, so create menu entry
		  $UIMI_idcode = $create_def_sec_menu_code_prefix . $dbtb_record['IDCODE'];
            $qty = ClxCommon::$db->oneExistsRecords('CLX_UI_MENUITEMS', 'IDCODE=' . singleQuote($UIMI_idcode));
		  if ($qty === CLX_NUM_ZERO) {
			 $record = array();
			 $record['IDCODE'] = $UIMI_idcode;
			 $record['NAME'] = $dbtb_record['RECORDNAME_PLURAL'];
			 $record['ACTIVE'] = '1';
			 $record['DESCRIPTION'] = 'User Table ' . $dbtb_record['IDCODE'] . ' - ' . $dbtb_record['MODULECODE'] . ' ' . $dbtb_record['IDNAME'];
			 $record['TOOLCODE'] = '';
			 $record['MODULECODE'] = uppercase($dbtb_record['MODULECODE']);
			 $record['VERSION'] = '1.0.0';
			 $record['VERSION_COUNTER'] = 0;
			 $record['CDATE'] = $dbtb_record['CDATE'];
			 $record['MDATE'] = $dbtb_record['MDATE'];
			 $record['USERNAME'] = $dbtb_record['USERNAME'];
			 $record['TYPE'] = 'LSTR';
			 $record['PARENT'] = $create_def_sec_menu;
			 $record['SEQNUM'] = 0;
			 $record['EXTRA1'] = $dbtb_record['IDCODE'];
			 $record['EXTRA2'] = null;
			 $record['EXTRA3'] = null;
			 $record['EXTRA4'] = null;
			 $record['EXTRA5'] = null;
			 ClxCommon::$db->createRecord('CLX_UI_MENUITEMS', $record, false);
		  }
        }
        if($create_def_sec_grp !== '') {
		  // Default security group defined, so create security entries
		  if($create_def_sec_menu !== '') {
			 // Default parent menu defined, so create menu entries
                $qty = ClxCommon::$db->oneExistsRecords('CLX_AC_MENUITEMS', 'OWNERCODE=' . singleQuote('GRUP') . ' AND OWNERREF in(' . singleQuoteArray(explode(",",uppercase($create_def_sec_grp))) . ') AND MENUID=' . singleQuote($UIMI_idcode) . ' AND TOOLCODE=' . singleQuote('EDIT') . ' AND TABLECODE=' . singleQuote($dbtb_record['IDCODE']) );
			 if ($qty === CLX_NUM_ZERO) {
					$create_def_sec_grpx=explode(',',$create_def_sec_grp);
					foreach($create_def_sec_grpx as $key=>$sec_grpx) {	
						$record = array();
						$record['OWNERCODE'] = 'GRUP';
						$record['OWNERREF'] = UPPERCASE($sec_grpx);
						$record['MENUID'] = $UIMI_idcode;
						$record['TOOLCODE'] = 'EDIT';
						$record['TABLECODE'] = $dbtb_record['IDCODE'];
						$record['ISADMIN'] = CLX_NUM_ONE;
						$record['CANREAD'] = CLX_NUM_ONE;
						$record['CANMODIFY'] = CLX_NUM_ONE;
						$record['CANDELETE'] = CLX_NUM_ONE;
						$record['CANAPPEND'] = CLX_NUM_ONE;
						ClxCommon::$db->createRecord('CLX_AC_MENUITEMS', $record, false);
					}	
				}
		  }
		  // Create security for table entry
            $qty = ClxCommon::$db->oneExistsRecords('CLX_AC_TABLES', 'OWNERCODE=' . singleQuote('GRUP') . ' AND OWNERREF in (' . singleQuoteArray(explode(",",uppercase($create_def_sec_grp))) . ') AND TABLECODE=' . singleQuote($dbtb_record['IDCODE']) );
		  if ($qty === CLX_NUM_ZERO) {
			 $record = array();
				$create_def_sec_grpx=explode(',',$create_def_sec_grp);
				foreach($create_def_sec_grpx as $key=>$sec_grpx) {	
					$record['OWNERCODE'] = 'GRUP';
					$record['OWNERREF'] = UPPERCASE($sec_grpx);
					$record['TABLECODE'] = $dbtb_record['IDCODE'];
					$record['ISADMIN'] = CLX_NUM_ONE;
					$record['CANREAD'] = CLX_NUM_ONE;
					$record['CANMODIFY'] = CLX_NUM_ONE;
					$record['CANDELETE'] = CLX_NUM_ONE;
					$record['CANAPPEND'] = CLX_NUM_ONE;
					ClxCommon::$db->createRecord('CLX_AC_TABLES', $record, false);
				}	
			}
		  // Create user entry for security group
            $qty = ClxCommon::$db->oneExistsRecords('CLX_AC_GROUP_MEMBERS', 'USERNAME=' . singleQuote($dbtb_record['USERNAME']) . ' AND GROUPCODE in (' . singleQuoteArray(explode(",",UPPERCASE($create_def_sec_grp))).')');
		  if ($qty === CLX_NUM_ZERO) {
				$create_def_sec_grpx=explode(',',$create_def_sec_grp);
				foreach($create_def_sec_grpx as $key=>$sec_grpx) {	
					$record = array();
					$record['USERNAME'] = $dbtb_record['USERNAME'];
					$record['GROUPCODE'] = UPPERCASE($sec_grpx);
					ClxCommon::$db->createRecord('CLX_AC_GROUP_MEMBERS', $record, false);
				}	
		  }
        }
    }
		  
    function duplicateRecord() {
        $DBTI = &getTool('DBTI');
        $tableid = $this->createRecord_();
        $tablecode = UserRequest::$post['DBTB_idcode'];
        $originaltableid = UserRequest::$post['DBTB_id'];
        $originaltablerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $originaltableid);
		$originaltablecode='';
		if (!empty($originaltablerec['IDCODE'])) {
	        $originaltablecode = $originaltablerec['IDCODE'];
	        $tablefields = ClxCommon::$db->getRecords('CLX_DB_TABLEFIELDS', 'TABLECODE=' . singleQuote($originaltablecode));
	        while (($tablefield = ClxCommon::$db->getNextRecord($tablefields))) {
			  if ($tablefield['FIELDNAME'] !== 'ID') {
				 unset($tablefield['ID']);
				 $tablefield['TABLECODE'] = $tablecode;
					$extras=array(	'type'=>'SELECT',
												'returnid'=>false,	
												'extracriteria'=>	' NOT EXISTS (SELECT 1 FROM CLX_DB_TABLEFIELDS WHERE TABLECODE='.SINGLEQUOTE($tablefield['TABLECODE']).	' AND FIELDNAME= '.SINGLEQUOTE($tablefield['FIELDNAME']).')');
				    $tablefieldid = ClxCommon::$db->createRecord('CLX_DB_TABLEFIELDS', $tablefield, false,$extras);
				    unset($tablefieldid);
			  }
	        }
			ClxCommon::$db->freeResult($tablefields);
		}
        $indexcounter = (int) CLX_NUM_ONE;
        
		$tableindexesdeletenew = ClxCommon::$db->executequery('SELECT COUNT(1) X, MIN(ID) ID FROM CLX_DB_TABLEINDEXES WHERE FIELDNAME='.SINGLEQUOTE('ID').' AND TABLECODE=' . singleQuote($tablecode).' GROUP BY TABLECODE HAVING COUNT(1)>1');
		//echo "-----".$tablecode."------1";
		while ($tableindex = ClxCommon::$db->getNextRecord($tableindexesdeletenew)) {
			ClxCommon::$db->executequery($SQL='DELETE FROM CLX_DB_TABLEINDEXES WHERE ID='.$tableindex['ID']);
			ClxCommon::$db->executequery('COMMIT');
		//	PRE($SQL);
		}
		 ClxCommon::$db->freeResult($tableindexesdeletenew);
		$tableindexesdeletenew = ClxCommon::$db->executequery('SELECT ID,IDCODE FROM CLX_DB_TABLEINDEXES A WHERE A.TABLECODE='.singlequote($tablecode).' AND A.FIELDNAME IS NULL ');
		//echo "-----".$tablecode."------2";
		while ($tableindex = ClxCommon::$db->getNextRecord($tableindexesdeletenew)) {
			ClxCommon::$db->executequery($SQL='DELETE FROM CLX_DB_TABLEINDEXES WHERE ID='.$tableindex['ID']);
			//PRE($SQL);
			ClxCommon::$db->executequery($SQL='DELETE FROM CLX_DB_INDEX_COLUMN WHERE INDEX_CODE='.$tableindex['IDCODE']);
			ClxCommon::$db->executequery('COMMIT');
			//PRE($SQL);
		}
		//echo "-----".$tablecode."------3";
        $tableindexes = ClxCommon::$db->getRecords('CLX_DB_TABLEINDEXES', 'TABLECODE=' . singleQuote($originaltablecode));
        while ($tableindex = ClxCommon::$db->getNextRecord($tableindexes)) {
		  $oldindex_code = $tableindex['IDCODE'];
		  $indexcounter++;
		  $tableindex['IDCODE'] = $DBTI->indexDefCode($tablecode) . $indexcounter;
		  //setting new index code
			if ($tableindex['TYPE'] === 'PRI') {
			 continue;
		  }
		  $tableindex['TABLECODE'] = $tablecode;
		  unset($tableindex['ID']);
			$extras=array(	'type'=>'SELECT',
							'returnid'=>false,	
							'extracriteria'=>	' NOT EXISTS (SELECT 1 FROM CLX_DB_TABLEINDEXES WHERE TABLECODE='.SINGLEQUOTE($tableindex['TABLECODE']).	' AND FIELDNAME= '.SINGLEQUOTE($tableindex['FIELDNAME']).' AND TYPE= '.SINGLEQUOTE($tableindex['TYPE']) .' AND FIELDNAME_SEC= '.SINGLEQUOTE($tableindex['FIELDNAME_SEC']).')'
			);
			
			ClxCommon::$db->createRecord('CLX_DB_TABLEINDEXES', $tableindex, false,$extras) ;
			
		  //adding index columns
		  $tableindexescols = ClxCommon::$db->getRecords('CLX_DB_INDEX_COLUMNS', 'INDEX_CODE=' . singleQuote($oldindex_code));
		  while (($tableindexescol = ClxCommon::$db->getNextRecord($tableindexescols))) {
			 unset($tableindexescol['ID']);
			 $tableindexescol['INDEX_CODE'] = $tableindex['IDCODE'];
			 ClxCommon::$db->createRecord('CLX_DB_INDEX_COLUMNS', $tableindexescol, false);
		  }
		  ClxCommon::$db->freeResult($tableindexes);
        }
		 ClxCommon::$db->freeResult($tableindexescols);
        if (!(UserRequest::$post['DBTB_type'] === 'VIEW' || (UserRequest::$post['DBTB_type'] === 'TEMP' && kDatabaseType <> 'oci'))) {
		  //only include events if not temporary table
		  $events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote('DBTB'), 'OWNERID=' . singlequote($originaltableid));
		  while (($eventrec = ClxCommon::$db->getNextRecord($events))) {
			 unset($eventrec['ID']);
			 $eventrec['OWNERID'] = $tableid;
			 ClxCommon::$db->createRecord('CLX_EVENTS', $eventrec, false);
		  }
		  ClxCommon::$db->freeResult($events);
        }
        $tabledfields = ClxCommon::$db->getRecords('CLX_DB_TABLEDFIELDS', 'TABLECODE=' . singleQuote($originaltablecode));
        while (($tabledfield = ClxCommon::$db->getNextRecord($tabledfields))) {
		  if ($tabledfield['FIELDNAME'] !== 'ID') {
			 unset($tabledfield['ID']);
			 $tabledfield['TABLECODE'] = $tablecode;
				$extras=array(	'type'=>'SELECT',
								'returnid'=>false,	
								'extracriteria'=>		' NOT EXISTS (SELECT 1 FROM CLX_DB_TABLEFIELDS WHERE TABLECODE='. SINGLEQUOTE($tabledfield['TABLECODE']). ' AND FIELDNAME= '.SINGLEQUOTE($tabledfield['FIELDNAME']).')' );
		
			 $tabledfieldid = ClxCommon::$db->createRecord('CLX_DB_TABLEDFIELDS', $tabledfield, false,$extras );
			 unset($tabledfieldid);
		  }
        }
		ClxCommon::$db->freeResult($tabledfields);
        return $tableid;
    }

    function deleteRecordSk($id) {
        $DBDR = &getTool('DBDR'); // Table Dynamic Relations
        $DBTI = &getTool('DBTI'); // Table Indexes
        $EVNT = &getTool('EVNT'); // Events
        $tablerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $tablecode = $tablerec['IDCODE'];
        $relations = ClxCommon::$db->getRecords('CLX_DB_DYNRELATIONS', 'PARENT_TABLECODE=' . singleQuote($tablecode) . ' OR CHILD_TABLECODE=' . singleQuote($tablecode));
        while (($relation = ClxCommon::$db->getNextRecord($relations))) {
		  $DBDR->deleteRecordSk($relation['ID']);
        }
		ClxCommon::$db->freeResult($relations);
        $events = ClxCommon::$db->getRecords('CLX_EVENTS', 'OWNERCODE=' . singleQuote($this->code), 'OWNERID=' . singlequote($id));
        while ($event = ClxCommon::$db->getNextRecord($events)) {
		  $EVNT->deleteRecordSk($event['ID']);
        }
		ClxCommon::$db->freeResult($events);
        $indexes = ClxCommon::$db->getRecords('CLX_DB_TABLEINDEXES', 'TABLECODE=' . singleQuote($tablecode));
        while ($index = ClxCommon::$db->getNextRecord($indexes)) {
		  $DBTI->deleteRecordSk($index['ID']);
        }
		ClxCommon::$db->freeResult($indexes);
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEFIELDS', 'TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords('CLX_DB_TABLEDFIELDS', 'TABLECODE=' . singleQuote($tablecode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', uppercase($tablerec['MODULECODE']));
        if ($tablerec['TYPE'] <> 'VIEW') {
		  $this->deleteTable($tablerec);
        } else {
		  ClxCommon::$db->dropView(uppercase($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME']));
        }
    }

    function defaultRecord() {
        $record = array();
        $record['DBTB_id'] = CLX_INIT_ZERO;
        $record['DBTB_idcode'] = '';
        $record['DBTB_idname'] = '';
        $record['DBTB_version'] = '1.0.0';
        $record['DBTB_cdate'] = '';
        $record['DBTB_mdate'] = '';
        $record['DBTB_username'] = $_SESSION['user_record']['username'];
        $record['DBTB_description'] = '';
        $record['DBTB_modulecode'] = '';
        $record['DBTB_view_query'] = '';
        $record['DBTB_type'] = $this->getDefaultTableType();
        $record['DBTB_audit'] = CLX_INIT_ZERO;
        //
        $record['DBTB_truncable'] = CLX_NUM_ONE;
        //
        $record['DBTB_status'] = CLX_INIT_ZERO;
        $record['DBTB_pttn_enabled'] = CLX_INIT_ZERO;
        $record['DBTB_pttn_options'] = '';
        $record['DBTB_recordname_singular'] = '';
        $record['DBTB_recordname_singular_DICT'] = '';
        $record['DBTB_recordname_plural'] = '';
        $record['DBTB_recordname_plural_DICT'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', uppercase($modulecode));
    }

    /**
     * Returns array with default record based on table structure
     * @param string $tablecode
     * @return array
     */
    function tableDefaultRecord($tablecode) {
        $record = array();
        $sql = 'SELECT F.FIELDNAME, F.FIELDTYPE, F.FIELD_DEFAULT FROM CLX_DB_FIELDS F, CLX_DB_TABLEFIELDS T WHERE F.FIELDNAME=T.FIELDNAME AND T.TABLECODE=' . singleQuote($tablecode);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  if ($r['FIELDTYPE'] === 'BOOL' && $r['FIELD_DEFAULT'] === CLX_CHAR_EMPTY_STRING)
			 $r['FIELD_DEFAULT'] = CLX_INIT_ZERO;
		  $record[$r['FIELDNAME']] = $r['FIELD_DEFAULT'];
		  if ($r['FIELDTYPE'] === 'DATE') {
			 $record[$r['FIELDNAME']] = xFormat::date($record[$r['FIELDNAME']], 'Y-m-d');
		  } elseif ($r['FIELDTYPE'] === 'DATI') {
			 $record[$r['FIELDNAME']] = xFormat::date($record[$r['FIELDNAME']], 'Y-m-d H:i:s');
		  } elseif ($r['FIELDTYPE'] === 'TIME') {
			 $record[$r['FIELDNAME']] = xFormat::date($record[$r['FIELDNAME']], 'Y-m-d H:i:s');
		  }
        }
		ClxCommon::$db->freeResult($rs);
        return $record;
    }

    /**
     * DEPRECATED in favor of chooseRecord($name, $default)
     * @param array $msg
     * @param string $prompt
     * @param string $name
     * @param string $default
     * @param obj $table
     * @param bool $required
     * @return string
     */
    function placeChooseTable($msg, $prompt, $name, $default, &$table, $required = false) {
        if (!empty($msg[$name]))
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => (($required) ? CLX_CLASS_PROMPTREQUIRED : CLX_CLASS_PROMPT)));
        $table->addCell($this->chooseRecord($name, $default, null, CLX_NUM_TEN, CLX_NUM_FOUR), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    // DEPRECATED in favor of recordExists($field_label, &$field_value, $msg)
    function tableExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, $this->primary_field, $this->primary_field . '=' . singleQuote($idcode));
        }
        return($qty > CLX_NUM_ZERO) ? true : false;
    }

    function createTable($tablerec) {
        if ($tablerec['TYPE'] === 'VIEW') {
		  //nothing to do
        } else {
		  $fieldinstance = getToolObject('DBFT', 'PRIM');
		  $field = $fieldinstance->fieldProperties(array());
		  $tablename = uppercase($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME']);
              $tabledef = array('idcode' => $tablerec['IDCODE'], 'idname' => $tablerec['IDNAME'], 'name' => $tablename, 'type' => $tablerec['TYPE'], 'fields' => array('ID' => $field), 'indexes' => array());
          //  if ($tablerec['STATUS'] === '1') {
                $id = ClxCommon::$db->createTable($tablename, $tablerec['TYPE'], $tabledef['fields']);
				apc_deleteFct($tablename);
           // }
            unset($id);
        }
    }
    
    //FUNCIÓN DUPLICADA DE LA ORIGINAL PERO CON LINEAS HABILITADAS PARA QUE QUE FUNCIONE LA CREACIÓN DEL CUESTIONARIO EN MERCADEO
    function createTable2($tablerec) {
        if ($tablerec['TYPE'] === 'VIEW') {
            //nothing to do
        } else {
            $fieldinstance = getToolObject('DBFT', 'PRIM');
            $field = $fieldinstance->fieldProperties(array());
            $tablename = uppercase($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME']);
		  $tabledef = array('idcode' => $tablerec['IDCODE'], 'idname' => $tablerec['IDNAME'], 'name' => $tablename, 'type' => $tablerec['TYPE'], 'fields' => array('ID' => $field), 'indexes' => array());
		  if ($tablerec['STATUS'] === '1') {
			 $id = ClxCommon::$db->createTable($tablename, $tablerec['TYPE'], $tabledef['fields']);
                apc_deleteFct($tablename);
		  }
		  unset($id);
        }
    }

    function deleteTable($tablerec) {
		$sql='DELETE FROM CLX_DB_TABLE_HIST X WHERE TABLE_CODE='.singlequote($tablerec['IDCODE']).' OR  TABLE_CODE_HIST='.singlequote($tablerec['IDCODE']);
		if ($this->debugCF==true) {pre('after drop table'.__LINE__);pre($sql);}		
		ClxCommon::$db->executequery($sql);
		ClxCommon::$db->executequery('COMMIT');
		
		//pre($tablerec);
        $tablename = uppercase($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME']);
        ClxCommon::$db->dropTable($tablename);
		apc_deleteFct($tablename);
    }

    function tableInfo($tablecode) {
        $tablerec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($tablecode));
        $tablename = $tablerec['MODULECODE'] . '_' . $tablerec['IDNAME'];
        $tableinfo = array('idcode' => $tablecode,
		  'idname' => $tablerec['IDNAME'],
		  'name' => $tablename,
		  'type' => $tablerec['TYPE'],
		  'viewsql' => $tablerec['VIEW_QUERY'],
		  'fields' => array(),
		  'indexes' => array(),
		  'case_sensitive' => CLX_NUM_ZERO);
        // get Field Information
        /* IFNULL(TF.FIELD_DEFAULT, F.FIELD_DEFAULT) FIELD_DEFAULT, */
		switch (kDatabaseType) {
			case 'oci':
				$sql = 'SELECT F.ID, F.FIELDNAME, F.FIELDTYPE, F.FIELD_SIZE, NVL(TF.FIELD_REQUIRED, F.FIELD_REQUIRED) FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN, TF.FIELD_DEFAULT, F.FIELD_DISPLAYSIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9 FROM CLX_DB_TABLES TB, CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TB.IDCODE = TF.TABLECODE AND TF.FIELDNAME = F.FIELDNAME AND TB.TYPE NOT IN (' . CLX_CHAR_SINGLE_QUOTE . 'VIEW' . CLX_CHAR_SINGLE_QUOTE . ') AND TF.TABLECODE = ' . singleQuote($tablecode) . ' ORDER BY TF.SEQNUM ASC';
				break;
			default:
        $sql = 'SELECT F.ID, F.FIELDNAME, F.FIELDTYPE, F.FIELD_SIZE, IFNULL(TF.FIELD_REQUIRED, F.FIELD_REQUIRED) FIELD_REQUIRED, F.FIELD_FORMAT, F.FIELD_ALIGN, TF.FIELD_DEFAULT, F.FIELD_DISPLAYSIZE, F.FIELD_EXTRA1, F.FIELD_EXTRA2, F.FIELD_EXTRA3, F.FIELD_EXTRA4, F.FIELD_EXTRA5, F.FIELD_EXTRA6, F.FIELD_EXTRA7, F.FIELD_EXTRA8, F.FIELD_EXTRA9 FROM CLX_DB_TABLES TB, CLX_DB_TABLEFIELDS TF, CLX_DB_FIELDS F WHERE TB.IDCODE = TF.TABLECODE AND TF.FIELDNAME = F.FIELDNAME AND TB.TYPE NOT IN (' . CLX_CHAR_SINGLE_QUOTE . 'VIEW' . CLX_CHAR_SINGLE_QUOTE . ') AND TF.TABLECODE = ' . singleQuote($tablecode) . ' ORDER BY TF.SEQNUM ASC';
		}
		if (ClxCommon::$db->debugCF==true){
		   echo "--->db tables info table oracle".__LINE__;
		   pre($sql);	
		}	
        $records = ClxCommon::$db->executeQuery($sql);
        while ($field = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  if (ClxCommon::$db->debugCF==true){
				echo 'field'.__LINE__;
				pre($field);
			}
			$fieldobj = getToolObject('DBFT', $field['FIELDTYPE']);
		  $props = $fieldobj->fieldProperties($field);
			if (ClxCommon::$db->debugCF==true) {
				echo 'props'.__LINE__;
				pre($props);	
			}
				
		  if (!empty($fieldobj->multifield)) {
			 reset($props);
			 foreach ($props as $key => $prop) {
			     $tableinfo['fields'][$field['FIELDNAME'] . '_' . $key] = $prop;
			 }
		  } else {
				$props['original_type']=$field['FIELDTYPE'];
			 $tableinfo['fields'][$field['FIELDNAME']] = $props;
		  }
        }
		ClxCommon::$db->freeResult($records);
		if (ClxCommon::$db->debugCF==true){
			echo "<BR>====> db tables infotable".__LINE__ .'<BR>';
			pre($tableinfo);	
		}	
        // get indexes information
        $sql = 'SELECT DBTI.IDCODE, DBTI.TYPE, DBTC.FIELDNAME FROM CLX_DB_TABLEINDEXES DBTI, CLX_DB_INDEX_COLUMNS DBTC WHERE DBTI.IDCODE = DBTC.INDEX_CODE AND DBTI.TABLECODE = ' . singleQuote($tablecode) . ' ORDER BY DBTC.SEQNUM ASC';
        $records = ClxCommon::$db->executeQuery($sql);
        while ($index = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
		  $tableinfo['indexes'][$index['IDCODE']]['index'] = $index['TYPE'];
		  if (empty($tableinfo['indexes'][$index['IDCODE']]['fieldname'])) {
			 $tableinfo['indexes'][$index['IDCODE']]['fieldname'] = $index['FIELDNAME'];
		  } else {
			 $tableinfo['indexes'][$index['IDCODE']]['fieldname'] .= ',' . $index['FIELDNAME'];
		  }
        }
		ClxCommon::$db->freeResult($records);
        return $tableinfo;
    }

    //function checkTable($tablerec, $fix, $drop_fields=false, $fields_only=false, $indexes_only=false) {
    function checkTable($tablerec, $fix, $drop_fields = false, $fields_only = false, $indexes_only = false, $dump = false,$analize=true ) {
        $msg = array();
        $tablecode = $tablerec['IDCODE'];
        $tablename = uppercase($tablerec['MODULECODE'] . '_' . $tablerec['IDNAME']);
        $tabletype = $tablerec['TYPE'];
        if ($tabletype === 'TEMP' && kDatabaseType <> 'oci') {
		  //mysql temporary table alwais remove any aditional field
		  return array();
        }
        $tableinfo = $this->tableInfo($tablecode);
        //Test of disabling autoincrement
        $sql = 'SELECT FIELDNAME FROM CLX_DB_TABLEFIELDS WHERE DISABLE_AUTOINCR = 1 AND TABLECODE = ' . singleQuote($tablecode);
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $tableinfo['fields'][$r['FIELDNAME']]['properties'] = str_replace('auto_increment', '', $tableinfo['fields'][$r['FIELDNAME']]['properties']);
		  if ($tableinfo['fields'][$r['FIELDNAME']]['index'] === 'PRI') {//no primary key required
			 $tableinfo['fields'][$r['FIELDNAME']]['index'] = ''; //no primary
		  }
        }
		ClxCommon::$db->freeResult($rs);
		if (ClxCommon::$db->debugCF==true){
			echo "tableinfo checktable en clxdtabase".__LINE__;
				pre($tableinfo);
				echo "DUMP";
				pre($dump);
				
		}	
        ClxCommon::$db->checkTable($tableinfo, $fix, $msg, $fields_only, $indexes_only, $dump,false,$analize );
        if ($tabletype === 'VIEW') {
		  return $msg;
        }
        //Checking and fixing partitions
        if (!empty($fix) && !empty($tablerec['PTTN_ENABLED']) && !empty($tablerec['PTTN_OPTIONS'])) {
		  $sql = 'ALTER TABLE ' . $tablename . ' ' . $tablerec['PTTN_OPTIONS'];
		  ClxCommon::$db->executeQuery($sql);
        }
        $skeep = array();
        //Backward - clear all fields not found on metadata.
        if (!empty($drop_fields)) {
		  $fields = ClxCommon::$db->fieldList($tablename);
		  while ($field = ClxCommon::$db->getNextRecord($fields)) {
			 if (array_key_exists('COLUMN_NAME', $field))
			     $field['Field'] = $field['COLUMN_NAME'];
			 $recurrent = substr($field['Field'], CLX_NUM_ZERO, -CLX_NUM_THREE);
			 if (!empty($skeep[$recurrent])) {
			     continue;
			 }
                if (ClxCommon::$db->oneExistsRecords('CLX_DB_FIELDS', 'FIELDNAME = ' . singleQuote($recurrent) . ' AND FIELDTYPE = ' . singleQuote('RECU')) > CLX_NUM_ZERO) {
			     $skeep[$recurrent] = CLX_NUM_ONE;
			     continue;
			 }
                if (ClxCommon::$db->oneExistsRecords('CLX_DB_TABLEFIELDS', 'TABLECODE = ' . singleQuote($tablecode) . ' AND FIELDNAME = ' . singleQuote($field['Field'])) == CLX_NUM_ZERO) {
			     $error = array('tablename' => $tablecode,
			         'fieldname' => $field['Field'],
			         'error' => dict('GEN_ERR_DROP_REQUIRED'),
			         'comments' => '');
			     if (!empty($fix)) {
			         if (!empty($dump))
					   ClxCommon::$db->dumpScript = true;
			         ClxCommon::$db->dropField($tablename, $field['Field']);
			         if (!empty($dump))
					   ClxCommon::$db->dumpScript = false;
			         $error['comments'] = dict('GEN_LBL_FIXED');
			     }
			     $msg[] = $error;
			 }
		  }
		  ClxCommon::$db->freeResult($fields);
        }
        return $msg;
    }

    /**
     * Method will construct temporary table if not exists
     *
     * @param unknown_type $tablecode
     */
    function buildTemporaryTable($tablecode) {
        $tablename = tableName($tablecode);
        if (empty($tablename))
		  return false;
        if (ClxCommon::$db->tempTableExists($tablename)) {
		  $sql = 'DELETE FROM ' . $tablename;
		  ClxCommon::$db->executeQuery($sql);
        } else {
		  $tableinfo = $this->tableInfo($tablecode);
		  if (!ClxCommon::$db->tableExists($tableinfo['name'])) {
			 ClxCommon::$db->createTable($tableinfo['name'], $tableinfo['type'], $tableinfo['fields']);
                apc_deleteFct($tableinfo['name']);
			 foreach ($tableinfo['indexes'] as $indexname => $indexinfo) {
			     switch ($indexinfo['index']) {
			         case 'UNI':
					   ClxCommon::$db->createUniqueIndex($tableinfo['name'], $indexinfo['fieldname'], $indexname);
					   break;
			         case 'COM':
					   ClxCommon::$db->createUniqueIndex($tableinfo['name'], $indexinfo['fieldname'], $indexname);
					   break;
			         case 'MUL':
					   ClxCommon::$db->createIndex($tableinfo['name'], $indexinfo['fieldname'], $indexname);
					   break;
			         case 'PRI':
					   break;
			         default :
					   break;
			     }
			 }
		  }
        }
        return true;
    }

    function create_ui($idlist) {
        UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
        UserResponse::setRunTimeLimit(CLX_NUM_2000);
        $tableid = $idlist[CLX_NUM_ZERO];
        $tablerec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $tableid);
        $tablecode = $tablerec['IDCODE'];
        $tablename = $tablerec['IDNAME'];
        $modulecode = uppercase($tablerec['MODULECODE']);
        $DBMO = &getTool('DBMO');
        $d = xFormat::getNow('Y-m-d');
        // PREVIOUS VERIFICATIONS
        $errmsg = '';
        $dict_title_value = dict('GEN_LBL_QUICKSEARCH'); // "GEN_LBL_QUICKSEARCH" = B�squeda R�pida
        $dict_title_value_p2 = dict('GEN_LBL_ADV_SEARCH'); // "GEN_LBL_ADV_SEARCH" = B�squeda Avanzada
        if(strpos($dict_title_value, '&ac=CREATE_FAST') !== false) {
		  $errmsg .= 'Missing dictionary: GEN_LBL_QUICKSEARCH<br>';
        }
        if(strpos($dict_title_value_p2, '&ac=CREATE_FAST') !== false) {
		  $errmsg .= 'Missing dictionary: GEN_LBL_ADV_SEARCH<br>';
        }
        if($errmsg !== '') {
		  return '<br><br>' . note($errmsg, 'OK');
        }
        // 
        // Report Table -----------------------------------------
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('GEN_LBL_REPORT'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        // Table Columns -------------------------------------------------
        $fields = array();
        $fields_rs = ClxCommon::$db->executeQuery('SELECT F.FIELDNAME, UPPER(F.FIELDFORMLABEL)FIELDFORMLABEL, F.FIELDPREFIX, F.FIELDSUFIX, TB.FIELD_DEFAULT TABLE_FIELD_DEFAULT, TB.SEQNUM FROM CLX_DB_FIELDS F, CLX_DB_TABLEFIELDS TB WHERE TB.TABLECODE=' . singleQuote($tablecode) . ' AND TB.FIELDNAME = F.FIELDNAME ORDER BY TB.SEQNUM');
        while (($fieldrec = ClxCommon::$db->getNextRecord($fields_rs))) {
		  if ($fieldrec['FIELDNAME'] !== 'ID') {
			 $fields[] = $fieldrec;
		  }
        }
		ClxCommon::$db->freeResult($fields_rs);
        unset($fields_rs);
        // Table Index Columns -------------------------------------------------
        $index_fields = array();
        $fields_rs = ClxCommon::$db->executeQuery('SELECT DBIC.FIELDNAME, DBTI.TYPE NDX_TYPE FROM CLX_DB_TABLEINDEXES DBTI, CLX_DB_INDEX_COLUMNS DBIC WHERE DBTI.IDCODE = DBIC.INDEX_CODE AND DBTI.TABLECODE = ' . singleQuote($tablecode) . ' AND DBIC.FIELDNAME <> ' . singleQuote('ID') );
        while (($fieldrec = ClxCommon::$db->getNextRecord($fields_rs))) {
		  $index_fields[$fieldrec['FIELDNAME']] = $fieldrec['NDX_TYPE'];
        }
		ClxCommon::$db->freeResult($fields_rs);
        $index_fields_qty = count($index_fields);
        unset($fields_rs);
        // Query -------------------------------------------------
        $record = array();
        $querycode = $this->get_unique_code('CLX_DB_QUERIES', $tablecode);
        $record['IDCODE'] = $querycode;
        $record['IDNAME'] = $tablename;
        $record['VERSION'] = '1.0.0';
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = 'Auto generated default query';
        $record['MODULECODE'] = uppercase($modulecode);
        $queryid = ClxCommon::$db->createRecord('CLX_DB_QUERIES', $record, false);
        $table->addData(dict('GEN_LBL_QUERY'), '<a href="tool.php?to=DBQY&ac=DISPLAY&idlist=' . $queryid . '">' . $tablename . ' (' . $querycode . ')</a>');
        // Query Table ------------------------------------------
        $record = array();
        $record['QUERYCODE'] = $querycode;
        $record['TABLECODE'] = $tablecode;
        $record['INVISIBLE'] = '0';
        ClxCommon::$db->createRecord('CLX_DB_QUERY_TABLES', $record, false);
        // Query Columns -------------------------------------------------
        $record = array();
        $record['QUERYCODE'] = $querycode;
        $record['TABLECODE'] = $tablecode;
        $record['VISIBLE'] = '1';
        $record['FILTER'] = '0';
        $record['GROUPED'] = '0';
        $record['EXPRESSION'] = '';
        $record['CURRENCY_FIELD_TBL'] = '';
        $record['CURRENCY_FIELD'] = '';
        $record['FIELD_FORMAT'] = '0';
        $i = CLX_INIT_ZERO;
        foreach($fields as $fieldrec) {
		  if ($fieldrec['FIELDNAME'] !== 'ID') {
			 $record['FIELDNAME'] = $fieldrec['FIELDNAME'];
			 $record['SORT'] = (isset($index_fields[$fieldrec['FIELDNAME']]) ? '1' : '0');
			 $record['SEQNUM'] = $i;
			 ClxCommon::$db->createRecordVoid('CLX_DB_QUERY_FIELDS', $record, false);
			 $i++;
		  }
        }
        // Form -------------------------------------------------
        $record = array();
        $formcode = $this->get_unique_code('CLX_UI_FORMS', $tablecode);
        $record['IDCODE'] = $formcode;
        $record['IDNAME'] = $tablename;
        $record['VERSION'] = '1.0.0';
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = 'Auto generated default form';
        $record['TABLECODE'] = $tablecode;
        $record['MODULECODE'] = uppercase($modulecode);
        $record['DEFAULT_FORM'] = '0';
        $formid = ClxCommon::$db->createRecord('CLX_UI_FORMS', $record, false);
        $table->addData(dict('GEN_LBL_FORM'), '<a href="tool.php?to=UIFM&ac=DISPLAY&idlist=' . $formid . '">' . $tablename . ' (' . $formcode . ')</a>');
        // Form Panel -------------------------------------------------
        $record = array();
        $record['IDCODE'] = $formcode . '_001';
        $record['IDNAME'] = 'Information';
        $record['FORMCODE'] = $formcode;
        $record['DESCRIPTION'] = 'Auto generated panel';
        $record['PANEL_CLASS'] = '';
        $record['NUMCOLS'] = '1';
        $record['TYPE'] = '0';
        $record['INITIAL_STATE'] = '1';
        $record['TITLE'] = '';
        $record['CAPTION'] = '';
        $record['SEQNUM'] = '0';
        ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANELS', $record, false);
        $panelcode = $record['IDCODE'];
        // Form Panel Fields -------------------------------------------------
        $record = array();
        $record['PANELID'] = $panelcode;
        $record['CAPTION'] = '';
        $record['CAPTION_CLASS'] = '';
        //$fields = ClxCommon::$db->executeQuery('SELECT F.*, TB.FIELD_DEFAULT TABLE_FIELD_DEFAULT FROM CLX_DB_FIELDS F, CLX_DB_TABLEFIELDS TB WHERE TB.TABLECODE=' . singleQuote($tablecode) . ' AND TB.FIELDNAME = F.FIELDNAME ORDER BY TB.SEQNUM');
        $i = CLX_INIT_ZERO;
        //while (($fieldrec = ClxCommon::$db->getNextRecord($fields))) {
        foreach($fields as $fieldrec) {
		  if ($fieldrec['FIELDNAME'] !== 'ID') {
			 $record['FIELDNAME'] = $fieldrec['FIELDNAME'];
			 $record['FIELDSEQNUM'] = $i;
			 $record['FIELDFORMLABEL'] = UPPERCASE($fieldrec['FIELDFORMLABEL']);
			 $record['FIELDPREFIX'] = $fieldrec['FIELDPREFIX'];
			 $record['FIELDSUFIX'] = $fieldrec['FIELDSUFIX'];
			 $record['FIELD_DISPLAYSIZE'] = CLX_INIT_ZERO;
			 $record['FIELD_REQUIRED'] = CLX_NUM_TWO; //field default
			 $record['FIELD_DEFAULT'] = $fieldrec['TABLE_FIELD_DEFAULT'];
			 ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANEL_FIELDS', $record, false);
			 $i++;
		  }
        }
		ClxCommon::$db->freeResult($fields);
        // Search Form -------------------------------------------------
        $create_def_search_form = (defined('DBTB_CREATE_DEF_SRCH_FRM') ? DBTB_CREATE_DEF_SRCH_FRM : '');
        if($create_def_search_form === strval(CLX_NUM_ONE)) {
		  // Create panel title dictionaries
		  $DICT = &getTool('DICT');
		  $dict_title_entry = $DICT->createEntry($dict_title_value);
		  if($index_fields_qty > CLX_NUM_ZERO) {
			 $dict_title_entry_p2 = $DICT->createEntry($dict_title_value_p2);
		  }
		  // Search Form -------------------------------------------------
		  $formcode_lsrch = $this->get_unique_code('CLX_UI_FORMS', $formcode . '_SRCH');
		  $record = array();
		  $record['IDCODE'] = $formcode_lsrch;
		  $record['IDNAME'] = $tablename . ' (Search Form)';
		  $record['VERSION'] = '1.0.0';
		  $record['VERSION_COUNTER'] = '0';
		  $record['CDATE'] = $d;
		  $record['MDATE'] = $d;
		  $record['USERNAME'] = $_SESSION['user_record']['username'];
		  $record['DESCRIPTION'] = 'Auto generated default search form';
		  $record['TABLECODE'] = $tablecode;
		  $record['MODULECODE'] = uppercase($modulecode);
		  $record['DEFAULT_FORM'] = '0';
		  $formid_lsrch = ClxCommon::$db->createRecord('CLX_UI_FORMS', $record, false);
		  $table->addData(dict('GEN_LBL_FORM'), '<a href="tool.php?to=UIFM&ac=DISPLAY&idlist=' . $formid_lsrch . '">' . $tablename . ' (' . $formcode_lsrch . ')</a>');
		  // Search Form Panel -------------------------------------------------
		  $record = array();
		  $record['IDCODE'] = $formcode_lsrch . '_001';
		  $record['IDNAME'] = $dict_title_value;
		  $record['FORMCODE'] = $formcode_lsrch;
		  $record['DESCRIPTION'] = 'Auto generated search panel - ' . $dict_title_value;
		  $record['PANEL_CLASS'] = '';
		  $record['NUMCOLS'] = ($index_fields_qty > CLX_NUM_ZERO ? CLX_NUM_ONE : CLX_NUM_TWO);
		  $record['TYPE'] = CLX_INIT_ZERO;
		  $record['INITIAL_STATE'] = CLX_NUM_ONE;
		  $record['TITLE'] = $dict_title_entry;
		  $record['CAPTION'] = '';
		  $record['SEQNUM'] = CLX_INIT_ZERO;
		  ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANELS', $record, false);
		  $panelcode_lsrch = $record['IDCODE'];
		  // Advanced Search Form Panel -------------------------------------------------
		  if($index_fields_qty > CLX_NUM_ZERO) {
			 $record = array();
			 $record['IDCODE'] = $formcode_lsrch . '_002';
			 $record['IDNAME'] = $dict_title_value_p2;
			 $record['FORMCODE'] = $formcode_lsrch;
			 $record['DESCRIPTION'] = 'Auto generated search panel - ' . $dict_title_value_p2;
			 $record['PANEL_CLASS'] = 'form';
			 $record['NUMCOLS'] = CLX_NUM_TWO;
			 $record['TYPE'] = CLX_NUM_ONE;
			 $record['INITIAL_STATE'] = CLX_INIT_ZERO;
			 $record['TITLE'] = $dict_title_entry_p2;
			 $record['CAPTION'] = '';
			 $record['SEQNUM'] = CLX_INIT_ZERO;
			 ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANELS', $record, false);
			 $panelcode_lsrch_p2 = $record['IDCODE'];
		  }
		  // Form Panel Fields -------------------------------------------------
		  $record = array();
		  $record['CAPTION'] = '';
		  $record['CAPTION_CLASS'] = '';
		  $i = CLX_INIT_ZERO;
		  foreach($fields as $fieldrec) {
			 if ($fieldrec['FIELDNAME'] !== 'ID') {
			     $record['PANELID'] = ($index_fields_qty > CLX_NUM_ZERO && !isset($index_fields[$fieldrec['FIELDNAME']]) ? $panelcode_lsrch_p2 : $panelcode_lsrch);
			     $record['FIELDNAME'] = $fieldrec['FIELDNAME'];
			     $record['FIELDSEQNUM'] = $i;
			     $record['FIELDFORMLABEL'] = UPPERCASE($fieldrec['FIELDFORMLABEL']);
			     $record['FIELDPREFIX'] = $fieldrec['FIELDPREFIX'];
			     $record['FIELDSUFIX'] = $fieldrec['FIELDSUFIX'];
			     $record['FIELD_DISPLAYSIZE'] = CLX_INIT_ZERO;
			     $record['FIELD_REQUIRED'] = CLX_INIT_ZERO; // always optional field
			     //$record['FIELD_DEFAULT'] = $fieldrec['TABLE_FIELD_DEFAULT'];
			     $record['FIELD_DEFAULT'] = '';
			     ClxCommon::$db->createRecordVoid('CLX_UI_FORM_PANEL_FIELDS', $record, false);
			     $i++;
			 }
		  }
        }
        // Listing -------------------------------------------------
        $record = array();
        $listingcode = $this->get_unique_code('CLX_UI_LISTINGS', $tablecode);
        $record['IDCODE'] = $listingcode;
        $record['IDNAME'] = $tablename;
        $record['VERSION'] = '1.0.0';
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = 'Auto generated default listing';
        $record['TITLE'] = $tablerec['RECORDNAME_PLURAL'];
        $record['CAPTION'] = '';
        $record['MODULECODE'] = uppercase($modulecode);
        $record['TABLECODE'] = $tablecode;
        $record['QUERYCODE'] = $querycode;
        $record['CREATE_FORM'] = $formcode;
        $record['MODIFY_FORM'] = $formcode;
        $record['DISPLAY_FORM'] = $formcode;
        $record['DELETE_FORM'] = $formcode;
        //$record['SEARCH_FORM'] = '';
        $record['SEARCH_FORM'] = ($create_def_search_form === strval(CLX_NUM_ONE) ? $formcode_lsrch : '');
        $record['CONTROL_VIEWS'] = '1';
        $record['CONTROL_FILTERS'] = '1';
        $record['VIEW_SUMMARY'] = '0';
        //$record['CONTROL_EXPORT'] = '0';
        $record['CONTROL_EXPORT'] = '1';
        $record['EXEONSEARCH'] = '0';
        $listingid = ClxCommon::$db->createRecord('CLX_UI_LISTINGS', $record, false);
        $table->addData(dict('GEN_LBL_LISTING'), '<a href="tool.php?to=UILG&ac=DISPLAY&idlist=' . $listingid . '">' . $tablename . ' (' . $listingcode . ')</a> --&gt; <a href="tool.php?to=EDIT&ac=LIST&li='.$listingcode.'&menuid=">'.uppercase(dict('GEN_LBL_EXECUTE')).'</a>');
        // Module Version Increase
        $DBMO->versionIncrease('IDCODE', uppercase($modulecode));
        $c = new htmlButtonCancel(dict('GEN_LBL_RETURN'));
        $table->addCell($c->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        return $table->source();
    }

    function get_unique_code($tablename, $code) {
        $number = '';
        $done = false;
        while (empty($done)) {
            $done = (ClxCommon::$db->oneExistsRecords($tablename, 'IDCODE=' . singleQuote($code . $number)) == CLX_NUM_ZERO);
		  if (empty($done))
			 $number++;
        }
        return $code . $number;
    }

    function getDefaultTableType() {
        return 'InnoDB';
    }

    /**
     * Returns table record from tool metadata
     * @param string $fieldname
     * @return array
     */
    function getTable($tablecode) {
        $tablerec = array();
		  $fetched = false;
        if (kApcTtl_14>0) $tablerec = apc_fetchFct($findVar=kSpaceName . '.dbtb.' . $tablecode, $fetched);
		  if (empty($fetched)) {
			if (isset($GLOBALS['DBTB_'.$tablecode]))
				$tablerec=$GLOBALS['DBTB_'.$tablecode];
			else
				$GLOBALS['DBTB_'.$tablecode]=$tablerec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($tablecode));
		    if (kApcTtl_14>0 && is_array($tablerec) && count($tablerec)>0) { 
				apc_addFct($findVar, $tablerec, kApcTtl_14);
		  }
        }
        return $tablerec;
    }

    /**
     * Returns table fields and derived field from table definition
     * @param string $fieldname
     * @return array
     */
    function getTableAllFields($tablecode) {
		  $fetched = false;
		if (kApcTtl_14>0) $tablefields = apc_fetchFct(kSpaceName . '.dbtb.dbfl.' . $tablecode, $fetched);
		  if (empty($fetched)) {
			 $tablefields = array();
			 $sql = 'SELECT DBFL.*, 0 ISWRK_FLD FROM CLX_DB_FIELDS DBFL, CLX_DB_TABLEFIELDS DBTF WHERE DBFL.FIELDNAME = DBTF.FIELDNAME AND DBTF.TABLECODE=' . singleQuote($tablecode) . ' UNION ALL SELECT DBFL.*, 1 ISWRK_FLD FROM CLX_DB_FIELDS DBFL, CLX_DB_TABLEDFIELDS DBTD WHERE DBFL.FIELDNAME = DBTD.FIELDNAME AND DBTD.TABLECODE=' . singleQuote($tablecode) . ' AND NOT EXISTS(SELECT DBTF.ID FROM CLX_DB_TABLEFIELDS DBTF WHERE DBTD.TABLECODE = DBTF.TABLECODE AND DBTD.FIELDNAME = DBTF.FIELDNAME)';
			 $rs = ClxCommon::$db->executeQuery($sql);
			 while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
			     $tablefields[] = $r;
			 }
			 ClxCommon::$db->freeResult($rs);
			if (kApcTtl_14>0 && !empty($tablefields)) {
				apc_addFct(kSpaceName . '.dbtb.dbfl.' . $tablecode, $tablefields, kApcTtl_14);
		  }
        }
        return $tablefields;
    }
	function gen_Historic($table_info) {
		if (ClxCommon::$db->debugCF==true) {pre('gen_historic'.__LINE__);pre($table_info);pre('sql'); pre('SELECT * FROM CLX_DB_TABLE_HIST WHERE '.' TABLE_CODE='.singleQuote($table_info['DBTB_idcode']).' OR TABLE_CODE_HIST='.singleQuote($table_info['DBTB_idcode']));}	
		//pre($table_info); 
		$info_hist_exist=ClxCommon::$db->getRecord('CLX_DB_TABLE_HIST', ' TABLE_CODE='.singleQuote($table_info['DBTB_idcode']).' OR TABLE_CODE_HIST='.singleQuote($table_info['DBTB_idcode'])) ;
		if (isset($table_info['DBTB_idcode']) && isset($info_hist_exist['TABLE_CODE']) && $info_hist_exist['TABLE_CODE']==$table_info['DBTB_idcode']) {
			if (ClxCommon::$db->debugCF==true) {pre('condicion1'.__LINE__);pre(singlequote($info_hist_exist['TABLE_CODE']).'=='.singlequote($table_info['DBTB_idcode']));}
		}
		else if (isset($table_info['DBTB_idcode']) && isset($info_hist_exist['TABLE_CODE_HIST']) && $info_hist_exist['TABLE_CODE_HIST']==$table_info['DBTB_idcode']) {
			if (ClxCommon::$db->debugCF==true) {pre('condicion2'.__LINE__);pre(singlequote($info_hist_exist['TABLE_CODE_HIST']).'=='.singlequote($table_info['DBTB_idcode']));}
		}
		else {
			$newtablecode=ClxCommon::$db->getOne('SELECT MAX(TABLE_CODE_HIST) FROM CLX_DB_TABLE_HIST') ;
			if ($newtablecode!='') {
			
				$exist=1;
				while ($exist==1) {
					// BASE CONVERT DE DECIMAL A BASE 36 (LETRAS Y NUMEROS) Y AL REVES PRODUCE MINUSCULAS RELLENADO CON HASTA 3 0 IZQUIERDA
					$newtablecode=uppercase('H'.SUBSTR("00".base_convert(base_convert(uppercase(substr($newtablecode,1,3)),36,10)+1	,10,36),-3));
					// BUSCADO CODIGO QUE COMIENCE CON H QUE NO EXISTA SI COUNT ES 1 EXISTE (REPITE CICLO) SI ES 0 SE SALE
					$exist= ClxCommon::$db->getOne('SELECT COUNT(*) COUNT FROM CLX_DB_TABLES WHERE IDCODE='.singleQuote($newtablecode));
				}
			
			}
			else {
				$newtablecode='H000';
			}
			$d = xFormat::getNow('Y-m-d');
			$db_info_table=ClxCommon::$db->getRecord('CLX_DB_TABLES','IDCODE='.singleQuote(UserRequest::$post['DBTB_idcode']));
			$record = array();
			$record['TABLE_CODE'] = UserRequest::$post['DBTB_idcode'];
			$record['TABLE_CODE_HIST'] = $newtablecode;
			$record['CDATE'] = $d;
			$record['MDATE'] = $d;
			$record['USERNAME'] = $_SESSION['user_record']['username'];
			$record['DESCRIPTION'] = (!empty(UserRequest::$post['DBTB_description']))? UserRequest::$post['DBTB_description']:uppercase($table_info['DBTB_idname']);
			ClxCommon::$db->createrecord('CLX_DB_TABLE_HIST', $record, false);
			UserRequest::$request=UserRequest::$post=array (
			  'to' => 'DBTB',
			  'ac' => 'DUPLICATE',
			  'confirm' => '1',
			  'idlist' => '',
			  'DBTB_id' => $table_info['DBTB_id'],
			  //'DBTB_recordname_singular' => '38677CC74BED1C1E6542A687A55A2B5A',
			  //'DBTB_recordname_plural' => '07763313FA455E5C0AD6D78D7627F41F',
			  'DBTB_idcode' => $newtablecode,
			  'DBTB_idname' => uppercase($table_info['DBTB_idname'])."_HIST",
			  'DBTB_type' => $table_info['DBTB_type'],
			  'DBTB_status' => (!empty($table_info['DBTB_status'])) ? 1 : 0,
			  'DBTB_version' => '0.0.0',
			  'DBTB_description' => (!empty(UserRequest::$post['DBTB_description']))? UserRequest::$post['DBTB_description']:uppercase($table_info['DBTB_idname'])." Historic",
			  'DBTB_auditu' => (empty( $table_info['DBTB_auditu'])? (empty($table_info['DBTB_audit'])? '': $table_info['DBTB_audit']) : $table_info['DBTB_auditu']),
			  'DBTB_audit' => (empty( $table_info['DBTB_auditu'])? (empty($table_info['DBTB_audit'])? '': $table_info['DBTB_audit']) : $table_info['DBTB_auditu']),
			  'DBTB_truncable' => '0',
			  'DBTB_modulecode' => uppercase($table_info['DBTB_modulecode']),
			  'DBTB_view_query' => '',
			  'DBTB_pttn_enabled' => '0', 
			  'DBTB_pttn_options' => $table_info['DBTB_pttn_options'],
			  'DBTB_recordname_plural_DICT' =>(empty($table_info['DBTB_recordname_plural_DICT'])? (empty(dict($db_info_table['RECORDNAME_PLURAL']) )? '':dict($db_info_table['RECORDNAME_PLURAL'])):$table_info['DBTB_recordname_plural_DICT']). ' '.((getCurrentLanguage() =='us' ||getCurrentLanguage() =='en')? 'Historics' :'Historicos'),
			  'DBTB_recordname_singular_DICT' => (empty($table_info['DBTB_recordname_singular_DICT'])?(empty(dict($db_info_table['RECORDNAME_SINGULAR']) )? '':dict($db_info_table['RECORDNAME_SINGULAR'])):$table_info['DBTB_recordname_singular_DICT']).' '.((getCurrentLanguage() =='us' ||getCurrentLanguage() =='en' )? 'Historic' :'Historico'),
			  'submit1' => 'Guardar');
			
			$this->duplicateRecord();	
			$sql='update CLX_DB_TABLES set historic=1 where IDCODE='.singleQuote($table_info['DBTB_idcode']);
			ClxCommon::$db->executeQuery($sql) ;
			
			//create bakup $var internal
			$antpost=UserRequest::$post;
			$antget=UserRequest::$get;
			$antrequest=UserRequest::$request;
			//find id
			$id=ClxCommon::$db->getOne('SELECT ID FROM CLX_DB_TABLES WHERE IDCODE='.singleQuote($newtablecode));
			if (ClxCommon::$db->tableExists_bycode($newtablecode)) {
				if (ClxCommon::$db->getOne('SELECT COUNT(1) FROM CLX_DB_QUERY_TABLES WHERE TABLECODE='.singleQuote($newtablecode))){
					//create listados
					UserRequest::$request=UserRequest::$post=UserRequest::$post= array('ac' => 'CREATE_UI', 'to' => 'DBTB', 'idlist' => $id, 'next' => '1', 'relfm' => '',  'submit1' => 'Generar Query, Forma y Listado Default');
					$contents = $this->create_ui(array($id));
					// historic fix table
				}
			}
			UserRequest::$request=UserRequest::$post=UserRequest::$post= array ( 'ac' => 'FIX',  'to' => 'DBTB', 'idlist' => $id,  'next' => '1', 'relfm' => '', 'submit1' => 'Arreglar Tabla');
			$content=$this->action('FIX',array($id));

			//revert $var;
			UserRequest::$post=$antpost;
			UserRequest::$request=$antrequest;
			UserRequest::$get=$antget;
		
		}
		ClxCommon::$db->executequery('commit');
	
	}

    /**
     * Build description file used on interfases
     * @param: string Tablecode
     */
    function importStructure() {
		$xcontents='';
		ob_start();
        $line = str_repeat('_', CLX_NUM_HUNDRED);
        if (!class_exists('clx_msexcell'))
		  include_once kPathToWebSite . 'classes/clx_msexcel.php';
        $idlist = explode(',', UserRequest::$get['DBTB_id']);
        if (empty($idlist[CLX_NUM_ZERO]))
		  return alert(dict('GEN_LBL_ERROR'));
        $tableid = $idlist[CLX_NUM_ZERO];
        $data = array();
        //Table General Information
        $tb = ClxCommon::$db->getRecord('CLX_DB_TABLES', 'ID = ' . $tableid);
        $filename = lowercase($tb['IDCODE'] . '_' . str_replace('.', '', $tb['VERSION'] . '.' . $tb['VERSION_COUNTER']) . 'xls');
        $data[] = array($line, '', '', '', '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_TBL_STREPORT')), '', '', '', '', '');
        $data[] = array($line, '', '', '', '', '', '', '');
        $data[] = array('', '', '', '', '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_CODE')), $tb['IDCODE'], '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_NAME')), $tb['MODULECODE'] . '_' . $tb['IDNAME'], '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_VERSION')), $tb['VERSION'] . '.' . $tb['VERSION_COUNTER'], '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_MODULE')), $tb['MODULECODE'], '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_DESCRIPTION')), dict($tb['RECORDNAME_PLURAL']), '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_DATE')), date('Y-m-d'), '', '', '', '');
        $data[] = array('', '', '', '', '', '', '', '');
        $data[] = array(uppercase(dict('TOOL_DBTF_TITLE_FIELDS')), '', '', '', '', '');
        $data[] = array('', '', '', '', '', '', '', '');
        $data[] = array('###',
		  uppercase(dict('GEN_LBL_FIELDNAME')), uppercase(dict('GEN_LBL_TYPE')), uppercase(dict('GEN_LBL_REQUIRED')), uppercase(dict('GEN_LBL_DEFAULT')), uppercase(dict('GEN_LBL_DESCRIPTION')), uppercase(dict('GEN_LBL_LIST')), uppercase(dict('GEN_LBL_TABLE')),);
        $data[] = array('', '', '', '', '', '', '', '');
        //Fields Details
       switch (kDatabaseType) {
			case 'oci':
				$sql = 'SELECT DBTF.SEQNUM, DBTF.FIELDNAME, NVL((SELECT CONCAT(' . CLX_CHAR_SINGLE_QUOTE . 'LOOKUP,' . CLX_CHAR_SINGLE_QUOTE . ', CONCAT(X.FIELDTYPE, CONCAT(' . CLX_CHAR_SINGLE_QUOTE . '(' . CLX_CHAR_SINGLE_QUOTE . ', CONCAT(X.FIELD_SIZE, ' . CLX_CHAR_SINGLE_QUOTE . ')' . CLX_CHAR_SINGLE_QUOTE . ')))) FROM CLX_DB_FIELDS X WHERE X.FIELDNAME = DBFL.FIELD_EXTRA2 AND DBFL.FIELDTYPE=' . CLX_CHAR_SINGLE_QUOTE . 'LOOK' . CLX_CHAR_SINGLE_QUOTE . '), DBFL.FIELDTYPE) FIELDTYPE, DBFL.FIELD_SIZE, DBTF.FIELD_REQUIRED, DBTF.FIELD_DEFAULT, DBFL.DESCRIPTION, UPPER(DBFL.FIELDFORMLABEL) LABEL, DBFL.FIELD_EXTRA1, DBFL.FIELD_FORMAT FROM CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBFL.FIELDNAME = DBTF.FIELDNAME AND DBFL.FIELDTYPE NOT IN (' . CLX_CHAR_SINGLE_QUOTE . 'RECU' . CLX_CHAR_SINGLE_QUOTE . ',' . CLX_CHAR_SINGLE_QUOTE . 'PRIM' . CLX_CHAR_SINGLE_QUOTE . ') AND DBTF.TABLECODE =' . singleQuote($tb['IDCODE']) . ' ORDER BY DBTF.SEQNUM ASC';
				break;
			default:
				$sql = 'SELECT DBTF.SEQNUM, DBTF.FIELDNAME, IFNULL((SELECT CONCAT(' . CLX_CHAR_SINGLE_QUOTE . 'LOOKUP,' . CLX_CHAR_SINGLE_QUOTE . ', CONCAT(X.FIELDTYPE, CONCAT(' . CLX_CHAR_SINGLE_QUOTE . '(' . CLX_CHAR_SINGLE_QUOTE . ', CONCAT(X.FIELD_SIZE, ' . CLX_CHAR_SINGLE_QUOTE . ')' . CLX_CHAR_SINGLE_QUOTE . ')))) FROM CLX_DB_FIELDS X WHERE X.FIELDNAME = DBFL.FIELD_EXTRA2 AND DBFL.FIELDTYPE=' . CLX_CHAR_SINGLE_QUOTE . 'LOOK' . CLX_CHAR_SINGLE_QUOTE . '), DBFL.FIELDTYPE) FIELDTYPE, DBFL.FIELD_SIZE, DBTF.FIELD_REQUIRED, DBTF.FIELD_DEFAULT, DBFL.DESCRIPTION, UPPER(DBFL.FIELDFORMLABEL) LABEL, DBFL.FIELD_EXTRA1, DBFL.FIELD_FORMAT FROM CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL WHERE DBFL.FIELDNAME = DBTF.FIELDNAME AND DBFL.FIELDTYPE NOT IN (' . CLX_CHAR_SINGLE_QUOTE . 'RECU' . CLX_CHAR_SINGLE_QUOTE . ',' . CLX_CHAR_SINGLE_QUOTE . 'PRIM' . CLX_CHAR_SINGLE_QUOTE . ') AND DBTF.TABLECODE =' . singleQuote($tb['IDCODE']) . ' ORDER BY DBTF.SEQNUM ASC';
	    }
        $rs = ClxCommon::$db->executeQuery($sql);
        $i = CLX_INIT_ZERO;
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $r['SEQNUM'] = $i;
		  $i++;
		  $r['FIELD_REQUIRED'] = ($r['FIELD_REQUIRED'] == CLX_NUM_ONE) ? '*' : '';
		  $r['LABEL'] = capitalize(str_replace(CLX_CHAR_CR, '', dict($r['LABEL'])));
		  $r['EXTRACOL_LIST'] = '';
		  $r['EXTRACOL_TABLE'] = '';
		  switch ($r['FIELDTYPE']) {
			 case 'PRIM':
			     $r['FIELDTYPE'] = 'dec(11)';
			     break;
			 case 'BOOL':
			     $r['FIELDTYPE'] = 'BOOLEAN(1)';
			     $r['LABEL'] .= '. ' . dict('GEN_LBL_FORMAT') . ': 0 / 1';
			     break;
			 case 'TLOK': /* Access Tree lookup field */
			     $r['FIELDTYPE'] = 'ACCESS TREE, CHAR(255)';
			     $r['LABEL'] .= '. ' . dict('GEN_LBL_TREECODE') . ': ' . $r['FIELD_EXTRA1'];
			     break;
			 case 'TEXT':
			     break;
			 case 'DATI':
			 case 'DATE':
			     $r['FIELDTYPE'] = 'char(19)';
			     $r['LABEL'] .= '. ' . dict('GEN_LBL_FORMAT') . ': YYYY-MM-DD 24:MM:SS';
			     break;
			 case 'TIME':
			     $r['FIELDTYPE'] = 'char(8)';
			     $r['LABEL'] .= '. ' . dict('GEN_LBL_FORMAT') . ': 24:MM:SS';
			     break;
			 case 'DECN':
			     $r['FIELDTYPE'] = 'dec(' . $r['FIELD_FORMAT'] . ')';
			     break;
			 case 'INTN':
			     $r['FIELDTYPE'] = 'dec(' . $r['FIELD_SIZE'] . ')';
			     break;
			 case 'LIST':
			     $r['FIELDTYPE'] = 'LIST, CHAR(10)';
			     $r['LABEL'] .='. ' . dict('GEN_LBL_SEE_LIST') . ': ' . $r['FIELD_EXTRA1'];
			     $r['EXTRACOL_LIST'] = $r['FIELD_EXTRA1'];
			     break;
			 case 'SOBJ':
			     $r['FIELDTYPE'] = 'char(255)';
			     break;
			 case 'TLOK':
			     $r['FIELDTYPE'] = 'char(255)';
			     break;
			 case 'SLUK':
			     $r['FIELDTYPE'] = 'char(255)';
			     break;
			 default:
			     if (substr($r['FIELDTYPE'], CLX_NUM_ZERO, CLX_NUM_SIX) === 'LOOKUP') {
			         $r['EXTRACOL_TABLE'] = $r['FIELD_EXTRA1'];
			     } else {
			         $r['FIELDTYPE'] = 'char(' . $r['FIELD_SIZE'] . ')';
			     }
			     break;
		  }
		  $r['FIELDTYPE'] = uppercase($r['FIELDTYPE']);
		  unset($r['DESCRIPTION'], $r['FIELD_SIZE'], $r['FIELD_EXTRA1'], $r['FIELD_FORMAT']);
		  $data[] = $r;
        }
		ClxCommon::$db->freeResult($rs);
        //List Options
        $data[] = array('', '', '', '', '', '', '', '');
        $data[] = array($line, '', '', '', '', '', '', '');
        $data[] = array(uppercase(dict('GEN_LBL_DB_LISTS')), '', '', '', '', '');
        $data[] = array($line, '', '', '', '', '', '', '');
        $data[] = array('', '', '', '', '', '', '', '');
        $sql = 'SELECT DISTINCT DBLT.IDCODE, DBLT.IDNAME, DBLT.MODULECODE, DBLT.VERSION, DBLT.VERSION_COUNTER FROM CLX_DB_TABLEFIELDS DBTF, CLX_DB_FIELDS DBFL, CLX_DB_LISTS DBLT WHERE DBFL.FIELDNAME = DBTF.FIELDNAME AND DBLT.IDCODE = DBFL.FIELD_EXTRA1 AND DBFL.FIELDTYPE = ' . CLX_CHAR_SINGLE_QUOTE . 'LIST' . CLX_CHAR_SINGLE_QUOTE . ' AND DBTF.TABLECODE =' . singleQuote($tb['IDCODE']) . ' ORDER BY DBLT.IDNAME ASC';
        $rs = ClxCommon::$db->executeQuery($sql);
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $data[] = array(uppercase(dict('GEN_LBL_CODE')), $r['IDCODE'], '', '', '', '');
		  $data[] = array(uppercase(dict('GEN_LBL_NAME')), $r['IDNAME'], '', '', '', '');
		  $data[] = array(uppercase(dict('GEN_LBL_VERSION')), $r['VERSION'] . '.' . $r['VERSION_COUNTER'], '', '', '', '');
		  $data[] = array(uppercase(dict('GEN_LBL_MODULE')), $r['MODULECODE'], '', '', '', '');
		  $data[] = array('', '', '', '', '', '', '', '');
		  $data[] = array('###', uppercase(dict('GEN_LBL_CODE')), uppercase(dict('GEN_LBL_NAME')), '', '', '', '', '');
		  $data[] = array('', '', '', '', '', '', '', '');
		  $sql = 'SELECT SEQNUM, IDCODE, IDNAME,' . CLX_CHAR_SINGLE_QUOTE . '' . CLX_CHAR_SINGLE_QUOTE . ',' . CLX_CHAR_SINGLE_QUOTE . '' . CLX_CHAR_SINGLE_QUOTE . ',' . CLX_CHAR_SINGLE_QUOTE . '' . CLX_CHAR_SINGLE_QUOTE . ' FROM CLX_DB_LIST_ITEMS WHERE LISTCODE = ' . singleQuote($r['IDCODE']) . ' AND INACTIVE = 0';
		  $rs2 = ClxCommon::$db->executeQuery($sql);
		  $i = CLX_INIT_ZERO;
		  while ($r2 = ClxCommon::$db->getNextRecord($rs2, ASSOCIATIVE)) {
			 $r2['SEQNUM'] = $i;
			 $r2['IDNAME'] = trim(dict($r2['IDNAME']));
			 $data[] = $r2;
			 $i++;
		  }
		  ClxCommon::$db->freeResult($rs2);
		  $data[] = array($line, '', '', '', '', '', '', '');
		  $data[] = array('', '', '', '', '', '', '', '');
        }
		ClxCommon::$db->freeResult($rs);
		if (ob_get_contents())
		ob_clean();
        sendDownloadFile($filename, 'xls', null);
	   $contents='';
	   //pre($xcontents);
	   
        foreach ($data as $key => $val) {
		  $contents .=implode(CLX_CHAR_TAB, $val) . CLX_CHAR_NL;
        }
		//echo $contents;die;	
        return decode_string($contents);
    }
	function changeh() {
		$sqlDELhist='DELETE FROM CLX_DB_TABLE_HIST WHERE
				NOT EXISTS (SELECT 1 FROM CLX_DB_TABLES WHERE IDCODE=TABLE_CODE) OR
				NOT EXISTS (SELECT 1 FROM CLX_DB_TABLES WHERE IDCODE=TABLE_CODE_HIST)' ;
		ClxCommon::$db->executequery($sqlDELhist);
		//PRE(UserRequest::$request);
		$form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		$form->setProp(CLX_PROP_ID, 'DBTBList');
		$form->addArgument(CLX_PROP_TO, $this->code);
		$form->addArgument('ac', UserRequest::$request['ac']);
		IF(!ISSET(UserRequest::$request['submit1'])) {
			$t = time();
			$to = $this->code;
			$here = 'tool.php?to=' . $to . '&t=' . $t;
			clearHistory();
			pushHistory($this->code, 'LIST', array());
			
			$form->addArgument('IDCODE', UserRequest::$request['IDCODE']);
			$table = new htmlTable(CLX_NUM_FOUR);

			$table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
			$table->addCell(dict('GEN_LBL_HIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_NUM_FOUR));
			
			$sqlhist1='SELECT TABLE_CODE,TABLE_CODE_HIST FROM CLX_DB_TABLE_HIST WHERE TABLE_CODE='.singlequote(UserRequest::$request['IDCODE']) ;
			$recordhists1 = ClxCommon::$db->getOneRecord($sqlhist1);
			
			$sqlhist2='SELECT TABLE_CODE,TABLE_CODE_HIST FROM CLX_DB_TABLE_HIST WHERE TABLE_CODE_HIST='.singlequote(UserRequest::$request['IDCODE']) ;
			$recordhists2 = ClxCommon::$db->getOneRecord($sqlhist2);	
		//	ECHO 'A';	
		//	PRE($recordhists1);
		//	ECHO 'B';
		//	PRE($recordhists2);
			
		//echo "-----".$tablecode."------1";
			IF (empty($recordhists1) && empty($recordhists2)) {
				$sql='SELECT UPPER(IDNAME)IDNAME FROM CLX_DB_TABLES WHERE IDCODE='.singlequote(UserRequest::$request['IDCODE']);
				if (substr(ClxCommon::$db->getone($sql),-5)!='_HIST') {
					$recordhists1['TABLE_CODE']=UserRequest::$request['IDCODE'];
					$recordhists2['TABLE_CODE']=UserRequest::$request['IDCODE'];
					$recordhists1['TABLE_CODE_HIST']='';
					$recordhists2['TABLE_CODE_HIST']='';
				}	
				else {
					$recordhists1['TABLE_CODE']='';
					$recordhists2['TABLE_CODE']='';
					$recordhists1['TABLE_CODE_HIST']=UserRequest::$request['IDCODE'];
					$recordhists2['TABLE_CODE_HIST']=UserRequest::$request['IDCODE'];
				}
			}
			else {
				if (empty($recordhists1)) {
					$recordhists1['TABLE_CODE']=$recordhists2['TABLE_CODE'];
					$recordhists1['TABLE_CODE_HIST']=$recordhists2['TABLE_CODE_HIST'];
				}	
				IF (empty($recordhist2)) {
					$recordhists2['TABLE_CODE']=$recordhists1['TABLE_CODE'];
					$recordhists2['TABLE_CODE_HIST']=$recordhists1['TABLE_CODE_HIST'];
				} 
			}
		//	ECHO '--------------';
		//	PRE($recordhists2);
			$select1 = new htmlSelect('TABLE_CODE');
			$select2 = new htmlSelect('TABLE_CODE_HIST');
			$sql='SELECT UPPER(IDCODE) IDCODE,UPPER(MODULECODE) MODULECODE,UPPER(IDNAME) IDNAME FROM CLX_DB_TABLES ORDER BY MODULECODE,IDNAME';
			$req = ClxCommon::$db->executequery($sql);
			
			 
			while ($tablerec = ClxCommon::$db->getNextRecord($req)) {
				IF (SUBSTR($tablerec['IDNAME'],-5)!=='_HIST')
					$select1->addItem($tablerec['MODULECODE'].'_'.$tablerec['IDNAME'].'('.$tablerec['IDCODE'].')', $tablerec['IDCODE'], ($recordhists2['TABLE_CODE'] === $tablerec['IDCODE']));
				ELSE
					$select2->addItem($tablerec['MODULECODE'].'_'.$tablerec['IDNAME'].'('.$tablerec['IDCODE'].')', $tablerec['IDCODE'], ($recordhists2['TABLE_CODE_HIST'] === $tablerec['IDCODE']));
			}
			ClxCommon::$db->freeResult($req);
			$table->addData(dict('GEN_LBL_TABLENAME') . ':', $select1->source());
			$table->addData(dict('GEN_LBL_TABLENAME_HIST') . ':', $select2->source());
			$buttons=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'CHANGEH', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
			$c = new HTMLButtonCancel(dict('GEN_LBL_CANCEL'));
			$buttons .= $c->source();
        
			$table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
			
			$form->addContents($table->source());
			return $form->source();
		}
		ELSE {
			$REC=UserRequest::$request;//pre($REC);
			$b = new htmlFormSubmit(dict('GEN_LBL_RETURN'), 'LIST', '-1');
		  $table = new htmlTable(CLX_NUM_ONE);
			$table->addCell($b->SOURCE(), array(CLX_PROP_CLASS => 'buttons'));
			$form->addContents($table->source());
			
			if (ClxCommon::$db->getOne('SELECT COUNT(1) FROM CLX_DB_TABLE_HIST WHERE TABLE_CODE ='.singlequote($REC['TABLE_CODE']).' AND TABLE_CODE_HIST ='.singlequote($REC['TABLE_CODE_HIST']))>0) {
				return note(dict('GEN_LBL_RECORD_EXISTS')).$form->source();
			}
			else if (ClxCommon::$db->getOne('SELECT COUNT(1) FROM CLX_DB_TABLE_HIST WHERE TABLE_CODE ='.singlequote($REC['TABLE_CODE']).' OR TABLE_CODE_HIST ='.singlequote($REC['TABLE_CODE_HIST']))>0) {
				return note(dict('GEN_LBL_RECORD_COMBINATION_EXISTS')).$form->source();
			}	
			else {//RETURN 'ELSE';
				$d = xFormat::getNow('Y-m-d');
				$record = array();
				$record['TABLE_CODE'] = $REC['TABLE_CODE'];
				$record['TABLE_CODE_HIST'] = $REC['TABLE_CODE_HIST'];
				$record['CDATE'] = $d;
				$record['MDATE'] = $d;
				$record['USERNAME'] = $_SESSION['user_record']['username'];
				$record['DESCRIPTION'] = 'ADD or Change '.ClxCommon::$db->getOne('SELECT IDNAME FROM CLX_DB_TABLES WHERE IDCODE ='.singlequote($REC['TABLE_CODE'])) ;
				ClxCommon::$db->createrecord('CLX_DB_TABLE_HIST', $record, false);
				
		
       		
				RETURN NOTE(DICT('GEN_LBL_SAVE_GOOD')).$form->source();
			}	
		}
	}
}
