<?php

/**
 * e-Mails and Alerts Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: mchirino $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-28 12:59:35 +0000 (Mon, 28 Jan 2019) $
 * @version      $Rev: 10505 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class ma_mail extends clx_tool {

    var $debug = false;
    var $status; //Message Status List
    var $priority; //Message Priority1450
    var $folders; //List of mail folders
    var $def_folder; //Default mail folder on open
    var $outtype = 'smtp'; //Outgoing mail type: smtp(Smtp Server),
    //sendmail(Sendmail), mail(Php Mail).
    var $store_attondb = false; //Set to true if attachment are store on db,
    //on file fail it will be stored on db
    var $cont_tablecode = 'CONT'; //Table code for address book search

    /**
     * Constructor
     * @param void
     * @return void
     */

    function __construct() {
        $this->code = 'MAIL';
        $this->name = 'Clientix Webmail';
        $this->system = 'Mail';
        $this->version = '1.0.0.1';

        $this->has_access = true;

        $this->has_pagelet = true;
        $this->tablename = 'CLX_MA_MAILS';
        $this->record_name_singular = 'MAIL_MA_MAIL';
        $this->record_name_plural = 'MAIL_MA_MAILS';
        $this->primary_field = 'FROM_EMAIL';
        $this->secondary_field = 'SUBJECT';

        $this->status = array('OLD' => 'GEN_LBL_OLD', 'NEW' => 'GEN_LBL_NEW', 'QUEUE' => 'GEN_LBL_QUEUE', 'PRO' => 'GEN_LBL_PROCESING', 'ERR' => 'GEN_LBL_ERROR');
        $this->priority = array('GEN_LBL_NOTSET', 'GEN_LBL_HIGH', 'GEN_LBL_HIGHEST', 'GEN_LBL_NORMAL', 'GEN_LBL_LOW', 'GEN_LBL_LOWEST');
        $this->folders = array('INBX' => 'GEN_LBL_INBOX', 'DRFT' => 'GEN_LBL_DRAFTS', 'OUBX' => 'GEN_LBL_OUTBOX', 'SENT' => 'GEN_LBL_SENTITEMS');
        $this->def_folder = 'INBX';
    }

    /**
     * Install Method
     * @param void
     * @return void
     */
    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_PERSONAL_INFORMATION:MENU_MAIL', 'TOOL', $this->code, 'ac=LIST');
    }

    /**
     * Tool table structure
     * @param void
     * @return array $table_structure
     */
    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['MESSAGEID'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['SUBJECT'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['FROM_EMAIL'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['TO_EMAIL'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['CC_EMAIL'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['BCC_EMAIL'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['REPLY_EMAIL'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['SEND_DATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['RECEIVED_DATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['EMAIL_STATUS'] = array('type' => 'varchar(32)', 'null' => 'YES', 'properties' => '');
        $fields['EMAIL_PRIORITY'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['EMAIL_HEADERS'] = array('type' => 'blob', 'null' => 'YES', 'properties' => '');
        $fields['EMAIL_CONTENTS'] = array('type' => 'blob', 'null' => 'YES', 'properties' => '');
        $fields['ERROR_MSG'] = array('type' => 'blob', 'null' => 'YES', 'properties' => '');
        $fields['MIME_TYPE_PRIMARY'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['MIME_TYPE_SECONDARY'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['MIME_ENCODING'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['FOLDER'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $fields['EMAIL_DIRECTION'] = array('type' => 'varchar(4)', 'null' => 'YES', 'properties' => '');
        $fields['EMAIL_ACCOUNT'] = array('type' => 'varchar(100)', 'null' => 'YES', 'properties' => '');
        $fields['IS_SIGNATURE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'YES', 'properties' => '', 'default' => '0');
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['EMAIL_CONT_TXT'] = array('type' => 'text', 'null' => 'YES', 'properties' => '');
        $fields['TEXT_ONLY'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['REQ_RECEIPT'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'MESSAGEID');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'USERNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'EMAIL_ACCOUNT');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'SUBJECT');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB', 'fields' => $fields, 'indexes' => $indexes);
    }

    /**
     * Action handler
     *
     * @param string $ac
     * @param array $idlist
     * @return string $contents
     */
    function action($ac, $idlist) {
        $contents = '';
        $msg = array();
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'FORWARD':
			     $idlist[CLX_NUM_ZERO] = $this->mailAction($idlist[CLX_NUM_ZERO], $ac);
			     $contents .= parent::action('MODIFY', $idlist);
			     break;
			 case 'REPLYALL':
			     $idlist[CLX_NUM_ZERO] = $this->mailAction($idlist[CLX_NUM_ZERO], $ac);
			     $contents .= parent::action('MODIFY', $idlist);
			     break;
			 case 'REPLY':
			     $idlist[CLX_NUM_ZERO] = $this->mailAction($idlist[CLX_NUM_ZERO], $ac);
			     $contents .= parent::action('MODIFY', $idlist);
			     break;
			 case 'CHECKMAIL':
			     $contents = $this->getMail();
			     $contents .= parent::action('LIST', $idlist);
			     break;
			 case 'SEND':
			     $fail = false;
			     if (array_key_exists('MAIL_id', UserRequest::$post) && !empty(UserRequest::$post['MAIL_id'])) {
			         parent::action('MODIFY', $idlist);
			         $idlist[CLX_NUM_ZERO] = UserRequest::$post['MAIL_id'];
			     } elseif (array_key_exists('MAIL_id', UserRequest::$post)) {
			         if ($this->getAccess()->canDisplay()) {
					   if (isset(UserRequest::$post['confirm'])) {
						  if ($this->checkForm($msg)) {
						      $idlist[CLX_NUM_ZERO] = $this->createRecord_();
						  } else {
						      $contents = $this->editForm($ac, array(), UserRequest::$post, $msg);
						      $fail = true;
						  }
					   } else {
						  $record = $this->defaultRecord();
						  $contents = $this->editForm($ac, array(), $record, $msg);
						  $fail = true;
					   }
			         } else {
					   //$contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_MAIL-CAN_APPEND_1][' . $ac . '] ' . $this->getCode(false));
					   $fail = true;
			         }
			     }
			     if (empty($fail))
			         $contents = $this->sendMails($idlist);
			     $contents .= parent::action('LIST', $idlist);
			     break;
			 case 'UNREAD':
			     $this->markUnread();
			     $contents .= parent::action('LIST', $idlist);
			     break;
			 case 'ATTACHFILE':
			     if (empty(UserRequest::$post['MAIL_id'])) {
			         if ($this->getAccess()->canDisplay()) {
					   if (isset(UserRequest::$post['confirm'])) {
						  if ($this->checkForm($msg)) {
						      UserRequest::$post['MAIL_id'] = $this->createRecord_();
						      $this->attachFile(UserRequest::$post['MAIL_id']);
						      //Return to Edit Form
						      $contents = $this->editForm('MODIFY', array(), UserRequest::$post, $msg);
						  } else {
						      $contents = $this->editForm('CREATE', array(), UserRequest::$post, $msg);
						  }
					   } else {
						  $record = $this->defaultRecord();
						  $contents = $this->editForm('CREATE', array(), $record, $msg);
					   }
			         } else {
					   $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_MAIL-CAN_APPEND_2][' . $ac . '] ' . $this->getCode(false));
			         }
			     } else {//if not empty idlist
			         $this->attachFile(UserRequest::$post['MAIL_id']);
			         //Return to Edit Form
			         $contents = $this->editForm('MODIFY', array(), UserRequest::$post, $msg);
			     }
			     break;
			 case 'DELATTACH':
			     if (count(UserRequest::$post['attachments']) > CLX_NUM_ZERO) {
			         foreach (UserRequest::$post['attachments'] as $key => $val) {
					   $this->deleteAttach($val);
			         }
			     }
			     $contents = $this->editForm('MODIFY', array(), UserRequest::$post, $msg);
			     break;
			 case 'SEARCHADDRESS':
			     $contents = $this->searchLibAddress();
			     break;
			 case 'DELETEDISPLAY':
			     if ($this->getAccess()->canDelete()) {
			         if (isset(UserRequest::$post['confirm'])) {
					   foreach ($idlist as $id) {
						  $this->deleteRecordSk($id);
					   }
					   //$info = popHistory();
					   $contents = parent::action('LIST', array());
					   //executeHistory($info);
			         } else {
					   $contents = $this->confirmDelete($ac, $idlist);
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_MAIL-CAN_DELETE][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
			     break;
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MA_MAIL-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    /**
     * Build tool listing
     * @param void
     * @return string $contents
     */
    function makelist($listingcode = null) {
        ClxFreeReturn($listingcode);
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        if (stristr(UserRequest::$server['PHP_SELF'], 'index.php') === false) {
		  pushHistory($this->code, 'LIST', array());
        }
        if (!array_key_exists('MAIL', $_SESSION)) {
		  $_SESSION['MAIL'] = array();
        }
        //Setting Change of folder
        if (array_key_exists('a', UserRequest::$get)) {
		  $_SESSION['MAIL']['account'] = UserRequest::$get['a'];
		  UserRequest::$post['MAIL_folder'] = $_SESSION['MAIL']['folder'];
		  UserRequest::$post['MAIL_email_account'] = UserRequest::$get['a'];
		  $this->setfilter();
        }
        //Setting Change of folder
        if (array_key_exists('f', UserRequest::$get) && (array_key_exists(UserRequest::$get['f'], $this->folders) || UserRequest::$get['f'] === '')) {
		  $_SESSION['MAIL']['folder'] = UserRequest::$get['f'];
		  UserRequest::$post['MAIL_folder'] = UserRequest::$get['f'];
		  UserRequest::$post['MAIL_from_email'] = $_SESSION['MAIL']['account'];
		  $this->setfilter();
        }
        //Set Accountr defaults view
        if (!array_key_exists('account', $_SESSION['MAIL'])) {
		  $_SESSION['MAIL']['account'] = '';
        }
        //Set Folder defaults view
        if (!array_key_exists('folder', $_SESSION['MAIL'])) {
		  $_SESSION['MAIL']['folder'] = $this->def_folder;
		  UserRequest::$post['MAIL_folder'] = $this->def_folder;
		  $this->setfilter();
        }
        $accounts = $this->getAccounts($_SESSION['user_record']['username']);
        $accountin = array('NUNCAUNEMAIL');
        foreach ($accounts as $key => $val) {
		  $accountin[] = $val['IDNAME'];
        }
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'MAILList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        //$whereclause = ' EMAIL_ACCOUNT IN (' . singleQuote(implode("''", $accountin)) . ') ';
        $whereclause = ' EMAIL_ACCOUNT IN (' . singleQuote(implode("','", $accountin)) . ') ';
        $sortclause = 'ID DESC';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $show_err = ($_SESSION['MAIL']['folder'] === 'OUBX') ? true : false;
        if (!empty($show_err)) {
		  $tableWidth = CLX_WIDTH_NINE;
        } else {
		  $tableWidth = CLX_WIDTH_EIGHT;
        }
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAIL_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => $tableWidth));
        $page->header($table);
        $c = new htmlSelect('MAIL_email_account');
        //$c->addItem(dict('GEN_LBL_ALLACCOUNTS'), '', false);
        $c->addItem(dict('GEN_LBL_ALLACCOUNTS'), '', empty($_SESSION['MAIL']['account']));
        foreach ($accounts as $key => $val) {
		  //$c->addItem($val['IDNAME'] . '(' . $val['EMAIL'] . ')', $val['IDNAME'], ($_SESSION['MAIL']['account'] == $val['IDNAME'] || empty($_SESSION['MAIL']['account'])));
		  $c->addItem($val['IDNAME'] . '(' . $val['EMAIL'] . ')', $val['IDNAME'], ($_SESSION['MAIL']['account'] == $val['IDNAME']));
		  $c->setProp('onchange', 'window.location=' . singleQuote('tool.php?to=MAIL&ac=LIST&a=') . '+this.value');
        }
        //Adding Folder List
        $c1 = new htmlSelect('MAIL_folder');
        //$c1->addItem(dict('GEN_LBL_ALLFOLDERS'), '', false);
        foreach ($this->folders as $key => $val) {
		  $c1->addItem(dict($val), $key, ($_SESSION['MAIL']['folder'] == $key));
		  $c1->setProp('onchange', 'window.location=' . singleQuote('tool.php?to=MAIL&ac=LIST&f=') . '+this.value');
        }
        $a = new htmlAnchor(dict('GEN_LBL_MAILACCCONF'), 'tool.php?to=MAAC&ac=LIST');
        $table->addCell($a->source() . ' | ' . dict('GEN_LBL_MAILACCOUNTS') . $c->source() . dict('GEN_LBL_MAILFOLDER') . $c1->source(), array(CLX_PROP_CLASS => '', CLX_PROP_ALIGN => CLX_PROP_RIGHT, CLX_PROP_COLSPAN => $tableWidth));
        $in = ($_SESSION['MAIL']['folder'] === 'INBX'); //Check if in folder
        if (!empty($show_err)) {
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MAILACCOUNT'), dict('GEN_LBL_PRIORITY'), ($in) ? dict('GEN_LBL_FROM') : dict('GEN_LBL_TO'), dict('GEN_LBL_SUBJECT'), dict('GEN_LBL_DATE'), dict('GEN_LBL_ATTACHMENTS'), dict('GEN_LBL_MAILFOLDER'), dict('GEN_LBL_ERROR_MSG')));
        } else {
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MAILACCOUNT'), dict('GEN_LBL_PRIORITY'), ($in) ? dict('GEN_LBL_FROM') : dict('GEN_LBL_TO'), dict('GEN_LBL_SUBJECT'), dict('GEN_LBL_DATE'), dict('GEN_LBL_ATTACHMENTS'), dict('GEN_LBL_MAILFOLDER')));
        }
        $i = CLX_INIT_ZERO;
        $MAAT = getTool('MAAT');
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $pfix = '';
		  $sfix = '';
		  if ($record['EMAIL_STATUS'] === 'NEW') {
			 $pfix = '<b>';
			 $sfix = '</b>';
		  }
		  if (!empty($show_err)) {
			 if ($record['EMAIL_STATUS'] === 'ERR')
			     $record['ERROR_MSG'] = substr(dict('GEN_LBL_ERROR') . ': ' . $record['ERROR_MSG'], CLX_NUM_ZERO, strpos($record['ERROR_MSG'], '[') + CLX_NUM_SIX);
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($pfix . $record['EMAIL_ACCOUNT'] . $sfix, $pfix . dict($this->priority[$record['EMAIL_PRIORITY']]) . $sfix, $pfix . (($in) ? stripslashes($record['FROM_EMAIL']) :
					   stripslashes($record['TO_EMAIL'])) . $sfix, $pfix . stripslashes($record['SUBJECT']) . $sfix, $pfix . $record['RECEIVED_DATE'] . $sfix, $pfix . ClxCommon::$db->countRecords($MAAT->tablename, 'MAILID =' . singleQuote($record['ID'])) . $sfix, $pfix . dict($this->folders[$record['FOLDER']]) . $sfix, $record['ERROR_MSG']));
		  }else {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($pfix . $record['EMAIL_ACCOUNT'] . $sfix, $pfix . dict($this->priority[$record['EMAIL_PRIORITY']]) . $sfix, $pfix . (($in) ? stripslashes($record['FROM_EMAIL']) :
					   stripslashes($record['TO_EMAIL'])) . $sfix, $pfix . stripslashes($record['SUBJECT']) . $sfix, $pfix . $record['RECEIVED_DATE'] . $sfix, $pfix . ClxCommon::$db->countRecords($MAAT->tablename, 'MAILID =' . singleQuote($record['ID'])) . $sfix, $pfix . dict($this->folders[$record['FOLDER']]) . $sfix));
		  }
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_MAIL_CREATEMSG')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $b = new htmlFormSubmit(dict('GEN_LBL_CHECKMAIL'), 'CHECKMAIL', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'ON');
        $buttons .= $b->source();
        $buttons .='<br>';
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REPLY'), 'REPLY', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ONE');
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REPLYALL'), 'REPLYALL', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ONE');
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_FORWARD'), 'FORWARD', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ONE');
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canModify()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_MARKUNREAD'), 'UNREAD', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'MULT');
		  $buttons .= $b->source();
        }
        $b = new htmlFormSubmit(dict('GEN_LBL_SEND'), 'SEND', $form->getProp(CLX_PROP_ID));
        $b->setProp(CLX_PROP_ID, 'MULT');
        $buttons .= $b->source();
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    /**
     * Tool search form
     * @param void
     * @return string $contents
     */
    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAIL_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_SUBJECT'), 'MAIL_subject', UserRequest::$post['MAIL_subject'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_FROM'), 'MAIL_from_email', UserRequest::$post['MAIL_from_email'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MESSAGE'), 'MAIL_email_contents', UserRequest::$post['MAIL_email_contents'], CLX_NUM_32, CLX_NUM_64);
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['MAIL_subject'] <> '') {
			 $criteria .= 'SUBJECT LIKE ' . singleQuote(UserRequest::$post['MAIL_subject'] . '%');
		  }
		  if (UserRequest::$post['MAIL_from_email'] <> '') {
			 $criteria .= 'FROM_EMAIL LIKE ' . singleQuote(UserRequest::$post['MAIL_from_email'] . '%');
		  }
		  if (UserRequest::$post['MAIL_email_contents'] <> '') {
			 $criteria .= 'EMAIL_CONTENTS LIKE ' . singleQuote(UserRequest::$post['MAIL_email_contents'] . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['SUBJECT']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ma_mail></page><a href=' . singleQuote($js) . '>' .
			         $record['SUBJECT'] . '</a>', $record['FROM_EMAIL']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    /**
     * Tool form validations
     *
     * @param array $msg
     * @return bool $status
     */
    function checkForm(&$msg) {
        $msg = array();
        xValidate::Required($msg, 'MAIL_from_email', UserRequest::$post['MAIL_from_email']);
        xValidate::Required($msg, 'MAIL_subject', UserRequest::$post['MAIL_subject']);
        //xValidate::Mail($msg, 'MAIL_to_email', UserRequest::$post['MAIL_to_email']);
        if (!empty(UserRequest::$post['MAIL_cc_email']))
		  xValidate::Mail($msg, 'MAIL_cc_email', UserRequest::$post['MAIL_cc_email']);
        if (!empty(UserRequest::$post['MAIL_bcc_email']))
		  xValidate::Mail($msg, 'MAIL_bcc_email', UserRequest::$post['MAIL_bcc_email']);
        if (!empty(UserRequest::$post['MAIL_reply_email']))
		  xValidate::Mail($msg, 'MAIL_reply_email', UserRequest::$post['MAIL_reply_email']);
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_FROM') . ':', $record['MAIL_from_email']);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $record['MAIL_subject']);
    }

    /**
     * Tool Display Form
     *
     * @param array $record
     * @return string $contents
     */
    function viewFormFill($record) {
        $idlist = getIdlist();
        $contents = '';
        //Change Email Status to old
        ClxCommon::$db->updateRecord($this->tablename, array('EMAIL_STATUS' => 'OLD'), 'ID = ' . singleQuote($record['MAIL_id']));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument('ac', 'NOT_DEFINED');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('idlist', $idlist[CLX_NUM_ZERO]);
        $buttons = '';
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REPLY'), 'REPLY', $form->getProp(CLX_PROP_ID));
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_REPLYALL'), 'REPLYALL', $form->getProp(CLX_PROP_ID));
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('GEN_LBL_FORWARD'), 'FORWARD', $form->getProp(CLX_PROP_ID));
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonGoto(dict('GEN_LBL_RETURN'), 'tool.php?ac=LIST&to=MAIL');
        $buttons .= $c->source();
        $table->addCell(dict('TOOL_MAIL_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO - CLX_NUM_ONE));
        $table->addData(dict('GEN_LBL_MAILACCOUNT') . ':', $record['MAIL_email_account']);
        $table->addData(dict('GEN_LBL_TO') . ':', $record['MAIL_to_email']);
        $table->addData(dict('GEN_LBL_FROM') . ':', $record['MAIL_from_email']);
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', dict($this->priority[$record['MAIL_email_priority']]));
        $table->addData(dict('GEN_LBL_CC') . ':', $record['MAIL_cc_email']);
        $table->addData(dict('GEN_LBL_BCC') . ':', $record['MAIL_bcc_email']);
        $table->addData(dict('GEN_LBL_REPLYTO') . ':', $record['MAIL_reply_email']);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $record['MAIL_subject']);
        $record['MAIL_email_contents'] = $record['MAIL_email_contents'];
        $record['MAIL_email_contents'] = str_replace(CLX_CHAR_CR . CLX_CHAR_NL, '<br>', $record['MAIL_email_contents']);
        $record['MAIL_email_contents'] = str_replace(CLX_CHAR_NL, '<br>', $record['MAIL_email_contents']);
        $record['MAIL_email_contents'] = str_replace(CLX_CHAR_CR, '<br>', $record['MAIL_email_contents']);
        $table->addData(dict('GEN_LBL_MESSAGE') . ':', stripslashes($record['MAIL_email_contents']));
        $form->setContents($table->source());
        $contents .= $form->source();
        //Attachments
        $MAAT = &getTool('MAAT');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (ClxCommon::$db->oneExistsRecords($MAAT->tablename, 'MAILID =' . singleQuote($record['MAIL_id'])) > CLX_NUM_ZERO) {
		  $results = ClxCommon::$db->getRecords($MAAT->tablename, 'MAILID = ' . singleQuote($record['MAIL_id']));
		  $i = CLX_INIT_ZERO;
		  while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
			 $link = new htmlAnchor((!empty($result['name'])) ? $result['name'] :
                                $results['MAILID'] . $results['ID'], 'javascript:openWindow("fi' . singleQuote($result['ID']) . '", "window.php?to=MAAT&ac=SHOW_FILE&ff=' . $result['ID'] . '", 400, 400,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');');
			 $table->addCell($link->source(), array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $i++;
		  }
		 ClxCommon::$db->freeResult($results);
        } else {
		  $table->addCell(dict('GEN_LBL_NOATTACHMENTS'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $contents .= buildPanelInterface(dict('GEN_LBL_ATTACHMENTS'), 'MAILATTACHMENTS', $table->source(), CLX_NUM_MINUS_1) . '<br>';
        return $contents;
    }

    /**
     * Tool Edit Form
     *
     * @param array $record
     * @return string $contents
     */
    function editForm($ac, $idlist, $record, $msg) {
        ClxFreeReturn($ac);
        $contents = '';
        $form = new htmlForm('name', 'theForm', CLX_PROP_ACTION, 'tool.php', 'enctype', 'multipart/form-data');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', (!empty($record['MAIL_id'])) ? 'MODIFY' : 'CREATE');
        $form->addArgument('confirm', '1');
        $form->addArgument('MAIL_id', (!empty($record['MAIL_id'])) ? $record['MAIL_id'] : '');
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('MAIL_folder', (empty($record['MAIL_folder'])) ? 'DRFT' : $record['MAIL_folder']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAIL_TITLE_CREATE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $accounts = $this->getAccounts($_SESSION['user_record']['username']);
        $c = new htmlSelect('MAIL_from_email');

        foreach ($accounts as $key => $val) {
		  $c->addItem($val['IDNAME'] . '(' . $val['EMAIL'] . ')', $val['IDNAME'], ($record['MAIL_from_email'] == $val['IDNAME']));
        }
        if (!empty($msg['MAIL_from_email']))
		  $table->addCell($msg['MAIL_from_email'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_FROM') . ':', $c->source());
        $link = getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=' . $this->code . '&ac=SEARCHADDRESS&fl=to_email') . ', 700, 500,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');', dict('GEN_LBL_SEARCH'));
        if (!empty($msg['MAIL_to_email']))
		  $table->addCell($msg['MAIL_to_email'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $f = new htmlField('MAIL_to_email', $record['MAIL_to_email'], CLX_NUM_60, '');
        $table->addData(dict('GEN_LBL_TO') . ':', $f->source() . $link);
        $link = getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=' . $this->code . '&ac=SEARCHADDRESS&fl=cc_email') . ',700,500,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');', dict('GEN_LBL_SEARCH'));
        if (!empty($msg['MAIL_cc_email']))
		  $table->addCell($msg['MAIL_cc_email'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $f = new htmlField('MAIL_cc_email', $record['MAIL_cc_email'], CLX_NUM_40, '');
        $table->addData(dict('GEN_LBL_CC') . ':', $f->source() . $link);
        $link = getSkinIcon('view', 'javascript:openWindow(' . singleQuote(CLX_CHAR_EMPTY_STRING) . ', ' . singleQuote('pop.php?to=' . $this->code . '&ac=SEARCHADDRESS&fl=bcc_email') . ',700,500,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');', dict('GEN_LBL_SEARCH'));
        if (!empty($msg['MAIL_bcc_email']))
		  $table->addCell($msg['MAIL_bcc_email'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $f = new htmlField('MAIL_bcc_email', $record['MAIL_bcc_email'], CLX_NUM_40, '');
        $table->addData(dict('GEN_LBL_BCC') . ':', $f->source() . $link);
        if (!empty($msg['MAIL_reply_email']))
		  $table->addCell($msg['MAIL_reply_email'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $f = new htmlField('MAIL_reply_email', $record['MAIL_reply_email'], CLX_NUM_40, CLX_NUM_255);
        $table->addData(dict('GEN_LBL_REPLYTO') . ':', $f->source());
        if (!empty($msg['MAIL_subject']))
		  $table->addCell($msg['MAIL_subject'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $f = new htmlField('MAIL_subject', $record['MAIL_subject'], CLX_NUM_60, CLX_NUM_255);
        $table->addData(dict('GEN_LBL_SUBJECT') . ':', $f->source());
        $c = new htmlSelect('MAIL_email_priority');
        $c->addItem('', '', false);
        foreach ($this->priority as $key => $val) {
		  $c->addItem(dict($val), $key, ($record['MAIL_email_priority'] == $key));
        }
        $table->addData(dict('GEN_LBL_PRIORITY') . ':', $c->source());
        $table->addText($msg, dict('GEN_LBL_MESSAGE') . ':', 'MAIL_email_contents', stripslashes($record['MAIL_email_contents']), CLX_NUM_20, CLX_NUM_60);
        $contents .= $table->source();
        //Attachments
        $MAAT = &getTool('MAAT');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $file = new htmlObject('input', '', array('type' => 'file', 'name' => 'MAIL_attachments', 'value' => ''));
        $c = new htmlFormSubmit(dict('GEN_LBL_ATTACH'), 'ATTACHFILE', $form->getProp(CLX_PROP_ID));
        $table->addData(dict('GEN_LBL_ATTACHFILE') . ':', $file->source() . $c->source());
        if (ClxCommon::$db->oneExistsRecords($MAAT->tablename, 'MAILID =' . singleQuote($record['MAIL_id'])) > CLX_NUM_ZERO) {
		  $results = ClxCommon::$db->getRecords($MAAT->tablename, 'MAILID = ' . singleQuote($record['MAIL_id']));
		  $i = CLX_INIT_ZERO;
		  while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
			 $s = new htmlObject('input', '', 'type', 'checkbox', 'name', 'attachments[]', 'value', $result['ID']);
                $link = new htmlAnchor(((!empty($result['name'])) ? $result['name'] : $results['MAILID'] . $results['ID']), 'javascript: openWindow(' . singleQuote('fi' . $result['ID']) . ', ' . singleQuote('window.php?to=MAAT&ac=SHOW_FILE&ff=' . $result['ID']) . ', 400, 400,null,null,'.singlequote( str_replace('\\','/',__FILE__) .'_'. __LINE__ ) .');');
			 $table->addCell($s->source() . $link->source(), array(CLX_PROP_CLASS => ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
			 $c = htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID)));
			 $table->addCell($c->source(), array(CLX_PROP_CLASS, ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT));
			 $i++;
		  }
		 ClxCommon::$db->freeResult($results);
        } else {
		  $table->addCell(dict('GEN_LBL_NOATTACHMENTS'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        $contents .= buildPanelInterface(dict('GEN_LBL_ATTACHMENTS'), 'MAILATTACHMENTS', $table->source(), CLX_NUM_MINUS_1);
        $form->addContents($contents . '<br>');
        //-- Footers
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => '90%'));
        if (UserRequest::$request['ac'] === 'CREATE') {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
        $c = new htmlFormSubmit(dict('GEN_LBL_SEND'), 'SEND', $form->getProp(CLX_PROP_ID));
        $buttons = $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS));
        $form->addContents($table->source());
        return $form->source();
    }

    /**
     * Tool Filter Form
     * @param array $msg
     * @param object $table
     * @return string $contents
     */
    function filterFormFill($msg, &$table) {
        if (!array_key_exists('MAIL_subject', UserRequest::$post))
		  UserRequest::$post['MAIL_subject'] = '';
        if (!array_key_exists('MAIL_from_email', UserRequest::$post))
		  UserRequest::$post['MAIL_from_email'] = '';
        if (!array_key_exists('MAIL_modulecode', UserRequest::$post))
		  UserRequest::$post['MAIL_modulecode'] = '';
        if (!array_key_exists('MAIL_folder', UserRequest::$post))
		  UserRequest::$post['MAIL_folder'] = '';
        if (!array_key_exists('MAIL_email_account', UserRequest::$post))
		  UserRequest::$post['MAIL_email_account'] = '';
        if (!array_key_exists('MAIL_email_contents', UserRequest::$post))
		  UserRequest::$post['MAIL_email_contents'] = '';
        $table->addField($msg, dict('GEN_LBL_SUBJECT'), 'MAIL_subject', UserRequest::$post['MAIL_subject'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_FROM'), 'MAIL_from_email', UserRequest::$post['MAIL_from_email'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MESSAGE'), 'MAIL_email_contents', UserRequest::$post['MAIL_email_contents'], CLX_NUM_32, CLX_NUM_64);
        $accounts = $this->getAccounts($_SESSION['user_record']['username']);
        $c = new htmlSelect('MAIL_email_account');
        $c->addItem(dict('GEN_LBL_ALLACCOUNTS'), '', false);
        foreach ($accounts as $key => $val) {
		  $c->addItem($val['IDNAME'] . '(' . $val['EMAIL'] . ')', $val['IDNAME'], ($_SESSION['MAIL']['account'] == $val['IDNAME']));
        }
        $table->addData(dict('GEN_LBL_MAILACCOUNTS') . ':', $c->source());
        $c = new htmlSelect('MAIL_folder');
        $c->addItem(dict('GEN_LBL_ALLFOLDERS'), '', false);
        foreach ($this->folders as $key => $val) {
		  $c->addItem($val, $key, false);
        }
        $table->addData(dict('GEN_LBL_MAILFOLDER') . ':', $c->source());
    }

    /**
     * Tool apply filter
     *
     * @param void
     * @return void
     */
    function setfilter() {
        $filter = '';
        $boolean = '';
        if (array_key_exists('MAIL_subject', UserRequest::$post) && !empty(UserRequest::$post['MAIL_subject'])) {
		  $filter .= $boolean . ' SUBJECT LIKE ' . singleQuote(UserRequest::$post['MAIL_subject'] . '%');
		  $boolean = 'AND';
        }
        if (array_key_exists('MAIL_from_email', UserRequest::$post) && !empty(UserRequest::$post['MAIL_from_email'])) {
		  $filter .= $boolean . ' FROM_EMAIL LIKE ' . singleQuote(UserRequest::$post['MAIL_from_email'] . '%');
		  $boolean = 'AND';
        }
        if (array_key_exists('MAIL_email_contents', UserRequest::$post) && !empty(UserRequest::$post['MAIL_email_contents'])) {
		  $filter .= $boolean . ' EMAIL_CONTENTS LIKE ' . singleQuote(UserRequest::$post['MAIL_email_contents'] . '%');
		  $boolean = 'AND';
        }
        if (array_key_exists('MAIL_folder', UserRequest::$post) && !empty(UserRequest::$post['MAIL_folder'])) {
		  $filter .= $boolean . ' FOLDER = ' . singleQuote(UserRequest::$post['MAIL_folder']);
		  $boolean = 'AND';
		  $_SESSION['MAIL']['folder'] = UserRequest::$post['MAIL_folder']; //Adding Folder Option for folder list
        }
        if (array_key_exists('MAIL_email_account', UserRequest::$post) && !empty(UserRequest::$post['MAIL_email_account'])) {
		  $filter .= $boolean . ' EMAIL_ACCOUNT = ' . singleQuote(UserRequest::$post['MAIL_email_account']);
		  $boolean = 'AND';
		  //Adding Folder Option for folder list
		  $_SESSION['MAIL']['account'] = UserRequest::$post['MAIL_email_account'];
        }
        $_SESSION['paging']['MAILList']['sortby'] = '';
        $_SESSION['paging']['MAILList']['criteria'] = $filter;
        $_SESSION['paging']['MAILList']['startat'] = '0';
    }

    /**
     * Tool Display Form
     *
     * @param void
     * @return void
     */
    function clearfilter() {
        $_SESSION['MAIL']['folder'] = ''; //Adding Folder Option for folder list
		parent::clearfilter();
    }

    /**
     * Load tool record
     *
     * @param int $recordid
     * @return array $record
     */
    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['MAIL_id'] = $recordid;
        $record['MAIL_from_email'] = stripslashes($values['FROM_EMAIL']);
        $record['MAIL_subject'] = stripslashes($values['SUBJECT']);
        $record['MAIL_to_email'] = stripslashes($values['TO_EMAIL']);
        $record['MAIL_cc_email'] = stripslashes($values['CC_EMAIL']);
        $record['MAIL_bcc_email'] = stripslashes($values['BCC_EMAIL']);
        $record['MAIL_reply_email'] = stripslashes($values['REPLY_EMAIL']);
        $record['MAIL_email_priority'] = $values['EMAIL_PRIORITY'];
        $record['MAIL_email_contents'] = stripslashes($values['EMAIL_CONTENTS']);
        $record['MAIL_folder'] = $values['FOLDER'];
        $record['MAIL_email_direction'] = $values['EMAIL_DIRECTION'];
        $record['MAIL_email_account'] = $values['EMAIL_ACCOUNT'];
        $record['MAIL_is_signature'] = $values['IS_SIGNATURE'];
        $record['MAIL_cdate'] = $values['CDATE'];
        $record['MAIL_mdate'] = $values['CDATE'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['FROM_EMAIL'] = UserRequest::$post['MAIL_from_email'];
        $record['SUBJECT'] = UserRequest::$post['MAIL_subject'];
        $record['TO_EMAIL'] = UserRequest::$post['MAIL_to_email'];
        $record['CC_EMAIL'] = UserRequest::$post['MAIL_cc_email'];
        $record['BCC_EMAIL'] = UserRequest::$post['MAIL_bcc_email'];
        $record['REPLY_EMAIL'] = UserRequest::$post['MAIL_reply_email'];
        $record['EMAIL_PRIORITY'] = UserRequest::$post['MAIL_email_priority'];
        $record['EMAIL_CONTENTS'] = UserRequest::$post['MAIL_email_contents'];
        $record['SEND_DATE'] = xFormat::getNow();
        $record['RECEIVED_DATE'] = xFormat::getNow();
        $record['FOLDER'] = 'DRFT';
        $record['EMAIL_DIRECTION'] = (!empty(UserRequest::$post['MAIL_email_direction'])) ? UserRequest::$post['MAIL_email_direction'] : '';
        $record['EMAIL_ACCOUNT'] = UserRequest::$post['MAIL_from_email'];
        $record['TEXT_ONLY'] = CLX_INIT_ZERO;
        if (array_key_exists('MAIL_is_signature', UserRequest::$post))
		  $record['IS_SIGNATURE'] = UserRequest::$post['MAIL_is_signature'];
        $record['MDATE'] = $d;
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['MAIL_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['FROM_EMAIL'] = UserRequest::$post['MAIL_from_email'];
        $record['SUBJECT'] = UserRequest::$post['MAIL_subject'];
        $record['TO_EMAIL'] = UserRequest::$post['MAIL_to_email'];
        $record['CC_EMAIL'] = UserRequest::$post['MAIL_cc_email'];
        $record['BCC_EMAIL'] = UserRequest::$post['MAIL_bcc_email'];
        $record['REPLY_EMAIL'] = UserRequest::$post['MAIL_reply_email'];
        $record['EMAIL_PRIORITY'] = UserRequest::$post['MAIL_email_priority'];
        $record['EMAIL_CONTENTS'] = UserRequest::$post['MAIL_email_contents'];
        $record['FOLDER'] = UserRequest::$post['MAIL_folder'];
        $record['EMAIL_DIRECTION'] = (!empty(UserRequest::$post['MAIL_email_direction'])) ? UserRequest::$post['MAIL_email_direction'] : '';
        $record['EMAIL_ACCOUNT'] = UserRequest::$post['MAIL_from_email'];
        $record['IS_SIGNATURE'] = (!empty(UserRequest::$post['MAIL_is_signature'])) ? UserRequest::$post['MAIL_is_signature'] : '0';
        $record['VERSION_COUNTER'] = (!empty(UserRequest::$post['MAIL_version_counter'])) ? UserRequest::$post['MAIL_version_counter'] : '0';
        $record['SEND_DATE'] = xFormat::getNow();
        $record['RECEIVED_DATE'] = xFormat::getNow();
        $record['TEXT_ONLY'] = CLX_INIT_ZERO;
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        return $recordid;
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $MAAT = &getTool('MAAT');
        $MAAT->deleteAttachments($id);
    }

    function defaultRecord() {
        $record = array();
        $record['MAIL_id'] = CLX_INIT_ZERO;
        $record['MAIL_email_status'] = '';
        $record['MAIL_subject'] = '';
        $record['MAIL_from_email'] = '';
        $record['MAIL_to_email'] = '';
        $record['MAIL_cc_email'] = '';
        $record['MAIL_bcc_email'] = '';
        $record['MAIL_reply_email'] = '';
        $record['MAIL_send_date'] = xFormat::getNow();
        $record['MAIL_received_date'] = xFormat::getNow();
        $record['MAIL_email_status'] = '';
        $record['MAIL_email_priority'] = '';
        $record['MAIL_email_headers'] = '';
        $record['MAIL_email_contents'] = '';
        $record['MAIL_mime_type_primary'] = '';
        $record['MAIL_mime_type_secondary'] = '';
        $record['MAIL_mime_encoding'] = '';
        $record['MAIL_folder'] = 'DRFT';
        $record['MAIL_email_direction'] = 'DRFT';
        $record['MAIL_email_account'] = '';
        $record['MAIL_is_signature'] = '0';
        $record['MAIL_cdate'] = '';
        $record['MAIL_mdate'] = '';
        $record['MAIL_version'] = '1.0.0';
        $record['MAIL_version_counter'] = '0';
        $record['MAIL_username'] = $_SESSION['user_record']['username'];
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        ClxFreeReturn($fieldname);
    }

// C O M P O U S E  M A I L
    /**
     * Creates a new email for Reply, Replyall and Forward and store it into mail draft table
     * Returns last createed message id
     */
    function mailAction($msg_id, $action) {//Replay && Forward
        $mailid = CLX_INIT_ZERO;
        $d = xFormat::getNow('Y-m-d H:i:s');
        $mailrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . singleQuote($msg_id));
        if (empty($mailrec)) {
		  return $mailid;
        }
        //Cleaning From
        $from = trim($mailrec['FROM_EMAIL']);
        if (($pos = strpos($from, '<')) > CLX_NUM_ZERO) {
		  $from = substr($from, $pos + CLX_NUM_ONE, strpos($from, '>', $pos) - $pos - CLX_NUM_ONE);
        }
        //Find to account
        $to = trim($mailrec['TO_EMAIL']);
        if (($pos = strpos($to, '<')) > CLX_NUM_ZERO) {
		  $to = substr($to, $pos + CLX_NUM_ONE, strpos($to, '>', $pos) - $pos - CLX_NUM_ONE);
        }
        $to = str_replace(CLX_CHAR_SINGLE_QUOTE, '', $to);
        $to = str_replace(CLX_CHAR_SINGLE_QUOTE, '', $to);
        $mailrec['CC_EMAIL'] = str_replace(CLX_CHAR_SINGLE_QUOTE, '', $mailrec['CC_EMAIL']);
        $mailrec['CC_EMAIL'] = str_replace(CLX_CHAR_SINGLE_QUOTE, '', $mailrec['CC_EMAIL']);
        //Carbon copy
        $cc = trim($mailrec['CC_EMAIL']);
        //Reply to
        $reply = trim($mailrec['REPLY_EMAIL']);
        //Find email Account code
        $accountmail = explode(',', str_replace(';', ',', $to));
        reset($accountmail);
        $MAAC = &getTool('MAAC');
        foreach ($accountmail as $key => $val) {
		  $accountrec = array();
		  $accountrec = ClxCommon::$db->getRecord($MAAC->tablename, 'USERNAME=' . singleQuote($_SESSION['user_record']['username']), 'EMAIL LIKE ' % ' . addslashes($val) . ' % '');
		  if (!empty($accountrec))
			 break;
        }
        $mailrec['SEND_DATE'] = $d;
        $mailrec['RECEIVED_DATE'] = $d;
        //Clear not req values and Set new record
        unset($mailrec['ID'], $mailrec['EMAIL_STATUS'], $mailrec['CC_EMAIL'], $mailrec['EMAIL_HEADERS'], $mailrec['REPLY_EMAIL']);
        $mailrec['ID'] = '';
        //User General Information and Dates
        $mailrec['USERNAME'] = $_SESSION['user_record']['username'];
        $mailrec['EMAIL_DIRECTION'] = 'OUT';
        $mailrec['FOLDER'] = 'DRFT';
        $mailrec['SEND_DATE'] = xFormat::getNow();
        $mailrec['CDATE'] = $d;
        $mailrec['MDATE'] = $d;
        $mailrec['FOLDER'] = 'DRFT';
        //Clean prev Content
        $content = trim($mailrec['EMAIL_CONTENTS']);
        switch ($action) {
		  case 'FORWARD':
			 $mailrec['SUBJECT'] = '[Fwd: ' . $mailrec['SUBJECT'] . ']';
			 $mailrec['FROM_EMAIL'] = '';
			 $mailrec['TO_EMAIL'] = '';
			 $mailrec['EMAIL_CONTENTS'] = $content;
			 $mailid = ClxCommon::$db->createRecord($this->tablename, $mailrec, false);
			 //Adding Attachments
			 $MAAT = &getTool('MAAT');
			 $files = ClxCommon::$db->getRecords($MAAT->tablename, 'MAILID=' . $msg_id);
			 while (($file = ClxCommon::$db->getNextRecord($files))) {
			     unset($file['ID']);
			     $file['MAILID'] = $mailid;
			     ClxCommon::$db->createRecord($MAAT->tablename, $file, false);
			 }
 			 ClxCommon::$db->freeResult($files);
			 break;
		  case 'REPLY':
			 $mailrec['SUBJECT'] = 'Re: ' . $mailrec['SUBJECT'];
			 $mailrec['FROM_EMAIL'] = '';
			 if (!empty($reply))
			     $mailrec['TO_EMAIL'] = $reply;
			 else
			     $mailrec['TO_EMAIL'] = $from;
			 $mailrec['EMAIL_CONTENTS'] = $from . ' wrote: ' . '<br>' . $content . CLX_CHAR_NL;
			 $mailid = ClxCommon::$db->createRecord($this->tablename, $mailrec, false);
			 break;
		  case 'REPLYALL':
			 $mailrec['SUBJECT'] = 'Re: ' . $mailrec['SUBJECT'];
			 $mailrec['FROM_EMAIL'] = '';
			 $mailrec['TO_EMAIL'] = addslashes($from . ',' . $to);
			 if (!empty($cc))
			     $mailrec['CC_EMAIL'] = addslashes($cc);
			 $mailrec['EMAIL_CONTENTS'] = addslashes($content);
			 $mailid = ClxCommon::$db->createRecord($this->tablename, $mailrec, false);
			 break;
		  default:
			 break;
        }
        return $mailid;
    }

// R E C I E V E
    /**
     * Get Email Action
     */
    function getMail() {
        $contents = '';
        $accounts = $this->getAccounts($_SESSION['user_record']['username']);
        if (count($accounts) > CLX_NUM_ZERO) {
		  foreach ($accounts as $key => $val) {
			 $contents .= $this->getMailAccount($val) . ' ' . '<br>';
		  }
        } else {
		  $contents = note(dict('BASE_MAIL_NO_ACCOUNTS'));
        }
        return $contents;
    }

    /**
     * Get Mail from Email account on POP3 server
     */
    function getMailAccount($mail_account) {
        ini_set('include_path', kPathToWebSite . kPathToPear);
        require_once'Net/POP3.php';
        require_once'Mail/mimeDecode.php';
        if (!class_exists('Net_POP3')) {
		  return alert(dict('GEN_MSG_MAILDISABLED'));
		  salir();
        }
        UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
        UserResponse::setRunTimeLimit(CLX_NUM_ONE * CLX_NUM_60 * CLX_NUM_60);
        $decode_options = array();
        $decode_options['include_bodies'] = true;
        $decode_options['decode_bodies'] = true;
        $decode_options['decode_headers'] = true;
        $pop3 = new Net_POP3();
        $in_port = (isset($mail_account['IN_PORT']) ? $mail_account['IN_PORT'] : CLX_NUM_110);
        $ok = $pop3->connect($mail_account['IN_HOSTNAME'], $in_port);
        if (empty($ok)) {
		  return alert(dict('BASE_MAIL_ERROR_CANT_CONNECT', $mail_account['IDNAME'] . ' -> ' . $mail_account['IN_HOSTNAME'].':'.$in_port));
        }
        $ok = $pop3->login($mail_account['IN_USERNAME'], $mail_account['IN_PASSWORD']);
        if (empty($ok))
		  return alert(dict('BASE_MAIL_ERROR_CANT_LOGIN', $mail_account['IDNAME']));
        $qty = $pop3->numMsg();
        if (empty($qty))
		  return alert(dict('BASE_MAIL_ERROR_CANT_GETQTY', $mail_account['IDNAME']));
        if ($qty > CLX_NUM_ZERO) {
		  for ($i = $qty; $i >= CLX_NUM_ONE; $i--) {
			 $hdrs = $pop3->getParsedHeaders($i);
			 $messageid = md5( $hdrs['From'] . (isset($hdrs['Delivery-date'])?$hdrs['Delivery-date']:$hdrs['Date']) );
                if (ClxCommon::$db->oneExistsRecords($this->tablename, 'MESSAGEID = ' . singleQuote($messageid)) > CLX_NUM_ZERO) {
			     $qty--;
			     continue;
			 }
			 $raw_msg = $pop3->getMsg($i);
			 $mime = new Mail_mimeDecode($raw_msg);
			 $msg = $mime->decode($decode_options);
			 //e cho xDebug::dump($msg);
			 $record = array();
			 $record['FOLDER'] = 'INBX';
			 $record['EMAIL_ACCOUNT'] = $mail_account['IDNAME'];
			 $record['MESSAGEID'] = $messageid;
			 $record['FROM_EMAIL'] = $msg->headers['from'];
			 $record['TO_EMAIL'] = $msg->headers[CLX_PROP_TO];
			 $record['CC_EMAIL'] = (isset($msg->headers['cc'])) ? $msg->headers['cc'] : '';
			 $record['REPLY_EMAIL'] = (isset($msg->headers['reply-to'])) ? $msg->headers['reply-to'] : '';
			 $record['SUBJECT'] = $msg->headers['subject'];
			 $record['MIME_TYPE_PRIMARY'] = $msg->ctype_primary;
			 $record['MIME_TYPE_SECONDARY'] = $msg->ctype_secondary;
			 if ($msg->ctype_primary === 'multipart') {
			     $record['MIME_ENCODING'] = (array_key_exists('content-type', $msg->headers)) ? $msg->headers['content-type'] : '';
			 } else {
			     $record['MIME_ENCODING'] = (array_key_exists('content-transfer-encoding', $msg->headers)) ? $msg->headers['content-transfer-encoding'] : '';
			 }
			 $record['SEND_DATE'] = xFormat::date($msg->headers['date'], 'Y-m-d H:i:s');
			 $record['RECEIVED_DATE'] = xFormat::getNow('Y-m-d H:i:s');
			 $record['EMAIL_STATUS'] = 'NEW';
			 //Setting Priority
			 if (!isset($msg->headers['x-priority']))
			     $record['EMAIL_PRIORITY'] = CLX_INIT_ZERO;
			 elseif (stristr($msg->headers['x-priority'], 'HIGH'))
			     $record['EMAIL_PRIORITY'] = CLX_NUM_ONE;
			 elseif (stristr($msg->headers['x-priority'], 'HIGHEST'))
			     $record['EMAIL_PRIORITY'] = CLX_NUM_TWO;
			 elseif (stristr($msg->headers['x-priority'], 'NORMAL'))
			     $record['EMAIL_PRIORITY'] = CLX_NUM_THREE;
			 elseif (stristr($msg->headers['x-priority'], 'LOW'))
			     $record['EMAIL_PRIORITY'] = CLX_NUM_FOUR;
			 elseif (stristr($msg->headers['x-priority'], 'LOWEST'))
			     $record['EMAIL_PRIORITY'] = CLX_NUM_FIVE;
			 $record['EMAIL_HEADERS'] = $this->niceHeaders($msg->headers);
			 $record['EMAIL_HEADERS'] = '';
			 $record['EMAIL_CONTENTS'] = '';
			 if (!empty($msg->body)) {
			     $record['EMAIL_CONTENTS'] = addslashes(trim($msg->body));
			 }
			 $record['EMAIL_DIRECTION'] = 'IN';
			 $record['USERNAME'] = $mail_account['USERNAME'];
			 $record['CDATE'] = xFormat::getNow('Y-m-d H:i:s');
			 $record['MDATE'] = xFormat::getNow('Y-m-d H:i:s');
			 foreach ($record as $key => $val) {
			     $record[$key] = addslashes($val);
			 }
			 $mailid = ClxCommon::$db->createRecord($this->tablename, $record, false);
			 if (isset($msg->parts)) {
			     ClxFreeReturn($this->saveMailAttachments($record, $mailid, $msg->parts));
			 }
			 if ($mail_account['IN_DELETEMSG'] === true)
			     $pop3->deleteMsg($i);
		  }
        }
        $pop3->disconnect();
        return note(dict('BASE_MAIL_NOTE_GET_MAIL', $qty, $mail_account['IDNAME']));
    }

    function niceHeaders($raw_headers) {
        $headers = '';
        foreach ($raw_headers as $key => $value) {
		  if(is_array($value)) {
			 $aux_value = '';
			 foreach ($value as $key1 => $value1) {
			     $aux_value .= CLX_CHAR_NL . '<b>' . $key1 . '</b>: ' . $value1;
			 }
			 $headers .= '<b>' . $key . '</b>:' . $aux_value . CLX_CHAR_NL;
		  } else {
			 $headers .= '<b>' . $key . '</b>: ' . $value . CLX_CHAR_NL;
		  }
        }
        return $headers;
    }

    /**
     * Save Email Attachements
     */
    function saveMailAttachments($mailrec, $mailid, $parts) {
        $MAAT = &getTool('MAAT');
        $hasContents = false;
        foreach ($parts as $part) {
		  if (isset($part->parts)) {
			 $mailrec = $this->saveMailAttachments($mailrec, $mailid, $part->parts);
		  } else {
			 if ($part->ctype_primary === 'text' && ($part->ctype_secondary === 'plain' || $part->ctype_secondary === 'html') && !$hasContents) {
			     $hasContents = true;
			     if (empty($part->body))
			         continue;
			     $contents = $part->body;
			     $mailrec['EMAIL_CONTENTS'] .= addslashes($contents);
			     ClxCommon::$db->updateRecord($this->tablename, $mailrec, 'ID=' . $mailid);
			 }else {
			     $contents = (isset($part->body)) ? $part->body : '';
			     if (empty($part->d_parameters['filename']))
			         continue;
			     $name = $part->d_parameters['filename'];
			     $type = $part->ctype_primary . '/' . $part->ctype_secondary;
			     $contents = (isset($part->body)) ? $part->body : '';
			     $this->store_attondb = false;
			     $path = '';
			     $encoding = $part->headers['content-transfer-encoding'];
			     $fileid = $MAAT->saveFile($mailid, $name, $type, $contents, $this->store_attondb, $path, $encoding, '');
			     if (strstr($part->disposition, 'inline')) {
			         $content_id = $part->headers['content-id'];
			         $mailrec['EMAIL_CONTENTS'] = addslashes(str_replace($content_id, 'window.php?to=MAAT&ac=SHOW_FILE&ff=' . $fileid, $mailrec['EMAIL_CONTENTS']));
			         ClxCommon::$db->updateRecord($this->tablename, $mailrec, 'ID=' . $mailid);
			     }
			 }
		  }
        }
        return $mailrec;
    }

    /**
     * Set all emails as unread
     */
    function markUnread() {
        $idlist = getIdlist();
        if (!empty($idlist)) {
		  ClxCommon::$db->updateRecord($this->tablename, array('EMAIL_STATUS' => 'NEW'), 'ID IN (' . implode(',', $idlist) . ')');
        }
    }

    function attachmentListing() {

    }

    /**
     * Returns Array of email accounts
     */
    function getAccounts($username) {
        $return = array();
        $MAAC = &getTool('MAAC');
        $results = ClxCommon::$db->getRecords($MAAC->tablename, 'UPPER(USERNAME)=' . singleQuote(uppercase($username)));
        while ($result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  $return[] = $result;
        }
		ClxCommon::$db->freeResult($results);
        return $return;
    }

    /**
     * Returns Array of email account
     */
    function getAccount($idname) {
        $MAAC = &getTool('MAAC');

        return ClxCommon::$db->getRecord($MAAC->tablename, 'IDNAME=' . singleQuote($idname));
    }

// S E N D
    function sendMails($idlist) {


        ini_set('include_path', kPathToWebSite . kPathToPear);
        require_once'Net/POP3.php';
        require_once'Mail/mimeDecode.php';
        if (!class_exists('Net_POP3')) {

		  return alert(dict('GEN_MSG_MAILDISABLED'));
		  salir();
        }

        foreach ($idlist as $id) {
		  ClxFreeReturn($this->sendMail($id));
        }
    }

    function sendMail($mailid, $batch = false) {
        UserResponse::setIgnoreUserAbort(CLX_NUM_ONE);
        UserResponse::setRunTimeLimit(CLX_NUM_24 * CLX_NUM_60 * CLX_NUM_60);
        ini_set('include_path', kPathToWebSite . kPathToPear);
        require_once'Mail.php';
        require_once'Mail/mime.php';
        if (!class_exists('Mail_mime')) {
		  return alert(dict('GEN_MSG_MAILDISABLED'));
		  salir();
        }
        $mailrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $mailid);
        $accountrec = $this->getAccount($mailrec['EMAIL_ACCOUNT']);
        if (empty($mailrec)) {
		  addMessage(dict('GEN_MAIL_MAILNOTFOUND'), CLX_CLASS_ALERT);
		  return CLX_CHAR_EMPTY_STRING;
        }

        if (empty($accountrec)) {
		  //addMessage(dict('GEN_MAIL_INVALIDACCOUNTFROM'), CLX_CLASS_ALERT);
		  //return CLX_CHAR_EMPTY_STRING;
		  $accountrec = array('SIGNATURE' => 0, 'FULLNAME' => kDefaultEmail, 'EMAIL' => kDefaultEmail,
			 'OUT_HOSTNAME' => kDefaultSmtp, 'OUT_PORT' => kDefaultSmtpPort, 'OUT_AUTHENTICATE' => kDefaultSmtpReqAuth,
			 'OUT_USERNAME' => kDefaultSmtpUsername, 'OUT_PASSWORD' => kDefaultSmtpPassword);
		  $this->outtype = kDefaultMailer;
        }

        //signature
        if ($mailrec['IS_SIGNATURE'] == CLX_NUM_ONE) {
		  $mailrec['EMAIL_CONTENTS'] .='<hr noshade size="1"><font face="arial" size=1><i>' . $accountrec['SIGNATURE'] . '</i></font>';
        }
		if (kDefaultMailer!='phpMailer') {
			$mailrec['EMAIL_CONTENTS'] = decode_string(html_entity_decode ($mailrec['EMAIL_CONTENTS']));
			$mailrec['EMAIL_CONTENTS'] = stripslashes($mailrec['EMAIL_CONTENTS']);
			$mailrec['EMAIL_CONTENTS'] = str_replace(CLX_CHAR_NL . CLX_CHAR_NL, CLX_CHAR_NL, stripslashes($mailrec['EMAIL_CONTENTS']));
			//$mailrec['EMAIL_CONTENTS'] = decode_string(str_replace(CLX_CHAR_NL, '<br>', $mailrec['EMAIL_CONTENTS']));
			$mailrec['EMAIL_CONTENTS'] = str_replace(CLX_CHAR_NL, '<br>', $mailrec['EMAIL_CONTENTS']);
        }
		//Setting mail
        //Recipients - setting cc and bcc
        $recipients = array('To' => $mailrec['TO_EMAIL']);
        if (!empty($mailrec['CC_EMAIL']))
		  $recipients['Cc'] = stripslashes($mailrec['CC_EMAIL']);
        if (!empty($mailrec['BCC_EMAIL']))
		  $recipients['Bcc'] = stripslashes($mailrec['BCC_EMAIL']);
        //Message Headers
		if (kDefaultMailer=='phpMailer') {
			$text = strip_tags($mailrec['EMAIL_CONTENTS']);
			$html = '<html><body>' . $mailrec['EMAIL_CONTENTS'] . '</body></html>';
			$crlf = CLX_CHAR_CR . CLX_CHAR_NL;

			$hdrs = array('From' => ($accountrec['FULLNAME'] <> '') ? $accountrec['FULLNAME'] . ' <' . $accountrec['EMAIL'] . '>' : $accountrec['EMAIL'],
			  'To' => $mailrec['TO_EMAIL'],
				'Subject' => $mailrec['SUBJECT'],                            // comentado para probar stripslashes(decode_string(html_entity_decode ($mailrec['SUBJECT']))),
			  'Date' => xFormat::getNow('r'),
			  'Message-ID' => clx_mcript($mailrec['ID']) . strstr($accountrec['EMAIL'], '@'),
			  'Content-ID' => clx_mcript($mailrec['ID']) . strstr($accountrec['EMAIL'], '@')
			);
			
		}
		else {
			$text = strip_tags(stripslashes($mailrec['EMAIL_CONTENTS']));
			$html = '<html><body>' . stripslashes($mailrec['EMAIL_CONTENTS']) . '</body></html>';
			$crlf = CLX_CHAR_CR . CLX_CHAR_NL;

			$hdrs = array('From' => ($accountrec['FULLNAME'] <> '') ? $accountrec['FULLNAME'] . ' <' . $accountrec['EMAIL'] . '>' : $accountrec['EMAIL'],
			  'To' => $mailrec['TO_EMAIL'],
				'Subject' => stripslashes(html_entity_decode ($mailrec['SUBJECT'])),                            // comentado para probar stripslashes(decode_string(html_entity_decode ($mailrec['SUBJECT']))),
			  'Date' => xFormat::getNow('r'),
			  'Message-ID' => clx_mcript($mailrec['ID']) . strstr($accountrec['EMAIL'], '@'),
			  'Content-ID' => clx_mcript($mailrec['ID']) . strstr($accountrec['EMAIL'], '@')
			);
		
			//Build message
			$mime = new Mail_mime($crlf);
			//$mime->_build_params['html_charset'] = 'ISO-8859-1';
			/* $mime->_build_params['html_charset'] = 'UTF-8';
			$mime->_build_params['text_charset'] = $mime->_build_params['html_charset'];
			$mime->_build_params['head_charset'] = $mime->_build_params['html_charset'];
			 */
			$mime->setTXTBody($text);
			$mime->setHTMLBody($html);
			//Attachments
			$MAAT = &getTool('MAAT');
			$files = ClxCommon::$db->getRecords($MAAT->tablename, 'MAILID=' . $mailid);
			while (($filerec = ClxCommon::$db->getNextRecord($files))) {
			  if (strlen($filerec['FILENAME']) == CLX_NUM_ZERO) {
				 $filecontents = stripslashes($filerec['CONTENTS']);
			  } else {
				 $filecontents = file_get_contents(kPathToWebSite . kPathToData . $filerec['FILENAME']);
			  }
			  $mime->addAttachment($filecontents, $filerec['type'], $filerec['name'], false);
			}
			$body = $mime->get();
		}
		if (kDefaultMailer=='phpMailer') {
			
			require_once(kPathToWebSite."/../extlibs/php/PHPMailer/src/PHPMailer.php");
			require_once(kPathToWebSite."/../extlibs/php/PHPMailer/src/SMTP.php");
			
			//pre($recipients);
			pre($hdrs);
			//die;
			$mail = new PHPMailer(true);
			//Server settings
			$mail->SMTPDebug=1;                                                 // Pon en 1 para ver debug de si se ha enviado o no el correo
			$mail->isSMTP();                                                    // Enviar con SMTP
		//	pre($accountrec['OUT_HOSTNAME']  );
			$mail->Host       = $accountrec['OUT_HOSTNAME'];                     // Servidor SMTP
			$mail->SMTPAuth   = !empty($accountrec['OUT_AUTHENTICATE']);        // Habilitar SMTP authentication
			$mail->Username   = stripslashes($accountrec['OUT_USERNAME']);      // SMTP usuario
			$mail->Password   = stripslashes($accountrec['OUT_PASSWORD']);      // SMTP password
			$mail->SMTPSecure = PHPMailer::ENCRYPTION_STARTTLS;                 // Enable TLS encryption; `PHPMailer::ENCRYPTION_SMTPS` also accepted
			$mail->Port       = $accountrec['OUT_PORT'];                                    // TCP port to connect to
			$mail->SMTPDebug = 2;
			
			//$mail->isHTML(true);
			$mail->CharSet = 'UTF-8';
			//$mail->addCustomHeader('Content-Type', 'text/plain;charset=utf-8');


			//Recipients
			$from=explode('<',str_replace('>','',$hdrs['From']));
			$mail->setFrom($from[1], $from[0]);
			$mail->addAddress($hdrs['To']);     // Añadir destinatario

			// Contenido
			$mail->isHTML(true);                                  	// Enviar en HTML
			$mail->Subject = $hdrs['Subject'];
			$mail->Body    = $html;

			if (!$mail->send()) {
				echo 'Mailer Error: ' . $mail->ErrorInfo;
				
				if (empty($batch))
					addMessage(dict('GEN_MAIL_MSFSENDFAIL', $mail->ErrorInfo));
				$r = array();
				$r['SEND_DATE'] = xFormat::getNow();
				$r['EMAIL_DIRECTION'] = 'OUT';
				$r['EMAIL_STATUS'] = 'ERR';
				$r['ERROR_MSG'] = $mail->ErrorInfo;
				ClxCommon::$db->updateRecord($this->tablename, $r, 'ID=' . singleQuote($mailid));
				return false;
				
			} else {
				echo 'Email Send!!';
			}
		}
		else {
			 $hdrs = $mime->headers($hdrs);
			switch ($this->outtype) {
			  case 'sendmail':
				 $params = array();
				 $params['sendmail_path'] = (defined('kPathToSendmail')) ? kPathToSendmail : '';
				 $mail = & Mail::factory('sendmail', $params);
				 break;
			  case 'smtp':
				 $params = array();
				 $params['host'] = $accountrec['OUT_HOSTNAME']; //- The server to connect. Default is localhost
				 $params['port'] = $accountrec['OUT_PORT']; //- The port to connect. Default is 25
				 $params['auth'] = !empty($accountrec['OUT_AUTHENTICATE']); //- Whether or not to use SMTP authentication. Default is false
				 $params['username'] = stripslashes($accountrec['OUT_USERNAME']); //- The username to use for SMTP authentication.
				 $params['password'] = stripslashes($accountrec['OUT_PASSWORD']); //- The password to use for SMTP authentication.
				 $params['persist'] = false; //- Indicates whether or not the SMTP connection should persist over multiple calls to the send() method.
				 $mail = &Mail::factory('smtp', $params);
				 break;
			  default: //PHP Mailer at last

				 $mail = &Mail::factory('mail');
				 break;
			}
			/**
			 * GEN_MAIL_MSFSENDFAIL
			 * Message could not be sent, server response: <i>-^1</i>
			 * Mansaje no pudo ser enviado. Respuesta del servidor: <i>-^1</i>
			 */
			$result = $mail->send($recipients, $hdrs, $body);
			if (is_object($result)) {
				if (empty($batch))
					addMessage(dict('GEN_MAIL_MSFSENDFAIL', $result->message));
				$r = array();
				$r['SEND_DATE'] = xFormat::getNow();
				$r['EMAIL_DIRECTION'] = 'OUT';
				$r['EMAIL_STATUS'] = 'ERR';
				$r['ERROR_MSG'] = $result->message;
				ClxCommon::$db->updateRecord($this->tablename, $r, 'ID=' . singleQuote($mailid));
				return false;
			} elseif (!empty($result)) {
				$r = array();
				$r['SEND_DATE'] = xFormat::getNow();
				$r['RECEIVED_DATE'] = xFormat::getNow();
				$r['EMAIL_DIRECTION'] = 'OUT';
				$r['FOLDER'] = 'SENT';
				$r['FROM_EMAIL'] = $accountrec['EMAIL'];
				$r['ERROR_MSG'] = 'null';
				ClxCommon::$db->updateRecord($this->tablename, $r, 'ID=' . singleQuote($mailid));
				if (empty($batch))
					addMessage(dict('GEN_MAIL_SENT'), 'note');
			

			}
		}
	}

    function attachFile($mailid) {
        if (strlen(UserRequest::$files['MAIL_attachments']['name']) > CLX_NUM_ZERO) {
		  $MAAT = &getTool('MAAT');
		  $path = kPathToWebSite . kPathToData;
		  $name = UserRequest::$files['MAIL_attachments']['name'];
		  $type = UserRequest::$files['MAIL_attachments']['type'];
		  $contents = file_get_contents(UserRequest::$files['MAIL_attachments']['tmp_name']);
		  $MAAT->saveFile($mailid, $name, $type, $contents, $this->store_attondb, $path);
        }
    }

    function deleteAttach($attach_id) {
        if (!empty($attach_id)) {
		  $MAAT = &getTool('MAAT');
		  $MAAT->deleteRecordSk($attach_id);
        }
    }

    /**
     * Search contact table to add addresses
     */
    function searchLibAddress() {
        $msg = array();
        $tablecode = 'CONT';
        $tablename = tableName($tablecode);
        $fieldname = 'MAIL_' . UserRequest::$request['fl'];
        $script = '';
        $variables = array();
        $contents = setJavaScriptTag(getJavaScript('EDIT_search.js', $variables));
        // The Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, 'MAIL');
        $form->addArgument('ac', 'SEARCHADDRESS');
        $form->addArgument('confirm', '1');
        $form->addArgument('fl', UserRequest::$request['fl']);
        // Buttons
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => 'buttons', CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCHADDRESS', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => 'buttons', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $buttons = $table->source();
        $title = dict('TOOL_MAIL_TITLE_SEARCH_ADDRESS');
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!array_key_exists('firstname', UserRequest::$post))
		  UserRequest::$post['firstname'] = '';
        if (!array_key_exists('lastname', UserRequest::$post))
		  UserRequest::$post['lastname'] = '';
        if (!array_key_exists('email', UserRequest::$post))
		  UserRequest::$post['email'] = '';
        $table->addField($msg, dict('GEN_LBL_FIRSTNAME'), 'firstname', UserRequest::$post['firstname'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_LASTNAME'), 'lastname', UserRequest::$post['lastname'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_EMAIL'), 'email', UserRequest::$post['email'], CLX_NUM_32, CLX_NUM_32);
        $form->addContents($table->source());
        $form->addContents($buttons);
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $criteria = '';
		  $boolean = '';
		  if ($_SESSION['user_record']['isadmin'] !== '1') {
			 $tlooks = ClxCommon::$db->executeQuery('SELECT    TF.TABLECODE TABLECODE, F.* FROM CLX_DB_TABLEFIELDS TF,    CLX_DB_FIELDS F WHERE TF.TABLECODE = ' . singleQuote($tablecode) . ' AND TF.FIELDNAME = F.FIELDNAME AND F.FIELDTYPE = ' . singleQuote('TLO') . '');
			 $has_tlook_field = ClxCommon::$db->numRows($tlooks);
		  } else {
			 $has_tlook_field = false;
		  }
		  if (!empty($has_tlook_field)) {
			 $tcriteria = '';
			 $tboolean = '';
			 $ACTR = &getTool('ACTR');
			 while (($tlook_field = ClxCommon::$db->getNextRecord($tlooks))) {
			     $nodes = $ACTR->getUserNodes($tlook_field['FIELD_EXTRA1']);
			     foreach ($nodes as $node) {
			         $tcriteria .= $tboolean . '.' . $tlook_field['fieldname'] . ' LIKE ' . singleQuote($node . '%');
			         $tboolean = ' OR ';
			     }
			 }
			 ClxCommon::$db->freeResult($toolks);
			 if (strlen($tcriteria) > CLX_NUM_ZERO)
                    $criteria = '('.$tcriteria.')';
		  }
		  $criteria = '1=1 '; //TYPE = 'E'
		  if (strlen($criteria) > CLX_NUM_ZERO) {
			 $boolean = ' AND ';
		  }
		  if (strlen(UserRequest::$post['firstname']) > CLX_NUM_ZERO) {
			 $criteria .= $boolean . ' FIRSTNAME LIKE ' . singleQuote(UserRequest::$post['firstname'] . '%');
			 $boolean = ' AND ';
		  }
		  if (strlen(UserRequest::$post['lastname']) > CLX_NUM_ZERO) {
			 $criteria .= $boolean . ' LASTNAME LIKE ' . singleQuote(UserRequest::$post['lastname'] . '%');
			 $boolean = ' AND ';
		  }
		  if (strlen(UserRequest::$post['email']) > CLX_NUM_ZERO) {
			 $criteria .= $boolean . ' EMAIL LIKE ' . singleQuote(UserRequest::$post['email'] . '%');
			 $boolean = ' AND ';
		  }
		  //e cho $criteria;

		  $table = new htmlTable(CLX_WIDTH_FOUR);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST));
            $qty = ClxCommon::$db->oneExistsRecords($tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
			 $records = ClxCommon::$db->getRecords($tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 $table->addCell(dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDLISTLABEL', 'FIELDNAME = ' . singleQuote('IDNAM') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_NUM_HUNDRED));
			 $table->addCell(dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDLISTLABEL', 'FIELDNAME = ' . singleQuote('EMAI') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_NUM_HUNDRED));
			 $table->addCell(dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDLISTLABEL', 'FIELDNAME = ' . singleQuote('EMAIL_0') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_NUM_HUNDRED));
			 $table->addCell(dict(ClxCommon::$db->getField('CLX_DB_FIELDS', 'FIELDLISTLABEL', 'FIELDNAME = ' . singleQuote('EMAIL_0') . '')), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_WIDTH => CLX_NUM_HUNDRED));
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $class = ($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT;
			     $emails = array();
			     if (!empty($record['EMAIL']))
			         $emails[] = $record['EMAIL'];
			     if (!empty($record['EMAIL_02']))
			         $emails[] = $record['EMAIL_02'];
			     if (!empty($record['EMAIL_03']))
			         $emails[] = $record['EMAIL_03'];
			     $all_emails = implode(', ', $emails);
			     $js = 'javascript:setSearchValuesSingleMail(' . singleQuote($fieldname) . ', ' . singleQuote($all_emails) . ');';
			     $js1 = 'javascript:setSearchValuesSingleMail(' . singleQuote($fieldname) . ', ' . singleQuote($record['EMAIL']) . ');';
			     $js2 = 'javascript:setSearchValuesSingleMail(' . singleQuote($fieldname) . ', ' . singleQuote($record['EMAIL_02']) . ');';
			     $js3 = 'javascript:setSearchValuesSingleMail(' . singleQuote($fieldname) . ', ' . singleQuote($record['EMAIL_03']) . ');';
			     if (empty($record["IDNAME"]))
			         $record["IDNAME"] = '[-]';

			     $table->addCells($class, array('<a href=' . quote($js) . '>' . $record['IDNAME'] . '</a>',
			         '<a href=' . quote($js1) . '>' . $record['EMAIL'] . '</a>',
			         '<a href=' . quote($js2) . '>' . $record['EMAIL_02'] . '</a>',
			         '<a href=' . quote($js3) . '>' . $record['EMAIL_03'] . '</a>'));
			     $i++;
			 }
		  }
		  ClxCommon::$db->freeResult($records);
		  $contents .= $table->source();
        }
        return $contents . $script;
    }

    function pageletform() {
        return $this->makeList(false);
    }

// QUEUE
    /**
     * Sets message to queue
     * @param type $from
     * @param type $to
     * @param type $msg
     * @param type $subject
     * @param type $html
     * @param type $cc
     * @param type $bcc
     * @param type $replay_to
     * @param type $attachments
     * @param type $msg_txt
     * @param type $request_receipt
     * @param type $encoding
     * @param type $debug
     */
    function setMessageQueue($from, $to, $msg, $subject, $html = true, $cc = '', $bcc = '', $replay_to = '', $attachments = array(), $msg_txt = '', $request_receipt = false, $encoding = 'ISO-8859-1', $debug = false, $maac_email_account = '') {
		
        //1.store email information
        //2.store attachment information
        //...later runMessageQueue will do its job
        if ($maac_email_account === '') {
		  //if not email account then try to search for one. First by sender or then by default.
		  $maac_email_account = ClxCommon::$db->getField('CLX_MA_MAILACCOUNTS', 'IDNAME', 'EMAIL = ' . singleQuote($from));
		  if (empty($maac_email_account)) {
			 $maac_email_account = ClxCommon::$db->getField('CLX_MA_MAILACCOUNTS', 'IDNAME', 'SYSTEM_DEFAULT = 1');
		  }
		  if (empty($maac_email_account)) {
			 $maac_email_account = $from;
		  }
        }
        $d = xFormat::getNow('Y-m-d H:i:s');
        $record = $this->defaultTableRecord();
        $record['SUBJECT'] = $subject;
        $record['FROM_EMAIL'] = $from;
        $record['TO_EMAIL'] = $to;
        $record['CC_EMAIL'] = $cc;
        $record['BCC_EMAIL'] = $bcc;
        $record['REPLY_EMAIL'] = $replay_to;
        $record['EMAIL_CONTENTS'] = $msg;
        $record['EMAIL_CONT_TXT'] = $msg_txt;
        $record['EMAIL_STATUS'] = 'QUEUE';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['FOLDER'] = 'OUBX';
        $record['EMAIL_DIRECTION'] = 'OUT';
        $record['EMAIL_ACCOUNT'] = $maac_email_account;
        $record['MIME_ENCODING'] = $encoding;
        $record['USERNAME'] = (isset($_SESSION['user_record']['username'])) ? $_SESSION['user_record']['username'] : 'system';
        $record['TEXT_ONLY'] = ($html) ? CLX_NUM_ZERO : CLX_NUM_ONE;
        $record['REQ_RECEIPT'] = ($request_receipt) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        //Creating message record
        $record['ID'] = ClxCommon::$db->createRecord($this->tablename, $record, false);
		$MAAT = &getTool('MAAT');
        //Storing Attachments
        foreach ($attachments as $val) {
		  $MAAT->saveFileByName($record['ID'], $val, true);
        }
		ClxCommon::$db->commit();
    }

    /**
     * Run message queue
     */
    private function runMessageQueue($batch = false) {

        $batch_id = batch_id();
        $sql = 'UPDATE ' . $this->tablename . '  SET MESSAGEID = ' . singleQuote($batch_id) . ', EMAIL_STATUS = ' . singleQuote('PRO') . '  WHERE FOLDER = ' . singleQuote('OUBX') . '  AND EMAIL_DIRECTION = ' . singleQuote('OUT') . '  AND EMAIL_STATUS = ' . singleQuote('QUEUE');
        ClxCommon::$db->executeQuery($sql);
        ClxCommon::$db->commit();
        if (!empty($batch)) {
		  $qty = ClxCommon::$db->countRecords($this->tablename, 'MESSAGEID = ' . singleQuote($batch_id) . ' AND EMAIL_STATUS = ' . singleQuote('PRO'));
		  echoLog('Total emails to process: ' . $qty);
        }
        ob_flush();
        $sql = 'SELECT ID FROM ' . $this->tablename . ' WHERE MESSAGEID = ' . singleQuote($batch_id) . ' AND EMAIL_STATUS = ' . singleQuote('PRO');
        $rs = ClxCommon::$db->executeQuery($sql);
        $i = 0;
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  ClxFreeReturn($this->sendMail($r['ID'], true));
		  ClxCommon::$db->commit();
		  $i++;
		  if ($i === 50) {
			 sleep(1);
			 $i = 0;
		  }
        }
		ClxCommon::$db->freeResult($rs);
        if (!empty($batch)) {
		  $qty = ClxCommon::$db->countRecords($this->tablename, 'MESSAGEID = ' . singleQuote($batch_id) . ' AND EMAIL_STATUS = ' . singleQuote('ERR'));
		  echoLog('Total emails with sent error: ' . $qty);
		  echoLog('Mail processing completed');
		  echoLog('');
        }
    }

    function runMessageQueueCron() {
        ClxCommon::$db->commit();
        $this->runMessageQueue(true);
    }

    var $IMAP_uri;
    var $IMAP_mbox;
    var $IMAP_uri_folder;
    var $IMAP_folders;
    var $IMAP_headers;
    var $IMAP_header;
    var $IMAP_mail_account;
    var $IMAP_errormsg = '';
    
    function IMAP_connect($mail_account, $folder = '') {
        $this->IMAP_errormsg = '';
        $this->IMAP_mail_account = $mail_account;
        /*
        $this->IMAP_mail_account['EMAIL_TYPE'];    // "POP3" => "GEN_LBL_POP3", "IMAP" => "GEN_LBL_IMAP"
        $this->IMAP_mail_account['IN_HOSTNAME'];
        $this->IMAP_mail_account['IN_PORT'];
        $this->IMAP_mail_account['IN_USERNAME'];
        $this->IMAP_mail_account['IN_PASSWORD'];
        $this->IMAP_mail_account['IN_DELETEMSG'];
        */
        $this->IMAP_uri_folder = $folder;
        if($this->IMAP_uri_folder === '') {
		  $this->IMAP_uri_folder = 'INBOX';
        }
        $this->IMAP_uri = '{' . $this->IMAP_mail_account['IN_HOSTNAME'] . ':' . $this->IMAP_mail_account['IN_PORT'] . '}';
        $this->IMAP_mbox = imap_open($this->IMAP_uri.$this->IMAP_uri_folder, $this->IMAP_mail_account['IN_USERNAME'], $this->IMAP_mail_account['IN_PASSWORD']);
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
        }
    }
    
    function IMAP_mbox_error($msg = null) {
        if(is_null($msg)) {
		  $msg = dict("BASE_MAIL_ERROR_CANT_CONNECT", $this->IMAP_mail_account["IDNAME"] . ' -> ' . $this->IMAP_mail_account["IN_HOSTNAME"]) . ' [IMAP-ER-010]';
        }
        $this->IMAP_errormsg = $msg;
    }
    
    function IMAP_get_folders($wildcard = '*') {
        $this->IMAP_errormsg = '';
        $options = array();
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return $options;
        }
        $this->IMAP_folders = imap_listmailbox($this->IMAP_mbox, $this->IMAP_uri, $wildcard);
        if ($this->IMAP_folders === false) {
		  $this->IMAP_errormsg = dict("GEN_MSG_TRANSMITION_FAIL", $this->IMAP_mail_account["IDNAME"] . ' -> ' . $this->IMAP_mail_account["IN_HOSTNAME"]) . ' [IMAP-ER-020]';
		  return $options;
        }
        foreach ($this->IMAP_folders as $val) {
		  $folder_name = str_replace($this->IMAP_uri, '', $val);
		  $options[$folder_name] = $folder_name;
        }
        return $options;
    }

    function IMAP_get_headers() {
        $this->IMAP_errormsg = '';
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return false;
        }
        $this->IMAP_headers = imap_headers($this->IMAP_mbox);
        if ($this->IMAP_headers == false) {
		  $this->IMAP_errormsg = dict("GEN_MSG_TRANSMITION_FAIL", $this->IMAP_mail_account["IDNAME"] . ' -> ' . $this->IMAP_mail_account["IN_HOSTNAME"]) . ' [IMAP-ER-030]';
		  return false;
        }
        return true;
    }

    function IMAP_get_numMessages() {
        $this->IMAP_errormsg = '';
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return false;
        }
        $this->IMAP_numMessages = imap_num_msg($this->IMAP_mbox);
        return $this->IMAP_numMessages;
    }

    function IMAP_get_uid($i) {
        $this->IMAP_errormsg = '';
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return false;
        }
        $uid = imap_uid($this->IMAP_mbox, $i);
        return $uid;
    }
    
    function IMAP_get_message($i) {
        $this->IMAP_errormsg = '';
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return false;
        }
        $this->IMAP_header = imap_header($this->IMAP_mbox, $i);
        $fromInfo = $this->IMAP_header->from[CLX_NUM_ZERO];
        $replyInfo = $this->IMAP_header->reply_to[CLX_NUM_ZERO];
        $details = array(
		  'fromAddr' => (isset($fromInfo->mailbox) && isset($fromInfo->host)) ? $fromInfo->mailbox . '@' . $fromInfo->host : '',
		  'fromName' => (isset($fromInfo->personal)) ? $fromInfo->personal : '',
		  'replyAddr' => (isset($replyInfo->mailbox) && isset($replyInfo->host)) ? $replyInfo->mailbox . '@' . $replyInfo->host : '',
		  'replyName' => (isset($replyTo->personal)) ? $replyto->personal : '',
		  'subject' => (isset($this->IMAP_header->subject)) ? $this->IMAP_header->subject : '',
		  'udate' => (isset($this->IMAP_header->udate)) ? $this->IMAP_header->udate : ''
        );
        return $details;
    }

    function IMAP_get_body($uid) {
        $body = $this->IMAP_get_part($uid, 'TEXT/HTML');
        // if HTML body is empty, try getting text body
        if ($body === '') {
		  $body = $this->IMAP_get_part($uid, 'TEXT/PLAIN');
        }
        return $body;
    }

    function IMAP_get_part($uid, $mimetype, $structure = false, $partNumber = false) {
        $this->IMAP_errormsg = '';
        if(!$this->IMAP_mbox) {
		  $this->IMAP_mbox_error();
		  return false;
        }
        if (!$structure) {
		  $structure = imap_fetchstructure($this->IMAP_mbox, $uid, FT_UID);
        }
        if ($structure) {
		  if ($mimetype == $this->IMAP_get_mime_type($structure)) {
			 if (!$partNumber) {
			     $partNumber = CLX_NUM_ONE;
			 }
			 $text = imap_fetchbody($this->IMAP_mbox, $uid, $partNumber, FT_UID);
			 switch ($structure->encoding) {
			     case CLX_NUM_THREE: return imap_base64($text);
			     case CLX_NUM_FOUR: return imap_qprint($text);
			     default: return $text;
			 }
		  }

		  // multipart 
		  if ($structure->type === CLX_NUM_ONE) {
			 foreach ($structure->parts as $index => $subStruct) {
			     $prefix = '';
			     if ($partNumber) {
			         $prefix = $partNumber . '.';
			     }
			     $data = $this->IMAP_get_part($uid, $mimetype, $subStruct, $prefix . ($index + CLX_NUM_ONE));
			     if ($data) {
			         return $data;
			     }
			 }
		  }
        }
        return false;
    }

    function IMAP_get_mime_type($structure) {
        $primaryMimetype = array('TEXT', 'MULTIPART', 'MESSAGE', 'APPLICATION', 'AUDIO', 'IMAGE', 'VIDEO', 'OTHER');
        if ($structure->subtype) {
		  return $primaryMimetype[(int) $structure->type] . '/' . $structure->subtype;
        }
        return 'TEXT/PLAIN';
    }

    function IMAP_disconnect() {
        if($this->IMAP_mbox) {
		  imap_close($this->IMAP_mbox);
		  $this->IMAP_mbox = false;
		  $this->IMAP_mail_account = null;
        }
    }
    
    function parseTextForEmail($text, $doNotIncludeThisAdresses = null) {
        $email = array();
        $text = ereg_replace('[^A-Za-z._0-9@ ]', ' ', $text);
        $token = trim(strtok($text, ' '));
        while ($token !== '') {
		  if (strpos($token, '@') !== false) {
			 $token = ereg_replace('[^A-Za-z._0-9@]', '', $token);
			 $token = lowercase($token);
			 if(is_null($doNotIncludeThisAdresses)) {
			     $email[] = $token;
			 } elseif(array_search($token, $doNotIncludeThisAdresses) === false ) {
			     $email[] = $token;
			 }
		  }
		  $token = trim(strtok(' '));
        }
        $email = array_unique($email);
        return $email;
    }
}
