<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-31 12:15:06 +0000 (Thu, 31 Jan 2019) $
 * @version        $Rev: 10514 $
 *
 * CHANGE LOG
 *  Included label
 */
class manager extends clx_tool {

    function __construct() {
        $this->code = 'MAGR';
        $this->name = 'Tool Manager';
        $this->system = 'System';
        $this->version = '0.4.4.0';

        $this->has_access = true;


        $this->tablename = 'CLX_TOOLS';
        $this->record_name_singular = 'tool';
        $this->record_name_plural = 'tools';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->viewform = array(CLX_CLASS_TITLE => 'TOOL_MAGR_TITLE_VIEW',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE', 'fields' => array(array('name' => 'idcode', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_NAME', 'fields' => array(array('name' => 'idname', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_VERSION', 'fields' => array(array('name' => 'version', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_SYSTEM', 'fields' => array(array('name' => 'system', 'type' => 'string'))),
			         array('label' => 'GEN_LBL_CLASS', 'fields' => array(array('name' => CLX_PROP_CLASS, 'type' => 'string'))),
			         array('label' => 'GEN_LBL_IDATE', 'fields' => array(array('name' => 'idate', 'type' => 'string'))
			         )
			     )
			 )
		  )
        );
    }

    function install() {

        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEM:MENU_SYSTOOLS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['SYSTEM'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['CLASS'] = array('type' => 'varchar(255)', 'null' => 'NO', 'properties' => '');
        $fields['VERSION'] = array('type' => 'varchar(32)', 'null' => 'NO', 'properties' => '');
        $fields['IDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['HAS_PREFERENCES'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO, 'properties' => '');
        $fields['HAS_PARAMETERS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO, 'properties' => '');
        $fields['HAS_ACCESS'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO, 'properties' => '');
        $fields['HAS_PACKAGE'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO, 'properties' => '');
        $fields['HAS_PAGELET'] = array('type' => 'tinyint(1) unsigned', 'null' => 'NO', 'default' => CLX_NUM_ZERO, 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'SYSTEM');
        $indexes[$ipf . '04'] = array('index' => 'UNI', 'fieldname' => 'CLASS');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'MyISAM',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function action($ac, $idlist) {
        $contents = '';
        if ($this->getAccess()->canRead()) {
		  switch ($ac) {
			 case 'INSTALL':
			     if ($this->getAccess()->canDisplay()) {
			         $newtools = $this->scantools('');
			         if (count($newtools) > CLX_NUM_ZERO) {
					   $list = array();
					   reset($newtools);
					   foreach ($newtools as $newtoolid => $toolcode) {
						  ClxCommon::$tools[$toolcode]->install();
						  $list[] = ClxCommon::$tools[$toolcode]->name;
						  unset($newtoolid);
					   }
					   $contents = note(dict('TOOL_MAGR_NEWTOOLS') . ':<br>' . implode('<br>', $list), 'Return');
			         } else {
					   $contents = note(dict('TOOL_MAGR_NONEWTOOLTOINSTALL'), 'Return');
			         }
			     } else {
			         $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MANAGER-CAN_APPEND][' . $ac . '] ' . $this->getCode(false));
			     }
			     break;
			 case 'LIST_TOOL_TABLES':
			     $contents = $this->listToolTables();
			     break;
			 case 'CHECK':
			     $contents = $this->checkTableForm($idlist, false);
			     break;
			 case 'FIX':
			     //   - Audit dump to .SQL Script file instead of excecuting SQL senences directly
			     $dump = (defined('UTIL_DUMP_AUPK_SQL_SCRIPT') && UTIL_DUMP_AUPK_SQL_SCRIPT === strval(CLX_NUM_ONE));
			     $flmsg = '';
			     if (!empty($dump)) {
			         ClxCommon::$db->scriptFileOut = kPathToTemp . clx_mcript('FORM_TOOL_FIX') . '.sql';
			         $flmsg = '<br>' . dict('GEN_LBL_TO') . ' <a href=' . singleQuote(clxCommon::$db->scriptFileOut) . ' target=_blank>' . ClxCommon::$db->scriptFileOut . '</a>';
			     }
			     $contents = $this->checkTableForm($idlist, true, $dump);
			     //   - Audit dump to .SQL Script file instead of excecuting SQL senences directly
			     if (!empty($dump)) {
			         $contents .= '<br><br>' . note(dict('GEL_LBL_OPER_COMP_SUCCESS') . ' ' . $flmsg, 'OK');
			     }
			     break;
			 default:
			     $contents = parent::action($ac, $idlist);
		  }
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [MANAGER-CAN_READ][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        $yesno = array(dict('GEN_LBL_NO'), dict('GEN_LBL_YES'));
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'MAGRList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = $this->tablename;
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_7);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MNGR_LIST_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_7));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_SYSTEM'), dict('GEN_LBL_IDATE'), dict('GEN_LBL_ACCESS')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME'], $record['VERSION'], $record['SYSTEM'], xFormat::date($record['IDATE'],$_SESSION['prefs']['DATE_FORMAT'].' h:i:s'), $yesno[$record['HAS_ACCESS']]));
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('TOOL_MNGR_BUTTON_INSTALL'), 'INSTALL', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $b = new htmlFormSubmit(dict('TOOL_MNGR_BUTTON_LIST_TOOL_TABLES'), 'LIST_TOOL_TABLES', $form->getProp(CLX_PROP_ID));
		  $b->setProp(CLX_PROP_ID, 'ON');
		  $buttons .= $b->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        $sqlstatement = $page->getProp('statement');
        $records = ClxCommon::$db->executeQuery($sqlstatement);
        $totals = array(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO);
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $pieces = explode('.', $record['VERSION']);
		  for ($i = CLX_INIT_ZERO; $i < CLX_NUM_FOUR; $i++) {
			 $totals[$i]+= $pieces[$i];
		  }
        }
		ClxCommon::$db->freeResult($records);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MNGR_STATISTICS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('TOOL_MNGR_VERSION_TOTAL'), $totals[CLX_NUM_ZERO] . '.' . $totals[CLX_NUM_ONE] . '.' . $totals[CLX_NUM_TWO] . '.' . $totals[CLX_NUM_THREE]);
        return $form->source() . $table->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MAGR_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'MAGR_idcode', (!empty(UserRequest::$post['MAGR_idcode'])) ? UserRequest::$post['MAGR_idcode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'MAGR_idname', (!empty(UserRequest::$post['MAGR_idname'])) ? UserRequest::$post['MAGR_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons =htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['MAGR_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['MAGR_idcode']) . '%');
		  }
		  if (UserRequest::$post['MAGR_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['MAGR_idname'])) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=manager></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult($records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('MAGR_idcode', UserRequest::$post))
		  UserRequest::$post['MAGR_idcode'] = '';
        if (!array_key_exists('MAGR_idname', UserRequest::$post))
		  UserRequest::$post['MAGR_idname'] = '';
        if (!array_key_exists('MAGR_system', UserRequest::$post))
		  UserRequest::$post['MAGR_system'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'MAGR_idcode', UserRequest::$post['MAGR_idcode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'MAGR_idname', UserRequest::$post['MAGR_idname'], CLX_NUM_32, CLX_NUM_32);
        $table->addField($msg, dict('GEN_LBL_SYSTEM') . ':', 'MAGR_system', UserRequest::$post['MAGR_system'], CLX_NUM_32, CLX_NUM_32);
        $sort = new HtmlSelect('MAGR_sort');
        $sort->addItem('', '', true);
        $sort->addItem(dict('TOOL_MAGR_IDCODE'), 'IDCODE', false);
        $sort->addItem(dict('TOOL_MAGR_IDNAME'), 'IDNAME', false);
        $sort->addItem(dict('TOOL_MAGR_SYSTEM'), 'SYSTEM', false);
        $sort->addItem(dict('TOOL_MAGR_INSTDATEASC'), 'IDATE ASC', false);
        $sort->addItem(dict('TOOL_MAGR_INSTDATEDESC'), 'IDATE DESC', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['MAGR_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['MAGR_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['MAGR_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['MAGR_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['MAGR_system'])) {
		  $filter .= $boolean . ' SYSTEM LIKE ' . singleQuote(UserRequest::$post['MAGR_system'] . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['MAGRList']['sortby'] = UserRequest::$post['MAGR_sort'];
        $_SESSION['paging']['MAGRList']['criteria'] = $filter;
        $_SESSION['paging']['MAGRList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['MAGRList']['criteria'] = '';
    }*/

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['MAGR_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['MAGR_idname']);
    }
	function change_case_null_default_case_not_null_field($structure) {
		$def= get_defined_constants(true);
		switch (kDatabaseType) {
				case 'oci':
					$bdtype="ORACLE";
					$size=11;
					break;
				default:
					$bdtype="MYSQL";
					$size=10;
					break;
		}
		$defaults=array();
		foreach($def['user'] as $key=>$value) {
			if (!empty($value)) { 
				if (substr($key,0,$size)=='DBTB_'.$bdtype) {
					$splt=explode("_",$key);
					$defaults[$splt[6]]=$value;
				}
			}
		}	
		$structure2=$structure;
		if (isset(ClxCommon::$db->debugCF) && ClxCommon::$db->debugCF==true){
			pre("OLD default BY PHP");
			pre($structure2);
			echo "-----------------";
		}
		if (isset($structure2['fields']) && is_array($structure2['fields'])) {
			foreach($structure2['fields'] as $key=>&$value) {
				if($key=='ID')
					continue;
				$predefaults=array('CHAR'=>'','INT'=>0,'TEXT'=>'','DATE'=>'1970/01/01' ,'TIMESTAMP'=>'1970/01/01');
				if (!isset($value['null']) || $value['null']=='NO') {
					foreach($defaults as $key2=>$value2) {
						if (stripos(" ".$value['type'],$key2)>0) {
							if ($key2=='CHAR' or $key2=='INT' or $key2=='TEXT'or $key2=='DATE') {
								if (!isset($value['default']))
									$value['default']=$predefaults[$key2];
								if ($value['default']=='' or $value['default']=='null') {
									if ($value=='[ROWID]') {
										$value['default']='ROWID';
								
									}	
									else if ($value=='[TIMESTAMP]') {
										IF ($bdtype=='ORACLE')
											$value['default']='CURRENT_TIMESTAMP';
										else
											$value['default']='CURRENT_TIMESTAMP';
									}
									else if ($value=='[DATE]') {
										IF ($bdtype=='ORACLE')
											$value['default']='CURRENT_DATE';
										else
											$value['default']='CURRENT_DATE';
									}
									else if ($value2=='[SPACE]') { 
										$value['default']=singlequote(' ');
									}	
									else {
										$value['default']=singlequote($value2);
									}
								}
							}
	
						}	
					}	
				}	
			}
		}
		if (isset(ClxCommon::$db->debugCF) && ClxCommon::$db->debugCF==true){
			pre("nuevos default CHANGE NULL");
			pre($structure2);
		}
		$structure=$structure2;
		return $structure;
	}	
    function viewFormFill($record) {		if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+"); 	fwrite($tta,chr(13).chr(10)."1197-manager-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,chr(13).chr(10)."record: ".var_export($record,true).chr(13).chr(10));	$te=debug_backtrace(); for ($t=0;$t<12;$t++) {	if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));}fclose($tta);}	
        $contents = '';
        $tool = &getTool($record['MAGR_idcode']);
		
        $structure = toupperfields($tool->toolTableStructure());
		$structure=$this->change_case_null_default_case_not_null_field($structure);
		
        // Table Structure
		$table = new htmlTable(CLX_NUM_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($structure)) {
		  $table->addCell($structure['name'] . ' (' . $structure['type'] . ')', array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_NUM_FOUR));
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_FIELD'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_null'), dict('GEN_LBL_PROPERTIES')));
		  $i = CLX_INIT_ZERO;
		  if (empty($structure['fields']))
			 $structure['fields'] = array();
		  reset($structure['fields']);
		  foreach ($structure['fields'] as $fieldname => $field) {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($fieldname, $field['type'], (empty($field['null']) || $field['null'] === 'NO') ? dict('GEN_LBL_NO') : dict('GEN_LBL_YES'), (array_key_exists('properties', $field)) ? $field['properties'] : ''));
			 $i++;
		  }
        } else {
		  $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_NUM_FOUR));
		  $table->addCell(dict('GEN_LBL_NOTABLE'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_FOUR));
        }
        $contents .= buildPanelInterface(dict('TOOL_MAGR_TITLE_LIST'), 'tolldbstructure', $table->source(), CLX_NUM_MINUS_1);
        $DBTI = getTool('DBTI');
        //Table indexes
        $i = CLX_INIT_ZERO;
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($structure)) {
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_CODE'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_FIELDNAME')));
		  foreach ($structure['indexes'] as $key => $val) {
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($key, dict($DBTI->indexes[$val['index']]), $val['fieldname']));
			 $i++;
		  }
        }
        $contents .= buildPanelInterface(dict('TOOL_MAGR_TABLE_INDEXES'), 'tolldbindexes', $table->source(), CLX_NUM_MINUS_1);
        // Properties

        $table = new htmlTable(CLX_NUM_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        if (!empty($structure)) {
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_PROPERTY'), dict('GEN_LBL_VALUE')));
		  $i = CLX_INIT_ZERO;
		  $tableinfo = ClxCommon::$db->getTableInfo($tool->tablename);
		  reset($tableinfo);
		  foreach ($tableinfo as $prop => $value) {
			 if (empty($value))
			     continue;
			 if (is_array($value)) {
			     $table2 = new htmlTable(CLX_NUM_TWO);
			     $table2->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
			     foreach ($value as $k => $v) {
			         $table2->addData($k, $v);
			     }
			     $table->addData($prop, $table2->source());
			 } else {
			     $table->addData($prop, $value);
			 }
			 $i++;
		  }
        } else {
		  $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_NUM_TWO));
		  $table->addCell(dict('GEN_LBL_NOPROPERTIES'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_NUM_TWO));
        }
        $contents .= buildPanelInterface(dict('GEN_LBL_PROPERTIES'), 'tolldbprops', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function viewFormButtons($form) {
        $buttons = '';
        if ($this->getAccess()->getIsadmin()) {
		  $c = new htmlFormSubmit(dict('GEN_LBL_CHECK_TABLE'), 'CHECK', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
		  $c = new htmlFormSubmit(dict('GEN_LBL_FIX_TABLE'), 'FIX', $form->getProp(CLX_PROP_ID));
		  $buttons .= $c->source();
        }
        return $buttons;
    }

    function checkTableForm($idlist, $fix, $dump = false) {
        //$toolrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $idlist[CLX_NUM_ZERO]);
        $idlst_ndx = count($idlist)-1;
        $toolrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $idlist[$idlst_ndx]);
        $contents = '';
        $htmltable = new htmlTable(CLX_WIDTH_TWO);
        $htmltable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $htmltable->addCell(dict('GEN_LBL_RESULTS'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $htmltable->addData(dict('GEN_LBL_TOOLCODE'), $toolrec['IDCODE']);
        $htmltable->addData(dict('GEN_LBL_TOOLNAME'), $toolrec['IDNAME']);
        $contents .= $htmltable->source();

        $htmltable = new htmlTable(CLX_WIDTH_FOUR);
        $htmltable->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $htmltable->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_TABLE'), dict('GEN_LBL_FIELD'), dict('GEN_LBL_ERROR'), dict('GEN_LBL_COMMENTS')));
        $msg = $this->checkTable($toolrec, $fix, false, false, $dump);
        $i = CLX_INIT_ZERO;
        if (count($msg) == CLX_NUM_ZERO) {
		  $htmltable->addCell(dict('GEN_LBL_NOERROR_FOUND'), array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        } else {
		  reset($msg);
		  foreach ($msg as $key => $line) {
			 $htmltable->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($line['tablename'], $line['fieldname'], $line['error'], $line['comments']));
		  }
        }
        $buttons = '';
        $c = new HTMLButtonCancel(dict('GEN_LBL_RETURN'));
        $buttons .= $c->source();
        $htmltable->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $contents .= $htmltable->source();
        return $contents;
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $recordid);
        $record = array();
        $record['MAGR_id'] = $recordid;
        $record['MAGR_idcode'] = $values['IDCODE'];
        $record['MAGR_idname'] = $values['IDNAME'];
        $record['MAGR_system'] = $values['SYSTEM'];
        $record['MAGR_class'] = $values['CLASS'];
        $record['MAGR_version'] = $values['VERSION'];
        $record['MAGR_idate'] = xFormat::date($values['IDATE']);
        return $record;
    }

    function deleteRecordSk($id) {
        $toolrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        $toolcode = $toolrec['IDCODE'];
        $class = $toolrec['CLASS'];
        include_once kPathToWebSite . kPathToTools . $class . '.php';
        $tool = new $class;
        if (!empty($tool->tablename))
		  ClxCommon::$db->droptable($tool->tablename);
        ClxCommon::$db->deleteRecords('CLX_UI_MENUITEMS', 'TYPE = ' . singleQuote('TOO') . ' AND EXTRA1=' . singleQuote($toolcode));
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    /**
     * Scan and install tools
     * @param type $sourcepath
     * @return array
     */
    private function scantools($sourcepath = '') {

        $newtools = array();

        $pathname = ($sourcepath == '') ? (kPathToWebSite . kPathToTools) : $sourcepath;
        if (is_dir($pathname)) {
		  $dir = opendir($pathname);
		  while ($file = readdir($dir)) {
			 if (!is_dir($pathname . $file) && ($file !== '.') && ($file !== '..') && stristr($file, '.php')) {

			     $pos = strpos($file, '.');
			     $class = substr($file, CLX_NUM_ZERO, $pos);
			     require_once $pathname . $file;
			     if (!class_exists($class)) {
			         addMessage('Class not exists or fail include: ' . $class);
			         continue;
			     }
			     $tool = new $class;
                    if (ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($tool->code)) === CLX_NUM_ZERO) {
			         $this->installtool($class, $tool);
			         $newtools[] = $tool->code;
			     } else {
			         $this->updatetool($class, $tool);
			     }
			 }
		  }
		  closedir($dir);
        } else {
		  addMessage('Can\'t access the tools directory: ' . $pathname);
        }
        return $newtools;
    }

    /**
     * Scan Tool for bath installer.
     */
    function scantoolsinstaller($pathname) {
        return $this->scantools($pathname);
    }

    function copyToolData($toolcode, $fromdbhost, $fromdbuser, $fromdbpassword) {

    }

    function installtool($toolclass, $toolinstance) {
        $this->savetool($toolclass, $toolinstance);
        ClxCommon::$tools[$toolinstance->code] = $toolinstance;
        $toolinstance->createToolTable();
        //Added during oracle. to fix indexes
        ClxFreeReturn($this->checkTable(array('IDCODE' => $toolinstance->code), true));
    }

    function savetool($toolclass, $toolinstance) {
        $record = array();
        $record['IDCODE'] = $toolinstance->code;
        $record['IDNAME'] = $toolinstance->name;
        $record['SYSTEM'] = $toolinstance->system;
        $record['HAS_PREFERENCES'] = ($toolinstance->has_preferences) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['HAS_ACCESS'] = ($toolinstance->has_access) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['HAS_PACKAGE'] = ($toolinstance->has_package) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['HAS_PAGELET'] = ($toolinstance->has_pagelet) ? CLX_NUM_ONE : CLX_NUM_ZERO;
        $record['CLASS'] = $toolclass;
        $record['VERSION'] = $toolinstance->version;
        $record['IDATE'] = xFormat::getNow('Y-m-d H:i:s');
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function updatetool($toolclass, $toolinstance) {

        $record = ClxCommon::$db->getRecord($this->tablename, 'IDCODE=' . singleQuote($toolinstance->code));
        if ($record['VERSION'] != $toolinstance->version) {
		  $record = array();
		  $record['IDCODE'] = $toolinstance->code;
		  $record['IDNAME'] = $toolinstance->name;
		  $record['SYSTEM'] = $toolinstance->system;
		  $record['HAS_PREFERENCES'] = ($toolinstance->has_preferences) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  $record['HAS_PARAMETERS'] = ($toolinstance->has_parameters) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  $record['HAS_ACCESS'] = ($toolinstance->has_access) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  $record['HAS_PACKAGE'] = ($toolinstance->has_package) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  $record['HAS_PAGELET'] = ($toolinstance->has_pagelet) ? CLX_NUM_ONE : CLX_NUM_ZERO;
		  $record['CLASS'] = $toolclass;
		  $record['VERSION'] = $toolinstance->version;
		  $record['IDATE'] = xFormat::getNow('Y-m-d H:i:s');
		  ClxCommon::$db->updateRecord($this->tablename, $record, 'IDCODE=' . singleQuote($toolinstance->code));
        }
    }

    function placeChooseTool($msg, $prompt, $name, $default, &$table) {
        $searchIcon = getSkinIcon('view', 'javascript:openWindow("", "pop.php?to=MAGR&ac=SEARCH&target=' . $name . ', 500,300,null,null,'.singlequote( str_replace('\\','/',__FILE__).'_'. __LINE__ ).');', 'Search for a tool');
        if ($msg[$name] <> '') {
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        }
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $c = new htmlField($name, $default, CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addCell($c->source() . $searchIcon, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

    function toolPop($name, $default) {
        $pop = new HTMLSelect($name);
        $records = ClxCommon::$db->getRecords($this->tablename, '1=1 ORDER BY IDNAME');
        while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
		  $pop->addItem($record['IDNAME'] . ' (' . $record['IDCODE'] . ')', $record['IDCODE'], ($record['IDCODE'] == $default));
        }
		ClxCommon::$db->freeResult($records);
        return $pop;
    }

    /**
     * Verify if tools exists
     * Return Boolean
     */
    function toolExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, 'IDCODE=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, 'IDCODE', 'IDCODE=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    //function checkTable($toolrec, $fix, $fields_only=false, $indexes_only=false) {
    function checkTable($toolrec, $fix, $fields_only = false, $indexes_only = false, $dump = false,$audit=false) {
        $tool = &getTool($toolrec['IDCODE']);
        if(!$tool == NULL){
            $structure=toupperfields($tool->toolTableStructure());
            $structure=$this->change_case_null_default_case_not_null_field($structure);
            $structure = ClxCommon::$db->translateTableStructure($structure);
            $msg = array();
            //pre($structure);
            ClxCommon::$db->checkTable($structure, $fix, $msg, $fields_only, $indexes_only, $dump,$audit);
        }

        return $msg;
    }

    function listToolTables() {
        $contents = '<textarea rows="10" cols="100">' . implode(CLX_CHAR_NL, $this->getListToolTables()) . '</textarea>';
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_MNGR_LIST_TITLE'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_ONE));
        $table->addCell($contents, array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return $table->source();
    }

    /**
     * Returns array with all tables from tools
     */
    function getListToolTables() {
        $tables = array();
        $rs = ClxCommon::$db->getRecords($this->tablename, '1=1 ORDER BY IDNAME');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
		  $tool = &getTool($r['IDCODE']);
		  if ($tool->tablename <> '') {
			 $tables[$r['IDCODE']] = $tool->tablename;
		  }
        }
		ClxCommon::$db->freeResult($rs);
        return $tables;
    }

}
