<?php

/**
 * Tool Class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  Included label
 */
class sd_mimegroups extends clx_tool {

    // CONSTRUCTOR //
    function __construct() {
        $this->code = 'SDMG';
        $this->name = 'MIME Groups Manager';
        $this->system = 'System Data';
        $this->version = '0.1.2.0';

        $this->has_access = true;


        $this->tablename = 'CLX_SD_MIMEGROUPS';
        $this->record_name_singular = 'MIME Group';
        $this->record_name_plural = 'MIME Groups';
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->editform = array(CLX_CLASS_TITLE => 'TOOL_SDMG_TITLE_CREATE',
		  'panels' => array(
			 array('type' => 'normal',
			     'entries' => array(
			         array('label' => 'GEN_LBL_CODE',
					   'fields' => array(
						  array('name' => 'idcode', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '32', 'dontmodify' => '1',
						      'validations' => array(
								array('type' => 'sqlobject', 'required' => '1'), array('type' => 'unique', 'fieldname' => 'idcode', 'tablename' => $this->tablename))
						  )
					   )
			         ), array('label' => 'GEN_LBL_NAME',
					   'fields' => array(
						  array('name' => 'idname', 'type' => 'string', 'default' => '', 'size' => '64', 'displaysize' => '48', 'validations' => array(
								array('type' => 'string', 'required' => '1'))
						  )
					   )
			         )
			     )
			 ), array('type' => 'callback',
			     'function' => 'editGroupMembers'
			 )
		  )
        );
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_SYSTEMDATA:MENU_MIMEGROUPS', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

//-- M I M E    T Y P E S
    function makelist($listingcode = null) {
	ClxFreeReturn($listingcode);
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'SDMGList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_SD_MIMEGROUPS';
        $whereclause = '';
        $sortclause = 'IDNAME';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_THREE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDMG_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_THREE));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['IDCODE'], $record['IDNAME']));
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_MIMEGROUP')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDMG_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'SDMG_idcode', (!empty(UserRequest::$post['SDMG_idcode'])) ? UserRequest::$post['SDMG_idcode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'SDMG_idname', (!empty(UserRequest::$post['SDMG_idname'])) ? UserRequest::$post['SDMG_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['SDMG_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['SDMG_idcode']) . '%');
		  }
		  if (UserRequest::$post['SDMG_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['SDMG_idname'])) . '%');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while (($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE))) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=sd_mimegroups></page><a href="' . $js . '">' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			  ClxCommon::$db->freeResult( $records);
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['SDMG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['SDMG_idname']);
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDMG_TITLE_VIEW') . ': ' . $record['SDMG_idname'], array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['SDMG_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['SDMG_idname']);
        $contents .= $table->source();
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('SDMG_idcode', UserRequest::$post))
		  UserRequest::$post['SDMG_idcode'] = '';
        if (!array_key_exists('SDMG_idname', UserRequest::$post))
		  UserRequest::$post['SDMG_idname'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE'), 'SDMG_idcode', UserRequest::$post['SDMG_idcode'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_NAME'), 'SDMG_idname', UserRequest::$post['SDMG_idname'], CLX_NUM_32, CLX_NUM_64);
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['SDMG_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['SDMG_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['SDMG_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase(clx_change_especial_char_html (UserRequest::$post['SDMG_idname'])) . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['SDMGList']['criteria'] = $filter;
        $_SESSION['paging']['SDMGList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['SDMGList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['SDMG_id'] = $recordid;
        $record['SDMG_idcode'] = $values['IDCODE'];
        $record['SDMG_idname'] = $values['IDNAME'];
        return $record;
    }

    function saveRecord() {
        $record = array();
        $record['IDNAME'] = UserRequest::$post['SDMG_idname'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['SDMG_id']);
        $this->saveMembers(UserRequest::$post['SDMG_idcode']);
    }

    function createRecord_() {
        $record = array();
        $record['IDCODE'] = UserRequest::$post['SDMG_idcode'];
        $record['IDNAME'] = UserRequest::$post['SDMG_idname'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $this->saveMembers(UserRequest::$post['SDMG_idcode']);
        return $recordid;
    }

    function saveMembers($groupcode) {
        ClxCommon::$db->deleteRecords('CLX_SD_MIMEGROUP_MEMBERS', 'GROUPCODE=' . singleQuote($groupcode));
        $record = array();
        $record['GROUPCODE'] = $groupcode;
        foreach (UserRequest::$request as $key => $value) {
		  if (substr($key, CLX_NUM_ZERO, CLX_NUM_FOUR) === 'XXX_') {
			 $id = substr($key, CLX_NUM_FOUR);
			 $record['MIMECODE'] = ClxCommon::$db->getField('CLX_SD_MIMETYPES', 'IDCODE', 'ID=' . $id);
			 ClxCommon::$db->createRecord('CLX_SD_MIMEGROUP_MEMBERS', $record, false);
		  }
        }
    }

    function deleteRecordSk($id) {
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
    }

    function defaultRecord() {
        $record = array();
        $record['SDMG_id'] = CLX_INIT_ZERO;
        $record['SDMG_idcode'] = '';
        $record['SDMG_idname'] = '';
        return $record;
    }

// DEPRECATED in favor of chooseRecord($name, $default)
    function placeChooseMimeGroup($msg, $prompt, $name, $default, &$table) {
        if (!empty($msg[$name]))
		  $table->addCell($msg[$name], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addCell($prompt, array(CLX_PROP_CLASS => CLX_CLASS_PROMPT));
        $table->addCell($this->chooseRecord($name, $default), array(CLX_PROP_CLASS => CLX_CLASS_DATA));
        return CLX_CHAR_EMPTY_STRING;
    }

// DEPRECATED in favor of recordExists($field_label, &$field_value, $msg)
    function MimeGroupExists(&$idcode) {
        $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $this->primary_field . '=' . singleQuote($idcode));
        if ($qty > CLX_NUM_ZERO) {
		  $idcode = ClxCommon::$db->getField($this->tablename, $this->primary_field, $this->primary_field . '=' . singleQuote($idcode));
        }
        return $qty > CLX_NUM_ZERO;
    }

    function editGroupMembers($record) {
        $table = new htmlTable(CLX_WIDTH_FOUR);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_SDMG_MEMBERS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_FOUR));
        $members = ClxCommon::$db->getRecords('CLX_SD_MIMETYPES', '1=1 ORDER BY IDCODE');
        $i = CLX_INIT_ZERO;
        while (($member = ClxCommon::$db->getNextRecord($members))) {
		  $status = (ClxCommon::$db->countRecords('CLX_SD_MIMEGROUP_MEMBERS', 'GROUPCODE=' . singleQuote($record['SDMG_idcode']) . ' AND MIMECODE=' . singleQuote($member['IDCODE'])) > CLX_NUM_ZERO);
		  $check = new htmlCheckbox('XXX_' . $member['ID'], $status);
		  $table->addCell($check->source() . $member['IDNAME'], array(CLX_PROP_CLASS => CLX_CLASS_DATA));
		  $i++;
        }
		ClxCommon::$db->freeResult( $members);
        $j = CLX_WIDTH_FOUR - ($i % CLX_WIDTH_FOUR);
        $table->addCell(CLX_CHAR_HTML_NBSP, array(CLX_PROP_CLASS => CLX_CLASS_DATA, CLX_PROP_COLSPAN => $j));
        return $table->source();
    }

}
