<?php

/**
 * Calendar manager
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-01-22 15:45:22 -0400 (ma., 22 ene. 2019) $
 * @version        $Rev: 10496 $
 *
 * CHANGE LOG
 *  Included label
 */
class ui_calendar extends clx_tool {

    var $ts = CLX_INIT_ZERO;   // Time Stamp
    var $cal_types = ''; // Calendar Type to display, cal only, listing + cal
    var $cal_views = ''; // Calendar Type to display, cal only, listing + cal
    var $viewType = '';  // User view type, month, yearly, etc
    var $description = array();
    var $listingQuery = array(); //Listing query
    var $listingCode = '';
    var $listingRecord = array();
    var $queryRecord = array();
    var $queryAccess = false;

    /**
     * Constructor
     */
    function __construct() {
        $this->code = 'UICA';
        $this->name = 'Calendar Manager';
        $this->system = 'Interface';
        $this->version = '4.0.2.2';

        $this->has_access = true;
        $this->has_package = true;
        $this->has_pagelet = true;
        $this->tablename = 'CLX_UI_CALENDAR';
        $this->record_name_singular = 'calendar';
        $this->record_name_plural = 'calendars';
        $this->primary_key = array('IDCODE');
        $this->primary_field = 'IDCODE';
        $this->secondary_field = 'IDNAME';

        $this->cal_types = array('CAL' => dict('GEN_LBL_CALENDAR_ONLY'), 'COMP' => dict('GEN_LBL_CALENDAR_AND_LISTING'));
        $this->cal_views = array('day' => dict('GEN_LBL_DAYVIEW'), 'week' => dict('GEN_LBL_WEEKVIEW'), 'work' => dict('GEN_LBL_WORKWEEKVIEW'), 'month' => dict('GEN_LBL_MONTHVIEW'), 'year' => dict('GEN_LBL_YEARVIEW'), CLX_CLASS_LIST => dict('GEN_LBL_LISTINGVIEW'));
        $this->cal_menu = array('1' => dict('GEN_LBL_YES'), '0' => dict('GEN_LBL_NO'));
    }

    function install() {
        $UIMI = getTool('UIMI');
        $UIMI->add($this->code, 'MENU_TOOLS:MENU_INTERFACE:MENU_CALENDAR', 'TOOL', $this->code, 'ac=LIST');
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['IDCODE'] = array('type' => 'varchar(16)', 'null' => 'NO', 'properties' => '');
        $fields['IDNAME'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['LISTINGCODE'] = array('type' => 'varchar(16)', 'null' => 'YES', 'properties' => '');
        $fields['TABLECODE'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['FIELDNAME'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['EXTRA_FIELD1'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['RECURRENCE_FIELD'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['TYPE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $fields['VIEWS'] = array('type' => 'varchar(10)', 'null' => 'NO', 'properties' => '');
        $fields['MENU'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['DISPLAYFIELD'] = array('type' => 'varchar(64)', 'null' => 'YES', 'properties' => '');
        $fields['VERSION'] = array('type' => 'tinyint(1)', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['VERSION_COUNTER'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['CDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['MDATE'] = array('type' => 'datetime', 'null' => 'YES', 'properties' => '');
        $fields['USERNAME'] = array('type' => 'varchar(30)', 'null' => 'NO', 'properties' => '');
        $fields['DESCRIPTION'] = array('type' => 'mediumtext', 'properties' => '');
        $fields['MODULECODE'] = array('type' => 'varchar(4)', 'null' => 'NO', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'UNI', 'fieldname' => 'IDCODE');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'IDNAME');
        $indexes[$ipf . '04'] = array('index' => 'MUL', 'fieldname' => 'LISTINGCODE');
        $indexes[$ipf . '05'] = array('index' => 'MUL', 'fieldname' => 'TYPE');
        $indexes[$ipf . '06'] = array('index' => 'MUL', 'fieldname' => 'MODULECODE');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

//-- C A L E N D A R
    function action($ac, $idlist) {
        if ($this->getAccess()->canRead()) {
		  $contents = parent::action($ac, $idlist);
        } else {
		  $contents = warning(dict('GEN_ERR_NO_ACCESS') . ' [UI_CALENDAR-CAN_READ_1][' . $ac . '] ' . $this->getCode(false));
        }
        return $contents;
    }

    function makelist($listingcode = null) {
         //$t = time('U'); genera un warning
        $t = time();
        $to = $this->code;
        $here = 'tool.php?to=' . $to . '&t=' . $t;
        clearHistory();
        pushHistory($this->code, 'LIST', array());
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->setProp(CLX_PROP_ID, 'UICAList');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'NOT_DEFINED');
        $selectclause = '*';
        $fromclause = 'CLX_UI_CALENDAR';
        $whereclause = '';
        $sortclause = '';
        $page = new xPaging();
        $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
        $table = new htmlTable(CLX_WIDTH_TEN);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UICA_TITLE_LIST'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TEN));
        $page->header($table);
        $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_MODULECODE'), dict('GEN_LBL_CODE'), dict('GEN_LBL_NAME'), dict('GEN_LBL_TYPE'), dict('GEN_LBL_VIEW'), dict('GEN_LBL_MENU'), dict('GEN_LBL_LISTINGCODE'), dict('GEN_LBL_VERSION'), dict('GEN_LBL_DESCRIPTION')));
        $i = CLX_INIT_ZERO;
        while ($record = $page->nextRecord()) {
		  $recordid = $record['ID'];
		  $select = new htmlCheckbox('selector_' . $recordid, false);
		  $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
		  $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
		  $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array(
			 $record['MODULECODE'], $record['IDCODE'], $record['IDNAME'], $this->cal_types[$record['TYPE']], (!empty($record['VIEWS'])) ? $this->cal_views[$record['VIEWS']] : '',
			 $this->cal_menu[$record['MENU']], $record['LISTINGCODE'], $record['VERSION'] . '-' . $record['VERSION_COUNTER'], xFormat::text($record['DESCRIPTION'])));
		  $i++;
        }
        $buttons = '';
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_DISPLAY'), array('action' => 'DISPLAY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        if ($this->getAccess()->canModify()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDelete()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DUPLICATE'), array('action' => 'DUPLICATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ONE'))->source();
        }
        if ($this->getAccess()->canDisplay()) {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE_SOMETHING', dict('GEN_LBL_CALENDAR')), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        }
        $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_FILTER'), array('action' => 'FILTER', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
        $form->addContents($page->footers($table, $buttons));
        return $form->source();
    }

    function searchForm() {
        $msg = array();
        $contents = '';
        // Search Form
        $form = new htmlForm(CLX_PROP_ACTION, UserRequest::$server['PHP_SELF']);
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', 'SEARCH');
        $form->addArgument('confirm', '1');
        $form->addArgument(CLX_PROP_TARGET, UserRequest::$request[CLX_PROP_TARGET]);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell(dict('TOOL_UICA_TITLE_SEARCH'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UICA_idcode', (!empty(UserRequest::$post['UICA_idcode'])) ? UserRequest::$post['UICA_idcode'] : '', CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UICA_idname', (!empty(UserRequest::$post['UICA_idname'])) ? UserRequest::$post['UICA_idname'] : '', CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UICA_modulecode', (!empty(UserRequest::$post['UICA_modulecode'])) ? UserRequest::$post['UICA_modulecode'] : '', CLX_NUM_FOUR, CLX_NUM_FOUR);
        $buttons = htmlButton::getButton('submit', dict('GEN_LBL_SEARCH'), array('action' => 'SEARCH', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCloseWindow(dict('GEN_LBL_CANCEL'));
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->setContents($table->source());
        $contents .= $form->source();
        // Search Procedure
        if (isset(UserRequest::$post['confirm'])) {
		  $table = new htmlTable(CLX_WIDTH_TWO);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $criteria = '';
		  if (UserRequest::$post['UICA_idcode'] <> '') {
			 $criteria .= 'UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UICA_idcode']) . '% ');
		  }
		  if (UserRequest::$post['UICA_idname'] <> '') {
			 $criteria .= 'UPPER(IDNAME) LIKE ' . singleQuote(uppercase(UserRequest::$post['UICA_idname']) . '% ');
		  }
		  if (UserRequest::$post['UICA_modulecode'] <> '') {
			 $criteria .= 'UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UICA_modulecode']) . '% ');
		  }
            $qty = ClxCommon::$db->oneExistsRecords($this->tablename, $criteria);
		  if ($qty == CLX_NUM_ZERO) {
			 $table->addCell(dict('TOOL_GEN_ERR_NOMATCH'), array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		  } else {
			 $table->addCell(dict('TOOL_GEN_MSG_FOUNDRECORDS'), array(CLX_PROP_CLASS => CLX_CLASS_SUBTITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 $records = ClxCommon::$db->getRecords($this->tablename, $criteria);
			 $i = CLX_INIT_ZERO;
			 while ($record = ClxCommon::$db->getNextRecord($records, ASSOCIATIVE)) {
			     $js = 'javascript:setLookupValue(' . quote(UserRequest::$request[CLX_PROP_TARGET]) . ', ' . quote($record['IDCODE']) . ');';
			     $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array('<page t=ui_calendar></page><a href=' . singleQuote($js) . '>' . $record['IDCODE'] . '</a>', $record['IDNAME']));
			     $i++;
			 }
			 ClxCommon::$db->freeResult( $records );
		  }
		  $contents .= $table->source();
        }
        return $contents;
    }

    function editForm($ac, $idlist, $record, $msg) {
        $modify = (UserRequest::$request['ac'] === 'MODIFY');
        $title = ($modify) ? dict('TOOL_UICA_TITLE_MODIFY') : dict('TOOL_UICA_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UICA_id', $record['UICA_id']);
        $form->addArgument('idlist', toString($idlist));
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        if (!empty($modify)) {
		  $table->addData(dict('GEN_LBL_CODE'), $record['UICA_idcode']);
		  $form->addArgument('UICA_idcode', $record['UICA_idcode']);
        } else {
		  $table->addField($msg, dict('GEN_LBL_CODE'), 'UICA_idcode', $record['UICA_idcode'], CLX_NUM_16, CLX_NUM_16);
        }
        $table->addField($msg, dict('GEN_LBL_NAME'), 'UICA_idname', $record['UICA_idname'], CLX_NUM_48, CLX_NUM_64);
        $pop = new HTMLSelect('UICA_type');
        foreach ($this->cal_types as $key => $value) {
		  $pop->addItem($value, $key, ($record['UICA_type'] == $key));
        }
        $table->addData(dict('GEN_LBL_TYPE'), $pop->source());
        $pop = new HTMLSelect('UICA_view');
        foreach ($this->cal_views as $key => $value) {
		  $pop->addItem($value, $key, ($record['UICA_view'] == $key));
        }
        $table->addData(dict('GEN_LBL_VIEW'), $pop->source());
        $pop = new HTMLSelect('UICA_menu');
        foreach ($this->cal_menu as $key => $value) {
		  $pop->addItem($value, $key, ($record['UICA_menu'] == $key));
        }
        $table->addData(dict('GEN_LBL_MENU'), $pop->source());
        $UILG = &getTool('UILG');
        $UILG->placeChooseListing($msg, dict('GEN_LBL_LISTINGCODE'), 'UICA_listingcode', $record['UICA_listingcode'], $table);
        $DBTB = &getTool('DBTB');
        $DBTB->placeChooseTable($msg, dict('GEN_LBL_TABLECODE'), 'UICA_tablecode', $record['UICA_tablecode'], $table);
        $DBFL = &getTool('DBFL');
        $DBFL->placeChooseField($msg, dict('UICA_LBL_FIELDNAME'), 'UICA_fieldname', $record['UICA_fieldname'], $table);
        $DBFL->placeChooseField($msg, dict('UICA_LBL_EXTRA_FIELD'), 'UICA_extra_field1', $record['UICA_extra_field1'], $table);
        $DBFL->placeChooseField($msg, dict('UICA_LBL_RECURRENCE'), 'UICA_recurrence_field', $record['UICA_recurrence_field'], $table);
        $DBFL->placeChooseField($msg, dict('UICA_LBL_DISPLAYFIELD'), 'UICA_displayfield', $record['UICA_displayfield'], $table);
        $table->addField($msg, dict('GEN_LBL_VERSION'), 'UICA_version', $record['UICA_version'], CLX_NUM_16, CLX_NUM_16);
        $table->addText($msg, dict('GEN_LBL_DESCRIPTION'), 'UICA_description', $record['UICA_description'], CLX_NUM_FOUR, CLX_NUM_48);
        $DBMO = &getTool('DBMO');
        $DBMO->placeChooseModule($msg, dict('GEN_LBL_MODULECODE'), 'UICA_modulecode', $record['UICA_modulecode'], $table);
        $form->addContents($table->source());
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons = '';
        if (!empty($modify)) {
		  $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        } else {
		  $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID)))->source();
        }
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
        $action = UserRequest::$post['ac'];
        if ($action === 'CREATE') {
		  xValidate::unique($msg, 'UICA_idcode', 'code', $this->tablename, 'IDCODE=' . singleQuote(UserRequest::$post['UICA_idcode']));
        }
        if (strlen(UserRequest::$post['UICA_idname']) == CLX_NUM_ZERO) {
		  $msg['UICA_idname'] = dict('GEN_ERR_REQUIRED_FIELD');
        }
        if (strlen(UserRequest::$post['UICA_tablecode']) == CLX_NUM_ZERO) {
		  $msg['UICA_tablecode'] = dict('GEN_ERR_REQUIRED_FIELD');
        }
        $UILG = &getTool('UILG');
        if (strlen(UserRequest::$post['UICA_listingcode']) > CLX_NUM_ZERO && !$UILG->listingExists(UserRequest::$post['UICA_listingcode'])) {
		  $msg['UICA_listingcode'] = dict('GEN_ERR_INDEX_DOES_NOT_EXISTS');
        }
        $DBFL = &getTool('DBFL');
        if (strlen(UserRequest::$post['UICA_fieldname']) > CLX_NUM_ZERO) {
		  if (!$DBFL->fieldExists(UserRequest::$post['UICA_fieldname'])) {
			 $msg['UICA_fieldname'] = dict('GEN_ERR_INDEX_DOES_NOT_EXISTS');
		  } else {
			 $count = ClxCommon::$db->getOne('SELECT	COUNT(TF.ID) FROM CLX_UI_LISTINGS L, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF WHERE L.QUERYCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND L.IDCODE = ' . singleQuote(UserRequest::$post['UICA_listingcode']) . ' AND TF.FIELDNAME = ' . singleQuote(UserRequest::$post['UICA_fieldname']));
			 if ($count == CLX_NUM_ZERO) {
			     $msg['UICA_fieldname'] = dict('GEN_ERR_NOFIELDONTABLE');
			 }
		  }
        } else {
		  $msg['UICA_fieldname'] = dict('GEN_ERR_REQUIRED_FIELD');
        }
        if (strlen(UserRequest::$post['UICA_displayfield']) > CLX_NUM_ZERO) {
		  if (!$DBFL->fieldExists(UserRequest::$post['UICA_displayfield'])) {
			 $msg['UICA_displayfield'] = dict('GEN_ERR_INDEX_DOES_NOT_EXISTS');
		  } else {
			 $count = ClxCommon::$db->getOne('SELECT	COUNT(TF.ID) FROM CLX_UI_LISTINGS L, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF WHERE L.QUERYCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND L.IDCODE = ' . singleQuote(UserRequest::$post['UICA_listingcode']) . ' AND TF.FIELDNAME = ' . singleQuote(UserRequest::$post['UICA_displayfield']));
			 if ($count == CLX_NUM_ZERO)
			     $msg['UICA_displayfield'] = dict('GEN_ERR_NOFIELDONTABLE');
		  }
        }else {
		  $msg['UICA_displayfield'] = dict('GEN_ERR_REQUIRED_FIELD');
        }
        if (strlen(UserRequest::$post['UICA_recurrence_field']) > CLX_NUM_ZERO) {
		  if (!$DBFL->fieldExists(UserRequest::$post['UICA_recurrence_field'])) {
			 $msg['UICA_recurrence_field'] = dict('GEN_ERR_INDEX_DOES_NOT_EXISTS');
		  } else {
			 $count = ClxCommon::$db->getOne(' 								SELECT	COUNT(TF.ID) FROM CLX_UI_LISTINGS L, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF WHERE L.QUERYCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND L.IDCODE = ' . singleQuote(UserRequest::$post['UICA_listingcode']) . ' AND 										TF.FIELDNAME = ' . singleQuote(UserRequest::$post['UICA_recurrence_field']));
			 if ($count == CLX_NUM_ZERO)
			     $msg['UICA_recurrence_field'] = dict('GEN_ERR_NOFIELDONTABLE');
		  }
        }
        if (strlen(UserRequest::$post['UICA_extra_field1']) > CLX_NUM_ZERO) {
		  if (!$DBFL->fieldExists(UserRequest::$post['UICA_extra_field1'])) {
			 $msg['UICA_extra_field1'] = dict('GEN_ERR_INDEX_DOES_NOT_EXISTS');
		  } else {
			 $count = ClxCommon::$db->getOne(' 								SELECT	COUNT(TF.ID) FROM CLX_UI_LISTINGS L, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF WHERE L.QUERYCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND L.IDCODE = ' . singleQuote(UserRequest::$post['UICA_listingcode']) . ' AND 										TF.FIELDNAME = ' . singleQuote(UserRequest::$post['UICA_extra_field1']));
			 if ($count == CLX_NUM_ZERO)
			     $msg['UICA_extra_field1'] = dict('GEN_ERR_NOFIELDONTABLE');
		  }
		  if (UserRequest::$post['UICA_fieldname'] == UserRequest::$post['UICA_extra_field1']) {
			 $msg['UICA_extra_field1'] = dict('GEN_ERR_UNIQUE_KEY');
		  }
        }
        $DBMO = &getTool('DBMO');
        if (!$DBMO->moduleExists(UserRequest::$post['UICA_modulecode'])) {
		  $msg['UICA_modulecode'] = dict('TOOL_DBMO_ERR_NO_RECORD');
        }
        return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CODE') . ':', $record['UICA_idcode']);
        $table->addData(dict('GEN_LBL_NAME') . ':', $record['UICA_idname']);
    }

    function viewFormFill($record) {
        $contents = '';
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addCell(dict('TOOL_UICA_TITLE_VIEW'), array(CLX_PROP_CLASS => CLX_CLASS_TITLE));
        $contents .= $table->source();
        $infotable = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $infotable->addData(dict('GEN_LBL_CODE'), $record['UICA_idcode']);
        $infotable->addData(dict('GEN_LBL_NAME'), $record['UICA_idname']);
        $infotable->addData(dict('GEN_LBL_TYPE'), $this->cal_types[$record['UICA_type']]);
        $infotable->addData(dict('GEN_LBL_LISTINGCODE'), $record['UICA_listingcode']);
        $infotable->addData(dict('GEN_LBL_TABLECODE'), $record['UICA_tablecode']);
        $infotable->addData(dict('UICA_LBL_FIELDNAME'), $record['UICA_fieldname']);
        $infotable->addData(dict('UICA_LBL_EXTRAFIELDNAME'), $record['UICA_extra_field1']);
        $infotable->addData(dict('UICA_LBL_RECURRENCE'), $record['UICA_recurrence_field']);
        $infotable->addData(dict('GEN_LBL_VERSION'), $record['UICA_version'] . '-' . $record['UICA_version_counter']);
        $infotable->addData(dict('GEN_LBL_DESCRIPTION'), xFormat::text($record['UICA_description']));
        $infotable->addData(dict('GEN_LBL_MODULECODE'), $record['UICA_modulecode']);
        $contents .=$infotable->source();
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM));
        $table->addData(dict('GEN_LBL_CDATE'), $record['UICA_cdate']);
        $table->addData(dict('GEN_LBL_MDATE'), $record['UICA_mdate']);
        $table->addData(dict('GEN_LBL_USERNAME'), $record['UICA_username']);
        $contents .= buildPanelInterface(dict('GEN_TTL_SYSTEMINFO'), 'SYSTEMINFO', $table->source(), CLX_NUM_MINUS_1);
        return $contents;
    }

    function filterFormFill($msg, &$table) {
        if (!array_key_exists('UICA_idcode', UserRequest::$post))
		  UserRequest::$post['UICA_idcode'] = '';
        if (!array_key_exists('UICA_idname', UserRequest::$post))
		  UserRequest::$post['UICA_idname'] = '';
        if (!array_key_exists('UICA_modulecode', UserRequest::$post))
		  UserRequest::$post['UICA_modulecode'] = '';
        if (!array_key_exists('UICA_listingcode', UserRequest::$post))
		  UserRequest::$post['UICA_listingcode'] = '';
        $table->addField($msg, dict('GEN_LBL_CODE') . ':', 'UICA_idcode', UserRequest::$post['UICA_idcode'], CLX_NUM_16, CLX_NUM_16);
        $table->addField($msg, dict('GEN_LBL_NAME') . ':', 'UICA_idname', UserRequest::$post['UICA_idname'], CLX_NUM_32, CLX_NUM_64);
        $table->addField($msg, dict('GEN_LBL_MODULECODE') . ':', 'UICA_modulecode', UserRequest::$post['UICA_modulecode'], CLX_NUM_FOUR, CLX_NUM_FOUR);
        $table->addField($msg, dict('GEN_LBL_QUERYCODE') . ':', 'UICA_listingcode', UserRequest::$post['UICA_listingcode'], CLX_NUM_16, CLX_NUM_16);
        $sort = new HtmlSelect('UICA_sort');
        $sort->addItem('', '', true);
        $sort->addItem('Module Code', 'modulecode', false);
        $sort->addItem('Code', 'idcode', false);
        $sort->addItem('Name', 'idname', false);
        $table->addData(dict('GEN_LBL_ORDERBY') . ':', $sort->source());
    }

    function setfilter() {
        $filter = '';
        $boolean = '';
        if (!empty(UserRequest::$post['UICA_idcode'])) {
		  $filter .= $boolean . ' UPPER(IDCODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UICA_idcode']) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UICA_idname'])) {
		  $filter .= $boolean . ' UPPER(IDNAME) LIKE ' . singleQuote(uppercase (clx_change_especial_char_html (UserRequest::$post['UICA_idname'])) . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UICA_listingcode'])) {
		  $filter .= $boolean . ' LISTINGCODE LIKE ' . singleQuote(UserRequest::$post['UICA_listingcode'] . '%');
		  $boolean = 'AND';
        }
        if (!empty(UserRequest::$post['UICA_modulecode'])) {
		  $filter .= $boolean . ' UPPER(MODULECODE) LIKE ' . singleQuote(uppercase(UserRequest::$post['UICA_modulecode']) . '%');
		  $boolean = 'AND';
        }
        $_SESSION['paging']['UICAList']['sortby'] = UserRequest::$post['UICA_sort'];
        $_SESSION['paging']['UICAList']['criteria'] = $filter;
        $_SESSION['paging']['UICAList']['startat'] = '0';
    }
	///////////////////////////////////////////////////
	///comentado 22/01/2019 para utilizar la clx_tool
	//////////////////////////////////////////////////
    /*function clearfilter() {
        $_SESSION['paging']['UICAList']['criteria'] = '';
    }*/

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UICA_id'] = $recordid;
        $record['UICA_idcode'] = $values['IDCODE'];
        $record['UICA_idname'] = $values['IDNAME'];
        $record['UICA_listingcode'] = $values['LISTINGCODE'];
        $record['UICA_tablecode'] = $values['TABLECODE'];
        $record['UICA_fieldname'] = $values['fieldname'];
        $record['UICA_extra_field1'] = $values['EXTRA_FIELD1'];
        $record['UICA_displayfield'] = $values['DISPLAYFIELD'];
        $record['UICA_recurrence_field'] = $values['RECURRENCE_FIELD'];
        $record['UICA_type'] = $values['TYPE'];
        $record['UICA_view'] = (!empty($values['VIEWS'])) ? $values['VIEWS'] : '';
        $record['UICA_menu'] = $values['MENU'];
        $record['UICA_version'] = $values['VERSION'];
        $record['UICA_version_counter'] = $values['VERSION_COUNTER'];
        $record['UICA_cdate'] = xFormat::date($values['CDATE']);
        $record['UICA_mdate'] = xFormat::date($values['MDATE']);
        $record['UICA_username'] = $values['USERNAME'];
        $record['UICA_description'] = $values['DESCRIPTION'];
        $record['UICA_modulecode'] = $values['MODULECODE'];
        return $record;
    }

    function saveRecord() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDNAME'] = UserRequest::$post['UICA_idname'];
        $record['VERSION'] = UserRequest::$post['UICA_version'];
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['UICA_description'];
        $record['MODULECODE'] = UserRequest::$post['UICA_modulecode'];
        $record['LISTINGCODE'] = UserRequest::$post['UICA_listingcode'];
        $record['TABLECODE'] = UserRequest::$post['UICA_tablecode'];
        $record['FIELDNAME'] = UserRequest::$post['UICA_fieldname'];
        $record['EXTRA_FIELD1'] = UserRequest::$post['UICA_extra_field1'];
        $record['RECURRENCE_FIELD'] = UserRequest::$post['UICA_recurrence_field'];
        $record['DISPLAYFIELD'] = UserRequest::$post['UICA_displayfield'];
        $record['TYPE'] = UserRequest::$post['UICA_type'];
        $record['VIEWS'] = UserRequest::$post['UICA_view'];
        $record['MENU'] = UserRequest::$post['UICA_menu'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UICA_id']);
        $this->versionIncrease('ID', UserRequest::$post['UICA_id']);
    }

    function createRecord_() {
        $d = xFormat::getNow('Y-m-d');
        $record = array();
        $record['IDCODE'] = UserRequest::$post['UICA_idcode'];
        $record['IDNAME'] = UserRequest::$post['UICA_idname'];
        $record['VERSION'] = UserRequest::$post['UICA_version'];
        $record['VERSION_COUNTER'] = '0';
        $record['CDATE'] = $d;
        $record['MDATE'] = $d;
        $record['USERNAME'] = $_SESSION['user_record']['username'];
        $record['DESCRIPTION'] = UserRequest::$post['UICA_description'];
        $record['MODULECODE'] = UserRequest::$post['UICA_modulecode'];
        $record['LISTINGCODE'] = UserRequest::$post['UICA_listingcode'];
        $record['TABLECODE'] = UserRequest::$post['UICA_tablecode'];
        $record['FIELDNAME'] = UserRequest::$post['UICA_fieldname'];
        $record['EXTRA_FIELD1'] = UserRequest::$post['UICA_extra_field1'];
        $record['RECURRENCE_FIELD'] = UserRequest::$post['UICA_recurrence_field'];
        $record['DISPLAYFIELD'] = UserRequest::$post['UICA_displayfield'];
        $record['TYPE'] = UserRequest::$post['UICA_type'];
        $record['VIEWS'] = UserRequest::$post['UICA_view'];
        $record['MENU'] = UserRequest::$post['UICA_menu'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $record['MODULECODE']);
        return $recordid;
    }

    function duplicateRecord() {
        return $this->createRecord();
    }

    function deleteRecordSk($id) {
        $calrec = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $calrec['MODULECODE']);
    }

    function defaultRecord() {
        $record = array();
        $record['UICA_id'] = CLX_INIT_ZERO;
        $record['UICA_idcode'] = '';
        $record['UICA_idname'] = '';
        $record['UICA_version'] = '1.0.0';
        $record['UICA_cdate'] = '';
        $record['UICA_mdate'] = '';
        $record['UICA_username'] = $_SESSION['user_record']['username'];
        $record['UICA_description'] = '';
        $record['UICA_modulecode'] = '';
        $record['UICA_listingcode'] = '';
        $record['UICA_tablecode'] = '';
        $record['UICA_fieldname'] = '';
        $record['UICA_extra_field1'] = '';
        $record['UICA_recurrence_field'] = '';
        $record['UICA_displayfield'] = '';
        $record['UICA_type'] = 'CAL';
        $record['UICA_view'] = '';
        $record['UICA_menu'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
        $modulecode = ClxCommon::$db->getField($this->tablename, 'MODULECODE', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

    function pagelet($code, $ac = null, $objectid = null) {
        return $this->buildMaster($code);
    }

    /**
     * Master table for all calendars ui
     */
    function buildMaster($code, $ts = CLX_NUM_ZERO) {
        $contents = (string) '';
        $contents = setJavaScriptTag(getJavaScript('UICA_toolTip.js', '')) . CLX_CHAR_NL;
        if (empty($ts)) {
		  $ts = adodb_mktime();
        }
        $calRec = ClxCommon::$db->getRecord($this->tablename, 'IDCODE = ' . singleQuote($code));
        $this->viewType = $calRec['VIEWS'];
        $ts = $this->calendarSession($calRec); //Set preffered time stamp
        if (empty($this->viewType))
		  $this->viewType = ''; //day, month, workweek,week, listing
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT, CLX_PROP_CLASS => CLX_CLASS_LIST));
        if (!empty($calRec['LISTINGCODE']) && $this->viewType <> CLX_CLASS_LIST) {
		  $listingRec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
		  $table->addCell(dict($listingRec['TITLE']), array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        }
        switch ($this->viewType) {
		  case 'year':
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildYear($calRec, $ts), array(CLX_PROP_WIDTH => '*', CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 break;
		  case 'week':
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildWeek($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 break;
		  case 'work':
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildWeek($calRec, $ts, 'workweek'), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 break;
		  case 'mont':
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildMonth($calRec, $ts, 'month'), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 break;
		  case 'day':
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_ALIGN => 'center', CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildDay($calRec, $ts, 'day'), array(CLX_PROP_WIDTH => '*', CLX_PROP_ALIGN => 'center'));
			 $table->addcell($this->buildMonth($calRec, $ts), array(CLX_PROP_WIDTH => '10px', CLX_PROP_ALIGN => 'center'));
			 break;
		  case CLX_CLASS_LIST:
			 if ($calRec['MENU'] === true) {
			     $table->addcell($this->calMenu($calRec, $ts), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 }
			 $table->addcell($this->buildListing($calRec, $ts), array(CLX_PROP_WIDTH => '*'));
			 $table->addcell($this->buildMonth($calRec, $ts), array(CLX_PROP_WIDTH => '10px'));
			 break;
		  default: //Day calendar
			 $table->addcell($this->buildMonth($calRec, $ts), array(CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
			 break;
        }
        $contents .= $table->source();
        return $contents;
    }

    /**
     * Generate output for a day.
     * @parameters
     * @return
     */
    function buildDay($calRec, $ts = CLX_NUM_ZERO, $view = '') {
        $contents = (string) '';
        if (empty($ts))
		  $ts = adodb_mktime();
        $listingRec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
        $li = $listingRec['IDCODE'];
        $qy = $listingRec['QUERYCODE'];
        // Get date information
        $dateRec = adodb_getdate($ts);
        // Init Table
        $table = new htmlTable(CLX_WIDTH_ONE);
        $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT, 'height' => CLX_PROP_100PCT));
        if ($view === 'week' || $view === 'workweek') {
		  $table->addCell($this->weekDays($dateRec['wday'], 'long') . '<br>' . adodb_date('Y-m-d', $ts), array(CLX_PROP_CLASS => 'month_title', 'nowrap' => 'true'));
        } elseif ($view !== 'week' && $view !== 'workweek' && $view !== 'day') {
		  $table->addCell($this->calNavMenu($calRec, $ts, $view), array(CLX_PROP_CLASS => 'month_title'));
        }
        // Calendar get data
        $data = $this->getDayData($calRec, $ts);
        // Set Number of colums
        $tableWidth = $data[CLX_NUM_ - CLX_NUM_TWO] + CLX_NUM_ONE; //	-2 means count of colums based on source data
        $allday = (bool) false;  //	All day activity flag
        $store = array();   //	Tracks columb where acti was set
        $store2 = array();   //	Tracks if and id was displayed or not
        $hourTable = new htmlTable($tableWidth);
        $hourTable->setProps(array(CLX_PROP_CLASS => 'calendarMonthView'));
        reset($data);
        foreach ($data as $key => $val) {
		  // All Day display
		  if (empty($allday)) {
			 // All day label
			 $allday = true;
			 $url = 'tool.php?to=EDIT&ac=CREATE&li=' . $li . '&qy=' . $qy . '&ds=' . $dateRec['year'] . '-' . $dateRec['mon'] . '-' . $dateRec['mday'] . ' ' . $key . ':00:00&de=' . $dateRec['year'] . '-' . $dateRec['mon'] . '-' . $dateRec['mday'] . ' 24:00:00&cal=' . $calRec['IDCODE'];
			 $hourTable->addCell(dict('GEN_UICA_ALLDAY') . getSkinIcon('UICA_plus', $url, 'MODIFY'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT, 'nowrap' => 'true'));
			 if (!empty($data['-1'])) {
			     $allDayTable = new htmlTable('1');
			     $table->setProps(array(CLX_PROP_WIDTH => CLX_PROP_100PCT, CLX_PROP_CLASS => CLX_CLASS_HEADERS));
			     reset($data['-1']);
			     foreach ($data['-1'] as $key2 => $val2) {
			         $allDayTable->addCell($this->displayCalItem($calRec, $val2, $li, $qy, (($view === 'week' || $view === 'workweek') ? CLX_NUM_FOUR : CLX_NUM_15), $view, $ts), array(CLX_PROP_CLASS => 'day'));
			     }
			     $hourTable->addCell($allDayTable->source(), array(CLX_PROP_CLASS => 'dayalt', 'border' => '1', CLX_PROP_COLSPAN => $tableWidth));
			 } else {
			     $hourTable->addCell(CLX_CHAR_HTML_NBSP, CLX_PROP_CLASS, 'dayalt', 'border', '1', CLX_PROP_COLSPAN, $tableWidth);
			 }
		  }
		  // Skeep table source and all day entries
		  if ($key < CLX_NUM_ZERO)
			 continue;
		  $url = 'tool.php?to=EDIT&ac=CREATE&li=' . $li . '&qy=' . $qy . '&ds=' . $dateRec['year'] . '-' . $dateRec['mon'] . '-' . $dateRec['mday'] . ' ' . $key . ':00:00&de=' . $dateRec['year'] . '-' . $dateRec['mon'] . '-' . $dateRec['mday'] . ' ' . ($key + CLX_NUM_ONE) . ':00:00&cal=' . $calRec['IDCODE'];
		  $hourTable->addCell($this->thisHour($key) . ' ' . getSkinIcon('UICA_plus', $url, 'Add'), array(CLX_PROP_CLASS => CLX_CLASS_HEADERS, CLX_PROP_ALIGN => CLX_PROP_RIGHT, CLX_PROP_WIDTH => '15%', 'nowrap' => 'true'));
		  if (empty($val)) {
			 // Add cell filler for each empty cell on no item schedule.
			 for ($i = CLX_INIT_ZERO; $i < ($data['-2']); $i++)
			     $hourTable->addCell(CLX_CHAR_HTML_NBSP, CLX_PROP_CLASS, 'dayalt');
		  } else {
			 $j = CLX_INIT_ZERO;
			 foreach ($val as $key2 => $val2) {
			     if (!isset($store[$val2['ID']])) {
			         $store[$val2['ID']] = $j;
			     }
			     if (!isset($store2[$val2['ID']])) {
			         //Create label
			         $hourTable->addCell($this->displayCalItem($calRec, $val2, $li, $qy, (($view === 'week' || $view === 'workweek') ? CLX_NUM_FOUR : CLX_NUM_15), $view, $ts), array(CLX_PROP_CLASS => 'day', CLX_PROP_ROWSPAN => $val2[CLX_PROP_ROWSPAN]));
			         $store2[$val2['ID']] = CLX_NUM_ONE;
			     }
			     $j++;
			 }
			 for ($i = $j; $i < ($data['-2']); $i++) {
			     $hourTable->addCell(CLX_CHAR_HTML_NBSP, CLX_PROP_CLASS, 'dayalt');
			     $j++;
			 }
		  }
        }
        $table->addCell($hourTable->source());
        $contents = $table->source();
        return $contents;
    }

    /**
     * Buids week or work week calendar
     * @param: calendar Calendar Record, time stamp, view type(month, year)
     * @return: string
     */
    function buildWeek($calRec, $ts = CLX_NUM_ZERO, $view = '') {
        if (empty($ts))
		  $ts = adodb_mktime();
        $contents = (string) '';
        $tableWidth = (($view === 'workweek') ? CLX_NUM_FIVE : CLX_NUM_7);
        $table = new htmlTable($tableWidth);
        $table->setProps(array(CLX_PROP_CLASS => 'calendarMonthView'));
        $table->addCell($this->calNavMenu($calRec, $ts, 'week'), array(CLX_PROP_CLASS => 'month_title', CLX_PROP_COLSPAN => $tableWidth));
        $dateRec = adodb_getdate($ts);
        // Build Previous days
        $start = (($view === 'workweek') ? CLX_NUM_ONE : CLX_NUM_ZERO);
        $end = (($view === 'workweek') ? CLX_NUM_SIX : CLX_NUM_7);
        $day = ($dateRec['mday'] - $dateRec['wday']) + (($view === 'workweek') ? CLX_NUM_ONE : CLX_NUM_ZERO);
        for ($i = $start; $i < $end; $i++) {
		  $ts = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $day, $dateRec['year']);
		  $table->addcell($this->buildDay($calRec, array($ts => 'week')));
		  $day++;
        }
        $contents = $table->source();
        return $contents;
    }

    /**
     * Buids month calendar
     * @param: calendar Calendar Record, time stamp, view type(month, year)
     * @return: string
     */
    function buildMonth($calRec, $ts = CLX_NUM_ZERO, $view = '') {
        $listingRec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
        $li = $listingRec['IDCODE'];
        $qy = $listingRec['QUERYCODE'];
        if (empty($ts))
		  $ts = adodb_mktime();
        // Retrive next and prev month information
        $info = adodb_getdate($ts);
        $year = $info['year'];
        $month = $info['mon'];
        $day = $info['mday'];
        $table = new htmlTable(CLX_NUM_7);
        $table->setProps(array(CLX_PROP_CLASS => (($view === 'month') ? 'calendarMonthView' : 'calendar')));
        if ($view === 'year') {
		  $table->addCell(dict('UICA_LBL_MONTH_' . adodb_date('m', $ts)) . ' ' . adodb_date('Y', $ts), array(CLX_PROP_CLASS => 'month_title', CLX_PROP_COLSPAN => CLX_NUM_7));
        } else {
		  $table->addCell($this->calNavMenu($calRec, $ts, $view), array(CLX_PROP_CLASS => 'month_title', CLX_PROP_COLSPAN => CLX_NUM_7));
        }
        for ($i = CLX_INIT_ZERO; $i < CLX_NUM_7; $i++) {
		  $table->addCell($this->weekDays($i, 'short'), array(CLX_PROP_CLASS => 'day_title', CLX_PROP_WIDTH => '14%', 'height' => '14'));
        }
        // Previous month days
        $padding = adodb_date('w', adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, CLX_NUM_ONE, $year));
        $qty = adodb_date('t', adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, $day, $year));
        $empty = ($qty - (CLX_NUM_7 - $padding)) % CLX_NUM_7;
        $cTs = (int) CLX_NUM_ZERO;
        // Display Previous Month Days
        for ($i = CLX_NUM_ONE; $i <= $padding; $i++) {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, (CLX_NUM_ONE - ($padding - ($i - CLX_NUM_ONE))), $year);
		  $desc = '';
		  //Searching Recurence For a Day
		  $data = $this->existsMonthData($calRec, $cTs);
		  if ($view === 'month') {
			 $data = $this->getMonthData($calRec, $cTs);
			 foreach ($data as $key => $val) {
			     if (empty($val)) {
			         continue;
			     }
			     $desc .= '<br>' . $this->displayCalItem($calRec, $val, $li, $qy, CLX_NUM_FOUR, $view, $cTs);
			 }
			 unset($data);
			 $url = 'tool.php?to=EDIT&ac=CREATE&li=' . $li . '&qy=' . $qy . '&ds=' . adodb_date('Y-m-d', $cTs) . ' 00:00:00&de=' . adodb_date('Y-m-d', $cTs) . ' 01:00:00&cal=' . $calRec['IDCODE'];
			 $add = getSkinIcon('UICA_plus', $url, 'Add') . ' ';
		  } else {
			 $add = '';
			 $desc = '';
		  }
		  $prompt = adodb_date('d', $cTs) . '<br>';
		  $dt = $this->anchorURI() . '&dt' . $calRec['ID'] . '=' . $cTs;
		  if ($view === 'year')
			 $dt .='&dt' . $calRec['ID'] . 't=' . base64_encode('day');
		  $anchor = new htmlAnchor($prompt, $dt);
		  $anchor->setProp(CLX_PROP_CLASS, 'day');
		  $table->addCell($add . $anchor->source() . $desc, array(CLX_PROP_CLASS => 'dayalt', CLX_PROP_WIDTH => '14%', 'height' => '14%'));
        }
        // Display Current Month Days
        $special = array();
        for ($i = CLX_NUM_ONE; $i <= $qty; $i++) {
		  if (array_key_exists($i, $special))
			 $prompt = $special[$i];
		  else
			 $prompt = $i;
		  $desc = '';
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, $prompt, $year);
		  if ($view === 'month') {
			 $data = $this->getMonthData($calRec, $cTs);
			 foreach ($data as $key => $val) {
			     if (empty($val))
			         continue;
			     $desc .= '<br>' . $this->displayCalItem($calRec, $val, $li, $qy, CLX_NUM_FOUR, $view, $cTs);
			 }
			 unset($data);
			 $url = 'tool.php?to=EDIT&ac=CREATE&li=' . $li . '&qy=' . $qy . '&ds=' . adodb_date('Y-m-d', $cTs) . ' 00:00:00&de=' . adodb_date('Y-m-d', $cTs) . ' 01:00:00&cal=' . $calRec['IDCODE'];
			 $add = getSkinIcon('UICA_plus', $url, 'Add') . ' ';
		  }else {
			 $add = '';
			 $desc = ($this->existsActivity($calRec, $cTs)) ? '@' : '';
		  }
		  $dt = $this->anchorURI() . '&dt' . $calRec['ID'] . '=' . $cTs;
		  if ($view === 'year')
			 $dt .='&dt' . $calRec['ID'] . 't=' . base64_encode('day');
		  $anchor = new htmlAnchor($prompt, $dt);
		  $anchor->setProp(CLX_PROP_CLASS, 'day');
		  if ($prompt == adodb_date('j') && $year == adodb_date('Y') && $month == adodb_date('m')) {
			 $prompt = $add . $anchor->source() . '*<br>' . $desc;
		  } else {
			 $prompt = $add . $anchor->source() . '&nbsp;<br>' . $desc;
		  }
		  if ($view === 'year') {
			 if (adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month, $i, $year) != $this->ts)
			     $table->addCell($prompt, array(CLX_PROP_CLASS => 'day', 'height' => '14%'));
			 else
			     $table->addCell($prompt, array(CLX_PROP_CLASS => 'daycur', CLX_PROP_WIDTH => '14%', 'height' => '14%'));
		  }else {
			 if ($i != $day)
			     $table->addCell($prompt, array(CLX_PROP_CLASS => 'day', CLX_PROP_WIDTH => '14%', 'height' => '14%'));
			 else
			     $table->addCell($prompt, array(CLX_PROP_CLASS => 'daycur', CLX_PROP_WIDTH => '14%', 'height' => (($view === 'month') ? '50px' : '14%')));
		  }
        } //End for
        // Display Next Month Days
        if ($empty > CLX_NUM_ZERO) {
		  $j = CLX_NUM_ONE;
		  for ($i = $empty; $i < CLX_NUM_7; $i++) {
			 $desc = '';
			 $prompt = $j;
			 $j++;
			 $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $month + CLX_NUM_ONE, $prompt, $year);
			 $data = $this->existsMonthData($calRec, $cTs);
			 if ($view === 'month') {
			     $data = $this->getMonthData($calRec, $cTs);
			     foreach ($data as $key => $val) {
			         if (empty($val))
					   continue;
			         $desc .= '<br>' . $this->displayCalItem($calRec, $val, $li, $qy, CLX_NUM_FOUR, $view, $cTs);
			     }
			     unset($data);
			     $url = 'tool.php?to=EDIT&ac=CREATE&li=' . $li . '&qy=' . $qy . '&ds=' . adodb_date('Y-m-d', $cTs) . ' 00:00:00&de=' . adodb_date('Y-m-d', $cTs) . ' 01:00:00&cal=' . $calRec['IDCODE'];
			     $add = getSkinIcon('UICA_plus', $url, 'Add') . ' ';
			 }else {
			     $add = '';
			     $desc = '';
			 }
			 $prompt = ($i - $empty) + CLX_NUM_ONE;
			 $dt = $this->anchorURI() . '&dt' . $calRec['ID'] . '=' . $cTs;
			 if ($view === 'year')
			     $dt .='&dt' . $calRec['ID'] . 't=' . base64_encode('day');
			 $anchor = new htmlAnchor($prompt, $dt);
			 $anchor->setProp(CLX_PROP_CLASS, 'day');
			 $table->addCell($add . $anchor->source() . $desc, array(CLX_PROP_CLASS => 'dayalt', CLX_PROP_WIDTH => '14%', 'height' => (($view === 'month') ? '40px' : '')));
		  }
        }
        $contents = $table->source();
        return $contents;
    }

    /**
     * Builds year calendar
     */
    function buildYear($calRec, $ts = o) {
        if (empty($ts))
		  $ts = adodb_mktime();
        $contents = (string) '';
        $table = new htmlTable(CLX_NUM_THREE);
        $table->setProps(array(CLX_PROP_CLASS => 'calendar'));
        $table->addCell($this->calNavMenu($calRec, $ts, 'year'), array(CLX_PROP_CLASS => 'month_title', CLX_PROP_COLSPAN => CLX_NUM_THREE));
        $dateRec = adodb_getdate($ts);
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_12; $i++) {
		  $ts = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $i, CLX_NUM_ONE, $dateRec['year']);
		  $table->addcell($this->buildMonth($calRec, array($ts => 'year')));
        }
        $contents = $table->source();
        return $contents;
    }

    /**
     * Builds calendar listing
     */
    function buildListing($calRec, $ts) {
        $contents = '';
        $criteria = '';
        $listingcode = $calRec['LISTINGCODE'];
        $fn = $calRec['FIELDNAME'];
        $ex = $calRec['EXTRA_FIELD1'];
        if (strlen($fn) > CLX_NUM_ZERO) {
		  if (empty($ts))
			 $ts = strtotime('now');
		  $listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($listingcode));
		  if(empty($listingrec)) {
			 // Listing not found
			 $contents = dict('GEN_MSG_RECORD_NOTFOUND', $listingcode) . ' [UILG-ERR-010]';
			 return $contents;
		  }
		  $querycode = $listingrec['QUERYCODE'];
		  // Set Filter Criteria
		  switch (kDatabaseType) {
			 case 'oci': // Oracle
			     //$criteria .= $listingrec['TABLECODE'] . '.' . $fn . ' = ' . ClxCommon::$db->db_to_date(date('Y-m-d', $ts));
			     $criteria .= $listingrec['TABLECODE'] . '.' . $fn . ' = ' . ClxCommon::$db->db_to_date(singleQuote(date('Y-m-d', $ts)));
			     break;
			 default: // MySQL Version all others
			     $criteria .= $listingrec['TABLECODE'] . '.' . $fn . ' LIKE ' . singleQuote(date('Y-m-d', $ts) . '%');
			     break;
		  }
		  if (strlen($ex) > CLX_NUM_ZERO) {
			 $criteria .= ' OR (';
			 //$criteria .= $listingrec['TABLECODE'] . '.' . $fn . ' < ' . ClxCommon::$db->db_to_date(date('Y-m-d', $ts));
			 $criteria .= $listingrec['TABLECODE'] . '.' . $fn . ' < ' . ClxCommon::$db->db_to_date(singleQuote(date('Y-m-d', $ts)));
			 //$criteria .= ' AND ' . $listingrec['TABLECODE'] . '.' . $ex . ' >= ' . ClxCommon::$db->db_to_date(date('Y-m-d', $ts));
			 $criteria .= ' AND ' . $listingrec['TABLECODE'] . '.' . $ex . ' >= ' . ClxCommon::$db->db_to_date(singleQuote(date('Y-m-d', $ts)));
			 $criteria .=')';
		  }
		  // Load filter tool and clear/set qfilter
		  $UIFT = &getTool('UIFT');
		  $UIFT->clearPref($querycode);
		  $_SESSION['paging'][$querycode . '_LIST']['criteria'] = $criteria;
		  // Build Listing
		  $UILG = &getTool('UILG');
		  $table = new htmlTable(CLX_NUM_ONE);
		  $table->setProps(array(CLX_PROP_ALIGN => 'center', CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $table->addCell($UILG->buildListing($listingcode), array());
		  $contents = $table->source();
		  // Clean Filters
		  $UIFT->clearPref($querycode);
        }
        return $contents;
    }

// C a l e n d a r  D a t a  S o u r c e s
    /**
     * Checks the day and displays this exists
     */
    function existsMonthData($calRec = false, $ts = CLX_NUM_ZERO) {
        $count = CLX_INIT_ZERO;
        if (empty($ts)) {
		  $ts = $this->ts;
        }
        $tsDate = (string) adodb_date('Y-m-d', $ts);
        //Search Query if not exists
        $haslisting = !empty($calRec['LISTINGCODE']);
        if ($haslisting === true && empty($this->listingQuery)) {
		  $DBQY = &getTool('DBQY');
		  $querycode = ClxCommon::$db->getField('CLX_UI_LISTINGS', 'QUERYCODE', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
		  if(empty($listingrec)) {
			 // Listing not found
			 return 0;
		  }
		  $qaccess = $DBQY->getAppAccess($querycode);
		  if ($qaccess->canRead()) {
			 $query = $DBQY->getquery('UICA', $querycode);
			 $this->listingQuery['WHERECLAUSE'] = $query['WHERECLAUSE'];
			 $this->listingQuery['FROMCLAUSE'] = $query['FROMCLAUSE'];
			 $this->listingQuery['SELECTCLAUSE'] = $query['SELECTCLAUSE'];
			 $this->listingQuery['GROUPCLAUSE'] = $query['GROUPCLAUSE'];
		  }
        }
        if (!empty($this->listingQuery)) {
		  $fn = $calRec['TABLECODE'] . '.' . $calRec['FIELDNAME'];
		  $criteria = (string) $fn . ' LIKE ' . singleQuote($tsDate . '%') . ' ' . ((strlen($this->listingQuery['WHERECLAUSE']) > CLX_NUM_ZERO) ? ' AND ' . $this->listingQuery['WHERECLAUSE'] : '');
		  $criteria .= (strlen($this->listingQuery['WHERECLAUSE']) <> CLX_NUM_ZERO) ? ' AND ' . $this->listingQuery['WHERECLAUSE'] : '';
		  $sql = 'SELECT COUNT(*) FROM ' . $this->listingQuery['FROMCLAUSE'] . ' WHERE ' . $criteria;
		  $sql .=((strlen($this->listingQuery['GROUPCLAUSE']) > CLX_NUM_ZERO) ? ' GROUP BY ' . $this->listingQuery['GROUPCLAUSE'] : '');
		  $count = ClxCommon::$db->getOne($sql);
        }
        return $count;
    }

    function getMonthData($calRec = false, $ts = CLX_NUM_ZERO) {
        $return = array();
        $fn = $calRec['FIELDNAME'];
        $ex = $calRec['EXTRA_FIELD1'];
        $df = $calRec['DISPLAYFIELD'];
        $re = $calRec['RECURRENCE_FIELD'];
        $records = $this->getData($calRec, $ts);
        if (strlen($re) > CLX_NUM_ZERO) {
		  $records = $this->recurrence($calRec, $records, $ts); // Get Recurrences
        }
        reset($records);
        foreach ($records as $key => $row) {
		  $return[$row['ID']] = array('sdate' => $row[$fn],
			 'edate' => ((!empty($row[$ex])) ? $row[$ex] : ''),
			 CLX_CLASS_TITLE => $row[$df],
			 CLX_PROP_ROWSPAN => CLX_NUM_ONE, 'offset' => CLX_NUM_ONE, 'ID' => $row['ID']);
        }
        return $return;
    }

    /**
     * Test if activity exists
     *
     */
    function existsActivity($calRec = false, $ts = CLX_NUM_ZERO) {
		$fetched = false;
		//pre($ts);pre($calRec);echo'----';
	    if(kApcTtl_18>0) $data = apc_fetchFct($findVar=kSpaceName . '_calendar_'.base64_encode(md5(implode('',$calRec))). $ts, $fetched);
		if (empty($fetched)) {
        $data = $this->getMonthData($calRec, $ts);
			if (kApcTtl_18>0 /*&&!empty($data)*/) {
				apc_addFct($findVar,json_encode($data), kApcTtl_18);  //actualizacion forzada cada 4 minutos no ilimitada
				}
			}
		else {
			$data=json_decode($data,true);
		}
        
        return (empty($data)) ? false : true;
    }

    /**
     * Builds array array
     * @parameter: array for day description
     * @return: Descripion string
     */
    function getDayData($calRec = false, $ts = CLX_NUM_ZERO) {
        $hours = (int) CLX_NUM_24;
        $return = array();
        for ($i = CLX_INIT_ZERO; $i < $hours; $i++) {
		  $return[$i] = false;
        }
        $dateRec = adodb_getdate($ts);
        if (empty($ts)) {
		  $ts = $this->ts;
        }
        $fn = uppercase($calRec['fieldname']);
        $ex = uppercase($calRec['EXTRA_FIELD1']);
        $df = uppercase($calRec['DISPLAYFIELD']);
        $re = uppercase($calRec['RECURRENCE_FIELD']);
        $records = $this->getData($calRec, $ts);
        if (strlen($re) > CLX_NUM_ZERO) {
		  $records = $this->recurrence($calRec, $records, $ts); // Get Recurrences
        }
        reset($records);
        foreach ($records as $key => $row) {
		  $offset = (int) CLX_NUM_ZERO;
		  $edateTs = (int) '';
		  $hour = (int) adodb_date('H', adodb_strtotime($row[$fn]));
		  $min = (int) adodb_date('i', adodb_strtotime($row[$fn]));
		  //All day activity
		  if (!empty($row['ISALLDAY_SYS'])) {
			 $offset = CLX_INIT_ZERO;
			 $rowspan = CLX_NUM_ONE;
			 $hour = - CLX_NUM_ONE;
		  } else {
			 // Calculatign offet and rowspan
			 if ($min >= CLX_NUM_30)
			     $offset = CLX_NUM_ONE;
			 if (strlen($ex) > CLX_NUM_ZERO) {
			     $dateDif = (int) adodb_strtotime($row[$ex]) - adodb_strtotime($row[$fn]);
			     $edateTs = adodb_strtotime($row[$ex]);
			     if ($dateDif < CLX_NUM_ZERO)
			         $dateDif *= - CLX_NUM_ONE;
			     // Calculate rowspan
			     $rowspan = round(($dateDif / CLX_NUM_60) / CLX_NUM_60);
			     // Adjust hour so it will start at 0 for new +1 day or more  after start
			     $tmp = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $dateRec['mday'], $dateRec['year']);
			     if (adodb_strtotime($row[$fn]) < $tmp) {
			         $hour = CLX_INIT_ZERO;
			     }
			     if ($rowspan > CLX_NUM_ONE) {
			         $x = $rowspan;
			         for ($i = CLX_NUM_ONE; $i < $x; $i++) {
					   if (($hour + $i) > CLX_NUM_23) {
						  break; // Break for long days
					   }
					   $tmp = adodb_mktime(($hour + $i), CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $dateRec['mday'], $dateRec['year']);
					   if ($edateTs == $tmp) {
						  $rowspan = adodb_date('H', $edateTs);
						  break;
					   }
					   $return[$hour + $i][] = array('sdate' => $row[$fn], 'edate' => ((!empty($row[$ex])) ? $row[$ex] : ''), CLX_CLASS_TITLE => $row[$df], CLX_PROP_ROWSPAN => CLX_NUM_ONE, 'offset' => CLX_NUM_ZERO, 'ID' => $row['ID']);
			         }//end for
			     }
			 }
		  }
		  $return[$hour][] = array('sdate' => $row[$fn], 'edate' => ((!empty($row[$ex])) ? $row[$ex] : ''), CLX_CLASS_TITLE => substr($row[$df], CLX_NUM_ZERO, CLX_NUM_15), CLX_PROP_ROWSPAN => $rowspan, 'offset' => $offset, 'ID' => $row['ID']);
        }
        $count = CLX_INIT_ZERO;
        foreach ($return as $key => $val) {
		  if ($key < CLX_NUM_ZERO) {
			 continue;
		  }
		  $x = (int) count($val);
		  if ($x > $count) {
			 $count = $x;
		  }
        }
        $return[CLX_NUM_ - CLX_NUM_TWO] = $count;
        return $return;
    }

    /**
     * Builds array array
     * @parameter: array for data
     * @return: Descripion string
     */
    function getData($calRec = false, $ts = CLX_NUM_ZERO, $return_count = false) {
        $return = array();
        if (empty($ts))
		  $ts = $this->ts;
        $tsDate = (string) adodb_date('Y-m-d', $ts);
        $fn = $calRec['FIELDNAME'];
        $ex = $calRec['EXTRA_FIELD1'];
        if (empty($this->listingRecord)) {
		  $this->listingRecord = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
		  if (empty($this->listingRecord)) {
			 // Listing not found
			 return $return;
		  }
        }
        $listingRec = $this->listingRecord;
        if (empty($this->queryRecord)) {
		  $DBQY = &getTool('DBQY'); // The query manager
		  $this->queryRecord = $DBQY->getQuery($listingRec['MODULECODE'], $listingRec['QUERYCODE']);
		  $this->queryAccess = $DBQY->getUserAccess($listingRec['QUERYCODE']);
        }
        $queryrec = $this->queryRecord;
        $cmd = $queryrec['FROMCLAUSE'];
        $criteria = (string) ' WHERE (' . $fn . ' LIKE ' . singleQuote($tsDate . '%');
        if (strlen($ex) > CLX_NUM_ZERO) {
		  $criteria .= ' OR (' . $fn . ' < ' . singleQuote($tsDate) . ' AND ' . $ex . ' >= ' . singleQuote($tsDate) . '))';
        } else {
		  $criteria .= ')';
        }
        if (strlen($queryrec['WHERECLAUSE']) > CLX_NUM_ZERO)
		  $criteria .= ' AND ' . $queryrec['WHERECLAUSE'];
        $criteria .=' ORDER BY ' . $fn;
        $sql = (string) 'SELECT * FROM ' . $cmd . ' ' . $criteria;
        $results = ClxCommon::$db->executeQuery($sql);
        if ($return_count === true) {
		  if (ClxCommon::$db->numRows($results) > CLX_NUM_ZERO) {
			 $return[] = CLX_NUM_ONE;
		  } else {
			 $return = array();
		  }
        } else {
		  while ($row = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
			 $return[] = $row;
		  }//End while
		  ClxCommon::$db->freeResult( $results );
        }
        return $return;
    }

// R E C U R R E N C E
    /**
     * Prepare recurrences for an item
     */
    function recurrence($calRec, $records, $ts = CLX_NUM_ZERO) {
        $fn = $calRec['FIELDNAME'];
        $ex = $calRec['EXTRA_FIELD1'];
        $re = $calRec['RECURRENCE_FIELD'];
        if (empty($this->listingRecord)) {
		  //$this->listingRec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
		  $this->listingRecord = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($calRec['LISTINGCODE']));
		  if (empty($this->listingRecord)) {
			 // Listing not found
			 return $records;
		  }
        }
        $listingRec = $this->listingRecord;
        if (empty($this->queryRecord)) {
		  $DBQY = &getTool('DBQY'); // The query manager
		  $this->queryRecord = $DBQY->getQuery($listingRec['MODULECODE'], $listingRec['QUERYCODE']);
		  $this->queryAccess = $DBQY->getUserAccess($listingRec['QUERYCODE']);
        }
        $queryrec = $this->queryRecord;
        $cmd = $queryrec['FROMWHERESORT'];
        if (strpos($cmd, 'WHERE') > CLX_NUM_ZERO) {
		  $criteria = ' AND ';
        } else {
		  $criteria = ' WHERE ';
        }
        $criteria .= ' (' . $re . '_R0 = 1 OR ' . $re . '_R9 >= ' . singleQuote(adodb_date('Y-m-d', $ts)) . ')';
        if (strlen($ex) > CLX_NUM_ZERO) {
		  $criteria .= ' AND ' . $ex . ' <' . singleQuote(adodb_date('Y-m-d', $ts));
        } else {
		  $criteria .= ' AND ' . $fn . '<' . singleQuote(adodb_date('Y-m-d', $ts));
        }
        $criteria .= ' AND ' . $re . '_R1 NOT IN (' . singleQuoteArray(array('NONE', 'ONE')) . ')';
        $criteria .= ' ORDER BY ' . $fn;
        $sql = (string) 'SELECT * ' . $cmd . ' ' . $criteria;
        $results = ClxCommon::$db->executeQuery($sql);
        while ($row = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE)) {
		  switch ($row[$re . '_R1']) {
			 case 'DAY':
			     $row = $this->recurrence_day($calRec, $row, $ts);
			     if (!empty($row)) {
			         $records[] = $row;
			     }
			     break;
			 case 'WEEK':
			     $row = $this->recurrence_week($calRec, $row, $ts);
			     if (!empty($row)) {
			         $records[] = $row;
			     }
			     break;
			 case 'MNTH':
			     $row = $this->recurrence_month($calRec, $row, $ts);
			     if (!empty($row)) {
			         $records[] = $row;
			     }
			     break;
			 case 'YEAR':
			     $row = $this->recurrence_year($calRec, $row, $ts);
			     if (!empty($row)) {
			         $records[] = $row;
			     }
			     break;
			 default:
			     break;
		  }//End switch
        }//end while
		ClxCommon::$db->freeResult( $results );
        return $records;
    }

    function recurrence_day($calRec, $record, $ts = CLX_NUM_ZERO) {
        $fn = $calRec['fieldname'];
        $ex = $calRec['EXTRA_FIELD1'];
        $re = $calRec['RECURRENCE_FIELD'];
        $dateRec = adodb_getdate($ts);
        $type = ($record[$re . '_R3']) ? 'day' : 'weekdays';
        $until = (empty($record[$re . '_R0'])) ? CLX_NUM_ZERO : adodb_strtotime($record[$re . '_R9']);
        $every = $record[$re . '_R2'];
        $rdateRec = adodb_getdate(adodb_strtotime($record[$fn]));
        $daydif = $dateRec['yday'] - $rdateRec['yday'];
        if (strlen($record[$ex]) > CLX_NUM_ZERO) {
		  $shours = ((adodb_strtotime($record[$fn])) / CLX_NUM_60) / CLX_NUM_60;
		  $ehours = ((adodb_strtotime($record[$ex])) / CLX_NUM_60) / CLX_NUM_60;
		  $duration = ceil(($ehours - $shours) / CLX_NUM_24);
        } else {
		  $duration = CLX_NUM_ONE;
        }
        switch ($type) {
		  case 'day':
			 //If recurrence until is less than today do nothing
			 if ($until > CLX_NUM_ZERO && $ts > $until)
			     break;
			 // Test if more than one day
			 $extra = CLX_INIT_ZERO;
			 //todo verificar porque $every==0 arrojaba un warning division por 0
			 if ($every <> CLX_NUM_ZERO && $duration > CLX_NUM_ONE)
			     $extra = ($duration - ($daydif % $every));
			 if ($every == CLX_NUM_ZERO || $daydif % $every == CLX_NUM_ZERO || $extra > CLX_NUM_ZERO) {
			     $record = $this->increaseDates($calRec, $record, $ts, CLX_NUM_ZERO, CLX_NUM_ZERO, $daydif, $extra, $duration);
			 } else {
			     $record = false;
			 }
			 break;
		  case 'weekdays':
			 if ($dateRec['wday'] <> CLX_NUM_ZERO && $dateRec['wday'] <> CLX_NUM_SIX) {
			     $record = $this->increaseDates($calRec, $record, $ts, CLX_NUM_ZERO, CLX_NUM_ZERO, $daydif);
			 } else {
			     $record = false;
			 }
			 break;
		  default:
			 break;
        }
        return $record;
    }

//End Method
    function recurrence_week($calRec, $record, $ts = CLX_NUM_ZERO) {
        $fn = $calRec['FIELDNAME'];
        $re = $calRec['RECURRENCE_FIELD'];
        $dateRec = adodb_getdate($ts);
        $every = $record[$re . '_R4'];
        $weekdays = ($record[$re . '_R7'] == CLX_NUM_7) ? CLX_NUM_ZERO : $record[$re . '_R7']; //List has sunday = 7
        $rdateRec = adodb_getdate(adodb_strtotime($record[$fn]));
        //Same day setting
        if ($record[$re . '_R7'] == CLX_NUM_8 && $dateRec['wday'] == $rdateRec['wday']) {
		  $weekdays = $dateRec['wday'];
        }
        $tsweek = floor($dateRec['yday'] / CLX_NUM_7);
        $recweek = floor($rdateRec['yday'] / CLX_NUM_7);
        if (($tsweek > $recweek) && (($tsweek - $recweek) % $every == CLX_NUM_ZERO) && ($weekdays == $dateRec['wday'])) {
		  $monthdif = $dateRec['mon'] - $rdateRec['mon'];
		  $record = $this->increaseDates($calRec, $record, $ts, CLX_NUM_ZERO, $monthdif, CLX_NUM_ZERO);
        } else {
		  $record = false;
        }
        return $record;
    }

//End Method
    function recurrence_month($calRec, $record, $ts = CLX_NUM_ZERO) {
        $fn = $calRec['fieldname'];
        $re = $calRec['RECURRENCE_FIELD'];
        $dateRec = adodb_getdate($ts);
        $every = $record[$re . '_R5'];
        $type = $record[$re . '_R6'];
        $rdateRec = adodb_getdate(adodb_strtotime($record[$fn]));
        $monthdif = abs($dateRec['mday'] - $rdateRec['mday']);
        switch ($type) {
		  case '1': //Date, ex. every 22 of the month
			 if (($dateRec['mday'] == $rdateRec['mday']) && $monthdif % $every == CLX_NUM_ZERO) {
			     $record = $this->increaseDates($calRec, $record, $ts, CLX_NUM_ZERO, $monthdif, CLX_NUM_ZERO);
			 } else {
			     $record = false;
			 }
			 break;
		  case '0': //Day, ex. every 1st friday of the month
			 $rdateWeek = floor(($rdateRec['mday'] - CLX_NUM_ONE) / CLX_NUM_7);
			 $dateWeek = floor(($dateRec['mday'] - CLX_NUM_ONE) / CLX_NUM_7);
			 if (($rdateWeek == $dateWeek) && ($dateRec['wday'] == $rdateRec['wday']) && $monthdif % $every == CLX_NUM_ZERO) {
			     $record = $this->increaseDates($calRec, $record, $ts, CLX_NUM_ZERO, $monthdif, CLX_NUM_ZERO);
			 } else {
			     $record = false;
			 }
			 break;
		  default:
			 break;
        }
        return $record;
    }

//End Method
    function recurrence_year($calRec, $record, $ts = CLX_NUM_ZERO) {
        $fn = $calRec['fieldname'];
        $re = $calRec['RECURRENCE_FIELD'];
        $dateRec = adodb_getdate($ts);
        $every = $record[$re . '_R8'];
        $rdateRec = adodb_getdate(adodb_strtotime($record[$fn]));
        $yeardif = $dateRec['year'] - $rdateRec['year'];
        if ($dateRec['mon'] . '-' . $dateRec['mday'] == $rdateRec['mon'] . '-' . $rdateRec['mday']) {
		  $daytest = true;
        } else {
		  $daytest = false;
        }
        if ($yeardif > CLX_NUM_ZERO && ($yeardif % $every == CLX_NUM_ZERO) && $daytest) {
		  $record = $this->increaseDates($calRec, $record, $ts, $yeardif, CLX_NUM_ZERO, CLX_NUM_ZERO);
        } else {
		  $record = false;
        }
        return $record;
    }

//End Method
    function increaseDates($calRec, $record, $ts, $yinc = CLX_NUM_ZERO, $minc = CLX_NUM_ZERO, $dinc = CLX_NUM_ZERO, $extra = CLX_NUM_ZERO, $duration = CLX_NUM_ZERO) {
        if ($extra > CLX_NUM_ZERO) {
		  $dif = $duration - $extra;
        } else {
		  $dif = CLX_INIT_ZERO;
        }
        if ($dinc - $dif < CLX_NUM_ZERO) {
		  $dif = CLX_INIT_ZERO;
        }
        $rdateRec = adodb_getdate(adodb_strtotime($record[$calRec['FIELDNAME']]));
        $record[$calRec['FIELDNAME']] = adodb_date('Y-m-d H:i:s', adodb_mktime($rdateRec['hours'], $rdateRec['minutes'], $rdateRec['seconds'], ($rdateRec['mon'] + $minc), ($rdateRec['mday'] + ($dinc - $dif)), ($rdateRec['year'] + $yinc)));
        if (strlen($calRec['EXTRA_FIELD1']) > CLX_NUM_ZERO) {
		  $rdateRec = adodb_getdate(adodb_strtotime($record[$calRec['EXTRA_FIELD1']]));
		  $record[$calRec['EXTRA_FIELD1']] = adodb_date('Y-m-d H:i:s', adodb_mktime($rdateRec['hours'], $rdateRec['minutes'], $rdateRec['seconds'], ($rdateRec['mon'] + $minc), ($rdateRec['mday'] + ($dinc - $dif)), ($rdateRec['year'] + $yinc)));
        }
        unset($ts);
        return $record;
    }

    /**
     * Return next run date for a record
     *
     */
    function nextRun() {

    }

    /**
     * Format date into formated time
     * @parameters: integer hour 0-23
     */
    function thisHour($hour = CLX_NUM_ZERO) { // May be deprecated.
        return adodb_date('g A', adodb_mktime($hour));
    }

    function anchorURI() {
        $info = peekHistory();
        $return = (string) UserRequest::$server['PHP_SELF'] . '?to=' . $info[CLX_PROP_TO] . '&ac=' . $info['ac'] . '&pc=' . $info['pc'] . '&';
        return $return;
    }

// C A L E N D A R  M E N U S
    /**
      // Calendar General Menu includes today, week, work week, month, year, listing
     */
    function calMenu($calRec, $ts = CLX_NUM_ZERO) {
        $table = new htmlTable(CLX_WIDTH_SIX);
        $table->setProps(array('cellspacing' => '3', CLX_PROP_WIDTH => '95%', CLX_PROP_ALIGN => 'center'));
        $anchor = new htmlAnchor(dict('UICA_LBL_DAY'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode('day'));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        $anchor = new htmlAnchor(dict('UICA_LBL_WEEK'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode('week'));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        $anchor = new htmlAnchor(dict('UICA_LBL_WORKWEEK'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode('work'));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        $anchor = new htmlAnchor(dict('UICA_LBL_MONTH'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode('mont'));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        $anchor = new htmlAnchor(dict('UICA_LBL_YEAR'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode('year'));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        $anchor = new htmlAnchor(dict('UICA_LBL_LISTING'), $this->anchorURI() . '&dt' . $calRec['ID'] . 't=' . base64_encode(CLX_CLASS_LIST));
        $table->addcell($anchor->source(), array(CLX_PROP_CLASS => 'day_title', 'cellpading' => '2'));
        return $table->source();
    }

    /**
     * Build navigation menu for calendars next day, previous day, today button, etc
     * @parameters: String
     * @return string;
     */
    function calNavMenu($calRec, $ts, $view = '') {
        $contents = (string) '';
        if (empty($ts))
		  $ts = adodb_strtotime('now');
        $cTs = (int) CLX_NUM_ZERO;
        // Get date information
        $dateRec = adodb_getdate($ts);
        // Build Header section
        $buttonTable = new htmlTable(CLX_WIDTH_FIVE);
        $buttonTable->setProps(array(CLX_PROP_ALIGN => CLX_PROP_CENTER));
        $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . adodb_mktime();
        $anchor = new htmlButtonGoto(dict('UICA_LBL_today'), $dt);
        $buttonTable->addcell($anchor->source(), array(CLX_PROP_CLASS => 'month_title', CLX_PROP_COLSPAN => CLX_NUM_FIVE));
        $cTs = CLX_INIT_ZERO;
        if ($view === 'month') {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, ($dateRec['mon'] - CLX_NUM_ONE), $dateRec['mday'], $dateRec['year']);
        } elseif ($view === 'year') {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $dateRec['mday'], ($dateRec['year'] - CLX_NUM_ONE));
        } elseif ($view === 'week' || $view === 'workweek') {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $dateRec['mday'], ($dateRec['year'] - CLX_NUM_ONE));
        } else {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], ($dateRec['mday'] - CLX_NUM_ONE), $dateRec['year']);
        }
        $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . $cTs;
        $buttonTable->addcell(getSkinIcon('UICA_prevpagen', $dt, 'Previous'), array(CLX_PROP_CLASS => 'month_title'));
        // Creation jump year
        $select = new htmlSelect('chooseYear');
        //$select->setProp('onchange', 'window.location=document.getElementsByName("chooseYear")[0].value');
        $select->setProp('onchange', 'window.location=document.getElementsByName("chooseYear").value');
        $y = (int) adodb_date('Y') - CLX_NUM_TWO;
        $x = (int) adodb_date('Y', $ts);
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_15; $i++) {
		  $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], ($dateRec['mday']), $y);
		  $select->addItem($y, $dt, (($x == $y) ? true : false));
		  $y++;
        }
        $buttonTable->addcell($select->source(), array(CLX_PROP_CLASS => 'month_title', 'valign' => 'center'));
        // Creating jump month
        $select = new htmlSelect('chooseMonth');
        //$select->setProp('onchange', 'window.location=document.getElementsByName("chooseMonth")[0].value');
        $select->setProp('onchange', 'window.location=document.getElementsByName("chooseMonth").value');
        for ($i = CLX_NUM_ONE; $i <= CLX_NUM_12; $i++) {
		  $cTs = mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $i, '01', $dateRec['year']);
		  $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . $cTs;
		  $select->addItem(dict('UICA_LBL_MONTH_' . adodb_date('m', $cTs)), $dt, ((adodb_date('m', $ts) == $i) ? true : false));
        }
        $buttonTable->addcell($select->source(), array(CLX_PROP_CLASS => 'month_title', 'valign' => 'center'));
        // Creating jump day
        $select = new htmlSelect('chooseDay');
        //$select->setProp('onchange', 'window.location=document.getElementsByName("chooseDay")[0].value');
        $select->setProp('onchange', 'window.location=document.getElementsByName("chooseDay").value');
        $x = adodb_date('t', $this->ts);
        for ($i = CLX_NUM_ONE; $i <= $x; $i++) {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $i, $dateRec['year']);
		  $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . $cTs;
		  $select->addItem($i, $dt, (($i == $dateRec['mday']) ? true : false));
        }
        $buttonTable->addcell($select->source(), array(CLX_PROP_CLASS => 'month_title', 'valign' => 'center'));
        // Adding Next
        $cTs = CLX_INIT_ZERO;
        if ($view === 'month') {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, ($dateRec['mon'] + CLX_NUM_ONE), $dateRec['mday'], $dateRec['year']);
        } elseif ($view === 'year') {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], $dateRec['mday'], ($dateRec['year'] + CLX_NUM_ONE));
        } else {
		  $cTs = adodb_mktime(CLX_NUM_ZERO, CLX_NUM_ZERO, CLX_NUM_ZERO, $dateRec['mon'], ($dateRec['mday'] + CLX_NUM_ONE), $dateRec['year']);
        }
        $dt = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . $cTs;
        $buttonTable->addcell(getSkinIcon('UICA_nextpagen', $dt, 'Next'), array(CLX_PROP_CLASS => 'month_title'));
        $contents = $buttonTable->source();
        return $contents;
    }

    /**
     * Prepare calendar item description with edit buttons and all requirements
     * @parameters: String
     * @return string;
     */
    function displayCalItem($calRec, $data, $li, $qy, $size = CLX_NUM_15, $view = '', $ts = CLX_NUM_ZERO) {
        $contents = (string) '';
        $url1 = 'tool.php?to=EDIT&ac=MODIFY&li=' . $li . '&qy=' . $qy . '&idlist=' . $data['ID'];
        $contents = getSkinIcon('UICA_modify', $url1, 'MODIFY') . ' ';
        $url2 = 'tool.php?to=EDIT&ac=DELETE&li=' . $li . '&qy=' . $qy . '&idlist=' . $data['ID'];
        $contents .=getSkinIcon('UICA_delete', $url2, 'DELETE');
        if (strlen($data[CLX_CLASS_TITLE]) > CLX_NUM_ZERO) {
		  $url = $url1;
		  if ($view === 'month' || $view === 'week' || $view === 'workweek') {
			 $url = $this->anchorURI() . 'dt' . $calRec['ID'] . '=' . $ts . '&dt' . $calRec['ID'] . 't=' . base64_encode('day');
		  }
		  $anchor = new htmlAnchor(substr($data[CLX_CLASS_TITLE], CLX_NUM_ZERO, $size), $url);
		  $prop = (string) '';
		  $prop .= 'tooltipLink(\'&lt;pre style=&quot;border: 1px; ';
		  $prop .= 'border-style:solid;background-color:#FFFFFF;margin:1px;';
		  $prop .= '&quot;<b>' . $data[CLX_CLASS_TITLE] . '</b><br>' . $data['sdate'] . '<br>' . $data['edate'] . '&lt;/pre&gt;\', ' . singleQuote($data[CLX_CLASS_TITLE]) . ');return true;';
		  $anchor->setProp('onmouseover', $prop);
		  $anchor->setProp('onmouseout', 'tooltipClose();');
		  $contents .=$anchor->source();
        }
        return $contents;
    }

    /**
     * Get session information for calendar.
     * @parameters: String
     * @return string;
     */
    function calendarSession($record) {
        //Set Session history
        if (isset($_SESSION['calendar'][$record['IDCODE']])) {
		  $this->ts = $_SESSION['calendar'][$record['IDCODE']];
        } else {
		  $_SESSION['calendar'][$record['IDCODE']] = $this->ts;
        }
        if (!empty(UserRequest::$request['dt' . $record['ID']])) {
		  $this->ts = UserRequest::$request['dt' . $record['ID']];
		  $_SESSION['calendar'][$record['IDCODE']] = $this->ts;
        }
        // Calendar Type
        if (isset($_SESSION['calendar'][$record['IDCODE'] . 't'])) {
		  $this->viewType = $_SESSION['calendar'][$record['IDCODE'] . 't'];
        } else {
		  $_SESSION['calendar'][$record['IDCODE'] . 't'] = $this->viewType;
        }
        if (!empty(UserRequest::$request['dt' . $record['ID'] . 't'])) {
		  $this->viewType = base64_decode(UserRequest::$request['dt' . $record['ID'] . 't']);
		  $_SESSION['calendar'][$record['IDCODE'] . 't'] = $this->viewType;
        }
        return $this->ts;
    }

    function weekDays($day, $type = 'short') {
        $return = (string) '';
        $days = array();
        switch ($type) {
		  case 'short':
			 $days[] = 'UICA_LBL_SUNDAY_SHORT';
			 $days[] = 'UICA_LBL_MONDAY_SHORT';
			 $days[] = 'UICA_LBL_TUESDAY_SHORT';
			 $days[] = 'UICA_LBL_WEDNESDAY_SHORT';
			 $days[] = 'UICA_LBL_THRUESDAY_SHORT';
			 $days[] = 'UICA_LBL_FRIDAY_SHORT';
			 $days[] = 'UICA_LBL_SATURDAY_SHORT';
			 break;
		  case 'long':
			 $days[] = 'UICA_LBL_SUNDAY';
			 $days[] = 'UICA_LBL_MONDAY';
			 $days[] = 'UICA_LBL_TUESDAY';
			 $days[] = 'UICA_LBL_WEDNESDAY';
			 $days[] = 'UICA_LBL_THRUESDAY';
			 $days[] = 'UICA_LBL_FRIDAY';
			 $days[] = 'UICA_LBL_SATURDAY';
			 break;
		  default:
			 break;
        }
        if (isset($days[$day]))
		  $return = dict($days[$day]);
        return $return;
    }

    function loadRequest($record) {
        if (!empty(UserRequest::$request['cal'])) {
		  $sql = 'SELECT UILG.TABLECODE, CALC.* FROM CLX_UI_CALENDAR CALC, CLX_UI_LISTINGS UILG WHERE CALC.LISTINGCODE = UILG.IDCODE AND CALC.IDCODE = ' . singleQuote(UserRequest::$request['cal']);
		  $results = ClxCommon::$db->executeQuery($sql);
		  if ($results === true) {
			 $result = ClxCommon::$db->getNextRecord($results, ASSOCIATIVE);
			 if (array_key_exists($result['TABLECODE'] . "'_'" . $result['fieldname'], $record)) {
			     $record[$result['TABLECODE'] . "'_'" . $result['fieldname']] = UserRequest::$request['ds'];
			 }
			 if (array_key_exists($result['TABLECODE'] . "'_'" . $result['EXTRA_FIELD1'], $record)) {
			     $record[$result['TABLECODE'] . "'_'" . $result['EXTRA_FIELD1']] = UserRequest::$request['de'];
			 }
		  }
		  ClxCommon::$db->freeResult( $results );
        }
        return $record;
    }

//End Method
}

//End Class
