<?php

/**
 * Application Form Listing Manager
 * For more information contact  support@alenet.com
 *
 * @package		Clientix.Tool.Interface
 * @abstract		User Interface - Form Listing Criteria
 * @author       $Author: mchirino $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-07-16 13:28:41 -0400 (Mon, 16 Jul 2018) $
 * @version    	$Rev: 9849 $
 *
 * CHANGE LOG
 *  Oracle uppercase record changed
 *  UPDATE CLX_UI_FORM_LISTING_CRITS SET OPERATOR = 'EQ'
 */
class ui_form_listing_criteria extends clx_tool {

    /**
     * Constructor
     */
	var $size;
    var $displaysize; 
    function __construct() {
        $this->code = 'UIFI';
        $this->name = 'Form Listing Limiters';
        $this->system = 'Interface';
        $this->version = '0.2.1.1';
        $this->accesskey = 'UIFM';
        $this->tablename = 'CLX_UI_FORM_LISTING_CRITS';
        $this->record_name_singular = 'listing criteria';
        $this->record_name_plural = 'listing criterias';
        $this->primary_field = 'FORMLISTINGID';
        $this->secondary_field = 'FIELDNAME1';
        $this->primary_key = array('FORMLISTINGID', 'FIELDNAME1');
		$this->size = isset($data['size']) ? $data['size'] : CLX_NUM_20;
        $this->displaysize = isset($data['displaysize']) ? $data['displaysize'] : CLX_NUM_20;
    }

    function toolTableStructure() {
        $fields = array();
        $fields['ID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => 'auto_increment', 'index' => 'PRI');
        $fields['FORMLISTINGID'] = array('type' => 'int(11) unsigned', 'null' => 'NO', 'properties' => '', 'default' => CLX_NUM_ZERO);
        $fields['FIELDNAME1'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['OPERATOR'] = array('type' => 'varchar(4)', 'null' => 'NO', 'default' => '' . singleQuote('E') . '', 'default_raw' => 'EQ', 'properties' => '');
        $fields['FIELDNAME2'] = array('type' => 'varchar(64)', 'null' => 'NO', 'properties' => '');
        $fields['VALUE'] = array('type' => 'varchar(255)', 'null' => 'YES', 'properties' => '');
        $indexes = array();
        $ipf = 'CLX' . $this->code;
        $indexes[$ipf . '01'] = array('index' => 'PRI', 'fieldname' => 'ID');
        $indexes[$ipf . '02'] = array('index' => 'MUL', 'fieldname' => 'FORMLISTINGID');
        $indexes[$ipf . '03'] = array('index' => 'MUL', 'fieldname' => 'FIELDNAME1');
        return array('idcode' => $this->code, 'idname' => $this->name, 'name' => $this->tablename, 'type' => 'InnoDB',
		  'fields' => $fields,
		  'indexes' => $indexes);
    }

    function unpack($parameters) {
        $parameters = array_values(array_merge(array('package' => null, 'id' => ''), $parameters));
        list($package, $formid) = $parameters;
	    $record = $this->unpackRecord($package);
        $record['FORMLISTINGID'] = $formid;
        $record['FIELDNAME1'] = uppercase($record['FIELDNAME1']);
        $record['FIELDNAME2'] = uppercase($record['FIELDNAME2']);
        ClxCommon::$db->createRecord($this->tablename, $record, false);
    }

    function makepanel($groupcode = null) {
        $formlistingid = $groupcode;
        if ($this->getAccess()->canRead()) {
		  $t = time();
		  $to = $this->code;
		  $here = 'tool.php?to=' . $to . '&t=' . $t;
		  $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
		  $form->setProp(CLX_PROP_ID, 'UIFIPanel');
		  $form->addArgument(CLX_PROP_TO, $this->code);
		  $form->addArgument('formcode', ''); // Verify formcode
		  $form->addArgument('ac', 'NOT_DEFINED');
		  $selectclause = '*';
		  $fromclause = 'CLX_UI_FORM_LISTING_CRITS';
			$whereclause = 'FORMLISTINGID=' . singleQuote($formlistingid);
			$sortclause = '';
		  $page = new xPaging();
		  $page->setProp('rpp', CLX_NUM_TEN_THOUSAND);
		  $page->setProp('printheaders', false);
		  $page->pageInit($form->getProp(CLX_PROP_ID), $here, $selectclause, $fromclause, $whereclause, $sortclause, '');
		  $table = new htmlTable(CLX_WIDTH_FIVE);
		  $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_LIST, CLX_PROP_WIDTH => CLX_PROP_100PCT));
		  $page->header($table);
		  //$table->addCells(CLX_CLASS_HEADERS,array( dict('GEN_LBL_SEL'),  dict('GEN_LBL_FIELDNAME'), dict('GEN_LBL_VALUE')));
		  $table->addCells(CLX_CLASS_HEADERS, array(dict('GEN_LBL_SEL'), dict('GEN_LBL_QRY_FIELDNAME'), dict('GEN_LBL_OPERATOR'), dict('GEN_LBL_FRM_FIELDNAME'), dict('GEN_LBL_VALUE')));
		  $i = CLX_INIT_ZERO;
		  while ($record = $page->nextRecord()) {
			 $recordid = $record['ID'];
			 $select = new htmlCheckbox('selector_' . $recordid, false);
			 $select->setProp('onclick', 'selectButtons(' . singleQuote($form->getProp(CLX_PROP_ID)) . ');');
			 $table->addCell($select->source(), array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_WIDTH => '40', CLX_PROP_ALIGN => 'center'));
			 $table->addCells(($i % CLX_NUM_TWO == CLX_NUM_ZERO) ? CLX_CLASS_DATA : CLX_CLASS_DATAALT, array($record['FIELDNAME1'], dict($this->operators[$record['OPERATOR']][CLX_NUM_ZERO]), $record['FIELDNAME2'], $record['VALUE']));
			 $i++;
		  }
		  $buttons = '';
		  if ($this->getAccess()->canDisplay()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_CREATE'), array('action' => 'CREATE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'ON'))->source();
		  }
		  if ($this->getAccess()->canModify()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_MODIFY'), array('action' => 'MODIFY', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  if ($this->getAccess()->canDelete()) {
			 $buttons .=htmlButton::getButton('submit', dict('GEN_LBL_DELETE'), array('action' => 'DELETE', 'form' => $form->getProp(CLX_PROP_ID), CLX_PROP_ID => 'MULT'))->source();
		  }
		  $form->addContents($page->footers($table, $buttons));
		  return buildPanelInterface(dict('TOOL_UIFI_TITLE_LIST'), 'FORMLISTINGSFOR' . $formlistingid, $form->source(), CLX_NUM_MINUS_1);
        } else {
		  return CLX_CHAR_EMPTY_STRING;
        }
    }

    function editForm($ac, $idlist, $record, $msg) {IF ($GLOBALS['ADDRDEBUGEDITFORM_']==1) { ECHO '__METHOD____FILE____LINE__'. __METHOD__.' '.__FILE__.':'.__LINE__.'<br>'; }
        $formlistingrec = ClxCommon::$db->getRecord('CLX_UI_FORM_LISTINGS', 'ID=' . $record['UIFI_formlistingid']);
        $listingcode = $formlistingrec['LISTINGCODE'];
	    $formcode = $formlistingrec['FORMCODE'];
      $listingrec = ClxCommon::$db->getRecord('CLX_UI_LISTINGS', 'IDCODE=' . singleQuote($listingcode));
        $querycode = $listingrec['QUERYCODE'];
        // Query Fields -------------------------
        switch (kDatabaseType) {
		  case 'oci':
			 $sql = 'SELECT T.MODULECODE||' . singleQuote('_') . '||t.IDNAME||' . singleQuote('.') . '||TF.FIELDNAME FIELDNAME, T.IDCODE||' . singleQuote('.') . '||TF.FIELDNAME FIELDCODE FROM CLX_DB_QUERIES Q, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF, CLX_DB_TABLES T WHERE Q.IDCODE=' . singleQuote($querycode) . ' AND Q.IDCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND TF.TABLECODE = T.IDCODE ORDER BY QT.TABLECODE ASC, TF.SEQNUM ASC';
			 break;
		  default:
			 $sql = 'SELECT CONCAT(T.MODULECODE, CONCAT(' . singleQuote('_') . ', CONCAT(T.IDNAME, CONCAT(' . singleQuote('.') . ', TF.FIELDNAME)))) FIELDNAME, '
			     . 'CONCAT(T.IDCODE, CONCAT(' . singleQuote('.') . ', TF.FIELDNAME)) FIELDCODE '
			     . 'FROM CLX_DB_QUERIES Q, CLX_DB_QUERY_TABLES QT, CLX_DB_TABLEFIELDS TF, CLX_DB_TABLES T WHERE Q.IDCODE=' . singleQuote($querycode) . ' AND Q.IDCODE = QT.QUERYCODE AND QT.TABLECODE = TF.TABLECODE AND TF.TABLECODE = T.IDCODE ORDER BY QT.TABLECODE ASC, TF.SEQNUM ASC';
			 break;
        }//End switch

        $fields = ClxCommon::$db->executeQuery($sql);
        $queryfields = new htmlSelect('UIFI_fieldname1');
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
		  $queryfields->addItem(
			$field['FIELDNAME'], $field['FIELDCODE'], ($record['UIFI_fieldname1'] == $field['FIELDCODE']));
        }
		ClxCommon::$db->freeResult( $fields);
        // Form Fields -------------------------
        switch (kDatabaseType) {
		  case 'oci':
			 $sql = ' SELECT M.IDCODE||' . singleQuote('_') . '||T.IDNAME||' . singleQuote('.') . '||TB.FIELDNAME FIELDNAME,  '
			     . 'TB.TABLECODE||' . singleQuote('_') . '||TB.FIELDNAME FIELDCODE FROM CLX_UI_FORMS F, CLX_DB_TABLEFIELDS TB, CLX_DB_TABLES T, CLX_DB_MODULES M WHERE F.IDCODE=' . singleQuote($formcode) . ' AND F.TABLECODE = TB.TABLECODE AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE ORDER BY TB.SEQNUM ASC';
			 break;
		  default:
			 $sql = ' SELECT CONCAT(M.IDCODE, CONCAT(' . singleQuote('_') . ', CONCAT(T.IDNAME, CONCAT(' . singleQuote('.') . ', TB.FIELDNAME)))) FIELDNAME, '
			     . 'CONCAT(TB.TABLECODE, CONCAT(' . singleQuote('_') . ', TB.FIELDNAME)) FIELDCODE '
			     . 'FROM CLX_UI_FORMS F, CLX_DB_TABLEFIELDS TB, CLX_DB_TABLES T, CLX_DB_MODULES M WHERE F.IDCODE=' . singleQuote($formcode) . ' AND F.TABLECODE = TB.TABLECODE AND TB.TABLECODE = T.IDCODE AND T.MODULECODE = M.IDCODE ORDER BY TB.SEQNUM ASC';
			 break;
        }//End switch
        $fields = ClxCommon::$db->executeQuery($sql);
        $formfields = new htmlSelect('UIFI_fieldname2');
        while ($field = ClxCommon::$db->getNextRecord($fields)) {
		  $formfields->addItem($field['FIELDNAME'], $field['FIELDCODE'], ($record['UIFI_fieldname2'] == $field['FIELDCODE']));
        }
		ClxCommon::$db->freeResult( $fields);
        $title = dict('TOOL_UIFI_TITLE_CREATE');
        $form = new htmlForm(CLX_PROP_ACTION, 'tool.php');
        $form->addArgument(CLX_PROP_TO, $this->code);
        $form->addArgument('ac', $ac);
        $form->addArgument('confirm', '1');
        $form->addArgument('UIFI_id', $record['UIFI_id']);
        $form->addArgument('idlist', toString($idlist));
        $form->addArgument('UIFI_formlistingid', $record['UIFI_formlistingid']);
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $table->addCell($title, array(CLX_PROP_CLASS => CLX_CLASS_TITLE, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
		$table->addData(dict('TOOL_UIFI_PROMPT_QUERYFIELD') . ':', $queryfields->source());
        // The operators
        $pop = new HTMLSelect($this->code . '_operator');
        foreach ($this->operators as $key => $value) {
		  $pop->addItem(dict($value[CLX_NUM_ZERO]), $key, ($key == $record[$this->code . '_operator']));
        }
        $table->addData(dict('GEN_LBL_OPERATORS') . ':', $pop->source());
        //$table->addData(dict('TOOL_UIFI_PROMPT_FORMFIELD') . ':', $formfields->source());
        $field = new htmlTextArea('UIFI_value', $record['UIFI_value'], CLX_NUM_7, CLX_NUM_50);
        if (!empty($msg[$this->code . '_frm_fieldname']))
		  $table->addCell($msg[$this->code . '_frm_fieldname'], array(CLX_PROP_CLASS => CLX_CLASS_ALERT, CLX_PROP_COLSPAN => $table->width));
        $table->addData(dict('TOOL_UIFI_PROMPT_FORMFIELDORVALUE') . ':', $formfields->source() . '<br>' . dict('GEN_LBL_OR') . '<br>' . $field->source());
        $form->addContents($table->source());
        $buttons = '';
        $table = new htmlTable(CLX_WIDTH_TWO);
        $table->setProps(array(CLX_PROP_CLASS => CLX_CLASS_FORM, CLX_PROP_WIDTH => CLX_PROP_100PCT));
        $buttons .= htmlButton::getButton('submit', dict('GEN_LBL_SAVE'), array('action' => $ac, 'form' => $form->getProp(CLX_PROP_ID)))->source();
        $c = new htmlButtonCancel();
        $buttons .= $c->source();
        $table->addCell($buttons, array(CLX_PROP_CLASS => CLX_CLASS_FOOTERS, CLX_PROP_COLSPAN => CLX_WIDTH_TWO));
        $form->addContents($table->source());
        return $form->source();
    }

    function checkForm(&$msg) {
        $msg = array();
		return count($msg) == CLX_NUM_ZERO;
    }

    function confirmDeleteFill(&$table, $record) {
        $table->addData(dict('GEN_LBL_CRITERIA') . ':', $record['UIFI_fieldname1'] . ' = ' . $record['UIFI_fieldname2']);
    }

    function loadToolRecord($recordid) {
        $values = ClxCommon::$db->getRecord($this->tablename, 'ID= ' . $recordid);
        $record = array();
        $record['UIFI_id'] = $recordid;
        $record['UIFI_formlistingid'] = $values['FORMLISTINGID'];
        $record['UIFI_fieldname1'] = $values['FIELDNAME1'];
        $record['UIFI_fieldname2'] = $values['FIELDNAME2'];
        $record['UIFI_operator'] = $values['OPERATOR'];
        $record['UIFI_value'] = $values['VALUE'];
        return $record;
    }

    function createRecord_() {
		//cambio para incluir los campos donde embasurado el post por funcion sanatize (quote o singlequote)
		UserRequest::$post['UIFI_value']=str_replace('&quot;','"', UserRequest::$post['UIFI_value']);
		UserRequest::$post['UIFI_value']=str_replace('&#039;',"'", UserRequest::$post['UIFI_value']);
        $record = array();
		$record['ID'] = 0;
		$record['FORMLISTINGID'] = UserRequest::$post['UIFI_formlistingid'];
        $record['FIELDNAME1'] = UserRequest::$post['UIFI_fieldname1'];
        $record['FIELDNAME2'] = UserRequest::$post['UIFI_fieldname2'];
        $record['OPERATOR'] = UserRequest::$post['UIFI_operator'];
        $record['VALUE'] = UserRequest::$post['UIFI_value'];
        $recordid = ClxCommon::$db->createRecord($this->tablename, $record, false);
        $UIFL = &getTool('UIFL');
        $UIFL->versionIncrease('ID', UserRequest::$post['UIFI_formlistingid']);
        return $recordid;
    }

    function saveRecord() {
		//cambio para incluir los campos donde embasurado el post por funcion sanatize (quote o singlequote)
		UserRequest::$post['UIFI_value']=str_replace('&quot;','"', UserRequest::$post['UIFI_value']);
		UserRequest::$post['UIFI_value']=str_replace('&#039;',"'", UserRequest::$post['UIFI_value']);
        $record = array();
		$record['FORMLISTINGID'] = UserRequest::$post['UIFI_formlistingid'];
        $record['FIELDNAME1'] = UserRequest::$post['UIFI_fieldname1'];
        $record['FIELDNAME2'] = UserRequest::$post['UIFI_fieldname2'];
        $record['OPERATOR'] = UserRequest::$post['UIFI_operator'];
        $record['VALUE'] = UserRequest::$post['UIFI_value'];
        ClxCommon::$db->updateRecord($this->tablename, $record, 'ID=' . UserRequest::$post['UIFI_id']);
        $UIFL = &getTool('UIFL');
        $UIFL->versionIncrease('ID', UserRequest::$post['UIFI_formlistingid']);
    }

    function deleteRecordSk($id) {
        $record = ClxCommon::$db->getRecord($this->tablename, 'ID=' . $id);
        ClxCommon::$db->deleteRecords($this->tablename, 'ID=' . $id);
        $UIFL = &getTool('UIFL');
        $UIFL->versionIncrease('ID', $record['FORMLISTINGID']);
    }

    function defaultRecord() {
        $callingpage = peekHistory();
        $record = array();
        $record['UIFI_id'] = '';
        $record['UIFI_formlistingid'] = $callingpage['fl'];
        $record['UIFI_fieldname1'] = '';
        $record['UIFI_fieldname2'] = '';
        $record['UIFI_operator'] = '';
        $record['UIFI_value'] = '';
        return $record;
    }

    function versionIncrease($fieldname, $fieldvalue) {
        parent::versionIncrease($fieldname, $fieldvalue);
		$modulecode = ClxCommon::$db->getField($this->tablename, 'modulecode', $fieldname . '=' . singleQuote($fieldvalue));
        $DBMO = &getTool('DBMO');
        $DBMO->versionIncrease('IDCODE', $modulecode);
    }

}
