<?php

/**
 * CRM Account interface (CLX <--> IBS)
 * For more information contact  support@alenet.com
 *
 * @author       $Author: cramirez $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2018-08-23 18:18:58 -0400 (Thu, 23 Aug 2018) $
 * @version        $Rev: 9902 $
 *
 * CHANGE LOG
 *  Included svn prop
 *
 * Record of changes
 * 			Desarrollo inicial para BT - Caso 1000787 - Exportación de Clientes a IBS c creac y modif
 * 			mejoras para Caja de Ahorros (panama) - Caso 1001335 - CA-Analisis nueva interfaz CLX-IBS y/o modif.exist
 * 			Inclusión de todos los metodos que tienen que ver con import/export de cuentas CRM, se trajeron de EIBS_utilities.php
 * 			Caso 1001370 - CDA-Eliminar validacion entre estado y ciudad (método client_data_validation)
 *
 * General Parameters:
 *
 * ACCO_XPRT_SOCKET_CREATION = IBS comunication type
 * 	1 = use sockets (real time)
 * 	0 = use batch file (mass processing)
 * DEFAULT_COMPANY = default company code
 * EIBS_LST_ACCO_XPRT_DATE = last date of crm accounts synchronization
 * EIBS_CLIEX_FILESPEC_I = nombre de archivo para export de personas naturales
 * EIBS_CLIEX_FILESPEC_C = nombre de archivo para export de personas jurídicas
 *
 * Librerías de Código: EIBS_5
 * Motor de Proceso: IBS_CLI_EXP
 * Cronograma: IBS_CLI_EXP
 *
 * Depende de:
 * tools/eb_sockets.php
 * data/code/EIBS_utilities.php
 * data/code/PROC_class_aid.php
 * data/code/EIBS_util_ibiq.php
 */
if (!class_exists('ibs'))
    includeCodeLib('EIBS');

/**
 * e-IBS Client interface common method and properties class
 */
class ibs_client_interface_commons extends ibs {

    var $debug = false;
    var $lang;
    var $username;
    var $html = true;
    var $o_createClient_outputFileHandle = null;
    var $s_cli_scktcode_individual = 'CLX008001';
    var $s_cli_scktcode_company = 'CLX008002';
    var $contents;

    /**
     * Constructor
     *
     * @param text $s_comp_code
     * @param array $parameters
     * @return void
     */
    function __construct($s_comp_code = null, $parameters = array()) {
		parent::__construct($this->s_comp_code, $parameters);
        if (!defined('IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID')) {
            // Tokens to identify clients socket type
            define('IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID', 'CLXNAT');
            define('IBS_MASSIVE_IMPRT_CLI_COMPANY_TOKEN_ID', 'CLXJUR');
            // file extension added to the error report filename
            define('EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT', '.err.html');
        }
        if (!empty($this->debug)) {
            echoLog('IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID=' . IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID);
            echoLog('IBS_MASSIVE_IMPRT_CLI_COMPANY_TOKEN_ID=' . IBS_MASSIVE_IMPRT_CLI_COMPANY_TOKEN_ID);
            echoLog('EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT=' . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT);
        }
    }

// Method End
    /**
     * Assign IBS client code (CUSCUN) to the corresponding CRM account in Clientix
     *
     * @param text $s_idcode
     * @param text $s_ibs_cuscun
     * @return array
     */
    function client_assgn_cuscun($s_idcode, $s_ibs_cuscun) {
        
        $a_response = array();
        $a_response['error'] = false;
        $a_response['errormsg'] = '';
        if ($s_idcode === '') {
            /**
             * GEN_ERR_IBS_CAC_NOIDCODE
             * IBS Client Code assigment error: CRM acc. code empty
             * Error en la asignación del Código de Cliente IBS: el código de la cta. CRM está vacio
             * */
            $a_response['error'] = true;
            $a_response['errormsg'] = dict('GEN_ERR_IBS_CAC_NOIDCODE');
        } else {
            $record = ClxCommon::$db->getRecord($this->acconame, 'IDCODE = ' . singleQuote($s_idcode));
            if (empty($record)) {
                /**
                 * GEN_ERR_IBS_CAC_IDNOTFND
                 * IBS Client Code assigment error: CRM acc. code not found: ^1
                 * Error en la asignación del Código de Cliente IBS: cód. de cta. CRM no existe: ^1
                 * */
                $a_response['error'] = true;
                $a_response['errormsg'] = dict('GEN_ERR_IBS_CAC_IDNOTFND', $s_idcode);
            }
        }
        if (trim($s_ibs_cuscun) === '' || $s_ibs_cuscun == null || $s_ibs_cuscun === '0') {
            /**
             * GEN_ERR_IBS_CAC_NOCUSCUN
             * IBS Client Code assigment error: invalid CUSCUN code (^1)
             * Error en la asignación del Código de Cliente IBS: el código CUSCUN inválido (^1)
             * */
            $a_response['error'] = true;
            if ($a_response['errormsg'] !== '')
                $a_response['errormsg'] .= ', ';
            $a_response['errormsg'] .= dict('GEN_ERR_IBS_CAC_NOCUSCUN', $s_ibs_cuscun);
        }
        if ($a_response['error'] == false) {
            ClxCommon::$db->updateRecord($this->acconame, array('IBS_CUSCUN' => $s_ibs_cuscun), 'IDCODE = ' . singleQuote($s_idcode));
        }
        return $a_response;
    }

// Method End
    /**
     * Write the error log file or records
     *
     * @param text $msg
     * @param boolean $error
     * @param array $a_fields
     * @return void
     */
    function proc_log_write_ibs($msg, $error = true, $a_fields = array()) {
        if ($error === true)
            $this->proc_eng_params->report_exceptions($msg, $a_fields);
    }

    /**
     * Returns report body contents
     *
     * @param void
     * @return text
     */
    function contents() {
        return $this->contents;
    }

    /**
     * Returns report body contents
     *
     * @param void
     * @return text
     */
    function source() {
        return $this->contents;
    }

    /**
     * Returns formatted title
     *
     * @param text $s_titulo
     * @return text
     */
    function titulo($s_titulo) {
        if ($this->html === true) {
            $s_titulo = '<spam class=' . singleQuote('title') . '><b>' . $s_titulo . '</b></spam><br>' . CLX_CHAR_NL . '<BR>' . CLX_CHAR_NL;
        } else {
            $s_titulo = $s_titulo . CLX_CHAR_NL . CLX_CHAR_NL . CLX_CHAR_NL . '';
        }
        return $s_titulo;
    }

// Method End
}

// Class End
/**
 *  CRM Account Massive Export (from CLX to IBS)
 */
class ibs_crm_account_mass_export extends ibs_client_interface_commons {

    var $b_socket_creation = CLX_INIT_ZERO; // 1 = encendido, usa socket; 0= apagado, usa batch file
    var $a_aditional_condition = null;
    var $s_last_export_date;

    /**
     * Constructor
     *
     * @param text $s_comp_code
     * @param array $parameters
     * @return void
     */
    function __construct($s_comp_code = null, $parameters = array()) {
        parent::__construct($s_comp_code, $parameters);
        // Flag to configure socket or batch massive client export
        // 1 = usa socket; 0 = usa batch file
        $xprt_client_socket_creation = companyParameter('ACCO_XPRT_SOCKET_CREATION', $this->s_comp_code);
        if ($xprt_client_socket_creation !== '') {
            $this->b_socket_creation = CLX_NUM_ZERO + $xprt_client_socket_creation;
        } else {
            $this->b_socket_creation = CLX_INIT_ZERO;
        }
        // Extrae la fecha de la última exportación realizada para poder saber cuales registros de ACCO se modificaron desde esa fecha
        $this->s_last_export_date = $this->get_last_export_date();
        // Massive Client Export parameters
        if (!defined('TEMPORARY_OPERATIONS_DIRECTORY')) {
            // Directorios temporales
            define('TEMPORARY_OPERATIONS_DIRECTORY', kPathToWebSite . kPathToTemp);
            define('IBS_MASSIVE_CLIENT_CREATION_FILENAME', TEMPORARY_OPERATIONS_DIRECTORY . 'ibs_mass_cli_creat.txt'); // masivo de clientes
            define('IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME', TEMPORARY_OPERATIONS_DIRECTORY . 'ibs_mass_cli_creat_rpt.html'); // masivo de clientes
            // Definiciones de sockets para los procesos de creación batch de clientes
            define('S_CLI_SCKTCODE_INDIVIDUAL', 'CLXNAT');
            define('S_CLI_SCKTCODE_COMPANY', 'CLXJUR');
            // Receive Mass Client Creation from IBS parameters
            // Titulos reporte
            define('DICT_TIT_INDIVIDUAL', '8D5EB91660868567713B9F631E3ADB15-1'); // Persona Natural
            define('DICT_TIT_CORPORATE', '4a833c1d522139d325119f57368d06a2'); // Persona Jurídica
        }
        if (!empty($this->debug)) {
            echoLog('TEMPORARY_OPERATIONS_DIRECTORY=' . TEMPORARY_OPERATIONS_DIRECTORY);
            echoLog('IBS_MASSIVE_CLIENT_CREATION_FILENAME=' . IBS_MASSIVE_CLIENT_CREATION_FILENAME);
            echoLog('IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME=' . IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME);
            echoLog('S_CLI_SCKTCODE_INDIVIDUAL=' . S_CLI_SCKTCODE_INDIVIDUAL);
            echoLog('S_CLI_SCKTCODE_COMPANY=' . S_CLI_SCKTCODE_COMPANY);
            echoLog('xprt_client_socket_creation=' . $xprt_client_socket_creation);
            echoLog('this->b_socket_creation=' . $this->b_socket_creation);
            echoLog('DICT_TIT_INDIVIDUAL=' . DICT_TIT_INDIVIDUAL);
            echoLog('DICT_TIT_CORPORATE=' . DICT_TIT_CORPORATE);
        }
    }

// Method end
    /*
     * Gets the last export date. If there's no defined parameter, returns current date
     *
     * @param void
     * @return text
     */
    function get_last_export_date() {
        $this->s_last_export_date = trim(companyParameter('EIBS_LST_ACCO_XPRT_DATE', $this->s_comp_code));
        if ($this->s_last_export_date === '' || $this->s_last_export_date == null)
            $this->s_last_export_date = adodb_date('Y-m-d');
        return $this->s_last_export_date;
    }

    /*
     * Record the last export date for future exports reference (incremental export)
     *
     * @param void
     * @return void
     */

    function put_last_export_date() {
        $this->companyParameters_insert_update('EIBS_LST_ACCO_XPRT_DATE', adodb_date('Y-m-d'), $this->s_comp_code);
    }

    /*
     * Creates or updates general parameters
     *
     * @param text $parameter_name
     * @param text $parameter_value
     * @param text $comp_code
     * @return void
     */

    function companyParameters_insert_update($parameter_name, $parameter_value, $comp_code = '') {
        $s_curr_date = adodb_date('Y-m-d');
        $n_record_qty = ClxCommon::$db->countRecords($this->prmtname, 'COMP_CODE=' . singleQuote($comp_code) . ' AND IDNAME=' . singleQuote($parameter_name));
        $a_record = array();
        $a_record['PVALUE'] = $parameter_value;
        $a_record['MODIFIEDBY'] = $this->username;
        $a_record['MDATE'] = $s_curr_date;
        if ($n_record_qty > CLX_NUM_ZERO) {
            // Parameter exists
            ClxCommon::$db->updateRecord($this->prmtname, $a_record, 'COMP_CODE=' . singleQuote($comp_code). ' AND IDNAME=' . singleQuote($parameter_name));
        } else {
            // Parameter doesn't exists
            $a_record['COMP_CODE'] = $comp_code;
            $a_record['IDNAME'] = $parameter_name;
            $a_record['DESCRIPTION'] = '';
            $a_record['CREATEDBY'] = $this->username;
            $a_record['CDATE'] = $s_curr_date;
            ClxCommon::$db->createRecord($this->prmtname, $a_record, false);
        }
    }

    /*
     * eIbs CRM Account Mass Interface excecution entry point
     *
     * @param void
     * @return boolean
     */

    function run() {
        $contenidoHTML = '';
        // Identifica el sub-modulo
        $this->submodule = 'EIBS';
        $this->lang = $this->proc_eng_params->process_parameters['LANGUAGE'];
        $this->username = $this->proc_eng_params->process_parameters['USERNAME'];
        $this->contents = '';
        // Trata de abrir el reporte de salida
        $s_response = $this->open_rpt_process_run(IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME);
        if ($s_response !== '') {
            $contenidoHTML.= '<h1>$s_response</h1>';
            echo $contenidoHTML;
            return false;
        }
        // Si los parametros del proceso no se pasaron debidamente, emite error y sale
        $this->proc_eng_params->set_tablename('ACCO');
        if (!$this->proc_eng_params->verify_parameters()) {
            $this->contents = '<h1>' . $this->proc_eng_params->errormsg . '</h1>';
            $this->o_response_rpt->write($this->proc_eng_params->errormsg);
            $this->o_response_rpt->close();
            return false;
        }
        // Esta variable permite chequear el cambio del tipo de cliente (ruptura de control) para poder escribir
        // 2 archivos distintos segun el tipo de persona ya que contiene el ultimo tipo de persona procesado
        $s_tipo_persona = '';
        // Titulo del reporte
        $s_title = ClxCommon::$db->getField($this->proc_eng_params->prscname, 'IDNAME', 'IDCODE = ' . singleQuote($this->proc_eng_params->process_parameters['SCHEDULECODE']));
        $this->o_response_rpt->write($this->titulo($s_title) . '<BR><BR>' . CLX_CHAR_NL);
        /*
         * GEN_LBL_USER
         * User
         * Usuario
         */
        $this->o_response_rpt->write(dict('GEN_LBL_USER') . ': $this->username<BR>' . CLX_CHAR_NL);
        $DBLT = getTool('DBLT');
		$tindividual=array('0'=>'C','1'=>'I');
//		pre(tindividual);
		$i_proc_counter = CLX_INIT_ZERO;
		foreach ($tindividual as $Ikey=>$Itype) {
			$sql1=' select count(1) FROM '.$this->acconame.' WHERE true AND ISINDIVIDUAL='.$Ikey.' %OTHER_WHERE_CONDITION%';
			$sql2=' SELECT ACCO.* , '.singlequote($Itype).' TIPO_PERSONA FROM ' . $this->acconame . ' ACCO WHERE true AND ISINDIVIDUAL='.$Ikey.' %OTHER_WHERE_CONDITION%';
			if ($this->a_aditional_condition == null) {
				$sql2 = str_replace('%OTHER_WHERE_CONDITION%', '', $sql2);
				$sql1 = str_replace('%OTHER_WHERE_CONDITION%', '', $sql1);
			} else {
				$s_aditional_condition = '';
				reset($this->a_aditional_condition);
				foreach ($this->a_aditional_condition as $name => $val) {
					$s_aditional_condition .= ' AND ' . $val;
				}
				$sql2 = str_replace('%OTHER_WHERE_CONDITION%', $s_aditional_condition, $sql2);
				$sql1 = str_replace('%OTHER_WHERE_CONDITION%', $s_aditional_condition, $sql1);
			}
			$max=ClxCommon::$db->getone($sql1);
			for ($rx=0;$rx<$max;$rx+=10000) {
			//	$_SESSION['ADDRDEBUGMYSQL']=$_SERVER['REMOTE_ADDR'];
				$sql = $sql2 . ' limit '.$rx.',10000';
//				pre($sql);	   
				$tta=fopen(kPathToWebSite."/".kPathToTemp."memory.txt","a+");
				fwrite($tta,$sql2.chr(13).chr(10).substr('        '.$rx,-8).' '.memory_get_usage().chr(13).chr(10));
				fclose($tta);
				
				if (!empty($this->debug)) {
					echoLog('<br>' . CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL);
				}
				
				$rs = ClxCommon::$db->executeQuery($sql);
			
				while (($clientrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
					if (!empty($this->debug))
						echoLog(debug_dump('clientrec', $clientrec));
					$i_proc_counter++;
					
					/*
					 * CODE_MSG_CRMEXPRT_LINEDET
					 * ^1) ^2 - ^3 - ^4
					 * ^1) ^2 - ^3 - ^4
					 */
					$rec_id_info = dict('CODE_MSG_CRMEXPRT_LINEDET', $i_proc_counter, $clientrec['IDCODE'], $clientrec['IDNAME'], $clientrec['IDENTIFICATION']) . ' ';
					$ar_acco_record = array('ACCOUNTCODE' => $clientrec['IDCODE'], // codigo de acc. en CRM
						'REC_STATUS' => 'ERRO'     // asume con error por defecto
					);
					$s_response = '';
					// Crea la cuenta CRM en el IBS
					$ar_other_dat = array('UC9' =>
						trim($clientrec['IDCODE']) . // cód. cta. CRM
						',ACCO,' . // Nombre tabla
						$clientrec['ID'] . // ID de cta. CRM
						',' . adodb_date('Y-m-d')    // Nro. de Lote
					);
					$ar_create_ibs_client_response = $this->create_ibs_client($clientrec['IDCODE'], $this->b_socket_creation, $s_tipo_persona, $clientrec, $ar_other_dat);
					$ar_client_record = $ar_create_ibs_client_response['client_record'];
					if ($ar_create_ibs_client_response['other_info'] !== '') {
						$this->o_response_rpt->write($ar_create_ibs_client_response['other_info'] . '<br><br>');
					}
					if ($ar_create_ibs_client_response['severe_error'] === true) {
						// Si hubo un error severo, por ejemplo abriendo el archivo de salida del masivo de clientes, detiene el proceso
						$this->report_error($ar_create_ibs_client_response['errormsg']);
						$this->o_response_rpt->close();
						return $ar_create_ibs_client_response['errormsg'];
					} else {
						$s_tipo_persona = $ar_create_ibs_client_response['tipo_persona'];
						if ($ar_create_ibs_client_response['error'] === true) {
							// Hubo problemas creando el cliente en IBS
							$s_response .= $ar_create_ibs_client_response['errormsg'];
							$ar_acco_record['REC_STATUS'] = 'ENDR'; // Completado con Errores (ENDR)
						} else {
							// El cliente en IBS fue creado con saliro
							if ($this->b_socket_creation != CLX_NUM_ONE && $ar_client_record['IBS_CUSCUN'] === '') {
								// Batch
								$ar_acco_record['REC_STATUS'] = 'RWOC'; // Waiting for IBS code
							} else {
								// Socket
								$ar_acco_record['REC_STATUS'] = 'RUN'; // En marcha (RUN)
							}
						}
					}
					/*
					 * GEN_LBL_OK
					 * OK
					 * OK
					 */
					if (empty($s_response)) {
						$s_response = dict('GEN_LBL_OK');
						$error = false;
					} else {
						$error = true;
					}
					// Reporta resultado en el Log del process scheduler
					if (!isset($GLOBALS['PROS'.'_'. $ar_acco_record['REC_STATUS']]))
						$GLOBALS['PROS'.'_'. $ar_acco_record['REC_STATUS']]=$DBLT->getListItem('PROS', $ar_acco_record['REC_STATUS']);
					$info_response=	$GLOBALS['PROS'.'_'. $ar_acco_record['REC_STATUS']];
					$s_msg_out = $rec_id_info . ' : ' . $info_response . ' [$s_response]';
					$this->report_error($s_msg_out, $error);
					if (!empty($this->debug)) {
						echoLog($s_msg_out);
					}
				}
			}
		}

        /*
         * Si era la creacion batch de clientes, verifica si quedo algo pendiente
         */
        $ar_create_ibs_client_moverfile = $this->create_ibs_client_moverfile($this->b_socket_creation, $s_tipo_persona);
        if ($ar_create_ibs_client_moverfile['error'] === true) {
            $this->report_error($ar_create_ibs_client_moverfile['errormsg']);
        }
        // Si se especificaron los parametros de codigo de producto y tipo de producto,
        // llama al metodo que genera la solicitud masiva de creación de cuentas de banco
        // Closes the output file for clients creation on IBS
        if ($this->b_socket_creation != CLX_NUM_ONE && $this->o_createClient_outputFileHandle != null) {
            // Batch
            $this->o_createClient_outputFileHandle->close();
            // Load the sockets interface (just in case)
            $this->massive_socket_ondemand_loader();
        }
        $this->o_response_rpt->close();
        // Updates last sinchronization date general parameter
        $this->put_last_export_date();
        echo $contenidoHTML;
        return true;
    }

// Method end
    /*
     * Creates client on eIBS from CRM Account data
     *
     * @param text $s_acco_idcode
     * @param integer $n_client_socket_creation
     * @param text $s_tipo_persona
     * @param array $clientrec
     * @param array $ar_other_dat
     * @return array
     */
    function create_ibs_client($s_acco_idcode, $n_client_socket_creation, $s_tipo_persona, $clientrec, $ar_other_dat = null) {
        $ar_client_record = array();
        $s_response = '';
        $b_severe_error = false;
        $b_error = false;
        $s_other_info = '';
        // Prepares the output file for CUSCUN assignment performed by IBS
        if ($n_client_socket_creation != CLX_NUM_ONE) {
            // Batch
            // Verifica cambio de tipo de persona, para poder generar archivos distintos
            if ($s_tipo_persona != $clientrec['TIPO_PERSONA']) {
                // Cuando se estaba procesando un tipo de persona anterior (no es la primera vez), se cierra el archivo que se estaba generando
                $ar_create_ibs_client_moverfile = $this->create_ibs_client_moverfile($n_client_socket_creation, $s_tipo_persona);
                $b_severe_error = $ar_create_ibs_client_moverfile['error'];
                $s_response .= $ar_create_ibs_client_moverfile['errormsg'];
                if ($b_severe_error == false) {
                    // Asigna el nuevo tipo de persona
                    $s_tipo_persona = $clientrec['TIPO_PERSONA'];
                    if ($s_tipo_persona === 'Z') {
                        // Tipo de persona invalida, por lo tanto se acaba la salida para los archivos
                        $this->o_createClient_outputFileHandle = null;
                    } else {
                        // Abre el archivo correspondiente segun el tipo de persona
                        // CLIENTS EXPORT DIRECTORY - Directorio para la creación del archivo con los clientes a ser creados en IBS masivamente
                        $s_output_filename_theoretical = companyParameter('EIBS_CLIEX_FILESPEC_' . $s_tipo_persona, $this->s_comp_code);
                        $s_output_filename = IBS_MASSIVE_CLIENT_CREATION_FILENAME;
                        $s_other_info = singleQuote($s_output_filename_theoretical) . '<br>' . CLX_CHAR_NL;
                        $this->o_createClient_outputFileHandle = new clx_file($s_output_filename, 'w');
                        $this->o_createClient_outputFileHandle->open();
                        if ($this->o_createClient_outputFileHandle->error) {
                            $s_response .= $this->o_createClient_outputFileHandle->errorMsg;
                            $b_severe_error = true;
                        } else {
                            // Para el batch se usan unos formatos alternativos que tienen la misma estructura que CLX008001/2 pero con unos campos
                            // adicionales parea poder realizar la creacion directamente sin pasar por el proceso de aprobacion del operador IBS
                            $this->s_cli_scktcode_individual = S_CLI_SCKTCODE_INDIVIDUAL;
                            $this->s_cli_scktcode_company = S_CLI_SCKTCODE_COMPANY;
                        }
                    }
                }
            }
        }
        if ($b_severe_error == false) {
            if (!empty($this->debug)) {
                echoLog('VA A ASIGNAR EL CUSCUN / n_acco_id='.$s_acco_idcode);
            }
            $ar_client_record = $clientrec;
            $n_acco_id = $clientrec['ID'];
            $a = new ibs_crm_account_socket_export($this->s_comp_code, $this->proc_eng_params->process_parameters);
            $ar_response_cli_crea_ibs = $a->createClientFromIdlist(array($n_acco_id), $ar_other_dat,$clientrec);
            $a = null;
            if (!empty($this->debug)) {
                debug_dump('ar_response_cli_crea_ibs', $ar_response_cli_crea_ibs);
            }
            $b_error = $ar_response_cli_crea_ibs[$n_acco_id]['error'];
            if ($b_error === true) {
                // Hubo problemas creando el cliente en IBS
                $s_response .= $ar_response_cli_crea_ibs[$n_acco_id]['contents'];
            }
        }
        $ar_response = array();
        $ar_response['tipo_persona'] = $s_tipo_persona;
        $ar_response['error'] = $b_error;
        $ar_response['severe_error'] = $b_severe_error;
        $ar_response['errormsg'] = $s_response;
        $ar_response['other_info'] = $s_other_info;
        $ar_response['client_record'] = $ar_client_record;
        return $ar_response;
    }

// Method End
    /**
     * Moves client mass creation temporary file from temporary to definitive directory, just if it's a valid person type(different than 'Z')
     * and at least one record was processed (person type not empty)
     *
     * @param integer $n_client_socket_creation
     * @param text $s_tipo_persona
     * @return array
     */
    function create_ibs_client_moverfile($n_client_socket_creation, $s_tipo_persona) {
        $contenidoHTML = '';
        $ar_response = array();
        $ar_response['error'] = false;
        $ar_response['errormsg'] = '';
        if ($n_client_socket_creation != CLX_NUM_ONE) {
            if ($s_tipo_persona !== '' && $s_tipo_persona !== 'Z') {
                // Closes the current output file
                $this->o_createClient_outputFileHandle->close();
                // Closes the current report (one report for each customer type (individuals / corporate)
                $this->o_response_rpt->close();
                $s_output_filename = companyParameter('EIBS_CLIEX_FILESPEC_' . $s_tipo_persona, $this->s_comp_code); // CLIENTS EXPORT DIRECTORY - Directorio para la creación del archivo con los clientes a ser creados en IBS masivamente
                //rename(IBS_MASSIVE_CLIENT_CREATION_FILENAME, $s_output_filename);
                $contenidoHTML.= 'copy(' . IBS_MASSIVE_CLIENT_CREATION_FILENAME . ', ' . $s_output_filename . ')<BR>' . CLX_CHAR_NL;
                $a = new ibs_interface_queue(array('comp_code' => $this->s_comp_code, 'interface' => 'CLI',
                    'submodule' => $this->submodule, 'subtype' => $s_tipo_persona
                        )
                );
                if (!$a->initpar()) {
                    $ar_response['errormsg'] .= '<BR>' . $a->get_errormsg();
                    $ar_response['error'] = true;
                } elseif (!$a->enter_queue(IBS_MASSIVE_CLIENT_CREATION_FILENAME, IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME)) {
                    /*
                     * GEN_ERR_FILESYS_COPYFAIL
                     * Error copying from file  ^1 to file ^2
                     * Error realizando la copia del archivo ^1 al archivo ^2
                     */
                    $ar_response['errormsg'] .= $a->get_errormsg();
                    $ar_response['error'] = true;
                } else {
                    // Elimina el temporal
                    unlink(IBS_MASSIVE_CLIENT_CREATION_FILENAME);
                    unlink(IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME);
                }
                // Re-open the report (one report for each customer type (individuals / corporate)
                $ar_response['errormsg'] = $this->open_rpt_process_run(IBS_MASSIVE_CLIENT_CREATION_RPT_FILENAME);
                if ($ar_response['errormsg'] !== '') {
                    $ar_response['error'] = true;
                }
                // Schedule queue monitor run
                $this->ibiq_schedule_4_exec();
            }
        }
        echo $contenidoHTML;
        return $ar_response;
    }

// Method end
    /**
     * Method to be called from menu to activate excecution of massive client export
     *
     * @param text $s_process_code
     * @param text $s_title
     * @return text
     */
    function proc_eng_fire($s_process_code, $s_title = null) {
        $this->contents = '';
        $this->lang = UserRequest::$request['lan'];
        if ($s_title == null) {
            $s_title = UserRequest::$request['clientix_MenuBar_option'];
        }
        //Create Process Schedule Schedule
        $ar_pars_2_send = array();
        $ar_pars_2_send['LANGUAGE'] = clx_language_lower_upper($this->lang);
        $ar_pars_2_send['USERNAME'] = $this->username;
        $this->contents = $this->create_proc_sched_sched($s_title, $s_process_code, $ar_pars_2_send);
        return $this->contents;
    }

// Method End
    /**
     * Adds condition to sinchronize crm accounts between CLX and IBS. The criteria es:
     * All crm account that's new (no IBS customer code 'cuscun' assigned) or modified since last sinchronization process.
     *
     * @param void
     * @return text
     */
    function add_sinc_criteria() {
        $s_response = '(IBS_CUSCUN = ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' OR IBS_CUSCUN IS null OR MDATE >= ' . singleQuote($this->s_last_export_date) . ')';
        if ($this->a_aditional_condition == null)
            $this->a_aditional_condition = array();
        $this->a_aditional_condition[] = $s_response;
        return $s_response;
    }

    /**
     * Method to be called from process engine to verify mass client interface creation file generated by eIBS as response of CLX request
     *
     * @param void
     * @return text
     */
    function recibe_ibs_clients_cuscun() {
        return $this->receive_ibs_clients_cuscun();
    }

// Method End
    /**
     * Método llamado por el motor de procesos para realizar la verificación de las respuestas del IBS a la creacion masiva de clientes
     *
     * @param void
     * @return text
     */
    function receive_ibs_clients_cuscun() {
        $s_response = $this->receive_ibs_massive_clients_creation_response();
        return $s_response;
    }

// Method End
    /**
     * This method is intended to be called by the process engine to verify the IBS responses to the massive client creation from Clientix
     *
     * @param text $s_eval_function
     * @return text
     */
    function receive_ibs_massive_clients_creation_response($s_eval_function = null) {
        /*
         * GEN_MSG_IBS_MASSIMPCLI_TITLE
         * Massive Client Export to  IBS process status
         * Estado del proceso de Importación Masiva de Clientes a IBS
         */
        $s_response = '<BR>' . CLX_CHAR_NL;
        $this->proc_eng_params->set_tablename('ACCO');
        if (!$this->proc_eng_params->verify_parameters()) {
            $s_response = '<h1>' . $this->proc_eng_params->errormsg . '</h1>';
            return $s_response;
        }
        // Verifica respuesta de los clientes NATURALES
        $s_input_filename = companyParameter('EIBS_CLIEX_RESP_FILESPEC_I', $this->s_comp_code);
        $ar_response = $this->client_mass_creat_ibs_response_read('I', $s_input_filename, $s_eval_function);
        $s_response .= '<BR>' . CLX_CHAR_NL;
        $s_response .= $ar_response['content'];
        if ($ar_response['filefound'] === true) {
            if ($ar_response['error'] === true) {
                $s_response .= $ar_response['errormsg'];
            } else {
                /*
                 * GEN_MSG_READY
                 * Ready
                 * Listo
                 */
                $s_response .= dict('GEN_MSG_READY');
                // deletes the log file
                unlink($s_input_filename);
            }
        } else {
            /*
             * GEN_LBL_NO_ANSWER
             * No answer
             * Sin respuesta
             */
            $s_response .= dict('GEN_LBL_NO_ANSWER');
        }
        // Verifica respuesta de los clientes JURIDICOS
        $s_input_filename = companyParameter('EIBS_CLIEX_RESP_FILESPEC_C', $this->s_comp_code);
        $ar_response = $this->client_mass_creat_ibs_response_read('C', $s_input_filename, $s_eval_function);
        $s_response .= '<BR>' . CLX_CHAR_NL;
        $s_response .= $ar_response['content'];
        if ($ar_response['filefound'] === true) {
            if ($ar_response['error'] === true) {
                $s_response .= $ar_response['errormsg'];
            } else {
                /*
                 * GEN_MSG_READY
                 * Ready
                 * Listo
                 */
                $s_response .= dict('GEN_MSG_READY');
                // deletes the log file
                unlink($s_input_filename);
            }
        } else {
            /*
             * GEN_LBL_NO_ANSWER
             * No answer
             * Sin respuesta
             */
            $s_response .= dict('GEN_LBL_NO_ANSWER');
        }
        $s_response .= '<BR>' . CLX_CHAR_NL . '<BR>' . CLX_CHAR_NL;
        return $s_response;
    }

// Method End
    /**
     * Clients masive creation on IBS from Clientix response file processing
     *
     * @param text $s_tipo_persona
     * @param text $s_import_filename
     * @param text $s_eval_function
     * @return array
     */
    function client_mass_creat_ibs_response_read($s_tipo_persona, $s_import_filename, $s_eval_function = null) {
        if (!empty($this->debug))
            echoLog('s_tipo_persona=$s_tipo_persona, s_import_filename=$s_import_filename, s_eval_function=$s_eval_function');
        $EBSK = getTool('EBSK');
        if($EBSK->debug === false) $EBSK->debug = $this->debug;
        $s_title = dict('GEN_MSG_IBS_MASSIMPCLI_TITLE') . ' (' . dict($s_tipo_persona === 'I' ? DICT_TIT_INDIVIDUAL : DICT_TIT_CORPORATE) . ')' . '<BR>' . dict('GEN_LBL_FILE') . ': ' . $s_import_filename . '<BR>' . dict('GEN_LBL_RESULTS') . ': ' . $s_import_filename . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT . '<BR>';
        $ar_response = array();
        $ar_response['error'] = false;
        $ar_response['error_line'] = false;
        $ar_response['filefound'] = true;
        $ar_response['errormsg'] = '';
        $ar_response['content'] = $s_title;
        if (!file_exists($s_import_filename)) {
            // There's nothing to import
            /*
             * GEN_ERR_FILE_NOTFOUND
             * File not found: ^1
             * Archivo no fue encontrado: ^1
             */
            $ar_response['error'] = true;
            $ar_response['filefound'] = false;
            $ar_response['errormsg'] = dict('GEN_ERR_FILE_NOTFOUND', $s_import_filename);
        } else {
            // Open input response file
            $o_clx_file = new clx_file($s_import_filename, 'r');
            $o_clx_file->open();
            if ($o_clx_file->error === true) {
                $ar_response['error'] = true;
                $ar_response['errormsg'] = $o_clx_file->errorMsg;
            } else {
                // Open log file for status report output
                $this->o_response_rpt = new clx_logfile_mngr($s_import_filename . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT);
                if ($this->o_response_rpt->error) {
                    $ar_response['error'] = true;
                    $ar_response['errormsg'] = $this->o_response_rpt->errorMsg;
                }
                $this->o_response_rpt->write($s_title);
            }
        }
        // If there was any error opening the input file, stops execution
        if ($ar_response['error'] === true) {
            return $ar_response;
        }
        $buffer = $o_clx_file->read_line();
        $i_proc_counter = CLX_INIT_ZERO;
        $this->o_response_rpt->write('<br>' . CLX_CHAR_NL);
        while (!$o_clx_file->eof) {
            $b_line_error = false;
            $a_exception_flds = array();
            $i_proc_counter++;
            if ($o_clx_file->delete_ending_crlf(trim($buffer)) !== '') {
                $submodule = '';
                $s_tipo_trama = substr($buffer, CLX_NUM_TEN, CLX_NUM_TEN);
                // Persona Natural
                $s_tipo_trama = trim($s_tipo_trama);
                switch ($s_tipo_trama) {
                    case IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID :
                        $persona == 'Natural';
                        $typ_cliente = 'I';
                        break;
                    case $s_tipo_trama == IBS_MASSIVE_IMPRT_CLI_COMPANY_TOKEN_ID:
                        $persona == 'Juridica';
                        $typ_cliente = 'C';
                        break;
                    default:
                        $s_response = $i_proc_counter . ' ) ' . dict('GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID') . ' (' . $s_tipo_trama . ' )';
                        $b_line_error = true;
                        break;
                }
                if ($s_tipo_trama == IBS_MASSIVE_IMPRT_CLI_INDIVIDUAL_TOKEN_ID ||
                        $s_tipo_trama == IBS_MASSIVE_IMPRT_CLI_COMPANY_TOKEN_ID) {
                    $msg_code = $s_tipo_trama;
                    $ar_record = $EBSK->readMessage($msg_code, $buffer);
                    if (!empty($this->debug))
                        echoLog(debug_dump('AR Persona ' . $persona, $ar_record));
                    $ar_response_rp = $this->client_mass_creat_ibs_response_read_processing($ar_record, $o_clx_file);
                    $ar_response_rp['client_type'] = $typ_cliente; // individual
                    if ($s_eval_function != null) {
                        // Calls the function to perform specific actions
                        // This function takes the array returned by the client_mass_creat_ibs_response_read_processing() method as parameter
                        // and it must returns it intactct or with additional information, even it could change those paramenter's elements
                        $cal = new ClxExpression();
                        $ar_response_rp = $cal->parseRecordFormula($s_eval_function($ar_response_rp));
                        $submodule = $ar_response_rp['submodulo'];
                    }
                    $s_response = $i_proc_counter . ') ' . $ar_response_rp['errormsg'];
                    $b_line_error = $ar_response_rp['error'];
                    $a_exception_flds['RECORDID'] = $ar_response_rp['idcode1'];
                }
                // Writes error file
                $this->report_error($s_response, $b_line_error, $a_exception_flds);
                $ar_response['submodule'] = $submodule;
            }
            // Reads next flat file line
            $buffer = $o_clx_file->read_line();
            // If any line has errors, report it to the output, just to know...
            $ar_response['error_line'] = $ar_response['error_line'] || $b_line_error;
        }
        // close input file
        $o_clx_file->close();
        if ($o_clx_file->error === true) {
            $ar_response['error'] = true;
            $ar_response['errormsg'] = $o_clx_file->errorMsg;
        }
        // There was at least on line with errors
        if ($ar_response['error_line'] === true) {
            /*
             * GEN_MSG_IBS_MASSIMPCLI_EILINE
             * Note: there was errors creating clients at IBS
             * Nota: hubo errores en la creación de algunos de los clientes en IBS
             */
            $s_response = '<br>' . CLX_CHAR_NL . dict('GEN_MSG_IBS_MASSIMPCLI_EILINE');
            $ar_response['content'] .= $s_response;
            $this->o_response_rpt->write($s_response);
        }
        // close output error file
        $this->o_response_rpt->close();
        if ($this->o_response_rpt->error === true) {
            $ar_response['error'] = true;
            $ar_response['errormsg'] = $this->o_response_rpt->errorMsg;
        }
        // Reset the queue record to free the resource
        $a = new ibs_interface_queue(array('comp_code' => $this->s_comp_code, 'interface' => 'CLI',
            'submodule' => $ar_response['submodule'],
            'subtype' => $s_tipo_persona
                )
        );
        if (!$a->initpar()) {
            $ar_response['error'] = true;
            $ar_response['content'] .= '<br>' . CLX_CHAR_NL . $a->get_errormsg();
        } elseif (!$a->salir_queue(array($s_import_filename . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT))) {
            $ar_response['error'] = true;
            $ar_response['content'] .= '<br>' . CLX_CHAR_NL . $a->get_errormsg();
        }
        if (!empty($this->debug))
            echoLog(debug_dump('client_mass_creat_ibs_response_read-ar_response', $ar_response));
        // Schedule queue monitor run
        $this->ibiq_schedule_4_exec();
        return $ar_response;
    }

// Method End
    /**
     * Process the response from IBS for one line
     *
     * @param array $ar_record
     * @param object $o_clx_file
     * @return array
     */
    function client_mass_creat_ibs_response_read_processing($ar_record, $o_clx_file) {
        $s_ibs_cuscun = '';
        $s_line_status = '';
        $s_idcode = '';
        $total = count($ar_record);
        for ($i_i = CLX_INIT_ZERO; $i_i < $total; $i_i++) {
            $val = $ar_record[$i_i]['VAL'];
            if (trim($val) === '')
                continue;
            // Reads each filed n the line, assigns the corresponding table column in the correct array
            switch ($ar_record[$i_i]['FILEFIELD']) {
                case 'H01WK3': // Estatus de Cliente Natural
                case 'H02WK3': // Estatus de Cliente Juridico
                    $s_line_status = $val;
                    break;
                case 'E01CUN': // CUSCUN Cliente Natural: codigo asignado al cliente dentro de IBS
                case 'E02CUN': // CUSCUN Cliente Juridico: codigo asignado al cliente dentro de IBS
                    $s_ibs_cuscun = trim($val);
                    break;
                case 'D01UC9': // Codigo Clientix para cliente natural: OJO SI E01UC9 llega a tener información, se pierde la info de este campo
                case 'D02UC9': // Codigo Clientix para cliente juridico: OJO SI E02UC9 llega a tener información, se pierde la info de este campo
                    $s_idcode = $o_clx_file->delete_ending_crlf(trim($val));
                    break;
                default:
                    break;
            }
        }
        /*
         * GEN_MSG_IBS_MASSIMPCLI_LINEOK
         * Client: Clientix code: ^1, IBS code: ^2, OK
         * Cliente: código Clientix: ^1, código IBS: ^2, OK
         */
        $s_errormsg = dict('GEN_MSG_IBS_MASSIMPCLI_LINEOK', $s_idcode, $s_ibs_cuscun);
        list($s_idcode1, $s_tablename, $s_id_tablename, $s_lote) = explode(',', $s_idcode);
        $b_error = false;
        if ($s_line_status === '1') {
            // There was an error in the line
            if ($s_ibs_cuscun === '999999999' || $s_ibs_cuscun === '000000000') {
                /*
                 * GEN_ERR_IBS_MASSIMPCLI_ERRDAT
                 * Client could not be created. There's errors in his/her/its data. Clientix code: ^1, IBS code: ^2
                 * El Cliente no pudo ser creado. Tiene errores en sus datos. Código Clientix: ^1, código IBS: ^2
                 */
                $s_errormsg = dict('GEN_ERR_IBS_MASSIMPCLI_ERRDAT', $s_idcode, $s_ibs_cuscun);
                $b_error = true;
            } else {
                // Client Already exists at IBS, so this is the customer code...
                $s_ibs_cuscun = $this->ibs_cuscun_code_fix($s_ibs_cuscun);
                $a_resp = $this->client_assgn_cuscun($s_idcode1, $s_ibs_cuscun);
                if ($a_resp['error'] === true) {
                    $s_errormsg = $a_resp['errormsg'] . ' [$s_idcode]';
                    $b_error = true;
                }
            }
        } else {
            // New Client created at IBS
            $s_ibs_cuscun = $this->ibs_cuscun_code_fix($s_ibs_cuscun);
            $a_resp = $this->client_assgn_cuscun($s_idcode1, $s_ibs_cuscun);
            if ($a_resp['error'] === true) {
                $s_errormsg = $a_resp['errormsg'] . ' [$s_idcode]';
                $b_error = true;
            }
        }
        $ar_response = array();
        $ar_response['error'] = $b_error;
        $ar_response['errormsg'] = $s_errormsg;
        $ar_response['ibs_cuscun'] = $s_ibs_cuscun;
        $ar_response['idcode'] = $s_idcode;
        $ar_response['idcode1'] = $s_idcode1;
        $ar_response['line_status'] = $s_line_status;
        return $ar_response;
    }

// Method End
    /**
     * Creates the process engine schedule to run the client massive creation verification program
     *
     * @param void
     * @return string
     */
    function mass_client_creation_response_checker() {
        /*
         * GEN_MSG_IBS_MASSIMPCLI_TITLE
         * Massive Client Export to  IBS process status
         * Estado del proceso de Importación Masiva de Clientes a IBS
         */
        $s_title = dict('GEN_MSG_IBS_MASSIMPCLI_TITLE');
        $s_process_code = 'IBS_MASCLIEX_MON';
        $ar_pars_2_send = array();
        $contents = $this->create_proc_sched_sched($s_title, $s_process_code, $ar_pars_2_send, 'HOUR');
        return $contents;
    }

// End method
}

// Class End
/**
 * CLX > e-IBS Client export socket interface
 */
class ibs_crm_account_socket_export extends ibs_client_interface_commons {

    /**
     * Constructor
     *
     * @param text $s_comp_code
     * @param array $parameters
     * @return void
     */
    function __construct($s_comp_code = null, $parameters = array()) {
        parent::__construct($s_comp_code, $parameters);
        if (!defined('EIBS_CLIEX_COD_MAIN_OFFICER')) {
            // 9999 = Cualquiera - tabla del IBS: codigo oficial principal - LISTA CLX = * NO HAY *
            define('EIBS_CLIEX_COD_MAIN_OFFICER', trim(companyParameter('EIBS_CLIEX_COD_MAIN_OFFICER', $this->s_comp_code)));
            // tablas del IBS: 10 OFICINA/REPRESENTANTE/SUSTITUTO - LISTA CLX = VEAG
            // Por ejemplo: 0101 = ofic. los palos grandes
            define('EIBS_CLIEX_COD_OFIC_REPRESENTATIVE', trim(companyParameter('EIBS_CLIEX_COD_OFIC_REPRESENTATIVE', $this->s_comp_code)));
            // STATUS: anteriormente era 2 Ronald  dijo que se le pusiera 1=inactivo
            define('EIBS_CLIEX_ACCSTAT_VALUE', trim(companyParameter('EIBS_CLIEX_ACCSTAT_VALUE', $this->s_comp_code)));
        }
        if (!empty($this->debug)) {
            echoLog('EIBS_CLIEX_COD_MAIN_OFFICER=' . EIBS_CLIEX_COD_MAIN_OFFICER);
            echoLog('EIBS_CLIEX_COD_OFIC_REPRESENTATIVE=' . EIBS_CLIEX_COD_OFIC_REPRESENTATIVE);
            echoLog('EIBS_CLIEX_ACCSTAT_VALUE=' . EIBS_CLIEX_ACCSTAT_VALUE);
        }
    }

    /**
     * Creates a Client into IBS
     * Takes idlist from the calling Form and calls the appropiate method passing it as parameter
     *
     * @param array $ar_other_dat
     * @return text
     */
    function createClient($ar_other_dat = null) {
        $title = dict('GEN_LBL_IBS_CLIENT_INTERFACE');
        $return = '';
        $EBSK = &getTool('EBSK');
        if($EBSK->debug === false) $EBSK->debug = ($this->eibs_send_debug || $this->debug); //   = EBSK message debug enabling by system parameters
        $rs = ClxCommon::$db->getRecords('CRM_ACCOUNTS', 'ID IN(' . toString(getIdlist()) . ')');
        while ($r = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE)) {
            $cl_display = array('IDNAME' => $r['IDNAME'],
                'IDENTIFICATION' => $r['IDENTIFICATION'],
                'LEGAL_TYPE' => $r['LEGAL_TYPE'],
                'EBSK_MSG_CODE' => '',
                'PROCESS_MSG' => '');
            $isindividual = ($r['LEGAL_TYPE'] === 'I') ? true : false;
            $prmtrs = array('IDENTIFICATION' => $r['IDENTIFICATION']);
            $resp1 = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_SRCH, $prmtrs, 'SELECT');
            if ($resp1[CLX_NUM_ZERO]['HEADERMSG']['ERNU01'] === 'SCKERR') {
                //connection fail
                return dialog($title, CLX_CLASS_DATA, $resp1[CLX_NUM_ZERO]['HEADERMSG']['ERDS01'] . ' (' . $resp1[CLX_NUM_ZERO]['HEADERMSG']['ERNU01'] . ')', false);
            }
            if (!empty($resp1[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {//client not found
                //create client
                if ($isindividual === true) {
                    $resp2 = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_NAT_MSG, $r, 'CREATE_REMOTE');
                    //Updating cuscun
                    if (!empty($resp2[CLX_NUM_ZERO]['E01CUN'])) {
                        ClxCommon::$db->updateRecord('CRM_ACCOUNTS', array('IBS_CUSCUN' => $resp2[CLX_NUM_ZERO]['E01CUN']), 'IDCODE = ' . singleQuote($r['IDCODE']));
                    }
                    $cl_display['EBSK_MSG_CODE'] = EBSK_IBS_DEF_CLI_NAT_MSG;
                    $cl_display['PROCESS_MSG'] = $this->displaySocketErrorMesage($resp2);
                } else {
                    $resp2 = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $r, 'CREATE_REMOTE');
                    //Updating cuscun
                    if (!empty($resp2[CLX_NUM_ZERO]['E02CUN'])) {
                        ClxCommon::$db->updateRecord('CRM_ACCOUNTS', array('IBS_CUSCUN' => $resp2[CLX_NUM_ZERO]['E02CUN']), 'IDCODE = ' . singleQuote($r['IDCODE']));
                    }
                    $cl_display['EBSK_MSG_CODE'] = EBSK_IBS_DEF_CLI_JUR_MSG;
                    $cl_display['PROCESS_MSG'] = $this->displaySocketErrorMesage($resp2);
                }
            } else {
                //update client
                if ($isindividual === true) {
                    $resp2 = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_NAT_MSG, $r, 'UPDATE_REMOTE');
                    //Updating cuscun
                    if (!empty($resp2[CLX_NUM_ZERO]['E01CUN'])) {
                        ClxCommon::$db->updateRecord('CRM_ACCOUNTS', array('IBS_CUSCUN' => $resp2[CLX_NUM_ZERO]['E01CUN']), 'IDCODE = ' . singleQuote($r['IDCODE']));
                    }
                    $cl_display['EBSK_MSG_CODE'] = EBSK_IBS_DEF_CLI_NAT_MSG;
                    $cl_display['PROCESS_MSG'] = $this->displaySocketErrorMesage($resp2);
                } else {
                    $resp2 = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $r, 'UPDATE_REMOTE');
                    //Updating cuscun
                    if (!empty($resp2[CLX_NUM_ZERO]['E02CUN'])) {
                        ClxCommon::$db->updateRecord('CRM_ACCOUNTS', array('IBS_CUSCUN' => $resp2[CLX_NUM_ZERO]['E02CUN']), 'IDCODE = ' . singleQuote($r['IDCODE']));
                    }
                    $cl_display['EBSK_MSG_CODE'] = EBSK_IBS_DEF_CLI_JUR_MSG;
                    $cl_display['PROCESS_MSG'] = $this->displaySocketErrorMesage($resp2);
                }
            }
            ClxCommon::$db->commit();
            $return .= record_display_dialog($cl_display, dict('GEN_LBL_CLIENT_INFO'), CLX_CLASS_DATA, '', false);
        }//end while
        return dialog($title, CLX_CLASS_DATA, $return, false);
    }

// Method End
    /**
     * Returns formated error string to display to user
     * @param array $response
     */
    function displaySocketErrorMesage($response) {
        $return = '';
        if (!empty($response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {
            foreach ($response[CLX_NUM_ZERO]['HEADERMSG'] as $key => $val) {
                if (!(strpos($key, 'ERDS') !== false) || $val === '')
                    continue;
                $ekey = str_replace('ERDS', 'ERNU', $key);
                $err_key = (array_key_exists($ekey, $response[CLX_NUM_ZERO]['HEADERMSG'])) ? $response[CLX_NUM_ZERO]['HEADERMSG'][$ekey] : '';
                $ekey = str_replace('ERDS', 'ERDF', $key);
                $err_key2 = (array_key_exists($ekey, $response[CLX_NUM_ZERO]['HEADERMSG'])) ? uppercase($response[CLX_NUM_ZERO]['HEADERMSG'][$ekey]) : '';
                $return .= '<li>' . $val . ' (' . $key . '.' . $err_key . '.' . $err_key2 . ')</li>';
            }
            $return .= '</ul>';
        }else {
            $return .= dict('GEN_LBL_PROCESS_COMPLETE');
        }
        return $return;
    }

    /**
     * Creates a Client into IBS
     * According account it will choose from Natural or Client Account.
     * It will test if account is complete to be able to be sent. Checking contacts, phones, etc.
     * If found, it will update account information, otherwise will send new client.
     *
     * @param array $idlist
     * @param array $ar_other_dat
     * @return array
     */
    function createClientFromIdlist($idlist, $ar_other_dat = null,$recordclient=array()) {
        $contents = '';
        $ar_response = array();
        if ($ar_other_dat == null)
            $ar_other_dat = array();
        if (empty($idlist))
            $idlist = array(CLX_NUM_ZERO);
        //Retrive Account
		if (count($recordclient)==0) {
			$sql = 'SELECT ACCO.ID, ACCO.IDCODE, ACCO.IDNAME, ACCO.ISINDIVIDUAL, ACCO.IBS_CUSCUN FROM ' . $this->acconame . ' ACCO WHERE ID IN (' . toString($idlist) . ')';
			if (!empty($this->debug))
				echoLog(debug_dump('createClientFromIdlist - <br>' . CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL, $idlist));
			$rs = ClxCommon::$db->executeQuery($sql);
			while (($clientrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
				$contents=get_contents_createClientFromIdlist($clientrec);
			}
		}
		else {
			$clientrec=array($recordclient);
			$contents=$this->get_contents_createClientFromIdlist($clientrec);
		}
        $ar_response['contents'] = note($contents);
        return $ar_fresponse;
    }
	function get_contents_createClientFromIdlist($clientrec) {
		   $s_id_nombre = '<b>' . $clientrec['IDNAME'] . '(' . $clientrec['IDCODE'] . ')<br>  ' . dict('GEN_LBL_STATUS') . ': </b>';
            if (!empty($this->debug))
                echoLog(debug_dump('<b>$s_id_nombre</b>', $results));
            // Extrae la informacion del cliente desde la tablas correspondientes segun su tipo
            $clientrec = $this->buildClient($clientrec);
            // Valida que la data esté correctamente preparada para el envio
            $ar_client_data_validation = $this->client_data_validation($clientrec);
            if ($ar_client_data_validation['error'] == false) {
                // Add the additional parameter data to the client record array
                $clientrec = array_merge($clientrec, $ar_other_dat);
                // Hace algunos arreglos en los datos (trunca)
                $clientrec = $this->client_data_fix($clientrec);
                $contents .= $s_id_nombre;
                switch ($clientrec['ISINDIVIDUAL']) {
                    case '1':
                        // persona natural
                        $ar_response['' . $clientrec['ID']] = $this->createClientNatural($clientrec);
                        $contents .= $ar_response['' . $clientrec['ID']]['contents'];
                        break;
                    case '0':
                        // persona juridica
                        $ar_response['' . $clientrec['ID']] = $this->createClientCorp($clientrec);
                        $contents .= $ar_response['' . $clientrec['ID']]['contents'];
                        break;
                    default:
                        /*
                         * GEN_LBL_IBS_MISSING_ACCOLEGALTYPE
                         * Missing individual/company type
                         * El tipo de persona no fue especificado
                         */
                        $ar_response['' . $clientrec['ID']]['error'] = true;
                        $ar_response['' . $clientrec['ID']]['contents'] = dict('GEN_LBL_IBS_MISSING_ACCOLEGALTYPE');
                        $contents .= $ar_response['' . $clientrec['ID']]['contents'];
                        break;
                }
            } else {
                /*
                 * GEN_ERR_IBSMISSINGACCOINFO
                 * Error, Missing Account Information
                 * Error, Falta información de cuenta.
                 */
                $ar_response['' . $clientrec['ID']]['error'] = true;
                $ar_response['' . $clientrec['ID']]['contents'] = dict('GEN_LBL_FAIL') . $this->ar_error_display_out($ar_client_data_validation['ar_errormsg']);
                $contents .= $s_id_nombre . $ar_response['' . $clientrec['ID']]['contents'];
            }
            $contents .= '<br>' . CLX_CHAR_NL;
			return $contents;
		
	}
// Method End
    /**
     * If client exists in either IBS and CLX, link one to eachother
     *
     * @param array $clientrec
     * @param array $ar_create_response
     * @param text $s_operation
     * @return array
     */
    function createClient_VerifAssCuscun($clientrec, $ar_create_response, $s_operation) {
        if (!empty($this->debug))
            echoLog('createClient_VerifAssCuscun');
        $ibs_crm_account_socket_import = new ibs_crm_account_socket_import($this->s_comp_code, $this->proc_eng_params->process_parameters);
        if ($this->o_createClient_outputFileHandle != null || $s_operation !== '0001') {
            return $ar_create_response;
        }
        // Only if there's a real time operation, via sockets $this->o_createClient_outputFileHandle == null) and it's creation
        $b_client_exists_in_both_sides = false;
        $b_error = !(isset($ar_create_response[CLX_NUM_ZERO]) && $ar_create_response[CLX_NUM_ZERO]['ERRNUM'] === '0');
        if ($b_error === true) {
            $n_hasta = (isset($ar_create_response[CLX_NUM_ZERO])) ? CLX_NUM_ZERO + $ar_create_response[CLX_NUM_ZERO]['ERRNUM'] : CLX_NUM_ZERO;
            for ($i = CLX_NUM_ONE; $i <= $n_hasta; $i++) {
                $s_i = (($i < CLX_NUM_TEN) ? '0' : '') . $i;
                if ($ar_create_response[CLX_NUM_ZERO]['ERNU$s_i'] === '1084') {
                    // The IDENTIFICACION of the CRM acc. in CLX already exist in IBS
                    $b_client_exists_in_both_sides = true;
                    break;
                }
            }
            if ($b_client_exists_in_both_sides === true) {
                $ar_response = $ibs_crm_account_socket_import->getClientDataFromIbs($clientrec);
                if ($ar_response['ar_result'][CLX_NUM_ZERO]['ERRNUM'] === '0') {
                    // If theresn't any error, assigns the IBS client code in the CLX's CRM account record
                    $s_ibs_cuscun = $ar_response['ar_result'][CLX_NUM_ONE]['IBS_CUSCUN'];
                    if (!empty($this->debug))
                        echoLog('Asigna CLX: clientrec[idcode]=' . $clientrec['IDCODE'] . ', s_ibs_cuscun= $s_ibs_cuscun');
                    $a_resp = $this->client_assgn_cuscun($clientrec['IDCODE'], $s_ibs_cuscun);
                    if ($a_resp['error'] === true) {
                        $s_i = (($n_hasta < CLX_NUM_TEN) ? '0' : '') . $n_hasta;
                        $ar_create_response[CLX_NUM_ZERO]['ERRNUM'] = $n_hasta;
                        $ar_create_response[CLX_NUM_ZERO]['ERRNU' . $s_i] = $a_resp['errormsg'];
                    } else {
                        $ar_create_response[CLX_NUM_ONE]['IBS_CUSCUN'] = $s_ibs_cuscun;
                        $ar_create_response[CLX_NUM_ZERO]['ERRNUM'] = '0';
                    }
                }
            }
        }
        $ibs_crm_account_socket_import = null;
        return $ar_create_response;
    }

// Method End
    /**
     * This method will add information to client record
     *
     * @param array $clientrec
     * @return array
     */
    function buildClient($clientrec) {
        // Constantes del export de clientes a IBS
        switch ($clientrec['ISINDIVIDUAL']) {
            // Personas naturales
            case '1':
                if ($this->sw_old_acco_struct === true) {
                    // old structure
                    $sql = 'SELECT ACCO.IBS_CUSCUN, ACCO.IDCODE, ACCO.IDENTIFICATION IDENTIFICATION, ACCO.ID_TYPE ID_TYPE, ACCO.INCORPORATION_COUNTRY ID_COUNTRY, ACCO.BIRTH_COUNTRY, ACCO.INCORPORATION_COUNTRY INCORPORATION_COUNTRY, LEFT(ACCO.IDNAME, 15) SHORT_NAME, LEFT(ACCO.IDNAME, 45) IDNAME, ACPH_W.PHONE PHONE_W, ACPH_F.PHONE PHONE_F, ACPH_M.PHONE PHONE_M, ACEA.ADDRESS EMAIL, UPPER(ACLO.ADDRESS1) ADDRESS1, UPPER(ACLO.ADDRESS2) ADDRESS2, UPPER(ACLO.ZIPCODE) ZIPCODE, ACLO.CITY CITY,' . 'UPPER(ACLO.STATE) STATE, UPPER(ACLO.COUNTRY) COUNTRY, UPPER(ACLO.COUNTRY) RESIDENCE_COUNTRY, CONT.FIRSTNAME FIRSTNAME, CONT.SECOND_NAME SECOND_NAME, CONT.LASTNAME LASTNAME, CONT.SECOND_LASTNAME SECOND_LASTNAME, CONT.MARRIED_NAME MARRIED_NAME,' . '' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' POBOX, UPPER(CONT.CITIZENSHIP) CITIZENSHIP, DATE_FORMAT(CONT.BIRTHDATE, ' . singleQuote('%m') . ') BIRTHDATE_MM, DATE_FORMAT(CONT.BIRTHDATE, ' . singleQuote('%d') . ' ) BIRTHDATE_DD, DATE_FORMAT(CONT.BIRTHDATE, ' . singleQuote('%Y') . ' ) BIRTHDATE_YY, CONT.SEX SEX, CONT.MARITAL_STATUS MARITAL_STATUS,' . '' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' CUSTOMER_CLASSIFICATION, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' RISK_LEVEL,' . '' . singleQuote('N') . '  CUSTOMER_TAX,' . 'ACCO.REFERREDBYCODE REFERRED_BY, CONT.INCOME_SOURCE INCOME_SOURCE,' . 'ACCO.CORRESPONDENCE_LANGUAGE LANGUAGE_FLAG,' . '' . singleQuote('R') . ' CUSTOMER_TYPE_IBS,' . singleQuote(EIBS_CLIEX_ACCSTAT_VALUE) . ' ACCSTAT,' . singleQuote(EIBS_CLIEX_COD_MAIN_OFFICER) . ' OFICIAL, ' . singleQuote(EIBS_CLIEX_COD_OFIC_REPRESENTATIVE) . '  OFICIAL2, ACCO.INDUSTRYCLASS INDUSTRYCLASS, ' . singleQuote('2') . ' E01FL2, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' PREVISIONTABLE_IBS FROM ( ' . $this->acconame . ' ACCO ) LEFT JOIN ' . $this->dyconame . ' DYCO ON ACCO.ID = DYCO.PARENT_RECORDID LEFT JOIN ' . $this->contname . ' CONT ON ACCO.ID = DYCO.PARENT_RECORDID AND CONT.ID = DYCO.CHILD_RECORDID AND CONT.ID_TYPE = ACCO.ID_TYPE AND CONT.IDENTIFICATION = ACCO.IDENTIFICATION LEFT JOIN ' . $this->acphname . ' ACPH_W ON ACCO.IDCODE = ACPH_W.ACCOUNTCODE AND ACPH_W.PHTYPE = ' . singleQuote('W') . ' LEFT JOIN ' . $this->acphname . ' ACPH_F ON ACCO.IDCODE = ACPH_F.ACCOUNTCODE AND ACPH_F.PHTYPE = ' . singleQuote('FX') . ' LEFT JOIN ' . $this->acphname . ' ACPH_M ON ACCO.IDCODE = ACPH_M.ACCOUNTCODE AND ACPH_M.PHTYPE = ' . singleQuote('M') . ' LEFT JOIN ' . $this->acphname . ' ACPH_E ON ACCO.IDCODE = ACPH_E.ACCOUNTCODE AND ACPH_E.ISPRIMARY = 1 LEFT JOIN ' . $this->aceaname . ' ACEA ON ACCO.IDCODE = ACEA.ACCOUNTCODE AND ACEA.ISPRIMARY = 1 LEFT JOIN ' . $this->acloname . ' ACLO ON ACCO.IDCODE = ACLO.ACCOUNTCODE AND ACLO.ISPRIMARY = 1 WHERE ACCO.IDCODE = ' . singleQuote($clientrec['IDCODE']) . ' GROUP BY ACCO.IDCODE';
                } else {
                    // new structure
                    $sql = 'SELECT ACCO.IBS_CUSCUN, ACCO.IDCODE, ACCO.IDENTIFICATION IDENTIFICATION, ACCO.ID_TYPE ID_TYPE, ACCO.BIRTH_COUNTRY ID_COUNTRY, ACCO.BIRTH_COUNTRY, ACCO.INCORPORATION_COUNTRY INCORPORATION_COUNTRY, LEFT(ACCO.IDNAME, 15) SHORT_NAME, LEFT(ACCO.IDNAME, 45) IDNAME, ACCO.PHONE PHONE_W, ACCO.FAX PHONE_F, ACCO.MOBILE_PHONE PHONE_M, ACCO.EMAIL EMAIL, UPPER(ACCO.ADDRESS1) ADDRESS1, UPPER(ACCO.ADDRESS2) ADDRESS2, UPPER(ACCO.ZIPCODE) ZIPCODE, ACCO.CITY CITY, UPPER(ACCO.STATE) STATE, UPPER(ACCO.COUNTRY) COUNTRY, UPPER(ACCO.COUNTRY) RESIDENCE_COUNTRY, ACCO.FIRSTNAME FIRSTNAME, ACCO.SECOND_NAME SECOND_NAME, ACCO.LASTNAME LASTNAME, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' SECOND_LASTNAME, ACCO.MARRIED_NAME MARRIED_NAME, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' POBOX, UPPER(ACCO.CITIZENSHIP) CITIZENSHIP, DATE_FORMAT(ACCO.BIRTHDATE, ' . singleQuote('%m') . ') BIRTHDATE_MM, DATE_FORMAT(ACCO.BIRTHDATE, ' . singleQuote('%d') . ') BIRTHDATE_DD, DATE_FORMAT(ACCO.BIRTHDATE, ' . singleQuote('%Y') . ') BIRTHDATE_YY, ACCO.SEX SEX, ACCO.MARITAL_STATUS MARITAL_STATUS, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' CUSTOMER_CLASSIFICATION, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' RISK_LEVEL, ' . singleQuote('N') . ' CUSTOMER_TAX, ACCO.REFERREDBYCODE REFERRED_BY, ACCO.INCOME_SOURCE INCOME_SOURCE,' . 'ACCO.CORRESPONDENCE_LANGUAGE LANGUAGE_FLAG, ' . singleQuote('R') . '  CUSTOMER_TYPE_IBS, ' . singleQuote(EIBS_CLIEX_ACCSTAT_VALUE) . ' ACCSTAT, EXT_OFFICER OFICIAL, EXT_REP_BRANCH OFICIAL2, ACCO.INDUSTRYCLASS INDUSTRYCLASS, ' . singleQuote('2') . '  E01FL2, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' PREVISIONTABLE_IBS FROM ' . $this->acconame . ' ACCO WHERE ACCO.IDCODE = ' . singleQuote($clientrec['IDCODE']);
                }
                if (!empty($this->debug))
                    echoLog('<b>Client Personal = ' . $sql . '</b>');
                $rs = ClxCommon::$db->executeQuery($sql);
                $result = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE);
                if (!empty($result)) {
                    $clientrec = array_merge($clientrec, $result);
                    if ($clientrec['ID_COUNTRY'] === '')
                        $clientrec['ID_COUNTRY'] = $clientrec['BIRTH_COUNTRY'];
                }
                break;
            // Personas juridicas, entes gubernamentales
            case '0':
                //NOTE: E01FL2 debe estar en 2 para que no solicite la table de previciones
                if ($this->sw_old_acco_struct === true) {
                    // old structure
                    $sql = 'SELECT ACCO.IBS_CUSCUN, ACCO.IDCODE, ACCO.IDENTIFICATION ID_NO, ACCO.ID_TYPE ID_TYPE, ACCO.INCORPORATION_COUNTRY ID_COUNTRY, ACCO.INCORPORATION_COUNTRY INCORPORATION_COUNTRY, LEFT(ACCO.IDNAME, 15) SHORT_NAME, LEFT(ACCO.IDNAME, 45) IDNAME, ACPH_W.PHONE PHONE_W, ACPH_F.PHONE PHONE_F, ACPH_M.PHONE PHONE_M, ACEA.ADDRESS EMAIL, UPPER(ACLO.ADDRESS1) ADDRESS1, UPPER(ACLO.ADDRESS2) ADDRESS2, UPPER(ACLO.ZIPCODE) ZIPCODE, ACLO.CITY CITY,' . 'UPPER(ACLO.STATE) STATE, UPPER(ACLO.COUNTRY) COUNTRY, UPPER(ACLO.COUNTRY) RESIDENCE_COUNTRY, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' POBOX, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ' ) INCORPORATION_DATEMM, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%d') . ' ) INCORPORATION_DATEDD, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%Y') . ' ) INCORPORATION_DATEYY, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ' ) OPESTART_DATE_MM, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ' ) OPESTART_DATE_DD, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%d') . ' ) OPESTART_DATE_YY, ACCO.CORRESPONDENCE_LANGUAGE LANGUAGE_FLAG, ' . singleQuote('1') . ' ROLLOVER_VOLUME, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' GROUP_CODE, ' . singleQuote('S') . ' E02SEX, ' . singleQuote('R') . ' CUSTOMER_TYPE_IBS, ' . singleQuote(EIBS_CLIEX_ACCSTAT_VALUE) . '  ACCSTAT, ' . singleQuote('N') . ' CUSTOMER_CLASSIFICATION, ' . singleQuote('N') . ' CUSTOMER_TAX, ' . singleQuote(EIBS_CLIEX_COD_MAIN_OFFICER) . ' OFICIAL, ' . singleQuote(EIBS_CLIEX_COD_OFIC_REPRESENTATIVE) . '  OFICIAL2, ACCO.INDUSTRYCLASS INDUSTRYCLASS, ' . singleQuote('2') . ' E02FL2, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' PREVISIONTABLE_IBS FROM ' . $this->acconame . ' ACCO LEFT JOIN ' . $this->acphname . ' ACPH_W ON ACCO.IDCODE = ACPH_W.ACCOUNTCODE AND ACPH_W.PHTYPE = ' . singleQuote('W') . ' LEFT JOIN ' . $this->acphname . ' ACPH_F ON ACCO.IDCODE = ACPH_F.ACCOUNTCODE AND ACPH_F.PHTYPE = ' . singleQuote('FX') . ' LEFT JOIN ' . $this->acphname . ' ACPH_M ON ACCO.IDCODE = ACPH_M.ACCOUNTCODE AND ACPH_M.PHTYPE = ' . singleQuote('M') . ' LEFT JOIN ' . $this->acphname . ' ACPH_E ON ACCO.IDCODE = ACPH_E.ACCOUNTCODE AND ACPH_E.ISPRIMARY = 1 LEFT JOIN ' . $this->aceaname . ' ACEA ON ACCO.IDCODE = ACEA.ACCOUNTCODE AND ACEA.ISPRIMARY = 1 LEFT JOIN ' . $this->acloname . ' ACLO ON ACCO.IDCODE = ACLO.ACCOUNTCODE AND ACLO.ISPRIMARY = 1 WHERE ACCO.IDCODE = ' . singleQuote($clientrec['IDCODE']) . ' GROUP BY ACCO.IDCODE';
                } else {
                    // new estructure
                    $sql = 'SELECT ACCO.IBS_CUSCUN, ACCO.IDCODE, ACCO.IDENTIFICATION IDENTIFICATION, ACCO.ID_TYPE ID_TYPE, ACCO.INCORPORATION_COUNTRY ID_COUNTRY, ACCO.INCORPORATION_COUNTRY INCORPORATION_COUNTRY, LEFT(ACCO.IDNAME, 15) SHORT_NAME, LEFT(ACCO.IDNAME, 45) IDNAME, ACCO.PHONE PHONE_W, ACCO.FAX PHONE_F, ACCO.MOBILE_PHONE PHONE_M, ACCO.EMAIL EMAIL, UPPER(ACCO.ADDRESS1) ADDRESS1, UPPER(ACCO.ADDRESS2) ADDRESS2, UPPER(ACCO.ZIPCODE) ZIPCODE, ACCO.CITY CITY, UPPER(ACCO.STATE) STATE, UPPER(ACCO.COUNTRY) COUNTRY, UPPER(ACCO.COUNTRY) RESIDENCE_COUNTRY, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' POBOX, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ') INCORPORATION_DATEMM, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%d') . ') INCORPORATION_DATEDD, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%Y') . ') INCORPORATION_DATEYY, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ') OPESTART_DATE_MM, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%m') . ') OPESTART_DATE_DD, DATE_FORMAT(ACCO.INCORPORATION_DATE, ' . singleQuote('%d') . ') OPESTART_DATE_YY, ACCO.CORRESPONDENCE_LANGUAGE LANGUAGE_FLAG, ' . singleQuote('1') . ' ROLLOVER_VOLUME, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' GROUP_CODE, ' . singleQuote('S') . ' E02SEX, ' . singleQuote('R') . ' CUSTOMER_TYPE_IBS, ' . singleQuote(EIBS_CLIEX_ACCSTAT_VALUE) . ' ACCSTAT, ' . singleQuote('N') . ' CUSTOMER_CLASSIFICATION, ' . singleQuote('N') . ' CUSTOMER_TAX, EXT_OFFICER OFICIAL, EXT_REP_BRANCH OFICIAL2, ACCO.INDUSTRYCLASS INDUSTRYCLASS, ' . singleQuote('2') . ' E02FL2, ' . singleQuote(CLX_CHAR_EMPTY_STRING) . ' PREVISIONTABLE_IBS FROM ' . $this->acconame . ' ACCO WHERE ACCO.IDCODE = ' . singleQuote($clientrec['IDCODE']);
                }
                if (!empty($this->debug))
                    echoLog('<b>Client Corporate = ' . $sql . '</b>');
                $rs = ClxCommon::$db->executeQuery($sql);
                $result = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE);
                if (!empty($result))
                    $clientrec = array_merge($clientrec, $result);
                break;
            default:
                break;
        }//End switch
        return $clientrec;
    }

// Method End
    /**
     * Validate if client if complete to be sent to ibs.
     *
     * @param array $clientrec
     * @return array
     */
    function client_data_validation($clientrec) {
        $ar_error_msg = array();
        $s_address = $clientrec['ADDRESS1'] . $clientrec['ADDRESS2'] . $clientrec['ZIPCODE'] . $clientrec['CITY'] . $clientrec['STATE'] . $clientrec['COUNTRY'];
        if (empty($s_address)) {
            /*
             * GEN_ERR_IBS_MISSINGADDRESS
             * Error, missing address
             * Error, no tiene dirección física.
             */
            $ar_error_msg[] = dict('GEN_ERR_IBS_MISSINGADDRESS');
        } else {
            if (empty($clientrec['CITY'])) {
                /*
                 * GEN_ERR_IBS_MISSINGCITY
                 * Error, missing city
                 * Error, la dirección no tiene ciudad.
                 */
                $ar_error_msg[] = dict('GEN_ERR_IBS_MISSINGCITY');
            } elseif (empty($clientrec['STATE'])) {
                /*
                 * GEN_ERR_IBS_MISSINGSTATE
                 * Error, missing State
                 * Error, la dirección no tiene Estado.
                 */
                $ar_error_msg[] = dict('GEN_ERR_IBS_MISSINGSTATE');
            }
        }
        return array('error' => (count($ar_error_msg) > CLX_NUM_ZERO), 'ar_errormsg' => $ar_error_msg,
        );
    }

// Method End
    /**
     * Returns the client array fixed
     * City: truncates it to the last 2 digits after the validation.
     *
     * @param array $clientrec
     * @return array
     */
    function client_data_fix($clientrec) {
        $DBLT = getTool('DBLT');
        switch ($clientrec['ISINDIVIDUAL']) {
            case '1':
                // persona natural
                $s_subnombrecampo = '01';
                break;
            case '0':
                // persona juridica
                $s_subnombrecampo = '02';
                break;
            default:
                // Tipo invalido (por aqui nunca deberia entrar)
                $s_subnombrecampo = '';
                break;
        }
        $s_city_name = $DBLT->getListItem('LOCI', $clientrec['CITY']);
        // Agrega el nombre de la ciudad
        $clientrec['CITY'] .= $s_city_name;
        // Pica la direccion (linea 1)
        $clientrec['ADDRESS1'] = fija($clientrec['ADDRESS1'], CLX_NUM_60);
        $clientrec['E' . $s_subnombrecampo . 'NA3'] = substr($clientrec['ADDRESS1'], CLX_NUM_MINUS_25);
        $clientrec['ADDRESS1'] = substr($clientrec['ADDRESS1'], CLX_NUM_ZERO, CLX_NUM_35);
        if ($this->o_createClient_outputFileHandle != null) {
            // If it's the massive client creation process, it must send the client's IDCODE in order to retrieve it later at response read & process time
            // Warning: it uses the D0?UC9 field, but if the E0?UC9 field has any content into it, the D0?UC9 will be blanked...
            $clientrec['D' . $s_subnombrecampo . 'UC9'] = $clientrec['UC9'];
        }
        // Trunca la direccion (linea 2)
        $clientrec['ADDRESS2'] = substr($clientrec['ADDRESS2'], CLX_NUM_ZERO, CLX_NUM_35);
        // Trunca el correspondence_lang para que quede como E o S (originalmente viene como EN o SP)
        if (isset($clientrec['LANGUAGE_FLAG']))
            $clientrec['LANGUAGE_FLAG'] = substr($clientrec['LANGUAGE_FLAG'], CLX_NUM_ZERO, CLX_NUM_ONE);
        if (!empty($this->debug))
            echoLog(debug_dump('client_data_fix - clientrec', $clientrec));
        return $clientrec;
    }

// Method End
    /**
     * Create/Update Natural Client to IBS
     *
     * @param array $clientrec
     * @return array
     */
    function createClientNatural($clientrec) {
        if (!empty($this->debug))
            echoLog('createClientNatural');
        $clientrec['USERNAME'] = $this->username;
        if ($clientrec['IBS_CUSCUN'] === '') {
            $clientrec['H01OPE'] = '0001'; //Create
        } else {
            if ($this->o_createClient_outputFileHandle == null) {
                // Only if there's a real time operation (via sockets)
                $clientrec['H01OPE'] = '0002'; //Query prior to the update operation
                $this->EBSK_direction = CLX_NUM_ONE; // translation OUT (reverse direction because is an export)
                $this->EBSK_timeout = CLX_NUM_THREE; // specific timeout for this operation
                $result = $this->send($this->s_cli_scktcode_individual, $clientrec);
                if ($result[CLX_NUM_ZERO]['ERRNUM'] !== '0') {
                    // Inquiry fails...
                    return $this->client_results_process($clientrec, $result, $this->ar_last_sent_skt);
                } else {
//					pre($result);
                    $clientrec = $this->Client_complete_other_flds($clientrec, $result[CLX_NUM_ONE]);
                }
            }
            $clientrec['H01OPE'] = '0005'; //Update
        }
        $this->EBSK_direction = CLX_NUM_ONE; // translation OUT (reverse direction because is an export)
        $this->EBSK_timeout = CLX_NUM_FIVE; // specific timeout for this operation
        $result = $this->send($this->s_cli_scktcode_individual, $clientrec);
        $result = $this->createClient_VerifAssCuscun($clientrec, $result, $clientrec['H01OPE']);
        //adding to now module
        $clientrec['H01OPE'] = $this->s_cli_scktcode_individual . '.' . $clientrec['H01OPE'];
        return $this->client_results_process($clientrec, $result, $this->ar_last_sent_skt);
    }

// Method End
    /**
     * Creates a Corporate Client on IBS
     *
     * @param array $clientrec
     * @return array
     */
    function createClientCorp($clientrec) {
        if (!empty($this->debug))
            echoLog('createClientCorp');
        if ($clientrec['INCORPORATION_DATEMM'] === '') {
            // Si no hay fecha de fundacion, asigna la del dia
            $clientrec['INCORPORATION_DATEMM'] = adodb_date('m');
            $clientrec['INCORPORATION_DATEDD'] = adodb_date('d');
            $clientrec['INCORPORATION_DATEYY'] = adodb_date('Y');
        }
        $clientrec['USERNAME'] = $this->username;
        $clientrec['LEGAL_TYPE'] = '1';
        if ($clientrec['IBS_CUSCUN'] === '') {
            $clientrec['H02OPE'] = '0001'; //Create
        } else {
            if ($this->o_createClient_outputFileHandle == null) {
                // Only if there's a real time operation (via sockets)
                $clientrec['H02OPE'] = '0002'; //Query prior to the update operation
                /* 	OPERATION CODE:
                  Create: 0001
                  Query: 0004
                  Manteinance: First: '0002 Para solicitar la info del cliente' Second: '0005 enviar los datos modificados'
                 */
                $this->EBSK_direction = CLX_NUM_ONE; // translation OUT (reverse direction because is an export)
                $result = $this->send($this->s_cli_scktcode_company, $clientrec);
                if ($result[CLX_NUM_ZERO]['ERRNUM'] !== '0') {
                    // Inquiry fails...
                    return $this->client_results_process($clientrec, $result, $this->ar_last_sent_skt);
                } else {
                    $clientrec = $this->Client_complete_other_flds($clientrec, $result[CLX_NUM_ONE]);
                }
            }
            $clientrec['H02OPE'] = '0005'; //Update
        }
        $this->EBSK_direction = CLX_NUM_ONE; // translation OUT (reverse direction because is an export)
        $result = $this->send($this->s_cli_scktcode_company, $clientrec);
        $result = $this->createClient_VerifAssCuscun($clientrec, $result, $clientrec['H02OPE']);
        //adding to now module
        $clientrec['H01OPE'] = $this->s_cli_scktcode_company . '.' . $clientrec['H02OPE'];
        return $this->client_results_process($clientrec, $result, $this->ar_last_sent_skt);
    }

// Method End
    /**
     * Complements the Client Record from CLX DB with record from IBS inquiry
     *
     * @param array $a_clientrec: client's record from CLx
     * @param array $a_result: client data from IBS inquiry
     * @return array: client's record complemented
     */
    function Client_complete_other_flds($a_clientrec, $a_result) {
        reset($a_result);
        foreach ($a_result as $s_cod => $v_element) {
            if (!isset($a_clientrec[$s_cod]) ||
                    (isset($a_clientrec[$s_cod]) && (trim($a_clientrec[$s_cod]) === '' || is_null($a_clientrec[$s_cod])) )) {
                $v_element = trim($v_element);
                $field = $this->get_sktdef_size($s_cod, $this->ar_last_sent_skt, '*', 'FIELD');
                if ($field['TYPE'] === 'DECN') {
                    //Validate number of decimals on string
                    if (!empty($field['DECIMALS']) > CLX_NUM_ZERO && strlen($v_element) > CLX_NUM_ZERO) {
                        //has decimal?
                        if (strpos($v_element, '.') === false) {
                            $v_element .= '.00';
                        }
                    }
                }
                $a_clientrec[$s_cod] = $v_element;
            }
        }
        return $a_clientrec;
    }

    /**
     * If the transmission was Ok, process the CUSCUN code incorporating it to the source ACCO record.
     * Returns an array with the following structure:
     * Array(
     * 	'contents' => the sending result message of a items sent.
     * 	'error'=> true if it was a transmision error
     * 	'cuscun' => if the operation was Ok, is the code assigned by IBS
     * )
     *
     * @param array $clientrec
     * @param array $result
     * @param array $ar_last_sent_skt
     * @return array
     */
    function client_results_process($clientrec, $result, $ar_last_sent_skt) {
        if (!empty($this->debug))
            echoLog('client_results_process');
        $ar_response = array();
        $ar_response['error'] = true;
        $ar_response['ar_errormsg'] = $result;
        $ar_response['cuscun'] = '';
        if (isset($result[CLX_NUM_ZERO]) && $result[CLX_NUM_ZERO]['ERRNUM'] === '0') {
            // Updates the ACCO.IBS_CUSCUN field
            $ar_response['error'] = false;
            $s_ibs_cuscun = (isset($result[CLX_NUM_ONE]) && $result[CLX_NUM_ONE]['IBS_CUSCUN']) ? $result[CLX_NUM_ONE]['IBS_CUSCUN'] : '';
            $s_idcode = $clientrec['IDCODE'];
            $ar_response['cuscun'] = $s_ibs_cuscun;
            if (!empty($this->debug))
                echoLog('s_ibs_cuscun=' . $s_ibs_cuscun . ', s_idcode=' . $s_idcode);
            $this->client_assgn_cuscun($s_idcode, $s_ibs_cuscun);
        }
        // Returns the sending result message
        $ar_response['contents'] = $this->print_error_out($result, $ar_last_sent_skt);
        $ar_response['contents'] .= '<BR> Oper: ' . $clientrec['H01OPE'] . '<br>';
        return $ar_response;
    }

// Method End
}

// Class End
/**
 * e-IBS > CLX Client socket import interface
 */
class ibs_crm_account_socket_import extends ibs_client_interface_commons {
    /*
     * Constructor
     *
     * @param text $s_comp_code
     * @param array $parameters
     * @return void
     */

    function __construct($s_comp_code = null, $parameters = array()) {
        parent::__construct($s_comp_code, $parameters);
        if (!empty($this->debug)) {

        }
    }

    /**
     * General Client Interfase for Queriing IBS
     *
     * @param void
     * @return text
     */
    function getClients() {
        $contents = '';
        //Connect and quote for client
        $idlist = getIdlist();
        //Retrive Account
        $sql = 'SELECT ACCO.IDCODE, ACCO.ISINDIVIDUAL, ACCO.IBS_CUSCUN FROM ' . $this->acconame . ' ACCO  WHERE ID IN (' . singleQuote($implode("','", $idlist)) . ')';
        $rs = ClxCommon::$db->executeQuery($sql);
        while (($clientrec = ClxCommon::$db->getNextRecord($rs, ASSOCIATIVE))) {
            $ar_response = $this->getClientDataFromIbs($clientrec);
            $contents .= $ar_response['contents'];
        }
        return $this->display($contents);
    }

// Method End
    /**
     * Get client data from IBS.
     * $clientrec is an array with the elements IDCODE, ISINDIVIDUAL and IBS_CUSCUN. See ACCO table for details.
     * Those values will be replaced in the corresponding data fields at socket definition.
     *
     * @param array $clientrec
     * @return arry
     */
    function getClientDataFromIbs($clientrec) {
        if (!empty($this->debug))
            echoLog(debug_dump('getClientDataFromIbs - clientrec', $clientrec));
        switch ($clientrec['ISINDIVIDUAL']) {
            case '1':
                $ar_response = $this->getClientNatural($clientrec);
                break;
            case '0':
                $ar_response = $this->getClientCorp($clientrec);
                break;
            default:
                break;
        }
        if (!empty($this->debug))
            echoLog(debug_dump('ar_response', $ar_response));
        return $ar_response;
    }

// Method End
    /*
     * Query Natural Clients from ibs MSG CLX008001
     * @param array $clientrec
     * @return array
     */
    function getClientNatural(&$clientrec) {
        if (!empty($this->debug))
            echoLog('getClientNatural');
        $clientrec['H01OPE'] = '0004'; //0004->Query
        $clientrec['USERNAME'] = $this->username;
        $result = $this->send('CLX008001', $clientrec);
        $ar_response = array('ar_result' => $result,
            'ar_last_sent_skt' => $this->ar_last_sent_skt, 'contents' => $this->print_error_out($result, $this->ar_last_sent_skt),);
        return $ar_response;
    }

// Method End
    /*
     * Query Corporate Clients from ibs
     *
     * @param array $clientrec
     * @return array
     */
    function getClientCorp(&$clientrec) {
        if (!empty($this->debug))
            echoLog('getClientCorp');
        //Consulta 8002 Cliente Persona Juridica
        $clientrec['H02OPE'] = '0004'; //Query
        $clientrec['USERNAME'] = $this->username;
        $result = $this->send('CLX008002', $clientrec);
        //$msg =($result[CLX_INIT_ZERO]['ERRNUM'] === '0')?dict('GEN_LBL_OK'):dict('GEN_LBL_FAIL') . ', <li>'.implode('</li><li>',$this->formatIbsErrorMsg($result[CLX_INIT_ZERO], $this->ar_last_sent_skt)).'</li>';
        $ar_response = array('ar_result' => $result,
            'ar_last_sent_skt' => $this->ar_last_sent_skt, 'contents' => $this->print_error_out($result, $this->ar_last_sent_skt),);
        return $ar_response;
    }

// Method End
    /**
     * Inquiry to E-IBS by identification - main method
     *
     * @param array $a_record
     * @return string
     */
    function inbid_run($a_record, &$msg = array()) {
        // Only if EIBS interface active and the authenticated user is an IBS user
//e cho '<br>s_eibs_interface_active = $this->s_eibs_interface_active';
//e cho '<br>_SESSION['others']['eibs_auth_user'] = ' . $_SESSION['others']['eibs_auth_user'];
        if ($this->verify_eibs_auth_user()) {
            if ($a_record['ACCO_IDENTIFICATION'] === true) {
                // Identification query
                $a_response = $this->GetGenClientByIdentification($a_record['ACCO_IDENTIFICATION']);
                $this->contents = $a_response['contents'];
                if ($a_response['error'] === true) {
                    $msg['ACCO_IDENTIFICATION'] = $this->contents;
                } elseif ($a_response['create'] === true) {
                    $a_ret = $this->inbid_cli_creation($a_response);
                    $this->contents = $a_ret['errormsg'];
                    if ($a_ret['error'] === true) {
                        $msg['ACCO_IDENTIFICATION'] = $this->contents;
                    }
                }
            } elseif ($a_record['ACCO_IBS_CUSCUN'] === true) {
                // EIBS code query
                $a_response = $this->GetGenClientByCuscum($a_record['ACCO_IBS_CUSCUN']);
                $this->contents = $a_response['contents'];
                if ($a_response['error'] === true) {
                    $msg['ACCO_IBS_CUSCUN'] = $this->contents;
                } elseif ($a_response['create'] === true) {
                    $a_ret = $this->inbid_cli_creation($a_response);
                    $this->contents = $a_ret['errormsg'];
                    if ($a_ret['error'] === true) {
                        $msg['ACCO_IDENTIFICATION'] = $this->contents;
                    }
                }
            }
        }
        return $this->source();
    }

// Method End
    /*
     * Create client from data received in IBS inquiry
     *
     * @param array $a_response
     * @return array
     */
    function inbid_cli_creation($a_response) {
        $b_is_individual = ($a_response['ar_result'][CLX_NUM_ONE]['LEGAL_TYPE'] === '2');
        $a_response['ar_result'][CLX_NUM_ONE]['H0' . ($b_is_individual ? '1' : '2') . 'OPE'] = '0001';
        $a_client_rec = $this->inbid_build_cli_creation_array($a_response['ar_result'], $a_response['ar_last_sent_skt']);
        $a_parameters = $this->proc_eng_params->process_parameters;
        $a_parameters['FLG_PRODUCTION'] = '1'; // just to effectively create the clients, bacuse it's not a process engine procedure...
        $a = new ibs_crm_account_mass_import($this->s_comp_code, $a_parameters);
        $a->b_sw_validations_on = false;
        $s_s = ''; // reference info...
        if ($b_is_individual === true) {
            $a_ret = $a->create_client_individual_frm_ibs(CLX_NUM_ONE, $a_client_rec, $s_s);
        } else {
            $a_ret = $a->create_client_company_frm_ibs(CLX_NUM_ONE, $a_client_rec, $s_s);
        }
        $a = null;
        return $a_ret;
    }

// Method End
    /*
     * Prepares array for the client creation, input for create_client_individual_frm_ibs() and
     * create_client_company_frm_ibs() methods
     *
     * @param array $ar_result
     * @param array $ar_last_sent_skt
     * @return array
     */
    function inbid_build_cli_creation_array($ar_result, $ar_last_sent_skt) {
//e cho '<br>inbid_build_cli_creation_array';
//e cho '<br><br>ar_result<br>'; print_r($ar_result);
//e cho '<br><br>ar_last_sent_skt<br>'; print_r($ar_last_sent_skt);
//e cho '<br><br>';
        $total = count($ar_last_sent_skt);
        for ($i = CLX_INIT_ZERO; $i < $total; $i++) {
            if ($ar_last_sent_skt[$i]['FIELD'] !== '' &&
                    isset($ar_result[CLX_NUM_ONE][$ar_last_sent_skt[$i]['FIELD']])) {
                $ar_last_sent_skt[$i]['VAL'] = $ar_result[CLX_NUM_ONE][$ar_last_sent_skt[$i]['FIELD']];
            }
        }
        return $ar_last_sent_skt;
    }

// Method End
    /*
     * Query Clients from EIBS by identification - MSG CLX00????
     *
     * @param string $s_rif
     * @return array
     */
    function GetGenClientByIdentification($s_rif) {
        if (!empty($this->debug))
            echoLog('GetGenClient');
        $b_ok = true;
        $b_create = false;
        $n_record_qty = ClxCommon::$db->countRecords($this->acconame, 'IDENTIFICATION = ' . singleQuote($s_rif));
        $EBSK = getTool('EBSK');
        if ($n_record_qty > CLX_NUM_ZERO) {
            $ar_response = array('ar_result' => $EBSK->build_ibs_style_error_array(array()), // OK
                'error' => false,
                'create' => $b_create,
                'ar_last_sent_skt' => array(),
                'contents' => ''
            );
        } else {
            $a_param = array();
            $a_param['USERNAME'] = $this->username;
            $a_param['IDENTIFICATION'] = $s_rif;
            $a_param['H01OPE'] = '0004';
            $a_result = $this->send('CLX0070', $a_param);
         //   pre(htmlentities(print_r($a_result, true)));
            $b_ok = ($a_result[CLX_NUM_ZERO]['ERRNUM'] === '0');
            if (!empty($b_ok)) {
                if (trim($a_result[CLX_NUM_ONE]['IBS_CUSCUN']) !== '') {
                    // Client was found...
                    $a_clientrec = array();
                    $a_clientrec['IBS_CUSCUN'] = $a_result[CLX_NUM_ONE]['IBS_CUSCUN'];
                    // LEGAL TYPE, 1 Juridica, 2 Natural, 3 Other
                    if ($a_result[CLX_NUM_ONE]['LEGAL_TYPE'] === '2') {
                        $a_clientrec['ISINDIVIDUAL'] = '1';
                    } else {
                        $a_clientrec['ISINDIVIDUAL'] = '0';
                    }
                    $a_result0 = $this->getClientDataFromIbs($a_clientrec);
                    $a_result = $a_result0['ar_result'];
                    $b_ok = ($a_result[CLX_NUM_ZERO]['ERRNUM'] === '0');
                    $b_create = $b_ok;
                } else {
                    // Identification Not found...
                    $a_result = $EBSK->build_ibs_style_error_array(array('NOTFOUND' => dict('GEN_MSG_RECORD_NOTFOUND', $s_rif)
                    ));
                    $b_ok = false;
                }
            }
            $ar_response = array('ar_result' => $a_result,
                'ar_last_sent_skt' => $this->ar_last_sent_skt, 'error' => (!$b_ok),
                'create' => $b_create,
                'contents' => $this->print_error_out($a_result, $this->ar_last_sent_skt),);
        }
//e cho debug_dump('GetGenClientByIdentification-a_result', $ar_response);
        return $ar_response;
    }

// Method End
    /*
     * Query Clients from EIBS by Cuscum
     *
     * @param string $s_rif
     * @return array
     */
    function GetGenClientByCuscum($s_cuscum) {
        if (!empty($this->debug))
            echoLog('GetGenClient');
        $n_record_qty = ClxCommon::$db->oneExistsRecords($this->acconame, 'IBS_CUSCUN = ' . singleQuote($s_cuscum));
        $EBSK = getTool('EBSK');
        $b_create = false;
        if ($n_record_qty > CLX_NUM_ZERO) {
            $ar_response = array('ar_result' => $EBSK->build_ibs_style_error_array(array()), // OK
                'ar_last_sent_skt' => array(),
                'create' => $b_create,
                'error' => false,
                'contents' => ''
            );
        } else {
            $a_clientrec = array();
            $a_clientrec['IBS_CUSCUN'] = $s_cuscum; //$a_result[1]['IBS_CUSCUN'];not defined
            $a_clientrec['ISINDIVIDUAL'] = '0';
            $a_result = $this->getClientDataFromIbs($a_clientrec);
            $b_ok = (array_key_exists(CLX_NUM_ZERO, $a_result) && $a_result[CLX_NUM_ZERO]['ERRNUM'] === '0');
            if (empty($b_ok)) {
                $a_clientrec['ISINDIVIDUAL'] = '1';
                $a_result = $this->getClientDataFromIbs($a_clientrec);
                $b_ok = (array_key_exists(CLX_NUM_ZERO, $a_result) && $a_result[CLX_NUM_ZERO]['ERRNUM'] === '0');
                $b_create = $b_ok;
            }
            $ar_response = array('ar_result' => $a_result,
                'ar_last_sent_skt' => $this->ar_last_sent_skt, 'create' => $b_create,
                'error' => (!$b_ok),
                'contents' => $this->print_error_out($a_result, $this->ar_last_sent_skt),);
        }
        return $ar_response;
    }
   function searchClientOnIbsForm() {
        //ACCO_IBS_SRCH
        $contents = '';
        $ac = UserRequest::$request['ac'];
        $to = UserRequest::$request[CLX_PROP_TO];
        $it = UserRequest::$request['it'];
        $title = dict('GEN_LBL_SEARCH_IBS_CLIENT');
        $idlist = getIdlist();
        //setting back in case or return
        pushHistory($to, $ac, $idlist, 'it', $it);
        //RTBL_REPCINV
        $EDIT = &getTool('EDIT');
        $record = array();
        $msg = array();
        $error = false;
        $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE = ' . singleQuote('ACCO_IBS_SRCH') . '');
        $record = UserRequest::$post;
        if (!empty($record['confirm'])) {
            $UIFM = &getTool('UIFM');
            $UIFM->userCheckForm($formrec, $record, $msg);
            if (empty($msg)) {//is not error just connect
                if ($record['ACCO_IDENTIFICATION'] !== '' && $record['ACCO_LEGAL_TYPE'] === '') {
                    /*
                     * GEN_ERR_REQUIRED_FIELD
                     * Este es un campo requerido.  Por favor llenelo.
                     * This is a required field.  Please fill it.
                     */
                    $msg['ACCO_LEGAL_TYPE'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if ($record['ACCO_IDENTIFICATION'] === '' && $record['ACCO_LEGAL_TYPE'] !== '') {
                    /*
                     * GEN_ERR_REQUIRED_FIELD
                     * Este es un campo requerido.  Por favor llenelo.
                     * This is a required field.  Please fill it.
                     */
                    $msg['ACCO_IDENTIFICATION'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if ($record['ACCO_IDENTIFICATION'] === '' && $record['ACCO_IBS_CUSCUN'] === '') {
                    /*
                     * ACCO_IBS_SRCH_ME01
                     * Some date must be specified
                     * Se debe especificar alguno de los campos
                     */
                    $msg['ACCO_IDENTIFICATION'] = dict('ACCO_IBS_SRCH_ME01');
                    $msg['ACCO_IBS_CUSCUN'] = dict('ACCO_IBS_SRCH_ME01');
                }
                if (user_is_in_acctree() == false) {
                    $error = true;
                    /*
                     * GEN_ERR_USR_NOTFND_ACTREE
                     * User is not registered in Access Tree
                     * Usuario no esta registrado en el Arbol de Acceso
                     */
                    $contents .= note(dict('GEN_ERR_USR_NOTFND_ACTREE'));
                }
                if ($record['ACCO_IBS_CUSCUN'] !== '') {
                    $n_record_qty = ClxCommon::$db->countRecords($this->acconame, 'IBS_CUSCUN = ' . singleQuote($record['ACCO_IBS_CUSCUN']));
                    if ($n_record_qty > CLX_NUM_ONE) {
                        $error = true;
                        /*
                         * GEN_ERR_IBS_REPEATED_CUSCUN
                         * IBS Client Code repated more than one time
                         * El código de cliente de IBS está repetido más de una vez
                         */
                        $contents .= note(dict('GEN_ERR_IBS_REPEATED_CUSCUN'));
                    }
                }
            }
            if (empty($msg) && $error == false) { //is not error just connect
                $id = '';
                if ($record['ACCO_IBS_CUSCUN'] !== '') {
                    $this->searchClientOnIbsByCuscum_force_ibs_search = true;
                    $a_response = $this->searchClientOnIbsByCuscum($record['ACCO_IBS_CUSCUN']);
                    if ($a_response['error'] == false) {
                        // OK, client was created from IBS
                        $id = $a_response['clx_id'];
                        $isindividual = ($a_response[CLX_CLASS_DATA]['LEGAL_TYPE'] === 'I') ? true : false;
                        $a_response['dataToDisplay']['IDCODE'] = $a_response[CLX_CLASS_DATA]['IDCODE'];
                        $a_response['dataToDisplay']['IDNAME'] = $a_response[CLX_CLASS_DATA]['IDNAME'];
                        $a_response['dataToDisplay']['LEGAL_TYPE'] = $a_response[CLX_CLASS_DATA]['LEGAL_TYPE'];
                        $a_response['dataToDisplay']['ID_TYPE'] = $a_response[CLX_CLASS_DATA]['ID_TYPE'];
                        $a_response['dataToDisplay']['IDENTIFICATION'] = $a_response[CLX_CLASS_DATA]['IDENTIFICATION'];
                        $a_response['dataToDisplay']['IBS_CUSCUN'] = $a_response[CLX_CLASS_DATA]['IBS_CUSCUN'];
                        $contents = record_display_dialog($a_response['dataToDisplay'], $title, CLX_CLASS_DATA, dict('GEN_LBL_RECORD_FOUND'), false);
                    } else {
                        // There was an error
                        $contents .= note($a_response['errormsg']);
                    }
                } else {
                    $parameters = array('IDENTIFICATION' => $record['ACCO_IDENTIFICATION']);
                    $EBSK = &getTool('EBSK');
                    if($EBSK->debug === false) $EBSK->debug = ($this->eibs_send_debug || $this->debug); //   = EBSK message debug enabling by system parameters
                    $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_SRCH, $parameters, 'SELECT');
                    if (!empty($response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {//client not found
                        $contents .= note($response[CLX_NUM_ZERO]['HEADERMSG']['ERDS01'] . ' (' . $response[CLX_NUM_ZERO]['HEADERMSG']['ERNU01'] . ')');
                    } else {
                        $isindividual = ($response[CLX_NUM_ZERO]['LEGAL_TYPE'] === 'I') ? true : false;
                        $contents = record_display_dialog($response[CLX_NUM_ZERO], $title, CLX_CLASS_DATA, dict('GEN_LBL_RECORD_FOUND'), false);
                        //Check if record already exists
                        $ibs_cuscun = $response[CLX_NUM_ZERO]['IBS_CUSCUN'];
                        $parameters = array('IBS_CUSCUN' => $ibs_cuscun);
                        //$id = ClxCommon::$db->getOne('SELECT ID FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = '. singleQuote($ibs_cuscun) );
                        if ($isindividual === true) {
                            $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_NAT_MSG, $parameters, 'LOAD');
                        } else {
                            $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $parameters, 'LOAD');
                        }
                        $id = ClxCommon::$db->getOne('SELECT ID FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = ' . singleQuote($ibs_cuscun));
                    }
                }
                //setting more link
                if (!empty($id)) {
                    if ($isindividual === true) {
                        //$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_PERS&qy=ACCO_PERS&relfm=&idlist=' . $id;
                        $url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_PERS_IBS&qy=ACCO_PERS&relfm=&idlist=' . $id;
                    } else {
                        //$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_CORP&qy=ACCO_CORP&relfm=&idlist=' . $id;
                        $url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_CORP_IBS&qy=ACCO_CORP&relfm=&idlist=' . $id;
                    }
                    $a = new htmlAnchor(dict('GEN_LBL_CLICK_HERE'), $url);
                    $contents .= '<br>' . dict('GEN_LBL_FOR_MORE') . ' ' . $a->source() . '<br>';
                } else {
                    $contents .= smsg('MSG-252') . '<br>';
                }
            }
        }
        //Build search form
        $form = $EDIT->defaultForm($formrec, $record, $msg, $ac, $to, $it, 'tool.php', '', $title, dict('GEN_LBL_EXECUTE'));
        return $form->source() . '<br>' . $contents;
        ;
    }
// Method End
    /**
     * Build IBS Client Search Form and Performs socket call,
     */
   function searchClientOnIbsForm_old() {
        //ACCO_IBS_SRCH
        $contents = '';
        $ac = UserRequest::$request['ac'];
        $to = UserRequest::$request[CLX_PROP_TO];
        $it = UserRequest::$request['it'];
        $title = dict('GEN_LBL_SEARCH_IBS_CLIENT');
        $idlist = getIdlist();
        //setting back in case or return
        pushHistory($to, $ac, $idlist, 'it', $it);
        //RTBL_REPCINV
        $EDIT = &getTool('EDIT');
        //$record = array();
	   
		//pre($_SESSION);
		//pre($_POST);
	   
        $msg = array();
        $error = false;
        $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE = ' . singleQuote('ACCO_IBS_SRCH') . '');
        $record = UserRequest::$post;
        if (!empty($record['confirm'])) {
            $UIFM = &getTool('UIFM');
            $UIFM->userCheckForm($formrec, $record, $msg);
            if (empty($msg)) {//is not error just connect
 
                if ($record['ACCO_LEGAL_TYPE'] == '') {
                    /*
                     * GEN_ERR_REQUIRED_FIELD
                     * Este es un campo requerido.  Por favor llenelo.
                     * This is a required field.  Please fill it.
                     */
                    $msg['ACCO_LEGAL_TYPE'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if ($record['ACCO_IBS_CUSCUN'] === '') {
                    /*
                     * ACCO_IBS_SRCH_ME01
                     * Some date must be specified
                     * Se debe especificar alguno de los campos
                     */
                    //$msg['ACCO_IDENTIFICATION'] = dict('ACCO_IBS_SRCH_ME01');
                    $msg['ACCO_IBS_CUSCUN'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if (user_is_in_acctree() == false) {
                    $error = true;
                    /*
                     * GEN_ERR_USR_NOTFND_ACTREE
                     * User is not registered in Access Tree
                     * Usuario no esta registrado en el Arbol de Acceso
                     */
                    $contents .= note(dict('GEN_ERR_USR_NOTFND_ACTREE'));
                }
                if ($record['ACCO_IBS_CUSCUN'] !== '') {
                    $n_record_qty = ClxCommon::$db->countRecords($this->acconame, 'IBS_CUSCUN = ' . singleQuote($record['ACCO_IBS_CUSCUN']));
                    if ($n_record_qty > CLX_NUM_ONE) {
                        $error = true;
                        /*
                         * GEN_ERR_IBS_REPEATED_CUSCUN
                         * IBS Client Code repated more than one time
                         * El código de cliente de IBS está repetido más de una vez
                         */
                        $contents .= note(dict('GEN_ERR_IBS_REPEATED_CUSCUN'));
                    }
                }
            }
            if (empty($msg) && $error == false) { //is not error just connect
    			if (!class_exists('web_services')){
    				includeCodeLib('WEBSERV');
				}
				$webserv= new web_services();
				$response=$webserv->ws_login(); 
				
				if ($response!=null){
					//$contents .= note($response['access_token']);
					//$contents .= note('Autenticado');
					$response_search=$webserv->ws_search_client($record);
					
					if ($response_search!=null){
						//pre ($response_search);
						
						if ($response_search['legalType']=='2'){
							$ar_acco= $webserv->ws_mapping_fields_acco_personal_response($response_search);
						}else{
							$ar_acco= $webserv->ws_mapping_fields_acco_corporate_response($response_search);
						}
						
   						$ibs_cuscun = $ar_acco['IBS_CUSCUN'];
						
						$cliente_clx = ClxCommon::$db->getRecord('CRM_ACCOUNTS', 'IBS_CUSCUN = ' . singleQuote($ibs_cuscun));

						if (!empty($cliente_clx)){
							//echo 'encontro algo';
							$ar_acco['IDCODE'] = $cliente_clx['IDCODE'];
							$ar_acco['MODIFIEDBY'] = '';
							$ar_acco['MDATE'] = '';
							// Prepares the ACCO record (CRM Accounts)
							$ar_acco = setCreationData($ar_acco);
							ClxCommon::$db->updateRecord($this->acconame, $ar_acco, 'ID = ' . $cliente_clx['ID']);
							
						}else{
							
							//echo 'a create';
							$ar_acco['USERNAME'] = '';
							$ar_acco['CREATEDBY'] = '';
							$ar_acco['MODIFIEDBY'] = '';
							$ar_acco['CDATE'] = '';
							$ar_acco['MDATE'] = '';
							// Prepares the ACCO record (CRM Accounts)
							$ar_acco = setCreationData($ar_acco);
							//obtenemos el IDCODE
							$ar_acco = setCreationIdCode($ar_acco, 'ACCO');
							ClxCommon::$db->createRecord($this->acconame, $ar_acco, true);
							
							$cliente_clx = ClxCommon::$db->getRecord('CRM_ACCOUNTS', 'IBS_CUSCUN = ' . singleQuote($ibs_cuscun));
							$ar_acco['IDCODE'] = $cliente_clx['IDCODE'];
						    
						}
						
						$ar_acco['dataToDisplay']['IDCODE'] = $ar_acco['IDCODE'];
                        $ar_acco['dataToDisplay']['IDNAME'] = $ar_acco['IDNAME'];
                        $ar_acco['dataToDisplay']['LEGAL_TYPE'] = $ar_acco['LEGAL_TYPE'];
                        $ar_acco['dataToDisplay']['ID_TYPE'] = $ar_acco['ID_TYPE'];
                        $ar_acco['dataToDisplay']['IDENTIFICATION'] = $ar_acco['IDENTIFICATION'];
                        $ar_acco['dataToDisplay']['IBS_CUSCUN'] = $ar_acco['IBS_CUSCUN'];
						
						$contents = record_display_dialog($ar_acco['dataToDisplay'], $title, CLX_CLASS_DATA, dict('GEN_LBL_RECORD_FOUND'), false);
						
						//setting more link
						if (!empty($cliente_clx)) {
							if ($ar_acco['ISINDIVIDUAL'] === true) {
								$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_PERS_IBS&qy=ACCO_PERS&relfm=&idlist=' . $cliente_clx['ID'];
							} else {
								$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_CORP_IBS&qy=ACCO_CORP&relfm=&idlist=' . $cliente_clx['ID'];
							}
							$a = new htmlAnchor(dict('GEN_LBL_CLICK_HERE'), $url);
							$contents .= '<br>' . dict('GEN_LBL_FOR_MORE') . ' ' . $a->source() . '<br>';
						}
						
					}else{
					    $contents .= note($webserv->getError());	
					}
					
					
				}else{
					$contents .= note($webserv->getError());
				}
				$webserv->ws_logout();
            }
        }
        //Build search form
		
		$form = $EDIT->defaultForm($formrec, $record, $msg, $ac, $to, $it, 'tool.php', '', $title, dict('GEN_LBL_EXECUTE'));
        return $form->source() . '<br>' . $contents;
        
    }

	
	
	function searchClientOnIbsForm2() {
        //ACCO_IBS_SRCH
        $contents = '';
        $ac = UserRequest::$request['ac'];
        $to = UserRequest::$request[CLX_PROP_TO];
        $it = UserRequest::$request['it'];
        $title = dict('GEN_LBL_SEARCH_IBS_CLIENT');
        $idlist = getIdlist();
        //setting back in case or return
        pushHistory($to, $ac, $idlist, 'it', $it);
        //RTBL_REPCINV
        $EDIT = &getTool('EDIT');
        $record = array();
        $msg = array();
        $error = false;
        $formrec = ClxCommon::$db->getRecord('CLX_UI_FORMS', 'IDCODE = ' . singleQuote('ACCO_IBS_SRCH') . '');
        $record = UserRequest::$post;
        if (!empty($record['confirm'])) {
            $UIFM = &getTool('UIFM');
            $UIFM->userCheckForm($formrec, $record, $msg);
            if (empty($msg)) {//is not error just connect
                if ($record['ACCO_IDENTIFICATION'] !== '' && $record['ACCO_LEGAL_TYPE'] === '') {
                    /*
                     * GEN_ERR_REQUIRED_FIELD
                     * Este es un campo requerido.  Por favor llenelo.
                     * This is a required field.  Please fill it.
                     */
                    $msg['ACCO_LEGAL_TYPE'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if ($record['ACCO_IDENTIFICATION'] === '' && $record['ACCO_LEGAL_TYPE'] !== '') {
                    /*
                     * GEN_ERR_REQUIRED_FIELD
                     * Este es un campo requerido.  Por favor llenelo.
                     * This is a required field.  Please fill it.
                     */
                    $msg['ACCO_IDENTIFICATION'] = dict('GEN_ERR_REQUIRED_FIELD');
                }
                if ($record['ACCO_IDENTIFICATION'] === '' && $record['ACCO_IBS_CUSCUN'] === '') {
                    /*
                     * ACCO_IBS_SRCH_ME01
                     * Some date must be specified
                     * Se debe especificar alguno de los campos
                     */
                    $msg['ACCO_IDENTIFICATION'] = dict('ACCO_IBS_SRCH_ME01');
                    $msg['ACCO_IBS_CUSCUN'] = dict('ACCO_IBS_SRCH_ME01');
                }
                if (user_is_in_acctree() == false) {
                    $error = true;
                    /*
                     * GEN_ERR_USR_NOTFND_ACTREE
                     * User is not registered in Access Tree
                     * Usuario no esta registrado en el Arbol de Acceso
                     */
                    $contents .= note(dict('GEN_ERR_USR_NOTFND_ACTREE'));
                }
                if ($record['ACCO_IBS_CUSCUN'] !== '') {
                    $n_record_qty = ClxCommon::$db->oneExistsRecords($this->acconame, 'IBS_CUSCUN = ' . singleQuote($record['ACCO_IBS_CUSCUN']));
                    if ($n_record_qty > CLX_NUM_ONE) {
                        $error = true;
                        /*
                         * GEN_ERR_IBS_REPEATED_CUSCUN
                         * IBS Client Code repated more than one time
                         * El código de cliente de IBS está repetido más de una vez
                         */
                        $contents .= note(dict('GEN_ERR_IBS_REPEATED_CUSCUN'));
                    }
                }
            }
            if (empty($msg) && $error == false) { //is not error just connect
                $id = '';
                if ($record['ACCO_IBS_CUSCUN'] !== '') {
                    $this->searchClientOnIbsByCuscum_force_ibs_search = true;
                    $a_response = $this->searchClientOnIbsByCuscum($record['ACCO_IBS_CUSCUN']);
                    if ($a_response['error'] == false) {
                        // OK, client was created from IBS
                        $id = $a_response['clx_id'];
                        $isindividual = ($a_response[CLX_CLASS_DATA]['LEGAL_TYPE'] === 'I') ? true : false;
                        $a_response['dataToDisplay']['IDCODE'] = $a_response[CLX_CLASS_DATA]['IDCODE'];
                        $a_response['dataToDisplay']['IDNAME'] = $a_response[CLX_CLASS_DATA]['IDNAME'];
                        $a_response['dataToDisplay']['LEGAL_TYPE'] = $a_response[CLX_CLASS_DATA]['LEGAL_TYPE'];
                        $a_response['dataToDisplay']['ID_TYPE'] = $a_response[CLX_CLASS_DATA]['ID_TYPE'];
                        $a_response['dataToDisplay']['IDENTIFICATION'] = $a_response[CLX_CLASS_DATA]['IDENTIFICATION'];
                        $a_response['dataToDisplay']['IBS_CUSCUN'] = $a_response[CLX_CLASS_DATA]['IBS_CUSCUN'];
                        $contents = record_display_dialog($a_response['dataToDisplay'], $title, CLX_CLASS_DATA, dict('GEN_LBL_RECORD_FOUND'), false);
                    } else {
                        // There was an error
                        $contents .= note($a_response['errormsg']);
                    }
                } else {
                    $parameters = array('IDENTIFICATION' => $record['ACCO_IDENTIFICATION']);
                    $EBSK = &getTool('EBSK');
                    if($EBSK->debug === false) $EBSK->debug = ($this->eibs_send_debug || $this->debug); //   = EBSK message debug enabling by system parameters
                    $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_SRCH, $parameters, 'SELECT');
                    if (!empty($response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {//client not found
                        $contents .= note($response[CLX_NUM_ZERO]['HEADERMSG']['ERDS01'] . ' (' . $response[CLX_NUM_ZERO]['HEADERMSG']['ERNU01'] . ')');
                    } else {
                        $isindividual = ($response[CLX_NUM_ZERO]['LEGAL_TYPE'] === 'I') ? true : false;
                        $contents = record_display_dialog($response[CLX_NUM_ZERO], $title, CLX_CLASS_DATA, dict('GEN_LBL_RECORD_FOUND'), false);
                        //Check if record already exists
                        $ibs_cuscun = $response[CLX_NUM_ZERO]['IBS_CUSCUN'];
                        $parameters = array('IBS_CUSCUN' => $ibs_cuscun);
                        //$id = ClxCommon::$db->getOne('SELECT ID FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = '. singleQuote($ibs_cuscun) );
                        if ($isindividual === true) {
                            $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_NAT_MSG, $parameters, 'LOAD');
                        } else {
                            $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $parameters, 'LOAD');
                        }
                        $id = ClxCommon::$db->getOne('SELECT ID FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = ' . singleQuote($ibs_cuscun));
                    }
                }
                //setting more link
                if (!empty($id)) {
                    if ($isindividual === true) {
                        //$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_PERS&qy=ACCO_PERS&relfm=&idlist=' . $id;
                        $url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_PERS_IBS&qy=ACCO_PERS&relfm=&idlist=' . $id;
                    } else {
                        //$url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_CORP&qy=ACCO_CORP&relfm=&idlist=' . $id;
                        $url = 'tool.php?to=EDIT&ac=DISPLAY&li=ACCO_CORP_IBS&qy=ACCO_CORP&relfm=&idlist=' . $id;
                    }
                    $a = new htmlAnchor(dict('GEN_LBL_CLICK_HERE'), $url);
                    $contents .= '<br>' . dict('GEN_LBL_FOR_MORE') . ' ' . $a->source() . '<br>';
                } else {
                    $contents .= smsg('MSG-252') . '<br>';
                }
            }
        }
        //Build search form
        $form = $EDIT->defaultForm($formrec, $record, $msg, $ac, $to, $it, 'tool.php', '', $title, dict('GEN_LBL_EXECUTE'));
        return $form->source() . '<br>' . $contents;
        ;
    }

    /**
     * Create CLX Client from IBS Client
     * @param Integer $ibs_cuscun
     */
    public $searchClientOnIbsByCuscum_force_ibs_search = false;

    function searchClientOnIbsByCuscum($ibs_cuscun, $debug = false) {
//$debug = true;
        if (!empty($debug))
            echoLog('paso 1<br>');
        $a_response = array('error' => false,
            'errormsg' => '',
            'ibs_cuscun' => $ibs_cuscun,
            'clx_idcode' => null,
            'clx_id' => null,
            CLX_CLASS_DATA => null,
        );
        if (!empty($debug))
            echoLog('paso 2<br>');
        if (empty($ibs_cuscun)) {
            $a_response['error'] = true;
            /*
             * GEN_LBL_SOCK_CUSCUN_BLK
             * IBS Cuscun (Client code) could not be blank
             * Código CUSCUN (Cliente de IBS) no puede estar en blanco
             */
            $a_response['errormsg'] = dict('GEN_LBL_SOCK_CUSCUN_BLK');
        } else {
            if (!empty($debug))
                echoLog('paso 3<br>');
            if ($this->searchClientOnIbsByCuscum_force_ibs_search == false) {
                $accorec = ClxCommon::$db->getOneRecord('SELECT * FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = ' . singleQuote($ibs_cuscun));
                if (!empty($accorec)) {
                    if (!empty($debug))
                        echoLog('paso 4<br>');
                    $a_response['clx_id'] = $accorec['ID'];
                    $a_response['clx_idcode'] = $accorec['IDCODE'];
                    $a_response[CLX_CLASS_DATA] = $accorec;
                    return $a_response;
                }
            }
            if (!empty($debug))
                echoLog('paso 5<br>');
            $EBSK = &getTool('EBSK');
            if($EBSK->debug === false) $EBSK->debug = ($this->eibs_send_debug || $this->debug); //   = EBSK message debug enabling by system parameters
            $parameters = array('IBS_CUSCUN' => $ibs_cuscun
            );
            $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $parameters, 'SELECT');
            //pre($response);
            if (!empty($debug))
                echoLog('paso 6<br>');

            if (isset($response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM']) &&
                    $response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'] !== '0' && $response[CLX_NUM_ZERO]['HEADERMSG']['ERNU01'] === '0477') {
                // Cuscun does not exists
                $response[CLX_NUM_ZERO]['LEGAL_TYPE'] = '';
            }
            if (empty($response [CLX_NUM_ZERO] ['LEGAL_TYPE'])) {
                if (!empty($debug)) {
                    echoLog('paso 7<br>');
                }
                //error reading socket
                $a_response['error'] = true;

                // check if there´s a specific error
                if (isset($response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM'])) {
                    /*
                     * [0][HEADERMSG][ERRNUM] => 1
                     * [[0][HEADERMSG][ERNU01] => 0477
                     * [[0][HEADERMSG][ERDS01] => Numero de Cliente No Esta Definido en el Sistema
                     */
                    $a_response['errormsg'] = '';
                    for ($qq = CLX_NUM_ONE; $qq <= (CLX_NUM_ZERO + $response[CLX_NUM_ZERO]['HEADERMSG']['ERRNUM']); $qq++) {
                        $qqtxt = substr('0' . $qq, -CLX_NUM_TWO);
                        if ($a_response['errormsg'] !== '')
                            $a_response['errormsg'] .= '<br>';
                        $a_response['errormsg'] .= ( '* ' . $response[CLX_NUM_ZERO]['HEADERMSG']['ERNU' . $qqtxt] . ' - ' . $response[CLX_NUM_ZERO]['HEADERMSG']['ERDS' . $qqtxt] );
                    }
                } else {
                    /*
                     * GEN_LBL_SOCK_CNX_FAIL
                     * Fail to connect to socket ^1
                     * Error al conectar al socket ^1
                     */
                    $a_response['errormsg'] = dict('GEN_LBL_SOCK_CNX_FAIL', EBSK_IBS_DEF_CLI_JUR_MSG);
                }
            } else {
                if (!empty($debug))
                    echoLog('paso 8<br>');
                $isindividual = ($response [CLX_NUM_ZERO] ['LEGAL_TYPE'] === 'I') ? true : false;
                if (!empty($debug))
                    echoLog('paso 8.1<br>');
                if ($isindividual === true) {
                    if (!empty($debug))
                        echoLog('paso 8.2<br>');
                    $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_NAT_MSG, $parameters, 'LOAD');
                    if (!empty($debug)) { 
                        pre('$response 8.2');
                        pre($response); 
                    }
                } else {
                    if (!empty($debug))
                        echoLog('paso 8.3<br>');
                    $response = $EBSK->sendMessage(EBSK_IBS_DEF_CLI_JUR_MSG, $parameters, 'LOAD');
                    if (!empty($debug)) { 
                        pre('$response 8.3'); 
                        pre($response); 
                    }
                }
                if (!empty($debug))
                    echoLog('paso 9<br>');
                $accorec = ClxCommon::$db->getOneRecord('SELECT * FROM CRM_ACCOUNTS WHERE IBS_CUSCUN = ' . singleQuote($ibs_cuscun));
                if(isset($response[CLX_INIT_ZERO]['HEADERMSG']['ERRNUM']) && isset($response[CLX_INIT_ZERO]['HEADERMSG']['ERNU01']) && $response[CLX_INIT_ZERO]['HEADERMSG']['ERRNUM'] === 1 && $response[CLX_INIT_ZERO]['HEADERMSG']['ERNU01'] === 0) {
                    // Error en la trama
                    $a_response['error'] = true;
                    /*
                     * GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID
                     * General Header invalid
                     * El tipo de trama es inválido
                     */
                    $a_response['errormsg'] = dict('GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID') . '. ' . $response[CLX_INIT_ZERO]['HEADERMSG']['ERDS01'] . ' [' . $response[CLX_INIT_ZERO]['HEADERMSG']['ERNU01'] . ']';
        	} elseif(isset($response[CLX_INIT_ZERO]['HEADERMSG']['ERRNUM']) && $response[CLX_INIT_ZERO]['HEADERMSG']['ERRNUM'] !== 0) {
		// Error en la respuesta al envia de la trama
                    $a_response['error'] = true;
                    /*
                     * GEN_LBL_ERROR
                     * Error
                     * Error
                     */
                    $ii = substr('0' . $response[CLX_INIT_ZERO][HEADERMSG][ERRNUM], -2);
                    $a_response['errormsg'] = dict('GEN_LBL_ERROR') . ': ' . $response[CLX_INIT_ZERO]['HEADERMSG']['ERDS' . $ii] . ' [' . $response[CLX_INIT_ZERO]['HEADERMSG']['ERNU' . $ii] . ']';
                } elseif (empty($accorec)) {
                    if (!empty($debug))
                        echoLog('paso 10<br>');
                    $a_response['error'] = true;
                    /*
                     * GEN_ERR_IBS_MASSIMPCLI_ERRDAT
                     * Client could not be created. There's errors in his/her/its data. Clientix code: ^1, IBS code: ^2
                     * El Cliente no pudo ser creado. Tiene errores en sus datos. Código Clientix: ^1, código IBS: ^2
                     */
                    $a_response['errormsg'] = dict('GEN_ERR_IBS_MASSIMPCLI_ERRDAT', '??????', $ibs_cuscun);
                } else {
                    if (!empty($debug))
                        echoLog('paso 11<br>');
                    $a_response['clx_id'] = $accorec['ID'];
                    $a_response['clx_idcode'] = $accorec['IDCODE'];
                    $a_response[CLX_CLASS_DATA] = $accorec;
                }
            }
        }
        if (!empty($debug))
            echoLog('paso 12<br>');
        return $a_response;
    }

}

// Class End
/**
 * e-IBS > CLX Client massive import interface
 */
class ibs_crm_account_mass_import extends ibs_client_interface_commons {

    var $ar_clientImport_aditional_fields = null;
    var $s_clientimport_report_base_filespec = 'export/ibs_client_import_';
    var $a_parameters;
    var $o_logfile_mngr;
    var $o_rptfile_mngr;
    var $ar_rpt_rec_type_desc;
    var $i_year_limit_tobe_xxi_century = CLX_NUM_20;
    var $s_custimport_def_cust_type = 'CUST';
    var $b_create_client_procesa_error_ar = false; // false = errores con mensajes amigables, true = errores con trama ELEERR
    var $s_clientImport_operation_type;
    var $a_clientImport_id;
    var $b_sw_validations_on = true;
    var $b_sw_strict_action_valid = false;
    var $b_sw_acco_exist;

    /*
     * Constructor
     *
     * @param text  $s_comp_code
     * @param array $parameters
     * @return void
     */

    function __construct($s_comp_code = null, $parameters = array()) {
        parent::__construct($s_comp_code, $parameters);
        $this->a_parameters = $parameters;
        if (!defined('EIBS_IMPORT_DIR')) {
            // flat files import directory from ibs
            define('EIBS_IMPORT_DIR', trim(companyParameter('EIBS_CLIENTS_IMPORT_DIR', $this->s_comp_code))); // /home/fideicomiso/fide/ibs
            // Tokens identificadores de tramas
            define('IBS_IMPRT_CLI_INDIVIDUAL_TOKEN_ID', '2');
            define('IBS_IMPRT_CLI_INDIVIDUAL_SOCKET_ID', 'CLXNAT');
            define('IBS_IMPRT_CLI_COMPANY_TOKEN_ID', '1');
            define('IBS_IMPRT_CLI_COMPANY_TOKEN_ID_2', '1');
            define('IBS_IMPRT_CLI_COMPANY_SOCKET_ID', 'CLXJUR');
            // access tree code default value
            define('EIBS_CLIENTS_ACCDEP_CODE_DEFVAL', trim(companyParameter('EIBS_CLIENTS_ACCDEP_CODE_DEFVAL', $this->s_comp_code))); // access tree code default value
            // ibs_client_import.txt
            define('EIBS_CLIENTS_IMPORT_FILENAME', trim(companyParameter('EIBS_CLIENTS_IMPORT_FILENAME', $this->s_comp_code)));
        }
        if (!empty($this->debug)) {
            echoLog('EIBS_IMPORT_DIR=' . EIBS_IMPORT_DIR);
            echoLog('IBS_IMPRT_CLI_INDIVIDUAL_TOKEN_ID=' . IBS_IMPRT_CLI_INDIVIDUAL_TOKEN_ID);
            echoLog('IBS_IMPRT_CLI_INDIVIDUAL_SOCKET_ID=' . IBS_IMPRT_CLI_INDIVIDUAL_SOCKET_ID);
            echoLog('IBS_IMPRT_CLI_COMPANY_TOKEN_ID=' . IBS_IMPRT_CLI_COMPANY_TOKEN_ID);
            echoLog('IBS_IMPRT_CLI_COMPANY_TOKEN_ID_2=' . IBS_IMPRT_CLI_COMPANY_TOKEN_ID_2);
            echoLog('IBS_IMPRT_CLI_COMPANY_SOCKET_ID=' . IBS_IMPRT_CLI_COMPANY_SOCKET_ID);
            echoLog('EIBS_CLIENTS_ACCDEP_CODE_DEFVAL=' . EIBS_CLIENTS_ACCDEP_CODE_DEFVAL);
            echoLog('EIBS_CLIENTS_IMPORT_FILENAME=' . EIBS_CLIENTS_IMPORT_FILENAME);
        }
    }

    /**
     * Clients import (contract participants/crm accounts) from a flat file created by IBS
     *
     * @param text $s_import_filename
     * @return boolean
     */
    function clientImport($s_import_filename = null) {
        $this->ar_rpt_rec_type_desc = array('1' => dict('4a833c1d522139d325119f57368d06a2'), // Corporate / Persona Jurídica
            '2' => dict('e625663a5621ee43f4bca9dce436dd85'), // Individual / Persona Natural
            '3' => dict('GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID') // General Header invalid / El tipo de trama es inválido
        );
        $EBSK = getTool('EBSK');
        if($EBSK->debug === false) $EBSK->debug = $this->debug;
        //$EBSK->tfields_only = true;
        //$EBSK->remove_decimal_point = true;
        // Parameters conversion...
        if (is_array($s_import_filename)) {
            // New version: pass the whole parameters array
            $this->a_parameters = $s_import_filename;
            $this->proc_eng_params = new proc_eng_params($this->a_parameters);
            if (isset($this->a_parameters['ANYFILE'])) {
                $s_import_filename = $this->a_parameters['ANYFILE'];
            } else {
                $s_import_filename = null;
            }
        } else {
            // Old version: pass only ANYFILE element
        }
        // Verify process engine parameters
        if (!$this->proc_eng_params->verify_parameters()) {
            $s_response = '<h1>' . $this->proc_eng_params->errormsg . '</h1>';
            echoLog($s_response);
            return false;
        }
        // Verify test file only flag
        $this->put_def_par_value('FLG_PRODUCTION', '1'); // In production by default
        // Debug flag: to view file's content even in production
        $this->put_def_par_value('FLG_DEBUG', '0'); // Debug off by default
        // Default import dir
        $dir = EIBS_IMPORT_DIR;
        if (substr($dir, CLX_NUM_MINUS_1) != DIRECTORY_SEPARATOR)
            $dir .= DIRECTORY_SEPARATOR;
        // Default import filename
        if ($s_import_filename == null || $s_import_filename === '') {
            $s_import_filename = $dir . EIBS_CLIENTS_IMPORT_FILENAME;
        }
        // Validates import file's filespec
        if (!file_exists($s_import_filename)) {
            // There's nothing to import
            echoLog('File not found: ' . $s_import_filename);
            return true;
        }

        $o_clx_file = new clx_file($s_import_filename, 'r');
        $o_clx_file->open();
        if ($o_clx_file->error === true) {
            echoLog($o_clx_file->errorMsg);
            return false;
        }
        if ($this->a_parameters['FLG_DEBUG'] === '1') {
            $s_import_log_filename = $this->s_clientimport_report_base_filespec . xFormat::getNow() . '_DEBUG' . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT;
            $this->o_logfile_mngr = new clx_logfile_mngr($s_import_log_filename);
            if ($this->o_logfile_mngr->error === true) {
                echoLog($this->o_logfile_mngr->errorMsg);
                return false;
            }
        }
        $s_import_rpt_filename = $this->s_clientimport_report_base_filespec . xFormat::getNow() . '_REPORT' . EIBS_CLIENTS_IMPORT_FILENAME_ERRFILE_EXT;
        $this->o_rptfile_mngr = new clx_logfile_mngr($s_import_rpt_filename);
        if ($this->o_rptfile_mngr->error) {
            echoLog($this->o_rptfile_mngr->errorMsg);
            return false;
        }
        $s_htm_open_tags = '<br>' . CLX_CHAR_NL . htmlentities(dict('GEN_MSG_IBS_IMPCLI_TITLE')) . '<br>' . CLX_CHAR_NL . '{SUBTITLE}' . '<br>' . CLX_CHAR_NL . $s_import_filename . '<br>' . CLX_CHAR_NL . $s_import_rpt_filename . ($this->a_parameters['FLG_DEBUG'] === '1' ? '<br>' . CLX_CHAR_NL . $s_import_log_filename : '') . '<br>' . CLX_CHAR_NL . '<br>' . CLX_CHAR_NL;
        $s_htm_closing_tags = '';
        $s_title_debug_data = 'DEBUG DATA';
        // Open html tags
        if ($this->a_parameters['FLG_DEBUG'] === '1') {
            $this->o_logfile_mngr->write(str_replace('{SUBTITLE}', '<br>' . CLX_CHAR_NL . $s_title_debug_data, $s_htm_open_tags)
            );
        }
        $this->o_rptfile_mngr->write(str_replace('{SUBTITLE}', '', $s_htm_open_tags) . '###' . ' | ' . htmlentities(dict('GEN_LBL_TYPE')) . // Type / Tipo
                ' | ' . htmlentities(dict('18bb425cd546e695449a5293174faa65-1')) . // IBS Customer No. / 	# Cliente IBS
                ' | ' . htmlentities(dict('GEN_LBL_NAME')) . // Name / Nombre
                ' | ' . htmlentities(dict('23486c49ec6dafacc37cd66524980107')) . // Memo / Observación
                '<br>' . CLX_CHAR_NL
        );
        $buffer = $o_clx_file->read_line();
        $i_proc_counter = CLX_INIT_ZERO;
        /*
         * GEN_MSG_IBS_IMPCLI_TITLE
         * Participans (clients) import from IBS
         * Importación de Participantes (Clientes) desde IBS
         */
        echoLog('<br>' . CLX_CHAR_NL);
        echoLog('<br>' . CLX_CHAR_NL . dict('GEN_MSG_IBS_IMPCLI_TITLE'));
        echoLog('<br>' . CLX_CHAR_NL . $s_import_filename);
        echoLog('<br>' . CLX_CHAR_NL . $s_import_rpt_filename);
        if (array_key_exists('FLG_DEBUG', $this->a_parameters) && $this->a_parameters['FLG_DEBUG'] === '1') {
            echoLog('<br>' . CLX_CHAR_NL . $s_import_log_filename);
        }
        if (!empty($this->debug)) {
            echoLog('<br>' . CLX_CHAR_NL . debug_dump('Param.', $this->a_parameters));
        } else {
            echoLog('<br>' . CLX_CHAR_NL . 'FLG_DEBUG = ' . $this->a_parameters['FLG_DEBUG']);
            echoLog('<br>' . CLX_CHAR_NL . 'FLG_PRODUCTION = ' . $this->a_parameters['FLG_PRODUCTION']);
        }
        echoLog('<br>' . CLX_CHAR_NL);
        while (!$o_clx_file->eof) {
            $i_proc_counter++;
            $s_tipo_trama = substr($buffer, CLX_NUM_42, CLX_NUM_ONE);
            $this->s_clientImport_operation_type = '';
            // Reads next flat file line
            if (trim($buffer) !== '') {
                // Line with contents... (no blank)	//
                $rec_id_info = null; // To hold the client info to be showed on debug screen
                switch (trim($s_tipo_trama)) {
                    // Persona Natural
                    case IBS_IMPRT_CLI_INDIVIDUAL_TOKEN_ID:
                        $msg_code = IBS_IMPRT_CLI_INDIVIDUAL_SOCKET_ID;
                        $persona = 'Natural';
                        $yes_record = true;
                        break;
                    case IBS_IMPRT_CLI_COMPANY_TOKEN_ID:
                    case IBS_IMPRT_CLI_COMPANY_TOKEN_ID_2:
                        $msg_code = IBS_IMPRT_CLI_COMPANY_SOCKET_ID;
                        $persona = 'Juridica';
                        $yes_record = true;
                        break;
                    default :
                        $yes_record = false;
                        break;
                }
                if (!empty($yes_record)) {
                    $ar_record = $EBSK->readMessage($msg_code, $buffer);
                    if ($this->a_parameters['FLG_DEBUG'] === '1') {
                        $this->o_logfile_mngr->write('<hr>' . $this->get_eibs_data_packet_debug_info($ar_record) . '<br>' . CLX_CHAR_NL);
                    }
                    if (!empty($this->debug))
                        echoLog(debug_dump('AR Persona ' . $persona, $ar_record));
                }
                switch (trim($s_tipo_trama)) {
                    // Persona Natural
                    case IBS_IMPRT_CLI_INDIVIDUAL_TOKEN_ID:
                        $ar_response = $this->create_client_individual_frm_ibs($i_proc_counter, $ar_record, $rec_id_info);
                        $s_response = $ar_response['errormsg'];
                        break;
                    // Persona Juridica
                    case IBS_IMPRT_CLI_COMPANY_TOKEN_ID:
                    case IBS_IMPRT_CLI_COMPANY_TOKEN_ID_2:
                        $ar_response = $this->create_client_company_frm_ibs($i_proc_counter, $ar_record, $rec_id_info);
                        $s_response = $ar_response['errormsg'];
                        break;
                    // Invalid socket id
                    default:
                        /**
                         * GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID
                         * General Header invalid
                         * El tipo de trama es inválido
                         */
                        $s_desc = dict('GEN_ERR_IBS_IMPCLI_WRONG_SOCKET_ID') . ' ($s_tipo_trama)';
                        $s_response = $this->create_client_procesa_error_ar($i_proc_counter, array('0010' => $this->create_error_record_ar($s_desc, 'HEADER')));
                        break;
                }
                // Prepares line data
                $rec_id_info_txt = '';
                $rec_id_info_html = '';
                if ($rec_id_info != null) {
                    reset($rec_id_info);
                    $s_sep = '';
                    foreach ($rec_id_info as $name => $val) {
                        $val = htmlentities($val);
                        $rec_id_info_txt .=' ' . $name . ' = ' . $val . ' ';
                        $rec_id_info_html .= $s_sep . $val;
                        $s_sep = ' | ';
                    }
                } else {
                    $rec_id_info_txt .= $this->ar_rpt_rec_type_desc['3'] . ' ';
                    $rec_id_info_html = htmlentities($this->ar_rpt_rec_type_desc['3']);
                }
                // Writes log  file
                $s_response = htmlentities($s_response);
                if ($this->a_parameters['FLG_DEBUG'] === '1') {
                    $rec_id_info_txt = $i_proc_counter . ') ' . $rec_id_info_txt . ' : ->' . $s_response . '<-';
                    $this->o_logfile_mngr->write($rec_id_info_txt . '<br>' . CLX_CHAR_NL);
                }
                $rec_id_info_html = $i_proc_counter . ' | ' . $rec_id_info_html . ' | ' . $s_response . '<br>' . CLX_CHAR_NL;
                $this->o_rptfile_mngr->write($rec_id_info_html);
            }
            // Reads next flat file line
            $buffer = $o_clx_file->read_line();
        }
        // Link to the log file
        $a_enlaces = array();
        $a_enlaces[] = array('FILENAME' => kWebSite . $s_import_rpt_filename,
            'FILENAME_PLAIN' => kWebSite . $s_import_rpt_filename,
        );
        if ($this->a_parameters['FLG_DEBUG'] === '1') {
            $a_enlaces[] = array('FILENAME' => kWebSite . $s_import_log_filename,
                'FILENAME_PLAIN' => kWebSite . $s_import_log_filename,
                'TITLE' => $s_title_debug_data
            );
        }
        echoLog($this->proc_eng_params->get_file_link($a_enlaces));
        // Close html tags
        if ($this->a_parameters['FLG_DEBUG'] === '1') {
            $this->o_logfile_mngr->write($s_htm_closing_tags);
        }
        // close input file
        $o_clx_file->close();
        if ($o_clx_file->error === true) {
            echoLog($o_clx_file->errorMsg);
        }
        // close output error file
        if ($this->a_parameters['FLG_DEBUG'] === '1') {
            $this->o_logfile_mngr->close();
            if ($this->o_logfile_mngr->error === true) {
                echoLog($this->o_logfile_mngr->errorMsg);
            }
        }
        $this->o_rptfile_mngr->close();
        if ($this->o_rptfile_mngr->error) {
            echoLog($this->o_rptfile_mngr->errorMsg);
        }
        return true;
    }

// Method End
    /**
     * $parameters variable element default value assignment
     * @param text $s_par_name
     * @param variant $v_par_def_value
     * @return void
     */
    function put_def_par_value($s_par_name, $v_par_def_value) {
        if (!isset($this->a_parameters[$s_par_name])) {
            // In production by default
            $this->a_parameters[$s_par_name] = '';
        }
        if (trim($this->a_parameters[$s_par_name]) === '') {
            // In production by default
            $this->a_parameters[$s_par_name] = $v_par_def_value;
        }
    }

// Method End
    /**
     * Dumps eIbs data packet array into text variable and returns it
     *
     * @param array $ar_record
     * @param text $s_sep_real
     * @return text
     */
    function get_eibs_data_packet_debug_info($ar_record, $s_sep_real = '<BR>') {
        $s_response = '';
        $s_sep = '';
        $total = count($ar_record);
        for ($i_i = CLX_INIT_ZERO; $i_i < $total; $i_i++) {
            $s_response .= $s_sep . $ar_record[$i_i]['FILEFIELD'] . ' = ' . $ar_record[$i_i]['VAL'];
            $s_sep = $s_sep_real;
        }
        return $s_response;
    }

    /**
     * ACCO Company type record creation (persona juridica)
     * @param integer $i_proc_counter
     * @param array $ar_record
     * @param text $rec_id_info
     * @return array
     */
    function create_client_company_frm_ibs($i_proc_counter, $ar_record, &$rec_id_info) {
        $rec_id_info = array();
        if (!empty($this->debug))
            echoLog(debug_dump('create_client_company_frm_ibs(ar_record)', $ar_record));
        // arrays to store the record columns values to be saved in the rows of each table that
        // conforms the contact info (acco, aclo, acph, etc.)
        $ar_acco = array();
        $ar_cont = array();
        $ar_dyco = array();
        $ar_acph = array();
        $ar_acea = array();
        $ar_aclo = array();
        $ar_error = array();
        $total = count($ar_record);
        for ($i_i = CLX_INIT_ZERO; $i_i < $total; $i_i++) {
            $val = trim($ar_record[$i_i]['VAL']);
            if (!empty($this->debug))
                echoLog('<br>FILEFIELD=' . $ar_record[$i_i]['FILEFIELD'] . ', valor=' . $val);
            // Reads each filed n the line, assigns the corresponding table column in the correct array
            switch ($ar_record[$i_i]['FILEFIELD']) {
                case 'H02OPE': // tabla: , campo: H02OPE, comentario: H02OPE
                    $this->s_clientImport_operation_type = $val;
                    if (!empty($this->debug))
                        echoLog('<br>OPERACION - ' . $ar_record[$i_i]['FILEFIELD'] . ' / s_clientImport_operation_type=' . $this->s_clientImport_operation_type);
                    break;
                case 'E02LGT': // tabla: CRM_ACCOUNTS, campo: LEGAL_TYPE, comentario: LEGAL TYPE, 1 Juridica, 2 Natural, 3 Other
                    $ar_acco['LEGAL_TYPE'] = $val;
                    $rec_id_info['T'] = $val;
                    break;
                case 'E02CUN': // tabla: CRM_ACCOUNTS, campo: IBS_CUSCUN, comentario: CUSTOMER NUMBER Blank on create: Modify and Query: # Customer
                    //$ar_acco['IBS_CUSCUN'] = 0+$val;	//  eliminacion de zeros del cuscun
                    $ar_acco['IBS_CUSCUN'] = $this->ibs_cuscun_code_fix($val);
                    $rec_id_info['CUSCUN'] = $val;
                    break;
                case 'E02NA1': // tabla: CRM_ACCOUNTS, campo: IDNAME, comentario: CUSTOMER NAME
                    $ar_acco['IDNAME'] = $val;
                    $rec_id_info['N'] = $val;
                    break;
                case 'E02NA2': // tabla: CRM_LOCATIONS, campo: ADDRESS1, comentario: CUSTOMER NAME 2 (CRM_ACCOUNT.E02NA2)  Venezuela: calle/avenida
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS1'] = $val;
                    } else {
                        $ar_acco['ADDRESS1'] = $val;
                    }
                    break;
                case 'E02NA3': // tabla: CRM_ACCOUNT, campo: E02NA3, comentario: CUSTOMER NAME 3 (CRM_ACCOUNT.E02NA3)  Venezuela: Casa/Edif. (15), Piso/Nivel (5), Apto/Of/Local (15), Total=35
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS1'] .= ' ' . $val;
                    } else {
                        $ar_acco['ADDRESS1'] .= ' ' . $val;
                    }
                    break;
                case 'E02NA4': // tabla: CRM_LOCATIONS, campo: ADDRESS2, comentario: Antes: CRM_ACCOUNT.E02NA4  Venezuela: referencia de la dirección
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS2'] = $val;
                    } else {
                        $ar_acco['ADDRESS2'] = $val;
                    }
                    break;
                case 'E02CTY': // tabla: CRM_LOCATIONS, campo: CITY, comentario: CITY
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['CITY'] = substr($val, CLX_NUM_ZERO, CLX_NUM_FOUR);
                    } else {
                        $ar_acco['CITY'] = substr($val, CLX_NUM_ZERO, CLX_NUM_FOUR);
                    }
                    break;
                case 'E02STE': // tabla: CRM_LOCATIONS, campo: STATE, comentario: STATE
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['STATE'] = $val;
                    } else {
                        $ar_acco['STATE'] = $val;
                    }
                    break;
                case 'E02CTR': // tabla: CRM_LOCATIONS, campo: COUNTRY, comentario: COUNTRY
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['COUNTRY'] = $val;
                    } else {
                        $ar_acco['COUNTRY'] = $val;
                    }
                    break;
                case 'E02ZPC': // tabla: CRM_LOCATIONS, campo: ZIPCODE, comentario: Zip postal code
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ZIPCODE'] = $val;
                    } else {
                        $ar_acco['ZIPCODE'] = $val;
                    }
                    break;
                case 'E02IDN': // tabla: CRM_ACCOUNTS, campo: ID_NO, comentario: Identification
                    $val = ($ar_record[$i_i]['VAL']);
                    $ar_acco['IDENTIFICATION'] = $val;
                    break;
                case 'E02TID': // tabla: CRM_ACCOUNTS, campo: ID_TYPE, comentario: Identification Type CEDU -> Cedula. RUC -> Registro unico de contribuyente
                    $ar_acco['ID_TYPE'] = $val;
                    break;
                case 'E02PID': // tabla: CRM_ACCOUNTS, campo: ID_COUNTRY, comentario: COUNTRY OF ID
                    if ($val !== '')
                        $ar_acco['INCORPORATION_COUNTRY'] = $val;
                    break;
                case 'E02BDM': // tabla: CRM_ACCOUNTS, campo: INCORPORATION_DATEMM, comentario: Incorporation Date Month
                    if (isset($ar_acco['INCORPORATION_DATE']) && !is_array($ar_acco['INCORPORATION_DATE']))
                        $ar_acco['INCORPORATION_DATE'] = array();
                    $ar_acco['INCORPORATION_DATE']['MM'] = $val;
                    break;
                case 'E02BDD': // tabla: CRM_ACCOUNTS, campo: INCORPORATION_DATEDD, comentario: Incorporation Date Day
                    if (isset($ar_acco['INCORPORATION_DATE']) && !is_array($ar_acco['INCORPORATION_DATE']))
                        $ar_acco['INCORPORATION_DATE'] = array();
                    $ar_acco['INCORPORATION_DATE']['DD'] = $val;
                    break;
                case 'E02BDY': // tabla: CRM_ACCOUNTS, campo: INCORPORATION_DATEYY, comentario: Incorporation Year YY
                    if (isset($ar_acco['INCORPORATION_DATE']) && !is_array($ar_acco['INCORPORATION_DATE']))
                        $ar_acco['INCORPORATION_DATE'] = array();
                    $ar_acco['INCORPORATION_DATE']['YY'] = $val;
                    break;
                case 'E02LIF': // tabla: CRM_ACCOUNTS, campo: LANGUAGE_FLAG, comentario: Language Flag E/S
                    $ar_acco['CORRESPONDENCE_LANGUAGE'] = ($val === 'S' ? 'SP' : ($val === 'E' ? 'EN' : $val));
                    break;
                case 'E02STS': // tabla: CRM_ACCOUNTS, campo: ACCSTAT, comentario: CUSTOMER STATUS '1.Inactivo 2.Activo 3.Lista Negra'
                    $val = ($val === '2') ? $val = 'I' : $val = 'A'; //  Se cambio ya que en IBS: 1=activo, 2=inactivo
                    $ar_acco['ACCSTAT'] = $val;
                    break;
                case 'E02HPN': // tabla: CRM_PHONES, campo: PHONE_W, comentario: Office phone # 1
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Of. 1',
                            'PHTYPE' => 'W',
                            'ISPRIMARY' => '1'
                        );
                    } else {
                        $ar_acco['PHONE'] = $val;
                    }
                    break;
                case 'E02PHN': // tabla: , campo: , comentario: Office Phone # 2
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Of. 2',
                            'PHTYPE' => 'W',
                            'ISPRIMARY' => '0'
                        );
                    }
                    break;
                case 'E02PH1': // tabla: CRM_PHONES, campo: PHONE_M, comentario:  Mobile Phone
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Mob.',
                            'PHTYPE' => 'M',
                            'ISPRIMARY' => '0'
                        );
                    }
                    break;
                case 'E02FAX': // tabla: CRM_PHONES, campo: PHONE_F, comentario:  Venezuela: Fax
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Fax',
                            'PHTYPE' => 'FX',
                            'ISPRIMARY' => '0'
                        );
                    } else {
                        $ar_acco['FAX'] = $val;
                    }
                    break;
                case 'E02IAD': // tabla: CRM_EADDRESSES, campo: EMAIL, comentario:  E-mail Address
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acea['ADDRESS'] = $val;
                        $ar_acea['EFFTDATE'] = adodb_date('Y-m-d');
                        $ar_acea['IDNAME'] = 'E-mail principal';
                        $ar_acea['TYPE'] = 'E';
                        $ar_acea['ISPRIMARY'] = '1';
                        $ar_acea['SEQNUM'] = CLX_NUM_ONE;
                    } else {
                        $ar_acco['EMAIL'] = $val;
                    }
                    break;
                case 'E02INC': // tabla: CRM_ACCOUNTS, campo: INDUSTRYCLASS, comentario: COD INDUSTRIA
                    $ar_acco['INDUSTRYCLASS'] = $val;
                    break;
                case 'E02GEC': // tabla: CRM_ACCOUNTS, campo: INCORPORATION_COUNTRY, comentario: PAIS RESIDENCIA OR CORPORATION INCORPORATION COUNTRY
                    if ($val !== '')
                        $ar_acco['INCORPORATION_COUNTRY'] = $val;
                    break;
                case 'E02UC4': // tabla: CRM_ACCOUNT, campo: VE_CLI_SUDEBAN, comentario:  Venezuela: código SUDEBAN clientes
                    $ar_acco['VE_CLI_SUDEBAN'] = $val;
                    break;
                    break;
                default:
                    break;
            }
/*
default:
                case 'HEADER': // tabla: , campo: , comentario: General Header must be first 10 positions of every message.
                    break;
                case 'H02USR': // tabla: CLX_USERS, campo: USERNAME, comentario: User Name
                    break;
                case 'H02PGM': // tabla: , campo: , comentario: H02PGM
                    break;
                case 'H02TIM': // tabla: , campo: , comentario: H02TIM
                    break;
                case 'H02SCR': // tabla: , campo: , comentario: H02SCR
                    break;
   
 *              case 'H02MAS': // tabla: , campo: , comentario: MORE RECORD
                    break;
                case 'H02WK1': // tabla: , campo: , comentario: LENGUAJE USER
                    break;
                case 'H02WK2': // tabla: , campo: , comentario: H02WK2
                    break;
                case 'H02WK3': // tabla: , campo: , comentario: FLAG 2
                    break;
   
 *              case 'E02SHN': // tabla: CRM_ACCOUNTS, campo: SHORT_NAME, comentario: SHORT NAME
                    break;
                case 'E02POB': // tabla: CRM_LOCATIONS, campo: POBOX, comentario: POBOX
                    break;
                case 'E02CP1': // tabla: , campo: , comentario: E02CP1
                    break;
                case 'E02CP2': // tabla: , campo: , comentario: E02CP2
                    break;
                case 'E02CP3': // tabla: , campo: , comentario: E02CP3
                    break;
                case 'E02FN2': // tabla: , campo: , comentario: E02FN2
                    break;
                case 'E02LN1': // tabla: , campo: , comentario: E02LN1
                    break;
                case 'E02LN2': // tabla: , campo: , comentario: E02LN2
                    break;
                case 'E02CCS': // tabla: , campo: , comentario: E02CCS
                    break;
   
 *              case 'E02IDF': // tabla: , campo: , comentario: E02IDF
                    break;
                case 'E02TIF': // tabla: , campo: , comentario: E02TIF
                    break;
                case 'E02PIF': // tabla: , campo: , comentario: E02PIF
                    break;
                case 'E02ID3': // tabla: , campo: , comentario: E02ID3
                    break;
                case 'E02TI3': // tabla: , campo: , comentario: E02TI3
                    break;
                case 'E02PI3': // tabla: , campo: , comentario: E02PI3
                    break;
                case 'E02ID4': // tabla: , campo: , comentario: E02ID4
                    break;
                case 'E02TI4': // tabla: , campo: , comentario: E02TI4
                    break;
                case 'E02PI4': // tabla: , campo: , comentario: E02PI4
                    break;
                case 'E02FL2': // tabla: , campo: , comentario: E02FL2
                    break;
   
                case 'E02IDM': // tabla: , campo: , comentario: E02IDM
                    break;
                case 'E02IDD': // tabla: , campo: , comentario: E02IDD
                    break;
                case 'E02IDY': // tabla: , campo: , comentario: E02IDY
                    break;
                case 'E02IEM': // tabla: CRM_ACCOUNTS, campo: OPESTART_DATE_MM, comentario: Operation Start Month
                    break;
                case 'E02IED': // tabla: CRM_ACCOUNTS, campo: OPESTART_DATE_DD, comentario: Operation Start Day
                    break;
                case 'E02IEY': // tabla: CRM_ACCOUNTS, campo: OPESTART_DATE_YY, comentario: Operation Date Year
                    break;
                case 'E02REM': // tabla: , campo: , comentario: E02REM
                    break;
                case 'E02RED': // tabla: , campo: , comentario: E02RED
                    break;
                case 'E02REY': // tabla: , campo: , comentario: E02REY
                    break;
                case 'E02REN': // tabla: , campo: , comentario: E02REN
                    break;
 
 *               case 'E02RLV': // tabla: CRM_ACCOUNTS, campo: ROLLOVER_VOLUME, comentario: ROLLOVER_VOLUME
                    break;
                case 'E02GRP': // tabla: CRM_ACCOUNTS, campo: GROUP_CODE, comentario: GROUP CODE
                    break;
                case 'E02SEX': // tabla: CRM_ACCOUNTS, campo: E02SEX, comentario: Natural: 'M Male' 'F Female' Juridicas: 'I International' 'T Multinational' 'S Small Business' 'O Other''
                    break;
                case 'E02TYP': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_TYPE_IBS, comentario: CUSTOMER TYPE R.Regular,M.Master,G.Group
                    break;
                case 'E02MST': // tabla: , campo: , comentario: MARITAL STATUS
                    break;
                case 'E02ILV': // tabla: , campo: , comentario: E02ILV
                    break;
   
                case 'E02MLC': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'E02CCL': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_CLASSIFICATION, comentario: Customer Classification 1- Ente Gubernamental, 2 - Entedecentralizado, ''
                    break;
                case 'E02RSL': // tabla: , campo: , comentario: RISK LEVEL '' No aplica CNOFC:31
                    break;
                case 'E02ATM': // tabla: , campo: , comentario: ATM ISSUED Y/N '*blanks '1' '2' '3' 'Y' 'N' '
                    break;
                case 'E02TAX': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_TAX, comentario: CHARGE TAX 1/2/3/N '1 RETENCION SOBRE INTERESS ISR 2 COBRE DEL IVA 3 IVA MAS ISR 4 IVA SOLO EN COMISIONES 5 IVA SOLO EN INTERESES 6 DEBITO BANCARIO IDB 7 IDB MAS ISR 8 IDB MAS IVA 9 CUALQUIERA TIPO DE IMPUESTO N NO CALCULA IMPUESTOS '
                    break;
 
                case 'E02EDL': // tabla: , campo: , comentario: E02EDL
                    break;
                case 'E02INL': // tabla: , campo: , comentario: E02INL
                    break;
                case 'E02BUC': // tabla: , campo: , comentario: E02BUC
                    break;
                case 'E02COB': // tabla: , campo: , comentario: E02COB
                    break;
                case 'E02RBY': // tabla: , campo: , comentario: E02RBY
                    break;
                case 'E02RBN': // tabla: , campo: , comentario: E02RBN
                    break;
                case 'E02FL3': // tabla: , campo: , comentario: E02FL3
                    break;
                case 'E02STF': // tabla: , campo: , comentario: E02STF
                    break;
                case 'E02SOI': // tabla: , campo: , comentario: E02SOI
                    break;
                case 'E02NSO': // tabla: , campo: , comentario: E02NSO
                    break;
                case 'E02FL1': // tabla: , campo: , comentario: E02FL1
                    break;
     
               case 'E02NST': // tabla: , campo: , comentario: E02NST
                    break;
                case 'E02NSH': // tabla: , campo: , comentario: E02NSH
                    break;
                case 'E02CAI': // tabla: , campo: , comentario: E02CAI
                    break;
                case 'E02CAS': // tabla: , campo: , comentario: E02CAS
                    break;
                case 'E02CAP': // tabla: , campo: , comentario: E02CAP
                    break;
                case 'E02AMT': // tabla: , campo: , comentario: E02AMT
                    break;
                case 'E02OFC': // tabla: , campo: OFICIAL, comentario: E02OFC
                    break;
                case 'E02OF2': // tabla: , campo: OFICIAL2, comentario: OFICIAL2
                    break;
 
 *                case 'E02UC1': // tabla: , campo: , comentario: E02UC1
                    break;
                case 'E02UC2': // tabla: , campo: , comentario: E02UC2
                    break;
                case 'E02UC3': // tabla: , campo: , comentario: E02UC3
                    break;
 
                  case 'E02UC5': // tabla: , campo: , comentario: E02UC5
                    break;
                case 'E02UC6': // tabla: , campo: , comentario: E02UC6
                    break;
                case 'E02UC7': // tabla: , campo: , comentario: E02UC7
                    break;
                case 'E02UC8': // tabla: , campo: , comentario: E02UC8
                    break;
                case 'E02UC9': // tabla: , campo: , comentario: E02UC9
                    break;
                case 'E02PRV': // tabla: CRM_ACCOUNTS, campo: PREVISIONTABLE_IBS, comentario: TABLE'PREVISION
                    break;
                case 'E02FL4': // tabla: , campo: , comentario: E02FL4
                    break;
                case 'E02FL5': // tabla: , campo: , comentario: E02FL5
                    break;
                case 'E02SFR': // tabla: , campo: , comentario: E02SFR
                    break;
                case 'E02SDY': // tabla: , campo: , comentario: E02SDY
                    break;
                case 'D02TID': // tabla: , campo: , comentario: D02TID
                    break;
                case 'D02PID': // tabla: , campo: , comentario: D02PID
                    break;
                case 'D02TIF': // tabla: , campo: , comentario: D02TIF
                    break;
                case 'D02PIF': // tabla: , campo: , comentario: D02PIF
                    break;
                case 'D02TI3': // tabla: , campo: , comentario: D02TI3
                    break;
                case 'D02PI3': // tabla: , campo: , comentario: D02PI3
                    break;
                case 'D02TI4': // tabla: , campo: , comentario: D02TI4
                    break;
                case 'D02PI4': // tabla: , campo: , comentario: D02PI4
                    break;
                case 'D02STE': // tabla: , campo: , comentario: D02STE
                    break;
                case 'D02CCS': // tabla: , campo: , comentario: D02CCS
                    break;
                case 'D02RBY': // tabla: , campo: , comentario: D02RBY
                    break;
                case 'D02EDL': // tabla: , campo: , comentario: D02EDL
                    break;
                case 'D02INC': // tabla: , campo: , comentario: D02INC
                    break;
                case 'D02BUC': // tabla: , campo: , comentario: D02BUC
                    break;
                case 'D02COB': // tabla: , campo: , comentario: D02COB
                    break;
                case 'D02RSL': // tabla: , campo: , comentario: D02RSL
                    break;
                case 'D02GEC': // tabla: , campo: , comentario: D02GEC
                    break;
                case 'D02OFC': // tabla: , campo: , comentario: D02OFC
                    break;
                case 'D02OF2': // tabla: , campo: , comentario: D02OF2
                    break;
                case 'D02UC1': // tabla: , campo: , comentario: D02UC1
                    break;
                case 'D02UC2': // tabla: , campo: , comentario: D02UC2
                    break;
                case 'D02UC3': // tabla: , campo: , comentario: D02UC3
                    break;
                case 'D02UC4': // tabla: , campo: , comentario: D02UC4
                    break;
                case 'D02UC5': // tabla: , campo: , comentario: D02UC5
                    break;
                case 'D02UC6': // tabla: , campo: , comentario: D02UC6
                    break;
                case 'D02UC7': // tabla: , campo: , comentario: D02UC7
                    break;
                case 'D02UC8': // tabla: , campo: , comentario: D02UC8
                    break;
                case 'D02UC9': // tabla: , campo: , comentario: D02UC9
                    break;
                case 'D02MLC': // tabla: , campo: , comentario: D02MLC
                    break;
                case 'D02SOI': // tabla: , campo: , comentario: D02SOI
                    break;
                case 'F02LGT': // tabla: , campo: , comentario: F02LGT
                    break;
                case 'F02CUN': // tabla: , campo: , comentario: F02CUN
                    break;
                case 'F02NA1': // tabla: , campo: , comentario: F02NA1
                    break;
                case 'F02NA2': // tabla: , campo: , comentario: F02NA2
                    break;
                case 'F02NA3': // tabla: , campo: , comentario: F02NA3
                    break;
                case 'F02NA4': // tabla: , campo: , comentario: F02NA4
                    break;
                case 'F02CTY': // tabla: , campo: , comentario: F02CTY
                    break;
                case 'F02STE': // tabla: , campo: , comentario: F02STE
                    break;
                case 'F02CTR': // tabla: , campo: , comentario: F02CTR
                    break;
                case 'F02SHN': // tabla: , campo: , comentario: F02SHN
                    break;
                case 'F02POB': // tabla: , campo: , comentario: F02POB
                    break;
                case 'F02CP1': // tabla: , campo: , comentario: F02CP1
                    break;
                case 'F02CP2': // tabla: , campo: , comentario: F02CP2
                    break;
                case 'F02CP3': // tabla: , campo: , comentario: F02CP3
                    break;
                case 'F02FN2': // tabla: , campo: , comentario: F02FN2
                    break;
                case 'F02LN1': // tabla: , campo: , comentario: F02LN1
                    break;
                case 'F02LN2': // tabla: , campo: , comentario: F02LN2
                    break;
                case 'F02CCS': // tabla: , campo: , comentario: F02CCS
                    break;
                case 'F02ZPC': // tabla: , campo: , comentario: F02ZPC
                    break;
                case 'F02IDN': // tabla: , campo: , comentario: F02IDN
                    break;
                case 'F02TID': // tabla: , campo: , comentario: F02TID
                    break;
                case 'F02PID': // tabla: , campo: , comentario: F02PID
                    break;
                case 'F02IDF': // tabla: , campo: , comentario: F02IDF
                    break;
                case 'F02TIF': // tabla: , campo: , comentario: F02TIF
                    break;
                case 'F02PIF': // tabla: , campo: , comentario: F02PIF
                    break;
                case 'F02ID3': // tabla: , campo: , comentario: F02ID3
                    break;
                case 'F02TI3': // tabla: , campo: , comentario: F02TI3
                    break;
                case 'F02PI3': // tabla: , campo: , comentario: F02PI3
                    break;
                case 'F02ID4': // tabla: , campo: , comentario: F02ID4
                    break;
                case 'F02TI4': // tabla: , campo: , comentario: F02TI4
                    break;
                case 'F02PI4': // tabla: , campo: , comentario: F02PI4
                    break;
                case 'F02BDM': // tabla: , campo: , comentario: F02BDM
                    break;
                case 'F02BDD': // tabla: , campo: , comentario: F02BDD
                    break;
                case 'F02BDY': // tabla: , campo: , comentario: F02BDY
                    break;
                case 'F02IDM': // tabla: , campo: , comentario: F02IDM
                    break;
                case 'F02IDD': // tabla: , campo: , comentario: F02IDD
                    break;
                case 'F02IDY': // tabla: , campo: , comentario: F02IDY
                    break;
                case 'F02IEM': // tabla: , campo: , comentario: F02IEM
                    break;
                case 'F02IED': // tabla: , campo: , comentario: F02IED
                    break;
                case 'F02IEY': // tabla: , campo: , comentario: F02IEY
                    break;
                case 'F02REM': // tabla: , campo: , comentario: F02REM
                    break;
                case 'F02RED': // tabla: , campo: , comentario: F02RED
                    break;
                case 'F02REY': // tabla: , campo: , comentario: F02REY
                    break;
                case 'F02REN': // tabla: , campo: , comentario: F02REN
                    break;
                case 'F02LIF': // tabla: , campo: , comentario: F02LIF
                    break;
                case 'F02RLV': // tabla: , campo: , comentario: F02RLV
                    break;
                case 'F02GRP': // tabla: , campo: , comentario: F02GRP
                    break;
                case 'F02SEX': // tabla: , campo: , comentario: F02SEX
                    break;
                case 'F02TYP': // tabla: , campo: , comentario: F02TYP
                    break;
                case 'F02MST': // tabla: , campo: , comentario: F02MST
                    break;
                case 'F02ILV': // tabla: , campo: , comentario: F02ILV
                    break;
                case 'F02STS': // tabla: , campo: , comentario: F02STS
                    break;
                case 'F02MLC': // tabla: , campo: , comentario: F02MLC
                    break;
                case 'F02CCL': // tabla: , campo: , comentario: F02CCL
                    break;
                case 'F02RSL': // tabla: , campo: , comentario: F02RSL
                    break;
                case 'F02ATM': // tabla: , campo: , comentario: F02ATM
                    break;
                case 'F02TAX': // tabla: , campo: , comentario: F02TAX
                    break;
                case 'F02HPN': // tabla: , campo: , comentario: F02HPN
                    break;
                case 'F02FL2': // tabla: , campo: , comentario: F02FL2
                    break;
                case 'F02PHN': // tabla: , campo: , comentario: F02PHN
                    break;
                case 'F02PH1': // tabla: , campo: , comentario: F02PH1
                    break;
                case 'F02FAX': // tabla: , campo: , comentario: F02FAX
                    break;
                case 'F02EDL': // tabla: , campo: , comentario: F02EDL
                    break;
                case 'F02INL': // tabla: , campo: , comentario: F02INL
                    break;
                case 'F02BUC': // tabla: , campo: , comentario: F02BUC
                    break;
                case 'F02COB': // tabla: , campo: , comentario: F02COB
                    break;
                case 'F02RBY': // tabla: , campo: , comentario: F02RBY
                    break;
                case 'F02RBN': // tabla: , campo: , comentario: F02RBN
                    break;
                case 'F02FL3': // tabla: , campo: , comentario: F02FL3
                    break;
                case 'F02STF': // tabla: , campo: , comentario: F02STF
                    break;
                case 'F02SOI': // tabla: , campo: , comentario: F02SOI
                    break;
                case 'F02NSO': // tabla: , campo: , comentario: F02NSO
                    break;
                case 'F02FL1': // tabla: , campo: , comentario: F02FL1
                    break;
                case 'F02IAD': // tabla: , campo: , comentario: F02IAD
                    break;
                case 'F02NST': // tabla: , campo: , comentario: F02NST
                    break;
                case 'F02NSH': // tabla: , campo: , comentario: F02NSH
                    break;
                case 'F02CAI': // tabla: , campo: , comentario: F02CAI
                    break;
                case 'F02CAS': // tabla: , campo: , comentario: F02CAS
                    break;
                case 'F02CAP': // tabla: , campo: , comentario: F02CAP
                    break;
                case 'F02AMT': // tabla: , campo: , comentario: F02AMT
                    break;
                case 'F02OFC': // tabla: , campo: , comentario: F02OFC
                    break;
                case 'F02OF2': // tabla: , campo: , comentario: F02OF2
                    break;
                case 'F02INC': // tabla: , campo: , comentario: F02INC
                    break;
                case 'F02GEC': // tabla: , campo: , comentario: F02GEC
                    break;
                case 'F02UC1': // tabla: , campo: , comentario: F02UC1
                    break;
                case 'F02UC2': // tabla: , campo: , comentario: F02UC2
                    break;
                case 'F02UC3': // tabla: , campo: , comentario: F02UC3
                    break;
                case 'F02UC4': // tabla: , campo: , comentario: F02UC4
                    break;
                case 'F02UC5': // tabla: , campo: , comentario: F02UC5
                    break;
                case 'F02UC6': // tabla: , campo: , comentario: F02UC6
                    break;
                case 'F02UC7': // tabla: , campo: , comentario: F02UC7
                    break;
                case 'F02UC8': // tabla: , campo: , comentario: F02UC8
                    break;
                case 'F02UC9': // tabla: , campo: , comentario: F02UC9
                    break;
                case 'F02PRV': // tabla: , campo: , comentario: F02PRV
                    break;
                case 'F02FL4': // tabla: , campo: , comentario: F02FL4
                    break;
                case 'F02FL5': // tabla: , campo: , comentario: F02FL5
                    break;
                case 'F02SFR': // tabla: , campo: , comentario: F02SFR
                    break;
                case 'F02SDY': // tabla: , campo: , comentario: F02SDY

  	    
	    */
	    //Excel =CONCATENATE('case );TRIM(B3);'': // tabla: ';TRIM(D3);', campo: ';TRIM(E3);', comentario: ';TRIM(G3); '---break;')
        }
        // Default value for access tree code, witch is mandatory but doesn't come in the input file
        // getTool('ACTR');
        $ar_acco['ACCDEP'] = EIBS_CLIENTS_ACCDEP_CODE_DEFVAL; // ' ';  CR it was blank before....
        // Value for ISINDIVIDUAL field. 0=company
        $ar_acco['ISINDIVIDUAL'] = '0';
        // Error checking
        $ar_error = array_merge($ar_error, $this->create_client_record_check_before_saving($ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo));
        if (!empty($this->debug)) {
            ClxCommon::$db->debug = true;
        }
        // If no errors found, inserts the record into DB
        $b_error = (count($ar_error) != CLX_NUM_ZERO);
        if (empty($b_error)) {
            $ar_acco_id = $this->create_client_rows_insert($ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo);
        } else {
            $ar_acco_id = array(CLX_PROP_ID => '', 'idcode' => '');
        }
        $s_result = $this->create_client_procesa_error_ar($i_proc_counter, $ar_error);
        return array('error' => $b_error,
            'errormsg' => $s_result,
            'ar_error' => $ar_error,
            CLX_PROP_ID => $ar_acco_id[CLX_PROP_ID],
            'idcode' => $ar_acco_id['idcode'],
        );
    }

// Method End
    /**
     * ACCO Individual type record creation (persona natural)
     *
     * @param integer $i_proc_counter
     * @param array $ar_record
     * @param text $rec_id_info
     * @return array
     */
    function create_client_individual_frm_ibs($i_proc_counter, $ar_record, &$rec_id_info) {
        $rec_id_info = array();
        if (!empty($this->debug))
            echoLog(debug_dump('create_client_individual_frm_ibs(ar_record)', $ar_record));
        // arrays to store the record columns values to be saved in the rows of each table that
        // conforms the contact info (acco, aclo, acph, etc.)
        $ar_acco = array();
        $ar_cont = array();
        $ar_dyco = array();
        $ar_acph = array();
        $ar_acea = array();
        $ar_aclo = array();
        $ar_error = array();
        $ar_adic = array();
        $ar_adic['IDNAME_TMP'] = '';
        $total = count($ar_record);
        for ($i_i = CLX_INIT_ZERO; $i_i < $total; $i_i++) {
            $val = trim($ar_record[$i_i]['VAL']);
            if (!empty($this->debug))
                echoLog('<br>FILEFIELD=' . $ar_record[$i_i]['FILEFIELD'] . ', valor=' . $val);
            // Reads each filed n the line, assigns the corresponding table column in the correct array
            switch ($ar_record[$i_i]['FILEFIELD']) {
                case 'H01OPE': // tabla: , campo: H01OPE, comentario: OPERAT CODE Create: 0001 Query: 0004 Manteinance: First: '0002 Para solicitar la info del cliente' Second: '0005 enviar los datos modificados'
                    $this->s_clientImport_operation_type = $val;
                    if (!empty($this->debug))
                        echoLog('<br>OPERACION - ' . $ar_record[$i_i]['FILEFIELD'] . ' / s_clientImport_operation_type=' . $this->s_clientImport_operation_type);
                    break;
      
                case 'E01LGT': // tabla: CRM_ACCOUNTS, campo: LEGAL_TYPE, comentario: LEGAL TYPE, 1 Juridica, 2 Natural, 3 Other
                    $ar_acco['LEGAL_TYPE'] = $val;
                    $rec_id_info['T'] = $val;
                    break;
                case 'E01CUN': // tabla: CRM_ACCOUNTS, campo: IBS_CUSCUN, comentario: CUSTOMER NUMBER Blank on create: Modify and Query: # Customer
                    //$ar_acco['IBS_CUSCUN'] = 0+$val;	//  eliminacion de zeros del cuscun
                    $ar_acco['IBS_CUSCUN'] = $this->ibs_cuscun_code_fix($val);
                    $rec_id_info['CUSCUN'] = $val;
                    break;
                case 'E01FNA': // tabla: CRM_CONTACTS, campo: FIRSTNAME, comentario: CUST FIRST NAME
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['FIRSTNAME'] = $val;
                    } else {
                        $ar_acco['FIRSTNAME'] = $val;
                    }
                    $ar_adic['IDNAME_TMP'] .= $val . ' ';
                    if (!array_key_exists('N', $rec_id_info))
                        $rec_id_info['N'] = '';
                    $rec_id_info['N'] .= $val . ' ';
                    break;
                case 'E01FN2': // tabla: CRM_CONTACTS, campo: SECOND_NAME, comentario: CUST SECOND NAME
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['SECOND_NAME'] = $val;
                    } else {
                        $ar_acco['SECOND_NAME'] = $val;
                    }
                    $ar_adic['IDNAME_TMP'] .= $val . ' ';
                    $rec_id_info['N'] .= $val . ' ';
                    break;
                case 'E01LN1': // tabla: CRM_CONTACTS, campo: LASTNAME, comentario: CUST LAST NAME 1
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['LASTNAME'] = $val;
                    } else {
                        $ar_acco['LASTNAME'] = $val;
                    }
                    $ar_adic['IDNAME_TMP'] .= $val . ' ';
                    $rec_id_info['N'] .= $val . ' ';
                    break;
                case 'E01LN2': // tabla: CRM_CONTACTS, campo: SECOND_LASTNAME, comentario: CUST LAST NAME 2
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['SECOND_LASTNAME'] = $val;
                    }
                    $ar_adic['IDNAME_TMP'] .= $val . ' ';
                    $rec_id_info['N'] .= $val . ' ';
                    break;
                case 'E01ACA': // tabla: CRM_CONTACTS, campo: MARRIED_NAME, comentario: CUST MARRIED NAME`
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['MARRIED_NAME'] = $val;
                    } else {
                        $ar_acco['MARRIED_NAME'] = $val;
                    }
                    break;
                case 'E01NA1': // tabla: , campo: , comentario: NAME
                    $ar_acco['IDNAME'] = $val;
                    break;
                case 'E01NA2': // tabla: CRM_LOCATIONS, campo: ADDRESS1, comentario: NAME  Venezuela BAV: Linea 1 de la dirección
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS1'] = $val;
                    } else {
                        $ar_acco['ADDRESS1'] = $val;
                    }
                    break;
                case 'E01NA3': // tabla: , campo: E02NA3, comentario: NAME
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS1'] .= ' ' . $val;
                    } else {
                        $ar_acco['ADDRESS1'] .= ' ' . $val;
                    }
                    break;
                case 'E01NA4': // tabla: CRM_LOCATIONS, campo: ADDRESS2, comentario: NAME  Venezuela BAV linea 2 de la dirección
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ADDRESS2'] = $val;
                    } else {
                        $ar_acco['ADDRESS2'] = $val;
                    }
                    break;
                case 'E01CTY': // tabla: CRM_LOCATIONS, campo: CITY, comentario: CITY
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['CITY'] = substr($val, CLX_NUM_ZERO, CLX_NUM_FOUR);
                    } else {
                        $ar_acco['CITY'] = substr($val, CLX_NUM_ZERO, CLX_NUM_FOUR);
                    }
                    break;
                case 'E01STE': // tabla: CRM_LOCATIONS, campo: STATE, comentario: STATE
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['STATE'] = $val;
                    } else {
                        $ar_acco['STATE'] = $val;
                    }
                    break;
                case 'E01CTR': // tabla: CRM_LOCATIONS, campo: COUNTRY, comentario: COUNTRY
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['COUNTRY'] = $val;
                        if (isset($ar_cont['COUNTRY']) && $ar_cont['COUNTRY'] === '')
                            $ar_cont['COUNTRY'] = $ar_aclo['COUNTRY'];
                    } else {
                        $ar_acco['COUNTRY'] = $val;
                    }
                    break;
                case 'E01SHN': // tabla: CRM_ACCOUNTS, campo: IDNAME, comentario: SHORT NAME
                    if (isset($ar_acco['IDNAME']) && $ar_acco['IDNAME'] === '')
                        $ar_acco['IDNAME'] = $val;
                    $rec_id_info['N'] .= $val . ' ';
                    break;
                case 'E01CCS': // tabla: CRM_CONTACTS, campo: CITIZENSHIP, comentario: CITIZENSHIP
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['CITIZENSHIP'] = $val;
                    } else {
                        $ar_acco['CITIZENSHIP'] = $val;
                    }
                    break;
                case 'E01ZPC': // tabla: CRM_LOCATIONS, campo: ZIPCODE, comentario: ZIP CODE
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['ZIPCODE'] = $val;
                    } else {
                        $ar_acco['ZIPCODE'] = $val;
                    }
                    break;
                case 'E01IDN': // tabla: CRM_ACCOUNTS, campo: IDENTIFICATION, comentario: IDENTIFICATION
                    $val = ($ar_record[$i_i]['VAL']);
                    $ar_acco['IDENTIFICATION'] = $val;
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['IDENTIFICATION'] = $ar_acco['IDENTIFICATION'];
                    }
                    break;
                case 'E01TID': // tabla: CRM_ACCOUNTS, campo: ID_TYPE, comentario: ID. TYPE
                    $ar_acco['ID_TYPE'] = $val;
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['ID_TYPE'] = $ar_acco['ID_TYPE'];
                    }
                    break;
                case 'E01PID': // tabla: CRM_ACCOUNTS, campo: ID_COUNTRY, comentario: ID. COUNTRY ORIG
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acco['INCORPORATION_COUNTRY'] = $val;
                        if (isset($ar_cont['COUNTRY']) && $ar_cont['COUNTRY'] === '')
                            $ar_cont['COUNTRY'] = $ar_acco['INCORPORATION_COUNTRY'];
                    } else {
                        $ar_acco['BIRTH_COUNTRY'] = $val;
                    }
                    break;
                case 'E01BDM': // tabla: CRM_CONTACTS, campo: BIRTHDATE_MM, comentario: BIRTH DATE MM
                    if ($this->sw_old_acco_struct === true) {
                        if (isset($ar_cont['BIRTHDATE']) && !is_array($ar_cont['BIRTHDATE']))
                            $ar_cont['BIRTHDATE'] = array();
                        $ar_cont['BIRTHDATE']['MM'] = $val;
                    } else {
                        if (isset($ar_acco['BIRTHDATE']) && !is_array($ar_acco['BIRTHDATE']))
                            $ar_acco['BIRTHDATE'] = array();
                        $ar_acco['BIRTHDATE']['MM'] = $val;
                    }
                    break;
                case 'E01BDD': // tabla: CRM_CONTACTS, campo: BIRTHDATE_DD, comentario: BIRTH DATE DD
                    if ($this->sw_old_acco_struct === true) {
                        if (isset($ar_cont['BIRTHDATE']) && !is_array($ar_cont['BIRTHDATE']))
                            $ar_cont['BIRTHDATE'] = array();
                        $ar_cont['BIRTHDATE']['DD'] = $val;
                    } else {
                        if (isset($ar_acco['BIRTHDATE']) && !is_array($ar_acco['BIRTHDATE']))
                            $ar_acco['BIRTHDATE'] = array();
                        $ar_acco['BIRTHDATE']['DD'] = $val;
                    }
                    break;
                case 'E01BDY': // tabla: CRM_CONTACTS, campo: BIRTHDATE_YY, comentario: BIRTH DATE YY
                    if ($this->sw_old_acco_struct === true) {
                        if (isset($ar_cont['BIRTHDATE']) && !is_array($ar_cont['BIRTHDATE']))
                            $ar_cont['BIRTHDATE'] = array();
                        $ar_cont['BIRTHDATE']['YY'] = $val;
                    } else {
                        if (isset($ar_acco['BIRTHDATE']) && !is_array($ar_acco['BIRTHDATE']))
                            $ar_acco['BIRTHDATE'] = array();
                        $ar_acco['BIRTHDATE']['YY'] = $val;
                    }
                    break;
                case 'E01SEX': // tabla: CRM_CONTACTS, campo: SEX, comentario: Natural: 'M Male' 'F Female' Juridicas: 'I International' 'T Multinational' 'S Small Business' 'O Other'
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['SEX'] = $val;
                    } else {
                        $ar_acco['SEX'] = $val;
                    }
                    break;
                case 'E01MST': // tabla: CRM_CONTACTS, campo: MARITAL_STATUS, comentario: MARITAL STATUS '1' >Soltero(a); '2' >Casado(a); '3' >Divorciado(a); '4' >Viudo(a); '5' >Cohabitante
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['MARITAL_STATUS'] = $val;
                    } else {
                        $ar_acco['MARITAL_STATUS'] = $val;
                    }
                    break;
                case 'E01STS': // tabla: CRM_ACCOUNTS, campo: ACCSTAT, comentario: CUSTOMER STATUS '1.Inactivo 2.Activo 3.Lista Negra'
                    $val = ($val === '2') ? 'I' : 'A'; //  Se cambio ya que en IBS: 1=activo, 2=inactivo
                    $ar_acco['ACCSTAT'] = $val;
                    break;
                case 'E01HPN': // tabla: CRM_PHONES, campo: PHONE_W, comentario: HOME PHONE NUMBER  Venezuela BAV: teléfono oficina 1
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Of. 1',
                            'PHTYPE' => 'W',
                            'ISPRIMARY' => '1'
                        );
                    } else {
                        $ar_acco['PHONE'] = $val;
                    }
                    break;
                case 'E01PHN': // tabla: , campo: , comentario: BUSS PHONE NUMBER  Venezuela BAV: teléfono oficina 2
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Of. 2',
                            'PHTYPE' => 'W',
                            'ISPRIMARY' => '0'
                        );
                    } else {
                        $ar_acco['OTHER_PHONE'] = $val;
                    }
                    break;
                case 'E01PH1': // tabla: CRM_PHONES, campo: PHONE_M, comentario: PHONE NUMBER 1
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Mobil.',
                            'PHTYPE' => 'M',
                            'ISPRIMARY' => '0'
                        );
                    } else {
                        $ar_acco['MOBILE_PHONE'] = $val;
                    }
                    break;
                case 'E01FAX': // tabla: CRM_PHONES, campo: PHONE_F, comentario: FAX PHONE NUMBER
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acph[] = array('PHONE' => $val,
                            'EFFTDATE' => adodb_date('Y-m-d'),
                            'IDNAME' => 'Fax',
                            'PHTYPE' => 'FX',
                            'ISPRIMARY' => '0'
                        );
                    } else {
                        $ar_acco['FAX'] = $val;
                    }
                    break;
                case 'E01FL3': // tabla: , campo: REFERRED_BY, comentario: REFERRED-BY REL.FAMIL CLX_CHAR_SINGLE_QUOTE3''Padre ''4''Madre ''5''Hermano(a) ''6''Esposo(a) ''7''Abuelo(a)'
                    $ar_acco['REFERREDBYCODE'] = $val;
                    break;
                case 'E01SOI': // tabla: CRM_CONTACTS, campo: INCOME_SOURCE, comentario: SOURCE OF INCOME CNOFC:30
                    if ($this->sw_old_acco_struct === true) {
                        $ar_cont['INCOME_SOURCE'] = $val;
                    } else {
                        $ar_acco['INCOME_SOURCE'] = $val;
                    }
                    break;
                case 'E01IAD': // tabla: CRM_EADDRESSES, campo: EMAIL, comentario: INTERNET E-MAIL
                    if ($this->sw_old_acco_struct === true) {
                        $ar_acea['ADDRESS'] = $val;
                        $ar_acea['EFFTDATE'] = adodb_date('Y-m-d');
                        $ar_acea['IDNAME'] = 'E-mail principal';
                        $ar_acea['TYPE'] = 'E';
                        $ar_acea['ISPRIMARY'] = '1';
                        $ar_acea['SEQNUM'] = CLX_NUM_ONE;
                    } else {
                        $ar_acco['EMAIL'] = $val;
                    }
                    break;
                case 'E01INC': // tabla: CRM_ACCOUNTS, campo: INDUSTRYCLASS, comentario: COD INDUSTRIA
                    $ar_acco['INDUSTRYCLASS'] = $val;
                    break;
                case 'E01GEC': // tabla: CRM_CONTACTS, campo: RESIDENCE_COUNTRY, comentario: PAIS RESIDENCIA
                    if ($this->sw_old_acco_struct === true) {
                        $ar_aclo['COUNTRY'] = $val;
                        if (isset($ar_cont['COUNTRY']) && $ar_cont['COUNTRY'] === '')
                            $ar_cont['COUNTRY'] = $ar_aclo['COUNTRY'];
                    } else {
                        $ar_acco['COUNTRY'] = $val;
                    }
                    break;
                case 'E01UC4': // tabla: CRM_ACCOUNT, campo: VE_CLI_SUDEBAN, comentario: COOOD. USUARIO 4 207-10-27 Venezuela BAV cód. SUDEBAN del cliente
                    $ar_acco['VE_CLI_SUDEBAN'] = $val;
                    break;
                default:
                    break;
            }
        }
   /*
    default:
                case '': // vacio !!!
                    break;
                case 'HEADER': // tabla: , campo: , comentario: General Header must be first 10 positions of every message.
                    break;
                case 'H01USR': // tabla: CLX_USERS, campo: USERNAME, comentario: USER ID
                    break;
                case 'H01PGM': // tabla: , campo: , comentario: PROGRAM antes: CLX008001
                    break;
                case 'H01TIM': // tabla: , campo: , comentario: TIME-DATE
                    break;
                case 'H01SCR': // tabla: , campo: , comentario: SCREEN CODE
                    break;

    *     *           case 'H01MAS': // tabla: , campo: , comentario: MORE RECORD
                    break;
                case 'H01WK1': // tabla: , campo: , comentario: LENGUAJE USER
                    break;
                case 'H01WK2': // tabla: , campo: , comentario: FLAG 2
                    break;
                case 'H01WK3': // tabla: , campo: , comentario: FLAG 3
                    break;
                case 'E01POB': // tabla: CRM_LOCATIONS, campo: POBOX, comentario: P.O. BOX
                    break;
      
    *           case 'E01LIF': // tabla: , campo: , comentario: LANGUAGE FLAG Values E/S
                    break;
                case 'E01RLV': // tabla: , campo: , comentario: ROLL OVER VOLUME
                    break;
                case 'E01GRP': // tabla: , campo: , comentario: GROUP CODE Para E01TYP=G
                    break;

    *                 case 'E01TYP': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_TYPE_IBS, comentario: CUSTOMER TYPE R.Regular,M.Master,G.Group
                    break;

    *                 case 'E01IDF': // tabla: , campo: , comentario: IDENTIFICATION 2
                    break;
                case 'E01TIF': // tabla: , campo: , comentario: ID. TYPE 2
                    break;
                case 'E01PIF': // tabla: , campo: , comentario: ID. COUNTRY ORIG 2
                    break;
                case 'E01ID3': // tabla: , campo: , comentario: IDENTIFICATION 3
                    break;
                case 'E01TI3': // tabla: , campo: , comentario: ID. TYPE 3
                    break;
                case 'E01PI3': // tabla: , campo: , comentario: ID. COUNTRY ORIG 3
                    break;
                case 'E01ID4': // tabla: , campo: , comentario: IDENTIFICATION 4
                    break;
                case 'E01TI4': // tabla: , campo: , comentario: ID. TYPE 4
                    break;
                case 'E01PI4': // tabla: , campo: , comentario: ID. COUNTRY ORIG 4
                    break;

    * 
    *                 case 'E01IDM': // tabla: , campo: , comentario: INIT DATE MM
                    break;
                case 'E01IDD': // tabla: , campo: , comentario: INIT DATE DD
                    break;
                case 'E01IDY': // tabla: , campo: , comentario: INIT DATE YY
                    break;
                case 'E01ILV': // tabla: , campo: , comentario: INQUIRY LEVEL
                    break;
                case 'E01FL2': // tabla: , campo: , comentario: FORMA CALIFICACION CLIENTE '?? No Aplica 1 Utiliza la Calificación mas alta entre la Objetiva y la Subjetiva para el Cálculo de Previsiones 2 No realiza ninguna Calificación del cliente 3 Utiliza la Calificación Subjetiva para el Cálculo de la Previsión' Colocado en 2 para no hacer validacion contra prevision.
                    break;
      
    *           case 'E01MLC': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'E01CCL': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_CLASSIFICATION, comentario: Customer Classification 1- Ente Gubernamental, 2 - Entedecentralizado, ''
                    break;
                case 'E01RSL': // tabla: CRM_ACCOUNTS, campo: RISK_LEVEL, comentario: RISK LEVEL '' No aplica CNOFC:31
                    break;
                case 'E01ATM': // tabla: , campo: , comentario: ATM ISSUED Y/N '*blanks '1' '2' '3' 'Y' 'N' '
                    break;
                case 'E01TAX': // tabla: CRM_ACCOUNTS, campo: CUSTOMER_TAX, comentario: CHARGE TAX 1/2/3/N '1 RETENCION SOBRE INTERESS ISR 2 COBRE DEL IVA 3 IVA MAS ISR 4 IVA SOLO EN COMISIONES 5 IVA SOLO EN INTERESES 6 DEBITO BANCARIO IDB 7 IDB MAS ISR 8 IDB MAS IVA 9 CUALQUIER TIPO DE IMPUESTO N NO CALCULA IMPUESTOS '
                    break;
      
    *           case 'E01EDL': // tabla: , campo: , comentario: EDUCATIONAL LEVEL
                    break;
                case 'E01INL': // tabla: , campo: , comentario: INCOME LEVEL
                    break;
                case 'E01BUC': // tabla: , campo: , comentario: CODIGO NEGOCIO
                    break;
                case 'E01COB': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'E01RBY': // tabla: , campo: , comentario: REFERED BY
                    break;
                case 'E01RBN': // tabla: , campo: , comentario: REFERRED-BY NAME
                    break;
    
    *             case 'E01STF': // tabla: , campo: , comentario: REFERRED-BY STAFF MEMBR '1 Empleado 2 No Empleado'
                    break;
    
                case 'E01NSO': // tabla: , campo: , comentario: NUMBER OF DEPENDENTS
                    break;
                case 'E01FL1': // tabla: , campo: , comentario: REFERRED-BY STATUS '1' SI residente '2' NO residente ' ' No aplica'
                    break;
    
                case 'E01OFC': // tabla: , campo: OFICIAL, comentario: OFICIAL
                    break;
                case 'E01OF2': // tabla: , campo: OFICIAL2, comentario: OFICIAL 2
                    break;

                case 'E01UC1': // tabla: , campo: , comentario: COOOD. USUARIO 1
                    break;
                case 'E01UC2': // tabla: , campo: , comentario: COOOD. USUARIO 2
                    break;
                case 'E01UC3': // tabla: , campo: , comentario: COOOD. USUARIO 3
                    break;

                case 'E01UC5': // tabla: , campo: , comentario: COOOD. USUARIO 5
                    break;
                case 'E01UC6': // tabla: , campo: , comentario: COOOD. USUARIO 6
                    break;
                case 'E01UC7': // tabla: , campo: , comentario: COOOD. USUARIO 7
                    break;
                case 'E01UC8': // tabla: , campo: , comentario: COOOD. USUARIO 8
                    break;
                case 'E01UC9': // tabla: , campo: , comentario: PROFESION
                    break;
                case 'E01PRV': // tabla: CRM_ACCOUNTS, campo: PREVISIONTABLE_IBS, comentario: TABLE'PREVISION
                    break;
                case 'E01CMP': // tabla: , campo: , comentario: E01CMP
                    break;
                case 'E01FL4': // tabla: , campo: , comentario: FLAG 4 MULT-USE
                    break;
                case 'E01SFR': // tabla: , campo: , comentario: E01SFR
                    break;
                case 'E01SDY': // tabla: , campo: , comentario: E01SDY
                    break;
                case 'E01FL5': // tabla: , campo: , comentario: FLAG 5 MULT-USE
                    break;
                case 'E01FL6': // tabla: , campo: , comentario: FLAG 6 MULT-USE
                    break;
                case 'E01FL7': // tabla: , campo: , comentario: FLAG 7 MULT-USE
                    break;
                case 'E01FL8': // tabla: , campo: , comentario: FLAG 8 MULT-USE  Venezuela - Ronald - F
                    break;
                case 'E01WEB': // tabla: , campo: , comentario: INTERNET WEB SITE
                    break;
                case 'E01BRA': // tabla: , campo: , comentario: BRANCH NUMBER
                    break;
                case 'E01SCH': // tabla: , campo: , comentario: CANAL DE VENTAS
                    break;
                case 'E01SST': // tabla: , campo: , comentario: TIPO FUENTE INFORMACION
                    break;
                case 'E01FC1': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'E01FC2': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'E01TER': // tabla: , campo: , comentario: TERM
                    break;
                case 'E01AM1': // tabla: , campo: , comentario: AMOUNT 152
                    break;
                case 'E01DD1': // tabla: , campo: , comentario: E01DD1
                    break;
                case 'E01DD2': // tabla: , campo: , comentario: E01DD2
                    break;
                case 'E01DD3': // tabla: , campo: , comentario: E01DD3
                    break;
                case 'E01D11': // tabla: , campo: , comentario: FEC/SUSCRIP
                    break;
                case 'E01D12': // tabla: , campo: , comentario: FEC/SUSCRIP
                    break;
                case 'E01D13': // tabla: , campo: , comentario: FEC/SUSCRIP
                    break;
                case 'E01F01': // tabla: , campo: , comentario: CODE CNOFC
                    break;
                case 'D01TID': // tabla: , campo: , comentario: D01TID
                    break;
                case 'D01PID': // tabla: , campo: , comentario: D01PID
                    break;
                case 'D01TIF': // tabla: , campo: , comentario: D01TIF
                    break;
                case 'D01PIF': // tabla: , campo: , comentario: D01PIF
                    break;
                case 'D01TI3': // tabla: , campo: , comentario: D01TI3
                    break;
                case 'D01PI3': // tabla: , campo: , comentario: D01PI3
                    break;
                case 'D01TI4': // tabla: , campo: , comentario: D01TI4
                    break;
                case 'D01PI4': // tabla: , campo: , comentario: D01PI4
                    break;
                case 'D01STE': // tabla: , campo: , comentario: D01STE
                    break;
                case 'D01CCS': // tabla: , campo: , comentario: D01CCS
                    break;
                case 'D01RBY': // tabla: , campo: , comentario: D01RBY
                    break;
                case 'D01EDL': // tabla: , campo: , comentario: D01EDL
                    break;
                case 'D01INC': // tabla: , campo: , comentario: D01INC
                    break;
                case 'D01BUC': // tabla: , campo: , comentario: D01BUC
                    break;
                case 'D01COB': // tabla: , campo: , comentario: D01COB
                    break;
                case 'D01RSL': // tabla: , campo: , comentario: D01RSL
                    break;
                case 'D01GEC': // tabla: , campo: , comentario: D01GEC
                    break;
                case 'D01OFC': // tabla: , campo: , comentario: D01OFC
                    break;
                case 'D01OF2': // tabla: , campo: , comentario: D01OF2
                    break;
                case 'D01UC1': // tabla: , campo: , comentario: D01UC1
                    break;
                case 'D01UC2': // tabla: , campo: , comentario: D01UC2
                    break;
                case 'D01UC3': // tabla: , campo: , comentario: D01UC3
                    break;
                case 'D01UC4': // tabla: , campo: , comentario: D01UC4
                    break;
                case 'D01UC5': // tabla: , campo: , comentario: D01UC5
                    break;
                case 'D01UC6': // tabla: , campo: , comentario: D01UC6
                    break;
                case 'D01UC7': // tabla: , campo: , comentario: D01UC7
                    break;
                case 'D01UC8': // tabla: , campo: , comentario: D01UC8
                    break;
                case 'D01UC9': // tabla: , campo: , comentario: D01UC9
                    break;
                case 'D01MLC': // tabla: , campo: , comentario: D01MLC
                    break;
                case 'D01SOI': // tabla: , campo: , comentario: D01SOI
                    break;
                case 'D01SCH': // tabla: , campo: , comentario: D01SCH
                    break;
                case 'D01SST': // tabla: , campo: , comentario: D01SST
                    break;
                case 'D01FC1': // tabla: , campo: , comentario: D01FC1
                    break;
                case 'D01FC2': // tabla: , campo: , comentario: D01FC2
                    break;
                case 'D01F01': // tabla: , campo: , comentario: D01F01
                    break;
                case 'F01LGT': // tabla: , campo: , comentario: F01LGT
                    break;
                case 'F01CUN': // tabla: , campo: , comentario: F01CUN
                    break;
                case 'F01FNA': // tabla: , campo: , comentario: F01FNA
                    break;
                case 'F01FN2': // tabla: , campo: , comentario: F01FN2
                    break;
                case 'F01LN1': // tabla: , campo: , comentario: F01LN1
                    break;
                case 'F01LN2': // tabla: , campo: , comentario: F01LN2
                    break;
                case 'F01ACA': // tabla: , campo: , comentario: F01ACA
                    break;
                case 'F01NA1': // tabla: , campo: , comentario: F01NA1
                    break;
                case 'F01NA2': // tabla: , campo: , comentario: F01NA2
                    break;
                case 'F01NA3': // tabla: , campo: , comentario: F01NA3
                    break;
                case 'F01NA4': // tabla: , campo: , comentario: F01NA4
                    break;
                case 'F01CTY': // tabla: , campo: , comentario: F01CTY
                    break;
                case 'F01STE': // tabla: , campo: , comentario: F01STE
                    break;
                case 'F01CTR': // tabla: , campo: , comentario: F01CTR
                    break;
                case 'F01SHN': // tabla: , campo: , comentario: F01SHN
                    break;
                case 'F01POB': // tabla: , campo: , comentario: F01POB
                    break;
                case 'F01CCS': // tabla: , campo: , comentario: F01CCS
                    break;
                case 'F01ZPC': // tabla: , campo: , comentario: F01ZPC
                    break;
                case 'F01IDN': // tabla: , campo: , comentario: F01IDN
                    break;
                case 'F01TID': // tabla: , campo: , comentario: F01TID
                    break;
                case 'F01PID': // tabla: , campo: , comentario: F01PID
                    break;
                case 'F01IDF': // tabla: , campo: , comentario: F01IDF
                    break;
                case 'F01TIF': // tabla: , campo: , comentario: F01TIF
                    break;
                case 'F01PIF': // tabla: , campo: , comentario: F01PIF
                    break;
                case 'F01ID3': // tabla: , campo: , comentario: F01ID3
                    break;
                case 'F01TI3': // tabla: , campo: , comentario: F01TI3
                    break;
                case 'F01PI3': // tabla: , campo: , comentario: F01PI3
                    break;
                case 'F01ID4': // tabla: , campo: , comentario: F01ID4
                    break;
                case 'F01TI4': // tabla: , campo: , comentario: F01TI4
                    break;
                case 'F01PI4': // tabla: , campo: , comentario: F01PI4
                    break;
                case 'F01BDM': // tabla: , campo: , comentario: F01BDM
                    break;
                case 'F01BDD': // tabla: , campo: , comentario: F01BDD
                    break;
                case 'F01BDY': // tabla: , campo: , comentario: F01BDY
                    break;
                case 'F01LIF': // tabla: , campo: , comentario: F01LIF
                    break;
                case 'F01RLV': // tabla: , campo: , comentario: F01RLV
                    break;
                case 'F01GRP': // tabla: , campo: , comentario: F01GRP
                    break;
                case 'F01SEX': // tabla: , campo: , comentario: F01SEX
                    break;
                case 'F01TYP': // tabla: , campo: , comentario: F01TYP
                    break;
                case 'F01MST': // tabla: , campo: , comentario: F01MST
                    break;
                case 'F01IDM': // tabla: , campo: , comentario: F01IDM
                    break;
                case 'F01IDD': // tabla: , campo: , comentario: F01IDD
                    break;
                case 'F01IDY': // tabla: , campo: , comentario: F01IDY
                    break;
                case 'F01ILV': // tabla: , campo: , comentario: F01ILV
                    break;
                case 'F01STS': // tabla: , campo: , comentario: F01STS
                    break;
                case 'F01MLC': // tabla: , campo: , comentario: F01MLC
                    break;
                case 'F01CCL': // tabla: , campo: , comentario: F01CCL
                    break;
                case 'F01RSL': // tabla: , campo: , comentario: F01RSL
                    break;
                case 'F01ATM': // tabla: , campo: , comentario: F01ATM
                    break;
                case 'F01TAX': // tabla: , campo: , comentario: F01TAX
                    break;
                case 'F01HPN': // tabla: , campo: , comentario: F01HPN
                    break;
                case 'F01FL2': // tabla: , campo: , comentario: F01FL2
                    break;
                case 'F01PHN': // tabla: , campo: , comentario: F01PHN
                    break;
                case 'F01PH1': // tabla: , campo: , comentario: F01PH1
                    break;
                case 'F01FAX': // tabla: , campo: , comentario: F01FAX
                    break;
                case 'F01EDL': // tabla: , campo: , comentario: F01EDL
                    break;
                case 'F01INL': // tabla: , campo: , comentario: F01INL
                    break;
                case 'F01BUC': // tabla: , campo: , comentario: F01BUC
                    break;
                case 'F01COB': // tabla: , campo: , comentario: F01COB
                    break;
                case 'F01RBY': // tabla: , campo: , comentario: F01RBY
                    break;
                case 'F01RBN': // tabla: , campo: , comentario: F01RBN
                    break;
                case 'F01FL3': // tabla: , campo: , comentario: F01FL3
                    break;
                case 'F01STF': // tabla: , campo: , comentario: F01STF
                    break;
                case 'F01SOI': // tabla: , campo: , comentario: F01SOI
                    break;
                case 'F01NSO': // tabla: , campo: , comentario: F01NSO
                    break;
                case 'F01FL1': // tabla: , campo: , comentario: F01FL1
                    break;
                case 'F01IAD': // tabla: , campo: , comentario: F01IAD
                    break;
                case 'F01OFC': // tabla: , campo: , comentario: F01OFC
                    break;
                case 'F01OF2': // tabla: , campo: , comentario: F01OF2
                    break;
                case 'F01INC': // tabla: , campo: , comentario: F01INC
                    break;
                case 'F01GEC': // tabla: , campo: , comentario: F01GEC
                    break;
                case 'F01UC1': // tabla: , campo: , comentario: F01UC1
                    break;
                case 'F01UC2': // tabla: , campo: , comentario: F01UC2
                    break;
                case 'F01UC3': // tabla: , campo: , comentario: F01UC3
                    break;
                case 'F01UC4': // tabla: , campo: , comentario: F01UC4
                    break;
                case 'F01UC5': // tabla: , campo: , comentario: F01UC5
                    break;
                case 'F01UC6': // tabla: , campo: , comentario: F01UC6
                    break;
                case 'F01UC7': // tabla: , campo: , comentario: F01UC7
                    break;
                case 'F01UC8': // tabla: , campo: , comentario: F01UC8
                    break;
                case 'F01UC9': // tabla: , campo: , comentario: F01UC9
                    break;
                case 'F01PRV': // tabla: , campo: , comentario: F01PRV
                    break;
                case 'F01CMP': // tabla: , campo: , comentario: F01CMP
                    break;
                case 'F01FL4': // tabla: , campo: , comentario: F01FL4
                    break;
                case 'F01FL5': // tabla: , campo: , comentario: F01FL5
                    break;
                case 'F01SFR': // tabla: , campo: , comentario: F01SFR
                    break;
                case 'F01SDY': // tabla: , campo: , comentario: F01SDY
                    break;
                case 'F01FL6': // tabla: , campo: , comentario: F01FL6
                    break;
                case 'F01FL7': // tabla: , campo: , comentario: F01FL7
                    break;
                case 'F01FL8': // tabla: , campo: , comentario: F01FL8
                    break;
                case 'F01WEB': // tabla: , campo: , comentario: F01WEB
                    break;
                case 'F01BRA': // tabla: , campo: , comentario: F01BRA
                    break;
                case 'F01SCH': // tabla: , campo: , comentario: F01SCH
                    break;
                case 'F01SST': // tabla: , campo: , comentario: F01SST
                    break;
                case 'F01FC1': // tabla: , campo: , comentario: F01FC1
                    break;
                case 'F01FC2': // tabla: , campo: , comentario: F01FC2
                    break;
                case 'F01TER': // tabla: , campo: , comentario: F01TER
                    break;
                case 'F01AM1': // tabla: , campo: , comentario: F01AM1
                    break;
                case 'F01DD1': // tabla: , campo: , comentario: F01DD1
                    break;
                case 'F01DD2': // tabla: , campo: , comentario: F01DD2
                    break;
                case 'F01DD3': // tabla: , campo: , comentario: F01DD3
                    break;
                case 'F01D11': // tabla: , campo: , comentario: F01D11
                    break;
                case 'F01D12': // tabla: , campo: , comentario: F01D12
                    break;
                case 'F01D13': // tabla: , campo: , comentario: F01D13
                    break;
                case 'F01F01': // tabla: , campo: , comentario: F01F01
                    break;
      
    
    *  
    */
	
	// Default value for access tree code, witch is mandatory but doesn't come in the input file
        // getTool('ACTR');
        $ar_acco['ACCDEP'] = EIBS_CLIENTS_ACCDEP_CODE_DEFVAL; // ' ';  CR it was blank before....
        // Value for ISINDIVIDUAL field. 1=individual person
        $ar_acco['ISINDIVIDUAL'] = '1';
        // *** Caso específico: Venezuela ***
        // Adjusto foreigner condition. LEGAL_TYPE='E' if ID_TYPE='CIE'
        if ($ar_acco['LEGAL_TYPE'] === 'V' && $ar_acco['ID_TYPE'] === 'CIE') {
            $ar_acco['LEGAL_TYPE'] = 'E';
        }
        // Error checking
        $ar_error = array_merge($ar_error, $this->create_client_record_check_before_saving($ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo, $ar_adic));
        // If no errors found, inserts the record into DB
        $b_error = (count($ar_error) != CLX_NUM_ZERO);
        if (empty($b_error)) {
            $ar_acco_id = $this->create_client_rows_insert($ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo);
        } else {
            $ar_acco_id = array(CLX_PROP_ID => '', 'idcode' => '');
        }
        $s_result = $this->create_client_procesa_error_ar($i_proc_counter, $ar_error);
        return array('error' => $b_error,
            'errormsg' => $s_result,
            'ar_error' => $ar_error,
            CLX_PROP_ID => $ar_acco_id[CLX_PROP_ID],
            'idcode' => $ar_acco_id['idcode'],
        );
    }

// Method End
    /**
     * Returns the operation description between braces
     *
     * @param void
     * @return text
     */
    function create_client_operation_descrip() {
        $s_response = '???';
        switch ($this->s_clientImport_operation_type) {
            case '0001':
                /*
                 * GEN_LBL_CREATION
                 * Creation
                 * Creación
                 */
                $s_response = dict('GEN_LBL_CREATION');
                if (empty($this->b_sw_acco_exist))
                    $s_response . ' (Orig. 0005)';
                break;
            case '0005':
                /*
                 * GEN_LBL_UPDATE
                 * Update
                 * Modificación
                 */
                $s_response = dict('GEN_LBL_UPDATE');
                break;
            default:
                break;
        }
        $s_response = '[' . $s_response . '] ';
        return $s_response;
    }

    /**
     * Creates rows in ACCO, CONT, DYCO, ACPH, ACEA, ACLO
     *
     * @param array $ar_acco
     * @param array $ar_cont
     * @param array $ar_dyco
     * @param array $ar_acph
     * @param array $ar_acea
     * @param array $ar_aclo
     */
    function create_client_rows_insert($ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo) {
        if (!empty($this->debug)) {
            echoLog('create_client_rows_insert');
            echoLog(debug_dump('ACCO', $ar_acco));
            echoLog(debug_dump('CONT', $ar_cont));
            echoLog(debug_dump('DYCO', $ar_dyco));
            echoLog(debug_dump('ACPH', $ar_acph));
            echoLog(debug_dump('ACEA', $ar_acea));
            echoLog(debug_dump('ACLO', $ar_aclo));
	}    
        // Effective date for the rest of rows to be created
        $s_effdate = adodb_date('Y-m-d');
        // Prepares the ACCO record (CRM Accounts)
        $ar_acco['USERNAME'] = '';
        $ar_acco['CREATEDBY'] = '';
        $ar_acco['MODIFIEDBY'] = '';
        $ar_acco['CDATE'] = '';
        $ar_acco['MDATE'] = '';
        if (!empty($this->debug)) {
            ClxCommon::$db->debug = true;
        }
        $ar_acco = setCreationData($ar_acco);
        $ar_acco['IDCODE'] = 'AUTO';
        $ar_acco = setCreationIdCode($ar_acco, 'ACCO');
        // Prepares the CONT record (Contacts of CRM Accounts)
        if (count($ar_cont) > CLX_NUM_ZERO) {
            $ar_cont['CDATE'] = '';
            $ar_cont['MDATE'] = '';
            $ar_cont['MODIFIEDBY'] = '';
            $ar_cont['CREATEDBY'] = '';
            $ar_cont = setCreationData($ar_cont);
        }
        if (!empty($this->debug)) {
            ClxCommon::$db->debug = false;
        }
        // Date fixes
        if (isset($ar_acco['INCORPORATION_DATE']))
            $ar_acco['INCORPORATION_DATE'] = $this->create_client_date_fix($ar_acco['INCORPORATION_DATE']);
        if (isset($ar_cont['BIRTHDATE']))
            $ar_cont['BIRTHDATE'] = $this->create_client_date_fix($ar_cont['BIRTHDATE']);
        if (isset($ar_acco['BIRTHDATE']))
            $ar_acco['BIRTHDATE'] = $this->create_client_date_fix($ar_acco['BIRTHDATE']); //  faltaba esto, asi que en el caso de nueva estructura de acco no estaba trayendo la fecha de nacimiento
// Process additional field values
        if ($this->ar_clientImport_aditional_fields != null && is_array($this->ar_clientImport_aditional_fields)) {
            // For each table...
            foreach ($this->ar_clientImport_aditional_fields as $s_table_sname => $ar_table_values) {
                // For each field in that table...
                foreach ($ar_table_values as $s_fld_name => $v_fld_value) {
                    // Add the field value to the corresponding field array
                    switch (uppercase($s_table_sname)) {
                        case 'ACCO':
                            $ar_acco[$s_fld_name] = $v_fld_value;
                            break;
                        case 'CONT':
                            $ar_cont[$s_fld_name] = $v_fld_value;
                            break;
                        case 'ACPH':
                            // The phones case is special, because it could be more than one phone....
                            $total = count($ar_acph);
                            for ($i_j = CLX_INIT_ZERO; $i_j < $total; $i_j++) {
                                $ar_acph[$i_j][$s_fld_name] = $v_fld_value;
                            }
                            break;
                        case 'ACEA':
                            $ar_acea[$s_fld_name] = $v_fld_value;
                            break;
                        case 'ACLO':
                            $ar_aclo[$s_fld_name] = $v_fld_value;
                            break;
                        default:
                            break;
                    }
                }
            }
        }
        // Another columns fixes
        if (isset($ar_acco['IDNAME'])) {
            $ar_acco['IDNAME'] = fija($ar_acco['IDNAME'], $this->get_fld_attrib('IDNAME'));
        }
        if ($this->a_parameters['FLG_PRODUCTION'] === '1') {
            // OPERAT CODE: Create: 0001...
            if ($this->s_clientImport_operation_type === '0001') {
                // Performs CRM record creation
                if (!empty($this->debug)) {
                    ClxCommon::$db->debug = true;
                }
                // Creates the account (client/particpant) record and retrieve the IDCODE
                if (!empty($this->debug))
                    echoLog(debug_dump('create_client_rows_insert - AR_ACCO', $ar_acco));
                $i_acco_id = ClxCommon::$db->createRecord($this->acconame, $ar_acco, true);
                $s_idcode = $ar_acco['IDCODE'];
                // Creates the various phones
                $total = count($ar_acph);
                for ($i_j = CLX_INIT_ZERO; $i_j < $total; $i_j++) {
                    $ar_acph[$i_j]['ACCOUNTCODE'] = $s_idcode;
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = true;
                    }
                    ClxCommon::$db->createRecord($this->acphname, $ar_acph[$i_j], true);
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = false;
                    }
                }
                // Creates the E-mail
                if (count($ar_acea) > CLX_NUM_ZERO) {
                    $ar_acea['ACCOUNTCODE'] = $s_idcode;
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = true;
                    }
                    ClxCommon::$db->createRecord($this->aceaname, $ar_acea, true);
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = false;
                    }
                }
                // Creates the location
                if (count($ar_aclo) > CLX_NUM_ZERO) {
                    $ar_aclo['ACCOUNTCODE'] = $s_idcode;
                    $ar_aclo['EFFTDATE'] = $s_effdate;
                    $ar_aclo['ISPRIMARY'] = '1';
                    // If city code is 2 ch. long, inserts the state code
                    if (isset($ar_aclo['CITY']) && strlen($ar_aclo['CITY']) == CLX_NUM_TWO && trim($ar_aclo['STATE']) !== '')
                        $ar_aclo['CITY'] = trim($ar_aclo['STATE']) . $ar_aclo['CITY'];
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = true;
                    }
                    ClxCommon::$db->createRecord($this->acloname, $ar_aclo, true);
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = false;
                    }
                }
                // Creates the contact
                if (count($ar_cont) > CLX_NUM_ZERO) {
                    if (!isset($ar_cont['STATUS']))
                        $ar_cont['STATUS'] = 'A';
                    $s_srch_criteria = 'IDENTIFICATION = ' . singleQuote(($ar_cont['IDENTIFICATION'])) . ' AND ' . 'ID_TYPE = ' . singleQuote(trim($ar_cont['ID_TYPE']));
                    $ar_cont_record = ClxCommon::$db->getRecord($this->contname, $s_srch_criteria);
                    if (!empty($this->debug))
                        echoLog(debug_dump('s_srch_criteria: $s_srch_criteria<br>' . CLX_CHAR_NL . 'ar_cont_record', $ar_cont_record));
                    if (empty($ar_cont_record)) {
                        if (!empty($this->debug)) {
                            ClxCommon::$db->debug = true;
                        }
                        $i_cont_id = ClxCommon::$db->createRecord($this->contname, $ar_cont, true);
                        if (!empty($this->debug)) {
                            ClxCommon::$db->debug = false;
                        }
                    } else {
                        $i_cont_id = $ar_cont_record['ID'];
                    }
                    // Creates the relationship between ACCO & CONT through DYCO
                    $ar_dyco = array('PARENT_TABLECODE' => 'ACCO',
                        'PARENT_RECORDID' => $i_acco_id,
                        'CHILD_TABLECODE' => 'CONT',
                        'CHILD_RECORDID' => $i_cont_id
                    );
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = true;
                    }
                    ClxCommon::$db->createRecord($this->dyconame, $ar_dyco, true);
                    if (!empty($this->debug)) {
                        ClxCommon::$db->debug = false;
                    }
                    if (!empty($this->debug))
                        echoLog(debug_dump('2-CONT', $ar_cont) . debug_dump('2-DYCO', $ar_dyco));
                }
                // OPERAT CODE: 0005 enviar los datos modificados...
            } elseif ($this->s_clientImport_operation_type === '0005') {
                // Performs CRM record modification (only for version 55, because updates ACCO record only)
                if (!empty($this->debug)) {
                    ClxCommon::$db->debug = true;
                }
                // Updates the account (client/particpant) record and retrieve the IDCODE
                if (!empty($this->debug))
                    echoLog(debug_dump('create_client_rows_insert (update) - AR_ACCO', $ar_acco));
                $i_acco_id = $this->a_clientImport_id['ID'];
                $s_idcode = $this->a_clientImport_id['IDCODE'];
                if (!empty($this->debug)) {
                    ClxCommon::$db->debug = true;
                }
                ClxCommon::$db->updateRecord($this->acconame, $ar_acco, 'ID = ' . $i_acco_id);
                if (!empty($this->debug)) {
                    ClxCommon::$db->debug = false;
                }
            }
            if (!empty($this->debug)) {
                ClxCommon::$db->debug = false;
            }
            if (!empty($this->debug))
                echoLog('<br>' . CLX_CHAR_NL . 'Saliendo de aquiiiiiii.... UPDATE CRM');
        }
        return array(CLX_PROP_ID => $i_acco_id,
            'idcode' => $s_idcode,
        );
    }

// Method End
    /**
     * Returns an array of eventual errors during validation phase,
     * just before saving the row in th DB
     *
     * @param array $ar_acco
     * @param array $ar_cont
     * @param array $ar_dyco
     * @param array $ar_acph
     * @param array $ar_acea
     * @param array $ar_aclo
     * @param array $ar_adic
     * @return array
     */
    function create_client_record_check_before_saving(&$ar_acco, $ar_cont, $ar_dyco, $ar_acph, $ar_acea, $ar_aclo, $ar_adic = null) {
       ECHO 'ACA'; if ($ar_adic == null)
            $ar_adic = array();
        if (isset($ar_adic['IDNAME_TMP']) && trim($ar_acco['IDNAME']) === '') {
            $ar_acco['IDNAME'] = trim($ar_adic['IDNAME_TMP']);
        }
        // Assigns default Account Type //
        if (!isset($ar_acco['ACCTYPE'])) {
            $ar_acco['ACCTYPE'] = $this->s_custimport_def_cust_type;
        }
        if (!empty($this->debug)) {
            echoLog('create_client_record_check_before_saving');
            echoLog(debug_dump('ACCO', $ar_acco));
            echoLog(debug_dump('CONT', $ar_cont));
            echoLog(debug_dump('DYCO', $ar_dyco));
            echoLog(debug_dump('ACPH', $ar_acph));
            echoLog(debug_dump('ACEA', $ar_acea));
            echoLog(debug_dump('ACLO', $ar_aclo));
            ClxCommon::$db->debug = true;
        }
        if (array_key_exists('FLG_DEBUG', $this->a_parameters) && $this->a_parameters['FLG_DEBUG'] === '1') {
            // View file´s contents
            $this->o_logfile_mngr->write('---------------------------------------------------------------------------------------');
            $this->o_logfile_mngr->write(debug_dump(dict('3A2FA2E1600A3021EABBB021CB14D4EB') . ' (ACCO)', $ar_acco));
            if (!empty($ar_cont))
                $this->o_logfile_mngr->write(debug_dump(dict('9AA698F602B1E5694855CEE73A683488') . ' (CONT)', $ar_cont));
            if (!empty($ar_acph))
                $this->o_logfile_mngr->write(debug_dump(dict('178ea9d484e72f881fec8cfd1f217d1f') . ' (ACPH)', $ar_acph));
            if (!empty($ar_acea))
                $this->o_logfile_mngr->write(debug_dump(dict('082E55A73F10496681C84231DC1C88F7') . ' (ACEA)', $ar_acea));
            if (!empty($ar_aclo))
                $this->o_logfile_mngr->write(debug_dump(dict('Locations') . ' (ACLO)', $ar_aclo));
            if (!empty($ar_dyco))
                $this->o_logfile_mngr->write(debug_dump(dict('GEN_LBL_DYNAMIC_RELATIONS') . ' (DYCO)', $ar_dyco));
        }
        $ar_error = array();
        if ($this->b_sw_validations_on === true) {
            // Check data before saving database row
            $ar_col_lst_cannot_b_blk_errcodes = array('IDNAME' => array('errcode' => '0260', 'field' => 'E0*NA1'),
                'IDENTIFICATION' => array('errcode' => '0220', 'field' => 'E0*IDN'),
                'ID_TYPE' => array('errcode' => '0230', 'field' => 'E0*TID'),
                'ISINDIVIDUAL' => array('errcode' => '0240', 'field' => 'E0*LGT')
            );
            foreach ($ar_acco as $s_key => $v_val) {
                // if the element name exist in the list, it's a mandatory field, so check if it has any value
                if (array_key_exists($s_key, $ar_col_lst_cannot_b_blk_errcodes) && trim($v_val) === '') {
                    /*
                     * GEN_ERR_IBS_IMPCLI_MISSING_VALUE
                     * Mandatory field. Must be filled
                     * Campo obligatorio. Debe tener algún contenido
                     */
                    $ar_error[$ar_col_lst_cannot_b_blk_errcodes[$s_key]['errcode']] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_MISSING_VALUE') . ' ' . '(' . $s_key . ')', $ar_col_lst_cannot_b_blk_errcodes[$s_key]['field']);
                }
            }
            foreach ($ar_col_lst_cannot_b_blk_errcodes as $s_key => $v_val) {
                // if the element name in the list doesn't in acco array, is a error because it's a mandatory field
                if (!array_key_exists($s_key, $ar_acco)) {
                    /*
                     * GEN_ERR_IBS_IMPCLI_MISSING_VALUE
                     * Mandatory field. Must be filled
                     * Campo obligatorio. Debe tener algún contenido
                     */
                    $ar_error[$ar_col_lst_cannot_b_blk_errcodes[$s_key]['errcode']] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_MISSING_VALUE') . ' (' . $s_key . ') ', $ar_col_lst_cannot_b_blk_errcodes[$s_key]['field']);
                }
            }
        }
        // Check if the participant exists by looking his/her/it ID number & ID Type
        if (trim($ar_acco['IDENTIFICATION']) . trim($ar_acco['ID_TYPE']) !== '') {
            // OPERAT CODE: Create: 0001...
            if ($this->s_clientImport_operation_type === '0001') {
                $s_srch_criteria = 'IDENTIFICATION = ' . singleQuote(($ar_acco['IDENTIFICATION'])) . ' AND ' . 'ID_TYPE = ' . singleQuote(trim($ar_acco['ID_TYPE']));
                $ar_cli_record = ClxCommon::$db->getRecord($this->acconame, $s_srch_criteria);
                if (!empty($this->debug))
                    echoLog(debug_dump(',<pre>s_srch_criteria: ' . $s_srch_criteria . '</pre><br>' . CLX_CHAR_NL . 'ar_cli_record', $ar_cli_record));
                $this->b_sw_acco_exist = (!empty($ar_cli_record));
                if ($this->b_sw_acco_exist === true) {
                    // Record already exists and it's creation
                    /*
                     * GEN_ERR_IBS_IMPCLI_ACCO_ALREADYEXIST
                     * Error, the CRM Account already exists
                     * Error, la Cuenta CRM ya existe
                     */
                    $ar_error['0210'] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_ACCO_ALREADYEXIST') . ' (' . $ar_acco['ID_TYPE'] . '-' . $ar_acco['IDENTIFICATION'] . ')', '');
                }
                // OPERAT CODE: 0005 enviar los datos modificados...
            } elseif ($this->s_clientImport_operation_type === '0005') {
                $s_srch_criteria = 'IBS_CUSCUN = ' . singleQuote($ar_acco['IBS_CUSCUN']);
                $ar_cli_record = ClxCommon::$db->getRecord($this->acconame, $s_srch_criteria);
                if (!empty($this->debug))
                    echoLog(debug_dump('s_srch_criteria: ' . $s_srch_criteria . '<br>' . CLX_CHAR_NL . 'ar_cli_record', $ar_cli_record));
                $this->b_sw_acco_exist = (!empty($ar_cli_record));
                if (empty($this->b_sw_acco_exist)) {
                    if (!empty($this->b_sw_strict_action_valid)) {
                        // Record doesn't exists and it's modificaion
                        /*
                         * GEN_ERR_IBS_IMPCLI_ACCO_NOTEXIST
                         * Error, the CRM Account doesn't exists
                         * Error, la Cuenta CRM NO existe
                         */
                        $ar_error['0215'] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_ACCO_NOTEXIST') . ' (' . $ar_acco['ID_TYPE'] . '-' . $ar_acco['IDENTIFICATION'] . ')', '');
                    } else {
                        $this->s_clientImport_operation_type = '0001';
                    }
                } else {
                    // Stores IDs for further use at update
                    $this->a_clientImport_id['ID'] = $ar_cli_record['ID'];
                    $this->a_clientImport_id['IDCODE'] = $ar_cli_record['IDCODE'];
                }
            } else {
                /*
                 * GEN_ERR_IBS_IMPCLI_INVALID_OPE
                 * Error, cannot handle Operation Type (^1)
                 * Error, Tipo de Operación no manejada (^1)
                 */
                $ar_error['0225'] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_INVALID_OPE', $this->s_clientImport_operation_type), '');
            }
        }
        if (!empty($this->debug)) {
            echoLog(debug_dump(CLX_CHAR_NL . 'ar_error', $ar_error) . '<br>' . CLX_CHAR_NL);
        }
        if (!empty($this->debug)) {
            ClxCommon::$db->debug = false;
        }
        return $ar_error;
    }

// Method End
    /**
     * Process an array of one record processing errors during any kind of
     * client import and generates a message using ELERR Clx template
     *
     * @param integet $i_proc_counter
     * @param array $ar_error
     * @return text
     */
    function create_client_procesa_error_ar($i_proc_counter, $ar_error) {
        if ($this->b_create_client_procesa_error_ar == false) {
            // Friendly error messages
            $a_response = $this->create_client_procesa_error_friendly($ar_error);
            $s_response = $a_response['s_response'];
            $i_error_counter = $a_response['i_error_counter'];
        } else {
            // IBS Type error menssage
            if (!empty($this->debug))
                echoLog(debug_dump('create_client_procesa_error_ar - ERROR (ar_error)', $ar_error));
            $EBSK = getTool('EBSK');
            if($EBSK->debug === false) $EBSK->debug = $this->debug;
            $EBSK->tfields_only = true;
            $EBSK->remove_decimal_point = true;
            $msg_code = 'ELEERR';
            $i_error_counter = CLX_INIT_ZERO;
            $record = array('HEADER' => $msg_code);
            foreach ($ar_error as $key => $val) {
                $i_error_counter++;
                $s_error_counter = substr('00' . $i_error_counter, CLX_NUM_ - CLX_NUM_TWO);
                $record['ERNU' . $s_error_counter] = $key;     // Error code
                $record['ERDS' . $s_error_counter] = $val['ds'];   // Description
                $record['ERDF' . $s_error_counter] = $val['df'];   // Definition (fieldname)
                $record['ERDR' . $s_error_counter] = $val['dr'];
                $record['ERWF' . $s_error_counter] = $val['wf'];
            }
            $record['ERRNUM'] = $i_error_counter;
            $record['ERWRNG'] = '';
            $ar_socket_line = $EBSK->createMessage($msg_code, $record);
            $total = count($ar_socket_line);
            for ($i_i = CLX_INIT_ZERO; $i_i < $total; $i_i++) {
                if (isset($record[$ar_socket_line[$i_i]['FILEFIELD']])) {
                    $ar_socket_line[$i_i]['VAL'] = $record[$ar_socket_line[$i_i]['FILEFIELD']];
                }
            }
            $EBSK->msgrecord = $ar_socket_line;
            if (!empty($this->debug))
                echoLog(debug_dump('ar_socket_line', $ar_socket_line));
            $s_response = $EBSK->toString_();
        }
        // Add the operation type description
        $s_response = $this->create_client_operation_descrip() . $s_response;
        if ($i_error_counter > CLX_NUM_ZERO) {
            // Create entry in process engine exception log
            $this->proc_log_write_ibs($s_response, true, array('RECORDID' => $i_proc_counter));
        }
        // Returns error menssage(s) in one string
        return $s_response;
    }

// Method End
    /**
     * Process an array of one record processing errors during any
     * kind of client import and generates a message using ELERR Clx template
     *
     * @param array $ar_error
     * @return array
     */
    function create_client_procesa_error_friendly($ar_error) {
        $i_error_counter = CLX_INIT_ZERO;
        $s_response = '';
        $s_sep = '';
        foreach ($ar_error as $key => $val) {
            $i_error_counter++;
            $s_fldname = '';
            if ($val['df'] !== '') {
                $s_fldname = ', ' . $val['df'];
            }
            $s_response .= $s_sep . $val['ds'] . ' (ERR. # ' . $key . $s_fldname . ')';
            $s_sep = ', ';
        }
        if ($i_error_counter == CLX_NUM_ZERO) {
            $s_response = dict('GEN_LBL_OK');
        }
        return array('s_response' => $s_response,
            'i_error_counter' => $i_error_counter
        );
    }

// Method End
    /**
     * Returns an error array entry
     *
     * @param text $s_desc
     * @param text $s_definition
     * @param text $i_dr
     * @param text $s_wf
     * @return void
     */
    function create_error_record_ar($s_desc, $s_definition, $i_dr = '', $s_wf = '') {
        if (!empty($this->debug))
            echoLog('create_error_record_ar - desc=' . $s_desc . ', def=' . $s_definition . ', dr=' . $i_dr . ', wf=' . $s_wf);
        return array('ds' => $s_desc,
            'df' => $s_definition,
            'dr' => $i_dr,
            'wf' => $s_wf,
        );
    }

// Method End
    /**
     * Do some fixed to dates that comes in the flat file
     *
     * @param array $ar_date
     * @return text
     */
    function create_client_date_fix($ar_date) {
        $ar_date['YY'] = trim($ar_date['YY']);
        $ar_date['MM'] = trim($ar_date['MM']);
        $ar_date['DD'] = trim($ar_date['DD']);
        // If it has more chars than expected, maybe cotains a leading zero
        if (strlen($ar_date['YY']) > CLX_NUM_FOUR)
            $ar_date['YY'] = substr($ar_date['YY'], CLX_NUM_ - CLX_NUM_FOUR);
        if (strlen($ar_date['MM']) > CLX_NUM_TWO)
            $ar_date['MM'] = substr($ar_date['MM'], CLX_NUM_ - CLX_NUM_TWO);
        if (strlen($ar_date['DD']) > CLX_NUM_TWO)
            $ar_date['DD'] = substr($ar_date['DD'], CLX_NUM_ - CLX_NUM_TWO);
        if (CLX_NUM_ZERO + substr($ar_date['YY'], CLX_NUM_ZERO, CLX_NUM_TWO) == CLX_NUM_ZERO) {
            $s_year = substr($ar_date['YY'], CLX_NUM_ - CLX_NUM_TWO);
            $s_century = '19';
            if (CLX_NUM_ZERO + $s_year <= $this->i_year_limit_tobe_xxi_century) {
                $s_century = '20';
            }
            $ar_date['YY'] = $s_century . $s_year;
        }
        // Concatenates the elements and returns it according to the YYYY-MM-DD format
        $s_tmp_date = $ar_date['YY'] . '-' . $ar_date['MM'] . '-' . $ar_date['DD'];
        return $s_tmp_date;
    }

// Method End
    /**
     * Convert legal type field
     *
     * @param text $val
     * @param array $ar_error
     * @return text
     */
    function create_client_legatype_conversion($val, &$ar_error) {
        switch ($val) {
            case '1':
                $val = 'J';
                break;
            case '2':
                $val = 'V';
                break;
            case 'V': // Venezolano
            case 'E': // Extranjero
            case 'G': // Gobierno
            case 'J': // Jurídico
            case 'R': // Firma Personal
                break;
            default:
                /*
                 * GEN_ERR_IBS_IMPCLI_WRONGLEGTYP
                 * Wrong legal type value: ^1. Must be V o J
                 * Error en el tipo de persona: ^1. Debe ser V o J
                 */
                $ar_error['0270'] = $this->create_error_record_ar(dict('GEN_ERR_IBS_IMPCLI_WRONGLEGTYP', $val), '');
                break;
        }
        return $val;
    }

// Method End
}
