<?php
/**
 * Oracle database class
 * For more information contact  support@alenet.com
 *
 * @author       $Author: alenetdev $
 * @copyright    Alenet, Inc.
 * @since        $Date: 2019-02-22 13:21:30 +0000 (Fri, 22 Feb 2019) $
 * @version    	$Rev: 10555 $
 * @package		Clientix.Classes.Database
 * @abstract		OCI - Database Abstraction Layer
 *
 * 	Intallation instructions for WIN32 + Apache.
 * 	1.	Install Oracle Db. and Create the db and a listener to connect to.
 * 	2.	If php not in same computer, download the instant Thin client
 * 		and copy it to a folder. For example E:\OracleInstantClientxxx
 * 	3.	Add the oracle client folder to the path(system variable) so it will be accesible.
 * 	4.	Copy the tnsnames.ora to the client folder.
 * 		Sample sintax for tnsnames.ora:
 * 		REPLACE_FOR_LISTENER_NAME =
 * 			(DESCRIPTION =
 *	 			(ADDRESS = (PROTOCOL = TCP)
 *					(HOST = REPLACE_HOST_OR_IP)
 *					(PORT = REPLACE_PORT_USUALLY_1521)
 *				)
 * 			(CONNECT_DATA =
 * 			  (SERVER = DEDICATED)
 * 			  (SERVICE_NAME = REPLACE_DATABASENAME)
 * 			)
 * 		  )
 * Note: if using instant client tnsora is not required just use for url as [//]HOST[:PORT]/SERVICE
 * 5. Set enviroment Variables or create Entries on the registry.
 *  SET PATH=E:\OracleInstantClientxxx
 *   This is obviously for finding the executables.
 *  SET TNS_ADMIN=E:\OracleInstantClientxxx
 *   This is only for finding the tnsnames.ora file. You do not have to have this in the Instant Client directory but if you were on a database box or one that already has the Oracle Client on it, where else would you put it?
 *  SET LD_LIBRARY_PATH=E:\OracleInstantClientxxx
 *   Obviously to find the shared libraries shipped with Instant Client.
 *  SET SQLPATH=E:\OracleInstantClientxxx
 *   If you use a glogin.sql file, you will need to set this to find it.
 * Others Registry alternatives:
 *  a. Open regedit and add a Key called ORACLE to HKEY_LOCAL_MACHINE\SOFTWARE 
 * 	b. To the ORACLE key add a string value called TNS_ADMIN and set value ora client folder.
 *  c. add the following additional strign for language and date formats on the same oracle key:
 *		string: LD_LIBRARY_PATH        value: /pathtoclient
 *		string: NLS_DATE_FORMAT        value: YYYY-MM-DD HH24:MI:SS
 *		string: NLS_LANG			value: ENGLISH_AMERICA.WE8ISO8859P1
 *		string: NLS_TIMESTAMP_FORMAT:  value: YYYY-MM-DD HH24:MI:SS
 * 	6. Install php with oci8 library enabled.
 * 	7. Important Notes: In apache with php as module some times oci library make apache crash. No problem found under apache with php as cgi
 * Just remember to start
 * 	    CREATE USER username IDENTIFIED BY passwrod;
 * 		GRANT ALL PRIVILEGES TO username;
 * 	Display sessions and cursors
 * 	SELECT s.sid,
 * 		   s.serial#,
 * 		   s.machine,
 * 		   s.osuser,
 * 		   s.username,
 * 		   s.program,
 * 		   v.value numopencursors
 *		FROM V$SESSTAT v, V$SESSION s
 *		WHERE v.statistic# = 3 and v.sid = s.sid;
 * 	Killing sessions
 * 	ALTER SYSTEM KILL SESSION 'sid,serial#';
 * 	ALTER SESSION SET CURSOR_SHARING='FORCE'
 *
 */
class OciDatabase extends ClxDatabase {

    var $error = array();
    var $lastId = CLX_INIT_ZERO; //Holds the last insert id
    var $nlsInit = true; //Set false to prevent init session nls
    var $enable_purge = true; //set false for oracle version < 10i
    var $sessionMode = OCI_DEFAULT; //Session mode for connection options: OCI_DEFAULT, OCI_SYSOPER and OCI_SYSDBA.
    var $nlsSession = array(
        'nls_language' => 'AMERICAN'
        , 'nls_timestamp_format' => 'YYYY-MM-DD HH24:MI:SS'
        , 'nls_date_format' => 'YYYY-MM-DD HH24:MI:SS'
#        , 'nls_sort' => 'BINARY_CI' // To allow index to be case insensitive
        , 'nls_sort' => 'BINARY' // To allow index to be case insensitive
#        , 'nls_comp' => 'LINGUISTIC'    // To allow searches to be case insensitive
        , 'nls_comp' => 'BINARY'    // To allow searches to be case insensitive
#        , 'nls_lang' => 'ENGLISH_AMERICA.WE8ISO8859P1'    // 2018-06-22 CR - Para evitar "ORA-64204: encountered partial multibyte character" en columna CONTENTS de la tabla CLX_SD_FILES
    );
    var $stmt = null;
    var $debug = false;
	var $debugVIEW=false;    
    //var $temporary_supress_squote_change = false;
    private $table_structure_has_changes = array();
    private $renamed_columns = array();
    private $table_struc_temp_name = array();
    private $temp_fieldRename = false;
    public $affected_rows=0;
	public $tmp_fld_name='';
    /**
     * Constructor
     */
    function __construct($url, $user, $pass, $name) {
        $this->databaseURL = $url;
        $this->databaseUsername = $user;
        $this->databasePassword = $pass;
        $this->databaseName = $name;
        $this->permanentCnx = kDatabasePermanent;
        $this->charset = kDatabaseCharset;
        $this->debug = false;
        $this->timelog = false;
        $this->charset = 'AL32UTF8';
        $this->sessionMode = null;
		$this->numerictype=array(); //e cho $url,",",$pass,",",$user;
		$this->debugVIEW=false;
		$this->debugCF=false;
		$this->errorGeneral=false;
		
    }
	//function exists_sequence($sequencename) {
	//	Return  $this->getOne('SELECT count(*) FROM USER_SEQUENCES WHERE SEQUENCE_NAME='.singlequote($sequencename))>0;
	//} 
    /**a
     * Database Error Number
     * @param: SQL Query
     */
    function errorNum() {
        return (!empty($this->error['code'])) ? $this->error['code'] : '';
    }

    /**
     * Database Error Message
     * @param: SQL Query
     */
    function errorMsg() {
        return (!empty($this->error['message'])) ? $this->error['message'] : '';
    }

    /**
     * Returns string with information about the most recent query.
     * @param: SQL Query
     */
    function info() {
        // Should return the last sql executed
        return $this->stmt;
    }

    /**
     * Set oci error property to be used by erronum and msg from db error
     * @param: query result or nothing for error during connection
     */
    function error($entry = false) {
        $this->error = array();
        if (empty($entry)) {
		  $this->error = oci_error();
        } else {
		  $this->error = oci_error($entry);
        }
    }

    function nlsSessionInit() {
		if (!empty($this->nlsInit)) {
		  foreach ($this->nlsSession as $key => $val) {
			 if (isset($_GET['ADDRDEBUGOCISESSION'])) {
				pre("session");pre("$key=>$val");}
			 $retorno = $this->executeQuery('ALTER SESSION SET ' . $key . '=' . singleQuote($val));
			 if($this->hasError()) {
		
			     $this->exitOnConnError('ERR2');
				 
			 }
			 unset($retorno);
		  }
        }
		if (isset($_GET['ADDRDEBUGOCISESSION'])) {
			pre("pre cursor number");}
		//$this->executequery('ALTER SYSTEM SET open_cursors = 2000 SCOPE=both');
		if (isset($_GET['ADDRDEBUGOCISESSION'])) {
			pre("fin function alter");}
		
    }

    function exitOnConnError($errNum, $otherInfo = 'Connection to ORACLE database fail') {
        salir('<center><h2>Clientix</h2><br><p>' . $errNum .' - ' . $otherInfo . '<br><i><font size=2>' . $this->errorMsg() . '</font></i>. <br><br>Please contact system administrator. <br><br>Thank You.</p></center>');
    }

    function dumpSessionParameters() {
		
        $rs = $this->executeQuery('SELECT * FROM NLS_SESSION_PARAMETERS');
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  print_r($result);
        }
        $this->freeResult($rs);
    }

    /**
     * Open a connection to a Database Server
     */
    function connectToDatabase() {
		//echo "<pre>".$this->databaseUsername ,",", $this->databasePassword,",", $this->databaseURL,",", $this->charset,",", $this->sessionMode."</pre>";

		//$db = "(DESCRIPTION=(ADDRESS_LIST = (ADDRESS = (PROTOCOL = TCP)(HOST = 129.146.240.155)(PORT = 1521)))(CONNECT_DATA=(SID=gff56dev)))";
		$tnsCloud = "(description= (retry_count=20)(retry_delay=3)(address=(protocol=tcps)(port=1522)(host=adb.us-ashburn-1.oraclecloud.com))(connect_data=(service_name=rdbakkhhe8rxuhm_clxoradev1_high.adb.oraclecloud.com))(security=(ssl_server_dn_match=yes)))";
        
		$cloudConn = strpos($this->databaseURL, 'oraclecloud');
		for ($t=3;$t>0;$t--) {
			if($cloudConn === false){
				//conexion tradicional
				if (!empty($this->permanentCnx) && function_exists('oci_pconnect') ){
					//echo "oci_pconnect($this->databaseUsername, $this->databasePassword, $this->databaseURL, $this->charset, $this->sessionMode";
					  $this->dblink = oci_pconnect($this->databaseUsername, $this->databasePassword, $this->databaseURL, $this->charset, $this->sessionMode);
					} else {
					  $this->dblink = oci_connect($this->databaseUsername, $this->databasePassword, $this->databaseURL, $this->charset, $this->sessionMode);
					}
			}else{
				//conexion cloud
				//echo "<pre>".$this->databaseUsername ,",", $this->databasePassword,",", $this->databaseURL,",", $this->charset,",", $this->sessionMode."</pre>";
				if (!empty($this->permanentCnx) && function_exists('oci_pconnect') ){
					//echo "oci_pconnect($this->databaseUsername, $this->databasePassword, $this->databaseURL, $this->charset, $this->sessionMode";
					  //$this->dblink = oci_pconnect($this->databaseUsername, $this->databasePassword, $tnsCloud);
					  $this->dblink = oci_pconnect($this->databaseUsername, $this->databasePassword, 'tcps://adb.us-ashburn-1.oraclecloud.com:1522/rdbakkhhe8rxuhm_clxoradev1_high.adb.oraclecloud.com?wallet_location=C:\oracle\network\admin',$this->charset, $this->sessionMode);
					} else {
					  //$this->dblink = oci_connect($this->databaseUsername, $this->databasePassword, $tnsCloud);
					  $this->dblink = oci_connect($this->databaseUsername, $this->databasePassword, 'tcps://adb.us-ashburn-1.oraclecloud.com:1522/rdbakkhhe8rxuhm_clxoradev1_high.adb.oraclecloud.com?wallet_location=C:\oracle\network\admin',$this->charset, $this->sessionMode);
					}
			}
			if ($this->dblink)
				break;
		}	
		

        if (!$this->dblink) {
		  $this->error();
		  $this->exitOnConnError('ERR1');
		  $this->reportError('oci_connect...', $this->errorNum(), $this->errorMsg());
        } else {
		  $this->nlsSessionInit();
			$this->executeQueryCiInitSesVars();
			$this->sethourdefault();
        }
    }
	function sethourdefault(){
		$hourdefault=ClxCommon::$db->getOne('select SYPRVALUE from clx_sys_parameters WHERE SYPRKEY='.singlequote('SET_DATABASE_HOUR_DEFAULT'));
		//echo ClxCommon::$db->getOne('select current_date from dual');
		if (!empty($hourdefault)) 
			ClxCommon::$db->executequery('ALTER SESSION SET TIME_ZONE = '.singlequote($hourdefault));
		//echo ClxCommon::$db->getOne('select current_date from dual');
    }

    /**
     * Selects the default database for queries
     */
    function selectDatabase() {
        //Do nothing
    }

    /**
     * Close database connection
     */
    function closeConnection() {
		oci_close($this->dblink);
    }

    /**
     * Returns commit mode
     * Return integer
     */
    function getCommitMode() {
        //$mode = ($this->getAutocommit()) ? OCI_COMMIT_ON_SUCCESS : OCI_DEFAULT;
        $mode = ($this->getAutocommit()) ? OCI_COMMIT_ON_SUCCESS : OCI_NO_AUTO_COMMIT;
        return $mode;
    }

    /**
     * Parse sql Statement to database connection
     *f= @param: SQL Query
     */
    function parse($sql) {
        return oci_parse($this->dblink, $sql);
    }

    /**
     * Execute sql statement
     * @param: Statement, optional commit mode default auto commit
     * Modes: OCI_COMMIT_ON_SUCCESS -> Commit its executed automatically
     * 		 OCI_DEFAULT -> Commit should be called Explicitelly after execute
     */
	function execute(&$stmt) {
	//echo '<br>';
	//$te=debug_backtrace();$posx=0;for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";}}	
			
	//pre($this->dblink);
//echo "------1-<br>";
//echo $stmt;
	/*te=debug_backtrace(); 
	$pos=$posx=0;
	for ($t=0;$t<12;$t++) {
		if (isset($te[$t]["file"])){
			if ($t==$posx)
				echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";
			echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
		}
	}
		echo "------2-<br>";
		*/
		$result=(!empty($stmt)) ? oci_execute($stmt, $this->getCommitMode()) : false;
	//re(oci_error($stmt));
		if (!empty($stmt) &&!oci_error($stmt) ) {
		
			$this->affected_rows=oci_num_rows($stmt) ;
		}
		else {
			$this->affected_rows=0;
		}
		return $result;
    }

    /**
     * Commit Execute sql statement
     */
    function commit() {
        return oci_commit($this->dblink);
    }

    /**
     * Rollback
     */
    function rollback() {
        return oci_rollback($this->dblink);
    }

    function statementType($statement) {
        return oci_statement_type($statement);
    }

    function isSelect($statement) {
        return $this->statementType($statement) === 'SELECT';
    }

    /** Change autocommit status
     *
     * @param boolean $boolean
     */
    function setAutocommit($boolean) {
        $this->autocommit = $boolean;
    }

    /**
     * Executes SQL Query and returns the correspondant result
     * resource or error on failure.
     * @param: SQL Query
     */
    function executeQuery($sql, $binds = array()) {
		$this->error['message']="";
		if (($this->debugCF and stripos(' '.$sql,'alter' )>0) OR $this->debugVIEW==true) {
			PRE('INICIO EXECUTEQUERY via create field');
			PRE($sql);
			echo $sql,chr(10),chr(10);echo $sql,chr(10),chr(10);$te=debug_backtrace();$posx=0;for ($t=0;$t<12;$t++) {
				if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";}}
		
		}
		
		IF (ISSET($_SESSION['ADDRDEBUGOCI']) && $_SESSION['ADDRDEBUGOCI']==$_SERVER['REMOTE_ADDR'])
			$this->debug=true;
		$this->sql=$sql;
		if ( !empty($_SESSION["_dEBUG"]))
			list($h1,$m1,$s1)=explode(':', date('h:i:s'));
		if (!empty($this->debug)) {
			$te=debug_backtrace();$mc1=microtime(true);	
		    echoLog($sql . ';     <br>');
			print_r($binds) ; echo '<br><br>'.CLX_CHAR_NL;
			
			for ($atx=1;$atx<5;$atx++) {
				if (isset($te[$atx]["file"])) {
					echoLog('<nobr>'.str_repeat(' &nbsp;',50).  substr((isset($te[$atx]["file"])?$te[$atx]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[$atx]["line"])?$te[$atx]["line"]:'').chr(9).$te[$atx]["function"] . ';</nobr><br>');
				}
			}
			pre($binds);
		}
		
        if (!empty($this->debug)) {
			if (stripos($sql,'menu')==false && (stripos($sql,'diction')==false)&& (stripos($sql,'CLX_T')==false))
		  echoLog($sql . ';<br>');
        }
	    if($this->debug) {
			if (stripos($sql,'menu')==false && (stripos($sql,'diction')==false)&& (stripos($sql,'CLX_T')==false)){
				echo CLX_CHAR_NL . '<br>executeQuery | '. (1==2 ? ('<br>'. CLX_CHAR_NL . $sql . '<br>' . CLX_CHAR_NL) : '') . 'BEFORE - Binds:' . CLX_CHAR_NL ; 
			
				print_r($binds) ; echo '<br><br>'.CLX_CHAR_NL;
			}	
		}
		if ($this->debugCF and stripos(' '.$sql,'alter' )>0) {
			PRE("EN EXECUTEQUERY CORRIDA DE ALTER");
			$te=debug_backtrace();$posx=0;for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";}}	
			PRE($sql);
			echo "--------------------------->>>>>'";
			pre($this->error);
		}
        $this->lastId = CLX_INIT_ZERO; // Init insert id.
        $this->logStart();
        //rewrite mysql functions
        $this->clx_parse($sql);
        if (!empty($this->debug)) {
			if (stripos($sql,'menu')==false && (stripos($sql,'diction')==false)&& (stripos($sql,'CLX_T')==false))
				echoLog($sql . ';<br>');
         
        }
        //Creating statement
        $stmt = $this->parse($sql);
        $this->stmt = $stmt;
        // Execute lobs binding variables
        if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));
			$te=debug_backtrace();$mc1=microtime(true);	
		    
			for ($atx=1;$atx<5;$atx++) {
				if (isset($te[$atx]["file"])) {
					fwrite($tta,chr(13).chr(10).str_repeat(' ;',50).  substr((isset($te[$atx]["file"])?$te[$atx]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[$atx]["line"])?$te[$atx]["line"]:'').chr(9).$te[$atx]["function"] . ';');
				}
			}
			fwrite($tta,chr(13).chr(10)."sql:".$sql.chr(13).chr(10));
			
			fwrite($tta,chr(13).chr(10)."Binds:".print_r($binds,true).chr(13).chr(10));fclose($tta);}
        reset($binds);
		
		foreach ($binds as $key => $val) {
		  //$this->bindNameVoid($stmt, $key, $binds[$key]);
		  if($key==='ID') {
				if($this->debug) {echo CLX_CHAR_NL . '<br> executeQuery | ** ENVIO ID COMO SQLT_INT **';}
			 $this->bindNameVoid($stmt, $key, $binds[$key], -CLX_NUM_ONE, SQLT_INT);
		  } else if (is_Array($val)) {
				unset($binds[$key]);	
			}
			else {	
				$typebind=null;
				if (isset($this->numerictype)){
					foreach($this->numerictype as $table=>$values) {
						if (isset($values[$key.'_CHANGE'])) {
							$values[$key]=$values[$key.'_CHANGE'];
						}
						
						if (isset($values[$key])) {	//pre($values[$key]);
							switch ($values[$key]) {
								case 'DECN':
									$typebind=SQLT_LNG;//SQLT_INT;
									break;
								case 'INTN':
									$typebind=SQLT_INT;
						
									break;	
							}
							break;
						}	
						else {
								
						}	
					}
				}
				$this->bindNameVoid($stmt, $key, $binds[$key],-CLX_NUM_ONE,$typebind);
			}
				
        }
        // Executing record
      	
        if (!$this->execute($stmt)) {
			
		  $this->error($stmt);
			$xsqlxA=beautiful_sql($sql,'<br>', " &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",false);
			$xsqlxB=beautiful_sql($this->sql,'<br>', " &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",false);
			if ($xsqlxA<>$xsqlxB) {
				$xsqxA='Modificate sql:<br>'.$xsqlxA.'Original sql:<br>'.$xsqlxB;
			}	
			$this->reportError($xsqlxA,
						$this->errorNum(), $this->errorMsg(), $binds);
			if ( !empty($_SESSION["_dEBUG"])) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"ERROR: ".$this->errorNum().",". $this->errorMsg().chr(13).chr(10));fclose($tta);}
        				
        }

        
        if (!empty($binds['ID'])) {
		  $this->lastId = $binds['ID'];
        }
        $this->logEnd();
        $this->writeTimeLog($sql); //Writing log
		
        if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));list($h2,$m2,$s2)=explode(':', date('h:i:s'));	fwrite($tta,"total segundos:". (($h2-$h1)*3600+($m2-$m1)*60+($s2-$s1)).chr(13).chr(10));fclose($tta);}
		if (!empty($this->debug)) {
		
			$mc2=microtime(true);
			$mcresta=$mc2-$mc1;
			$mxt1=''; $mxt2='';
			if ($mcresta>1) {$mxt1='<strong style="color:red;background-color: yellow">'; $mxt2='</strong>'; }
			echoLog(str_repeat(' &nbsp;',50). "total segundos:".$mxt1. (SPRINTF("%7.4f",($mcresta)))  .$mxt2.'<br><br>');

		}
		if (ISSET($_SESSION['ADDRDEBUGNEXTRECORD'])) {ECHO '<strong style="color:WHITE;background-color: RED">', $stmt,"</STRONG>"; $GLOBALS["Xmc1"][(string)$stmt]=microtime(true);}

        return $stmt;
    }

    function dumpCursors() {
        echoLog('Open Cursors and totals');
        $sql = 'select a.value, s.username, s.sid, s.serial# serial from v$sesstat a, v' . $statname . ' b, v' . $session . ' s ' . ' WHERE a.statistic# = b.statistic#  and s.sid=a.sid and b.name = ' . singleQuote('opened cursors current') . ' and a.value>0 and s.username =' . singleQuote(uppercase($this->databaseUsername));
        $rs = $this->executeQuery($sql);
        while (($rs = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  echoLog('Total cursors: ' . $result['VALUE'] . ' sid: ' . $result['SID'] . ' serial: ' . $result['SERIAL']);
        }
		$this->freeResult($rs);		
		
        $sql = 'SELECT MAX(A.VALUE) AS HIGHEST_OPEN_CUR, P.VALUE AS MAX_OPEN_CUR ' . 'from v$sesstat a, v$statname b, v$parameter p ' . ' WHERE a.statistic# = b.statistic# ' . 'and b.name = ' . singleQuote('opened cursors current') . ' ' . 'and p.name= ' . singleQuote('open_cursors') . ' ' . 'group by p.value';
        $rs = $this->executeQuery($sql);
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  echoLog('Total server cursors: ' . $result['HIGHEST_OPEN_CUR'] . ' Max allowed: ' . $result['MAX_OPEN_CUR']);
        }
		$this->freeResult($rs);		
		echoLog('Open Cache Cursors and totals');
        $sql = 'select a.value, s.username, s.sid, s.serial# serial ' . 'from v' . $sesstat . ' a, v' . $statname . ' b, v' . $session . ' s where a.statistic# = b.statistic#  and s.sid=a.sid ' . 'and b.name = ' . singleQuote('session cursor cache count') . ' and a.value>0 and s.username =' . singleQuote(uppercase($this->databaseUsername));
        $rs = $this->executeQuery($sql);
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  echoLog('Total cached cursors: ' . $result['VALUE'] . ' sid: ' . $result['SID'] . ' serial: ' . $result['SERIAL']);
        }
			
		$this->freeResult($rs);		
			
        $sql = 'SELECT MAX(A.VALUE) AS HIGHEST_OPEN_CUR, P.VALUE AS MAX_OPEN_CUR from v' . $sesstat . ' a, v' . $statname . ' b, v' . $parameter . ' p ' . ' WHERE a.statistic# = b.statistic# ' . 'and b.name = ' . singleQuote('session cursor cache count') . ' ' . 'and p.name= ' . singleQuote('session_cached_cursors') . ' ' . 'group by p.value';
        $rs = $this->executeQuery($sql);
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  echoLog('Total cached server cursors: ' . $result['HIGHEST_OPEN_CUR'] . ' Max allowed: ' . $result['MAX_OPEN_CUR']);
        }
		$this->freeResult($rs);		
    }

    /**
     * Executes SQL Query (Case Insensitive)
     * @param: SQL Query
     */
    function executeQueryCi($sql) {
		
        // Preference to enable CI (Case Insensitive) capability undex Oracle Database CLX installation
        $clx_lsrch_ci = (isset($_SESSION['prefs']['LSRCH_CI']) ? $_SESSION['prefs']['LSRCH_CI'] : null);
        if(is_null($clx_lsrch_ci) || $clx_lsrch_ci === CLX_CHAR_EMPTY_STRING . CLX_NUM_ZERO) {
		  return $this->executeQuery($sql);
        }
        $not_ci = (!isset($this->nlsSession['nls_comp']) || $this->nlsSession['nls_comp'] === 'BINARY');
        //if ($not_ci) {
         //   $this->executeQueryCiInitSesVars();
        //}
        $response = $this->executeQuery($sql);
        //if ($not_ci) { // no se puede hacer aca, porque deja de funcionar en la lectura del resultado del record set
         //   $this->executeQueryCiResetSesVars();
        //}
        return $response;
    }
    
    /**
     * Reset sesion variables altered before Case Insensitive query execution and result set processing
     * @param: void
     */
    function executeQueryCiResetSesVars() {
		 $this->executeQueryCiInitSesVars();
	//	if (!isset($GLOBALS['CiInitSesVars2'])) {
	//		$GLOBALS['CiInitSesVars2']=1;
	//		$this->executeQuerySilent('ALTER SESSION SET NLS_SORT=' . singleQuote('BINARY'));
	//		$this->executeQuerySilent('ALTER SESSION SET NLS_COMP=' . singleQuote('BINARY'));
	//}	
    }
    
    /**
     * Init sesion variables for Case Insensitive query execution
     * @param: void
     */
    function executeQueryCiInitSesVars() {
		if (!isset($GLOBALS['CiInitSesVars'])) {
			$GLOBALS['CiInitSesVars']=1;
			if (isset($_GET['ADDRDEBUGOCISESSION'])) {
				pre("pre alter sort comp");}	
			$this->executeQuerySilent('ALTER SESSION SET NLS_SORT=' . singleQuote('BINARY'));
			$this->executeQuerySilent('ALTER SESSION SET NLS_COMP=' . singleQuote('LINGUISTIC'));
			if (isset($_GET['ADDRDEBUGOCISESSION'])) {
				pre("post alter sort comp");}
		}
    }

    /**
     * Equivalent to executeQuery with the exception of no error reported on
     * failure
     * @params: string sql, array ocilobs containing  all records that need
     * bindings
     * @param: SQL Query
     */
    function executeQuerySilent($sql, $binds = array()) {
		list($h1,$m1,$s1)=explode(':', date('h:i:s'));		
		if (($this->debugCF and stripos(' '.$sql,'alter' )>0) OR $this->debugVIEW==true) {
			PRE('INICIO EXECUTEQUERY via create field');
			PRE($sql);
			
			for ($atx=1;$atx<5;$atx++) {
				if (isset($te[$atx]["file"])) {
					echoLog('<nobr>'.str_repeat(' &nbsp;',50).  substr((isset($te[$atx]["file"])?$te[$atx]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[$atx]["line"])?$te[$atx]["line"]:'').chr(9).$te[$atx]["function"] . ';</nobr><br>');
				}
			}
		}
		IF (ISSET($_SESSION['ADDRDEBUGOCI']) && $_SESSION['ADDRDEBUGOCI']==$_SERVER['REMOTE_ADDR'])
			$this->debug=true;
		
		$this->sql=$sql;
	//	if ( !empty($_SESSION["_dEBUG"]) and (((strlen($sql)>65 or (strlen($sql)<=65 and stripos($sql, "IDCODE")<1)) and stripos($sql, "CLX_DICTIONARY")<1 and stripos($sql, "dictionary")<1 /*and stripos($sql, "SER_SESSIONS")<1  */ /* and stripos($sql, "events")<1 */ and stripos($sql, "CLX_AC_GROUP_MEMBERS")<1 and stripos($sql, "CLX_UI_FORMS")<1 /* and stripos($sql, "CLX_DB_TABLE")<1 */		   and stripos($sql, "CLX_OBJECTS")<1 and stripos($sql, "CLX_DB_DYNRELATIONS")<1 and stripos($sql, "CLX_AC_USERS")<1 /*and stripos($sql, "CLX_SYS_PARAMETERS")<1 */ and stripos($sql, "CLX_CODELIB")<1 and stripos($sql, "ALRT_ALERT_SCHEDU")<1 and stripos($sql, "CLX_TOOLS")<1 and stripos($sql, "CLX_UI_LISTINGS")<1 and stripos($sql, "CLX_UI_FORM_PANEL")<1 and stripos($sql, "CLX_DB_LISTS")<1 and stripos($sql, "CLX_DB_FIELDS")<1 and stripos($sql, "CLX_UI_VIEW_PREFS")<1) or $_SESSION["_dEBUG"]==99)) {list($h1,$m1,$s1)=explode(':', date('h:i:s'));IF (!ISSET($_SESSION["maxdebug"])) $_SESSION["maxdebug"]=400000; if (filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")>$_SESSION["maxdebug"]) {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","r"); $conte=fread($tta,filesize(kPathToWebSite."/".kPathToTemp."archivo.txt")); fclose($tta);$arreglos=explode("GET-",$conte);			for($t=0;$t<count($arreglos)-6;$t++) {unset($arreglos[$t]);	}  
		if (!empty($this->debug)) {
			$te=debug_backtrace();$mc1=microtime(true);	
		  echoLog($sql . ';     <br>');
			echoLog('<nobr>'.str_repeat(' &nbsp;',50).  substr((isset($te[1]["file"])?$te[1]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[1]["line"])?$te[1]["line"]:'').chr(9).$te[1]["function"] . ';</nobr><br>');
			echoLog('<nobr>'.str_repeat(' &nbsp;',50).  substr((isset($te[2]["file"])?$te[2]["file"]:'').str_repeat(" ",80),0,80).chr(9).(isset($te[2]["line"])?$te[2]["line"]:'').chr(9).$te[2]["function"] . ';<nobr><br>');
			pre($binds);
		}//pre($this->dblink);//		
			
		/*if (!isset($conte)) {$conte='';}        
		if (is_array($conte))   
			$conte=implode("GET-",$conte);	$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","w+");	fwrite($tta,substr($conte,-$_SESSION["maxdebug"]*80/100));}else {$tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");}	fwrite($tta,chr(13).chr(10)."1- hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,				"   remote:".(isset($_SERVER["REMOTE_ADDR"]) ? $_SERVER["REMOTE_ADDR"]:"").chr(13).chr(10));	fwrite($tta,chr(13).chr(10).beautiful_sql($sql,chr(13).chr(10),"    ",false).chr(13).chr(10));$te=debug_backtrace();for ($t=0;$t<18;$t++) { if (isset($te[$t]["file"]))	fwrite($tta,substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"].chr(13).chr(10));	}fwrite($tta,chr(13).chr(10));
		fclose($tta);}*/
        if (!empty($this->debug)) {
		  echoLog($sql . '<br>');
        }
		
        $this->lastId = CLX_INIT_ZERO; // Init insert id.
        $this->logStart();
        //rewrite mysql functions
        $this->clx_parse($sql);
        //Creating statement
        $stmt = $this->parse($sql);
        $this->stmt = $stmt;
        // Execute lobs binding variables
        reset($binds);
        foreach ($binds as $key => $val) {
		  //$this->bindNameVoid($stmt, $key, $binds[$key]);
		  if($key==='ID') {
			 if($this->debug) {echo CLX_CHAR_NL . '<br> executeQuerySilent | ** ENVIO ID COMO SQLT_INT **';}
			 $this->bindNameVoid($stmt, $key, $binds[$key], -CLX_NUM_ONE, SQLT_INT);
		  } else {
			 $this->bindNameVoid($stmt, $key, $binds[$key]);
		  }
        }
        if (!empty($stmt)) {
		  oci_set_prefetch($stmt, CLX_NUM_HUNDRED);
        }
        // Executing record
        if (!$this->execute($stmt)) {
			
		  $this->error($stmt);
			$xsqlxA=beautiful_sql($sql,'<br>', " &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",false);
			$xsqlxB=beautiful_sql($this->sql,'<br>', " &nbsp; &nbsp; &nbsp; &nbsp; &nbsp; &nbsp;",false);
			if ($xsqlxA<>$xsqlxB) {
				$xsqxA='Modificate sql:<br>'.$xsqlxA.'Original sql:<br>'.$xsqlxB;
			}
				
			$this->reportError('Not show Execuction ERROR:'.chr(13).chr(10).  $xsqlxA,$this->errorNum(), $this->errorMsg(), $binds,false);
			if ( !empty($_SESSION["_dEBUG"])) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"ERROR: ".$this->errorNum().",". $this->errorMsg().chr(13).chr(10));fclose($tta);}
        				
        }
		if ($this->debugCF and stripos(' '.$sql,'alter' )) {PRE("EN SQLSILENT CORRIDA DE ALTER");$te=debug_backtrace();$posx=0;for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";}}	PRE($sql);echo "--------------------------->>>>>'";pre($this->error);}
		
        if (!empty($binds['ID'])) {
		  $this->lastId = $binds['ID'];
        }
        $this->logEnd();
        $this->writeTimeLog($sql); //Writing log

		if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));list($h2,$m2,$s2)=explode(':', date('h:i:s'));	fwrite($tta,"total segundos:". (($h2-$h1)*3600+($m2-$m1)*60+($s2-$s1)).chr(13).chr(10));fclose($tta);}
		if (!empty($this->debug)) {$mc2=microtime(true);
			$mcresta=$mc2-$mc1;
			$mxt1=''; $mxt2='';
			if ($mcresta>1) {$mxt1='<strong style="color:red;background-color: yellow">'; $mxt2='</strong>'; }
			echoLog(str_repeat(' &nbsp;',50). "total segundos:".$mxt1. (SPRINTF("%7.4f",($mcresta)))  .$mxt2.'<br><br>');

		}		
        if (!empty($binds['ID'])) {
		  $this->lastId = $binds['ID'];
        }
        $this->logEnd();
        $this->writeTimeLog($sql); //Writing log
		if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));list($h2,$m2,$s2)=explode(':', date('h:i:s'));	fwrite($tta,"total segundos:". (($h2-$h1)*3600+($m2-$m1)*60+($s2-$s1)).chr(13).chr(10));fclose($tta);}
        if (ISSET($_SESSION['ADDRDEBUGNEXTRECORD'])) {ECHO '<strong style="color:WHITE;background-color: RED">', $stmt,"</STRONG>"; $GLOBALS["Xmc1"][(string)$stmt]=microtime(true);}

		
        return $stmt;
    }

    /**
     * Free SQL Results
     */
    function freeResult($statement) {
        //oci_cancel ( $statement );
		if (!empty($statement) and !is_null($statement)) {
	/*		$te=debug_backtrace(); 
	$pos=$posx=0;
	for ($t=0;$t<12;$t++) {
		if (isset($te[$t]["file"])){
			if ($t==$posx)
				echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";
			echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";
		}
	}
			echo("'".$statement."'");*/
			oci_free_statement($statement);
		}
    }

    /**
     * Creates a new database
     */
    function createDatabase() {

    }

    /**
     * Remove a Mysql database if exist
     */
    function dropDatabase() {

    }

    /**
     * Return true if table exists or false if not found on database
     * @param: String tablename
     */
    function tableExists($tablename) {
		if (isset($GLOBALS['apc_cache_get_tablesnames']['t_'.$tablename])) 
			return true;
		else {	
			//$binds = array('TABLE_NAME' => $tablename);
			$binds = array('TABLE_NAME' => uppercase($tablename));
			$sql = 'SELECT COUNT(TABLE_NAME) FROM USER_TABLES WHERE TABLE_NAME = :TABLE_NAME';
			$qty = $this->getOne($sql, $binds);
			return ($qty > CLX_NUM_ZERO) ? true : false;
		}
	}
	function tableExists_bycode($tablecode) {
		$binds = array('TABLE_CODE' => uppercase($tablecode));
		$sql = 'SELECT COUNT(T.TABLE_NAME) FROM CLX_DB_TABLES X JOIN USER_TABLES T ON X.MODULECODE|| '.singlequote('_').' ||X.IDNAME= T.TABLE_NAME WHERE X.IDCODE=:TABLE_CODE';
		$qty = $this->getOne($sql, $binds);
		return ($qty > CLX_NUM_ZERO) ? true : false;
	}

    /**
     * Return true if view exists or false if not found on database
     * @param: String viewname
     */
    function viewExists($viewname) {
        $binds = array('VIEW_NAME' => $viewname);
        $sql = 'SELECT COUNT(VIEW_NAME) FROM USER_VIEWS WHERE VIEW_NAME = :VIEW_NAME';
        $qty = $this->getOne($sql, $binds);
        return ($qty > CLX_NUM_ZERO) ? true : false;
    }

    /**
     * Creates physical table on database.
     * @param string $viewname Name of view
     * @param string $viewsql View's SQL
     */
    function createView($viewname, $viewsql) {
        $viewsql = $this->extrac_dbms_sql($viewsql);
        $sql = 'CREATE OR REPLACE VIEW ' . $viewname . ' AS ' . $viewsql;
        $retorno = $this->executeQuery($sql);
        unset($retorno);
    }

    /**
     * Alters a view on the database.
     * @param string $viewname Name of view
     * @param string $viewsql View's SQL
     */
    function alterView($viewname, $viewsql) {
        $viewsql = $this->extrac_dbms_sql($viewsql);
        $sql = 'CREATE OR REPLACE VIEW ' . $viewname . ' AS ' . $viewsql;
        $retorno = $this->executeQuery($sql);
        unset($retorno);
    }

    /**
     * Creates physical table on database.
     * @param string $tablename Name of table
     * @param string $tabletype Type of table VIEW, TEMP, ETC
     * @param array $fields Table specifications, $fields['fields'] = List of field and its caracteristicts
     * @param boolean $casesensitive Additiona parameter to define case sensitivity. only for mysql
     */
    function createTable($tablename, $tabletype, $fields, $casesensitive = false) {
        if ($tabletype === 'VIEW') {
		  return;
        }
        $sql = '';
        $comma = '';
        $indexes = array();
        $autoincrement = '';
		
	/*	$fields_keys=array_keys($record);
		foreach ($fields_keys as $name => $info) {
			if (uppercase($name)!=$name) {
				unset($record[$name]);
				$record[uppercase($name)]=$info;
			}
		}
	*//////////////	
        reset($fields);
        foreach ($fields as $name => $info) {
			
		  if (!array_key_exists('null', $info)) {
			 $info['null'] = 'YES';
		  }
		  $type = $info['type'];
		  $null = (!empty($info['null']) && $info['null'] === 'NO') ? 'NOT null' : 'null';
		  if (stristr($info['properties'], 'auto_increment') !== false) {
			 $autoincrement = $name;
		  }
		  if (!array_key_exists('default', $info)) {
			 $info['default'] = '';
		  }
		  $info['default'] = trim($info['default']);
		  $has_default = (strlen($info['default']) > CLX_NUM_ZERO) ? true : false;
		  $default = '';
		  if ($has_default) {
			 $default = 'DEFAULT ' . $info['default'];
		  }
			if (uppercase($default)=='DEFAULT TODAY')
				$default='DEFAULT CURRENT_DATE';

		  $sql .= $comma . str_replace('  ', ' ', $name . ' ' . $type . ' ' . $default . ' ' . $null);
		  $comma = ', ';
        }
        $comma = '';
        if ($tabletype === 'TEMP') {
		  $temporary = 'GLOBAL TEMPORARY  ';
		  $type = ' ON COMMIT PRESERVE ROWS';
        } else {
		  $temporary = '';
		  $type = '';
        }
        //constraint pk_iot_ primary key (a, b)
        $sql = 'CREATE ' . $temporary . 'TABLE ' . $tablename . ' (' . $sql;
        if (count($indexes) > CLX_NUM_ZERO) {
		  $sql .= $comma . implode(',', $indexes);
		  $comma = ',';
        }
        $sql .= ') ' . $type;
        $results = $this->executeQuery($sql);
        // Creating Autoincrements
        if (strlen($autoincrement) > CLX_NUM_ZERO) {
		  $this->createAutoIncrement($tablename, $autoincrement);
        }
        unset($results);
    }

    /**
     * Translate table structure according db implementation current default mysql
     * @param array $table Table structure as defined in tool or application
     * @return array Modified table structure accordin to specific database
     */
    function translateTableStructure($table) {
        $table['case_sensitive'] = false; //Case sensitivity already handled
        $table['type'] = ''; // No tabletype for oracle at the moment
        if (!empty($table['fields'])) {
		  reset($table['fields']);
		  foreach ($table['fields'] as $key => $val) {
			 $table['fields'][$key]['type'] = $this->mysqlToOci($val['type']);
				if ($this->debugCF==true) {
					pre('[[[[[>>>type old conversion');
					pre($val['type']);
					pre('[[[[[>>>type new conversion');
					pre($table['fields'][$key]['type']);
				}
		  }
        }
        return $table;
    }

    /**
     * Changes table name
     * @param string $oldtablename
     * @param string $newtablename
     */
    function renameTable($oldtablename, $newtablename) {
        $sql = 'RENAME ' . $oldtablename . ' TO ' . $newtablename;
        $results = $this->executeQuery($sql);
        unset($results);
    }

    /**
     * Drop Table from database
     */
    function dropTable($tablename) {//DI E('DROP3');
		$sql='DELETE FROM CLX_DB_TABLE_HIST X WHERE EXISTS ( SELECT 1 FROM CLX_DB_TABLES Y WHERE (X.TABLE_CODE_HIST=Y.IDCODE OR X.TABLE_CODE=Y.IDCODE) AND Y.MODULECODE||'.singlequote('_').'||Y.IDNAME='.singlequote($tablename).')';
		if ($this->debugCF==true) {pre('after drop table');pre($sql);}		
		$this->executequery($sql);
		$this->executequery('COMMIT');
		
        if ($this->tableExists($tablename)) {
		  $sql = 'DROP TABLE ' . $tablename . ' CASCADE CONSTRAINTS';
		  if ($this->debugCF==true) {pre ('drop sql');pre($sql);}	
		  $results = $this->executeQuery($sql);
		  unset($results);
        }
        $this->dropSequence($this->sequenceName($tablename));
        //deleted on drop table no need to drop
		
        $this->purgeTrash();
    }

    /**
     * Returns array containing tables names
     */
    function tables() {
        $result = array();
        $results = $this->getRecords('SELECT TABLE_NAME FROM ALL_TABLES');
        while (($row = $this->getNextRecord($results, NUMERIC)) === true) {
		  $result[] = $row[CLX_NUM_ZERO];
        }
		$this->freeResult($results);		
        return $result;
    }

    /**
     * Locks table
     */
    function tableLock($tablename) {
        return true;
    }

    /**
     * Unlock table, for oracle this has no meaning,
     * unlock us always after commit or rollback
     */
    function tableUnlock() {
        return true;
    }

    /**
     * Create Field on database
     * @param: string tablename, string $fieldname, array info
     */
    function createField($tablename, $fieldname, $info) {
		if ($this->debugCF==true){
			PRE( "FUNCTION CREATEFIELD");
			$te=debug_backtrace();$posx=0;for ($t=0;$t<12;$t++) {if (isset($te[$t]["file"])){if ($t==$posx)echo $pos."_".$te[$posx]["function"]."_util(pre)-hora:".date("h:i:s d/m/Y")."<br>";echo substr($te[$t]["file"].str_repeat(" ",80),0,80).chr(9).$te[$t]["line"].chr(9).$te[$t]["function"]."<br>";}}	
				echo "tablename";
				pre($tablename);
				echo "fieldname";
				pre($fieldname);
				echo "info";
				pre($info);
		}
		$fieldname=uppercase($fieldname);
        $type = $info['type'];
        $null = ($info['null'] === 'NO') ? 'NOT null' : 'null';
        $properties = $info['properties'];
        $default = '';//e cho "========";
        if (!array_key_exists('default', $info)) {
		  $info['default'] = false;
        } elseif (empty($info['default']) && !( $info['default']===0 ||  $info['default']==='0')) {
		  $info['default'] = false;
        }
		//ECHO $null;
	    if (!empty($info['default']) ||  $info['default']===0 ||  $info['default']==='0' ) {
		  $default = 'DEFAULT null';
		  if (strlen($info['default']) > CLX_NUM_ZERO && ($info['default'] === 'NOW()' || $info['default'] === 'CURRENT_DATE'))
			 $default = 'DEFAULT ' . $info['default'];
		  else if (strlen($info['default']) > CLX_NUM_ZERO) {
			 $default = 'DEFAULT ' . $info['default'];
			}
			else if ($info['default']===0 ||  $info['default']==='0')	 {
				$default = 'DEFAULT 0' ;
			}	
			
		}
		if (($this->real_fisic_fields($tablename, $fieldname) || $this->getone('SELECT COUNT(*) FROM '.$tablename)>0) && $fieldname<>'ID' ) {
			if ($info['default']!==0 && empty($default)) {
				$null='';
			}
		}	
		ELSE IF ($fieldname=='ID') {
			$null= ' NOT NULL ';
		}
		$properties2=str_replace('auto_increment','',$properties);
		if (uppercase($default)=='DEFAULT TODAY')
			$default='DEFAULT CURRENT_DATE';
		IF ($this->debugCF) {
			PRE('PRE_ALTER_TABLE ANTES');
		}
	    $this->preAlterTable($tablename,'C' ,$fieldname, $type, $default, $properties, $null);
        IF ($this->debugCF) {
			PRE('PRE_ALTER_TABLE DESPUES');
		}
        //$this->table_struc_temp_name[$tablename]['tmp_table_name']
		IF ($this->real_fisic_fields($tablename,$fieldname)) {
			IF ($this->debugCF) {
				PRE('IF DISYUNTIVA REAL_FISIC_FIELDS AFIRMATIVA');
			}
			$results = $this->executeQuery('COMMIT');
			$sql = 'ALTER TABLE ' . $tablename . ' DROP COLUMN ' . $fieldname ;
			IF ($this->debugCF) {
				PRE ('DROP FIELD IN TABLE');
				PRE($sql);
			}
			$this->temporary_supress_squote_change = true;
			$results = $this->executeQuerySILENT($sql);
		    $results = $this->executeQuery('COMMIT');
		}
		ELSE {
			IF ($this->debugCF) {
				PRE('IF DISYUNTIVA REAL_FISIC_FIELDS FALSE');
			}
		}	
		$sql = 'ALTER TABLE ' . $tablename . ' ADD  ' . quote($fieldname) . ' ' . $type . ' ' . $default . ' ' . $properties2 . ' ' . $null;
    	IF ($this->debugCF) {
			PRE ('ADD FIELD IN TABLE SIGUE EXECUTE');
			PRE($sql);
		}	
		//die;
    	$this->temporary_supress_squote_change = true;
        $results = $this->executeQuery($sql);
        unset($results);
        if(!isset($this->renamed_columns[$tablename])) {
		  $this->renamed_columns[$tablename] = array();
        }
        $this->renamed_columns[$tablename][$fieldname] = '';
        if (stristr($info['properties'], 'auto_increment') !== false) {
		  $this->createAutoIncrement($tablename, $fieldname);
        }
    }

    /**
     * Modify a field on database,
     * WARNING review the data type control type -> blob you
     * need to create and modify blob
     */
    function modifyField($tablename, $oldfieldname, $newfieldname, $info) {
		if ($this->debugCF==true) {
			pre("------->function en ocidatabase modifyField");
					
		}
		$infooriginal=$info;
		$nullfield=$info['null'];
		$MsgReturn='';
		$existe_field_pre_renamed=false;
		$newfieldname=uppercase($newfieldname);
        $old_col_info = $this->fieldInfo($tablename, $oldfieldname);
        if ($oldfieldname <> $newfieldname) {
		  $this->renameField($tablename, $oldfieldname, $newfieldname);
        }
		$null='';
        $type = $info['type'];
     //   $null = 'null';// ($info['null'] === 'NO') ? 'NOT null' : 'null';
        if (!array_key_exists('default', $info))
		  $info['default'] = '';
        $default = '';
        if (!array_key_exists('default', $info)) {
		  $info['default'] = false;
        } elseif (empty($info['default']) && $info['default']!==0 ) {
		  $info['default'] = false;
        }
        if (!empty($info['default'])) {
			IF ($oldfieldname!='ID' ) {
		  $default = 'DEFAULT null';
				if (strlen($info['default']) > CLX_NUM_ZERO && ($info['default'] === 'NOW()' || $info['default'] === 'CURRENT_DATE'))
			 $default = 'DEFAULT ' . $info['default'];
		  else if (strlen($info['default']) > CLX_NUM_ZERO)
			 $default = 'DEFAULT ' . $info['default'];
        }
        }
		
        $properties = ''; //autoincrement is for mysql
        $has_not_null = $this->hasNotNull($tablename, $newfieldname);
        if ((!$has_not_null && $null === 'null') || ($has_not_null && $null === 'NOT null')) {
		  $null = '';
        }
		$this->tmp_fld_name=$tmp_fld_name = $newfieldname.'___TMP';
        if($this->fieldExists($tablename, $tmp_fld_name)||$this->real_fisic_fields($tablename, $tmp_fld_name)) {
		  $this->dropField($tablename, $tmp_fld_name);
			$this->executeQuery('COMMIT');
        }
		else if  ($this->real_fisic_fields($tablename, $tmp_fld_name)) {
			$sql = 'ALTER TABLE ' . $tablename . ' DROP COLUMN ' . $tmp_fld_name;
			$this->temporary_supress_squote_change = true;
			$retorno = $this->executeQuery($sql);
			$this->executeQuery('COMMIT');
		}	
		$existe_field_pre_renamed=$this->real_fisic_fields($tablename, $newfieldname);
		if  ($existe_field_pre_renamed) {
			$this->preAlterTable($tablename,'C' ,$newfieldname, $type, $default, $properties, $null);
		
			$sql = 'ALTER TABLE ' . $tablename . ' RENAME COLUMN ' .$newfieldname . ' TO '. $tmp_fld_name;	
			$this->temporary_supress_squote_change = true;
			$retorno = $this->executeQuery($sql);
			$this->executeQuery('COMMIT');
			$null = '';
		}	
        else {
			$null = '';
		}	
        // Verifica cambio de tipo de datos
		$countnull=0;
	    if ($existe_field_pre_renamed) {
			if ($this->debugCF==true) {
					pre("------->existe_field_pre_renamed");
			}	
			
			if ($nullfield=='NO') {
				$countnull =$this->getone('SELECT COUNT(*) FROM '.$tablename.' WHERE '.$oldfieldname. '___TMP IS NULL ');
			}
			$this->createField($tablename, $newfieldname, $infooriginal);
			if ($this->debugCF==true) {
				pre('oldfieldname');
				pre ($oldfieldname);
				pre('prefieldname');
				pre ($prefieldname);
				pre('tmp_fld_name');
				pre($tmp_fld_name);
			}
			$this->temporary_supress_squote_change = true;
			$col_name_old = $this->cast_to_target_type($newfieldname, $type);
			$sql = 'UPDATE ' . $tablename . ' SET ' . quote($newfieldname) . ' = ' . $tmp_fld_name. ($nullfield=='NO' ? ' WHERE '.$tmp_fld_name.' IS NOT NULL ': '' );
			if ($this->debugCF) {
				PRE('ACTUALIZACION DE LO VIEJO DE defauult');
				pre( $sql);
			}
			$retorno = $this->executeQuerySilent($sql);
			if (!empty($this->errorMsg())) {
				
				$MsgReturn .= note(dict('CHANGE_WITH_DETAILS').'<br>'.
						dict('INFORMATIONSQL').':'.$sql.'<br>'.
						dict('MESSAGEBD').':'.$this->errorMsg().'<br>'.
						dict('OLDFIELDINFORMATION').':'.$tmp_fld_name.'<br>'.
						dict('OLDALLTABLEINFORMATION').':'.$this->table_struc_temp_name[$tablename]['tmp_table_name'].'<br>');
				$GLOBALS['DETAILS_ACUMULATION'].= $MsgReturn;
			}
			else {
				if (uppercase($default)=='DEFAULT TODAY')
					$default='DEFAULT CURRENT_DATE';
				switch (kDatabaseType) {
					case 'oci': // Oracle
						$this->temporary_supress_squote_change = true;
						$sql='SELECT COUNT(1) FROM USER_CONSTRAINTS WHERE TABLE_NAME='.	singlequote($tablename).' AND CONSTRAINT_TYPE= '.singlequote('C').' AND SEARCH_CONDITION_VC='.singlequote(quote($newfieldname).' IS NOT NULL');
						$count1=$this->getone($sql);
						if ($this->debugCF) {
							pre('####count1');
							PRE($count1);
							pre('###nullfield');
							pre("[".$nullfield."]");
						}	
						if ($count1>0 and $nullfield=='NO') {
							$nullfield='';
						}
						else if ($count1>1 and $nullfield=='YES' && empty ($nullfield)) {
							$nullfield=' NULL ';
						}
						else if ($count1<1 and $nullfield=='YES' && empty ($nullfield)) {
							$nullfield=' NULL ';
						}
						else if ($count1<1 and $nullfield=='NO' && empty ($nullfield)) {
							$nullfield=' NOT NULL ';
						}
						$sql = 'ALTER TABLE ' . $tablename . ' MODIFY ' . $newfieldname . ' ' . $type . ' ' . $default. ' ' . ($nullfield=='NO' ? ' NOT NULL ': '' ) . ' ' . $properties ;
				
						break;
					default: 
						$sql = 'ALTER TABLE ' . $tablename . ' MODIFY ' . $newfieldname . ' ' . $type . ' ' . ($nullfield=='NO' ? ' NOT NULL ': '' ) . ' ' . $properties . ' ' . $default;
				
					break;
				
				}
				$this->executeQuerySilent($sql);		
			}	

		}	
        else {
			if ($this->debugCF==true) {
					pre("------->no existe_field_pre_renamed");
			}
        if($old_col_info['type'] === $type) {
		  // Mismo tipo de datos
		  $do_alter_modify = true;
        } else {
		  // Distinto tipo de datos
		  if(uppercase($type) === 'CLOB') {
			 $do_alter_modify = false;
			 /*
 '					 * You can't modify from VARCHAR2 to CLOB but you can achieve your result like this:
			  * 1. Add a new column as CLOB
			  * 2. UPDATE varchar date to CLOB column
			  * 3. DROP VARCHAR column
			  * 4. Rename CLOB column to VARCHAR column name
			  */
		  }
        }
        if($do_alter_modify === true) { 
				if ($this->debugCF==true) {
					pre("------->alter modify");
				}	
				if(trim($old_col_info['default']) !== trim($info['default']) || trim($old_col_info['null']) !== trim($info['null']) || trim($old_col_info['type']) !== trim($info['type']) /* || $old_col_info['extra'] !== $info['properties'] */) {
			 // Hay cambios, hacer ALTER TABLE ... MODIFY
			 $this->temporary_supress_squote_change = true;
			 //$sql = 'ALTER TABLE ' . $tablename . ' MODIFY ' . $newfieldname . ' ' . $type . ' ' . $null . ' ' . $properties . ' ' . $default;
					if ($existe_field_pre_renamed) { 
						$sql = 'ALTER TABLE ' . $tablename . ' CREATE ' . quote($newfieldname) . ' ' . $type . ' ' . $default . ' ' . $properties . ' ' . $null;
					}
					else { 	
			 $sql = 'ALTER TABLE ' . $tablename . ' MODIFY ' . quote($newfieldname) . ' ' . $type . ' ' . $default . ' ' . $properties . ' ' . $null;
					}
			 $retorno = $this->executeQuery($sql);
					
					
			 unset($retorno);
		  }
        } else {
				if ($existe_field) {
		  $this->createField($tablename, $tmp_fld_name, $info);
					
		  // 2. UPDATE varchar date to CLOB column
		  //$sql = 'UPDATE ' . $tablename . ' SET ' . $tmp_fld_name . ' = ' . $newfieldname;
		  $this->temporary_supress_squote_change = true;
		  $col_name_old = $this->cast_to_target_type($newfieldname, $type);
					$sql = 'UPDATE ' . $tablename . ' SET ' . quote($tmp_fld_name) . ' = ' . $col_name_old. quote($tmp_fld_name). ($nullfield=='NO' ? ' WHERE IS NOT NULL ': '' );
					//UPDATE CASI IMPOSIBLE VARIAS CONVERSIONES DARAN ERROR @
					
					$retorno = $this->executeQuerySilent($sql);
		  unset($retorno);
		  // 3. DROP VARCHAR column
		  $this->dropField($tablename, $newfieldname);
		  // 4. Rename CLOB column to VARCHAR column name
		  // 4. Rename CLOB column to VARCHAR column name
		  $this->temp_fieldRename = true;
		  $this->renameField($tablename, $tmp_fld_name, $newfieldname);
        }
				else {
					$this->createField($tablename, $oldfieldname, $info);
				}		
			}
		}
		if($this->real_fisic_fields($tablename, $tmp_fld_name) && empty($MsgReturn)) {
		  $this->dropField($tablename, $tmp_fld_name);
        }
        if (strstr($info['properties'], 'auto_increment') !== false) {
		  $this->createAutoIncrement($tablename, $newfieldname);
        }
		
		
		
    }

    /**
     * Similar to modify Field but Multiple
     *
     * @param string $tablename
     * @param array $fields array('oldfieldname'=>string, 'newfieldname'=>string, 'info'=>array(),'isaddfield'=>bool)
     */
    function reorderFields($tablename, $fields) {

    }

    /**
     * Reorder a field on table after selected field.
     * @param string $tablename
     * @param string $fieldname
     * @param string $after_fieldname
     */
    function reorderField($tablename, $fieldname, $after_fieldname) {

    }

    function hasDefault($tablename, $fieldname) {
        $sql = 'SELECT F.DATA_DEFAULT FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME AND F.COLUMN_NAME = :COLUMN_NAME';
        $binds = array('TABLE_NAME' => $tablename, 'COLUMN_NAME' => $fieldname);
        return (strlen($this->getOne($sql, $binds)) > CLX_NUM_ZERO) ? true : false;
    }

    function hasNotNull($tablename, $fieldname) {
        $sql = 'SELECT F.NULLABLE FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME AND F.COLUMN_NAME = :COLUMN_NAME';
        $binds = array('TABLE_NAME' => $tablename, 'COLUMN_NAME' => $fieldname);
        return ($this->getOne($sql, $binds) === 'Y') ? false : true;
    }

    /**
     * Rename a field only in oci class.  private method
     */
    function renameField($tablename, $oldfieldname, $newfieldname) {
        $this->preAlterTable($tablename,'R',$oldfieldname,$newfieldname);
        if($this->temp_fieldRename === false) {
		  if(!isset($this->renamed_columns[$tablename])) {
			 $this->renamed_columns[$tablename] = array();
		  }
		  $this->renamed_columns[$tablename][$newfieldname] = $oldfieldname;
        } else {
		  $this->temp_fieldRename = false;
        }
        $this->temporary_supress_squote_change = true;
        $sql = 'ALTER TABLE ' . $tablename . ' RENAME COLUMN ' . quote($oldfieldname) . ' TO ' . quote($newfieldname);
        $results = $this->executeQuery($sql);
        unset($results);
    }

    /**
     * Retrives information regarding a field and returns an
     * array with field information Verify auto increment on record extra
     */
    function fieldInfo($tablename, $fieldname) {
        $oraField = array();
        $record = false;
        $sql = 'SELECT F.COLUMN_NAME, F.DATA_TYPE, F.DATA_LENGTH, F.CHAR_LENGTH, F.DATA_PRECISION, F.DATA_SCALE, F.NULLABLE, F.DATA_DEFAULT FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME AND F.COLUMN_NAME = :COLUMN_NAME';
        $binds = array('TABLE_NAME' => $tablename, 'COLUMN_NAME' => $fieldname);
        $oraField = $this->getOneRecord($sql, $binds);
        // Setting field information
        $record['field'] = $oraField['COLUMN_NAME'];
        $record['default'] = $oraField['DATA_DEFAULT'];
        if(is_null($record['default'])) {
		  $record['default'] = 'null';
        }
        $record['null'] = ($oraField['NULLABLE'] === 'Y') ? 'YES' : 'NO';
        switch ($oraField['DATA_TYPE']) {
		  case 'NVARCHAR2':
			 $record['type'] = $oraField['DATA_TYPE'] . '(' . $oraField['CHAR_LENGTH'] . ')';
			 break;
		  case 'VARCHAR2':
			 $record['type'] = $oraField['DATA_TYPE'] . '(' . $oraField['CHAR_LENGTH'] . ')';
			 break;
		  case 'NUMBER':
			 if ($oraField['DATA_SCALE'] > CLX_NUM_ZERO) {
			     $record['type'] = $oraField['DATA_TYPE'] . '(' . $oraField['DATA_PRECISION'] . ',' . $oraField['DATA_SCALE'] . ')';
			 } else {
			     $record['type'] = $oraField['DATA_TYPE'] . '(' . $oraField['DATA_PRECISION'] . ')';
			 }
			 break;
		  default:
			 $record['type'] = $oraField['DATA_TYPE'];
			 break;
        }
        if ($this->existsAutoIncrement($tablename, $fieldname)) {
		  $record['extra'] = 'auto_increment';
        } else {
		  $record['extra'] = '';
        }
	//	pr e($record);
        return $record;
    }

    /**
     * Performs testing of current index to see status
     *
     * @param string $tablename
     * @param string $fieldname
     * @param array $fieldinfo
     * @param array $msg
     * @param int $fix
     */
    function checkFieldIndex($tablename, $indexname, &$indexinfo, &$msg, $fix, $dump = false) {
		if ($this->debugCI==true) {
			pre ('########inicio de checkfieldindex($indexinfo)');
			pre($indexinfo);	
		}	
        $indexes = $this->getTableIndexes($tablename, $indexname);
		if ($this->debugCI==true) {
			pre ('########despues de gettableIndexes(indexes)');
			pre($indexes);
		}
        if (empty($indexes[$indexname]))
		  $indexes[$indexname] = array('index' => '', 'fieldname' => '');
		if ($this->debugCI==true) {
			pre('indexname');
			pre($indexname);
			pre('$indexes[$indexname]["index"]');
			pre($indexes[$indexname]['index']);
			pre('$indexinfo["index"]');
			pre($indexinfo['index'] );
		
		}		
        if ($indexes[$indexname]['index'] <> $indexinfo['index'] || $indexes[$indexname]['fieldname'] <> $indexinfo['fieldname']) {
			$x=explode(',',$indexes[$indexname]['fieldname']);
			$y=explode(',',$indexinfo['fieldname']);
			if ($this->debugCI==true) {
				pre("indexes");
				pre($x);
				pre("indexinfo");
				pre($y);
			}	
			$noverificar=false;//$verificar=/oracle cambia el orden
			if (($maxX=count($x))==count($y)){
				$noverificar=true;
				$contar=0;
				for ($t=0;$t<$maxX;$t++){
					if (in_array ($x[$t],$y)){
						$contar++;
				}
				}
				$noverificar=$contar==$maxX;
			}
			if (!$noverificar) {
				$error = array('tablename' => $tablename,
					'fieldname' => $indexname . '(' . $indexinfo['fieldname'] . ')',
					'error' => dict('GEN_ERR_FIELD_WRONG_INDEX'),
					'comments' => dict('GEN_LBL_SET') . ': ' . 
						($indexes[$indexname]['index'] <> $indexinfo['index'] ? $indexes[$indexname]['index'] . ' -> ' . $indexinfo['index'] . ' | ' : '') .
						($indexes[$indexname]['fieldname'] <> $indexinfo['fieldname'] ? $indexes[$indexname]['fieldname'] . ' -> ' . $indexinfo['fieldname'] : '')
				);
				if ($this->debugCI==true) {
					pre("error de indice IN checkFieldIndex");
					pre($error);
				}	
		
				if (!empty($fix)) {
					switch ($indexinfo['index']) {
						case 'UNI':
						case 'COM':
							if ($this->debugCI==true) {
								pre('[[[[llamando createUniqueIndex]]]]]');	
							}
							$this->createUniqueIndex($tablename, $indexinfo['fieldname'], $indexname);
							break;
						case 'MUL':
							if ($this->debugCI==true) {
								pre('[[[[llamando createIndex]]]]]');	
							}
							$this->createIndex($tablename, $indexinfo['fieldname'], $indexname);
							break;
						case 'PRI':
							if ($this->debugCI==true) {
								pre('[[[[llamando createPrimaryIndex]]]]]');	
							}
							$this->createPrimaryIndex($tablename, $indexinfo['fieldname'], $indexname);
							break;
						default:
							if ($this->debugCI==true) {
								pre('[[[[llamando a ningun valor para Index]]]]]');	
							}
							break;
					}
					$error['comments'] .= ' ' . dict('GEN_LBL_FIXED');
				}
				else
					echo "no verificar";
				$msg[] = $error;
			}
		}
		else {
			if ($this->debugCI==true) {
				pre('condiciones de indexes son correctas, no hay diferencia');
					
			
		}
	}
    }

    /**
     * Verify table and it structure
     */ 
    function checkTable($tabledef, $fix, &$msg, $fields_only = false, $indexes_only = false, $dump = false,$audit=false,$analize=true) {
        if ($this->debugCF==true or $this->debugCI==true){echo '------------##>inicio de checktable';}
		if ($audit==true)
			$this->audit=true;		
        if (empty($tabledef['name'])) {
		  $msg[] = array('tablename' => '',
			 'fieldname' => '',
			 'error' => dict('GEN_ERR_EMPTY_TABLENAME')." IDCODE:".$tabledef,
			 'comments' => '');
		  return;
        }
        if (!$this->tableExists($tabledef['name']) && !$this->viewExists($tabledef['name'])) {
		  $error = array('tablename' => $tabledef['name'],
			 'fieldname' => 'DBTB: ' . $tabledef['idname'] . ' (' . $tabledef['idcode'] . ')',
			 'error' => dict('GEN_ERR_TABLE_DOES_NOT_EXISTS'),
			 'comments' => '');
		  if (!empty($fix)) {
			 if ($tabledef['type'] === 'VIEW') {
			     $this->createView($tabledef['name'], $tabledef['viewsql']);
			 } else {
			     //$this->createTable($tabledef['name'], $tabledef['type'], $tabledef, false);
			     $this->createTable($tabledef['name'], $tabledef['type'], $tabledef['fields'], false);
			 }
			 $error['comments'] = dict('GEN_LBL_FIXED');
		  }
		  $msg[] = $error;
		  if ($tabledef['type'] === 'VIEW') {//nothing else to do if view
			 return;
		  }
		  //checking indexes only if table was created on fix
		  if (!empty($fix)) {
			 if (!empty($indexes_only) || (empty($indexes_only) && empty($fields_only))) {
			     foreach ($tabledef['indexes'] as $indexname => $indexinfo) {
			         $this->checkFieldIndex($tabledef['name'], $indexname, $indexinfo, $msg, $fix, $dump);
			     }
			 }
		  }
        } else {
		  if ($tabledef['type'] === 'VIEW') {//nothing else to do if view
			 return;
		  }
			
		    $tableinfo = $this->getTableInfo($tabledef['name']);
			if ($this->debugCF==true){
				echo "tableINFO OCIDATABASE CHECKTABLE";
				pre($tableinfo);
		
			}
		  if ($tableinfo['type'] <> 'default' && $tableinfo['type'] <> $tabledef['type']) {
			 if ($this->debugCF==true){
				echo "tableINFO OCIDATABASE CHECKTABLE CONDICION 1<BR>";
				}
			 $error = array('tablename' => $tabledef['name'],
			     'fieldname' => 'DBTB: ' . $tabledef['idname'] . ' (' . $tabledef['idcode'] . ')',
			     'error' => dict('GEN_ERR_TABLE_WRONG_TYPE') . ' Req: ' . $tableinfo['type'] . '  Cur:' . $tabledef['type'],
			     'comments' => '');
			 if (!empty($fix)) {
			     $this->changeTableType($tabledef['name'], $tabledef['type']);
			     $error['comments'] = dict('GEN_LBL_FIXED');
					
			 }
			 $msg[] = $error;
		  }
			
		    $this->check_data_and_convert_fields_to_upper('',$tabledef['name']);
			
		  if (!empty($fields_only) || (empty($indexes_only) && empty($fields_only))) {
			 if ($this->debugCF==true){
					echo "tableINFO OCIDATABASE CHECKTABLE CONDICION 2 linea apr. 1325<BR>";
				}
			 foreach ($tabledef['fields'] as $fieldname => $fieldinfo) {
					if ($this->debugCF==true){
						pre('--->fields en Ocidatabase '.$fieldname);
						pre($fieldinfo);
					}
			     $this->checkField($tabledef['name'], $fieldname, $fieldinfo, $msg, $fix, $dump);
			 }
					
		  }
		  //checking indexes
			if ($this->debugCF==true or $this->debugCI==true){
				pre('%%%%previo a indexes');	
			}
		  if (!empty($indexes_only) || (empty($indexes_only) && empty($fields_only))) {
				if ($this->debugCF==true or $this->debugCI==true){
					echo "--->tableINFO OCIDATABASE CHECKTABLE CONDICION 3<BR>";
					pre('%%%%indexname');
					pre($indexinfo);
					pre('%%%%conjunto index');
					pre($tabledef['indexes']);
				}	
			 foreach ($tabledef['indexes'] as $indexname => $indexinfo) {
			     //$this->checkFieldIndex($tabledef['name'], $indexname, $indexinfo, $msg, $fix);
			     if ($this->debugCI==true) {
						pre('%%%%revisando index');	
						pre($indexinfo);
					}	
			     $this->checkFieldIndex($tabledef['name'], $indexname, $indexinfo, $msg, $fix, $dump);
			 }
		  }
		  $pat_err_msg = $this->postAlterTable($tabledef['name']);
		  if($pat_err_msg !== '') {
				if ($this->debugCF==true){
				echo "tableINFO OCIDATABASE CHECKTABLE CONDICION 4<BR>";
				}
			 // Deja linea en blanco de por medio para destacar error
			 $msg[] = array('tablename' => '&nbsp;', 'fieldname' => '&nbsp;', 
			     'error' => '<center><b>*** '.uppercase(dict('GEN_LBL_ERROR')).' ***</b></center>', 
			     'comments' => '&nbsp;');
			 // Reporta error en la parte final del Audit
			 $msg[] = array('tablename' => $tabledef['name'],
			     'fieldname' => 'DBTB: ' . $tabledef['idname'] . ' (' . $tabledef['idcode'] . ')',
			     'error' => $pat_err_msg,
			     'comments' => dict('GEN_LBL_REVERSE'));
		  }
			
			$this->checkTableSequence($tabledef['name']);
				$this->analizetable($tabledef['name']);
			return;
        }
    }
    function analizetable($tablename) {
		$reg=$this->getone('select count(1) from '.$tablename);
	//	echo 'aq:',$reg,"<br>";
        
		if ($reg<2*1000*1000) {
			$rs = $this->executeQuery('ANALYZE TABLE '.uppercase($tablename).' COMPUTE STATISTICS');
		}
		else  { 
			$porc= (int) (3000/pow($reg, 1/4)+1 ); ##segun tamaño tabla disminuye porcentaje 
		//	echo 'aca:',$porc;
			$rs = $this->executeQuery('ANALYZE TABLE  '.uppercase($tablename).' ESTIMATE STATISTICS FOR TABLE SAMPLE '.$porc.' PERCENT');
        }
    }

    /**
     * Returns true is a field is found on table or false on failure.
     */
    function fieldExists($tablename, $fieldname) {
		//aumento de velocidad del importador buscando lo mismo en cada registro en base de datos
		$fetched = false;
		if (kApcTtl_24>0) $returno = apc_fetchFct($findVar='field_'.$tablename.'_'.$fieldname.'_' , $fetched);
		if (empty($fetched)) {
			$sql = 'SELECT COUNT(F.COLUMN_NAME) FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME AND F.COLUMN_NAME = :COLUMN_NAME';
        //$binds = array('TABLE_NAME' => $tablename, 'COLUMN_NAME' => $fieldname);
       
			$binds = array('TABLE_NAME' => uppercase($tablename), 'COLUMN_NAME' => $fieldname);
		//	p re($binds);
		//	p re($this->getOne($sql, $binds));
			$returno=$this->getOne($sql, $binds) > CLX_NUM_ZERO;
			if (kApcTtl_24>0 ){
				apc_addFct($findVar,$returno,kApcTtl_24);
			}
		}
		
        return $returno;
    }

    function tableFieldTypes($tablename) {
        $return = array();
        $sql = 'SELECT ' . 'F.COLUMN_NAME, ' . 'F.DATA_TYPE, ' . 'F.NULLABLE ' . 'FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME';
        //$binds = array('TABLE_NAME' => $tablename);
        $binds = array('TABLE_NAME' => uppercase($tablename));
        $rs = $this->executeQuery($sql, $binds);
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
		  $return[$result['COLUMN_NAME']] = array('TYPE' => $result['DATA_TYPE'], 'null' => ($result['NULLABLE'] === 'Y') ? CLX_NUM_ONE : CLX_NUM_ZERO);
        }
		$this->freeResult($rs);		
        return $return;
    }

    /**
     * Return resultset from query of fields on table,
     * Apparently not in use any case reviwe on field info
     */
    function fieldList($tablename, $fieldname = '') {
        $sql = 'SELECT ' . 'F.COLUMN_NAME, ' . 'F.DATA_TYPE, ' . 'F.DATA_LENGTH, ' . 'F.DATA_SCALE, ' . 'F.NULLABLE ' . 'FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME';
        //$binds = array('TABLE_NAME' => $tablename);
        $binds = array('TABLE_NAME' => uppercase($tablename));
        if (strlen($fieldname) > CLX_NUM_ZERO) {
		  $sql .= ' AND F.COLUMN_NAME = :COLUMN_NAME';
		  $binds['COLUMN_NAME'] = $fieldname;
        }
        return $this->executeQuery($sql, $binds);
    }

    /**
     * Create a table index
     */
    function createUniqueIndex($tablename, $fieldname, $indexname) {
        //Prepares multiple fields unique index
        if (is_array($fieldname)) {
		  //$fieldname = implode(',', $fieldname);
		  $fieldname = implode(quote(','), $fieldname);
        } else {
		  $fieldname = str_replace(',', quote(','), $fieldname);
        }
        $this->dropIndex($tablename, $fieldname, $indexname);
        $sql = 'CREATE UNIQUE INDEX ' . quote($indexname) . ' ON ' . quote($tablename) . ' (' . quote($fieldname) . ') ';
        $this->temporary_supress_squote_change = true;
        $results = $this->executeQuery($sql);
        unset($results);
    }

    /**
     * Create a table index
     */
    function createIndex($tablename, $fieldname, $indexname) {
		if ($this->debugCI==true) {
			pre('########inicio create index createIndex');
		}	
        $this->dropIndex($tablename, $fieldname, $indexname);
        if (is_array($fieldname)) {
		  //$fieldname = implode(',', $fieldname);
		  $fieldname = implode(quote(','), $fieldname);
        } else {
		  $fieldname = str_replace(',', quote(','), $fieldname);
        }
        $sql = 'CREATE INDEX ' . quote($indexname) . ' ON ' . quote($tablename) . ' (' . quote($fieldname) . ')';
        $this->temporary_supress_squote_change = true;
        $results = $this->executeQuery($sql);
        unset($results);
    }

    /**
     * Creates a primary index
     *
     * @param string $tablename
     * @param mixed $fieldname
     * @param string $indexname
     */
    function createPrimaryIndex($tablename, $fieldname, $indexname) {
		if ($this->debugCI==true) {
			pre('########inicio create index createPrimaryindex');
		
		}	
        if (is_array($fieldname)) {//	 multiple fields
		  //$fieldname = implode(',', $fieldname);
		  $fieldname = implode(quote(','), $fieldname);
        } else {
		  $fieldname = str_replace(',', quote(','), $fieldname);
        }
        $this->dropPrimaryIndex($tablename, $fieldname, $indexname);
        $sql = 'ALTER TABLE ' . $tablename . ' ADD CONSTRAINT ' . quote($indexname) . ' PRIMARY KEY (' . quote($fieldname) . ')';
        $this->temporary_supress_squote_change = true;
        $retorno = $this->executeQuery($sql);
        unset($retorno);
    }

    /**
     * check if table field exists
     *
     * @param string $tablename
     * @param mixed $fieldname
     * @param string $indexname
     * @return boolean
     */
    function hasPrimaryIndex($tablename, $fieldname, $indexname) {
		if ($this->debugCI==true) {	
			pre('########inicio hasPrimaryIndex');
		}	
        $name = $this->getPrimaryIndexName($tablename);
        return (!empty($name)) ? true : false;
    }

    function getPrimaryIndexName($tablename) {
        if ($this->debugCI==true) {	
			pre('########inicio getPrimaryIndexName');
		}
        //$sql = 'SELECT INDEX_NAME FROM USER_CONSTRAINTS C WHERE TABLE_NAME = ' . singleQuote($tablename) . ' AND CONSTRAINT_TYPE = ' . singleQuote('P') . ' AND ROWNUM = 1';
        $sql = 'SELECT INDEX_NAME FROM USER_CONSTRAINTS C WHERE TABLE_NAME = ' . singleQuote(uppercase($tablename)) . ' AND CONSTRAINT_TYPE = ' . singleQuote('P') . ' AND ROWNUM = 1';
        return $this->getOne($sql);
    }

    /**
     * Drops a primary index key
     *
     * @param unknown_type $tablename
     * @param unknown_type $fieldname
     * @param unknown_type $indexname
     */
    function dropPrimaryIndex($tablename, $fieldname, $indexname) {
        $indexname = $this->getPrimaryIndexName($tablename);
        if (!empty($indexname)) {
		  $sql = 'ALTER TABLE ' . $tablename . ' DROP CONSTRAINT ' . $indexname;
		  $retorno = $this->executeQuery($sql);
		  unset($retorno);
        }
    }

    /**
     * Drop table index
     */
    function dropIndex($tablename, $fieldname, $indexname) {
		
        if ($this->indexExists($indexname)) {
		  $sql = 'DROP INDEX ' . $indexname;
		  $results = $this->executeQuery($sql);
		  unset($results);
        }
    }

    function indexExists($indexname) {
		if ($this->debugCI==true) {	
			pre('########inicio IndexExists');
		}
        $sql = 'SELECT COUNT(0) FROM USER_INDEXES WHERE INDEX_NAME = ' . singleQuote($indexname);
        return $this->getOne($sql) <> CLX_NUM_ZERO;
    }

    /**
     * To test this has changed on clx database to add indexname
     */
    function hasIndex($tablename, $fieldname, $indexname) {
        $sql = 'SELECT COUNT(0)  FROM USER_INDEXES  WHERE INDEX_NAME = ' . singleQuote($indexname);
        return $this->getOne($sql) <> CLX_NUM_ZERO;
    }
	function getTablebyIndexes($tablename, $type='UNI') {
		$return = array();
		$binds = array('TABLENAME' => uppercase($tablename));
        $sql = 'SELECT IC.COLUMN_NAME, (SELECT NVL(' . singleQuote('PRI') . ',' . singleQuote('') . ') FROM USER_CONSTRAINTS C WHERE C.INDEX_NAME = I.INDEX_NAME AND C.CONSTRAINT_TYPE = ' . "'P'" . ') PRIM, CASE WHEN I.UNIQUENESS= ' . singleQuote('UNIQUE') . ' THEN ' . singleQuote('UNI') . ' ELSE ' . singleQuote('MUL') . ' END AS INDEX_TYPE, I.INDEX_NAME FROM USER_INDEXES I, USER_IND_COLUMNS IC WHERE IC.INDEX_NAME = I.INDEX_NAME AND I.TABLE_NAME = IC.TABLE_NAME AND IC.TABLE_NAME = :TABLENAME';
        $indexes = $this->executeQuery($sql, $binds);
        while (($index = $this->getNextRecord($indexes, ASSOCIATIVE))) {
			if ($type==$index['INDEX_TYPE'] ) {
				if (empty($return[$index['INDEX_NAME']]['fieldname'])) {
					$return[$index['INDEX_NAME']]['fieldname'] = $index['COLUMN_NAME'];
				} else {
					$return[$index['INDEX_NAME']]['fieldname'] .= ',' . $index['COLUMN_NAME'];
				}
			}	
			if ($index['PRIM'] === 'PRI' ) {
				if ($index['PRIM']==$type) {
					$return[$index['INDEX_NAME']]['index'] = 'PRI';
			}
		  } /*else {
			 $return[$index['INDEX_NAME']]['index'] = $index['INDEX_TYPE'];
		  }*/
		}	
		$this->freeResult($indixes);		
        return $return;
	}
    /**
     * Returns array containing current table indexes
     * Verify this in oracle
     */
    function getTableIndexes($tablename, $indexname) {
		if ($this->debugCI==true) {	
			pre('########inicio getTableIndexes');
		}
        $return = array();
        $binds = array('TABLENAME' => uppercase($tablename));
        $sql = 'SELECT IC.COLUMN_NAME, (SELECT NVL(' . singleQuote('PRI') . ',' . singleQuote('') . ') FROM USER_CONSTRAINTS C WHERE C.INDEX_NAME = I.INDEX_NAME AND C.CONSTRAINT_TYPE = ' . "'P'" . ') PRIM, CASE WHEN I.UNIQUENESS= ' . singleQuote('UNIQUE') . ' THEN ' . singleQuote('UNI') . ' ELSE ' . singleQuote('MUL') . ' END AS INDEX_TYPE, I.INDEX_NAME FROM USER_INDEXES I, USER_IND_COLUMNS IC WHERE IC.INDEX_NAME = I.INDEX_NAME AND I.TABLE_NAME = IC.TABLE_NAME AND IC.TABLE_NAME = :TABLENAME';
        if (strlen($indexname) > CLX_NUM_ZERO) {
		  $binds['INDEXNAME'] = $indexname;
		  $sql .=' AND I.INDEX_NAME = :INDEXNAME';
        }
        $indexes = $this->executeQuery($sql, $binds);
        while (($index = $this->getNextRecord($indexes, ASSOCIATIVE))) {
		  if (empty($return[$index['INDEX_NAME']]['fieldname'])) {
			 $return[$index['INDEX_NAME']]['fieldname'] = $index['COLUMN_NAME'];
		  } else {
			 $return[$index['INDEX_NAME']]['fieldname'] .= ',' . $index['COLUMN_NAME'];
		  }
		  if ($index['PRIM'] === 'PRI') {
			 $return[$index['INDEX_NAME']]['index'] = 'PRI';
		  } else {
			 $return[$index['INDEX_NAME']]['index'] = $index['INDEX_TYPE'];
		  }
        }
		$this->freeResult($indixes);		
        return $return;
    }

    /**
     * Remove a field from table
     */
    function dropField($tablename, $fieldname) {
        $this->preAlterTable($tablename,'D',$fieldname);
		if ($this->real_fisic_fields($tablename, $fieldname)) {
					
        $sql = 'ALTER TABLE ' . $tablename . ' DROP COLUMN ' . quote($fieldname);
        $this->temporary_supress_squote_change = true;
        //$results = $this->executeQuery(uppercase($sql));
        $results = $this->executeQuery($sql);
        unset($results);
    }
	}
    
    /**
     * Insert new record in database table.
     * @param: string table, array record
     */
    function createRecordVoid($table, $record, $silent) {
        ClxFreeReturn($this->createRecord($table, $record, $silent));
    }

	function createRecord($table, $record, $silent = false,$extras=array('type'=>"VALUES",'returnid'=>true,'extracriteria'=>'','selectexistsid'=>'')) {
        $fields = '';
        $values = '';
        $comma = '';
        if (!isset($this->numerictype[$table])) {
			$this->numerictype[$table]=$this->getFieldstypeNumeric($table);
		}
		$record_keys=array_keys($record);
		foreach ($record_keys as $field => $value) {
			if (uppercase($value)!=$value) {
				$ant=$record[$value];
				unset($record[$value]);
				$record[uppercase($value)]=$ant;
				
			}
		}	
		//unset($record['caption']);
		
		
        foreach ($record as $field => $value) {
			$this->replaceInvalidValues($record[$field]);
		  //$fields .= $comma . $field;
		  //$values .= $comma . ':' . $field;
		  $fields .= $comma . quote($field);
			if (is_array($value)) { // sample $record['RUNID']=array('ESPECIAL_FIELD'=>' (SELECT MAX(RUNID)+1 FROM RCTA_TRN_MATCH_EXECS) ');
				$values.= $comma .$value['ESPECIAL_FIELD'];
		  } else {
			  $values .= $comma . ':' . quote(uppercase($field));
		  }
			
		  $comma = ', ';
        }
        /* Creating binding variable for id to emulate first insert id used in mysql */
		
        if ($this->fieldExists($table, 'ID')) {
		  $bindid = ' RETURNING ID INTO :ID';
		  if (!array_key_exists('ID', $record)) {
			 $record['ID'] = CLX_INIT_ZERO;
				//$fields .= $comma . quote('ID');
				//$values .= $comma .  ':'. quote('ID');
			 //$record['ID'] = null; // 2018-07-26 CR : Para que asigne de manera efectiva el proximo en la Secuence
		  }
        } else {
		  $bindid = '';
        }
		
        // Continuing insert
		$insert_id=-1;
        $this->temporary_supress_squote_change = true;
		if ($extras['type']!='VALUES') {
			if ($extras['returnid'] && $extras['selectexistsid']!=='')
				$valid_insert=empty($this->getOne($extras['selectexistsid']));
			else {
				$valid_insert=true;
			}
		}
		else {
			$valid_insert=true;
		}
		
		if ($valid_insert) {
			if ($extras['returnid'] || $extras['extracriteria']=='' ) {
				$sql = 'INSERT INTO ' . $table . ' (' . $fields . ' ) VALUES (' . $values . ') ' . $bindid;
			}
			else  {
				unset($record['ID']);
				$sql = 'INSERT INTO ' . $table . ' (' . $fields . ') SELECT ' . $values . ' FROM DUAL '. ((empty($extras['extracriteria']))?'': ' WHERE '.$extras['extracriteria'] );
			}
			//PRE($record);
			if ( !empty($_SESSION["_dEBUG"]) and $_SESSION["_dEBUG"]==99) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,$sql.print_r($record,true).chr(13).chr(10));fclose($tta);}
			if ($silent === true) {
				$results = $this->executeQuerySilent($sql, $record);
			} else {
				$results = $this->executeQuery($sql, $record); //sql, binds
			}
			
			
			$insert_id = $this->createdId();
			$this->freeResult($this->stmt);
			unset($results);
		}
			
        return $insert_id;
    }

	
	function MulticreateRecord($table, $records, $silent = false) {
        if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite.'/'.kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- inicio-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"Multicreate:".$table.chr(13).chr(10));fclose($tta);}

		$sql='';
		$cont=0;
		$totrecords=array();
		
		$record_keys=array_keys($record);
		foreach ($record_keys as $field => $value) {
			if (uppercase($value)!=$value) {
				$ant=$record[$value];
				unset($record[$value]);
				$record[uppercase($value)]=$ant;
				
			}
		}	
	
		
		if (!isset($this->numerictype[$table])) {
			$this->numerictype[$table]=$this->getFieldstypeNumeric($table);
		}	
		/* Creating binding variable for id to emulate first insert id used in mysql */
		$existingID=false;
        if ($this->fieldExists($table, 'ID')) {
			$existingID=true;	
		  
			 //$record['ID'] = null; // 2018-07-26 CR : Para que asigne de manera efectiva el proximo en la Secuence
        }
		foreach($records as &$record) {
			if ($existingID===true) {
				$record['ID'] = CLX_INIT_ZERO;
			}	

			$cont++;
			$fields = '';
			$values = '';
			$comma = '';
			$changetype=array();
			foreach ($record as $field => $value) {
				$this->replaceInvalidValues($record[$field]);
				//$fields .= $comma . $field;
				//$values .= $comma . ':' . $field;
				$fields .= $comma . quote($field);
				
				
				
				if (is_array($value)) { // sample $record['RUNID']=array('ESPECIAL_FIELD'=>' (SELECT MAX(RUNID)+1 FROM RCTA_TRN_MATCH_EXECS) ');
					$values.= $comma .$value['ESPECIAL_FIELD'];
				} else if ($this->numerictype[$table][$field]!='DECN') {
					$values .= $comma . ':' . quote($field.'_'.$cont);
				}
				
				else {
					if (lowercase($value) =='null') {
						$value=0;
					}
					$this->numerictype[$table][$field.'_CHANGE']='CHAR';
					$this->numerictype[$table][$field.'_MASK_'.$cont]='CHAR';
				
					$values .= $comma . 'TO_NUMBER(:' . quote($field.'_'.$cont).',:'.quote($field.'_MASK_'.$cont).')';	
					$totrecords[$field.'_MASK_'.$cont]=preg_replace('/[-0123456789]/','9',$value);
				
				}
				$totrecords[$field.'_'.$cont]=$value;	
				
				$comma = ', ';
			}
			/* Creating binding variable for id to emulate first insert id used in mysql */
			$this->temporary_supress_squote_change = true;
			if ($extras['returnid'] || $extras['extracriteria']=='' ) {
				$sql .= ' INTO ' . $table . ' (' . $fields . ' ) VALUES (' . $values . ') ' ;
			}
				
		}
		$sql = 'INSERT ALL '.$sql.' SELECT 1 FROM DUAL';
		if ($silent === true) {
			$results = $this->executeQuerySilent($sql, $totrecords);
		} else {
			$results = $this->executeQuery($sql, $totrecords); //sql, binds
		}
		$this->freeResult($this->stmt);
		unset($results);
		
        if ( !empty($_SESSION["_dEBUG"]) ) { $tta=fopen(kPathToWebSite."/".kPathToTemp."archivo.txt","a+");	fwrite($tta,"1- fin-hora:".date("h:i:s d/m/Y").chr(13).chr(10));fwrite($tta,"Multicreate:".$table.chr(13).chr(10));fclose($tta);}

	}

	
	
    /**
     * Returns the ID number of Last Insert
     */
    function createdId() {
        return $this->lastId;
    }

    /**
     * Update record on database
     * @param: string table, string [, criteria 1]...[, criteria n]
     * Arguments for this function: table, $fields. array $field has
     * key as the fieldname and the value is the field's contents,
     * You can specify the criteria of the update starting in argument 3.
     */
    function updateRecord($table, $record, $criteria = '') {
        $comma = '';
        $sqlfields = '';
		$record_keys=array_keys($record);
		foreach ($record_keys as $field => $value) {
			if (uppercase($value)!=$value) {
				$ant=$record[$value];
				unset($record[$value]);
				$record[uppercase($value)]=$ant;
				
			}
		}	
	
  
		foreach ($record as $field => $value) {
	
		  $this->replaceInvalidValues($record[$field]);
		  
		
		  $sqlfields .= $comma . ' ' . quote($field) . '=:' . quote($field);
		  $comma = ', ';
        }	
        $this->temporary_supress_squote_change = true;
        $sql = 'UPDATE ' . $table . ' SET ' . $sqlfields . ' WHERE 1=1 ';
        if (strlen($criteria) > CLX_NUM_ZERO) {
		  $sql .= ' AND  ' . $criteria;
        }
		 $result = $this->executeQuery($sql, $record);
        $this->freeResult($this->stmt);

       
        return $result;
    }

    function getRecord($table, $critaria = '') {
        $sql = 'SELECT * FROM ' . $table;
        if (strlen($critaria) > CLX_NUM_ZERO) {
		  $sql .= ' WHERE ' . $critaria;
		  if (stripos($sql, 'ORDER BY') !== false) {
			 $sql = str_replace('ORDER BY', ' AND ROWNUM<=1 ORDER BY ', $sql);
		  } else {
			 $sql .= ' AND ROWNUM<=1 ';
		  }
        } else {
		  $sql .= ' WHERE ROWNUM<=1';
        }
        $rs = $this->executeQuery($sql);
        $r = $this->getNextRecord($rs, ASSOCIATIVE);
        $this->freeResult($rs);
        return $r;
    }

    /**
     * Perform sql and return selected field value from result set
     * @param:string table, string fieldname, string [, criteria 1]...[, criteria n]
     */
    function getField($table, $field) {
        // arguments for this function:
        // table, field [, criteria 1]...[, criteria n]
        $field = uppercase($field);
        $sql = 'SELECT ' . $field . ' FROM ' . $table;
        $qty = func_num_args();
        if ($qty > CLX_NUM_TWO === true) {
		  $sql .= ' WHERE ';
		  $boolean = '';
		  for ($i = CLX_NUM_TWO; $i < $qty; $i++) {
			 $sql .= $boolean . func_get_arg($i);
			 $boolean = ' AND ';
		  }
		  if (stripos($sql, 'ORDER BY') !== false) {
			 $sql = str_replace('ORDER BY', ' AND ROWNUM<=1 ORDER BY ', $sql);
		  } else {
			 $sql .= ' AND ROWNUM<=1 ';
		  }
        } else {
		  $sql .= ' WHERE ROWNUM<=1';
        }
        $result = $this->executeQuery($sql);
        $value = $this->getNextRecord($result, NUMERIC);
		$this->freeResult($result);		
        return ($value != false) ? $value[CLX_NUM_ZERO] : false;
    }

    /**
     * Perform sql and return selected fields value from result set
     * @param:string table, string fieldname, string [, criteria 1]...[, criteria n]
     */
    function getFields($table, $field) {
        // arguments for this function:
        // table, field [, criteria 1]...[, criteria n]
        $sql = 'SELECT ' . $field . ' FROM ' . $table;
        $qty = func_num_args();
        if ($qty > CLX_NUM_TWO === true) {
		  $sql .= ' WHERE ';
		  $boolean = '';
		  for ($i = CLX_NUM_TWO; $i < $qty; $i++) {
			 $sql .= $boolean . func_get_arg($i);
			 $boolean = ' AND ';
		  }
		  if (stripos($sql, 'ORDER BY') !== false) {
			 $sql = str_replace('ORDER BY', ' AND ROWNUM<=1 ORDER BY ', $sql);
		  } else {
			 $sql .= ' AND ROWNUM<=1 ';
		  }
        } else {
		  $sql .= ' WHERE ROWNUM<=1';
        }
        $result = $this->executeQuery($sql);
        $value = $this->getNextRecord($result, ASSOCIATIVE);
		$this->freeResult($result);		
        return $value;
    }

    /**
     * Select records from database and return resource of results.
     * @param: string table, string [, criteria 1]...[, criteria n]
     */
    function getRecords($table) {
        $qty = func_num_args();
        $sql = 'SELECT * FROM ' . uppercase($table);
        if ($qty > CLX_NUM_ONE === true) {
		  $boolean = ' WHERE ';
		  for ($i = CLX_NUM_ONE; $i < $qty; $i++) {
			 $c = func_get_arg($i);
			 if (!empty($c)) {
			     $sql .= $boolean . $c;
			     $boolean = ' AND ';
			 }
		  }
        }
        if($this->ci_search === false) {
		  $result = $this->executeQuery($sql);
        } else {
		  $result = $this->executeQueryCi($sql);
        }
        return $result;
    }

    /**
     * Fetch and array containing the variables for
     * the next record on result set
     */
    function getNextRecord(&$id, $type = ASSOCIATIVE) {
		$XX= oci_fetch_array($id, ($type == ASSOCIATIVE) ? (OCI_ASSOC + OCI_RETURN_NULLS + OCI_RETURN_LOBS) : (OCI_NUM + OCI_RETURN_NULLS + OCI_RETURN_LOBS));
 		if (ISSET($_SESSION['ADDRDEBUGNEXTRECORD'])) {$GLOBALS["Xmc2"]=microtime(true);$mcresta=$GLOBALS["Xmc2"]-$GLOBALS["Xmc1"][(string)$id];echoLog(str_repeat(' &nbsp;',30). " Id ".$id. " NEXT record segundos:". (SPRINTF("%7.4f",($mcresta)))  .'<br>');}
	    return $XX;
    }

    /**
     * Returns boolean after counting result from query
     * @param: string $table, string fieldname, string value
     */
    function keyExists($table, $field, $value) {
        $field = uppercase($field);
        return $this->oneExistsRecords($table, $field . '=' . singleQuote($value)) > CLX_NUM_ZERO;
    }

    /**
     * Moves the internal row pointer to the specified
     * row number on result set. Improve by oci_fetch_all but if feasible
     * @param: resource cursor, int position
     */
    function moveTo($cursor, $position) {
        for ($i = CLX_INIT_ZERO; $i < $position; $i++) {
		  oci_fetch_array($cursor, OCI_NUM);
        }
    }

    /**
     * Returns the number of field columns on result set
     * @param: resource cursor
     */
    function numFields($cursor) {
        return oci_num_fields($cursor);
    }

    /**
     * Returns the number of rows on result set
     * @param: resource cursor
     */
    function numRows($cursor) {
        return oci_num_rows($cursor);
    }

    /**
     * ?????
     * @param: resource cursor
     */
    function fieldName($cursor, $field_num) {
        return oci_field_name($cursor, ($field_num + CLX_NUM_ONE));
    }

    /**
     * Strip slashes from sql query
     */
    function addslashes(&$something) {
        if (is_array($something)) {
		  foreach ($something as $key => $value) {
			 $value = $this->addslashes($value);
			 $something[$key] = $value;
		  }
        } else {
		  $something = str_replace(CLX_CHAR_SINGLE_QUOTE, "''", $something);
        }
        return $something;
    }

    /**
     * Return readible string from lob object, not used for the moment since fetch array has OCI_LOBS
     */
    function getLob($lob) {
        $return = '';
        if (is_object($lob))
		  $size = $lob->size();
        else
		  $size = CLX_INIT_ZERO;
        if ($size > CLX_NUM_ZERO) {
		  $lob->rewind();
		  $return = $lob->read($size);
        }
        return $return;
    }

//	ORACLE FUNCTIONS
    /**
     * Clears the Oracle's recyclebin
     */
    function purgeTrash() {
        if ($this->enable_purge === true) {
		  $sql = 'PURGE recyclebin';
		  $results = $this->executeQuery($sql);
		  unset($results);
        }
    }

    /**
     * Select Oracle recyclebin contents
     */
    function getTrash() {
        $sql = 'SELECT * FROM recyclebin';
        return $this->executeQuery($sql);
    }

    /**
     * Generates auto increment field
     */
    function createAutoIncrement($tablename, $fieldname) {
        $tablename = uppercase($tablename);
        $fieldname = uppercase($fieldname);
        $this->dropSequence($this->sequenceName($tablename));
        $this->dropTrigger($this->triggerName($tablename));
        $this->createSequence($this->sequenceName($tablename));
        //$sql = 'CREATE OR REPLACE TRIGGER ' . $this->triggerName($tablename) . ' BEFORE INSERT ON ' . $tablename . ' FOR EACH ROW BEGIN IF (:new.' . $fieldname . ' IS NULL) THEN SELECT ' . $this->sequenceName($tablename) . '.NEXTVAL INTO :new.' . $fieldname . ' FROM DUAL; END IF; END;';
        $sql = 'CREATE OR REPLACE TRIGGER ' . $this->triggerName($tablename) . ' BEFORE INSERT ON ' . $tablename . ' FOR EACH ROW BEGIN IF (:new.' . $fieldname . ' IS NULL OR :new.' . $fieldname . ' = 0) THEN SELECT ' . $this->sequenceName($tablename) . '.NEXTVAL INTO :new.' . $fieldname . ' FROM DUAL; END IF; END;';
        $results = $this->executeQuery($sql);
        unset($results);
    }

    /**
     * Search table to see if an autoincrement field exists
     */
    function existsAutoIncrement($tablename, $fieldname) {
        $return = '';
        $tablename = uppercase($tablename);
        $fieldname = uppercase($fieldname);
		
        $triggerrec = $this->getTrigger($this->triggerName($tablename));
        if (!empty($triggerrec['TRIGGER_BODY']) && stristr($triggerrec['TRIGGER_BODY'], ':new.' . $fieldname)) {
		  $return = 'auto_increment';
        }
        return $return;
    }

    /**
     * Return sequence name
     */
    function sequenceName($tablename) {
        return 'S' . str_replace('_', '', uppercase($tablename));
    }

    /**
     * Creates Oracle sequence
     * @param: string sequencename
     */
    function createSequence($sequencename,$init=0) {
        $results = $this->executeQuery('CREATE SEQUENCE ' . $sequencename . ' MINVALUE 0 START WITH '.$init.' INCREMENT BY 1 NOCACHE');
        unset($results);
        $this->increaseSequenceVoid($sequencename);
    }

    /**
     * Test if sequence exists
     *
     * @param string $sequencename
     * @return boolean
     */
    function sequenceExists($sequencename) {
        return $this->oneExistsRecords('USER_SEQUENCES', 'SEQUENCE_NAME=' . singleQuote($sequencename)) > CLX_NUM_ZERO;
    }

    /**
     * Drops Oracle sequence
     * @param: string sequencename
     */
    function dropSequence($sequencename) {
        if ($this->sequenceExists($sequencename)) {
		  $sql = 'DROP SEQUENCE ' . $sequencename;
		  $results = $this->executeQuery($sql);
		  unset($results);
        }
    }

    /**
     * Get sequence information
     * @param: string sequencename
     */
    function getSequence($sequencename) {
        return $this->getRecord('USER_SEQUENCES', 'SEQUENCE_NAME=' . singleQuote($sequencename));
    }

    /**
     * Get sequence information
     * @param: string sequencename
     */
    function getSequences() {
        return $this->getRecords('USER_SEQUENCES');
    }

    /**
     * Gets the value for the next sequence
     */
    function nextSequence($sequencename) {
        $sequence = $this->getSequence($sequencename);
        if (!empty($sequence['LAST_NUMBER'])) {
		  $return = $sequence['LAST_NUMBER'];
        } else {
		  $return = CLX_INIT_ZERO;
        }
        return $return;
    }

    /**
     * Increases selected sequent to the next value
     */
    function increaseSequenceVoid($sequencename) {
        ClxFreeReturn($this->increaseSequence($sequencename));
    }

    function increaseSequence($sequencename) {
        if (empty($sequencename)) {
		  return CLX_CHAR_EMPTY_STRING;
        }
        $retorno = $this->executeQuery('SELECT ' . $sequencename . '.nextval FROM DUAL');
        unset($retorno);
    }

    /**
     * Increases selected sequent to the next value
     */
    function setNextSequence($sequencename) {
        if (empty($sequencename)) {
		  return CLX_CHAR_EMPTY_STRING;
        }
       $retorno= $this->getone('SELECT ' . $sequencename . '.nextval FROM DUAL');
		
        return $retorno;
    }

    /**
     * Changes Sequence interval
     */
    function alterSequenceInterval($sequencename, $interval = CLX_NUM_ONE) {
        $retorno = $this->executeQuery('ALTER SEQUENCE ' . $sequencename . ' INCREMENT BY ' . $interval);
        unset($retorno);
    }

    /**
     * Verifies inserted id to check if less than next
     * sequence or increase sequence
     */
    function checkSequenceId($sequencename, $id) {
        if (empty($sequencename) || empty($id)) {
		  return CLX_CHAR_EMPTY_STRING;
        }
        $seq = $this->nextSequence($sequencename);
        if ($seq < $id) {
		  $interval = ($id - $seq) + CLX_NUM_ONE;
		  $this->alterSequenceInterval($sequencename, $interval);
		  $this->increaseSequenceVoid($sequencename);
		  $this->alterSequenceInterval($sequencename);
        }
    }

    /**
     * Set the next Sequence starting from tablename
     */
    function resetSequenceByTableName($tablename) {
        $this->resetSequence($this->sequenceName($tablename));
    }

    /**
     * Sets the last value for a sequence
     */
    function resetSequence($sequencename) {
        $this->dropSequence($sequencename);
        $this->createSequence($sequencename);
    }

    function checkTableSequence($tablename) {
        if ($this->existsAutoIncrement($tablename, 'ID')) {
		  $id = $this->getOne('SELECT MAX(ID) ID FROM ' . $tablename);
		  if ($id <> '')
			 $retorno = $this->checkSequenceId($this->sequenceName($tablename), $id);
		  unset($retorno);
        }
    }

    /**
     * Creates Oracle trigger
     * @param: string trigger
     */
    function createTrigger($triggername) {
        $retorno = $this->executeQuery('CREATE TRIGGER ' . $triggername);
        unset($retorno);
    }

    /**
     * Trigger Name
     */
    function triggerName($tablename) {
        return 'T' . str_replace('_', '', uppercase($tablename));
    }

    /**
     * Returns Oracle trigger
     * @param: string $sequencename
     */
    function getTrigger($triggername) {
        return $this->getRecord('USER_TRIGGERS', 'TRIGGER_NAME = ' . singleQuote($triggername));
    }

    /**
     * Test for trigger existance
     *
     * @param string $triggername
     * @return boolean
     */
    function triggerExists($triggername) {
        return $this->oneExistsRecords('USER_TRIGGERS', 'TRIGGER_NAME=' . singleQuote($triggername)) > CLX_NUM_ZERO;
    }

    /**
     * Drops Oracle trigger
     * @param: string $sequencename
     */
    function dropTrigger($triggername) {
        if ($this->triggerExists($triggername)) {
		  $sql = 'DROP TRIGGER ' . $triggername;
		  $results = $this->executeQuery($sql);
		  unset($results);
        }
    }
	function getFieldstypeNumeric($table) {
        $return = array();
		$sql='SELECT X.FIELDNAME,X.FIELDTYPE FROM CLX_DB_FIELDS X,CLX_DB_TABLEFIELDS Z,CLX_DB_TABLES Y WHERE Z.TABLECODE=Y.IDCODE AND X.FIELDNAME=Z.FIELDNAME AND X.FIELDTYPE IN('.singleQuoteArray(array('DECN','INTN')).') AND CONCAT(CONCAT(Y.MODULECODE,'.singlequote('_').'),Y.IDNAME)='.singleQuote($table);	
        $rs = $this->executeQuery($sql);
	    while ($result = $this->getNextRecord($rs, ASSOCIATIVE)) {
		  $return[$result['FIELDNAME']] = $result['FIELDTYPE'];
			for ($t=1;$t<51;$t++) {
				$return[$result['FIELDNAME'].'_'.$t] = $result['FIELDTYPE'];
			}
        }
		$this->freeResult($rs);		
        return $return;
	}
    /**
     * Searches date and date time fields to process them as in oracle
     * @param: tablename, array data_type
     */
    function getDateFields($table) {
        $return = array();
        $sql = 'SELECT  C.COLUMN_NAME, C.DATA_TYPE FROM USER_TAB_COLUMNS C WHERE C.TABLE_NAME=' . singleQuote(uppercase($table)) . ' AND (C.DATA_TYPE LIKE ' . singleQuote('TIMESTAMP%') . ' OR C.DATA_TYPE LIKE ' . singleQuote('DATE') . ')';
        $rs = $this->executeQuery($sql);
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) ) {
		  $return[$result['COLUMN_NAME']] = $result['DATA_TYPE'];
        }
		$this->freeResult($rs);		
        return $return;
    }

    /**
     * Searches lob fields on selected table
     * @param: tablename, array data_type
     */
    function getLobFields($table, $datatypes = false) {
        
		$fetched = false;
		if (kApcTtl_24>0) $returno = apc_fetchFct($findVar='fieldinfo_'.$tablename.'_'.$fieldname.'_' , $fetched);
        // Setting default datatypes
		if (empty($fetched)) {
			if (empty($datatypes)) {
			  $datatypes = array('FILE', 'CFILE', 'CLOB', 'BLOB', 'ROWID');
			}
			$sql = 'SELECT C.COLUMN_NAME, C.DATA_TYPE FROM USER_TAB_COLUMNS C WHERE C.TABLE_NAME=' . singleQuote(uppercase($table)) . ' AND C.DATA_TYPE IN (' . singleQuote(implode("','", $datatypes)) . ')';
			$rs = $this->executeQuery($sql);
			$returno = array();
			while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) === true) {
				$c = 'OCI_B_' . $result['DATA_TYPE'];
				$returno[$result['COLUMN_NAME']]['type'] = $c;
			}
			$this->freeResult($rs);	
			if (count($returno)==0)
				$returno ='vacio';
			if (kApcTtl_24>0 ){
				apc_addFct($findVar,$returno,kApcTtl_24);
			}	
		}	
		if (!is_array($returno)) {
			$returno=array();
		}
        return $returno;
    }

    /**
     * Creates a lob object for file, blob and clobs from record
     * @param: Fieldname, $record to be prepared, type of lob
     * Allowed types: OCI_D_FILE, OCI_D_LOB and OCI_D_ROWID.
     * The Default type OCI_D_LOB.
     */
    function bindDescriptor($type = '') {
        return oci_new_descriptor($this->dblink);
    }

    /**
     * Creates a lob object for file, blob and clobs from record
     * @param: Fieldname, $record to be prepared, type of lob
     * Allowed types: OCI_D_FILE, OCI_D_LOB and OCI_D_ROWID.
     * The Default type OCI_D_LOB.
     */
	 
	 
    //function bindNameVoid(&$stmt, $fieldname, &$value, $size = -CLX_NUM_ONE) {
	
    function bindNameVoid(&$stmt, $fieldname, &$value, $size = -CLX_NUM_ONE, $type_par = null) {
        //ClxFreeReturn($this->bindName($stmt, $fieldname, $value, $size));
        ClxFreeReturn($this->bindName($stmt, $fieldname, $value, $size, $type_par));
    }

    function bindName(&$stmt, $fieldname, &$value, $size = -CLX_NUM_ONE, $type_par = null) {
		$verificadordebindcreado=md5($this->sql.$fieldname);//echo " $verificadordebindcreado ";
		if (1==0 /*isset($GLOBALS[$verificadordebindcreado])*/) {
			$GLOBALS[$verificadorbindcreado]=$value;
			
		}
		else {
		//	echo "--0--";
		//	echo $fieldname;
        if ($fieldname === 'ID') {
		  $size = CLX_NUM_22; //Verify what should be the correct size
		 
				return oci_bind_by_name($stmt, ':' . $fieldname,$value, 22,SQLT_INT);		
				
        }
			else {		
			
			
        $type = (is_null($type_par) ? SQLT_CHR : $type_par);
        if (is_integer($value)) {
		  $type = SQLT_INT;
				
		  //$size = 22;
        } elseif (is_float($value)) {
				$type =	SQLT_NUM; // LNG;
				$size = CLX_NUM_22;
			}
				if( lowercase($value)!='null' &&$type ==	SQLT_FLT){
				$type=SQLT_NUM;
				//$size = CLX_NUM_22;
        }
        if ($fieldname === 'ID') {
		  $size = CLX_NUM_22; //Verify what should be the correct size
		  $type = SQLT_INT;
        }
			$valuenormal=$value;
			if ($type ==	SQLT_FLT || $type ==	SQLT_NUM) {
				$type=SQLT_LNG;
				if (uppercase($value)=='NULL') {
					$value=0;
				}
				else {
					$value=$value+0;
				}
			}
			
			
			
			if (lowercase($value)!='null' && $type==SQLT_CHR) {
				$value=singleQuote($value);
			}	
			$GLOBALS[$verificadordebindcreado]=$valuenormal;
			return oci_bind_by_name($stmt, ':' . $fieldname,$GLOBALS[$verificadordebindcreado], $size, $type);
		}	
			
		}	
    }

    function readBind($fieldname, $record) {
        unset($fieldname, $record);
    }

// Other functions
    /**
     * Replace non oci on sql
     * @see ClxDatabase::clx_parse()
     */
    function clx_parse(&$sql) {
        $sql = str_ireplace('IFNULL(', 'NVL(', $sql);
        $sql = str_ireplace('now()', 'CURRENT_DATE', $sql);
        $sql = str_ireplace('NOW()', 'CURRENT_DATE', $sql);
        $sql = str_ireplace('CURDATE()', 'CURRENT_DATE', $sql);
        if($this->temporary_supress_squote_change === false ) {
		  $sql = str_ireplace('"', CLX_CHAR_SINGLE_QUOTE, $sql);
        } else {
		  $this->temporary_supress_squote_change = false;
        }
        $sql = str_ireplace(', rowid asc', '', $sql);
        $sql = str_ireplace('POW(', 'POWER(', $sql);
    }

    /**
     * Prepares dates acording database implementation
     */
    function prepareDate(&$date) {

    }

    /**
     * Retuns SQL SELECT with MySQL's LIMIT emulation.
     */
    function select_limit($sql, $limit_string, $fromclause = '', $groupclause = '') {
        // Emula LIMIT de MySQL en instrucciones SELECT. Tener cuidado con los que tengan GROUP BY ya que puede no funcionar | Caso 1001929 - BNYM-Migracion de clx5dev a oracle | CR 2018-07-19 
        // En $limit_string viene algo como "LIMIT 1" o "LIMIT 1,20"
        $limit_string = str_replace('LIMIT ', '', $limit_string);
        return $this->oracle_limit($sql, $limit_string, $fromclause, $groupclause);
    }
    
    /**
     * Retuns SQL with Oracle's version o MySQL's LIMIT
     */
    function oracle_limit($sql, $limit_string, $fromclause, $groupclause = '') {
        //use to sort internal top n results before paging
        //if not sorted by row id paging could be not ordered correctly
        //the idea here is to get table codes to assign all rowid that may
        //apply
        // This may not work if there is a group by clause, so use carefully
        $rowidsort = '';
        $comma = '';
        if ($fromclause <> '' && $groupclause === '') {
		  $from = explode(',', $fromclause);
		  if (count($from) == CLX_NUM_ONE && strpos($from[CLX_NUM_ZERO], '_VW') === false) {
			 $tblkey = explode(' ', trim($fromclause));
			 
				if (isset($tblkey[1]) && $this->getone('SELECT COUNT(1) FROM CLX_DB_TABLES WHERE IDCODE='.singlequote(TRIM($tblkey[1])) .' and TYPE='.singlequote('VIEW'))  ==0) {
				
					$rowidsort = (isset($tblkey[CLX_NUM_ONE]) ? $tblkey[CLX_NUM_ONE] : $tblkey[CLX_NUM_ZERO]).'.rowid asc';
				}
		  } else {
			 foreach ($from as $key => $val) {
			     $tblkey = explode(' ', trim($val));
			     if (array_key_exists(CLX_NUM_ONE, $tblkey)) {
			         if (strpos($tblkey[CLX_NUM_ZERO], '_VW') === false) {
					   $rowidsort .= $comma . $tblkey[CLX_NUM_ONE] . '.rowid asc';
					   $comma = ', ';
			         }
			     }
			 }
		  }
		  if ($rowidsort <> '') {
			 if (strpos(uppercase($sql), 'ORDER BY') !== false) {
			     $sql .= ', ' . $rowidsort;
			 } else {
			     $sql .= ' ORDER BY ' . $rowidsort;
			 }
		  }
        }
        $return = '';
/////////OFFSET 20 ROWS FETCH NEXT 5 ROWS ONLY;
		//PRE( get_defined_constants(TRUE));

		if (strlen($limit_string) > CLX_NUM_ZERO && strlen($sql) > CLX_NUM_ZERO) {
				$constrains = explode(',', $limit_string);
				if (array_key_exists(CLX_NUM_ONE, $constrains)) {
					if  (!defined('UTIL_PAGE_NEXT') or  uppercase(UTIL_PAGE_NEXT)=='ROWNUM') {
						$return = 'select * from ( select a.*, ROWNUM rnum from (' . $sql . '  ) a where ROWNUM <= ' . ($constrains[CLX_NUM_ZERO] + $constrains[CLX_NUM_ONE]) .
							') where rnum  > ' . $constrains[CLX_NUM_ZERO];
					}
					ELSE {
						$return=$sql.' OFFSET '. $constrains[CLX_NUM_ZERO].' ROWS FETCH NEXT '.$constrains[CLX_NUM_ONE].'  ROWS ONLY';
					}
				} else {
						if  (!defined('UTIL_PAGE_NEXT') or  uppercase(UTIL_PAGE_NEXT)=='ROWNUM') {
						
							$return = 'select * from ( select    a.*, ROWNUM rnum from ( ' . $sql . ' ) a where ROWNUM <= ' . ($constrains[CLX_NUM_ZERO]) .
							') where rnum  > 0';
						}
						ELSE {
							$return=$sql.' OFFSET 0 ROWS FETCH NEXT '.$constrains[CLX_NUM_ZER0].'  ROWS ONLY';
						}
						
				}
		} else {
			$return = $sql;
		}
        return $return;
    }

    /**
     * Converts Oracle data types into mysql data types
     */
    function ociToMysql($type, $size, $decimal, $unsigned = false) {
        $return = '';
        $type = uppercase(trim($type));
        switch ($type) {
		  case 'VARCHAR2':
			 $return = 'varchar(' . $size . ')';
			 break;
		  case 'NUMBER':
			 if (empty($decimal))
			     $return = 'int(' . $size . ')';
			 else
			     $return = 'decimal(' . $size . ',' . $decimal . ')';
			 break;
		  case 'DATE':
			 $return = 'datetime()';
			 break;
		  case 'TIMESTAMP':
			 $return = 'timestamp';
			 break;
		  case 'CLOB':
			 $return = 'mediumblob';
			 break;
		  case 'BLOB':
			 $return = 'largeblob';
			 break;
		  default:
			 break;
        }
        return $return;
    }

    function mysqlToOci($old) {
		if ($this->debugCF==true) {
			pre('[[[[[>>> MYSQLTOOCI type old conversion');
			pre($old);
		}
        $old = str_ireplace('unsigned', '', $old);
        $old = trim($old);
        if (stristr($old, 'varchar')) {
		  return str_ireplace('varchar', 'NVARCHAR2', $old);
		  //return str_ireplace('varchar', 'VARCHAR2', $old);
        }
        if (stristr($old, 'varchar2')) {
		  return str_ireplace('varchar2', 'NVARCHAR2', $old);
		  //return str_ireplace('varchar', 'VARCHAR2', $old);
        }
        if (stristr($old, 'char')) {
		  return str_ireplace('char', 'NVARCHAR2', $old);
		  //return str_ireplace('char', 'VARCHAR2', $old);
        }
        //()A fixed section from 0 to 255 characters long.
        if (stristr($old, 'tinytext')) {
		  return str_ireplace('tinytext', 'NVARCHAR2(255)', $old);
		  //return str_ireplace('tinytext', 'VARCHAR2(255)', $old);
        }
        if (stristr($old, 'mediumtext')) {
		  return 'CLOB';
        }
        if (stristr($old, 'text')) {
		  return 'CLOB';
        }
        if (stristr($old, 'mediumblob')) {
		  return 'CLOB';
		  //return 'BLOB';
        }
        if (stristr($old, 'longblob')) {
		  return 'CLOB';
		  //return 'BLOB';
        }
        if (stristr($old, 'blob')) {
		  return str_ireplace('blob', 'CLOB', $old);
		  //return str_ireplace('blob', 'BLOB', $old);
        }
        //Numbers
        if (stristr($old, 'tinyint')) {
		  //()-128 to 127 normal 0 to 255 unsigned.
		  if (stristr($old, 'tinyint(1)'))
			 //return 'NUMBER(3)';
			 return 'NUMBER(1)';
		  else
			 return str_ireplace('tinyint', 'NUMBER', $old);
        }
        if (stristr($old, 'smallint')) {
		  //()-32768 to 32767 normal 0 to 65535 unsigned.
		  return str_ireplace('smallint', 'NUMBER', $old);
        }
        if (stristr($old, 'mediumint')) {
		  //()-8388608 to 8388607 normal0 to 16777215 unsigned.
		  return str_ireplace('mediumint', 'NUMBER', $old);
        }
        if (stristr($old, 'bigint')) {
		  //()-9223372036854775808 to 9223372036854775807 normal0 to 18446744073709551615 unsigned.
		  return str_ireplace('bigint', 'NUMBER', $old);
        }
        if (stristr($old, 'int')) {
		  //()-2147483648 to 2147483647 normal 0 to 4294967295 unsigned.
		  return str_ireplace('int', 'NUMBER', $old);
        }
        if (stristr($old, 'float')) {
		  //a small number with a floating decimal point.
		  return str_ireplace('float', 'NUMBER', $old);
        }
        if (stristr($old, 'double')) {
		  //(,)a large number with a floating decimal point.
		  return str_ireplace('double', 'NUMBER', $old);
        }
        if (stristr($old, 'decimal')) {
		  /*
		  $p = str_ireplace('decimal(', '', $old);
		  $p = str_replace(')', '', $p);
		  $p = explode(',', $p);
		  return 'NUMBER(' . ($p[CLX_NUM_ZERO] + CLX_NUM_ONE) . ',' . $p[CLX_NUM_ONE] . ')';
		  //(,)a double as string , allow a fixed dec point.
		  */
		  return str_ireplace('decimal', 'NUMBER', $old);
        }
        if (stristr($old, 'datetime')) {
		  //yyyy-mm-dd.
		  //return 'DATE';
		  return 'TIMESTAMP';
        }
        if (stristr($old, 'date')) {
		  //yyyy-mm-dd.
		  return 'DATE';
        }
        if (stristr($old, 'timestamp')) {
		  //yyyy-mm-dd.
		  return 'TIMESTAMP';
        }
        if (stristr($old, 'time')) {
		  return 'NVARCHAR2(8)';
		  //yyyy-mm-dd.
		  //return 'DATE';
        }
    }

// DB MANTEINANCE RELATED METHOD
    /**
     * Reindex table or Tables if not tablecode is provided.
     *
     * @param string [$tablename]
     * @return void
     */
    function buldStats($tablename = '') {
        $tablename = ($tablename <> '') ? ',' . singleQuote($tablename) : '';
        $retorno = $this->executeQuery('BEGIN dbms_stats.gather_table_stats(' . singleQuote(kDatabaseUsername) . $tablename . ', cascade=>true); END;');
        unset($retorno);
    }

    /**
     * Clear cache spool
     *
     * @param string [$tablename]
     * @return void
     */
    function clearSpool() {
        $retorno = $this->executeQuery('alter system flush shared_pool');
        unset($retorno);
    }

    // Check Procedures and functions  //Create
    function createFunction($name, $parameters, $body, $return) {
        $sql = 'CREATE OR REPLACE FUNCTION ' . $name . ' (' . $parameters . ') RETURN ' . $return . ' AS ' . (strpos($body, 'BEGIN')===false ? 'BEGIN' : '') . ' ' . $body . ' END ' . $name . ';';
        $results = $this->executeQuerySilent($sql);
        unset($results);
        $err = $this->errorMsg();
        if (!empty($err)) {
		  //return $err;
		  return $err . ' <br> Line: ' . $this->error['offset'] . ' <br> Instruction: <br> ' . $this->error['sqltext'];
        } else {
		  return true;
        }
    }

    function createProcedure($name, $parameters, $body) {
        $sql = 'CREATE OR REPLACE PROCEDURE ' . $name . ' (' . $parameters . ') AS ' . (strpos($body, 'BEGIN')===false ? 'BEGIN' : '') . ' ' . $body . ' END ' . $name . ';';
        $results = $this->executeQuerySilent($sql);
		$this->freeResult($results);		
        unset($results);
        $err = $this->errorMsg();
        if (!empty($err)) {
		  //return $err;
		  return $err . ' <br> Line: ' . $this->error['offset'] . ' <br> Instruction: <br> ' . $this->error['sqltext'];
        } else {
		  return true;
        }
    }

    //Exists
    function existsFunction($name) {
        $sql = 'SELECT * FROM ALL_OBJECTS WHERE OBJECT_TYPE = ' . singleQuote('FUNCTION') . ' AND OBJECT_NAME = ' . singleQuote($name);
        $rs = $this->executeQuery($sql);
		$xx=$this->getNextRecord($rs, ASSOCIATIVE);
		$this->freeResult($rs);		
        return $xx;
    }

    function existsProcedure($name) {
        $sql = 'SELECT * FROM ALL_OBJECTS WHERE OBJECT_TYPE = ' . singleQuote('PROCEDURE') . ' AND OBJECT_NAME = ' . singleQuote($name);
        $rs = $this->executeQuery($sql);
		$xx=$this->getNextRecord($rs, ASSOCIATIVE);
		$this->freeResult($rs);	
        return $xx;
    }

    function db_to_date($colname, $fmt = 'yyyy-mm-dd') {
    //function db_to_date($date_par, $fmt = 'yyyy-mm-dd') {
        //return 'TO_DATE(' . singleQuote($date_par) . ', ' . singleQuote($fmt) . ')';
        return 'TO_DATE(' . $colname . ', ' . singleQuote($fmt) . ')';
    }

    function db_to_datetime($colname, $fmt = 'yyyy-mm-dd hh24:mi:ss') {
        return $this->db_to_date($colname, $fmt);
    }

    function db_to_time($colname, $fmt = 'hh24:mi:ss') {
        return $this->db_to_date($colname, $fmt);
    }
    
    function db_to_char($colname) {
        return 'TO_CHAR(' . $colname . ')';
    }
    
    function db_to_blob($colname) {
        return 'TO_BLOB(UTL_RAW.CAST_TO_RAW(TO_CHAR(' . $colname . ')))';
    }
    
    function db_to_number($colname) {
        return 'TO_NUMBER(' . $colname . ')';
    }
    
    function cast_to_char($colname) {
        return $this->db_to_char($colname);
    }

    function cast_to_target_type($col_name, $target_data_type_par) {
        $col_name_new = quote($col_name);
        // Antes de evaluar el tipo, le quita eventual tama�o, especificado por ejemplo asi: NUMBER(12,2) >> NUMBER
        $target_data_type = $target_data_type_par;
        $pos = strpos($target_data_type, '(');
        if($pos !== false) {
		  $target_data_type = substr($target_data_type, 0, $pos);
        }
        // Si el tipo de datos es distinto, hace la conversion al tipo dato de destino
        switch ($target_data_type) {
		  case 'VARCHAR':
		  case 'VARCHAR2':
		  case 'NVARCHAR2':
		  case 'CLOB':
			 $col_name_new = $this->db_to_char($col_name_new);
			 break;
		  case 'BLOB':
			 $col_name_new = $this->db_to_blob($col_name_new);
			 break;
		  case 'DATE':
			 $col_name_new = $this->db_to_datetime($col_name_new);
			 break;
/*
		  case 'TIME':
			 $col_name_new = $this->db_to_time($col_name_new);
			 break;
*/
		  case 'TIMESTAMP':
			 $col_name_new = $this->db_to_datetime($col_name_new);
			 break;
		  case 'NUMBER':
			 $col_name_new = $this->db_to_number($col_name_new);
			 break;
        }
        return $col_name_new;
    }
    
    /**
     * Duplicate Table y truncate from database
     * Se duplica la tabla a una temporal y se vacia la tabla original
     * Esto se hace para poder hacer un ALTER TABLE en Oracle cuando la tabla tiene datos,
     * porque en algunos casos, Oracle da error cuando se intenta hacer esto.
     */
    function preAlterTable($tablename,$optype='M', $newfieldname='', $type='', $default='', $properties=array(), $null='') {
		
        $pat_err_msg = '';
        if(isset($this->table_structure_has_changes[$tablename]) && $this->table_structure_has_changes[$tablename] === true) {
		  return $pat_err_msg;
        }
//$this->debug=true;
        // Inicia transaccion para evitar que el proceso quede por la mitad si hay error y la tabla quede vacia
        $this->commit();
        $this->table_struc_temp_name[$tablename] = array();
		$nowPHP=str_replace('+','',str_replace('.','',hexdec(uniqid())));
        $this->table_struc_temp_name[$tablename]['tmp_table_name'] = 'Z__TMP_'.uppercase($tablename).'___'.$nowPHP;
        $this->table_struc_temp_name[$tablename]['transaction_name'] = uppercase($tablename.'_trn');
        $this->table_struc_temp_name[$tablename]['save_point_name'] = uppercase($tablename.'_sp');
        $this->table_struc_temp_name[$tablename]['prev_autocommit'] = $this->getAutocommit();
        // Apaga autocommit
        $this->setAutocommit(false);
		
		$sql='SELECT * FROM ( SELECT TABLE_NAME,  CASE WHEN INSTR(TABLE_NAME,'.singlequote('____').')<1 THEN 0 ELSE TO_NUMBER(SUBSTR(TABLE_NAME, INSTR(TABLE_NAME,'.singlequote('____').')+4)) END X  FROM USER_TABLES WHERE INSTR(TABLE_NAME,'.singlequote('Z_').')=1 ) Y WHERE X<'.($nowPHP-(86400*7));
		$resultsX = $this->executeQuery($sql);
		while (($rX = $this->getNextRecord($resultsX))) { 
			$sql = 'DROP TABLE '.$rX['TABLE_NAME'];
			$this->executeQuery($sql);
			$this->commit();
		}
		$this->freeResult($resultsX);	
		  // Si la tabla ya existe, es porque quedo una operacion de Audit sin concluir... Si no existe, la crea...
		  // Crea copia de la tabla original en una tabla temporal
		  //$sql = 'CREATE GLOBAL TEMPORARY TABLE '.$this->table_struc_temp_name[$tablename]['tmp_table_name'].' ON COMMIT PRESERVE ROWS AS SELECT * FROM '.$tablename;
		  $sql = 'CREATE TABLE '.$this->table_struc_temp_name[$tablename]['tmp_table_name'].' AS SELECT * FROM '.$tablename;
		
		  $results = $this->executeQuery($sql);
		  $this->freeResult($resulsts);	
		  unset($results);
		  $this->commit();
		
		
		
		
		$this->temporary_supress_squote_change = true;
		
	    // Prepara variables para guardar datos de la estructura anterior
        $sql = 'SELECT \'Antes\' STARE, F.COLUMN_NAME, F.DATA_TYPE, F.DATA_LENGTH, F.CHAR_LENGTH, F.DATA_PRECISION, F.DATA_SCALE, F.NULLABLE, F.DATA_DEFAULT FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME';
        // La estructura de tabla anterior a guardar, va a ser el de la tabla temporal
        //$binds = array('TABLE_NAME' => $tablename);
        $binds = array('TABLE_NAME' => uppercase($this->table_struc_temp_name[$tablename]['tmp_table_name']));
        // Guarda datos de la estructura anterior
        $results = $this->executeQuery($sql, $binds);
        $this->table_struc_temp_name[$tablename]['struc'] = array();
        if($this->debug===true) {
		  $this->table_struc_temp_name[$tablename]['struc_debug'] = array();
		  $i=0;
        }
        while(($row = $this->getNextRecord($results))) {
		  $this->table_struc_temp_name[$tablename]['struc'][$row['COLUMN_NAME']] = array('DATA_TYPE' => $row['DATA_TYPE']);
		  if($this->debug===true) {
			 $this->table_struc_temp_name[$tablename]['struc_debug'][$i] = $row;
			 $i++;
		  }
        }
        if($this->debug===true) {
		  $this->showRecordSet($this->table_struc_temp_name[$tablename]['struc_debug']);
        }
		//copiar indexes y constraints en tabla temporal
		
		
        // Vacia la tabla original para poder alterar las columnas
		//if ($optype!='C') {
		//	$sql = 'TRUNCATE TABLE '.$tablename;
		//	$results = $this->executeQuery($sql);
		//	$results = $this->executeQuery('COMMIT');
		//}	
        unset($results);
        $this->table_structure_has_changes[$tablename] = true;
		
        return $pat_err_msg;
		
    }
    
    /**
     * Insert data and Drop Table from database
     * Inserta la data desde la temporal hacia la tabla original 
     * y luego borra la tabla temporal.
     * Esto se hace para poder hacer un ALTER TABLE en Oracle cuando la tabla tiene datos,
     * porque en algunos casos, Oracle da error cuando se intenta hacer esto.
     */
    function postAlterTable($tablename) {
		$this->error['message']='';
		
        $pat_err_msg = '';
        if(!isset($this->table_structure_has_changes[$tablename]) || $this->table_structure_has_changes[$tablename] === false) {
		  // Si no hubo cambios en las collumnas, no hay nada que hacer
		  return $pat_err_msg;
        }
        // Extrae la lista de las columnas de la vieja estructura de la tabla original, que se guardo en una tabla temporal
        $col_list_new = array();
        $col_list_old = array();
        //$tableinfo = $this->getTableInfo($this->table_struc_temp_name[$tablename]['tmp_table_name']);
        $sql = 'SELECT F.COLUMN_NAME, F.DATA_TYPE, F.DATA_LENGTH, F.CHAR_LENGTH, F.DATA_PRECISION, F.DATA_SCALE, F.NULLABLE, F.DATA_DEFAULT FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME';
        $binds = array('TABLE_NAME' => uppercase($tablename));
        $rs = $this->executeQuery($sql, $binds);
        while (($r = $this->getNextRecord($rs))) {
		  $col_name_old = $r['COLUMN_NAME'];
			if(isset($this->renamed_columns[$tablename][$r['COLUMN_NAME']]) && !empty($this->renamed_columns[$tablename][$r['COLUMN_NAME']]) ) {
			 // En este caso se renombro la columna o se elimino
			 // Si fue eliminada, col_name_old queda en blanco...
			 $col_name_old = $this->renamed_columns[$tablename][$r['COLUMN_NAME']];
				
		  }
		  if($col_name_old !== '') {
			 // Si es blanco, es una nueva columna que no va a encontrar en la tabla temporal
			 // De lo contrario, la incluye en el INSERT ... SELECT
			 if(!isset($this->table_struc_temp_name[$tablename]['struc'][$col_name_old])&& !empty($this->renamed_columns[$tablename][$r['COLUMN_NAME']])) {
			     // Si no existe en la vieja estructura, es una nueva columna que no existe en la tabla temporal
			     continue;
			 }
				if (substr($col_name_old,-6)=='___TMP')
					continue;
			    if(isset($this->table_struc_temp_name[$tablename]['struc'][$col_name_old]) && $this->table_struc_temp_name[$tablename]['struc'][$col_name_old]['DATA_TYPE'] !== $r['DATA_TYPE']) {
			     // Si el tipo de datos es distinto, hace la conversion al tipo dato de destino
			     $col_name_old = $this->cast_to_target_type($col_name_old, $r['DATA_TYPE']);
			 } else {
					$col_name_old = quote($col_name_old);
//					$col_name_old = str_replace('___TMP', '', uppercase(quote($col_name_old)));
				}
				if ($this->real_fisic_fields('Z__TMP_'.uppercase($tablename), $col_name_old)) {
				 $col_list_old[] = $col_name_old; // el quote() ya se lo pone cast_to_target_type()
				 $col_list_new[] = quote($r['COLUMN_NAME']);
			  }
			}
			$this->freeResult($rs);	
        }
        // Se hace el INSERT...SELECT desde la tabla temporal que se guardo hacia la tabla original
        if (count($col_list_old)>0) {
        $col_name_old = implode(',', $col_list_old);
        $col_name_new = implode(',', $col_list_new);
        if($this->debug===true) {
		  $sql = 'SELECT \'Despues\' STARE, F.COLUMN_NAME, F.DATA_TYPE, F.DATA_LENGTH, F.CHAR_LENGTH, F.DATA_PRECISION, F.DATA_SCALE, F.NULLABLE, F.DATA_DEFAULT FROM USER_TAB_COLUMNS F WHERE F.TABLE_NAME = :TABLE_NAME';
				
		  $binds = array('TABLE_NAME' => uppercase($tablename));
		  $results = $this->executeQuery($sql, $binds);
		  $this->showRecordSet($results);
        }
		$this->freeResult($results);	
        $sql = 'INSERT INTO '.$tablename.' ('.$col_name_new.') SELECT '.$col_name_old.' FROM '.$this->table_struc_temp_name[$tablename]['tmp_table_name'];
			
        //
        // Guarda valores de variables de sesion que deben ser asignadas para evitar el throw normal del error
        $prev_disable_execute_query_error_trap = (isset($_SESSION['disable_execute_query_error_trap']) ? $_SESSION['disable_execute_query_error_trap'] : null);
        $_SESSION['disable_execute_query_error_trap'] = true; 
        $this->temporary_supress_squote_change = true;
			$this->clearErrorMsg();
		
        //
			// Ejecuta SQL, si hay error, reporta que no hay acceso tipo 'SYS'
			$results = $this->executeQuery($sql);
        //
		}	
        // Restaura variables de sesion asignadas para evitar el throw normal del error
		if (isset($disable_execute_query_error_trap ) ) {
        if(is_null($disable_execute_query_error_trap)) {
		  unset($_SESSION['disable_execute_query_error_trap']);
        } else {
		  $_SESSION['disable_execute_query_error_trap'] = $disable_execute_query_error_trap; // Restaura variable de session
        }
		}
        // Verifica eventual error
        if($this->errorMsg() !== '') {
		  $pat_err_msg = $this->errorMsg();
		  if($this->debug===true) {
			 pre('+++++ ERRORMSG 3: ' . $this->errorMsg());
		  }
		  $this->clearErrorMsg();
		  
		  // Para poder hacer rollback, hay que eliminar la tabla original ...
		  $sql = 'DROP TABLE '.$tablename;
		  $results = $this->executeQuery($sql);
		  unset($results);
		  if($this->errorMsg() !== '') {
			 $pat_err_msg .= '<br>' . CLX_CHAR_NL . $this->errorMsg();
		  }
		  if($this->debug===true) {
			 pre('+++++ ERRORMSG 4: ' . $this->errorMsg());
		  }
		  $this->clearErrorMsg();
		  // Y luego restaurarla desde la copia en la tabla temporal
		  $sql = 'CREATE TABLE '.$tablename.' AS SELECT * FROM '.$this->table_struc_temp_name[$tablename]['tmp_table_name'];

		  $results = $this->executeQuery($sql);
		  unset($results);
		  if($this->errorMsg() !== '') {
			 $pat_err_msg .= '<br>' . CLX_CHAR_NL . $this->errorMsg();
		  }
		  if($this->debug===true) {
			 pre('+++++ ERRORMSG 5: ' . $this->errorMsg());
		  }
		  $this->clearErrorMsg();
        } else {
		  // Si se pudo hacer el INSERT
		  unset($results);
        }

		$this-> review_and_delete_old_tmp ($this->table_struc_temp_name[$tablename]['tmp_table_name']);
		
        unset($results);
        // Commit para finiquitar la transaccion
        $this->commit();
        // Reinicia propiedades utilizadas
        $this->table_structure_has_changes[$tablename] = false;
        $this->renamed_columns[$tablename] = array();
        $this->table_struc_temp_name[$tablename] = array();
		
		if (!empty($this->tmp_fld_name) && $this->real_fisic_fields($tablename, $this->tmp_fld_name)) {
			$sqlx = 'ALTER TABLE ' . QUOTE($tablename) . ' DROP COLUMN ' . QUOTE($this->tmp_fld_name) ;
			$rename=false;
			$results = $this->executeQuerysilent($sqlx);
			$results = $this->executeQuery('COMMIT');
		}
		if (isset($this->table_struc_temp_name[$tablename]['prev_autocommit']))
        $this->setAutocommit($this->table_struc_temp_name[$tablename]['prev_autocommit']);
//$this->debug=false;

		
        return $pat_err_msg;
    }
    
    function showRecordSet($record_set, $cli_interfase = false) {
        if($cli_interfase === true) {
		  $str_init = '';
		  $str_end = '';
		  $col_init = '';
		  $col_sep = CLX_CHAR_TAB;
		  $row_init = '';
		  $row_sep = CLX_CHAR_NL;
        } else {
		  $str_init = '<table>';
		  $str_end =  '</table>';
		  $col_init = '<td>';
		  $col_sep = '</td>';
		  $row_init = '<tr>';
		  $row_sep = '</tr>';
        }
       
        $i=0;
        while($i >= 0){
		  if(is_array($record_set)) {
			 if($i >= count($record_set)) {
			     break;
			 }
			 $row = $record_set[$i];
		  } else {
			 $row = $this->getNextRecord($record_set);
			 if(empty($row)) {
			     break;
			 }
		  }
		  if($i==0) {
			 // Nombres de las columnas
			 $linea = '';
			 foreach($row as $key => $val){
			     $linea .= $col_init . $key . $col_sep;
			 }
			 
		  }
		  $linea = '';
		  foreach($row as $key => $val){
			 $linea .= $col_init . $val . $col_sep;
		  }
		  
		  $i++;
        }
		$this->freeResult($record_set);	
        
    }
	function check_data_and_convert_fields_to_upper($fieldname='',$tablename='') {
		return ;
		$campos=array();
		if (!empty($fieldname)&&!empty($tablename) ){
			$sql1='SELECT COLUMN_NAME,TABLE_NAME,DATA_TYPE FROM USER_TAB_COLUMNS F WHERE UPPER(F.COLUMN_NAME)<>COLUMN_NAME AND COLUMN_NAME='.singleQUOTE($fieldname).' AND TABLE_NAME = '.singleQUOTE($tablename); 
		}
		else if (!empty($fieldname)){
			$sql1='SELECT COLUMN_NAME,TABLE_NAME,DATA_TYPE FROM USER_TAB_COLUMNS F WHERE UPPER(F.COLUMN_NAME)<>COLUMN_NAME AND COLUMN_NAME='.singleQUOTE($fieldname); 
		
		}
		else if (! empty($tablename) ){
			$sql1='SELECT COLUMN_NAME,TABLE_NAME,DATA_TYPE FROM USER_TAB_COLUMNS F WHERE UPPER(F.COLUMN_NAME)<>COLUMN_NAME AND TABLE_NAME = '.singleQUOTE($tablename); 
		}
		else{
			$sql1='SELECT COLUMN_NAME,TABLE_NAME,DATA_TYPE FROM USER_TAB_COLUMNS F WHERE UPPER(F.COLUMN_NAME)<>COLUMN_NAME '; 
		}
		
		$this->temporary_supress_squote_change = true;
		$records = $this->executeQuery($sql1);
		
		while ($record = $this->getNextRecord($records, ASSOCIATIVE)) {
			IF($record['DATA_TYPE']=='CLOB') {
				CONTINUE;
			}	
			$sql2='SELECT COUNT(1) CUENTA FROM USER_TAB_COLUMNS F WHERE F.COLUMN_NAME='.singlequote(uppercase($record['COLUMN_NAME'])).'  AND TABLE_NAME = '.singleQUOTE($record['TABLE_NAME']); 
		
			$this->temporary_supress_squote_change = true;
			$contarx = $this->getone($sql2);
			
			if ($contarx>0) {
				$this->temporary_supress_squote_change = true;
				$sql2='SELECT COUNT(1) FROM '.QUOTE($record['TABLE_NAME']).' WHERE '.quote(uppercase($record['COLUMN_NAME'])).' IS NULL';
				$ycontar=$this->getOne($sql2);
				
				$sql2='SELECT COUNT(1) FROM '.QUOTE($record['TABLE_NAME']);
				$ycontar=$this->getOne($sql2);
				
				$rename=false;
				
				IF ($ycontar<0) {
					$this->temporary_supress_squote_change = true;
					$sql3='SELECT COUNT(1) FROM USER_TAB_COLUMNS F WHERE COLUMN_NAME='.singlequote(uppercase($record['COLUMN_NAME'])).' AND TABLE_NAME = '. singlequote($record['TABLE_NAME']); 
				//	e cho 'sql3:',$sql3,"<br>";
					$xcontar=$this->getOne($sql3);
					IF ($xcontar>0) {
						$this->temporary_supress_squote_change = true;
						$sql6 = 'ALTER TABLE ' . QUOTE($record['TABLE_NAME']) . ' DROP COLUMN ' . QUOTE($record['COLUMN_NAME']) ;
						$rename=false;
						$results = $this->executeQuery($sql6);
						$results2 = $this->executeQuery('COMMIT');
						$this->freeResult($results);
						$this->freeResult($results2);
					}
					else {
						$rename=true;	
					}
					
				}
				else {
					
					$this->temporary_supress_squote_change = true;
				//	IF($record['DATA_TYPE']=='NUMBER')
					$sql4='SELECT count(1)-COUNT("'.$record['COLUMN_NAME'].'")  Y FROM '.QUOTE($record['TABLE_NAME']); //.' WHERE NVL('.QUOTE(uppercase($record['COLUMN_NAME'])).',9999999999999999999999999999999999999999)=9999999999999999999999999999999999999999'; 
				//	else if ($record['DATA_TYPE']=='DATE') 
				//		$sql4='SELECT COUNT(1) FROM '.QUOTE($record['TABLE_NAME']).' WHERE NVL('.QUOTE(uppercase($record['COLUMN_NAME'])).','.SINGLEQUOTE('01-01-01') .')='.SINGLEQUOTE('01-01-01'); 
				
				//	else
				//		$sql4='SELECT COUNT(1) FROM '.QUOTE($record['TABLE_NAME']).' WHERE NVL('.QUOTE(uppercase($record['COLUMN_NAME'])).','.SINGLEQUOTE('.EMPTY.') .')='.SINGLEQUOTE('.EMPTY.'); 
					IF (($xx=$this->getOne($sql4))>0) {
						$rename=true;
						$this->temporary_supress_squote_change = true;
						$sql7 = 'ALTER TABLE ' . QUOTE($record['TABLE_NAME']) . ' DROP COLUMN ' . QUOTE(uppercase($record['COLUMN_NAME'])) ;
						
					
						$results = $this->executeQuery($sql7);
						$results2 = $this->executeQuery('COMMIT');
						$this->freeResult($results);
						$this->freeResult($results2);
						
					}
					else {
						$rename=false;
						$this->temporary_supress_squote_change = true;
						$sql8 = 'ALTER TABLE ' . QUOTE($record['TABLE_NAME']) . ' DROP COLUMN ' . QUOTE($record['COLUMN_NAME']) ;
						
						
						$results = $this->executeQuery($sql8);
						$results2 = $this->executeQuery('COMMIT');
						$this->freeResult($results);
						$this->freeResult($results2);
					}
					
					
				}
					
			}
			else {
				$rename=true;
			}	
			if ($rename==true) {
				$sql5 = 'ALTER TABLE ' . QUOTE($record['TABLE_NAME']) . ' RENAME COLUMN '.QUOTE($record['COLUMN_NAME']). ' TO ' . QUOTE(uppercase($record['COLUMN_NAME'])) ;
				
				$this->temporary_supress_squote_change = true;
				$results = $this->executeQuery($sql5);
				$results2 = $this->executeQuery('COMMIT');
				$this->freeResult($results);
				$this->freeResult($results2);
				
			}
			
		}
		$this->freeResult($records);	
	}
	function real_fisic_fields($tablename='',$fieldname) {
		if (empty($tablename)) {
			return array();
		}
		switch (kDatabaseType) {
			case 'oci': // Oracle
//				$rs = $this->executeQuerySilent('SELECT column_name, data_type,data_length  FROM all_tab_columns  WHERE table_name = '.singleQuote($tablename)); 
				$rs = $this->executeQuerySilent($sql='SELECT column_name, data_type,CASE WHEN data_type='.singlequote('NUMBER').' THEN TRIM(TO_CHAR(DATA_PRECISION))||'.singlequote('.').'||TRIM(TO_CHAR(DATA_SCALE)) ELSE  TO_CHAR(data_length) END  data_length  FROM all_tab_columns WHERE table_name = '.singleQuote($tablename)); 
				
				break;
			default:
				$rs = $this->executeQuerySilent('describe '.$tablename);
			break;
		
		}
		$retorno=false;
	  	while ($r = $this->getNextRecord($rs, NUMERIC)) {
		//	no colocar upper hay campos buscados con quote devolver que un campo sea distinto "NAME" de "Name" de "name" ...
			if ($r[0]==$fieldname) {
				$retorno=true;
				break;
			}
		
		}
		$this->freeResult($rs);
		return $retorno;
	
	}
	function describe_table($tablename='') {
		if (empty($tablename)) {
			return array();
		}
		switch (kDatabaseType) {
			case 'oci': // Oracle
//				$rs = $this->executeQuerySilent('SELECT column_name, data_type,data_length  FROM all_tab_columns  WHERE table_name = '.singleQuote($tablename)); 
				$rs = $this->executeQuerySilent($sql='SELECT column_name, data_type,CASE WHEN data_type='.singlequote('NUMBER').' THEN TRIM(TO_CHAR(DATA_PRECISION))||'.singlequote('.').'||TRIM(TO_CHAR(DATA_SCALE)) ELSE  TO_CHAR(data_length*2) END  data_length  FROM all_tab_columns WHERE table_name = '.singleQuote($tablename)); 
			
			//	e cho $sql;
				break;
			default:
				$rs = $this->executeQuerySilent('describe '.$tablename);
			break;
		
		}
		$fisic_fields=array();
		$key_fisic_fields=array();
		$size_fields=array();
        if (!empty($rs)) {
			while ($r = $this->getNextRecord($rs, NUMERIC)) {
				$r[1]=uppercase(preg_replace('/\((.*?)\)/i', '',$r[1]));
				$fisic_fields[uppercase($r[0])]=$r[1];
				$key_fisic_fields[]=uppercase($r[0]);
				IF ($r[1]=='NUMBER' && $r[2]=='.')
					$r[2]='22.0';
				
				$size_fields[uppercase($r[0])]=$r[2];
				//$type_fields[uppercase($r[0])]=$r[1];
			}
		}
		
		$x=  array('fisic_fields'=>$fisic_fields,'key_fisic_fields'=>$key_fisic_fields,'size_fields'=>$size_fields);
		return $x;
	}
	function create_procedure_bulk($importcode,$BATCH_ID,$LOG_ID,$onlyInsert=true,$extra=array(),$mappings=array(),$tablename='',$describe=array(),$name_procedure='',$verificar_exists=array(),$fields_single=array(),$fields_array=array(),$error_general=false) {
		$debug=false;
		 IF ($this->debug==true)
			$debug=true;
		IF (ISSET($GLOBALS['ADDRDEBUGIMPORT']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGIMPORT']==$_SERVER['REMOTE_ADDR'])) {
			$debug=true;
		}
		$tablecode = $this->getone($sql='SELECT IDCODE FROM CLX_DB_TABLES WHERE MODULECODE||'.singlequote(  '_' ).'||IDNAME='.singlequote($tablename));
	//	e cho $tablecode,",",$sql;
		$DBTI = &getTool('DBTI');
		$indx = $DBTI->getTableIndexes($tablecode, 'UNI');
		$indicesUni=array();
		foreach($indx as $key=>$vl) {
			$indicesUni[]=array('indices'=>explode(',',$vl['FIELDNAME']),'where'=>'');
		}

		if ($debug==true) {
			echo "error general";
			pre($error_general);
			echo "importcode";
			pre($importcode);
			echo "batch_id";
			pre($BATCH_ID);
			echo "log_id";
			pre ($LOG_ID);
			echo "extra";
			pre($extra);
			echo "tablename";
			pre($tablename);
			echo "procedure";
			pre($name_procedure);
			echo "mapping";
			pre($mappings);
			echo "vericar_exists";
			
			pre($verificar_exists);
			echo "fields_single";
			
			pre($fields_single);
			echo "fields_array";
			pre($fields_array);
			echo "describe";
			pre($describe);			
			
			
			
			
		
			if (empty($tablename)) {
				
				echo 'aqui salio1';
				return false;
			}
			else if( empty($name_procedure)){
				echo 'aqui salio2';
				return false;
				
			}	

			else if (empty($fields_single)){
				echo ':::::::::::::::::advertencia no hay single';
			
				
			}	

			else if ( empty($fields_array)) {
				echo 'aqui salio4';
				return false;
				
			}
		}
		$name_procedure=uppercase($name_procedure);
		$fisic_fields=$describe['fisic_fields'];
		$key_fisic_fields=$describe['key_fisic_fields'];
		$size_fields=$describe['size_fields'];
		
		$string_init_objects='';
		$string_init_objects2='';
		$string_init_use_objects='';
		$cont_pos=0;
		$name_fields='';
		$objects='';
		$string_use_objects='';
		$string_use_objects_alias='';
		$string_use_objects_alias_view='';
		$string_use_objects_with_alias='';
		
		
		$WHEREON='';
	
		$update_objects='';
		$update_objects_='';
		
		$fields=array();
		$entry_single=false;
	/////////////////////////single////////////////////////	
		$parsebind ='ARRAYBIND_'.$name_procedure.'.'.$name_procedure.'(';
		$max1=count($fields_single);
		$sibatch=false;
		$siinitpos=false;
		$initcommas=0;
		$textoextraposicion='';
		$textoextraposicion2='';
		
		
		for ($t=0;$t<$max1;$t++)  {
			if ($debug==true) {
				pre( "[[[[[pos single ".$t);
				pre($fields_single[$t]);
			}				
			if ($fields_single[$t]=='BATCH_ID') {
				$sibatch=true;
				$initcommas++;
				continue;
			}	
			if ($fields_single[$t]=='INIT_POS') {
				$siinitpos=true;
				$initcommas++;
				continue;
			}
			$size='';
			$type=$fisic_fields[uppercase($fields_single[$t])];
			$oldtype=$type;
			if (strpos(' ,CHAR,VARCHAR,SOBJ,NVARCHAR,VARCHAR2,NVARCHAR2,',','. $type.',') >0 ) {
				//$size='('.$size_fields[uppercase($fields_array[$t])].')';
				$type='VARCHAR2';
			}
			else if($type=='DATE' ) {
				$type='VARCHAR2';
						
			}
			
			$fields[]=$fields_single[$t];	
			if ($debug==true) {
				pre("[[[[[[[[[[fields");
				pre($fields);				
			}	
			$valid=true;
			foreach($indicesUni as $key=>&$posIndice) {
				foreach($posIndice['indices'] as $key =>$value) {
					if ($value==$fields_single[$t])
						$valid=false;
					
				}
			}		
			if ($t>$initcommas) {
				if(substr($string_init_use_objects,-1)!=',')
					$string_init_use_objects.=',';
				$textoextraposicion2.='||\',\'||';
				$name_fields.=',';
				
				$string_use_objects.=',';
				$string_use_objects_alias.=',';
				$string_use_objects_with_alias.=',';
				$string_use_objects_alias_view.='||\',\'||';
				

				if($valid==true)
				
					$update_objects.=',';
				$parsebind.=',';
			}
			$textoextraposicion2.=$fields_single[$t];
			$parsebind .= ':'.$fields_single[$t];
			$bindes[$fields_single[$t]]=$fields_single[$t];
			$string_init_use_objects.=chr(13).chr(10). chr(9).chr(9).$fields_single[$t].' IN OUT '.$type. $size;
		//	if($oldtype=='DATE' ) {
		//		$string_use_objects.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).'TO_DATE('.$fields_single[$t].','.singlequote('YYYY-MM-DD').')';
		//	}	
		//	else {
				$string_use_objects.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).$fields_single[$t];
				$string_use_objects_alias.=	$fields_single[$t].' "'.$fields_single[$t].'"';
				$string_use_objects_alias_view.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).$fields_single[$t].'||\' "'.$fields_single[$t].'"\'';

				$string_use_objects_with_alias.='T2.'.$fields_single[$t];
				if($valid==true)
				
					$update_objects.='T1.'.$fields_single[$t].'=T2.'.$fields_single[$t];
				
		//	}
			foreach($indicesUni as $key=>&$posIndice) {
				foreach($posIndice['indices'] as $key =>$value) {
					if ($value==$fields_single[$t]) {
						if (!empty($posIndice['where']))
							$posIndice['where'].=' AND ';
						$posIndice['where'].=' T1.'.$value.' = T2.'.$value;

					}
				}
			}

		//	$name_fields.=$fields_array[$t];
			$entry_single=true;
		}
		if ($debug==true) {
				pre("[[[[[[[[[[ exit from if for fields");
				pre($fields);				
		}
		if($sibatch==true) {
			if (!empty($string_init_use_objects))
				$string_init_use_objects.=',';
			$string_init_use_objects.=chr(13).chr(10). chr(9).chr(9).'BATCH_ID IN OUT NUMBER';
		}	
		/*($siinitpos==false) {e cho 'aqu';
			foreach($extra as $keys=>$vals) {e cho 'aco';
				if ($vals[0]=='INIT_POS')
					$siinitpos==true;
			}
		}	*/
		if($siinitpos==true) {
			if (!empty($string_init_use_objects))
				$string_init_use_objects.=',';
			$string_init_use_objects.=chr(13).chr(10). chr(9).chr(9).'INIT_POS IN OUT NUMBER';
		}
		if (!empty($string_init_use_objects))
				$string_init_use_objects.=',';
		//pr e($fisic_fields);
		$posicionesnumber='';
		$extraposicion=2;
		
		$cont_pos=0;
		
		IF($debug===true) {
			PRE("FIELDS TO ARRAY BULK PROCEDURE INSERT CONSTRUCT");
			pre($fields_array);
			pre("[[[[[[[[[[ next from if for fields");
			pre($fields);				
		}		
		for ($t=0;$t<count($fields_array);$t++)  {
			IF($debug===true) {
				
				echo "[[[[[[[[[ in for ", $textoextraposicion,"fields $t<br>";
			}	
			$size='';
			$type=$fisic_fields[uppercase($fields_array[$t])];
			//e cho uppercase($fields_array[$t]),",.",$fisic_fields[uppercase($fields_array[$t])],"<br>";
			$oldtype=$type;
			$fieldtextoinsert='col_'.$cont_pos.'(SQL%BULK_EXCEPTIONS (i).ERROR_INDEX);';
			IF($debug===true) {
				pre("[[[[[[[[[[ next 2 from if for fields in for");
				pre($fields);				
			}	
			if (strpos(' ,CHAR,VARCHAR,NVARCHAR,VARCHAR2,NVARCHAR2,',','. $type.',') >0 ) {
				$size='('.$size_fields[uppercase($fields_array[$t])].')';
				$type='VARCHAR2';
				
			}
			else if ($type=='NUMBER') {
				$size='(22,6)';
				$fieldtextoinsert='TO_CHAR(col_'.$cont_pos.'(SQL%BULK_EXCEPTIONS (i).ERROR_INDEX));';
			
			}
			else if($type=='DATE' ) {
	//			if($fields_array[$t]!='CLX_EFFTDATE'){
			
				$type='VARCHAR2';
				$size='(25)';
	//			}
	//			else {
	//				$type='DATE';
	//				$size='';
	//			}
			}
			else if($type=='CLOB' ) {
				$type='VARCHAR2';
				$size='(4000)';
			}
			if (!in_array($fields_array[$t],$fields)) {
			$textoextraposicion.=chr(9).chr(9).'IF (LENGTH(col_'.$cont_pos.'(SQL%BULK_EXCEPTIONS (i).ERROR_INDEX))+LENGTH(TEXTRA3))>2000 THEN '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA3:=\'\';'.chr(13).chr(10).		
			chr(9).chr(9).'END IF;'.chr(13).chr(10);
			
			
			$fields[]=$fields_array[$t];
			
			$valid=true;
			foreach($indicesUni as $key=>&$posIndice) {
				foreach($posIndice['indices'] as $key =>$value) {
					if ($value==$fields_array[$t])
						$valid=false;
					
				}
			}		
				IF($debug===true) {
					pre("[[[[[[[[[[ next 3 from if for fields in for");
					pre($fields);				
				}		
			if ($t>0 ||$entry_single==true ) {
					if (!empty($fieldtextoinsert))
				$fieldtextoinsert='\',\'||'.$fieldtextoinsert;
					if (!empty($name_fields))
				$name_fields.=',';
					if (!empty($string_use_objects))
				$string_use_objects.=',';
					if (!empty($string_use_objects_alias))
				$string_use_objects_alias.=',';
					if (!empty($string_use_objects_with_alias))
				$string_use_objects_with_alias.=',';
					if (!empty($string_use_objects_alias_view)) 
						$string_use_objects_alias_view.='||\',\'||';
					if($valid==true and !empty($update_objects))
					$update_objects.=',';
					if ($t>0 and !empty($string_init_objects))
					$string_init_objects.=';';
				if (!empty(trim($string_init_use_objects))) {
					if(substr($string_init_use_objects,-1)!=',')
						$string_init_use_objects.=',';
					
				}	
				$entry_single=false;
				$parsebind.=',';
			}
				IF($debug===true) {
					pre("fieldtextoinsert");
					pre($fieldtextoinsert);
				}	
			$parsebind .= ':col_'.$cont_pos;
			if ($debug==true) {
				echo "BULK INDICES====================== ON WHERE'",'<br>';
			}
			foreach($indicesUni as $key=>&$posIndice) {
				foreach($posIndice['indices'] as $key =>$value) {
					if ($debug==true) {
						echo $value,'==',$fields_array[$t],'<br>';
					}
					if ($value==$fields_array[$t]) {
						if (!empty($posIndice['where']))
							$posIndice['where'].=' AND ';
						$posIndice['where'].=' T1.'.$value.' = T2.'.$value;
						if ($debug==true) {
							echo '-------------<br>';
						}	
					}
				}
			}
			if ($debug==true) {
				pre($posIndice['where']);
			}
			$bindes[uppercase($fields_array[$t])]='col_'.$cont_pos;
			$string_init_objects.=chr(13).chr(10).chr(9).chr(9).'TYPE ARRTYPE'.$t.' IS TABLE OF '.$type.$size.' INDEX BY BINARY_INTEGER';
			if ($type=="NUMBER"&&$fields_array[$t]=='ID')  {
				$posicionesnumber.='TO_CHAR(col_'.$cont_pos.' (SQL%BULK_EXCEPTIONS (i).ERROR_INDEX))||\',\'||';
			}
			$string_init_use_objects.= chr(13).chr(10).chr(9).chr(9).('col_'.$cont_pos).(' IN OUT ARRTYPE'.$t);	
//			if($oldtype=='DATE' ) {
	//			$string_use_objects.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).'TO_DATE(TRIM('.('col_'.$cont_pos).'(i)),'.singlequote('YYYY-DD-MM').')';
		//	}	
			//else
			$string_use_objects.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).('col_'.$cont_pos).'(i)';
			$string_use_objects_alias.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).('col_'.$cont_pos).'(i) "'.$fields_array[$t].'"';
			$string_use_objects_alias_view.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).('col_'.$cont_pos).'(SQL%BULK_EXCEPTIONS (i).ERROR_INDEX) ||\' "'.$fields_array[$t].'"\'';
			$string_use_objects_with_alias.= chr(13).chr(10). chr(9).chr(9).chr(9).chr(9).chr(9).'T2.'.$fields_array[$t];
				
				IF($debug===true) {
					pre("[[[[[[[[[[[[string_use_objects");
					pre($string_use_objects);
					pre("[[[[[[[[[[[[string_use_objects_alias");
					pre($string_use_objects_alias);
					pre("[[[[[[[[[[[[string_use_objects_alias_view");
					pre($string_use_objects_alias_view);
					pre("[[[[[[[[[[[[[string_use_objects_with_alias");
					pre($string_use_objects_with_alias);
				}
				
				
			if($valid==true)
				$update_objects.='T1.'.$fields_array[$t].'=T2.'.$fields_array[$t];
			$textoextraposicion.=chr(9).chr(9).'TEXTRA3:=TEXTRA3||'.$fieldtextoinsert.chr(13).chr(10);
			++$cont_pos;
		}
		}
		if ($debug==true) {
			pre($bindes);
			pre($string_init_objects);
		} 
		
		if ($debug==true) {	
			pre($indicesUni);
		}
		
		
		foreach($indicesUni as $key=>&$posIndice) {
			 
				
			if (!empty($posIndice['where'])) {
				if (empty($WHEREON))
					$WHEREON= '  ('. $posIndice['where'].') ';
				else 
					$WHEREON.= ' OR ('. $posIndice['where'].') ';
			}
				
			
		}
		IF($debug===true) {
			pre("[[[[[[[[[[ next 4 from if for fields");
			pre($fields);				
		}		
		IF (empty($WHEREON)) {
			$onlyInsert=true;
			$WHEREON ='0=1';
		}	
		$create_pkg = 'CREATE OR REPLACE PACKAGE ARRAYBIND_'.$name_procedure. ' AS '.chr(13).chr(10).
			  $string_init_objects.';'.chr(13).chr(10).chr(9).
			  'PROCEDURE '.$name_procedure.'('.$string_init_use_objects .');'.chr(13).chr(10).
			'END ARRAYBIND_'.$name_procedure.';'.chr(13).chr(10);
		$fields=uppercase(implode(','.chr(13).chr(10).chr(9).chr(9).chr(9).chr(9).chr(9),$fields));
		
		
		
		$create_pkg_body = 'CREATE OR REPLACE PACKAGE BODY ARRAYBIND_'.$name_procedure.' AS '.chr(13).chr(10).
			' PROCEDURE '.$name_procedure.' ('.$string_init_use_objects.') IS'.chr(13).chr(10).
			'	ERR_MSG VARCHAR2(2000);'. chr(13).chr(10).
			'	TEXTRA1 VARCHAR2(4000);'. chr(13).chr(10).
			'	TEXTRA2 VARCHAR2(4000);'. chr(13).chr(10).
			'	TEXTRA3 VARCHAR2(4000);'. chr(13).chr(10).
			'	ERR_NUM NUMBER;'. chr(13).chr(10).
			'   ERR_LINE NUMBER;'. chr(13).chr(10).
			'	BEGIN'.chr(13).chr(10).
			'	-- Bulk Insert'.chr(13).chr(10). 
//			'	INSERT INTO XX VALUES(IMPORTCODE,BATCH_ID,LOG_ID);'.chr(13).chr(10). 
			
//			'   COMMIT;'.chr(13).chr(10).
		//	'	ALTER SESSION SET nls_date_format = '.singlequote('yyyy-mm-dd').';'.chr(13).chr(10).
		//		chr(9). '	TEXTRA2:=TEXTRA1;'.chr(13).chr(10). 
			'   TEXTRA1:=\'.\';'.chr(13).chr(10).
		
			'   TEXTRA2:=\'.\';'.chr(13).chr(10).
			'   TEXTRA3:=\'\';'.chr(13).chr(10).
			'		FORALL i IN INDICES OF col_0 SAVE EXCEPTIONS'.chr(13).chr(10);
		//	'			BEGIN'.chr(13).chr(10).
		//	'				LINE=i;'.chr(13).chr(10).
		
		if ($onlyInsert==true) {
				
			$create_pkg_body .=	chr(9). '	INSERT INTO '.$tablename.' ('.$fields.') '.chr(13).chr(10).
				chr(9).	'	VALUES ('.chr(13).chr(10).$string_use_objects.');'.chr(13).chr(10).
				'	COMMIT;'.chr(13).chr(10);
	
		}
	
		else {
			$create_pkg_body .=chr(9).'MERGE INTO '.$tablename.' T1 '.CHR(13).CHR(10).
				chr(9).' USING (SELECT '. $string_use_objects_alias .' FROM DUAL) T2 '.CHR(13).CHR(10).
				chr(9).' ON ('.$WHEREON.')  '.CHR(13).CHR(10).
				chr(9).' WHEN MATCHED THEN  '.CHR(13).CHR(10).
				chr(9).' UPDATE SET '.$update_objects. CHR(13).CHR(10).

				chr(9).' WHEN NOT MATCHED THEN  '.CHR(13).CHR(10).
				chr(9).' INSERT ( '.$fields.')  '.CHR(13).CHR(10).
				chr(9).' VALUES ('.$string_use_objects_with_alias.'); '.CHR(13).CHR(10);
		
		}

		$create_pkg_body .=	'	EXCEPTION '.chr(13).chr(10).
			chr(9).	'	WHEN OTHERS THEN'.chr(13).chr(10).   
			chr(9).chr(9).'COMMIT;'.chr(13).chr(10).   
			
			chr(9).chr(9).	'FOR i IN 1 .. SQL%BULK_EXCEPTIONS.COUNT'.chr(13).chr(10).   
			chr(9).chr(9).	'LOOP'.chr(13).chr(10);   
//			chr(9).chr(9).chr(9).'INSERT INTO CLX_DB_IMPORT_ERRORS ("ID","IMPORTCODE","FIELDNAME","ERRORMSG") VALUES '.chr(13).chr(10). 
//			chr(9).chr(9).chr(9).'INSERT INTO CLX_DB_IMPORT_ERRORS ("ID","IMPORTCODE","FIELDNAME","ERRORMSG") VALUES '.chr(13).chr(10). 
//			chr(9).chr(9).chr(9).'(NULL, \'XX\',\'-\',\'-\');'.chr(13).chr(10).    ///,'.singlequote('').',0,0,0,0,0,\'ERR_MSG\',\'2019-01-01\');'.chr(13).chr(10).  
//			chr(9).chr(9).'COMMIT;'.chr(13).chr(10). 
			//&'.singlequote(' ').'&TO_CHAR(col_47(0))  //TO_CHAR(col_47 (SQL%BULK_EXCEPTIONS (i).ERROR_INDEX))
		
		if ($debug==true) {
			pre("[[[[[[create fields exceptions");
			pre("fields");
			pre($fields);
			pre("fields_array");
			pre($fields_array);
		}
		if (in_array("LINE_OF_LOG_ID",$fields_single)) {
			$create_pkg_body.=	chr(9).chr(9).chr(9).'ERR_LINE:=col_'.(count($fields_array)-1).'(SQL%BULK_EXCEPTIONS (i).ERROR_INDEX);'.chr(13).chr(10);			
		}
		else {
			$create_pkg_body.=	chr(9).chr(9).chr(9).'ERR_LINE:=0;'.chr(13).chr(10);

		}			
		$create_pkg_body.=	chr(9).chr(9).chr(9).'ERR_MSG:=SQLERRM(\'-\'||SQL%BULK_EXCEPTIONS (i).ERROR_CODE);'.chr(13).chr(10).  
			chr(9).chr(9). '	TEXTRA1:=\'\';'.chr(13).chr(10).
			chr(9).chr(9). '	TEXTRA2:=\'\';'.chr(13).chr(10);
			
		if($onlyInsert==true){
			if (empty($textoextraposicion2)) {
				$create_pkg_body .=chr(9). '	TEXTRA3:=\'INSERT INTO '.$tablename.' ('.$fields.') VALUES (\';'.chr(13).chr(10);
			}
			else {
				$create_pkg_body .=chr(9). '	TEXTRA3:=\'INSERT INTO '.$tablename.' ('.$fields.') VALUES (\'||'.$textoextraposicion2.';'.chr(13).chr(10);
			}
		
		}
		else {
			$create_pkg_body .=chr(9).chr(9). '	TEXTRA3:=\'MERGE INTO '.$tablename.' T1 '.
								chr(9).chr(9).' 	USING (SELECT \'||'. $string_use_objects_alias_view .'||\' FROM DUAL) T2 \';'.CHR(13).CHR(10).
			
			chr(9).chr(9).'IF (LENGTH(\''.$WHEREON.'\')+LENGTH(TEXTRA3))>1984 THEN '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA3:=\' ON ('.$WHEREON.')  \';'.chr(13).chr(10).
			chr(9).chr(9).'ELSE '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA3:=TEXTRA3||\' ON ('.$WHEREON.')  \';'.chr(13).chr(10).
			chr(9).chr(9).'END IF;'.chr(13).chr(10);
			
		$create_pkg_body .=chr(9).chr(9).'IF (LENGTH(\''.$update_objects.'\')+LENGTH(TEXTRA3))>1950 THEN '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA3:=\' WHEN MATCHED THEN  '.CHR(13).CHR(10).
			chr(9).chr(9).chr(9).	chr(9).'UPDATE SET '.$update_objects.'\';'.chr(13).chr(10);

		$create_pkg_body .=chr(9).chr(9).'ELSE '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'   TEXTRA3:=TEXTRA3||\' WHEN MATCHED THEN  '.CHR(13).CHR(10).
			chr(9).chr(9).chr(9).	chr(9).' UPDATE SET '.$update_objects.'\';'.chr(13).chr(10);

		$create_pkg_body .=	chr(9).chr(9).'END IF;'.chr(13).chr(10);
			
		$create_pkg_body .=chr(9).chr(9).'IF (LENGTH(\''.$fields.'\')+LENGTH(TEXTRA3))>1951 THEN '.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).chr(9).chr(9).'TEXTRA3:=\' WHEN NOT MATCHED THEN  '.CHR(13).CHR(10).
			chr(9).chr(9).	chr(9).chr(9).' UPDATE SET '.$fields.'\';'.chr(13).chr(10).

			chr(9).'ELSE '.chr(13).chr(10).
			chr(9).'   TEXTRA3:=TEXTRA3||\' WHEN NOT MATCHED THEN  '.CHR(13).CHR(10).
				chr(9).' INSERT ( '.$fields.')\';'.chr(13).chr(10).

			chr(9).'END IF;'.chr(13).chr(10);
			
			$create_pkg_body .=chr(9).chr(9).'IF (LENGTH(\''.$string_use_objects_with_alias.'\')+LENGTH(TEXTRA3))>1976 THEN '.chr(13).chr(10).
			chr(9).chr(9).'	TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).chr(9).'	TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).chr(9).'	TEXTRA3:=\' VALUES ('.$string_use_objects_with_alias.')  \';'.chr(13).chr(10).

			chr(9).'ELSE '.chr(13).chr(10).
			chr(9).'   TEXTRA3:=TEXTRA3||\' VALUES ('.$string_use_objects_with_alias.')  \';'.chr(13).chr(10).
			chr(9).'END IF;'.chr(13).chr(10);
			
	//	chr(9).'END IF;'.chr(13).chr(10);	
			
			
		}
		
				
			//chr(9).chr(9).chr(9).'ERR_MSG:=SQLERRM(\'-\'||SQL%BULK_EXCEPTIONS (i).ERROR_CODE);'.chr(13).chr(10).   
//			chr(9).chr(9).chr(9).'ERR_MSG:=ERR_MSG||(SELECT value FROM nls_session_parameters WHERE  parameter = '.singlequote('NLS_DATE_FORMAT').');'.chr(13).chr(10).   
		//	chr(9).chr(9).chr(9).'ERR_MSG:=SQL%BULK_EXCEPTIONS (i).ERROR_CODE&col_1(ERR_LINE);'.chr(13).chr(10).
			
		//	chr(9).chr(9).chr(9).'ERR_MSG:=SQL%BULK_EXCEPTIONS (i).ERROR_CODE&\'(\'&col_0(ERR_LINE)&\';\'&col_1(ERR_LINE)&\'...)\';'.chr(13).chr(10).
			
			
		$create_pkg_body .=		chr(9).chr(9).chr(9).'ERR_NUM:=SQL%BULK_EXCEPTIONS (i).ERROR_CODE;'.chr(13).chr(10).
			$textoextraposicion
		//	;$e=
			.
			chr(9).' TEXTRA3:=	 TEXTRA3||\')\';'.chr(13).chr(10).
			chr(9).' IF LENGTH(TEXTRA1)<=1 THEN '.chr(13).chr(10).
			chr(9).'	 TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).'	 TEXTRA2:=TEXTRA3;'.chr(13).chr(10).
			chr(9).'	 TEXTRA3:=\'\';'.chr(13).chr(10).
			chr(9).' 	 IF LENGTH(TEXTRA1)<=1 THEN '.chr(13).chr(10).
			chr(9).'		 TEXTRA1:=TEXTRA2;'.chr(13).chr(10).
			chr(9).'	 	 TEXTRA2:=\'\';'.chr(13).chr(10).
			chr(9).'	 END IF;'.chr(13).chr(10).
			chr(9).'END IF;'.chr(13).chr(10);
			
			if (in_array("IMPORTCODE",$fields_single)) {
				$texto_code='IMPORTCODE';
			}
			else {
				$texto_code=singlequote($importcode);
		
			}
			if (in_array("BATCH_ID",$fields_single)) {
				$texto_code2='BATCH_ID';
			}
			else {
				$texto_code2=singlequote($BATCH_ID);
			
			}
			if (in_array("LOG_ID",$fields_single)) {
				$texto_code3='LOG_ID';
			}
			else {
				$texto_code3=singlequote($LOG_ID);
			
			}
			if ($error_general) {	
				$create_pkg_body .=chr(9).chr(9).chr(9).
									'INSERT INTO CLX_BUGS ("ID","URL","USERNAME","PRIORITY","TYPE","STATUS","ERRMSG","ERRNUM","CDATE","MDATE") '.
									' VALUES (NULL,'.singlequote('-').','.singlequote('administrator').',0,1,1,ERR_MSG||'.singlequote('(line:').'||ERR_LINE||'.singlequote(')').',ERR_NUM,  CURRENT_DATE, CURRENT_DATE);'.chr(13).chr(10);
			}
			else {
				$create_pkg_body .=chr(9).chr(9).chr(9).'INSERT INTO CLX_DB_IMPORT_ERRORS ("ID","FIELDNAME","ERRORMSG","LINENUM","ERRORNUM","CDATE","BATCH_ID","IMPORTCODE","LOG_ID","EXTRA1","EXTRA2","EXTRA3") VALUES '.chr(13).chr(10). 
									//chr(9).chr(9).chr(9).'( NULL, '.singlequote('bulk not field information').',ERR_MSG,INIT_POS+ERR_LINE ,ERR_NUM, CURRENT_DATE,BATCH_ID,IMPORTCODE,LOG_ID);'.chr(13).chr(10);
									chr(9).chr(9).chr(9).'( NULL, '.singlequote('bulk not field information').',ERR_MSG,ERR_LINE ,ERR_NUM, CURRENT_DATE,'.$texto_code2.','.$texto_code.','.$texto_code3.',TEXTRA1,TEXTRA2,TEXTRA3);'.chr(13).chr(10);
				
			}
			

			
			
			$create_pkg_body .=chr(9).chr(9).	'COMMIT;'.chr(13).chr(10).
//			chr(9).chr(9).chr(9).'INSERT INTO CLX_DB_IMPORT_ERRORS ("ID","IMPORTCODE","FIELDNAME","ERRORMSG","LINENUM","ERRORNUM","CDATE") VALUES'.chr(13).chr(10). 
//			chr(9).chr(9).chr(9).'( NULL, \'RR\'|| IMPORTCODEX,'.singlequote('bulk not field').',ERR_MSG,ERR_LINE ,ERR_NUM, TRUNC(CURRENT_DATE) ) ;'.chr(13).chr(10).    // BATCH_ID,IMPORTCODE,LOG_ID );'.chr(13).chr(10).  
//			chr(9).chr(9).	'COMMIT;'.chr(13).chr(10).
				
			chr(9).chr(9).	'END LOOP;'.chr(13).chr(10).   
			'  END  '.$name_procedure.';'.chr(13).chr(10).
			'END ARRAYBIND_'.$name_procedure.';';
			if ($debug==true) {
				pre($create_pkg);
				pre($create_pkg_body);
			}
		return array('parsebind'=>$parsebind,'create_pkg'=>$create_pkg ,'create_pkg_body'=>$create_pkg_body,'bindes'=>$bindes);
	} 
	
	function bulk_insert(&$fd,&$import,$tablename='',$name_procedure='') {
		$error_general=false;
		$debug=false;
		$tiempoOracle=0;
		
		IF (ISSET($GLOBALS['ADDRDEBUGIMPORT']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGIMPORT']==$_SERVER['REMOTE_ADDR'])) {
			$debug=true;
		} 
		if ($debug==true) {echo 'activado restore error handler';
			restore_error_handler();
		}	
		$options_import=&$import->options_import;
		$namefilebytes=kPathToWebSite.'/temp/'.str_replace('.','',$options_import['original_filename']).'.barr';
		$insertfields=&$import->options_import['insertfields'];
		$mappings=$import->options_import['mappings'];
		$BATCH_ID=&$import->options_import['BATCH_ID'];
		$IMPORTCODE=&$import->options_import['IMPORTCODE'];
		$LOG_ID=$import->options_import['log_id'];
		$CREATEDBY=&$import->options_import['CREATEDBY'];
		$MODIFIEDBY=&$import->options_import['MODIFIEDBY'];
		$CDATE=&$import->options_import['CDATE'];
		$MDATE=&$import->options_import['MDATE'];
		$FILEDATE_REQ=&$import->options_import['FILEDATE_REQ'];
	//	echo "...........in ocidatabase............";
	//	pre($import->options_import);
	//	echo "...........in ocidatabase............";
		$insert_only=($import->options_import['IF_DUPLICATES'] ==  CLX_NUM_ZERO);
		
		if (empty($BATCH_ID)) {
			$BATCH_ID=0.0;
		}
		if (empty($BATCH_ID)) {
			$INIT_POS=0;
		}
		
		$dblink=&$this->dblink;
		$describe=$this->describe_table($tablename);
		
		$maxlot=$fd->max_num_lot;	
		$insertfields=&$options_import['insertfields'];
		
		$restrictions=&$options_import['restrictions'];
		$max=0;
		$superlotnum=0;
		$xx=0;
		if ($debug==true) {
			echo 'fd',__LINE__.' ',__METHOD__;
			pre($fd);
		}
		for ($superlot=0;$superlot<$fd->countsuperlot;$superlot++) {
			if ($debug==true) {
				echo "<br>total:",$fd->countsuperlot," superlot ", $superlot,"<br>";
				echo "c superlotnum",$superlotnum,"<br>";
			
				echo"<br><br>superlot",$superlot,",",$fd->countsuperlot,"<br><br>";
				echo "=================================================";
			}
			for ($numlot=0;$numlot<$maxlot;$numlot++) {
			//	e cho " numlot ", $numlot,",maxlot",$maxlot, "<br>";
			//	e cho "<br><br>numlot",$numlot,"<br><br>";
			//	e cho "'occcco: $xx  '";
			//	if ($xx++>30) {
			//		die;
			//	}
				if (!$import->getmappingsx($fd,$import,$restrictions,$insertfields,$options_import,$numlot,$superlot) ) {
					if ($debug==true) {
						echo "<br><br>no insertar numlot en bulk",$numlot,"continue<br><br>";
					}
					continue;
				}
				if ($fd->filesize>0) {
					file_put_contents($namefilebytes,(int) ($fd->file_read_tonow_bytes*100 /$fd->filesize));
				}
				else {
					file_put_contents($namefilebytes,0);
				}
				$verificar_exists=&$options_import['verificar_column'];
				$single_fields=&$options_import['single_column'];
				if ($debug==true) {
					pre('(((((((((((single_fields in bulk))))))))))');
					pre($single_fields);
			
				}	
				$array_data=&$options_import['array_valid_column'];
				if ($debug==true) {
					pre('((((((((((array_data in bulk OCI '. __LINE__ .')))))))) ');
					pre($array_data);
				}	
		//		if ($superlot>14)
				
			//	$superlotnum+=count($array_data);
			//	$superlotnum+=count($array_data['LINE_OF_LOG_ID']);
			//	p re($array_data['LINE_OF_LOG_ID']);
			//	ec ho "suma superlotnum",$superlotnum,"<br>";
				
//p re($array_data);
				//p re($array_data); ;die;
				//for ($t=0;$t<181;$t++){
				//	$options_import['array_valid_column']['FECHA_POSTEO'][$t]='2019/02/19';
				//}
				$IDS=array();			  
				if ($debug==true) {
					echo"<br>numlot", $numlot,",array_data:",count($array_data["LINE_OF_LOG_ID"]),"<br>";
					pre($array_data);
				}
			
			
				if (1==1) {
					$fields_array=array_keys($array_data);
								
					$extra=array(array('INIT_POS', 'NUMBER'));
					$fields_array_single=array_keys($single_fields);
					if ($debug==true) {
						echo "=================single array ----". __LINE__ ."----";
						pre($fields_array_single);
						
					}
					if ($numlot==0&& $superlot==0) {
						if ($debug==true) {
							echo "creando procedure IN ". __LINE__ ;
						}	
						$retorno_create_procedure=$this->create_procedure_bulk($IMPORTCODE,$BATCH_ID,$LOG_ID,$insert_only,$extra,$mappings,$tablename,$describe,$name_procedure,$verificar_exists,$fields_array_single,$fields_array,false)	;
														
						
						$bindes=$retorno_create_procedure['bindes'];
					
				
						$create_pkg=$retorno_create_procedure['create_pkg'];
						$create_pkg_body=$retorno_create_procedure['create_pkg_body'];
						$parsebind=$retorno_create_procedure['parsebind'];
						$size_fields=$describe['size_fields'];
						$fisic_fields=$describe['fisic_fields'];

						$datos_descripcion_bind=array();
						$max1=count($array_data);
						
						$max2=count($single_fields);
						if ($debug==true) {
							
							//paso 1 crear paquete bd
							echo "<br><b>paso1 CREATE Package create_pkg</b><br>";
							echo "<font color='blue'><b>";
								pre( $create_pkg);
							echo "</b></font>";
						}
						
						$stid = oci_parse($dblink, $create_pkg);
						oci_execute($stid);
						if ($debug==true) {
							//paso 2 crear procedure cuerpo paquete
							echo "<br><b>paso2 Create Package Body create_pkg_body</b><br>";
							echo "<font color='blue'><b>";
							pre($create_pkg_body);
							echo "</b></font>";
						}
						$stid = oci_parse($dblink, $create_pkg_body);
						oci_execute($stid);
						$this->commit();
						//paso 3 llenar los binds
						if ($debug==true) {
							echo "<br><b>paso3 create text for binds array instruccion</b><br>";
						}
						for ($t=0;$t<$max1;$t++) {
							$type=$fisic_fields[uppercase($fields_array[$t])];
							$typevalue=$this->typebind($type,"array");
							$size=$size_fields[uppercase($fields_array[$t])];
							$bind=$bindes[uppercase($fields_array[$t])];	
							if ($debug==true) {
								echo "<br>linea:$t<br>";
								echo "------>", $bind,",",$size,"<br>";
							}
							$datos_descripcion_bind[]=array('1',':'.$bind,uppercase($fields_array[$t]),$size,$typevalue,$type); 
							
						}	
						if ($debug==true) {
							echo "<br><b>paso4 create text for array array instruccion</b><br>";
						}
						$array_keys=array_keys($single_fields);
						
						$fisic_fields['BATCH_ID']='NUMBER';
						$size_fields['BATCH_ID']='10.0';
						$bindes['BATCH_ID']='BATCH_ID';
						$fisic_fields['INIT_POS']='NUMBER';
						$size_fields['INIT_POS']='10.0';
						$bindes['INIT_POS']='INIT_POS';
						
						for ($t=0;$t<$max2;$t++) {
							if ($debug==true) {
								echo "<br>linea:$t<br>";
								echo uppercase($array_keys[$t]);
							}
							$type=$fisic_fields[uppercase($array_keys[$t])];
							$typevalue=$this->typebind($type,"");
							$size=$size_fields[uppercase($array_keys[$t])];
							$bind=$bindes[uppercase($array_keys[$t])];	
							if ($debug==true) {
								echo "------>", $bind,",",$size,"<br>";
							}
							$datos_descripcion_bind[]=array('2',':'.$bind,uppercase($array_keys[$t]),$size,$typevalue,$type); 
							
						}
						$this->commit();
						if ($debug==true) {
							echo "<br><b>paso5 ejecutar text bind to create in bd bloque begin</b><br>";
						}
						$textoforzado='';
						if (in_array("BATCH_ID",$single_fields)) {
							$textoforzado.=':BATCH_ID,';
		
			
						}
						if (in_array("INIT_POS",$single_fields)) {
							$textoforzado.=':INIT_POS,';
						}
						
						$parsebind=str_replace(':col_0',$textoforzado.':col_0',$parsebind);
						$parsebind='BEGIN '.chr(13).chr(10).$parsebind.'); END;';
						
						
						if ($debug==true) {
							echo '<b><font color=red>parsebind<br>';
							pre($parsebind);
							echo"</font></b><br>datos_descripcion_bind";
							pre($datos_descripcion_bind);
						
						}
						
						$stid = oci_parse($dblink, $parsebind);
						$mxx=count($datos_descripcion_bind);
						$xx=0;
					}
					$maxtotal=$max;
					$sicorre=false;
					for ($t=0;$t<$mxx;$t++) {
						
						if ($debug==true) {
								echo "<br><b>paso6 ejecutar text bind to create php in bd individual: $t</b><br>";
						}
						$opcion=$datos_descripcion_bind[$t][0];
						$bind=$datos_descripcion_bind[$t][1];
						$name=$datos_descripcion_bind[$t][2];
						$size=$datos_descripcion_bind[$t][3];
						$type=$datos_descripcion_bind[$t][4];
						$typetext=$datos_descripcion_bind[$t][5];
						
						if ($opcion==1) {
							
							
							$max=count($array_data[$name]);
						//	ec ho "___________";
						//	pr e($array_data[$name]);
						//	e cho ",", $max;
							if ($max>0) {$dsi=false;
								foreach($array_data[$name] as $dkey=>$dvalue) {
									if (!empty($dvalue)) {
										$dsi=true;
										break;
									}	
								}
								if ($dsi) {
								//	ec ho "en max";
								//	pr e($name);
									$sicorre=true;
									break;
								}	
							}
							
						}
					}
					if ($sicorre) {
						for ($t=0;$t<$mxx;$t++) {
							
							if ($debug==true) {
								echo "<br><b>paso6 ejecutar text bind to create php in bd individual: $t</b><br>";
							}
							$opcion=$datos_descripcion_bind[$t][0];
							$bind=$datos_descripcion_bind[$t][1];
							$name=$datos_descripcion_bind[$t][2];
							$size=$datos_descripcion_bind[$t][3];
							$type=$datos_descripcion_bind[$t][4];
							$typetext=$datos_descripcion_bind[$t][5];
							
							if ($opcion==1) {
								$arr=$name;
								$$arr=$array_data[$name];
								$max=count($$arr);
								if ($debug==true /*or ($numlot>1331 and$superlot==3)*/) {
									echo "<br>lineayy:  $bind,$name,$type,$typetext,$size<br>";
								}
								if (!empty($$arr)) {
									if ($debug==true) {
										echo '<br>opcion 1 ===bind ', $bind,' , arr';
										pre($$arr);
										echo $max,' ,-1 ', $type,')';
									}
									oci_bind_array_by_name($stid, $bind, $$arr, $max, -1, $type);
								}	
								else {
								//	pre($$arr);
								}
								
							}
							else if ($opcion==2 ) {
								IF (1==1 or $numlot==0&&$superlot==0) {
									IF ($typetext=='DATE')
										$size=25;
									if ($debug==true) {
								
										echo "<br> oopcion 2 ====lineaxx:  $bind,$name,$type,$typetext,$size,$typevalue,<br>";
										echo 'aca----->',$name,",",$single_fields[$name],"<br>";
									
									}
									$$name=$single_fields[$name];
									if ($debug==true) {
										echo 'xxx--->'. $name,",",$$name,"<br>";
									}
									if ($name=='BATCH_ID') {
										$BATCH_ID=0;
										
									}
									if ($debug==true) {
										echo '<br>bbbbbbbbbbbbbbbb1 ', $bind,' ,2 ',$name,' ,3 ', $size,' ,4 ', $type,')';
									}
						//			$x2stime = microtime_float();
									oci_bind_by_name($stid, $bind, $$name, $size, $type);
						//			$tiempoOracle+=microtime_float()-$x2stime;
									
								}	
								if ($name=='INIT_POS') {
							
									$INIT_POS=$superlotnum;//+$LINE_OF_LOG_ID[0]-1;	
					//				$x2stime = microtime_float();
									oci_bind_by_name($stid, $bind, $$name, $size, $type);
									
								}						
								
							
							}
						}	
					}
					if ($debug==true) {
						echo "<br><b>paso7 execute final</b><br>";
					}
					//
					if ($sicorre){
				//		$x2stime = microtime_float();
						oci_execute($stid);
			//			$tiempoOracle+=microtime_float()-$x2stime;
								
						if (isset($array_data['LINE_OF_LOG_ID']))
							$superlotnum+=$x=count($array_data['LINE_OF_LOG_ID']);
						else {
							$rr=array_keys($array_data);
							$superlotnum+=$x=count($array_data[$rr[0]]);
						}	
					//	ec ho $array_data['LINE_OF_LOG_ID'][0],"<br>";
					//	e cho $array_data['LINE_OF_LOG_ID'][$x-1],"<br>";
					//	ec ho $x,"<br>";
					//	e cho "sAuperlot ,", $superlotnum,"<br>";
					}
					else {
					//	ec ho 'nocorre';
					}
					
				}	
				
			}
			
			$maxlot=$fd->max_num_lot;
			
		
		}	
		if ($debug==true) {
			echo 'FIN  BULK',__LINE__.' ',__METHOD__;
			pre($fd);
		}
		$import->options_import['superlotnum']=$superlotnum-1;	
		file_put_contents($namefilebytes,100);
			
	} 
	function bulk_insert_simple($array_data,$keys_data,$tablename='',$name_procedure='') {
		$debug=false;
		IF (ISSET($GLOBALS['ADDRDEBUGIMPORT']) && (empty($_SERVER['REMOTE_ADDR']) || $GLOBALS['ADDRDEBUGIMPORT']==$_SERVER['REMOTE_ADDR'])) {
			$debug=true;
		}	 
		$error_general=true;
		$dblink=&$this->dblink;
		$describe=$this->describe_table($tablename);
		$single_fields=array();
		$IDS=array();			  
		$fields_array=array_keys($array_data);
		$fields_array_single=array_keys($single_fields);
		$mappings=array_keys();
		$retorno_create_procedure=$this->create_procedure_bulk($fields_array,$tablename,$describe,$name_procedure,false,$fields_array_single,$fields_array,$error_general)	;
		$bindes=$retorno_create_procedure['bindes'];
		$create_pkg=$retorno_create_procedure['create_pkg'];
		$create_pkg_body=$retorno_create_procedure['create_pkg_body'];
		$parsebind=$retorno_create_procedure['parsebind'];
		$size_fields=$describe['size_fields'];
		$fisic_fields=$describe['fisic_fields'];

		$datos_descripcion_bind=array();
		$max1=count($array_data);
						
		$stid = oci_parse($dblink, $create_pkg);
		oci_execute($stid);
		$stid = oci_parse($dblink, $create_pkg_body);
		oci_execute($stid);
		$this->commit();
		for ($t=0;$t<$max1;$t++) {
			$type=$fisic_fields[uppercase($fields_array[$t])];
			$typevalue=$this->typebind($type,"array");
			$size=$size_fields[uppercase($fields_array[$t])];
			$bind=$bindes[uppercase($fields_array[$t])];	
			if ($debug==true) {
				echo "<br>linea:$t<br>";
				echo "------>", $bind,",",$size,"<br>";
			}
			$datos_descripcion_bind[]=array('1',':'.$bind,uppercase($fields_array[$t]),$size,$typevalue,$type); 
			
		}	
		$this->commit();
		$parsebind='BEGIN '.chr(13).chr(10).$parsebind.'); END;';
			
		$stid = oci_parse($dblink, $parsebind);
		$mxx=count($datos_descripcion_bind);
		$xx=0;
		$maxtotal=$max;
		for ($t=0;$t<$mxx;$t++) {
			$opcion=$datos_descripcion_bind[$t][0];
			$bind=$datos_descripcion_bind[$t][1];
			$name=$datos_descripcion_bind[$t][2];
			$size=$datos_descripcion_bind[$t][3];
			$type=$datos_descripcion_bind[$t][4];
			$typetext=$datos_descripcion_bind[$t][5];
					
			$arr=$name;
			$$arr=$array_data[$name];
			$max=count($$arr);
			if ($debug==true /*or ($numlot>1331 and$superlot==3)*/) {
				echo "<br>lineayy:  $bind,$name,$type,$typetext,$size<br>";
			}
			if (!empty($$arr)) {
			/*	e cho "================1=";
				e cho("stid");
				pr e($stid);
				e cho ("bind");
				pr e($bind);
				e cho ("arr");
				pr e($$arr);
				e cho ("max");
				pr e($max);
				e cho ("type");
				pr e($type);
				e cho "================2=";*/
				oci_bind_array_by_name($stid, $bind, $$arr, $max, -1, $type);
			}	
			else {
			//	pr e($$arr);
			}
	
		}	
			
		oci_execute($stid);
		
		$this->commit()		;
	} 

	function typebind($type,$typefunction=''){
		$typevalue='';
		if (strpos(' ,DATE,TIMESTAMP,CHAR,NCHAR,VARCHAR,VARCHAR2, ',','. $type.',') >0 ) {
			if ($typefunction=='array') {
				$typevalue =SQLT_CHR;
			}	
			else {
				$typevalue =SQLT_CHR;
			}	
		}
		elseif (strpos(' ,NCHAR,NVARCHAR,NVARCHAR2, ',','. $type.',') >0 ) {
			$typevalue =SQLT_CHR;
		}
		else if (strpos(' ,FLOAT,NUMBER, ',   ','. $type.',') >0 ) {
			if ($typefunction=='array') {
				$typevalue =SQLT_FLT;
			}	
			else {
				$typevalue = SQLT_LNG;
			}	
		}
		
		else if (strpos(' ,LONG, ',   ','. $type.',') >0 ) {
			$typevalue =SQLT_LNG;		
		}
		else if (strpos(' ,RAW,LONG RAW, ',  ','. $type.','	)>0) {
			$typevalue =SQLT_LBI;

		}
		else if (strpos(' ,CLOB,NCLOW, ',  ','. $type.','	)>0) {
			$typevalue =SQLT_CHR;
		
		}
		else if (strpos(' ,LOB,FILE, ',  ','. $type.','	)>0) {
			$typevalue = SQLT_BLOB;

		}
		
		return $typevalue;
	}
	function read_time_tmp() {
		$sql = 'SELECT  SYPRVALUE FROM CLX_SYS_PARAMETERS WHERE SYPRKEY='.singlequote('MAX_TIME_HOUR_DELETE_TMP').' AND TOOLCODE = ' . singleQuote('DBTB');
		$maxtime=$this->getOne($sql);
		if ($maxtime<=1)
			$maxtime=1;
		define('MAX_TIME_HOUR_DELETE_TMP',$maxtime); 
	}
	function review_and_delete_old_tmp($tabletmp='',$tiempo=8640) {
		$this->read_time_tmp();
		$extra='';
		if (!empty($tabletmp))
			$extra=' TABLE_NAME ='.singlequote($tabletmp).' AND ';
		
		if ($this->debug==true) {
			echo "Review Tmp function".chr(10);
		}
		$this->temporary_supress_squote_change = true;
		$sql = 'SELECT TABLE_NAME FROM USER_TABLES WHERE '.$extra.' TABLE_NAME LIKE '.singlequote('Z__TMP_%');
		$rs = $this->executeQuery($sql);       
	
		$nowPHP=hexdec(uniqid())-200000;
	
		
        while (($result = $this->getNextRecord($rs, ASSOCIATIVE)) ) {
			if ($this->debug==true) {
				echo "Review Tmp function",$result['TABLE_NAME'];
			}
			$cortes=explode('___',$result['TABLE_NAME']);
			
			if (count($cortes)>1 && $nowPHP>intval($cortes[1])) {
				if ($this->debug==true) {
					echo " Delete temp file ".$result['TABLE_NAME'];
				}
				$this->executeQuery('DROP TABLE '.$result['TABLE_NAME']);
			}
			
		}
		$this->freeResult($rs);
	}
	function fetch_all($id, $type = ASSOCIATIVE) {
		if ($id != NULL){
			oci_fetch_all($id, $res);
			return $res;
		}else {
			return NULL;
		}	
	}
	
}
